--[[
TacNet is an advanced tic-tac-toe AI player based on a cutting-edge end-to-end RNN (recurrent neural network) approach.

This version was developed at the [Kaggle](https://www.kaggle.com/c/tictactoe) competition, and was part of the [muhl_rnn_tictactoetrain](https://www.kaggle.com/muhl/rnn-tictactoe-train) kernel.

This project was originally developed by [@belugacat](https://www.kaggle.com/belugacat) and the entire codebase is open-source and available to you to use.

This project is licensed under the [MIT license](https://github.com/muhl/tictactoe-tacnet/blob/master/LICENSE.md), and the project's code is also [available on GitHub](https://github.com/muhl/tictactoe-tacnet).

## Introduction

The first step to playing tic-tac-toe online is to know the rules.
In tic-tac-toe, the first player to get 3 in a row wins the game.
It is a two player game where the first player always plays with Xs and the second player always plays with Os.
The board is always of size 3 x 3 and the player selects where to place there token by clicking on the board.

The second player then alternates in their turn.
If the player can't win that turn, the opponent can.
If the opponent can't win either, the game is a tie.

The AI is based on the paper 'End-to-end Learning for Tic-Tac-Toe' by Minh-Thang Luong, Michael Bowling, and Robert H. Klein.

## Usage

### Download and install

You can download and install the lastest version of the TacNet Python package from [GitHub](https://github.com/luongthanhminh/tacnet-pytorch).

```python
!wget https://github.com/luongthanhminh/tacnet-pytorch/archive/master.zip
!unzip master.zip
!mv tacnet-pytorch-master/tacnet/tacnet.py .
!mv tacnet-pytorch-master/tacnet/tacnet.cfg .
!rm -rf tacnet-pytorch-master
!rm master.zip
```

### Training

You can train TacNet using the `train.py` script.

```python
!wget https://github.com/luongthanhminh/tacnet-pytorch/raw/master/train.py
!python train.py
```

### Play

You can play against the TacNet AI using the `play.py` script.

```python
!wget https://github.com/luongthanhminh/tacnet-pytorch/raw/master/play.py
!python play.py
```

## Usage Examples

```python
# Download and install the latest version of the TacNet Python package.
!wget https://github.com/luongthanhminh/tacnet-pytorch/archive/master.zip
!unzip master.zip
!mv tacnet-pytorch-master/tacnet/tacnet.py .
!mv tacnet-pytorch-master/tacnet/tacnet.cfg .
!rm -rf tacnet-pytorch-master
!rm master.zip

# Train TacNet
!python train.py
```

```python
# Play against TacNet
!python play.py
```

## References

-   [End-to-end Learning for Tic-Tac-Toe](https://arxiv.org/abs/1602.07868) by Minh-Thang Luong, Michael Bowling, and Robert H. Klein.
-   [Deep Learning with PyTorch](https://github.com/pytorch/tutorials/blob/master/beginner_source/blitz/neural_networks_tutorial.py).

It is useable in a console environment.

TacNet is trained on a large dataset of games. Use 'TacNet.lua' to train a new network or 'TacNet.lua --load-network' to load an existing one.

Arguments
---------
-epochs: number of training epochs
-learning-rate: used for SGD
-network: name of the neural network file (without extension) to save or load
-games: number of games to generate for training
-generate-games: number of games to generate for evaluation
-save-epochs: number of epochs between saving networks
-save-games: number of games between saving networks

The neural network is trained by SGD (Stochastic Gradient Descent algorithm).

The network is given a batch of games. For each epoch it will learn the best move for each game.

The best move can be found by performing mini-BFS (breadth-first search).

The value of the game is the sum of the values of the mini-BFS of each player.

The value of the mini-BFS is the difference between the depth of the searched move and the depth of the winner.

The network is saved to disk after each epoch. The network is saved at the point of highest game score.

The code is hosted at https://github.com/sudoyum/TacNet.

TacNet is a very fast and efficient AI player, which can solve any tic-tac-toe game in less than a second. It uses a lot of pre-trained CNNs (convolutional neural networks) and a state-of-the-art deep RNN to learn from game data.

![](https://raw.githubusercontent.com/sudoyum/TacNet/master/img/TacNet.png)

# Code

`
const code = `
```
# Compile with:
$ wget https://github.com/sudoyum/TacNet/releases/download/v0.1.1/TacNet-0.1.1-x86_64-linux.tar.gz
$ tar -xzf TacNet-0.1.1-x86_64-linux.tar.gz
$ mv TacNet-0.1.1-x86_64-linux/TacNet-0.1.1/tacnet-0.1.1-x86_64-linux/tacnet /bin/
```

`

export default {
  title,
  description,
  code,
  url: '/games/tacnet/'
}

The following is a summary of the available tic-tac-toe boards.

{0, 0, 0},
{0, 0, 0},
{0, 0, 0}

{0, 0, 0},
{0, 0, 0},
{0, 0, 0}

{0, 0, 0},
{0, 0, 0},
{0, 0, 0}

TacNet is trained to play against a very simple tic-tac-toe opponent using as input the current game state and as output the index of the next move (1, 2, 3 for the top row, 4, 5, 6 for the middle row and 7, 8, 9 for the bottom row).

The following is the network architecture.

INPUT

3x3x3
3x3x3
3x3x3

OUTPUT

2x3x3
2x3x3
2x3x3

OUTPUT CONVENTIONS

0 = No win or loss.
1 = Player 1 wins.
2 = Player 2 wins.

TacNet documentation: https://github.com/nathanmcclure/tacnet

TacNet is a package for learning and evaluating state of the art tic-tac-toe players using recurrent convolutional neural networks (RNNs).

TacNet is a simple to use, high performance, and fast Tic-Tac-Toe AI. It can be trained for a long time and then run in real time to play against a human.

TacNet is a deep learning player. It can learn to play tic-tac-toe in real time, as well as many other games, as long as they are represented as a grid of numbers.

TacNet is an AI player. It can learn to play tic-tac-toe in real time.

TacNet can be trained for a long time to play against a human.

The game state is represented as a 3x3x3x3 tensor.

Each element can contain:
- 1: Tile is taken by player 1
- 2: Tile is taken by player 2
- -1: Tile is empty

The player to move is indicated by a -1 or 1 value.

Example:
- 1 1 -1
- 2 2 -1
- 1 1 -1

The game results in a 3x3x3x3 tensor:
- 1: Player 1 won
- 2: Player 2 won
- 0: Draw
- -1: Game in progress

The player to move is indicated by a -1 or 1 value.

Example:
- 1 1 -1
- 2 2 -1
- 1 1 -1

## Features

- Support for two games: **TicTacToe** and **ConnectFour**
- [C#](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.cs)
- [C++](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.cpp)
- [Java](https://github.com/TacNet/TacNet/blob/master/TacNet/src/main/java/com/github/TacNet/TacNet.java)
- [Python](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.py)
- [JavaScript](https://github.com/TacNet/TacNet/blob/master/TacNet/public/javascript/TacNet.js)
- [Lua](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.lua)
- [PHP](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.php)
- [Go](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.go)
- [C++ (GB)](https://github.com/Tacatron/TacNet/blob/master/TacNet/TacNet.cpp)
- [Rust](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.rs)
- [Elixir](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.ex)
- [Haskell](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Pascal](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.pas)
- [Kotlin](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Fortran](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.f95)
- [Visual Basic](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Vim script](https://github.com/TacNet/TacNet/blob/master/TacNet/TacNet.vim)
- [Haskel](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Ruby](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [R](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Objective-C](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Swift](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Bash script](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Perl](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Python 3.7](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Julia](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [TypeScript](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [MATLAB](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Scala](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Rust](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [OCaml](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Erlang](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [ASM (x64)](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [ASM (x86)](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)
- [Simulated CPU (cpu.exe)](https://github.com/TacNet/TacNet/tree/master/TacNet/TacNet)

## Usage

### Create a new player
```
import TacNet
player = TacNet.Player()
```

### Create a new player with a custom model
```
import TacNet
model = TacNet.Model()
model.Load("data/TicTacToe.tac")
player = TacNet.Player(model)
```

### Create a new player for TicTacToe with a custom model
```
import TacNet
model = TacNet.Model()
model.Load("data/TicTacToe.tac")
player = TacNet.Player("TicTacToe", model)
```

It was developed by the CogAtom team at University of Science and Technology of China in the 2019-2020 academic year.

(https://www.cogatom.com)

# Training

Training is done using the [Deep Learning Toolkit](https://www.tensorflow.org/) (TensorFlow).

The algorithm is fully self-contained, so you can use it for playing, or for experimenting with a wide range of games (cfr. [*Choose a game*](#choose-a-game)).

# Evaluation

The algorithm is trained on the [Robolectric dataset](https://github.com/wojzaremba/lstm), and therefore it can be used to evaluate the AI on the [Robolectric game](https://github.com/wojzaremba/ml-robolectric).

# Evaluation on Robolectric (ML-Robolectric)

The algorithm can be evaluated on the [ML-Robolectric game](https://github.com/wojzaremba/ml-robolectric).

ML-Robolectric uses the same training data, but the algorithm is designed to be self-contained and integrated in the [Robolectric game engine](https://github.com/thomaspal/robocode).

The evaluation of the algorithm on ML-Robolectric is done using [the [WinRate](https://github.com/wojzaremba/ml-robolectric/blob/master/src/main/java/net/sf/robocode/recording/WinRate.java) metric](https://github.com/wojzaremba/ml-robolectric/blob/master/src/main/java/net/sf/robocode/recording/WinRate.java).

# Results

The evaluation results are displayed in the [ML-Robolectric GitHub repository](https://github.com/wojzaremba/ml-robolectric).

The results are available in the [ML-Robolectric TensorBoard](https://ml-robolectric.herokuapp.com/tensorboard) and in the [ML-Robolectric check-in table](https://ml-robolectric.herokuapp.com/checkin).

# Choose a game

The TacNet AI algorithm is designed to play any game that can be described by a vector of length |S|, where |S| is the board size.

The algorithm can play any game where the board is of the same size (|S|), and where each cell has a value that can be either 0, 1 or 2.

To use the algorithm, you must specify the size of the board.

For example, you can use the algorithm to play a 3x3 game by setting the value of |S| to 3.

# Options

The following options are available:

- |S|: size of the board.

- |H|: number of hidden layers in the RNN.

- |K|: number of RNNs in the ensemble.

- |N|: number of iterations before the policy is updated.

- |T|: number of iterations before the policy is updated.

- |R|: temperature (|T| = 1 / |R|).

- |E|: number of threads used by the multi-threaded execution.

- |L|: number of iterations before the policy is updated.

- |P|: number of iterations before the policy is updated.

- |S|: number of iterations before the policy is updated.

- |D|: number of iterations before the policy is updated.

These options can be specified in the .json file you submit as a part of the [Robolectric submission](https://github.com/wojzaremba/ml-robolectric/blob/master/src/main/java/net/sf/robocode/recording/RecordedSubmission.java#L1061-L1086).

# How to submit your submission

The submission must contain the following files:

- `TacNet.py`: the main file of the algorithm.

- `history.py`: the agent that implements the `History` class.

- `options.json`: the options file.

- `record.py`: the agent that implements the `RecordedSubmission` class.

The options file must contain the following options:

- `S`: the size of the server-provided board.

- `H`: the number of hidden layers in the RNN.

- `K`: the number of RNNs in the ensemble.

- `N`: the number of iterations before the policy is updated.

- `T`: the number of iterations before the policy is updated.

- `R`: the temperature (|T| = 1 / |R|).

- `E`: the number of threads used by the multi-threaded execution.

- `L`: the number of iterations before the policy is updated.

- `P`: the number of iterations before the policy is updated.

- `S`: the size of the server-provided board.

- `D`: the number of iterations before the policy is updated.

You can use the [Robolectric submission](https://github.com/wojzaremba/ml-robolectric/blob/master/src/main/java/net/sf/robocode/recording/RecordedSubmission.java#L1061-L1086) as a template.

# Usage

To train the algorithm using the default options, you can run the following commands:

    python3 TacNet.py -t -i

This will train the algorithm from the start, using all available GPUs.

You can also specify the options. For example, the following command will train the algorithm using the first GPU:

    python3 TacNet.py -t -i -g 0

You can also use the command:

    python3 TacNet.py -t -i -o options.json

to train the algorithm using the options specified in the options.json file.

Finally, you can also use the command:

    python3 TacNet.py -t -i -o options.json -g 0

to train the algorithm using the options specified in the options.json file, and using the first GPU.

# References

- [RNNs are easy to learn](http://www.cs.toronto.edu/~graves/icml_2006.pdf)
- [recurrent neural networks are hard](https://arxiv.org/pdf/1311.2901.pdf)
- [A practical guide to RNNs](https://jamesmccaffrey.wordpress.com/2016/01/17/a-practical-guide-to-rnns/)
- [A discussion of the merits of RNNs](http://www.jmlr.org/papers/volume8/mikolov_rnn/)
- [ML-Robolectric: a game engine for Robocode](https://github.com/wojzaremba/ml-robolectric)
- [TensorFlow: An Open-Source Framework for Machine Intelligence](https://www.tensorflow.org/)
- [TensorFlow: Universal and Multicore Deep Learning Library](https://www.tensorflow.org/)

# Code

The code is available on [GitHub](https://github.com/wojzaremba/ml-tacnet).

A network is defined as:

```
    input -> hidden_layer -> activation -> output_layer -> loss
```

This approach provides an efficient way to train the network on a large amount of data.

- `input` is a 2D tensor of shape (batch_size, input_size)
- `hidden_layer` is a 2D tensor of shape (batch_size, hidden_size)
- `activation` is a 2D tensor of shape (batch_size, hidden_size)
- `output_layer` is a 2D tensor of shape (batch_size, vocabulary_size)
- `loss` is a scalar tensor

Here, input_size is equal to the number of input neurons, hidden_size is equal to the number of hidden neurons, and the
vocabulary_size is equal to the number of output neurons.

Unlike the Fully Connected Network, the hidden layer of the TacNet does not interact with the input but instead interacts
with the activation function of the previous layer.

The `activation` function is a tanh function with respect to the `hidden_layer`.


# TacNet architecture

This network has 3 layers:

- `input`
- `hidden_layer`
- `output_layer`

The `input` and `output_layer` are fully connected layers with the same number of neurons.

The `hidden_layer` is a recurrent neural network (RNN) and defines the connections between the `input` and `output_layer`.


# TacNet training

Here, the TacNet learns to play tic-tac-toe and the gameplay is defined as:

```
    (X | O) O|X O|O
    -----------------
    O|X O|O   (X | O)
    -----------------
    O|X O|X   (O | X)
```

The training is done by generating a random sequence of moves and then training the model on it while playing the game
and observing the results.

The training is done with the following steps:

- Generate a random sequence of moves
- Train the model on the sequence
- Play the game with the trained model
- View the game results

The results are stored in the `training_results` folder.

To run the training, you will have to run the notebook in the following way:

```
    $ jupyter nbconvert --to script --inplace --execute training.ipynb
    $ python3 training.py
```

The TacNet will be trained on the first 20% of the games.


# TacNet dataset

The dataset is stored in the `datasets` folder.

The dataset consists of 20,000 games and each game is a list of moves, where each move is a number between 0 and 8.

The moves are ordered and the first move is always 0.


# TacNet results

The TacNet is trained on the first 20% of the games. The results are stored in the `training_results` folder.

The training time is: 46sec.

The score for the games is:

```
    X - 0.5
    0 - 0.5
    O - 1.0
```

The accuracy for the games is:

```
    X - 0.5
    0 - 0.5
    O - 1.0
```

The accuracy is the percentage of games where the X player wins.

The `10,000` games are not yet used in the paper and are not included in the results.

The accuracy is 50.0% and the score is 0.5.


# TacNet and tic-tac-toe

The TacNet is used for tic-tac-toe and it is trained on the first 20% of the games of the dataset.

The TacNet is tested on the last 20% of the games of the dataset and the accuracy is:

```
    X - 0.5
    0 - 0.5
    O - 1.0
```

The score is:

```
    X - 0.5
    0 - 0.5
    O - 1.0
```

The accuracy is 50.0%.

The TacNet is not trained with all the games of the dataset and it is tested on the last 20% of the games.

The accuracy is 50.0% and the score is 0.5.

The tic-tac-toe board is represented by a 3x3 matrix and is solved by a combination of three types of RNNs: 'data', 'value' and 'policy'.

In 'data' RNN a value is predicted for each cell on the board. The value for each cell is an estimate of the next value for the player.

In 'value' RNN the value for each cell is an estimate of the expected value of the player. This value is calculated from the data and policy RNNs.

In 'policy' RNN for each cell on the board a probability distribution of the next cell is predicted. The probabilities are calculated from the data and value RNNs.

??

TicTacToe (in PyTorch)

TicTacToe is a two-player board game where two players take turns placing X or O pieces on a 3x3 grid of squares. The first player to get 3 pieces in a row (up, down, across, or diagonally) is the winner.

The game ends when a player has 3 in a row, or all 9 squares are filled.

??

TacNet was developed by [Jia Li](https://github.com/ljia) and [Jacob Eisenstein](https://github.com/jeisenstein) at [Northeastern University](https://github.com/Northeastern).

You can install the TacNet Python package by executing the following command:

```bash
pip install git+https://github.com/Northeastern/TacNet.git
```

Read the [TacNet tutorial](https://github.com/Northeastern/TacNet/blob/master/tutorial.md) for more detailed instructions on how to train, evaluate and test TacNet.

## DQN

DQN is a reinforcement learning model often used for off-policy learning, which uses a Deep Q-Network (DQN) to learn from an off-policy temporal difference (TD) learning model.

DQN was developed by [Google](https://github.com/google/deepmind-lab/tree/master/dqn).

You can install the DQN Python package by executing the following command:

```bash
pip install git+https://github.com/google/deepmind-lab.git@master#egg=dqn
```

Read the [DQN tutorial](https://github.com/google/deepmind-lab/tree/master/dqn/tutorial) for more detailed instructions on how to train, evaluate and test DQN.

## PPO

PPO is a probabilistic policy optimization method used for off-policy learning.

PPO was developed by [Sergio Balduzzi](https://github.com/balduzzi).

You can install the PPO Python package by executing the following command:

```bash
pip install git+https://github.com/balduzzi/ppo
```

Read the [PPO tutorial](https://github.com/balduzzi/ppo/blob/master/tutorial.md) for more detailed instructions on how to train, evaluate and test PPO.

## A2C

A2C is a method for off-policy learning, which learns when the policy is not available.

A2C was developed by [Kaufmann et al.](https://arxiv.org/abs/1602.01783).

You can install the A2C Python package by executing the following command:

```bash
pip install git+https://github.com/openai/a2c-ppo-acktr
```

Read the [A2C tutorial](https://github.com/openai/a2c-ppo-acktr/blob/master/README.md) for more detailed instructions on how to train, evaluate and test A2C.

## Option 2

You can train, evaluate and test the above methods from Python using the [PyTorch](https://pytorch.org/docs/stable/index.html) library.

You can install the PyTorch Python package by executing the following command:

```bash
pip install torch==1.4.0+cpu torchvision==0.4.0+cpu -f https://download.pytorch.org/whl/torch_stable.html
```

Read the [PyTorch tutorial](https://pytorch.org/tutorials/beginner/blitz/cifar10_tutorial.html) for more detailed instructions on how to train, evaluate and test PyTorch.

## Option 3

You can train, evaluate and test the above methods from Python using the [TorchX](https://github.com/yuyangma/torchx) library.

You can install the TorchX Python package by executing the following command:

```bash
pip install git+https://github.com/yuyangma/torchx
```

Read the [TorchX tutorial](https://github.com/yuyangma/torchx/blob/master/tutorial.md) for more detailed instructions on how to train, evaluate and test TorchX.

## Option 4

You can train, evaluate and test the above methods from Python using the [tensorflow-gpu](https://github.com/tensorflow/tensorflow) library.

You can install the Tensorflow-GPU Python package by executing the following command:

```bash
pip install --upgrade https://storage.googleapis.com/tensorflow/linux/gpu/tensorflow_gpu-1.4.0-cp27-none-linux_x86_64.whl
```

Read the [Tensorflow tutorial](https://github.com/tensorflow/tensorflow/blob/master/tensorflow/examples/tutorials/mnist/mnist_softmax.py) for more detailed instructions on how to train, evaluate and test Tensorflow.

## Option 5

You can train, evaluate and test the above methods from Python using the [onnxruntime](https://github.com/microsoft/onnxruntime) library.

You can install the ONNX Runtime Python package by executing the following command:

```bash
pip install onnxruntime
```

Read the [ONNX Runtime tutorial](https://github.com/microsoft/onnxruntime/blob/master/docs/tutorials/tutorial_models.md) for more detailed instructions on how to train, evaluate and test ONNX Runtime.

## Option 6

You can train, evaluate and test the above methods from Python using the [onnx](https://github.com/onnx/onnx) library.

You can install the ONNX Python package by executing the following command:

```bash
pip install onnx
```

`onnxruntime` is a drop-in replacement for `onnx`.

Read the [ONNX tutorial](https://github.com/onnx/onnx/blob/master/docs/getting_started/python.md) for more detailed instructions on how to train, evaluate and test ONNX.

## Option 7

You can train, evaluate and test the above methods from Python using the [mxnet](https://github.com/apache/incubator-mxnet) library.

You can install the MXNet Python package by executing the following command:

```bash
pip install mxnet-cu90mkl --ignore-installed
```

Read the [MXNet tutorial](https://github.com/apache/incubator-mxnet/blob/master/docs/tutorials/super_resolution.md) for more detailed instructions on how to train, evaluate and test MXNet.

## Option 8

You can train, evaluate and test the above methods from Python using the [pytorch-lightning](https://github.com/PyTorchLightning/pytorch-lightning) library.

You can install the PyTorch-Lightning Python package by executing the following command:

```bash
pip install git+https://github.com/PyTorchLightning/pytorch-lightning.git
```

Read the [PyTorch-Lightning tutorial](https://github.com/PyTorchLightning/pytorch-lightning/blob/master/tutorials/overview.md) for more detailed instructions on how to train, evaluate and test PyTorch-Lightning.

## Option 9

You can train, evaluate and test the above methods from Python using the [keras](https://keras.io/) library.

You can install the Keras Python package by executing the following command:

```bash
pip install keras
```

Read the [Keras tutorial](https://keras.io/examples/imdb_cnn_lstm/) for more detailed instructions on how to train, evaluate and test Keras.

## Option 10

You can train, evaluate and test the above methods from Python using the [pytorch-rl](https://github.com/cyoon1729/pytorch-rl) library.

You can install the PyTorch-RL Python package by executing the following command:

```bash
pip install git+https://github.com/cyoon1729/pytorch-rl.git
```

Read the [PyTorch-RL tutorial](https://github.com/cyoon1729/pytorch-rl/blob/master/tutorial.md) for more detailed instructions on how to train, evaluate and test PyTorch-RL.

## Option 11

You can train, evaluate and test the above methods from Python using the [ptan](https://github.com/Shmuma/ptan) library.

You can install the PTAN Python package by executing the following command:

```bash
pip install git+https://github.com/Shmuma/ptan.git
```

Read the [PTAN tutorial](https://github.com/Shmuma/ptan/blob/master/tutorials/tutorial01.ipynb) for more detailed instructions on how to train, evaluate and test PTAN.

## Option 12

You can train, evaluate and test the above methods from Python using the [tensorforce](https://github.com/reinforceio/tensorforce) library.

You can install the TensorForce Python package by executing the following command:

```bash
pip install tensorforce
```

Read the [TensorForce tutorial](https://github.com/reinforceio/tensorforce/blob/master/examples/quickstart.py) for more detailed instructions on how to train, evaluate and test TensorForce.

## Option 13

You can train, evaluate and test the above methods from Python using the [torchbeast](https://github.com/deepmind/torchbeast) library.

You can install the TorchBeast Python package by executing the following command:

```bash
pip install torchbeast
```

Read the [TorchBeast tutorial](https://github.com/deepmind/torchbeast/blob/master/docs/tutorials.md) for more detailed instructions on how to train, evaluate and test TorchBeast.

## Option 14

You can train, evaluate and test the above methods from Python using the [torch-rl](https://github.com/lcswillems/torch-rl) library.

You can install the Torch-RL Python package by executing the following command:

```bash
pip install git+https://github.com/lcswillems/torch-rl.git
```

Read the [Torch-RL tutorial](https://github.com/lcswillems/torch-rl/blob/master/tutorial.md) for more detailed instructions on how to train, evaluate and test Torch-RL.

## Option 15

You can train, evaluate and test the above methods from Python using the [tensorrt](https://github.com/NVIDIA/TensorRT) library.

You can install the TensorRT Python package by executing the following command:

```bash
pip install tensorrt
```

Read the [TensorRT tutorial](https://docs.nvidia.com/deeplearning/tensorrt/api/python_api/inference.html) for more detailed instructions on how to train, evaluate and test TensorRT.

## Option 16

You can train, evaluate and test the above methods from Python using the [tensorpack](https://github.com/tensorpack/tensorpack) library.

You can install the TensorPack Python package by executing the following command:

```bash
pip install tensorpack
```

Read the [TensorPack tutorial](https://github.com/tensorpack/tensorpack/tree/master/examples) for more detailed instructions on how to train, evaluate and test TensorPack.

## Option 17

You can train, evaluate and test the above methods from Python using the [pytorch-playground](https://github.com/Tongpan/pytorch-playground) library.

You can install the PyTorch-Playground Python package by executing the following command:

```bash
pip install git+https://github.com/Tongpan/pytorch-playground.git
```

Read the [PyTorch-Playground tutorial](https://github.com/Tongpan/pytorch-playground/blob/master/tutorial.md) for more detailed instructions on how to train, evaluate and test PyTorch-Playground.

## Option 18

You can train, evaluate and test the above methods from Python using the [tensorflow-tutorials](https://github.com/pkmital/tensorflow_tutorials) library.

You can install the Tensorflow-Tutorials Python package by executing the following command:

```bash
pip install git+https://github.com/pkmital/tensorflow_tutorials.git
```

Read the [Tensorflow-Tutorials tutorial](https://github.com/pkmital/tensorflow_tutorials/blob/master/notebooks/01_beginner_tutorials/basic_graph.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-Tutorials.

## Option 19

You can train, evaluate and test the above methods from Python using the [rl-baselines-zoo](https://github.com/araffin/rl-baselines-zoo) library.

You can install the RL-Baselines-Zoo Python package by executing the following command:

```bash
pip install git+https://github.com/araffin/rl-baselines-zoo.git
```

Read the [RL-Baselines-Zoo tutorial](https://github.com/araffin/rl-baselines-zoo/blob/master/tutorials/01_getting_started.ipynb) for more detailed instructions on how to train, evaluate and test RL-Baselines-Zoo.

## Option 20

You can train, evaluate and test the above methods from Python using the [pytorch-qrnn](https://github.com/salesforce/pytorch-qrnn) library.

You can install the PyTorch-QRNN Python package by executing the following command:

```bash
pip install pytorch-qrnn
```

Read the [PyTorch-QRNN tutorial](https://github.com/salesforce/pytorch-qrnn/blob/master/tutorial.md) for more detailed instructions on how to train, evaluate and test PyTorch-QRNN.

## Option 21

You can train, evaluate and test the above methods from Python using the [tensorflow-addons](https://github.com/tensorflow/addons) library.

You can install the Tensorflow-Addons Python package by executing the following command:

```bash
pip install tensorflow-addons
```

Read the [Tensorflow-Addons tutorial](https://github.com/tensorflow/addons/blob/master/docs/tutorials/tensorflow_addons_basics.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-Addons.

## Option 22

You can train, evaluate and test the above methods from Python using the [tensorflow-datasets](https://github.com/tensorflow/datasets) library.

You can install the Tensorflow-Datasets Python package by executing the following command:

```bash
pip install tensorflow-datasets
```

Read the [Tensorflow-Datasets tutorial](https://github.com/tensorflow/datasets/tree/master/docs/tutorials) for more detailed instructions on how to train, evaluate and test Tensorflow-Datasets.

## Option 23

You can train, evaluate and test the above methods from Python using the [tensorflow-probability](https://github.com/tensorflow/probability) library.

You can install the Tensorflow-Probability Python package by executing the following command:

```bash
pip install tensorflow-probability
```

Read the [Tensorflow-Probability tutorial](https://github.com/tensorflow/probability/blob/master/tensorflow_probability/examples/jupyter_notebooks/Probabilistic_Layers_Regression.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-Probability.

## Option 24

You can train, evaluate and test the above methods from Python using the [tensorflow-graphics](https://github.com/tensorflow/graphics) library.

You can install the Tensorflow-Graphics Python package by executing the following command:

```bash
pip install tensorflow-graphics
```

Read the [Tensorflow-Graphics tutorial](https://github.com/tensorflow/graphics/blob/master/tensorflow_graphics/notebooks/basic.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-Graphics.

## Option 25

You can train, evaluate and test the above methods from Python using the [tensorflow-hub](https://github.com/tensorflow/hub) library.

You can install the Tensorflow-Hub Python package by executing the following command:

```bash
pip install tensorflow-hub
```

Read the [Tensorflow-Hub tutorial](https://github.com/tensorflow/hub/blob/master/examples/image_retraining/retrain.py) for more detailed instructions on how to train, evaluate and test Tensorflow-Hub.

## Option 26

You can train, evaluate and test the above methods from Python using the [tensorflow-io](https://github.com/tensorflow/io) library.

You can install the Tensorflow-IO Python package by executing the following command:

```bash
pip install tensorflow-io
```

Read the [Tensorflow-IO tutorial](https://github.com/tensorflow/io/blob/master/docs/tutorials/tensorflow_io.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-IO.

## Option 27

You can train, evaluate and test the above methods from Python using the [tensorflow-lattice](https://github.com/tensorflow/lattice) library.

You can install the Tensorflow-Lattice Python package by executing the following command:

```bash
pip install tensorflow-lattice
```

Read the [Tensorflow-Lattice tutorial](https://github.com/tensorflow/lattice/blob/master/tensorflow_lattice/demo/california_housing_hparams_search.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-Lattice.

## Option 28

You can train, evaluate and test the above methods from Python using the [tensorflow-text](https://github.com/tensorflow/text) library.

You can install the Tensorflow-Text Python package by executing the following command:

```bash
pip install tensorflow-text
```

Read the [Tensorflow-Text tutorial](https://github.com/tensorflow/text/blob/master/tensorflow_text/python/examples/embedding_sentence_features.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-Text.

## Option 29

You can train, evaluate and test the above methods from Python using the [tensorflow-transform](https://github.com/tensorflow/transform) library.

You can install the Tensorflow-Transform Python package by executing the following command:

```bash
pip install tensorflow-transform
```

Read the [Tensorflow-Transform tutorial](https://github.com/tensorflow/transform/blob/master/docs/examples.md) for more detailed instructions on how to train, evaluate and test Tensorflow-Transform.

## Option 30

You can train, evaluate and test the above methods from Python using the [tensorflow-gan](https://github.com/tensorflow/gan) library.

You can install the Tensorflow-GAN Python package by executing the following command:

```bash
pip install tensorflow-gan
```

Read the [Tensorflow-GAN tutorial](https://github.com/tensorflow/gan/blob/master/docs/tutorials/infogan.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-GAN.

## Option 31

You can train, evaluate and test the above methods from Python using the [tensorflow-federated](https://github.com/tensorflow/federated) library.

You can install the Tensorflow-Federated Python package by executing the following command:

```bash
pip install tensorflow-federated
```

Read the [Tensorflow-Federated tutorial](https://github.com/tensorflow/federated/blob/master/tensorflow_federated/python/research/simple_fedavg_cnn_mnist.py) for more detailed instructions on how to train, evaluate and test Tensorflow-Federated.

## Option 32

You can train, evaluate and test the above methods from Python using the [tensorflow-datasets](https://github.com/tensorflow/datasets) library.

You can install the Tensorflow-Datasets Python package by executing the following command:

```bash
pip install tensorflow-datasets
```

Read the [Tensorflow-Datasets tutorial](https://github.com/tensorflow/datasets/blob/master/tutorial.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-Datasets.

## Option 33

You can train, evaluate and test the above methods from Python using the [tensorflow-probability](https://github.com/tensorflow/probability) library.

You can install the Tensorflow-Probability Python package by executing the following command:

```bash
pip install tensorflow-probability
```

Read the [Tensorflow-Probability tutorial](https://github.com/tensorflow/probability/blob/master/tensorflow_probability/examples/jupyter_notebooks/Probabilistic_Layers_Regression.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-Probability.

## Option 34

You can train, evaluate and test the above methods from Python using the [tensorflow-addons](https://github.com/tensorflow/addons) library.

You can install the Tensorflow-Addons Python package by executing the following command:

```bash
pip install tensorflow-addons
```

Read the [Tensorflow-Addons tutorial](https://github.com/tensorflow/addons/blob/master/docs/tutorials/tensorflow_addons_basics.ipynb) for more detailed instructions on how to train, evaluate and test Tensorflow-Addons.

![TacNet Logo](https://raw.githubusercontent.com/ddrscott/tacnet/master/logo/tacnet_logo_small.png)

## Usage

Install the requirements:

```
pip install -r requirements.txt
```

Play against TacNet:

```
python play.py
```

or

```
python play.py -p tacnet
```

Train TacNet:

```
python train.py
```

Play against a trained TacNet:

```
python play.py -p tacnet -w <weights>
```

For example:

```
python play.py -p tacnet -w weights/tacnet_2018-08-01_02-01-13/weights.hdf5
```

## Notes

TacNet is designed to be run on GNU/Linux. It may work on other platforms, but this has not been tested.

## License

TacNet is licensed under the GNU General Public License v3.0.

It is able to learn from a sequence of moves as efficiently as a player would learn from that sequence alone.

This AI is designed to play tic-tac-toe against humans and can be trained with a human player.

The board state and action space is encoded as:
    num_squares = 9
    board_structure = [0, 1, 2, 3, 4, 5, 6, 7, 8]
    actions = {
        "board_structure": [board_structure],
        "num_squares": [num_squares],
        "win_state": [board_structure],
        "is_draw": [board_structure]
    }

This code is licensed, but can be adapted to use your own dataset and trained on your own dataset.

For more information see:
    https://tacnet.readthedocs.io/en/latest/tacnet.html
    https://github.com/nolte/tacnet


It can be trained on the classic and advanced tic-tac-toe game.

The board is represented by a sequence of 9 binary encoded features.

A feature 0 is added to the sequence at the end of a game and a feature 1 is added in the beginning of a game.

The expected input is a sequence of length 9.

For training, the input is fed to the RNN and the output of the RNN is a linear layer.

To play the game, the output of the network is read to predict the next input.

The game is ended if there is a draw or if there is an immediate winner.

A winner is defined as the player who has 3 in a row.

TacNet is a `TensorFlow` model.

The implementation is based on the following paper:

@techreport{klepka2015deep,
  title={Deep Tic-Tac-Toe Learner},
  author={Klepka, Tadeusz and Pluczny, Maciej},
  year={2015},
  institution={Politechnika Warszawska}
}

@inproceedings{klepka2015playing,
  title={Playing tic-tac-toe with a deep neural network},
  author={Klepka, Tadeusz and Pluczny, Maciej},
  year={2015},
  booktitle={Proceedings of the International Conference on Machine Learning},
  pages={21--32},
  organization={ACM}
}

@inproceedings{klepka2015playing2,
  title={Playing tic-tac-toe with a deep neural network},
  author={Klepka, Tadeusz and Pluczny, Maciej},
  year={2015},
  booktitle={Proceedings of the International Conference on Machine Learning},
  pages={33--44},
  organization={ACM}
}

The current version of TacNet can play a single round against an opponent.

*A single round is a sequence of 3 turns by the two players.
*For a single turn, a player may play an action or pass.
*Each turn is defined by three values:
  - the current player (an integer in the range [1, 2])
  - the current state of the board (a list of 9 integers in the range [1, 2])
  - the previous action of the opponent (an integer in the range [0, 3])

The board is a 3x3 matrix, with the cells numbered as follows:

1  2  3
4  5  6
7  8  9

The possible states are as follows.

1 : Empty cell
2 : X mark
3 : O mark

The parameters are as follows:

1. Learning rate (alpha)
2. Discount factor (gamma)
3. Neural network architecture (layers)
4. Optimizer (optimizer)

The current version of TacNet is published on GitHub: https://github.com/HaroSachan/TacNet

The algorithm can be found in a paper by Haro Sachan: https://arxiv.org/abs/1808.06993
]]

local m1 = {{0.0155074093490839, 0.006488165818154812, 0.00027295813197270036, -0.004079065751284361, 0.006884671747684479, 0.0042692008428275585, -0.009449069388210773, 0.0015732233878225088, -0.021253908053040504, -0.0015808097086846828, 0.02058781497180462, 0.014156852848827839, 0.0011407096171751618, 0.0026757230516523123, 0.10612194985151291, 0.0009431670187041163, -0.0068403566256165504, -2.948203109554015e-05, 0.014330151490867138, -0.010280955582857132, 0.00607257941737771, -0.007597192190587521, 0.010801678523421288, -0.4398154318332672, -0.011297371238470078, 0.0011513150529935956, 0.18024559319019318, -0.0026264109183102846, 0.2545485496520996, 0.005268874578177929, -0.22284995019435883, -0.0025158366188406944}, {-0.023265661671757698, -0.028199685737490654, 0.0019726157188415527, 0.002311018295586109, 0.0039128935895860195, -0.010445198975503445, -0.003814024617895484, 0.011082574725151062, 0.004536514636129141, -0.002593920798972249, 0.009368406608700752, 0.006720294710248709, 8.915900980355218e-05, 0.004164125304669142, 0.9897564053535461, 0.019276781007647514, -0.01060003973543644, -0.014678665436804295, -0.011010600253939629, -0.010455616749823093, -0.013942350633442402, 0.0043983678333461285, -0.002270421478897333, -2.5565435886383057, 0.005101943388581276, 0.005614021793007851, -0.07775966078042984, 0.004709700588136911, 1.195217490196228, -0.0025556227192282677, -0.011072738096117973, -0.003579859621822834}, {-0.0008956045494414866, 5.668430821970105e-05, -0.00031418423168361187, -0.01421699021011591, -0.008675345219671726, -0.0025554841849952936, 0.006331453565508127, -0.001803689869120717, -0.009321389719843864, -0.0074706594459712505, 0.01738801598548889, -0.008258081041276455, 0.013161792419850826, 0.01379439327865839, -0.041612766683101654, 0.010264900512993336, -0.008259480819106102, -0.005292787216603756, 0.013369467109441757, -0.02200317569077015, -0.0027856286615133286, -0.015384763479232788, 0.007832403294742107, 0.14879383146762848, 0.0022518739569932222, -0.0015280295629054308, 0.08750196546316147, -0.010823709890246391, -0.01591183803975582, -0.00044037486077286303, 0.024126602336764336, -0.013408261351287365}, {0.0006285661365836859, -0.011370187625288963, -0.014266930520534515, -0.016016153618693352, -0.005611760076135397, 0.02668766863644123, 0.0008600738365203142, 0.008757492527365685, -0.007828972302377224, -0.014785801991820335, 0.004311458673328161, 0.0020335528533905745, -0.015151389874517918, -0.01958364248275757, 0.8698611259460449, 0.0009036201518028975, 0.008898426778614521, 0.00037994986632838845, -0.0009526663925498724, -0.0005932285566814244, -0.021463865414261818, 0.0022706245072185993, -0.0007221369305625558, -0.28683385252952576, -0.004136236384510994, -0.008284393697977066, -0.0014376738108694553, 0.010943891480565071, -0.6084831357002258, -0.007891763933002949, 0.0010044678347185254, 0.010963504202663898}, {0.008662886917591095, 0.0016641751863062382, -0.004643036983907223, -0.002748311497271061, -0.004758197348564863, -0.007227372378110886, 0.0021537886932492256, -0.009522989392280579, 0.024872008711099625, -0.003176902187988162, -0.012698564678430557, 0.013463514856994152, -0.0038242435548454523, -0.013188377022743225, 0.12998083233833313, -0.001050448277965188, -0.0090628731995821, -0.005227814894169569, 0.013407512567937374, 0.003268970875069499, 0.00863635540008545, -0.0067290812730789185, -0.017506981268525124, 0.27209416031837463, -0.004974703304469585, 0.014676785096526146, 0.005096870474517345, -0.0023669705260545015, -0.09735122323036194, 0.00852327048778534, 0.029738785699009895, -0.00018516465206630528}, {0.0013851794647052884, 0.0075482577085494995, 0.0033859265968203545, -0.007900326512753963, 0.011987993493676186, 0.005661900620907545, 0.0027204789221286774, 0.0013737846165895462, -0.0040061213076114655, -0.01702849753201008, -0.024045083671808243, -0.001812430564314127, -0.007875361479818821, 0.0038145040161907673, 0.6438373327255249, -0.01725444197654724, 0.013205562718212605, -0.008944562636315823, 0.0036043482832610607, -0.013157244771718979, 0.00829711277037859, 0.006997830234467983, -0.0030326240230351686, -0.222147598862648, 0.006741775199770927, 0.004146956838667393, 0.14480897784233093, -0.014564628712832928, -0.35692766308784485, 0.004153877962380648, 0.08814334124326706, 0.002427420811727643}, {0.00910109467804432, -0.008598258718848228, 0.01343824714422226, 0.001383208204060793, -0.002559378044679761, 0.001373131526634097, 0.0014705450739711523, 0.0032749706879258156, 0.01015828549861908, 0.010105757042765617, 0.00574147142469883, 0.0006056641577742994, 0.01798497512936592, -0.015823135152459145, -0.31035223603248596, -0.002140038413926959, 0.016672104597091675, 0.011084435507655144, -0.0034442502073943615, 0.0036647820379585028, -0.0003174898447468877, 0.005208584014326334, 0.00717482203617692, -0.019229311496019363, -0.0026643441524356604, -0.0011569760972633958, -0.08449070900678635, -0.0014608472120016813, 0.45100903511047363, -0.011754516512155533, -0.13711124658584595, -0.002602459629997611}, {0.0023644124157726765, -0.004162526223808527, -0.010107123292982578, -0.006765483412891626, -0.0003944530326407403, 0.007642676588147879, -0.001435710582882166, 0.009934192523360252, -0.0040167272090911865, -0.007702975533902645, 0.002442356199026108, 0.00729410583153367, 0.0078088766895234585, 0.010027991607785225, 0.23553720116615295, 0.007433400955051184, -0.004391684662550688, -0.0022256416268646717, 0.00921260379254818, -0.01799924299120903, -0.005462481174618006, -0.0010499764466658235, -0.013332435861229897, 0.11314482986927032, 0.012240801937878132, 0.00824181642383337, 0.004330344032496214, 0.010675545781850815, -0.2455327957868576, -0.00013463516370393336, 0.12761424481868744, 0.007782581262290478}, {-0.011366243474185467, 0.0018246296094730496, -0.0022160231601446867, 0.0061631351709365845, -0.0035403594374656677, 0.005753170233219862, -0.019338183104991913, 0.007676220498979092, -0.0008359102648682892, 0.0007485417299903929, -0.00024197850143536925, -0.01142578013241291, 0.013969842344522476, -0.016157053411006927, -0.20114679634571075, 0.007503764238208532, 0.00981162954121828, -0.0030910917557775974, -0.013341110199689865, 0.015079936943948269, 0.020745517686009407, 0.016959166154265404, 0.008520898409187794, 0.6666644215583801, 0.00705732312053442, -0.00035834251320920885, -0.040182117372751236, -0.002583657158538699, 0.17494314908981323, 0.0010625030845403671, -0.059903163462877274, 0.005773482378572226}, {-0.007678381633013487, 0.002520439913496375, -0.004013577476143837, -0.01878165826201439, 0.012584494426846504, -0.0037011418025940657, -0.0033026309683918953, -0.009220370091497898, -0.026551881805062294, 0.01225411705672741, 0.018922442570328712, -0.005660835187882185, 0.0013797376304864883, -0.014362804591655731, -0.055424951016902924, 0.0013512697769328952, 0.008233402855694294, -0.012236997485160828, -0.007361895404756069, -2.8791342629119754e-05, -0.004312260542064905, 0.00871257670223713, 0.00483307521790266, -0.4072363078594208, 0.0003002468147315085, 0.017628004774451256, 0.23043157160282135, 0.01635270193219185, 0.09681729227304459, 0.004842303693294525, 0.00962389912456274, 0.0165789183229208}, {0.008463777601718903, 0.005835637915879488, 0.019334182143211365, 0.008868386968970299, 0.030270814895629883, -0.003703873371705413, 0.007132847793400288, -0.004389679059386253, -0.015160123817622662, -0.002356915967538953, 0.01377823669463396, 0.005026481579989195, -0.005125989206135273, 0.014332245104014874, 0.04954835772514343, 0.005082275718450546, 0.016706328839063644, -0.007861177437007427, -0.007798521779477596, 0.007554527372121811, 0.003882315941154957, 0.00274745374917984, -0.004454579669982195, 1.0273360013961792, 0.008192003704607487, 0.005677070468664169, -0.3240997791290283, -0.005624113138765097, -0.5029873251914978, 0.004352062940597534, 0.20909498631954193, 0.005946814082562923}, {0.007961279712617397, -0.002574071753770113, 0.00475683156400919, -0.009174681268632412, 0.0010233214125037193, 0.00551092391833663, -0.004898770246654749, 0.0037236118223518133, 7.516503046645084e-06, -0.0008922286797314882, 0.025253884494304657, 0.005892513319849968, -0.0047020697966217995, -0.007162168622016907, 0.32343247532844543, -3.781467239605263e-05, 0.0059041487984359264, -0.001953915925696492, 0.0019380724988877773, 0.02236943133175373, 0.017855066806077957, 0.01295614242553711, -0.0040580639615654945, 0.20835179090499878, -0.008371136151254177, -0.004913396202027798, -0.0005826794658787549, -0.017705054953694344, 0.18477511405944824, -0.0001414541620761156, -0.29335635900497437, 0.010106767527759075}, {0.0002344588137930259, 0.0025874092243611813, 0.009495864622294903, -0.021352164447307587, -0.024142401292920113, 0.007893292233347893, 0.0026527801528573036, 0.0013875330332666636, 0.011581790633499622, 0.0004448537074495107, -0.0016932431608438492, -0.0028149879071861506, -0.0038879599887877703, -0.01954953372478485, 0.04286988824605942, 0.00953431986272335, -0.0017635500989854336, 0.006374882999807596, 0.001639889320358634, -0.00917183980345726, -0.005705657880753279, 0.018049191683530807, 0.020710719749331474, -0.2560116648674011, 0.009193428792059422, -0.0027770977467298508, -0.020678028464317322, -0.0023401372600346804, 0.3231610953807831, 0.0018426543101668358, 0.02713758684694767, -0.007318727672100067}, {0.00999587494879961, -0.004326492082327604, -0.005933282431215048, 0.006787762511521578, -0.0010744559112936258, -0.005218882579356432, 0.010040942579507828, -0.004779490176588297, 0.0034480791073292494, 0.0003073903208132833, -0.006072813645005226, 0.011277906596660614, -4.2437783122295514e-05, -0.012230828404426575, 2.2358367443084717, 0.018093623220920563, 0.009005763567984104, 0.01569017954170704, 0.009916383773088455, -0.003629440674558282, -0.002423341851681471, 0.017028452828526497, 0.0013429394457489252, -1.745781421661377, 0.005478706210851669, 0.014355435967445374, 0.17193350195884705, -0.0012465441832318902, -0.5434186458587646, -0.005541139282286167, -0.3876783847808838, 0.006193301174789667}, {-0.011838851496577263, -0.0014600695576518774, -0.004608154296875, 0.004301847890019417, 0.013464231044054031, -0.0052960943430662155, -0.0032688837964087725, -0.005021826364099979, 0.02697138674557209, -0.0054878913797438145, 0.005354130640625954, -0.0008739845361560583, 0.002267936011776328, -0.0059895263984799385, 1.3956201076507568, 0.013601827435195446, -0.0031355363316833973, -0.01821182481944561, 0.007091544568538666, 0.00037203714600764215, -0.001940289861522615, -0.006597144994884729, 0.0050049894489347935, -1.0322291851043701, 0.014268926344811916, -0.009316129609942436, 0.11792915314435959, -0.0006313536432571709, -0.25273841619491577, -0.028244702145457268, -0.25464674830436707, -0.008866600692272186}, {0.006295053754001856, 0.015551751479506493, 0.008335161954164505, 0.0024206601083278656, -0.011420453898608685, 0.012042788788676262, 0.005882774014025927, 0.0038776351138949394, -0.0015964305493980646, -0.00846458412706852, 0.01727195270359516, 0.00712951272726059, 0.015257446095347404, -0.003497837111353874, 0.5673947930335999, -0.0012727105058729649, -0.0002802102826535702, -0.01785474270582199, -0.01954907737672329, -0.007803395856171846, -0.007196509279310703, -0.011492637917399406, -0.01220529805868864, -0.02263224497437477, -0.0002206426433986053, -0.0004323915345594287, -0.5113746523857117, 0.012961449101567268, -0.03973303735256195, -0.00018462285515852273, 0.22049693763256073, -0.010228252969682217}, {-0.022760629653930664, -0.004683106206357479, 0.00023272473481483757, 0.0014962187269702554, -0.009687814861536026, -0.004751282744109631, 0.008663992397487164, -0.004671190865337849, -0.00027339966618455946, 0.0001737860729917884, 0.00949001219123602, -0.009102960117161274, 0.0028603901155292988, -0.013804564252495766, 0.12300854176282883, 0.02123611606657505, 0.008207395672798157, 0.0062142484821379185, -0.017146123573184013, -0.023124074563384056, 0.013865934684872627, 0.009486544877290726, 0.015979226678609848, -0.6912488341331482, 0.0033945380710065365, 0.015426114201545715, -0.041613370180130005, -0.0008396520279347897, 0.29853543639183044, 0.01688922941684723, -0.20449283719062805, -0.005967861972749233}, {-0.002992942463606596, 0.007640726864337921, -0.011330755427479744, -0.0019663837738335133, -0.0196280125528574, 0.006057250313460827, -0.011782298795878887, -0.013972729444503784, -0.0031226202845573425, -0.005345412530004978, -0.005861162208020687, -0.0017062772531062365, 0.01449880562722683, -0.007051106542348862, 0.2775218188762665, -0.017276080325245857, -0.016753733158111572, -0.01146726030856371, -0.016536392271518707, -0.021406197920441628, 0.008091616444289684, 0.018006308004260063, -0.002667082007974386, -0.4650105834007263, 0.0030770229641348124, -0.014944464899599552, 0.005430223420262337, 0.013998590409755707, 0.18908405303955078, -0.002684327308088541, 0.14591224491596222, 0.011360613629221916}, {0.007309080101549625, 0.02003169059753418, -0.007227093912661076, 0.004758843686431646, -0.011289023794233799, 0.004746735095977783, 0.006925847381353378, -0.00031600831425748765, 0.0005477790837176144, 0.004413597751408815, -0.014275522902607918, -0.01456927414983511, -0.012411847710609436, -0.01958138681948185, -0.2060476392507553, -0.0010495326714590192, -0.01403555367141962, -0.0028164435643702745, 0.025137322023510933, 0.0013672183267772198, 0.009440260007977486, -0.002848238917067647, 0.00465045589953661, -0.5837608575820923, 0.019277794286608696, 0.0012692173477262259, -0.15042655169963837, 0.0030940049327909946, 0.8414435982704163, -0.0038456120528280735, -0.18147273361682892, 0.0030172744300216436}, {0.021595794707536697, 0.006755348294973373, 0.00033285722020082176, 0.014256930910050869, 0.006475409492850304, -0.00768862571567297, 0.009614584036171436, 0.0029460384976118803, 0.004349169787019491, 0.0018986242357641459, 0.003726147348061204, 0.0002773219021037221, 0.00156972196418792, 0.011213733814656734, 0.19019386172294617, -0.003840723540633917, 0.012743695639073849, -0.013779637403786182, -0.004551803693175316, 0.007531711831688881, 0.0002619501610752195, 0.012804988771677017, -0.004039091523736715, 0.018934743478894234, -0.00980527326464653, -0.005556164775043726, 0.056682039052248, 0.0034354692324995995, -0.27922576665878296, 0.01434984989464283, 0.17072038352489471, -0.014937011525034904}, {-0.0004250899364706129, 0.020555192604660988, 0.006900623440742493, -0.0004815379506908357, 0.012397639453411102, -0.022670798003673553, -0.01783890277147293, 0.008039688691496849, -0.019612282514572144, 0.009405957534909248, -0.021159417927265167, 0.003996067680418491, -0.004415162838995457, -0.008351555094122887, 0.11765477061271667, -0.01510944776237011, -0.0032197516411542892, 0.005010230932384729, -0.005155005492269993, 0.006272091530263424, -0.011793545447289944, -0.0006023283931426704, -0.00838419795036316, 0.07389222830533981, -0.001242431695573032, 0.001168850576505065, -0.20375217497348785, 0.0203931275755167, 0.032299429178237915, 0.007556915748864412, -0.0251461211591959, -0.01360306702554226}, {0.01720031537115574, 0.017595967277884483, -0.023394400253891945, -0.0024268226698040962, 0.013579871505498886, 0.008203074336051941, -0.010327765718102455, -0.006758249364793301, 0.015322613529860973, 0.00457569258287549, 0.005823150742799044, 0.0041708434000611305, -0.010186250321567059, 6.722846592310816e-05, 2.422327756881714, -0.00537190493196249, -0.00594719871878624, 0.001047942670993507, -0.0035609938204288483, -0.010251734405755997, 0.0001624504802748561, 0.0029074170161038637, 0.02916661463677883, -2.3680927753448486, 0.005345210898667574, -0.0018212622962892056, -0.5605747699737549, 0.0064812577329576015, 0.48748913407325745, -0.005252801813185215, 0.5184818506240845, -0.0035550601314753294}, {-0.00350739317946136, 0.0028106244280934334, 0.016154631972312927, 0.012970928102731705, -9.665269317338243e-05, 0.006430949550122023, 0.0015391813358291984, 0.003283039666712284, 0.028646882623434067, 0.02120162732899189, -0.007803512737154961, -0.0006225521210581064, 0.005522683262825012, -0.0029032528400421143, 2.1437668800354004, -0.012594827450811863, -0.005891080480068922, -0.008683832362294197, 0.008454344235360622, 0.0008953133947215974, 0.012537669390439987, 0.004333336837589741, 0.007742403540760279, -0.8338155150413513, 0.0034864547196775675, 0.004580571781843901, -0.02134294994175434, -0.0042412010952830315, -1.041390299797058, 0.002848125994205475, -0.1439482569694519, -0.009627903811633587}, {0.010137836448848248, -0.0002164143224945292, -0.01380458939820528, -0.00025968498084694147, 0.01061310525983572, -0.002975899027660489, 0.0016238931566476822, -0.003277283161878586, -0.0023763622157275677, 0.0013641455443575978, -0.009027227759361267, -0.008124607615172863, -0.0035706430207937956, 0.0024337281938642263, 0.29575595259666443, 0.013523370027542114, 0.009316027164459229, 0.007083297241479158, 0.015968330204486847, -0.0030933781526982784, 0.015304774045944214, 0.02084755338728428, -0.005831829737871885, -0.1477351188659668, -0.0023298615124076605, 0.0017819888889789581, 0.04277261346578598, 0.022653717547655106, 0.1122182086110115, -0.013171616941690445, 0.0047762454487383366, 0.01781332865357399}, {0.0059210192412137985, 0.0039638313464820385, -0.002468657214194536, 0.010272827930748463, 0.00925086997449398, -0.002116486197337508, -0.007040772121399641, 0.002964052138850093, 0.01627231389284134, -0.011032355017960072, -0.009207062423229218, 0.00037024987977929413, -0.020575417205691338, 0.01575477607548237, -0.23132358491420746, -0.006453773006796837, -0.003471862990409136, -0.005602561868727207, -0.009721090085804462, -0.0003853075031656772, 0.005798095837235451, 0.017872406169772148, -0.011227503418922424, -0.03552557900547981, -0.0018877786351367831, -0.0039934623055160046, -0.07391141355037689, -0.012165524996817112, -0.29988542199134827, -0.014030699618160725, 0.2632637321949005, 0.0030967979691922665}, {-0.010359683074057102, 0.008645826950669289, 0.005462129600346088, -0.0017050117021426558, -0.010269451886415482, -0.008230799809098244, 0.0027327085845172405, 0.0011999324196949601, -0.009228027425706387, 0.015352505259215832, 0.008644680492579937, 0.003235778072848916, -0.029569154605269432, -0.0026577117387205362, -0.09923073649406433, 0.001781285973265767, 0.012274907901883125, 0.005337791983038187, -0.0075800674967467785, -0.009259702637791634, -0.0037415537517517805, -0.029852144420146942, 0.0006089151720516384, -0.22381828725337982, 0.006609015166759491, -0.008426161482930183, -0.19403594732284546, 0.013292504474520683, 0.2958134114742279, -0.014622041024267673, 0.2764447033405304, -0.0004318822466302663}, {0.005162656307220459, 0.0038524046540260315, -0.018908431753516197, 0.00815515872091055, 0.0076013486832380295, -0.002819803776219487, 0.0007749981014057994, -0.0023178979754447937, 0.004365979693830013, -0.001526201143860817, -0.010979930870234966, 0.019656872376799583, -0.034296564757823944, -0.012073432095348835, 0.7911325693130493, -0.014717507176101208, -0.013003758154809475, 0.010652810335159302, -0.006323471665382385, 0.018213078379631042, 0.0028590690344572067, -0.007349267601966858, -0.002440360374748707, -1.1037330627441406, -0.005201247986406088, 0.008841908536851406, -0.8246519565582275, -0.0005337618640623987, 0.5403968095779419, -0.007494116201996803, 0.7339386940002441, -0.0009258344070985913}, {-0.007001653779298067, -0.0029639985878020525, 0.003187933238223195, 0.00856404472142458, -0.006656302604824305, 0.011751877143979073, -0.011819296516478062, 0.0013858400052413344, 8.016217179829255e-05, -0.0018162505002692342, -0.017235318198800087, -0.010504263453185558, -0.011256792582571507, -0.01854492723941803, 0.5100098252296448, 0.00857385154813528, -0.01088624820113182, 0.00113320117816329, -0.003322965232655406, 0.011355170980095863, 0.012501568533480167, -0.0006142749334685504, 0.010679394006729126, -0.24822533130645752, -0.013326587155461311, -0.004500436130911112, -0.010036827996373177, 0.005451040342450142, -0.2595650255680084, -0.0079718716442585, 0.04016327112913132, 0.020109569653868675}, {0.006614176090806723, -0.0003985618823207915, 0.005110369063913822, 0.007090490777045488, 0.01140976045280695, -0.003928148187696934, 0.009614460170269012, -0.0048528932966291904, -0.014673779718577862, 0.008452821522951126, 0.011034606024622917, 0.0055699762888252735, -0.0035388621035963297, 0.006536735687404871, 0.2735954523086548, -0.014832736924290657, 0.01922849752008915, 0.006414779461920261, 0.011040978133678436, 0.006147244945168495, -1.3859130376658868e-05, -0.015357669442892075, -0.00857450533658266, 0.3382336497306824, 0.006084175780415535, -0.004567731637507677, 0.01761762611567974, -0.008517568930983543, -0.23889747262001038, 0.0035906813573092222, -0.059288542717695236, 0.009438827633857727}, {-0.006240782793611288, 0.006520673632621765, -0.011046296916902065, -0.007184114772826433, -0.004050386603921652, -0.01534113846719265, 0.005468952469527721, -0.005159454885870218, -0.007752264384180307, -0.00102034246083349, -0.007896733470261097, 0.001336373621597886, 0.002179165603592992, 0.0037392477970570326, 0.30625832080841064, -0.007515127304941416, -0.0032487427815794945, 0.0009368255268782377, 0.0007011882844381034, -0.004245438612997532, -0.014757059514522552, -0.000808987591881305, 0.0014807824045419693, 0.5305205583572388, -0.01118148397654295, 0.009553276933729649, -0.1147003248333931, -0.015234882943332195, -0.22546260058879852, 0.0140473498031497, -0.029370222240686417, -0.0014925332507118583}, {0.008892392739653587, 0.0058849286288022995, 0.016086775809526443, -0.00788002647459507, 0.01675669476389885, 0.005864041391760111, -0.005471899174153805, -0.006908921990543604, 0.002914976794272661, 0.004107952583581209, 0.013766258955001831, 0.005361507646739483, 0.000983613426797092, 0.006700466852635145, 0.10473813116550446, 0.0020034518092870712, -0.012917661108076572, -0.007167542353272438, 0.0006538002053275704, -0.0010931834112852812, -0.007915395312011242, 0.0007228231406770647, 0.002101707039400935, -0.5960894227027893, 0.0066304258070886135, 0.014326727017760277, 0.4352636933326721, -0.0111786974593997, 0.22216002643108368, 0.00633050175383687, -0.35208845138549805, 0.03122672811150551}, {0.0173582062125206, -0.016059082001447678, -0.01019311510026455, -0.004172830376774073, 0.0004863333306275308, 0.00564173748716712, 0.003949705045670271, -0.006820242386311293, 0.0028518852777779102, 0.01576649583876133, 0.004323855973780155, -0.012238276191055775, -0.0042002834379673, -0.0011740436311811209, 0.8920463919639587, -0.006746762897819281, -0.001338079571723938, -0.0008473091875202954, 0.00038101791869848967, 0.002132843481376767, -0.0022256511729210615, -0.012223639525473118, -0.015994152054190636, -0.5304014682769775, -0.007734508253633976, 0.0006795509834773839, -0.5735889077186584, -0.01317320205271244, -0.06162974238395691, 0.0012352400226518512, 0.27667689323425293, -0.008798453956842422}, {0.0033794555347412825, -0.005683994386345148, -0.0032179018016904593, 0.008215335197746754, 0.011074275709688663, 0.003851932007819414, -0.0006066964706405997, -0.007915233261883259, 0.012808844447135925, -0.0004890863783657551, 0.002982101868838072, 0.0020291528198868036, -0.0031272817868739367, -0.003961733542382717, -0.1229405552148819, -0.0008771060383878648, 0.003939097747206688, 0.0009215877507813275, -0.0004285993636585772, 0.008563905023038387, -0.0110769746825099, 0.004088870249688625, -0.02231914922595024, 1.06755530834198, -0.015936139971017838, 0.006211400963366032, -0.40952378511428833, -0.015173296444118023, -0.5746299028396606, 0.009706497192382812, 0.3354443907737732, -0.014239845797419548}, {-0.008780053816735744, 0.008039740845561028, 0.006555104162544012, -0.014294088818132877, 0.012589702382683754, 0.008166513405740261, -0.008018292486667633, 0.0007273680530488491, -0.010220068506896496, -0.009000743739306927, 0.012910781428217888, -0.010541696101427078, -0.0003876472474075854, 0.0002541446883697063, 0.4868677258491516, -0.02082294225692749, 0.018607523292303085, 0.018823210150003433, 0.00010715555981732905, 0.012373347766697407, 0.0031854661647230387, 0.0009744242415763438, 0.01507482584565878, -0.6108127236366272, 0.006947392597794533, 0.01692979224026203, 0.11310839653015137, -0.0026719069574028254, -0.29061466455459595, 0.0015633560251444578, -0.12157130241394043, 0.017271509394049644}, {-0.01586422137916088, -0.0034744220320135355, 0.0030524744652211666, -0.012168961577117443, -0.007069684565067291, -0.0028405359480530024, -0.004513164050877094, -0.004312749952077866, -0.0009805629961192608, -0.02407277375459671, -0.00895270798355341, -0.025859644636511803, 9.92574859992601e-05, -0.003607373684644699, 0.2850044071674347, 0.00981427077203989, 0.0042574405670166016, -0.00906427577137947, 0.006644279696047306, -0.002597900340333581, -0.007120296824723482, 0.020599395036697388, -0.005730542354285717, 0.04532996937632561, -0.02033020369708538, 0.004505982622504234, -0.9248276352882385, 0.011849433183670044, 0.5616374611854553, 0.0027719195932149887, 0.7208003401756287, 0.003946095705032349}, {0.00297547597438097, 0.0046413494274020195, 0.005545536056160927, -0.010619503445923328, 0.013323492370545864, -0.009556121192872524, -0.008641518652439117, 0.008254984393715858, -0.006743925157934427, 0.0060625323094427586, -0.014092615805566311, -0.0037866015918552876, -0.016217147931456566, -0.004164053592830896, 0.17801235616207123, 0.016106413677334785, 0.005785125773400068, -0.0037925168871879578, -0.015046726912260056, -0.004622855689376593, 0.0020136504899710417, 0.007647139485925436, -0.0023625269532203674, -0.16201135516166687, -0.0054625594057142735, -0.0036936532706022263, -0.10806986689567566, 0.010460564866662025, 0.120651476085186, -0.01747700572013855, 0.15308551490306854, -0.007904927246272564}, {-0.0030873108189553022, -0.007039747200906277, 0.01218599732965231, 0.0178381334990263, 0.0069063762202858925, -0.013897763565182686, 0.010092929005622864, 0.0015568158123642206, 0.004759186878800392, -0.0208585187792778, 0.022480912506580353, 0.014490448869764805, 0.009759310632944107, 0.009754403494298458, 0.9622986912727356, 0.004031396005302668, -0.01898365281522274, -0.0018159489845857024, 0.022627662867307663, 0.008225568570196629, -0.020284738391637802, 7.059131894493476e-05, -0.012563937343657017, -1.2337385416030884, 0.009144807234406471, -0.017236746847629547, -0.20978380739688873, -0.008039156906306744, 0.2330615371465683, -0.015282263047993183, 0.16654664278030396, -0.004177235998213291}, {-0.012868708930909634, 0.006628962699323893, -0.009018054232001305, 0.0004572176549118012, -0.013645769096910954, 0.008651990443468094, -0.0031672546174377203, 0.012945332564413548, -0.011295433156192303, 0.003849491709843278, -0.0040724496357142925, -0.008456755429506302, -0.0036356402561068535, 0.010718976147472858, 0.08901102840900421, -0.018486768007278442, -0.003663771552965045, 0.002014784375205636, 0.01753438264131546, 0.00397184444591403, 0.0140986368060112, 0.007685468532145023, 0.011802918277680874, 0.6926411986351013, -0.02336411364376545, 0.011189805343747139, -0.34039464592933655, 0.018170475959777832, 0.0040993355214595795, -0.00019540679932106286, 0.10230708122253418, 0.001245888532139361}, {0.008461126126348972, 0.0038956445641815662, 0.0018692887388169765, 0.0014304430224001408, 0.002436253475025296, 0.010677268728613853, -0.00100515887606889, -0.014335714280605316, -0.006795018911361694, 0.005881910677999258, -0.0051907566376030445, -0.0005231950199231505, -0.014138308353722095, 0.006314144469797611, -0.318935751914978, 0.004221760202199221, 0.008559792302548885, 0.004712349269539118, -0.018020154908299446, 0.00146305316593498, -0.004989307373762131, 0.00903232116252184, 0.006381976418197155, 0.8654659986495972, -0.001794707728549838, -0.007051557302474976, -0.3227330148220062, 0.004027347080409527, -0.2958993911743164, -0.0038505839183926582, 0.34709370136260986, 0.0036492743529379368}, {-0.006846962962299585, 0.0029628355987370014, 0.006698574870824814, 0.011709448881447315, 0.021204445511102676, 0.002057144185528159, -0.0015197537140920758, -0.011825071647763252, -0.0026321709156036377, -0.015054442919790745, -0.0015582034830003977, -0.0025954130105674267, 0.014344828203320503, 0.012855570763349533, 0.1647699475288391, 0.005597682669758797, 0.0003189501876477152, 0.0039534661918878555, -0.0007317602285183966, 0.00734653091058135, 0.0026104485150426626, 0.00580316036939621, 0.00045697393943555653, 0.4583854675292969, -0.004042318556457758, 0.005431761499494314, 0.04121946915984154, -0.020807020366191864, -0.25554168224334717, 0.0061490521766245365, -0.15077759325504303, -0.011594502255320549}, {-0.010718814097344875, 0.006459695752710104, -0.005154430866241455, -0.0008595313411206007, -0.012273606844246387, -0.0006207322585396469, -0.006138981319963932, 0.007468762341886759, 0.010954896919429302, -0.001873114495538175, 0.01425949577242136, -0.00903376005589962, -0.0011547993635758758, 0.009651616215705872, -0.16399818658828735, -0.0031447734218090773, 0.015825023874640465, 0.0043020183220505714, 0.009213349781930447, -0.014580613002181053, 0.013059481978416443, -0.015092502348124981, -0.0027982702013105154, 0.13211427628993988, -0.010161450132727623, 0.007455398794263601, -0.07916515320539474, 0.005949478130787611, 0.03373967111110687, -0.0018358755623921752, 0.07131317257881165, -0.010439868085086346}, {-0.0017300840700045228, -0.005456352606415749, 0.005344930104911327, -0.003861891571432352, -0.010812627151608467, 0.0013852727133780718, 0.0009397559915669262, -0.006243269890546799, -0.010989510454237461, -0.0010937363840639591, 0.009593739174306393, -0.0009616070310585201, 0.0048910388723015785, -0.0014861068921163678, 0.0833776593208313, 0.009172380901873112, -0.013469957746565342, -0.011567402631044388, -0.0034251296892762184, -0.012562820687890053, 0.008968330919742584, 0.012423557229340076, 0.0025523826479911804, -0.18154186010360718, -0.0035861199721693993, -0.01687934435904026, 0.02780507691204548, -0.006289830431342125, 0.2898888885974884, -0.00031025547650642693, -0.009473610669374466, 0.00884348526597023}, {0.0153956962749362, 0.003045641351491213, 0.018017390742897987, 0.03377203270792961, -0.009923610836267471, 0.006952296476811171, -0.003027435392141342, 0.005469093099236488, -0.0007845655782148242, 0.002251051366329193, 0.00753064127638936, -0.007106596603989601, -0.000705783546436578, -0.0009070360683836043, -0.5108087658882141, -0.013427072204649448, 0.008119869977235794, 0.0032966025173664093, -0.008180543780326843, 0.003269396722316742, -0.017021391540765762, -0.0014012850588187575, -0.014967391267418861, -0.7015854120254517, -0.00629345141351223, 9.776089427759871e-05, -0.08117961883544922, 0.0051593598909676075, 1.023720145225525, 0.006400030571967363, -0.07157018035650253, 0.006776622962206602}, {0.011734962463378906, 0.0058901370503008366, -0.006743274163454771, -0.003598144045099616, -0.00035111166653223336, 0.0041435188613832, 0.011784900911152363, -0.00627515884116292, 0.0006721486570313573, -0.012778899632394314, -0.0034352061338722706, -0.011382090859115124, -0.014941535890102386, 0.007554147858172655, 1.2933003902435303, -0.008422618731856346, 0.00018076419655699283, 0.009881652891635895, 0.005079108756035566, -0.01532857958227396, -0.0015293235192075372, -0.03261828422546387, -0.0065913419239223, -1.6234227418899536, -0.010348673909902573, 0.006259975954890251, 0.12973617017269135, -0.015141244977712631, 0.08701419085264206, 0.00024833864881657064, -0.2960708737373352, -0.004375736694782972}, {-0.005560705903917551, -0.00504433736205101, 0.004393595736473799, 0.011787895113229752, -0.009017761796712875, -0.00040022734901867807, 0.002957200398668647, -0.001993598183616996, -0.004533910192549229, 0.011395782232284546, 0.0066032991744577885, 0.007257696241140366, 0.002694334601983428, 0.0053198812529444695, 1.3774529695510864, 0.003496281336992979, -0.006101998966187239, 0.008389756083488464, 0.007327154278755188, -0.01713748089969158, -0.006129145622253418, -0.008181999437510967, 0.000238084772718139, -1.8722048997879028, 0.004043621476739645, -0.02197415754199028, -0.1731913983821869, 0.004444460850208998, 0.326384037733078, 0.006305214483290911, 0.21745696663856506, -0.004557858221232891}, {-0.0028835188131779432, -0.0027241052594035864, 0.005281674675643444, 0.0073751225136220455, 0.003353394102305174, -0.0023297080770134926, 0.001170873292721808, -0.012430661357939243, 0.008188405074179173, 0.0023525008000433445, 0.008020911365747452, 0.0027977696154266596, -0.0005270705441944301, 0.009270629845559597, 0.21976524591445923, 0.01337168924510479, -0.020600873976945877, 0.020252613350749016, 0.0038679279386997223, -0.006875889841467142, -0.011776589788496494, 0.013874881900846958, 0.0023955306969583035, 0.19433395564556122, 0.01061862800270319, 0.0021969075314700603, 0.028877703472971916, 0.006310808006674051, -0.024622276425361633, -0.020778171718120575, -0.015500501729547977, -0.0022517500910907984}, {0.004413754679262638, -0.010314388200640678, 0.007509896531701088, 0.009477331303060055, 0.0014978309627622366, 0.0014237610157579184, -0.002601437270641327, 0.011967186816036701, -0.00878132227808237, 0.005793747957795858, 0.01041154470294714, 0.004061668645590544, 0.005833055358380079, -0.0036397224757820368, 0.04227959364652634, -0.00020310244872234762, -0.0017184014432132244, 0.01483656745404005, -0.005605859681963921, 0.009112900123000145, -0.02270951308310032, 0.004150915890932083, -0.0013729118509218097, -0.39651960134506226, 0.020499709993600845, 0.001163111417554319, 0.028550010174512863, 0.0045937662944197655, 0.03312791511416435, -0.005467553157359362, 0.030246898531913757, -0.003007565625011921}, {0.014319347217679024, 0.003108490724116564, 0.0042740656062960625, 0.019261784851551056, -0.007011936977505684, 0.0023930922616273165, -0.0035123915877193213, -0.01015328150242567, -0.011781366541981697, -0.0032002585940063, -0.006941428408026695, 0.006474744528532028, -0.00498548150062561, 0.009368191473186016, -0.8636953234672546, 0.004942610859870911, -0.00413477374240756, -0.029770120978355408, 0.004046228248625994, 0.010175751522183418, 0.010445651598274708, -0.006689436733722687, 0.02259686030447483, 1.15212082862854, -0.009847714565694332, 0.0023805408272892237, 0.04233204573392868, 0.006150438450276852, -0.3825833797454834, -0.00248582661151886, 0.0942077711224556, -0.004856327548623085}, {0.000377402117010206, 0.015206708572804928, -0.01684383489191532, 0.0098119480535388, -0.002793008927255869, -0.007868927903473377, 0.0029065983835607767, 0.016479628160595894, -0.013595404103398323, -0.008703724481165409, -0.006655496079474688, -0.015741216018795967, 0.006104935891926289, -0.02398134395480156, 1.4204118251800537, -0.005821539554744959, 0.0032279740553349257, 0.0009140830370597541, 0.010563774965703487, -0.003189684124663472, 0.0036000339314341545, 0.0003147487295791507, -0.0026019266806542873, -1.0659358501434326, 0.012572629377245903, -0.022716552019119263, 0.094023197889328, -0.008598028682172298, -0.7785492539405823, 0.0029815330635756254, -0.11093975603580475, -0.010490098968148232}, {0.022413674741983414, -0.00018257444025948644, 0.003451781114563346, 0.004607020411640406, -9.550163667881861e-06, 0.0015477476408705115, -0.009842857718467712, -0.001994015881791711, -0.010535942390561104, 0.00921684317290783, 0.0018471635412424803, 0.009137516841292381, 0.008808061480522156, 0.004947697278112173, 0.5903924703598022, 0.004113267175853252, -0.004494170658290386, 0.0030600184109061956, -0.009147217497229576, 0.00151265156455338, -0.002908028895035386, -0.009530724957585335, -0.003634287742897868, -1.520931601524353, -0.005452077370136976, -0.008647685870528221, -0.2413868010044098, 0.011954373680055141, 0.9784976840019226, 0.00321060954593122, 0.20798730850219727, 0.006019740365445614}, {-0.00028955715242773294, -0.004998998716473579, -0.0047384463250637054, 0.009268560446798801, -0.019637243822216988, -0.0032789232209324837, 0.004871797282248735, 0.006337468512356281, 0.00626107444986701, -0.007949510589241982, -0.004659641068428755, 0.00022334170353133231, 0.01179510448127985, 0.0037437009159475565, 0.27122172713279724, 0.003012745874002576, -0.0011597657576203346, -0.010833702981472015, 0.0015411534113809466, 0.011690356768667698, 0.005023285746574402, 0.011056666262447834, 0.021284684538841248, 0.5557917356491089, -0.00645210687071085, -0.02245199680328369, 0.021919574588537216, -0.014527675695717335, -1.0116419792175293, 0.005725824274122715, 0.23041608929634094, -0.01869591325521469}, {0.0037888747174292803, 0.00023524041171185672, 0.02261774241924286, -0.0024982253089547157, 0.008180863223969936, 0.0029447609558701515, 0.002110699424520135, -0.013803699053823948, 0.004792613442987204, 0.011219077743589878, 0.008885455317795277, 0.003931349143385887, -0.008137065917253494, 0.008572239428758621, 0.005839966703206301, -0.003558106254786253, -0.0015135749708861113, 0.004119107499718666, -0.007498005870729685, -0.0006587299285456538, 0.005290151108056307, 0.01106665376573801, -0.013219449669122696, -0.08735901117324829, 0.0022746191825717688, 0.0036210196558386087, -0.45510080456733704, -0.01454657968133688, 0.20719970762729645, 0.0012548703234642744, 0.14513877034187317, 0.0033796492498368025}, {0.0015982782933861017, 0.0037036268040537834, 0.010695419274270535, -0.008662442676723003, 0.010339277796447277, -0.006490954197943211, 0.0009194416925311089, -0.0041922349482774734, -0.018605556339025497, 0.007425906602293253, -0.0035581474658101797, 0.015132023952901363, 0.0035097452346235514, 0.010277511551976204, -0.30054035782814026, 0.0020664541516453028, 0.005522075574845076, 0.00837700068950653, -0.0016751822549849749, 0.013250148855149746, 0.009434542618691921, -0.017060359939932823, 0.008893693797290325, -0.4694260060787201, -0.005090749356895685, -0.0074449097737669945, 0.05481927469372749, -0.0076225255616009235, 0.2891460955142975, 0.0004510802391450852, -0.018694190308451653, 0.006721340119838715}, {-0.0019400424789637327, 0.00666714645922184, 0.0075624375604093075, 0.006634674035012722, -0.007485761307179928, 0.007035349495708942, 0.01825571060180664, 0.004636073485016823, 0.00413728691637516, -0.0006979085155762732, -0.0030804569832980633, 0.006873710546642542, 0.014589806087315083, -0.010647255927324295, 0.4902038872241974, -0.01172525342553854, 0.0017911377362906933, 0.00793724600225687, 0.007483789697289467, 0.011834397912025452, -0.0023873643949627876, 0.015295608900487423, 0.0015650660498067737, -0.12149312347173691, 0.006144994404166937, 0.0056977239437401295, -0.13775306940078735, 0.00692197447642684, -0.34338825941085815, -0.008654027245938778, -0.008851024322211742, -0.0037415551487356424}, {0.0034707679878920317, -0.018031813204288483, -0.014898540452122688, -0.003179261926561594, -0.0022433411795645952, -0.005922497250139713, 0.020223842933773994, -0.021807236596941948, -0.013690214604139328, 0.015635818243026733, 0.008591432124376297, 0.010000980459153652, -0.005600565578788519, 0.006912258919328451, 0.8864721655845642, -0.0023647313937544823, -0.016260091215372086, -0.0032681599259376526, 0.010858542285859585, 0.012707047164440155, -0.006528387311846018, -0.001002654666081071, -0.011482064612209797, -0.7828675508499146, -0.0020987375173717737, -0.007100383285433054, -0.1764313280582428, -0.015062990598380566, -0.20175747573375702, 0.0046180235221982, 0.3736259639263153, -0.01488625817000866}, {-0.002932619769126177, 0.014992397278547287, -0.0037653532344847918, -0.0023827950935810804, 0.011494198814034462, 0.0032843202352523804, -0.0038666161708533764, -0.005795083474367857, -0.00548570929095149, 0.006053662858903408, 0.018601126968860626, 0.012196660041809082, -0.00421601627022028, -0.004567616619169712, 0.24827848374843597, 0.0029313599225133657, -0.005873829126358032, 0.005051097832620144, -0.005881758406758308, 0.012883937917649746, -0.009322045370936394, -0.013379733078181744, 0.0026146566960960627, 0.23393391072750092, 0.007938421331346035, 0.0009247367852367461, -0.6310185194015503, 0.012592550367116928, 0.35633307695388794, 0.006652607582509518, 0.3990229368209839, 0.0005239237216301262}, {0.0064538149163126945, -0.0033531528897583485, -0.002259618602693081, -0.004055867437273264, 0.005167911760509014, 0.0035417990293353796, 0.013541482388973236, 0.015779869630932808, -0.0021099757868796587, 0.011611580848693848, -0.010454745031893253, -0.010423053056001663, 0.001011635991744697, -0.0018086594063788652, 0.16246767342090607, 0.0051494017243385315, -0.019240323454141617, 0.0039085159078240395, 0.0014380713691934943, 0.007622634060680866, 0.0016384487971663475, 0.00035617200774140656, 0.004994714632630348, -0.628596305847168, 0.012597665190696716, 0.004471564665436745, 0.23298780620098114, 0.012079291045665741, 0.0721396878361702, 0.014137059450149536, -0.010779929347336292, 0.007987440563738346}, {0.00272116856649518, -0.0023610414937138557, -0.0058840299025177956, 0.00011757702304748818, 0.0059938859194517136, 0.0078513128682971, -0.01382844801992178, 0.0020456062629818916, -0.007877725176513195, 0.0021117241121828556, 0.010554813779890537, -0.0003174782614223659, 0.025073431432247162, 0.0017574355006217957, -0.3731347620487213, 0.0017952820053324103, -0.012638858519494534, 0.0010092416778206825, 0.015895966440439224, -0.028335269540548325, -0.005915159359574318, -0.0034802749287337065, -0.005873407702893019, -0.38809919357299805, 0.012479324825108051, 0.005365911405533552, 0.42108139395713806, -0.006153268273919821, 0.21959926187992096, 0.01529600564390421, -0.26058676838874817, -0.005158999469131231}, {0.008034645579755306, -0.013429638929665089, -0.0028250273317098618, 0.00575611088424921, -0.036062926054000854, -0.006872337311506271, 0.011826741509139538, -0.00337657006457448, 0.00395972141996026, -0.012808036059141159, -0.0062608299776911736, -0.009322432801127434, -0.008140623569488525, 0.008855437859892845, 1.1066402196884155, 0.01569012925028801, -0.0028499960899353027, -0.0010963802924379706, 0.00375345372594893, 0.006674269679933786, 0.013987326063215733, 0.0025015058927237988, 0.001963916467502713, 0.04287942871451378, -0.0002667322405613959, -0.0077779171988368034, -0.19549648463726044, 0.002646145410835743, -0.6665232181549072, -0.01506990846246481, 0.16192081570625305, 0.03429887816309929}, {-0.009681252762675285, -0.01843881979584694, 0.01620442047715187, -0.00015273282770067453, -0.007216218393296003, -0.0062203374691307545, -0.0036143367178738117, 0.0026355271693319082, -0.008217466063797474, 0.011637640185654163, -0.010258760303258896, -0.008859051391482353, -0.0027254214510321617, -0.001378231798298657, -0.3353939950466156, -3.748868766706437e-05, 0.012702842243015766, -0.014568441547453403, 0.00743389967828989, -0.006839959416538477, -0.013425092212855816, 0.012803524732589722, -0.005190189927816391, 0.20938341319561005, -0.0011577418772503734, 0.003994160331785679, 0.025610804557800293, 0.004814654588699341, 0.04955940693616867, 0.0007036123424768448, 0.07954787462949753, -0.02114303596317768}, {0.00713853957131505, 0.010299758985638618, 0.021393753588199615, 0.0076976558193564415, 0.0057075112126767635, -0.0006159147014841437, -0.019180607050657272, 0.0018433108925819397, -0.032000117003917694, 0.004184340126812458, 0.002264013048261404, -0.006581857800483704, 0.0055992333218455315, 0.0016870052786543965, 0.015026785433292389, -0.011554954573512077, 0.0009383441065438092, -0.004238937981426716, 0.001015392248518765, 0.010564484633505344, 0.0037651374004781246, 0.0070240371860563755, -0.003702398855239153, 0.20245304703712463, -0.0018635328160598874, 0.006614563520997763, 0.4727795422077179, 0.012741335667669773, -0.07287051528692245, -0.005960103590041399, -0.793310284614563, 0.013840537518262863}, {-0.009674151428043842, -0.007628649473190308, 0.0037036959547549486, -0.010914463549852371, 0.005111267324537039, -0.01144898496568203, 0.014393829740583897, 0.0029361143242567778, -0.0071519664488732815, -0.017537575215101242, 0.026873139664530754, -0.003795014461502433, 0.007141451817005873, -0.0005755925667472184, 0.1290082484483719, 0.000773948326241225, 0.023007506504654884, 0.004231610801070929, -0.005313219968229532, -0.01848871074616909, 0.0047460212372243404, 0.00017811359430197626, 0.0024374686181545258, 0.25170212984085083, 0.001491536502726376, -0.0017347452230751514, 0.12498722970485687, 0.007044399622827768, -0.4667525887489319, 0.006837911903858185, 0.11361753195524216, 0.019817279651761055}, {-0.0003738536615855992, -0.009449335746467113, 0.007791479583829641, -0.018305985257029533, -0.0028596296906471252, -0.001568327657878399, 0.001057672081515193, -0.0025373809039592743, -0.0011823981767520308, 0.01264435425400734, 0.002615241566672921, -0.008244971744716167, 0.013817396014928818, -0.013536546379327774, 0.29683515429496765, 0.012639585882425308, 0.018709583207964897, -0.02253628708422184, -0.006311038043349981, 0.00277473172172904, 0.004385561682283878, 0.002143606310710311, 0.006119642872363329, -0.05155418813228607, 0.0019609504379332066, 0.004037267528474331, -0.044647570699453354, -0.023253658786416054, 0.2900846302509308, -0.009125045500695705, -0.02266404591500759, -0.005517847370356321}, {0.003717579646036029, 0.014931305311620235, 8.04742521722801e-05, -0.005190812051296234, 0.008710644207894802, -0.01595386676490307, -0.00581169780343771, -0.004494520835578442, 0.010374418459832668, -0.011000202968716621, -0.002431610133498907, -0.009859814308583736, -0.0032792328856885433, 0.01112688984721899, -0.3106549084186554, 0.00830837432295084, -0.0067187147215008736, 0.005400925874710083, -0.018216721713542938, 0.021550409495830536, -0.001057867193594575, 0.004474982153624296, 0.009147260338068008, 0.8495011329650879, -0.019809743389487267, 0.007849239744246006, -0.4499781131744385, 0.0039027200546115637, 0.15474459528923035, -0.00204128073528409, 0.41888856887817383, 0.000226989621296525}, {0.013693145476281643, -0.0009643144439905882, -0.0025107162073254585, 0.006841821130365133, 0.009304606355726719, 0.008100268431007862, -0.004147065803408623, 0.0001560077944304794, 0.001902711926959455, 0.0013708946062251925, 0.01222242321819067, -0.008131842128932476, 0.0014721319312229753, -0.013721398077905178, 0.822214663028717, 0.0041611590422689915, 0.012736844830214977, -0.012335126288235188, -0.0075733857229352, -0.00451070349663496, 0.026504864916205406, 0.011723659932613373, -0.006205831654369831, 0.049485933035612106, -0.0032489164732396603, -0.0027860074769705534, -0.0013235355727374554, -0.001787649467587471, -0.4540238678455353, 0.0105318957939744, -0.01635146327316761, -0.001671737409196794}, {0.0006014675018377602, -0.003175221849232912, 0.002140157390385866, -0.018630441278219223, 0.004623390268534422, 0.014840408228337765, -0.004464225843548775, -0.011490434408187866, -0.004601206164807081, -0.0003515926655381918, 0.0040201302617788315, -0.006668470799922943, 0.0017498816596344113, 0.008664502762258053, -0.30113181471824646, -0.004603833891451359, 0.004736659117043018, -0.004646891262382269, -0.01431203167885542, -0.0036503642331808805, 0.006770434323698282, 0.0035960497334599495, 0.01816456764936447, 0.023810509592294693, 0.0003384816227480769, -0.011125676333904266, 0.5001276135444641, -0.002047996735200286, -0.20280034840106964, 0.00016357396089006215, -0.19500793516635895, 0.009822666645050049}, {0.007812765426933765, -0.013253094628453255, -0.005301748402416706, -0.01164939347654581, 0.0021940101869404316, -0.008025392889976501, 0.012996196746826172, 0.015384029597043991, 0.0037568064872175455, 0.005524591077119112, 0.025448773056268692, -0.000570120697375387, 0.0018668757984414697, 0.00292897317558527, -1.8858592510223389, -4.836159132537432e-05, 0.014798779040575027, -0.012915472500026226, 0.0027388478629291058, -0.004877944011241198, -0.0025558534543961287, 0.004589103627949953, 0.013023106381297112, 0.5489970445632935, -0.006027292925864458, -0.01281022559851408, -0.06358113884925842, 0.00956639088690281, 1.0817807912826538, 0.016115551814436913, 0.10828260332345963, -0.028788315132260323}, {0.0027377125807106495, 0.01350532379001379, -0.012280987575650215, 0.005925880745053291, -0.0014706733636558056, -0.003515507560223341, -0.007376664783805609, 0.008393784984946251, 0.013156420551240444, -0.0064244321547448635, -0.0037143805529922247, -0.0036785665433853865, -0.008275610394775867, 0.006114162039011717, 0.24415519833564758, 0.013568205758929253, 0.007645045407116413, -0.013667361810803413, -0.010849333368241787, -0.017175747081637383, 0.008274300955235958, -0.009987096302211285, -0.009136462584137917, 0.23403730988502502, -0.0022823421750217676, 0.0006509104860015213, -0.019683467224240303, -0.011434296146035194, 0.09317898750305176, -0.013572005555033684, 0.018053719773888588, 0.010368462651968002}, {0.02019379287958145, -0.004792197607457638, 0.007080283015966415, -0.008818096481263638, 0.004779737908393145, 0.0033787605352699757, -0.0031730053015053272, -0.02241269126534462, -0.010720164515078068, -0.0033378887455910444, -0.00345600675791502, -0.013182520866394043, -0.01861160434782505, -0.00644936366006732, 0.375754714012146, 0.00697458116337657, 0.012161051854491234, -0.004727574530988932, -0.0076898569241166115, -0.009964870288968086, 0.012827884405851364, -0.010310339741408825, -0.007135890424251556, -0.2419154942035675, -0.02226799912750721, 0.022001899778842926, -0.36666032671928406, 0.0020009528379887342, 0.40793582797050476, 0.018008243292570114, 0.13316716253757477, 0.0068336534313857555}, {0.0038628345355391502, 0.009442461654543877, 0.003448192961513996, -0.0037388126365840435, -0.00568122323602438, -0.0021051226649433374, 0.008300391025841236, -0.003362898947671056, -0.010928724892437458, -0.004925722721964121, 0.005644796881824732, -0.01327014435082674, 0.0031373598612844944, 0.004399737808853388, -1.154597282409668, -0.012483439408242702, -0.0030373164918273687, 0.00934181734919548, -0.004023402463644743, 0.011622611433267593, 0.00852002389729023, 0.019520414993166924, -0.0006200737552717328, 0.9404209852218628, -0.019378043711185455, -0.0051538157276809216, 0.27367106080055237, -0.0005468346644192934, 0.26073482632637024, 0.0005009472370147705, -0.3358244001865387, -0.006580785382539034}, {0.011598908342421055, -0.009401636198163033, 0.013139601796865463, 0.002708343556150794, 0.006224534474313259, -0.005710717290639877, -0.006733583752065897, -0.0041635287925601006, 0.00010684492735890672, -0.006545495241880417, -0.009033813141286373, -0.01252875104546547, 0.021070104092359543, 0.012580106034874916, -0.5528908371925354, -0.0034441372845321894, 0.0027663668151944876, -0.00010969310824293643, 0.013248343952000141, 0.01156100444495678, 0.008538398891687393, -0.0007927265833131969, 0.0023636200930923223, 0.5444974899291992, -0.0019598372746258974, -0.0032086404971778393, 0.25277402997016907, 0.0096916314214468, 0.04992739483714104, -0.011946305632591248, -0.38157445192337036, 0.00864340364933014}, {0.013728356920182705, -0.01618446595966816, -0.006572845857590437, 0.008531682193279266, -0.004136913456022739, -0.005414141807705164, 0.014904447831213474, -0.0014231319073587656, 0.0017749412218108773, -0.009793632663786411, -0.007166539318859577, 0.0039048257749527693, 0.002283533103764057, 0.014682062901556492, 1.4643940925598145, 0.0006237176130525768, -0.013604975305497646, 0.0033989117946475744, 0.003554892959073186, 0.0046551707200706005, 0.006120816804468632, -0.02746679075062275, -0.008679886348545551, 0.46001508831977844, -0.01341188419610262, 0.00026057701325044036, 0.13263702392578125, 0.006318786181509495, -1.793030858039856, -0.0060426704585552216, -0.2780059576034546, -0.016000621020793915}, {0.0033134734258055687, -0.01925513707101345, -0.012610490433871746, -0.008031778037548065, -0.005615156143903732, -0.006505276542156935, 0.009850556030869484, 0.024179212749004364, -0.0007058086339384317, -0.003627528203651309, 0.004344156943261623, 0.007392717991024256, 0.004584337119013071, 0.012606093659996986, -0.1500832736492157, -0.009389294311404228, 0.0030503161251544952, -0.005587253253906965, -0.01559816487133503, -0.01710355095565319, 0.008956369012594223, -0.012489068321883678, -0.00220631156116724, -0.7242831587791443, -0.021763885393738747, 0.005924916826188564, 0.742066502571106, 0.0034255264326930046, 0.0798354297876358, 0.009845002554357052, -0.4267733097076416, -0.0010374871781095862}, {0.009792286902666092, 0.007792288437485695, 0.014455048367381096, -0.007117435801774263, 0.012144179083406925, -0.003729808609932661, -0.006495623383671045, -0.0009008133783936501, 0.003186366753652692, 0.002566257957369089, 0.008810765109956264, 0.014525607228279114, 0.007505977991968393, -0.013229461386799812, -0.4057508409023285, -0.007480442989617586, 0.010159177705645561, 0.01610690727829933, -0.011428140103816986, 0.0025311990175396204, 0.0035847644321620464, 0.014891366474330425, -0.0011322690406814218, -0.027671461924910545, 0.011173705570399761, -0.0009863502345979214, 0.2871130704879761, 0.012259983457624912, -0.005230902228504419, 0.006271274760365486, -0.2621975541114807, -0.00433738250285387}, {0.005447731353342533, 0.008062049746513367, 0.010707009583711624, -0.004448845982551575, 0.009570776484906673, -0.02666393294930458, 0.0008456503273919225, 0.0016328272176906466, 0.0034735435619950294, -0.0060929469764232635, 0.005297164432704449, -0.006436783354729414, -0.003968692850321531, 0.020756646990776062, 1.4954867362976074, -0.004610046744346619, -0.010709274560213089, 0.007140100002288818, 0.0204764474183321, 0.004943330772221088, -0.0010440621990710497, 0.0042566717602312565, 0.0005371326114982367, -2.972578525543213, 0.009541142731904984, -0.0031944336369633675, -0.32403841614723206, 0.005253891460597515, 1.4527209997177124, 0.007163472007960081, 0.29896557331085205, 0.0027644187211990356}, {0.0050862012431025505, 0.0032396134920418262, -0.025463838130235672, 0.013788249343633652, 0.008127368055284023, -0.01999920979142189, -0.0036417576484382153, -0.002314926590770483, 0.004282727371901274, -0.01694484055042267, -0.011683370918035507, -0.011567880399525166, 0.0010878965258598328, -0.005665101110935211, -0.32936376333236694, -0.018022289499640465, 0.012879077345132828, 0.007441787514835596, -0.009942973032593727, 0.003126332303509116, 0.004828511271625757, 0.020455654710531235, -0.006885096896439791, 0.04946235567331314, -0.0064235650934278965, 0.01172743272036314, -0.12743353843688965, -0.015348857268691063, -0.030131889507174492, 0.02068145014345646, 0.13690784573554993, 0.006739385426044464}, {0.011939678341150284, 0.004566522315144539, -0.003011977067217231, -0.011112848296761513, -0.014615021646022797, -0.0048138112761080265, -0.01719021610915661, 0.00552169606089592, 0.020250579342246056, 0.014399346895515919, 0.003658775705844164, -0.0075196437537670135, -0.001206760760396719, 0.018810944631695747, 0.12085544317960739, 0.0100087346509099, 0.004927431233227253, 0.005051444750279188, -0.0021456803660839796, -0.007424819748848677, 0.0027740993537008762, 0.00024200892949011177, -0.02716205269098282, 0.18763229250907898, -0.007417386397719383, 0.009431506507098675, 0.06059970706701279, -0.011541146785020828, -0.27588653564453125, 0.004898990038782358, -0.15259474515914917, 0.0023981058038771152}, {-0.0027321239467710257, 0.0017550768097862601, -0.008444557897746563, 0.002200540155172348, -0.0026753367856144905, 0.007108062505722046, -0.011411008425056934, -0.010263376869261265, -0.00799320824444294, -0.005442308261990547, -0.012810840271413326, -0.0024105398915708065, -0.0024711834266781807, -0.006567088421434164, -0.010088706389069557, -0.010650868527591228, 0.0012120321625843644, 0.008650685660541058, 0.003717537736520171, -0.029740994796156883, -0.001674459781497717, -0.0038065826520323753, 0.004330556374043226, -0.5561919212341309, 0.02242318168282509, 0.00654522143304348, 0.0881061926484108, 0.00502417329698801, -0.03826649487018585, -0.0001341269089607522, -0.05361044034361839, -0.0019517275504767895}, {0.014491644687950611, -0.005933044943958521, 0.007610498461872339, -0.018079012632369995, 0.005189388059079647, -0.004048764239996672, 0.010355012491345406, -0.001803381135687232, -0.013658805750310421, 0.008741442114114761, 0.0017387670231983066, -0.012517309747636318, 0.002240163739770651, 0.002659615594893694, 0.036132439970970154, 0.029647158458828926, 0.0026663111057132483, -0.003029467072337866, -0.0008763544610701501, 0.01294295396655798, 0.023018181324005127, 0.005804979242384434, 0.00035422909422777593, 0.058301810175180435, -0.004016195423901081, 0.01082670222967863, -0.2524283826351166, 0.004931164439767599, 0.15152062475681305, -0.008058939129114151, 0.036259543150663376, -0.0014488016022369266}, {0.010476726107299328, 0.008730607107281685, -0.006574747618287802, -0.016895394772291183, 0.008337041363120079, 0.008665522560477257, -0.0035615181550383568, -0.009079476818442345, -0.005307461600750685, -0.006506814621388912, -0.01671406626701355, 0.006145923398435116, -0.017987200990319252, -0.011208045296370983, -0.41031327843666077, 0.00860328134149313, -0.01269823033362627, -0.0040465546771883965, -0.005097216926515102, 0.004876154474914074, 0.005433978512883186, -0.00045728558325208724, 0.006682941690087318, 0.2796829044818878, 0.004254576284438372, 0.00950804352760315, 0.2543877065181732, -0.00045794545440003276, -0.3798869848251343, 0.0146236103028059, -0.5877758860588074, 0.005626112222671509}, {-0.0014600643189623952, -0.003938994836062193, -0.011335094459354877, -0.004512375220656395, 0.005569130647927523, -0.005901461001485586, -0.0010782910976558924, 0.009504567831754684, 0.00022067854297347367, 0.0007336531998589635, 0.002677950309589505, -0.005371156148612499, -0.0055966912768781185, -0.019452616572380066, -0.08685757964849472, 0.010683197528123856, 0.010723995976150036, 0.005999147891998291, 0.003985514864325523, -0.004449458327144384, -0.02569340169429779, 0.0008484116988256574, -0.010166921652853489, -0.34029239416122437, 0.006168062333017588, 0.011377187445759773, -0.1519761085510254, -0.015085826627910137, 0.163273885846138, -0.006254120729863644, 0.28420817852020264, 0.0038659116253256798}, {-0.008747839368879795, -0.007529204245656729, 0.0008318023756146431, -0.006275946740061045, 0.0016234178328886628, 0.007662393152713776, 0.016838032752275467, -0.0050284541212022305, 0.01019036490470171, -0.009833862073719501, -0.015920190140604973, -0.003563612699508667, -0.004273143131285906, 0.013610814698040485, 0.2575470507144928, 0.010334718972444534, -0.012171758338809013, -0.005055723711848259, 0.015175050124526024, 0.004164066631346941, 0.012588568963110447, -0.006197178270667791, 0.0006897783605381846, -0.05522988736629486, 0.008003252558410168, 0.009031281806528568, -0.25434020161628723, -0.011250961571931839, 0.0064957416616380215, 0.0004316429258324206, 0.42113587260246277, 0.007466908078640699}, {0.002095179632306099, 0.006906971801072359, 0.001650893478654325, 0.0008705806685611606, 0.0003000612778123468, 0.0030316575430333614, -0.0017755752196535468, -0.014038411900401115, -0.002356555312871933, -0.008899230509996414, -0.011815536767244339, 0.015387315303087234, -0.0057608080096542835, 0.0013931476278230548, 0.3507184088230133, 0.0061013768427073956, -0.010784946382045746, -0.004704837221652269, 0.004635840654373169, -0.003515861928462982, -0.009157976135611534, -0.00868538673967123, 0.008194668218493462, -0.6607871651649475, -0.0017909210873767734, 0.010724440217018127, -0.04571821540594101, -0.0066141835413873196, 0.41337984800338745, 0.006443400401622057, 0.0016603519907221198, 0.011377527378499508}, {0.008441660553216934, 0.013180958107113838, -0.013345547951757908, 0.0010656813392415643, -0.009619180113077164, -0.002672079484909773, 0.004406829830259085, -0.011427623219788074, 0.019083714112639427, -0.00085437175584957, 0.0037859645672142506, 0.009672165848314762, 0.003235078416764736, -0.00569189852103591, 0.019894130527973175, 0.00858587957918644, 0.012241934426128864, 0.005701894406229258, 0.0040030572563409805, -0.004790720995515585, 0.01931430958211422, 0.014865460805594921, 0.0010234552901238203, -0.5530552268028259, -0.001310777966864407, -0.002114332979544997, 0.04647281765937805, -0.0063054910860955715, 0.06643840670585632, -0.007681717164814472, -0.029432497918605804, 0.01789059303700924}, {0.011263225227594376, 0.011946423910558224, 0.00743219256401062, 0.009593179449439049, -0.010620739310979843, -0.006425719708204269, -0.010928941890597343, 0.009293373674154282, -0.004722288344055414, -0.0003521263424772769, 0.0055525293573737144, 0.013637689873576164, 0.0004512036975938827, 0.01677597314119339, 0.4003985822200775, -0.0036150244995951653, -0.00740208150818944, 0.0034053425770252943, -0.0026342214550822973, 0.008773209527134895, -0.008194491267204285, 0.000524995441082865, -0.02129598893225193, 0.048135269433259964, 0.011608046479523182, -0.00843984354287386, -0.12289825081825256, -0.0033036363311111927, -0.164742112159729, 0.003935418091714382, 0.021129854023456573, -0.012464337050914764}, {-0.0014700527535751462, -0.011882281862199306, -0.002456367714330554, 0.011321079917252064, 9.066319762496278e-05, -0.004570025950670242, -0.006155114620923996, -0.007518840953707695, 0.005136461928486824, -0.0038208900950849056, -0.0015626467065885663, -0.000531495432369411, 0.010380009189248085, 0.012572190724313259, -0.06677141040563583, -0.006559638772159815, 0.00549412751570344, -0.017625054344534874, -0.01705813966691494, 0.008883925154805183, 0.0027917888946831226, -0.012000905349850655, 0.003726951777935028, 0.08408636599779129, -0.008685837499797344, -0.00844157300889492, 0.19813968241214752, 0.004502727650105953, -0.22158029675483704, -0.013312004506587982, 0.022016871720552444, -0.00853664893656969}, {0.004533539526164532, -0.0048971027135849, 0.001492862356826663, 0.0007816112483851612, 0.0068567353300750256, 0.0019724194426089525, -0.021513385698199272, -0.010537396185100079, 0.008525803685188293, -0.002020477782934904, -0.008830408565700054, 0.002471655374392867, -0.005374210886657238, -0.013500447385013103, -0.04195530340075493, -0.01022807601839304, 0.003560583805665374, 0.008961828425526619, -0.0015912165399640799, 0.01156007219105959, -0.00489197438582778, 0.011872096918523312, 0.007629557512700558, -1.4361045360565186, 0.0002043987187789753, 0.0041682301089167595, 0.5628525614738464, -0.006393151357769966, 0.6482865810394287, 0.011120880953967571, -0.20804880559444427, -0.009758614003658295}, {0.010924858041107655, 0.0031204873230308294, -0.004337076563388109, -0.012302094139158726, 0.008340494707226753, -0.004733006935566664, 0.00824509933590889, -0.008006184361875057, 0.004490439314395189, -0.001542321522720158, -0.010655616410076618, 0.009545503184199333, -0.002830605488270521, 0.0002001081738853827, 0.005888020619750023, -0.009449506178498268, 0.015185405500233173, 0.0010019598994404078, -0.012569932267069817, -0.003061093855649233, -0.016698133200407028, -0.0042747026309370995, -0.001803950173780322, 0.7988687753677368, -0.01228475384414196, -0.012601341120898724, -0.19048690795898438, 0.010183289647102356, -0.5576179027557373, 0.010797292925417423, 0.18290449678897858, 0.0057405405677855015}, {-0.0032620918937027454, -0.019783146679401398, 0.007587571162730455, -0.00024042905715759844, -0.009659526869654655, -0.0022730641067028046, -0.01299258228391409, -0.005337582901120186, -0.011457427404820919, 0.011532741598784924, 0.0023860083892941475, 0.003045552410185337, -0.004124872852116823, 0.013100366108119488, 1.1550869941711426, 0.006715318188071251, -0.0037889655213803053, 0.012520945630967617, -0.010965792462229729, -0.00038555279024876654, 0.005098786670714617, 0.0029102375265210867, -0.0038344967179000378, -1.5191713571548462, 0.002783350180834532, 0.018558340147137642, -0.16639834642410278, 0.0039614965207874775, 0.08915652334690094, -0.024094972759485245, -0.09754916280508041, 0.012645173817873001}, {-0.013879073783755302, 0.011097258888185024, 0.0053237727843225, 0.01725262776017189, -0.008739413693547249, 0.022879011929035187, -0.00037922151386737823, 0.01131818350404501, 0.014176888391375542, -0.02098342403769493, -0.0010661014821380377, 0.00400083651766181, 0.012658425606787205, -0.0023939646780490875, 0.937130868434906, 0.00574643537402153, 0.015463631600141525, 0.003691712860018015, -0.009760760702192783, -0.0029243065509945154, 0.010750355198979378, 0.011957277543842793, 0.01120374072343111, 0.48832887411117554, 0.004659459460526705, 0.006058844737708569, 0.13989382982254028, -0.006050860043615103, -1.2120521068572998, 0.006419450975954533, -0.13552641868591309, -0.017860179767012596}, {0.013868005014955997, -0.003563950303941965, 0.006866789888590574, 0.0004333931428845972, 0.00994531624019146, 0.009392728097736835, 0.005441504064947367, 0.009013714268803596, -0.0018062317976728082, 0.0026150753255933523, 0.0069923484697937965, 0.010542218573391438, -0.006847707089036703, -0.011977426707744598, 2.018977403640747, -0.0262544434517622, -0.014112132601439953, -0.024484923109412193, -0.009380675852298737, 0.0025749397464096546, -0.008070055395364761, 0.020453600212931633, -0.0014426331035792828, -1.460649013519287, -0.0061403014697134495, 0.011571254581212997, 0.009492048993706703, 0.00759924529120326, -0.44586890935897827, 0.004631470423191786, -0.13176192343235016, 0.005669428501278162}, {0.009249240159988403, -0.0009414040832780302, 0.008789917454123497, -0.003795584663748741, -0.012366235256195068, 0.004978249315172434, -0.0031250263564288616, 0.014995504170656204, -0.0114047946408391, 0.0007565980777144432, 0.0029906295239925385, 0.0006143457139842212, 0.008935308083891869, -0.009071682579815388, 0.5580523014068604, -0.007814927957952023, 0.012551014311611652, 0.017643673345446587, -0.004162383731454611, 0.00812404416501522, -0.004973896313458681, 0.011604759842157364, -0.0011780756758525968, -1.1778254508972168, 5.440797394840047e-05, 0.009357515722513199, 0.11533631384372711, 0.0029620262794196606, -0.11892195791006088, -0.024088792502880096, 0.2933386266231537, -0.012519482523202896}, {0.002229450037702918, -0.0006438971031457186, -0.000348119210684672, 0.012164452113211155, -0.011071041226387024, 0.00916776992380619, 0.0006010390352457762, -0.003934840671718121, 0.0008034061174839735, -0.010592839680612087, 0.008878746069967747, -0.008603203110396862, 0.001835413626395166, 0.0038068266585469246, -0.7030117511749268, 0.002733521396294236, -0.010255042463541031, -0.007834861986339092, 0.01383650116622448, -0.00869856309145689, 0.009602661244571209, -0.002556031569838524, 0.0067230877466499805, 0.2251707762479782, 0.006531451363116503, -0.0172018613666296, 0.10995302349328995, -0.006505719386041164, 0.35330721735954285, -0.0009766101138666272, -0.06268288195133209, 0.008745876140892506}, {-0.0055772350169718266, -0.021413398906588554, -0.017232464626431465, -0.006389035377651453, -0.01282583363354206, 0.016281429678201675, -0.007671080529689789, 0.006753706373274326, -0.012410281226038933, -0.0007521864026784897, -0.025868143886327744, 0.0011886402498930693, -0.01663482002913952, 0.01773606799542904, 0.12402231991291046, 0.0019670280162245035, 0.025721322745084763, -0.00014876853674650192, -0.0065719191916286945, 0.0056245820596814156, -0.0014133559307083488, -0.005064335186034441, -0.0012295827036723495, 0.06883303076028824, 0.010659989900887012, -0.00037986008101142943, -0.08610173314809799, 0.0009588028187863529, -0.003570250468328595, -0.018238816410303116, 0.0862610787153244, 0.00031178182689473033}, {0.001788854249753058, 0.009090757928788662, -0.003923875745385885, -0.0008692166302353144, 0.0279973316937685, -0.0035695682745426893, -0.013893595896661282, -0.01140778511762619, -0.02000926062464714, -0.0067971753887832165, -0.011142868548631668, 0.00905978586524725, -7.416881271637976e-05, -0.014056768268346786, 0.5209183096885681, 0.021471554413437843, -0.0013400281313806772, -0.00018882697622757405, 0.005427220836281776, -0.01585942693054676, -0.008188821375370026, 0.0017534219659864902, 0.006122142542153597, -0.4891663193702698, -0.020064616575837135, -0.0008115090895444155, 0.042796943336725235, -0.005169988144189119, 0.016140716150403023, -0.004673058167099953, -0.29270708560943604, 0.018419938161969185}, {0.006230891682207584, -0.0019047949463129044, 0.004199582617729902, -0.0019825443159788847, -0.005121150985360146, 0.011637975461781025, -0.005065586417913437, 0.00800733920186758, 0.00988239236176014, -0.003134403144940734, 0.01364724151790142, 0.012428387068212032, 0.002130090491846204, -0.006305631250143051, -0.5036334991455078, -0.004635895136743784, 0.01038442924618721, -0.008225726895034313, -0.016638612374663353, -0.0070225950330495834, -0.003893617307767272, -0.0006866959156468511, 0.0023295683786273003, -0.46810382604599, 0.007534955628216267, 0.003502944717183709, 0.07797086238861084, -0.0018017124384641647, 0.4523923397064209, 0.007218232378363609, -0.3134578764438629, -0.016771970316767693}, {0.003807065077126026, 0.008921320550143719, 0.013448175974190235, -0.003665145020931959, 0.00021436906536109746, -0.010512721724808216, -0.003006797982379794, -0.004981648176908493, -0.006247334182262421, -0.005943718366324902, 0.018764004111289978, 0.0015742742689326406, 7.7946184319444e-05, -0.006594012025743723, 0.0725848376750946, 0.013971959240734577, 0.013720995746552944, 0.001046995515935123, 0.0009261933737434447, 0.0038602305576205254, -0.0005736189777962863, 0.0027586419600993395, -0.00015262377564795315, 0.1676677018404007, -0.011975463479757309, 0.016228457912802696, 0.03606119006872177, -0.014436479657888412, 0.045193612575531006, -0.002782799769192934, -0.046226274222135544, -0.007706447038799524}, {-0.0015500038862228394, -0.0029864555690437555, 0.0039030674379318953, -0.008195266127586365, -0.013274679891765118, 0.014563783071935177, 0.004151358734816313, -0.013036410324275494, -0.009225457906723022, -0.003657326567918062, -0.006431820336729288, -0.015049705281853676, 0.01585714891552925, -0.003397677093744278, -0.5114689469337463, 0.004844662267714739, -0.016381366178393364, -0.010699105449020863, 0.0016863017808645964, -0.016091832891106606, 0.0023529643658548594, -0.017513928934931755, -0.0026029455475509167, 0.41202840209007263, -0.00046346461749635637, 0.006785403937101364, -0.049445752054452896, -0.0019985116086900234, -0.04472038149833679, -0.006834299303591251, 0.10648646950721741, -0.007245991844683886}, {-0.002816013991832733, 0.02058715559542179, -0.015999631956219673, -0.014555822126567364, -0.003366469871252775, 0.002063034102320671, -0.014902452938258648, -0.006665930151939392, 0.005191567353904247, 0.00866971630603075, -0.008201084099709988, -0.004271871410310268, -0.009886289946734905, -0.0326651968061924, 0.3315158784389496, -0.010622688569128513, -0.0036710230633616447, -0.0014351027784869075, -0.0018136908765882254, 0.0038953213952481747, 0.00729859434068203, -0.002161588752642274, -0.011273643933236599, 0.08147363364696503, -0.009148899465799332, 0.0012850004713982344, -0.40737730264663696, -0.010140235535800457, 0.2435673177242279, -0.00015642994549125433, 0.47686833143234253, -0.008813210763037205}, {0.023231687024235725, 0.008172422647476196, -0.004777746275067329, -0.00593797629699111, -0.024769719690084457, 0.002163815312087536, 0.004987653810530901, -0.006890727672725916, 9.049363870872185e-05, 0.003357132663950324, 0.0008635282865725458, 0.005232539493590593, 0.025140587240457535, 0.013637714087963104, 1.5131127834320068, -0.000501633039675653, 0.002618253231048584, 0.008642667904496193, -0.0011440477101132274, -0.006349638104438782, -0.0005724424263462424, -0.005965156946331263, 0.016866758465766907, -2.9543192386627197, 0.0033251589629799128, -0.013699958100914955, -0.3274044692516327, 0.009519302286207676, 1.7540112733840942, -0.013159902766346931, 0.35767295956611633, 0.004917983431369066}, {-0.004763943608850241, 0.00014240786549635231, -0.011841480620205402, 0.029821982607245445, -0.002895528916269541, -0.005843411665409803, 0.0030774616170674562, 0.0043882750906050205, -0.0033365406561642885, -0.004266075789928436, 0.0018820094410330057, 0.0052664270624518394, 0.00297848554328084, 0.00819916371256113, 0.34093958139419556, 0.01828707382082939, -0.010293370112776756, -0.0055389925837516785, 0.006577306427061558, -0.0004397696175146848, -0.0050056991167366505, -0.004358754493296146, 0.018798166885972023, -0.8475100994110107, -0.006736552342772484, 0.014715956524014473, -0.11813310533761978, -0.011060456745326519, 0.08618708699941635, -0.004148556385189295, 0.030137283727526665, -0.002885687630623579}, {0.005558243487030268, 0.009094014763832092, -0.01642855443060398, -0.0024175094440579414, 0.012437236495316029, 0.006940203253179789, -0.016191568225622177, 0.0059687355533242226, -0.00680579524487257, -0.019638532772660255, -0.007091908250004053, 0.004927946720272303, -0.0033208641689270735, 0.011318735778331757, 0.3342839181423187, 0.0032633233349770308, 0.014882172457873821, -0.013397409580647945, 0.01526456419378519, 0.003543668892234564, 0.02027754671871662, 0.015288295224308968, 0.0005559808341786265, 0.2874232530593872, 0.017986904829740524, -0.002559233922511339, 0.056863900274038315, 0.008268660865724087, -0.28839561343193054, 0.006003526505082846, -0.20182178914546967, 0.003279514843598008}, {0.01078027207404375, 0.024472085759043694, -0.015866652131080627, -0.0046178437769412994, -0.005127196200191975, 0.0010162548860535026, -0.005628394894301891, 0.004317294806241989, -0.00694844638928771, 0.0018908805213868618, 0.013610041700303555, -0.01714596152305603, 0.008070566691458225, -0.007102406583726406, -0.09241832047700882, 0.009297745302319527, 0.00012288690777495503, -0.018719134852290154, 0.010449235327541828, 0.0005188171635381877, -0.017667993903160095, -0.003784621600061655, 0.008915361948311329, 0.04296271875500679, 0.01956282928586006, -0.00969009567052126, -0.038558896631002426, 0.016054578125476837, -0.3961338996887207, -0.012275871820747852, 0.16002261638641357, 0.008228632621467113}, {0.009998372755944729, -0.0011915682116523385, 0.0008609442156739533, 0.012717293575406075, 0.003909687977284193, -0.00045874222996644676, -0.005379348993301392, -0.007563952822238207, 0.002211478538811207, 0.011215622536838055, -0.0020527902524918318, 0.0037243603728711605, 0.003712140489369631, -0.006468728184700012, 1.0313904285430908, 0.002605828922241926, 0.013730368576943874, 0.0007066910620778799, 0.008438191376626492, 0.013093698769807816, 0.0072220247238874435, -0.005848945118486881, -0.008363217115402222, -0.2447998970746994, -0.016460344195365906, -0.01823066733777523, -0.0003931912360712886, -0.003215175587683916, -0.5784081220626831, -0.011383634991943836, -0.20329450070858002, -0.0011098511749878526}, {0.0013233578065410256, -0.00017927699082065374, 0.017871489748358727, 0.0017231603851541877, 0.001843766774982214, -0.006686931010335684, 0.002834906103089452, 0.005644622258841991, 0.0006612755823880434, 0.0007178038358688354, -0.015060076490044594, 0.006010768469423056, 0.006076463498175144, 0.00917851459234953, -0.07915183156728745, -0.005572393536567688, 0.009970875456929207, -0.014290809631347656, -0.01610514521598816, -0.015524784103035927, -0.008286062628030777, -0.008483446203172207, -0.004044571425765753, 0.7177749872207642, 0.0043851411901414394, 0.010726157575845718, -0.4142846465110779, -0.014713117852807045, -0.030990712344646454, 0.013917718082666397, 0.14226314425468445, -0.004509244579821825}, {0.0019145680125802755, -0.022243380546569824, -0.0020992113277316093, 0.006167557556182146, 0.012127971276640892, -0.0021329293958842754, -0.008381154388189316, -0.003030473133549094, 0.009086057543754578, -0.01770109124481678, -0.008334501646459103, -0.027701174840331078, -0.00422624172642827, 0.00479848962277174, 3.6991348266601562, -0.00516539067029953, 0.003460866631940007, 0.018126672133803368, -0.007274155970662832, 0.008661416359245777, 0.01827988028526306, -0.007384574972093105, -0.00431987177580595, -2.149343252182007, 0.00978346448391676, -0.0007030978449620306, -0.4647919535636902, 0.0031868943478912115, -1.907062292098999, -0.005964711774140596, 0.6886760592460632, 0.014016631990671158}, {-0.021848656237125397, -4.0638449718244374e-05, 0.001992849400267005, -0.009700998663902283, -0.0062055266462266445, -0.0031604324467480183, -0.000745932396966964, 0.008902285248041153, 0.0009035823168233037, -0.0034988406114280224, 0.005067187361419201, -0.008764837868511677, 0.005509696435183287, 0.015421672724187374, -1.2487003803253174, -0.007581756915897131, 0.0017421375960111618, 0.002256294246762991, 0.0028181655798107386, 0.012328872457146645, 0.014735563658177853, 0.006774619221687317, 0.008577754721045494, 1.6160132884979248, -0.013983272016048431, -0.01570257544517517, 0.01477998960763216, -0.013092529028654099, -0.10078764706850052, 0.010892516002058983, -0.06153648719191551, 0.01696765422821045}, {-0.021775441244244576, 0.009852834977209568, -0.0024437173269689083, 0.002343439031392336, -0.009707944467663765, -0.018071969971060753, -0.001063961535692215, 0.0007221453706733882, 0.02008868008852005, -0.00073980912566185, 0.0025344006717205048, -0.006918821018189192, -0.01446517463773489, -0.00573331443592906, -0.017289863899350166, -0.003554518101736903, 0.0041465191170573235, 0.017094748094677925, -0.0019501772476360202, -0.013302410952746868, 0.006014520302414894, -0.014609419740736485, -0.005629881285130978, 0.12575304508209229, -0.002937828190624714, 0.00815710797905922, -0.029362166300415993, 0.0018800643738359213, 0.16471773386001587, -0.015603539533913136, -0.06418494880199432, 0.0016066613607108593}, {0.0006781116244383156, 0.0015034300740808249, 0.0027747948188334703, 0.0026524888817220926, -0.004999964963644743, 0.004041988868266344, 0.00019426837388891727, 0.008359215222299099, -0.00775161711499095, -0.014225302264094353, -0.007082353811711073, 0.0033513749949634075, 0.0014975317753851414, -0.012275353074073792, -0.3795620799064636, -0.01237317081540823, -0.00805801060050726, 0.0052681309171020985, -0.011923003941774368, 0.007192594930529594, -0.00747700035572052, 0.0019653863273561, -0.0007729847566224635, 0.48152977228164673, 0.00022813494433648884, -0.013125975616276264, -0.17511597275733948, 0.00039480673149228096, 0.06923608481884003, 0.011396264657378197, 0.1087648868560791, -0.0032993836794048548}, {0.0025785588659346104, -0.006120673846453428, -0.006142458412796259, 0.01793505810201168, -0.0019699493423104286, 0.014611351303756237, -0.018746299669146538, 0.015706023201346397, -0.023431777954101562, -0.011284924112260342, 0.0038133233319967985, 0.0018835277296602726, -0.0006591930869035423, 0.02056807465851307, -0.31141337752342224, -0.008595908991992474, -0.005809520836919546, -0.022990452125668526, 0.0082303611561656, -0.0030420341063290834, 0.007951928302645683, 0.004320107400417328, -0.005641857627779245, -0.3570516109466553, -0.011530679650604725, 0.005222282838076353, 0.10110382735729218, 0.0017357594333589077, 0.2030581384897232, -0.0013273541117087007, 0.15517225861549377, 0.015413084998726845}, {0.007703585084527731, 0.014453995041549206, -0.0023229862563312054, 0.0018793278140947223, 0.024726981297135353, -0.0003419119166210294, -0.010567120276391506, 0.0164286270737648, -0.005735412705689669, 0.015029002912342548, -0.010940346866846085, -0.013891276903450489, -0.003070581005886197, 0.005454095080494881, -0.36861759424209595, 0.0012733986368402839, 0.007247076835483313, 0.027036063373088837, 0.011039049364626408, 0.009528212249279022, 0.008207435719668865, -0.0069082495756447315, -0.005173118785023689, 1.293960690498352, -0.022173890843987465, 0.03685790300369263, -0.0558423213660717, -0.001133639132604003, -0.011229022406041622, 0.0008766773389652371, -0.03838745504617691, -0.002451626816764474}, {-0.00921376422047615, -0.003911409992724657, 0.0016151961171999574, -0.004013497848063707, -0.004384216386824846, 0.008214429020881653, -0.010341967456042767, 0.011886960826814175, -0.0018582370830699801, -0.00306445499882102, 0.003444208763539791, 0.0019898745231330395, 0.008841698989272118, 0.01107083261013031, 0.2795574963092804, 0.01331027690321207, 0.0018622870557010174, -0.013041718862950802, 0.00195868662558496, -0.0071691786870360374, 0.008567586541175842, 0.008746927604079247, 0.005601423792541027, 0.5751889944076538, 0.0035627020988613367, -0.0056533305905759335, -0.021150488406419754, -0.003953021485358477, -0.3813439905643463, 0.009570338763296604, 0.12897947430610657, -0.004022308625280857}, {0.004368350841104984, 0.003426674287766218, 0.013158883899450302, 0.018221424892544746, -0.018059657886624336, 0.0009058949071913958, 0.005769847892224789, 0.004876273218542337, 0.009118490852415562, 0.0036164175253361464, -0.011804470792412758, -0.011665207333862782, 0.0057702879421412945, -0.0038577772211283445, 0.0024431690108031034, 0.005188777111470699, -0.005777235608547926, -0.007841652259230614, 0.0164017453789711, -0.008007443509995937, 0.005781174637377262, 0.002793446648865938, 0.00800141878426075, 0.13371093571186066, -0.01262402068823576, -0.0038705419283360243, 0.07733834534883499, 0.00720185786485672, 0.004789233673363924, -0.00854016188532114, 0.1303461641073227, 0.018968556076288223}, {0.012679506093263626, -0.02000555954873562, -0.016986606642603874, -0.0089263916015625, -0.009427402168512344, -0.001545190461911261, -0.0031768502667546272, -0.020122813060879707, 0.01927988789975643, -0.009799433872103691, -0.0026504730340093374, -0.001579602132551372, 0.01024398673325777, -0.016225172206759453, 0.12196294963359833, -0.0011253010015934706, 0.0036267468240112066, -0.002135704504325986, -0.00611478928476572, 0.0029552243649959564, 0.00022081396309658885, 0.0004138506483286619, -0.020824814215302467, 0.07637114077806473, 0.01723751425743103, 0.013863133266568184, -0.10689548403024673, 0.01077597588300705, 0.11275950819253922, 0.006465534213930368, 0.17683039605617523, 0.009025298058986664}, {-0.005188262555748224, 0.01018021535128355, -0.008891639299690723, -0.005114147439599037, -0.001488547888584435, -0.01824520342051983, 0.0014244712656363845, 0.01184050552546978, -0.0029200895223766565, -0.005034375470131636, -0.01228412240743637, -0.0007479324121959507, -0.0025793802924454212, 0.00016539411444682628, -0.828350305557251, 0.0006929431692697108, -0.00608531478792429, 0.026834802702069283, 0.0031170453876256943, 0.01922493800520897, 0.00968509167432785, 0.0016474125441163778, -0.0009176447638310492, -0.7409163117408752, -0.004222977440804243, 0.011295178905129433, -0.3642919659614563, 0.02113443613052368, 1.577730655670166, 0.0026202104054391384, 0.3899795413017273, -0.007277796510607004}, {0.003032384440302849, 0.000800118432380259, 0.009892614558339119, 0.007703291717916727, 0.00014926998119335622, 0.009663164615631104, -0.006408589892089367, 0.019151998683810234, 0.008686289191246033, 0.0008564209565520287, -0.017640938982367516, -0.004589139483869076, -0.0062445299699902534, 0.005094143562018871, 0.04929112270474434, -0.0038247599732130766, 0.004007254261523485, -0.0015206639654934406, 0.004203410819172859, -0.007300919853150845, -0.0019057461759075522, 0.007991700433194637, -0.004201160743832588, -0.8298936486244202, 0.013840327039361, 0.004207161720842123, 0.0661272332072258, -0.005071495659649372, 0.1364854872226715, 0.004758385010063648, -0.021998295560479164, -0.0018966506468132138}, {0.004208915866911411, -0.007832041941583157, -0.018017418682575226, -0.010094846598803997, 0.02294796332716942, -0.005766760557889938, -0.00942234881222248, -0.011006956920027733, 6.908144132466987e-05, 0.019521357491612434, -0.0003679138608276844, -0.010504537262022495, -0.009506254456937313, 0.015650350600481033, 0.6277840733528137, -0.004973620641976595, -0.003622236428782344, -0.004900560714304447, -0.012659699656069279, -0.00810931995511055, 0.0018900544382631779, -0.0021888217888772488, -0.0029242008458822966, -0.044883791357278824, 0.011602737940847874, -0.0031107827089726925, -0.02858668565750122, 0.0021307484712451696, -0.28342482447624207, 0.014845998957753181, -0.05797060579061508, 0.01269990298897028}, {0.003624806646257639, -0.01523729506880045, 0.007721294648945332, -0.005643165670335293, 0.011598367244005203, -0.001978452317416668, -0.003330447245389223, 0.011369732208549976, 0.00617345841601491, 0.0005211348761804402, 0.003926767967641354, -0.014783767983317375, 0.0013368917861953378, -0.0021703389938920736, -0.14537782967090607, -0.010481727309525013, -0.025055624544620514, -0.0051028309389948845, -0.01169948186725378, -0.01276847068220377, 0.027006374672055244, 0.008830903097987175, 0.011239857412874699, -0.595710039138794, -0.013949996791779995, 0.0053832740522921085, 0.019544247537851334, 0.0046095168218016624, 0.20257271826267242, 0.0029527987353503704, 0.12671104073524475, -0.009607387706637383}, {0.00018733493925537914, -0.010723658837378025, 0.010189962573349476, -0.01988101750612259, -0.010751732625067234, -0.002699192613363266, 0.020473668351769447, 0.002798179630190134, -0.013086031191051006, 0.0023977276869118214, -0.009223517030477524, 0.011079179123044014, -0.010719457641243935, 0.007089667953550816, 0.06853598356246948, -0.006564941722899675, 0.008091536350548267, -0.001803756458684802, -0.001701588393189013, 0.014070669189095497, -0.006653826683759689, -0.0007345676422119141, -0.004329286981374025, 0.6378514170646667, 0.009400843642652035, -0.016521142795681953, 0.014858182519674301, 0.011498446576297283, -0.3433677554130554, -0.002207167912274599, -0.3988279104232788, -0.007463812828063965}, {-0.014221923425793648, 0.009596137329936028, -0.005820258520543575, 0.0023928082082420588, 0.008823715150356293, 0.0009122125338762999, -0.005544137675315142, 0.011769099161028862, 0.019834117963910103, 0.001822065096348524, 0.001778769656084478, 0.0063759940676391125, 0.01645277440547943, 0.008564496412873268, -0.8403871655464172, 0.02044067159295082, 0.008128363639116287, 0.01144111342728138, 0.00877675786614418, -0.0006878793356008828, 0.0007027370156720281, -0.00016835820861160755, -0.006906889379024506, 0.310293585062027, 0.002205370459705591, -0.008072573691606522, 0.0071587213315069675, 0.002980937249958515, 0.3380507230758667, -0.004885903559625149, 0.06990015506744385, -0.004038792569190264}, {0.007522132713347673, -0.0029500345699489117, 0.011738434433937073, 0.014085294678807259, -0.016999896615743637, -0.00224890885874629, 0.014276694506406784, 0.015225517563521862, 0.008757203817367554, -0.00752935791388154, -0.00478382920846343, -0.005870178807526827, 0.0015013671945780516, 0.011362635530531406, 0.1813206821680069, -0.003178318263962865, -0.0016155182383954525, 0.0008088430040515959, -0.0007192455232143402, -0.006964152678847313, -0.005050403997302055, -0.008012538775801659, -0.01026439480483532, 0.46910983324050903, -0.012670104391872883, 0.006910022813826799, 0.012708915397524834, -0.012931345030665398, -0.01541957724839449, -0.0037759554106742144, -0.1695682406425476, 0.019986940547823906}, {-0.011516159400343895, -0.010264482349157333, 0.016586901620030403, -0.0012373881181702018, -0.014399779960513115, 0.015041138976812363, -0.01485537737607956, -0.011631782166659832, -0.01837099716067314, -0.001790686510503292, 0.008958136662840843, -0.006665549241006374, -0.010131741873919964, -0.007949728518724442, 0.039144787937402725, -0.011294268071651459, 0.012802738696336746, 0.009975175373256207, -0.032656479626894, 0.001924345619045198, 0.01946553960442543, -0.012665548361837864, -0.0021391743794083595, 0.4955490529537201, -0.0013577048666775227, 0.012396897189319134, -0.3387344777584076, 0.001091587240807712, -0.003133622230961919, -0.021746577695012093, 0.6456449031829834, -0.01643124409019947}, {-0.006759372074157, 0.013256264850497246, -0.0050512622110545635, 0.0037678065709769726, -0.019813932478427887, -0.01570981554687023, 0.01237241830676794, -0.01330329105257988, -0.016168568283319473, -0.012482563965022564, -0.00014412133896257728, 0.003634801134467125, -0.0035946331918239594, 0.011773302219808102, 0.00747923506423831, 0.007064907345920801, -0.0070249224081635475, 0.0007306904881261289, -0.009876486845314503, 0.0038654946256428957, 0.0004209229664411396, -0.013261331245303154, -0.010201537050306797, -0.4191267490386963, 0.006363824009895325, 0.0011826399713754654, 0.033436987549066544, -0.0045689987018704414, 0.10191275179386139, -0.015180295333266258, 0.05881484970450401, 0.001729860669001937}, {-0.015011714771389961, 0.005821851547807455, 0.0027861767448484898, 0.007362840697169304, -0.005890885833650827, -0.010359789244830608, 0.003799342317506671, -0.00214792019687593, 0.005410486366599798, 0.0054886797443032265, 0.010755615308880806, -0.0031214000191539526, 0.014288526028394699, 0.0071395463310182095, -0.2578656077384949, 0.00904133077710867, 0.008163615129888058, -0.0033185663633048534, -0.0022939154878258705, -0.012497681193053722, -0.01267087459564209, 0.003853103844448924, 0.00032802377245388925, -0.032784365117549896, 0.008782748132944107, 0.0013153398176655173, -0.08644115924835205, -0.0021981738973408937, 0.08071829378604889, 0.004600899294018745, 0.0282292440533638, 0.0017173337982967496}, {0.003392991377040744, 0.005670482292771339, 0.003032407723367214, -0.003181448206305504, -0.004634695593267679, 0.00021776286303065717, 0.021534046158194542, -0.00017181328439619392, 0.002798540750518441, 0.007658341899514198, -0.005755938123911619, -0.0018711595330387354, 0.007288094609975815, 0.009663197211921215, 0.19687002897262573, 0.009762575849890709, -0.01038158405572176, 0.009993068873882294, -0.00425700331106782, -0.0002874670608434826, -0.006217722315341234, -0.0036651238333433867, -0.004955461714416742, 0.004045445472002029, -0.004525375552475452, 0.020419469103217125, 0.003378279274329543, -0.0005133458762429655, -0.10057488828897476, -0.011184646748006344, -0.18560585379600525, 0.015192501246929169}, {-0.010723856277763844, -0.00773308239877224, 0.004291201941668987, 0.007521321065723896, 0.013168145902454853, -0.020968090742826462, 0.012300578877329826, -0.00021675144671462476, 0.0028231111355125904, -0.000909577589482069, 0.0059721628203988075, -0.0007993582985363901, -0.015832556411623955, -0.008611025288701057, 0.08160137385129929, 0.013600473292171955, 0.015150783583521843, -0.0015390500193461776, 0.0007257809047587216, 0.012946827337145805, 0.008317709900438786, 0.0003506818611640483, -0.003791227238252759, -0.01914336159825325, 0.0032988907769322395, -0.007754347752779722, -0.17737676203250885, -0.003223067382350564, 0.13523805141448975, -0.007852896116673946, -0.09711050242185593, -0.0187575314193964}, {-0.01472486462444067, -0.011818474158644676, 0.00024254487652797252, 0.003988038282841444, 0.01284883450716734, 0.004646592307835817, 0.017867187038064003, 0.013123724609613419, 0.013477019034326077, 0.004406084772199392, -0.008614876307547092, 0.007179405074566603, 0.004629475064575672, -0.014334939420223236, -0.5280975103378296, 0.005046452395617962, 0.010905412025749683, -0.014277107082307339, -0.011816750280559063, 0.004220126662403345, 0.01440457534044981, -0.011993056163191795, 0.008028927259147167, 1.264237642288208, -0.010895808227360249, 0.0028500540647655725, 0.023097747936844826, 0.01611553691327572, -0.7890207767486572, -0.0021729683503508568, 0.10092610865831375, -0.008044710382819176}, {0.0005306642851792276, -0.00840944703668356, -0.007615706883370876, -0.006194261834025383, 0.002236701548099518, -0.007785644382238388, -0.01359135564416647, 0.009628430008888245, 0.008513995446264744, 0.002150553511455655, 0.0110121238976717, -0.006500736810266972, 0.004767825361341238, -0.0006265938864089549, -1.1796679496765137, -0.012592051178216934, -0.003036283887922764, 0.026534609496593475, -0.00280085951089859, 0.009583286009728909, -0.0011116217356175184, -0.002855763304978609, 0.0033554243855178356, 0.9806472063064575, 0.012701956555247307, 0.015435597859323025, 0.1664041429758072, -0.007803710643202066, -0.17596131563186646, 0.015273675322532654, -0.10878127813339233, 0.008217206224799156}, {-0.0020307940430939198, 0.007922089658677578, -0.004269401077181101, 0.013339366763830185, -0.0076467376202344894, -0.007335615810006857, -0.013250304386019707, 0.007408088073134422, 0.0024297127965837717, 0.0004923390224575996, -0.0027617220766842365, 0.004638085141777992, -0.005101864226162434, -0.014056052081286907, -0.23337271809577942, -0.007699030917137861, 0.02978718839585781, 0.0029684414621442556, 0.006956505589187145, -0.001069735735654831, 0.003133067162707448, 0.016426268965005875, 0.002809352707117796, 0.15454311668872833, 0.008774054236710072, 0.004296714905649424, -0.04369091987609863, 0.009264722466468811, 0.3873540461063385, -0.001410207012668252, 0.0632505714893341, -0.01526960451155901}, {-0.001444022636860609, -0.007025657221674919, -0.000885321875102818, -6.11911018495448e-05, -0.023505548015236855, -0.0012316637439653277, -0.00876102689653635, -0.007202802691608667, 0.0010826120851561427, 0.031713519245386124, 0.013659989461302757, -0.01122712716460228, -0.00972828920930624, 0.009949713945388794, -0.14886516332626343, 0.011431819759309292, -0.015989216044545174, -0.009928368963301182, 0.005998936016112566, 0.002873631427064538, -0.009234138764441013, -0.004594519734382629, -0.004772537853568792, -0.35107991099357605, -0.011245818808674812, -0.0053537944331765175, 0.04032008722424507, 0.011743421666324139, 0.1317799985408783, 0.0030277930200099945, 0.05281608924269676, -0.01392159704118967}, {0.0058815632946789265, 7.954682223498821e-05, -0.006439278367906809, 0.0076631647534668446, 0.009938880801200867, -0.0033009271137416363, -0.0042827678844332695, 0.007693497464060783, -0.0022824497427791357, 0.0016057147877290845, 0.0030045565217733383, -0.01655413955450058, -0.0014195980038493872, -0.004456321708858013, -0.038923852145671844, -0.005740263499319553, -0.005237349774688482, 0.0036832180339843035, 0.006338995415717363, 0.018141644075512886, -0.016147777438163757, 0.004249134100973606, -0.01402352750301361, 0.10585568845272064, 0.0015110471285879612, -0.01059320755302906, -0.026643283665180206, 0.0017998997354879975, 0.09484680742025375, -0.015764856711030006, -0.11976026743650436, 0.00036619335878640413}, {0.0010008665267378092, 0.015908492729067802, 0.00012229983985889703, -0.004491904750466347, 0.004087394569069147, -0.0011268595699220896, 0.014238872565329075, 0.0017944539431482553, -0.002509759273380041, 0.02115546353161335, -3.787740206462331e-05, 0.007491761818528175, -0.004582581575959921, -0.009799012914299965, -0.45803385972976685, 0.01537342183291912, 0.003079855116084218, 0.0020070867612957954, -0.00022547546541318297, -0.0019121336517855525, 0.019301867112517357, 0.006505061872303486, 0.002073424868285656, 1.2624058723449707, 0.0025747239124029875, 0.002082805847749114, 0.11825913935899734, 0.006527741439640522, -0.256416380405426, -0.0008959762053564191, -0.2483440637588501, 0.007288203574717045}, {-0.002530155936256051, -0.008548391051590443, -0.007444852497428656, -0.010617890395224094, -0.02418903261423111, 0.0040042828768491745, 0.006753379944711924, -0.01690155267715454, -0.004690302070230246, 0.0016861238982528448, 0.004702623002231121, 0.004272317048162222, -0.015466269105672836, -0.0007190515170805156, -0.4969441890716553, 0.002317943377420306, 0.007441326975822449, 0.000723439094144851, -0.004519951995462179, -0.00900985300540924, -0.006353121716529131, -0.01057642325758934, -0.005779067520052195, 0.5255576372146606, 0.0028045817743986845, 0.00299025047570467, -0.3427550494670868, 0.007227187044918537, 0.4377492070198059, 0.008214621804654598, 0.08459912985563278, 0.023599615320563316}, {2.4250979549833573e-05, 0.014879646711051464, 0.007443985901772976, -0.0033356589265167713, 0.014734802767634392, -0.001164303277619183, 0.01819773204624653, -0.004872916731983423, 0.01182973850518465, -0.01016391534358263, 0.008576411753892899, -0.0056812092661857605, -0.0029134361539036036, -0.004498813766986132, -0.6627014875411987, 0.007430760655552149, -0.01296484936028719, 0.0011937044328078628, 6.343177665257826e-05, -0.007977216504514217, 0.00548071414232254, 0.001423204317688942, 0.003369727171957493, 0.029297301545739174, 0.0010649426840245724, -0.017131434753537178, 0.040456775575876236, 0.0004991167224943638, 0.5567248463630676, 0.00762605108320713, 0.028159117326140404, 0.004833000712096691}, {0.017784560099244118, -0.01503084972500801, 0.0031452816911041737, 0.004058633930981159, 0.0032102300319820642, -0.005583110265433788, -0.00588392186909914, 0.00555363530293107, 0.01654793508350849, -0.009092814289033413, -0.017623692750930786, 0.006991959176957607, 0.009298461489379406, 0.007954993285238743, 0.20090226829051971, 0.006528057157993317, 0.018148688599467278, -0.00483626639470458, -0.006020371336489916, 0.008106672205030918, 0.001098755863495171, -0.005880808923393488, -0.0005303838988766074, 0.3777278959751129, -0.01936507225036621, -0.009759805165231228, -0.18135681748390198, 0.0053975749760866165, 0.036150604486465454, 0.005050295498222113, 0.512326717376709, 0.01282289158552885}, {-0.026956424117088318, -0.008229129016399384, -0.004681508522480726, -0.014875506050884724, 0.005958174355328083, 0.005628556478768587, -0.010568245314061642, 0.026947302743792534, 0.0012680746149271727, -0.00042169992229901254, 0.001443227636627853, 0.023861264809966087, 0.00034218691871501505, 0.002098370809108019, -0.3895093500614166, -0.023079967126250267, -0.016712255775928497, 0.006723967846482992, -6.414026574930176e-05, 0.003953143954277039, -0.0016416864236816764, -0.0028380213771015406, -0.004225980956107378, 0.2926005721092224, 0.00933800358325243, 0.00167893897742033, -0.13326670229434967, -0.014064829796552658, -0.050028007477521896, 0.0040122210048139095, 0.2890048921108246, 0.006288771517574787}, {-0.007442865055054426, -0.007902109064161777, 0.0115680992603302, 0.005339406430721283, 0.001628876430913806, 0.0054479921236634254, 0.006125304382294416, 0.01452439185231924, -0.0034129235427826643, 0.01428739633411169, -0.005331368185579777, -0.009131109341979027, 0.0006337643135339022, -0.013128823600709438, 0.4105270504951477, -0.00447887135669589, 0.011799340136349201, -0.012737746350467205, 0.011477652937173843, 0.013967319391667843, -0.009607161395251751, 0.006188502069562674, 0.0052192541770637035, 0.45628872513771057, -0.0034310691989958286, 0.004497977904975414, -0.1768135130405426, -0.007883845828473568, -0.31211724877357483, 0.0033530602231621742, -0.026590704917907715, -0.003933063708245754}, {0.002276442712172866, -0.006292302627116442, 0.013481419533491135, -0.00424002343788743, 0.010197299532592297, 0.011122428812086582, 0.01966913230717182, 0.01300590205937624, -0.007970969192683697, 0.011447901837527752, -0.00011976266978308558, 0.015313261188566685, -0.010593628510832787, 0.0061314706690609455, -0.06426399946212769, -0.002892873017117381, 0.00575006240978837, 0.006964169442653656, 0.0057107917964458466, -0.005680042318999767, 0.00027028564363718033, 0.003089661942794919, -0.009933832101523876, -0.013728807680308819, -0.008829531259834766, -0.0017621538136154413, 0.20726150274276733, -0.004697146359831095, 0.07801046222448349, -0.007681291084736586, -0.32097411155700684, 0.009935472160577774}, {0.008758855983614922, 0.006315196398645639, -0.0018635735614225268, 0.007324679754674435, 0.005611149128526449, -0.018922921270132065, -0.015588343143463135, -0.009080520831048489, 0.0020535418298095465, -0.0029790985863655806, 0.023318521678447723, -0.011344916187226772, 0.02420281060039997, 4.413968781591393e-05, -0.46848440170288086, -0.014841016381978989, -0.008614863269031048, 0.008982953615486622, 0.0009276865748688579, 0.013371354900300503, -0.016509877517819405, -0.006368910428136587, 0.0018372467020526528, -0.23373855650424957, 0.00858317967504263, -0.01243025716394186, 0.017713306471705437, -0.016987016424536705, 0.11757703125476837, 0.0037819710560142994, 0.20192962884902954, 0.0006145023507997394}, {0.01554787252098322, 0.009144335985183716, 0.01702907681465149, -0.0032226007897406816, -0.002682535210624337, 0.01634868048131466, -0.011824042536318302, -0.004820718429982662, 0.008365511894226074, 0.006553669925779104, 0.014465305022895336, 0.012175134383141994, -0.006570933386683464, 0.0007720082066953182, 0.24505598843097687, -0.00329802418127656, 0.011135966517031193, 0.006541151087731123, -0.008595877327024937, 0.0054241628386080265, -0.0071494379080832005, 0.01071198284626007, 0.0012556342408061028, -0.12640395760536194, 0.0019357363926246762, -0.012701309286057949, -0.3775402307510376, 0.013850390911102295, -0.043136149644851685, -0.001277127885259688, 0.08204292505979538, -0.01490710861980915}, {-0.0021176219452172518, 0.002322529675439, 0.003764472436159849, -0.006741084158420563, 0.008372187614440918, 0.004756909795105457, 0.004118060693144798, 0.005645400378853083, -0.02328726276755333, -0.007516835816204548, 0.006621628999710083, 0.0038796616718173027, 0.0053023770451545715, -0.012115346267819405, 0.8202271461486816, -0.025446411222219467, -0.0031855779234319925, -0.004971392918378115, 0.003037683665752411, 0.003254825482144952, -0.002215923508629203, 0.008548678830265999, 0.008702426217496395, 0.3225776255130768, 0.00012889729987364262, 0.0016892127459868789, 0.11496468633413315, 0.010663612745702267, -1.2762713432312012, 0.00445647444576025, -0.15179003775119781, -0.008432385511696339}, {-0.007211609277874231, 0.002618652768433094, 0.007094988599419594, 0.00783701241016388, 0.011721023358404636, 0.007862914353609085, 0.004752925131469965, -3.037452552234754e-05, -0.0032068490982055664, -0.005365646444261074, -0.017245080322027206, 0.00013922540529165417, 0.013874382711946964, 0.0012864847667515278, -0.21384300291538239, -0.0005649847444146872, 0.005020493641495705, 0.012764647603034973, -0.008748000487685204, -0.007312388624995947, 0.005509214475750923, -0.002540719462558627, 0.016168637201189995, -0.3466107249259949, 0.012979330494999886, 0.0010399409802630544, -0.02214794233441353, 0.007591506466269493, 0.07037059962749481, -0.017128316685557365, 0.08693868666887283, -0.004700100515037775}, {-0.004930516704916954, 0.007502839434891939, -0.014277207665145397, -0.010640017688274384, -0.0065117934718728065, -0.016336865723133087, 0.003035780508071184, -0.010678892955183983, 0.008283101953566074, 0.003373272018507123, 0.005330368876457214, 0.011160865426063538, 0.01572697050869465, 0.00850879680365324, 0.020727569237351418, 2.9011340302531607e-05, 0.01160039659589529, -0.003161126282066107, 0.013382529839873314, -0.00734221376478672, 0.0002315981691936031, -0.000512311642523855, -0.012558023445308208, -0.6705604791641235, -0.003101769369095564, -0.014398917555809021, 0.38617339730262756, -0.0013391149695962667, 0.009015788324177265, -0.011759154498577118, -0.10736070573329926, -0.0016808041837066412}, {0.0072573828510940075, -0.018392374739050865, 0.0027416381053626537, -6.798742106184363e-05, -0.008528739213943481, -0.016463182866573334, 0.024403389543294907, -0.0011778826592490077, -0.016620351001620293, 0.017617356032133102, -0.007598064839839935, -0.007012377958744764, 0.0007242438150569797, -0.0048832884058356285, -1.3651516437530518, 0.026255490258336067, -0.013616159558296204, -0.009224725887179375, -0.020416002720594406, 0.011714931577444077, -0.0040112691931426525, 0.005870649591088295, 0.007885345257818699, 2.324061155319214, -0.0021658556070178747, -0.015752047300338745, 0.10208664834499359, -0.0017149533377960324, -1.0704193115234375, 0.0076815164647996426, 0.031376827508211136, 0.007997196167707443}, {0.00014744378859177232, -0.0015701972879469395, -0.016790926456451416, 0.007573732640594244, -0.008459395729005337, 0.0005625305930152535, 0.014530109241604805, 0.0005184616893529892, -0.0029161800630390644, -0.003398157889023423, -0.011066228151321411, -0.004512940067797899, -0.0122006731107831, 0.006064934656023979, 0.1565837264060974, 0.0032377014867961407, -0.008910865522921085, 0.0106056435033679, 0.004971494432538748, -0.001449569477699697, -0.008200175128877163, 4.768905910168542e-06, -0.004328681156039238, -0.13323189318180084, 0.010730745270848274, -0.015544851310551167, -0.0933036357164383, -0.005734246224164963, 0.04134109988808632, -0.0034355991519987583, -0.19287608563899994, 0.0011965266894549131}, {0.024838179349899292, 0.0005282458732835948, 0.010260280221700668, -0.0060029844753444195, 0.0030939492862671614, 0.005670429207384586, 0.004931476432830095, -0.009641606360673904, -0.006438649725168943, 0.0023482264950871468, -0.002403541235253215, -0.015355953015387058, 0.009727099910378456, -0.0010791108943521976, 0.2561543881893158, -0.003976848442107439, -0.0012736336793750525, -0.0073223658837378025, -0.0021828922908753157, -0.013879027217626572, 0.01105247437953949, 0.003116693114861846, -0.006163254380226135, -0.6299400925636292, -0.012649842537939548, -0.01723998412489891, -0.01771613024175167, 0.019438009709119797, 0.11919927597045898, -0.018338661640882492, 0.052181176841259, -0.00680294306948781}, {-0.015161697752773762, -0.0001784571068128571, -0.007046030834317207, -0.0031867048237472773, -0.006449154578149319, 0.0061958846636116505, -0.0023907844442874193, -0.003385293297469616, -0.03531085327267647, 0.018384665250778198, -0.00900005828589201, 0.0021073766984045506, 0.00694469315931201, -0.008893064223229885, 0.3023476004600525, -0.01076789852231741, -0.0027471769135445356, -0.004104721359908581, -0.006209088023751974, 0.010696849785745144, -0.0037844653706997633, 0.0014932064805179834, 0.02183990739285946, -0.4186633229255676, 0.0043626511469483376, -0.0074135069735348225, 0.5857838988304138, -0.009519977495074272, -0.1797502189874649, 0.0034381647128611803, -0.6410287022590637, -0.0014796381583437324}, {-0.006276578642427921, -0.006274199113249779, -0.0016049925470724702, -0.004123723600059748, 0.0016468159155920148, -0.015270345844328403, -0.015923451632261276, -0.004808933008462191, 0.004562508314847946, 0.028876665979623795, 0.003222744446247816, -0.012744300998747349, -0.02779371850192547, 0.012297860346734524, 0.22207173705101013, -0.005261209327727556, 0.00042314178426750004, -0.004254560451954603, -0.01298255193978548, -0.005743745248764753, -0.010827851481735706, 0.007669815327972174, -0.017802530899643898, -0.5189176797866821, -0.0031304555013775826, -0.006461854558438063, -0.11281085759401321, 0.016324853524565697, 0.23696817457675934, -0.012472260743379593, 0.10298802703619003, 0.0044508869759738445}, {-0.002724876394495368, 0.01964869350194931, 0.007774542551487684, 0.016546690836548805, 0.017528587952256203, 0.00852831918746233, 4.7228382754838094e-05, 0.0018009091727435589, -0.0037172697484493256, 0.013681034557521343, 0.015686387196183205, 0.0010566451819613576, -0.001864162739366293, 0.0023356531746685505, -0.23285546898841858, 0.010006913915276527, 0.004246214404702187, -0.018879519775509834, 0.012069804593920708, 0.008761810138821602, -0.001468914677388966, 0.0017640492878854275, 0.003294730093330145, 0.48736482858657837, 0.0029223423916846514, -0.004856284242123365, 0.2094704508781433, 0.008683188818395138, -0.5847821831703186, -0.007074279710650444, 0.05152560770511627, 0.004965899512171745}, {0.0035760030150413513, 0.01139393262565136, -0.0049989414401352406, 0.0007940613431856036, -0.017718402668833733, -0.009772432036697865, -0.02865144982933998, 0.006409558467566967, -0.015636514872312546, 0.002757987240329385, 6.101052713347599e-05, 0.004612159449607134, -0.02147892862558365, 0.01087497640401125, -0.7384495139122009, 0.015360964462161064, -0.021944059059023857, -0.007850722409784794, 0.002680645789951086, -0.0012610352132469416, 0.0033808567095547915, -0.0038932361640036106, -0.009296849370002747, 0.2951136529445648, -0.003852335037663579, -0.006247336510568857, -0.04758556932210922, 0.008904773741960526, 0.3616582453250885, -0.00843017641454935, 0.013548927381634712, -0.015262666158378124}, {-0.0004030559794045985, 0.0034528726246207952, 0.004769741557538509, 0.003721817396581173, -6.194448815222131e-06, 0.011740771122276783, -0.007530970964580774, 0.020399941131472588, 0.010433320887386799, -0.006056727841496468, 0.003357219509780407, -0.017862988635897636, 0.01198579091578722, 0.019258353859186172, 0.4008706212043762, -0.020689163357019424, -0.017194073647260666, 0.0047072903253138065, 0.003797460114583373, -0.0011501730186864734, -0.005167662166059017, -0.0034417062997817993, 0.002968480810523033, 0.04373408481478691, 0.0009736701031215489, 0.002636969555169344, -0.09760556370019913, -0.020777499303221703, -0.09508555382490158, 0.004700637422502041, -0.008973909541964531, -0.014171566814184189}, {0.007237975485622883, -0.003725629299879074, 0.013132398016750813, 0.0009936424903571606, -0.006846067029982805, -0.0067800600081682205, 0.003184189787134528, 0.01920706033706665, 0.009824058972299099, 0.0030824244022369385, 0.010249833576381207, -0.011796649545431137, 0.004637728910893202, 0.006297433748841286, 0.5889281630516052, 0.009105811826884747, 0.0027500432915985584, -0.00011696307774400339, -0.007268097717314959, -0.00859475415199995, 0.0016599196242168546, 0.0038461785297840834, 0.021041378378868103, -1.698487639427185, -0.003974738996475935, 0.014801042154431343, 0.28037095069885254, 0.005055575631558895, 0.5542747974395752, 0.020315377041697502, -0.21590644121170044, -0.028899451717734337}, {-0.009162221103906631, -0.012015794403851032, -0.0019619574304670095, 0.013192735612392426, -0.0226267222315073, -0.006066983100026846, 0.004312629345804453, -0.01424369215965271, -0.010522907599806786, 0.007536787074059248, 0.007758601102977991, 0.007551184855401516, -0.005781649611890316, -0.017623838037252426, 1.3502254486083984, -0.0060758404433727264, -0.006591544020920992, 0.012749187648296356, -0.004695562180131674, -0.011447799392044544, -0.010217373259365559, -0.004329139366745949, 0.0004990142770111561, -0.5940030217170715, 0.00918797217309475, -0.01212659478187561, -0.023373352363705635, 0.0037901971954852343, -0.3806847035884857, 0.007269010413438082, -0.10614022612571716, -0.007450851146131754}, {0.009617422707378864, -0.008621505461633205, 0.008228783495724201, 0.003958456218242645, -0.010336953215301037, -0.013597335666418076, -0.011407758109271526, -0.014735420234501362, -0.014885293319821358, -0.006657357793301344, -0.010049696080386639, 0.00029347342206165195, -7.496064790757373e-05, -0.0034685146529227495, -1.6725513935089111, -0.005989335477352142, 0.0015497212298214436, -0.0034858097787946463, 0.005808827001601458, 0.010825107805430889, 0.00522603141143918, 0.013125617057085037, -0.011198324151337147, 0.030470654368400574, -0.01428094319999218, 0.001957545755431056, -0.10771672427654266, 0.008979503065347672, 1.3913073539733887, -0.016643013805150986, 0.19438879191875458, 0.004505982156842947}, {0.005410637240856886, 0.010245034471154213, 0.005656176712363958, -0.0027119636069983244, -0.0035516906064003706, 0.014571454375982285, 0.003362573217600584, 0.012215975672006607, 0.0035236296243965626, -0.00596311129629612, -0.0071174041368067265, 0.0006262697861529887, -0.012400461360812187, -0.007348352577537298, 0.30303826928138733, -0.004604985471814871, -0.0009926524944603443, -0.008325503207743168, 0.006290805991739035, -0.02317984774708748, 0.006169741041958332, 0.026690863072872162, 0.0057903784327209, 0.1066274419426918, 0.00012071592209395021, -0.014785902574658394, 0.10702039301395416, 0.0022940412163734436, -0.15024115145206451, 0.004243799485266209, -0.10155980288982391, -0.008891522884368896}, {0.0035652650985866785, -0.0038948305882513523, -0.011767791584134102, -0.0005329680861905217, 0.01856575720012188, 0.003921880852431059, -0.009553647600114346, -0.0032044153194874525, 0.007337922230362892, -0.01255046296864748, 0.010489868000149727, 0.009905371814966202, 0.013575769029557705, 0.004879112355411053, -0.24333639442920685, -0.006192863918840885, -0.008766820654273033, -0.00259797484613955, 0.007609974592924118, -0.00032413742155767977, -0.0034344568848609924, -0.006053427699953318, 0.008196717128157616, -0.11894519627094269, 0.010517053306102753, 0.010760175995528698, 0.1964629590511322, -0.006653232965618372, 0.4188118577003479, -0.010721706785261631, -0.09134767949581146, -0.009709681384265423}, {-0.006449736189097166, -0.015788108110427856, -0.0021414512302726507, 0.006361076142638922, 0.016664670780301094, 0.00991720799356699, -0.004131148103624582, -0.006248342338949442, 0.011619837023317814, -0.007588509935885668, 0.016263822093605995, -0.005134233273565769, -0.004212872125208378, -0.022540798410773277, -0.625268280506134, 0.031559523195028305, -0.00818229652941227, 0.014458773657679558, 0.008201071992516518, 0.002214952139183879, 0.015234443359076977, 0.00973846297711134, -0.013752369210124016, -0.2578301727771759, 0.007367415819317102, -0.011270848102867603, 0.07269088923931122, 0.009107385762035847, 0.4155069887638092, -0.0048271347768604755, -0.0979328528046608, -0.016687652096152306}, {-0.00974266603589058, -0.002495769876986742, -0.005610800348222256, 0.01343715563416481, 0.016050394624471664, -0.012997252866625786, 0.0060952650383114815, 0.020790740847587585, 0.0008788644336163998, -0.0038002978544682264, 0.002508474513888359, 0.006295462138950825, -0.011446456424891949, -0.005083139054477215, 0.5062437057495117, 0.00047726588672958314, -0.008497948758304119, 0.004210915882140398, -0.0041810572147369385, -0.003350257407873869, 0.010496716015040874, -0.01637771539390087, -0.0006971830152906477, 0.20804108679294586, -0.006916761863976717, -0.006009907461702824, -0.668445348739624, 0.03225305303931236, 0.41392770409584045, 0.0017533086938783526, 0.684627890586853, -0.008310899138450623}, {0.005823182407766581, -0.016321910545229912, 0.0016772137023508549, -0.02053205855190754, 0.0036174103152006865, -0.0005237669101916254, 0.00017618406855035573, 0.007421230431646109, 0.008108151145279408, -0.011339559219777584, 0.0015466635813936591, 0.0012400727719068527, 0.015060065314173698, 0.018037188798189163, 0.24475659430027008, 0.010621190071105957, 0.004432473331689835, -0.012011713348329067, 0.00786958821117878, -0.0031002582982182503, -0.002016240730881691, -0.0017029924783855677, 0.0025501931086182594, -0.1465919464826584, 0.009640631265938282, 0.005878642201423645, -0.09938641637563705, -0.0012160106562078, 0.0810091644525528, -0.007215588353574276, -0.14088578522205353, 0.014349019154906273}, {-0.01445271261036396, -0.0039536659605801105, 0.007308544125407934, 0.01677539385855198, -0.014461093582212925, -0.006022934801876545, -0.014473466202616692, 0.0033406487200409174, 0.000585090194363147, -0.002978863660246134, -0.011088013648986816, -0.014763121493160725, 0.001451162388548255, 0.006044709589332342, -1.4867650270462036, -0.0001726146147120744, -0.0005400796653702855, 0.001896955887787044, 0.017047196626663208, 0.0027918117120862007, 0.004399564582854509, -0.0028106081299483776, 0.00853421539068222, -1.550203800201416, 0.0014336243038997054, 0.0177525095641613, 0.08078861236572266, 0.027229931205511093, 2.9626643657684326, 0.00196943455375731, 0.08330786973237991, -0.001244169077835977}, {0.01754004694521427, -0.019899804145097733, -0.004572614096105099, 0.0004966125707142055, 0.001772262156009674, -0.010251414962112904, 0.002061569131910801, 0.0067234942689538, -0.003145230235531926, 0.008230249397456646, -0.030690550804138184, 0.01037265732884407, 0.005093130748718977, -0.016029536724090576, -0.13168564438819885, 0.0020986709278076887, 0.00786527805030346, -0.011094396002590656, 0.001656935433857143, 0.002902185544371605, -0.010433467105031013, -0.018601451069116592, -0.013594381511211395, -0.24490900337696075, 0.012414265424013138, 0.009312969632446766, 0.00518338568508625, -0.0009912968380376697, 0.29764750599861145, -0.0002744950179476291, 0.10547535866498947, 0.0036586110945791006}, {0.0027460139244794846, -0.006297809537500143, -0.001511028385721147, 0.0030022799037396908, 0.011438055895268917, -0.002536744112148881, -0.013736097142100334, -0.008454354479908943, 0.011586259119212627, 0.0007765444461256266, -0.0045959013514220715, -0.006752092856913805, -0.0068331812508404255, 0.005751891061663628, -0.17260921001434326, -0.006476723589003086, -0.008793479762971401, -0.002519527217373252, -0.017870385199785233, -0.02568676881492138, 0.003237695898860693, -0.006256421096622944, -0.009183583781123161, 0.19640333950519562, 0.0021621217019855976, 0.0015574503922834992, 0.03456667438149452, -0.0006781886331737041, 0.01612955890595913, -0.007716079242527485, -0.15233279764652252, 0.001747951959259808}, {-0.0019575064070522785, 0.005940387491136789, -0.001458074082620442, 0.009181026369333267, 0.013560578227043152, -0.0011109039187431335, 0.030923089012503624, 0.0032757504377514124, -0.002511457074433565, -0.016591444611549377, -0.002846882212907076, 0.012200052849948406, -0.009413466788828373, 0.011368262581527233, 0.08931370824575424, -0.009911381639540195, -0.02766033634543419, 0.01389022171497345, 0.010437351651489735, 0.010145225562155247, 0.008971239440143108, -0.009308146312832832, 0.015565944835543633, 0.33902063965797424, -0.007686664815992117, -0.004978158511221409, -0.044367510825395584, 0.019084924831986427, -0.18147297203540802, -0.01026266347616911, -0.030651401728391647, 0.010203282348811626}, {-0.017292087897658348, 0.012996641919016838, -0.017494672909379005, 0.00942771602421999, 0.00749240443110466, 0.0019768907222896814, -0.005806208122521639, -0.0011257481528446078, -0.006780252326279879, -0.001635220367461443, 0.005425858777016401, -0.003909675870090723, 0.0007855735020712018, -0.006619018968194723, 0.2504537105560303, 0.008543318137526512, 0.009943854995071888, -0.009160323068499565, 0.005414943210780621, -0.0113995261490345, -0.002265926683321595, 0.007487474475055933, -9.071653767023236e-05, -0.10596290230751038, 0.0032114547211676836, -0.006379051133990288, -6.837134424131364e-05, 0.014887934550642967, -0.08451256155967712, -0.004256255924701691, -0.19972585141658783, -0.0033021036069840193}, {0.0009327666484750807, -0.005813489202409983, 0.004444671794772148, 0.005511227063834667, -0.0007594052003696561, -0.005138691049069166, -0.002266614930704236, 0.0013692916836589575, -0.008955275639891624, 0.006600010208785534, 0.0008694112184457481, 0.004535631276667118, -0.0018565395148470998, 0.005904379300773144, 0.4420353174209595, -0.01284870132803917, 0.002169910119846463, 0.003780301660299301, 0.000637260964140296, -0.009815729223191738, 0.0005078939720988274, -0.007631266489624977, -0.028898706659674644, -0.548669159412384, 0.016903668642044067, -0.0062558394856750965, 0.0886930525302887, 0.0034056762233376503, 0.017483429983258247, 0.0018228074768558145, -0.1768854558467865, 0.011315185576677322}, {0.01902051828801632, -0.006781934294849634, -0.008779683150351048, 0.0006620794301852584, -0.004076472949236631, -0.007927464321255684, 0.008112008683383465, 0.019992563873529434, -0.00413030618801713, -0.010017609223723412, 0.005538196302950382, 0.018452579155564308, -0.01964801736176014, -0.0035155145451426506, -0.2643126845359802, 0.018908459693193436, 0.01235621701925993, -0.010799902491271496, 0.0021849109325557947, -0.006271360442042351, 0.01995631866157055, 0.005527618806809187, -0.003603453515097499, -0.2958451509475708, -0.02416081540286541, -0.004408963955938816, 0.006222996395081282, -0.01577150821685791, 0.0361754335463047, 0.012246440164744854, -0.15808206796646118, 0.010441665537655354}, {0.023440390825271606, 0.0073818243108689785, -0.017572496086359024, -0.014769116416573524, 0.014477889984846115, -0.001897075679153204, -0.008356337435543537, 0.0036718787159770727, -0.015581806190311909, -0.0006635200115852058, -0.01567976363003254, -0.002482911804690957, -0.0010862593771889806, 0.004881864879280329, -0.12488264590501785, -0.00976424477994442, -0.013755624182522297, -0.009036331437528133, 0.014809899032115936, 0.010898157954216003, 0.002037925412878394, -0.006952271331101656, -0.002684419509023428, 0.18991652131080627, 0.008377443999052048, 0.00633557653054595, -0.2840627431869507, -0.00897313468158245, 0.18183092772960663, -0.007438370026648045, 0.13352936506271362, 0.020512551069259644}, {-0.0040080430917441845, -0.0026029085274785757, -0.0038185666780918837, 0.0021411569323390722, -0.004656312055885792, 0.004800115711987019, -0.006765441037714481, 0.009390132501721382, -0.004619928076863289, 0.006904000882059336, 0.005745721515268087, -0.005819798447191715, 0.017265668138861656, 0.0015195858431980014, -0.45111972093582153, 0.005955245811492205, -0.015628300607204437, 0.0037712689954787493, -0.0017465444980189204, 0.007172157522290945, -0.0033244991209357977, 0.0021839544642716646, 0.010260794311761856, -0.36153799295425415, -0.013904083520174026, -0.007982464507222176, -0.1787119209766388, -0.012075561098754406, 0.8011026382446289, -0.012250746600329876, 0.11997444927692413, 0.008376579731702805}, {0.012970746494829655, 0.009845954366028309, 0.02247481234371662, -0.001103292335756123, -0.01677011139690876, -0.013901110738515854, 0.008008238859474659, -0.007357005961239338, -0.021060211583971977, 0.0006037743296474218, -0.001413755351677537, 0.019221359863877296, -0.0042940410785377026, -0.009336289018392563, 0.005575745832175016, -0.0041214050725102425, -0.024974126368761063, 0.005963467992842197, -0.008215995505452156, 0.002189401537179947, 0.018180539831519127, 0.0010617149528115988, 0.008678280748426914, 0.008244206197559834, 0.005856512114405632, -0.017841298133134842, -0.16339123249053955, -0.012419173493981361, -0.08118274807929993, 0.0030208402313292027, 0.2763150930404663, 0.02407701127231121}, {0.003031439147889614, -0.008826382458209991, 0.005060946103185415, 0.006841855589300394, -0.0024683955125510693, -0.005229911766946316, 0.0176029521971941, -0.02448096312582493, -0.011844952590763569, 0.001622003735974431, 0.00594465434551239, -0.003186138579621911, 0.005208441987633705, 0.009758396074175835, 0.5698365569114685, 0.024237653240561485, -0.009737740270793438, -0.00403040973469615, -0.0005675707361660898, 0.002637488767504692, 0.012766215950250626, -0.010895212180912495, 0.00481833191588521, -1.0418262481689453, 0.0029250127263367176, -0.007139923516660929, 0.06632966548204422, 0.01657848246395588, -0.05934548005461693, 0.006678505800664425, 0.05749554932117462, 0.011837730184197426}, {-0.010217102244496346, 0.010798892006278038, -0.008999570272862911, -0.008461098186671734, -0.001681999652646482, 0.00997238326817751, 0.007681122049689293, 0.000908014306332916, -0.00903762225061655, 0.018558241426944733, -0.007612366694957018, -0.0023410136345773935, -0.012730111368000507, -0.0033033331856131554, -0.2758776843547821, -0.0011423821561038494, -0.013387519866228104, -0.01224770862609148, -0.016398532316088676, 0.021135613322257996, -0.010958963073790073, 0.0025764780584722757, -0.019830631092190742, 0.4975963830947876, 0.0040962169878184795, 0.005981777794659138, 0.08229994028806686, -0.009254519827663898, -0.13810934126377106, -0.0036222778726369143, -0.21904848515987396, 0.011421882547438145}, {-0.022603658959269524, 0.0013503023656085134, -0.009472512640058994, 0.0013420077739283442, 0.0125869857147336, -0.010185611434280872, -0.004413809161633253, -0.0036155921407043934, 0.0213411096483469, -0.01709156297147274, 0.00854011345654726, 0.008685833774507046, -0.018738605082035065, -0.00809765700250864, 0.368906170129776, 0.008181418292224407, -0.004807020537555218, -0.010620487853884697, -0.006827911362051964, -0.008791734464466572, -0.005570261739194393, 0.0013490933924913406, 0.013248633593320847, 0.42548853158950806, -0.017552459612488747, -0.0013437522575259209, -0.11284719407558441, 0.0060781375505030155, -0.3322095274925232, -0.002617528894916177, 0.16972659528255463, -0.014463834464550018}, {-0.0033108009956777096, -0.007694424595683813, -0.012786922045052052, -0.0023284261114895344, -0.005900037009268999, -0.007698335684835911, -0.002966176485642791, -0.001186951994895935, 0.011085158213973045, 0.013052334077656269, -0.010705158114433289, -0.005866076331585646, 0.006620087660849094, -0.003868864383548498, 0.08326370269060135, 0.003001841949298978, -0.01750604622066021, 0.006770951207727194, 3.768940223380923e-05, -0.006130393128842115, -0.0034218127839267254, 0.041497550904750824, 0.014082961715757847, -0.5843687057495117, -0.013734279200434685, -0.009123878553509712, -0.0654427707195282, 0.00021640605700667948, 0.2664254307746887, 0.01444550696760416, 0.1621028631925583, 0.007346660364419222}, {0.018509112298488617, -0.01721794717013836, 0.009577115997672081, -0.006857880391180515, 0.004515584092587233, 0.002752348082140088, -0.006390290334820747, -0.00916062481701374, -0.013312472961843014, 0.00040646849083714187, 0.008721006102859974, 0.0022294465452432632, -0.009994970634579659, 0.021376347169280052, 0.10251080244779587, -0.006550557445734739, -0.012539821676909924, 0.01312266569584608, -0.00358596327714622, 0.003882665652781725, 0.008158670738339424, -0.005739368032664061, 0.0007123788818717003, -0.28391480445861816, 0.013638470321893692, 0.021119995042681694, -0.06386759877204895, -0.0007768702926114202, 0.21159756183624268, -0.005913428030908108, 0.0651780515909195, 0.0014803486410528421}, {-0.013165304437279701, 0.0011390955187380314, 0.007554082199931145, -0.001486530527472496, -0.0191736351698637, -0.0032908457797020674, 0.011871211230754852, -0.0012072331737726927, -0.002518640598282218, -0.01094064675271511, -0.00392746739089489, 0.00560760498046875, 0.0021769190207123756, -0.004754045512527227, 0.3384712040424347, 0.005864543840289116, -0.0033918197732418776, 0.0011282203486189246, 0.0037187940906733274, -0.004291074816137552, 0.018019916489720345, 0.012083296664059162, 0.0021561007015407085, 0.13577774167060852, 0.009853064082562923, -0.005021423101425171, -0.02101955935359001, 0.012028996832668781, -0.1798419952392578, -0.006961624603718519, 0.0963880866765976, 0.0018406263552606106}, {0.019628217443823814, 0.011682208627462387, -0.0031137766782194376, 0.002160498406738043, 0.003360369708389044, 0.006636593025177717, 0.013032817281782627, 0.012198226526379585, -0.003328061895444989, 0.002404195489361882, -0.007684783078730106, 0.01303787063807249, -0.01855078525841236, -0.004257773514837027, -1.0457152128219604, 0.007229130249470472, 0.021391436457633972, -0.002803961280733347, -0.0023852218873798847, -0.017485974356532097, 0.02014719694852829, -0.006288233678787947, -0.021438810974359512, 1.8846570253372192, -0.006024859379976988, 0.0017776757013052702, 0.036348748952150345, -0.0006231901352293789, -0.7821537852287292, -0.01069655828177929, 0.18447738885879517, 0.017400145530700684}, {-0.022588247433304787, -0.007548258174210787, -0.014757071621716022, -0.0020004487596452236, 0.0033964866306632757, -0.005488998256623745, -0.011479289270937443, -0.008782561868429184, -0.008151991292834282, -0.018593844026327133, -0.006563406903296709, -0.00513001112267375, -0.006785161793231964, 0.0006852023652754724, 1.6174055337905884, -0.0017089212778955698, 0.004414996597915888, 0.011783239431679249, 0.010732526890933514, 0.024278951808810234, -0.0023572470527142286, 0.010512782260775566, 0.0074454001151025295, -0.4266422390937805, -0.010745665989816189, -0.0031049479730427265, -0.1543857604265213, 0.005794717464596033, -0.8473488092422485, -0.004376482684165239, -0.1433885246515274, -0.0011289996327832341}, {0.005837023723870516, -0.013592798262834549, 0.012771695852279663, 0.010874058119952679, 0.008008037693798542, 0.02007785439491272, 0.01057360414415598, -0.005801644641906023, -0.001967875985428691, 0.00867527723312378, -0.018228719010949135, -0.010811449028551579, 0.003952734172344208, 0.015370783396065235, 0.15136459469795227, -0.0029131630435585976, 0.009424343705177307, -0.016594499349594116, 0.002321006963029504, 0.013497188687324524, -0.010374142788350582, 0.007239442318677902, -0.003956958651542664, 0.19907814264297485, 0.001047651981934905, 0.01837150938808918, 0.006724705919623375, 0.002238222397863865, -0.3674190640449524, 0.005795696284621954, 0.02480938844382763, 0.001725447247736156}, {-0.014439580030739307, -0.004599854815751314, 0.01012462843209505, 0.005989979952573776, -0.0041777146980166435, -0.009534936398267746, 0.01352986041456461, -0.009218992665410042, -0.004151993431150913, -0.003498398931697011, 0.011118186637759209, 0.0015553385019302368, -0.008083485998213291, -0.009769090451300144, -0.8626474142074585, -0.0048417868092656136, -0.008623704314231873, 0.006527441553771496, -0.0066496096551418304, -0.009729570709168911, -0.011154022999107838, -0.010989075526595116, -0.02114090882241726, 0.3730989694595337, -0.0006822225404903293, 0.015177901834249496, 0.2929590940475464, -0.01224274281412363, 0.21116682887077332, -0.003673003753647208, -0.06494737416505814, -0.014637993648648262}, {-0.004134416580200195, 0.012527878396213055, -0.002450853120535612, -0.0023727959487587214, -0.02109496109187603, -0.003037166316062212, -0.0010187625885009766, 0.0058116549625992775, 0.011460504494607449, 0.012921162880957127, -0.010006827302277088, 0.0020269732922315598, 0.013205758295953274, 0.015096055343747139, 0.37236037850379944, -0.0023026608396321535, 0.0002922413405030966, 0.014931289479136467, 0.0010789281222969294, -0.01581927016377449, -0.008498829789459705, 0.023372719064354897, 0.0023265269119292498, 0.18655237555503845, 0.010421783663332462, -0.004358499776571989, -0.27717384696006775, -0.0011285042855888605, -0.16275887191295624, -0.0008407454588450491, 0.12687961757183075, -0.004245776683092117}, {0.012802030891180038, -0.015221375040709972, 0.015742814168334007, -0.018601732328534126, -0.001977483509108424, -0.0013727931072935462, -0.011221912689507008, 0.0017512760823592544, 0.018630173057317734, -0.014433461241424084, -0.005398239940404892, 0.00685573136433959, -0.0025759818963706493, 0.0097916629165411, 0.9530646204948425, 0.0037708859890699387, 0.004266709089279175, 0.0026690831873565912, 0.013508708216249943, -0.0009963734773918986, -0.003146034898236394, 0.0015091135865077376, -0.0032236760016530752, -0.635870635509491, -0.003747125156223774, 0.004741987679153681, -0.06863682717084885, 0.0037557913456112146, -0.1955101490020752, 0.0012342226691544056, 0.014297571033239365, 0.008904160000383854}, {-0.0012489977525547147, 0.011947820894420147, 0.012303272262215614, 0.0030925648752599955, 0.010470572859048843, -0.01340505015105009, -0.006427356507629156, 0.0013736099936068058, -0.0045744250528514385, -0.004029478877782822, 0.00040875314152799547, 0.01825438067317009, -0.001912651932798326, -0.0024170319084078074, 0.25735709071159363, -0.02255021035671234, -0.0017869871808215976, -0.013304050080478191, 0.0008061480475589633, 0.004235424101352692, 0.010700538754463196, 0.00021377028315328062, 0.0007801075698807836, 0.3383835554122925, -0.011431205086410046, 0.002060513710603118, 0.13629639148712158, 0.003176787169650197, 0.31675586104393005, 0.008340252563357353, -0.6843183636665344, 0.006333749741315842}, {-0.002113792346790433, -0.00664024893194437, 0.0005575971445068717, -0.0029642863664776087, -0.006586432456970215, 0.005157249514013529, 0.004441434517502785, 0.00512718828395009, -0.014609389938414097, 0.007003484293818474, -0.013303147628903389, -0.003970079589635134, 0.005253429524600506, 0.0014758540783077478, -0.3142129182815552, -0.012097977101802826, -0.017966298386454582, -0.0025451036635786295, -0.0012032986851409078, 0.003375650616362691, 0.0031533571891486645, 0.0012782609555870295, -0.00669381208717823, 0.4224318861961365, -0.02026183344423771, 0.000287674309220165, 0.0024122425820678473, 0.010587849654257298, -0.269916832447052, 0.00444750813767314, 0.199297696352005, 0.0014934205682948232}, {0.015563038177788258, -0.005223121959716082, -0.025748640298843384, 0.0033095479011535645, -0.007855945266783237, -0.012004737742245197, -0.013801364228129387, -0.007944470271468163, 0.005781514570116997, -0.018509315326809883, 0.005777093581855297, -0.0016509370179846883, 0.019321558997035027, -0.01254176814109087, -0.3377060890197754, -0.0065797907300293446, -0.0183576513081789, -0.011136524379253387, -0.0030230635311454535, 0.005208856891840696, -0.0058237360790371895, 0.00705189211294055, 0.0023318955209106207, -0.3921847939491272, 0.015042334794998169, -0.002433893969282508, 0.8170562982559204, 0.004379053600132465, 0.097404845058918, 0.002283552661538124, -0.2295650839805603, -0.011813889257609844}, {-0.0029765754006803036, 0.0008254364947788417, 0.009356972761452198, -0.021826932206749916, -0.0005045282887294888, 0.007350175175815821, -0.005426681134849787, -0.000619170255959034, 0.0007737404084764421, 0.003561189631000161, -0.008540065959095955, -0.0028692358173429966, -0.020173395052552223, -0.007279981859028339, 0.2599954903125763, -0.015583153814077377, -0.01231780368834734, 0.020007021725177765, -5.108361801831052e-05, 0.004404677078127861, 0.009672964923083782, -0.0010527782142162323, 0.008109520189464092, 0.4281577169895172, 0.013970443047583103, -0.0028387364000082016, 0.0031816849950701, 0.003983506932854652, -0.14707981050014496, -0.0005422119866125286, 0.15751107037067413, 0.003088477300480008}, {0.013627284206449986, 0.014053290709853172, -0.01082073524594307, -0.013644125312566757, 0.012959392741322517, 0.008170424029231071, -0.00318147917278111, 0.003190061077475548, -0.008170626126229763, 0.00313971028663218, -0.0017763407668098807, -0.005043341778218746, 0.00823121052235365, -0.016965802758932114, 0.17181998491287231, 0.009897852316498756, -0.00712544983252883, 0.009801017120480537, 0.023363636806607246, -0.006887319963425398, 0.01949547417461872, 0.006263053044676781, 0.02166507951915264, 0.21654951572418213, 0.0014148041373118758, -0.004893074743449688, -0.029191140085458755, 0.004014397040009499, -0.13877259194850922, -0.01643279194831848, 0.09992184489965439, 0.0025923363864421844}, {0.0006360338302329183, -0.0016398201696574688, 0.003505461849272251, 0.0015616829041391611, -0.0018346129218116403, -0.006639342289417982, 0.0199726652354002, 0.00026425730902701616, -0.005834260024130344, 0.003432592609897256, 0.003405822440981865, -0.020064804702997208, 0.016812171787023544, 0.007059566210955381, 0.3310113847255707, -0.00921227689832449, -0.0027926694601774216, 0.0017679043812677264, -0.0009098667651414871, 0.011114650405943394, -0.020039763301610947, -0.0014208940556272864, 0.0020626969635486603, 0.2695602774620056, 0.00718164537101984, -0.006235257722437382, -0.8823777437210083, -0.018384264782071114, 0.25546756386756897, 0.022362882271409035, 0.3189644515514374, -0.008116119541227818}, {0.00748443603515625, 0.00013410650717560202, -0.0030629553366452456, 0.007989170029759407, -0.0032620641868561506, 0.026408597826957703, 0.009098893031477928, -0.01451032143086195, 0.006967493332922459, -0.008825352415442467, 0.005585421342402697, -0.01116385031491518, -0.014785457402467728, 0.01799873076379299, -1.255319356918335, 0.0072313291020691395, 0.001909528742544353, -0.0034397768322378397, -0.008941723965108395, 0.0007257435936480761, 0.012635545805096626, 0.01064046286046505, -0.004677245393395424, -0.8649332523345947, 0.0013814312405884266, 0.00815717875957489, -0.06808560341596603, 0.009183600544929504, 1.5310096740722656, 0.011146892793476582, 0.00672684283927083, -0.0038288035430014133}, {-0.00047781309694983065, 0.0033042216673493385, -0.003965646959841251, -0.009826133958995342, 0.013776767067611217, 0.0018595665460452437, -0.015334200114011765, 0.0018152964767068624, 0.00142335903365165, 0.0103301377967, 0.003392501501366496, 0.004275232087820768, 0.0037648009601980448, -0.010288736782968044, 0.2371048629283905, -0.010602733120322227, -0.008282320573925972, -0.006103511434048414, 0.0010306347394362092, -0.0036692942958325148, 0.0048039574176073074, -0.002602599561214447, 0.014879420399665833, -0.3338552713394165, 0.004923210479319096, -0.01546389702707529, -0.36034852266311646, -0.007725386880338192, 0.32645460963249207, 0.011728424578905106, 0.24288462102413177, -0.0017459341324865818}, {0.0027565746568143368, 0.025403188541531563, 0.00919746607542038, -0.005599241703748703, 0.018812919035553932, -0.0039849551394581795, 0.0001252443325938657, 0.005421467125415802, -0.01276436448097229, 0.011045243591070175, -0.017174340784549713, -0.0001776702411007136, 0.005171193741261959, -0.008032277226448059, -0.1377897411584854, 0.004888483323156834, -0.008942806161940098, -0.0027727552223950624, 0.010984940454363823, 0.0033412848133593798, 0.020906882360577583, -0.004760235548019409, -0.005664887838065624, 0.5305887460708618, 0.01652670092880726, 0.014841871336102486, 0.11660425364971161, -0.0027648629620671272, -0.11661919206380844, 0.007085944060236216, -0.2246445268392563, -0.0008204415207728744}, {-0.00022477767197415233, 0.009158161468803883, -0.01250419206917286, -0.003306814469397068, 0.012559326365590096, -0.011697704903781414, 0.012545054778456688, -0.010850895196199417, -0.005646504461765289, -0.007477669045329094, -0.0002770356077235192, 0.001565182814374566, -0.015998689457774162, -0.012452015653252602, 1.7656011581420898, -0.005417088512331247, -0.0016427547670900822, 0.005309923551976681, 0.003408732358366251, 0.01504663098603487, -0.01361769251525402, 0.012989535927772522, 0.004410890396684408, -1.5855681896209717, 0.0038858281914144754, 0.0037905843928456306, -0.20884591341018677, -0.02290327288210392, -0.4759805500507355, 0.021604230627417564, 0.2951700985431671, 0.01769861951470375}, {0.0017966425511986017, 0.017799288034439087, 0.00420044781640172, -0.017748834565281868, -0.0016681634588167071, -0.012332248501479626, 0.005775431636720896, -0.010163860395550728, 0.011131905019283295, -0.005558067467063665, 0.01615777611732483, 0.005891730543226004, 0.004146280698478222, 0.002029265044257045, -0.010976587422192097, 0.019239867106080055, 0.00026862084632739425, 0.01593012735247612, -0.0009544849744997919, -0.004906734451651573, 0.004020539578050375, 0.005391024053096771, -0.00660170940682292, 0.45750147104263306, 0.0015382011188194156, -0.016252359375357628, -0.38226866722106934, 0.0005818378995172679, -0.010785236954689026, 0.014530225656926632, 0.2191477119922638, -0.002025697147473693}, {0.005316906608641148, -0.0026940149255096912, -0.014170666225254536, -0.018377073109149933, 0.0026366126257926226, -0.0004058373742736876, -0.008143807761371136, -0.01609928533434868, -0.02182580716907978, 0.006092646159231663, -0.0008604954928159714, 0.006753177382051945, 0.007858925499022007, 0.00897094514220953, -0.036426667124032974, 0.0028746898751705885, -0.023744987323880196, 0.010497182607650757, 0.0019445320358499885, -0.003362382296472788, -0.018587540835142136, 0.02509313076734543, 0.013758226297795773, -0.3800261318683624, -0.014136193320155144, -0.02089298516511917, -0.11793113499879837, -0.00805977825075388, 0.03973245248198509, 0.0032156535889953375, 0.3244277834892273, -0.01887378841638565}, {-0.02880181185901165, -0.007051622495055199, -0.00589545164257288, -0.006742553785443306, 0.00983564741909504, -0.006779010873287916, 0.0014120660489425063, 0.00015922950115054846, 0.006220450159162283, 0.010863421484827995, -0.001480920473113656, 0.008634092286229134, -0.0019597122445702553, -0.00038646921166218817, 0.1678469032049179, -0.005333075299859047, -0.0014059088425710797, 0.000812768645118922, -0.004236276727169752, 0.010692770592868328, -0.006624298635870218, -0.007290728390216827, -0.00678026769310236, -0.20054535567760468, 0.011567030102014542, -0.0028098549228161573, -0.1252186894416809, 0.00555866165086627, 0.04496512934565544, -0.0022843836341053247, 0.2468954175710678, 0.004149012267589569}, {-0.004707233048975468, 0.00027366288122721016, -0.0001731965458020568, 0.013013309799134731, -0.0024421566631644964, -0.01034445222467184, 0.00574524188414216, -0.007184335961937904, -0.009498191066086292, -0.003286544233560562, -0.008744394406676292, 0.006289069540798664, -0.0013141590170562267, -0.003522701095789671, -0.1181517243385315, 0.0008503549615852535, 0.015213673934340477, 0.010946921072900295, -0.004907484631985426, -0.005812692455947399, 0.010824507102370262, -0.00028162545640952885, 0.009765516966581345, -0.07817203551530838, 0.005382970441132784, -0.0239923857152462, 0.496378093957901, -0.004277516156435013, -0.3438139855861664, -0.002131762681528926, -0.09622084349393845, 0.0011519779218360782}, {-0.014255745336413383, -0.00029268028447404504, 0.003689025528728962, 0.003769412636756897, 0.01690921001136303, -0.008463806472718716, -0.0130233746021986, 0.0044684638269245625, 0.002310454845428467, -0.010653859004378319, -0.003109067678451538, -0.0056071835570037365, -0.0020489420276135206, 0.009397240355610847, -0.3098646402359009, 0.0030390960164368153, -0.003021290758624673, 0.007654760964214802, 0.0010399899911135435, -0.013086189515888691, 2.669937202881556e-05, 0.003226072760298848, -0.00922587513923645, -0.236496239900589, 0.014671772718429565, -0.025691239163279533, 0.9494403004646301, 0.006225929129868746, -0.6951291561126709, -0.010681514628231525, -0.6733139753341675, -0.007314205635339022}, {-0.00946354866027832, -0.004505178891122341, 0.02119985595345497, -0.00027933079400099814, 0.009985055774450302, 0.005745877977460623, -0.0006761772674508393, 0.01134219765663147, -0.015182734467089176, 0.0008882851689122617, 0.0034279723186045885, -0.005870956461876631, 0.0034488036762923002, -0.00983402132987976, 0.21495701372623444, -0.011882791295647621, -0.001189701957628131, -0.023314369842410088, 0.00025165945407934487, -0.0019035397563129663, -0.011296069249510765, 0.006465771235525608, -0.014073197729885578, 0.24900880455970764, -0.010523264296352863, -0.0078675402328372, -0.052546605467796326, -0.0018632832216098905, -0.13615882396697998, -0.0007071774452924728, 0.028355196118354797, -0.010111779905855656}, {-0.006973207462579012, -0.012874656356871128, 0.008979092352092266, 0.005166154820472002, -0.007374165579676628, 0.0024899826385080814, 0.013234458863735199, 0.0036252655554562807, 0.016378430649638176, -0.018262425437569618, -0.010200433433055878, 0.018257366493344307, -0.002106489846482873, 0.010321320965886116, 0.042012546211481094, 0.013489874079823494, 0.0037553401198238134, -0.0015309287700802088, 0.005858900956809521, 0.00474808644503355, 0.008774954825639725, 0.0021387282758951187, -0.0045030489563941956, 0.683665931224823, -0.007404659874737263, -0.025351382791996002, -0.009723041206598282, -0.0009486309718340635, -0.05167060345411301, -0.022576995193958282, -0.2404511719942093, 0.008951754309237003}, {0.004302585031837225, -0.020630383864045143, 0.003408186137676239, 0.0142986373975873, 0.012123224325478077, -0.007692037615925074, -0.0008739854674786329, 0.0012403273722156882, 0.004569269251078367, -0.008742800913751125, -0.004711436573415995, -0.0010580699890851974, -0.0015393001958727837, 0.004813310690224171, -2.612484931945801, -0.0006987293600104749, -0.00650584464892745, -0.022174803540110588, -0.004278987646102905, 0.013811171986162663, 0.0005936169764027, -0.018425190821290016, 0.00974405836313963, 1.5105150938034058, -0.004760730545967817, -0.007344881538301706, 0.34655746817588806, -0.00956080574542284, 0.9014913439750671, -0.001970744226127863, -0.27392181754112244, -0.002733433386310935}, {-0.00026059066294692457, 0.009324193000793457, -0.008263700641691685, 0.0016751326620578766, -0.004468715284019709, 0.007024296559393406, 0.0002890866599045694, -0.0027769054286181927, -0.012665648013353348, -0.013399959541857243, 0.010179806500673294, 0.0001384928764309734, -0.0015740395756438375, -0.0054654632695019245, 0.2699774503707886, -0.006619125604629517, -0.0015805104048922658, -0.009355409070849419, 0.006796718575060368, -0.003122850554063916, 0.0072444723919034, -3.855016620946117e-05, 0.01830965094268322, -0.19541187584400177, -0.008899461477994919, -0.004916549660265446, 0.07913264632225037, -0.005415369290858507, -0.24113136529922485, -0.005709758959710598, -0.15347521007061005, 0.003778687445446849}, {0.00583221111446619, 0.014440766535699368, -0.0019121792865917087, -0.008566494099795818, 0.020946964621543884, -0.010295988991856575, 0.007452761754393578, -0.012113583274185658, -0.021248526871204376, 0.0016871150583028793, 0.020293068140745163, 0.002929546870291233, -0.008972542360424995, -0.00012519712618086487, 0.9137343764305115, 0.007667242083698511, -0.005309141241014004, 0.003780097933486104, -0.008666187524795532, -0.00418015755712986, -0.00962761975824833, 0.0216074176132679, -0.010694297030568123, 0.3684985935688019, -0.004267346113920212, -0.006525983568280935, -0.07150320708751678, -9.024303290061653e-05, -0.5794177055358887, 0.0014657847350463271, -0.0708429142832756, 0.000770575599744916}, {-0.019297368824481964, -0.005871417000889778, 0.010561252944171429, -0.020022403448820114, -0.008287033997476101, -0.020413316786289215, -0.007274710573256016, 0.004177843686193228, 0.0013381876051425934, 0.008278338238596916, 0.01770029589533806, 0.0031455655116587877, 0.020507710054516792, -0.012309293262660503, 0.47055521607398987, -0.008797327987849712, -0.008976933546364307, 0.0043131206184625626, -0.009200022555887699, 0.0011523683788254857, 0.007571016903966665, -0.010014117695391178, -0.0002787587291095406, -0.3576958477497101, 0.0013247159076854587, -0.004055588971823454, -0.24770809710025787, 0.006288447882980108, 0.039108823984861374, 0.026658963412046432, 0.3268760144710541, 0.00819896999746561}, {-0.003369454760104418, 0.009384640492498875, -0.0037035313434898853, 0.005045242607593536, -0.012513041496276855, 0.008515267632901669, 7.025852391961962e-05, -0.024455733597278595, 0.01360025629401207, -0.008208193816244602, 0.000387437641620636, 0.015583401545882225, -0.004377276636660099, -0.00756120914593339, 0.273694783449173, -0.002124349819496274, 0.003955159801989794, 0.005727510899305344, 0.0013521879445761442, 0.008135535754263401, 0.02004890702664852, 0.003536414820700884, 0.010722286067903042, 0.1491689831018448, 0.011080639436841011, -0.006221788004040718, -0.08603323996067047, 0.013889700174331665, -0.2897060215473175, 0.012764248996973038, -0.12202728539705276, 0.002606224035844207}, {-0.010519747622311115, -0.015844689682126045, 0.002002098597586155, -0.015699230134487152, 0.006779283285140991, -0.0048325154930353165, 0.013242468237876892, 0.004068633075803518, -0.0060938578099012375, -0.0046468693763017654, 0.04463482275605202, 0.0003507899527903646, -0.017301028594374657, -0.010410502552986145, -0.34456706047058105, -0.012579775415360928, -0.006008772179484367, -0.016521083191037178, 0.010846753604710102, 0.002235273364931345, 0.0009679590002633631, -0.012991230003535748, -0.018182367086410522, 0.22803077101707458, -0.0012717358767986298, 0.004019090440124273, 0.03979583457112312, -0.026045432314276695, -0.0020858123898506165, -0.0036266439128667116, 0.041517890989780426, -0.011736731044948101}, {-0.014326526783406734, -0.004093676805496216, 0.013334027491509914, -0.02130373939871788, -0.021731168031692505, 0.008998831734061241, -0.00797504372894764, 0.005208859220147133, 0.0006727111176587641, -0.014800839126110077, 0.008392857387661934, -0.005436872132122517, 0.010731314308941364, 0.0003939596062991768, -2.008207321166992, -0.0017968014581128955, 0.007809341885149479, 0.0011235629208385944, 0.01992795616388321, 0.012234030291438103, -0.008878055959939957, 0.011852423660457134, -0.008899507112801075, 1.4013291597366333, -0.00021104331244714558, 0.014549787156283855, 0.39096423983573914, 0.017354005947709084, 0.8026222586631775, 0.0024376939982175827, -0.5775858163833618, -0.009504473768174648}, {0.012705671600997448, -0.015027789399027824, -0.0015936170239001513, -0.013611542992293835, -0.0034170544240623713, 0.022194795310497284, -0.005742000415921211, 0.012999136932194233, 9.701064846012741e-05, -0.006914690602570772, -0.00796203501522541, -0.01033999677747488, -0.016103746369481087, 0.007791975978761911, 0.08931227028369904, -0.008262134157121181, -0.010088303126394749, -0.003887750208377838, -0.008816629648208618, -0.01951664127409458, 0.0070607345551252365, -0.009801791049540043, 0.02371848374605179, -0.05035752058029175, -0.009455477818846703, -0.00921539030969143, 0.14118318259716034, -0.004854791332036257, -0.3294413685798645, 0.0024716700427234173, 0.03424126282334328, 0.01203429326415062}, {0.015433605760335922, 0.001680864836089313, 0.015939941629767418, -0.00785164162516594, -0.0249424260109663, -0.011862573213875294, -0.008955535478889942, -0.004072272684425116, 0.0014586822362616658, -0.01011540275067091, 0.00737565103918314, -0.0028757313266396523, -0.004528549965471029, 0.015894128009676933, 0.7598644495010376, -0.01395968534052372, 0.006781451404094696, 0.004128621891140938, 0.0011801847722381353, 0.00536817591637373, 0.002323690801858902, 0.0028914634604007006, -0.005944873671978712, 0.12777559459209442, 0.0048515391536056995, -0.006366281770169735, -0.08050935715436935, -0.010335776954889297, -0.599403440952301, -0.01115320809185505, -0.12540017068386078, -0.0017762442585080862}, {0.008844836615025997, -0.001271508284844458, 0.022578727453947067, 0.010599926114082336, 0.013485020957887173, 0.01040654256939888, -0.012740937061607838, -6.662636587861925e-05, -0.017519226297736168, -0.0132519556209445, -0.0030073290690779686, 0.0005313106230460107, -0.0028284811414778233, 0.0018171303672716022, 0.05264557525515556, -0.014011724852025509, 0.002859109081327915, 0.012366495095193386, -0.005062643904238939, -0.009906036779284477, -0.017269326373934746, 0.012968290597200394, -0.003285819198936224, 0.45722678303718567, -0.006666394881904125, 0.0058592515997588634, 0.28127333521842957, 0.006597193889319897, -0.025676889345049858, -0.005308611784130335, -0.39804598689079285, 0.0017176952678710222}, {-0.002094124909490347, -0.01028066873550415, -0.0051792035810649395, 0.008730161935091019, 0.017993547022342682, -0.007204856723546982, -0.021182121708989143, 0.03806500509381294, -0.0006403339793905616, -0.001130916876718402, 0.00406952528283, 0.0015303882537409663, -0.013687883503735065, -0.007625422440469265, 0.10718489438295364, -0.01005767285823822, -0.012267854996025562, -0.010073539800941944, -0.000886023452039808, 0.00519552081823349, 0.0051189810037612915, 0.004205666482448578, -0.014110748656094074, 0.47895264625549316, -0.0022248937748372555, 0.0045941718854010105, 0.08142966032028198, -0.01875671185553074, -0.5402437448501587, -0.005083739757537842, 0.20355623960494995, -0.009415660053491592}, {-0.0011446240823715925, 0.0043582008220255375, -0.012598136439919472, 0.004268219228833914, 0.005505930166691542, -0.011109690181910992, -0.0024832147173583508, 0.013443092815577984, 0.009889526292681694, -0.02055424638092518, -0.0034847750794142485, 0.007962675765156746, -0.0043673706240952015, 0.006915199104696512, 0.4151364862918854, 0.008389489725232124, 0.004740049131214619, -0.005407889373600483, -0.006378974299877882, -0.008470477536320686, -0.00553085608407855, 0.0024230927228927612, -0.006884677801281214, 0.06547625362873077, 0.006321737077087164, -0.004393880721181631, 0.10821903496980667, -0.00025185037520714104, -0.24220715463161469, -0.0001018126931739971, 0.017839066684246063, -0.0009125275537371635}, {-0.015252611599862576, 0.006293220445513725, -0.00437461631372571, 0.008386981673538685, -0.008985227905213833, 0.0037419022992253304, -0.019018134102225304, 0.006845102179795504, -0.0028080001939088106, -0.006517051253467798, -0.003772978438064456, -0.006040943320840597, 0.0009187620016746223, 0.018588656559586525, -0.04521051421761513, 0.015321739949285984, -0.004365043714642525, -0.010205424390733242, -0.008995776996016502, -0.00749193225055933, 0.016493597999215126, -0.009747912175953388, -0.022585799917578697, 0.9953658580780029, -0.009473639540374279, 0.005421409383416176, 0.06639298051595688, 0.0017310228431597352, -0.4245797395706177, -0.010630707256495953, 0.22248445451259613, -0.01576950214803219}, {0.01883615367114544, -0.006312820129096508, 0.011763734742999077, 0.008847283199429512, 0.007980721071362495, 0.006715604569762945, 0.00687714759260416, 0.015496568754315376, -0.015482839196920395, -0.005852592643350363, 0.02130662091076374, -0.008838700130581856, 0.003923715092241764, 0.0016151003073900938, 0.10930798202753067, -0.013547822833061218, 0.001900915871374309, 0.016410473734140396, -0.009530054405331612, -0.00021085029584355652, -0.022326190024614334, 0.012824912555515766, 0.01120662223547697, 0.34145888686180115, 0.011069711297750473, -0.010484461672604084, -0.7385779023170471, -0.006013007834553719, 0.2423318326473236, 0.0028461869806051254, 0.5627086162567139, 0.007709837984293699}, {0.0046925353817641735, 0.002158632269129157, -0.0004523672687355429, -0.006569341756403446, -0.0032282357569783926, 0.02280004881322384, -0.007649158127605915, -0.003186112502589822, -0.00918668881058693, -0.012478898279368877, -0.005823374725878239, -0.001973648788407445, -0.012284453958272934, 0.011325354687869549, -0.6502009630203247, -0.0011538468534126878, -0.007237473037093878, 0.011687101796269417, -0.009063995443284512, -0.0007804153137840331, -0.0024551120586693287, -0.010432302951812744, -0.004616296850144863, 0.7350390553474426, 0.006522148847579956, -0.0020374103914946318, -0.0416133850812912, -0.009182782843708992, -0.13478463888168335, 0.014886954799294472, -0.00826265662908554, -0.006660113111138344}, {-0.005979590117931366, -0.0024589470122009516, -0.009336426854133606, -0.001170176430605352, -0.006792867090553045, 0.0027029113844037056, -0.012241262942552567, -0.007684748619794846, 0.011742010712623596, 0.007929904386401176, 0.0077241756953299046, 0.001423702109605074, 0.00862052384763956, -0.0069580660201609135, -0.6521844267845154, -0.006502844393253326, -0.007828823290765285, 0.006648474372923374, -0.0032384039368480444, 0.0038450537249445915, 0.011403251439332962, 0.0027883609291166067, -0.02151813544332981, 0.1668330579996109, 0.00189782225061208, -0.0017812581500038505, 0.012124264612793922, 0.006152538117021322, 0.20917804539203644, -0.002569768577814102, 0.09024372696876526, 0.010378089733421803}, {0.007862336002290249, -0.0010410930262878537, 0.010467877611517906, 0.011830578558146954, -8.315921877510846e-05, -0.0030563499312847853, -0.015585253946483135, -0.013530301861464977, 0.0013805415946990252, 0.013046357780694962, 0.00823177769780159, 0.017913520336151123, -0.0038943856488913298, -0.0037776401732116938, -0.020772693678736687, 0.01624307595193386, 0.00801041629165411, -0.00046303996350616217, -0.027677230536937714, -0.0044555338099598885, 0.009268454276025295, -0.0022286667954176664, -0.0017208491917699575, 0.04068143293261528, 0.01088434923440218, 0.000665001105517149, 0.03195101395249367, 0.0009032276575453579, -0.17499305307865143, -0.002592506119981408, -0.04163685813546181, 0.00359644228592515}, {0.013989564031362534, 0.0019236612133681774, -0.017995594069361687, 0.011317803524434566, 0.007702067494392395, -0.014555102214217186, -0.005507622845470905, -0.000873013399541378, -0.014916065149009228, 0.0033929599449038506, -0.010929777286946774, 0.0002455889480188489, 0.028367608785629272, 0.008486002683639526, 0.3838285207748413, -0.005988094490021467, 0.0037020863965153694, -0.015697022899985313, 0.0001658936234889552, 0.007648325525224209, 0.015361225232481956, 0.006994183640927076, 0.01435270719230175, 0.9948990345001221, -0.012518142350018024, -0.0018553818808868527, 0.02335866540670395, 0.003009623847901821, -1.4282917976379395, -0.004731685854494572, 0.07774358987808228, 0.01393017265945673}, {-0.0173021387308836, -0.0001412788697052747, -0.003441969631239772, 0.0017504867864772677, -0.005500121042132378, -0.000321697152685374, -0.006127617787569761, 0.003893929533660412, 0.01681383140385151, 0.0034825322218239307, -0.00662196334451437, 0.002592728938907385, 0.018886009231209755, -0.0008237117435783148, -0.954181432723999, 0.0006768417079001665, -0.006418454460799694, -0.00684389378875494, -0.006128134671598673, -0.017142845317721367, -0.0005036438233219087, -0.003033363725990057, 0.014881854876875877, 1.1938952207565308, 0.0007182535482570529, 0.009624134749174118, -0.041490670293569565, 0.0015513630351051688, -0.18845374882221222, -0.011271466501057148, 0.06728246808052063, 0.004112313501536846}, {0.0006787908496335149, -0.0058518508449196815, 0.01804986409842968, 0.0032335349824279547, 9.903718455461785e-05, 0.0012401562416926026, 0.0010059404885396361, 0.004410200752317905, -0.008500314317643642, -0.00568357715383172, 0.017155423760414124, -0.008919966407120228, -0.0070907967165112495, 0.000692230009008199, -0.18222945928573608, -0.008915347047150135, 0.012581678107380867, 0.022160958498716354, 0.011731620877981186, 0.004351078066974878, 0.02304665558040142, -0.00809453148394823, -0.009470856748521328, 0.2770206928253174, 0.0026313024573028088, -0.00358256115578115, -0.013449768535792828, 0.006510835140943527, 0.12961101531982422, -0.0006402585422620177, -0.11092332005500793, 0.0015617593890056014}, {-0.01671847701072693, -0.00817493163049221, -0.004317732993513346, 0.0055815731175243855, 0.011641262099146843, -0.011508994735777378, -0.004798290319740772, 0.013714872300624847, -0.003916406072676182, -0.009203750640153885, 0.011768513359129429, 0.015193996019661427, 0.011280772276222706, -0.007261608261615038, 0.3090111017227173, 0.01931787095963955, -0.005338846240192652, 0.0081228818744421, 0.0014011005405336618, -0.0019171881722286344, -0.004519724287092686, -0.01628659851849079, -0.0013507785042747855, -0.5106307864189148, -0.0011678349692374468, 0.014311823062598705, -0.06860680133104324, 0.005034234374761581, -0.3025408685207367, 0.005614957772195339, 0.4635484218597412, -0.0015568805392831564}, {-0.011060239747166634, 0.008948998525738716, -0.005757736042141914, 0.00239211181178689, -0.006258845329284668, -0.018232496455311775, 0.013464324176311493, -0.00015963717305567116, 4.850898403674364e-05, 0.004470679443329573, -0.004925849381834269, -0.005719983950257301, 0.004321087151765823, 0.0032459937501698732, -0.06010414659976959, 0.0016256156377494335, 0.012160604819655418, 0.007233565207570791, -0.0014761893544346094, 0.003692809259518981, 0.015774324536323547, 0.002147513208910823, 0.00980992428958416, 0.28586384654045105, -0.001101189642213285, 0.008338844403624535, -0.17376665771007538, -0.005604740232229233, -0.009391183033585548, 0.001142955501563847, -0.06663937866687775, 0.005938508547842503}, {-0.017114846035838127, -0.010787748731672764, -0.0010165151907131076, -0.00503510981798172, -0.011585014872252941, 0.004464506171643734, 0.0048122587613761425, -0.00043143858783878386, -0.013460097834467888, -0.004114317242056131, -0.008388600312173367, 0.002123388694599271, 0.0047422489151358604, 0.003828339744359255, -0.47127053141593933, -0.012492687441408634, -0.005688032601028681, 0.0002407816209597513, -0.027769479900598526, -0.0012022571172565222, 0.0015967298531904817, -0.002318170852959156, 0.009672116488218307, -1.278228998184204, 0.008096546865999699, -0.0113288015127182, -0.20100313425064087, -0.0036062540020793676, 1.781897783279419, -0.010637212544679642, 0.29590025544166565, 0.001058140187524259}, {-0.008731286972761154, 0.00036229018587619066, -0.00638555409386754, 0.012403041124343872, -0.008156979456543922, -0.004302245564758778, 0.0027904435992240906, -0.007614738307893276, 0.005855244118720293, -0.0007920180214568973, 0.016687698662281036, -0.007276427466422319, 0.0037778248079121113, 0.014486906118690968, 0.8240101337432861, 0.0014499267563223839, 0.004717196337878704, 0.02124135009944439, 0.007424087729305029, 0.007881891913712025, 0.009209359996020794, -0.007251603063195944, 0.0022568143904209137, -1.055596113204956, 0.003546070074662566, -0.0022254325449466705, 0.12839123606681824, -0.0062439399771392345, -0.18686574697494507, -0.001106301904655993, 0.08630619198083878, -0.015189330093562603}, {-0.026278439909219742, -8.52584998938255e-05, -0.011630137450993061, 0.006815379019826651, 0.00920063629746437, -0.027699073776602745, 0.005717304069548845, 0.01284814067184925, 0.0072870575822889805, -0.004900691565126181, 0.010803989134728909, 0.01427517645061016, 0.00036708961124531925, 0.01322995126247406, -0.33038368821144104, -0.008993309922516346, -0.0018799748504534364, 0.013670642860233784, -0.01646762527525425, -0.006223647855222225, -0.006778523791581392, -0.003022195538505912, -0.008863178081810474, 0.19793440401554108, -0.005723615642637014, 0.0017092005582526326, 0.14179867506027222, -0.0038499890360981226, 0.029100656509399414, -0.0067552197724580765, -0.07018585503101349, 0.0008859901572577655}, {0.00017955986550077796, 0.016587810590863228, -0.005737910512834787, -0.0028639405500143766, -0.002743565710261464, 0.003169039962813258, -0.0038269199430942535, 0.006855467800050974, -0.008213979192078114, 0.017660023644566536, 0.005498128477483988, 0.00832302588969469, -0.009563790634274483, -0.0005123906303197145, 0.2607417404651642, 0.0106063112616539, 0.0031613383907824755, 0.0058802454732358456, -0.00877994205802679, -0.0005078318645246327, -0.0022397530265152454, -0.005897845607250929, -0.009381884709000587, 0.2889520525932312, 0.002735338406637311, -0.009317578747868538, -0.9532237648963928, 0.010693488642573357, 0.19199791550636292, 0.002401755889877677, 0.6279938817024231, -0.006676698103547096}, {-0.01882229372859001, -0.007757897954434156, 0.0077288164757192135, 0.007422233000397682, -0.007756715640425682, 0.0005112322396598756, -0.006919044069945812, 0.0019303604494780302, 0.0006749688764102757, -0.011613745242357254, 0.004873437341302633, 0.007648087106645107, 0.019582116976380348, 0.002784988610073924, 0.003213999792933464, -0.014582502655684948, -0.02035207860171795, 0.009194713085889816, -0.011096717789769173, 0.007611635606735945, -0.0013488134136423469, 0.0005516005330719054, 0.002417668467387557, 0.022187933325767517, -0.0007532450254075229, 0.018058499321341515, -0.46276482939720154, 0.011540687642991543, 0.05488663539290428, 0.004564089700579643, 0.06985080242156982, -0.0059310561046004295}, {-0.0046498035080730915, -0.017899421975016594, -0.0038922401145100594, 0.0025290946941822767, -0.00876825675368309, 0.02203284204006195, -0.0014471763279289007, -0.007632204797118902, -0.01025534700602293, -0.0056945523247122765, 0.0036313440650701523, 0.007401577662676573, -0.007773900404572487, 0.00449730409309268, 0.2746421992778778, 0.002628880087286234, 0.012343242764472961, -0.0032100998796522617, 0.004165010526776314, 0.004861319437623024, -0.018856054171919823, -0.005117783322930336, 0.00713720079511404, 0.26301226019859314, -0.011864785104990005, 0.017655683681368828, -0.8786892294883728, -0.0025923214852809906, 0.04798232391476631, 0.006403360981494188, 0.47371143102645874, -0.023537592962384224}, {-0.008183464407920837, -0.0070475428365170956, -0.004199951887130737, 0.004294677637517452, 0.01921960897743702, -0.00912279449403286, 0.004145923536270857, -0.015611140057444572, -0.0035483480896800756, -0.007822810672223568, 0.0011085688602179289, 0.008160264231264591, -0.014430751092731953, 0.012794057838618755, -0.21641165018081665, 0.002358384430408478, -0.01457128580659628, -0.01277525071054697, -0.011962872929871082, 0.01876533031463623, 0.0015151564730331302, -0.00694830808788538, -0.004810230806469917, -0.04463370144367218, 0.007433569058775902, -0.0028032707050442696, 0.704921305179596, -0.003814916592091322, -0.1455690562725067, -0.009729087352752686, -0.20164957642555237, 0.009870585985481739}, {0.011192798614501953, 0.008417737670242786, -0.00494789332151413, -0.005181012209504843, -0.008126407861709595, 0.019729461520910263, -0.008511058054864407, -0.01579400897026062, -0.006578544154763222, -0.0060997623950243, 0.016720673069357872, -0.0026279583107680082, 0.010928896255791187, -0.01717877760529518, -0.571550190448761, -0.006021940615028143, -0.00961801316589117, 0.007095841225236654, 0.012284820899367332, 0.014352036640048027, -0.010458767414093018, -0.002691433997824788, 0.0017845525871962309, 0.4870610237121582, 0.006521112285554409, -0.0017803936498239636, 0.21892549097537994, 0.017863579094409943, -0.15568402409553528, 0.0008811685256659985, -0.36090371012687683, -0.010284787975251675}, {0.01695803925395012, 0.004668329376727343, 0.013731038197875023, -0.0006610367563553154, -0.003683580784127116, 0.002040194347500801, 0.004481193143874407, 0.01739346981048584, -0.01749259978532791, -0.00150605000089854, -0.001261883764527738, 0.004987095482647419, -0.013862397521734238, -0.003373224288225174, 0.5482799410820007, -0.000251210352871567, -0.005691549275070429, 0.0008944497094489634, -0.0042014350183308125, -0.009818055666983128, 0.016299311071634293, -0.003352639963850379, 0.008146768435835838, -1.7366567850112915, -0.022478198632597923, 0.0034085686784237623, -0.012993999756872654, -0.008166909217834473, 0.5268067121505737, 0.011446272023022175, -0.012792808935046196, 0.005576222203671932}, {-0.001021853182464838, -0.002548342803493142, 0.006047029513865709, 0.00034879500162787735, 0.0017504229908809066, -0.008949823677539825, 0.024268584325909615, -0.00025647759321145713, 0.006283195689320564, -0.009963646531105042, -0.0015707660932093859, -0.01010301522910595, -0.0036810035817325115, 0.0021187018137425184, 0.13717035949230194, -0.009176639840006828, -0.005766019225120544, 0.005046231206506491, -0.003852885914966464, 0.003986063413321972, -0.001679121982306242, 0.01209661178290844, -0.005461334250867367, -0.5511596202850342, -0.01039851363748312, -0.008597102016210556, 0.07773006707429886, -0.00329068792052567, -0.1939219832420349, 0.007336026057600975, -0.11108648031949997, -0.014236382208764553}, {-0.012016559951007366, 0.024046266451478004, 0.005932841915637255, -0.004675707779824734, 0.01921355351805687, 0.0023154125083237886, 0.0012818111572414637, 0.01166502945125103, -0.012887628749012947, 0.022856995463371277, -0.004218075890094042, 0.005285054445266724, 0.009396227076649666, 0.012310149148106575, 0.5886894464492798, -0.0014921566471457481, -0.011555103585124016, 0.004731415770947933, 0.006048165261745453, 0.018258865922689438, -0.007709577679634094, -0.006046511232852936, 0.015347299166023731, 0.14998427033424377, -0.005848034285008907, 0.001452601165510714, -0.07281292229890823, -0.004149652551859617, -0.5920799374580383, -0.02883557416498661, 0.11474139988422394, -0.006323861423879862}, {0.010742751881480217, -0.004260104615241289, -0.01811162754893303, -0.02232068032026291, 0.0008989855996333063, 0.012115338817238808, -0.006640581414103508, 0.004064588807523251, -0.005698551889508963, 0.0025190727319568396, 0.00431984756141901, -0.01341266930103302, -0.007823597639799118, -0.0002865406568162143, 0.7801105976104736, 0.012743137776851654, 0.017675597220659256, 0.0087044108659029, -0.009385857731103897, -0.006385907996445894, 0.012447972781956196, -0.007915273308753967, 0.014073330909013748, -1.4963384866714478, -0.0012630005367100239, 0.019318921491503716, 0.0613178052008152, -0.012257235124707222, 0.5523742437362671, -0.007337681017816067, -0.20898112654685974, -0.014458720572292805}, {0.0040196520276367664, 0.00019041348423343152, 0.010969694703817368, 0.013005238026380539, 0.014006049372255802, -0.0052814302034676075, 0.004236973822116852, -0.009414151310920715, 0.002290321048349142, 0.0036118021234869957, 0.0013165203854441643, 0.006428111810237169, 0.015406559221446514, 0.016356654465198517, 0.18678830564022064, 0.007080893497914076, -0.002906534820795059, -0.006056297104805708, -0.011183340102434158, -0.001108357566408813, 0.012104151770472527, -0.01698305271565914, 0.022668415680527687, -1.110999584197998, 0.0021215290762484074, -0.0009735113708302379, 0.060137636959552765, -0.008761153556406498, 0.32802680134773254, -0.0043737152591347694, 0.1556149274110794, -0.0001774476986611262}, {-0.012795140035450459, 0.007765559479594231, -0.008511013351380825, -0.01671147532761097, -0.01837245747447014, -0.0008933364297263324, -0.005987877491861582, 0.01554278377443552, -0.005105904303491116, -0.0024063566233962774, 0.010770815424621105, -0.002114978153258562, 0.008397838100790977, 0.000620546517893672, 0.25301340222358704, -0.009564587846398354, -0.008579639717936516, -0.0007471913122572005, -0.009538772515952587, -0.0037542369682341814, -0.008060491643846035, -0.0016473139403387904, -0.010120389983057976, -0.11336757987737656, -0.0016634183702990413, 0.016532955691218376, -0.03799823299050331, -0.0008709058165550232, -0.23165445029735565, -0.009847940877079964, 0.022294938564300537, 0.002965533174574375}, {-0.005020159762352705, -0.00920107401907444, 0.005468405317515135, -0.001523842802271247, -0.0007368343649432063, -0.011844552122056484, -0.010375185869634151, -0.0030682224314659834, 0.011779095977544785, -0.008834164589643478, 0.006041740532964468, 0.012341399677097797, -0.0018395206425338984, 0.001240696175955236, -0.4701346158981323, -0.024677356705069542, -0.0044013154692947865, -0.0033833570778369904, -0.005462727975100279, 0.02054559625685215, -0.01770714670419693, -0.017447426915168762, -0.01621139794588089, 0.8812867403030396, -0.010648337192833424, 0.0015123640187084675, 0.1140235960483551, -0.0001303165772696957, -0.2140807807445526, 0.00396310118958354, 0.02027037926018238, -0.00837236549705267}, {0.00015590096882078797, 0.005815830547362566, 0.011908547952771187, -0.0058570075780153275, -0.0058036367408931255, 0.007884526625275612, -0.0036899824626743793, -0.019459813833236694, -0.013782461173832417, 0.0009690406150184572, -0.012010791338980198, -0.008305785246193409, 0.00566106429323554, -0.0054381671361625195, 0.1468065232038498, 0.0017813424346968532, -0.012420506216585636, 0.000451487343525514, -0.0007492427830584347, 0.002454877831041813, 0.0038802914787083864, 0.004155557602643967, -0.00808786042034626, -0.010830736719071865, -0.00894737709313631, 0.00674274330958724, -0.02153606154024601, 0.0004749818763229996, 0.04153309017419815, 0.004556551575660706, 0.08195625245571136, -0.01781519688665867}, {0.007314854767173529, -0.010473216883838177, 0.006407582201063633, -0.0017580506391823292, 0.016412651166319847, -0.0038856908213347197, 0.010074960999190807, -0.00209025084041059, -0.012902824208140373, 0.0014710725517943501, -0.006537661422044039, 0.0004844970244448632, -0.004704933147877455, 0.02830476500093937, 0.11752547323703766, -0.017645224928855896, 0.0028850066009908915, 0.02212139032781124, 0.000965243496466428, -0.0031497194431722164, 0.006207204889506102, -0.00023452773166354746, 0.004606650676578283, 0.015506801195442677, -0.012737933546304703, -0.02381865493953228, 0.41990774869918823, -0.014536782167851925, -0.4621352255344391, -0.01712549664080143, -0.5409174561500549, -0.016021279618144035}, {0.004548993892967701, 0.0018832760397344828, -0.017709702253341675, 0.0025541819632053375, -0.010920722037553787, -0.011380461975932121, 0.003086111741140485, 0.004576966632157564, -0.01862965151667595, -0.006511597894132137, 0.016161099076271057, -0.009588701650500298, -0.006689867936074734, -0.015623698011040688, 0.8398613333702087, -0.003474690020084381, -0.0070507824420928955, -0.004584434907883406, -0.0006909557851031423, 0.010792037472128868, 0.00785554014146328, -0.012110063806176186, -0.007179776672273874, 0.6970676183700562, 0.004639278631657362, 0.004724718164652586, -0.2428540587425232, 0.003920180257409811, -0.9467878341674805, -0.005944396834820509, 0.09242506325244904, -0.004027450457215309}, {0.008569542318582535, -0.0140541335567832, -0.011470023542642593, -0.006905089598149061, -0.0002452149346936494, 0.01140702236443758, 0.0013509407872334123, -0.005273750983178616, 0.024289363995194435, 0.0036771565210074186, -0.023817768320441246, 0.002036936115473509, 0.0025118098128587008, 0.004039059393107891, -0.2395647019147873, 0.007832935079932213, 0.02654797025024891, -0.008646195754408836, -0.002737900009378791, -0.025346936658024788, 0.011620470322668552, 0.0029179868288338184, -0.008681626059114933, -0.3048113286495209, -0.009561731480062008, 0.009074843488633633, 0.2173011749982834, 0.0012122886255383492, 0.0831003487110138, -0.0015821539564058185, -0.27678561210632324, -0.00620201276615262}, {-0.008066189475357533, -0.011677561327815056, 0.0036836927756667137, -0.011098009534180164, 0.005148475524038076, 0.01981469988822937, 0.0036498927511274815, 0.004978426266461611, -0.006961531471461058, 0.0018258431227877736, 0.0014630543300881982, 0.00499314907938242, -0.008045053109526634, -0.0051472545601427555, 1.9538230895996094, 0.004863455891609192, -0.021835817024111748, -0.008852323517203331, -0.006739744450896978, 0.006841880735009909, -0.013607502914965153, 0.01853671669960022, 0.004628083668649197, -3.5181994438171387, -0.006925547961145639, -0.001021113945171237, -0.16462671756744385, 0.006780700758099556, 1.8306384086608887, 0.004082250874489546, 0.27698445320129395, -0.009871616028249264}, {0.021023906767368317, 0.001219740603119135, -0.008358851075172424, 0.016960101202130318, 0.005152593832463026, 0.001773107796907425, -0.00020033985492773354, 0.003026326885446906, 0.0025480526965111494, -0.012204240076243877, 0.004596139304339886, 0.009215261787176132, -0.007695729378610849, 0.0013809828087687492, -0.170059934258461, -0.008657957427203655, 0.005831180606037378, 0.014587534591555595, 0.009912343695759773, 0.00968001689761877, 0.0004393734270706773, -0.005192511714994907, 0.0025161211378872395, 0.08975930511951447, -0.002695908537134528, 0.0024522088933736086, 0.08889984339475632, 0.008665192872285843, -0.14179974794387817, 0.008653830736875534, -0.11907720565795898, -0.002880419371649623}, {-0.0007022092468105257, -0.008478114381432533, -0.011235020123422146, -0.003037627087906003, 0.0031121158972382545, -0.005332602187991142, -0.016124794259667397, -0.007792696822434664, -0.00937636848539114, 0.0022976486943662167, 0.0027388939633965492, -0.005580332595854998, -0.0016009329119697213, -0.01002312172204256, -0.48588675260543823, -0.015149850398302078, -0.014092441648244858, -0.003861137432977557, 0.0016733275260776281, -0.013300555758178234, -0.003727618372067809, 0.00041676510591059923, 0.0023206647019833326, 0.5345004200935364, -0.0015005101449787617, -0.009151151403784752, 0.09916796535253525, 0.014648563228547573, -0.1076761782169342, -0.0135109918192029, -0.05641091614961624, 0.019579095765948296}, {0.005117643158882856, 0.015429961495101452, -0.006974669173359871, -0.003118827473372221, -0.009716770611703396, 0.007452317513525486, -0.017314720898866653, -0.0041357711888849735, -0.0011388740967959166, -0.00755542004480958, 0.002645289758220315, -0.011573553085327148, 0.007042020559310913, -0.01587202399969101, -0.40052303671836853, 0.008021592162549496, -0.00161265442147851, -0.0003530886606313288, 0.0063297804445028305, -0.005135528277605772, -0.0005090453196316957, -0.0062582725659012794, 0.01080105360597372, -0.12487513571977615, -0.011918634176254272, -0.009431115351617336, 0.16413836181163788, -0.0025666351430118084, 0.39792588353157043, -0.015517402440309525, -0.23686577379703522, 0.012323512695729733}, {0.015675321221351624, 0.004335131961852312, 0.0017523589776828885, 0.02053224667906761, 0.014101605862379074, -0.0009810234187170863, -0.013557357713580132, 0.0012533209519460797, -0.0033537819981575012, -0.0008090948103927076, 0.009881953708827496, -0.01352431159466505, -0.014045427553355694, 0.030478257685899734, -0.3392571806907654, 0.009146167896687984, 0.0009617057512514293, 0.006771579850465059, 0.00397218344733119, -0.0017371755093336105, -0.0028826366178691387, 0.002544463612139225, -0.005626806523650885, 0.20436137914657593, 0.01573360711336136, 0.0032964192796498537, -0.02395455166697502, 0.000552298326510936, -0.010223230347037315, 0.011898228898644447, -0.025278853252530098, -0.007696817163378}, {0.011738552711904049, -0.0004351411771494895, -0.0033306069672107697, -0.010848820209503174, -0.0019923606887459755, -0.0010251576313748956, -0.001499138423241675, -0.0061128148809075356, -0.01204062718898058, 0.01893927901983261, 0.023133227601647377, 0.0025050484109669924, -0.0013553060125559568, 0.00030864396831020713, 0.0862475037574768, -0.025295427069067955, -0.020480994135141373, 0.0010346464114263654, -0.013303935527801514, -0.023517362773418427, -0.003753453493118286, -0.014111148193478584, -0.00824723206460476, -0.042686399072408676, 0.01318754069507122, -0.013210445642471313, -0.1370115727186203, -0.012058612890541553, -0.022724101319909096, 0.008152528665959835, 0.32748091220855713, 0.011988632380962372}, {0.008513596840202808, 0.0034016205463558435, -0.004436387214809656, -0.0020333793945610523, 0.0037927171215415, -0.01632261462509632, -0.0021242171060293913, -0.004422105383127928, 0.005612078122794628, -0.0009475821861997247, -0.030673155561089516, -0.005462887696921825, -0.0035202093422412872, 0.005703245289623737, 0.26556843519210815, 0.011831187643110752, 0.004700415767729282, 0.008753851056098938, 0.0027370250318199396, 0.016973329707980156, 0.005424144212156534, -0.03193441405892372, -0.00866378378123045, 0.10287093371152878, -0.007959031499922276, 0.016049141064286232, -0.02613513544201851, 0.007610232103615999, 0.041938040405511856, -0.022692495957016945, -0.027722090482711792, 0.0004160026437602937}, {-0.0020272748079150915, 0.018298368901014328, -0.003248860128223896, -0.016154028475284576, 0.011660607531666756, -0.002700468758121133, 0.017200440168380737, 0.0021442703437060118, -0.007759239990264177, -0.008611006662249565, 0.014879611320793629, -0.0013615190982818604, 0.010674158111214638, -0.004150038119405508, 0.8805178999900818, -0.007217955309897661, 0.009332808665931225, 0.012022970244288445, 0.008811536245048046, -0.004436055663973093, 0.0008477665251120925, -0.0031317411921918392, 0.0027941828593611717, 0.4945751428604126, 0.013622869737446308, -0.0026552474591881037, 0.02107224240899086, 0.005619728937745094, -0.9575036764144897, -0.014987768605351448, -0.12691444158554077, 0.005981742404401302}, {0.01633911207318306, -0.005087671801447868, 0.011366665363311768, -0.010245022363960743, -0.002495635999366641, -0.006257564760744572, 0.004401103593409061, -0.016108520328998566, -0.008496761322021484, 0.005025968886911869, -0.004516370128840208, -0.008030284196138382, -0.002477553440257907, -0.005425713025033474, -0.22144047915935516, -0.008157984353601933, -0.006547385826706886, 0.006636794190853834, -0.0011396112386137247, -0.0027569904923439026, 0.003383104922249913, -0.0017249009106308222, -0.006834729108959436, -0.45175158977508545, -0.010304410010576248, -0.0119627071544528, 0.6448363661766052, 0.004813619889318943, 0.1329927146434784, -0.014719835482537746, -0.47638267278671265, -0.01245646458119154}, {-0.008551230654120445, 0.002702161902561784, 0.002917086472734809, -0.0020894454792141914, -0.018415868282318115, -0.013502361252903938, 0.015095519833266735, -0.007071893196552992, -0.020772656425833702, 0.0009129734244197607, -0.02280620113015175, -0.0114537188783288, -0.004391901195049286, 0.016693521291017532, -0.7182605862617493, -0.0062895785085856915, -0.0004027656395919621, -0.021303869783878326, -0.009837730787694454, -0.0026928605511784554, 0.003064022632315755, 0.0014945276780053973, 0.0016808181535452604, 0.2727340757846832, 0.01016821339726448, 0.008887491188943386, 0.029697217047214508, 0.0034942207857966423, 0.2308860421180725, -0.01814865693449974, 0.07772787660360336, 0.013940921984612942}, {-0.009298410266637802, -0.02217075228691101, -0.014754176139831543, -0.018121402710676193, -0.008494794368743896, 0.00642102025449276, -0.0008366162655875087, 0.021815041080117226, 0.01350400224328041, 0.003075669053941965, -0.0278959721326828, -0.009338761679828167, 0.009682177565991879, -0.01089197862893343, -0.1251877248287201, -0.008654650300741196, -0.015168377198278904, -0.002230613259598613, 0.010106418281793594, 0.013339562341570854, 0.0008317127358168364, -0.009277849458158016, -0.00851539894938469, 0.6832456588745117, 0.004571130033582449, 0.01032345648854971, -0.128317192196846, -0.008085914887487888, -0.04063321650028229, 0.014252768829464912, -0.025185367092490196, 0.003842115867882967}, {0.005905442871153355, 0.0025302525609731674, -0.003274730173870921, -0.012180056422948837, -0.018519427627325058, -0.001448831637389958, 0.008212861604988575, 0.010553753934800625, 0.0008754991577006876, 0.003380324225872755, 0.0033846807200461626, 0.01092398352921009, -0.010426893830299377, 0.0032876962795853615, -0.08118019998073578, -0.004666084889322519, -0.004544710274785757, -0.00027511033113114536, -0.01121020969003439, -0.002709499793127179, 0.0006804837030358613, -0.002415494294837117, -0.012304659001529217, 0.9243329763412476, 0.005930008832365274, 0.005026611965149641, 0.062868632376194, -0.01125501561909914, -0.3897669315338135, 0.009423971176147461, -0.37528491020202637, 0.016011428087949753}, {-0.007882446050643921, -0.00648008706048131, -0.0008106948807835579, -0.0089844586327672, -0.005361623130738735, -0.03240559995174408, 0.006637488957494497, 0.010131618939340115, 0.006634494289755821, -0.010604954324662685, 0.0023329532705247402, -0.008095317520201206, -0.008720370940864086, 0.004207896068692207, 0.26060983538627625, -0.004169980529695749, 0.0162725280970335, 0.003860272467136383, 0.013399409130215645, 0.0067508709616959095, 0.005731974262744188, -0.013057667762041092, 0.003072833875194192, 0.14948736131191254, 0.00450711976736784, -0.01846998557448387, -0.11540378630161285, 0.002548708813264966, -0.0518491305410862, 0.00784639734774828, -0.10920872539281845, 0.007269538007676601}, {0.012851033359766006, -0.017619939520955086, -0.015571428462862968, 0.0206863135099411, 0.0056789591908454895, -0.01660270243883133, 0.0029475304763764143, -0.00887849647551775, 0.01814260520040989, -0.011349220760166645, -0.01832384243607521, -0.0014496782096102834, -0.007421853952109814, -0.010653210803866386, -0.08406684547662735, 0.0018375178333371878, -0.0034854901023209095, -0.0029776303563266993, -0.00017241436580661684, -0.005297346040606499, -0.001342040253803134, -0.0074242642149329185, 0.013391617685556412, -0.02316817082464695, -0.0004253576626069844, -0.006398546509444714, 0.24771995842456818, 5.7645735068945214e-05, -0.1534799337387085, -0.006404482759535313, 0.013800832442939281, 0.00401326036080718}, {0.01252643670886755, 0.0004607568553183228, 0.0010485609527677298, 0.005197734106332064, -0.00740663381293416, -0.00045759990462101996, 0.011411504819989204, -0.013983208686113358, -0.0007999989320524037, 0.006059171631932259, 0.0005877322400920093, 0.00163577264174819, -0.025481635704636574, -0.012442917563021183, -1.6178072690963745, 0.014423991553485394, 0.007778570055961609, 0.0037239163648337126, 0.004157962277531624, -0.0003589336993172765, -0.0017671043751761317, 0.0022288288455456495, -0.006328006740659475, 2.713900566101074, 0.011035000905394554, -0.006051553413271904, 0.1416744887828827, 0.011919629760086536, -0.7616187930107117, -0.010922365821897984, 0.011119845323264599, 0.011700807139277458}, {0.013836589641869068, -0.004841845016926527, 0.007015629671514034, -0.008430431596934795, -0.007098667323589325, 0.002263834932819009, 0.019322169944643974, 0.005857313517481089, 0.004806213080883026, -0.003882350865751505, -0.0035535271745175123, -0.025134583935141563, 0.0017808139091357589, 0.007039083633571863, 0.1613527536392212, -0.0009759082458913326, 0.007734541315585375, 0.005546704865992069, 0.02073284052312374, 0.0075544570572674274, 0.0038460141513496637, 0.011708599515259266, 0.007096314337104559, 0.012104494497179985, -0.007899141870439053, 0.005530244205147028, 0.16051828861236572, 0.02486652508378029, -0.14484865963459015, -0.01882404088973999, 0.08006145805120468, -0.007428336422890425}, {-0.006318057421594858, -0.009432467631995678, -0.004266307223588228, -0.004593187943100929, 0.002370404312387109, 0.0017141716089099646, 0.019751863554120064, 0.0014261662727221847, 0.002897408325225115, 0.004503769334405661, 0.01277683675289154, 0.002194806467741728, -0.006131253205239773, 0.0038365046493709087, -0.5332598090171814, -0.008998318575322628, -0.006654054392129183, 0.015662837773561478, -0.016525981947779655, -0.008652298711240292, -0.008806091733276844, -0.008276082575321198, -0.01309353206306696, 0.335989773273468, 0.0027774181216955185, 0.0003527376102283597, -0.3302786350250244, -0.017316268756985664, 0.5530577898025513, 0.010181345976889133, 0.19236694276332855, 0.005518049467355013}}
local m2 = {{0.07553824782371521, -0.0645296722650528, 0.10721559077501297, -0.03344632312655449, 0.04402335733175278, -0.14498655498027802, -0.24824872612953186, -0.028136080130934715, 0.010257996618747711, 0.04899561032652855, -0.07254855334758759, 0.04425831139087677, -0.12577120959758759, -0.06210765615105629, -0.2474345862865448, 0.19734469056129456, 0.08223488181829453, -0.2642064690589905, 0.20032833516597748, 0.1393246352672577, 0.24789603054523468, -0.029128318652510643, 0.03991561755537987, 0.08747734874486923, -0.15210981667041779, 0.21348346769809723, -0.004202730022370815, -0.07640890777111053, 0.035374537110328674, 0.20545758306980133, -0.13402153551578522, -0.013360240496695042, -0.06283942610025406, 0.004215680994093418, 0.052041616290807724, 0.17439034581184387, -0.04691479727625847, -0.10672907531261444, 0.06983888894319534, 0.06806627660989761, 0.09846071153879166, -0.015826642513275146, 0.0821738988161087, -0.09562093764543533, -0.026398606598377228, 0.1365816593170166, 0.10816899687051773, 0.053475674241781235, -0.03520507737994194, 0.05663060396909714, -0.0377238430082798, 0.051254600286483765, 0.15148167312145233, 0.02784220688045025, 0.12553270161151886, 0.10538196563720703, -0.03774934262037277, -0.030157925561070442, -0.018928879871964455, 0.21677447855472565, -0.15169253945350647, 0.03154894337058067, -0.07218336313962936, 0.20766714215278625, 0.06265395879745483, -0.11403381824493408, -0.040245164185762405, 0.09963522851467133, -0.06630754470825195, -0.0850624144077301, -0.006549973506480455, -0.023176562041044235, -0.24850808084011078, -0.10374903678894043, -0.027541888877749443, -0.043780986219644547, 0.16565898060798645, -0.1047585979104042, -0.1458377242088318, -0.21487028896808624, 0.2625219523906708, 0.10952800512313843, -0.080295130610466, -0.2025158852338791, -0.0033290924038738012, 0.010512113571166992, 0.06078812852501869, 0.044913001358509064, -0.010809013620018959, -0.05210274085402489, -0.08819323033094406, -0.012975985184311867, -0.12405148148536682, 0.16844694316387177, 0.01159491017460823, -0.0008553077350370586, -0.07364997267723083, 0.12349097430706024, 0.051770057529211044, -0.05794773995876312, -0.08141735196113586, -0.11702139675617218, 0.012413940392434597, 0.06812078505754471, 0.04283491522073746, 0.14569199085235596, -0.06163358315825462, -0.16245973110198975, 0.15673525631427765, 0.09435007721185684, -0.002995515475049615, -0.07004749029874802, -0.015164253301918507, 0.24665884673595428, 0.04964752867817879, -0.053110793232917786, -0.12439381331205368, 0.015544100664556026, -0.009431213140487671, -0.027016587555408478, -0.1101924255490303, 0.0286633912473917, 0.020475197583436966, -0.1841467022895813, 0.2083078920841217, 0.04791541025042534, -0.10505610704421997, 0.21289026737213135, 0.012760705314576626, 0.013979974202811718, -0.028596483170986176, -0.09427878260612488, 0.2807009220123291, 0.15835528075695038, 0.03745485097169876, 0.00626502325758338, -0.11637209355831146, 0.026969268918037415, 0.18618597090244293, -0.24378374218940735, 0.31604015827178955, 0.1815677434206009, -0.03278563544154167, 0.11942233145236969, 0.18154408037662506, 0.09858276695013046, -0.20427648723125458, -0.1102781742811203, -0.08439134061336517, 0.14521808922290802, 0.06375597417354584, 0.0015419838018715382, -0.11102668941020966, 0.004924634005874395, -0.10762850195169449, -0.03029109723865986, 0.1219550222158432, 0.043739549815654755, -0.056693390011787415, -0.1550617665052414, 0.09157174080610275, -0.0006331920740194619, -0.14751747250556946, 0.125230610370636, 0.0348643995821476, -0.005419035442173481, 0.12913982570171356, 0.020559782162308693, -0.010819525457918644, 0.00659601017832756, -0.031850166618824005, 0.19830839335918427, -0.1730603277683258, 0.04745963588356972, 0.38217002153396606, -0.12155912071466446, -0.12539425492286682, 0.005400398280471563, -0.30477866530418396, 0.1125781312584877, 0.030136732384562492, 0.22387877106666565, 0.17369307577610016, 0.010449954308569431, 0.1646101176738739, -0.05209542065858841, -0.1663273274898529, -0.13515152037143707, 0.03673654422163963, 0.012869671918451786, 0.10327444970607758, 0.0564858503639698, 0.06724126636981964, 0.32876431941986084, 0.028737790882587433, -0.1291785091161728, -0.17721149325370789, -0.09683606773614883, -0.00985441543161869, -0.03744896873831749, -0.08007701486349106, -0.05477055162191391, 0.04622100666165352, 0.0479375422000885, 0.07327546179294586, 0.026617979630827904, 0.051947008818387985, -0.09582288563251495, -0.08590096235275269, 0.023444991558790207, 0.10288529843091965, -0.0530620776116848, 0.06574056297540665, -0.06182540953159332, 0.030548494309186935, 0.013667287304997444, -0.04355970025062561, -0.012437908910214901, -0.16329090297222137, 0.12948742508888245, 0.0007588548469357193, 0.05267022177577019, 0.004539476241916418, -0.1282203048467636, -0.14793500304222107, -0.15002919733524323, 0.010542599484324455, -0.043998997658491135, -0.11263101547956467, -0.18641728162765503, 0.1533362865447998, -0.04775574430823326, -0.029846692457795143, 0.03540625795722008, -0.04648526385426521, 0.011842698790133, -0.1759014129638672, 0.30782219767570496, 0.02270948700606823, 0.003084807889536023, 0.031982287764549255, 0.162301167845726, 0.03156519681215286, 0.02931559830904007, 0.05432320386171341, -0.17275650799274445, 0.14650259912014008, 0.03948226571083069, 0.0054491362534463406, 0.07094229757785797, -0.01836157590150833, -0.34315308928489685, -0.128196582198143, -0.12173353880643845, -0.1546563059091568, -0.07082843035459518}, {-0.02728372998535633, 0.3094256520271301, 0.13681696355342865, -0.03489336371421814, 0.15425071120262146, 0.05781418830156326, 0.2304074764251709, -0.04954364150762558, -0.0801316574215889, -0.06007550656795502, 0.09495189785957336, -0.04763296991586685, -0.04995645582675934, 0.02520843967795372, -0.045993510633707047, -0.13820725679397583, -0.04554274305701256, -0.013680517673492432, -0.020157935097813606, -0.04373137652873993, 0.09958980232477188, 0.16232725977897644, -0.11984661221504211, 0.022269481793045998, -0.08791957050561905, -0.009939032606780529, -0.057124119251966476, 0.17283880710601807, -0.12795330584049225, 0.04373588785529137, -0.1667201817035675, -0.06839058548212051, 0.14418916404247284, 0.0033992889802902937, -0.06297118961811066, 0.05012241378426552, -0.14048892259597778, 0.14756405353546143, 0.03784217685461044, 0.08076676726341248, 0.09488382190465927, -0.07277587056159973, -0.14543557167053223, 0.08005378395318985, 0.1455630213022232, 0.00772758387029171, -0.05988319590687752, -0.02518567629158497, 0.11176996678113937, -0.1276417076587677, -0.23241235315799713, 0.23212790489196777, 0.03471804037690163, 0.08605179190635681, -0.008751894347369671, -0.037525877356529236, -0.1226656436920166, 0.16871696710586548, 0.07385734468698502, -0.05624299496412277, 0.19423547387123108, 0.13468244671821594, 0.06022940203547478, -0.06600119918584824, -0.0516185462474823, -0.054874684661626816, -0.05874790996313095, 0.08022963255643845, 0.008692051284015179, -0.026514779776334763, 0.15020138025283813, -0.1393825262784958, -0.27732884883880615, -0.09755850583314896, 0.31010547280311584, 0.060758356004953384, -0.016161778941750526, 0.1189986914396286, -0.13229802250862122, -0.0943041443824768, -0.03998078405857086, -0.09734953939914703, 0.3211958706378937, -0.24176807701587677, -0.12425483018159866, 0.09686177223920822, 0.17351798713207245, 0.030806275084614754, -0.0653504878282547, -0.05967776104807854, 0.08424053341150284, 0.21023449301719666, 0.02604489214718342, 0.0974486917257309, 0.009998016990721226, -0.12372997403144836, -0.09160374850034714, 0.03909308835864067, -0.03298598527908325, 0.4521943926811218, 0.032015975564718246, -0.07071983069181442, -0.11676423251628876, 0.1388312131166458, -0.029647119343280792, 0.413319855928421, 0.13996796309947968, 0.11763148009777069, 0.019911503419280052, -0.04680810868740082, 0.012783645652234554, -0.039585329592227936, 0.0698767751455307, -0.14570410549640656, 0.05435517430305481, 0.019169140607118607, 0.002016907325014472, 0.0009872021619230509, -0.04650728404521942, 0.012023921124637127, 0.055079080164432526, -0.09767348319292068, 0.09241950511932373, -0.07166027277708054, 0.1503613442182541, 0.00722036138176918, -0.022383667528629303, -0.1297389417886734, 0.053759731352329254, 0.2157285064458847, -0.0018176864832639694, 0.01696557179093361, 0.16556422412395477, -0.022775953635573387, -0.09655208885669708, -0.3447644114494324, -0.02053697779774666, -0.03967021778225899, -0.03035440854728222, -0.14022623002529144, 0.20124265551567078, -0.14743423461914062, 0.004841111600399017, -0.2294231355190277, 0.17334960401058197, 0.05659297853708267, 0.02902568131685257, -0.12936760485172272, -0.03650292381644249, 0.22720475494861603, 0.03330650180578232, -0.028463512659072876, -0.009251575917005539, 0.008955095894634724, 0.046671219170093536, 0.05235414579510689, -0.09475912898778915, 0.3099921941757202, -0.2880564332008362, 0.31613343954086304, 0.2641652524471283, 0.09309516847133636, 0.022187814116477966, -0.02821979485452175, 0.11057060211896896, 0.06569042801856995, -0.037640467286109924, -0.02235998772084713, 0.026783203706145287, -0.15570399165153503, -0.04027846083045006, -0.044160567224025726, 0.13357871770858765, 0.00041408673860132694, 0.1072656512260437, -0.11985613405704498, -0.06191052123904228, 0.09033704549074173, 0.038861457258462906, 0.32047560811042786, -0.09030328691005707, 0.15823613107204437, -0.048775091767311096, 0.1005413755774498, -0.11469133198261261, -0.1509971022605896, -0.031119657680392265, 0.026095271110534668, 0.17603951692581177, -0.019992047920823097, 0.0663491040468216, 0.02301461435854435, -0.09382133930921555, -0.16305969655513763, 0.11642659455537796, -0.25455421209335327, -0.1380578875541687, -0.13986171782016754, -0.024489352479577065, -0.10820097476243973, -0.10483638942241669, -0.07773183286190033, 0.052661631256341934, -0.579707145690918, 0.10840272903442383, -0.24992014467716217, 0.11665712296962738, -0.1851416379213333, -0.10740803182125092, -0.048771463334560394, -0.08199280500411987, 0.036724891513586044, -0.07377081364393234, -0.09076258540153503, 0.04461950808763504, -0.15167778730392456, 0.033832333981990814, -0.052003562450408936, -0.09886360913515091, -0.036041952669620514, 0.10240880399942398, -0.14541023969650269, -0.14290979504585266, 0.16456268727779388, -0.005537515506148338, -0.017260383814573288, -0.23519647121429443, 0.04493423178792, 0.09809033572673798, -0.08625214546918869, -0.24748952686786652, -0.013726356439292431, -0.05959045886993408, -0.13292038440704346, 0.0927032083272934, -0.10505411028862, 0.08368368446826935, 0.016396943479776382, -0.19527684152126312, 0.6551891565322876, -0.039505887776613235, -0.09328990429639816, -0.19822804629802704, 0.06394889950752258, 0.20049823820590973, -0.016769394278526306, -0.03828824684023857, -0.18619054555892944, -0.22150659561157227, -0.006677386350929737, -0.024629563093185425, 0.3346157670021057, -0.09573792666196823, -0.07653823494911194, 0.07833965122699738, 0.03166183456778526}, {0.18789231777191162, -0.2663653492927551, 0.056473854929208755, -0.24804969131946564, 0.0024461811408400536, -0.14461097121238708, 0.031124543398618698, 0.028775719925761223, -0.04444010555744171, 0.3414222300052643, -0.11442744731903076, 0.010692869313061237, 0.006606009788811207, 0.011977237649261951, -0.1890801042318344, 0.11281431466341019, -0.12087033689022064, -0.20796871185302734, -0.04706870764493942, 0.12305813282728195, -0.28358548879623413, 0.15542705357074738, -0.047649532556533813, 0.11799050122499466, -0.10524605214595795, -0.11202822625637054, -0.08047524094581604, 0.058477677404880524, -0.17978690564632416, -0.0649360716342926, 0.07056969404220581, -0.026404550299048424, 0.03752041980624199, -0.07661814987659454, 0.11584262549877167, 0.16472820937633514, 0.006472548935562372, 0.09633836150169373, 0.08207628130912781, -0.05737932771444321, 0.1448771208524704, -0.08065441995859146, -0.08302979916334152, -0.004628981929272413, 0.05326779931783676, -0.03365084528923035, -0.000651234295219183, 0.16891179978847504, 0.016824442893266678, 0.03498232737183571, -0.055634237825870514, -0.10649215430021286, 0.17419494688510895, 0.17864446341991425, 0.002789880149066448, 0.04549875855445862, 0.15050344169139862, 0.0009329082677140832, 0.0046189771965146065, -0.10517939925193787, -0.12446054071187973, -0.10460156947374344, 0.2402220070362091, 0.11500900983810425, 0.09159964323043823, 0.21280358731746674, -0.11909649521112442, -0.034357473254203796, -0.5487101078033447, 0.014322996139526367, 0.11105513572692871, -0.12462449818849564, -0.2307923138141632, 0.06285577267408371, 0.14370527863502502, 0.09494772553443909, -0.3332452178001404, 0.03805658221244812, -0.3060656785964966, -0.011025959625840187, 0.0860077515244484, 0.11562284082174301, -0.010111321695148945, -0.11020936071872711, -0.059427738189697266, -0.066803939640522, 0.05923130735754967, -0.28157809376716614, -0.1766921728849411, -0.13754712045192719, -0.008699203841388226, 0.0577048696577549, -0.11901002377271652, 0.04431845247745514, -0.13891860842704773, -0.03624266758561134, -0.05485635623335838, 0.11929795891046524, -0.09560492634773254, 0.010043497197329998, -0.12695002555847168, -0.13555888831615448, -0.13704563677310944, -0.10674194246530533, -0.06010684743523598, -0.059071026742458344, -0.03856593370437622, -0.365299254655838, 0.166033536195755, -0.049215011298656464, 0.14524342119693756, 0.09761958569288254, 0.07039131969213486, 0.015163878910243511, 0.06191658973693848, 0.1272771656513214, -0.17516520619392395, -0.04639413207769394, 0.15187717974185944, 0.20382718741893768, -0.04697991907596588, -0.002150318818166852, -0.2875746190547943, -0.05204058066010475, -0.00613014493137598, -0.02972419373691082, 0.01044453401118517, -0.10557134449481964, 0.06903418153524399, -0.08822721987962723, 0.15559662878513336, 0.12983959913253784, 0.12919557094573975, 0.0730154886841774, 0.05629771575331688, -0.04462619870901108, 0.0207821037620306, 0.015732480213046074, 0.028058521449565887, -0.26227110624313354, 0.04690876230597496, -0.051767513155937195, -0.23424279689788818, 0.11549807339906693, -0.04085405543446541, -0.04119854420423508, 0.03747829422354698, -0.1763671338558197, 0.13171155750751495, -0.12580223381519318, 0.2838408052921295, -0.06732840836048126, -0.05677531659603119, -0.1697133183479309, 0.2105003148317337, -0.08023945987224579, -0.3089483082294464, 0.03380474075675011, -0.09505274891853333, 0.24203376471996307, 0.13484631478786469, 0.2899252474308014, -0.013054472394287586, -0.2645091414451599, 0.050246331840753555, 0.27098050713539124, 0.12099877744913101, -0.04133554548025131, 0.059678755700588226, -0.10189205408096313, -0.15338505804538727, 0.0844903513789177, 0.023310868069529533, 0.06053079292178154, 0.21229861676692963, 0.02699541673064232, -0.11498130112886429, 0.19225037097930908, -0.24059376120567322, 0.28179430961608887, -0.027072547003626823, -0.21392959356307983, -0.12939877808094025, 0.13577617704868317, 0.1978847235441208, 0.028068194165825844, -0.005027199164032936, 0.08776901662349701, 0.041852422058582306, -0.005717986263334751, 0.046226467937231064, 0.1278323084115982, 0.04192196577787399, 0.13371726870536804, -0.14117127656936646, -0.006562639027833939, -0.09053431451320648, -0.03563113138079643, -0.20480449497699738, -0.17315024137496948, 0.052411872893571854, -0.1571156531572342, -0.12530401349067688, -0.043635595589876175, 0.11315684020519257, -0.06468771398067474, 0.13303959369659424, -0.07769843935966492, -0.1530730426311493, -0.027041805908083916, -0.061433807015419006, -0.1965993195772171, -0.11814466118812561, 0.11871923506259918, 0.012145224027335644, -0.012135007418692112, -0.01941503770649433, 0.1517835110425949, -0.08395731449127197, 0.01343189924955368, -0.06935924291610718, -0.21057845652103424, -0.08680684119462967, -0.34159764647483826, -0.12802523374557495, -0.2550918161869049, -0.1562325656414032, -0.06978543847799301, 0.05962064117193222, -0.04223155230283737, -0.10966531187295914, -0.15861502289772034, 0.07445000857114792, -0.0062680901028215885, 0.3697664737701416, 0.1147150918841362, -0.23933005332946777, 0.1918131560087204, -0.013762653805315495, 0.05481640249490738, 0.09081649780273438, 0.1333468109369278, -0.022965187206864357, 0.06293127685785294, 0.11922886222600937, -0.148219496011734, -0.06130451336503029, -0.0966750755906105, -0.16050966084003448, 0.03160293400287628, 0.009930362924933434, -0.18024735152721405, -0.05328625068068504, 0.031023189425468445, -0.1825554519891739, 0.280359148979187}, {-0.1411728858947754, -0.0458160862326622, 0.12760184705257416, 0.028427472338080406, 0.11128614097833633, 0.07151156663894653, -0.08531025052070618, 0.022155994549393654, -0.11154010146856308, -0.15380509197711945, 0.09125892072916031, -0.07792563736438751, -0.07366794347763062, 0.09446954727172852, 0.19898952543735504, 0.1341608464717865, 0.07682580500841141, -0.016473937779664993, -0.11173202842473984, -0.2832520008087158, 0.17234307527542114, -0.11348790675401688, 0.22204098105430603, -0.029341531917452812, -0.20957061648368835, -0.1558002233505249, 0.08460333198308945, 0.0535971000790596, -0.007354096509516239, 0.2076958268880844, -0.021173397079110146, 0.16964325308799744, 0.045830488204956055, 0.05883689597249031, -0.02971859835088253, -0.18677102029323578, 0.09022609144449234, -0.15684694051742554, 0.03830762952566147, 0.11624763160943985, 0.05731517821550369, -0.053538743406534195, 0.12954340875148773, 0.12464070320129395, -0.12417800724506378, -0.06504851579666138, -0.008441231213510036, -0.11853615939617157, -0.0752125158905983, 0.09159088134765625, 0.09367761015892029, -0.04182731732726097, -0.19058962166309357, 0.10926114767789841, 0.06435036659240723, -0.08846200257539749, 0.09138990938663483, 0.2558283805847168, 0.07891438156366348, -0.35012364387512207, -0.08682352304458618, 0.027257459238171577, 0.01652788743376732, 0.1384017914533615, 0.10857321321964264, -0.16100092232227325, 0.10940363258123398, 0.33800047636032104, -0.02908424474298954, 0.03463917225599289, -0.01734801195561886, 0.09636793285608292, -0.14042378962039948, 0.03642166033387184, -0.012425318360328674, -0.014025197364389896, -0.02737690880894661, 0.06698945164680481, 0.02734987996518612, 0.07144085317850113, -0.10060478001832962, 0.045683082193136215, 0.027346573770046234, -0.21394085884094238, 0.09001502394676208, -0.10149621963500977, 0.03153599426150322, 0.1951771378517151, -0.18860496580600739, -0.06216024234890938, 0.014283665455877781, 0.27250003814697266, -0.11100303381681442, -0.15441925823688507, 0.07766629010438919, 0.004945016000419855, 0.0029507153667509556, -0.20475222170352936, 0.022764593362808228, 0.0008588015916757286, -0.05565224960446358, 0.1629742681980133, -0.14212016761302948, 0.06366785615682602, 0.10258021950721741, 0.3242984712123871, 0.10652831941843033, 0.021015124395489693, -0.15866027772426605, -0.1475399136543274, -0.05357588082551956, -0.07724949717521667, 0.0841090977191925, 0.11951178312301636, -0.11857956647872925, -0.07497494667768478, 0.19720615446567535, -0.0653882697224617, 0.41107308864593506, -0.030339378863573074, 0.004398603457957506, -0.06680700927972794, -0.024463722482323647, 0.21406283974647522, 0.07680536806583405, 0.18296125531196594, 0.07365131378173828, -0.07533332705497742, -0.10374939441680908, 0.007566016633063555, 0.09206518530845642, 0.13361024856567383, 0.17045854032039642, 0.016458090394735336, -0.05303604155778885, -0.14304500818252563, 0.07214716821908951, 0.13116107881069183, -0.016799749806523323, 0.21812234818935394, -0.016673067584633827, 0.021569794043898582, -0.12463070452213287, -0.10490181297063828, 0.21047426760196686, -0.09001445025205612, -0.055842649191617966, -0.12716849148273468, 0.019642410799860954, -0.11618021875619888, 0.14001774787902832, 0.06971871107816696, 0.19664955139160156, -0.20208559930324554, -0.07953760772943497, 0.27760422229766846, -0.11469274014234543, 0.24129244685173035, -0.20313763618469238, -0.02967802621424198, -0.21300674974918365, -0.08506882935762405, 0.1668366938829422, 0.15715618431568146, 0.02125023677945137, -0.17824184894561768, -0.3232785761356354, -0.08549816161394119, 0.030350202694535255, -0.008738293312489986, -0.0253891758620739, 0.01800009049475193, -0.0683240219950676, -0.23502878844738007, -0.18899156153202057, 0.08495408296585083, 0.09585680812597275, -0.2684246599674225, 0.014620468020439148, 0.0157783180475235, -0.11918817460536957, -0.1215553730726242, 0.05814022198319435, 0.07502996176481247, 0.07393911480903625, 0.043111614882946014, -0.18748807907104492, 0.17922642827033997, 0.03600972145795822, 0.27177971601486206, 0.07222970575094223, -0.11645867675542831, 0.09713725000619888, -0.02045069821178913, 0.05416058748960495, -0.2550361156463623, -0.13690392673015594, 0.11263839155435562, 0.029375990852713585, 0.1501055508852005, 0.08463457971811295, 0.28857117891311646, -0.053159166127443314, -0.1993197351694107, -0.13973000645637512, -0.039925217628479004, 0.2891709506511688, 0.20469507575035095, 0.060798630118370056, 0.008562822826206684, -0.06195209547877312, -0.2479662001132965, -0.054896704852581024, 0.021930629387497902, 0.03477628529071808, 0.1817992478609085, -0.04728825390338898, 0.004485124256461859, -0.1274811178445816, -0.08852465450763702, -0.17500197887420654, 0.002103799022734165, -0.0876028835773468, -0.3573761284351349, -0.29475757479667664, -0.03703370690345764, -0.12212719768285751, -0.0032035773620009422, 0.09173871576786041, -0.1113557294011116, 0.08737268298864365, -0.006357439327985048, -0.04757898673415184, -0.30357825756073, -0.22715309262275696, -0.03045503795146942, -0.03761206567287445, 0.30221590399742126, 0.08400681614875793, 0.13066111505031586, 0.0293758325278759, 0.253775030374527, -0.08983828872442245, -0.043720878660678864, -0.0707339346408844, -0.016166402027010918, -0.030078496783971786, -0.14314678311347961, 0.045139629393815994, 0.12596547603607178, 0.06875073164701462, 0.06378676742315292, -0.07342831790447235, 0.18694591522216797, -0.030308224260807037, -0.021727556362748146}, {0.08545998483896255, -0.014946091920137405, 0.07108346372842789, 0.18369698524475098, 0.12234175205230713, 0.26522818207740784, -0.09459041804075241, -0.12781928479671478, 0.21912069618701935, -0.06072276830673218, -0.06321089714765549, -0.03521016612648964, 0.07015015929937363, 0.06725794076919556, 0.061393506824970245, 0.05410446971654892, 0.06863277405500412, -0.09650374948978424, -0.0714624673128128, -0.06815068423748016, 0.21052055060863495, -0.024898912757635117, -0.13166749477386475, 0.09912578761577606, -0.10013452917337418, -0.18814793229103088, -0.10401596873998642, -0.10962624102830887, 0.12825517356395721, 0.0815679132938385, -0.40898397564888, 0.03053554892539978, 0.2021428793668747, -0.06909818202257156, -0.09819317609071732, 0.08497694134712219, -0.050031643360853195, 0.24217884242534637, -0.11546020954847336, 0.20405513048171997, -0.09661733359098434, 0.1295623481273651, -0.03919718787074089, 0.06825987249612808, 0.02191590890288353, 0.028503047302365303, -0.28291091322898865, -0.03898272663354874, -0.13487283885478973, -0.13775643706321716, 0.13721174001693726, -0.023645024746656418, 0.09108833968639374, 0.022543802857398987, -0.005584531929343939, -0.1986369639635086, 0.07618137449026108, 0.11791875958442688, 0.12744946777820587, 0.010060402564704418, 0.13391633331775665, -0.0900907814502716, 0.14784584939479828, -0.07317537069320679, -0.144760861992836, -0.06221388652920723, -0.06283126026391983, 0.01509859599173069, 0.17176540195941925, 0.0021601682528853416, -0.055745795369148254, 0.23996129631996155, 0.013411851599812508, -0.10256537050008774, 0.08147556334733963, -0.1849367618560791, 0.01550468523055315, -0.0029294313862919807, -0.23247948288917542, -0.17676019668579102, 0.06105200573801994, 0.07470382004976273, 0.09362860023975372, -0.14065535366535187, 0.13337096571922302, -0.028677955269813538, 0.09914026409387589, -0.05798235535621643, -0.0422697551548481, -0.10166067630052567, 0.0710768774151802, 0.13585659861564636, 0.017715152353048325, 0.21834152936935425, -0.0716632828116417, -0.0640224814414978, 0.10196264833211899, -0.08319850265979767, 0.016697516664862633, 0.050356943160295486, -0.021055711433291435, 0.05326859652996063, -0.1903984397649765, -0.04025079682469368, 0.32582318782806396, 0.22718968987464905, -0.016698138788342476, -0.10340259969234467, -0.09216390550136566, -0.09842918068170547, 0.1257646232843399, -0.03695391118526459, -0.0810454934835434, -0.13998763263225555, 0.0393836610019207, -0.03552980720996857, -0.003610617946833372, -0.14685693383216858, -0.044699158519506454, -0.019063884392380714, -0.024871326982975006, 0.09484332799911499, -0.35771724581718445, -0.21375228464603424, -0.06528979539871216, 0.061234794557094574, -0.01020177360624075, -0.19816730916500092, 0.08235909044742584, -0.013223981484770775, 0.22226153314113617, 0.18780864775180817, 0.11896827071905136, -0.18035078048706055, 0.096565380692482, -0.08926135301589966, -0.032088037580251694, -0.049753084778785706, 0.0328189879655838, 0.2920432686805725, -0.06511703133583069, -0.190404012799263, -0.032531045377254486, 0.04357298091053963, 0.13458780944347382, 0.08452556282281876, -0.016803786158561707, 0.015184177085757256, 0.026193123310804367, 0.006973696406930685, 0.06671837717294693, -0.2877015471458435, -0.0862904042005539, -0.025058839470148087, 0.04333919286727905, -0.09921544045209885, 0.03553992509841919, -0.014033169485628605, 0.025186985731124878, -0.05429898947477341, -0.007962343282997608, 0.09648660570383072, 0.19002552330493927, 0.2523263394832611, -0.1021135225892067, 0.015147173777222633, 0.038375917822122574, -0.015115326270461082, -0.30283793807029724, -0.2657110095024109, 0.040069255977869034, -0.015779923647642136, 0.08012853562831879, 0.07657713443040848, 0.025520410388708115, -0.10871636122465134, 0.04864851012825966, 0.0433674082159996, -0.07612869143486023, 0.11414527148008347, 0.31666094064712524, 0.008974657393991947, 0.027087468653917313, -0.10691683739423752, 0.07556454092264175, 0.22541634738445282, 0.07929567992687225, 0.09747005254030228, -0.029542112722992897, -0.014825417660176754, 0.0631449967622757, -0.03349709138274193, -0.04016498848795891, 0.04871068522334099, 0.0031991377472877502, 0.10263200104236603, 0.060201339423656464, 0.04021575674414635, 0.0374068059027195, 0.04621129855513573, -0.2265619933605194, -0.0554962120950222, 0.04291793331503868, -0.12576912343502045, 0.07851990312337875, -0.33653342723846436, 0.15157945454120636, 0.37699517607688904, 0.1782018542289734, 0.042219508439302444, 0.08210290223360062, 0.02751857228577137, -0.18019868433475494, -0.05202450975775719, -0.07671993225812912, 0.11744121462106705, 0.05925599858164787, -0.006921623833477497, 0.07139196991920471, 0.21195361018180847, 0.052303001284599304, -0.14125484228134155, -0.01919507049024105, -0.010419613681733608, 0.10583563894033432, 0.05347595736384392, -0.025808628648519516, 0.033457666635513306, 0.0459500290453434, -0.06468670815229416, -0.0024559872690588236, 0.0464162714779377, 0.2873096168041229, 0.1917448341846466, 0.00035461076186038554, 0.24517056345939636, 0.2775239944458008, 0.022000975906848907, -0.08878876268863678, 0.15600523352622986, -0.25198936462402344, -0.06291886419057846, -0.10057010501623154, -0.06194508075714111, -0.030336221680045128, 0.21039609611034393, 0.028232363983988762, -0.030417414382100105, -0.2442587912082672, 0.05293731391429901, 0.12695831060409546, 0.050021201372146606, -0.03653797507286072, -0.11565183103084564, -0.004641571547836065, 0.06271575391292572}, {0.10260943323373795, 0.12995721399784088, 0.030607100576162338, 0.1337808519601822, -0.054948143661022186, -0.010093111544847488, -0.12467387318611145, 0.1293758898973465, -0.1459619104862213, -0.12011223286390305, 0.07024671882390976, 0.02490350976586342, -0.024209601804614067, -0.04295007511973381, -0.13575881719589233, -0.08813094347715378, -0.06249666586518288, 0.10569654405117035, -0.0658629834651947, 0.010210931301116943, -0.24173609912395477, 0.047512203454971313, 0.12488903850317001, -0.05750759318470955, -0.09123198688030243, -0.10746973752975464, 0.09389479458332062, 0.05209951102733612, 0.18789033591747284, -0.1480512171983719, -0.0909527912735939, -0.10426463931798935, 0.2660325765609741, -0.08166195452213287, -0.1745878905057907, 0.1804867386817932, 0.09550587087869644, -0.17084935307502747, 0.1394968181848526, 0.26453086733818054, 0.04912083223462105, -0.07849469780921936, -0.1672496199607849, -0.017425669357180595, 0.004347749520093203, -0.05675781890749931, -0.39331045746803284, 0.10100618004798889, 0.09198153764009476, 0.035129204392433167, 0.40206247568130493, -0.040647923946380615, -0.0922795981168747, -0.006535769905894995, -0.14735068380832672, -0.07463790476322174, -0.03358682245016098, 0.11215667426586151, -0.12798316776752472, -0.049157414585351944, 0.047711554914712906, -0.03332068771123886, -0.2654893696308136, -0.1810060292482376, -0.15279392898082733, -0.1713119000196457, -0.05789254605770111, 0.11696553975343704, 0.005363997537642717, 0.0780239924788475, -0.12016462534666061, -0.12340372800827026, -0.15565158426761627, 0.020731952041387558, 0.030791960656642914, 0.21904779970645905, -0.11469485610723495, 0.0028007549699395895, 0.03143995255231857, -0.015503590926527977, 0.020176008343696594, 0.07481592893600464, 0.18548600375652313, -0.06270652264356613, 0.03458832576870918, -0.064839206635952, 0.1386813521385193, 0.2743752896785736, -0.06485630571842194, 0.0061527094803750515, -0.023373836651444435, 0.059084225445985794, -0.08547569811344147, 0.09624526649713516, -0.026559380814433098, 0.015494443476200104, -0.23784607648849487, 0.10874807089567184, 0.07659437507390976, 0.12296740710735321, -0.016521360725164413, -0.34536656737327576, 0.05771307274699211, -0.04608102887868881, -0.08738898485898972, 0.3055952191352844, -0.0026995521038770676, 0.3359753489494324, -0.19511401653289795, 0.06469349563121796, 0.0716187134385109, -0.0019827247597277164, 0.019578153267502785, 0.17048166692256927, 0.14598557353019714, -0.0011910267639905214, 0.16641025245189667, 0.01836216263473034, 0.013105662539601326, 0.17465481162071228, 0.22739236056804657, 0.05048038437962532, -0.13744784891605377, 0.05255569890141487, -0.14020460844039917, 0.08868256211280823, -0.04045114666223526, 0.08812341839075089, 0.03599859029054642, 0.09157713502645493, 0.23406748473644257, 0.04483212158083916, 0.11589539796113968, -0.24735067784786224, -0.024685673415660858, -0.051423557102680206, -0.3183276951313019, 0.01845000870525837, -0.057841744273900986, -0.08459141105413437, -0.035311270505189896, 0.1042783334851265, -0.05428213253617287, -0.10048595815896988, -0.05802779272198677, 0.12857189774513245, 0.21804922819137573, 0.013786292634904385, 0.009592273272573948, 0.14305628836154938, 0.006174882408231497, 0.06346842646598816, 0.04263247177004814, 0.03384486213326454, 0.02478432096540928, -0.11989761143922806, -0.3015817701816559, 0.19561520218849182, -0.29702576994895935, 0.2371406853199005, -0.1406482458114624, 0.0006592663703486323, 0.006274511106312275, -0.04376101493835449, -0.01594744622707367, -0.20842532813549042, -0.11385661363601685, -0.318857878446579, -0.03329305350780487, -0.011150365695357323, 0.09909085184335709, 0.22967910766601562, 0.051172856241464615, 0.2113489806652069, 0.0036278816405683756, 0.04093758016824722, 0.06793920695781708, -0.15222933888435364, 0.058667805045843124, 0.010107552632689476, 0.00886356458067894, -0.18442516028881073, 0.12054838985204697, -0.04283060505986214, 0.10724068433046341, 0.12029892951250076, -0.1534358412027359, -0.21583938598632812, 0.19410264492034912, -0.26559579372406006, -0.009975475259125233, -0.35663866996765137, -0.11102618277072906, 0.14261427521705627, -0.10375956445932388, 0.1576836258172989, 0.1359700858592987, -0.27732786536216736, 0.008800043724477291, 0.04597795754671097, 0.01996796578168869, -0.004889470990747213, 0.0945473462343216, -0.06290090084075928, -0.025503067299723625, 0.04088003560900688, -0.007345164660364389, -0.12342590093612671, -0.08831450343132019, -0.14188750088214874, 0.16806650161743164, -0.03654783219099045, -0.20199701189994812, 0.06784608960151672, 0.04401716962456703, 0.04687299579381943, 0.21414799988269806, 0.1307971328496933, 0.10456568002700806, 0.3183343708515167, 0.02303219400346279, 0.02846882864832878, 0.059468816965818405, -0.11185505986213684, -0.04692007601261139, -0.0651136040687561, 0.03389306738972664, -0.14472270011901855, 0.0662916973233223, -0.046572305262088776, 0.28836172819137573, 0.009605927392840385, 0.006030685268342495, 0.13878175616264343, -0.05986177176237106, 0.24685226380825043, 0.04455762729048729, 0.16246306896209717, -0.1250186264514923, 0.1462993621826172, -0.04772763326764107, 0.014323118142783642, -0.2577000856399536, -0.011215593665838242, -0.07346948236227036, -0.24094654619693756, 0.028571022674441338, 0.1767875850200653, 0.06743154674768448, 0.1357431709766388, 0.16851790249347687, 0.0057334271259605885, 0.23420874774456024, -0.030207164585590363, -0.09007996320724487, -0.06371061503887177}, {-0.011933979578316212, 0.1746484637260437, -0.26408934593200684, -0.11765541136264801, -0.06381959468126297, -0.037154670804739, -0.08193150907754898, -0.07148467749357224, 0.09030888974666595, -0.0429145023226738, 0.04421713948249817, -0.13432741165161133, -0.14318136870861053, -0.21455107629299164, -0.19601640105247498, 0.1934695690870285, -0.18496939539909363, -0.005353547167032957, -0.035769540816545486, 0.22621405124664307, 0.07165588438510895, 0.12133600562810898, 0.07329847663640976, -0.08295363187789917, 0.0696970596909523, -0.14191241562366486, -0.015321486629545689, -0.07724791765213013, 0.1640530675649643, 0.017798397690057755, -0.06305749714374542, -0.11559722572565079, -0.15499740839004517, 0.011396070942282677, -0.10058309137821198, 0.217381089925766, 0.09834938496351242, 0.17044927179813385, -0.025639686733484268, -0.020985856652259827, -0.05295189842581749, 0.046312909573316574, -0.009506146423518658, 0.04493597894906998, -0.05229014903306961, 0.015898102894425392, 0.09683528542518616, -0.13711902499198914, -0.17519141733646393, 0.06136385351419449, -0.09256614744663239, -0.1037105917930603, 0.01905500888824463, -0.1427183300256729, -0.09381530433893204, -0.10660593211650848, -0.12565071880817413, -0.16506722569465637, -0.19706875085830688, -0.03197513148188591, -0.141442209482193, 0.08695356547832489, 0.05387717857956886, -0.03536555543541908, -0.17699450254440308, -0.029765741899609566, 0.13782893121242523, -0.30724748969078064, 0.05741003155708313, 0.1729215681552887, -0.009197051636874676, -0.10139290243387222, -0.11294781416654587, -0.052407633513212204, 0.07045014947652817, -0.04641621559858322, -0.08840305358171463, -0.15362529456615448, -0.0995713397860527, 0.2043166607618332, 0.06806893646717072, 0.07862408459186554, -0.013440735638141632, 0.11914810538291931, 0.10360627621412277, -0.08482784032821655, 0.03291865438222885, -0.06949248164892197, 0.132097065448761, -0.24654684960842133, -0.0740276649594307, -0.1148747131228447, 0.11271461099386215, -0.04787615314126015, -0.15066589415073395, 0.18019960820674896, 0.09234948456287384, 0.077049620449543, -0.010028371587395668, 0.5080245733261108, -0.013958359137177467, -0.21091192960739136, 0.16553084552288055, 0.017124993726611137, -0.08747578412294388, -1.1071228981018066, 0.028063809499144554, -0.12044408172369003, -0.07811018079519272, -0.00759004894644022, 0.15025551617145538, -0.16972315311431885, 0.06269628554582596, -0.16043636202812195, -0.04339540749788284, 0.01845984160900116, -0.15555697679519653, 0.055845070630311966, -0.11094176024198532, 0.20821072161197662, -0.08011866360902786, -0.07852713018655777, -0.2571629285812378, -0.03130188211798668, 0.008083361200988293, 0.10006836801767349, 0.04090900719165802, -0.029489761218428612, -0.006794186308979988, 0.10778839141130447, -0.22244268655776978, -0.1384192556142807, 0.05713344365358353, 0.029162148013710976, 0.020971201360225677, 0.03449184447526932, -0.206882506608963, -0.1556856483221054, 0.18172027170658112, -0.15259414911270142, -0.06819099187850952, -0.008498577401041985, 0.024871762841939926, -0.25388234853744507, -0.1730736643075943, -0.06738892942667007, -0.11327666789293289, 0.19892308115959167, -0.12762457132339478, 0.03058570995926857, 0.014796532690525055, -0.009477643296122551, -0.3409569263458252, 0.16498830914497375, 0.19923105835914612, -0.04741459712386131, 0.05994243919849396, 0.004604006186127663, -0.1069926768541336, 0.8574925661087036, -0.004990970715880394, 0.022145189344882965, -0.1443607360124588, -0.023367973044514656, -0.10249429941177368, 0.12455464154481888, -0.47062408924102783, 0.2715282440185547, 0.004305726382881403, -0.3495917320251465, 0.03000635839998722, -0.008020881563425064, 0.31136244535446167, 0.06990567594766617, 0.12152455002069473, -0.28880074620246887, -0.38947856426239014, -0.21544450521469116, 0.14112302660942078, 0.10669085383415222, -0.01825135387480259, -0.2397092580795288, -0.16378790140151978, 0.13750842213630676, -0.0392112135887146, 0.14909760653972626, 0.09427677094936371, 0.05592653900384903, 0.0721832811832428, -0.09891919791698456, 0.037188511341810226, 0.03551185131072998, 0.0404961071908474, 0.20068059861660004, 0.09575619548559189, 0.2559761703014374, -0.021764473989605904, -0.010452168993651867, 0.07388131320476532, -0.14762352406978607, -0.21715907752513885, 0.04687075316905975, 0.1832982450723648, -0.1607946753501892, 0.18762782216072083, 0.024586409330368042, -0.05368407815694809, 0.020788995549082756, -0.16255956888198853, -0.107111357152462, -0.08947063237428665, -0.21226324141025543, 0.07174695283174515, -0.05899474769830704, 0.07269906997680664, -0.23839546740055084, -0.12763209640979767, 0.06759846955537796, -0.01603700779378414, -0.08425819128751755, 0.28298503160476685, 0.06029736250638962, -0.017299707978963852, 0.03255673497915268, -0.2299795001745224, -0.050870250910520554, 0.034184060990810394, 0.03052368387579918, -0.017814846709370613, -0.001649104175157845, -0.031332828104496, -0.058596860617399216, 0.04506736993789673, 0.09696939587593079, -0.11646810173988342, 0.04840452969074249, 0.05116607993841171, -0.04906078055500984, -0.08201422542333603, 0.23501960933208466, 0.0030081490986049175, -0.08127690106630325, 0.14349105954170227, -0.1735210120677948, 0.13615216314792633, -0.14544200897216797, -0.01350369956344366, -0.047843098640441895, -0.16170905530452728, 0.022104427218437195, -0.017216864973306656, 0.10727688670158386, 0.15548546612262726, -0.1335546374320984, 0.01003008522093296, -0.1977895349264145}, {-0.07331862300634384, 0.08673611283302307, 0.05003689229488373, -0.18212109804153442, 0.057100776582956314, 0.4220196306705475, 0.004665008746087551, -0.014902664348483086, -0.025232290849089622, -0.07124782353639603, -0.2378830462694168, 0.08272146433591843, -0.1217295452952385, 0.028972510248422623, 0.09121455997228622, -0.04918329790234566, -0.008824571967124939, 0.27543604373931885, -0.06316155195236206, -0.008535322733223438, -0.004731933586299419, -0.0016366811469197273, 0.033827416598796844, 0.19566486775875092, 0.04654579982161522, 0.04631626605987549, 0.05712439864873886, -0.19380535185337067, -0.12969079613685608, -0.0470866858959198, -0.044530827552080154, 0.19144810736179352, -0.06448709964752197, -0.09888099879026413, -0.10984162986278534, 0.38755854964256287, -0.07511240988969803, 0.062099624425172806, -0.09236159920692444, 0.08902484178543091, -0.23611092567443848, -0.09214841574430466, 0.007579382974654436, -0.10257089883089066, -0.02757154405117035, -0.09826478362083435, -0.03697233647108078, -0.007226140238344669, -0.05533263087272644, 0.06964011490345001, 0.21455812454223633, -0.12696786224842072, -0.002679636934772134, -0.10290095955133438, 0.07905471324920654, -0.005711367819458246, -0.05678863078355789, 0.0782107561826706, 0.041076309978961945, 0.10879574716091156, 0.06813909113407135, -0.01919225975871086, 0.360311895608902, -0.02860909514129162, -0.12593421339988708, 0.2085391879081726, 0.05379191413521767, 0.13306599855422974, -0.07356894016265869, -0.10596601665019989, 0.01582658477127552, 0.051140788942575455, -0.009878475219011307, -0.038140565156936646, 0.08160755783319473, -0.3501938581466675, 0.15434938669204712, -0.06910455971956253, -0.06855221837759018, -0.09909580647945404, 0.06560184806585312, -0.06649158149957657, -0.07647121697664261, -0.10981691628694534, 0.1477070301771164, 0.020855696871876717, 0.053192138671875, 0.06855467706918716, -0.036541495472192764, -0.11184734106063843, 0.014428702183067799, 0.14712835848331451, 0.017518440261483192, 0.12545877695083618, -0.15532737970352173, -0.07582423090934753, 0.14086060225963593, -0.15002025663852692, 0.07860587537288666, -0.2405233234167099, -0.08639566600322723, 0.0029976083897054195, -0.05597766116261482, 0.03838365152478218, -0.008112866431474686, 0.14346802234649658, -0.07636932283639908, -0.07832150906324387, -0.10743528604507446, 0.006640393752604723, -0.031196847558021545, 0.20242026448249817, 0.10678629577159882, -0.16774244606494904, -0.0812329575419426, -0.01771659404039383, 0.0972343236207962, -0.1988871842622757, 0.08977444469928741, -0.17655815184116364, -0.006320492830127478, 0.06691381335258484, -0.024873914197087288, -0.05835016816854477, 0.23121164739131927, 0.052396345883607864, 0.022983085364103317, 0.011947834864258766, 0.12915468215942383, -0.00747723551467061, 0.07130618393421173, 0.1621752679347992, -0.04701542854309082, -0.0680762231349945, 0.2789101302623749, -0.18456153571605682, 0.1658085733652115, -0.029495641589164734, 0.06032463163137436, -0.0753229409456253, -0.08101671189069748, -0.0768965631723404, -0.06883171200752258, 0.13328073918819427, -0.09514591842889786, -0.06268341839313507, -0.16011077165603638, -0.015027575194835663, -0.19125786423683167, -0.06714005023241043, 0.020734021440148354, -0.004685760475695133, 0.0029202490113675594, 0.0030782357789576054, 0.17761848866939545, 0.026415416970849037, 0.21186088025569916, -0.04917777702212334, 0.18298104405403137, 0.14034460484981537, -0.05515868216753006, -0.0829443633556366, 0.1649155616760254, -0.14841842651367188, -0.04126212000846863, -0.07343710213899612, -0.010079103522002697, -0.006459842901676893, -0.21183782815933228, -0.24026048183441162, -0.07671792805194855, -0.17310287058353424, 0.028944339603185654, 0.4563204050064087, 0.3050350248813629, -0.1791929453611374, -0.11144822835922241, 0.11042044311761856, -0.11533535271883011, -0.07906807959079742, 0.13248871266841888, -0.011919277720153332, -0.004475396126508713, -0.005236471071839333, 0.0602312795817852, 0.3738132119178772, 0.05341760441660881, -0.15370571613311768, 0.07905099540948868, 0.021233558654785156, 0.0005850843153893948, -0.02170742116868496, -0.04480160400271416, -0.03959474712610245, -0.21583908796310425, 0.12399086356163025, 0.22151440382003784, 0.11609295010566711, -0.03594442456960678, -0.235814169049263, -0.09975221008062363, -0.12954586744308472, 0.14993958175182343, 0.1363726705312729, -0.10907459259033203, -0.2383815050125122, 0.05570584163069725, 0.14047423005104065, 0.35413858294487, 0.06626470386981964, 0.041778188198804855, -0.13261836767196655, 0.002146110637113452, -0.1653454750776291, -0.09402927756309509, -0.03904388099908829, 0.04237792640924454, -0.061641354113817215, -0.09986559301614761, -0.017237437888979912, 0.00480148009955883, -0.004137569572776556, 0.049575336277484894, -0.17684046924114227, 0.01942279189825058, -0.14390264451503754, -0.0942850336432457, 0.007115810643881559, 0.07964679598808289, 0.09071469306945801, -0.06604582071304321, 0.07981900870800018, -0.14000660181045532, 0.3055696189403534, -0.15012003481388092, 0.09741710871458054, 0.14629806578159332, -0.04022965207695961, -0.090471550822258, -0.034225355833768845, -0.18957765400409698, 0.052400875836610794, 0.3199569880962372, -0.20500609278678894, 0.1762450486421585, -0.020219527184963226, 0.05769295245409012, 0.012864142656326294, 0.010370255447924137, -0.11629197001457214, -0.053784992545843124, -0.05422992631793022, 0.08627308160066605, -0.03135759010910988, 0.2006099969148636, 0.10759257525205612}, {0.1548110991716385, -0.22285687923431396, 0.1381268948316574, -0.11443258076906204, 0.1399209052324295, -0.13958799839019775, 0.10830003768205643, -0.008241606876254082, 0.27772408723831177, 0.22480425238609314, 0.25692400336265564, 0.007173281162977219, -0.05190069228410721, -0.04366032034158707, -0.1465989351272583, 0.018385156989097595, -0.1734086573123932, 0.035966601222753525, -0.03792283684015274, 0.08311425894498825, -0.030125943943858147, 0.062163133174180984, -0.013322467915713787, 0.07488244026899338, 0.0326712392270565, -0.16092418134212494, 0.01383118610829115, -0.11299236863851547, 0.12040866911411285, 0.1494845151901245, -0.3960197865962982, -0.07610061019659042, 0.3266535699367523, 0.03491945564746857, 0.15719598531723022, -0.026420695707201958, -0.05506850406527519, 0.322715699672699, -0.17739443480968475, -0.08703016489744186, 0.06024491414427757, 0.13501441478729248, -0.008617252111434937, -0.38485971093177795, -0.07624133676290512, 0.05291355773806572, -0.07669814676046371, 0.12438400089740753, -0.15621356666088104, -0.10521290451288223, 0.2535055875778198, -0.10731009393930435, -0.0040817237459123135, 0.0066558909602463245, -0.05294480919837952, 0.11544105410575867, -0.18709491193294525, -0.07609889656305313, -0.13524609804153442, 0.2259172648191452, 0.2540636658668518, -0.12178725004196167, -0.06277117133140564, 0.1629912555217743, -0.04387538507580757, 0.1329568326473236, 0.1557694524526596, 0.12554575502872467, 0.009871226735413074, 0.09852169454097748, -0.07787914574146271, 0.06211487203836441, 0.05958058685064316, 0.0564635768532753, -0.36244282126426697, -0.087112195789814, -0.008342280983924866, -0.3805389106273651, -0.05840861797332764, 0.13118061423301697, 0.28274574875831604, -0.02293199859559536, -0.10023593157529831, -0.10559291392564774, 0.13648246228694916, 0.03481544926762581, 0.051615770906209946, 0.12077497690916061, -0.19688478112220764, -0.03950659558176994, -0.02346019074320793, -0.09129439294338226, 0.09707418084144592, 0.22449016571044922, -0.06372469663619995, -0.07186542451381683, -0.003897419199347496, 0.1547076255083084, 0.12244962155818939, -0.15271630883216858, -0.30145710706710815, -0.11087475717067719, -0.011731507256627083, -0.09938091039657593, 0.3345158100128174, -0.49201393127441406, -0.0808701366186142, 0.06498700380325317, -0.027205653488636017, -0.06192199885845184, 0.010222895070910454, 0.28826937079429626, -0.03191002830862999, -0.17338183522224426, 0.18845818936824799, -0.01399477943778038, 0.10992144793272018, -0.03946623206138611, 0.010924718342721462, 0.0954488143324852, -0.0021274476312100887, 0.2457304745912552, 0.2503296136856079, -0.44318464398384094, 0.02043410949409008, -0.09089798480272293, -0.02885577641427517, 0.06637385487556458, 0.08592304587364197, -0.12860184907913208, 0.3102247416973114, 0.12033411115407944, -0.07398214936256409, 0.1511811912059784, 0.25724363327026367, -0.11108072102069855, -0.039774928241968155, 0.09649312496185303, -0.09252341091632843, 0.240426704287529, -0.09693383425474167, -0.15795888006687164, -0.08874329179525375, 0.2529217004776001, 0.1251814216375351, -0.09550657868385315, 0.024661682546138763, 0.010110119357705116, -0.27834397554397583, -0.11455974727869034, -0.05051789805293083, -0.3324929177761078, -0.11321919411420822, -0.08808945119380951, -0.07071896642446518, 0.11464651674032211, -0.040911413729190826, 0.22493931651115417, -0.1264653354883194, -0.17551054060459137, 0.11673832684755325, -0.06791024655103683, 0.030294910073280334, 0.09788858890533447, -0.23733289539813995, -0.057774562388658524, 0.10859543085098267, -0.08230860531330109, -0.244719997048378, -0.13007472455501556, -0.03605998679995537, -0.0007846330408938229, -0.14314645528793335, -0.06847339123487473, -0.10702908784151077, -0.020339403301477432, -0.030380045995116234, 0.2607913911342621, -0.0879369005560875, 0.17428524792194366, -0.004653428215533495, -0.031414225697517395, 0.05431005358695984, -0.07420294731855392, 0.08223128318786621, -0.03848839923739433, 0.23931041359901428, 0.14365243911743164, 0.01762421801686287, -0.10638607293367386, -0.052033353596925735, 0.03308921307325363, 0.00867487769573927, 0.15941216051578522, -0.07896523177623749, -0.3185669779777527, -0.20841416716575623, -0.05307212099432945, 0.09620983898639679, -0.010138464160263538, -0.0017158730188384652, -0.18781913816928864, -0.03895212337374687, 0.06888606399297714, 0.08979541063308716, -0.260362833738327, -0.16210515797138214, 0.03835315629839897, 0.00911005400121212, -0.07539292424917221, 0.15502557158470154, 0.2484988272190094, -0.10449002683162689, 0.046304233372211456, 0.021972814574837685, 0.0967702865600586, 0.08561746031045914, -0.060065463185310364, -0.026130512356758118, 0.08626750111579895, -0.049872275441884995, -0.01315218210220337, 0.2073560655117035, 0.2651321291923523, -0.09386664628982544, -0.006134359631687403, 0.10438290983438492, 0.16261723637580872, -0.16008873283863068, -0.08594998717308044, -0.06089315935969353, 0.033752452582120895, -0.030790958553552628, -0.027484554797410965, 0.29264745116233826, 0.09999901801347733, 0.11398640275001526, 0.24689772725105286, -0.13989728689193726, -0.21613912284374237, -0.21382462978363037, 0.02679935283958912, -0.036951325833797455, 0.17452210187911987, 0.10671689361333847, -0.006494737695902586, -0.12921646237373352, -0.1867515742778778, 0.1229068860411644, 0.030230797827243805, -0.02104666829109192, -0.08306662738323212, -0.01697622798383236, 0.21231131255626678, 0.21181891858577728, 0.08955433964729309}, {-0.048678092658519745, -0.08808552473783493, -0.06850840896368027, -0.1727704554796219, -0.051951926201581955, -0.05139247700572014, 0.2533107399940491, -0.09121792018413544, 0.28895479440689087, 0.2406209260225296, 0.2093186229467392, 0.03620629012584686, 0.01146304327994585, -0.02839738316833973, -0.04437221586704254, -0.02454305998980999, -0.01606656424701214, 0.07731383293867111, -0.04085422679781914, 0.16963250935077667, 0.15607982873916626, -0.09181290864944458, -0.1442541629076004, -0.06631390005350113, 0.18083421885967255, 0.0611780546605587, -0.013787937350571156, -0.0360347181558609, 0.18882577121257782, -0.025563836097717285, 0.14979347586631775, -0.09940656274557114, 0.13141779601573944, 0.022157644852995872, 0.09668529778718948, -0.05546938627958298, -0.21464398503303528, -0.013351338915526867, 0.11755776405334473, -0.09299115836620331, 0.0510157085955143, 0.016748888418078423, -0.12983672320842743, -0.14373934268951416, 0.17202314734458923, 0.15224529802799225, 0.12054574489593506, 0.42260873317718506, 0.04395432397723198, -0.09807486832141876, -0.14934027194976807, -0.0674089565873146, -0.023873688653111458, -0.0030574037227779627, -0.0004713157541118562, 0.36536186933517456, 0.20032773911952972, -0.15431958436965942, -0.16262196004390717, 0.24183183908462524, 0.05296977236866951, -0.011480798944830894, -0.1893136352300644, -0.07338038831949234, -0.16877129673957825, 0.0341854989528656, -0.000557789346203208, -0.26895377039909363, -0.02710518054664135, 0.05618095397949219, 0.040807757526636124, 0.05355989560484886, -0.06802167743444443, -0.06798569113016129, 0.0019846472423523664, 0.08833702653646469, 0.19851602613925934, -0.1417243480682373, 0.07132978737354279, 0.19257289171218872, 0.2649477422237396, -0.08697333931922913, 0.05518362298607826, 0.03441538289189339, -0.08231531083583832, 0.08514467626810074, -0.07342246919870377, -0.13503660261631012, -0.15645425021648407, 0.09347696602344513, -0.16900913417339325, -0.2280929982662201, 0.07086224108934402, -0.02047073468565941, 0.006536606699228287, 0.17556166648864746, -0.023401595652103424, 0.14321616291999817, -0.12352263182401657, 0.25592172145843506, -0.030997328460216522, -0.19992510974407196, 0.20797650516033173, 0.154952734708786, -0.1490558683872223, 0.35996630787849426, 0.021711990237236023, -0.06982764601707458, 0.07162180542945862, 0.04202491044998169, 0.011117244139313698, -0.037619929760694504, -0.11765234172344208, -0.06734684854745865, 0.1290002316236496, 0.11736433953046799, -0.176180899143219, 0.018457164987921715, 0.047756750136613846, 0.03081601671874523, 0.04713428393006325, 0.08629696071147919, 0.03639725223183632, 0.016914110630750656, 0.09474184364080429, 0.0895739495754242, 0.1261935830116272, 0.351442813873291, 0.21060335636138916, 0.22582247853279114, -0.0830278992652893, 0.14222270250320435, 0.025276726111769676, 0.2639184594154358, 0.06540848314762115, 0.10104309767484665, -0.019109757617115974, -0.06160387024283409, -0.09708695113658905, -0.059955790638923645, 0.23242397606372833, -0.0760100856423378, 0.06101565808057785, -0.09621688723564148, -0.030493319034576416, -0.06953082233667374, 0.3011457026004791, 0.033438555896282196, -0.007611519657075405, -0.03017077036201954, -0.26595234870910645, 0.060223426669836044, -0.21611709892749786, -0.16476412117481232, 0.02995632030069828, -0.1940779834985733, 0.23970752954483032, -0.10797717422246933, 0.07171564549207687, -0.1980883926153183, 0.013339201919734478, 0.022161975502967834, -0.32606104016304016, -0.11920082569122314, -0.17544981837272644, 0.14625833928585052, 0.6335100531578064, 0.09485964477062225, -0.03517287224531174, 0.027988966554403305, -0.0828770324587822, -0.06224408745765686, -0.10025308281183243, 0.03504597768187523, -0.19358479976654053, -0.014897429384291172, -0.18484681844711304, 0.10992995649576187, -0.16715925931930542, 0.11899688094854355, -0.03889494016766548, 0.05695868283510208, -0.025080107152462006, 0.058009784668684006, 0.08227499574422836, 0.07671026885509491, -0.28748252987861633, -0.05421328917145729, 0.10195368528366089, -0.4570443630218506, 0.17921040952205658, -0.15267401933670044, -0.00012886217155028135, -0.1463526040315628, 0.0216034185141325, 0.1616775244474411, -0.08508303016424179, 0.036618422716856, -0.05762503668665886, -0.11187289655208588, -0.1198628768324852, -0.07191520184278488, 0.16972704231739044, -0.12439670413732529, 0.09920340031385422, -0.004229849670082331, -0.4685412049293518, 0.0649869367480278, -0.07315509021282196, -0.19871069490909576, -0.18861909210681915, -0.15742546319961548, -0.12110567092895508, -0.03814044967293739, 0.2383648008108139, -0.17366625368595123, 0.06726470589637756, 0.05451546609401703, 0.1983715444803238, -0.07250544428825378, -0.07054714113473892, -0.12416517734527588, 0.0786258727312088, -0.2797376811504364, -0.12310287356376648, -0.41529688239097595, 0.05280749872326851, 0.10713276267051697, -0.410392165184021, 0.1810537874698639, -0.09360776841640472, 0.1281035840511322, -0.19559641182422638, 0.1550201177597046, 0.18265952169895172, 0.10588137805461884, -0.15713247656822205, -0.03837168961763382, 0.08061663806438446, 0.04541894048452377, 0.01235552504658699, 0.15024642646312714, -0.07501896470785141, 0.4514428675174713, 0.19962531328201294, -0.051571134477853775, 0.014066115953028202, -0.0015113666886463761, 0.1874881088733673, -0.06424850225448608, 0.004524655174463987, 0.019347285851836205, -0.07606054842472076, -0.2645666301250458, -0.10516844689846039, -0.1493036150932312}, {0.053666938096284866, -0.12152734398841858, 0.17216640710830688, -0.08640143275260925, -0.07718589901924133, 0.0176889356225729, -0.26766225695610046, 0.029417414218187332, 0.013361250050365925, 0.19630500674247742, -0.09859776496887207, -0.1846858114004135, 0.18031948804855347, -0.08816248178482056, -0.04732375964522362, -0.040246814489364624, 0.004343171138316393, -0.1770285665988922, -0.029417742043733597, 0.01564733125269413, -0.17710964381694794, -0.10398977994918823, 0.03192388266324997, 0.0535467304289341, -0.12148337066173553, -0.148272305727005, 0.08248946815729141, -0.004515127744525671, 0.33858564496040344, -0.11888211220502853, 0.1538851261138916, 0.11352446675300598, -0.3923536539077759, 0.17432764172554016, 0.027552546933293343, -0.16837053000926971, -0.06982944160699844, -0.0406862273812294, 0.003832488553598523, -0.07743597775697708, 0.055236998945474625, 0.027627207338809967, -0.09408168494701385, 0.022983480244874954, -0.06199657917022705, 0.10857771337032318, 0.16678504645824432, 0.17163506150245667, 0.13401974737644196, 0.09550046920776367, 0.012337658554315567, -0.011831087060272694, 0.11861901730298996, -0.23108157515525818, -0.004980437457561493, 0.11129948496818542, -0.09687294811010361, 0.18187947571277618, 0.3936244547367096, -0.18839886784553528, 0.2298397272825241, 0.11513324826955795, 0.08683770895004272, -0.13232259452342987, -0.13336898386478424, -0.010019846260547638, -0.10951828956604004, 0.031085941940546036, -0.03320512920618057, 0.039474423974752426, 0.10591959208250046, -0.041696950793266296, 0.1102306917309761, -0.009346924722194672, -0.30533143877983093, -0.3242815434932709, 0.25711727142333984, -0.11834188550710678, -0.037897076457738876, 0.015740077942609787, -0.27864494919776917, -0.22946278750896454, -0.04071531817317009, 0.1124170571565628, -0.26920655369758606, 0.09129572659730911, 0.14920443296432495, -0.009115948341786861, -0.03376790136098862, -0.06246405094861984, 0.014257198199629784, -0.15989819169044495, 0.23112133145332336, 0.045646168291568756, 0.053300682455301285, 0.03545485436916351, -0.0012250036234036088, 0.184162899851799, 0.03256184980273247, -0.26224228739738464, -0.05492749437689781, 0.13750262558460236, -0.0006854425300844014, 0.08518244326114655, -0.13467343151569366, -0.15552030503749847, 0.07979240268468857, 0.11250466853380203, 0.1284327208995819, 0.12173085659742355, 0.07513594627380371, 0.051498282700777054, -0.05275189131498337, -0.10588930547237396, -0.12565843760967255, 0.08930087834596634, -0.02456052042543888, 0.10103320330381393, 0.06576425582170486, -0.2990269362926483, -0.007918951101601124, 0.22375965118408203, 0.0917765349149704, 0.04104772210121155, -0.002488857600837946, -0.21980386972427368, 0.004363151732832193, -0.1464204043149948, 0.08230321854352951, -0.21363100409507751, -0.08803223073482513, -0.12615618109703064, -0.04784303903579712, 0.17659133672714233, 0.08177578449249268, 0.08711623400449753, 0.291268914937973, -0.012224104255437851, -0.24383136630058289, 0.01331188715994358, 0.09627141803503036, -0.16679812967777252, 0.09410510212182999, 0.18702469766139984, 0.03592213615775108, -0.11490961164236069, 0.0009773962665349245, -0.2737794816493988, -0.07426436990499496, 0.16811002790927887, 0.0685739815235138, 0.025958845391869545, 0.004754921421408653, -0.08976217359304428, 0.07763689011335373, -0.08864398300647736, 0.1571023166179657, -0.08044188469648361, -0.16516804695129395, 0.02915358543395996, 0.07532359659671783, 0.00032106778235174716, -0.0360136479139328, 0.1311461627483368, -0.005461504217237234, 0.005878556985408068, 0.03317854553461075, 0.05686729773879051, 0.023524755612015724, -0.013544535264372826, 0.04709162190556526, -0.21193933486938477, -0.013321898877620697, -0.08334977924823761, -0.10040900856256485, 0.05933242291212082, -0.16661542654037476, -0.02772020921111107, -0.08914350718259811, -0.0964915007352829, -0.03010874055325985, -0.22694173455238342, 0.02698216587305069, -0.16767369210720062, -0.09029924869537354, -0.031717024743556976, 0.12525488436222076, -0.08264797180891037, -0.05252110958099365, 0.2339923083782196, -0.06091110035777092, 0.09720110148191452, -0.2540763020515442, 0.005558745004236698, -0.23152780532836914, -0.08926323801279068, -0.024476971477270126, 0.21714095771312714, 0.15771502256393433, -0.0050298855639994144, 0.14705099165439606, -0.020735442638397217, -0.06398949772119522, 0.21861594915390015, 0.013767478056252003, -0.03434402495622635, 0.32328706979751587, 0.02938946522772312, -0.05874516814947128, 0.11517956107854843, -0.019585976377129555, 0.1912423074245453, -0.01747596450150013, 0.08115026354789734, -0.05292314291000366, 0.0570475198328495, -0.027822138741612434, -0.21641723811626434, -0.050818879157304764, -0.24517370760440826, 0.09658871591091156, 0.01732817105948925, -0.017141250893473625, 0.2264777272939682, 0.3573901653289795, 0.1587514728307724, -0.2007141411304474, -0.022233426570892334, -0.02411234751343727, -0.11933516710996628, -0.1666782945394516, 0.05102980509400368, 0.05469077453017235, -0.07489161938428879, 0.06938241422176361, -0.1955166757106781, -0.1022486537694931, -0.046946555376052856, 0.03333757817745209, -0.05956266447901726, 0.1820170283317566, 0.16086794435977936, 0.21250002086162567, -0.04543336480855942, 0.04088981822133064, 0.2900164723396301, -0.027974624186754227, 0.0618194080889225, -0.07780701667070389, 0.17668673396110535, -0.0944971889257431, 0.15107521414756775, 0.1273537278175354, -0.05903954431414604, 0.10476464033126831, 0.36211198568344116}, {-0.009634237736463547, -0.04754488915205002, -0.030762530863285065, -0.015405998565256596, 0.031664151698350906, 0.05735853686928749, 0.15212368965148926, -0.11445733159780502, 0.04064422473311424, -0.00736486678943038, -0.07595367729663849, 0.10660553723573685, 0.06971734017133713, -0.017052240669727325, -0.0007150660967454314, 0.173753023147583, -0.024420106783509254, 0.02768649347126484, 0.12364105880260468, 0.002150207059457898, 0.06074235215783119, 0.016690736636519432, 0.05388250574469566, 0.027022091671824455, -0.03728053346276283, 0.09150417894124985, 0.0648340955376625, -0.16371478140354156, -0.06164124980568886, 0.000790649326518178, -0.5643276572227478, 0.1947590410709381, 0.48218023777008057, 0.07031802833080292, 0.22134950757026672, 0.09189654886722565, -0.005710266996175051, 0.21076692640781403, 0.15039174258708954, 0.06138110160827637, 0.031194141134619713, 0.07462498545646667, 0.04547044262290001, -0.00655593303963542, 0.017020907253026962, -0.11361353099346161, -0.10333318263292313, -0.039375580847263336, -0.07093758136034012, -0.029979540035128593, -0.01296725869178772, 0.10138847678899765, 0.09047626703977585, 0.049245867878198624, -0.23569118976593018, 0.011232972145080566, -0.09706058353185654, -0.23599661886692047, 0.03477764129638672, 0.04608149453997612, -0.06622391194105148, -0.06956062465906143, 0.22094807028770447, 0.17946116626262665, 0.08213355392217636, -0.24588017165660858, 0.021430734544992447, 0.14233055710792542, 0.05364268645644188, -0.03640620410442352, 0.00048181493184529245, -0.16173535585403442, -0.4877063035964966, -0.17479796707630157, 0.02328248880803585, 0.11106278747320175, 0.07510276883840561, -0.09397394210100174, 0.07028321176767349, -0.3019602298736572, -0.02934395708143711, 0.0583951361477375, -0.009517237544059753, -0.08352604508399963, -0.027582868933677673, -0.057004690170288086, -0.22967785596847534, -0.14281833171844482, 0.11945926398038864, -0.12407545745372772, -0.0781044065952301, -0.14682666957378387, -0.18289531767368317, 0.034628257155418396, -0.03698357939720154, 0.03651925176382065, 0.028782686218619347, 0.013719750568270683, 0.07605298608541489, 0.021806934848427773, 0.03767482936382294, 0.09920598566532135, -0.12602630257606506, 0.012577422894537449, 0.674376368522644, 0.14603959023952484, 0.07353980094194412, 0.01232458371669054, -0.00961493793874979, -0.09292947500944138, 0.04024898633360863, -0.005389759782701731, 0.021045725792646408, -0.06730866432189941, -0.076146200299263, -0.026746325194835663, 0.10235687345266342, -0.11723379790782928, 0.0835295245051384, 0.0056531233713030815, -0.012960714288055897, -0.014698518440127373, -0.03765878826379776, 0.03769509121775627, -0.028516735881567, 0.05718123912811279, -0.04016726836562157, 0.08088749647140503, 0.044137243181467056, -0.061994925141334534, 0.015520704910159111, -0.1448182761669159, 0.42604580521583557, -0.06789703667163849, 0.09738922864198685, 0.029479455202817917, 0.062196917831897736, -0.08236297219991684, -0.014406725764274597, -0.06680306792259216, -0.025374872609972954, -0.08897379040718079, 0.07033675909042358, 0.0626455694437027, 0.024416450411081314, -0.06597133725881577, 0.0266508050262928, 0.013771474361419678, -0.04840182885527611, -0.030945951119065285, 0.1321275532245636, -0.059898462146520615, 0.027001293376088142, 0.045674074441194534, -0.03638477623462677, 0.12600909173488617, 0.010207216255366802, 0.4067884683609009, 0.08682616800069809, -0.08989767730236053, 0.10412987321615219, 0.058652669191360474, 0.028232116252183914, 0.07951387017965317, -0.07219734787940979, -0.09349362552165985, 0.08075461536645889, 0.047817911952733994, 0.20310841500759125, -0.18501314520835876, 0.08449380844831467, -0.08092208206653595, -0.1469125747680664, 0.03694000095129013, -0.024732200428843498, 0.13589905202388763, 0.07181861996650696, 0.024226514622569084, -0.024072187021374702, 0.01323645282536745, 0.09585578739643097, 0.1082848310470581, 0.021557984873652458, -0.313577800989151, 0.03267041593790054, 0.00014788865519221872, 0.32976365089416504, -0.059055108577013016, 0.11058656871318817, -0.018616698682308197, -0.057882774621248245, 0.11505202949047089, -0.0851561650633812, -0.011286015622317791, -0.3274616301059723, -0.2548714578151703, 0.10142171382904053, 0.037694841623306274, 0.05928065627813339, 0.030856501311063766, -0.004812371451407671, -0.032008640468120575, -0.025795061141252518, -0.04175558686256409, 0.034125979989767075, -0.16844899952411652, -0.05844319984316826, -0.15323123335838318, 0.013080362230539322, -0.06091583892703056, 0.1065891683101654, 0.1769893914461136, 0.005815523210912943, -0.08850426226854324, -0.025199156254529953, -0.008706307038664818, -0.0190893542021513, 0.06972049176692963, -0.17688316106796265, 0.07442465424537659, -0.05425221472978592, 0.004076323006302118, 0.01660066470503807, 0.2544711232185364, 0.08047010004520416, 0.19311906397342682, -0.1342841386795044, -0.09324168413877487, -0.017849482595920563, -0.06800166517496109, -0.06011950969696045, 0.08380303531885147, 0.17811976373195648, 0.05940883979201317, -0.03413164243102074, 0.018620455637574196, -0.1663629710674286, 0.054820332676172256, -0.016336746513843536, 0.05333658680319786, 0.05249151960015297, 0.06491008400917053, -0.00037482089828699827, 0.0022066740784794092, 0.07255680859088898, -0.005639857612550259, 0.035325683653354645, -0.126156747341156, -0.08493556827306747, 0.12952540814876556, 0.1908157616853714, 0.10906220972537994, -0.08527319878339767, 0.18261569738388062, -0.06403164565563202, 0.18651030957698822}, {-0.0024999030865728855, 0.1737988144159317, 0.04526708275079727, 0.0716598629951477, -0.42793184518814087, -0.11775792390108109, -0.11906379461288452, -0.05953400954604149, -0.13073232769966125, -0.1376941204071045, 0.058074526488780975, 0.5524536967277527, -0.21717986464500427, 0.09392466396093369, 0.041394297033548355, -0.07384772598743439, 0.082558773458004, -0.025395991280674934, 0.12408338487148285, 0.1853114813566208, 0.07969899475574493, -0.08545888960361481, -0.05865124240517616, -0.42928656935691833, -0.47377583384513855, -0.16484640538692474, 0.2038775533437729, 0.2043760120868683, -0.1708737313747406, -0.25107455253601074, -0.07304184883832932, 0.12552569806575775, -0.17893598973751068, 0.02759048342704773, -0.1622176319360733, -0.19968780875205994, -0.12405818700790405, 0.13159997761249542, -0.04787111654877663, -0.0106968954205513, -0.05165828764438629, 0.194013312458992, -0.28106629848480225, -0.12610583007335663, 0.09397149085998535, -0.143358051776886, -0.12071837484836578, -0.1082916334271431, 0.14079903066158295, -0.3469659090042114, 0.11931642144918442, -0.03823179006576538, 0.15244874358177185, -0.20934683084487915, 0.1537150889635086, -0.15819886326789856, -0.24932192265987396, -0.0950053259730339, -0.19934897124767303, -0.22500817477703094, 0.06712234765291214, 0.022090809419751167, -0.1091231107711792, 0.286482572555542, -0.16693015396595, -0.4906594157218933, 0.08019490540027618, 0.16422832012176514, -0.11308411508798599, 0.15685975551605225, -0.05135539546608925, -0.0643056184053421, -0.06210585683584213, 0.06106096878647804, 0.13253524899482727, 0.1662876009941101, -0.048695847392082214, 0.14009171724319458, -0.05533213168382645, -0.1453438550233841, -0.0138398427516222, -0.11510150134563446, -0.008289006538689137, -0.011816519312560558, -0.08363541215658188, -0.07139427959918976, -0.09517531096935272, 0.0237528495490551, 0.02268415503203869, -0.23768137395381927, 0.08351422846317291, 0.09817240387201309, 0.046996504068374634, -0.124565988779068, 0.14279843866825104, 0.1373033970594406, -0.0811220109462738, -0.10250594466924667, 0.20242035388946533, 0.028587371110916138, -0.14983992278575897, -0.07291724532842636, 0.2209218144416809, 0.3622109293937683, -0.46285825967788696, 0.19615501165390015, -0.10845699161291122, -0.089726023375988, -0.2354097217321396, 0.4635964632034302, 0.03293384984135628, -0.17980706691741943, 0.06463056802749634, 0.1959238201379776, 0.2750363051891327, 0.041513849049806595, 0.03727290406823158, -0.08729866147041321, 0.12697061896324158, 0.2558540105819702, -0.13185475766658783, -0.05492662638425827, -0.23518593609333038, 0.28095123171806335, 0.12734705209732056, 0.26143696904182434, 0.2262546271085739, -0.11678922921419144, -0.06940384209156036, 0.2257693111896515, 0.16401894390583038, 0.18918384611606598, 0.177890345454216, -0.07810469716787338, -0.09778720885515213, -0.1558883786201477, -0.16427716612815857, -0.056708354502916336, -0.3788747191429138, -0.011148913763463497, -0.14355899393558502, 0.19384415447711945, -0.11046627163887024, -0.10612839460372925, -0.17932279407978058, 0.11324898153543472, 0.042239878326654434, 0.24127881228923798, -0.18064023554325104, 0.05314826965332031, -0.11079119145870209, -0.020417630672454834, 0.026739632710814476, 0.11515795439481735, -0.1153121069073677, -0.04210889711976051, -0.31151914596557617, -0.21310292184352875, -0.07942727208137512, -0.08580644428730011, -0.1713263988494873, 0.17222489416599274, 0.1032714769244194, 0.039075396955013275, -0.009541134350001812, 0.04402061551809311, 0.01970897987484932, -0.1577737033367157, 0.15531298518180847, 0.23976324498653412, 0.045007169246673584, 0.3019292652606964, -0.07282319664955139, -0.09759511798620224, 0.1241447925567627, 0.1341945379972458, -0.031049320474267006, -0.09821661561727524, -0.0803871676325798, -0.2515207529067993, 0.10882703214883804, 0.22897198796272278, 0.06694868952035904, -0.13688859343528748, 0.2948477268218994, -0.07419846206903458, 0.3551042973995209, 0.03518260270357132, -0.01397521048784256, 0.05024068057537079, -0.026783673092722893, -0.17192964255809784, 0.11660600453615189, 0.009492479264736176, -0.4131215810775757, -0.03285222128033638, -0.0797121450304985, -0.004604651127010584, -0.0626070499420166, 0.3169747591018677, -0.06033891439437866, -0.1982177197933197, -0.06325329095125198, 0.0009530438110232353, -0.15114857256412506, 0.08111992478370667, -0.10683826357126236, -0.1227794662117958, -0.08172859251499176, -0.003069269238039851, 0.14777468144893646, 0.3189692497253418, -0.1861145943403244, 0.016158191487193108, 0.30354201793670654, 0.13693739473819733, -0.12697938084602356, 0.0014558382099494338, -0.24066950380802155, 0.23716911673545837, -0.13582687079906464, 0.2534899413585663, -0.0688105896115303, 0.21849429607391357, -0.029358377680182457, 0.10083088278770447, -0.19690771400928497, 0.11707998812198639, 0.0963437631726265, -0.11075204610824585, 0.11241678893566132, -0.20241011679172516, 0.2750570774078369, -0.14701758325099945, -0.2150869220495224, 0.04037167504429817, 0.11229122430086136, -0.23729319870471954, -0.10595878958702087, 0.1498914510011673, 0.07659132033586502, 0.011461218819022179, -0.1357216089963913, -0.10630538314580917, -0.08731920272111893, -0.09517337381839752, -0.13418535888195038, 0.05984891206026077, -0.14628678560256958, 0.17444680631160736, 0.17085465788841248, -0.13246628642082214, 0.41838353872299194, -0.11399859189987183, -0.18696974217891693, -0.09322662651538849}, {-0.14060385525226593, -0.025349261239171028, 0.07265276461839676, -0.08649376779794693, 0.07669738680124283, 0.07495440542697906, 0.014065325260162354, 0.013408232480287552, -0.11042940616607666, -0.16944792866706848, 0.012852194719016552, 0.045635636895895004, 0.14912734925746918, -0.09550352394580841, -0.025150606408715248, 0.35739564895629883, 0.11032573878765106, -0.08201109617948532, -0.09381068497896194, -0.036234233528375626, 0.009295322000980377, -0.011934962123632431, 0.009192144498229027, 0.26007989048957825, 0.31037595868110657, -0.36522942781448364, 0.04202435538172722, -0.1609889715909958, -0.05012287572026253, 0.07039032131433487, 0.010516459122300148, 0.05356256663799286, 0.020537039265036583, 0.19843947887420654, -0.02096163108944893, 0.21287201344966888, 0.18449383974075317, 0.10711367428302765, -0.08342400938272476, 0.08352797478437424, 0.12584224343299866, 0.04745899885892868, -0.10492605715990067, -0.0699266642332077, 0.09225068986415863, 0.0008395854383707047, -0.18198178708553314, -0.0008365890826098621, 0.02186739444732666, -0.053682874888181686, 0.04713346064090729, 0.12762677669525146, 0.07248899340629578, 0.08462867140769958, -0.049786388874053955, 0.06020614504814148, 0.09302389621734619, -0.26244935393333435, 0.19347277283668518, 0.11499155312776566, 0.11431300640106201, 0.19463537633419037, 0.1408807337284088, -0.014126039110124111, -0.11545108258724213, 0.02789422869682312, 0.07434754818677902, 0.20954111218452454, 0.16337740421295166, -0.2167588174343109, -0.023383604362607002, -0.057576268911361694, -0.15455487370491028, -0.0433838851749897, 0.31743642687797546, 0.08638965338468552, -0.10697711259126663, 0.11836934834718704, -0.13457465171813965, -0.06563650816679001, -0.10445977747440338, -0.09798851609230042, -0.029916830360889435, -0.20402874052524567, 0.014225447550415993, -0.020939074456691742, -0.23670406639575958, -0.05988679826259613, 0.15492244064807892, -0.2263306826353073, 0.0027871355414390564, -0.09153994917869568, 0.11830086261034012, 0.11435092240571976, -0.13958272337913513, 0.19544363021850586, -0.02129686437547207, -0.11219664663076401, 0.015298061072826385, 0.02796274796128273, 0.18407821655273438, 0.07086873799562454, 0.04549127817153931, 0.1305081695318222, 0.1154012531042099, 0.4274037778377533, -0.03453316539525986, -0.18985551595687866, 0.11595466732978821, -0.07956527173519135, 0.075471892952919, 0.07658793032169342, 0.09506727010011673, 0.018699880689382553, -0.007480067666620016, -0.030393559485673904, -0.14030028879642487, -0.061106204986572266, 0.08592405915260315, -0.03900730609893799, -0.09418123215436935, 0.18663395941257477, -0.030067045241594315, 0.4146929681301117, -0.023534582927823067, 0.3254566788673401, -0.04156670346856117, -0.0053850640542805195, -0.11890918761491776, 0.03047632798552513, -0.03536283224821091, -0.25345563888549805, 0.0682828351855278, -0.1671452671289444, 0.12983396649360657, 0.039712946861982346, 0.010183990933001041, 0.23494534194469452, -0.1001911610364914, 0.16542744636535645, -0.22276875376701355, -0.09862346947193146, 0.08216840773820877, 0.023315487429499626, 0.3050693869590759, 0.017610419541597366, -0.15088781714439392, -0.13263529539108276, 0.15765315294265747, 0.06967459619045258, 0.29739221930503845, -0.11933885514736176, 0.1315564662218094, 0.03084120899438858, 0.030350016430020332, 0.1468556672334671, 0.019912878051400185, 0.003669502679258585, -0.0820683166384697, 0.15622414648532867, -0.023368194699287415, -0.10342545807361603, -0.08978471904993057, 0.06240113452076912, 0.0562177337706089, -0.06961125135421753, 0.05425971746444702, -0.02211635932326317, 0.07682333886623383, -0.004353555850684643, 0.029665794223546982, -0.14598995447158813, 0.12829376757144928, -0.02980160154402256, 0.2434425950050354, -0.005543161649256945, -0.00026859407080337405, 0.06471934914588928, -0.07639597356319427, 0.13180595636367798, 0.16273866593837738, -0.06731288880109787, -0.028149355202913284, -0.25337356328964233, -0.004564115311950445, 0.061984576284885406, 0.08780131489038467, -0.2635922133922577, 0.0988774374127388, 0.1181979849934578, -0.050777699798345566, -0.022930406033992767, 0.039191894233226776, 0.20958316326141357, -0.03791886195540428, -0.0859619528055191, 0.018224317580461502, -0.04246167093515396, 0.0720548927783966, -0.06895872950553894, -0.01951371505856514, -0.0482342354953289, -0.04237740486860275, -0.13974842429161072, 0.08841179311275482, 0.09262655675411224, 0.1342945694923401, -0.024767354130744934, -0.1444445252418518, -0.03445246443152428, -0.08298998326063156, -0.0998677983880043, -0.14160534739494324, 0.08545386046171188, -0.13749182224273682, -0.03423766791820526, -0.03535262495279312, 0.020969346165657043, 0.05243656411767006, 0.0822916105389595, 0.23470696806907654, -0.1650388091802597, 0.03098588064312935, 0.2262991964817047, 0.3584231436252594, 0.13594233989715576, -0.3329181969165802, -0.04107120633125305, 0.18356014788150787, -0.14151689410209656, -0.09000441431999207, -0.022380180656909943, -0.009765103459358215, -0.01068071834743023, 0.06814298033714294, -0.031240273267030716, 0.07665199041366577, -0.04345894232392311, 0.19557945430278778, 0.08480122685432434, 0.047518521547317505, -0.05427444353699684, 0.05350588634610176, -0.22609345614910126, -0.1918741762638092, -0.08897560089826584, 0.03888194262981415, -0.15617585182189941, -0.0067055243998765945, -0.05104435607790947, -0.020851880311965942, 0.08422216027975082, -0.13351112604141235, 0.2253618836402893, 0.045303571969270706, 0.08820749074220657}, {0.049473509192466736, 0.017571933567523956, 0.15058502554893494, 0.10880982130765915, -0.01868121698498726, -0.048392150551080704, 0.060500629246234894, -0.17524230480194092, 0.1803593933582306, -0.13137643039226532, 0.3347964882850647, 0.27585339546203613, -0.09958931058645248, 0.12459123879671097, -0.18225166201591492, -0.19903792440891266, 0.007852120324969292, -0.1471932828426361, 0.05535214766860008, 0.09028413146734238, -0.11324489116668701, 0.0668363869190216, -0.009500343352556229, -0.14668679237365723, -0.1457507610321045, -0.12707914412021637, 0.20523066818714142, -0.005497401114553213, -0.18264450132846832, 0.014760278165340424, 0.01318203005939722, 0.1334330290555954, -0.01085728406906128, -0.048095703125, 0.2757379710674286, 0.01211730670183897, 0.14887236058712006, 0.25744569301605225, -0.06181638687849045, 0.11894133687019348, -0.10868237912654877, 0.05336569622159004, -0.10191229730844498, -0.06318261474370956, 0.14438538253307343, -0.003639745293185115, -0.08943329006433487, 0.09612621366977692, 0.22785000503063202, -0.18031011521816254, 0.14310157299041748, 0.01674807444214821, -0.184182271361351, 0.134541317820549, 0.03632659837603569, -0.041412439197301865, -0.3635610044002533, 0.0535658597946167, -0.09117519110441208, -0.04947170987725258, 0.1464240401983261, -0.41643181443214417, 0.1328415423631668, -0.12683086097240448, -0.054898109287023544, -0.06006426364183426, 0.1405622363090515, -0.11365648359060287, -0.2151065468788147, 0.08439650386571884, -0.18951407074928284, 0.20302027463912964, -0.3101600110530853, -0.008896258659660816, -0.07332608103752136, 0.1731330156326294, -0.006882976740598679, -0.031102877110242844, 0.06071752682328224, -0.23456288874149323, -0.07770891487598419, -0.015916643664240837, 0.056041669100522995, -0.14292743802070618, -0.08478967100381851, -0.10105068981647491, 0.03909694775938988, 0.18004874885082245, -0.05953278765082359, -0.05930304899811745, 0.03273363038897514, 0.05126819387078285, -0.1921812742948532, -0.17430177330970764, -0.12050171196460724, -0.020081138238310814, -0.10466185212135315, -0.13550612330436707, -0.13814543187618256, 0.15570446848869324, -0.11878598481416702, 0.25800421833992004, 0.18361760675907135, -0.023079659789800644, 0.0008782277582213283, 1.0155023336410522, 0.15954050421714783, -0.044066913425922394, -0.14930446445941925, -0.26579537987709045, -0.06466449797153473, -0.09071853756904602, -0.07765702903270721, 0.03853975608944893, -0.0046854144893586636, 0.004959849640727043, 0.09645984321832657, -0.2022865116596222, -0.03237808123230934, 0.014147224836051464, -0.00793664064258337, 0.011244301684200764, -0.13369953632354736, -0.05731569603085518, 0.17748843133449554, 0.2562280297279358, 0.000291655829641968, 0.0982762947678566, -0.27196958661079407, 0.2551012635231018, -0.015492809936404228, 0.058232538402080536, 0.23573854565620422, 0.04106982424855232, -0.0169108584523201, 0.09204956889152527, -0.016495231539011, -0.02562803402543068, 0.15345346927642822, -0.22560188174247742, 0.037404924631118774, -0.1740282028913498, -0.24126215279102325, -0.05373912677168846, -0.10207340121269226, 0.028776876628398895, 0.11130690574645996, -0.012121892534196377, 0.025320518761873245, 0.27509400248527527, -0.13653384149074554, 0.25926777720451355, 0.03437738120555878, -0.13922765851020813, -0.13454562425613403, 0.03706317022442818, 0.13952836394309998, 0.17510874569416046, -0.14885886013507843, -0.04172787442803383, 0.005301147233694792, -0.033172961324453354, 0.04012942686676979, 0.1809859275817871, -0.06468075513839722, -0.010055269114673138, 0.12366143614053726, 0.08456190675497055, 0.027598142623901367, 0.16603638231754303, -0.12222025543451309, 0.11526669561862946, -0.05933240056037903, -0.04432936757802963, 0.06038128212094307, 0.0267132967710495, 0.1270424723625183, 0.016200106590986252, -0.007721730973571539, -0.07621295005083084, 0.06645189225673676, 0.15907371044158936, 0.0809694305062294, -0.022203028202056885, 0.09856308996677399, 0.10361921787261963, 0.09557723253965378, -0.09325104206800461, -0.07676171511411667, 0.10455027967691422, 0.2373971939086914, 0.05079545080661774, -0.07169389724731445, -0.014907336793839931, 0.03844647482037544, 0.033959053456783295, 0.013740386813879013, -0.2773628830909729, 0.11022539436817169, -0.1582685261964798, -0.042001862078905106, 0.01068522036075592, -0.010618711821734905, 0.03420484438538551, -0.14140863716602325, 0.07981511205434799, -0.17899823188781738, 0.030806951224803925, -0.07037569582462311, 0.08048020303249359, 0.012253254652023315, -0.03011203184723854, -0.1732785403728485, 0.28649112582206726, 0.010528596118092537, 0.0350738987326622, 0.1098589152097702, 0.046510789543390274, -0.10139261931180954, 0.1404869258403778, -0.022213879972696304, 0.01690821163356304, -0.14800313115119934, -0.16196797788143158, -0.058998096734285355, 0.12161989510059357, 0.0680835172533989, -0.13091230392456055, -0.014643541537225246, -0.13639238476753235, 0.03540141507983208, 0.024043409153819084, 0.09289821982383728, 0.057370007038116455, 0.0032579165417701006, 0.06703991442918777, -0.009075826965272427, 0.04518697038292885, 0.020748421549797058, 0.24193638563156128, 0.05776327848434448, -0.12890884280204773, -0.13854646682739258, -0.036145519465208054, 0.06501929461956024, -0.18865005671977997, 0.02873663231730461, 0.013617658987641335, -0.23261481523513794, 0.03253304958343506, 0.10376730561256409, -0.23431362211704254, -0.12607789039611816, 0.015367638319730759, 0.15949276089668274, -0.1421550065279007}, {-0.18019360303878784, 0.2311915159225464, -0.08226895332336426, -0.08645473420619965, 0.01754898950457573, -0.12622496485710144, -0.04293039068579674, -0.01434231549501419, -0.34888017177581787, -0.26270365715026855, 0.0905022919178009, -0.0267767533659935, -0.13966870307922363, 0.07947801798582077, 0.012448660098016262, -0.03208834305405617, -0.18401043117046356, 0.0205181073397398, 0.36776670813560486, 0.2961757183074951, 0.18737415969371796, 0.015841132029891014, 0.14127343893051147, -0.045337602496147156, -0.0714547261595726, 0.035352159291505814, 0.3820827901363373, -0.09268252551555634, -0.07246299088001251, -0.0002543597074691206, 0.20816193521022797, 0.2299337536096573, -0.5849059224128723, 0.05108580365777016, -0.036141738295555115, -0.10208781063556671, 0.02397472970187664, -0.05889841169118881, -0.1325480341911316, 0.011182411573827267, 0.1830441802740097, -0.24370266497135162, -0.08343051373958588, 0.05232364684343338, 0.14065688848495483, -0.0004153724294155836, -0.08670803159475327, 0.04098184034228325, 0.029488321393728256, 0.09099359810352325, -0.2654779851436615, 0.30133479833602905, 0.11473387479782104, -0.1943429708480835, 0.19909748435020447, 0.0055382451973855495, -7.78826215537265e-05, -0.012054028920829296, 0.008038682863116264, -0.03175140172243118, -0.11604466289281845, -0.01938111148774624, 0.31997543573379517, 0.0016196167562156916, 0.1911987066268921, -0.40429264307022095, -0.056769877672195435, -0.042820531874895096, 0.05029522627592087, -0.027488021180033684, -0.020133953541517258, -0.03295876830816269, -0.0856829434633255, -0.15289856493473053, -0.11163253337144852, 0.12996989488601685, -0.1505647450685501, -0.05527544021606445, -0.005490822251886129, -0.00735889095813036, -0.19413109123706818, 0.07903237640857697, 0.0640520453453064, -0.015482156537473202, -0.015039073303341866, -0.28574830293655396, -0.1510939598083496, -0.00013732034130953252, 0.24913939833641052, 0.29800885915756226, 0.13878385722637177, -0.06801622360944748, 0.028183713555336, -0.10952671617269516, 0.06698990613222122, 0.06377175450325012, 0.023818790912628174, 0.14703407883644104, 0.07468408346176147, -0.011440319009125233, -0.0016271573258563876, 0.12449658662080765, 0.07657404243946075, 0.07906154543161392, -0.5492022037506104, -0.012084507383406162, -0.02289462462067604, -0.06591404229402542, 0.23872840404510498, -0.16030949354171753, -0.0011058622039854527, -0.19830019772052765, 0.025966035202145576, 0.031691521406173706, 0.07336266338825226, -0.09380075335502625, 9.659412899054587e-05, -0.13300199806690216, -0.02839767374098301, 0.03907220438122749, 0.022931328043341637, -0.2690151631832123, -0.07192175835371017, 0.13762004673480988, -0.06554572284221649, 0.28099769353866577, -0.0017224873881787062, -0.07666832953691483, -0.17088396847248077, 0.24375376105308533, 0.21524271368980408, 0.03246554732322693, 0.14797018468379974, -0.021146254613995552, -0.23845386505126953, -0.0025850229430943727, 0.0048792725428938866, -0.27798816561698914, 0.04379190132021904, -0.045947808772325516, -0.21389411389827728, 0.05766817554831505, -0.1403595358133316, -0.01092370506376028, -0.29553329944610596, -0.02236364781856537, -0.07462552189826965, -0.11400247365236282, 0.09887854009866714, 0.010274707339704037, -0.0003142059431411326, 0.22140803933143616, 0.17596741020679474, -0.1278054565191269, 0.06581200659275055, 0.06208757311105728, 0.05924764648079872, -0.14741159975528717, 0.24994465708732605, -0.1266603320837021, 0.1646626591682434, -0.019974056631326675, -0.021501323208212852, -0.010823973454535007, 0.06044202297925949, 0.022056670859456062, 0.05103863403201103, 0.12648378312587738, 0.22835446894168854, 0.270681768655777, -0.0734611228108406, -0.05208994448184967, 0.0026644873432815075, -0.0007053947192616761, -0.05124705657362938, -0.12232284247875214, -0.02757725492119789, -0.09734495729207993, -0.045942287892103195, -0.09916044771671295, -0.016943994909524918, -0.0293524581938982, -0.19228383898735046, -0.2637020945549011, 0.36865201592445374, -0.12593235075473785, 0.14877140522003174, -0.07062675803899765, -0.12152436375617981, 0.06535130739212036, 0.1684401035308838, 0.07644573599100113, -0.2323281615972519, -0.2393488734960556, -0.28085047006607056, -0.06526827067136765, 0.15025641024112701, 0.1553906798362732, -0.017991667613387108, -0.11678863316774368, 0.02997107058763504, 0.15235991775989532, -0.12845535576343536, -0.04153825342655182, -0.0930321216583252, 0.017618827521800995, -0.2300596386194229, -0.044204823672771454, -0.1068989709019661, 0.02081487886607647, 0.11975101381540298, 0.16290314495563507, 0.05268622934818268, 0.045916568487882614, 0.04235770180821419, -0.03017331101000309, -0.15433518588542938, -0.008908310905098915, -0.11503197997808456, 0.0584753192961216, 0.19128426909446716, 0.010897351428866386, -0.21232956647872925, 0.23507556319236755, 0.04789094626903534, 0.19627879559993744, -0.12545090913772583, -0.12311660498380661, 0.13105519115924835, -0.009632560424506664, 0.05396434664726257, -0.06246976926922798, -0.09904634207487106, -0.0635826587677002, -0.06174282357096672, 0.1533796489238739, -0.05056167021393776, -0.039496343582868576, 0.1626400500535965, -0.04829208552837372, 0.1407756358385086, 0.004505981225520372, 0.1556665003299713, -0.040085822343826294, 0.151852086186409, 0.16938328742980957, 0.06671066582202911, -0.017064746469259262, 0.1076807826757431, -0.21806560456752777, -0.08214306086301804, 0.07063702493906021, -0.04577759653329849, 0.039399631321430206, -0.140709787607193, 0.04465090483427048}, {0.043649349361658096, 0.07733968645334244, 0.031358908861875534, 0.020621947944164276, -0.20139305293560028, -0.0035812112037092447, -0.21155887842178345, -0.10616505891084671, 0.042144328355789185, -0.07467830926179886, 0.001243899459950626, -0.0913955569267273, 0.024288350716233253, -0.045645952224731445, -0.050049956887960434, 0.05280707776546478, 0.11403676867485046, -0.021875305101275444, -0.004210674203932285, -0.02253434807062149, 0.12585286796092987, 0.011946252547204494, 0.0039688111282885075, -0.03928494453430176, -0.020144451409578323, 0.056715238839387894, 0.18538634479045868, -0.023647215217351913, -0.14623357355594635, 0.03184476122260094, -0.09698209166526794, 0.08362550288438797, -0.07513231784105301, 0.17577756941318512, 0.1447439193725586, -0.03379525989294052, -0.04184327647089958, -0.009745382703840733, -0.006760149262845516, -0.09896152466535568, 0.09383455663919449, -0.09033972024917603, -0.014617612585425377, 0.036751072853803635, -0.005018940661102533, -0.22550496459007263, 0.17570427060127258, -0.1534697562456131, -0.10072234272956848, 0.20395082235336304, 0.02619059756398201, 0.2817544937133789, 0.18157339096069336, 0.0721750408411026, -0.15096424520015717, 0.11882051825523376, 0.07767617702484131, 0.03082728572189808, -0.06500833481550217, 0.08042332530021667, -0.1395123451948166, -0.1146666407585144, -0.01726909726858139, 0.09895770251750946, 0.04301607608795166, -0.2170722931623459, -0.0659748986363411, -0.060387033969163895, 0.09988555312156677, -0.007051984779536724, -0.022332875058054924, -0.028521332889795303, -0.09210557490587234, -0.0220768041908741, 0.028849950060248375, 0.16082528233528137, -0.10799869149923325, 0.030341457575559616, 0.10932162404060364, -0.027491582557559013, 0.051051296293735504, -0.14095401763916016, -0.04077308624982834, -0.0017905314452946186, -0.02364143542945385, 0.08203394711017609, -0.1249222606420517, -0.01586546003818512, -0.028405630961060524, -0.0645143985748291, -0.08702738583087921, -0.24175474047660828, 0.01407217513769865, 0.045833997428417206, 0.0459890253841877, 0.057178888469934464, 0.15560640394687653, -0.1679074913263321, 0.10618695616722107, 0.10044834762811661, 0.05755699425935745, 0.04769950732588768, -0.07859636843204498, 0.1777803897857666, -0.34311598539352417, 0.1728777438402176, 0.18475502729415894, -0.1942654699087143, 0.049586985260248184, 0.040674589574337006, -0.2776758074760437, -0.06038621813058853, -0.15555858612060547, 0.031828995794057846, 0.003895778441801667, -0.056792762130498886, 0.15392334759235382, -0.0566924512386322, 0.21445435285568237, 0.1484556645154953, 0.025554005056619644, -0.045175667852163315, 0.2568790316581726, 0.4632335901260376, 0.26263442635536194, -0.08152145892381668, 0.025836512446403503, 0.28389623761177063, 0.0029284870252013206, 0.16001693904399872, 0.23113934695720673, -0.13350573182106018, 0.1465536653995514, 0.03023943118751049, 0.08241921663284302, 0.19609172642230988, -0.029185298830270767, -0.04077614098787308, -0.001465607900172472, 0.1965499222278595, 0.009411207400262356, 0.17822331190109253, 0.24359679222106934, -0.1310378909111023, 0.23177820444107056, -0.16263443231582642, 0.1704276204109192, -0.010674151591956615, -0.0546124242246151, -0.056328628212213516, 0.02370668575167656, 0.06599682569503784, 0.08458751440048218, -0.1393522173166275, 0.15165232121944427, 0.16379320621490479, 0.08588454127311707, 0.10220158100128174, -0.02809746190905571, -0.02804817259311676, 0.09540180116891861, -0.11452896147966385, -0.14136332273483276, -0.03804919868707657, -0.06299259513616562, -0.09451717138290405, -0.02602333389222622, 0.03303999826312065, -0.13217498362064362, 0.1992071121931076, -0.10847686976194382, 0.170903280377388, -0.20315907895565033, -0.025828052312135696, -0.20345836877822876, 0.15694957971572876, 0.06235514581203461, -0.013680712319910526, -0.07778747379779816, -0.10829883813858032, 0.005114283412694931, -0.14397092163562775, 0.048676807433366776, -0.09058932214975357, 0.152869313955307, -0.33010274171829224, 0.04625144228339195, 0.11872448027133942, -0.06342519819736481, 0.1926501840353012, -0.051231641322374344, -0.10188493132591248, -0.09969925135374069, -0.04451471194624901, 0.07968492805957794, -0.1748359054327011, 0.05318291857838631, -0.024685624986886978, -0.006754131987690926, -0.07640987634658813, -0.04232366383075714, 0.08633417636156082, -0.07354095578193665, 0.025424787774682045, -0.09441716969013214, -0.2868722081184387, -0.020207032561302185, -0.05023680627346039, 0.06321819871664047, 0.09140969812870026, 0.01739254593849182, 0.05091342702507973, -0.026854144409298897, 0.10626542568206787, -0.02626907080411911, 0.056675832718610764, 0.09025730937719345, -0.17849396169185638, 0.0019499235786497593, -0.11471586674451828, 0.025117676705121994, 0.26040351390838623, -0.0961843729019165, 0.0381498858332634, 0.1367979794740677, 0.044427044689655304, -0.16997110843658447, -0.09272798150777817, 0.11930479854345322, 0.09454154223203659, 0.0680544450879097, -0.0475468672811985, 0.13027100265026093, -0.17724871635437012, -0.0441078282892704, 0.040724027901887894, 0.13338132202625275, 0.10646187514066696, 0.16457858681678772, 0.1419174075126648, 0.0026459447108209133, 0.07366491109132767, 0.08372106403112411, 0.014716577716171741, -0.028076108545064926, -0.07188262045383453, -0.191573828458786, 0.007271911017596722, 0.14046069979667664, 0.025116808712482452, 0.2515031397342682, -0.13005132973194122, -0.14851701259613037, -0.14934729039669037, -0.10735615342855453, -0.00486366730183363}, {0.0981731042265892, 0.18819023668766022, -0.09270507842302322, 0.033175285905599594, 0.19950437545776367, 0.19919933378696442, -0.12912653386592865, 0.10243956744670868, -0.1894107609987259, -0.48554304242134094, 0.06644999980926514, 0.03136032819747925, -0.01920980028808117, 0.06409455090761185, 0.20006407797336578, 0.06932606548070908, -0.11745963990688324, 0.2571156919002533, -0.06498830020427704, 0.11539619415998459, -0.3486327826976776, -0.15363898873329163, -0.07668512314558029, -0.20180097222328186, -0.10588139295578003, -0.12037024646997452, -0.03029748983681202, 0.1647237241268158, -0.08514472842216492, -0.08628679066896439, 0.088384248316288, 0.11070262640714645, -0.020975708961486816, -0.031138431280851364, -0.31231045722961426, 0.008057516068220139, -0.024433359503746033, -0.1590496003627777, -0.06007252261042595, -0.04452124983072281, 0.026365982368588448, -0.0028530326671898365, 0.2314988523721695, -0.1592940092086792, -0.3357228636741638, 0.030227413401007652, -0.0288393571972847, -0.22547177970409393, 0.0016207726439461112, -0.156843900680542, 0.24323631823062897, -0.012231498956680298, 0.03749261423945427, -0.13074931502342224, -0.05081569775938988, -0.05249979719519615, 0.052957553416490555, 0.15239444375038147, -0.04965794458985329, -0.07171428948640823, -0.3102397918701172, 0.04171724617481232, 0.14535431563854218, 0.055277157574892044, 0.049070268869400024, 0.04899315908551216, 0.07980256527662277, -0.026730747893452644, 0.1308017522096634, -0.08515287935733795, -0.1184161826968193, -0.0011088716564700007, 0.0808161199092865, -0.08657428622245789, 0.0053656394593417645, 0.028136439621448517, -0.0689484030008316, -0.039148978888988495, 0.09850551187992096, -0.3413410186767578, 0.08682824671268463, 0.10857778787612915, 0.10454460978507996, 0.11652281135320663, 0.03779211640357971, 0.01516001671552658, 0.14400263130664825, 0.05486840009689331, 0.08390187472105026, -0.08041109144687653, 0.09051157534122467, 0.09162377566099167, -0.001572666922584176, -0.01868310570716858, 0.016832226887345314, -0.19680139422416687, -0.13588806986808777, 0.012911679223179817, 0.13074758648872375, -0.11889870464801788, 0.3354875445365906, -0.25928249955177307, -0.027030805125832558, -0.06333015114068985, -0.17372171580791473, 0.24498054385185242, -0.0920671671628952, -0.12176411598920822, -0.05452859401702881, 0.05171578750014305, 0.0008076641825027764, 0.012758995406329632, 0.009836814366281033, 0.27054688334465027, 0.054718997329473495, 0.0028026143554598093, 0.18601344525814056, 0.051505837589502335, -0.17909768223762512, 0.09072525054216385, -0.02674161270260811, 0.09548404812812805, -0.20595809817314148, -0.17610087990760803, -0.001745685120113194, 0.06095096096396446, 0.1379258781671524, -0.25399893522262573, -0.09622395783662796, 0.10034526139497757, -0.028255490586161613, -0.029401158913969994, -0.12806472182273865, -0.057981330901384354, -0.012523768469691277, -0.06406396627426147, -0.016451621428132057, 0.08690977096557617, 0.057442184537649155, -0.3070722818374634, -0.0297680776566267, -0.09848468005657196, 0.10258103162050247, -0.0009862930746749043, -0.08764330297708511, 0.321532279253006, -0.1735759675502777, 0.07238876074552536, -0.09390715509653091, -0.06818678975105286, -0.004849449265748262, 0.08213412761688232, 0.17934879660606384, 0.1683834046125412, 0.09960981458425522, 0.03768249228596687, -0.02690460905432701, 0.08783001452684402, 0.09520136564970016, 0.11194480955600739, -0.2854420840740204, 0.09735983610153198, -0.021401934325695038, -0.13144618272781372, 0.15569144487380981, -0.1991642266511917, -0.41360148787498474, -0.08473111689090729, 0.057941749691963196, 0.062317706644535065, 0.2978993058204651, 0.05916592851281166, 0.21930523216724396, 0.153915137052536, 0.005916007328778505, -0.05401201546192169, 0.056698329746723175, -0.08844517916440964, 0.035055097192525864, -0.21693116426467896, 0.1287570744752884, -0.1419752985239029, -0.05383969098329544, 0.18544547259807587, -0.0504121333360672, 0.05485861375927925, -0.12124059349298477, 0.11226876080036163, 0.029289044439792633, -0.14023616909980774, -0.059298884123563766, -0.15558567643165588, 0.08540964126586914, 0.09705188870429993, 0.1890958994626999, 0.04175947979092598, 0.05724041536450386, 0.023846516385674477, -0.01162020955234766, 0.05618230253458023, -0.11579210311174393, 0.06692341715097427, 0.2076416164636612, 0.10735709965229034, -0.19741249084472656, 0.07542881369590759, 0.07299964129924774, -0.19230817258358002, 0.21470485627651215, 0.1808936446905136, 0.08556947857141495, -0.014078847132623196, 0.04932648316025734, -0.07120060920715332, -0.08717942237854004, 0.10432149469852448, 0.07008610665798187, -0.14635199308395386, -0.02319888025522232, 0.04753640294075012, 0.0031168931163847446, 0.038712240755558014, 0.012844483368098736, -0.09838096797466278, -0.1476317048072815, -0.1024518832564354, 0.25637975335121155, 0.05779942497611046, 0.13955754041671753, -0.04897642880678177, 0.04941171035170555, 0.05478445068001747, -0.0822916328907013, 0.15985016524791718, -0.23752278089523315, 0.013087687082588673, -0.08002549409866333, 0.04147155582904816, -0.17947277426719666, 0.15032881498336792, 0.03206994757056236, -0.11649857461452484, -0.1835104078054428, -0.0652177482843399, 0.13445037603378296, -0.10276752710342407, -0.1366281807422638, 0.07040154188871384, -0.0566517673432827, 0.020122749730944633, -0.14659397304058075, 0.08614286780357361, 0.17548857629299164, 0.06734488904476166, 0.0469367541372776, -0.15271416306495667}, {0.07357744872570038, -0.012546571902930737, 0.17114470899105072, 0.13152118027210236, -0.08951099216938019, -0.20527036488056183, 0.15437012910842896, -0.5089982151985168, 0.09243461489677429, -0.055357035249471664, 0.13798867166042328, -0.15828903019428253, 0.2339852750301361, -0.04862150922417641, -0.036938611418008804, -0.12270378321409225, 0.1504140943288803, 0.05166013538837433, 0.004268151242285967, -0.30805033445358276, 0.0022196192294359207, -0.16838659346103668, -0.24243664741516113, -0.23328790068626404, -0.23353280127048492, 0.042548295110464096, 0.13179591298103333, -0.11065781116485596, 0.07705645263195038, 0.1328798085451126, -0.15297089517116547, 0.32648491859436035, 0.14868202805519104, -0.08544276654720306, 0.08123950660228729, 0.2100757658481598, -0.008574679493904114, 0.031956177204847336, 0.10948144644498825, -0.16126316785812378, -0.16813747584819794, 0.056038036942481995, -0.09912215918302536, 0.09552987664937973, 0.11938218772411346, -0.1257079690694809, -0.008740309625864029, 0.05510404706001282, -0.16570807993412018, 0.05321764945983887, -0.03449524939060211, 0.06693380326032639, 0.17676100134849548, 0.04486044496297836, -0.08405231684446335, 0.05094718188047409, -0.23559844493865967, -0.055830005556344986, -0.11275069415569305, -0.09279480576515198, -0.12314027547836304, 0.03276032209396362, 0.1259499192237854, 0.04105401784181595, 0.10133451968431473, -0.34307795763015747, 0.047510623931884766, 0.04833626002073288, 0.15254829823970795, -0.07169713079929352, -0.05170520395040512, -0.07955481857061386, -0.6636428236961365, 0.014046769589185715, -0.03772221878170967, 0.16008751094341278, 0.029668768867850304, 0.05779039487242699, 0.19100691378116608, -0.18040981888771057, 0.008864914067089558, -0.20155860483646393, 0.04111076146364212, 0.3071812689304352, -0.24555352330207825, -0.09224268049001694, -0.11789547652006149, -0.04330672696232796, 0.12619201838970184, -0.13860823214054108, -0.02309645153582096, 0.13677984476089478, 0.0007360987947322428, 0.09003910422325134, 0.09929310530424118, -0.0012212940491735935, -0.036124877631664276, -0.03248428925871849, 0.19562655687332153, -0.23687897622585297, 0.0621911846101284, 0.023009372875094414, -0.2916012108325958, -0.005693479906767607, 0.2692308723926544, -0.23069719970226288, 0.20643435418605804, 0.13702961802482605, -0.011494225822389126, -0.2690081000328064, 0.07014265656471252, -0.28179603815078735, -0.2633962333202362, 0.01098190899938345, 0.032282691448926926, -0.09918388724327087, 0.10032336413860321, -0.040676139295101166, 0.10527528077363968, -0.0900343805551529, -0.27334126830101013, -0.2639695107936859, -0.08539509028196335, 0.44007423520088196, 0.07752985507249832, 0.08559921383857727, -0.08918745815753937, -0.12252192944288254, 0.022971129044890404, -0.009861045517027378, -0.13967758417129517, -0.11947652697563171, 0.49085986614227295, -0.00855652242898941, -0.24735695123672485, 0.10795016586780548, -0.062226176261901855, -0.12373457849025726, 0.08065824955701828, -0.020671209320425987, 0.09648576378822327, -0.06325837969779968, -0.07640714198350906, 0.1175338476896286, 0.28595107793807983, 0.08999652415513992, 0.02278311923146248, -0.11937858909368515, -0.10242386162281036, 0.05851740762591362, -0.03543819114565849, 0.1572125405073166, 0.06091072037816048, 0.24460557103157043, -0.2599337100982666, -0.1117754876613617, 0.13133494555950165, -0.08827164769172668, -0.043153341859579086, 0.19545546174049377, -0.10094807296991348, 0.20744650065898895, 0.08353620767593384, -0.17822229862213135, 0.14758118987083435, 0.13188445568084717, 0.08334444463253021, 0.21002992987632751, -0.005705490242689848, 0.11171220242977142, 0.06781021505594254, 0.0030033753719180822, -0.11072342097759247, 0.1913435012102127, -0.14050769805908203, 0.11510448902845383, 0.0028661286924034357, -0.02283129282295704, -0.01848371885716915, -0.21995973587036133, 0.09511721879243851, -0.08936864137649536, -0.036129727959632874, -0.17053712904453278, -0.1632494032382965, 0.005396110936999321, 0.25275951623916626, -0.03778567910194397, -0.09810470789670944, 0.18833380937576294, -0.18547092378139496, -0.10956372320652008, 0.15181167423725128, 0.11293292790651321, -0.4256037473678589, -0.057769861072301865, -0.10396037250757217, -0.08449719846248627, 0.21900822222232819, 0.1997867077589035, -0.008390235714614391, -0.012110390700399876, -0.0027810048777610064, -0.07284791767597198, 0.2053515762090683, 0.0403611958026886, -0.13327708840370178, -0.09100991487503052, -0.19293098151683807, -0.28131985664367676, -0.10072115808725357, 0.15879137814044952, 0.06779888272285461, -0.13469815254211426, 0.06767876446247101, 0.024119360372424126, -0.08129927515983582, 0.18032853305339813, -0.2522194981575012, 0.060863006860017776, 0.030580060556530952, -0.13095691800117493, -0.03037254884839058, 0.3430970013141632, 0.14027075469493866, 0.3980327248573303, -0.19349825382232666, -0.18125852942466736, -0.0359506830573082, 0.04679212346673012, 0.0880580022931099, 0.13150568306446075, -0.004491784144192934, 0.017193498089909554, 0.12683479487895966, -0.02702222391963005, -0.03267386928200722, -0.10065719485282898, 0.1343691200017929, -0.07050259411334991, -0.10483336448669434, 0.13273656368255615, -0.09484520554542542, 0.07531335949897766, 0.11687085032463074, 0.11365485191345215, -0.0020805823151022196, -0.07835392653942108, 0.07553789764642715, 0.07670319825410843, 0.026800958439707756, 0.05872301012277603, -0.2575063109397888, 0.10537443310022354, -0.088468998670578, 0.22445504367351532}, {-0.0334860123693943, 0.09215875715017319, 0.03673709183931351, -0.050236936658620834, 0.22236225008964539, 0.19123110175132751, -0.063517726957798, -0.07026825100183487, -0.018049001693725586, 0.11125802993774414, -0.17700257897377014, -0.01463031955063343, 0.02347560226917267, 0.09527619183063507, 0.19024929404258728, -0.10577955096960068, -0.07995028793811798, 0.0015085021732375026, 0.09559842199087143, -0.00980697013437748, -0.21057629585266113, -0.0463869646191597, -0.014280383475124836, 0.03521530330181122, -0.11797966063022614, -0.06825528293848038, 0.032536666840314865, -0.025681255385279655, -0.029570309445261955, 0.01473072450608015, -0.1270754635334015, -0.1049894243478775, 0.08951916545629501, -0.13516798615455627, -0.2148979902267456, 0.08620047569274902, -0.013558453880250454, 0.07644924521446228, -0.033143218606710434, 0.1849990040063858, -0.09777513146400452, -0.022206369787454605, -0.12077843397855759, -0.11419744789600372, 0.05267471447587013, 0.17864954471588135, 0.11254722625017166, -0.04301425442099571, 0.06323902308940887, 0.024152439087629318, 0.19336993992328644, -0.10078556090593338, 0.11515611410140991, 0.12549202144145966, 0.10705382376909256, -0.04805530235171318, 0.05394800379872322, 0.13529497385025024, 0.052141182124614716, 0.05257488414645195, -0.018404651433229446, -0.056872010231018066, -0.06904035806655884, -0.0336906760931015, -0.13109073042869568, 0.19598010182380676, 0.13207630813121796, 0.04736688733100891, 0.15315799415111542, -0.21111105382442474, -0.12918366491794586, -0.08935174345970154, -0.09718085825443268, -0.2884739637374878, -0.06342009454965591, -0.006758158095180988, 0.20540881156921387, -0.0657128244638443, 0.02755551226437092, 0.18682537972927094, 0.08223751187324524, 0.14914903044700623, -0.11289754509925842, -0.004286252427846193, 0.4105496108531952, -0.1976267546415329, 0.3580886423587799, 0.10057435184717178, 0.22940897941589355, -0.1509786993265152, -0.04984387010335922, -0.16102765500545502, 0.1987379789352417, 0.06613107025623322, -0.010912050493061543, -0.09623361378908157, -0.061468321830034256, 0.06625261902809143, 0.06882329285144806, -0.0455927737057209, -0.03926178812980652, 0.022053496912121773, 0.1905641108751297, -0.049210794270038605, 0.038575705140829086, 0.05612711235880852, -0.07208529859781265, 0.1615055501461029, -0.051641978323459625, -0.07423035800457001, -0.08807116746902466, 0.08673953264951706, -0.04220777377486229, -0.20200392603874207, -0.1825709044933319, -0.058211974799633026, 0.06718647480010986, -0.12763738632202148, 0.015309292823076248, -0.0898241177201271, -0.13564935326576233, -0.023759888485074043, -0.095718152821064, 0.20284798741340637, 0.25218191742897034, -0.012394571676850319, -0.04165029153227806, 0.08528070151805878, 0.22966115176677704, -0.08984944224357605, -0.028031619265675545, -0.08236497640609741, 0.052791155874729156, 0.06983579695224762, 0.028379205614328384, -0.1511780023574829, -0.3200065791606903, -0.07869748026132584, 0.049263279885053635, -0.1271914541721344, -0.20203372836112976, -0.03548178821802139, -0.06594958901405334, 0.14910031855106354, 0.043272465467453, 0.05104716122150421, 0.016637343913316727, -0.09040970355272293, -0.2534031569957733, -0.14061473309993744, 0.05619160458445549, 0.01338098756968975, -0.08080658316612244, 0.3125418722629547, 0.10273174941539764, -0.06576137244701385, -0.04764382913708687, 0.05564163252711296, 0.10568633675575256, 0.12279552221298218, 0.06284555047750473, 0.1380549669265747, 0.1365666389465332, -0.12244797497987747, -0.06020531803369522, 0.1566547453403473, -0.023651108145713806, -0.23290148377418518, -0.4394005835056305, -0.03837917372584343, 0.026166893541812897, -0.09847550839185715, 0.06913864612579346, 0.2602780759334564, -0.04973079636693001, 0.05676329508423805, 0.001175773679278791, 0.14188317954540253, 0.05611511692404747, 0.09394098818302155, 0.15909717977046967, -0.14588244259357452, -0.2261486053466797, 0.0011306054657325149, -0.1465139389038086, 0.15290355682373047, -0.0020665074698626995, 0.07574960589408875, 0.003941668197512627, 0.007422364316880703, 0.04269924759864807, -0.12860359251499176, -0.06766507774591446, 0.0119243785738945, 0.18539945781230927, 0.2746005058288574, 0.24153603613376617, -0.20697808265686035, 0.0588734932243824, 0.12670809030532837, 0.02568769082427025, -0.25767695903778076, -0.1737094670534134, 0.07035128027200699, -0.08831600099802017, 0.048505961894989014, -0.07720471918582916, -0.184727281332016, -0.010008555836975574, 0.00831643771380186, 0.007514075376093388, -0.18278862535953522, -0.05943068861961365, -0.05950029939413071, -0.033718276768922806, -0.16578982770442963, -0.031020449474453926, 0.004595593549311161, -0.01169420313090086, 0.08416087925434113, -0.13465115427970886, -0.15883781015872955, 0.07515303045511246, -0.1620868742465973, -0.051712047308683395, -0.05109849199652672, 0.002044287044554949, 0.04986337572336197, 0.21122455596923828, 0.12043964117765427, 0.09819122403860092, 0.09346302598714828, 0.180807426571846, 0.29300689697265625, -0.04925745353102684, 0.10538989305496216, 0.16289682686328888, -0.2904142737388611, 0.0056619239039719105, 0.20748353004455566, -0.008410387672483921, -0.005866159684956074, 0.1531396210193634, 0.06051056459546089, -0.1581835001707077, -0.14700475335121155, -0.008890061639249325, 0.05867806449532509, -0.14779363572597504, 0.09406635165214539, -0.06485197693109512, 0.08643882721662521, 0.014550196938216686, -0.07334604114294052, 0.02643686905503273, 0.05754672363400459}, {-0.013520576991140842, 0.056802887469530106, 0.022284982725977898, 0.03008340671658516, 0.06976376473903656, -0.06880420446395874, 0.012765012681484222, -0.1968320608139038, -0.1686861366033554, -0.13357405364513397, -0.10080205649137497, 0.10739284008741379, -0.06860930472612381, 0.008575724437832832, 0.0461597740650177, 0.3520505428314209, 0.11927299946546555, 0.16322556138038635, -0.02820153720676899, 0.07249874621629715, 0.33463743329048157, -0.008926039561629295, -0.029337739571928978, -0.09463059902191162, 0.13894866406917572, 0.06372187286615372, 0.17071174085140228, -0.012802457436919212, -0.0025193123146891594, 0.017210470512509346, 0.10627198219299316, 0.2133885622024536, 0.23572826385498047, 0.07799877226352692, 0.03341638296842575, 0.08186600357294083, -0.0006585688097402453, 0.2840116322040558, -0.10115805268287659, 0.1162784993648529, -0.10529884696006775, -0.15482725203037262, -0.02512124925851822, -0.021286888048052788, 0.028387775644659996, 0.112998828291893, -0.0017625425243750215, -0.019729578867554665, 0.16669972240924835, -0.06690090894699097, -0.28874707221984863, 0.12319660186767578, -0.09813570231199265, 0.09441503137350082, -0.06751859188079834, -0.010731471702456474, 0.050337500870227814, 0.016120783984661102, -0.3579074442386627, -0.05313316732645035, 0.00364967854693532, -0.10867564380168915, 0.44461047649383545, 0.17498727142810822, -0.013917290605604649, -0.3155827522277832, 0.04257607087492943, 0.27495405077934265, 0.3741854131221771, -0.0007697047549299896, 0.0874556377530098, 0.1895868331193924, -0.3290579617023468, -0.05495710298418999, -0.16875484585762024, -0.11723803728818893, -0.09492145478725433, -0.09623643010854721, -0.010052845813333988, -0.21998386085033417, -0.06677339971065521, -0.28533390164375305, -0.055692531168460846, -0.1075449287891388, 0.07514405995607376, -0.3482632040977478, -0.08465074747800827, -0.1450197398662567, 0.04890032485127449, -0.07548362761735916, 0.07461438328027725, -0.08266296237707138, 0.008713392540812492, -0.11688581854104996, 0.3429551422595978, 0.22898703813552856, -0.06030363216996193, -0.06796874850988388, -0.03281357139348984, -0.08929208666086197, -0.15188664197921753, 0.13361315429210663, -0.08984988927841187, 0.1674417406320572, 0.2210252583026886, -0.25083017349243164, -0.050757020711898804, 0.042608290910720825, 0.018392523750662804, -0.012081637978553772, -0.08514165878295898, -0.37709274888038635, -0.011619921773672104, 0.21248534321784973, -0.1123909130692482, -0.007383099291473627, 0.1774224191904068, -0.25750526785850525, 0.10211383551359177, -0.10065775364637375, -0.04176509007811546, -0.1477183699607849, 0.22848369181156158, 0.19144921004772186, -0.05179270729422569, 0.237864688038826, -0.06074981391429901, 0.12391325831413269, 0.12531894445419312, -0.012549997307360172, 0.15547288954257965, 0.0629962757229805, 0.10110113024711609, 0.04928947612643242, -0.21602292358875275, -0.26786181330680847, 0.012415571138262749, -0.1922769993543625, -0.037015561014413834, 0.3778637647628784, -0.09580154716968536, -0.02620365284383297, -0.13352008163928986, -0.25907549262046814, -0.03185642510652542, -0.06462639570236206, -0.026292799040675163, 0.18807443976402283, 0.14377808570861816, 0.05274299532175064, -0.3335146903991699, 0.027072645723819733, 0.03654872998595238, 0.018880460411310196, 0.3062969148159027, 0.17901936173439026, 0.1366819441318512, -0.14723481237888336, 0.23397228121757507, -0.15200190246105194, -0.06678938865661621, 0.07465346157550812, 0.021938474848866463, -0.022953467443585396, -0.08519101142883301, 0.21860405802726746, 0.053766049444675446, 0.2399434596300125, 0.018048839643597603, 0.05734590068459511, -0.08689846843481064, -0.05263581499457359, 0.008925764821469784, 0.05427524819970131, -0.018691791221499443, -0.1256658136844635, 0.08123858273029327, 0.056593094021081924, 0.07494378089904785, -0.029488535597920418, 0.14865753054618835, 0.17146357893943787, 0.04547445476055145, -0.2160160094499588, 0.04230642318725586, 0.044385720044374466, 0.5860061049461365, -0.08481959998607635, 0.01817728765308857, 0.042109593749046326, 0.05835084617137909, -0.24990150332450867, -0.2425745129585266, -0.02451045624911785, -0.15513308346271515, -0.015245121903717518, -0.12422267347574234, 0.22554588317871094, -0.09514380246400833, 0.02519562467932701, -0.34293073415756226, 0.028689296916127205, 0.055062320083379745, -0.024379415437579155, -0.12607720494270325, -0.033076219260692596, 0.0775030329823494, -0.06016611307859421, -0.12209814786911011, 0.1639554798603058, -0.06256953626871109, 0.230905681848526, -0.27715080976486206, 0.0505584180355072, 0.11233296245336533, 0.09467888623476028, -0.049582865089178085, 0.003930315840989351, -0.18580016493797302, 0.21892087161540985, -0.09778829663991928, -0.035138681530952454, -0.12477566301822662, 0.33149605989456177, 0.03783929720520973, 0.3639346659183502, -0.7214707136154175, -0.006062163505703211, 0.21490921080112457, -0.33973428606987, 0.07180190831422806, -0.06339137256145477, 0.05613742396235466, -0.044996265321969986, 0.005480123683810234, -0.14959615468978882, -0.00020223992760293186, -0.11292688548564911, 0.1476859599351883, -0.010516448877751827, 0.007251972332596779, -0.08497098088264465, 0.149601012468338, 0.07245419919490814, -0.02343059703707695, -0.029382584616541862, -0.034893814474344254, 0.03317742422223091, -0.10524577647447586, -0.20873193442821503, 0.22037151455879211, -0.17070645093917847, 0.10852387547492981, -0.09813867509365082, -0.15193234384059906, -0.05312182009220123}, {0.21613366901874542, 0.06609312444925308, -0.22602936625480652, 0.043522343039512634, -0.08128532022237778, -0.10755559056997299, -0.12829172611236572, -0.08546609431505203, 0.12670566141605377, -0.22529204189777374, 0.15777891874313354, -0.2791139483451843, -0.12828825414180756, 0.09007368236780167, 0.12828274071216583, -0.005371191073209047, -0.10223682969808578, -0.04935762286186218, 0.04357311502099037, 0.20127280056476593, 0.13039296865463257, 0.012131670489907265, -0.01761438325047493, -0.016879050061106682, -0.1724708378314972, 0.015901975333690643, 0.30955979228019714, 0.06770723313093185, -0.05323716998100281, -0.06682021915912628, 0.01039436087012291, 0.015510199591517448, -0.4608934819698334, 0.22505377233028412, 0.1502685844898224, 0.11341964453458786, 0.0032550878822803497, 0.10622061043977737, -0.19702816009521484, 0.09523052722215652, 0.1422450840473175, -0.20420807600021362, -0.08024627715349197, 0.2722003161907196, 0.18398621678352356, -0.037793707102537155, -0.28690987825393677, -0.06125560402870178, 0.045995745807886124, 0.20779506862163544, -0.14943274855613708, 0.09350693970918655, -0.062013644725084305, -0.2996685206890106, 0.024416739121079445, -0.15886472165584564, -0.14439500868320465, -0.14265988767147064, 0.03144071251153946, -0.46688443422317505, 0.17601537704467773, 0.06583236902952194, 0.17696964740753174, -0.1469011753797531, 0.07021815329790115, -0.31101757287979126, 0.050639934837818146, 0.2713104486465454, -0.0695720762014389, -0.020800335332751274, 0.2939389646053314, -0.0636923760175705, -0.39328184723854065, -0.32160884141921997, 0.2655445635318756, 0.08698771148920059, 0.012325872667133808, 0.02457313984632492, -0.2673540711402893, -0.05265360325574875, -0.13039174675941467, 0.12331034243106842, -0.046299584209918976, -0.19600555300712585, 0.039188630878925323, -0.07474575191736221, -0.35105785727500916, -0.06903291493654251, -0.08250793814659119, 0.2621515989303589, 0.2262566089630127, 0.008447287604212761, 0.02628641203045845, -0.1097651943564415, -0.021321238949894905, -0.14492617547512054, 0.0893532931804657, -0.3062421977519989, -0.009461106732487679, 0.5470337867736816, -0.28009119629859924, 0.14380335807800293, 0.058616336435079575, 0.1168472170829773, -0.4412403106689453, 0.6802645921707153, -0.0046998728066682816, -0.11376366019248962, 0.06212949380278587, -0.11744502931833267, 0.3952285945415497, -0.3096362352371216, 0.4092162847518921, -0.04480383172631264, 0.02675745263695717, -0.27519068121910095, -0.024940453469753265, -0.2860003113746643, -0.00526850949972868, 0.015040162019431591, -0.07043232023715973, -0.2578808069229126, 0.05834454670548439, 0.09453298151493073, 0.14184066653251648, 0.4708543121814728, -0.1452193558216095, -0.13176612555980682, -0.4884856939315796, 0.05544895678758621, -0.27623987197875977, 0.11631886661052704, 0.3730086088180542, 0.11632783710956573, -0.2457883059978485, -0.2535177171230316, -0.1319190263748169, 0.11230729520320892, -0.05562753602862358, -0.2957238554954529, -0.17338356375694275, 0.16372767090797424, 0.044529978185892105, -0.21709270775318146, 0.0465075820684433, -0.08559219539165497, -0.04506850242614746, 0.1556721180677414, -0.10567032545804977, 0.008866146206855774, -0.0592411644756794, 0.190365269780159, 0.11800342053174973, -0.027381090447306633, 0.15518100559711456, 0.037554867565631866, -0.12659573554992676, 0.07820366322994232, 0.18449127674102783, -0.0723918154835701, 0.13219186663627625, -0.059689439833164215, 0.021209903061389923, 0.01622674986720085, 0.013572506606578827, 0.6347599029541016, 0.06753586232662201, 0.04161098599433899, 0.2591334879398346, 0.11994051933288574, -0.16562984883785248, 0.1529507040977478, 0.04369109123945236, 0.11655690521001816, 0.33375948667526245, -0.36468708515167236, -0.039704106748104095, -0.1930254101753235, 0.050530217587947845, -0.014861153438687325, 0.20070011913776398, -0.11000305414199829, -0.07748294621706009, -0.05765977129340172, 0.5362400412559509, 0.16357854008674622, 0.3292315602302551, -0.0004592657496687025, 0.12133882939815521, -0.040276676416397095, 0.033425118774175644, 0.05960455164313316, -0.1319202184677124, -0.03742684796452522, -0.5230974555015564, -0.16739074885845184, 0.11943741887807846, 0.09148689359426498, -0.06219523027539253, 0.06390154361724854, 0.1268945336341858, -0.17195479571819305, 0.11697130650281906, -0.2292688637971878, -0.017924567684531212, 0.09272821247577667, -0.16085389256477356, -0.2738748788833618, -0.27724361419677734, 0.09652037173509598, -0.15485021471977234, 0.08753972500562668, 0.12367072701454163, -0.11417349427938461, 0.0794898048043251, -0.17775018513202667, -0.21611063182353973, -0.029180722311139107, -0.346011221408844, 0.19313320517539978, 0.028025921434164047, 0.19102643430233002, -0.2241138368844986, 0.004781174007803202, -0.17820367217063904, 0.09745752811431885, -0.3742808699607849, -0.07258514314889908, -0.008854212239384651, -0.2693387567996979, 0.010147818364202976, -0.009583705104887486, 0.1516849249601364, 0.0020767091773450375, -0.07833611965179443, 0.0942351222038269, -0.06284106522798538, -0.08861584961414337, -0.10193809866905212, 0.6960651278495789, -0.025133756920695305, -0.029064185917377472, 0.1318778097629547, -0.3335559070110321, 0.152306467294693, 0.02843633107841015, 0.06374924629926682, -0.0832672193646431, -0.3758070468902588, -0.24991796910762787, 0.1074262484908104, -0.045120056718587875, -0.0016844907077029347, -0.14524978399276733, -0.1898590475320816, 0.0690440759062767}, {-0.03097367100417614, -0.018791014328598976, 0.015023576095700264, 0.16587571799755096, 0.1144484207034111, 0.02337576262652874, -0.2102367877960205, -0.0032563256099820137, -0.0271117202937603, -0.2605258822441101, -0.0933728814125061, 0.021682094782590866, 0.10201083868741989, 0.18643848598003387, 0.08885295689105988, -0.2229262888431549, -0.10839451849460602, 0.06769979000091553, 0.1339809149503708, 0.005288978107273579, 0.24064582586288452, -0.03590000048279762, -0.034417953342199326, 0.043111223727464676, -0.07657451182603836, 0.01868291012942791, 0.07551167160272598, 0.034413717687129974, -0.12577815353870392, -0.13933409750461578, 0.06643367558717728, -0.01991535909473896, -0.14699985086917877, 0.07231105864048004, 0.17993101477622986, -0.0007853633142076433, 0.06696002185344696, 0.22668445110321045, -0.01266559399664402, -0.08088679611682892, 0.15979890525341034, -0.04590420424938202, -0.2586594820022583, 0.11017777025699615, 0.10776392370462418, 0.13328416645526886, 0.11705251038074493, -0.027707187458872795, 0.09708157181739807, -0.06974682211875916, -0.03527495637536049, 0.07813768088817596, 0.06409312039613724, -0.20938004553318024, -0.14933694899082184, -0.015702607110142708, -0.029278431087732315, 0.08391311019659042, 0.052317142486572266, -0.00461334316059947, 0.09136365354061127, 0.033399663865566254, 0.22963038086891174, -0.06433331221342087, 0.03781253844499588, -0.03021896630525589, 0.0866105929017067, 0.02136072888970375, -0.09033705294132233, -0.23617492616176605, 0.07319086790084839, 0.10418495535850525, 0.019632987678050995, -0.040987465530633926, -0.03378693386912346, -0.0023603930603712797, -0.09375803917646408, 0.1972995102405548, -0.002990450942888856, 0.005091771483421326, -0.08549686521291733, -0.14249557256698608, -0.20830556750297546, 0.07098684459924698, 0.048418737947940826, -0.33931541442871094, -0.07265502214431763, -0.020076340064406395, 0.12119884043931961, 0.07031846046447754, -0.04615475982427597, 0.1655302345752716, 0.1295911967754364, 0.034034740179777145, 0.07859096676111221, 0.10667682439088821, 0.029329266399145126, 0.015987571328878403, -0.1391264647245407, -0.06423075497150421, 0.004968234337866306, 0.007172838784754276, -0.1470457911491394, -0.06181132048368454, -0.19641010463237762, 1.3011833429336548, 0.07842697203159332, -0.14088957011699677, 0.053496092557907104, 0.058753617107868195, -0.20055876672267914, 0.05900666490197182, -0.101089246571064, 0.09176581352949142, -0.05376559868454933, 0.024184228852391243, 0.1270969957113266, -0.046301648020744324, -0.03534713387489319, -0.1154838502407074, -0.0443742610514164, -0.15342435240745544, -0.011923951096832752, 0.166060671210289, 0.017496438696980476, 0.12789268791675568, -0.002769479062408209, 0.16250528395175934, -0.6111264228820801, -0.14026299118995667, -0.11404671519994736, 0.1533845216035843, 0.04371289163827896, -0.09954475611448288, -0.11630754172801971, -0.12201777845621109, 0.11034061014652252, 0.10843779146671295, -0.19070900976657867, 0.1020876094698906, 0.008667303249239922, 0.05934953689575195, -0.21513856947422028, -0.09254857152700424, 0.05586916580796242, 0.12689901888370514, -0.13197733461856842, -0.12100043147802353, -0.06796365231275558, -0.14246831834316254, -0.0007592499605379999, 0.019443847239017487, 0.26337528228759766, -0.12038583308458328, 0.03728886693716049, -0.0020384793169796467, -0.13142076134681702, -0.1416490525007248, 0.22419337928295135, -0.3869773745536804, -0.009483039379119873, 0.06409610062837601, -0.10121512413024902, 0.27612975239753723, -0.0740753784775734, -0.17823222279548645, -0.004878692794591188, 0.054793909192085266, 0.0033311396837234497, 0.1280549019575119, 0.04609858989715576, 0.11454678326845169, -0.07188227027654648, 0.001865697791799903, 0.06767862290143967, 0.04891999438405037, 0.1851373314857483, 0.0905190110206604, 0.09614833444356918, -0.026746872812509537, -0.15928225219249725, 0.32125547528266907, 0.03720511123538017, 0.06986073404550552, 0.02962692640721798, 0.18003763258457184, 0.1420847624540329, -0.19136753678321838, 0.05932866036891937, 0.05025394260883331, 0.03022014908492565, -0.10708899050951004, -0.0661735087633133, -0.149734765291214, -0.11721280217170715, -0.03223660588264465, -0.07339122891426086, 0.25147560238838196, -0.07787400484085083, 0.06191137433052063, 0.13196738064289093, 0.025932401418685913, 0.048121143132448196, -0.09409356862306595, 0.010899510234594345, -0.0028348250780254602, 0.5104293823242188, -0.09571123868227005, 0.1222258135676384, 0.03835040330886841, 0.11786149442195892, -0.2038508653640747, -0.15806660056114197, -0.11327394843101501, -0.12287098169326782, 0.20155906677246094, -0.14198577404022217, 0.060273587703704834, -0.3441688120365143, 0.07313983887434006, -0.11554024368524551, 0.10350465029478073, -0.07091546058654785, -0.0288790762424469, 0.20290203392505646, -0.0014497096417471766, 0.004218588583171368, 0.040959686040878296, 0.25154128670692444, -0.12160072475671768, 0.08746623247861862, -0.21070995926856995, 0.015151984989643097, 0.07033642381429672, 0.028931841254234314, -0.02413974702358246, 0.15489695966243744, 0.017277436330914497, 0.25424864888191223, 0.11940622329711914, 0.16886460781097412, 0.014384204521775246, -0.01959790103137493, 0.08675818145275116, 0.0878014862537384, -0.21642108261585236, -0.119170643389225, -0.1962052583694458, -0.13975021243095398, 0.0732361301779747, -0.013196190819144249, 0.2715328335762024, 0.04231955111026764, 0.17642353475093842, -0.02533441036939621, 0.11142736673355103}, {-0.07987460494041443, 0.014641533605754375, -0.05182399973273277, 0.004093800205737352, 0.07202213257551193, 0.173025980591774, 0.044700827449560165, 0.10158532112836838, -0.2865317165851593, -0.0664994865655899, -0.017905425280332565, -0.1878729909658432, 0.020013684406876564, 0.08241941779851913, -0.028506116941571236, 0.09422079473733902, 0.05733918771147728, 0.17060433328151703, -0.08188512176275253, 0.1042851060628891, 0.04834481701254845, 0.07520587742328644, 0.015465333126485348, -0.0019501745700836182, 0.1878037303686142, -0.08344684541225433, 0.1637718379497528, -0.09840908646583557, -0.0644369050860405, -0.06922876834869385, -0.5438806414604187, 0.16564397513866425, 0.11188644915819168, 0.020194457843899727, -0.0921524241566658, -0.08391117304563522, -0.06671670824289322, 0.019720088690519333, 0.13725793361663818, 0.026996977627277374, 0.11458208411931992, -0.08363232761621475, -0.13128355145454407, -0.09753117710351944, 0.20705166459083557, -0.09441019594669342, 0.16470006108283997, -0.08019153773784637, -0.1624508798122406, 0.19575800001621246, 0.06606072932481766, 0.14590908586978912, 0.02454611472785473, -0.05363515391945839, 0.024971630424261093, 0.22012899816036224, 0.0403897650539875, -0.09587819129228592, 0.02400045283138752, 0.012258767150342464, -0.027782663702964783, 0.0004936527111567557, -0.2487708479166031, 0.043346256017684937, 0.01581118442118168, -0.11045435070991516, 0.2060038298368454, -0.0063979788683354855, 0.08399922400712967, -0.0699022114276886, -0.06192532554268837, -0.043108001351356506, 0.08934406191110611, 0.007400833535939455, 0.4708411991596222, 0.14175230264663696, -0.11765359342098236, -0.15516258776187897, -0.11692088097333908, 0.0688810870051384, 0.09801419824361801, -0.10841047763824463, 0.04276350885629654, -0.11318692564964294, -0.10115793347358704, -0.17734864354133606, -0.0012385454028844833, -0.06550240516662598, -0.03325377777218819, -0.02187100052833557, 0.017926722764968872, 0.18178635835647583, 0.11140905320644379, 0.1967032253742218, -0.03624868765473366, -0.0860593318939209, -0.00706480024382472, -0.1731332540512085, 0.0918630063533783, 0.49701109528541565, -0.03361637145280838, 0.10317225009202957, 0.04332248866558075, -0.1656455546617508, -0.13345687091350555, 0.641409695148468, 0.03173425421118736, -0.08022940158843994, -0.12754251062870026, 0.1936989277601242, 0.033470746129751205, -0.0531279556453228, -0.018112989142537117, -0.2109028548002243, 0.03583398461341858, 0.021244054660201073, 0.11018335074186325, 0.1498458981513977, -0.061984218657016754, 0.14981281757354736, -0.039184607565402985, 0.01004711352288723, 0.1384904831647873, 0.09607237577438354, -0.07695727050304413, 0.09747498482465744, 0.002757621929049492, 0.06694328039884567, 0.14675942063331604, 0.14920124411582947, 0.0531395822763443, 0.04886512830853462, -0.12274002283811569, -0.07525893300771713, 0.08688969910144806, 0.07593122869729996, 0.17697343230247498, -0.0367649644613266, 0.024252088740468025, 0.02962927334010601, -0.05370466411113739, -0.02299537882208824, 0.03739095851778984, 0.09424908459186554, -0.06617092341184616, -0.028453266248106956, 0.09933125972747803, 0.10607296973466873, 0.1771124303340912, 0.06162741407752037, 0.13491316139698029, -0.002976626856252551, 0.048322565853595734, 0.1085985004901886, 0.1849953681230545, 0.1751510351896286, -0.0845690667629242, -0.0043562185019254684, -0.04429749399423599, 0.011764513328671455, 0.09400591999292374, -0.04231707379221916, 0.04833861067891121, -0.19624683260917664, 0.02788386307656765, -0.07529667019844055, -0.019623586907982826, 0.0766749158501625, 0.07843685895204544, 0.0988563522696495, 0.10216405242681503, 0.011017675511538982, -0.04134279862046242, 0.019152183085680008, -0.09057645499706268, 0.20517878234386444, 0.09297014772891998, 0.12222672998905182, -0.22618399560451508, 0.12640255689620972, 0.1502358764410019, -0.27768683433532715, -0.003813129849731922, -0.1470334678888321, -0.06258539110422134, -0.1296636015176773, 0.15343771874904633, -0.001769360387697816, 0.050531674176454544, 0.028095373883843422, 0.08717505633831024, 0.04218043014407158, -0.07751703262329102, 0.011856533586978912, 0.04596900939941406, -0.07060519605875015, 0.08993615210056305, -0.13466741144657135, -0.05682045593857765, -0.04298018664121628, -0.07761713862419128, -0.002513495972380042, -0.15872903168201447, -0.28452688455581665, -0.11653512716293335, -0.29196906089782715, -0.16366638243198395, -0.00015739351511001587, 0.1469646692276001, 0.03884396329522133, -0.09810444712638855, 0.09053266793489456, 0.01475537195801735, -0.3044068217277527, -0.18232260644435883, 0.09024496376514435, 0.16538526117801666, -0.08793997019529343, 0.2096012830734253, -0.11913595348596573, -0.016715601086616516, 0.08001391589641571, 0.08696882426738739, 0.28228676319122314, 0.10038986802101135, 0.1487039476633072, -0.10160230845212936, -0.14847199618816376, 0.12008340656757355, 0.05699046328663826, 0.08685988932847977, 0.013895822688937187, 0.14704768359661102, -0.02998013235628605, -0.0808655247092247, 0.022912634536623955, 0.07820934802293777, 0.18893727660179138, -0.08932635933160782, 0.3788371980190277, -0.20100033283233643, -0.06425511091947556, 0.014798874966800213, 0.14915430545806885, 0.0005480011459439993, 0.05550120770931244, -0.08640486747026443, -0.17311932146549225, 0.12957671284675598, -0.1083240807056427, -0.11758819222450256, 0.03493364527821541, 0.0576956570148468, -0.13780473172664642, -0.13936243951320648, 0.0046118153259158134}, {-0.09531814604997635, -0.07729177176952362, -0.17751862108707428, 0.02572152391076088, 0.04774489253759384, 0.10105347633361816, 0.0732225701212883, -0.02098143845796585, -0.0008891681209206581, 0.17735697329044342, -0.03659674897789955, -0.24685952067375183, 0.09584256261587143, -0.018626827746629715, 0.023559749126434326, -0.1503051519393921, -0.06063244119286537, 0.19619745016098022, -0.1784539371728897, 0.15411321818828583, -0.16810975968837738, -0.06893958896398544, 0.08610006421804428, -0.042910877615213394, 0.261557012796402, -0.010183162055909634, -0.05470547452569008, -0.03827384486794472, -0.15313637256622314, -0.09227334707975388, 0.11579624563455582, -0.141377255320549, -0.019946405664086342, 0.10462456941604614, -0.16035285592079163, -0.13768278062343597, -0.1492215394973755, -0.017260780557990074, 0.007440546527504921, -0.010290251113474369, -0.08783219009637833, -0.09432471543550491, -0.16355568170547485, -0.1675354689359665, 0.00016404170310124755, -0.20856046676635742, -0.0075930263847112656, 0.14570321142673492, 0.1637534201145172, -0.09871304780244827, -0.07036089897155762, 0.1369510143995285, -0.02471095696091652, -0.1886352300643921, 0.003940205089747906, -0.20262190699577332, 0.14071281254291534, -0.05865335837006569, -0.047136642038822174, 0.027449818328022957, 0.09900820255279541, 0.10435304045677185, -0.10277555882930756, -0.037115633487701416, -0.04119473695755005, 0.11726812273263931, -0.054768964648246765, -0.26439887285232544, 0.05083080008625984, 0.0026309217792004347, 0.013944821432232857, 0.032520197331905365, 0.5384438037872314, 0.0880509465932846, -0.130384162068367, -0.07682564109563828, -0.1433696746826172, 0.02942013554275036, -0.032948363572359085, 0.2180820107460022, -0.10005804896354675, 0.007762844208627939, 0.12741683423519135, -0.06506418436765671, 0.032768696546554565, -0.028557246550917625, -0.10134012997150421, 0.043325524777173996, -0.2248489260673523, -0.0015458258567377925, -0.02739926613867283, 0.13195963203907013, 0.1016598716378212, -0.027906490489840508, 0.27752071619033813, -0.09587825834751129, 0.004696639254689217, -0.0007927846163511276, -0.11459407210350037, 0.06257136166095734, -0.22360509634017944, 0.03572531417012215, 0.2500637173652649, -0.08542457967996597, -0.09353369474411011, -0.21886973083019257, -0.17409075796604156, 0.11549457907676697, 0.01790906861424446, 0.16470499336719513, -0.025725753977894783, 0.0031768225599080324, 0.005628115963190794, 0.016944099217653275, 0.019974322989583015, 0.06277626007795334, 0.18774883449077606, 0.06086435168981552, -0.07123591005802155, 0.0313800573348999, -0.10223139077425003, 0.09931594878435135, 0.06944821774959564, 0.1160130649805069, -0.26957735419273376, -0.08752308785915375, 0.08416885137557983, -0.053039971739053726, 0.002474720822647214, 0.007445135153830051, 0.06509093195199966, 0.049485333263874054, -0.22628073394298553, -0.018818899989128113, 0.09729859232902527, -0.09709472954273224, -0.07652925699949265, 0.1675550490617752, 0.053374942392110825, -0.12591668963432312, -0.1268233358860016, 0.16634057462215424, 0.12713515758514404, 0.17762675881385803, -0.29542651772499084, -0.022266199812293053, 0.08297668397426605, -0.04076281189918518, 0.1748354136943817, -0.08941614627838135, -0.0015371593181043863, 0.10486627370119095, 0.008647155947983265, 0.018709883093833923, 0.008937929756939411, 0.05603060498833656, 0.21938708424568176, -0.2873641848564148, 0.13518880307674408, -0.03501037508249283, 0.02660665474832058, -0.03790830075740814, -0.01588394306600094, -0.20990319550037384, -0.02364988438785076, -0.04925538972020149, 0.07207968086004257, 0.01568899303674698, -0.03459783270955086, 0.06308550387620926, 0.044955141842365265, -0.02771477773785591, -0.27551376819610596, -0.047578442841768265, -0.16706308722496033, -0.20571956038475037, 0.00575853418558836, 0.07263606786727905, 0.002394924871623516, 0.0018452837830409408, 0.03350638970732689, -0.3286506235599518, -0.10269930958747864, 0.12321608513593674, 0.05333804339170456, -0.0879630446434021, -0.051187653094530106, -0.181361585855484, -0.03373808041214943, -0.2237253040075302, 0.03844132274389267, -0.08116699755191803, -0.13006214797496796, -0.08109613507986069, -0.002318643033504486, 0.1296105831861496, -0.1301727294921875, 0.12236428260803223, -0.023980094119906425, 0.1276930868625641, 0.03742905706167221, -0.10771995037794113, -0.10301000624895096, 0.149839386343956, -0.09461033344268799, -0.06756210327148438, 0.09443071484565735, -0.07700687646865845, -0.16676665842533112, 0.03545862436294556, 0.18686014413833618, 0.014833149500191212, 0.03178985044360161, -0.09806051105260849, 0.06046523526310921, 0.19322612881660461, 0.1464088261127472, 0.060096994042396545, 0.16530394554138184, -0.0012016703840345144, -0.2685084939002991, -0.014736234210431576, -0.2357865869998932, -0.018051765859127045, -0.021414538845419884, -0.042562808841466904, 0.1176612377166748, 0.08023989200592041, -0.02904505282640457, 0.21241819858551025, -0.16044050455093384, 0.09659651666879654, 0.0906512439250946, 0.031404413282871246, -0.2495165914297104, 0.05486046522855759, 0.2615908086299896, -0.12764866650104523, -0.06773927807807922, -0.2078307569026947, -0.007438449189066887, -0.10510411858558655, 0.014154180884361267, -0.0983470007777214, -0.05987242981791496, -0.028363831341266632, 0.20647777616977692, -0.10425863415002823, 0.05686705932021141, 0.0330648198723793, -0.13489551842212677, -0.08064503222703934, 0.24291227757930756, 0.011544504202902317, -0.08792191743850708, 0.1199970468878746}, {0.15025803446769714, -0.18768911063671112, -0.05450257658958435, -0.1374642699956894, 0.12867645919322968, 0.14341945946216583, 0.26021724939346313, 0.03288381174206734, -0.049956731498241425, 0.1304377168416977, -0.2167169451713562, -0.061001408845186234, 0.01795121096074581, 0.08620421588420868, 0.15577082335948944, -0.16047093272209167, -0.03962572291493416, -0.07271486520767212, 0.027472777292132378, -0.01982932910323143, 0.05366681143641472, -0.04967910796403885, -0.04462053254246712, 0.008153407834470272, 0.25445103645324707, 0.06797507405281067, -0.022174596786499023, -0.08485627919435501, -0.026917584240436554, -0.05270160362124443, 0.09666664898395538, -0.1487143188714981, -0.18693052232265472, -0.08102301508188248, -0.3243541121482849, -0.0031836621928960085, 0.01884823851287365, -0.07859103381633759, -0.21696363389492035, 0.02131863497197628, -0.23444530367851257, 0.05261380225419998, 0.19335731863975525, 0.09705953299999237, -0.10578696429729462, -0.055243685841560364, 0.059640996158123016, 0.0842084214091301, 0.13718505203723907, 0.06076851114630699, -0.04028022289276123, 0.10083284974098206, 0.0017592726508155465, -0.011792810633778572, -0.1433718502521515, -0.048214640468358994, 0.013257295824587345, -0.11701419204473495, -0.15619507431983948, -0.019458459690213203, 0.11164670437574387, -0.03392765298485756, -0.1614762246608734, -0.13324593007564545, 0.051433760672807693, 0.2229449301958084, -0.048385124653577805, -0.0020404993556439877, 0.04619250446557999, 0.09743203222751617, -0.057965345680713654, 0.07598520815372467, 0.05112127959728241, -0.09435492753982544, -0.3340947926044464, -0.00916240643709898, 0.06364618986845016, -0.007037381175905466, 0.09504645317792892, 0.3748333752155304, -0.007086203433573246, 0.012216281145811081, 0.2187861055135727, 0.011478671804070473, 0.05965525656938553, 0.0015834965743124485, -0.07889548689126968, -0.11639019846916199, -0.13769923150539398, 0.0951412245631218, 0.042687684297561646, -0.04018233343958855, 0.06528937071561813, -0.11420329660177231, -0.03440602123737335, -0.006875369232147932, 0.24195633828639984, 0.0014262438053265214, -0.3467206358909607, -0.02860022522509098, -0.06750842183828354, 0.11676166951656342, 0.1051868349313736, -0.3068080544471741, -0.12163707613945007, -0.08848114311695099, 0.10230620205402374, 0.19411517679691315, -0.19875118136405945, 0.14045016467571259, 0.14015662670135498, -0.06796588748693466, 0.08372602611780167, -0.19693537056446075, 0.0726064071059227, 0.03735949099063873, 0.17384421825408936, 0.0954585000872612, -0.10164937376976013, -0.14733727276325226, -0.154994934797287, -0.09963954240083694, -0.01591438055038452, -0.09105685353279114, -0.2508237957954407, -0.03138728439807892, -0.020934248343110085, 0.14048069715499878, 0.06292479485273361, -0.05025554820895195, 0.09852375090122223, -0.007270189933478832, -0.04334418475627899, 0.04804704338312149, -0.2816661596298218, -0.019192347303032875, -0.0036884599830955267, 0.07291456311941147, 0.21106520295143127, -0.19048097729682922, -0.13939589262008667, 0.007067136466503143, 0.12689658999443054, -0.09096969664096832, 0.0590401217341423, -0.3653430938720703, 0.053657256066799164, 0.22139248251914978, 0.15201391279697418, -0.02944592572748661, 0.012273205444216728, -0.013673247769474983, 0.0029827472753822803, -0.02215457707643509, 0.03679349273443222, -0.09173507988452911, 0.1708691567182541, -0.26757556200027466, 0.16644227504730225, -0.2084599733352661, 0.000483013252960518, -0.04919135197997093, 0.052504632622003555, 0.02692992426455021, -0.10562463849782944, 0.30359938740730286, 0.0315881222486496, 0.02014658786356449, -0.0522545725107193, 0.00018131747492589056, -0.11382246017456055, -0.19907382130622864, -0.057436954230070114, 0.05394753813743591, -0.19348064064979553, -0.009861034341156483, 0.14074115455150604, 0.17778722941875458, 0.08477262407541275, -0.011287125758826733, 0.4554870128631592, -0.20623695850372314, 0.06480547040700912, 0.027977919206023216, 0.19839531183242798, 0.03679632768034935, -0.05734756216406822, -0.0698317214846611, -0.17879745364189148, 0.10011453926563263, -0.05751502886414528, 0.22439147531986237, -0.1379672735929489, 0.023527538403868675, 0.18141239881515503, -0.0011328200343996286, 0.15196514129638672, -0.13489419221878052, 0.04002750292420387, 0.11763247102499008, -0.008002808317542076, -0.008570857346057892, -0.27312904596328735, -0.013542244210839272, 0.10238748043775558, -0.05123304948210716, -0.22460494935512543, 0.1320379078388214, -0.06936399638652802, -0.023027347400784492, 0.044402580708265305, 0.008364781737327576, 0.12583264708518982, -0.1446804702281952, -0.07495873421430588, 0.06242412328720093, -0.02449273318052292, -0.022920601069927216, 0.3515606224536896, 0.0012874766252934933, -0.04113531857728958, 0.07424004375934601, -0.0244253259152174, 0.02923532761633396, 0.07950492948293686, 0.015540148131549358, 0.1221582442522049, 0.34595605731010437, -0.07713466882705688, 0.1492445170879364, -0.1405368596315384, 0.06337644159793854, -0.055518440902233124, 0.021252229809761047, 0.024922914803028107, 0.015201719477772713, 0.40360110998153687, -0.25021815299987793, -0.05667179822921753, -0.13078217208385468, 0.10438594222068787, -0.17759010195732117, 0.2771782875061035, -0.14577968418598175, -0.10161669552326202, 0.08913493156433105, 0.08134308457374573, 0.5553650259971619, -0.18988122045993805, -0.12596532702445984, 0.1422792226076126, 0.11533869802951813, -0.0773838609457016, 0.14428754150867462, -0.0643795058131218, 0.19014422595500946}, {0.23909953236579895, 0.032302238047122955, 0.07961816340684891, -0.007970273494720459, -0.04101807251572609, -0.3014232814311981, -0.03658277168869972, -0.48136812448501587, -0.2961427867412567, -0.012292420491576195, -0.08707728981971741, 0.08074527233839035, 0.08985891938209534, 0.028971094638109207, -0.045262791216373444, -0.13486328721046448, -0.09055174887180328, 0.05352765694260597, 0.2545427978038788, 0.0641530305147171, 0.1970762461423874, 0.041223570704460144, -0.033238135278224945, -0.2934032082557678, 0.00803651474416256, -0.0826980322599411, 0.18499743938446045, 0.1164935901761055, -0.05974486842751503, -0.09138727188110352, 0.37894269824028015, 0.1208745613694191, -0.20689639449119568, 0.132822185754776, -0.26625391840934753, 0.1287006288766861, 0.018393972888588905, -0.490848183631897, -0.0851314589381218, 0.03954984247684479, 0.09712696075439453, -0.08835846930742264, 0.018185149878263474, 0.2351417988538742, -0.1562161147594452, 0.1808411329984665, 0.12949399650096893, 0.05933697149157524, 0.14796650409698486, 0.035529062151908875, -0.04109204187989235, 0.18623630702495575, 0.18085600435733795, -0.2909708619117737, -0.0673808678984642, -0.2611212730407715, 0.06530272960662842, 0.2733272612094879, 0.14253462851047516, 0.2981528043746948, 0.1056593507528305, 0.1818479746580124, -0.14923664927482605, -0.1473126858472824, -0.015957890078425407, -0.03489626944065094, 0.16310150921344757, -0.18076609075069427, 0.08987880498170853, 0.048015229403972626, -0.012628399766981602, -0.17344313859939575, -0.16417226195335388, -0.35238710045814514, -0.36394715309143066, 0.10316382348537445, 0.07312940061092377, 0.3232783377170563, -0.32434749603271484, -0.16102619469165802, 0.03710583224892616, -0.09990101307630539, -0.13154880702495575, 0.11551062017679214, -0.16383342444896698, -0.016915034502744675, 0.2036149650812149, -0.12882766127586365, -0.01311498787254095, -0.0977192372083664, -0.16131970286369324, 0.12968190014362335, 0.22617696225643158, -0.12892402708530426, -0.008275694213807583, 0.21227024495601654, 0.04566607624292374, -0.04158419370651245, -0.14169219136238098, -0.11293650418519974, 0.08044223487377167, -0.1371556967496872, 0.16799400746822357, 0.0035172589123249054, -0.459418386220932, 0.05738239362835884, 0.22470694780349731, -0.01884429156780243, 0.1542794555425644, 0.04671074450016022, -0.1965406835079193, -0.07680217176675797, -0.24338580667972565, 0.26626092195510864, 0.01909734308719635, 0.29834678769111633, -0.17298631370067596, 0.13024760782718658, -0.0481991320848465, 0.29207122325897217, 0.10158002376556396, -0.04345142841339111, 0.17459967732429504, 0.3107888996601105, 0.3163364827632904, -0.3677709400653839, 0.12383271008729935, -0.1412346363067627, 0.05635647848248482, 0.13168661296367645, 0.0517125278711319, -0.5090774297714233, 0.3373919129371643, 0.36845117807388306, -0.25835326313972473, 0.04406636580824852, -0.12992551922798157, -0.12625588476657867, 0.05491366982460022, -0.30093276500701904, 0.07885020226240158, 0.05598437041044235, 0.3641919791698456, -0.2792711555957794, 0.01662689819931984, 0.19123227894306183, -0.21053023636341095, -0.12256788462400436, -0.11935223639011383, -0.29120582342147827, -0.038235489279031754, 0.42492249608039856, 0.11357933282852173, 0.18661004304885864, -0.07984045892953873, 0.18248699605464935, 0.24439498782157898, -0.34068867564201355, -0.256961852312088, 0.10659052431583405, 0.08193059265613556, -0.1432352066040039, -0.2491125762462616, 0.1091310903429985, -0.051290448755025864, 0.3439333140850067, -0.07455357909202576, 0.19950418174266815, 0.2307320386171341, 0.4876875877380371, -0.018294107168912888, 0.03400519862771034, 0.11686621606349945, 0.14453333616256714, -0.18172313272953033, 0.08147447556257248, -0.08450809866189957, -0.01658191904425621, 0.09684721380472183, -0.15540936589241028, -0.05108046159148216, -0.22229045629501343, -0.19771939516067505, 0.06631510704755783, -0.2785950005054474, 0.11138823628425598, -0.11487129330635071, -0.05086011439561844, -0.11925176531076431, 0.10469860583543777, -0.1407156139612198, -0.20181263983249664, 0.01440696232020855, 0.055719196796417236, -0.29895931482315063, 0.5474636554718018, -0.15144284069538116, -0.05826614797115326, 0.13402853906154633, -0.011859540827572346, -0.039030592888593674, 0.004660039208829403, -0.026429442688822746, 0.19788888096809387, -0.06401794403791428, 0.36219361424446106, 0.02257305011153221, -0.2914358377456665, 0.06233788654208183, -0.014458969235420227, -0.28947046399116516, -0.14044323563575745, -0.06241375952959061, -0.10737662017345428, -0.051477283239364624, -0.1351703256368637, -0.42683637142181396, -0.1611073613166809, -0.3626205623149872, -0.14907336235046387, 0.07562007755041122, 0.29318973422050476, -0.07491795718669891, 0.021576883271336555, -0.00875675305724144, -0.19086475670337677, -0.05605071410536766, -0.16393855214118958, 0.12188103049993515, 0.30587106943130493, 0.2184779942035675, 0.05119875445961952, -0.021084923297166824, 0.0874943882226944, 0.08429834246635437, -0.055840808898210526, 0.003093012375757098, -0.08807084709405899, 0.3563609719276428, 0.0610576830804348, 0.23473159968852997, -0.030346384271979332, -0.16627730429172516, -0.0421249121427536, -0.010239111259579659, -0.12159948796033859, -0.2989959418773651, 0.10829493403434753, -0.10277649760246277, -0.3354901969432831, -0.2373887151479721, -0.03568877652287483, 0.03423578292131424, 0.02784094586968422, 0.18837174773216248, 0.20586995780467987}, {-0.13975849747657776, -0.07514023780822754, -0.020260876044631004, -0.11433453857898712, 0.09512736648321152, 0.037055693566799164, 0.002264520386233926, 0.14120960235595703, -0.10571953654289246, -0.15895718336105347, -0.15636174380779266, -0.15641909837722778, 0.004292933735996485, -0.01900489255785942, -0.09108662605285645, -0.04201453924179077, -0.018410783261060715, 0.12722547352313995, -0.06915105134248734, 0.1672191321849823, -0.21489134430885315, -0.005127929616719484, 0.016889186576008797, 0.021550551056861877, 0.09897870570421219, 0.016611209139227867, -0.08053924143314362, -0.08516785502433777, -0.01696649380028248, -0.14273788034915924, -0.05066319555044174, -0.014698144979774952, 0.034904297441244125, 0.09766146540641785, -0.10849221050739288, -0.047617875039577484, -0.11627884209156036, 0.016408471390604973, 0.05902036651968956, 0.16788896918296814, 0.0053313784301280975, -0.022731231525540352, -0.004957851022481918, -0.07515256106853485, 0.2356688231229782, 0.09330200403928757, 0.12796764075756073, 0.13776938617229462, -0.022691836580634117, -0.22654791176319122, 0.08130494505167007, -0.20110392570495605, 0.1295967996120453, -0.008476062677800655, -0.01684851013123989, -0.07409530878067017, 0.04448055103421211, -0.010576095432043076, 0.0439048632979393, 0.08253612369298935, 0.019723590463399887, 0.23936355113983154, 0.0303571168333292, 0.07890693098306656, 0.030848361551761627, -0.032756417989730835, -0.13346126675605774, 0.08962135016918182, -0.16685624420642853, 0.015996266156435013, -0.16388264298439026, 0.04627760499715805, -0.031379334628582, 0.05392855778336525, 0.03602686896920204, -0.0673724114894867, 0.14142510294914246, 0.06318284571170807, -0.03028719872236252, 0.1727764755487442, 0.10171658545732498, 0.026166610419750214, -0.03668174147605896, -0.10222923755645752, -0.01869926042854786, 0.31739693880081177, 0.01632268726825714, -0.14086127281188965, -0.08496256172657013, 0.08017626404762268, -0.02822151780128479, 0.1876375526189804, -0.03032475896179676, 0.17052464187145233, -0.0373435840010643, -0.021049071103334427, -0.023030636832118034, 0.15422408282756805, 0.016428088769316673, 0.047471482306718826, -0.02308894693851471, -0.06660155951976776, -0.04724237322807312, -0.09133732318878174, 0.09294264018535614, 0.29377785325050354, -0.02578805387020111, 0.00898798182606697, 0.04886890947818756, 0.017068300396203995, 0.03918492794036865, 0.14213460683822632, 0.053620658814907074, -0.03204338625073433, 0.04972987622022629, 0.01347243320196867, -0.06466700881719589, -0.10273853689432144, -0.13446325063705444, -0.014025202952325344, -0.12686926126480103, 0.022914566099643707, 0.12548623979091644, 0.0030683185905218124, -0.1931118220090866, -0.4336583614349365, 0.017990512773394585, 0.0017157127149403095, 0.07557329535484314, -0.02365095168352127, 0.0825205072760582, -0.06501853466033936, 0.14472128450870514, 0.05921264737844467, 0.05578678473830223, 0.41664832830429077, 0.09345762431621552, 0.0012864298187196255, 0.05284251645207405, -0.03589894250035286, 0.04097964987158775, -0.16452118754386902, 0.3360964059829712, 0.034899819642305374, 0.11380832642316818, -0.005733103025704622, 0.05416540056467056, -0.1550784856081009, 0.2248954474925995, -0.030175169929862022, 0.15203118324279785, -0.05023394525051117, -0.06996965408325195, -0.044135935604572296, -0.029534511268138885, -0.11696898937225342, 0.03075048327445984, -0.004993187729269266, -0.15064652264118195, 0.016224097460508347, -0.03491614758968353, -0.1146748811006546, 0.10640383511781693, -0.06441675871610641, -0.020578395575284958, 0.014240838587284088, 0.010129626840353012, -0.24356192350387573, 0.12447412312030792, -0.01085750013589859, -0.020013250410556793, 0.011028521694242954, -0.09534191340208054, -0.021254921332001686, 0.02747432515025139, -0.006882158573716879, 0.007144290488213301, 0.09300041198730469, -0.06559589505195618, -0.033120427280664444, 0.0774725005030632, -0.06058913469314575, 0.1465815007686615, -0.0029807016253471375, -0.038757260888814926, 0.021242719143629074, -0.027695177122950554, 0.031928256154060364, -0.09010275453329086, -0.170710027217865, 0.05669628456234932, 0.04136289283633232, 0.18235626816749573, 0.10625588148832321, -0.05393588915467262, -0.08592606335878372, 0.014144258573651314, -0.2314741611480713, 0.07972033321857452, -0.08450388163328171, -0.03153844177722931, -0.026105821132659912, 0.09254839271306992, -0.021210093051195145, -0.06101498752832413, -0.09067602455615997, 0.013574346899986267, 0.12781278789043427, 0.17014431953430176, -0.17723923921585083, -0.03269808366894722, 0.13440294563770294, 0.11447575688362122, 0.01963002234697342, -0.07137617468833923, -0.01454092189669609, 0.1053665280342102, -0.029136791825294495, 0.04248752444982529, -0.16968770325183868, -0.09164244681596756, 0.12812767922878265, -0.06599612534046173, 0.11205399036407471, -0.0034564826637506485, 0.09552644938230515, 0.11731534451246262, 0.008196357637643814, 0.0808577686548233, -0.052871737629175186, -0.07416914403438568, -0.0952867791056633, 0.07877923548221588, 0.0620088167488575, 0.033633895218372345, 0.0029278164729475975, -0.013127784244716167, 0.14777003228664398, -0.025978712365031242, 0.26454928517341614, 0.092179074883461, 0.005245319101959467, 0.033846694976091385, -0.048317380249500275, 0.1793060004711151, 0.12660753726959229, 0.012163653038442135, 0.021241962909698486, -0.025654487311840057, -0.06452676653862, -0.06369462609291077, 0.04351281002163887, -0.09573528170585632, -0.01340436190366745, -0.008924136869609356, 0.054196931421756744}, {0.07135505974292755, -0.05391550436615944, 0.1558268666267395, 0.15289098024368286, -0.10870462656021118, 0.0038803613279014826, 0.1579132378101349, 0.07236143201589584, 0.07622458040714264, -0.2083538919687271, 0.18013277649879456, 0.05438634380698204, -0.05372370034456253, 0.15020810067653656, -0.1330442726612091, -0.03899630531668663, -0.07566750794649124, 0.08879002928733826, 0.2028336226940155, 0.21194760501384735, 0.14985665678977966, -0.05583419278264046, -0.06150280684232712, -0.16619233787059784, -0.15829849243164062, -0.24874570965766907, 0.10831224918365479, -0.20323361456394196, 0.2405872493982315, 0.10621785372495651, -0.11098494380712509, 0.14839984476566315, -0.17322517931461334, 0.0019391858950257301, -0.0058104656636714935, -0.10192949324846268, -0.020617268979549408, 0.11357762664556503, -0.0578479990363121, 0.10718610137701035, -0.11317786574363708, 0.03295116871595383, 0.05830251798033714, 0.020646188408136368, -0.046932537108659744, 0.17489711940288544, 0.06220583990216255, 0.15734611451625824, 0.06211157515645027, 0.1767609864473343, -0.1107313260436058, -0.006964833941310644, 0.09649374336004257, -0.2686639726161957, 0.23347258567810059, -0.11473757773637772, -0.03974346071481705, -0.014722947962582111, -0.11919119209051132, 0.17681729793548584, -0.13616786897182465, 0.180665522813797, -0.15545381605625153, -0.015534774400293827, -0.06659364700317383, -0.1431703120470047, 0.010969055816531181, 0.06590934842824936, -0.2509990334510803, 0.029213909059762955, 0.051108650863170624, 0.015142392367124557, 0.1411277949810028, -0.43650108575820923, -0.12799645960330963, -0.005848643369972706, 0.1407686471939087, -0.11497662961483002, 0.3007067143917084, -0.07701408863067627, -0.16856686770915985, 0.041634462773799896, 0.05022776871919632, -0.044560614973306656, 0.0016237979289144278, 0.17622865736484528, 0.08285365253686905, 0.27045413851737976, 0.03837975114583969, 0.05247215926647186, 0.22049489617347717, 0.012339151464402676, 0.4006996154785156, 0.07996954768896103, -0.044561877846717834, -0.15244261920452118, 0.012276245281100273, 0.599611759185791, -0.0014005808625370264, 0.03766312450170517, -0.03203485533595085, -0.07965130358934402, -0.09959185868501663, 0.19684109091758728, -0.24829436838626862, 0.0026582113932818174, -0.009199997410178185, -0.09597719460725784, 0.1273118406534195, -0.1628090888261795, 0.06587240099906921, -0.01747943088412285, 0.0778866782784462, -0.19100995361804962, 0.024369286373257637, -0.17241781949996948, -0.16485251486301422, -0.16511206328868866, -0.1744772493839264, -0.10888143628835678, 0.06998353451490402, 0.16255155205726624, -0.013403902761638165, -0.22554394602775574, 0.031798116862773895, -0.10800310969352722, -0.1382809579372406, 0.1386861652135849, -0.13813447952270508, 0.022444257512688637, -0.030627382919192314, 0.05676868185400963, -0.08274958282709122, 0.18441693484783173, 0.05642314255237579, -0.030045589432120323, 0.11597512662410736, 0.0486445426940918, -0.2193852812051773, 0.24792487919330597, 0.03398214280605316, 0.11990290135145187, -0.17518675327301025, -0.004978069104254246, 0.10680972784757614, 0.08198941498994827, -0.03338191658258438, -0.14860454201698303, -0.05341755226254463, -0.045514997094869614, -0.2943027913570404, -0.049530062824487686, -0.0192717257887125, 0.24218884110450745, 0.04033481702208519, -0.16892755031585693, -0.16766873002052307, -0.04826504737138748, -0.14427907764911652, -0.10481150448322296, 0.1438583880662918, -0.20401974022388458, -0.22943910956382751, 0.047127436846494675, 0.18056856095790863, -0.17592069506645203, -0.01706317439675331, 0.023524396121501923, -0.043733637779951096, 0.15614981949329376, -0.10960955172777176, 0.11428239941596985, -0.07081709802150726, -0.1152711808681488, -0.1059960424900055, -0.13816961646080017, -0.008235684596002102, -0.004283515736460686, 0.01237420178949833, -0.048350103199481964, 0.011223754845559597, 0.05480317026376724, -0.027947230264544487, -0.2705481946468353, 0.004201006609946489, -0.22670668363571167, 0.25462186336517334, -0.021042179316282272, -0.1185264065861702, 0.037026070058345795, 0.08350683748722076, 0.08827272057533264, -0.043648213148117065, -0.16546094417572021, 0.16905638575553894, 0.15999186038970947, 0.018049519509077072, 0.19090090692043304, 0.07827881723642349, 0.16153952479362488, 0.16441789269447327, -0.14937260746955872, 0.0004793730622623116, -0.14982743561267853, 0.07521761208772659, 0.07260026782751083, -0.059606291353702545, 0.1376785933971405, -0.07283104956150055, 0.042437463998794556, 0.07138935476541519, 0.14353327453136444, 0.16994021832942963, 0.3147391378879547, 0.005593098700046539, -0.0792541354894638, 0.024077866226434708, 0.25470083951950073, -0.0994625985622406, -0.1877320259809494, -0.10011966526508331, -0.022319398820400238, 0.073879174888134, 0.17054897546768188, 0.09382303804159164, 0.28510910272598267, 0.06253117322921753, -0.04960837960243225, -0.08652985095977783, 0.024209409952163696, 0.01717410236597061, -0.10327029228210449, 0.06348155438899994, -0.18756525218486786, -0.0856851190328598, 0.16311925649642944, -0.28332462906837463, 0.04950881004333496, 0.25383296608924866, -0.027943043038249016, 0.09826154261827469, -0.037004031240940094, 0.21284228563308716, 0.11574717611074448, -0.09031400829553604, 0.05747067928314209, -0.03102913685142994, -0.005267789121717215, 0.02418111450970173, -0.038318004459142685, -0.24657894670963287, 0.19392219185829163, -0.03155187889933586, -0.0415152981877327, 0.17031897604465485, 0.13563337922096252}, {0.154473677277565, -0.1750091314315796, 0.1493767499923706, -0.17457622289657593, -0.022222040221095085, -0.11118236184120178, -0.09787765890359879, 0.22925646603107452, 0.141804501414299, 0.09243310987949371, 0.20498082041740417, -0.13058523833751678, -0.09592662751674652, -0.1179695799946785, -0.019136318936944008, 0.04945208504796028, 0.034592751413583755, -0.06598365306854248, -0.18703576922416687, 0.11574621498584747, 0.1929144561290741, -0.009367471560835838, -0.010990613140165806, -0.2998385727405548, 0.1526590883731842, 0.07395603507757187, -0.024985289201140404, -0.08549574017524719, 0.16116683185100555, 0.2250685691833496, 0.07870849967002869, -0.05461595207452774, -0.061136722564697266, 0.009843943640589714, -0.06865701824426651, -0.11117259413003922, -0.005564443301409483, 0.23268797993659973, -0.0010993772884830832, -0.1398535966873169, -0.0411582812666893, 0.12718181312084198, -0.031037896871566772, -0.08494381606578827, 0.029628673568367958, 0.27961936593055725, 0.009648296050727367, 0.1011316329240799, -0.007198940496891737, 0.057248931378126144, 0.2067338526248932, 0.01747238077223301, -0.11478692293167114, -0.14962507784366608, 0.06268328428268433, -0.011932967230677605, -0.277317076921463, -0.13036374747753143, 0.014789758250117302, 0.03597739711403847, -0.044558364897966385, 0.15483219921588898, -0.21664199233055115, -0.21995794773101807, 0.034489862620830536, -0.0447140708565712, 0.001155095174908638, -0.05191929265856743, 0.027937449514865875, 0.02325003780424595, 0.06021779403090477, 0.20290449261665344, 0.1094588115811348, -0.10509069263935089, -0.37653955817222595, 0.07550650089979172, 0.1826467663049698, -0.1532028317451477, -0.12959565222263336, 0.27644404768943787, -0.183321014046669, -0.007454576902091503, -0.14916297793388367, -0.014627289026975632, 0.08788785338401794, 0.06872720271348953, -0.032622452825307846, 0.18137015402317047, 0.1867968887090683, 0.048582956194877625, 0.02345995232462883, -0.22438576817512512, 0.1416054219007492, 0.1231086254119873, -0.15265044569969177, -0.06077878177165985, 0.14119300246238708, 0.045634523034095764, 0.05482171103358269, -0.1556236743927002, 0.05562353879213333, -0.13948209583759308, 0.19240471720695496, -0.05452677235007286, -0.05952516198158264, -0.37490642070770264, 0.013437153771519661, 0.011535516940057278, -0.09036199003458023, -0.16686750948429108, -0.008396818302571774, 0.11677474528551102, -0.06507711112499237, 0.011978544294834137, -0.0511033833026886, 0.002744129626080394, -0.06105111166834831, -0.006922160275280476, -0.016011917963624, -0.042844101786613464, 0.008503307588398457, 0.08043486624956131, 0.21448451280593872, 0.10466133803129196, -0.12371637672185898, 0.09611192345619202, -0.2370768040418625, 0.06500690430402756, -0.08523084968328476, -0.037288714200258255, 0.05511082708835602, -0.02890944294631481, -0.18999254703521729, -0.027405284345149994, 0.09283937513828278, -0.0034737559035420418, 0.07679886370897293, -0.009162899106740952, -0.06315070390701294, 0.14648985862731934, 0.021122924983501434, -0.013727088458836079, -0.14818303287029266, -0.08018921315670013, -0.19791868329048157, -0.0013591747265309095, 0.02142414264380932, -0.15260516107082367, 0.2056179642677307, 0.1290513128042221, -0.16673476994037628, 0.06371527165174484, -0.04542214795947075, -0.08107040822505951, 0.17526914179325104, 0.08598285913467407, -0.02869359776377678, 0.0037049828097224236, -0.21212570369243622, -0.09579454362392426, 0.043805576860904694, -0.201040580868721, -0.02260269969701767, 0.28367990255355835, -0.030157070606946945, -0.0024541784077882767, -0.08391301333904266, -0.11788760870695114, -0.060455817729234695, 0.09866854548454285, -0.09564290195703506, 0.1419522613286972, 0.1476203352212906, -0.1565832942724228, -0.15952466428279877, 0.013699174858629704, 0.143580362200737, 0.26249930262565613, -0.0960489884018898, -0.15402384102344513, -0.1613696962594986, -0.18269944190979004, 0.05378203094005585, -0.24395261704921722, -0.016795773059129715, -0.1292995661497116, 0.10619117319583893, 0.014440513215959072, -0.017589615657925606, -0.16937564313411713, 0.1512109935283661, 0.2755722403526306, -0.07032468169927597, 0.06519442051649094, -0.0320422537624836, 0.027985652908682823, 0.0017055082134902477, -0.0897635817527771, 0.02725747413933277, -0.05715280398726463, -0.02377457730472088, 4.3152958824066445e-05, 0.16145531833171844, -0.13047298789024353, -0.05398624390363693, 0.06282953172922134, 0.03165217861533165, 0.008470257744193077, 0.06194223836064339, 0.08288539201021194, -0.14654134213924408, 0.21311041712760925, 0.006986276712268591, 0.10262599587440491, 0.11186500638723373, -0.039078593254089355, 0.003773526521399617, -0.032811954617500305, 0.18258218467235565, -0.05346432700753212, 0.020495548844337463, 0.12982092797756195, 0.23460377752780914, 0.33850908279418945, 0.4315490126609802, 0.27083510160446167, -0.10676731169223785, 0.022233961150050163, -0.03346449136734009, 0.12990696728229523, -0.040773164480924606, -0.0949971079826355, -0.02700560912489891, -0.1361791491508484, -0.052321452647447586, 0.0705445259809494, -0.14334072172641754, 0.14021450281143188, 0.11783689260482788, -0.2789682149887085, 0.06616145372390747, -0.1812228560447693, -0.024817392230033875, 0.14637014269828796, 0.1643867790699005, -0.07158706337213516, -0.012800535187125206, 0.03233766555786133, 0.24724489450454712, 0.05254044756293297, 0.08812180906534195, 0.1341497302055359, -0.1410754919052124, 0.0663428083062172, 0.176308736205101, 0.33941635489463806}, {0.08492014557123184, 0.16593937575817108, 0.344076931476593, 0.07232294231653214, -0.023410359397530556, 0.1505337357521057, -0.13751745223999023, -0.05200846493244171, -0.024260325357317924, 0.1527673453092575, -0.03636539727449417, 0.25363609194755554, 0.22415407001972198, -0.15654976665973663, 0.16149957478046417, -0.09963298588991165, 0.06729607284069061, -0.07035747170448303, 0.2245873510837555, 0.16659443080425262, 0.17386411130428314, -0.02338782325387001, -0.03372791036963463, -0.1762140393257141, -0.1670643836259842, -0.05096810311079025, 0.10963162034749985, -0.11614897102117538, 0.0027428544126451015, -0.027252109721302986, 0.19424429535865784, 0.24480366706848145, -0.1816566288471222, -0.055384472012519836, 0.1012994572520256, 0.22920803725719452, 0.13337139785289764, 0.05386928468942642, 0.18763339519500732, 0.0009256122866645455, -0.030867891386151314, 0.08777961879968643, 0.2096712291240692, 0.201760932803154, 0.005259583238512278, 0.16406360268592834, -0.10121133923530579, 0.09859102964401245, 0.15878386795520782, -0.007735781371593475, 0.01031273789703846, -0.043506380170583725, 0.08337219804525375, -0.294799268245697, -0.11076483130455017, -0.06582041084766388, 0.08301512897014618, 0.04775696620345116, 0.0898422971367836, 0.06923636049032211, -0.12845896184444427, 0.19977165758609772, 0.3406936526298523, 0.07981428503990173, 0.03520479053258896, -0.11489630490541458, 0.04056134819984436, 0.20062507688999176, 0.06950322538614273, 0.08230556547641754, -0.114979587495327, -0.01000679936259985, 0.07881462574005127, 0.2381027489900589, 0.23861680924892426, 0.013663962483406067, -0.05877753719687462, 0.13537734746932983, -0.3346371650695801, -0.16207699477672577, 0.13268625736236572, -0.001508107641711831, -0.11799245327711105, -0.04195291921496391, 0.03219117969274521, 0.07018633186817169, 0.18343289196491241, 0.15381982922554016, 0.07273047417402267, -0.08176949620246887, 0.0377785786986351, 0.22171860933303833, 0.11454164236783981, -0.004599212668836117, -0.0674206018447876, 0.0015201031928882003, 0.053160104900598526, -0.11674007773399353, -0.09564045816659927, -0.07725096493959427, 0.04931003227829933, -0.03861676901578903, -0.12558479607105255, 0.08308685570955276, -0.20354755222797394, -0.12101663649082184, -0.010771316476166248, -0.015203597955405712, -0.3160226047039032, -0.037437740713357925, 0.1499260663986206, -0.0028483769856393337, 0.19568657875061035, -0.45618632435798645, -0.06186724454164505, -0.15940888226032257, 0.14536266028881073, -0.05739913508296013, 0.04339873790740967, -0.077395498752594, -0.18846775591373444, 0.011041630990803242, -0.04166395589709282, 0.22209972143173218, 0.3050896227359772, 0.04216175153851509, -0.127083420753479, -0.044063013046979904, 0.00732337636873126, -0.042309001088142395, -0.00442698085680604, 0.13953126966953278, 0.06591910868883133, -0.10852790623903275, 0.1146775335073471, -0.12675164639949799, 0.022335607558488846, 0.0418810099363327, -0.021578537300229073, -0.12000762671232224, 0.05278933048248291, -0.03091055154800415, 0.04780047759413719, 0.1927165538072586, -0.08941590040922165, 0.060678862035274506, -0.13105328381061554, -0.04076886922121048, -0.3499261736869812, -0.0066265505738556385, 0.24036742746829987, 0.09373956173658371, -0.001354619744233787, -0.029462365433573723, -0.0013595747295767069, 0.1272372156381607, 0.10022470355033875, -0.1473986655473709, 0.03757442161440849, 0.16513946652412415, 0.012038839049637318, 0.007444020360708237, 0.11229536682367325, 0.12403233349323273, 0.12316145747900009, -0.0019029934192076325, -0.05265418440103531, -0.034444570541381836, -0.19987334311008453, -0.18177980184555054, -0.16473639011383057, -0.1518060863018036, 0.025680843740701675, 0.15249809622764587, 0.00847155973315239, 0.020990395918488503, -0.1206107810139656, -0.014604181051254272, 0.022918956354260445, -0.24529732763767242, -0.07134682685136795, 0.4959205090999603, 0.0418163500726223, 0.13507790863513947, 0.11490200459957123, 0.12263603508472443, -0.054143574088811874, 0.06818701326847076, 0.000926923705264926, 0.12580491602420807, -0.09738073498010635, -0.04314499348402023, 0.16240519285202026, 0.1489020138978958, 0.07858669757843018, -0.054551202803850174, 0.011953692883253098, -0.06918202340602875, -0.11110024154186249, -0.06156153976917267, 0.005223971325904131, -0.003343255491927266, 0.07000558823347092, 0.08757225424051285, -0.020105434581637383, 0.19627317786216736, 0.007677966263145208, 0.17003993690013885, -0.007405423559248447, -0.05504065379500389, -0.21930381655693054, -0.18997150659561157, 0.03321089595556259, 0.09388551861047745, -0.04928291589021683, 0.09485672414302826, -0.09189026057720184, 0.08493883907794952, -0.25588229298591614, 0.1513693630695343, -0.07163377106189728, -0.1473783552646637, 0.20786315202713013, -0.25597673654556274, 0.042959462851285934, -0.25357672572135925, -0.07761885225772858, -0.10035982728004456, 0.04927403852343559, -0.04945769160985947, -0.09120158106088638, 0.12919653952121735, -0.25249242782592773, -0.12055712193250656, 0.0683891549706459, -0.10468202084302902, -0.2107871174812317, -0.028748532757163048, 0.026658223941922188, -0.06127336248755455, -0.08568789064884186, 0.14948266744613647, -0.007936582900583744, -0.14054135978221893, -0.06856835633516312, 0.15605038404464722, -0.0003663886454887688, 0.007223871070891619, -0.1302717626094818, 0.13494011759757996, 0.04721057042479515, -0.013729053549468517, 0.07403545826673508, -0.021968046203255653, -0.12104340642690659, -0.04778427630662918}, {0.08099450916051865, -0.16754227876663208, 0.028160959482192993, 0.04598093405365944, -0.11470015347003937, 0.04201526939868927, 0.03811623901128769, 0.04729705676436424, -0.041421566158533096, -0.11568842083215714, -0.1026221439242363, -0.1911325603723526, 0.0354980006814003, -0.07902778685092926, -0.04676719754934311, -0.17112573981285095, 0.16626214981079102, -0.15574321150779724, 0.0016677847597748041, 0.12271124869585037, 0.0815427154302597, 0.030829304829239845, -0.04060029238462448, -0.2533590793609619, 0.026555268093943596, -0.06287501752376556, 0.16953594982624054, 0.12456324696540833, -0.18260081112384796, 0.0017116464441642165, 0.07045772671699524, 0.1742756962776184, -0.0672345906496048, 0.036062292754650116, -0.2224043905735016, -0.17778687179088593, 0.08353699743747711, -0.14257991313934326, 0.23082078993320465, -0.09756463021039963, -0.11959894746541977, 0.13088177144527435, 0.1357542723417282, 0.03728405386209488, 0.0950847938656807, -0.3041788637638092, 0.07505959272384644, -0.09791075438261032, 0.2188156396150589, -0.09860245138406754, 0.09033545106649399, -0.07421840727329254, -0.034505441784858704, -0.03961251676082611, 0.09802708774805069, -0.19256378710269928, 0.18597735464572906, 0.05381401255726814, -0.0053016613237559795, -0.09283091127872467, -0.219376802444458, 0.008190266788005829, -0.17109687626361847, 0.03849872574210167, -0.021410569548606873, -0.0031301581766456366, -0.11681443452835083, -0.017033258453011513, -0.2541889250278473, 0.005592971108853817, -0.08769767731428146, -0.21662096679210663, -0.22937336564064026, 0.26792407035827637, 0.2145487368106842, 0.0523039735853672, 0.11807094514369965, 0.5764367580413818, -0.23470669984817505, 0.26202598214149475, -0.01852116733789444, -0.02504139579832554, 0.16821198165416718, 0.06837157160043716, 0.11527185142040253, 0.10292783379554749, -0.07086699455976486, 0.03482925519347191, 0.10543157160282135, -0.054124996066093445, -0.0872635766863823, 0.20618607103824615, 0.04257526621222496, -0.11023581027984619, -0.11716583371162415, 0.05160880088806152, -0.012709631584584713, -0.0685523971915245, -0.18466225266456604, -0.28006207942962646, 0.11615067720413208, 0.07987476885318756, 0.36951038241386414, -0.025905277580022812, -0.30045944452285767, 0.087622731924057, -0.27953866124153137, 0.03552243486046791, -0.24772608280181885, 0.01145834382623434, 0.03398440033197403, -0.1754850298166275, 0.10929253697395325, 0.05228574201464653, 0.08511895686388016, 0.0657850131392479, -0.02359936572611332, -0.06534937024116516, -0.11763498187065125, 0.218339741230011, -0.1280078887939453, -0.07536324113607407, 0.10107934474945068, -0.09236978739500046, 0.18046872317790985, 0.04159116744995117, -0.13232874870300293, 0.08150416612625122, -0.14176879823207855, 0.16584861278533936, -0.2142486274242401, -0.17864194512367249, 0.13724401593208313, -0.08134312927722931, -0.2007865160703659, 0.14713409543037415, -0.06660299748182297, 0.08452404290437698, 0.22884465754032135, 0.14287042617797852, 0.33712974190711975, 0.18984931707382202, 0.04325316473841667, 0.09902579337358475, -0.031845446676015854, 0.06753953546285629, -0.08137769252061844, 0.09758809208869934, -0.07502566277980804, -0.07511302828788757, -0.1964281052350998, 0.376547247171402, -0.07957647740840912, -0.1360437273979187, -0.029917964711785316, -0.13085326552391052, 0.04826493188738823, -0.42140889167785645, -0.01715245097875595, -0.0361148901283741, 0.2035800665616989, -0.011426087468862534, -0.03259532153606415, 0.09022728353738785, 0.2739304006099701, 0.2654024660587311, -0.055550020188093185, 0.23382419347763062, 0.04178627207875252, 0.1638716757297516, -0.12569643557071686, -0.015576754696667194, 0.020247459411621094, -0.05111172795295715, -0.1265084594488144, 0.03676214814186096, 0.009833592921495438, 0.02452208660542965, -0.1781938374042511, -0.13382162153720856, -0.11174384504556656, -0.41371914744377136, 0.030635105445981026, -0.018999136984348297, 0.1312505155801773, -0.24166305363178253, 0.019213153049349785, 0.18397186696529388, -0.08984696120023727, -0.05882960557937622, 0.211542010307312, 0.08634286373853683, 0.18431763350963593, -0.25547289848327637, 0.02971230261027813, 0.16641369462013245, -0.06366683542728424, -0.2602762281894684, -0.03366265445947647, 0.24071456491947174, -0.009983527474105358, -0.12133447080850601, 0.03436325117945671, -0.07381061464548111, -0.1826968491077423, 0.12543153762817383, 0.20860671997070312, -0.2515811324119568, -0.05607345327734947, 0.022008489817380905, -0.18797945976257324, -0.07667350769042969, -0.08259167522192001, 0.05654977262020111, -0.07314105331897736, -0.06256470829248428, 0.014722042717039585, -0.07125846296548843, -0.02283094823360443, -0.15024049580097198, -0.08842837065458298, -0.08529888838529587, 0.009554949589073658, -0.008475498296320438, 0.048468295484781265, 0.049814824014902115, 0.046995531767606735, -0.17705363035202026, 0.08714853972196579, 0.24758055806159973, 0.12636050581932068, -0.09562025219202042, 0.07872255146503448, -0.26360324025154114, -0.21722345054149628, -0.04980655387043953, -0.3178999423980713, 0.14127947390079498, 0.25675615668296814, -0.08926840871572495, 0.3048792779445648, 0.030559541657567024, 0.010764180682599545, -0.12417937070131302, -0.03798636421561241, -0.25105682015419006, -0.09350913763046265, 0.06426074355840683, 0.022209424525499344, 0.016103027388453484, -0.11519844084978104, 0.08426067978143692, 0.11126016080379486, 0.12260394543409348, -0.11267370730638504, -0.1641719937324524}, {-0.09776978939771652, 0.13930389285087585, -0.12870486080646515, -0.06428064405918121, -0.2742803990840912, 0.1159759908914566, -0.04166041314601898, 0.00735263479873538, -0.14149300754070282, 0.07758006453514099, 0.06082788482308388, -0.5804994106292725, -0.11728490889072418, -0.08865758031606674, 0.01721271127462387, -0.030074186623096466, -0.1904040426015854, -0.008305958472192287, -0.21663667261600494, -0.011976300738751888, -0.38162362575531006, -0.07035820186138153, -0.01245499774813652, -0.042858611792325974, 0.38110581040382385, -0.05323273688554764, -0.040163304656744, 0.05968550592660904, 0.012750389985740185, 0.10822460055351257, -0.02128419652581215, 0.08265144377946854, 0.14462855458259583, -0.15641316771507263, 0.1425027698278427, 0.10379552841186523, 0.04013066366314888, 0.17217937111854553, 0.14024539291858673, -0.10142431408166885, -0.09161113947629929, -0.03276039659976959, -0.033330123871564865, 0.16403815150260925, 0.04546778276562691, -0.23441165685653687, 0.15008753538131714, 0.18856893479824066, -0.08428367972373962, -0.1482190489768982, -0.25478681921958923, 0.15545029938220978, -0.18023183941841125, 0.031572986394166946, 0.04386484622955322, 0.017276177182793617, -0.00544548686593771, -0.17202995717525482, 0.04605241119861603, 0.02778395637869835, 0.03845162317156792, 0.1608189195394516, -0.028522832319140434, 0.13975486159324646, -0.16881801187992096, -0.02087877131998539, -0.03637392446398735, -0.20466333627700806, 0.07571471482515335, 0.09678126871585846, -0.213194340467453, 0.23726819455623627, -0.3082793951034546, -0.0848214402794838, 0.3279343247413635, -0.08424482494592667, 0.08268283307552338, 0.03232540190219879, 0.02379651926457882, 0.06615858525037766, 0.14454205334186554, 0.010349716059863567, 0.17031331360340118, 0.01424009632319212, 0.014417598024010658, 0.10347553342580795, 0.022387897595763206, -0.029096268117427826, -0.0158421378582716, -0.04877523332834244, 0.06930225342512131, 0.046259891241788864, -0.24112148582935333, -0.05877438187599182, -0.09622196853160858, -0.050138477236032486, -0.011607391759753227, -0.07024964690208435, 0.07261147350072861, -0.11608931422233582, -0.19062307476997375, -0.11633086204528809, 0.2371167093515396, -0.0182594433426857, -0.21557441353797913, 0.3168261647224426, 0.06308648735284805, 0.15012793242931366, 0.16345222294330597, -0.06447664648294449, -0.13025936484336853, -0.005908516701310873, 0.0031517765019088984, 0.07475192099809647, 0.1232752725481987, 0.14709503948688507, 0.07204228639602661, 0.13181491196155548, -0.09288027137517929, 0.050084758549928665, 0.09864740073680878, 0.06539343297481537, 0.1383695900440216, 0.1837901622056961, -0.4401315152645111, -0.18731658160686493, 0.08368799090385437, -0.046147141605615616, -0.013824142515659332, -0.1016262024641037, 0.11272177845239639, 0.34332960844039917, 0.10505551099777222, -0.30391860008239746, 0.09778854250907898, 0.49664175510406494, -0.01717630960047245, -0.15395428240299225, 0.13644155859947205, -0.20201173424720764, 0.07016895711421967, -0.03799252212047577, -0.1361037939786911, 0.08192317932844162, 0.026555614545941353, 0.061197828501462936, 0.17996613681316376, -0.13265268504619598, 0.4516622722148895, 0.23304985463619232, -0.15015456080436707, -0.1598976105451584, 0.08184516429901123, 0.03566697612404823, -0.05064773932099342, -0.09737824648618698, 0.12769968807697296, -0.07625783234834671, -0.08349940180778503, 0.012530682608485222, -0.1543741524219513, -0.293930321931839, -0.015319395810365677, -0.21948495507240295, -0.16192187368869781, 0.06534340232610703, 0.14509552717208862, 0.11128958314657211, 0.23571379482746124, 0.0324813649058342, -0.09475359320640564, 0.02991611137986183, 0.0041411155834794044, 0.1531255841255188, -0.07336489856243134, -0.14678767323493958, 0.10200448334217072, 0.00048707862151786685, 0.005998099688440561, 0.04972340539097786, -0.06158095970749855, -0.556369423866272, 0.07010049372911453, 0.1314583420753479, -0.14414943754673004, -0.10537503659725189, 0.018316365778446198, -0.03304773196578026, 0.08377071470022202, 0.06831001490354538, 0.20802141726016998, -0.017365949228405952, 0.2326444536447525, -0.06257883459329605, -0.1451922208070755, -0.019333945587277412, -0.1731652468442917, -0.022294746711850166, -0.08932477235794067, -0.05615624040365219, 0.11726085841655731, 0.032687246799468994, 0.018139371648430824, -0.21306435763835907, 0.12478889524936676, -0.08515620976686478, 0.1741599291563034, -0.0006742585101164877, 0.07837235927581787, -0.07126094400882721, -0.22087444365024567, -0.19037100672721863, -0.12800297141075134, 0.02331225760281086, 0.12081009894609451, -0.13182927668094635, 0.058085840195417404, -0.14143913984298706, 0.3315597474575043, -0.00785176269710064, 0.03954790532588959, -0.08611280471086502, 0.01742836833000183, -0.1802407205104828, -0.172902911901474, -0.1905202865600586, 0.09185554832220078, -0.015513689257204533, -0.15336036682128906, 0.03581392019987106, -0.010523711331188679, -0.10835403203964233, -0.15156684815883636, 0.12822562456130981, -0.032285261899232864, 0.03208876773715019, 0.022540418431162834, 0.05346039682626724, 0.002698509953916073, -0.07227711379528046, -0.0831100270152092, 0.05021360144019127, -0.043985944241285324, -0.11237048357725143, 0.23311026394367218, -0.021103667095303535, 0.041777871549129486, -0.034957945346832275, -0.07113958150148392, -0.11055140942335129, -0.09908776730298996, -0.03826703876256943, -0.07477924972772598, -0.05771522969007492, 0.03108992613852024, -0.13248737156391144}, {-0.11084248125553131, 0.26327672600746155, 0.008111171424388885, -0.05719617381691933, 0.4152507483959198, 0.12891249358654022, 0.013089231215417385, 0.169270321726799, 0.053265057504177094, 0.1122140884399414, -0.07889272272586823, -0.012945986352860928, 0.07607334107160568, -0.0064955162815749645, 0.05664098262786865, -0.12971991300582886, 0.13093511760234833, -0.13196857273578644, -0.10832187533378601, -0.055155713111162186, 0.1966084986925125, -0.1033930703997612, 0.05106977000832558, 0.34251266717910767, 0.18402785062789917, -0.08135990798473358, 0.024031566455960274, -0.0803917944431305, -0.13779602944850922, -0.046107564121484756, -0.03660382702946663, -0.27559220790863037, 0.01768116094172001, -0.08005576580762863, 0.011810173280537128, -0.0015311248134821653, -0.1693822741508484, -0.16454361379146576, 0.06488368660211563, 0.09279686212539673, -0.01802315190434456, -0.055953238159418106, -0.07789892703294754, -0.09558342397212982, -0.1601344347000122, -0.04845055192708969, 0.1941642016172409, 0.08013991266489029, 0.17495912313461304, 0.028286617249250412, 0.23461097478866577, -0.017860975116491318, 0.03970606252551079, 0.18506518006324768, -0.0033446012530475855, -0.013286775909364223, 0.17996381223201752, 0.23765699565410614, 0.24880000948905945, 0.01008359994739294, -0.11657250672578812, 0.0852343961596489, -0.09104401618242264, -0.30407097935676575, 0.1970391422510147, 0.35495826601982117, -0.008787994273006916, 0.04205838218331337, 0.1178489699959755, -0.16664281487464905, 0.01465067733079195, 0.11396273225545883, 0.048282090574502945, 0.023810459300875664, -0.22699719667434692, -0.1311260312795639, -0.029415443539619446, 0.02392580546438694, 0.020375968888401985, 0.13788677752017975, 0.02069755084812641, -0.19612206518650055, 0.013379393145442009, -0.15297198295593262, 0.08267252892255783, -0.004951820708811283, -0.12737952172756195, -0.08500508964061737, 0.08124593645334244, -0.06528978794813156, 0.009476623497903347, 0.07469844818115234, -0.017226627096533775, 0.059188224375247955, 0.06069405749440193, -0.05728849768638611, 0.11117143929004669, 0.07173756510019302, -0.19233182072639465, 0.06917186826467514, -0.01459585502743721, 0.23434695601463318, 0.015337098389863968, -0.042065802961587906, 0.02748621068894863, 0.11746790260076523, 0.10577520728111267, 0.009246417321264744, 0.10752265900373459, -0.04373801499605179, 0.11683639883995056, 0.037908535450696945, -0.11954821646213531, -0.11119677126407623, -0.12686778604984283, -0.09649281948804855, 0.03917244076728821, -0.22364911437034607, 0.18379215896129608, -0.07105496525764465, 0.0625026747584343, -0.1271759569644928, 0.13502733409404755, -0.1093326285481453, -0.06204180046916008, 0.021835828199982643, 0.06296394765377045, -0.1056017279624939, -0.01621408201754093, 0.09736102819442749, 0.057483330368995667, -0.035960204899311066, 0.02913639135658741, 0.11289837211370468, -0.28152093291282654, 0.0005714296130463481, 0.1803148239850998, 0.06658864766359329, 0.10662906616926193, -0.061965737491846085, 0.07320695370435715, -0.012073024176061153, 0.05389096960425377, -0.023820601403713226, 0.16900411248207092, 0.09219943732023239, 0.022214729338884354, -0.2346634566783905, 0.07770989835262299, -0.3189890384674072, 0.14977194368839264, 0.01949186809360981, 0.05826235190033913, -0.07001632452011108, 0.11654088646173477, 0.132334902882576, 0.0992918312549591, -0.056823547929525375, 0.21143247187137604, -0.23507654666900635, -0.03721226751804352, 0.06453821808099747, -0.06194676086306572, 0.030231697484850883, 0.2059323936700821, -0.03755762055516243, -0.047685518860816956, -0.06539209187030792, -0.1747530698776245, 0.11694829910993576, -0.019762450829148293, -0.30675938725471497, -0.02616482600569725, -0.10696011781692505, 0.04235834255814552, 0.0619099922478199, 0.08420863002538681, 0.11050154268741608, -0.1853201538324356, -0.013580946251749992, -0.026435792446136475, -0.00079008360626176, -0.04964945837855339, 0.07627537101507187, -0.16973452270030975, -0.21951062977313995, -0.33520033955574036, -0.09801805019378662, -0.11157204955816269, 0.2658391296863556, -0.10564190149307251, -0.0028024683706462383, 0.02775619551539421, 0.07204481959342957, 0.2997545301914215, -0.12534688413143158, 0.03383520245552063, -0.10432347655296326, 0.08699817955493927, 0.09154067933559418, 0.1489322930574417, 0.016980493441224098, -0.07624763995409012, 0.07090963423252106, -0.048946481198072433, -0.07202595472335815, -0.0235977191478014, 0.09154073894023895, 0.002996238647028804, 0.3281501233577728, -0.003462752792984247, -0.17154167592525482, 0.047565851360559464, 0.004832661245018244, -0.22665493190288544, 0.030500667169690132, 0.09708227962255478, -0.0682535469532013, 0.2659603953361511, -0.12448316812515259, -0.14187021553516388, -0.0025581582449376583, 0.008129103109240532, 0.09365259110927582, -0.05539153143763542, 0.05088407173752785, -0.17063945531845093, 0.07663842290639877, -0.03636870160698891, 0.07906197756528854, -0.07528217136859894, 0.012448357418179512, 0.10872048139572144, -0.20546701550483704, 0.10952243208885193, -0.11729993671178818, 0.06914238631725311, 0.03030257485806942, 0.04273254796862602, -0.024687252938747406, 0.09031427651643753, 0.057984236627817154, 0.036762624979019165, 0.02212463691830635, 0.01386752724647522, 0.17935878038406372, 0.07603698968887329, -0.012588062323629856, -0.02339131012558937, -0.15114907920360565, 0.04371405765414238, 0.07199110090732574, -0.09455206990242004, -0.03687562420964241, -0.11171220242977142, -0.03869301825761795}, {0.10964156687259674, 0.1572411060333252, 0.158089280128479, -0.1621941328048706, 0.06800894439220428, -0.02456645853817463, -0.2092965692281723, -0.06479746103286743, 0.08735696971416473, 0.4669836461544037, 0.06386791169643402, 0.031678538769483566, -0.14533424377441406, 0.006926799193024635, -0.06515181809663773, 0.12460725754499435, -0.029200589284300804, -0.16057458519935608, -0.254720002412796, -0.03963065519928932, -0.13877062499523163, 0.03558075428009033, 0.15008841454982758, -0.02326250821352005, 0.17309924960136414, 0.09905681759119034, 0.04971488192677498, -0.11652793735265732, 0.22797973453998566, 0.21544037759304047, -0.07509902119636536, 0.10918246954679489, -0.14037136733531952, -0.05255500599741936, 0.0723521038889885, 0.21413080394268036, -0.09439995139837265, 0.11047246307134628, 0.05975361168384552, 0.020866557955741882, -0.018141545355319977, 0.22713080048561096, -0.18827761709690094, -0.06610630452632904, 0.13050618767738342, 0.17317990958690643, -0.19943833351135254, 0.18817344307899475, -0.014317885972559452, 0.39910417795181274, -0.16875137388706207, 0.15679611265659332, -0.10927386581897736, -0.05334532633423805, -0.012885627336800098, 0.0689067468047142, 0.04937581717967987, -0.1245068684220314, 0.20692914724349976, -0.015728510916233063, 0.02332398295402527, -0.029903404414653778, -0.09342558681964874, -0.21898505091667175, -0.1599372923374176, -0.1017734706401825, 0.2673899829387665, 0.14277943968772888, 0.17595595121383667, 0.12548042833805084, 0.06464945524930954, 0.1444232165813446, -0.13051877915859222, -0.06696207076311111, 0.1697293519973755, -0.1905425637960434, 0.10741251707077026, 0.2304597944021225, -0.0324556939303875, 0.28075093030929565, -0.03618938475847244, 0.0065374793484807014, -0.005879909731447697, 0.014469500631093979, -0.02694539912045002, -0.12741324305534363, -0.18636946380138397, -0.1114516407251358, -0.07083947211503983, 0.12246747314929962, -0.08715636283159256, -0.2298050820827484, 0.21736687421798706, -0.0008012657053768635, 0.016979172825813293, 0.0607951320707798, 0.2196943312883377, 0.1741083860397339, -0.06210247799754143, -0.03917231410741806, 0.021290002390742302, 0.2110903412103653, 0.2716134786605835, 0.007572612725198269, -0.07485880702733994, -0.012559961527585983, -0.07453103363513947, 0.2502105236053467, -0.23534689843654633, 0.10009735077619553, -0.2184358388185501, -0.1403755247592926, -0.19600984454154968, -0.07263601571321487, 0.1606055349111557, 0.09719309955835342, 0.16870065033435822, -0.04789334535598755, -0.011885746382176876, -0.1074635311961174, -0.008515073917806149, -0.17196397483348846, 0.09527016431093216, 0.10764019936323166, -0.18493421375751495, 0.20274102687835693, 0.14410218596458435, 0.2064943164587021, -0.0871443822979927, 0.06377895176410675, 0.01967449113726616, -0.056553907692432404, -0.10385879874229431, 0.0015946229686960578, -0.05186865106225014, -0.13862483203411102, 0.11167017370462418, 0.10284636914730072, 0.01091453991830349, 0.25834259390830994, 0.12001104652881622, -0.054349079728126526, 0.01355727668851614, 0.03953484818339348, -0.01080779917538166, -0.1300487518310547, 0.06078983098268509, 0.06795109063386917, 0.09518159180879593, 0.08453153073787689, 0.18974433839321136, -0.014940976165235043, -0.1253591924905777, -0.024372588843107224, 0.06311795115470886, -0.0786660835146904, 0.1071806252002716, 0.020130787044763565, -0.0440097376704216, -0.030811576172709465, 0.21691548824310303, 0.12213451415300369, -0.29807642102241516, -0.3034482002258301, -0.2665039300918579, -0.062012434005737305, 0.14572963118553162, 0.11230181902647018, -0.06305507570505142, 0.018637970089912415, -0.22901412844657898, -0.0513731949031353, -0.03502989187836647, 0.21708671748638153, -0.08436509966850281, -0.20221857726573944, -0.13291090726852417, 0.19948209822177887, 0.048334673047065735, -0.048450954258441925, -0.060428187251091, -0.5558063387870789, -0.3144446611404419, -0.1808433085680008, 0.24685029685497284, -0.09313007444143295, 0.4276329278945923, -0.10277079790830612, -0.026397069916129112, 0.0004706840845756233, 0.053184814751148224, -0.1746746152639389, -0.0926152840256691, 0.0982276052236557, -0.042530253529548645, 0.03389260545372963, 0.007999707944691181, 0.24365006387233734, 0.051217205822467804, 0.009707842022180557, -0.16705213487148285, -0.11240256577730179, -0.07341655343770981, -0.07486382126808167, -0.034478336572647095, 0.17923301458358765, -0.011708477512001991, -0.15172185003757477, -0.1719641089439392, 0.04069611057639122, -0.16323186457157135, 0.13960318267345428, -0.009042929857969284, 0.019721001386642456, -0.026502329856157303, 0.1465074121952057, 0.04018344357609749, 0.01869257166981697, 0.2057543396949768, 0.16375073790550232, -0.08085809648036957, -0.05764385312795639, 0.1141633689403534, 0.19630292057991028, 0.02393312007188797, 0.2648470401763916, -0.20789973437786102, 0.12520210444927216, -0.12940077483654022, -0.12426160275936127, 0.019126582890748978, -0.015209256671369076, 0.03526374325156212, -0.09282802790403366, -0.22781087458133698, 0.1636309176683426, -0.18547631800174713, -0.08695128560066223, 0.01750042848289013, 0.1031842827796936, 0.1219729632139206, 0.05683166906237602, 0.27679187059402466, 0.06917794793844223, 0.1225551962852478, -0.11062590777873993, 0.055757783353328705, 0.04872749000787735, 0.18830348551273346, -0.10446939617395401, -0.03301668539643288, -0.001472772448323667, 0.101035937666893, -0.09992541372776031, 0.13327664136886597, 0.02122241072356701}, {-0.05041449889540672, -0.12017933279275894, -0.009931408800184727, -0.010246229358017445, 0.003170845564454794, -0.07622173428535461, 0.02054947428405285, 0.07515477389097214, -0.05837312713265419, 0.20130135118961334, 0.022971617057919502, -0.09273719042539597, 0.0019897196907550097, 0.0501227006316185, -0.014410218223929405, 0.0917661041021347, -0.19300569593906403, 0.18092043697834015, 0.06441944092512131, 0.18799738585948944, -0.2710755169391632, -0.049418095499277115, -0.002132757566869259, -0.000131211694679223, -0.0674581229686737, -0.06850478798151016, 0.15236905217170715, 0.3350711762905121, -0.0850764736533165, -0.0036380821838974953, -0.04105265066027641, 0.14545971155166626, 0.041423995047807693, -0.04111628234386444, 0.044791337102651596, -0.15997986495494843, -0.003657007124274969, 0.12806572020053864, -0.0017017192440107465, -0.14959140121936798, 0.13639351725578308, -0.16684696078300476, -0.31931617856025696, -0.15202265977859497, 0.06386670470237732, -0.2958303689956665, -0.09324417263269424, 0.056324105709791183, -0.01269302237778902, -0.0253001619130373, -0.23082050681114197, 0.04180449619889259, -0.16614167392253876, -0.029194984585046768, -0.03298370540142059, 0.24770742654800415, 0.005219775717705488, -0.039936330169439316, 0.007836375385522842, -0.25271865725517273, 0.004928893409669399, -0.15126322209835052, -0.04897942766547203, -0.012827431783080101, 0.10440134257078171, -0.05894153192639351, 0.049703340977430344, -0.14963963627815247, 0.15537364780902863, 0.11117170006036758, 0.16655747592449188, -0.13830916583538055, 0.08613722771406174, -0.029931850731372833, -0.046393029391765594, 0.04369768500328064, -0.06690913438796997, -0.08211002498865128, 0.03743681684136391, -0.1867506355047226, -0.1704261600971222, 0.1369658261537552, -0.029970085248351097, 0.11790760606527328, -0.1041477844119072, -0.2078915387392044, -0.14600110054016113, -0.3081793189048767, 0.024248410016298294, -0.011758372187614441, 0.04146129637956619, -0.20284713804721832, -0.04153454303741455, 0.07665279507637024, 0.24393679201602936, 0.02158946916460991, 0.01575595885515213, -0.07713916897773743, -0.040323905646800995, 0.02782100811600685, -0.01491642091423273, -0.034713488072156906, -0.06504533439874649, 0.004847818519920111, -0.0011631989618763328, 0.0584041103720665, 0.11017729341983795, 0.17254792153835297, 0.3691766560077667, 0.06148279458284378, 0.09919798374176025, 0.09581982344388962, -0.023823227733373642, 0.19161196053028107, -0.058171119540929794, 0.059713009744882584, 0.08658711612224579, 0.07794554531574249, 0.09203419834375381, 0.0074866279028356075, 0.12362360209226608, -0.07033081352710724, -0.21496038138866425, -0.18438200652599335, 0.04717942699790001, 0.1296873390674591, 0.1656070202589035, -0.041262492537498474, 0.07258294522762299, -0.006136184558272362, 0.06148968264460564, 0.11381419003009796, -0.10380919277667999, -0.07320351153612137, 0.062215857207775116, -0.12725573778152466, -0.15738500654697418, -0.0052428096532821655, 0.007336139213293791, 0.029644040390849113, -0.11727266013622284, 0.03354138135910034, -0.19056910276412964, -0.19863910973072052, -0.10929670929908752, 0.12285805493593216, 0.2584558427333832, 0.07166719436645508, -0.05716712772846222, 0.30930450558662415, 0.0004099496582057327, 0.08058413863182068, 0.004590285010635853, 0.0298711396753788, 0.021577145904302597, 0.06984853744506836, -0.1250527799129486, 0.07312379032373428, 0.20415519177913666, 0.07880253344774246, -0.11042773723602295, 0.21207426488399506, 0.04849473387002945, -0.29568636417388916, 0.04197150841355324, -0.20517589151859283, 0.04176861792802811, 0.15309447050094604, -0.004731129854917526, 0.010942436754703522, 0.26246243715286255, 0.18158258497714996, -0.05535068362951279, -0.06349248439073563, -0.20509637892246246, 0.016914110630750656, -0.010351065546274185, 0.03358934074640274, -0.07473974674940109, 0.02785087376832962, -0.044231969863176346, -0.11181308329105377, -0.15362520515918732, -0.09590919315814972, 0.02041647955775261, 0.08004752546548843, 0.0607401467859745, -0.0951605960726738, 0.02569223940372467, 0.1463228315114975, -0.00488412007689476, 0.03211025893688202, -0.02456667646765709, -0.09652364999055862, -0.34405189752578735, 0.038947463035583496, 0.04325531795620918, 0.13623079657554626, 0.019614357501268387, 0.2014877200126648, -0.06780929118394852, 0.2519095838069916, -0.07269135117530823, -0.07557304948568344, -0.034139204770326614, -0.014419491402804852, -0.09143966436386108, 0.061828650534152985, -0.12960626184940338, -0.0699140876531601, 0.23566369712352753, 0.09110315889120102, 0.04591519013047218, -0.2906727194786072, 0.021788883954286575, -0.05977458506822586, -0.14194801449775696, 0.01943102851510048, -0.15588094294071198, 0.16535799205303192, -0.016229305416345596, -0.0498754158616066, -0.06342347711324692, 0.07664355635643005, 0.06177638843655586, 0.25502654910087585, -0.00998612493276596, 0.14745382964611053, -0.020559919998049736, 0.1626301407814026, 0.022939488291740417, -0.011327029205858707, 0.1979943811893463, 0.04473474621772766, 0.0757375955581665, -0.04961521923542023, 0.05683242157101631, 0.08167973160743713, 0.06244899705052376, -0.03807133436203003, 0.1230815127491951, -0.00940095167607069, -0.5164400339126587, -0.10783309489488602, -0.06175839528441429, -0.0011775812599807978, 0.06790380924940109, 0.0157517958432436, 0.1180829256772995, -0.014316598884761333, 0.18489785492420197, 0.044316064566373825, 0.04265092685818672, -0.12107012420892715, -0.32942426204681396, 0.14266854524612427}, {0.10556141287088394, -0.012356165796518326, -0.031843990087509155, -0.08187044411897659, 0.17555291950702667, -0.0742054358124733, -0.2964724004268646, -0.06788189709186554, -0.16852471232414246, 0.05040610581636429, 0.34014421701431274, -0.01333936769515276, -0.13438090682029724, 0.10970282554626465, 0.3735651671886444, -0.013366050086915493, 0.1099042221903801, 0.05329505726695061, 0.05165591463446617, -0.16221314668655396, 0.19398610293865204, 0.08616572618484497, 0.35996171832084656, 0.017272742465138435, -0.2726837396621704, -0.20559385418891907, 0.2522887885570526, 0.15142694115638733, 0.10923780500888824, 0.10451092571020126, -0.020808544009923935, -0.09554323554039001, -0.115678571164608, 0.05463748052716255, 0.18266387283802032, 0.028717925772070885, -0.0026401642244309187, -0.11595837026834488, 0.09057658165693283, -0.13079263269901276, -0.08222589641809464, 0.15961197018623352, -0.024094153195619583, 0.24484892189502716, -0.03837059065699577, -0.05492398887872696, -0.24800187349319458, -0.3887418806552887, 0.09480931609869003, 0.07019162178039551, -0.04347174987196922, 0.3328285813331604, -0.010244577191770077, -0.15792474150657654, 0.0863836333155632, -0.03309699520468712, 0.048943717032670975, -0.00964763481169939, 0.009728590957820415, -0.4245806038379669, -0.15491607785224915, -0.07881201058626175, -0.018234629184007645, 0.07372448593378067, 0.09006395190954208, -0.008167463354766369, -0.18794149160385132, -0.18624190986156464, -0.13872385025024414, 0.021776216104626656, 0.02953910082578659, 0.3042985796928406, -0.20725536346435547, 0.05249844491481781, 0.14998340606689453, -0.0002532380458433181, -0.041698284447193146, 0.29212695360183716, -0.17288021743297577, -0.11120539158582687, 0.24864870309829712, 0.2483489066362381, 0.059550389647483826, 0.057275351136922836, -0.12819106876850128, -0.2942362427711487, -0.13116690516471863, -0.0044708154164254665, 0.12692371010780334, 0.2149004489183426, 0.16093531250953674, 0.12192723900079727, -0.05794049799442291, 0.05550678074359894, 0.09627224504947662, -0.04523709416389465, 0.10632473230361938, 0.018347308039665222, -0.1496497541666031, 0.07827989757061005, 0.17413823306560516, 0.1130620613694191, 0.11919736862182617, 0.18380199372768402, -0.18845070898532867, 0.6692194938659668, -0.21576829254627228, 0.03764034062623978, 0.16213735938072205, 0.17168687283992767, -0.089332714676857, -0.16247805953025818, -0.16518093645572662, -0.06156317517161369, -0.07712394744157791, -0.08177158981561661, 0.19354358315467834, 0.09212610870599747, 0.07288015633821487, -0.23798520863056183, -0.0369989387691021, -0.23959052562713623, -0.23090563714504242, 0.05073516443371773, -0.0332903154194355, 0.31004956364631653, -0.10150009393692017, -0.1542755514383316, -0.4768231213092804, 0.016347745433449745, -0.05789266154170036, 0.02260429784655571, 0.12989802658557892, 0.07640933990478516, 0.026049593463540077, -0.09812961518764496, 0.07511448860168457, -0.08770915120840073, -0.14271660149097443, 0.08586059510707855, 0.09630745649337769, 0.19793933629989624, -0.04584427550435066, 0.018270224332809448, -0.09439023584127426, -0.010045517235994339, 0.06741157174110413, -0.05413030833005905, -0.04338565468788147, 0.0012608709512278438, 0.16440770030021667, -0.025441113859415054, 0.09943561255931854, -0.30855852365493774, -0.11967623233795166, -0.004441889002919197, 0.010365208610892296, 0.09289643168449402, 0.10393086075782776, -0.28627097606658936, -0.05448951572179794, 0.16078080236911774, -0.013590101152658463, 0.25983110070228577, -0.016678208485245705, 0.19017164409160614, -0.0881367176771164, -0.03738442063331604, 0.13868848979473114, 0.15192455053329468, -0.018696928396821022, 0.11247174441814423, -0.02306014485657215, -0.0017591106006875634, 0.2766322195529938, -0.15621496737003326, 0.04290446639060974, 0.010373474098742008, -0.21568143367767334, 0.008957055397331715, -0.08300452679395676, -0.06639914959669113, -0.20920389890670776, 0.1699787676334381, 0.30203166604042053, 0.0816255658864975, -0.011942919343709946, 0.016997434198856354, -0.0354808084666729, 0.18179677426815033, 0.1315692514181137, -0.19726388156414032, -0.015052364207804203, -0.12056365609169006, -0.035732001066207886, -0.17786426842212677, -0.14977554976940155, 0.13401074707508087, -0.16214655339717865, -0.015585203655064106, 0.013924892991781235, 0.09922277927398682, -0.0519733726978302, -0.01064833253622055, -0.15268896520137787, 0.09873947501182556, 0.536035418510437, -0.21516962349414825, -0.011432675644755363, -0.04231301322579384, -0.08223871886730194, -0.07282483577728271, 0.07514806091785431, -0.30055510997772217, 0.07021888345479965, 0.032940637320280075, -0.10954638570547104, -0.0659918338060379, -0.1895190328359604, 0.23772959411144257, -0.2250434011220932, -0.005094354972243309, -0.17877227067947388, -0.12436054646968842, 0.013447252102196217, -0.10269946604967117, 0.03240566328167915, 0.057282526046037674, 0.12740793824195862, 0.002519805682823062, 0.1420920044183731, -0.1832824945449829, -0.042565327137708664, -0.25821006298065186, -0.214193195104599, 0.047750312834978104, -0.07512250542640686, -0.0008126964094117284, 0.0779053196310997, 0.09184088557958603, 0.3101315200328827, 0.03065192699432373, -0.20014864206314087, 0.10446248203516006, 0.17183050513267517, 0.0008686354267410934, 0.15701721608638763, 0.02123718336224556, -0.12039730697870255, 0.039171889424324036, 0.1405526101589203, -0.035073425620794296, 0.1778579205274582, -0.09586822241544724, -0.15377357602119446, 0.005991579033434391}, {-0.006342229899019003, -0.09863381832838058, -0.06963472813367844, -0.13059285283088684, -0.10896215587854385, 0.1137339398264885, 0.07731463015079498, 0.061598580330610275, 0.10908716917037964, -0.08042983710765839, 0.08267351984977722, -0.5814301371574402, 0.01782110705971718, -0.07845963537693024, 0.1217758059501648, -0.05865435674786568, 0.022702869027853012, 0.12056105583906174, -0.024465316906571388, -0.09571044147014618, -0.18590234220027924, -0.1666945368051529, -0.043408770114183426, 0.0440957248210907, 0.16145318746566772, 0.02599242515861988, 0.26236653327941895, -0.02207767777144909, 0.24862396717071533, 0.15199661254882812, 0.049854617565870285, 0.00502390880137682, -0.15175023674964905, -0.022968927398324013, 0.093470998108387, -0.08569731563329697, -0.032496847212314606, 0.164168119430542, 0.08084846287965775, -0.11953084170818329, 0.0016908850520849228, 0.05415751412510872, 0.012800845317542553, -0.08986546844244003, -0.116828553378582, -0.12931738793849945, -0.09362953156232834, -0.08399596065282822, -0.078133225440979, 0.0030410303734242916, -0.30767253041267395, -0.06207018718123436, -0.06969756633043289, -0.1458035260438919, 0.0005354424938559532, -0.08825869113206863, -0.13252779841423035, 0.11485935747623444, 0.10043062269687653, -0.1390971839427948, -0.08020028471946716, 0.28656917810440063, -0.20821180939674377, 0.022751258686184883, -0.0033751283772289753, 0.09733372181653976, -0.04594816640019417, -0.09975931793451309, -0.15484677255153656, 0.07404501736164093, -0.03057771734893322, 0.1053159162402153, -0.02705015055835247, 0.1500551998615265, 0.09450376778841019, 0.02453591115772724, 0.02097024954855442, 0.26804912090301514, 0.1585344672203064, 0.3130342662334442, -0.08152958005666733, 0.030339675024151802, 0.13898728787899017, 0.07322154939174652, -0.07500330358743668, 0.15997514128684998, -0.14022819697856903, 0.029069026932120323, -0.0024249397683888674, 0.12535716593265533, 0.16738222539424896, 0.014530546963214874, 0.24660572409629822, -0.09221775084733963, -0.1447455883026123, 0.03882833197712898, 0.05615601688623428, 0.0024767443537712097, -0.06261978298425674, -0.16785390675067902, -0.069864921271801, 0.10146012157201767, 0.36175233125686646, 0.07670192420482635, 0.02702006697654724, -0.35881906747817993, 0.02731918916106224, 0.11066056042909622, 0.145645409822464, 0.08335162699222565, 0.0683239996433258, -0.015252357348799706, 0.31083205342292786, -0.028078867122530937, 0.1060400903224945, -0.04660467058420181, 0.07206802070140839, 0.10475637018680573, -0.08253965526819229, -0.24622036516666412, 0.03831450641155243, 0.07607600837945938, -0.07762152701616287, 0.13579630851745605, -0.04695364832878113, -0.04125652834773064, 0.10330983251333237, -0.013564511202275753, -0.20757660269737244, 0.09858204424381256, 0.031554099172353745, 0.05831890180706978, 0.00644916296005249, -0.04742167890071869, 0.23893332481384277, 0.04413905367255211, 0.15100935101509094, -0.033471278846263885, -0.04238113760948181, 0.12695859372615814, 0.08155958354473114, 0.03431835398077965, 0.031017912551760674, 0.0482599139213562, -0.07159136980772018, -0.1436440348625183, -0.054531242698431015, -0.01409647986292839, 0.08288075774908066, -0.04475554823875427, 0.002207087818533182, 0.06037939712405205, -0.012267736718058586, -0.07102913409471512, -0.014247791841626167, -0.1276806741952896, 0.0042521958239376545, -0.12965162098407745, -0.1680499166250229, 0.09809298813343048, -0.12367570400238037, 0.046774331480264664, -0.009331950917840004, -0.12446007132530212, 0.18593300879001617, -0.0579434372484684, -0.14008723199367523, 0.06710005551576614, 0.13277296721935272, -0.21786350011825562, 0.05795519798994064, -0.10737824440002441, -0.05498518794775009, -0.13772621750831604, -0.12637114524841309, -0.024860020726919174, -0.036432571709156036, -0.09704074263572693, -0.017359435558319092, -0.15699464082717896, -0.14073443412780762, -0.6209313273429871, -0.003719984320923686, -0.014688234776258469, -0.10770609974861145, -0.11309874057769775, 0.07690470665693283, -0.055763229727745056, -0.04619570076465607, 0.08852595090866089, -0.07496888935565948, 0.08510508388280869, 0.029223758727312088, -0.15305998921394348, -0.14722606539726257, -0.12441239506006241, -0.12283941358327866, -0.03565158322453499, 0.12459716945886612, 0.27070701122283936, 0.16267426311969757, 0.10512877255678177, 0.0694977343082428, -0.037354420870542526, 0.07292163372039795, 0.09165344387292862, 0.009213645942509174, -0.07646196335554123, -0.07667955011129379, 0.0801708996295929, 0.02098258025944233, 0.1938694566488266, 0.07284022867679596, 0.11254160106182098, -0.08957433700561523, -0.048784419894218445, -0.08889041841030121, 0.047081850469112396, 0.06524042040109634, -0.1602492481470108, -0.09601566195487976, -0.012805582955479622, 0.051832396537065506, 0.21880117058753967, -0.04079544544219971, 0.17853066325187683, -0.003968683071434498, 0.01169397309422493, -0.06985410302877426, 0.217545285820961, 0.029400834813714027, -0.06094757094979286, -0.11257641762495041, -0.2549976110458374, 0.10690834373235703, -0.037788279354572296, -0.18394823372364044, -0.007392925210297108, 0.23074494302272797, -0.2390763908624649, 0.05861569195985794, 0.00993200484663248, 0.16640949249267578, -0.06624013930559158, -0.08158024400472641, 0.15596288442611694, 0.12477583438158035, -0.05329456180334091, 0.0010333128739148378, 0.1462618112564087, -0.1029614806175232, 0.1293942928314209, -0.0233570858836174, 0.17273154854774475, -0.03901694715023041, 0.19365201890468597}, {-0.07784256339073181, -0.06912307441234589, -0.04258792847394943, 0.06555941700935364, -0.07412748038768768, 0.14712625741958618, 0.16061782836914062, -0.042581263929605484, 0.005412914324551821, 0.1483740210533142, 0.05980530381202698, -0.1756303459405899, 0.061050642281770706, 0.23802663385868073, 0.25074490904808044, -0.0320875383913517, -0.011556107550859451, 0.16950605809688568, 0.018864238634705544, -0.004214081913232803, -0.2775072455406189, 0.14128729701042175, 0.042273007333278656, 0.12050038576126099, 0.10086831450462341, -0.14761239290237427, -0.04215209558606148, -0.01084993313997984, -0.014380231499671936, 0.06834478676319122, 0.1500524878501892, 0.11825165897607803, 0.00039699909393675625, 0.09654220193624496, -0.12283757328987122, -0.3028334379196167, 0.09740530699491501, 0.21745288372039795, -0.1551390439271927, -0.12224139273166656, -0.0910448431968689, -0.021196018904447556, 0.05347398295998573, 0.20086652040481567, 0.1255495548248291, -0.1266084611415863, 0.009653721004724503, -0.051191024482250214, 0.0034937243908643723, 0.1295270174741745, -0.1949302703142166, -0.13561229407787323, -0.10913579165935516, -0.13460463285446167, 0.024315446615219116, -0.01667320728302002, -0.11661074310541153, -0.053432006388902664, 0.13412751257419586, -0.2542397081851959, 0.10796818137168884, 0.08557040244340897, -0.0023875292390584946, -0.0976540744304657, -0.05997445434331894, 0.17713108658790588, 0.10866006463766098, -0.2633589506149292, -0.2181168496608734, -0.039446037262678146, 0.018613645806908607, 0.017882954329252243, 0.24753648042678833, 0.05018322914838791, 0.10509882122278214, -0.05155589431524277, 0.046024806797504425, 0.010217896662652493, 0.12072183936834335, 0.32673418521881104, -0.05529278889298439, 0.004291370511054993, 0.019346414133906364, 0.25072601437568665, 0.020860327407717705, -0.019485631957650185, -0.005939190275967121, -0.15179266035556793, 0.06502082943916321, -0.10983102768659592, 0.14179109036922455, 0.006042194552719593, 0.023536982014775276, -0.17477275431156158, 0.261123925447464, -0.16454093158245087, 0.004507537465542555, -0.10769198834896088, -0.061330556869506836, 0.3442866802215576, 0.01651838608086109, 0.1702587604522705, 0.006837706547230482, -0.032888252288103104, 0.00837595947086811, 0.10607064515352249, -0.027278898283839226, 0.11804826557636261, 0.046677365899086, -0.02871512994170189, -0.10470998287200928, 0.040940169245004654, -0.02402367815375328, -0.11348027735948563, -0.013795360922813416, 0.008725074119865894, -0.016020214185118675, 0.08406417816877365, 0.012690774165093899, -0.05456165969371796, -0.13145911693572998, -0.0585305392742157, -0.05750221759080887, -0.0504157654941082, -0.04543371498584747, 0.0564555823802948, -0.09015645831823349, 0.004849936347454786, -0.03805287927389145, 0.04472389817237854, 0.04972391575574875, 0.07745394855737686, -0.14129352569580078, 0.02325364761054516, -0.08632490038871765, -0.08357900381088257, 0.03472566977143288, -0.0478663295507431, -0.13330593705177307, 0.04101412743330002, -0.14165933430194855, -0.029628640040755272, 0.08980578929185867, -0.19117996096611023, 0.03024435229599476, 0.029231078922748566, 0.1004309430718422, 0.08330551534891129, 0.03137602657079697, 0.05360561981797218, -0.1438637524843216, -0.03800930827856064, -0.036704275757074356, 0.05977680906653404, -0.08584418147802353, -0.016722293570637703, 0.013241246342658997, -0.011715568602085114, -0.013850830495357513, 0.24302835762500763, -0.07531213760375977, 0.18267573416233063, 0.17051184177398682, -0.10012004524469376, 0.06256946176290512, 0.1246870830655098, 0.13430900871753693, 0.3267676532268524, -0.018349291756749153, -0.08282914757728577, -0.01347391214221716, 0.10406611114740372, -0.05791618674993515, -0.16936761140823364, -0.17065969109535217, -0.12498807907104492, -0.013989276252686977, -0.0610184445977211, -0.011144228279590607, -0.21349918842315674, -0.010488493368029594, -0.2688296139240265, -0.07487249374389648, -0.16901269555091858, -0.18590497970581055, -0.007807200774550438, 0.30200520157814026, 0.00758683355525136, 0.007373035419732332, -0.014004908502101898, 0.1257784515619278, -0.16757594048976898, -0.15944510698318481, -0.2538157105445862, -0.02070796862244606, -0.050414491444826126, 0.13794350624084473, 0.10743346065282822, -0.06376989930868149, 0.14846272766590118, -0.027467554435133934, -0.06692389398813248, -0.05679541826248169, -0.08483247458934784, 0.014394296333193779, 0.06182735413312912, -0.008125790394842625, 0.12067604064941406, -0.1870274543762207, -0.01903170719742775, -0.02707660384476185, 0.14368315041065216, 0.05059053376317024, 0.12859439849853516, 0.022544963285326958, -0.07627203315496445, 0.17256318032741547, 0.13815902173519135, 0.06179317086935043, -0.14522364735603333, 0.03177143260836601, -0.06561413407325745, -0.0063332379795610905, 0.08915381878614426, 0.1563171148300171, 0.3006722033023834, -0.056543637067079544, 0.15141192078590393, 0.03036588989198208, 0.18986289203166962, -0.13945186138153076, 0.11356035619974136, 0.1801021844148636, -0.04493905231356621, -0.017946090549230576, -0.13341011106967926, 0.049528397619724274, -0.16627174615859985, 0.15565913915634155, 0.3286513090133667, 0.06185139715671539, 0.0048122103326022625, 0.026944421231746674, -0.08023173362016678, 0.04059777036309242, 0.13916242122650146, 0.1253460794687271, -0.10140509158372879, -0.015808161348104477, 0.08524113148450851, -0.12613211572170258, -0.11730534583330154, 0.040546439588069916, -0.1723819375038147, -0.05518699064850807, 0.04180052503943443}, {0.05409486964344978, 0.04718130826950073, 0.06988578289747238, 0.27938729524612427, 0.0410415381193161, 0.19781096279621124, 0.0334378220140934, -0.05279003083705902, 0.1120767593383789, 0.16541141271591187, 0.18436498939990997, -0.012724439613521099, 0.3505665063858032, -0.1830669343471527, 0.07580437511205673, 0.20422524213790894, 0.017062518745660782, 0.024238981306552887, -0.18267689645290375, -0.3167697787284851, 0.08596113324165344, 0.0716819241642952, 0.01075380016118288, 0.05689375102519989, 0.04417624697089195, 0.10324927419424057, -0.3041727840900421, 0.07173238694667816, 0.19054313004016876, 0.10377107560634613, -0.1569940447807312, -0.13209141790866852, 0.3070671558380127, -0.1280917227268219, 0.0009434790117666125, 0.1984340399503708, -0.06516371667385101, 0.24341532588005066, -0.01819833740592003, 0.07821869105100632, -0.19785915315151215, 0.06985031813383102, -0.07777990400791168, 0.19636288285255432, 0.0735754743218422, -0.033921290189027786, 0.22231422364711761, 0.036724384874105453, 0.06686241924762726, -0.27092984318733215, -0.032979100942611694, -0.11725901067256927, -0.0376911498606205, 0.015834100544452667, -0.01441071555018425, -0.04785759001970291, -0.020053163170814514, -0.08749803900718689, 0.05660677328705788, 0.08137210458517075, 0.12233537435531616, -0.002493972657248378, 0.27913132309913635, 0.11117476969957352, -0.05406125634908676, 0.025719519704580307, 0.15014199912548065, -0.003454700345173478, -0.06897121667861938, -0.04626522213220596, -0.0035025456454604864, 0.12585462629795074, 0.20791228115558624, -0.06194853037595749, -0.11353731900453568, 0.10823845863342285, -0.03147818148136139, -0.1997075080871582, -0.09195691347122192, -0.1178906261920929, 0.04162823408842087, 0.1743668019771576, 0.10173140466213226, -0.2002314329147339, 0.14294543862342834, 0.2043852061033249, 0.15145276486873627, -0.029381319880485535, -0.08232367783784866, -0.044814903289079666, -0.022817308083176613, -0.1679563820362091, 0.04027712717652321, 0.2285306453704834, 0.011552080512046814, -0.07005397230386734, 0.05089810863137245, -0.09642979502677917, 0.14532053470611572, 0.12069377303123474, 0.09680353850126266, 0.020884886384010315, -0.2117757350206375, 0.06100417301058769, 0.3905010223388672, 0.09302420914173126, -0.20405623316764832, -0.1631004512310028, -0.012134379707276821, -0.008798593655228615, 0.27784863114356995, -0.0707647055387497, -0.09287163615226746, 0.0053565059788525105, -0.22538433969020844, 0.16071583330631256, 0.11401456594467163, -0.059544507414102554, 0.028391551226377487, -0.22311162948608398, 0.01682325266301632, 0.0813208743929863, -0.005888534709811211, -0.2491959035396576, 0.17476831376552582, -0.17014579474925995, 0.12284688651561737, -0.03873268887400627, 0.06918086111545563, -0.11298961192369461, 0.08820686489343643, 0.10068356990814209, -0.09983429312705994, -0.210841104388237, 0.03839787468314171, -0.02212822251021862, -0.03984898701310158, 0.10452929139137268, -0.14330963790416718, 0.1340593546628952, 0.21656997501850128, -0.2536848783493042, 0.17322535812854767, 0.14676906168460846, 0.013313110917806625, 0.11053433269262314, 0.12284049391746521, -0.02299453876912594, 0.05376814305782318, 0.020253216847777367, 0.08584358543157578, -0.13203151524066925, -0.10554474592208862, -0.05101303383708, 0.2768687307834625, -0.021893678233027458, -0.07689189165830612, 0.03595709800720215, 0.008740279823541641, -0.04374665021896362, -0.025183049961924553, 0.19527359306812286, 0.10054421424865723, 0.052542902529239655, 0.145759716629982, -0.052486833184957504, -0.27944907546043396, -0.047597069293260574, -0.21644233167171478, -0.27114054560661316, 0.1650254726409912, 0.16794587671756744, -0.024449016898870468, 0.12250293791294098, -0.07208192348480225, -0.017722999677062035, 0.0794212818145752, 0.055888671427965164, -0.02173003740608692, 0.1748376339673996, -0.042021337896585464, 0.06823679804801941, -0.005360065959393978, -0.0022796588018536568, -0.039247021079063416, 0.08684130758047104, 0.054611966013908386, -0.17773497104644775, 0.03622465208172798, 0.061456531286239624, 0.14835116267204285, -0.11294697225093842, -0.18880362808704376, 0.09114841371774673, 0.24546106159687042, -0.005721319001168013, -0.029378272593021393, -0.08365771174430847, -0.04412681981921196, -0.018629705533385277, -0.07016780227422714, -0.11143702268600464, -0.11106740683317184, -0.062059979885816574, 0.20098936557769775, -0.5273382067680359, 0.28816932439804077, 0.2569501996040344, -0.05736486613750458, 0.07308068126440048, 0.16097140312194824, 0.006627539172768593, -0.0010925406822934747, -0.28384900093078613, -0.06580397486686707, 0.2297387421131134, 0.0841006487607956, 0.10618181526660919, -0.0927814394235611, 0.018089070916175842, -0.22671832144260406, -0.32238268852233887, 0.14675529301166534, 0.00403904402628541, 0.075471892952919, 0.061152663081884384, -0.12845587730407715, 0.07796906679868698, 0.07882410287857056, -0.19775986671447754, 0.07432550936937332, 0.16870102286338806, 0.14260929822921753, 0.03765012323856354, -0.20490631461143494, 0.06563204526901245, 0.135582834482193, 0.05386333912611008, -0.06509989500045776, -0.08951916545629501, -0.24199211597442627, 0.014024036936461926, -0.1465204507112503, 0.12639960646629333, 0.020147761330008507, 0.23563943803310394, 0.09227559715509415, -0.07090513408184052, 0.09174633026123047, 0.2152795046567917, 0.08290568739175797, 0.173475444316864, -0.0578385591506958, 0.024061523377895355, -0.115214042365551, -0.03578752651810646}, {-0.10377286374568939, -0.024607138708233833, -0.1301053911447525, 0.01787632331252098, 0.15142714977264404, 0.21538487076759338, -0.06289849430322647, 0.27008673548698425, -0.10529778897762299, -0.1555868536233902, -0.12318544834852219, -0.17531734704971313, 0.09773585200309753, -0.06557273864746094, -0.2457398772239685, -0.20040880143642426, -0.07122231274843216, 0.1381482630968094, 0.07549569010734558, 0.18891294300556183, 0.1936812847852707, -0.010242088697850704, 0.011168617755174637, 0.0991712436079979, 0.04251113906502724, -0.013357022777199745, -0.01805618591606617, -0.03213442116975784, -0.23572273552417755, -0.12039823085069656, -0.15871867537498474, 0.03543095663189888, -0.09771526604890823, -0.08974920958280563, -0.0675504133105278, -0.030791504308581352, -0.15953993797302246, -0.16942578554153442, -0.26466861367225647, 0.14354485273361206, -0.011079104617238045, 0.071884386241436, 0.25121861696243286, -0.1714148223400116, -0.04584719240665436, -0.08515247702598572, 0.12876366078853607, -0.1515064388513565, -0.29399070143699646, 0.2367553859949112, 0.0624537356197834, -0.017697516828775406, -0.05608078092336655, -0.2293417602777481, 0.04409850016236305, -0.05302420258522034, -0.04726904258131981, -0.04811973124742508, 0.0921832025051117, 0.21798443794250488, 0.011595327407121658, 0.13654008507728577, 0.02464725822210312, -0.09757658839225769, 0.0856984332203865, 0.1954273134469986, -0.08094058185815811, 0.07458998262882233, -0.06328246742486954, -0.0016872452106326818, 0.02197086624801159, -0.15917348861694336, -0.12667608261108398, 0.03154827281832695, -0.15988975763320923, -0.022398248314857483, -0.1721348613500595, -0.08489134162664413, -0.1403157114982605, -0.274941086769104, -0.14010122418403625, -0.015236614271998405, 0.0390445739030838, -0.03304283693432808, 0.08402835577726364, 0.1249210461974144, -0.10500733554363251, 0.12437716871500015, 0.04170829802751541, -0.20293591916561127, 0.06929609179496765, 0.003494508098810911, 0.1878701001405716, -0.08206844329833984, 0.014301572926342487, 0.008405398577451706, 0.17697715759277344, -0.1423264890909195, -0.13816194236278534, -0.0451178140938282, 0.06203295290470123, -0.07813361287117004, 0.1303572654724121, 0.03299867734313011, 0.16693539917469025, -0.16807812452316284, -0.010631537064909935, -0.015483658760786057, 0.08753006905317307, 0.006622479762881994, -0.010437756776809692, -0.02701752446591854, -0.03430200740695, 0.06943878531455994, -0.012870692647993565, -0.1962590515613556, 0.28169548511505127, -0.1950886994600296, -0.0590023435652256, -0.2539263963699341, -0.1342368721961975, 0.005182075314223766, 0.20284150540828705, -0.24829533696174622, -0.12652842700481415, -0.2242620587348938, 0.07532580941915512, -0.21046273410320282, 0.021523455157876015, 0.06674119830131531, 0.10225255787372589, 0.023387905210256577, 0.008272740058600903, -0.055969126522541046, -0.08318663388490677, 0.16227690875530243, 0.1032046303153038, -0.04832566902041435, 0.017577053979039192, 0.21609145402908325, -0.03199449181556702, -0.13320085406303406, 0.048791080713272095, 0.0018951899837702513, -0.09457151591777802, -0.021629827097058296, -0.05557482689619064, 0.03362474963068962, -0.03409688547253609, -0.04826105386018753, 0.11717042326927185, -0.0971711054444313, 0.08326046168804169, -0.11721338331699371, 0.2079801708459854, 0.06331907212734222, 0.1412227749824524, -0.090666763484478, -0.031605832278728485, 0.0577719509601593, 0.017598189413547516, -0.2601267695426941, -0.09803260862827301, 0.12190674245357513, 0.02758490853011608, -0.1877240240573883, -0.0142238549888134, -0.027098629623651505, 0.06677448004484177, -0.07034514099359512, 0.054787829518318176, -0.21244709193706512, 0.019430430606007576, -0.07073019444942474, 0.10657936334609985, 0.08216986060142517, -0.023930523544549942, 0.06711766868829727, -0.18223626911640167, -0.08124345541000366, -0.024225793778896332, 0.024774465709924698, 0.0310297179967165, -0.03375384211540222, -0.0520324669778347, 0.07771964371204376, 0.21186377108097076, -0.03335309773683548, -0.09063461422920227, -0.0348515510559082, -0.1829383671283722, 0.2076631635427475, -0.3009333610534668, -0.0957513302564621, 0.1995706558227539, -0.006825288292020559, 0.16416522860527039, 0.1534266322851181, 0.1499735713005066, -0.1097249984741211, -0.010025227442383766, 0.04684385284781456, 0.004978237673640251, -0.12617520987987518, -0.011274787597358227, -0.011303304694592953, 0.0457300990819931, -0.11222022771835327, 0.2184164971113205, -0.01302780769765377, 0.029592344537377357, 0.3242108225822449, 0.07540340721607208, -0.009918893687427044, -0.2538853883743286, -0.0071528200060129166, -0.10053735971450806, -0.21761663258075714, -0.04174858331680298, -0.12702348828315735, 0.040435101836919785, 0.17572756111621857, 0.1331264227628708, 0.3701251149177551, -0.03870895877480507, 0.1785021722316742, -0.1322893649339676, 0.056645359843969345, 0.15197022259235382, 0.05615327134728432, -0.07347171008586884, 0.025806574150919914, 0.021525299176573753, 0.029612164944410324, -0.13638398051261902, -0.18047936260700226, 0.1372050940990448, -0.13854728639125824, -0.0481029711663723, 0.05815843492746353, 0.0027104683686047792, 0.13854773342609406, 0.23899111151695251, 0.004702366888523102, 0.1967664510011673, 0.18575042486190796, -0.15789520740509033, -0.04743698984384537, 0.04737019166350365, 0.026005180552601814, -0.03090485744178295, 0.04125530645251274, 0.03437406197190285, -0.00742111261934042, 0.03700924664735794, 0.06501505523920059}, {0.28333580493927, -0.15752920508384705, -0.17122608423233032, 0.020685674622654915, -0.4622308015823364, -0.1981009542942047, 0.005100760143250227, -0.09044325351715088, -0.19072112441062927, 0.12972700595855713, 0.20379555225372314, 0.1758124828338623, -0.1263204663991928, 0.12097291648387909, -0.08196621388196945, 0.0983738973736763, 0.06933020800352097, 0.06153619661927223, 0.15827161073684692, -0.07180449366569519, -0.09287456423044205, -0.06813281774520874, -0.14197774231433868, -0.21606187522411346, -0.24988511204719543, 0.002766096033155918, 0.3247557282447815, 0.04690041393041611, -0.12668955326080322, 0.058270055800676346, -0.1727043241262436, 0.02244061604142189, -0.10197950899600983, 0.011386224068701267, 0.23688815534114838, -0.02846045047044754, -0.03219885379076004, 0.05280853062868118, -0.06435035169124603, 0.04947960004210472, 0.0026236604899168015, -0.11914201825857162, -0.29721853137016296, -0.011451520025730133, 0.1092083752155304, -0.17132727801799774, -0.23108263313770294, 0.035707488656044006, 0.030383696779608727, -0.00734297651797533, -0.28793197870254517, 0.021054711192846298, -0.12844955921173096, -0.23660942912101746, 0.004543990362435579, 0.05061326175928116, -0.20227225124835968, -0.2158629298210144, -0.07150954008102417, 0.024197611957788467, 0.09013395011425018, -0.01968441531062126, 0.018478045240044594, 0.30323562026023865, -0.14033401012420654, -0.6480121612548828, 0.09157025814056396, 0.020740874111652374, -0.14882276952266693, 0.11621018499135971, 0.07525132596492767, 0.04359492287039757, -0.1078062653541565, -0.08559713512659073, 0.26962992548942566, 0.09372954070568085, -0.20841509103775024, 0.13301889598369598, -0.0368877574801445, -0.16161926090717316, 0.10510649532079697, -0.016743546351790428, 0.05918595939874649, 0.06891065090894699, -0.21732693910598755, -0.09475591778755188, -0.2745285928249359, -0.13689732551574707, -0.01418653316795826, -0.04994645342230797, 0.00019408763910178095, -0.2515980899333954, -0.029250379651784897, -0.0762297585606575, 0.04439922422170639, -0.26135092973709106, -0.06802396476268768, -0.053245458751916885, 0.1839042454957962, 0.2688101530075073, -0.2647929787635803, -0.026420312002301216, -0.022229237481951714, 0.13823537528514862, -0.11003724485635757, 0.0446942038834095, 0.19303207099437714, -0.2281816005706787, 0.060690078884363174, 0.11831247806549072, 0.1721247136592865, -0.08729863911867142, 0.15591250360012054, -0.16064973175525665, -0.04002154991030693, -0.047025199979543686, -0.038332268595695496, 0.1308629810810089, 0.037952788174152374, 0.21847191452980042, 0.012123827822506428, -0.16255562007427216, 0.01113399863243103, 0.06224764883518219, -0.10504695028066635, 0.24595551192760468, 0.05883697792887688, 0.11706779152154922, -0.08299729973077774, 0.3155134320259094, 0.09364915639162064, 0.27533045411109924, 0.3125724196434021, 0.03970588743686676, -0.06324484199285507, 0.07894971966743469, -0.11855393648147583, 0.13986599445343018, -0.19320663809776306, 0.2152034044265747, -0.14229045808315277, 0.2804901897907257, -0.2521205246448517, 0.16589820384979248, -0.009125239215791225, 0.010094566270709038, 0.2523603141307831, -0.24332232773303986, 0.1730383336544037, 0.025018787011504173, 0.2044144868850708, 0.14540791511535645, 0.14593592286109924, -0.10938474535942078, -0.15239202976226807, -0.0035104043781757355, -0.1930762380361557, -0.0873357355594635, 0.04678571969270706, 0.013560009188950062, 0.019632823765277863, -0.13438385725021362, -0.01878109760582447, -0.0022142708767205477, -0.06155967339873314, 0.05730952322483063, 0.19866527616977692, 0.09513355791568756, 0.15660566091537476, 0.20540767908096313, -0.07082470506429672, 0.19224081933498383, 0.0689561739563942, 0.04178912565112114, 0.008434876799583435, 0.10569412261247635, 0.1201365739107132, -0.13930188119411469, -0.1158226802945137, -0.12503598630428314, -0.02309819869697094, 0.33349359035491943, -0.1710091233253479, 0.13626345992088318, 0.3551587164402008, -0.02170267142355442, 0.2831140160560608, 0.06376244127750397, -0.08343403786420822, 0.07201121002435684, 0.13216260075569153, -0.02705748751759529, 0.028119582682847977, 0.1368725448846817, -0.5025038719177246, -0.17705479264259338, 0.05810576304793358, 0.04685515537858009, 0.005052056163549423, -0.10120885074138641, -0.17387908697128296, -0.06719724833965302, 0.004047737922519445, -0.03599105030298233, 0.09853845089673996, 0.1304577738046646, -0.2390880286693573, -0.005014505703002214, -0.2626523971557617, -0.36467477679252625, 0.13001354038715363, 0.041783709079027176, -0.12860873341560364, -0.0680588111281395, 0.1932799071073532, 0.05709988623857498, -0.10395746678113937, -0.1539411097764969, -0.004205932375043631, 0.04212335869669914, -0.12124886363744736, 0.2125948667526245, 0.02923024445772171, -0.07772022485733032, -0.15695171058177948, -0.0929926186800003, 0.05827385187149048, 0.06480223685503006, -0.025778094306588173, -0.08187728375196457, -0.12983503937721252, 0.06240766867995262, 0.29316842555999756, 0.07266438752412796, -0.006271977908909321, 0.24275726079940796, -0.15277236700057983, -0.00980082806199789, 0.05171305686235428, 0.047856371849775314, -0.08393283188343048, 0.05248698592185974, -0.1683386266231537, 0.11912804841995239, -0.055679623037576675, -0.33696427941322327, 0.08865820616483688, 0.052716340869665146, -0.19470550119876862, 0.036663830280303955, 0.26501694321632385, -0.1744522750377655, 0.0900314450263977, -0.21586182713508606, -0.12122287601232529, -0.0488167479634285}, {-0.005063334014266729, 0.1487163007259369, 0.017057528719305992, -0.06464032083749771, -0.29849210381507874, -0.20182517170906067, 0.18927410244941711, -0.4637466371059418, 0.016004065051674843, 0.12201546132564545, -0.013217497617006302, -0.06303198635578156, 0.004277560394257307, 0.012435468845069408, 0.04514437913894653, -0.11008884757757187, 0.11125507205724716, 0.07727210223674774, -0.0566096305847168, -0.024382159113883972, -0.05078192055225372, -0.1271824985742569, -0.11205330491065979, -0.17441442608833313, -0.1705540418624878, 0.28888818621635437, 0.16899700462818146, 0.004435759969055653, -0.03591267392039299, 0.06642373651266098, -0.1683003157377243, 0.17525500059127808, -0.04416470229625702, -0.020499151200056076, -0.0459456630051136, 0.15564829111099243, -0.03903256729245186, 0.20553058385849, -0.10003161430358887, -0.285584419965744, -0.19689588248729706, -0.09286829084157944, -0.12419705092906952, 0.11067706346511841, 0.011212962679564953, 0.0515068955719471, -0.24792371690273285, -0.19468215107917786, -0.2641295790672302, 0.08505858480930328, -0.30975934863090515, 0.11129646748304367, -0.03806042671203613, -0.013979200273752213, 0.2234102487564087, -0.08596096187829971, -0.37687790393829346, -0.07664600014686584, -0.2095392793416977, -0.43576574325561523, 0.011168326251208782, 0.13489365577697754, 0.12696430087089539, -0.04514384642243385, -0.00686066597700119, -0.40595924854278564, -0.07128260284662247, 0.1575341671705246, 0.11042167246341705, 0.09702765941619873, 0.10389531403779984, -0.14807046949863434, -0.19843176007270813, 0.02239004336297512, 0.13707074522972107, -0.18294751644134521, -0.12685668468475342, -0.0036098971031606197, -0.11288632452487946, -0.32053396105766296, -0.0681147649884224, -0.06286703050136566, 0.14961998164653778, 0.3598158061504364, -0.21974430978298187, -0.08988527953624725, -0.0046645477414131165, -0.06835497915744781, 0.11146383732557297, 0.11089307069778442, 0.047019533812999725, 0.1265058070421219, -0.1080542653799057, 0.14282643795013428, 0.09400168806314468, 0.058309800922870636, 0.0005590621731244028, -0.1142863854765892, 0.10723434388637543, 0.13626565039157867, -0.1846192330121994, -0.13846899569034576, 0.045152369886636734, -0.09344887733459473, -0.08022934198379517, -0.46467283368110657, 0.007182828150689602, -0.01183946244418621, -0.26908978819847107, -0.054729774594306946, -0.010990671813488007, -0.30692780017852783, -0.04657011479139328, -0.09879351407289505, 0.08997958153486252, 0.06868767738342285, 0.1565413624048233, 0.07249920070171356, -0.14506839215755463, 0.10079887509346008, -0.03149167820811272, -0.06443911790847778, 0.06587832421064377, 0.07752232998609543, 0.012410164810717106, 0.22224637866020203, -0.061638254672288895, -0.09919518977403641, 0.21276912093162537, 0.07203302532434464, 0.18610449135303497, -0.0647253766655922, 0.492051362991333, 0.0405786894261837, -0.04248329997062683, -0.2791909873485565, 0.10923366248607635, 0.11397732049226761, 0.08869102597236633, 0.07405675947666168, -0.2894112467765808, 0.08820203691720963, 0.14506986737251282, -0.01224004290997982, 0.17053109407424927, 0.09443266689777374, 0.10791800171136856, 0.18123595416545868, -0.10071878135204315, 0.038459476083517075, 0.07858861237764359, 0.06700577586889267, -0.0938325896859169, -0.04899504780769348, -0.17952145636081696, -0.04523112624883652, -0.11712093651294708, -0.02145840972661972, 0.0633435770869255, 0.3895239233970642, -0.07448924332857132, 0.16120892763137817, 0.23121371865272522, -0.15371467173099518, 0.09406765550374985, 0.06664726883172989, -0.06103236973285675, 0.2465389370918274, 0.00489854859188199, 0.010985353961586952, -0.10850515216588974, 0.025716928765177727, -0.07730752974748611, 0.17319226264953613, 0.16391925513744354, 0.006375913508236408, 0.05443999171257019, 0.13658669590950012, -0.09379713982343674, -0.010224117897450924, -0.05604657158255577, 0.12305662781000137, -0.0315532349050045, -0.013696848414838314, 0.04138336330652237, 0.08586671948432922, 0.36730924248695374, 0.061612408608198166, -0.12301050871610641, 0.1663838028907776, -0.08203978836536407, -0.001987440511584282, -0.10776444524526596, -0.07789316028356552, -0.2121429145336151, 0.019595207646489143, -0.03658728674054146, 0.11360151320695877, 0.02351083606481552, -0.00500847352668643, -0.11398417502641678, 0.04920414090156555, -0.08735833317041397, -0.05444842204451561, -0.08946536481380463, -0.2075156420469284, -0.0631733238697052, -0.11978621035814285, -0.039462532848119736, 0.16550090909004211, -0.28472092747688293, 0.33602944016456604, 0.14958426356315613, -0.011059015989303589, -0.011478730477392673, -0.040847912430763245, 0.1320890337228775, 0.00046013915562070906, -0.04087839648127556, 0.07198817282915115, -0.1269182562828064, 0.16243255138397217, -0.04329695552587509, 0.4017559289932251, 0.06291373074054718, 0.2722790539264679, -0.2211681306362152, -0.0948537290096283, 0.08832090348005295, -0.1502436250448227, 0.0169454887509346, 0.10310649126768112, 0.16153404116630554, -0.10252111405134201, -0.16970473527908325, 0.052390821278095245, -0.10572656244039536, -0.06269720196723938, 0.24432998895645142, 0.15297524631023407, 0.08077719062566757, 0.0028926352970302105, -0.05843053385615349, -0.05657960847020149, -0.05722484737634659, -0.07834552973508835, -0.07973158359527588, 0.03986799716949463, 0.13129019737243652, 0.017709514126181602, -0.06073750928044319, -0.021377863362431526, 0.2133655548095703, -0.13600091636180878, -0.0346032939851284, 0.06054994836449623}, {0.024614304304122925, 0.1623239517211914, 0.11939771473407745, -0.11192285269498825, 0.08919494599103928, -0.06685841828584671, 0.1565827876329422, 0.10156209021806717, 0.165535107254982, -0.005906819831579924, 0.1811273694038391, -0.12282141298055649, -0.03327573463320732, 0.1584235280752182, 0.029670659452676773, 0.16063864529132843, 0.09253814071416855, -0.14275433123111725, -0.010041214525699615, 0.014165196567773819, 0.3827130198478699, 0.28597745299339294, 0.14383308589458466, -0.205868199467659, -0.07258962839841843, -0.1283067911863327, -0.07209958881139755, 0.04539245367050171, 0.04635337367653847, 0.2253565788269043, 0.20564813911914825, -0.04585004970431328, -0.045055635273456573, 0.09403438866138458, 0.13713927567005157, 0.03921585530042648, -0.03564342483878136, 0.02507157251238823, 0.11911941319704056, -0.01189924031496048, -0.08013495802879333, 0.19506174325942993, -0.12083758413791656, 0.026108724996447563, 0.033782463520765305, -0.09039972722530365, -0.04132223129272461, -0.012957786209881306, -0.04578680545091629, -0.07164707034826279, -0.05758536979556084, -0.10396061837673187, -0.1028294563293457, -0.021008020266890526, 0.04666724428534508, 0.21382088959217072, 0.001982910092920065, -0.19385604560375214, 0.018739162012934685, -0.07655634731054306, 0.008165808394551277, -0.03238336369395256, 0.0642584040760994, 0.01460401713848114, -0.001227193046361208, 0.006520465016365051, 0.10757819563150406, 0.10034395009279251, 0.11096085608005524, 0.007100030779838562, -0.04479657858610153, 0.16635872423648834, -0.005526574794203043, 0.032978229224681854, 0.3142496943473816, -0.10657847672700882, -0.049807075411081314, 0.3538818657398224, -0.2026883065700531, 0.02712869830429554, -0.024771396070718765, 0.03841380774974823, 0.011276119388639927, 0.02139117568731308, -0.08579462766647339, 0.03668490797281265, -0.09384285658597946, 0.2241278737783432, -0.03478119522333145, 0.3155261278152466, -0.00291837309487164, 0.09220284968614578, -0.07354969531297684, -0.004216657020151615, 0.17665590345859528, -0.03062782622873783, -0.03249860182404518, -0.09667074680328369, -0.15014199912548065, -0.1309187114238739, 0.06829629838466644, 0.11897475272417068, -0.1893879920244217, 0.005148502066731453, 0.1671573370695114, 0.9371572732925415, -0.05205259844660759, -0.06243995577096939, -0.11002057045698166, -0.1671164482831955, 0.1007521003484726, -0.09157867729663849, -0.026903396472334862, -0.04497184604406357, -0.012889818288385868, -0.006837172899395227, 0.08318061381578445, 0.21939724683761597, -0.09040506184101105, 0.0090031111612916, -0.11005163937807083, 0.07904930412769318, -0.1662374883890152, 0.05562823638319969, -0.08697914332151413, 0.05629806965589523, -0.15475037693977356, -0.09784352779388428, -0.13442018628120422, 0.018501250073313713, -0.20656439661979675, -0.11813803017139435, -0.04942428693175316, -0.028294220566749573, 0.03248835355043411, 0.11832719296216965, -0.08591978996992111, -0.03049788996577263, 0.01059908140450716, 0.13892151415348053, -0.05438387021422386, 0.11111751198768616, -0.18319016695022583, -0.11686895787715912, 0.05753035470843315, 0.003216199344024062, -0.16139255464076996, 0.10386066883802414, 0.1066342443227768, -0.018689995631575584, 0.08361206948757172, -0.06234974041581154, -0.18868127465248108, -0.18131160736083984, 0.054131608456373215, -0.052680615335702896, -0.053374335169792175, -0.13725557923316956, 0.1478727161884308, -0.3174610733985901, 0.15725690126419067, -0.03215412423014641, -0.009447832591831684, 0.09044988453388214, 0.1437125951051712, -0.11355844885110855, -0.01837114803493023, 0.06402750313282013, 0.12075956910848618, 0.02495148777961731, -0.11121749877929688, 0.129071906208992, -0.028180338442325592, 0.10966458916664124, -0.12268009781837463, -0.03072207421064377, 0.054641783237457275, 0.03166758269071579, 0.09909648448228836, -0.12459178268909454, 0.08914142847061157, 0.04565851762890816, 0.12957483530044556, 0.026914330199360847, -0.022537488490343094, -0.016878638416528702, 0.0012783214915543795, -0.06858600676059723, -0.14917585253715515, -0.029693404212594032, 0.07010357826948166, -0.041318412870168686, 0.08085612952709198, -0.19298948347568512, 0.054478805512189865, 0.002611494157463312, -0.048016466200351715, -0.008382038213312626, -0.21930477023124695, 0.14719092845916748, -0.06344674527645111, 0.4469057619571686, -0.07884862273931503, 0.05441897734999657, -0.07025095820426941, 0.07523632049560547, 0.06985604763031006, 0.2572355270385742, -0.10575993359088898, 0.2080465406179428, -0.033772580325603485, -0.06859223544597626, 0.09361708909273148, 0.06918222457170486, -0.012063388712704182, 0.07960810512304306, 0.0703059583902359, 0.030506351962685585, -0.14152564108371735, 0.023088879883289337, -0.20778979361057281, 0.026461290195584297, -0.029646936804056168, -0.14028994739055634, 0.032490357756614685, -0.18191295862197876, -0.25356703996658325, -0.10781408101320267, -0.13332301378250122, -0.04254235699772835, -0.11552199721336365, 0.01230429857969284, -0.05683286860585213, 0.03350595012307167, 0.05435870215296745, 0.009790785610675812, 0.03951325640082359, -0.01597454398870468, 0.2915208637714386, -0.030762042850255966, 0.09064305573701859, -0.13403162360191345, 0.016596173867583275, 0.16752974689006805, -0.02895882911980152, -0.29437628388404846, 0.010062728077173233, -0.02600049413740635, 0.20851020514965057, -0.09566272795200348, 0.08665790408849716, -0.2831590473651886, 0.0019511544378474355, -0.21528972685337067, 0.11107024550437927, -0.39473485946655273}, {0.05260910838842392, 0.10563924908638, 0.05747585371136665, -0.13235509395599365, 0.07549653202295303, 0.12279678136110306, -0.24862003326416016, 0.3093087375164032, -0.04951036721467972, 0.43441328406333923, -0.1260574460029602, 0.19289502501487732, -0.037592723965644836, 0.04985564202070236, 0.07379711419343948, 0.19949722290039062, 0.19737419486045837, 0.022599557414650917, 0.14032815396785736, -0.05350843444466591, -0.2395547479391098, 0.10177014023065567, 0.005815065000206232, -0.17522037029266357, -0.12801775336265564, 0.05360489338636398, 0.0577629916369915, 0.042207855731248856, -0.04591929167509079, -0.1860361397266388, 0.13984760642051697, 0.1321772038936615, -0.0220340546220541, 0.11150544881820679, -0.004438129719346762, 0.1533304750919342, -0.1732078641653061, -0.1316644549369812, 0.023549368605017662, 0.07768361270427704, 0.03600802645087242, 0.21680289506912231, 0.032650671899318695, 0.0585017204284668, -0.19939498603343964, -0.03766028955578804, -0.01944563165307045, 0.014395719394087791, -0.08643431216478348, 0.06797176599502563, -0.06614311784505844, -0.033666934818029404, -0.009283050894737244, 0.09934122860431671, -0.02890102006494999, -0.11267759650945663, 0.0495414212346077, 0.015045123174786568, 0.12184429168701172, 0.18448635935783386, 0.09743648022413254, -0.035991691052913666, 0.005641032010316849, 0.020159678533673286, -0.060118045657873154, 0.07676154375076294, -0.0557069405913353, -0.1486334204673767, -0.029468603432178497, 0.02565082721412182, -0.18378648161888123, 0.03368499130010605, -0.1675577312707901, 0.019874926656484604, 0.1156277135014534, -0.31122681498527527, -0.0802982747554779, 0.25971078872680664, -0.12312024086713791, 0.051546208560466766, 0.059544529765844345, 0.23745544254779816, 0.03796648979187012, 0.0828484445810318, -0.008327925577759743, 0.03711981326341629, 0.17501522600650787, 0.03134565055370331, 0.017565790563821793, -0.20483095943927765, 0.08760669082403183, -0.2068861424922943, -0.030164595693349838, -0.12688063085079193, 0.039507828652858734, -0.06963202357292175, 0.23211568593978882, -0.057086214423179626, -0.0376712791621685, 0.13609199225902557, 0.049508560448884964, -0.14144381880760193, 0.13854755461215973, 0.06218963488936424, -0.028862204402685165, 0.9346612095832825, 0.033425502479076385, 0.04536870867013931, 0.10742952674627304, 0.2001783400774002, -0.06533650308847427, -0.06227394938468933, -0.1390656679868698, 0.19477450847625732, 0.020669754594564438, -0.12461953610181808, 0.04344955459237099, 0.10091990232467651, 0.037562280893325806, 0.024616550654172897, 0.062487177550792694, -0.2524206042289734, -0.04507824406027794, 0.2119092047214508, 0.02154874987900257, 0.10973849147558212, 0.08073955029249191, -0.19014978408813477, -0.024380214512348175, 0.14056839048862457, -0.0046179089695215225, -0.011952287517488003, -0.0143950916826725, 0.15882469713687897, -0.06042451038956642, -0.13935773074626923, 0.03503027185797691, 0.19232980906963348, 0.05503511056303978, -0.19085034728050232, 0.2195693999528885, 0.1731480062007904, 0.06646755337715149, -0.018240323290228844, 0.0198961291462183, -0.12047600001096725, 0.2889198362827301, -0.007292551454156637, -0.24922628700733185, -0.12408442050218582, 0.16500337421894073, 0.08389630913734436, -0.08758822828531265, 0.19061905145645142, -0.031007828190922737, -0.07609309256076813, -0.11005841940641403, 0.06991975754499435, 0.26221227645874023, -0.08197349309921265, -0.23421499133110046, -0.007805834989994764, 0.07347100973129272, -0.16523776948451996, -0.12660212814807892, 0.09768055379390717, 0.21453239023685455, -0.18434663116931915, -0.11869129538536072, 0.12544235587120056, -0.05184463784098625, 0.3386283218860626, 0.02858547493815422, 0.23532520234584808, 0.11761181801557541, -0.12773635983467102, -0.207215815782547, 0.01899264007806778, -0.23217535018920898, 0.10242188721895218, -0.2782886028289795, -0.07525505125522614, -0.06995084136724472, -0.0698731392621994, -0.007691803853958845, 0.05515234172344208, 0.24594086408615112, 4.517920024227351e-05, 0.06140342354774475, -0.28098368644714355, 0.08372829109430313, -0.16220422089099884, 0.000595875084400177, 0.3844332993030548, -0.04700811952352524, -0.08398529887199402, -0.19932809472084045, -0.0757032260298729, -0.04317815974354744, -0.26987069845199585, -0.026885593309998512, -0.013902339152991772, 0.03836936503648758, 0.010595014318823814, -0.11573293060064316, 0.14350955188274384, 0.4150818884372711, 0.05506225302815437, 0.05119878053665161, 0.03845946118235588, -0.2338586449623108, -0.327358603477478, -0.19454899430274963, -0.12281806021928787, 0.08104731887578964, -0.03174735978245735, 0.018110621720552444, -0.2212727665901184, -0.10787614434957504, 0.17437684535980225, -0.0628805011510849, -0.14461389183998108, 0.1590879261493683, -0.08956961333751678, -0.07524654269218445, 0.008404001593589783, -0.2781088948249817, 0.16569893062114716, 0.007941097021102905, 0.21153731644153595, -0.050694577395915985, 0.02156265452504158, 0.047211747616529465, 0.09122536331415176, -0.10594719648361206, 0.14060455560684204, 0.029031313955783844, 0.011954245157539845, -0.04223909229040146, 0.3333894610404968, -0.042150743305683136, 0.17126382887363434, 0.14902879297733307, 0.08894508332014084, 0.37211576104164124, -0.0017362791113555431, 0.03836340084671974, 0.03997958451509476, 0.04252413287758827, 0.1296183317899704, 0.16104856133460999, -0.3713974356651306, 0.17570270597934723, 0.012651536613702774, -0.18419815599918365, 0.21593031287193298}, {0.13646867871284485, -0.23744815587997437, 0.12970443069934845, 0.025528544560074806, -0.030177848413586617, 0.14091508090496063, -0.0745702013373375, 0.038442954421043396, 0.07734960317611694, 0.16179268062114716, -0.0027404739521443844, 0.10547994077205658, -0.04184180498123169, -0.00787401758134365, 0.06293395906686783, 0.0022688212338835, -0.001259895390830934, -0.2254142463207245, 0.15340451896190643, 0.05400446057319641, 0.23867526650428772, 0.009244783781468868, 0.06628574430942535, -0.11766938865184784, -0.2353808730840683, -0.008456571958959103, 0.052252575755119324, -0.08259587734937668, 0.09722958505153656, 0.14810140430927277, -0.014647907577455044, 0.15506213903427124, 0.013761251233518124, -0.008255163207650185, -0.18156322836875916, 0.1168082132935524, -0.09218420088291168, 0.010416907258331776, -0.13649988174438477, 0.04715041443705559, 0.016317380592226982, -0.058940619230270386, -0.12794765830039978, -0.013543992303311825, -0.290702760219574, -0.20536769926548004, -0.3800882399082184, 0.26757553219795227, -0.022110113874077797, -0.0526457317173481, 0.15265434980392456, -0.06671173125505447, 0.051736801862716675, -0.20199941098690033, 0.20050308108329773, -0.058810196816921234, -0.24225880205631256, -0.14132025837898254, 0.16790620982646942, -0.07535019516944885, 0.20482605695724487, 0.07711747288703918, -0.03268645331263542, -0.03912141174077988, 0.014815384522080421, -0.17326796054840088, 0.017305996268987656, 0.1450149416923523, 0.021537909284234047, -0.05758419632911682, 0.151988223195076, 0.1777670532464981, 0.006459117867052555, -0.2786763310432434, -0.022131605073809624, -0.2650212049484253, 0.18038420379161835, 0.056970950216054916, -0.18606515228748322, -0.08545288443565369, -0.20667371153831482, -0.07774748653173447, -0.15135402977466583, -0.06709888577461243, 0.0741267129778862, -0.13549821078777313, -0.2427036613225937, 0.16253718733787537, 0.031168119981884956, -0.028306517750024796, 0.07443620264530182, 0.022433334961533546, 0.17495273053646088, 0.01284533366560936, 0.09942224621772766, 0.005426222924143076, 0.12701387703418732, -0.03607383370399475, -0.03042401745915413, -0.4433492422103882, 0.06532548367977142, 0.015580358915030956, -0.2504775822162628, 0.07302755862474442, 0.10802645236253738, -0.47548961639404297, -0.01563403569161892, 0.038450632244348526, -0.10314393043518066, 0.021846266463398933, -0.1398727297782898, -0.03312653675675392, 0.009785540401935577, -0.2275020182132721, 0.060901790857315063, 0.11639884114265442, -0.03447885811328888, -0.034292418509721756, -0.04382958635687828, -0.07796930521726608, -0.10056289285421371, 0.062076158821582794, -0.2850244343280792, -0.05082271620631218, -0.19708576798439026, 0.3624468743801117, -0.17204277217388153, 0.16690689325332642, -0.14228837192058563, -0.13877376914024353, 0.11469487100839615, -0.12158599495887756, 0.10719285905361176, -0.012157067656517029, -0.16731473803520203, -0.2697858214378357, 0.22351118922233582, -0.013944940641522408, -0.15961286425590515, 0.10277736932039261, -0.06864644587039948, 0.06654918193817139, -0.29682955145835876, 0.24743688106536865, 0.011889776214957237, -0.08461732417345047, -0.13841666281223297, -0.06469183415174484, 0.130319744348526, 0.0066438340581953526, 0.17726944386959076, 0.006029842887073755, -0.08052050322294235, 0.2517056167125702, 0.011700528673827648, 0.05907502397894859, -0.02522214688360691, 0.05038343742489815, 0.11409314721822739, -0.16230185329914093, 0.15456220507621765, -0.13436807692050934, 0.18342362344264984, 0.32558849453926086, 0.1352740228176117, 0.04273001104593277, 0.05274010822176933, -0.003240793477743864, 0.1284993290901184, 0.09973231703042984, -0.09670428931713104, -0.07572934031486511, 0.08874652534723282, 0.01328413374722004, 0.23835872113704681, -0.17587444186210632, -0.06511484086513519, -0.035461731255054474, 0.05023185908794403, -0.16096407175064087, -0.028379248455166817, 0.1753990650177002, -0.016271427273750305, -0.14271403849124908, -0.018154671415686607, -0.07150212675333023, 0.43227359652519226, -0.01273154653608799, -0.046806734055280685, -0.014248823747038841, 0.07313438504934311, 0.0810215026140213, 0.01193061750382185, 0.12180842459201813, -0.19220902025699615, -0.006207642611116171, 0.024818023666739464, 0.13676272332668304, 0.10469722002744675, 0.20111800730228424, -0.09108328819274902, 0.08693653345108032, -0.14352235198020935, -0.041987642645835876, -0.17788799107074738, 0.1448666900396347, 0.1636676788330078, -0.11580492556095123, -0.13360527157783508, 0.1439078152179718, 0.08342939615249634, 0.08402252942323685, 0.17224165797233582, 0.001823418540880084, -0.12613072991371155, 0.07990866899490356, -0.07966464012861252, 0.13854128122329712, -0.22602654993534088, 0.3493172824382782, -0.019232003018260002, -0.05760452523827553, 0.021495642140507698, 0.26030200719833374, 0.14231348037719727, 0.2843583822250366, 0.10519370436668396, -0.15749309957027435, 0.05469666048884392, -0.1749688982963562, -0.07977069914340973, 0.1771271824836731, -0.010149449110031128, -0.03532681614160538, -0.08156578987836838, 0.19677451252937317, -0.030286673456430435, -0.09573253244161606, 0.2921161353588104, -0.49808335304260254, -0.020866895094513893, 0.05878917872905731, 0.17555853724479675, -0.07479294389486313, 0.007714746054261923, -0.19139701128005981, 0.22765684127807617, 0.07349233329296112, -0.06381906569004059, 0.2058590054512024, -0.06167071312665939, -0.03856434300541878, -0.008462467230856419, 0.1442677080631256, 0.031433723866939545, 0.09909208118915558}, {0.20477814972400665, 0.021494196727871895, 0.052367471158504486, -0.1847754567861557, -0.059387482702732086, 0.11115049570798874, 0.12170041352510452, 0.22204819321632385, -0.06039026752114296, -0.08751603960990906, -0.12612532079219818, 0.04898662492632866, -0.05480498448014259, 0.24574322998523712, -0.07752757519483566, -0.0492883175611496, 0.006604006513953209, -0.03222418203949928, 0.03311984986066818, 0.14949655532836914, -0.1297476887702942, -0.1745917648077011, -0.20636850595474243, 0.1438310295343399, -0.09345646947622299, 0.040778759866952896, -0.015090127475559711, -0.03571232780814171, 0.022004136815667152, 0.03785323724150658, -0.003522151382640004, -0.028641752898693085, -0.0744362622499466, -0.04044962674379349, 0.21114854514598846, -0.2912469208240509, -0.044365037232637405, 0.01949194259941578, -0.2534398138523102, 0.01936042681336403, 0.02697479911148548, -0.013942726887762547, 0.05010215565562248, -0.0006403722218237817, 0.1525515764951706, 0.022360971197485924, -0.23074141144752502, -0.0025113122537732124, -0.1569422036409378, 0.07923179119825363, -0.13131022453308105, -0.010390851646661758, -0.08361414819955826, -0.11030036956071854, -0.10978072881698608, -0.08853479474782944, 0.10577534139156342, 0.054629500955343246, 0.06254497915506363, 0.28684577345848083, -0.052006326615810394, -0.233668714761734, 0.04226459562778473, -0.6516677737236023, 0.009633785113692284, 0.18180270493030548, 0.13063211739063263, -0.04767323657870293, -0.014631249941885471, -0.09783326089382172, 0.07707315683364868, 0.1007830798625946, -0.048942189663648605, -0.06154194474220276, -0.08194074034690857, -0.11698759347200394, -0.11844713985919952, -0.12232592701911926, 0.023536397144198418, 0.17305754125118256, 0.013229544274508953, 0.08370010554790497, 0.07138673961162567, -0.00836925208568573, 0.1800355613231659, -0.25777778029441833, 0.2541680932044983, -0.017857156693935394, 0.22704949975013733, 0.09084338694810867, -0.013274021446704865, 0.0044997981749475, 0.10180003941059113, 0.12202002853155136, 0.13668227195739746, -0.02874385006725788, 0.41025033593177795, 0.12155137956142426, -0.20457683503627777, 0.24693414568901062, 0.2423965036869049, 0.1977580040693283, 0.01229818258434534, -0.0014887095894664526, -0.1894393265247345, 0.0011541771236807108, 0.07497810572385788, 0.03426872938871384, 0.25138676166534424, 0.07195425033569336, 0.20169799029827118, -0.003805355867370963, -0.19629424810409546, -0.03161797672510147, 0.11068086326122284, -0.22999460995197296, 0.22142812609672546, -0.05629752576351166, -0.07357870042324066, -0.39788907766342163, -0.15511585772037506, -0.14983871579170227, -0.14928306639194489, -0.16528688371181488, 0.06904681771993637, 0.06543154269456863, -0.028608521446585655, -0.03867190331220627, 0.05150750279426575, -0.021612778306007385, 0.259276419878006, 0.12681514024734497, -0.07469122856855392, -0.09386429190635681, -0.39223986864089966, -0.10486249625682831, 0.15975263714790344, -0.012508216314017773, 0.037189386785030365, 0.02195851318538189, 0.13025157153606415, -0.14133726060390472, -0.10689520090818405, -0.02008785307407379, 0.1831878274679184, -0.047492459416389465, -0.10394631326198578, -0.003465876681730151, -0.07569685578346252, -0.15844422578811646, 0.009735685773193836, 0.11707750707864761, 0.04615127295255661, -0.18848693370819092, 0.1663559377193451, 0.06466204673051834, 0.05335575342178345, -0.15483853220939636, -0.01666123792529106, 0.07563778758049011, -0.04161030799150467, -0.0005895275389775634, 0.06577202677726746, 0.1044246181845665, 0.06755287945270538, -0.2110775262117386, 0.051662519574165344, 0.07498608529567719, -0.2501465380191803, -0.0077468459494411945, 0.047254569828510284, 0.026771996170282364, 0.04274856299161911, -0.24825802445411682, -0.1939481496810913, -0.012531530112028122, 0.06410118192434311, 0.18311943113803864, 0.016582803800702095, -0.13656975328922272, -0.04050042852759361, 0.0010661222040653229, 0.01931905373930931, 0.005492257419973612, 0.17829932272434235, 0.0073943170718848705, -0.24832333624362946, 0.05609794333577156, -0.2540779411792755, -0.10217780619859695, -0.04759891703724861, 0.1328200250864029, 0.07125231623649597, 0.01949794590473175, 0.18497245013713837, -0.1877567023038864, 0.14590096473693848, -0.026285655796527863, 0.07391374558210373, -0.030797351151704788, -0.15472863614559174, -0.06549909710884094, 0.020300140604376793, -0.09248421341180801, -0.13664522767066956, -0.06149112805724144, -0.1593780219554901, 0.2762663960456848, -0.049479708075523376, -0.0035818989854305983, -0.08155779540538788, -0.12852542102336884, -0.05497702211141586, -0.017126524820923805, 0.06388645619153976, 0.015579995699226856, -0.12915198504924774, 0.09698017686605453, 0.03765695542097092, 0.10723523795604706, 0.024521298706531525, 0.09161939471960068, 0.05107172951102257, 0.0013936609029769897, 0.18667124211788177, -0.12391913682222366, -0.05552603304386139, 0.3210662603378296, -0.11672938615083694, 0.050762616097927094, -0.13690485060214996, -0.07034612447023392, -0.007524450775235891, -0.09469839185476303, 0.15061846375465393, 0.11840633302927017, 0.1514367312192917, 0.006099384743720293, 0.15276406705379486, 0.15380917489528656, -0.10935032367706299, 0.0954829528927803, -0.046666815876960754, 0.13097602128982544, 0.059359051287174225, 0.25117096304893494, 0.13661184906959534, 0.1968689113855362, 0.02227633260190487, 0.02422967739403248, -0.13292837142944336, -0.0721346065402031, -0.2708338797092438, -0.017920125275850296, 0.1501842439174652, -0.0892518162727356}, {-0.04664624482393265, -0.03379896655678749, -0.21618860960006714, 0.03352081775665283, -0.04894183948636055, -0.02127385139465332, 0.18332795798778534, 0.0928662121295929, -0.05559641495347023, 0.06119295582175255, 0.2070288509130478, 0.12236355990171432, -0.1692570000886917, 0.09886254370212555, 0.09534221142530441, 0.0804215669631958, 0.03894101083278656, 0.013539785519242287, 0.2286151498556137, -0.1673508584499359, -0.04053441807627678, 0.08116978406906128, 0.013314739800989628, 0.006370245944708586, -0.07938292622566223, 0.05873527377843857, 0.026328839361667633, 0.18365126848220825, -0.16402973234653473, -0.08218688517808914, -0.10701481997966766, -0.23624388873577118, -0.14647047221660614, 0.07616531103849411, -0.2796322703361511, -0.11523254960775375, -0.19547148048877716, -0.06551636010408401, -0.15782302618026733, 0.031241338700056076, 0.12947218120098114, 0.08854470402002335, -0.0014333033468574286, -0.28506672382354736, -0.20665888488292694, 0.009745090268552303, 0.025438904762268066, 0.027569545432925224, -0.1574556678533554, -0.08026360720396042, -0.23904399573802948, -0.1896561235189438, -0.005272610578685999, -0.04521217942237854, 0.1262650340795517, 0.12572062015533447, 0.02143784612417221, -0.041314076632261276, -0.10447070002555847, 0.07795725762844086, 0.021368760615587234, 0.02017323300242424, -0.09460705518722534, -0.08940939605236053, 0.028390487655997276, 0.05633487552404404, 0.13678869605064392, -0.0409533716738224, -0.28602299094200134, 0.08309587836265564, -0.021046463400125504, -0.022435789927840233, -0.08677788823843002, -0.05083989351987839, -0.10387638211250305, 0.051227468997240067, 0.008861160837113857, -0.048900727182626724, 0.07618993520736694, 0.21185748279094696, -0.2022424191236496, -0.03717183321714401, -0.04292862117290497, 0.023614848032593727, 0.07576847821474075, 0.2493685483932495, -0.07698551565408707, 0.03765730559825897, 0.16008009016513824, -0.1782015562057495, 0.08003193140029907, -0.1572568714618683, 0.16378425061702728, 0.056784868240356445, 0.13983803987503052, 0.13590951263904572, 0.03234780579805374, -0.026200484484434128, -0.0011975181987509131, 0.045589104294776917, -0.10386292636394501, -0.14902539551258087, 0.14265382289886475, -0.14736822247505188, -0.016060352325439453, -0.7150229811668396, 0.01425109151750803, -0.025267954915761948, 0.10461757332086563, 0.0005370740545913577, -0.005779621656984091, 0.12305758148431778, -0.14302901923656464, -0.0168132521212101, 0.2664490044116974, 0.026855256408452988, -0.10618273913860321, 0.010072623379528522, -0.08086489886045456, -0.024099361151456833, 0.01153351366519928, 0.15505307912826538, -0.04377755522727966, -0.0019146306440234184, -0.027552945539355278, -0.08694586902856827, -0.09256264567375183, 0.0022842828184366226, -0.21319478750228882, -0.10440528392791748, -0.009887680411338806, 0.012807835824787617, -0.21011370420455933, 0.04015711694955826, -0.014281134121119976, -0.1112026497721672, -0.26199495792388916, 0.12748368084430695, -0.07244987785816193, 0.03567906841635704, -0.2807196378707886, 0.07795552164316177, -0.05896836146712303, -0.12479525059461594, 0.13362811505794525, 0.1224413737654686, -0.06192898750305176, -0.09966209530830383, -0.06437152624130249, -0.009907892905175686, -0.08512160181999207, -0.17284119129180908, -0.03600502759218216, 0.3070175051689148, -0.11728553473949432, 0.09195798635482788, 0.04520053043961525, -0.03316027671098709, 0.2765733003616333, 0.2147643119096756, 0.10818464308977127, 0.17000095546245575, -0.07387915253639221, -0.026964610442519188, -0.13101749122142792, 0.03591702878475189, 0.323212593793869, 0.0558314174413681, -0.08560804277658463, -0.05848463624715805, -0.04534166306257248, 0.18124091625213623, -0.1970498263835907, -0.043716464191675186, 0.022095542401075363, 0.2773817181587219, 0.08374188840389252, -0.007114734034985304, -0.0847199335694313, 0.12526972591876984, 0.00549635523930192, 0.17589536309242249, -0.1315084546804428, -0.017329217866063118, -0.011629004031419754, -0.034727565944194794, 0.2666465938091278, -0.18916480243206024, 0.055507589131593704, -0.3108895719051361, -0.08400020003318787, -0.14879442751407623, -0.17101164162158966, -0.0741412416100502, -0.0761990025639534, 0.0665193647146225, -0.1961788386106491, 0.031514089554548264, -0.11335869878530502, 0.12207779288291931, 0.005904580000787973, -0.17358416318893433, 0.23277170956134796, 0.08174248039722443, 0.029624884948134422, 0.08541325479745865, -0.4611751437187195, -0.09676551818847656, 0.04532204940915108, 0.013205505907535553, 0.22893668711185455, 0.06309414654970169, -0.037631865590810776, -0.18245533108711243, -0.02428671158850193, -0.1420549601316452, -0.02269645780324936, 0.16852252185344696, -0.18980926275253296, -0.1047557145357132, -0.025147538632154465, -0.0565057210624218, 0.13737082481384277, 0.15929827094078064, -0.20554760098457336, 0.20257484912872314, 0.0325605683028698, 0.023086512461304665, -0.03437928482890129, 0.08417824655771255, -0.01240510493516922, -0.2480749487876892, -0.02303241193294525, 0.03205186128616333, -0.0381157211959362, 0.1921132355928421, -0.0689191222190857, -0.02617853693664074, 0.022335492074489594, -0.12305478751659393, 0.12974992394447327, 0.13499921560287476, -0.031077438965439796, 0.09062312543392181, -0.08108947426080704, -0.0032564611174166203, 0.08969544619321823, -0.0776853933930397, 0.1720569133758545, 0.14715249836444855, 0.03203415870666504, -0.025304334238171577, 0.031750697642564774, -0.1097823902964592, -0.13210396468639374, -0.029501108452677727}, {-0.030252724885940552, 0.18285571038722992, 0.14792783558368683, 0.0038621844723820686, -0.0008318155887536705, 0.03508285805583, -0.24396084249019623, -0.004158542957156897, 0.08245386928319931, -0.25812092423439026, 0.1677478551864624, 0.2791709899902344, -0.03376585245132446, 0.22621813416481018, 0.07231194525957108, 0.09887105226516724, 0.1924060583114624, -0.13286277651786804, -0.02598663419485092, 0.08728060871362686, 0.19451136887073517, 0.04098962992429733, 0.14987438917160034, 0.045450661331415176, -0.1482086181640625, -0.04881647601723671, -0.0765211433172226, -0.05570826679468155, 0.09381323307752609, 0.007888960652053356, 0.04807153344154358, -0.08110465109348297, 0.0003909467195626348, 0.12701615691184998, 0.38194555044174194, -0.08712173998355865, 0.17807243764400482, -0.09982465207576752, 0.0382184274494648, -0.026941141113638878, 0.07433738559484482, -0.021136518567800522, -0.0601293109357357, 0.005604838021099567, -0.10896912962198257, 0.20235197246074677, 0.16279533505439758, -0.033163074404001236, -0.2961469292640686, 0.00561349606141448, -0.002154208719730377, 0.010215885937213898, -0.047195155173540115, -0.1596786081790924, 0.20437897741794586, 0.0579020157456398, 0.10618922114372253, 0.07809580862522125, 0.02835826203227043, -0.6231948733329773, -0.17170028388500214, 0.040522053837776184, 0.35390761494636536, 0.1156686320900917, 0.021350018680095673, -0.003727043280377984, 0.036579232662916183, 0.10645311325788498, -0.053903717547655106, -0.05405489727854729, -0.037130195647478104, 0.40222644805908203, 0.007549712900072336, 0.12320602685213089, 0.30321240425109863, -0.11905781179666519, 0.2506687641143799, 0.15240076184272766, -0.013396551832556725, -0.031099626794457436, 0.08165071159601212, 0.03242320567369461, -0.022250279784202576, 0.028102949261665344, 0.1666828989982605, 0.13658259809017181, -0.03624984622001648, 0.01641152985394001, 0.10869674384593964, 0.1596270352602005, 0.24439281225204468, 0.18367013335227966, 0.109053835272789, 0.0887545794248581, -0.03162223473191261, 0.12265404313802719, 0.00310975662432611, 0.3450486958026886, 0.12603609263896942, 0.017912112176418304, 0.02172115445137024, 0.22135555744171143, -0.24524752795696259, 0.14964544773101807, 0.17745250463485718, 0.4482346475124359, -0.043260395526885986, 0.056763920933008194, 0.2751576900482178, -0.07648040354251862, -0.01926535554230213, 0.11911749094724655, -0.10506264865398407, 0.04491675645112991, 0.02255365438759327, 0.0402643047273159, -0.2117753028869629, -0.06835541874170303, -0.11965741962194443, -0.15102219581604004, -0.09523458033800125, 0.015366800129413605, 0.043724313378334045, 0.08368236571550369, 0.038505781441926956, 0.06735321134328842, 0.008995498530566692, 0.180495947599411, -0.39268070459365845, 0.007398609537631273, -0.08926469832658768, -0.05546635761857033, -0.14478370547294617, 0.13987226784229279, 0.17704395949840546, -0.048600293695926666, -0.09844444692134857, 0.06020348519086838, -0.08678677678108215, 0.004373413044959307, 0.049656711518764496, -0.12774257361888885, 0.07378268241882324, -0.026915766298770905, 0.22198043763637543, 0.003928339574486017, -0.2594676911830902, -0.015265131369233131, -0.3152969777584076, -0.23811420798301697, -0.06307139992713928, -0.02132096327841282, -0.1651475727558136, -0.01754828169941902, 0.10955828428268433, -0.09235599637031555, 0.014830052852630615, 0.18101468682289124, 0.10512221604585648, -0.09742675721645355, 0.18350261449813843, -0.08805388957262039, 0.05883563309907913, 0.022595347836613655, 0.09127139300107956, -0.14423127472400665, -0.1491382122039795, -0.20034222304821014, -0.024371342733502388, -0.13007979094982147, 0.11079102754592896, 0.11698739975690842, 0.047641098499298096, -0.10074461996555328, -0.05829496681690216, -0.004804807715117931, -0.029349172487854958, -0.10408347845077515, 0.0792296826839447, -0.1915513426065445, -0.07168276607990265, 0.2391432374715805, -0.18687660992145538, -0.004957166966050863, 0.009398075751960278, -0.1110558956861496, -0.017476139590144157, 0.1007886454463005, -0.26728159189224243, -0.10453075170516968, -0.0048607042990624905, 0.04588520526885986, 0.17735645174980164, -0.04145287349820137, 0.07248875498771667, -0.1752977818250656, -0.020942695438861847, 0.024980204179883003, 0.01173702534288168, 0.3221777379512787, 0.04012160375714302, -0.09974762052297592, 0.1500234603881836, 0.22498874366283417, -0.024076074361801147, -0.008587765507400036, 0.417177677154541, 0.050598420202732086, -0.09680819511413574, 0.2601469159126282, 0.1449030041694641, -0.06766068935394287, -0.013390395790338516, 0.029086697846651077, -0.08258231729269028, -0.01302263606339693, 0.08024097234010696, -0.05060330033302307, -0.010729395784437656, -0.14142265915870667, -0.19455517828464508, 0.05193638429045677, -0.004564463160932064, -0.12764449417591095, -0.05148765817284584, 0.1782812923192978, 0.003154737874865532, -0.08620639890432358, 0.2374604493379593, 0.00559307262301445, -0.03278343752026558, -0.14136803150177002, -0.025599604472517967, -0.07562381029129028, -0.2595251500606537, -0.07129789888858795, -0.1378178894519806, -0.11914966255426407, 0.0027426546439528465, -0.06292937695980072, 0.17892728745937347, 0.11938019841909409, 0.21665480732917786, 0.06865353137254715, 0.0633467435836792, -0.1077849343419075, 0.36677008867263794, -0.046680424362421036, 0.06804796308279037, -0.13470329344272614, -0.04959569871425629, 0.10257532447576523, -0.14891968667507172, 0.12063057720661163, 0.08828137814998627, -0.025102857500314713}, {-0.097211092710495, -0.07167046517133713, -0.1527804732322693, 0.07430177927017212, -0.1295291781425476, 0.04639437049627304, 0.1833317130804062, -0.18064621090888977, -0.026401719078421593, -0.0565473847091198, 0.0481058768928051, -0.10035915672779083, 0.08490731567144394, -0.0038208465557545424, -0.0610596239566803, -0.10916336625814438, 0.21974575519561768, 0.0993930995464325, 0.1671716570854187, 0.11333978921175003, 0.2087348997592926, -0.12371806800365448, 0.1497558057308197, -0.08929677307605743, 0.13779853284358978, 0.029874421656131744, 0.028030091896653175, -0.14573396742343903, -0.11256079375743866, 0.12829601764678955, -0.04920584335923195, -0.28247809410095215, -0.14085672795772552, -0.08185011148452759, 0.020142432302236557, -0.06413742899894714, -0.08824582397937775, 0.08664937317371368, -0.07729624956846237, -0.09732475131750107, -0.07193930447101593, -0.040191374719142914, 0.37346896529197693, -0.09889958798885345, 0.023735759779810905, -0.11439913511276245, 0.012661673128604889, -0.06654129922389984, -0.06673935800790787, 0.05922214686870575, 0.11953135579824448, 0.2737535834312439, 0.10793653875589371, 0.022841526195406914, -0.028819436207413673, 0.04356289654970169, -0.09193508327007294, -0.033743828535079956, -0.18749357759952545, 0.1911827027797699, 0.07440735399723053, -0.20318515598773956, 0.08969181776046753, -0.0053398567251861095, -0.024611715227365494, -0.12042711675167084, -0.07232341170310974, 0.0408991239964962, 0.12179660052061081, 0.07204948365688324, 0.11100082099437714, -0.11754289269447327, -0.2786710858345032, 0.08973823487758636, -0.044976986944675446, 0.12889209389686584, -0.02148824743926525, -0.032179947942495346, -0.30830588936805725, -0.21852347254753113, 0.09317554533481598, -0.07878237962722778, -0.03469444811344147, 0.15975981950759888, 0.21025621891021729, -0.15585441887378693, 0.15396000444889069, -0.04229909926652908, -0.03930993005633354, -0.12611618638038635, 0.034023467451334, -0.02932819165289402, 0.06953181326389313, 0.021731220185756683, 0.030554573982954025, 0.08985649794340134, 0.0347147211432457, -0.1760353446006775, 0.013107703067362309, 0.1790672093629837, -0.02832168899476528, 0.07914111763238907, 0.11925355345010757, -0.06591532379388809, -0.14085808396339417, -0.06207730621099472, -0.09484042972326279, -0.08281122893095016, -0.008159352466464043, 0.23581016063690186, -0.32514268159866333, -0.22481191158294678, 0.011994284577667713, 0.06318577378988266, 0.06598269939422607, 0.2735660970211029, 0.06207086145877838, -0.05535910278558731, -0.11640838533639908, -0.17093007266521454, -0.18074078857898712, 0.03038797341287136, -0.0528125986456871, 0.05430198460817337, 0.05840236321091652, 0.23212972283363342, 0.03412552922964096, 0.06417107582092285, -0.09409718960523605, 0.030520640313625336, 0.23106072843074799, 0.07731642574071884, 0.001475112745538354, -0.045457568019628525, -0.06286957859992981, -0.00932835228741169, 0.09336303174495697, -0.060516078025102615, 0.08212730288505554, 0.08311288058757782, -0.29076603055000305, 0.029302403330802917, -0.086717389523983, 0.07991188019514084, 0.05093055218458176, -0.056478194892406464, -0.07902923971414566, 0.10196338593959808, -0.0516536608338356, -0.18057265877723694, 0.00046234382898546755, 0.08045997470617294, 0.13649174571037292, -0.12203249335289001, -0.040622420608997345, 0.20891663432121277, 0.17447032034397125, -3.9901391573948786e-05, 0.07431421428918839, 0.2639637589454651, 0.1351001262664795, -0.06290168315172195, -0.2120819091796875, -0.050443436950445175, -0.0708751231431961, 0.1633249968290329, 0.07421721518039703, 0.09946533292531967, -0.2137616127729416, 0.23171842098236084, 0.17703160643577576, -0.027554253116250038, -0.11929880827665329, -0.02490996941924095, -0.0490298792719841, 0.020808730274438858, 0.18908020853996277, 0.09794444590806961, -0.13316956162452698, 0.18350298702716827, 0.06279514729976654, -0.15888501703739166, -0.08555281907320023, -0.24129585921764374, 0.009076578542590141, -0.09825459867715836, 0.21851293742656708, -0.08147677779197693, -0.02442309819161892, 0.11310968548059464, -0.009450418874621391, -0.06320594996213913, -0.08537774533033371, -0.25050386786460876, -0.16508929431438446, -0.011058328673243523, -0.1542353630065918, -0.19202494621276855, 0.11372885853052139, -0.005745104048401117, 0.0831393152475357, -0.18272368609905243, 0.02041296660900116, 0.09001772105693817, -0.07127721607685089, -0.000721378019079566, -0.2645173668861389, -0.11117777973413467, -0.00837776344269514, 0.008676161989569664, -0.11491195112466812, 0.15632641315460205, 0.05030376836657524, -0.12629403173923492, 0.02714385837316513, 0.13222981989383698, -0.12827631831169128, 0.017939012497663498, -0.06159025803208351, 0.10427498817443848, -0.09412986785173416, 0.033714812248945236, -0.08109983801841736, 0.19481705129146576, 0.17561765015125275, 0.13152563571929932, -0.43055930733680725, 0.11200816184282303, 0.08522855490446091, 0.08919642865657806, -0.050763703882694244, 0.07870075851678848, -0.0259383711963892, 0.08973169326782227, -0.13182513415813446, 0.04921242967247963, 0.02547363191843033, 0.06631843000650406, -0.005608372390270233, 0.2635536193847656, 0.0955597311258316, 0.06271591037511826, 0.2357621192932129, -0.0979374423623085, -0.0965266302227974, 0.03652802109718323, -0.004432967863976955, -0.002779716858640313, -0.04015493020415306, -0.2948109209537506, 0.040641818195581436, -0.07743071764707565, 0.09410487860441208, -0.019468456506729126, -0.046756189316511154, -0.12138957530260086}, {0.16762703657150269, 0.1146351769566536, -0.030704889446496964, -0.026997709646821022, 0.05863674357533455, -0.0934574231505394, -0.17444877326488495, 0.08792530000209808, 0.04418647289276123, -0.00431370222941041, 0.07950650900602341, 0.19946323335170746, -0.16441650688648224, 0.0032533174380660057, 0.2290506511926651, 0.10726047307252884, 0.007091260049492121, 0.15235033631324768, -0.11617670953273773, -0.026706717908382416, -0.04748915508389473, -0.10964234918355942, 0.01714775711297989, 0.193662628531456, -0.11086063086986542, 0.0513761080801487, -0.005998148117214441, 0.023249655961990356, -0.06402358412742615, 0.12328752875328064, 0.11663977801799774, -0.06582114845514297, -0.06052000820636749, 0.03716852515935898, 0.12176837772130966, 0.011021328158676624, 0.007331931032240391, -0.09769294410943985, -0.09405337274074554, 0.052934084087610245, 0.052955225110054016, -0.10431578755378723, -0.10082320868968964, 0.1675858050584793, -0.03060029447078705, -0.05920639634132385, 0.09219904243946075, -0.06461706757545471, 0.1672006994485855, 0.05887347087264061, -0.25557562708854675, 0.18367105722427368, -0.0014619191642850637, 0.06823486089706421, 0.03585776314139366, 0.20897556841373444, 0.19136348366737366, -0.10773034393787384, 0.11002229899168015, -0.24492402374744415, 0.0560150146484375, -0.0916428416967392, 0.06161556765437126, 0.016173217445611954, 0.013004113920032978, -0.14720530807971954, -0.156132772564888, 0.08236230909824371, 0.11109106242656708, 0.060002654790878296, 0.1660163551568985, -0.041116368025541306, 0.14003382623195648, -0.05291825532913208, -0.12751396000385284, -0.08735258132219315, 0.13605695962905884, -0.3155818581581116, 0.11965986341238022, -0.11319178342819214, -0.15202948451042175, -0.04626545310020447, 0.05165364220738411, 0.04021364822983742, 0.013542440719902515, -0.048507675528526306, -0.07046183943748474, -0.30633726716041565, 0.000979167059995234, -0.019974179565906525, -0.062183596193790436, -0.2753542959690094, -0.1397445797920227, -0.028689125552773476, 0.195929154753685, 0.13065704703330994, 0.25171980261802673, -0.04632125794887543, 0.20982256531715393, 0.0026552218478173018, -0.046716224402189255, 0.1064605712890625, -0.14713048934936523, -0.015056946314871311, -0.08326878398656845, 0.19861319661140442, 0.10640717297792435, -0.005802239291369915, 0.18192170560359955, 0.039683952927589417, 0.12391937524080276, 0.12996479868888855, -0.05916226655244827, 0.020408421754837036, -0.18295679986476898, 0.10743122547864914, 0.07865938544273376, -0.12925513088703156, 0.14553256332874298, -0.056227874010801315, 0.12937811017036438, -0.051229652017354965, -0.04647165909409523, 0.16032609343528748, 0.019630344584584236, 0.218757763504982, -0.22727257013320923, 0.059347447007894516, 0.05947035551071167, -0.07597948610782623, 0.008539782837033272, -0.015331653878092766, -0.0016616423381492496, 0.0487244613468647, 0.07100454717874527, -0.4104214012622833, 0.030191753059625626, 0.0951726958155632, -0.02019554376602173, -0.08038774132728577, 0.02287496067583561, -0.14164748787879944, 0.0017280898755416274, -0.2452394813299179, 0.08983606845140457, -0.03796067088842392, 0.4322112798690796, -0.12717144191265106, -0.01588231325149536, -0.007587112952023745, -0.03892620652914047, 0.0799490362405777, 0.016873015090823174, 0.03440477326512337, -0.01255030557513237, 0.024625428020954132, -0.0652407705783844, 0.10212762653827667, 0.12358584254980087, -0.0757332444190979, -0.17283640801906586, 0.011830736882984638, 0.019872119650244713, -0.11867057532072067, -0.07025931775569916, -0.11943654716014862, -0.011412784457206726, 0.3100528419017792, -0.10907810181379318, -0.027052514255046844, 0.02902473323047161, 0.0390470065176487, -0.0036442792043089867, 0.04038042947649956, 0.1363336741924286, -0.019378570839762688, -0.020089656114578247, -0.06830095499753952, 0.06002490967512131, -0.04536167159676552, 0.09759613126516342, 0.1228654757142067, -0.13309356570243835, 0.19120123982429504, -0.03543413430452347, 0.038165681064128876, -0.05512618273496628, -0.08046616613864899, 0.08396364748477936, 0.07750794291496277, 0.14284688234329224, 0.05615002289414406, -0.2148757427930832, -0.08055863529443741, -0.0920301005244255, 0.2716377079486847, -0.044034723192453384, 0.5665733218193054, 0.09873825311660767, 0.08305662870407104, 0.011250869370996952, 0.08092605322599411, -0.10984502732753754, -0.0915834978222847, 0.12404382973909378, 0.12890994548797607, 0.08024988323450089, 0.1702517867088318, -0.06720862537622452, -0.08265233039855957, 0.03315429016947746, -0.09881159663200378, -0.16305571794509888, -0.010746777057647705, 0.21337871253490448, 0.060030508786439896, -0.052148304879665375, -0.11837081611156464, 0.0706213191151619, 0.03483741357922554, 0.14603780210018158, 0.08147025853395462, 0.20050913095474243, -0.1284051388502121, -0.161111518740654, 0.0252667386084795, 0.05130237340927124, 0.0051493290811777115, 0.0016610632883384824, -0.06599290668964386, 0.012952332384884357, 0.15087318420410156, 0.11448436975479126, -0.008799638599157333, -0.003803543047979474, -0.020146476104855537, 0.04060357064008713, -0.021516263484954834, -0.1565888375043869, -0.03165047988295555, -0.03419816121459007, 0.040980130434036255, -0.04119798541069031, -0.052809711545705795, -0.1353764533996582, 0.038343943655490875, -0.06967891752719879, 0.175058975815773, 0.041916947811841965, -0.10227035731077194, 0.17984187602996826, 0.08915449678897858, -0.12186481058597565, -0.10010772198438644, -0.10951481014490128, 0.0024104630574584007}, {0.03639492392539978, 0.25005072355270386, -0.02453678287565708, -0.15141230821609497, 0.11229749023914337, -0.12560957670211792, 0.157371386885643, -0.08574984222650528, -0.058735594153404236, 0.260315477848053, -0.07814955711364746, -0.10315315425395966, 0.15084145963191986, 0.04450390487909317, 0.16489896178245544, -0.04511919990181923, 0.15321621298789978, 0.03313356265425682, -0.11593487113714218, -0.048975054174661636, 0.060360293835401535, -0.07373136281967163, 0.08292178809642792, -0.18968643248081207, -0.07608340680599213, -0.014519549906253815, 0.05385804921388626, -0.19294293224811554, 0.008190651424229145, -0.20985764265060425, 0.14474044740200043, 0.2013833224773407, -0.047825247049331665, 0.192788228392601, -0.06951819360256195, 0.020890330895781517, 0.019683167338371277, -0.2089146077632904, 0.05832960456609726, -0.0771460011601448, 0.10264682024717331, -0.014250200241804123, 0.023749517276883125, 0.19337332248687744, -0.004989677108824253, -0.1342988908290863, 0.0011298962635919452, -0.050459638237953186, -0.08575824648141861, 0.2624869644641876, 0.07982932776212692, 0.1163865327835083, 0.11625806242227554, -0.10984964668750763, 0.12285849452018738, -0.1584910899400711, 0.1404063105583191, 0.04595310240983963, -0.07624150812625885, 0.10161146521568298, -0.17095154523849487, 0.195109561085701, -0.16305853426456451, -0.09216809272766113, 0.06495700031518936, 0.10548406094312668, 0.1509384661912918, -0.19370664656162262, -0.04802129790186882, 0.037086885422468185, -0.0377592071890831, -0.05098900571465492, -0.04339020699262619, 0.10820423811674118, 0.1534012407064438, -0.18484699726104736, -0.2787841856479645, 0.23868899047374725, 0.17140206694602966, 0.08512596040964127, -0.08884582668542862, -0.0008750785491429269, 0.25781330466270447, 0.09722713381052017, 0.02437140792608261, 0.22233015298843384, 0.2933506667613983, 0.0072977556847035885, 0.014091387391090393, -0.11203877627849579, 0.07024924457073212, 0.18557259440422058, 0.14747437834739685, 0.07576069235801697, 0.06757055222988129, 0.18321342766284943, 0.22875036299228668, -0.018580695614218712, -0.38631513714790344, 0.18990333378314972, 0.1854613870382309, 0.17570166289806366, 0.28676000237464905, -0.10625331103801727, 0.04369794949889183, 0.030433794483542442, -0.10056103020906448, 0.05973739176988602, 0.01087727677077055, 0.1928769201040268, -0.22705607116222382, -0.18220563232898712, -0.07803458720445633, -0.014639981091022491, 0.20888446271419525, 0.32545769214630127, 0.14714811742305756, 0.03534986078739166, -0.13291187584400177, -0.13694977760314941, -0.06970374286174774, -0.15069784224033356, 0.08244708925485611, 0.17676898837089539, 0.12749013304710388, -0.039577800780534744, -0.11106375604867935, -0.0980534628033638, -0.039277855306863785, 0.09201571345329285, 0.1209605261683464, -0.14767974615097046, -0.10243739187717438, 0.0322674885392189, -0.2810218334197998, 0.12985822558403015, -0.019403206184506416, 0.05647881701588631, 0.2389051914215088, -0.028022123500704765, -0.15212877094745636, 0.1436053067445755, 0.3021157383918762, -0.3082526624202728, -0.10440931469202042, -0.04356793314218521, 0.03851702809333801, 0.16560755670070648, -0.2488679438829422, 0.04472590610384941, -0.09366386383771896, 0.1434679627418518, 0.0641949400305748, 0.05538148432970047, 0.15148510038852692, 0.05421615391969681, 0.1484193205833435, -0.3269653618335724, 0.13542035222053528, 0.244866743683815, 0.07486840337514877, -0.02747979760169983, -0.12715893983840942, -0.02137129195034504, 0.19312064349651337, 0.24469231069087982, -0.08156852424144745, 0.06491348892450333, -0.047726552933454514, 0.07729167491197586, -0.15845103561878204, -0.09416860342025757, 0.18905334174633026, -0.00494432682171464, -0.15330366790294647, -0.18537476658821106, -0.021213047206401825, 0.13284841179847717, 0.03920484334230423, -0.2114657759666443, -0.0754997655749321, -0.3140854835510254, 0.052367959171533585, -0.11183137446641922, 0.07500219345092773, -0.12106496095657349, -0.11124230921268463, 0.010886643081903458, -0.23582372069358826, 0.007089570164680481, 0.18253184854984283, -0.039079699665308, -0.06603032350540161, -0.037075839936733246, 0.12270160019397736, 0.21974821388721466, 0.03924233093857765, -0.02869911678135395, -0.09298042207956314, 0.06462886929512024, -0.24797478318214417, -0.103801429271698, -0.17151732742786407, -0.0621444433927536, 0.014523258432745934, -0.0006139578181318939, 0.0007902804645709693, -0.09321697801351547, 0.0955248773097992, -0.0007757682469673455, -0.2847341299057007, -0.060318075120449066, -0.2188444286584854, -0.08650463819503784, -0.20867718756198883, -0.14156579971313477, 0.04074499383568764, 0.12473611533641815, 0.012956950813531876, -0.10771021246910095, 0.1439327448606491, 0.008287199772894382, -0.15211325883865356, 0.05846491828560829, 0.027246618643403053, -0.01256502140313387, -0.03019283339381218, -0.017163071781396866, 0.1251736581325531, 0.22916458547115326, 0.06960605829954147, 0.18549767136573792, 0.1127937063574791, -0.23123331367969513, -0.1315508633852005, -0.1790841668844223, 0.05156540870666504, -0.1195310726761818, 0.02697785571217537, 0.10782705247402191, 0.17617911100387573, -0.031281597912311554, 0.23560504615306854, 0.05900295823812485, 0.10912077128887177, -0.06353583931922913, -0.17788156867027283, 0.26235178112983704, 0.11657453328371048, 0.01191747561097145, -0.17087934911251068, 0.2608947455883026, 0.08955612778663635, -0.35330188274383545, -0.11484680324792862, 0.102354995906353}, {0.05385240167379379, 0.047507017850875854, 0.21922814846038818, 0.17549476027488708, -0.277293860912323, -0.052974581718444824, 0.0426703505218029, 0.06878745555877686, 0.1943669617176056, -0.11493103951215744, -0.1415676325559616, 0.2135792374610901, 0.026828167960047722, 0.16715039312839508, -0.16724984347820282, 0.26727601885795593, 0.22036735713481903, -0.05215749889612198, 0.0345834381878376, -0.07117138057947159, 0.1948743760585785, 0.01677662506699562, -0.1445140838623047, 0.15503643453121185, -0.17303934693336487, 0.1315554678440094, -0.08430643379688263, -0.061571404337882996, -0.19369065761566162, -0.05306096002459526, 0.13023632764816284, -0.46416735649108887, 0.03304872661828995, 0.022605709731578827, -0.10186503827571869, -0.001207691035233438, 0.03185071423649788, -0.15768666565418243, -0.09553979337215424, 0.1289781928062439, 0.1400132030248642, -0.002372966380789876, 0.007328686770051718, -0.42651528120040894, 0.09729515761137009, -0.0823369100689888, 0.14223632216453552, 0.05891960859298706, -0.1088193878531456, 0.05567237362265587, -0.07546374946832657, 0.08392121642827988, 0.03075588122010231, 0.14042127132415771, 0.12906506657600403, 0.08837821334600449, 0.2287496030330658, 0.12339477241039276, -0.27315858006477356, -0.1199335902929306, 0.0476812869310379, -0.04014793783426285, 0.05873868986964226, 0.06157384812831879, 0.15558792650699615, -0.03152388334274292, 0.11442156881093979, -0.03668241575360298, 0.16632546484470367, 0.09312901645898819, 0.07440720498561859, 0.01252572052180767, -0.00264136865735054, 0.1359594166278839, 0.09156683087348938, -0.09280847012996674, -0.18271511793136597, 0.01760145090520382, -0.4620814323425293, -0.029203688725829124, 0.24267922341823578, -0.02054024487733841, -0.2230970710515976, 0.033240024000406265, 0.0010563035029917955, 0.040735021233558655, 0.06272081285715103, -0.06274468451738358, -0.006255654618144035, -0.20267246663570404, 0.03293114900588989, -0.11719702184200287, -0.24011538922786713, 0.16587750613689423, 0.3556654453277588, 0.18301326036453247, -0.04978350177407265, 0.08933225274085999, 0.4082394540309906, 0.17484524846076965, 0.02956247702240944, -0.01467453595250845, -0.11664250493049622, 0.08401181548833847, -0.1232244223356247, 0.09295691549777985, -0.14141561090946198, -0.15714870393276215, 0.2448071390390396, -0.017514795064926147, -0.040525760501623154, -0.1048937514424324, 0.08178514242172241, 0.14382526278495789, 0.0019349867943674326, -0.06586188077926636, -0.05707420036196709, -0.1342027634382248, 0.08963508158922195, 0.23560591042041779, -0.07582421600818634, 0.08427619934082031, -0.08412078022956848, 0.12705819308757782, -0.16279341280460358, 0.4126549959182739, -0.024469392374157906, 0.028806865215301514, 0.08046923577785492, -0.006946017500013113, 0.22292348742485046, 0.014382956549525261, 0.1112835481762886, 0.14939267933368683, -0.055137790739536285, 0.026637474074959755, -0.156428262591362, -0.10617166012525558, -0.18228018283843994, -0.00446542352437973, -0.059707287698984146, -0.09932305663824081, -0.06899060308933258, 0.12308457493782043, 0.10783571749925613, 0.07063842564821243, 0.03065209649503231, -0.05996878817677498, 0.023486964404582977, -0.043788861483335495, -0.01303676888346672, 0.047015994787216187, -0.036249544471502304, 0.02024950459599495, -0.07211649417877197, 0.12050122022628784, -0.11236979812383652, -0.024704216048121452, 0.22210150957107544, -0.044522177428007126, 0.04606969654560089, -0.08511829376220703, -0.14833024144172668, 0.04392806068062782, 0.06819041073322296, -0.19976328313350677, 0.00936259888112545, 0.0666748508810997, -0.06169413402676582, 0.1566658765077591, -0.004577120300382376, 0.15494811534881592, 0.048203375190496445, -0.07863727957010269, -0.012926172465085983, -0.23466414213180542, -0.010116634890437126, 0.098734050989151, -0.10368861258029938, 0.05280714109539986, 0.08790544420480728, 0.3383816182613373, -0.10680580884218216, 0.06995319575071335, -0.01693296805024147, -0.13050463795661926, -0.2575856149196625, -0.1550094336271286, 0.08728175610303879, -0.0943606048822403, 0.037401989102363586, -0.16480934619903564, 0.0034949558321386576, 0.10321386158466339, 0.032538704574108124, 0.02151288278400898, -0.043113112449645996, 0.12750473618507385, -0.08996674418449402, -0.21603459119796753, -0.0393809974193573, -0.17416349053382874, 0.07847046107053757, 0.39054015278816223, -0.03460568189620972, -0.07958073168992996, 0.1694411039352417, 0.1346825361251831, 0.04937580227851868, 0.057420749217271805, -0.06905841082334518, -0.1861635148525238, -0.07120050489902496, -0.04857322573661804, -0.02506435103714466, -0.012702667154371738, 0.04445793852210045, -0.060779668390750885, 0.01706808991730213, -0.04747115820646286, -0.01895451918244362, -0.0017659708391875029, -0.06377073377370834, -0.1882910430431366, -0.047658223658800125, -0.25037136673927307, -0.1481061428785324, -0.0926540195941925, 0.14649862051010132, 0.01931498572230339, 0.019232193008065224, 0.17677827179431915, 0.036314837634563446, -0.03771296516060829, -0.26532503962516785, 0.1204029768705368, -0.13849151134490967, 0.0037483079358935356, -0.04646269977092743, 0.1890038251876831, 0.0773981437087059, 0.24696063995361328, -0.004367704037576914, -0.0042106700129806995, 0.09469561278820038, 0.11199343949556351, -0.021809102967381477, 0.14752714335918427, 0.03457805886864662, -0.2064180076122284, -0.002804394345730543, -0.22105318307876587, -0.09660040587186813, -0.149234339594841, -0.27080392837524414, 8.252506813732907e-05}, {0.040786415338516235, -0.06203655153512955, -0.09151174128055573, -0.2175208032131195, 0.1005411446094513, 0.11423824727535248, -0.30769041180610657, 0.2732422351837158, 0.05367467552423477, 0.008616995997726917, -0.11089278757572174, -0.23667290806770325, -0.22139529883861542, 0.03144330903887749, -0.0339943952858448, -0.0020973370410501957, -0.1292884796857834, 0.04271407425403595, 0.04492393136024475, 0.3538544774055481, -0.007796948775649071, 0.0674985870718956, 0.11636708676815033, 0.2708965539932251, 0.11645670980215073, 0.19480140507221222, -0.10772176831960678, 0.024096861481666565, 0.12083372473716736, 0.1735740602016449, -0.2566869258880615, 0.13735002279281616, -0.10716378688812256, 0.3017370104789734, 0.2618175148963928, 0.007775305304676294, -0.076071597635746, -0.057858552783727646, -0.01186317391693592, 0.017477046698331833, 0.03903041407465935, -0.11405143141746521, 0.0915045440196991, -0.05971894785761833, 0.017585447058081627, -0.10634639859199524, -0.08872834593057632, -0.07541454583406448, -0.07130569964647293, 0.07231241464614868, 0.13437016308307648, 0.22662043571472168, -0.20048199594020844, 0.028073307126760483, -0.09689415991306305, -0.0031618280336260796, -0.057139743119478226, -0.14889663457870483, 0.0756467655301094, -0.07726623862981796, -0.14748619496822357, 0.11709710210561752, -0.09053286164999008, 0.013052916154265404, 0.008678251877427101, -0.14778944849967957, -0.055487532168626785, -0.3247068524360657, -0.19326557219028473, -0.09295007586479187, -0.18411032855510712, 0.06089043244719505, 0.0824945792555809, -0.01277218759059906, -0.13127122819423676, -0.0179139431566, -0.09121771901845932, 0.05860763043165207, -0.061084240674972534, 0.17412148416042328, -0.12279023975133896, 0.21230924129486084, 0.09696533530950546, 0.08374963700771332, -0.04486466571688652, -0.10858963429927826, -0.04843498393893242, -0.022095145657658577, -0.10030978173017502, 0.02987855300307274, 0.039724357426166534, 0.009359098970890045, -0.07928802818059921, -0.027034945785999298, -0.1689663678407669, -0.09720566868782043, 0.04833624139428139, -0.010221781209111214, -0.053280867636203766, 0.016421258449554443, -0.0910763144493103, -0.15439869463443756, 0.1151864156126976, -0.1466054767370224, -0.07914483547210693, 0.3660162389278412, 0.22142599523067474, 0.23499853909015656, -0.015028178691864014, -0.005596228875219822, 0.06503093242645264, -0.010240918956696987, 0.05262896418571472, -0.09137196838855743, 0.08404038101434708, -0.04967384785413742, 0.0910443365573883, -0.0035658429842442274, -0.2974754869937897, -0.09667164087295532, -0.09661813080310822, -0.024092845618724823, 0.20680344104766846, -0.1973232924938202, -0.343384325504303, 0.11566419899463654, 0.028996625915169716, -0.19465331733226776, 0.09074531495571136, 0.38918259739875793, 0.18082180619239807, 0.10156401991844177, -0.11986413598060608, 0.01327377837151289, -0.006914815399795771, 0.17327436804771423, 0.11531778424978256, -0.20190583169460297, 0.11109749972820282, 0.028733834624290466, 0.001696973224170506, -0.02384302392601967, 0.0823678970336914, -0.05861495062708855, -0.08872421085834503, -0.0410795696079731, 0.12994490563869476, 0.06906130164861679, -0.1149880513548851, -0.0232076533138752, -0.05339929088950157, 0.08073052763938904, 0.1115206703543663, 0.022273672744631767, 0.08393172174692154, -0.11366640031337738, -0.1308285892009735, 0.11472391337156296, -0.029738983139395714, -0.04676114395260811, 0.0747598260641098, 0.003032477106899023, -0.011046805419027805, -0.1477832794189453, -0.31452053785324097, -0.2674516439437866, -0.16694796085357666, 0.1889069676399231, 0.20364023745059967, 0.11249396204948425, 0.16881749033927917, 0.16724121570587158, 0.0665750578045845, 0.06799470633268356, 0.12180410325527191, 0.02849433571100235, 0.15465496480464935, 0.2918555736541748, -0.09003685414791107, -0.014880816452205181, -0.005724349059164524, -0.27026721835136414, 0.16162621974945068, -0.09386511892080307, 0.11696270853281021, 0.041265856474637985, 0.14555683732032776, 0.20782293379306793, -0.39269450306892395, -0.1914338767528534, 0.0593935027718544, 0.1911054402589798, -0.10778792202472687, -0.06193187087774277, 0.25141236186027527, -0.09460808336734772, 0.12912045419216156, -0.09538355469703674, -0.16091126203536987, -0.07186687737703323, -0.23933199048042297, 0.03538050130009651, -0.03267859295010567, -0.21649041771888733, -0.1351848989725113, -0.18277475237846375, 0.14283612370491028, -0.06924637407064438, -0.03472995012998581, -0.01557131391018629, -0.2047746628522873, 0.24270884692668915, -0.012647523544728756, -0.09286712855100632, 0.021819962188601494, 0.20733091235160828, -0.07158972322940826, -0.071660116314888, 0.1473088413476944, 0.1327737718820572, -0.10493652522563934, 0.22817476093769073, -0.23182933032512665, 0.22727136313915253, 0.21098209917545319, 0.08650398999452591, -0.21956419944763184, -0.08032704144716263, -0.0464395247399807, 0.20306307077407837, 0.1142229214310646, 0.02667241543531418, 0.05879013240337372, 0.10228300094604492, 0.14550505578517914, -0.0894334465265274, -0.020333824679255486, -0.0026474406477063894, -0.21652783453464508, 0.25146761536598206, 0.08787555247545242, -0.12476403266191483, -0.0354597233235836, -0.2851954400539398, 0.2458953857421875, -0.17446769773960114, -0.06227865070104599, 0.1848745346069336, 0.11398224532604218, -0.22182144224643707, -0.08359295129776001, 0.09385322034358978, -0.059930771589279175, -0.1191711351275444, 0.11208256334066391, 0.0677480697631836}, {0.19079066812992096, 0.24278178811073303, 0.03019036166369915, -0.08212833106517792, -0.11695174872875214, -0.12378127127885818, -0.08206643164157867, -0.17053692042827606, -0.20290955901145935, -0.07752526551485062, -0.03563927486538887, -0.1860773265361786, -0.13798238337039948, -0.0560455359518528, 0.057065967470407486, 0.1772761344909668, -0.18770451843738556, 0.05471523851156235, 0.15678638219833374, -0.1272544413805008, 0.01736474595963955, 0.05243130028247833, 0.03131552040576935, -0.25595834851264954, -0.27004796266555786, -0.024732977151870728, 0.1656169444322586, -0.014650044031441212, 0.09971007704734802, -0.05479820445179939, 0.011717785149812698, 0.33635562658309937, -0.19290843605995178, -0.029741158708930016, -0.09198450297117233, -0.04425814002752304, -0.01799161732196808, -0.2603713572025299, -0.101760134100914, -0.08592427521944046, 0.15489283204078674, -0.14868222177028656, -0.1520306020975113, 0.065040722489357, 0.04887225478887558, -0.031133299693465233, -0.02178310975432396, -0.10346853733062744, 0.3363584876060486, 0.08723435550928116, -0.1764838844537735, 0.2319021373987198, 0.10986286401748657, -0.1093815341591835, 0.018933573737740517, 0.0031073701102286577, -0.02036426030099392, 0.04679291322827339, -0.1858540028333664, -0.09272786974906921, 0.08218676596879959, 0.004442308563739061, -0.05960862338542938, -0.17396675050258636, 0.21361498534679413, -0.4355030953884125, -0.016968203708529472, 0.0631326362490654, 0.047045957297086716, 0.09219495207071304, 0.13823547959327698, -0.07914920151233673, -0.09305544942617416, -0.2145392745733261, -0.04974738880991936, 0.027725767344236374, 0.08731894940137863, -0.26121264696121216, -0.11815167218446732, -0.21723921597003937, -0.12284205108880997, -0.0596187449991703, -0.20723526179790497, 0.06562381237745285, -0.032629091292619705, -0.08582533895969391, -0.029967596754431725, -0.19440822303295135, 0.010511030443012714, 0.019165920093655586, 0.16598805785179138, -0.0700654610991478, -0.009143112227320671, 0.0018632470164448023, -0.20719356834888458, 0.041106220334768295, 0.04580889269709587, 0.03563005104660988, 0.04084208607673645, 0.23166289925575256, -0.18454673886299133, -0.10317506641149521, -0.15482622385025024, 0.2326446771621704, -0.26899775862693787, 0.3901122212409973, 0.030412454158067703, -0.019604401662945747, -0.13496753573417664, -0.12870442867279053, 0.17737706005573273, -0.2813127040863037, 0.22619305551052094, -0.02037893980741501, -0.1270313858985901, 0.052423145622015, -0.027245912700891495, -0.01569383032619953, 0.1937343329191208, 0.1657143384218216, 0.1853470653295517, -0.2172226905822754, -0.23922643065452576, 0.10449903458356857, 0.3193928897380829, 0.2082636058330536, -0.09852636605501175, 0.032381586730480194, 0.10650402307510376, 0.17179447412490845, -0.13507342338562012, -0.11534900963306427, 0.2832720875740051, 0.3096346855163574, -0.09806542098522186, -0.15371055901050568, -0.2539896070957184, -0.027237016707658768, -0.14380061626434326, -0.0762634351849556, 0.008246688172221184, -0.013927696272730827, 0.11912258714437485, -0.14106686413288116, 0.07085982710123062, 0.1649087816476822, 0.19964854419231415, -0.194247767329216, -0.023944256827235222, -0.050628289580345154, -0.03130476921796799, 0.051691193133592606, 0.10907142609357834, -0.14099572598934174, 0.12709639966487885, -0.10890604555606842, -0.10020094364881516, -0.03351764753460884, -0.07204386591911316, -0.19447647035121918, -0.06813893467187881, -0.1913057267665863, -0.022930074483156204, -0.08710937947034836, 0.051058344542980194, 0.13865076005458832, -0.17405465245246887, -0.008546274155378342, 0.14554308354854584, -0.10736776143312454, 0.15225881338119507, -0.07082878053188324, -0.05934937670826912, -0.00416220398619771, 0.12623335421085358, -0.14987091720104218, 0.03158164396882057, -0.1511252373456955, -0.1274539977312088, 0.025194201618433, 0.09320308268070221, -0.1360454261302948, -0.09851282089948654, 0.030604921281337738, 0.1308705359697342, 0.042130500078201294, -0.32767316699028015, -0.005956706590950489, 0.18383465707302094, 0.15725846588611603, 0.14027459919452667, -0.11330000311136246, -0.04067289084196091, 0.045855436474084854, -0.36552929878234863, 0.09907068312168121, 0.10640288889408112, 0.10851359367370605, -0.16726912558078766, 0.002042714273557067, 0.18456535041332245, 0.11184459924697876, 0.01773269847035408, 0.007207438349723816, -0.1527826189994812, -0.124974824488163, -0.19837965071201324, -0.07727976888418198, -0.07256127893924713, 0.2760048508644104, 0.010593331418931484, -0.21227113902568817, -0.06957188248634338, 0.022399602457880974, 0.32787248492240906, 0.12975044548511505, -0.1967564970254898, -0.18415924906730652, -0.4276583194732666, 0.0037013341207057238, -0.058784373104572296, 0.13973824679851532, -0.18484139442443848, -0.1220463290810585, 0.011090290732681751, -0.17131981253623962, -0.22719725966453552, -0.18785697221755981, 0.05266900733113289, -0.09150584042072296, -0.023573430255055428, 0.01858142390847206, 0.08987253904342651, -0.06627847254276276, -0.18360677361488342, 0.15754973888397217, -0.2936667501926422, 0.15136390924453735, -0.09785407036542892, 0.08662620186805725, 0.23226632177829742, -0.03391958400607109, 0.0840168371796608, 0.10367540270090103, -0.18503697216510773, 0.06188802793622017, -0.07309430092573166, 0.10764309018850327, 0.032739970833063126, -0.17456196248531342, 0.12877504527568817, -0.06863798946142197, -0.030537407845258713, -0.15017661452293396, -0.1565675586462021, 0.24061544239521027}, {-0.18315096199512482, 0.09537769854068756, 0.03350864723324776, -0.05819399654865265, 0.152884379029274, 0.10738813132047653, 0.09279106557369232, 0.23004169762134552, -0.10782894492149353, 0.06477846950292587, -0.15636879205703735, -0.0011445846175774932, -0.08313480764627457, -0.013730756938457489, 0.10996659845113754, 0.2606945335865021, -0.12463925033807755, 0.022634433582425117, 0.3179451823234558, 0.21509212255477905, -0.22059717774391174, 0.023315412923693657, -0.19564928114414215, -0.0801817774772644, 0.054655611515045166, -0.05657948926091194, 0.119926318526268, -0.06812039762735367, -0.04425910860300064, -0.13223274052143097, 0.22922934591770172, 0.04122559726238251, -0.13275569677352905, -0.22689861059188843, -0.015954643487930298, -0.1748771071434021, -0.07841384410858154, -0.03798164054751396, 0.06819501519203186, 0.16857634484767914, -0.1816304326057434, -0.3009209930896759, -0.15736190974712372, -0.08698440343141556, -0.13473860919475555, 0.03895125910639763, -0.2797521650791168, 0.17478308081626892, 0.030756602063775063, 0.12314767390489578, -0.19875457882881165, 0.07883690297603607, -0.025530030950903893, -0.2489652782678604, 0.07306286692619324, 0.09069789201021194, 0.011376685462892056, -0.0635840892791748, -0.002849193522706628, 0.1600233018398285, -0.04693987965583801, -0.04230468347668648, 0.15334561467170715, 0.10769447684288025, 0.06256954371929169, -0.09658269584178925, -0.04049590229988098, -0.006037640385329723, 0.013137337751686573, 0.12210555374622345, -0.21623942255973816, 0.07011807709932327, -0.1201033964753151, -0.1192445456981659, 0.009791433811187744, 0.028108445927500725, -0.056167423725128174, -0.08268413692712784, 0.10810264945030212, -0.1629568636417389, -0.11636542528867722, 0.11258342117071152, -0.06275743991136551, -0.07657284289598465, 0.15277397632598877, -0.3909410238265991, -0.12195257842540741, 0.06669015437364578, 0.07034789770841599, 0.09270217269659042, -0.15984167158603668, -0.01885339245200157, 0.1897730529308319, -0.0916040688753128, -0.1305413544178009, 0.07385396957397461, 0.12744057178497314, -0.01270239893347025, -0.10097277164459229, 0.08219876885414124, -0.0681624785065651, 0.1335538625717163, 0.184878408908844, 0.0628560408949852, -0.29413655400276184, -0.5867302417755127, -0.16604982316493988, 0.14682793617248535, 0.3473254144191742, 0.08818010985851288, -0.11928176879882812, -0.04393910989165306, 0.07877296209335327, -0.20838037133216858, 0.11668123304843903, -0.15618351101875305, 0.022361567243933678, 0.007028986234217882, -0.06734300404787064, -0.03969881683588028, 0.12679074704647064, -0.20674918591976166, 0.0035007328260689974, -0.07146994024515152, -0.08409243077039719, 0.14255569875240326, -0.05922384187579155, 0.030492525547742844, 0.04680247977375984, 0.1581045538187027, 0.25269508361816406, 0.09529680013656616, 0.00930735282599926, -0.06864407658576965, -0.12294100970029831, 0.14790476858615875, 0.13397249579429626, -0.1986425668001175, -0.14168842136859894, -0.11641275137662888, -0.021717770025134087, -0.2079550325870514, -0.39368313550949097, -0.1358482539653778, -0.3020734488964081, 0.017888274043798447, -0.13043077290058136, -0.009531235322356224, -0.013598495163023472, 0.05821221321821213, 0.14082996547222137, 0.08392304182052612, 0.11219357699155807, -0.1014230027794838, 0.04264284670352936, 0.1442893147468567, 0.12058849632740021, 0.005584731698036194, -0.015721982344985008, 0.016321048140525818, 0.0702504813671112, -0.2761545777320862, -0.201560378074646, -0.21798788011074066, 0.0009030306828208268, -0.2510215938091278, 0.012575560249388218, 0.1106976792216301, 0.2645855247974396, 0.2119935005903244, 0.009710198268294334, -0.07395491749048233, 0.1764724850654602, -0.023172622546553612, -0.25256043672561646, -0.18602098524570465, -0.10113729536533356, -0.004095273558050394, -0.06144802272319794, -0.0882132276892662, -0.13001658022403717, -0.32812392711639404, -0.05778244510293007, -0.33547502756118774, 0.27915292978286743, 0.02013997919857502, 0.4877054691314697, -0.09836354851722717, -0.007425524760037661, -0.05281747132539749, 0.13176439702510834, 0.06262192875146866, -0.29820531606674194, -0.08333960920572281, -0.36921122670173645, -0.059772882610559464, 0.34612202644348145, 0.16375933587551117, -0.05053173005580902, -0.24582740664482117, -0.017574945464730263, 0.12227790057659149, 0.01967644691467285, 0.09634494036436081, 0.07176507264375687, 0.191210076212883, -0.018687158823013306, -0.10459840297698975, 0.059852052479982376, -0.05890728160738945, 0.0038620138075202703, 0.3987939953804016, -0.018456755205988884, -0.028584221377968788, -0.05056988075375557, -0.09507053345441818, -0.07183121889829636, -0.14869950711727142, 0.008707522414624691, 0.12406595051288605, 0.09145873785018921, -0.20230723917484283, 0.10541963577270508, 0.4957261085510254, 0.17003633081912994, 0.41882434487342834, -0.11656239628791809, -0.06109939515590668, -0.012217068113386631, 0.2974362373352051, -0.3604201376438141, -0.05780110880732536, -0.13757877051830292, 0.2615945339202881, -0.2807108759880066, -0.050705693662166595, -0.03974591940641403, -0.04819333925843239, 0.1137116402387619, -0.02648773044347763, 0.0642906203866005, 0.09633802622556686, -0.07906600087881088, 0.04606645181775093, 0.23299285769462585, 0.05333815887570381, -0.001209871843457222, 0.10402947664260864, 0.1102224737405777, -0.09501118957996368, -0.1550457924604416, 0.06779517233371735, 0.2917720079421997, 0.04760710522532463, -0.19369614124298096, -0.047376878559589386}, {0.007578146643936634, 0.084568090736866, -0.23582835495471954, 0.17399489879608154, -0.007728348486125469, 0.1120968833565712, 0.007358412723988295, -0.016285467892885208, -0.2190243899822235, 0.11481334269046783, -0.08772873133420944, 0.5171709060668945, -0.10096882283687592, 0.1280420422554016, 0.08606765419244766, 0.014833192341029644, -0.06493452936410904, 0.05651092529296875, 0.20018284022808075, 0.016491791233420372, 0.1613358110189438, 0.08659439533948898, -0.05904349312186241, 0.08828489482402802, -0.3730391561985016, 0.030291415750980377, 0.11603377759456635, -0.1281765103340149, -0.1561419516801834, -0.02367362007498741, -0.01521080918610096, 0.05636066198348999, -0.023132314905524254, -0.012533323839306831, -0.08173127472400665, -0.00475608604028821, -0.09567064046859741, 0.1814272403717041, -0.08105514943599701, 0.12033558636903763, -0.1696472465991974, -0.06816484779119492, -0.028848588466644287, 0.2253555953502655, -0.12920042872428894, 0.011596660129725933, -0.19570805132389069, 0.07724766433238983, -0.15324793756008148, 0.18064309656620026, 0.15235581994056702, -0.07513340562582016, 0.06734483689069748, 0.04868868738412857, -0.12202771753072739, 0.0544632263481617, 0.36006683111190796, 0.005147866439074278, 0.005397153552621603, -0.04611434414982796, 0.1267271339893341, -0.15240144729614258, 0.2391507476568222, -0.13393434882164001, 0.15318937599658966, -0.24102871119976044, -0.04388483241200447, 0.26262545585632324, 0.04994344338774681, 0.019900882616639137, 0.03526385873556137, -0.001981121953576803, -0.05194215476512909, -0.08982107043266296, -0.08828189224004745, 0.07259198278188705, -0.18789733946323395, 0.1678915023803711, -0.1344112604856491, -0.47414353489875793, -0.215852752327919, 0.10709379613399506, 0.21414715051651, 0.038150522857904434, 0.23628735542297363, -0.09461171925067902, 0.029279006645083427, 0.04270617291331291, -0.017346419394016266, -0.17674779891967773, 0.10501173138618469, 0.1349785327911377, -0.15850181877613068, -0.17157398164272308, 0.08175237476825714, -0.0961848571896553, 0.1208256185054779, -0.3095073699951172, 0.13039928674697876, 0.029487131163477898, 0.0517597571015358, 0.07636934518814087, -0.0990094542503357, -0.02008129097521305, -0.035361647605895996, 0.1587338149547577, -0.14103387296199799, 0.11564553529024124, -0.026863371953368187, -0.09127088636159897, 0.07500776648521423, -0.09786941856145859, 0.09571525454521179, 0.07815592736005783, -0.11909174174070358, 0.00757321622222662, 0.3054296374320984, -0.13652898371219635, 0.14672911167144775, -0.07617875188589096, 0.01994018256664276, 0.0400460809469223, -0.14325109124183655, 0.07375543564558029, 0.0707593783736229, 0.15576553344726562, 0.2795383334159851, -0.03275700658559799, 0.17596933245658875, 0.02545308880507946, 0.2832520604133606, 0.058120448142290115, -0.022590991109609604, -0.18913674354553223, 0.032802924513816833, -0.14852620661258698, -0.16286085546016693, 0.06076314300298691, 0.1100282222032547, -0.12284886837005615, -0.11639393121004105, 0.11407678574323654, -0.003362815361469984, -0.14275109767913818, -0.051633864641189575, -0.09391707926988602, 0.09354756772518158, 0.23906566202640533, -0.03539383038878441, -0.1943427175283432, 0.030306223779916763, 0.10863253474235535, 0.1763617843389511, 0.0707707330584526, 0.2310040146112442, 0.1602650135755539, -0.014756225980818272, 0.22610293328762054, 0.2641202211380005, -0.023062899708747864, -0.10113298147916794, 0.08737121522426605, 0.000669009517878294, 0.19894669950008392, 0.0699847862124443, 0.006452521309256554, -0.03201640024781227, 0.010946481488645077, -0.15309551358222961, -0.11733897030353546, 0.013271148316562176, -0.051156774163246155, -0.0980624258518219, 0.14069415628910065, 0.13664203882217407, -0.055905841290950775, -0.11526373028755188, -0.13624116778373718, -0.06165934354066849, 0.08209790289402008, 0.09169593453407288, 0.5550490617752075, -0.005642704665660858, -0.06383320689201355, -0.16248619556427002, 0.15752597153186798, -0.018863465636968613, 0.0878242626786232, 0.048452459275722504, -0.20853731036186218, -0.11430525034666061, -0.15656505525112152, -0.0441431924700737, -0.10333554446697235, -0.03385002166032791, -0.1909942328929901, 0.03937522694468498, 0.1606174111366272, -0.016807524487376213, 0.010190248489379883, 0.08249308168888092, -0.10315060615539551, 0.21911539137363434, -0.060837432742118835, -0.02993788570165634, 0.03990345075726509, -0.10237105935811996, 0.08097220212221146, 0.11204089224338531, 0.040119439363479614, 0.09736433625221252, -0.0451509952545166, 0.06280569732189178, -0.09026269614696503, -0.09117284417152405, 0.08691240847110748, 0.11823713779449463, -0.062041446566581726, -0.04030979424715042, 0.06856681406497955, 0.03965592384338379, 0.16140517592430115, 0.038807883858680725, -0.08030752837657928, -0.29434630274772644, 0.01775946654379368, -0.13017360866069794, -0.02208157815039158, 0.06834830343723297, -0.2145966738462448, -0.10092604905366898, 0.12887603044509888, -0.10670968890190125, -0.16139937937259674, -0.10109628736972809, 0.0021949876099824905, 0.04070115089416504, 0.013223152607679367, -0.01345062255859375, 0.09284252673387527, -0.04494285210967064, 0.15417760610580444, 0.0027165429200977087, 0.030466480180621147, 0.09283966571092606, 0.058832716196775436, 0.17475982010364532, 0.04622398316860199, -0.19960960745811462, -0.06656861305236816, 0.23275162279605865, -0.04076129198074341, 0.08722570538520813, 0.14110489189624786, 0.03246348723769188, -0.5718332529067993}, {-0.031147604808211327, 0.09259817004203796, 0.13493990898132324, 0.23661217093467712, 0.27875134348869324, -0.1761058121919632, -0.12490856647491455, 0.06467337161302567, 0.12171891331672668, 0.1634058803319931, 0.1270088404417038, 0.4439714252948761, -0.047875817865133286, 0.09835807979106903, -0.035800617188215256, 0.20221860706806183, 0.13573358952999115, -0.08869624882936478, -0.15633037686347961, -0.12980163097381592, 0.17470592260360718, -0.04236214607954025, 0.032194629311561584, 0.14519844949245453, -0.4001830816268921, 0.10578059405088425, -0.21971629559993744, 0.05895344540476799, -0.1351713091135025, 0.0722699910402298, -0.018937449902296066, 0.04423382133245468, 0.33104413747787476, 0.23637400567531586, 0.09935755282640457, 0.15560127794742584, -0.0318911038339138, 0.029828190803527832, -0.2117318958044052, 0.07238762825727463, 0.19713719189167023, 0.04178212583065033, -0.12288262695074081, -0.07373280078172684, 0.07747949659824371, -0.17793117463588715, -0.04976439103484154, -0.07290323078632355, 0.10231688618659973, -0.1287582367658615, -0.024047385901212692, 0.04300568625330925, 0.021015847101807594, 0.1885066032409668, -0.0018954541301354766, -0.07676652818918228, 0.1716337502002716, 0.030328527092933655, -0.06873641908168793, -0.3106679320335388, -0.16919399797916412, -0.14079415798187256, 0.13290242850780487, 0.13788416981697083, 0.040071289986371994, 0.1778404265642166, 0.012804261408746243, 0.1282367706298828, -0.09516335278749466, -0.10763131082057953, 0.011570032685995102, 0.010118076577782631, -0.17219652235507965, 0.01857711747288704, 0.07575301826000214, 0.04431580379605293, -0.2048283964395523, 0.0489175021648407, -0.03512321412563324, -0.03421671688556671, -0.06666150689125061, 0.01845652610063553, 0.05165909603238106, -0.13621027767658234, -0.022651512175798416, 0.12326385825872421, 0.04214936122298241, 0.0905175432562828, -0.008414996787905693, -0.020267842337489128, -0.049525003880262375, 0.013073081150650978, -0.16513541340827942, 0.061925698071718216, 0.02350705862045288, -0.10676483064889908, 0.11614155024290085, 0.3765558898448944, 0.09895346313714981, -0.07447444647550583, -0.004672269336879253, -0.16923066973686218, -0.6088724732398987, 0.08044521510601044, 0.25204890966415405, -0.008980036713182926, 0.01795404963195324, 0.05058671906590462, 0.16993799805641174, -0.06558067351579666, -0.08558277785778046, -0.05396096035838127, -0.009705542586743832, 0.21150031685829163, 0.07189936935901642, 0.033033039420843124, -0.08107493817806244, -0.08277074992656708, 0.13742901384830475, 0.09851851314306259, 0.08883817493915558, 0.04168681800365448, -0.19524076581001282, -0.13398486375808716, -0.2387864589691162, -0.028877126052975655, 0.07065820693969727, -0.04414844885468483, 0.046218469738960266, -0.11923623830080032, 0.2423839569091797, 0.06951914727687836, 0.23688185214996338, 0.009985244832932949, -0.15729600191116333, 0.11991578340530396, 0.03151852265000343, 0.09303095191717148, -0.05302249640226364, -0.07773888111114502, 0.09423895925283432, -0.19051477313041687, 0.11517082899808884, -0.04094265028834343, 0.14209343492984772, 0.031025221571326256, -0.0017713903216645122, -0.06142738088965416, -0.023713815957307816, 0.015062700025737286, -0.06539009511470795, -0.06035495176911354, 0.03591400757431984, -0.0867907926440239, 0.08085369318723679, -0.09901096671819687, -0.1607331484556198, 0.08727540820837021, -0.01179532241076231, 0.16113100945949554, 0.1918889433145523, 0.0189934391528368, 0.050029207020998, 0.12256500869989395, 0.2858503460884094, -0.1094236969947815, -0.2942453920841217, -0.1450285017490387, 0.04395706206560135, 0.012428812682628632, 0.020698294043540955, 0.2717638611793518, 0.03710280731320381, 0.02714226208627224, 0.007333653047680855, 0.07237661629915237, -0.11150477826595306, 0.00868647638708353, -0.03313997760415077, 0.22982540726661682, 0.043134886771440506, 0.5274024605751038, 0.09999591112136841, 0.11491934210062027, -0.10145201534032822, -0.08973570168018341, -0.0007111190352588892, 0.047520775347948074, -0.16629701852798462, 0.07542751729488373, 0.07603149116039276, 0.17320463061332703, -0.060166679322719574, 0.06090334430336952, 0.21051768958568573, 0.12573295831680298, -0.2101794183254242, 0.07302038371562958, 0.1361270397901535, -0.0017280683387070894, -0.19284676015377045, 0.08055143803358078, 0.04145368933677673, -0.1398928314447403, -0.10959537327289581, -0.017643574625253677, 0.06654951721429825, 0.21377043426036835, 0.10588949173688889, -0.12394127249717712, -0.11753636598587036, 0.010013801977038383, -0.06554106622934341, 0.09529688954353333, 0.11083319038152695, 0.05859392136335373, 0.1327507197856903, 0.1057480201125145, 0.31122907996177673, 0.006392547395080328, -0.14784245193004608, 0.05455167219042778, -0.04623980447649956, -0.03817020729184151, 0.131394162774086, -0.012046228162944317, 0.010380949825048447, -0.10301097482442856, 0.23644091188907623, -0.03152548521757126, 0.19977174699306488, -0.11518111079931259, -0.061585698276758194, 0.00929858535528183, -0.03546668216586113, 0.07230173796415329, -0.13287754356861115, 0.3421650826931, -0.07387778908014297, -0.10329817235469818, -0.0852024182677269, 0.13128405809402466, -0.04674575850367546, 0.13878971338272095, -0.1076173484325409, -0.1680968999862671, -0.031168995425105095, -0.06855487823486328, 0.10015346109867096, -0.07151849567890167, -0.038928329944610596, 0.023610888049006462, -0.1010061502456665, -0.08838305622339249, -0.0810520276427269, -0.09386126697063446}, {0.13827960193157196, 0.024227812886238098, 0.10079317539930344, 0.03487248718738556, -0.06910710036754608, 0.11145928502082825, -0.11371218413114548, 0.1833658218383789, -0.08993161469697952, -0.045966289937496185, 0.2333851307630539, 0.0009334713104180992, -0.04239250719547272, 0.18052279949188232, 0.06593910604715347, -0.007475725375115871, -0.06765865534543991, -0.11665080487728119, 0.17370299994945526, -0.06456035375595093, -0.04055704176425934, 0.10935604572296143, -0.004227172117680311, -0.3069199323654175, -0.12178357690572739, -0.26493385434150696, 0.12932860851287842, -0.024896053597331047, 0.07438531517982483, 0.09982366114854813, 0.07480371743440628, 0.1803743988275528, -0.029489396139979362, -0.09839849174022675, -0.18300001323223114, 0.23535747826099396, -0.012419138103723526, 0.010885882191359997, 0.03606448695063591, 0.10647647082805634, 0.038728151470422745, -0.08524784445762634, -0.06872162967920303, -0.08967410773038864, 0.03561820462346077, 0.18176046013832092, -0.0723564624786377, -0.06220187619328499, -0.11082866042852402, 8.534501830581576e-05, 0.17851778864860535, 0.019281674176454544, 0.04649922996759415, -0.16072119772434235, 0.06295405328273773, -0.1381799727678299, -0.07066550105810165, -0.07291179150342941, -0.04806675761938095, 0.13958507776260376, 0.12090323865413666, 0.052450958639383316, -0.08370738476514816, -0.16745571792125702, 0.051183026283979416, 0.027561789378523827, -0.08873341977596283, -0.008945857174694538, -0.2631705105304718, -0.12240432947874069, -0.08632015436887741, 0.016090579330921173, 0.09097392112016678, -0.07130207121372223, 0.06040694937109947, 0.0899825468659401, -0.25265970826148987, 0.33788764476776123, 0.21836158633232117, 0.13782921433448792, -0.033190853893756866, 0.11375011503696442, -0.30397796630859375, 0.018181396648287773, -0.014887391589581966, 0.014827210456132889, -0.07810962945222855, 0.07091471552848816, 0.18544891476631165, 0.1954295039176941, -0.006627504713833332, -0.032232217490673065, 0.02139696478843689, 0.026979714632034302, -0.11373354494571686, -0.20728355646133423, -0.16401009261608124, -0.004813973791897297, -0.17899104952812195, 0.07607875019311905, -0.07649553567171097, 0.1263948380947113, 0.20090250670909882, 0.08829885721206665, -0.07311224192380905, 0.9269750714302063, -0.08823936432600021, -0.08109467476606369, -0.21726343035697937, -0.21624967455863953, 0.20461417734622955, 0.2200630009174347, 0.023360542953014374, 0.05150343477725983, 0.07379187643527985, -0.1937023401260376, 0.11728399246931076, -0.06800240278244019, 0.054185546934604645, -0.006776522845029831, -0.09868558496236801, -0.10458024591207504, -0.19575996696949005, -0.2771838903427124, -0.13143761456012726, -0.01239651907235384, 0.14606192708015442, -0.17325913906097412, -0.08377739042043686, 0.00015246290422510356, 0.18334926664829254, 0.23551496863365173, 0.03616597503423691, -0.15772999823093414, -0.08046115189790726, -0.05506652221083641, 0.08220576494932175, 0.06513553857803345, -0.018506111577153206, -0.22868254780769348, 0.0525062270462513, 0.09326993674039841, -0.2832314372062683, -0.17700310051441193, 0.006929520983248949, 0.02915620617568493, -0.024706389755010605, -0.1430012583732605, 0.02080301009118557, 0.05305879935622215, 0.0008529408951289952, -0.031365618109703064, 0.14152047038078308, -0.0005047020968049765, -0.004395653028041124, -0.265916645526886, -0.1396888941526413, -0.08291123807430267, -0.03419090062379837, -0.36284735798835754, 0.15996500849723816, -0.0828847736120224, -0.08271200954914093, 0.19924643635749817, 0.02105308324098587, -0.015220542438328266, 0.08120618015527725, -0.1672668159008026, 0.07721444964408875, -0.02232435531914234, -0.0665740817785263, 0.19997648894786835, 0.03957207500934601, -0.0016827116487547755, 0.07606848329305649, -0.013252501375973225, 0.16333681344985962, 0.2149048149585724, -0.029381154105067253, -0.19787700474262238, 0.12591338157653809, 0.07169990986585617, 0.08916155993938446, -0.06926572322845459, -0.06887403875589371, -0.05057351663708687, 0.10158541053533554, 0.018240375444293022, -0.03620881587266922, -0.05788860470056534, -0.04995395243167877, 0.0942547544836998, 0.17949289083480835, 0.029163748025894165, 0.056265100836753845, 0.008158721029758453, 0.08025575429201126, -0.10116903483867645, -0.13042189180850983, -0.20113709568977356, -0.021342212334275246, -0.1097642108798027, -0.05501086637377739, 0.08336290717124939, -0.16805705428123474, 0.02714286930859089, -0.1858048439025879, -0.09070135653018951, 0.10991561412811279, 0.15094353258609772, 0.10108928382396698, 0.11463386565446854, 0.0025954912416636944, 0.048342201858758926, -0.08382876217365265, 0.07059716433286667, -0.06481016427278519, 0.13528308272361755, -0.08721952140331268, 0.17516162991523743, -0.17365118861198425, -0.021104706451296806, -0.0668552815914154, 0.14348313212394714, 0.07596071064472198, 0.029923278838396072, 0.06652383506298065, 0.07738032937049866, -0.08309075236320496, -0.08712293952703476, 0.12164599448442459, -0.08226120471954346, 0.10211726278066635, 0.07893100380897522, -0.091011643409729, 0.08529973775148392, -0.012696152552962303, 0.1975015550851822, 0.05061771720647812, 0.2558129131793976, 0.06523890048265457, -0.0036750277504324913, -0.14534474909305573, 0.04999306797981262, -0.08453797549009323, 0.041684240102767944, 0.19040752947330475, 0.07974810898303986, -0.11130211502313614, 0.09429560601711273, 0.056214589625597, -0.1282777041196823, -0.030637988820672035, -0.03863966092467308, -0.14172959327697754, 0.18009236454963684}, {0.03288007155060768, -0.05463234707713127, 0.05472373962402344, -0.01860605739057064, -0.0511455200612545, -0.09250340610742569, -0.03676019608974457, 0.09163998067378998, -0.042927421629428864, 0.15940825641155243, 0.02193835936486721, 0.08861402422189713, 0.007340429816395044, -0.10161764919757843, -0.09512822329998016, -0.006750311236828566, -0.03679675608873367, 0.02492706850171089, 0.08926007151603699, -0.20883122086524963, -0.17130716145038605, -0.10495567321777344, -0.19613084197044373, 0.008515223860740662, -0.15458035469055176, 0.026302367448806763, 0.037171825766563416, 0.2112802267074585, -0.08485422283411026, -0.15460838377475739, -0.013844736851751804, 0.03264157101511955, 0.18528349697589874, 0.007602599915117025, 0.00806481670588255, -0.02575698494911194, 0.02385299652814865, -0.12031729519367218, 0.04321444034576416, -0.09773044288158417, -0.002749598817899823, -0.0978393480181694, -0.03568615764379501, -0.18996772170066833, -0.19284126162528992, -0.08957753330469131, -0.23771874606609344, -0.1260167956352234, -0.03410683199763298, 0.04033539816737175, -0.20895977318286896, 0.10094454139471054, -0.031157096847891808, 0.04266076534986496, -0.0008241178002208471, -0.03469793125987053, 0.03040200285613537, 0.09553399682044983, -0.15994863212108612, -0.24787743389606476, 0.0965593233704567, -0.05104963108897209, 0.024441799148917198, 0.03523024171590805, 0.01657947711646557, 0.04278424754738808, -0.02015320584177971, -0.10286308079957962, 0.12906993925571442, -0.14695018529891968, 0.011893948540091515, -0.1441233903169632, -0.03559562936425209, 0.05420629307627678, -0.16619227826595306, -0.022715382277965546, -0.11813091486692429, -0.04614769667387009, 0.08190124481916428, 0.15517005324363708, -0.20001870393753052, -0.020744189620018005, -0.04332634434103966, 0.12809835374355316, -0.07043998688459396, -0.07989662140607834, 0.1299002319574356, -0.017539484426379204, 0.18341968953609467, -0.09432997554540634, 0.0783689022064209, -0.052981823682785034, 0.03799628093838692, -0.17928311228752136, 0.03093278408050537, 0.14562591910362244, -0.027475565671920776, 0.06434197723865509, 0.040465936064720154, -0.2510789930820465, -0.010170210152864456, -0.03299223631620407, -0.06817582249641418, 0.028250068426132202, 0.188214972615242, -1.2782745361328125, 0.22812561690807343, 0.028617866337299347, 0.13141246140003204, -0.0787874311208725, 0.14408816397190094, 0.11325377225875854, -0.14478956162929535, 0.10002893209457397, 0.07563372701406479, -0.0969926044344902, -0.07182526588439941, -0.07815506309270859, -0.018015572801232338, -0.08204562216997147, 0.10878738015890121, -0.18428075313568115, -0.16392473876476288, 0.01547209732234478, 0.11803176254034042, 0.09599754959344864, 0.11630881577730179, 0.004740320611745119, -0.019708503037691116, 0.001179645536467433, -0.10825194418430328, 0.09384369850158691, 0.0029399688355624676, 0.01724027469754219, -0.16067664325237274, -0.21665118634700775, -0.07421597093343735, 0.0336783304810524, -0.03264826908707619, 0.009664328768849373, -0.14416755735874176, 0.05205968767404556, -0.10895857959985733, -0.09290114790201187, 0.0032861027866601944, 0.1616085320711136, 0.020842712372541428, -0.07179441303014755, -0.11601940542459488, 0.042050786316394806, -0.014886963181197643, 0.05445723980665207, -0.020215630531311035, 0.3059464991092682, 0.07332450896501541, -0.04943311959505081, -0.16243508458137512, -0.055894773453474045, 0.14075174927711487, 0.5722447037696838, 0.040591709315776825, 0.19863824546337128, -0.0916251465678215, -0.15799996256828308, 0.05822094529867172, 0.013687219470739365, 0.1593589186668396, 0.13177384436130524, -0.20104558765888214, 0.04171041026711464, -0.05014285072684288, 0.03364841639995575, -0.018625959753990173, -0.035755280405282974, -0.10631924122571945, 0.0877666249871254, -0.07664107531309128, -0.14842528104782104, 0.05974821373820305, -0.03803436458110809, -0.21461664140224457, 0.23367011547088623, -0.22960427403450012, 0.17868490517139435, -0.11543915420770645, -0.09521104395389557, -0.03549493849277496, -0.10921832919120789, 0.041385259479284286, 0.10667368024587631, -0.030740251764655113, -0.11103495210409164, -0.07756278663873672, -0.11277935653924942, 0.02555791661143303, 0.09521885216236115, 0.018353082239627838, 0.19189248979091644, 0.17549993097782135, 0.18312518298625946, -0.04491161182522774, -0.05459318310022354, 0.024146143347024918, 0.05091900750994682, 0.1819821298122406, 0.06199396774172783, 0.017362749204039574, 0.1306382715702057, 0.05569726601243019, 0.17694175243377686, 0.10533393174409866, -0.2039729505777359, 0.12168318778276443, 0.1829921156167984, -0.018106339499354362, -0.24321216344833374, 0.12798474729061127, 0.06667309999465942, -0.14129117131233215, 0.1445210725069046, 0.08619424700737, 0.013847271911799908, 0.012082942761480808, -0.19545304775238037, -0.10409295558929443, -0.11687175184488297, 0.15945962071418762, 0.028369689360260963, 0.05507097393274307, -0.05214357748627663, 0.10371237248182297, -0.10479121655225754, 0.0008343086228705943, -0.0391094908118248, 0.007800897117704153, -0.12660175561904907, 0.07379566133022308, -0.11789753288030624, 0.016788577660918236, -0.37421098351478577, -0.009510374628007412, 0.16994082927703857, -0.09549444913864136, -0.04671000689268112, -0.03532859683036804, -0.01585186831653118, -0.042605988681316376, 0.15241947770118713, 0.15218794345855713, -0.0835072472691536, 0.1281730681657791, -0.01989230513572693, 0.019012970849871635, 0.13887618482112885, 0.002886715577915311, 0.04267542436718941}, {-0.1564154028892517, 0.05287659540772438, -0.10454528778791428, 0.03769791126251221, 0.28515201807022095, -0.10976774245500565, -0.14720287919044495, 0.2054397612810135, -0.02064395695924759, -0.0070799277164042, 0.144048810005188, -0.21990272402763367, -0.02066045068204403, -0.13930511474609375, -0.02388959936797619, 0.00856152456253767, -0.06618660688400269, 0.1048905998468399, 0.06814349442720413, -0.290496826171875, -0.20189183950424194, -0.03570812940597534, 0.03819144889712334, 0.05010255053639412, 0.08822343498468399, 0.13536152243614197, 0.01626494713127613, 0.03577079996466637, 0.006608645431697369, -0.14538094401359558, -0.05076679214835167, -0.321340948343277, 0.3676224648952484, -0.09627047181129456, 0.023810168728232384, 0.03107221983373165, -0.16958481073379517, -0.02120787277817726, 0.11665812879800797, -0.05510489270091057, 0.031011337414383888, 0.11712110787630081, -0.045709531754255295, -0.01821441389620304, 0.17572681605815887, 0.32414835691452026, 0.4652634859085083, 0.04772967845201492, -0.185930997133255, 0.15657080709934235, 0.10091077536344528, -0.01620534062385559, 0.0016658862587064505, 0.09057051688432693, 0.08568435907363892, 0.1570053994655609, 0.17068281769752502, 0.011054574511945248, -0.05379490554332733, 0.14040698111057281, -0.3088958263397217, 0.009447254240512848, -0.01756066270172596, 0.34779492020606995, 0.09034884721040726, 0.3740316927433014, 0.06100274249911308, 0.09731253981590271, -0.004519083071500063, -0.043625086545944214, -0.1288701891899109, -0.2148156613111496, 0.14185237884521484, 0.15906311571598053, -0.11164236068725586, 0.10334570705890656, -0.01470883097499609, -0.3157329857349396, 0.1306229531764984, 0.12902992963790894, 0.5357140302658081, 0.2430545836687088, -0.06395969539880753, -0.26124998927116394, 0.13190732896327972, 0.0766381025314331, -0.1386847048997879, 0.05158696696162224, -0.12575501203536987, 0.0058229658752679825, -0.017124446108937263, 0.11833416670560837, -0.0073776463977992535, 0.09707784652709961, 0.0273745134472847, -0.20401431620121002, -0.06720246374607086, -0.021847764030098915, 0.2856827676296234, 0.24737676978111267, -0.03878038749098778, 0.11288680881261826, 0.2805529534816742, -0.13257919251918793, 0.35692861676216125, 0.3007517158985138, -0.019417211413383484, 0.041317690163850784, -0.06601090729236603, -0.09694869816303253, 0.15577594935894012, 0.24236328899860382, -0.09039267152547836, -0.011350381188094616, -0.18708258867263794, 0.0556841678917408, -0.1634904444217682, -0.01990029402077198, 0.14531418681144714, 0.0002148113853763789, -0.23606754839420319, 0.3213901221752167, -0.09392057359218597, -0.08472639322280884, 0.23768161237239838, -0.1481347531080246, 0.11800077557563782, -0.07665735483169556, -0.1260983794927597, -0.03629931062459946, -0.11070475727319717, -0.0396486297249794, -0.08853559195995331, 0.021284103393554688, 0.020873716101050377, 0.14200256764888763, -0.03281070664525032, 0.07329762727022171, -0.2005292773246765, 0.09570711851119995, 0.03823205083608627, 0.004848347045481205, -0.23550446331501007, 0.015556045807898045, 0.19414860010147095, 0.08996663987636566, -0.22002319991588593, 0.014979471452534199, 0.01913590542972088, -0.049559660255908966, 0.07737413048744202, -0.014973265118896961, -0.008331087417900562, -0.15285713970661163, 0.014653787948191166, 0.15385988354682922, 0.018048934638500214, 0.29341328144073486, 0.21436475217342377, 0.1074579656124115, -0.1237986609339714, -0.00898707378655672, -0.06461041420698166, 0.2171870917081833, 0.16359683871269226, -0.23674947023391724, 0.17854653298854828, -0.24508361518383026, 0.07450935244560242, -0.1993282586336136, -0.05482262745499611, 0.01998717710375786, -0.19754214584827423, -0.0410563126206398, -0.29655054211616516, 0.03704473748803139, 0.09597622603178024, 0.03529581427574158, 0.05487583205103874, -0.1598532646894455, -0.038050420582294464, -0.11946138739585876, 0.1447073519229889, 0.06013691425323486, -0.10895773023366928, -0.19746506214141846, -0.409849613904953, -0.15251904726028442, 0.13099326193332672, -0.19652798771858215, -0.13473862409591675, 0.009640408679842949, 0.20021501183509827, 0.2331623136997223, 0.12301334738731384, -0.24126698076725006, 0.10204801708459854, -0.17496070265769958, -0.09857960790395737, -0.033167801797389984, 0.0769144743680954, -0.0029649673961102962, -0.02259269915521145, 0.2878851592540741, -0.107332743704319, -0.03895332291722298, 0.055011916905641556, -0.07420660555362701, 0.026561392471194267, 0.08113411068916321, 0.2418716549873352, 0.04495777562260628, -0.21015259623527527, -0.1072707399725914, -0.10713636875152588, 0.02971920184791088, 0.005371611565351486, 0.024014350026845932, -0.36105039715766907, -0.08384247869253159, -0.19325819611549377, -0.012172425165772438, 0.127268448472023, -0.08836657553911209, -0.2403421550989151, -0.1409527063369751, 0.03347964584827423, 0.048680346459150314, -0.031118256971240044, -0.24199523031711578, 0.26461419463157654, 0.08163788914680481, -0.006302601657807827, -0.14093820750713348, 0.08788959681987762, -0.05932144448161125, -0.2930419147014618, 0.051504720002412796, 0.018947914242744446, 0.23864591121673584, -0.07193659245967865, 0.04531330615282059, -0.18797588348388672, 0.13861171901226044, -0.03907479718327522, 0.003971106372773647, 0.17373940348625183, 0.03997828811407089, 0.14031638205051422, -0.02626391313970089, -0.13233406841754913, -0.23402217030525208, 0.0076211923733353615, -0.10886526107788086, -0.0017854527104645967, -0.2368474155664444}, {0.02128341794013977, -0.08680839836597443, -0.004570901393890381, -0.0007577367359772325, -0.10692790895700455, 0.14506381750106812, -0.040902312844991684, 0.1481044739484787, 0.13448332250118256, -0.2705317437648773, -0.02334323152899742, -0.12584859132766724, -0.002089011948555708, -0.046044979244470596, -0.02764195017516613, -0.06888297945261002, -0.09908385574817657, 0.22820506989955902, -0.030266599729657173, 0.008060907945036888, 0.07704721391201019, 0.046286631375551224, 0.11293362826108932, 0.07313782721757889, 0.23398500680923462, -0.08249512314796448, 0.015141564421355724, 0.05454649031162262, -0.1570264846086502, 0.043721918016672134, 0.16351370513439178, 0.030984004959464073, -0.22405022382736206, -0.11645016074180603, -0.09809055179357529, -0.0317353829741478, -0.09683456271886826, -0.413798987865448, -0.03890298679471016, 0.10920517146587372, -0.06984470784664154, -0.06250205636024475, -0.057689547538757324, -0.039194900542497635, 0.041736871004104614, -0.0008108267793431878, 0.009406814351677895, 0.02067979797720909, -0.010058541782200336, -0.2922694683074951, -0.028358876705169678, -0.19475527107715607, 0.02753978781402111, -0.15901999175548553, 0.029908474534749985, -0.12556791305541992, 0.019010085612535477, 0.18055717647075653, 0.027242813259363174, -0.05974684655666351, 0.0820491686463356, -0.10749323666095734, -0.1872715950012207, -0.11702363193035126, 0.10587866604328156, 0.15322177112102509, -0.09470856934785843, -0.16632385551929474, -0.25240665674209595, 0.0695822536945343, 0.13361221551895142, -0.010080249048769474, 0.13313345611095428, 0.00020036309433635324, -0.25279849767684937, 0.06787513196468353, 0.09532075375318527, -0.05532119795680046, -0.0269793514162302, 0.20261672139167786, 0.05875539779663086, -0.004173716995865107, 0.03945166990160942, 0.03392789140343666, 0.1605067253112793, -0.12968674302101135, 0.0004750058287754655, -0.010143304243683815, 0.018538419157266617, -0.03287383168935776, -0.08119085431098938, -0.038210678845644, 0.0363340824842453, 0.21210674941539764, -0.029852503910660744, -0.12621881067752838, -0.01563136652112007, -0.06167633458971977, -0.038283705711364746, -0.07491619139909744, 0.003104667877778411, -0.015320499427616596, -0.15557847917079926, -0.04354454204440117, -0.22609882056713104, -0.09008732438087463, -0.058226730674505234, -0.010402485728263855, -0.042229775339365005, -0.0051983376033604145, 0.04839412122964859, 0.20261968672275543, 0.041199032217264175, -0.027771631255745888, 0.0019994894973933697, -0.15243352949619293, -0.0828317180275917, -0.040634892880916595, 0.04090280830860138, -0.08343663066625595, 0.043980322778224945, -0.042498230934143066, -0.07830466330051422, -0.29172080755233765, -0.1133604496717453, 0.053349100053310394, 0.05599573999643326, -0.1418268233537674, 0.032888393849134445, 0.00839447695761919, -0.11080871522426605, -0.026749862357974052, 0.037834230810403824, -0.03500061854720116, -0.3230411410331726, -0.06862375885248184, -0.05512387305498123, 0.07639968395233154, -0.1732865273952484, -0.2607034742832184, -0.038990285247564316, -0.14063437283039093, -0.10255508124828339, 0.20004302263259888, 0.17588166892528534, 0.06839784234762192, 0.03716323897242546, -0.10535658895969391, 0.16177216172218323, -0.22209683060646057, 0.05461303889751434, -0.16166962683200836, 0.14885564148426056, -0.09726736694574356, 0.05062403529882431, 0.10734783113002777, 0.04010195657610893, -0.25713470578193665, 0.18294966220855713, -0.12303145974874496, 0.12081800401210785, 0.15259426832199097, -0.019483745098114014, -0.011516919359564781, 0.06818442046642303, -0.192737877368927, -0.07638756930828094, 0.03861750289797783, 0.07622227072715759, 0.02172216773033142, -0.02668958343565464, 0.018457911908626556, 0.03215073049068451, 0.004612742457538843, -0.04048313945531845, -0.042149096727371216, 0.061422199010849, -0.03465239331126213, -0.03287868946790695, -0.0760859027504921, 0.08080127835273743, -0.10662473738193512, -0.11312543600797653, 0.2561631500720978, 0.09530964493751526, -0.1333673745393753, -0.32813698053359985, -0.10743706673383713, 0.03989753872156143, -0.16572131216526031, 0.0036234541330486536, -0.18136107921600342, -0.0916694775223732, -0.09190460294485092, -0.0037903529591858387, 0.1239459365606308, 0.31327396631240845, -0.05362981930375099, -0.15980243682861328, 0.08356068283319473, 0.19526787102222443, 0.1414574682712555, -0.007834913209080696, 0.16517896950244904, 0.02690959721803665, 0.06491115689277649, -0.20651176571846008, -0.026375068351626396, 0.05087275803089142, 0.22244782745838165, 0.34790557622909546, -0.15646231174468994, 0.1315334439277649, -0.02092273347079754, -0.23976491391658783, 0.08858974277973175, -0.07374819368124008, -0.1663389652967453, 0.10468042641878128, 0.02060772478580475, -0.08646784722805023, 0.02447725087404251, 0.10430850088596344, -0.16490983963012695, -0.06136324629187584, -0.3244687616825104, 0.3266820013523102, 0.15712998807430267, -0.05752645805478096, 0.1926201730966568, 0.02326919138431549, -0.01052605826407671, 0.10586962103843689, 0.017142819240689278, 0.027385463938117027, 0.11226202547550201, 0.020669247955083847, 0.06919141113758087, 0.07260701060295105, -0.1057877391576767, -0.0464472621679306, -0.03610827401280403, -0.005622602067887783, -0.19961188733577728, -0.26142624020576477, 0.09750482439994812, 0.015239574946463108, 0.08108552545309067, 0.08215058594942093, 0.15326662361621857, 0.07034054398536682, -0.12054934352636337, 0.017468702048063278, 0.167994886636734, 0.12106219679117203, 0.056826401501894}, {0.04790026694536209, -0.043918177485466, 0.1967952996492386, 0.20507214963436127, 0.08740866929292679, 0.08203490823507309, 0.20911720395088196, 0.10558343678712845, 0.1639699786901474, 0.0630803033709526, 0.006788305472582579, -0.02860606275498867, 0.010498340241611004, -0.14884816110134125, -0.020958494395017624, -0.006256780121475458, 0.014029622077941895, -0.12537536025047302, 0.00314375851303339, -0.02235267125070095, 0.008703604340553284, 0.045768335461616516, -0.04115806519985199, 0.01542498916387558, -0.05209483578801155, -0.07537802308797836, 0.012354431673884392, 0.1350616067647934, -0.07042883336544037, -0.013642560690641403, -0.2551843822002411, 0.06038407236337662, 0.2901623547077179, -0.20640867948532104, 0.11398769915103912, 0.08303473889827728, 0.02020116336643696, 0.12413908541202545, 0.019340001046657562, -0.01991323009133339, -0.08926723897457123, 0.06885664910078049, 0.05736425146460533, 0.07338622957468033, 0.05124133080244064, -0.0051865242421627045, -0.3554057478904724, -0.1311686784029007, -0.09940002113580704, 0.047216251492500305, -0.11238949745893478, -0.00433313800022006, 0.1121278926730156, 0.09559062868356705, -0.037572212517261505, -0.062226105481386185, 0.14379364252090454, 0.10635760426521301, -0.08316543698310852, -0.08490627259016037, 0.033447280526161194, -0.05039183050394058, -0.21906135976314545, 0.016764439642429352, -0.014149043709039688, 0.056815627962350845, -0.007876820862293243, -0.25819146633148193, 0.2652466595172882, 0.06033175438642502, 0.10765272378921509, 0.023643575608730316, -0.17140105366706848, 0.1745891124010086, -0.12263789772987366, 0.021043559536337852, -0.08387994021177292, -0.13913291692733765, -0.00782543420791626, 0.08724161982536316, -0.13244356215000153, -0.0814124271273613, -0.005370619241148233, 0.13287201523780823, -0.0017177442787215114, 0.008799715898931026, 0.10121294856071472, -0.00445901183411479, 0.03922218829393387, -0.03022962622344494, -0.03060910291969776, -0.11215221136808395, -0.10251296311616898, -0.0022058484610170126, -0.174296572804451, 0.059843797236680984, 0.2535189986228943, 0.018086278811097145, -0.005060579162091017, 0.05564996227622032, 0.06738138198852539, 0.12677963078022003, 0.39743170142173767, -0.11317887902259827, 0.30048197507858276, -0.015192407183349133, 0.05219343304634094, -0.15634982287883759, 0.08615557849407196, 0.014225799590349197, 0.12389529496431351, -0.02418486773967743, 0.12343450635671616, 0.19321675598621368, 0.11217866092920303, -0.009158546105027199, 0.17856940627098083, -0.09370356798171997, 0.23238971829414368, -0.14768999814987183, -0.08645960688591003, 0.029733123257756233, -0.3919511139392853, 0.010265611112117767, 0.04894890636205673, 0.01726275309920311, 0.0770350843667984, -0.018603013828396797, 0.09997352957725525, 0.19229662418365479, 0.051868852227926254, 0.21239933371543884, 0.08314947783946991, -0.06396295875310898, -0.09946709126234055, 0.05111623555421829, 0.10404252260923386, -0.09002679586410522, 0.03536379709839821, 0.1319151520729065, -0.19848878681659698, -0.06857535243034363, -0.03615102544426918, 0.05339654162526131, -0.0311964713037014, 0.04294579103589058, -0.1627100259065628, 0.12566137313842773, 0.023563027381896973, -0.04532072693109512, 0.19458770751953125, -0.04806124418973923, -0.00234337174333632, 0.031819213181734085, 0.18858827650547028, -0.02334064431488514, -0.04315454512834549, 0.09166306257247925, -0.02536761574447155, 0.054464761167764664, -0.07861292362213135, 0.19358697533607483, -0.1981525868177414, 0.024302907288074493, 0.023232068866491318, 0.025350771844387054, -0.04607482627034187, 0.014824388548731804, -0.1069173812866211, 0.2016405463218689, -0.028905034065246582, 0.17338958382606506, 0.011045720428228378, 0.04314505308866501, -0.055123016238212585, 0.0008605958428233862, 0.09191329032182693, 0.2013324350118637, 0.03918701410293579, -0.02804376557469368, 0.08679842203855515, -0.1115732192993164, 0.10734590888023376, -0.12155365198850632, 0.1016077846288681, -0.1324421912431717, 0.1494840532541275, -0.05512586981058121, -0.01699632592499256, 0.23704731464385986, -0.09371757507324219, -0.14125439524650574, -0.11222657561302185, -0.10098160803318024, -0.09617486596107483, -0.0837402492761612, 0.18980354070663452, 0.0525573268532753, 0.1268872171640396, -0.047081153839826584, -0.04052191600203514, 0.10497964918613434, -0.15662631392478943, -0.05254773795604706, -0.02002515085041523, 0.00760664464905858, -0.1916768103837967, 0.2586335241794586, -0.023821089416742325, -0.008724185638129711, -0.009490640833973885, -0.009519671089947224, -0.2175474464893341, 0.20745381712913513, -0.09589985758066177, -0.00573270907625556, 0.07975485920906067, 0.06512158364057541, 0.0843346118927002, 0.5093029737472534, -0.06695547699928284, -0.049563802778720856, -0.002777215326204896, 0.009012573398649693, 0.0420694500207901, 0.15937115252017975, -0.10679987072944641, 0.13878490030765533, 0.01729227416217327, 0.26628822088241577, 0.12159056216478348, 0.0006408406188711524, 0.06704579293727875, -0.2658903896808624, -0.04648329317569733, -0.061752576380968094, -0.20487968623638153, -0.09963994473218918, -0.15698838233947754, 0.07795350253582001, -0.03343530371785164, -0.022691940888762474, -0.16867418587207794, 0.022888153791427612, -0.014128929935395718, 0.15061433613300323, -0.08136189728975296, 0.11363314837217331, -0.006141947582364082, 0.07375700771808624, 0.2094576507806778, 0.09508296847343445, 0.11814656853675842, 0.09148488938808441, -0.015834735706448555, 0.10810748487710953}, {-0.03763248771429062, 0.05662357434630394, -0.045505039393901825, 0.02656954899430275, -0.13161443173885345, -0.02594214677810669, -0.028332827612757683, -0.013919874094426632, 0.006926228757947683, -0.06677190959453583, 0.04659708961844444, -0.13825233280658722, -0.17370763421058655, 0.1822051852941513, -0.012023426592350006, -0.09164860844612122, 0.0026762476190924644, 0.08528773486614227, -0.053749021142721176, 0.004265038296580315, -0.19211158156394958, -0.09740728884935379, -0.03401757776737213, -0.006619893480092287, 0.13534025847911835, -0.04291290417313576, 0.061268776655197144, 0.09688016027212143, 0.05117625370621681, 0.21007660031318665, 0.1618257611989975, 0.10159172862768173, 0.009038954973220825, 0.10981237888336182, 0.13230033218860626, -0.7113580703735352, -0.10521787405014038, -0.016972318291664124, 0.06666957587003708, -0.10724373906850815, 0.032343409955501556, -0.22051890194416046, -0.015341689810156822, 0.07552672922611237, -0.11400912702083588, 0.05334460362792015, -0.04271774739027023, -0.07360409200191498, 0.0940999984741211, 0.13785424828529358, -0.26098430156707764, -0.0753297433257103, -0.07348063588142395, -0.06309103965759277, -0.1886100023984909, -0.08296525478363037, -0.031481046229600906, -0.09435752034187317, 0.01992388255894184, -0.030926678329706192, 0.16853921115398407, -0.020958567038178444, -0.08547735214233398, -0.15608470141887665, 0.1223239153623581, 0.024909619241952896, -0.12357437610626221, -0.1381327211856842, 0.017006762325763702, 0.11878219246864319, 0.10594827681779861, 0.051341332495212555, -0.08519157022237778, 0.15316946804523468, -0.13152319192886353, 0.05793162062764168, -0.13948635756969452, -0.25679513812065125, 0.05435442179441452, 0.2626623511314392, -0.22710584104061127, -0.17404574155807495, -0.002990652108564973, 0.10399793833494186, -0.0888378843665123, -0.1081078052520752, 0.06278971582651138, -0.1372753381729126, 0.028122957795858383, -0.00410052202641964, 0.12605921924114227, -0.11953824013471603, -0.06571491807699203, -0.26669013500213623, 0.1738244593143463, 0.021316727623343468, 0.16799503564834595, -0.06929055601358414, -0.11152860522270203, 0.08082074671983719, -0.1585775464773178, 0.1154298409819603, -0.016058912500739098, 0.05777512118220329, 0.15178467333316803, -0.07394056022167206, 0.14889203011989594, 0.04842660576105118, 0.24115687608718872, 0.006987534463405609, 0.07740716636180878, 0.08655496686697006, 0.017269564792513847, 0.021200889721512794, 0.026687366887927055, -0.2223438173532486, -0.03765011206269264, -0.00083102349890396, 0.16242414712905884, 0.06817127019166946, 0.21866168081760406, -0.08681874722242355, 0.07903241366147995, 0.01914054900407791, -0.12292804569005966, 0.1400838941335678, 0.16024591028690338, 0.10859142988920212, -0.026150960475206375, 0.023615876212716103, 0.07679003477096558, -0.018835125491023064, 0.0679553970694542, 0.11993731558322906, -0.0903782770037651, 0.09700167179107666, 0.0377524234354496, 0.04403604567050934, -0.19311855733394623, 0.03142538666725159, -0.035737842321395874, -0.09134889394044876, 0.04946369677782059, -0.07178935408592224, 0.021725047379732132, 0.05076754093170166, -0.00905425101518631, 0.0009066491620615125, 0.12144743651151657, -0.014987480826675892, -0.10054833441972733, 0.17707185447216034, 0.10655377060174942, -0.04399881884455681, 0.06819813698530197, 0.11168494075536728, -0.15190353989601135, 0.05235442519187927, 0.05544249713420868, -0.039550404995679855, -0.012046120129525661, -0.02969244495034218, -0.07018846273422241, -0.03964289650321007, 0.07954384386539459, 0.005812392104417086, -0.09608932584524155, 0.26317164301872253, 0.030211664736270905, 0.00759863993152976, -0.013985267840325832, 0.08469375222921371, -0.015538557432591915, -0.41993266344070435, -0.46731036901474, 0.02577516809105873, 0.09642636030912399, -0.15553471446037292, 0.21859805285930634, -0.17102059721946716, -0.051663078367710114, -0.21457050740718842, -0.1521281599998474, 0.04142342507839203, -0.07308433949947357, -0.16158521175384521, -0.09042250365018845, 0.0015988796949386597, -0.028562357649207115, 0.0627959743142128, -0.03367617353796959, -0.03068450465798378, -0.2168068140745163, -0.20133163034915924, -0.11684028804302216, -0.10408172011375427, -0.0622008815407753, 0.12941579520702362, 0.16025398671627045, -0.021767202764749527, 0.19211624562740326, 0.06377723813056946, -0.002767727244645357, -0.06986010819673538, 0.045761946588754654, 0.05473194271326065, 0.06164292246103287, 0.02189456857740879, -0.15242378413677216, -0.014358370564877987, 0.043180618435144424, -0.002197000663727522, -0.10886909067630768, -0.09338539093732834, 0.21982863545417786, -0.1757238358259201, 0.081368088722229, -0.04355227202177048, 0.16618521511554718, -0.04141362011432648, 0.015160168521106243, -0.01660493016242981, -0.07814369350671768, 0.06700629740953445, -0.10154453665018082, -0.06322187185287476, 0.05086435377597809, 0.022758319973945618, -0.07392571866512299, 0.07511217892169952, 0.06499194353818893, 0.033721618354320526, -0.0017900840612128377, -0.1295623928308487, 0.19183437526226044, -0.22342537343502045, 0.026242537423968315, 0.10474234074354172, 0.15886516869068146, 0.10162029415369034, 0.15049251914024353, -0.014991541393101215, -0.1776888519525528, -0.021688314154744148, -0.08364821970462799, 0.10629136860370636, -0.01714068464934826, -0.0464274138212204, 0.05667528510093689, 0.1631080061197281, 0.11626158654689789, 0.0023235268890857697, -0.0918235033750534, 0.011787135154008865, -0.11780640482902527, 0.008187773637473583}, {-0.05627011880278587, -0.037890080362558365, -0.12106994539499283, 0.161429762840271, 0.1824347823858261, 0.09234411269426346, -0.14690425992012024, -0.016946623101830482, 0.07608220726251602, -0.37687698006629944, 0.1584589034318924, 0.1374680995941162, 0.03957917168736458, -0.15239602327346802, -0.07087133079767227, -0.10702383518218994, 0.11105357110500336, 0.28050678968429565, -0.05693293362855911, 0.04539996758103371, 0.04906962811946869, 0.08272518217563629, 0.013739125803112984, -0.179383784532547, 0.11862555146217346, 0.051222868263721466, -0.06516619026660919, 0.00418366864323616, -0.2673002779483795, 0.0018515978008508682, -0.034454595297575, 0.11590884625911713, 0.0004472713335417211, 0.19729912281036377, -0.07495459914207458, -0.04758213087916374, 0.036912526935338974, 0.12425202876329422, -0.14267970621585846, 0.17124693095684052, 0.03991192206740379, -0.04934816062450409, 0.0010499816853553057, -0.03555433824658394, 0.023388179019093513, 0.02283611334860325, -0.11840356141328812, -0.034264519810676575, -0.14181721210479736, 0.10524013638496399, 0.027863651514053345, -0.020140284672379494, -0.14866903424263, -0.1766529679298401, -0.04793297499418259, -0.10156308114528656, 0.03789282217621803, -0.16470909118652344, -0.15346284210681915, -0.1276000589132309, 0.07476355135440826, 0.14039689302444458, 0.19944171607494354, -0.004200253169983625, -0.08125030249357224, -0.05435417592525482, 0.20423728227615356, -0.018635591492056847, 0.0526166670024395, -0.1962440460920334, 0.0747784897685051, -0.03439497947692871, -0.003294208087027073, -0.2562164068222046, -0.18626311421394348, 0.03841578960418701, -0.06755882501602173, -0.3911330997943878, -0.18948081135749817, 0.08805247396230698, -0.2682751715183258, -0.004007598385214806, -0.11556645482778549, 0.10479950904846191, 0.22559985518455505, 0.05625496059656143, -0.07070404291152954, -0.04240238666534424, 0.09177771955728531, -0.1443493813276291, 0.004232111386954784, -0.0979759469628334, 0.22476069629192352, -0.13088582456111908, 0.028334680944681168, -0.11100564152002335, 0.20752349495887756, -0.051322825253009796, -0.09891903400421143, -0.060106419026851654, 0.09757383167743683, 0.16762657463550568, 0.09283317625522614, -0.16208922863006592, 0.1561911404132843, 0.175843745470047, 0.12332024425268173, -0.15341991186141968, 0.1816730499267578, 0.09669895470142365, -0.16336797177791595, -0.1133088544011116, -0.029048413038253784, -0.18191221356391907, 0.06645470857620239, -0.08516904711723328, -0.10580743104219437, -0.26970651745796204, -0.03228171914815903, -0.04440438747406006, 0.146040141582489, 0.03410430625081062, -0.14284880459308624, 0.28437572717666626, -0.18295973539352417, -0.08601555973291397, 0.09094364196062088, -0.0519532673060894, -0.2967187166213989, -0.04612583667039871, -0.04452555254101753, -0.00998256541788578, -0.023234214633703232, -0.14574342966079712, -0.17636975646018982, 0.14758606255054474, 0.015915846452116966, -0.33120501041412354, -0.08207007497549057, 0.24488331377506256, -0.09233663976192474, -0.13223285973072052, 0.003035670379176736, 0.1576755940914154, -0.05247463658452034, 0.20911526679992676, -0.25611111521720886, 0.0626058578491211, 0.10643398016691208, 0.03393235430121422, -0.17298278212547302, 0.0488806888461113, 0.00019080894708167762, 0.15542066097259521, 0.1485723853111267, -0.1157027930021286, -0.06314344704151154, -0.08476966619491577, 0.13407203555107117, 0.024166936054825783, -0.05548365041613579, -0.21511788666248322, -0.028640873730182648, 0.12796759605407715, -0.009517423808574677, 0.1820312738418579, -0.26078730821609497, -0.10582385212182999, 0.110415980219841, 0.1213822066783905, 0.07966539263725281, 0.11340738087892532, 0.1349257081747055, -0.0005827767890878022, 0.17518970370292664, 0.027354029938578606, -0.2160479575395584, -0.17452964186668396, -0.0646694153547287, -0.11817333847284317, 0.13682590425014496, 0.14914388954639435, -0.07185909152030945, -0.2570091784000397, -0.18813180923461914, 0.02495325170457363, 0.607703447341919, 0.04073919728398323, -0.10452090948820114, -0.26740267872810364, 0.17208626866340637, 0.13077564537525177, -0.1536596119403839, -0.04810766503214836, 0.005440655630081892, 0.09834864735603333, 0.04111864045262337, 0.09665153920650482, -0.023336002603173256, -0.11472529917955399, -0.21707573533058167, -0.0751279816031456, 0.2612660527229309, -0.08478369563817978, -0.010988933965563774, 0.10346775501966476, -0.05611943081021309, -0.08409323543310165, 0.17773979902267456, 0.0013872806448489428, -0.28431379795074463, 0.1379503458738327, -0.1345992237329483, -0.20439162850379944, 0.1431029886007309, 0.054115179926157, 0.05110381916165352, 0.01873408444225788, 0.09611095488071442, 0.0033121376764029264, 0.011220674961805344, -0.03678715229034424, 0.06592540442943573, 0.374124139547348, 0.014094588346779346, 0.42139437794685364, -0.3599301278591156, -0.06436312198638916, 0.1812347024679184, -0.018131140619516373, -0.1241670623421669, -0.09331110864877701, 0.04589942470192909, -0.01908227801322937, -0.23996102809906006, -0.001973634585738182, 0.13469982147216797, -0.11813417822122574, -0.023390643298625946, 0.003555525792762637, 0.25342223048210144, -0.21211883425712585, 0.20613346993923187, -0.27255263924598694, 0.229542076587677, -0.06546258181333542, 0.09758783131837845, -0.08973456919193268, -0.3623958230018616, 0.08013559877872467, 0.0573904886841774, -0.021446924656629562, 0.050758346915245056, -0.04738317057490349, -0.09350927174091339, -0.1275823414325714}, {-0.17706716060638428, -0.028320062905550003, -0.06790640950202942, 0.19684244692325592, 0.09953271597623825, 0.4515872597694397, -0.22755643725395203, -0.030274400487542152, 0.10874104499816895, 0.10181329399347305, -0.13540774583816528, 0.05832592770457268, 0.06467829644680023, 0.06269679963588715, -0.09060214459896088, -0.06881311535835266, 0.15727075934410095, 0.06361629068851471, -0.13237068057060242, -0.028490174561738968, -0.04873484745621681, -0.01894233748316765, -0.1319095939397812, 0.06667863577604294, -0.03239011764526367, -0.2644040584564209, 0.02123558521270752, -0.09675224870443344, 0.1497865468263626, 0.12242219597101212, -0.2113458514213562, 0.11588122695684433, 0.2030828595161438, -0.05731072276830673, 0.025040673092007637, 0.1958746314048767, -0.0864773690700531, 0.17985348403453827, 0.021179858595132828, 0.06766057759523392, 0.07707814872264862, 0.10995066165924072, -0.009340016171336174, -0.1732473373413086, 0.06543255597352982, 0.10776803642511368, -0.4334885776042938, 0.0676368847489357, -0.41667675971984863, -0.10879430919885635, 0.21125444769859314, -0.0680798888206482, -0.09731148928403854, -0.18500986695289612, -0.03099735826253891, 0.18806082010269165, -0.017243804410099983, 0.11403065919876099, 0.019528968259692192, 0.15628062188625336, -0.18790076673030853, -0.06454050540924072, 0.24632227420806885, 0.04349083453416824, 0.05394332483410835, -0.19528062641620636, 0.1150895431637764, 0.3761415183544159, -0.10900159180164337, 0.07105272263288498, 0.03629748895764351, 0.07724414765834808, 0.14113914966583252, -0.04365982487797737, -0.09210840612649918, -0.08243253082036972, 0.12836860120296478, -0.48106348514556885, -0.2728518545627594, -0.11574006825685501, 0.1613643616437912, -0.11130984872579575, 0.1288825273513794, -0.280411958694458, 0.044596876949071884, -0.01112513616681099, -0.07968366146087646, 0.2527409791946411, -0.1700276881456375, -0.04676279425621033, -0.09671278297901154, -0.03465835005044937, 0.08647572249174118, 0.19751760363578796, -0.2187470644712448, -0.1179836168885231, 0.04991341382265091, -0.12288764119148254, 0.2727120816707611, 0.33053597807884216, -0.18650537729263306, 0.07060085237026215, 0.07070004194974899, 0.03707221522927284, 0.19462279975414276, 0.12948761880397797, -0.020433640107512474, 0.09233161062002182, -0.03983186185359955, 0.12019659578800201, -0.032567910850048065, 0.16346950829029083, 0.0038145126309245825, -0.31483080983161926, 0.024675289168953896, -0.2676588296890259, 0.19123747944831848, -0.08949482440948486, -0.1073329746723175, -0.21737590432167053, -0.1020190417766571, 0.13965953886508942, -0.42089343070983887, 0.040622446686029434, -0.3555029034614563, 0.15593722462654114, 0.060193054378032684, -0.014877954497933388, 0.22882544994354248, -0.04104609042406082, 0.21074120700359344, 0.0900474414229393, 0.05472516641020775, -0.13026569783687592, 0.20166830718517303, 0.01343507133424282, 0.10190565139055252, -0.07762593030929565, -0.08070565015077591, -0.09872712194919586, -0.20685014128684998, -0.02485015243291855, -0.009297502227127552, 0.12647651135921478, 0.19710156321525574, -0.08843737840652466, -0.07344779372215271, -0.08531033247709274, 0.1860014796257019, 0.10536953806877136, 0.0964139997959137, -0.20941875874996185, 0.04289602115750313, -0.1608031690120697, 0.04866086319088936, 0.08879173547029495, -0.009735969826579094, 0.30226752161979675, -0.19342024624347687, 0.23512357473373413, -0.16515818238258362, -0.004192843567579985, 0.2011168897151947, 0.05232531949877739, -0.23471122980117798, -0.2244030237197876, -0.03932248055934906, -0.30802327394485474, 0.05482212081551552, 0.026520967483520508, 0.19331376254558563, -0.06465679407119751, -0.028385743498802185, 0.18228556215763092, 0.06020436808466911, 0.012276059947907925, 0.08437264710664749, 0.043918538838624954, 0.039669670164585114, 0.07954676449298859, 0.014367402531206608, 0.17281199991703033, 0.05112689733505249, 0.21554125845432281, 0.09787990152835846, 0.34725815057754517, -0.14927352964878082, -0.06128733605146408, -0.09573692083358765, 0.008730949833989143, -0.03272397443652153, -0.03494534268975258, -0.07873060554265976, 0.14609494805335999, -0.09843114763498306, 0.010947679169476032, 0.21324972808361053, 0.1441902071237564, 0.05370980501174927, -0.26855310797691345, -0.04691443592309952, -0.07728630304336548, -0.05317540466785431, 0.10737618058919907, -0.03831228241324425, -0.42648744583129883, -0.03848567605018616, 0.2587529718875885, -0.06522341072559357, -0.08531351387500763, 0.3185730278491974, 0.01781986840069294, 0.0410832054913044, -0.18935826420783997, 0.02168470062315464, 0.2856374680995941, 0.3035690486431122, 0.0256560780107975, 0.07220516353845596, 0.33161309361457825, -0.1041981652379036, 0.048213642090559006, 0.0321841686964035, -0.044937871396541595, 0.06843341886997223, -0.19622720777988434, 0.11334731429815292, 0.04762429744005203, -0.2881099581718445, -0.1853111982345581, -0.13141702115535736, -0.0014022180112078786, 0.04902447387576103, 0.22862224280834198, -0.04667647182941437, 0.2085755467414856, 0.2856377363204956, 0.008775025606155396, -0.120353102684021, -0.1117313802242279, -0.5556761026382446, -0.17200469970703125, -0.11440221965312958, -0.062265895307064056, 0.16504517197608948, 0.26125213503837585, -0.007822223007678986, -0.06401021778583527, 0.08098948746919632, 0.19388310611248016, 0.0783897414803505, -0.20294694602489471, 0.07162526994943619, -0.0697973370552063, 0.09553368389606476, 0.0625944435596466}, {0.0532514750957489, -0.05655265226960182, 0.04272698983550072, -0.16632935404777527, 0.02821064181625843, -0.24750933051109314, 0.10844632238149643, -0.1097828820347786, -0.18162952363491058, 0.26957204937934875, -0.48186299204826355, 0.16294075548648834, 0.06408053636550903, -0.20840667188167572, -0.059620656073093414, 0.3146870732307434, -0.16464418172836304, -0.08954210579395294, -0.18127596378326416, -0.19843702018260956, -0.10444596409797668, 0.09356587380170822, 0.0636020228266716, -0.05819311738014221, 0.02392738126218319, 0.15079259872436523, 0.07745999842882156, 0.21336591243743896, 0.21020281314849854, 0.21388666331768036, -0.04700896888971329, -0.0017190210055559874, 0.14851154386997223, -0.21749156713485718, -0.24144086241722107, 0.017797810956835747, -0.22024965286254883, -0.2987045347690582, -0.005925106815993786, -0.039139408618211746, 0.20796793699264526, 0.36622849106788635, 0.21324710547924042, 0.04430580884218216, 0.09736905992031097, 0.03504820913076401, 0.21426008641719818, 0.2614975869655609, -0.2557884156703949, -0.06016405299305916, 0.44008955359458923, 0.16107229888439178, -0.02918998710811138, 0.20763659477233887, -0.2271149605512619, 0.3957344591617584, 0.232648104429245, 0.19841578602790833, 0.19083787500858307, 0.03843999654054642, -0.11415938287973404, -0.36121052503585815, -0.17277935147285461, -0.15721778571605682, 0.3276282250881195, -0.25414279103279114, -0.7850620150566101, 0.2332185059785843, 0.5483168363571167, 0.258021742105484, 0.03675336018204689, -0.5129055380821228, -0.29730862379074097, -0.12200266122817993, -0.015133954584598541, -0.18509259819984436, -0.0027184486389160156, -0.31684258580207825, -0.1494251787662506, 0.17509891092777252, 0.21799536049365997, 0.12641268968582153, 0.06061798334121704, -0.3062628507614136, -0.4064469635486603, 0.3503761291503906, 0.12475771456956863, -0.1189017966389656, -0.43125447630882263, -0.028789158910512924, -0.412674218416214, -0.02382836490869522, -0.16578076779842377, 0.11222607642412186, -0.31682834029197693, -9.735830099089071e-05, 0.04469773545861244, -0.05015139281749725, -0.13212516903877258, -0.09780386090278625, 0.09175838530063629, -0.47025156021118164, 0.023879367858171463, -0.3644949197769165, 0.08166918158531189, -0.9687825441360474, 0.17707085609436035, 0.21488238871097565, -0.2347736805677414, -0.29469233751296997, 0.35421422123908997, -0.3863811790943146, -0.08491256833076477, -0.18633219599723816, 0.09279067814350128, 0.30220410227775574, -0.007665399927645922, 0.1762153059244156, 0.12631916999816895, 0.16219861805438995, 0.19107870757579803, 0.3759765326976776, 0.21907997131347656, -0.21879300475120544, 0.16663877665996552, 0.07015436142683029, 0.01635804772377014, -0.07699054479598999, 0.8577479720115662, 0.24611693620681763, 0.29268577694892883, -0.24003024399280548, 0.19990724325180054, 0.07037518918514252, -0.019231365993618965, -0.013663067482411861, 0.13959366083145142, 0.35103732347488403, 0.11923974752426147, -0.3180309236049652, 0.06872479617595673, -0.18757925927639008, 0.2945448160171509, -0.0692068412899971, 0.20683839917182922, -0.18740810453891754, 0.1235935389995575, -0.03485441580414772, -0.17076431214809418, 0.24684879183769226, 0.024088237434625626, -0.18529143929481506, -0.1652330905199051, -0.04110345244407654, -0.13156259059906006, -0.04031665250658989, -0.1270960122346878, 0.10007727891206741, 0.04090643301606178, 0.35708779096603394, -0.058397527784109116, 0.15272751450538635, 0.1299932897090912, -0.2584485411643982, 0.12816350162029266, -0.14696359634399414, -0.1165933832526207, -0.16446048021316528, -0.2402004897594452, -0.024803239852190018, 0.05443258211016655, -0.19393034279346466, -0.36440229415893555, -0.02939613163471222, -0.22364264726638794, 0.16919642686843872, -0.1093769371509552, -0.02686641551554203, -0.06375911831855774, 0.22273877263069153, -0.24802829325199127, -0.23808692395687103, 0.4906240999698639, 0.6209815144538879, -0.11152710765600204, -0.21745705604553223, -0.5706537365913391, -0.1581171751022339, -0.0827900841832161, 0.06012468412518501, -0.1815025806427002, 0.1404411345720291, 0.18384425342082977, 0.23210246860980988, 0.06032659858465195, -0.3081718683242798, -0.2897261083126068, 0.1503864824771881, 0.17567606270313263, -0.17340974509716034, 0.08883752673864365, 0.2047446221113205, -0.23608297109603882, -0.2188253551721573, 0.1417997181415558, -0.17054951190948486, -0.23261705040931702, -0.17494545876979828, -0.01253597717732191, -0.11295036226511002, -0.11307105422019958, -0.17407751083374023, 0.12811440229415894, 0.0036628313828259706, -0.12819801270961761, 0.12576042115688324, 0.1351241022348404, -0.01062888652086258, 0.05692380294203758, 0.017122119665145874, 0.004849185235798359, -0.028364187106490135, -0.02461954392492771, -0.13000746071338654, -0.13011540472507477, -0.5625357627868652, 0.20280712842941284, 0.027003824710845947, -0.45381197333335876, -0.13969850540161133, -0.01207555178552866, 0.19694750010967255, -0.31805360317230225, -0.2599395215511322, 0.4784713089466095, -0.009297548793256283, -0.013350731693208218, -0.06918385624885559, -0.1986527293920517, -0.02317586913704872, -0.3190354108810425, 0.17597626149654388, -0.12595286965370178, -0.034365128725767136, -0.0665862113237381, 0.13496078550815582, -0.2290622740983963, 0.2701771855354309, 0.33028456568717957, 0.0369122140109539, 0.06564687937498093, 0.010947106406092644, -0.24971304833889008, 0.05510346591472626, 0.08638761937618256, -0.08784843981266022}, {0.1025950163602829, 0.0497199110686779, 0.15472058951854706, 0.09920639544725418, 0.09315663576126099, -0.1315421462059021, 0.12172197550535202, -0.1618000566959381, 0.10900174081325531, -0.06608113646507263, -0.1072297915816307, 0.16448365151882172, 0.11081631481647491, -0.07307320833206177, -0.03433351218700409, 0.05893644317984581, 0.03445671126246452, -0.13982658088207245, -0.14057007431983948, -0.21212700009346008, 0.01608291082084179, 0.2580272853374481, 0.11901962757110596, 0.13217172026634216, 0.04044709354639053, -0.15663008391857147, -0.23167307674884796, 0.12650637328624725, -0.21341867744922638, 0.06687270849943161, -0.2266020029783249, -0.08168523758649826, 0.2743838131427765, -0.24530701339244843, 0.20928260684013367, -0.0823381319642067, 0.011961713433265686, 0.0657617598772049, 0.025254327803850174, -0.026672538369894028, 0.09858061373233795, 0.11149505525827408, 0.19492560625076294, 0.03129854425787926, 0.011401037685573101, -0.17934541404247284, 0.02498845010995865, -0.04022897779941559, -0.12048347294330597, 0.05539901182055473, -0.18945743143558502, -0.06708472222089767, -0.1420605331659317, 0.16498780250549316, -0.1329796314239502, 0.037216946482658386, 0.1878110021352768, -0.01853860914707184, -0.10804208368062973, -0.2649132311344147, -0.04236083850264549, -0.08263978362083435, -0.05738251656293869, 0.0550701804459095, 0.026463255286216736, 0.09793907403945923, -0.03961244598031044, -0.06513615697622299, 0.08511795103549957, -0.06905142217874527, 0.17457526922225952, -0.09362984448671341, -0.09112795442342758, -0.11023873835802078, 0.03440225496888161, 0.1400923728942871, 0.06858350336551666, -0.0385291762650013, 0.1352597326040268, -0.21182307600975037, -0.09605003893375397, 0.16648517549037933, 0.09340783953666687, -0.014711088500916958, 0.21343994140625, -0.09868855774402618, 0.07031925022602081, -0.11907802522182465, -0.13243241608142853, -0.12364936619997025, 0.039090346544981, -0.13456991314888, -0.010131464339792728, 0.08223170042037964, -0.022167690098285675, 0.006318297237157822, 0.16089192032814026, 0.09917640686035156, -0.057554692029953, 0.08418583124876022, -0.038205668330192566, 0.016039162874221802, 0.12364012002944946, 0.05581121891736984, 0.48215121030807495, -0.012021669186651707, 0.08675038814544678, -0.0954740047454834, 0.16255038976669312, -0.1833239644765854, 0.17576266825199127, -0.02659498155117035, -0.03191731870174408, 0.28800860047340393, 0.08916396647691727, 0.04016871005296707, -0.00031899739406071603, -0.19668146967887878, 0.23674693703651428, 0.007490186952054501, -0.10154026746749878, -0.1365535408258438, -0.20238012075424194, -0.029498761519789696, 0.05597323179244995, 0.0785764753818512, 0.0786949172616005, 0.009223883971571922, 0.048349857330322266, -0.15004052221775055, 0.3475983738899231, 0.1135956346988678, 0.07913175225257874, 0.03034435212612152, -0.2120969444513321, -0.0009891149820759892, 0.06252869963645935, 0.27807679772377014, -0.048942651599645615, 0.24671100080013275, 0.1376776546239853, -0.09024295210838318, -0.08473533391952515, -0.07725599408149719, 0.09962155669927597, 0.09447770565748215, 0.060773737728595734, 0.07796590775251389, 0.08412694185972214, -0.18379931151866913, 0.021626845002174377, -0.0566520169377327, -0.0033279128838330507, 0.05419454723596573, 0.17031095921993256, -0.011867513880133629, -0.07506810128688812, 0.1810324341058731, 0.08766977488994598, -0.05290505662560463, -0.10428325831890106, 0.18319955468177795, 0.05252519249916077, -0.07913339883089066, 0.15477892756462097, 0.17768512666225433, 0.1090606302022934, -0.02319704182446003, -0.15117096900939941, -0.16187965869903564, 0.09358108043670654, 0.10444799065589905, 0.12203040719032288, -0.027999958023428917, 0.02609594538807869, -0.041827891021966934, 0.01698307693004608, 0.07123026996850967, 0.009608622640371323, -0.002669173525646329, 0.09849313646554947, 0.0702618733048439, -0.01416296511888504, -0.1858808994293213, 0.010426524095237255, 0.10288001596927643, 0.19966796040534973, -0.05635388195514679, -0.0095194261521101, 0.09743189066648483, -0.07212334126234055, -0.11257627606391907, -0.02245909348130226, -0.10977178066968918, -0.06490855664014816, -0.1287594735622406, 0.15856234729290009, 0.17326052486896515, 0.0032076153438538313, 0.17130881547927856, 0.08116081357002258, 0.050090350210666656, -0.1308174431324005, -0.04850134253501892, 0.005325112957507372, 0.09316765516996384, 0.14085085690021515, 0.29534897208213806, -0.14724594354629517, -0.03322352096438408, -0.01924731768667698, 0.07742666453123093, -0.08254305273294449, 0.05524672195315361, -0.034988850355148315, 0.09469173848628998, 0.1330748051404953, 0.17335835099220276, 0.10301341861486435, 0.03625911474227905, -0.004727051593363285, 0.008375081233680248, 0.08202848583459854, 0.07828526198863983, 0.2238762378692627, 0.23270867764949799, -0.0071120718494057655, 0.044152528047561646, -0.045386236160993576, 0.1379895955324173, 0.04579561948776245, -0.03182623162865639, -0.07345754653215408, 0.06074145808815956, 0.029772846028208733, -0.0031603102106601, -0.08198434114456177, 0.19832652807235718, 0.024085137993097305, -0.09397879987955093, 0.13985426723957062, 0.1293456107378006, -0.20397187769412994, -0.03342762589454651, -0.05881142616271973, 0.08242900669574738, 0.015964578837156296, -0.08451948314905167, -0.07071710377931595, 0.02391945943236351, 0.3029448390007019, 0.19636011123657227, -0.17772945761680603, 0.11531520634889603, -0.053672902286052704, -0.09123159945011139}, {-0.10417485982179642, -0.011445265263319016, -0.056604981422424316, -0.2155735045671463, 0.020408544689416885, -0.02084488421678543, 0.07832050323486328, 0.08447326719760895, -0.2296016812324524, 0.09977435320615768, -0.13502927124500275, 0.10626471042633057, -0.14666181802749634, 0.08227967470884323, -0.0711401179432869, 0.05722102150321007, -0.15649783611297607, -0.0009077638387680054, 0.08485125005245209, 0.10751459747552872, 0.01775200478732586, 0.05530663579702377, -0.06761690974235535, 0.05824916809797287, -0.005658292211592197, 0.013806496746838093, -0.011328740045428276, 0.06946634501218796, -0.02046024799346924, -0.2387552559375763, 0.09346868842840195, -0.04944286122918129, 0.1000712588429451, -0.17223070561885834, -0.021322788670659065, 0.05193227902054787, 0.018856218084692955, 0.22158849239349365, 0.19613195955753326, -0.01799243874847889, -0.021199317649006844, -0.04211249202489853, -0.0039020953699946404, 0.13290780782699585, -0.08465151488780975, -0.17316721379756927, -0.11495643854141235, -0.014726689085364342, -0.0007415651925839484, -0.0714525356888771, 0.0660739615559578, 0.33232346177101135, -0.08042779564857483, -0.08561092615127563, -0.02329912781715393, 0.10378045588731766, -0.08533679693937302, -0.04554739221930504, -0.04890858381986618, 0.025320664048194885, -0.08962056785821915, 0.03418780118227005, -0.02354002557694912, 0.12682786583900452, -0.04166324436664581, -0.12514185905456543, -0.0006193722365424037, -0.10273004323244095, 0.06149106100201607, 0.18970531225204468, -0.004346761386841536, 0.01026552077382803, -0.11984574794769287, -0.07926858961582184, 0.29268723726272583, 0.0036976539995521307, -0.10437173396348953, 0.10263997316360474, 0.02072242833673954, -0.03217947855591774, -0.08331503719091415, 0.15159547328948975, 0.34795594215393066, -0.05301925167441368, 0.008422614075243473, -0.0889413058757782, -0.001443090382963419, 0.04474654048681259, 0.17834578454494476, 0.11217395961284637, 0.06400742381811142, 0.2696562111377716, 0.09596912562847137, -0.13729418814182281, -0.11274546384811401, 0.008279497735202312, 0.10213059186935425, 0.1359914243221283, -0.07508303225040436, 0.19831770658493042, 0.020826837047934532, 0.06314175575971603, 0.06789030879735947, -0.1042589619755745, 0.09620573371648788, -0.05388695374131203, 0.03742954507470131, -0.10403025150299072, 0.19865533709526062, 0.2457161247730255, -0.12747764587402344, -0.036341890692710876, -0.13176468014717102, -0.06908734142780304, 0.2175750881433487, 0.3672666847705841, 0.1097998172044754, 0.055855926126241684, -0.08881297707557678, -0.02970927394926548, 0.07992298156023026, -0.1565117985010147, -0.12817414104938507, 9.932398097589612e-05, 0.046493515372276306, -0.3496968448162079, 0.050171010196208954, -0.0961833968758583, -0.0036117578856647015, 0.028674976900219917, 0.3626636266708374, 0.13709814846515656, 0.11611142009496689, 0.01810920424759388, -0.09144032746553421, 0.21267622709274292, 0.04079725220799446, -0.15811452269554138, 0.10553896427154541, 0.045052770525217056, -0.0819525271654129, 0.17035168409347534, -0.05900552123785019, -0.3310447335243225, 0.0835469663143158, -0.06646331399679184, -0.1476600021123886, 0.015405643731355667, -0.0615251325070858, -0.0361858494579792, -0.011316653341054916, -0.0036081750877201557, -0.05766230821609497, -0.129095196723938, 0.08591002225875854, -0.10244102776050568, 0.0894118919968605, 0.06609185039997101, -0.07370829582214355, -0.0804448351264, 0.14243696630001068, -0.15913435816764832, -0.031708985567092896, 0.011534188874065876, 0.23750855028629303, -0.18572074174880981, 0.027182022109627724, -0.04952950030565262, 0.06654716283082962, 0.1178819090127945, 0.1467411071062088, -0.07719577848911285, 0.21892192959785461, 0.1448824554681778, 0.014812947250902653, -0.12002529203891754, -0.0049447896890342236, 0.16455866396427155, 0.0749620720744133, -0.1083347499370575, 0.15033917129039764, 0.238228902220726, 0.059090062975883484, -0.07876995950937271, 0.2172478884458542, -0.03154841065406799, 0.4399937689304352, 0.002724097343161702, -0.0797177255153656, -0.032617777585983276, 0.11686154454946518, 0.024479230865836143, -0.11711160838603973, -0.1581042855978012, -0.05388372018933296, 0.04682398959994316, -0.1550154983997345, 0.07020901143550873, -0.05273164063692093, 0.15794388949871063, -0.2059655785560608, 0.10337167978286743, -0.09585006535053253, -0.626493513584137, 0.10076655447483063, -0.022239698097109795, -0.08962705731391907, -0.04890910163521767, 0.06760584563016891, 0.03778348118066788, 0.04498931020498276, 0.30101385712623596, -0.052681468427181244, -0.05207166075706482, 0.04986625909805298, -0.0076178014278411865, 0.08981654047966003, 0.060873232781887054, 0.09128708392381668, -0.04701165854930878, 0.1624988615512848, 0.01972178742289543, -0.11492282897233963, 0.3761826455593109, 0.11414670199155807, 0.2790714204311371, 0.13166241347789764, -0.20907150208950043, 0.05248983949422836, 0.047437071800231934, -0.11442624777555466, 1.9655230062198825e-05, 0.021848510950803757, 0.0286695696413517, -0.07458064705133438, -0.10768333077430725, 0.203774094581604, -0.21066918969154358, 0.03767712786793709, 0.14719191193580627, 0.09291277825832367, -0.07220356166362762, 0.0021059263963252306, 0.040868185460567474, 0.13381220400333405, 0.03785928711295128, -0.22120103240013123, -0.045979201793670654, -0.03712717443704605, -0.21612177789211273, -0.12279932200908661, 0.4530286192893982, 0.15765681862831116, 0.03330039232969284, 0.05066158249974251, 0.04741340130567551}, {0.02127252146601677, 0.025002840906381607, -0.11548332124948502, 0.17100535333156586, 0.3059001863002777, -0.13975980877876282, 0.25350743532180786, 0.1864299476146698, 0.11959529668092728, 0.20723672211170197, 0.11218729615211487, 0.24318641424179077, -0.10768497735261917, -0.17920586466789246, -0.03386369347572327, 0.06752236932516098, -0.15794813632965088, -0.14216475188732147, 0.044959940016269684, -0.07830743491649628, 0.00881133321672678, -0.0916312038898468, -0.09952960908412933, 0.06855922192335129, -0.003134422702714801, 0.3454555869102478, -0.22059385478496552, 0.07094121724367142, 0.031754594296216965, 0.043655797839164734, 0.1654285192489624, -0.05864901468157768, 0.2644362151622772, -0.21778161823749542, 0.13582353293895721, -0.1565566509962082, 0.0567922405898571, 0.08715783804655075, 0.021294210106134415, 0.17922374606132507, 0.17260237038135529, 0.09935685247182846, 0.15577571094036102, 0.12473543733358383, 0.037327736616134644, 0.19133152067661285, 0.15644703805446625, -0.19854788482189178, -0.03522082790732384, 0.09670035541057587, 0.027838511392474174, -0.007947773672640324, -0.262235164642334, 0.14821888506412506, -0.1701039969921112, 0.022729376330971718, 0.09646310657262802, -0.1984463483095169, -0.008433382026851177, 0.13755643367767334, -0.0507342629134655, -0.040310222655534744, 0.12818093597888947, 0.06538896262645721, -0.08107361942529678, 0.09199569374322891, 0.09231270849704742, -0.03648172691464424, 0.1877506822347641, 0.06385326385498047, -0.036644596606492996, -0.12800607085227966, 0.030789310112595558, -0.0195445753633976, -0.10994447767734528, 0.18435046076774597, -0.15940813720226288, -0.04482029750943184, 0.47333577275276184, -0.03396212309598923, -0.03196783736348152, 0.20872855186462402, 0.11075927317142487, 0.090410977602005, 0.07378050684928894, 0.10522425919771194, 0.018030712381005287, -0.03558624908328056, -0.10380738228559494, 0.004417658317834139, -0.07024307548999786, -0.15412472188472748, -0.12417349964380264, -0.20485790073871613, 0.07687390595674515, 0.05438368394970894, 0.01541651226580143, -0.0684579536318779, 0.07312555611133575, -0.11089560389518738, 0.11326037347316742, 0.13606198132038116, -0.1257186233997345, -0.12740503251552582, 0.26424357295036316, -0.5061834454536438, -0.02384991943836212, 0.2590748965740204, 0.037551164627075195, -0.09253247082233429, -0.007253558840602636, 0.04042665287852287, -0.06341609358787537, 0.09549553692340851, 0.11976146697998047, 0.10015208274126053, 0.035017747431993484, -0.0033229412510991096, 0.016390785574913025, 0.15642786026000977, 0.13031145930290222, 0.06576398015022278, 0.11769041419029236, -0.23993928730487823, 0.03656557947397232, -0.08807886391878128, -0.10482774674892426, 0.0476873554289341, 0.12898442149162292, -0.367849737405777, -0.14137016236782074, -0.010816244408488274, -0.27853190898895264, 0.002290592063218355, 0.09633096307516098, -0.016938483342528343, -0.02457991987466812, 0.2737746238708496, -0.07892192900180817, 0.1869712471961975, 0.06877441704273224, 0.09911321103572845, -0.010234183631837368, 0.04868274927139282, 0.019666045904159546, 0.016686908900737762, 0.05141215771436691, 0.2290319800376892, -0.10150811076164246, -0.14284631609916687, -0.33994585275650024, -0.03473488986492157, -0.16415777802467346, 0.02472997084259987, -0.04336625337600708, -0.004909207578748465, -0.02563607692718506, 0.23749394714832306, 0.16127917170524597, -0.028879374265670776, 0.1487097144126892, 0.18794187903404236, -0.06528853625059128, 0.053434792906045914, 0.1131880134344101, 0.0361233726143837, 0.2967652380466461, 0.049644552171230316, -0.03634493798017502, -0.029070541262626648, -0.055768322199583054, 0.03866982087492943, -0.010994566604495049, -0.33565282821655273, 0.08086463063955307, 0.11179234832525253, 0.10278574377298355, 0.03632495179772377, 0.3105368912220001, -0.24878640472888947, -0.09007807821035385, 0.23999719321727753, -0.08825714886188507, 0.0993124395608902, -0.07028770446777344, -0.011522126384079456, 0.04515805095434189, -0.00570471677929163, 0.008891438134014606, -0.0839400514960289, -0.029280375689268112, 0.026378100737929344, -0.1504564881324768, -0.028379149734973907, 0.1140902191400528, -0.16635479032993317, -0.19130393862724304, 0.18433070182800293, 0.033845204859972, -0.10794954746961594, -0.07466913759708405, 0.21866142749786377, -0.15805107355117798, -0.3092731535434723, 0.30144813656806946, 0.2288183718919754, -0.3389301896095276, -0.04027164727449417, 0.005303805693984032, -0.20376050472259521, 0.013161173090338707, -0.02518923208117485, -0.0591120682656765, 0.3003941476345062, -0.020985253155231476, -0.03074852004647255, -0.09509585797786713, 0.18564225733280182, 0.039087940007448196, -0.0650584027171135, 0.06487094610929489, -0.09381680190563202, 0.026661992073059082, -0.07496046274900436, -0.33555614948272705, -0.13752220571041107, -0.02144087851047516, 0.09431751072406769, -0.13257215917110443, -0.1955856829881668, -0.10236354917287827, 0.011148390360176563, -0.13730280101299286, -0.0111025869846344, 0.15874120593070984, -0.25980430841445923, -0.1433693766593933, -0.07056507468223572, 0.10612151026725769, -0.20332174003124237, 0.11143921315670013, -0.058117661625146866, -0.12046485394239426, 0.35479357838630676, 0.09013888239860535, -0.11374856531620026, -0.07658381760120392, 0.03454442694783211, 0.01325453445315361, 0.07259410619735718, 0.14562880992889404, -0.020674221217632294, -0.03312891349196434, 0.10047150403261185, 0.2140464335680008, -0.27027472853660583}, {0.03748233616352081, -0.024020357057452202, -0.10677832365036011, 0.058451808989048004, -0.004106160718947649, -0.011591866612434387, -0.07214163988828659, 0.07544222474098206, 0.009695424698293209, -0.015911193564534187, -0.03715774044394493, -0.0402982123196125, -0.01987806148827076, -0.06259981542825699, -0.07247623801231384, 0.0864870622754097, 0.07617322355508804, -0.023021234199404716, -0.0725005716085434, 0.023663897067308426, 0.09840834885835648, 0.08127862960100174, 0.1223047524690628, -0.0765659436583519, 0.07847326248884201, 0.03557755425572395, 0.07734405249357224, 0.11154084652662277, -0.04175297170877457, 0.05175983905792236, -0.007150372955948114, 0.034425634890794754, 0.12782475352287292, -0.02118547447025776, -0.007569476962089539, 0.0742565467953682, 0.1684218943119049, 0.03838001936674118, 0.06299496442079544, -0.018697410821914673, 0.1105070635676384, 0.018702398985624313, -0.006309573072940111, 0.05936923623085022, 0.004955305717885494, 0.0840243399143219, 0.045698512345552444, -0.0060819401405751705, -0.07266245782375336, 0.053335435688495636, 0.2416134923696518, -0.09704045951366425, -0.007918920367956161, 0.21857033669948578, 0.10867170244455338, -0.06492142379283905, 0.03528055548667908, -0.03813931345939636, -0.09172070771455765, 0.08021355420351028, -0.17705900967121124, 0.013652091845870018, 0.018665771931409836, 0.30198580026626587, -0.05436884984374046, -0.05256232991814613, -0.1231522411108017, 0.013792435638606548, 0.0907118171453476, -0.008233149535953999, -0.17896433174610138, 0.09922897070646286, -0.022275704890489578, -0.0002560522116255015, 0.006863962858915329, 0.003931913059204817, 0.10128219425678253, 0.07423955947160721, -0.07467041164636612, -0.03738848492503166, 0.12749460339546204, 0.07617989927530289, -0.0451778843998909, -0.038373589515686035, 0.08687052875757217, 0.004271944053471088, -0.10040067136287689, 0.15325208008289337, 0.03655626252293587, -0.023482227697968483, 0.05826319754123688, 0.12091986835002899, -0.14516066014766693, 0.09558979421854019, -0.01727314293384552, -0.04627777636051178, -0.11914917826652527, 0.03871859237551689, 0.021883003413677216, -0.035297371447086334, 0.1487390697002411, 0.05302833020687103, 0.09181845933198929, 0.04781058430671692, 0.08047729730606079, 0.05993518605828285, -0.07134412229061127, -0.14632071554660797, -0.12413426488637924, 0.16506198048591614, -0.11149149388074875, -0.029378872364759445, 0.07753909379243851, 0.060740552842617035, 0.09142749011516571, -0.0671209767460823, -0.06314381211996078, -0.0896671861410141, -0.07165539264678955, 0.010177948512136936, -0.26353782415390015, 0.04968088120222092, -0.02089359611272812, 0.0023530442267656326, 0.0251773614436388, -0.12747423350811005, -0.11820387095212936, 0.0024341812822967768, 0.04289564490318298, -0.09014112502336502, -0.09501790255308151, -0.055086251348257065, 0.04602688550949097, 0.016086934134364128, 0.16757196187973022, 0.27160727977752686, 0.002069909358397126, -0.03274707496166229, 0.14113499224185944, 0.04977425932884216, -0.04795185476541519, 0.21630488336086273, -0.16255731880664825, -0.0018929577199742198, -0.07444266229867935, -0.04010878875851631, -0.15435893833637238, 0.006127013359218836, 0.013143192045390606, -0.10448329150676727, -0.04373537749052048, 0.07252747565507889, -0.061838485300540924, 0.04821603000164032, 0.0067445687018334866, -0.038836851716041565, 0.08083692938089371, -0.03595136106014252, -0.045924436300992966, -0.06893876194953918, -0.07608833909034729, -0.10744626075029373, 0.02331606298685074, 0.17910684645175934, -0.028113992884755135, 0.07828536629676819, -0.11300631612539291, -0.06613296270370483, 0.11076126992702484, 0.10266143083572388, 0.005986432079225779, 0.13115237653255463, 0.0547848679125309, 0.0027503950987011194, -0.014269289560616016, -0.03463965654373169, -0.03585633262991905, -0.05048028379678726, -0.14858557283878326, 0.06782960891723633, 0.010251226834952831, 0.01562538556754589, 0.03355815261602402, -0.0374019630253315, 0.030063625425100327, -0.14669683575630188, -0.001243231468833983, -0.04849258065223694, 0.020109029486775398, -0.2482028752565384, 0.17778125405311584, 0.09172941744327545, 0.13160592317581177, 0.1266801506280899, 0.13780750334262848, 0.03078429400920868, -0.005834078881889582, -0.15077678859233856, -0.05814654380083084, -0.1421303004026413, -0.12113804370164871, -0.039162226021289825, 0.003262871876358986, 0.03099747747182846, -0.030390821397304535, 0.07907402515411377, 0.07520132511854172, -0.09735794365406036, 0.10478765517473221, 0.05600019916892052, -0.1970822811126709, -0.018524855375289917, 0.11885710805654526, 0.03439939394593239, -0.03779182955622673, 0.10432681441307068, 0.1268882304430008, -0.018588770180940628, 0.11031553894281387, -0.0630929097533226, -0.05539955943822861, 0.12933693826198578, -0.03447515517473221, -0.028879975900053978, 0.09767016768455505, -0.05932703614234924, -0.09300870448350906, 0.09713233262300491, 0.004496573004871607, 0.0492517463862896, -0.004693248309195042, 0.0275247935205698, 0.007459128275513649, -0.07483935356140137, -0.2241036742925644, 0.008311019279062748, -0.05359466373920441, 0.05499277263879776, 0.037982456386089325, -0.17182226479053497, -0.008914261125028133, 0.039759956300258636, 0.012541409581899643, -0.019943930208683014, 0.02912609651684761, -0.15744276344776154, 0.061785995960235596, 0.05478009581565857, -0.02945712022483349, 0.12237474322319031, -0.05691944807767868, -0.16848190128803253, 0.07266383618116379, -0.03853877633810043, -0.06470760703086853, -0.12389841675758362}, {0.22895081341266632, 0.012290543876588345, -0.06329185515642166, 0.16121017932891846, -0.1578965038061142, -0.1760866641998291, -0.10212049633264542, 0.12562406063079834, -0.026732441037893295, -0.13639996945858002, 0.057090338319540024, -0.17004887759685516, -0.07561586797237396, 0.16866353154182434, 0.03572744503617287, 0.07045532017946243, -0.0026938014198094606, 0.011894561350345612, 0.02918155863881111, 0.032814785838127136, 0.09784910082817078, -0.17248417437076569, 0.05658683553338051, -0.09896296262741089, -0.16890858113765717, -0.022517401725053787, 0.22042915225028992, 0.012839434668421745, 0.31322208046913147, -0.022659113630652428, -0.0058433921076357365, 0.21010123193264008, -0.07000134140253067, 0.1443483829498291, -0.2382071167230606, 0.05610330030322075, 0.027183592319488525, 0.19247640669345856, 0.30057498812675476, 0.04314688220620155, 0.12236935645341873, -0.11243586242198944, -0.04680810496211052, -0.07278014719486237, -0.10971927642822266, -0.0341961495578289, -0.05670921504497528, 0.11764348298311234, 0.16482089459896088, 0.08346198499202728, -0.12026356160640717, 0.07699395716190338, 0.15825363993644714, -0.5213230848312378, -0.046998877078294754, 0.26358312368392944, 0.09879929572343826, 0.3495255708694458, 0.03333579748868942, -0.21800774335861206, 0.30225229263305664, 0.127201110124588, -0.15650954842567444, 0.014875617809593678, -0.0879981592297554, -0.15704326331615448, 0.11252466589212418, -0.14528131484985352, 0.03904686123132706, -0.07286882400512695, -0.011359171010553837, -0.06056346744298935, 0.12287513166666031, 0.16507047414779663, 0.15547333657741547, -0.042342402040958405, -0.12650559842586517, -0.0075173829682171345, 0.1595960259437561, 0.4292452335357666, -0.02203136868774891, -0.1697794646024704, 0.18359117209911346, -0.09916085004806519, -0.29796531796455383, -0.06144121289253235, -0.08051425218582153, 0.07940272986888885, 0.09982927143573761, 0.03336611017584801, 0.035811372101306915, -0.04709773138165474, 0.1670924425125122, 0.0054250420071184635, -0.1390811949968338, 0.06626494973897934, -0.09851396083831787, 0.20972542464733124, 0.1062561646103859, -0.04139760509133339, -0.061872806400060654, -0.14306478202342987, 0.13213492929935455, 0.30631089210510254, -0.05872250720858574, 0.12847763299942017, 0.09221170097589493, 0.04522344842553139, -0.0065497420728206635, -0.21333280205726624, 0.059141531586647034, 0.27557578682899475, 0.0012104596244171262, -0.12888307869434357, -0.15679074823856354, 0.17594274878501892, -0.12299595028162003, -0.05698157474398613, 0.10780181735754013, -0.2855392098426819, -0.006182441022247076, 0.24875614047050476, 0.10625054687261581, 0.04419500753283501, 0.22634759545326233, 0.3343825936317444, -0.19194342195987701, -0.09514379501342773, -0.1856338232755661, 0.015693316236138344, -0.14107690751552582, -0.10105300694704056, -0.10903381556272507, -0.035195834934711456, 0.560001790523529, -0.1133434996008873, -0.08981917053461075, 0.020382659509778023, -0.184478759765625, 0.15284477174282074, -0.05454360693693161, 0.1488284319639206, -0.09513632953166962, -0.05153019353747368, -0.08475109934806824, 0.08462662249803543, 0.03617439046502113, -0.08960118144750595, -0.1417853981256485, -0.027270233258605003, 0.06625497341156006, -0.31276875734329224, -0.012669578194618225, -0.011676376685500145, 0.011806993745267391, -0.03686599060893059, -0.17923307418823242, 0.13670893013477325, -0.30054184794425964, -0.22977207601070404, -0.045647867023944855, 0.18076056241989136, -0.0038678357377648354, 0.30773651599884033, -0.050710998475551605, -0.09401804953813553, -0.0692526251077652, 0.07985398173332214, 0.05429856479167938, -0.08787709474563599, -0.07184401154518127, -0.19029048085212708, -0.07595677673816681, 0.06550700217485428, -0.049255628138780594, -0.027996787801384926, -0.06949687004089355, -0.02041538991034031, -0.1671438068151474, 0.0272416640073061, -0.029110321775078773, -0.15777534246444702, 0.11220797896385193, -0.228707417845726, 0.11409962177276611, 0.08545506745576859, -0.1940964013338089, 0.1045098602771759, -0.05484882369637489, 0.1765950471162796, -0.05064682289958, 0.11421367526054382, 0.009395431727170944, -0.3131972849369049, -0.3687435984611511, -0.19493989646434784, 0.07649362087249756, -0.15818443894386292, 0.06070799008011818, 0.1619795560836792, 0.003355216234922409, 0.06597504019737244, -0.011823488399386406, -0.09473339468240738, 0.12323781102895737, -0.03213309869170189, 0.08505839854478836, 0.017635513097047806, -0.08259700983762741, 0.012305798940360546, 0.12220890820026398, -0.129408061504364, 0.11815488338470459, -0.23293046653270721, 0.020734066143631935, -0.1627473384141922, 0.1352485716342926, 0.09181508421897888, -0.2476169914007187, 0.2694995105266571, 0.11241689324378967, 0.09041401743888855, 0.04456439986824989, -0.19586697220802307, 0.4760846197605133, -0.010743065737187862, 0.3079688847064972, -0.11319104582071304, 0.02351546287536621, 0.05981503427028656, 0.11737915873527527, 0.048809681087732315, -0.12908564507961273, -0.1458074301481247, -0.022919999435544014, 0.0799492746591568, -0.12967327237129211, 0.012210629880428314, -0.14453575015068054, -0.18938389420509338, -0.07306236773729324, 0.012585658580064774, -0.14563411474227905, -0.0032299382146447897, -0.08360757678747177, 0.03046579100191593, 0.09028039127588272, 0.045638248324394226, 0.06376327574253082, -0.09639052301645279, -0.1283855140209198, 0.3242725133895874, -0.1745038628578186, 0.00930102914571762, -0.27395370602607727, 0.23276299238204956}, {0.10080740600824356, 0.07810318470001221, -0.0375373475253582, 0.07082974165678024, 0.27488258481025696, -0.14173822104930878, 0.1158384457230568, 0.28642401099205017, 0.08562438935041428, 0.012151971459388733, -0.04701134189963341, 0.20655614137649536, -0.023004505783319473, 0.03225208446383476, 0.021929392591118813, -0.024934520944952965, -0.16864602267742157, 0.008490626700222492, 0.18705202639102936, 0.035290103405714035, 0.028756609186530113, 0.0054695033468306065, -0.06170641630887985, 0.0034319651313126087, -0.44301271438598633, 0.035326652228832245, -0.024041902273893356, -0.12878026068210602, -0.0961655005812645, 0.03275800123810768, 0.13307474553585052, -0.19930221140384674, -0.11078084260225296, -0.05061262473464012, 0.09673083573579788, 0.17851266264915466, 0.04552358761429787, -0.1043972298502922, -0.10713103413581848, 0.12340763211250305, 0.07086291164159775, -0.08320679515600204, 0.04239986464381218, 0.07271062582731247, 0.03285146504640579, -0.18584945797920227, -0.0757393017411232, -0.01941274292767048, 0.0720219612121582, 0.06135305017232895, 0.19197990000247955, -0.31517472863197327, -0.008691233582794666, -0.23354627192020416, 0.1167316734790802, 0.020258991047739983, 0.14906463027000427, 0.19697682559490204, 0.01626923680305481, 0.059899888932704926, -0.2370842546224594, -0.02886226773262024, 0.15635713934898376, -0.23359885811805725, -0.00933939777314663, 0.0759238600730896, -0.07284554839134216, 0.2119983285665512, -0.009138326160609722, 0.002894303761422634, -0.20882149040699005, 0.17428158223628998, 0.18727880716323853, -0.0068142046220600605, 0.14297305047512054, 0.03406928852200508, 0.020177209749817848, -0.016658129170536995, 0.0935392826795578, -0.11344641447067261, 0.14645445346832275, 0.22074849903583527, 0.10275658220052719, -0.19950731098651886, 0.061677485704422, 0.014325723052024841, 0.2362758219242096, 0.046998925507068634, 0.06488294899463654, -0.07629480957984924, 0.04062230885028839, -0.06459714472293854, 0.226930171251297, -0.1115250289440155, -0.03671438619494438, 0.033511657267808914, 0.02656969428062439, -0.032441187649965286, -0.18947026133537292, -0.035217657685279846, 0.020476777106523514, -0.2824058532714844, -0.03233179450035095, -0.056106068193912506, -0.26988354325294495, 0.13626986742019653, -0.09273163974285126, -0.07321176677942276, -0.017136285081505775, 0.10144652426242828, 0.12748484313488007, 0.12055393308401108, 0.10609649121761322, -0.5370903015136719, -0.02545372024178505, -0.11084707081317902, -0.053419310599565506, -0.154556542634964, -0.07134614884853363, -0.10703688859939575, 0.175022691488266, -0.012778067030012608, -0.10146640241146088, -0.3966781795024872, 0.3510144352912903, 0.04627728462219238, -0.11406423896551132, 0.022681094706058502, -0.0005740924971178174, -0.04035333916544914, 0.11816238611936569, 0.029286567121744156, -0.06412658840417862, 0.060466468334198, 0.05783107876777649, -0.05949816852807999, -0.011234807781875134, 0.02066311240196228, 0.15289878845214844, -0.1793774664402008, -0.03193845972418785, -0.10041522979736328, -0.00965724978595972, -0.003262660000473261, -0.20730160176753998, 0.05061860382556915, -0.024878764525055885, 0.0047622062265872955, -0.5285788178443909, -0.06647366285324097, 0.04950644075870514, -0.018818389624357224, 0.014701585285365582, 0.0017829241696745157, 0.10867523401975632, -0.09978809952735901, -0.023554019629955292, 0.17929615080356598, -0.020290439948439598, 0.06112271547317505, -0.005583955906331539, 0.02244281768798828, -0.05296121537685394, -0.06318843364715576, -0.016988782212138176, -0.21730875968933105, -0.13917072117328644, -0.08287803083658218, -0.07209452986717224, -0.25202706456184387, -0.047618597745895386, -0.021993553265929222, 0.11876560002565384, 0.02426944114267826, 0.07140450924634933, 0.015809638425707817, 0.10762278735637665, -0.014906786382198334, -0.009837773628532887, 0.0037674852646887302, 0.07645181566476822, 0.008730980567634106, 0.060143474489450455, 0.37999728322029114, 0.3086429834365845, 0.1613352745771408, -0.11414644122123718, -0.27775245904922485, -0.19843612611293793, -0.025911500677466393, 0.1508379727602005, -0.06410767138004303, 0.24686424434185028, 0.1482764482498169, 0.21966546773910522, -0.13741150498390198, 0.024202127009630203, -0.13196255266666412, -0.04499059170484543, -0.06932919472455978, -0.1377231925725937, -0.08977137506008148, -0.03678784891963005, 0.014865833334624767, 0.08492138236761093, 0.0960683599114418, -0.16204683482646942, 0.05591990053653717, -0.04735248163342476, 0.13522154092788696, 0.1735667586326599, -0.32547393441200256, -0.09467806667089462, -0.08958503603935242, -0.12679755687713623, 0.13973811268806458, -0.0004551924066618085, 0.011041490361094475, -0.3636806309223175, 0.16764554381370544, -0.0008792818407528102, -0.049695175141096115, 0.010017585940659046, -0.21404428780078888, -0.07139187306165695, -0.2935289442539215, 0.35585951805114746, -0.2705302834510803, -0.008143019862473011, -0.2631947696208954, -0.16624143719673157, -0.03273705765604973, -0.06418763846158981, -0.04964229464530945, -0.039316631853580475, -0.01659988984465599, -0.09499945491552353, 0.05520401895046234, 0.025591298937797546, -0.1268921047449112, -0.023419395089149475, 0.03717596456408501, 0.2990204989910126, 0.062368497252464294, 0.12308690696954727, 0.15964345633983612, -0.03872308135032654, 0.08621153980493546, -0.18389888107776642, 0.15326473116874695, -0.06906051933765411, 0.10453388094902039, 0.11420091241598129, -0.18449708819389343, -0.07572431862354279, -0.07735040038824081}, {0.038588911294937134, -0.0873098224401474, -0.19853723049163818, 0.2054581493139267, 0.04553643614053726, 0.14146582782268524, 0.12425657361745834, -0.23422090709209442, 0.15305723249912262, 0.043984849005937576, -0.21716398000717163, 0.08187077194452286, 0.21873323619365692, -0.046562306582927704, 0.11279644072055817, -0.09797593206167221, -0.0337873212993145, 0.04379795119166374, -0.029573222622275352, -0.12950512766838074, 0.09903265535831451, 0.07253905385732651, -0.0598314106464386, -0.11515357345342636, -0.19675427675247192, -0.106624074280262, -0.13020601868629456, -0.08096876740455627, 0.008159334771335125, -0.33568549156188965, 0.017751544713974, 0.1269720196723938, -0.05439145117998123, -0.08039648830890656, 0.1816888451576233, 0.01757964864373207, -0.018892569467425346, -0.00391240743920207, -0.29059362411499023, 0.009428050369024277, -0.11900845915079117, 0.08638382703065872, 0.1621686816215515, -0.014319262467324734, -0.05147600173950195, 0.07315275073051453, 0.15339352190494537, 0.1463497430086136, -0.0886022076010704, 0.06625425815582275, 0.06322469562292099, -0.12971040606498718, -0.0053264787420630455, -0.04785916581749916, 0.02731507271528244, -0.07053221762180328, 0.3165184259414673, 0.024382658302783966, 0.07371814548969269, 0.16678962111473083, 0.14547601342201233, -0.021727344021201134, 0.21661534905433655, -0.23414552211761475, -0.04835386946797371, 0.07024406641721725, 0.200903058052063, 0.14253681898117065, -0.17061351239681244, 0.13413405418395996, -0.009412042796611786, 0.07398492097854614, 0.03052222728729248, 0.14526812732219696, 0.08702617883682251, 0.02970115840435028, -0.21728095412254333, -0.015462934039533138, -0.2510252296924591, -0.05413765087723732, 0.13858656585216522, 0.07243407517671585, -0.08782948553562164, 0.025562962517142296, 0.07436426728963852, 0.03611301630735397, -0.17565861344337463, 0.1447085589170456, -0.16115528345108032, -0.17463603615760803, -0.1223217323422432, 0.079147569835186, 0.041705790907144547, 0.09031666815280914, 0.13664016127586365, -0.04267231002449989, 0.049100812524557114, 0.007654526270925999, -0.0014632449019700289, -0.11315649747848511, 0.06124752387404442, 0.09549392759799957, -0.06733249872922897, 0.031887371093034744, -0.1850111335515976, 0.006190225947648287, -0.12518355250358582, -0.2297896146774292, -0.20462514460086823, 0.11811583489179611, 0.04710642248392105, -0.24061614274978638, 0.05936923995614052, -0.3425779938697815, -0.027719322592020035, 0.0469442680478096, 0.21431399881839752, -0.023958079516887665, 0.21031180024147034, 0.13343501091003418, 0.09573625028133392, -0.11066245287656784, 0.007057930342853069, -0.0009497313294559717, 0.0057450332678854465, 0.0357901006937027, -0.19022603332996368, 0.04435700178146362, 0.06942468136548996, -0.1934278905391693, 0.12009704858064651, 0.20198886096477509, 0.05861429125070572, -0.028900569304823875, -0.06711100041866302, -0.14776383340358734, 0.1313568353652954, 0.2302752435207367, 0.027587011456489563, 0.16745127737522125, 0.0460180789232254, -0.10733311623334885, 0.10170871764421463, 0.09479283541440964, -0.05550093576312065, -0.05508032068610191, 0.14554914832115173, 0.2381455898284912, -0.07279808819293976, 0.04562556743621826, 0.011718030087649822, -0.10137270390987396, 0.023398326709866524, -0.09134749323129654, -0.04313228279352188, 0.17737600207328796, 0.14341604709625244, -0.269550621509552, 0.10236213356256485, -0.24818933010101318, 0.014032337814569473, -0.03846709057688713, 0.12164615094661713, 0.10020662099123001, 0.0041094571352005005, 0.003974076360464096, -0.05943113565444946, -0.1681394875049591, -0.007447223179042339, -0.15336142480373383, -0.10783737897872925, -0.316712349653244, -0.11203807592391968, 0.178801491856575, -0.0009172037825919688, -0.0623864009976387, -0.06500755250453949, -0.03181186318397522, 0.07808568328619003, -0.03765818849205971, 0.03428720682859421, 0.24615904688835144, -0.12685388326644897, 0.23585186898708344, 0.020760584622621536, 0.1820441037416458, -0.20361892879009247, -0.10512741655111313, 0.031196122989058495, -0.10698307305574417, -0.011575632728636265, -0.18267394602298737, -0.14806541800498962, -0.006001036614179611, 0.05306273326277733, -0.08931907266378403, 0.017761308699846268, 0.07048898935317993, -0.01512366533279419, -0.15282101929187775, -0.11745025962591171, 0.08316480368375778, -0.005215398967266083, 0.14452436566352844, 0.05122720077633858, -0.07769276201725006, 0.003705553477630019, -0.023952165618538857, 0.16592638194561005, -0.05308215692639351, -0.02008483186364174, -0.24994659423828125, -0.10604655742645264, -0.03914123401045799, -0.0830235481262207, 0.07099217921495438, 0.07058719545602798, -0.07661345601081848, 0.0519883818924427, -0.12868013978004456, 0.016847068443894386, 0.0003251024172641337, 0.08050189167261124, -0.32592296600341797, -0.024695666506886482, -0.2233375608921051, 0.28833848237991333, 0.09371186047792435, 0.014236397109925747, -0.1558510810136795, -0.0600549690425396, 0.09804703295230865, -0.15527018904685974, -0.06807824224233627, -0.05816696211695671, 0.018564332276582718, 0.12216870486736298, -0.1010909304022789, 0.013242819346487522, -0.021484822034835815, 0.016586000099778175, 0.092995285987854, 0.10624928772449493, 0.017884477972984314, 0.049998439848423004, 0.038531187921762466, -0.08012786507606506, -0.07046615332365036, 0.015632230788469315, 0.10153944790363312, 0.036892447620630264, -0.16978465020656586, 0.22592608630657196, 0.08086401224136353, 0.10041572898626328, -0.17459365725517273}, {-0.007764284033328295, 0.22323590517044067, 0.007456996478140354, -0.058232441544532776, -0.0488702617585659, -0.03102213703095913, -0.043401651084423065, 0.05118374153971672, -0.03873617202043533, -0.085320383310318, -0.11062970757484436, -0.11741983890533447, -0.09310556203126907, -0.13765135407447815, -0.000935736286919564, -0.12748584151268005, 0.08533773571252823, 0.014871357008814812, 0.042894184589385986, 0.07115097343921661, -0.016105974093079567, -0.11516983807086945, -0.09789463132619858, 0.005040768533945084, 0.2832379937171936, -0.047107163816690445, 0.025966227054595947, -0.023158762603998184, -0.005552471149712801, 0.12589533627033234, -0.023050576448440552, 0.015805402770638466, -0.04800371453166008, -0.05498271808028221, 0.04019296541810036, -0.024959620088338852, 0.23996281623840332, 0.016641708090901375, -0.08155974000692368, 0.008474980480968952, -0.029917128384113312, -0.057791128754615784, -0.10722534358501434, 0.11460229009389877, 0.1248113289475441, 0.16715987026691437, -0.10260388255119324, -0.04660027474164963, 0.01944008097052574, -0.02908683381974697, -0.1347692608833313, -0.04168959707021713, 0.01413608156144619, -0.11368421465158463, 0.013039038516581059, -0.07110752165317535, 0.1470259577035904, 0.029220854863524437, 0.060206592082977295, 0.05687864124774933, 0.018599461764097214, 0.008982093073427677, -0.0516282320022583, -0.07448329031467438, -0.0664248988032341, -0.11117658019065857, 0.0013223692076280713, 0.12709924578666687, -0.1186373382806778, -0.0332917720079422, -0.07679726183414459, -0.0921371653676033, -0.1428537219762802, 0.04210406169295311, 0.4894324243068695, -0.019385840743780136, -0.12873390316963196, -0.03423531726002693, -0.047558221966028214, -0.17302708327770233, 0.10746525228023529, -0.00496683781966567, -0.06264235824346542, -0.05462302267551422, -0.026037810370326042, -0.1560899317264557, 0.20923833549022675, -0.0013672711793333292, 0.09733151644468307, -0.18334349989891052, 0.005839321296662092, 0.10288079082965851, -0.11290865391492844, 0.004946906119585037, -0.024271899834275246, -0.012716388329863548, -0.09461195766925812, -0.02156362310051918, -0.11772274225950241, 0.6360609531402588, -0.16805587708950043, -0.08388619869947433, -0.1269899308681488, 0.04805712029337883, -0.04221302643418312, 0.7711950540542603, -0.11491603404283524, -0.18479642271995544, -0.18945303559303284, 0.04651735723018646, 0.23061974346637726, 0.07329071313142776, -0.012271523475646973, 0.19513608515262604, -0.026644721627235413, -0.13099882006645203, 0.011464988812804222, -0.08755967020988464, -0.2172544151544571, -0.08188594877719879, -0.036896269768476486, -0.23453468084335327, 0.07168678939342499, -0.0957772508263588, 0.14294688403606415, -0.023045701906085014, -0.07881387323141098, -0.12707430124282837, 0.06866207718849182, 0.015043264254927635, 0.05850864201784134, 0.12540589272975922, 0.021648555994033813, -0.020524073392152786, 0.18625408411026, -0.14119920134544373, 0.04858537018299103, -0.014032812789082527, 0.0318799689412117, 0.06469206511974335, -0.05004258453845978, 0.04563238471746445, -0.09289821982383728, -0.3090471029281616, -0.028944771736860275, -0.04891685023903847, 0.10432390868663788, -0.10821229219436646, -0.060283154249191284, 0.1061009094119072, 0.0678151547908783, 0.13345730304718018, -0.01899593695998192, -0.10547874122858047, 0.06378510594367981, 0.11552257835865021, 0.09124156087636948, -0.08052525669336319, -0.14657971262931824, 0.3677496910095215, -0.010964172892272472, -0.02111092396080494, -0.05075928196310997, -0.061567872762680054, -0.06088520959019661, -0.029216155409812927, -0.11297176778316498, 0.06716179102659225, -0.07699456810951233, -0.11654789745807648, -0.154308021068573, -0.047981612384319305, 0.07327758520841599, -0.028789088129997253, -0.25819528102874756, -0.07581610232591629, -0.11352093517780304, 0.03369448333978653, 0.08833327144384384, -0.050937097519636154, 0.025919713079929352, -0.21623505651950836, -0.14753517508506775, -0.0037340752314776182, -0.1766475886106491, 0.03431270644068718, 0.026346242055296898, 0.10952137410640717, 0.025896796956658363, -0.09020819514989853, -0.13751240074634552, 0.15951085090637207, 0.051420338451862335, 0.2692897915840149, -0.2774927616119385, -0.12113875150680542, 0.19178764522075653, -0.15467095375061035, 0.09196382761001587, -0.10275772213935852, -0.048226382583379745, 0.09731141477823257, 0.016426801681518555, 0.09703916311264038, -0.052871331572532654, -0.10752014070749283, -0.13930389285087585, 0.03121321089565754, -0.11394514888525009, -0.056039195507764816, 0.06166209653019905, -0.0737370103597641, -0.2619394361972809, -0.02457721158862114, -0.014504069462418556, -0.06961307674646378, 0.022599712014198303, 0.21576666831970215, 0.03847988694906235, -0.07922817021608353, 0.14908576011657715, 0.02340920828282833, -0.14420515298843384, -0.10898822546005249, -0.0020608589984476566, -0.06788776069879532, -0.02549283392727375, -0.026372767984867096, 0.2347710132598877, -0.05075613409280777, 0.009651084430515766, 0.13867343962192535, 0.07253844290971756, -0.06921130418777466, -0.12192289531230927, -0.028899917379021645, -0.20038992166519165, -0.02849176712334156, -0.1879836767911911, 0.8432790637016296, -0.04883027449250221, -0.01052053738385439, 0.1055130660533905, -0.0049221147783100605, 0.08473597466945648, 0.15953481197357178, 0.0025333943776786327, -0.14294031262397766, 0.09431643784046173, 0.041999489068984985, -0.137514129281044, -0.17572857439517975, -0.06412812322378159, -0.05575608089566231, -0.06322839856147766, 0.08278566598892212}, {-0.23429667949676514, -0.030785348266363144, -0.01199624128639698, -0.155572772026062, -0.06120847538113594, -0.12436813116073608, -0.15886259078979492, -0.07354987412691116, 0.15348173677921295, 0.020733928307890892, 0.026493661105632782, -0.003446048591285944, -0.10081267356872559, 0.01041849609464407, 0.0010303236776962876, -0.12462328374385834, 0.004074983764439821, -0.09207011759281158, 0.07778501510620117, 0.07324311137199402, -0.07539243251085281, -0.1464461237192154, -0.1767842173576355, 0.08824349194765091, 0.06504248827695847, 0.1919664740562439, -0.04608340188860893, 0.06403964012861252, -0.06852996349334717, -0.005464798770844936, 0.06920929998159409, -0.2460358440876007, -0.11030431091785431, 0.13737446069717407, 0.22043955326080322, -0.2582050859928131, -0.09722582995891571, -0.08145282417535782, 0.10314440727233887, -0.08759886026382446, -0.21712148189544678, -0.055977415293455124, 0.05849077180027962, 0.3101520538330078, -0.0418609194457531, 0.06139159947633743, 0.06022883206605911, 0.04771466925740242, -0.0214590635150671, 0.07490582764148712, -0.06796830892562866, 0.10424666106700897, -0.08321285992860794, 0.20288658142089844, -0.022533023729920387, 0.017238667234778404, -0.05154946818947792, 0.08990632742643356, 0.14530165493488312, -0.3116905987262726, 0.224241703748703, 0.052927207201719284, 0.21313083171844482, 0.07070457935333252, -0.1117367148399353, 0.19465915858745575, -0.08224210143089294, 0.10129091143608093, -0.02106088027358055, 0.008358270861208439, 0.00931132398545742, -0.05428475886583328, -0.2287028580904007, 0.0037325541488826275, -0.05583357810974121, -0.22761164605617523, 0.3879753053188324, -0.11254537850618362, 0.06688909232616425, -0.04459407553076744, -0.09375207871198654, -0.034229326993227005, 0.09979205578565598, 0.10065591335296631, 0.02485053613781929, 0.33116549253463745, 0.28065839409828186, 0.03325638920068741, -0.19841249287128448, -0.07825305312871933, -0.019095029681921005, -0.0385221466422081, -0.13549089431762695, -0.24361786246299744, 0.13556011021137238, -0.04215274378657341, -0.1140512228012085, -0.06912192702293396, -0.12352563440799713, -0.32168132066726685, -0.030454197898507118, 0.03343961015343666, 0.05469977855682373, 0.0173562690615654, -0.04705022647976875, -0.6210757493972778, 0.24636521935462952, 0.09420044720172882, 0.02573472261428833, -0.09094155579805374, 0.04855683445930481, -0.16495206952095032, 0.03921568766236305, 0.0707734227180481, -0.07802357524633408, 0.12776164710521698, -0.16367945075035095, 0.07207919657230377, 0.06487864255905151, 0.01021042000502348, 0.08503841608762741, -0.0037179954815655947, 0.2666841745376587, 0.12953171133995056, 0.15353894233703613, -0.3183937072753906, -0.028159944340586662, 0.0855577364563942, 0.2274332195520401, 0.006062603089958429, -0.15288925170898438, -0.10776672512292862, 0.00830017402768135, 0.13057975471019745, -0.03717033565044403, 0.021422671154141426, 0.05458834767341614, -0.06321145594120026, 0.02131427638232708, -0.16555748879909515, 0.21635404229164124, -0.3504614233970642, 0.19475091993808746, 0.1590936928987503, -0.014116520993411541, -0.04227077588438988, -0.15071360766887665, 0.08415649086236954, -0.028009269386529922, -0.02100989781320095, -0.38054367899894714, 0.008642110042273998, -0.1619809865951538, 0.20373256504535675, -0.05765969678759575, 0.04288964718580246, 0.07410925626754761, -0.05244840681552887, -0.03934047743678093, 0.1840289831161499, -0.2732858657836914, 0.12291952967643738, 0.08085025101900101, -0.09126680344343185, -0.11331760138273239, 0.2689352035522461, -0.07388386130332947, -0.16438531875610352, 0.01239285059273243, 0.009188679978251457, 0.05029117316007614, -0.0020386900287121534, 0.007325212936848402, -0.16955579817295074, 0.02195248007774353, -0.11427883803844452, -0.1619948446750641, -0.18404333293437958, 0.0005989042110741138, 0.12379897385835648, -0.10394066572189331, -0.17465947568416595, 0.004372025839984417, 0.020321683958172798, -0.07142166048288345, 0.03749367222189903, 0.026524988934397697, 0.13979671895503998, -0.15418992936611176, 0.1434146910905838, 0.0032797495368868113, 0.10182174295186996, -0.1811726689338684, 0.10145126283168793, 0.14439059793949127, 0.14309488236904144, -0.060529544949531555, 0.10055936127901077, 0.15289607644081116, -0.011056246235966682, -0.0396064929664135, -0.04624605551362038, -0.03175262361764908, -0.0756232813000679, 0.19917769730091095, -0.10568850487470627, 0.20979440212249756, 0.05310874804854393, 0.03574135899543762, 0.10426228493452072, -0.5464590191841125, -0.05244522914290428, -0.14648684859275818, 0.04310350865125656, -0.045171916484832764, -0.11436162143945694, -0.022896606475114822, -0.1082710325717926, 0.0013192611513659358, -0.3507315218448639, 0.032159220427274704, 0.16035762429237366, 0.0010128107387572527, -0.21727575361728668, -0.16392245888710022, -0.05303733050823212, -0.31182578206062317, 0.11716092377901077, 0.12158913910388947, -0.04794897884130478, 0.019040485844016075, 0.16250509023666382, -0.08835837244987488, -0.1716204583644867, 0.1677953451871872, 0.16147810220718384, -0.06976281851530075, -0.16044272482395172, -0.02710937336087227, -0.25744983553886414, -0.10194016993045807, 0.1394965797662735, 0.01917373202741146, -0.1029093936085701, 0.06906291097402573, 0.15494859218597412, 0.008045634254813194, 0.05665993690490723, -0.3140605390071869, -0.04483003169298172, 0.04706157371401787, -0.10283850878477097, 0.16643989086151123, -0.05293163284659386, 0.1583860069513321, -0.14422690868377686}, {0.1530732959508896, 0.07492564618587494, 0.2690191864967346, -0.08517156541347504, -0.029272092506289482, -0.09928509593009949, -0.007003921549767256, 0.11886947602033615, -0.08724652230739594, -0.12196362018585205, 0.15986742079257965, 0.042530357837677, -0.09266464412212372, 0.11375743895769119, 0.18518884479999542, -0.05094593018293381, 0.05261093005537987, 0.07392976433038712, 0.05818061903119087, -0.008720768615603447, 0.12168754637241364, 0.01009930856525898, 0.05091127008199692, 0.03320677578449249, -0.03232977166771889, -0.1727844923734665, 0.03032512590289116, 0.06094929203391075, 0.1258789300918579, 0.34657540917396545, 0.06343626230955124, 0.16779842972755432, 0.004580319859087467, -0.2102724313735962, -0.11488131433725357, 0.17133896052837372, 0.08850372582674026, 0.11482766270637512, -0.10567829012870789, -0.11642863601446152, 0.22452567517757416, -0.023987552151083946, -0.20656009018421173, -0.14546744525432587, 0.14105969667434692, 0.07226818054914474, -0.004568321630358696, -0.13593260943889618, -0.06850610673427582, 0.05821240320801735, -0.02448100596666336, -0.1556851714849472, 0.06798142939805984, -0.03531615063548088, 0.010506819002330303, -0.08565030246973038, 0.05361943319439888, 0.01820121333003044, -0.10632894933223724, -0.10914047807455063, -0.010783664882183075, 0.025153199210762978, -0.01225655060261488, -0.1811632215976715, 0.18545973300933838, 0.05007617175579071, -0.05543431267142296, 0.03890277072787285, -0.009921777993440628, -0.03724759817123413, 0.12263958156108856, 0.14573121070861816, 0.11348573863506317, -0.12568847835063934, -0.0634206235408783, 0.07299745827913284, 0.02280518226325512, 0.07677567005157471, -0.37273889780044556, 0.049713462591171265, 0.13850760459899902, -0.06361747533082962, 0.007430220954120159, 0.10994109511375427, -0.08486457914113998, 0.025445345789194107, 0.15748803317546844, -0.060974009335041046, -0.08641497790813446, 0.1162993311882019, -0.04632779583334923, -0.01495019719004631, 0.0649019107222557, 0.021512027829885483, -0.3547368347644806, -0.0026589788030833006, 0.16520485281944275, 0.7232581973075867, 0.18705052137374878, 0.1040741428732872, -0.13936419785022736, -0.2540605962276459, -0.10046353191137314, 0.1558319479227066, -0.060041435062885284, -0.21322005987167358, -0.03940432518720627, 0.18973375856876373, 0.29397135972976685, -0.2558307647705078, 0.030974624678492546, -0.07695173472166061, -0.09229311347007751, -0.09232094138860703, 0.1516164392232895, -0.021509863436222076, -0.21784944832324982, -0.015985872596502304, 0.028918489813804626, -0.036254990845918655, -0.04569654166698456, 0.11392781138420105, -0.16949187219142914, 0.053504884243011475, 0.03752099722623825, 0.2068658471107483, -0.19841183722019196, 0.14154471457004547, -0.002594048622995615, -0.046099547296762466, 0.13587728142738342, 0.040466684848070145, -0.19628757238388062, 0.18704694509506226, 0.17043562233448029, -0.075865738093853, -0.22928790748119354, 0.009642594493925571, -0.0558752678334713, 0.05311328545212746, -0.025483259931206703, -0.061676252633333206, -0.02344014123082161, -0.013334018178284168, 0.03364177793264389, 0.11711303889751434, 0.0771067887544632, -0.29991400241851807, -0.002694321796298027, -0.03813774511218071, 0.10995641350746155, -0.07189574837684631, -0.14289315044879913, 0.00010073128214571625, 0.10885364562273026, -0.30905914306640625, -0.05100654438138008, 0.282940149307251, 0.005443916656076908, -0.050271738320589066, 0.12519285082817078, -0.0687381774187088, 0.04082451015710831, -0.18250958621501923, -0.048173122107982635, -0.07233718037605286, 0.13027538359165192, 0.0043202899396419525, -0.0942305251955986, 0.1428600549697876, 0.02258368395268917, 0.14113906025886536, 0.01736699417233467, 0.08966993540525436, 0.17196542024612427, 0.026380542665719986, 0.06483239680528641, 0.09653066843748093, 0.13576184213161469, -0.10004618763923645, 0.0029112016782164574, -0.0188764538615942, 0.062225691974163055, 0.03064350038766861, 0.008875437080860138, 0.010575387626886368, 0.07157117128372192, 0.025088535621762276, -0.032625049352645874, 0.0726926326751709, 0.04573583975434303, 0.14678768813610077, 0.11271359026432037, 0.02466529607772827, 0.12002964317798615, -0.009226982481777668, 0.13190320134162903, 0.1555040180683136, 0.07660626620054245, -0.03328176587820053, 0.13270683586597443, -0.1379772424697876, -0.13748350739479065, 0.1381317675113678, -0.2495545595884323, -0.03488573431968689, 0.013433472253382206, 0.013512525707483292, -0.009258117526769638, 0.2151118814945221, 0.1847621500492096, 0.046987343579530716, 0.08991289883852005, 0.13056693971157074, 0.24865593016147614, -0.13186438381671906, -0.05450752004981041, 0.02921389974653721, -0.08000295609235764, 0.17666971683502197, 0.009811957366764545, -0.08315864205360413, 0.0604269914329052, 0.01399066112935543, 0.37008631229400635, -0.039268434047698975, 0.07748032361268997, -0.20782677829265594, -0.10827945172786713, -0.048017896711826324, 0.06493984907865524, -0.3401322364807129, 0.06864593178033829, 0.09290678799152374, 0.19391152262687683, 0.1469961702823639, 0.20504198968410492, 0.2595891058444977, -0.1229335144162178, -0.07066994160413742, -0.055154282599687576, 0.14542101323604584, 0.17141400277614594, 0.12029233574867249, -0.28298670053482056, 0.03763705864548683, -0.027558883652091026, 0.1538393497467041, 0.16587568819522858, -0.11142314225435257, 0.01381081622093916, -0.057101961225271225, -0.441326379776001, -0.1438446044921875, 0.13272453844547272, 0.4391440451145172}, {0.22180739045143127, 0.14543528854846954, 0.12806269526481628, 0.1343155950307846, 0.1274867206811905, -0.1921297311782837, -0.18055294454097748, -0.05852431803941727, -0.045838430523872375, 0.07946720719337463, 0.04565160349011421, 0.3381322920322418, -0.04409649223089218, 0.19037947058677673, 0.035737719386816025, -0.15286873281002045, -0.0964551717042923, -0.2713663578033447, 0.23041212558746338, 0.15366758406162262, -0.18986429274082184, 0.05866728723049164, -0.07391513139009476, -0.03154270350933075, -0.1904992163181305, 0.14065326750278473, -0.12481194734573364, 0.15120753645896912, -2.3334467186941765e-05, -0.06721310317516327, 0.26181307435035706, -0.06550684571266174, 0.052942510694265366, 0.06878118962049484, -0.2505260109901428, -0.07582861185073853, 0.11704716086387634, -0.2559713125228882, -0.17318040132522583, -0.07638169825077057, -0.016666295006871223, 0.21779650449752808, -0.08961852639913559, 0.306436687707901, 0.34737056493759155, 0.1317199319601059, -0.09865743666887283, -0.10265735536813736, -0.015631312504410744, 0.08506706357002258, -0.004254197236150503, -0.20197521150112152, 0.1587177962064743, 0.029091080650687218, 0.08699224144220352, -0.05318274348974228, 0.08073936402797699, 0.007770090363919735, -0.11593076586723328, -0.07744593173265457, 0.05692587420344353, -0.1211431697010994, -0.10087402164936066, -0.2365482747554779, -0.15586432814598083, 0.15244708955287933, -0.011033573187887669, 0.18585820496082306, -0.25475549697875977, -0.018895259127020836, -0.056910187005996704, -0.04042431712150574, 0.0339759960770607, 0.17370879650115967, 0.18940657377243042, 0.09431395679712296, -0.028061779215931892, 0.12326899915933609, 0.011685285717248917, -0.07872997224330902, -0.06498456001281738, 0.05282028764486313, -0.10795724391937256, 0.055693089962005615, -0.14196422696113586, 0.14202991127967834, 0.21575124561786652, 0.006147386971861124, 0.118397057056427, -0.14002910256385803, 0.0017534957733005285, 0.08265317976474762, 0.16683271527290344, -0.06566443294286728, -0.06763474643230438, 0.06532002985477448, -0.1140037402510643, -0.2510110139846802, -0.04453000798821449, 0.5469027757644653, -0.03539999946951866, -0.060121145099401474, -0.24675101041793823, -0.06993257254362106, 0.3706851899623871, 0.4641052186489105, 0.08170770108699799, -0.2558812201023102, -0.2780105769634247, -0.11036426573991776, -0.13849031925201416, 0.09333467483520508, -0.24765698611736298, 0.0985446572303772, 0.006542212795466185, 0.00028373903478495777, 0.028491631150245667, 0.22379407286643982, -0.07049643993377686, 0.14733657240867615, -0.04710547626018524, 0.004527620039880276, -0.00010618440137477592, 0.024947457015514374, 0.28117719292640686, 0.08273504674434662, -0.0030682487413287163, -0.14025640487670898, 0.09041733294725418, -0.043582167476415634, 0.014339608140289783, -0.19591054320335388, -0.10742867738008499, 0.06928568333387375, -0.11984436959028244, -0.1361181139945984, -0.0449775755405426, 0.06842309981584549, -0.04666459932923317, -0.22554413974285126, -0.1363760381937027, 0.15862660109996796, 0.07874021679162979, -0.4523523151874542, 0.15576788783073425, 0.2511110007762909, 0.04521552100777626, 0.005083142779767513, -0.3095509111881256, -0.025177378207445145, 0.09825995564460754, 0.08273142576217651, -0.026710521429777145, 0.06468543410301208, -0.3394748270511627, 0.18149995803833008, -0.11406875401735306, -0.19399967789649963, -0.05945367366075516, 0.34371069073677063, -0.0625770166516304, -0.06155942752957344, 0.1457507163286209, 0.27681201696395874, -0.0032434870954602957, 0.20546792447566986, -0.0005779731436632574, 0.005800449289381504, -0.029007190838456154, -0.032326363027095795, -0.14891068637371063, 0.04180847853422165, -0.015168767422437668, -0.04149042069911957, 0.06411449611186981, -0.09318026155233383, -0.09155678004026413, 0.0051293764263391495, 0.03712834417819977, -0.08015827089548111, 0.08861695975065231, 0.16614559292793274, 0.01772799901664257, 0.05222519114613533, -0.08481668680906296, 0.12514042854309082, 0.07722941786050797, 0.12442599982023239, -0.13211452960968018, -0.3317984342575073, 0.06281320750713348, -0.06739407032728195, -0.008357217535376549, -2.7720296202460304e-05, 0.11662901192903519, 0.09319199621677399, 0.021113410592079163, -0.1611742526292801, -0.09016995131969452, 0.02145674079656601, -0.10754883289337158, -0.001163787441328168, 0.06911882013082504, 0.12652702629566193, 0.07914960384368896, -0.03918479382991791, -0.17839358747005463, 0.039157070219516754, -0.023732664063572884, -0.09687104821205139, -0.18463999032974243, -0.08122827112674713, -0.3503967821598053, -0.012928878888487816, -0.06808331608772278, -0.07884688675403595, -0.2608625888824463, 0.2030857652425766, 0.10888433456420898, 0.1312914490699768, 0.051968567073345184, 0.12512686848640442, 0.13090331852436066, 0.01853555254638195, -0.08598102629184723, 0.10388482362031937, 0.11413925886154175, -0.018154950812458992, 0.06336893886327744, 0.05911838263273239, 0.021579740568995476, 0.16993260383605957, 0.024083510041236877, 0.19598309695720673, -0.11521260440349579, -0.05067545920610428, -0.2148868441581726, -0.054126590490341187, 0.22243846952915192, 0.7552856802940369, 0.052564043551683426, -0.02290540188550949, -0.29705771803855896, 0.1748049259185791, -0.002876860089600086, -0.2618708908557892, -0.0647253543138504, -0.15993356704711914, -0.08397163450717926, 0.021494640037417412, -0.009271673858165741, -0.3172456622123718, -0.05755603313446045, -0.41993215680122375, 0.11427576839923859, -0.007554704789072275}, {0.23920094966888428, -0.023349642753601074, 0.2676427364349365, 0.010823599062860012, 0.06013396382331848, 0.045747075229883194, -0.03476448357105255, -0.20932088792324066, 0.13162867724895477, 0.04424046725034714, 0.004988230764865875, 0.06730932742357254, -0.13948673009872437, 0.019874796271324158, 0.31217527389526367, 0.019066382199525833, 0.08380147814750671, -0.10011501610279083, 0.09790632128715515, -0.08023406565189362, -0.16545185446739197, -0.16612254083156586, -0.19303803145885468, -0.136134535074234, -0.11820077896118164, -0.03690143674612045, -0.15069825947284698, -0.17432107031345367, 0.3218384087085724, 0.07474067807197571, 0.0533754900097847, 0.09806665778160095, 0.019300956279039383, 0.0701441615819931, 0.13383926451206207, 0.0718379095196724, 0.04300593584775925, 0.064521923661232, 0.05930403992533684, -0.02943454496562481, -0.2813326120376587, 0.11092332005500793, -0.058535270392894745, 0.16109465062618256, -0.0553198903799057, 0.0008318553445860744, 0.08258980512619019, -0.2869187295436859, 0.08381456881761551, -0.05683733522891998, 0.060397710651159286, 0.17555834352970123, -0.1374005824327469, 0.07999974489212036, 0.024739431217312813, -0.2256881147623062, -0.26312708854675293, 0.07373858988285065, 0.1946786493062973, 0.17140653729438782, 0.11972483992576599, -0.14908498525619507, 0.08484455943107605, -0.08905656635761261, -0.19929169118404388, 0.0032304751221090555, 0.037970952689647675, 0.061802636831998825, -0.021675752475857735, -0.018474353477358818, -0.19494110345840454, 0.14954179525375366, 0.04106883332133293, -0.1727433055639267, -0.06769198924303055, -0.044532373547554016, 0.05160735920071602, 0.19067984819412231, 0.5367538332939148, -0.14215189218521118, -0.1809140145778656, -0.08409806340932846, 0.00704650953412056, -0.02043915167450905, 0.034515608102083206, -0.16262948513031006, -0.12296730279922485, -0.03894435614347458, 0.11639539152383804, 0.06728838384151459, -0.05449467524886131, 0.05104301497340202, 0.1277886927127838, -0.14502444863319397, 0.08971811830997467, 0.03851914778351784, 0.10658339411020279, -0.03234099596738815, 0.11455743759870529, 0.008587009273469448, 0.1711362600326538, 0.3062668442726135, -0.06632550060749054, -0.14328217506408691, -0.12634992599487305, 0.0026693367399275303, 0.1363406926393509, 0.4117272198200226, -0.1149153783917427, 0.02444537915289402, -0.025093723088502884, -0.0470394566655159, 0.02433220110833645, -0.08970936387777328, -0.013534623198211193, -0.12042772024869919, 0.039463870227336884, -0.12514011561870575, -0.021249713376164436, -0.13720625638961792, 0.27605104446411133, -0.07198262214660645, 0.017157496884465218, 0.04877199977636337, -0.17468655109405518, 0.25032076239585876, 0.1592523604631424, -0.19665788114070892, 0.165618896484375, -0.06915845721960068, -0.08642017841339111, -0.01255004107952118, -0.09458132833242416, -0.08933939039707184, 0.2833264172077179, -0.29199448227882385, 0.29143670201301575, 0.28928878903388977, 0.006499277427792549, -0.07159166783094406, 0.2293226271867752, -0.18921543657779694, 0.02222943678498268, 0.012650550343096256, -0.207546204328537, -0.2636657655239105, -0.04890817031264305, -0.19417904317378998, -0.09149248898029327, 0.22567550837993622, 0.12836402654647827, 0.058627430349588394, -0.05360506847500801, 0.22460076212882996, 0.006959547288715839, -0.07875936478376389, -0.022297736257314682, 0.09833398461341858, -0.08518756181001663, -0.011406498029828072, -0.08723060041666031, 0.14729347825050354, 0.10002027451992035, -0.10393697768449783, 0.11241942644119263, -0.012178590521216393, -0.056545354425907135, 0.03158348798751831, -0.0008988275076262653, 0.1129257008433342, -0.03900160267949104, -0.14173847436904907, 0.022101664915680885, 0.04364122822880745, -0.06471162289381027, 0.07454104721546173, 0.09379389882087708, -0.11653735488653183, 0.3420066237449646, 0.06447513401508331, 0.12488426268100739, 0.024241965264081955, -0.05379344895482063, 0.032637301832437515, -0.16150076687335968, 0.0553148053586483, 0.24605464935302734, 0.13705244660377502, 0.21996505558490753, 0.5949996113777161, -0.12051945924758911, 0.2960785925388336, 0.1040293499827385, -0.005864349659532309, -0.01237513404339552, -0.2051672339439392, 0.04801376536488533, -0.16555288434028625, 0.1196485236287117, 0.0560857392847538, 0.1374533325433731, 0.041471485048532486, -0.2796512544155121, -0.025706037878990173, -0.053430210798978806, -0.08897686004638672, 0.028490319848060608, -0.1227470114827156, -0.08551458269357681, 0.06808905303478241, -0.06877575069665909, -0.0409901961684227, 0.011398098431527615, 0.2490861713886261, -0.06335891783237457, -0.06266026198863983, -0.05729363486170769, 0.18856367468833923, 0.09054695814847946, 0.04602931812405586, 0.20472651720046997, -0.1884492039680481, -0.3224007487297058, 0.011265739798545837, -0.018012451007962227, 0.07827521115541458, 0.21810200810432434, 0.136478453874588, 0.03271172195672989, -0.07997749000787735, -0.16317810118198395, 0.1034267321228981, 0.1095505952835083, 0.16610406339168549, 0.0991760790348053, -0.024106208235025406, 0.154915452003479, -0.046245235949754715, 0.23434188961982727, 0.11086665093898773, -0.00766114704310894, -0.020708823576569557, 0.07197095453739166, 0.15406005084514618, -0.1662682294845581, 0.3037760853767395, -0.21944116055965424, -0.16421271860599518, -0.16384172439575195, 0.0023424820974469185, -0.07903970032930374, 0.00048229467938654125, 0.06250134110450745, -0.011564967222511768, 0.2832508981227875, 0.0438365638256073}, {-0.12350272387266159, -0.12264099717140198, -0.20875483751296997, 0.20904430747032166, -0.012682654894888401, -0.07257933914661407, -0.009992311708629131, 0.06823474913835526, -0.1125335693359375, -0.03381872922182083, 0.18850016593933105, -0.030607635155320168, 0.05421778932213783, -0.01703505590558052, 0.12403968721628189, 0.1076405867934227, -0.150893896818161, -0.04810931906104088, 0.1199372336268425, -0.1016712337732315, -0.03871191665530205, -0.0009789299219846725, 0.07880522310733795, -0.24897204339504242, 0.006991902831941843, -0.04143286496400833, -0.06442351639270782, -0.27951470017433167, 0.10797422379255295, -0.009389234706759453, 0.2530270218849182, 0.008548329584300518, -0.045180339366197586, 0.1857481449842453, -0.16802307963371277, 0.10259711742401123, -0.050159599632024765, -0.1466546356678009, 0.012932267040014267, -0.07665498554706573, -0.1078244000673294, 0.1508607566356659, 0.05825747177004814, -0.1359846293926239, -0.04737832769751549, -0.07185382395982742, 0.4174672067165375, 0.09226623177528381, 0.11295386403799057, -0.08223722875118256, 0.2544695734977722, 0.11765330284833908, -0.02452387474477291, -0.44085371494293213, 0.0793856829404831, -0.1736113280057907, 0.08387665450572968, -0.13859309256076813, 0.000585507950745523, 0.34637323021888733, -0.1679576337337494, 0.09796342998743057, 0.020172372460365295, 0.015761643648147583, 0.017458384856581688, 0.1605302393436432, 0.11396715044975281, 0.045753028243780136, -0.25164884328842163, -0.09793815016746521, -0.16670961678028107, 0.05986948683857918, 0.21286973357200623, -0.06773307174444199, 0.010241691023111343, -0.010417995043098927, 0.005893809720873833, 0.07322605699300766, 0.1466303914785385, 0.11708129942417145, 0.085861936211586, -0.056595578789711, -0.13482534885406494, -0.2212810516357422, 0.021893225610256195, 0.08262161910533905, -0.23537708818912506, 0.1143055260181427, 0.11560948938131332, -0.23198406398296356, -0.017485106363892555, 0.19613197445869446, 0.31161990761756897, -0.04470478743314743, -0.2148294448852539, -0.10144653171300888, 0.07102838158607483, -0.06669704616069794, -0.023893700912594795, -0.2057860642671585, 0.05212756246328354, -0.06847714632749557, 0.05450676381587982, 0.1660298854112625, -0.18034307658672333, 0.15318407118320465, -0.017362283542752266, 0.004056215286254883, -0.3025974929332733, 0.07068376243114471, -0.13066989183425903, 0.057294271886348724, 0.04241708666086197, -0.33499088883399963, -0.059525180608034134, 0.2551553547382355, -0.14052748680114746, -0.018238991498947144, -0.111752450466156, -0.09232526272535324, 0.08738721162080765, 0.07400914281606674, 0.032673630863428116, -0.12777742743492126, 0.16749584674835205, -0.1355815827846527, 0.13869857788085938, 0.02000688761472702, -0.17479351162910461, -0.14817671477794647, 0.05415663495659828, -0.040951330214738846, -0.1594582051038742, -0.07175055891275406, 0.03397180885076523, -0.026334812864661217, -0.043467458337545395, 0.1613301783800125, -0.02675824798643589, -0.059548232704401016, -0.09031947702169418, 0.07268644124269485, 0.0650249794125557, 0.19288666546344757, -0.0830577090382576, 0.143661767244339, 0.03675474599003792, 0.15914975106716156, -0.10456881672143936, 0.027341753244400024, -0.04476320371031761, -0.020821256563067436, -0.15085119009017944, 0.14620128273963928, 0.04433886334300041, -0.15064293146133423, 0.05878492444753647, -0.18975235521793365, 0.009002411738038063, -0.1446942239999771, 0.0784912109375, -0.25826194882392883, 0.056826382875442505, -0.18259212374687195, -0.12569501996040344, -0.204091876745224, 0.08371662348508835, -0.17692549526691437, 0.07535416632890701, 0.059202149510383606, -0.05135685205459595, -0.2533404529094696, -0.12348182499408722, 0.1569581925868988, 0.023720847442746162, -0.0521041564643383, -0.09625233709812164, -0.08674625307321548, 0.1160687804222107, 0.025812048465013504, 0.0750967413187027, 0.1892854869365692, 0.13658824563026428, 0.11057673394680023, 0.04732806980609894, 0.10000922530889511, -0.4618087410926819, 0.04288705810904503, 0.1579044610261917, 0.03715457767248154, -0.17218756675720215, -0.010371388867497444, 0.0690256878733635, 0.029891284182667732, 0.2877569794654846, 0.07418709248304367, -0.04395776242017746, -0.08039063960313797, 0.008111687377095222, 0.019521638751029968, -0.02305152826011181, -0.29372501373291016, 0.1360480636358261, -0.01849755086004734, 0.19665580987930298, -0.04505423828959465, 0.08616563677787781, 0.02003304846584797, 0.07565226405858994, 0.005203233100473881, 0.3196650445461273, -0.1741446852684021, -0.014810379594564438, 0.06319345533847809, 0.06455929577350616, 0.15696121752262115, 0.04712185263633728, 0.07618095725774765, -0.08336009085178375, -0.26111987233161926, 0.08981530368328094, 0.06114071607589722, 0.005501330830156803, -0.07329845428466797, 0.105494923889637, -0.19793669879436493, 0.2378700077533722, 0.007125695236027241, 0.0006557134329341352, -0.15884815156459808, -0.17892974615097046, 0.08824896067380905, 0.1481383740901947, 0.17189449071884155, 0.022894108667969704, -0.015193823724985123, 0.036396197974681854, -0.0642540380358696, -0.009350499138236046, -0.13927261531352997, -0.04344324395060539, -0.04041891545057297, 0.18632423877716064, 0.018214570358395576, -0.08379367738962173, 0.028744621202349663, 0.17557719349861145, -0.17459739744663239, -0.08192992210388184, -0.005639389622956514, -0.1413230150938034, -0.021569494158029556, 0.07067589461803436, 0.03421641141176224, 0.09612946957349777, 0.010722972452640533}, {0.03476942703127861, -0.08746682107448578, -0.12797342240810394, 0.013413138687610626, -0.14395633339881897, -0.10044264048337936, -0.17361502349376678, 0.14304706454277039, -0.1043027713894844, 0.014975093305110931, 0.09067791700363159, 0.03940174728631973, 0.0332648903131485, 0.048242148011922836, -0.08467727154493332, 0.06448929756879807, 0.06353918462991714, -0.04834040626883507, 0.08518106490373611, 0.05151883140206337, -0.0825403481721878, -0.0837828740477562, 0.045068856328725815, -0.09002906829118729, 0.1006745845079422, 0.025143347680568695, 0.1342773735523224, -0.028089627623558044, -0.006493878550827503, 0.011462974362075329, 0.17552371323108673, 0.12941361963748932, 0.0912252739071846, -0.01882600598037243, -0.3222994804382324, -0.013088107109069824, -0.1658288985490799, 0.004253634717315435, -0.18454919755458832, -0.17577862739562988, 0.14708355069160461, 0.08425478637218475, -0.012517523020505905, -0.08198070526123047, -0.15349699556827545, 0.25145429372787476, -0.06202516332268715, -0.060980841517448425, -0.07677778601646423, 0.13049687445163727, -0.3558480441570282, -0.08818342536687851, -0.0043556466698646545, -0.12919089198112488, -0.0031796633265912533, -0.07498716562986374, 0.050777312368154526, -0.06998315453529358, -0.057834070175886154, 0.10836432129144669, 0.08248493075370789, 0.057079900056123734, -0.03817569091916084, -0.14818575978279114, 0.028731491416692734, -0.06741099804639816, -0.10252436995506287, -0.16622251272201538, 0.02917686104774475, -0.09617552906274796, 0.20852424204349518, 0.06842582672834396, 0.057888928800821304, 0.03134656324982643, -0.47913220524787903, 0.058558762073516846, -0.06424512714147568, -0.37904074788093567, -0.07837959378957748, 0.11247947067022324, -0.205280140042305, -0.13988184928894043, -0.21318253874778748, 0.1880006492137909, 0.04048877954483032, -0.04412813484668732, -0.05566868558526039, -0.1723189800977707, 0.05742295831441879, 0.029050039127469063, -0.014292176812887192, -0.26301172375679016, 0.07796034216880798, 0.04758847504854202, -0.03607850521802902, -0.017859002575278282, 0.1502278596162796, 0.040764618664979935, -0.05370672792196274, -0.10169760137796402, -0.024241723120212555, 0.11988625675439835, 0.13518908619880676, 0.00026382581563666463, 0.06927818804979324, -0.1713937520980835, 0.08007345348596573, -0.014678354375064373, 0.21116119623184204, 0.0026223475579172373, 0.006010058335959911, 0.09178167581558228, -0.2695557475090027, 0.09706752002239227, -0.016377849504351616, -0.14324918389320374, -0.1287662386894226, 0.10313798487186432, -0.041810646653175354, -0.014660608023405075, -0.033258501440286636, -0.05274619907140732, -0.1463312804698944, -0.08133041858673096, -0.016651911661028862, 0.08617778867483139, 0.0038666652981191874, -0.03174430876970291, -0.04768426716327667, -0.02115105651319027, -0.018213419243693352, 0.00037312827771529555, -0.1418626755475998, 0.06458070129156113, -0.08158184587955475, -0.188490092754364, 0.07554779201745987, -0.010314122773706913, -0.20119965076446533, 0.14316920936107635, 0.07059872895479202, 0.07512059807777405, -0.1349228471517563, 0.04330732300877571, -0.04587686434388161, 0.04531753435730934, 0.09309854358434677, -0.11437144130468369, 0.10261882841587067, -0.0642867460846901, -0.026977414265275, 0.035933662205934525, 0.02167617902159691, 0.09587928652763367, 0.06189270317554474, -0.07586894929409027, -0.06493731588125229, 0.01225073542445898, 0.06299493461847305, -0.037573665380477905, -0.018488993868231773, 0.260977178812027, -0.22144094109535217, -0.06344043463468552, 0.006859073415398598, -0.1492415815591812, 0.28565728664398193, 0.09122449904680252, -0.03223055601119995, 0.12791652977466583, 0.16825513541698456, 0.12941542267799377, -0.09490089118480682, -0.005056948401033878, 0.10498280823230743, 0.06522431969642639, -0.24377386271953583, 0.2116771787405014, 0.04747050628066063, -0.016458647325634956, -0.08005765080451965, 0.07117127627134323, -0.010733542032539845, 0.03154490888118744, -0.10472995787858963, -0.016345733776688576, 0.0419967919588089, -0.016501890495419502, 0.06857457011938095, 0.03311539813876152, -0.16394276916980743, 0.20384007692337036, -0.4071250259876251, -0.08774800598621368, -0.12152638286352158, 0.13815121352672577, 0.18977633118629456, 0.4513845443725586, 0.04580049589276314, -0.09678973257541656, -0.1313062161207199, -0.14725719392299652, 0.16775141656398773, 0.029680931940674782, -0.0597226619720459, 0.08746416121721268, -0.09760282188653946, 0.1240721344947815, -0.07875555008649826, -0.03344700112938881, -0.0013280418934300542, -0.04331184923648834, 0.050186462700366974, -0.012227908708155155, 0.02482856996357441, -0.004301914479583502, -0.04885248467326164, 0.2627237141132355, -0.08313580602407455, 0.12365459650754929, -0.02506139688193798, -0.08308679610490799, 0.049133967608213425, 0.0023298426531255245, 0.07425158470869064, -0.022568056359887123, 0.07192255556583405, 0.15279962122440338, 0.028264859691262245, -0.030819209292531013, 0.09500095993280411, -0.11888188123703003, 0.01792212575674057, -0.01922779530286789, 0.18946245312690735, -0.085270456969738, 0.05116491764783859, 0.005119435954838991, 0.06445722281932831, -0.18993203341960907, -0.04637421667575836, 0.015392092056572437, -0.15104752779006958, 0.15729033946990967, 0.04552731290459633, -0.061275579035282135, -0.03480143845081329, 0.05835620313882828, 0.12196285277605057, 0.03149876743555069, 0.09420548379421234, -0.14061634242534637, 0.011958691291511059, 0.13781021535396576, 0.005674137733876705, 0.035213224589824677}, {-0.1685689389705658, 0.014351911842823029, -0.12749749422073364, 0.12470211833715439, -0.1811680942773819, 0.12992848455905914, 0.09801653772592545, -0.1721918135881424, 0.09204740822315216, -0.1104719266295433, 0.19035224616527557, -0.045878391712903976, -0.07664897292852402, -0.016536803916096687, 0.2376384139060974, 0.026845581829547882, -0.0668567344546318, -0.12889137864112854, -0.03692972660064697, 0.05562599375844002, 0.07408164441585541, -0.10974536091089249, -0.054376840591430664, -0.2270820438861847, 0.03008355386555195, 0.07679042220115662, -0.00016004171629901975, -0.10867781937122345, 0.19126208126544952, 0.3814283013343811, -0.15438859164714813, 0.13597510755062103, -0.08006209880113602, -0.10183102637529373, -0.12924355268478394, 0.025770941749215126, -0.021834097802639008, -0.053532060235738754, -0.04816240817308426, -0.03651338070631027, 0.05201694369316101, -0.014386872760951519, -0.2313971072435379, 0.04182412102818489, -0.253688782453537, 0.022118372842669487, -0.10424641519784927, -0.14435407519340515, 0.023477977141737938, -0.09617386758327484, -0.19870339334011078, 0.05015581101179123, -0.18987014889717102, -0.2694787383079529, 0.09221240878105164, -0.11391304433345795, -0.23170852661132812, -0.017823226749897003, 0.17845897376537323, 0.04976154863834381, 0.1582861989736557, 0.15458743274211884, 0.13344016671180725, -0.11511803418397903, -0.22373902797698975, -0.03672923892736435, 0.008035391569137573, 0.07997051626443863, -0.2570894658565521, 0.08007406443357468, -0.011086538434028625, 0.16344521939754486, -0.125601127743721, -0.23700445890426636, -0.20848210155963898, 0.013582885265350342, 0.045995019376277924, -0.03106297180056572, 0.13099803030490875, -0.08970832824707031, -0.17965874075889587, 0.16324234008789062, -0.045551497489213943, 0.15186664462089539, 0.11049137264490128, 0.08381260186433792, -0.040606215596199036, 0.09434624761343002, 0.06643835455179214, 0.08602362126111984, 0.17871591448783875, -0.20588141679763794, -0.01783161610364914, -0.1327289342880249, -0.019559701904654503, -0.20882494747638702, -0.04212288558483124, -0.09779375791549683, -0.16569331288337708, -0.16009701788425446, 0.07032681256532669, 0.1685752272605896, 0.16992774605751038, 0.013582111336290836, -0.15830713510513306, -0.1195000633597374, 0.05345974862575531, 0.1773311048746109, -0.08143889904022217, -0.05214882269501686, 0.005458200816065073, -0.025923578068614006, 0.22521957755088806, -0.3057418465614319, -0.04408024251461029, -0.16369210183620453, -0.14396421611309052, 0.09476467221975327, -0.1927964985370636, 0.11458427459001541, 0.0528654120862484, -0.0034448213409632444, 0.009403718635439873, 0.11868015676736832, -0.08137839287519455, -0.057137176394462585, -0.052827395498752594, 0.05161561816930771, -0.004409780725836754, -0.023045329377055168, 0.04742198437452316, 0.0414535216987133, 0.28504207730293274, 0.06089731678366661, -0.06441306322813034, -0.0113262003287673, -0.1335800290107727, -0.1549379676580429, 0.21697264909744263, 0.035747431218624115, -0.20171035826206207, 0.030181441456079483, 0.09470517933368683, 0.15839338302612305, -0.029810499399900436, -0.23314037919044495, -0.048905596137046814, 0.183945432305336, 0.05676215887069702, -0.03853878751397133, -0.08504047989845276, -0.03670823946595192, -0.09970171749591827, 0.2226937711238861, -0.1737644076347351, -0.06418439000844955, -0.040199894458055496, -0.05931660905480385, 0.15630316734313965, -0.10218610614538193, 0.03195371478796005, 0.04903813451528549, 0.11747447401285172, 0.014430108480155468, -0.40715232491493225, 0.2170432060956955, 0.05165604129433632, -0.027613800019025803, 0.08871905505657196, -0.038346726447343826, 0.06717833131551743, 0.012121749110519886, -0.026753591373562813, 0.031078852713108063, -0.09619367122650146, -0.18160182237625122, 0.0014055987121537328, -0.17238105833530426, 0.1012231782078743, 0.029851755127310753, 0.0796133503317833, -0.05575151741504669, -0.30125799775123596, -0.1440339982509613, 0.13950203359127045, 0.30443763732910156, 0.22450195252895355, 0.023960774764418602, -0.22865666449069977, 0.06451074033975601, 0.007519019767642021, -0.0019365743501111865, -0.1266593039035797, -0.0019663332495838404, -0.16193804144859314, 0.15545795857906342, -0.0008249545353464782, 0.13411448895931244, 0.005635701585561037, -0.2190450131893158, 0.149480938911438, 0.07819781452417374, -0.10883139818906784, 0.0026895541232079268, 0.041056837886571884, 0.19875933229923248, -0.2363889068365097, 0.06569015234708786, -0.02008781209588051, 0.12202184647321701, -0.1465037614107132, 0.07991819828748703, 0.07331406325101852, -0.3277297914028168, 0.03972312808036804, -0.015280384570360184, -0.010795405134558678, -0.04059329628944397, 0.06006573885679245, -0.13572761416435242, -0.03482068330049515, 0.040434956550598145, 0.14671632647514343, 0.141331747174263, -0.1080005019903183, 0.022727789357304573, 0.0007998955552466214, 0.05930369719862938, -0.23331670463085175, -0.09312469512224197, 0.08376618474721909, 0.05362973362207413, -0.16266877949237823, 0.09149373322725296, -0.17257869243621826, 0.20484961569309235, 0.08434056490659714, -0.21367835998535156, 0.25184860825538635, -0.06941238045692444, 0.09003225713968277, -0.4101398289203644, 0.2098526954650879, 0.0749671533703804, -0.029697412624955177, -0.02261558547616005, 0.05115169659256935, -0.07913389056921005, -0.27191004157066345, -0.015288794413208961, 0.008182968012988567, 0.1543777585029602, 0.2256954461336136, -0.019616663455963135, 0.03364492580294609, 0.01312488503754139}, {-0.08198852092027664, 0.24307817220687866, 0.12171419709920883, -0.16000483930110931, 0.06493870168924332, 0.03316505625844002, 0.0002943069557659328, -0.06068025529384613, 0.12301117181777954, -0.17061208188533783, 0.14998403191566467, -0.043957073241472244, -0.09245641529560089, 0.03650527074933052, -0.02054518088698387, 0.07312802225351334, 0.08754316717386246, 0.09439186006784439, 0.04709632694721222, -0.007593631744384766, -0.009810704737901688, 0.010728041641414165, 0.036646466702222824, -0.08726468682289124, -0.041040852665901184, 0.0181544441729784, -0.08699019253253937, -0.05119188502430916, 0.03790229186415672, -0.09426931291818619, 0.018190301954746246, -0.08041246235370636, 0.01571499928832054, 0.018231894820928574, 0.14800617098808289, 0.24017676711082458, 0.1477528214454651, 0.14976587891578674, 0.11270132660865784, 0.0791284367442131, -0.04670372232794762, -0.1140170693397522, 0.13739804923534393, 0.25919705629348755, 0.10427820682525635, 0.13726985454559326, 0.08416689932346344, -0.13378824293613434, 0.003805666696280241, 0.12240676581859589, 0.0498945526778698, -0.03278675302863121, 0.10882110148668289, -0.09719793498516083, 0.06076229363679886, -0.006930924486368895, -0.18685783445835114, -0.07480426132678986, -0.14842405915260315, 0.05106710270047188, 0.003490886650979519, -0.11453374475240707, 0.018600357696413994, -0.0033541701268404722, -0.05106540024280548, 0.16168783605098724, 0.06850934773683548, -0.114019475877285, -0.020866887643933296, -0.0372992679476738, -0.027199281379580498, -0.04484807327389717, -0.276571661233902, -0.10349934548139572, 0.4868299663066864, -0.013247660361230373, -0.06970705091953278, 0.2179679572582245, 0.3368942141532898, -0.014036203734576702, 0.2182329297065735, -0.2694406807422638, -0.04270096868276596, 0.13765089213848114, 0.05688595026731491, 0.14397858083248138, 0.18349476158618927, -0.170286625623703, -0.05935137718915939, -0.11905460059642792, 0.13046196103096008, 0.08582326769828796, -0.022379104048013687, -0.045537568628787994, 0.009202235378324986, 0.04677613452076912, -0.02929558791220188, -0.07428481429815292, -0.10712775588035583, 0.4560706317424774, -0.06765781342983246, -0.1386420726776123, 0.11549698561429977, -0.0330851748585701, 0.15463140606880188, 0.6442792415618896, -0.06848375499248505, 0.08337690681219101, 0.09804026037454605, 0.051368240267038345, -0.03532823175191879, 0.06611879914999008, -0.04434933513402939, 0.008878488093614578, 0.09534820914268494, -0.041021574288606644, -0.183017298579216, -0.03782718628644943, 0.01533815823495388, 0.13071131706237793, 0.11032643169164658, -0.02293238788843155, -0.0871470496058464, 0.006716389674693346, 0.06271662563085556, -0.15770208835601807, -0.07947345823049545, -0.00560245243832469, 0.03855759650468826, -0.17220748960971832, -0.20714642107486725, 0.034967198967933655, -0.06761284172534943, 0.0380406528711319, 0.024325182661414146, -0.07937653362751007, 0.06136995553970337, 0.04893093928694725, -0.02196301333606243, 0.09853988885879517, -0.00028573148301802576, -0.11554940044879913, 0.13340748846530914, -0.0800568163394928, 0.024760160595178604, -0.1347232162952423, 0.12844529747962952, 0.1849006712436676, -0.10738915205001831, 0.21678337454795837, -0.0396663136780262, 0.028341097757220268, -0.014499706216156483, 0.1588633954524994, 0.17076100409030914, -0.0869079977273941, 0.0017871269956231117, 0.1692045032978058, -0.0829053595662117, 0.186784565448761, 0.05580552667379379, 0.04661177098751068, -0.06465009599924088, -0.23030045628547668, -0.15147839486598969, -0.06611765921115875, -0.1492672562599182, -0.06075045093894005, -0.046406276524066925, 0.18129536509513855, -0.07342848181724548, -0.2081778347492218, 0.28628838062286377, 0.22497527301311493, -0.054380785673856735, -0.09183721989393234, -0.06988007575273514, -0.054091066122055054, 0.02148972824215889, -0.08670691400766373, 0.09802122414112091, -0.008782163262367249, -0.034099772572517395, 0.037701237946748734, -0.17471791803836823, 0.17492739856243134, -0.018091844394803047, 0.1095086932182312, 0.1872822642326355, -0.13965535163879395, 0.18751707673072815, -0.052997611463069916, 0.11101984977722168, 0.10832840949296951, 0.11481581628322601, 0.07274802029132843, 0.018943918868899345, -0.08373520523309708, -0.12142045050859451, -0.06693296134471893, 0.025667065754532814, -0.031178845092654228, 0.08948313444852829, 0.00862647220492363, -0.05208735167980194, 0.04790265113115311, -0.042864784598350525, 0.07410139590501785, 0.03976431116461754, -0.04772493988275528, 0.0006440822035074234, 0.01265362836420536, -0.2220626175403595, 0.014845523051917553, -0.03560296818614006, -0.14186914265155792, -0.11739572882652283, 0.1492205113172531, 0.05502862483263016, -0.15674780309200287, 0.28399109840393066, -0.06102132052183151, -0.15272577106952667, 0.03759118542075157, -0.15582232177257538, -0.07331367582082748, 0.02018573321402073, 0.0699407085776329, 0.1550118327140808, 0.02707507088780403, 0.022048961371183395, 0.15757343173027039, -0.013902164995670319, 0.11857082694768906, 0.007240932900458574, -0.039600029587745667, 0.16270537674427032, -0.12092404067516327, -0.09146815538406372, 0.6148878335952759, -0.3225511610507965, -0.187361478805542, 0.13016310334205627, -0.1989680528640747, -0.17291253805160522, -0.0011047336738556623, -0.1298830360174179, -0.05074228346347809, -0.04748000577092171, 0.031532689929008484, -0.04224942624568939, 0.03919139504432678, 0.026810118928551674, -0.07262689620256424, 0.2654949128627777, -0.08247821033000946}, {-0.19011475145816803, -0.21092619001865387, -0.1786678433418274, -0.09316559880971909, -0.013993486762046814, -0.04606756195425987, -0.030984828248620033, 0.015163609758019447, -0.09819072484970093, -0.01779201813042164, 0.04968002438545227, -0.15606406331062317, 0.10535178333520889, -0.05739029496908188, 0.007935655303299427, -0.11704320460557938, -0.04615110158920288, 0.1682511419057846, -0.11271551996469498, 0.19482992589473724, 0.08900004625320435, -0.1194291040301323, 0.07362714409828186, -0.16569940745830536, -0.030562445521354675, 0.05221935361623764, 0.24096031486988068, -0.051367729902267456, 0.06509191542863846, -0.09746553003787994, 0.12881968915462494, 0.21396274864673615, -0.3118298053741455, -0.13410240411758423, -0.14618544280529022, 0.0594378337264061, -0.1023886650800705, -0.1342787891626358, 0.05197415500879288, -0.050852738320827484, 0.03820396959781647, -0.08722762018442154, 0.07080034166574478, -0.1067504808306694, -0.07444487512111664, -0.03279446065425873, 0.03543352335691452, 0.022829562425613403, 0.04018145427107811, 0.00815258827060461, 0.15551148355007172, -0.05545424297451973, -0.02388938143849373, 0.037940844893455505, 0.06292261928319931, 0.050860755145549774, -0.12622129917144775, 0.13106323778629303, 0.06679326295852661, -0.03075808472931385, -0.1006707176566124, -0.06807170063257217, -0.22376582026481628, 0.06844902783632278, 0.23948746919631958, 0.011332256719470024, -0.11535341292619705, -0.134118914604187, -0.0680168867111206, 0.04900464415550232, -0.052682578563690186, -0.025584332644939423, -0.06838112324476242, -0.0016009494429454207, -0.04781819507479668, 0.10756894201040268, -0.11502125859260559, 0.23174889385700226, 0.12715975940227509, 0.052054811269044876, 0.08813446760177612, 0.07978708297014236, 0.09045848995447159, 0.23954522609710693, -0.190477192401886, 0.06277977675199509, 7.578849181300029e-05, 0.14159925282001495, -0.11909906566143036, 0.22238688170909882, 0.056712646037340164, 0.06767459213733673, -0.039464447647333145, 0.03644535318017006, -0.08911921083927155, 0.2948095202445984, 0.034405164420604706, -0.03603314235806465, -0.027509812265634537, -0.10585939139127731, 0.10421586781740189, -0.26031437516212463, 0.051697105169296265, -0.06045917049050331, -0.26443082094192505, -0.1679212599992752, 0.055043838918209076, 0.06708969175815582, 0.0034801180008798838, -0.020560093224048615, -0.1507781445980072, 0.055743731558322906, 0.03357025980949402, -0.007502997759729624, 0.11484499275684357, 0.05245351418852806, -0.1313336342573166, 0.1604248732328415, -0.19836893677711487, 0.032134994864463806, -0.1264123171567917, 0.07014920562505722, 0.217194065451622, 0.09690946340560913, -0.1532321721315384, 0.009252794086933136, -0.02461201325058937, 0.20999254286289215, -0.022731943055987358, 0.12239416688680649, -0.13845312595367432, -0.2915613055229187, 0.01168214250355959, 0.10238482058048248, -0.04692039638757706, 0.20236046612262726, -0.13634145259857178, -0.232828289270401, 0.06430716067552567, -0.17368334531784058, -0.2932407259941101, 0.21146295964717865, 0.0013961817603558302, -0.007648767903447151, 0.07316095381975174, -0.0015153726562857628, -0.14849576354026794, -0.11819829791784286, 0.06615303456783295, -0.03478917479515076, 0.07618985325098038, -0.06599394977092743, -0.06256087124347687, -0.14368239045143127, -0.17458707094192505, 0.11440961062908173, -0.04684777557849884, -0.059682488441467285, -0.0880485251545906, 0.1720338612794876, -0.04956668242812157, -0.1972160041332245, -0.09129996597766876, 0.11961831152439117, -0.009549712762236595, 0.05573411285877228, -0.18909448385238647, -0.011534707620739937, 0.051452066749334335, 0.054853249341249466, 0.11873385310173035, -0.002462082076817751, -0.01570710726082325, 0.06027207523584366, 0.021058471873402596, -0.02238122560083866, -0.029462149366736412, -0.08299381285905838, 0.15425649285316467, -0.15676286816596985, 0.17414666712284088, -0.26078876852989197, 0.11785519868135452, 0.060243312269449234, 0.09844650328159332, -0.022010473534464836, -0.1731470227241516, 0.1496550440788269, -0.1385461986064911, 0.014460944570600986, -0.05200394615530968, 0.07045900821685791, 0.1665225327014923, 0.034482043236494064, -0.08822738379240036, -0.0007489850977435708, 0.03373042866587639, -0.20677770674228668, -0.029210086911916733, 0.04276411235332489, 0.04720934480428696, 0.25086942315101624, -0.023358561098575592, 0.22346580028533936, -0.1676209419965744, 0.2491539567708969, 0.0008405014523304999, -0.3465655446052551, -0.03012900985777378, -0.04216597229242325, -0.007561501115560532, -0.031926341354846954, 0.3236999809741974, -0.004895977675914764, -0.04559837654232979, -0.05868740379810333, -0.05099267140030861, -0.07434157282114029, 0.15317226946353912, -0.0026123314164578915, 0.15812388062477112, 0.14180022478103638, -0.1399218589067459, -0.0924573764204979, -0.20453178882598877, -0.1227371096611023, 0.2043484002351761, -0.11257769167423248, 0.013809318654239178, 0.17039667069911957, 0.022258562967181206, -0.08123749494552612, -0.13968981802463531, -0.007842469029128551, 0.21033430099487305, 0.06406315416097641, -0.09754365682601929, -0.1118498295545578, 0.009270967915654182, -0.012167938984930515, 0.25716763734817505, 0.0610957071185112, -0.05698439106345177, -0.017036449164152145, 0.03726270794868469, -0.1475471407175064, -0.056502364575862885, 0.0123047586530447, 0.058141566812992096, -0.0580960176885128, 0.0012728581205010414, -0.1425764262676239, -0.049561623483896255, 0.08421824872493744, 0.010509216226637363, -0.061074841767549515}, {-0.15769319236278534, -0.07325133681297302, -0.11128473281860352, 0.08711257576942444, -0.1687886118888855, 0.018226204439997673, -0.14960084855556488, 0.0492231510579586, -0.20801211893558502, -0.09323476254940033, -0.0636981725692749, 0.018194664269685745, 0.15572378039360046, 0.017699288204312325, -0.1740902215242386, 0.19906103610992432, -0.13404837250709534, 0.20035028457641602, -0.05419018119573593, 0.027806703001260757, 0.12160009890794754, -0.08994416147470474, 0.060593828558921814, 0.04971243068575859, 0.16565410792827606, -0.2416948527097702, 0.13042160868644714, -0.17906396090984344, 0.15323393046855927, -0.09142369031906128, -0.2550276815891266, -0.04592268168926239, -0.07475803047418594, 0.14448851346969604, 0.004974949639290571, 0.20282943546772003, 0.10659411549568176, 0.14655135571956635, 0.01888883113861084, -0.0634579062461853, 0.006038405001163483, -0.06347089260816574, 0.008193966932594776, -0.29010337591171265, -0.3132569193840027, 0.19711342453956604, -0.20464491844177246, 0.19418452680110931, -0.013506639748811722, -0.04828272759914398, 0.05931808054447174, 0.1685212403535843, -0.010823859833180904, -0.025603696703910828, -0.044309355318546295, 0.10023579746484756, -0.0003884485049638897, 0.21255289018154144, 0.04921453446149826, -0.13820351660251617, 0.03142701834440231, -0.13088758289813995, -0.049495890736579895, -0.022148672491312027, 0.289376437664032, -0.19980621337890625, -0.09370817989110947, 0.12906493246555328, 0.3186023235321045, 0.2013685256242752, 0.03125764802098274, -0.0640564113855362, 0.10240864753723145, -0.18063750863075256, -0.38039565086364746, -0.17393429577350616, 0.10565190762281418, 0.03992239385843277, -0.05661029368638992, 0.02480172924697399, 0.1461499184370041, -0.020988857373595238, -0.011917304247617722, -0.0537569522857666, 0.046662990003824234, -0.03294973075389862, -0.031901102513074875, 0.10973481088876724, 0.050154734402894974, 0.04736344888806343, 0.07286564260721207, -0.1368236243724823, 0.00714525394141674, 0.28417518734931946, -0.037463363260030746, -0.1275344341993332, -0.023898988962173462, 0.0003490553062874824, 0.16080176830291748, -0.05128246918320656, -0.011039813049137592, -0.1223999485373497, -0.04567297175526619, -0.08720066398382187, -0.07741823047399521, -0.03567970171570778, -0.011985895223915577, 0.09922245889902115, -0.05549731105566025, 0.0005045045982114971, -0.06750848144292831, 0.0073956516571342945, 0.00404695700854063, 0.06632880121469498, -0.12949341535568237, 0.09994451701641083, 0.01664525642991066, 0.04779521003365517, 0.23465153574943542, -0.18056032061576843, 0.06282593309879303, 0.2827112376689911, -0.19643470644950867, -0.006616848986595869, -0.04163912683725357, 0.02009500563144684, 0.13543228805065155, 0.07223229855298996, -0.07328299432992935, 0.009757265448570251, 0.12784609198570251, -0.178701713681221, 0.007218771614134312, -0.214434415102005, 0.14701369404792786, -0.19457148015499115, -0.23621061444282532, -0.008549951948225498, 0.06683095544576645, -0.1336352527141571, 0.07196956872940063, -0.016773482784628868, 0.10407227277755737, 0.0003107827506028116, 0.18880850076675415, 0.031820040196180344, -0.059241387993097305, -0.20615741610527039, 0.006047444883733988, 0.09693639725446701, -0.008841415867209435, -0.09874600172042847, 0.06436236202716827, -0.04061640053987503, 0.13677920401096344, 0.11083400249481201, -0.10356850922107697, 0.1114678680896759, -0.1063871681690216, 0.09289582073688507, -0.2117862105369568, 0.052868276834487915, 0.1739913821220398, 0.20753027498722076, -0.07889808714389801, -0.12505139410495758, -0.08130505681037903, -0.07096437364816666, -0.1161893755197525, 0.04372318461537361, 0.10857122391462326, 0.07645032554864883, 0.16547095775604248, 0.08688759058713913, 0.1971941739320755, -0.05165708810091019, 0.22498059272766113, -0.10433340072631836, 0.1952243149280548, 0.13514581322669983, 0.09353121370077133, -0.06359706819057465, -0.033852122724056244, 0.021775763481855392, 0.11986257880926132, 0.19146473705768585, -0.09199167042970657, 0.04435230419039726, 0.08681373298168182, -0.03139375150203705, 0.0388236828148365, -0.00869945902377367, 0.06570514291524887, 0.22057048976421356, -0.1909625232219696, -0.00568679952993989, -0.052297212183475494, -0.1776134967803955, -0.044595614075660706, -0.11219095438718796, 0.1996711790561676, 0.09013842791318893, 0.2537163197994232, 0.028801368549466133, 0.044131822884082794, 0.012128389440476894, 0.06757225096225739, -0.10626434534788132, -0.13132546842098236, 0.20281803607940674, 0.1342100203037262, -0.01995241455733776, 0.13821619749069214, -0.280364066362381, 0.05607058107852936, 0.049828361719846725, 0.12398730218410492, 0.012896422296762466, 0.01663072779774666, 0.09957266598939896, -0.11992467939853668, 0.19570687413215637, 0.033256128430366516, 0.18952979147434235, -0.10118260979652405, 0.0973074659705162, 0.021052192896604538, -0.017504019662737846, 0.011718055233359337, -0.20580150187015533, 0.05532946437597275, 0.13630029559135437, -0.08159659057855606, 0.034882381558418274, 0.18726810812950134, 0.12541493773460388, 0.24230633676052094, 0.1542331874370575, -0.07746425271034241, 0.0024219208862632513, -0.18811580538749695, -0.00896209292113781, 0.1576760709285736, -0.07593227177858353, -0.024319380521774292, -0.005559982266277075, 0.014260755851864815, 0.0035916545893996954, -0.13207828998565674, -0.12701284885406494, 0.03745990991592407, -0.017188619822263718, 0.011166204698383808, 0.183039590716362, 0.051384054124355316, -0.03951896354556084}, {0.3064798414707184, 0.09789222478866577, -0.0021468624472618103, -0.03627213463187218, 0.07347932457923889, 0.03413432464003563, -0.19025418162345886, 0.5103320479393005, 0.013367808423936367, 0.07487194240093231, -0.035845495760440826, 0.05221940204501152, -0.09671421349048615, 0.053388968110084534, -0.019873173907399178, 0.04153921827673912, -0.07350315898656845, 0.13236339390277863, -0.02795577421784401, 0.15930525958538055, 0.14577311277389526, -0.03268876671791077, 0.06758351624011993, 0.24271179735660553, -0.10558509826660156, -0.029826605692505836, 0.039954427629709244, 0.08074302971363068, -0.23608775436878204, 0.1290600299835205, 0.01423805020749569, -0.13761647045612335, -0.12639661133289337, -0.026885362342000008, 0.06264803558588028, -0.29537177085876465, -0.14820146560668945, -0.23169034719467163, -0.31349068880081177, -0.05794672667980194, 0.06992996484041214, -0.07450713217258453, 0.09254521131515503, 0.00041187350871041417, -0.05871112272143364, -0.03146951645612717, 0.15949198603630066, -0.09298434853553772, -0.12071216106414795, 0.0011747825192287564, -0.04386220499873161, -0.29131805896759033, -0.004149329848587513, 0.15831351280212402, -0.16361233592033386, -0.015912411734461784, 0.24731187522411346, -0.03763195499777794, 0.07532915472984314, -0.0972094014286995, -0.17773273587226868, 0.008410676382482052, -0.12917377054691315, -0.2643803656101227, 0.12167371064424515, 0.22281110286712646, -0.027845758944749832, -0.06367874890565872, -0.006638236343860626, 0.06605769693851471, 0.08445937186479568, -0.031880203634500504, 0.10517162084579468, -0.22427137196063995, -0.0710466280579567, -0.011176234111189842, 0.04773661494255066, -0.17388617992401123, 0.18278175592422485, 0.12287480384111404, 0.1453627496957779, 0.04245997220277786, 0.18579816818237305, 0.019145045429468155, 0.2146129459142685, 0.07905732840299606, -0.0037624489050358534, -0.11006243526935577, 0.019068267196416855, 0.08552062511444092, 0.0909058004617691, -0.033926259726285934, -0.18743301928043365, -0.1496867537498474, 0.07492291927337646, 0.033747196197509766, 0.18438279628753662, 0.01838616468012333, 0.01495448686182499, 0.2475319802761078, -0.12221227586269379, 0.2211712896823883, 0.027136223390698433, 0.08373618870973587, -0.17809723317623138, -0.013691696338355541, 0.018759598955512047, 0.11784585565328598, 0.26444754004478455, 0.021157313138246536, 0.07776309549808502, 0.18963713943958282, 0.06734230369329453, -0.08859594911336899, 0.08070661872625351, -0.12589387595653534, 0.02992217056453228, 0.06015259400010109, 0.05191142484545708, -0.14773066341876984, -0.10970122367143631, -0.12013819068670273, -0.08176819235086441, -0.34216710925102234, -0.035672083497047424, 0.07415008544921875, -0.042508564889431, 0.013713682070374489, 0.12104640156030655, 0.021923454478383064, -0.0026451321318745613, 0.13008244335651398, -0.15962012112140656, 0.11322268098592758, -0.1340634822845459, -0.24882394075393677, 0.08828143775463104, 0.17313683032989502, -0.04547819867730141, -0.08801137655973434, 0.03445620834827423, -0.1601668894290924, -0.07781188189983368, 0.036782845854759216, 0.12430359423160553, -0.04928915947675705, 0.10207688808441162, -0.003564700949937105, -0.02166769839823246, -0.16737718880176544, -0.049078647047281265, -0.06330049782991409, -0.11823113262653351, -0.027580002322793007, 0.07659856975078583, 0.07412681728601456, -0.03508025407791138, -0.0021721404045820236, 0.11582222580909729, 0.011893374845385551, 0.038065940141677856, 0.14588172733783722, 0.11617302894592285, -0.038431186228990555, -0.14665266871452332, -0.41687652468681335, 0.10997714847326279, -0.17581188678741455, 0.009677448309957981, -0.13368238508701324, 0.047304362058639526, -0.038578100502491, -0.04367056488990784, -0.21169644594192505, -0.0444774404168129, -0.0176830030977726, 0.03167201578617096, 0.13492177426815033, 0.30827638506889343, 0.02066088281571865, -0.14058703184127808, 0.054505687206983566, 0.09387639164924622, 0.25393611192703247, 0.0037918249145150185, -0.0036748743150383234, -0.263575941324234, -0.1429629623889923, -0.05009140074253082, -0.1899993121623993, -0.02008719928562641, 0.002413216745480895, -0.08404318988323212, -0.11759103089570999, 0.35054901242256165, -0.07376479357481003, 0.2816295623779297, 0.129591166973114, -0.050815314054489136, -0.06652235239744186, -0.07737510651350021, 0.10568572580814362, -0.0815049335360527, 0.12419719994068146, -0.042764533311128616, 0.09732132405042648, 0.0704190582036972, 0.22171759605407715, -0.0435209646821022, 0.23519788682460785, 0.305134654045105, -0.015428908169269562, 0.12164399027824402, -0.11628250777721405, 0.10870861262083054, -0.06860806047916412, -0.028310617431998253, -0.09307386726140976, 0.11358579248189926, -0.07991251349449158, -0.14405496418476105, 0.1336108297109604, -0.0012198933400213718, -0.17663420736789703, -0.13314135372638702, -0.2594161033630371, 0.18874989449977875, 0.3753521144390106, -0.14797571301460266, 0.11879283934831619, 0.008782329969108105, -0.12477675080299377, 0.12111023813486099, -0.05544092878699303, 0.15353897213935852, 0.0970725417137146, -0.0008267256198450923, -0.0789879858493805, -0.2775329351425171, 0.050562918186187744, 0.02639945037662983, 0.12919485569000244, 0.04148329421877861, 0.1546887308359146, -0.20958764851093292, 0.19156697392463684, -0.01688588224351406, 0.13841061294078827, 0.054267846047878265, -0.08685488253831863, 0.04607836902141571, -0.2267623245716095, -0.12703166902065277, 0.02596692554652691, 0.03774570673704147, 0.08032166212797165}, {0.20152044296264648, 0.02934170328080654, -0.05137639492750168, -0.00020332854182925075, -0.10243291407823563, -0.03820405155420303, 0.06401093304157257, -0.03217051923274994, -0.11375858634710312, 0.024205293506383896, -0.02912517450749874, 0.35921648144721985, 0.10384262353181839, 0.0791507437825203, 0.004431906621903181, 0.1765645146369934, -0.006417766213417053, -0.11293447017669678, -0.09736066311597824, -0.2000207155942917, -0.09390531480312347, 0.12428522855043411, -0.06343495845794678, 0.08129408955574036, -0.04405814781785011, 0.08288399875164032, -0.04327645152807236, 0.022054128348827362, -0.0698113739490509, 0.03138056769967079, -0.2749635577201843, 0.0027874871157109737, 0.14159844815731049, -0.07216136157512665, -0.1365121304988861, 0.16591493785381317, 0.006597858387976885, 0.1222110241651535, 0.04838436469435692, -0.12748785316944122, -0.04653942584991455, -0.016034405678510666, 0.18719500303268433, 0.023042988032102585, 0.07823876291513443, -0.09878885000944138, -0.0040468149818480015, 0.13276270031929016, -0.09450525790452957, 0.03989584743976593, -0.05253569781780243, -0.050097059458494186, -0.04990019276738167, 0.09782471507787704, 0.11874460428953171, -0.026413891464471817, 0.044594451785087585, -0.018126819282770157, -0.10770408809185028, -0.06580819934606552, -0.12750250101089478, 0.009087327867746353, -0.02096162736415863, 0.03524056449532509, -0.10347113013267517, -0.024264240637421608, 0.03738197311758995, 0.0780380368232727, 0.004803983960300684, -0.004549265839159489, 0.02303100749850273, -0.03203783184289932, -0.1309610903263092, -0.11884365230798721, 0.21270738542079926, 0.0100663211196661, 0.14560796320438385, 0.1981884092092514, -0.03337562456727028, -0.26781702041625977, 0.1875496655702591, -0.0786043331027031, 0.010457308031618595, 0.08006276935338974, -0.002638021484017372, 0.050247061997652054, -0.002614055061712861, 0.07269380986690521, 0.06355955451726913, -0.10677596926689148, -0.0443083792924881, 0.18659302592277527, -0.10508695989847183, -0.027111420407891273, -0.14448551833629608, 0.10151612758636475, 0.09375448524951935, 0.06468778103590012, 0.24648591876029968, 0.07646836340427399, 0.1710214465856552, -0.1912025511264801, -0.07354044169187546, 0.034782521426677704, 0.03851905092597008, 0.25588807463645935, -0.19616790115833282, 0.0003253156610298902, -0.31963589787483215, -0.08723790943622589, 0.22142638266086578, -0.1542307585477829, -0.032096538692712784, -0.16862204670906067, 0.20129793882369995, -0.011748610064387321, -0.026083173230290413, -0.10596100240945816, -0.0762663260102272, 0.1030382290482521, -0.10373518615961075, -0.004291473887860775, -0.10981781035661697, -0.04049641266465187, -0.08397572487592697, 0.09023906290531158, -0.022916121408343315, 0.1151355728507042, 0.17981241643428802, 0.1261955052614212, 0.06273028254508972, -0.12427537143230438, -0.08624132722616196, 0.02738933265209198, 0.08167733252048492, -0.049377989023923874, -0.03814920783042908, 0.028492668643593788, 0.19133412837982178, 0.05513916164636612, 0.12358057498931885, 0.008010128512978554, 0.017888348549604416, -0.045218877494335175, 0.10675777494907379, 0.03850703313946724, -0.04389749467372894, 0.06304231286048889, 0.07413390278816223, -0.019667895510792732, -0.03489113226532936, -0.010678455233573914, -0.23695018887519836, 0.14287801086902618, 0.16114294528961182, -0.14719144999980927, 0.12260834127664566, 0.2173556536436081, 0.028693055734038353, 0.10782760381698608, -0.0726640373468399, -0.09090830385684967, 0.09798901528120041, -0.007648375350981951, -0.08063790202140808, -0.04505694657564163, -0.19036658108234406, 0.05298981815576553, -0.009370386600494385, -0.02483789436519146, 0.31438377499580383, -0.17017275094985962, 0.07459524273872375, 0.2787714898586273, 0.08082029223442078, 0.003981067333370447, -0.14453065395355225, 0.014107225462794304, 0.0534876212477684, 0.052409011870622635, 0.0032162994612008333, 0.46535924077033997, -0.15908701717853546, -0.1831890493631363, -0.1340588629245758, 0.06385531276464462, 0.053062569350004196, 0.1371629238128662, 0.23541498184204102, -0.04721096530556679, 0.04882874712347984, -0.002531995065510273, -0.044519197195768356, 0.07354416698217392, 0.12606674432754517, 0.14329685270786285, 0.20975345373153687, 0.03742678463459015, -0.0259348563849926, 0.07083392143249512, -0.16398738324642181, -0.06318493187427521, 0.040114875882864, -0.07583503425121307, -0.1559630036354065, -0.06292472779750824, 0.12390874326229095, 0.10160066932439804, 0.09496806561946869, -0.07854647934436798, -0.021439146250486374, 0.050453342497348785, -0.03968316689133644, -0.015172569081187248, 0.15240707993507385, 0.03923601657152176, 0.10891209542751312, 0.09122869372367859, 0.27129697799682617, 0.06099439412355423, -0.016305342316627502, -0.0028019584715366364, 0.14571258425712585, -0.08889982104301453, 0.007904494181275368, 0.06003332883119583, -0.0011691834079101682, -0.13622291386127472, 0.0685839131474495, -0.054573506116867065, -0.020374814048409462, -0.041245799511671066, 0.23117029666900635, 0.0583065040409565, -0.023445621132850647, 0.2753862738609314, 0.02874271757900715, 0.04970596730709076, -0.07962702959775925, 0.10419805347919464, -0.0779944583773613, 0.2110924869775772, -0.04800324887037277, 0.286063015460968, 0.25477278232574463, 0.030967365950345993, 0.0013062499929219484, 0.08489146083593369, -0.13289980590343475, 0.07095178216695786, 0.04157426953315735, -0.10427377372980118, 0.050504378974437714, -0.12429605424404144, 0.018945325165987015, -0.1249014362692833}, {0.040478192269802094, -0.07873015105724335, -0.04411912336945534, 0.1457379162311554, -0.07924722135066986, 0.14870689809322357, 0.09112292528152466, 0.20832981169223785, -0.035997528582811356, -0.09031787514686584, -0.04751893877983093, -0.006027360912412405, 0.149688258767128, 0.09790439158678055, -0.04448782280087471, 0.15523874759674072, 0.014789526350796223, -0.15797318518161774, -0.11243236809968948, -0.08577622473239899, 0.1997886449098587, 0.09299589693546295, -0.028114205226302147, -0.06589023023843765, -0.1821662038564682, -0.09842683374881744, -0.10444942861795425, 0.02717020735144615, 0.18354150652885437, -0.03710554167628288, 0.2777869701385498, 0.21070148050785065, -0.1548430621623993, -0.18523511290550232, -0.016025885939598083, -0.056537430733442307, 0.10533808916807175, -0.19094157218933105, -0.06076296418905258, -0.03685402497649193, 0.01542105246335268, 0.1741216629743576, -0.15424004197120667, 0.17637936770915985, 0.050175271928310394, -0.10713271796703339, -0.053299084305763245, -0.19099123775959015, 0.10289296507835388, -0.05470035970211029, -0.05918876454234123, -0.006488699931651354, 0.055457208305597305, -0.187443807721138, 0.024739248678088188, -0.06509018689393997, 0.08655020594596863, 0.018797526136040688, -0.006781537085771561, -0.04871251806616783, 0.3061203360557556, 0.11423443257808685, 0.009136633016169071, -0.15702088177204132, -0.1945371925830841, -0.09791918843984604, 0.08501441776752472, 0.11118476837873459, -0.006622448097914457, 0.07858625054359436, 0.09057503193616867, 0.08265572786331177, 0.12649112939834595, -0.01886734738945961, 0.18449163436889648, -0.19778099656105042, 0.1219024807214737, -0.03572734445333481, -0.18091878294944763, 0.032510749995708466, -0.2576761841773987, -0.03667691722512245, -0.014270873740315437, -0.20516416430473328, -0.02734382636845112, 0.003720123553648591, -0.27951422333717346, -0.06346887350082397, -0.04221632704138756, -0.012780128046870232, 0.1320493370294571, -0.11961010843515396, 0.12336215376853943, 0.12560664117336273, 0.1481933444738388, 0.09312600642442703, 0.07497536391019821, -0.05088795721530914, 0.018665548413991928, 0.14727488160133362, -0.07866440713405609, -0.07167492061853409, -0.13457483053207397, 0.20049941539764404, -0.12594696879386902, 0.14197364449501038, 0.02363329939544201, -0.17346790432929993, -0.1474331170320511, 0.19618898630142212, 0.02548622153699398, 0.010006577707827091, 0.001045677810907364, -0.07399941980838776, -0.09487863630056381, 0.18515300750732422, -0.08660609275102615, 0.11035410314798355, 0.09374916553497314, 0.21688561141490936, 0.11046847701072693, 0.2841625213623047, -0.1557742953300476, -0.006304703652858734, -0.12015146762132645, 0.153961181640625, -0.11382906883955002, 0.004890579264611006, 0.09604038298130035, -0.10715236514806747, -0.07690601050853729, 0.045861199498176575, -0.2641351819038391, 0.028256764635443687, -0.035341136157512665, -0.4556659758090973, 0.15464575588703156, 0.20214056968688965, -0.34434300661087036, 0.1002894788980484, -0.11558487266302109, 0.019731679931282997, 0.037708234041929245, -0.0021105764899402857, 0.029440144076943398, 0.05470952019095421, 0.13116811215877533, -0.007785223890095949, 0.06323673576116562, -0.09404561668634415, 0.20540665090084076, 0.0005247446824796498, 0.179416224360466, -0.05132506787776947, 0.05450756847858429, 0.07457058131694794, -0.2057109922170639, 0.040370646864175797, 0.16207802295684814, -0.24727493524551392, 0.07784329354763031, 0.14875797927379608, 0.10433641821146011, 0.043169181793928146, 0.11424796283245087, 0.006450837478041649, -0.05219680815935135, 0.07962893694639206, 0.010405927896499634, 0.12260483205318451, -0.04403385892510414, -0.058965787291526794, -0.21663989126682281, -0.12842240929603577, 0.012308605946600437, -0.13188022375106812, 0.0739404708147049, -0.23132455348968506, 0.17000417411327362, 0.15620967745780945, -0.06774502247571945, 0.05601269006729126, 0.06191813200712204, -0.12387078255414963, -0.07902748882770538, -0.04718500003218651, 0.008651011623442173, -0.12745511531829834, 0.018398545682430267, 0.35181140899658203, 0.041822679340839386, -0.12015952169895172, -0.1969994157552719, -0.12153850495815277, -0.06682315468788147, 0.039121516048908234, -0.2808341383934021, 0.18329496681690216, 0.1424475461244583, 0.23896609246730804, 0.05914425104856491, 0.039258141070604324, -0.08886486291885376, -0.04611670598387718, -0.10962439328432083, -0.1857750564813614, 0.1739472597837448, 0.08171574026346207, -0.15895313024520874, 0.0720791295170784, 0.15381355583667755, -0.19707709550857544, -0.0031724402215331793, 0.11088860780000687, -0.01729687675833702, 0.09881237149238586, 0.0647614523768425, 0.058754634112119675, -0.006777467671781778, 0.06396625936031342, 0.11226565390825272, -0.12380317598581314, 0.06204352527856827, -0.0782422423362732, 0.11116284877061844, 0.08149255067110062, 0.1690632402896881, -0.008522777818143368, 0.2692936956882477, -0.18031089007854462, -0.07020539045333862, 0.2054114043712616, 0.11029022186994553, -0.09530691057443619, -0.017873650416731834, 0.049215804785490036, -0.1603146642446518, 0.07579875737428665, 0.22916468977928162, -0.23961754143238068, 0.09791801869869232, 0.04691426455974579, 0.1177680492401123, -0.019754547625780106, -0.17783123254776, 0.041839465498924255, -0.03934641554951668, -0.13231995701789856, -0.04651598632335663, 0.00380706787109375, -0.10012370347976685, 0.20181873440742493, -0.00844744872301817, -0.17947690188884735, -0.06412061303853989, 0.05577840656042099}, {-0.014810221269726753, 0.1995333433151245, 0.25312793254852295, 0.21418029069900513, -0.09889520704746246, 0.1270001381635666, -0.2759148180484772, -0.08333462476730347, -0.08146818727254868, -0.002086981665343046, -0.11370726674795151, -0.15761889517307281, 0.18882927298545837, 0.14655683934688568, -0.024749550968408585, 0.08249706029891968, 0.09776096791028976, 0.09206905961036682, 0.08620184659957886, -0.02963113784790039, 0.0374661348760128, -0.2625587582588196, 0.029144171625375748, 0.19122721254825592, 0.005683946423232555, -0.37987443804740906, 0.01318943127989769, -0.16232143342494965, -0.05127520486712456, -0.03252016007900238, -0.1266787052154541, 0.16302195191383362, -0.044525131583213806, 0.17754177749156952, -0.03930151090025902, 0.054865870624780655, 0.13031668961048126, -0.12481196969747543, 0.1394999474287033, 0.09610714763402939, -0.17144019901752472, -0.13906286656856537, 0.2096700221300125, 0.11955854296684265, 0.1718119978904724, -0.17023636400699615, 0.15728621184825897, -0.18269126117229462, 0.06484638154506683, -0.02897004969418049, 0.34102341532707214, 0.2895384728908539, 0.1465466320514679, -0.04441389441490173, -0.041853126138448715, -0.07511842250823975, -0.10250251740217209, 0.37490010261535645, -0.02550029754638672, -0.0036033624783158302, 0.07122664153575897, -0.14588285982608795, 0.017604133114218712, -0.0546896830201149, -0.11653588712215424, -0.26481130719184875, 0.09078256785869598, 0.060764990746974945, 0.3016132712364197, 0.009927340783178806, -0.027406971901655197, -0.016709789633750916, -0.1200753003358841, -0.18759846687316895, 0.07476421445608139, -0.09977328777313232, 0.13445760309696198, 0.16336099803447723, -0.038734711706638336, -0.07991380244493484, 0.02497085928916931, -0.2160225659608841, -0.12182216346263885, -0.11768554896116257, 0.11144531518220901, -0.3661647140979767, -0.025734219700098038, 0.1908792406320572, -0.06298849731683731, -0.14140541851520538, 0.1591164469718933, 0.38190433382987976, 0.10158025473356247, 0.10886790603399277, -0.07656188309192657, 0.31029075384140015, 0.03725142031908035, 0.0014025389682501554, 0.07358530908823013, -0.03115925006568432, -0.11768294125795364, -0.1193300113081932, 0.1931016743183136, 0.1255723237991333, -0.09139708429574966, -0.07955639809370041, -0.10047400742769241, -0.05157795548439026, -0.06927786767482758, -0.08421856164932251, -0.2633679509162903, -0.1147051453590393, 0.23237816989421844, 0.19915825128555298, -0.22003254294395447, 0.08891131728887558, 0.07569129765033722, -0.009603018872439861, 0.13430482149124146, -0.1963750720024109, -0.020074894651770592, 0.341746985912323, 0.16809824109077454, 0.14781847596168518, -0.1474674940109253, 0.31834328174591064, 0.1470927596092224, 0.07965254783630371, -0.13460521399974823, 0.15361149609088898, 0.005461881402879953, -0.2745439410209656, 0.07774962484836578, 0.01817048341035843, -0.028852567076683044, -0.07901240885257721, -0.03992096707224846, -0.04234251007437706, 0.20326177775859833, -0.0857606902718544, -0.028329044580459595, -0.012898573651909828, 0.0895589217543602, -0.09683722257614136, 0.33278802037239075, 0.2205672711133957, -0.2956762909889221, -0.1745654046535492, 0.21979111433029175, 0.0257128793746233, 0.15792539715766907, -0.08701542019844055, 0.05795230343937874, 0.24850691854953766, 0.0975043997168541, 0.13328297436237335, 0.3596539795398712, -0.040504153817892075, -0.3100355863571167, 0.004706158768385649, -0.09295856952667236, -0.23369619250297546, -0.0034472676925361156, 0.05193879082798958, 0.3161531090736389, -0.0236289594322443, -0.4193228781223297, -0.148389533162117, -0.01337303314357996, -0.10205357521772385, 0.17597439885139465, -0.2652505338191986, -0.0680069774389267, 0.036829423159360886, -0.019475210458040237, 0.1258215755224228, 0.0925283208489418, -0.08983875066041946, 0.031817276030778885, -0.031246459111571312, 0.17741529643535614, -0.03042137622833252, 0.07432396709918976, -0.36753228306770325, 0.03290414437651634, -0.2668178975582123, -0.09260071814060211, -0.16626279056072235, -0.007460087072104216, 0.19308502972126007, 0.07276607304811478, -0.2234126180410385, 0.05400984361767769, 0.12854275107383728, -0.18802356719970703, 0.053962983191013336, -0.014784267172217369, -0.09728264808654785, 0.06739639490842819, 0.21587897837162018, -0.002990542910993099, 0.032767005264759064, 0.01749051734805107, 0.05790387839078903, -0.2927199900150299, 0.15386025607585907, -0.07734300941228867, -0.3154853880405426, 0.06284087896347046, 0.2161383032798767, 0.10817591100931168, 0.02607145346701145, 0.2051158845424652, -0.02915884368121624, -0.08837554603815079, 0.203398659825325, 0.05989005044102669, -0.24285268783569336, -0.022037498652935028, 0.02412986382842064, -0.049280907958745956, 0.06374198943376541, -0.01473977416753769, -0.005420090164989233, 0.3071495294570923, 0.12619411945343018, -0.3090022802352905, -0.10290492326021194, 0.19957521557807922, -0.07764606177806854, -0.112922802567482, 0.12383784353733063, 0.09390684962272644, -0.1295548528432846, -0.38317811489105225, 0.20633205771446228, -0.04292415827512741, -0.013071201741695404, -0.1070328950881958, 0.16152997314929962, -0.04316890239715576, 0.08005277812480927, 0.07249706238508224, -0.11560549587011337, -0.2977750897407532, 0.15510299801826477, -0.0033781796228140593, -0.06412380933761597, -0.10640471428632736, -0.12095876783132553, -0.25118115544319153, 0.3473255932331085, -0.11144028604030609, 0.24679498374462128, 0.05138859525322914, 0.08464288711547852}, {-0.02222655527293682, 0.26797375082969666, -0.010281537659466267, 0.157333642244339, 0.06057225167751312, -0.012962653301656246, 0.06617426872253418, -0.006036717444658279, 0.055947184562683105, 0.028556333854794502, -0.1756606251001358, 0.1908516138792038, -0.002401547273620963, 0.12230348587036133, 0.0006328403833322227, -0.06782660633325577, 0.005148048512637615, -0.014386048540472984, 0.010371495969593525, 0.2721385955810547, -0.20741979777812958, -0.10678698122501373, 0.012537535279989243, -0.08938480913639069, -0.024560561403632164, 0.04727223888039589, 0.07393740862607956, 0.05490877106785774, 0.07837793231010437, 0.013240444473922253, -0.02120032161474228, 0.1702631264925003, -0.021273205056786537, 0.21137398481369019, 0.1285153478384018, -0.019577695056796074, -0.06229672208428383, 0.2174266129732132, 0.07518111169338226, 0.07304265350103378, -0.04152341187000275, 0.06768617779016495, -0.027896281331777573, 0.03009164333343506, 0.0845513790845871, 0.09497585892677307, -0.10875717550516129, 0.12432806193828583, 0.07986586540937424, 0.08907800167798996, -0.07806316018104553, 0.08505593985319138, 0.19502641260623932, -0.25023168325424194, -0.09009212255477905, 0.021169131621718407, 0.02747081220149994, -0.16560129821300507, 0.06361238658428192, -0.11665786802768707, 0.3123495280742645, 0.01197142992168665, 0.01384250819683075, -0.07147872447967529, -0.15825456380844116, 0.07507669180631638, -0.003930012229830027, 0.15976741909980774, -0.3173900544643402, -0.08759736269712448, -0.058808352798223495, -0.06158798933029175, 0.29540178179740906, -0.07531652599573135, 0.12477284669876099, -0.06227874383330345, 0.3367956578731537, -0.021572858095169067, 0.03921385109424591, 0.0448710098862648, -0.19441193342208862, -0.10739138722419739, 0.10098502039909363, -0.08292735368013382, -0.0810440257191658, 0.0007621386903338134, 0.018488140776753426, -0.06950153410434723, 0.06673569977283478, -0.0807277113199234, 0.010230003856122494, -0.0290270633995533, 0.010171426460146904, 0.09185436367988586, 0.08891360461711884, -0.11664899438619614, -0.003926342353224754, 0.163840651512146, -0.07909544557332993, 0.062205322086811066, -0.05993076413869858, 0.18774357438087463, -0.23543484508991241, 0.09805597364902496, 0.04119907319545746, 0.052179597318172455, 0.0783110186457634, 0.08898252993822098, -0.15363246202468872, -0.021300019696354866, 0.06695565581321716, -0.036174751818180084, -0.07088493555784225, -0.33878499269485474, 0.02808765135705471, 0.19535857439041138, 0.02978251688182354, 0.04800185188651085, -0.10334848612546921, -0.05106895789504051, 0.09247654676437378, -0.1041584461927414, 0.16422708332538605, -0.22658485174179077, 0.1017131507396698, 0.016474882140755653, 0.061010412871837616, 0.11703451722860336, -0.13006944954395294, -0.13265888392925262, -0.08296579122543335, -0.14751142263412476, -0.24329477548599243, 0.12394335865974426, 0.16575349867343903, -0.01540023647248745, -0.10779768228530884, -0.008857551030814648, -0.20753170549869537, 0.12472256273031235, 0.06979191303253174, -0.17112453281879425, -0.14520171284675598, 0.006072141695767641, 0.08030252903699875, -0.04808305948972702, 0.006441951729357243, 0.05468229576945305, -0.1138220727443695, 0.08055800944566727, -0.1650800257921219, 0.0677911639213562, -0.053098443895578384, 0.0691540539264679, 0.18269842863082886, -0.19943583011627197, -0.10171432793140411, -0.3035214841365814, 0.01377695333212614, 0.0916796401143074, 0.041856471449136734, -0.09948188811540604, 0.12438279390335083, 0.21320250630378723, -0.0432547926902771, -0.10083235055208206, 0.07411632686853409, -0.011166512034833431, 0.19451580941677094, 0.024479614570736885, 0.010669060982763767, -0.047336380928754807, 0.019771810621023178, 0.04413392394781113, -0.04803520068526268, 0.12549760937690735, -0.14263412356376648, -0.07490622252225876, 0.026604339480400085, -0.1685834676027298, 0.17337077856063843, -0.03648996725678444, -0.12050393968820572, -0.43938443064689636, 0.04396379739046097, 0.19902603328227997, 0.344835489988327, 0.06976915150880814, -0.20516881346702576, -0.07168122380971909, -0.04159624129533768, -0.0779060572385788, -0.09501893073320389, -0.1851498782634735, 0.03550105541944504, 0.2435491979122162, -0.02841329202055931, 0.11306118220090866, 0.04874769225716591, 0.24356363713741302, -0.2147943079471588, -0.03837776929140091, -0.0066052647307515144, -0.16965016722679138, -0.0443742461502552, 0.09305489808320999, 0.19369114935398102, -0.10428161174058914, -0.12606115639209747, 0.05290589854121208, 0.10393543541431427, 0.13083505630493164, 0.020955469459295273, 0.11925780773162842, 0.26740407943725586, -0.12992613017559052, -0.1288357526063919, -0.0002899557875934988, -0.09367942065000534, 0.05851060152053833, -0.008239690214395523, -0.031291671097278595, 0.030732326209545135, 0.3019391596317291, 0.24858011305332184, 0.43179967999458313, -0.18846505880355835, 0.01106051541864872, 0.1705867201089859, -0.020296715199947357, -0.2718665599822998, -0.03198074921965599, 0.03224579989910126, 0.09984171390533447, 0.04179614782333374, 0.05389191210269928, 0.022663988173007965, -0.044186726212501526, 0.10948016494512558, -0.1837833970785141, 0.2597953975200653, -0.11263477802276611, 0.18941721320152283, 0.2076384276151657, 0.06854220479726791, -0.15619388222694397, -0.009405797347426414, -0.03489445894956589, -0.17371568083763123, -0.014065864495933056, -0.06860567629337311, 0.11452905088663101, 0.07866864651441574, -0.12227440625429153, 0.02307981811463833, 0.35455644130706787}, {-0.017297571524977684, 0.13159334659576416, -0.048311036080121994, 0.08868873119354248, -0.025795698165893555, 0.21768750250339508, -0.11511211842298508, -0.020565612241625786, 0.17974953353405, -0.2375069111585617, 0.10803839564323425, 0.038465291261672974, 0.011160171590745449, 0.20661024749279022, -0.060368623584508896, 0.067647285759449, 0.11407358199357986, 0.3969535529613495, -0.11067917197942734, 0.16548481583595276, 0.011801048181951046, -0.20120200514793396, 0.1489163041114807, 0.2296004295349121, 0.3214731812477112, -0.34794843196868896, -0.08928480744361877, -0.07912666350603104, 0.2064732313156128, 0.3061845302581787, 0.07950980216264725, -0.09864673018455505, -0.05942516773939133, 0.1319790482521057, 0.30229613184928894, 0.12015806883573532, 0.09907784312963486, 0.22700844705104828, -0.015259076841175556, 0.2380649596452713, 0.22143790125846863, -0.3026871383190155, -0.3138883411884308, 0.020111342892050743, 0.16644833981990814, 0.2684650421142578, -0.21897947788238525, -0.26905113458633423, 0.008531969040632248, -0.0022534620948135853, 0.14669911563396454, 0.09156422317028046, 0.02695106714963913, -0.16696882247924805, -0.18155351281166077, 0.1895807832479477, 0.05988651514053345, 0.041206806898117065, -0.0385018065571785, -0.066051185131073, 0.16714489459991455, 0.2223210483789444, -0.10936929285526276, -0.028334302827715874, -0.3816989064216614, -0.1381922960281372, 0.3971022963523865, 0.2673734426498413, 0.07488233596086502, -0.3311660587787628, 0.05435062572360039, 0.21584753692150116, -0.3253685534000397, -0.03826797753572464, 0.3326337933540344, -0.07708115130662918, 0.2072238028049469, -0.07168328762054443, -0.2016511857509613, 0.41125157475471497, -0.1404687762260437, -0.3288739025592804, 0.08516546338796616, -0.20411090552806854, 0.16272929310798645, -0.10666602104902267, -0.14553236961364746, 0.26552680134773254, 0.20531940460205078, -0.0584210604429245, 0.09543642401695251, 0.058245979249477386, 0.3104478120803833, -0.005229318514466286, 0.24423711001873016, 0.12374404072761536, -0.018481245264410973, 0.08549819886684418, 0.012900474481284618, 0.2890506386756897, -0.2766212522983551, 0.14172950387001038, 0.3385028541088104, 0.10635655373334885, 0.029989274218678474, 0.7070924043655396, 0.08615217357873917, 0.014207461848855019, -0.000619174272287637, -0.08882154524326324, 0.061554547399282455, 0.26524823904037476, 0.16679537296295166, -0.32669568061828613, -0.044789690524339676, 0.07101821899414062, -0.16460749506950378, -0.1840498298406601, 0.02878250926733017, -0.09436005353927612, 0.16070285439491272, 0.2886849641799927, -0.059852853417396545, 0.2993893027305603, -0.16210828721523285, 0.04649636149406433, -0.20306387543678284, 0.06479864567518234, -0.2130614072084427, -0.22725030779838562, -0.1922549307346344, -0.10244657844305038, -0.08788697421550751, -0.06906132400035858, 0.12113475799560547, 0.055012885481119156, -0.028816763311624527, -0.03102060593664646, -0.029666699469089508, 0.1738756150007248, -0.14669008553028107, -0.16037243604660034, 0.02516808919608593, -0.3878737986087799, 0.06544539332389832, 0.11847308278083801, -0.07237594574689865, -0.1658090054988861, 0.0928419902920723, -0.11159485578536987, -0.2317374050617218, 0.07836401462554932, -0.03951721265912056, 0.050274841487407684, 0.2949110269546509, -0.12814640998840332, 0.04785000905394554, 0.08349253982305527, -0.1671883761882782, -0.12184176594018936, 0.05678500980138779, -0.3794454336166382, -0.024533454328775406, 0.192750483751297, 0.14345194399356842, 0.22666361927986145, -0.27500516176223755, -0.022235902026295662, -0.09244596213102341, 0.21676820516586304, -0.06970827281475067, -0.0791511982679367, -0.0838107019662857, -0.0023548086173832417, 0.11978904157876968, -0.45845696330070496, 0.12020702660083771, -0.15106280148029327, -0.1630973517894745, -0.007670220453292131, 0.2465333193540573, -0.043275136500597, -0.059306301176548004, -0.3388676345348358, 0.11586777865886688, 0.16989372670650482, 0.09118705242872238, -0.16115228831768036, -0.11883348971605301, 0.19556108117103577, -0.15885400772094727, -0.03451738879084587, -0.00342450849711895, -0.11187978088855743, -0.039246030151844025, 0.10920652747154236, -0.02463318407535553, 0.008416497148573399, -0.02283846214413643, 0.23791398108005524, -0.019618744030594826, -0.11946608126163483, 0.15725138783454895, -0.025758426636457443, -0.2817351222038269, 0.15797419846057892, 0.07319225370883942, 0.18383203446865082, -0.059538859874010086, 0.3187943398952484, -0.1862868070602417, -0.04086768254637718, -0.26967018842697144, -0.1749873161315918, 0.14069774746894836, -0.0012071237433701754, -0.1645120531320572, -0.004771114327013493, 0.15364642441272736, -0.01447107270359993, -0.10154882818460464, -0.03069656528532505, -0.00244809384457767, 0.24398252367973328, 0.0105829369276762, 0.21540358662605286, -0.33661845326423645, -0.17588172852993011, 0.32928791642189026, -0.034820109605789185, -0.0670747384428978, -0.12447641044855118, -0.07770182937383652, -0.18523754179477692, -0.14344273507595062, 0.11892351508140564, 0.056242696940898895, 0.12121207267045975, -0.02909936010837555, 0.47811225056648254, 0.14125001430511475, 0.025470003485679626, 0.390109658241272, -0.031265631318092346, -0.25576531887054443, 0.31036362051963806, 0.173807293176651, -0.2110023945569992, -0.1324741691350937, 0.13185329735279083, -0.12557204067707062, 0.14628176391124725, -0.17908430099487305, -0.2866455018520355, -0.1571027785539627, 0.10119587928056717}, {0.14051149785518646, 0.27965623140335083, -0.10175172239542007, 0.10224930197000504, -0.12591169774532318, -0.07843010127544403, -0.0805874615907669, 0.13924437761306763, -0.15928570926189423, 0.19164849817752838, -0.14288608729839325, 0.3101850748062134, 0.007359180133789778, 0.07866670936346054, 0.01970609277486801, -0.0456850528717041, 0.06728734821081161, -0.041264113038778305, -0.24509382247924805, -0.05256788432598114, 0.06195767968893051, 0.20714126527309418, -0.00924297422170639, 0.04485847055912018, -0.07473273575305939, 0.002582202199846506, -0.11658530682325363, -0.025253554806113243, 0.042053546756505966, -0.09423568844795227, 0.012127315625548363, -0.0093549694865942, 0.04980391636490822, 0.02608892135322094, 0.0727803185582161, 0.044289492070674896, 0.09836545586585999, -0.08007786422967911, -0.1482173502445221, -0.0064005362801253796, -0.02850017510354519, 0.13257429003715515, -0.0047534448094666, 0.2563956379890442, 0.25522926449775696, -0.13351508975028992, 0.032499153167009354, -0.22802391648292542, 0.2334175407886505, 0.004027183633297682, -0.023258691653609276, -0.18197192251682281, 0.14842349290847778, 0.049309324473142624, 0.00187048246152699, -0.1296982616186142, 0.03442327678203583, 0.017641613259911537, -0.05445489659905434, -0.22729599475860596, 0.16061203181743622, 0.046661701053380966, -0.13655133545398712, -0.1541876643896103, -0.008576068095862865, -0.08009805530309677, -0.09086831659078598, 0.01667754165828228, 0.13155846297740936, -0.22564463317394257, -0.00404254999011755, -0.14376062154769897, 0.01820306107401848, 0.1104578748345375, 0.565022349357605, -0.008379637263715267, -0.14956222474575043, -0.0007513711461797357, -0.0924626961350441, 0.08534693717956543, 0.1463678926229477, 0.05054841190576553, -0.029782669618725777, 0.20644836127758026, -0.062097374349832535, 0.07151355594396591, 0.07682820409536362, -0.042259279638528824, 0.049737926572561264, -0.12771116197109222, -0.0028825721237808466, -0.08171764761209488, -0.11107936501502991, -0.11142613738775253, 0.037037260830402374, 0.0008084442815743387, 0.001174839329905808, 0.066290944814682, -0.048856914043426514, 0.4808233976364136, -0.12982484698295593, 0.04067138209939003, -0.22372731566429138, -0.026585761457681656, 0.18401631712913513, 0.47187724709510803, 0.0022793228272348642, -0.0029963913839310408, -0.05580460652709007, -0.10167310386896133, 0.06748095899820328, -0.08007679879665375, 0.026044631376862526, -0.030236313119530678, 0.11559484153985977, -0.012558968737721443, -0.023467909544706345, 0.08667381852865219, 0.026164302602410316, 0.08523255586624146, -0.048711489886045456, -0.08408299833536148, 0.008568603545427322, 0.04291136935353279, 0.0037134033627808094, 0.11209061741828918, -0.07303766161203384, -0.1066567450761795, 0.03651341050863266, 0.033724889159202576, -0.023001564666628838, -0.07099802047014236, -0.017616158351302147, -0.033678095787763596, -0.009426450356841087, -0.29847195744514465, -0.061726827174425125, 0.19287917017936707, -0.17209190130233765, 0.003525917883962393, 0.043506134301424026, 0.00583537807688117, 0.35358598828315735, -0.09100798517465591, 0.05773860588669777, -0.0061143431812524796, 0.27670297026634216, -0.0764225497841835, -0.07203196734189987, 0.07408864796161652, 0.043097104877233505, 0.046210724860429764, 0.10570525377988815, -0.06273958086967468, -0.10299210250377655, -0.0064221336506307125, -0.1979573369026184, -0.15892323851585388, 0.2154359072446823, 0.4295031428337097, -0.04544031247496605, 0.10247514396905899, 0.048179734498262405, 0.05783754214644432, -0.036879852414131165, -0.04816851019859314, -0.11510158330202103, -0.040343936532735825, -0.07621509581804276, 0.13013584911823273, 0.0068390145897865295, 0.20027250051498413, -0.0010074533056467772, 0.0064230626448988914, 0.12363087385892868, -0.008485522121191025, 0.28135380148887634, 0.010657524690032005, 0.026176555082201958, 0.06484723091125488, 0.09539328515529633, 0.15038949251174927, -0.04122420772910118, 0.09229390323162079, -0.11666334420442581, 0.11435838043689728, -0.17067037522792816, 0.04322925582528114, 0.12339846044778824, -0.19587954878807068, -0.011345643550157547, -0.2063456028699875, -0.25142601132392883, -0.06131155788898468, -0.023983312770724297, 0.2532079219818115, 0.11742112040519714, 0.1835976243019104, -0.06104353442788124, 0.041677143424749374, -0.10187586396932602, 0.0010525680845603347, 0.03662798926234245, -0.03325311839580536, -0.10732219368219376, -0.018214507028460503, 0.22607873380184174, 0.2702781558036804, -0.0349036306142807, -0.07026247680187225, -0.05149924010038376, -0.321818470954895, -0.12436201423406601, 0.15883776545524597, 0.11173322051763535, 0.09814700484275818, 0.13870109617710114, 0.05549615994095802, 0.07528063654899597, -0.011396261863410473, 0.129481241106987, -0.07171493768692017, 0.06867919862270355, -0.29890280961990356, 0.11236713081598282, -0.21180084347724915, -0.026348302140831947, 0.14564235508441925, 0.27320873737335205, 0.1724890023469925, 0.05797549709677696, 0.07829438894987106, 0.16081911325454712, 0.1067509800195694, 0.06971738487482071, 0.004879191517829895, -0.05530546233057976, 0.06290401518344879, -0.25344347953796387, 0.7065738439559937, -0.005879351403564215, -0.01595514453947544, -0.12879566848278046, -0.007398516871035099, -0.07959562540054321, 0.028030946850776672, 0.04995858669281006, 0.30977901816368103, -0.13015714287757874, -0.048396799713373184, -0.01887945458292961, -0.0081662992015481, 0.0946551039814949, -0.27607807517051697, -0.112860307097435, -0.13107681274414062}, {0.01654876582324505, -0.06923757493495941, 0.09875056892633438, 0.17755338549613953, -0.16829612851142883, -0.12405689805746078, 0.060887642204761505, -0.08214623481035233, 0.06484326720237732, 0.1507725715637207, 0.03846053406596184, 0.10208994895219803, -0.06793171912431717, 0.05019308999180794, -0.1214379072189331, 0.014905453659594059, 0.22861023247241974, -0.1158565804362297, 0.09437092393636703, -0.34277671575546265, -0.002877495251595974, 0.019549351185560226, -0.032405588775873184, -0.22226835787296295, -0.13532964885234833, -0.40135693550109863, -0.001311811967752874, 0.048897672444581985, -0.036470215767621994, -0.052249208092689514, 0.0003844313323497772, -0.05619719624519348, 0.0728410929441452, -0.12995585799217224, -0.0608220249414444, -0.06640949100255966, -0.0024630562402307987, -0.038104403764009476, -0.07270653545856476, -0.0603046752512455, 0.10016639530658722, -0.06508740782737732, -0.12152498960494995, 0.14244677126407623, -0.12220002710819244, -0.12175700813531876, -0.1245906800031662, 0.042990073561668396, 0.0064290608279407024, -0.03907260671257973, 0.10467873513698578, -0.0059525142423808575, -0.018083667382597923, -0.16368375718593597, 0.009331310167908669, -0.1344473659992218, -0.1430826336145401, 0.15429843962192535, 8.694144344190136e-05, 0.000965284591075033, 0.1294533759355545, -0.1928119957447052, 0.15864533185958862, 0.03402862325310707, -0.03100433014333248, 0.07045522332191467, 0.053411733359098434, 0.12244053930044174, -0.10731559246778488, 0.07555996626615524, 0.09435976296663284, -0.10581962764263153, 0.028769247233867645, 0.03306836262345314, 0.04534869268536568, -0.1188349574804306, 0.039599012583494186, 0.05342519283294678, -0.23016230762004852, -0.06530516594648361, 0.10936681926250458, -0.035190142691135406, 0.07130659371614456, -0.00914769247174263, 0.11553070694208145, -0.14834631979465485, -0.10069642215967178, -0.013171501457691193, 0.09555650502443314, 0.07055255770683289, 0.0514129213988781, -0.19831159710884094, -0.011868723668158054, 0.11159791052341461, 0.1485941857099533, 0.2761549651622772, -0.016708021983504295, -0.09667838364839554, 0.0785573422908783, -0.22109262645244598, -0.07823862135410309, 0.01764276996254921, -0.07270565629005432, 0.12919862568378448, 0.13030068576335907, -0.41107144951820374, 0.0877535343170166, -0.1319723129272461, -0.13536716997623444, 0.13218556344509125, 0.1652301847934723, -0.10151572525501251, -0.20444555580615997, 0.03888188675045967, 0.13772544264793396, 0.04228539392352104, -0.012847930192947388, -0.12413444370031357, 0.388113409280777, 0.03498044237494469, 0.04126199707388878, -0.05549206584692001, -0.14798033237457275, -0.11324498802423477, 0.09848854690790176, 0.21108323335647583, 0.02266181819140911, 0.04712328687310219, -0.010660835541784763, -0.1959669589996338, -0.06239859759807587, -0.012644631788134575, 0.04717487841844559, 0.04067498445510864, -0.09931506961584091, -0.10935524106025696, -0.11826290190219879, 0.15055790543556213, -0.18961314857006073, 0.07198239862918854, 0.01120948139578104, 0.09990153461694717, -0.09104207903146744, 0.09570389986038208, 0.09452910721302032, 0.09157024323940277, -0.015360101126134396, -0.0959056094288826, 0.04704727232456207, 0.13379213213920593, -0.08892368525266647, 0.05822587013244629, 0.177298903465271, -0.0395539216697216, 0.06085725501179695, 0.125091090798378, 0.03718612715601921, -0.044045016169548035, 0.2352956235408783, 0.11405418813228607, 0.009656163863837719, 0.18416862189769745, -0.00816416461020708, 0.10283540189266205, 0.1056518480181694, 0.14584298431873322, -0.007390434388071299, -0.03326869010925293, -0.2512476444244385, 0.11321250349283218, -0.051215022802352905, -0.08131161332130432, -0.10107541084289551, -0.11749350279569626, -0.12555718421936035, -0.12668894231319427, -0.050450000911951065, -0.1048399806022644, 0.11402291804552078, -0.16623812913894653, -0.07676829397678375, 0.09971042722463608, 0.01258463878184557, 0.1487465500831604, -0.3556773364543915, -0.16594800353050232, -0.015589688904583454, -0.048793237656354904, 0.1287507265806198, 0.14932307600975037, 0.05447743833065033, -0.38877326250076294, 0.10793962329626083, -0.039353154599666595, 0.054762475192546844, 0.03477877750992775, -0.020182112231850624, -0.01496003195643425, 0.1481122225522995, 0.07814636081457138, 0.014272310771048069, -0.12318352609872818, -0.02783840335905552, 0.33033478260040283, 0.20176586508750916, -0.05289384722709656, -0.14950664341449738, -0.033106639981269836, -0.2568259835243225, -0.0886835977435112, 0.22184963524341583, -0.20441901683807373, 0.05687836930155754, 0.23563221096992493, -0.08524501323699951, -0.12696725130081177, 0.12934286892414093, 0.021408746019005775, -0.23021647334098816, 0.22862999141216278, -0.10253174602985382, -0.23441971838474274, 0.22965547442436218, -0.2021496444940567, 0.006903225556015968, -0.0784183219075203, 0.11193347722291946, 0.1391593962907791, 0.11409097164869308, -0.22143329679965973, -0.14153562486171722, 0.07178382575511932, -0.01969929225742817, -0.09882994741201401, 0.05092642456293106, 0.057974714785814285, 0.04344114288687706, 0.0816323459148407, 0.4037679135799408, -0.2664145231246948, 0.03531577065587044, 0.10488316416740417, -0.0312952920794487, 0.10809234529733658, -0.020067384466528893, -0.16095872223377228, 0.11919694393873215, -0.06541391462087631, 0.051050446927547455, -0.20670783519744873, 0.033323854207992554, 0.005742229521274567, 0.23131480813026428, -0.037330128252506256, 0.006766251754015684, -0.09668023139238358}, {-0.008652988821268082, 0.06130361184477806, -0.15579605102539062, -0.0064515103586018085, -0.08005399256944656, -0.21689462661743164, 0.025414925068616867, 0.20999859273433685, -0.018238713964819908, 0.02470928430557251, 0.06324904412031174, 0.05356179177761078, -0.12584879994392395, 0.004030722659081221, -0.08711111545562744, 0.14275819063186646, 0.023922407999634743, 0.03895600512623787, -0.04446547478437424, 0.10505544394254684, -0.07761067152023315, -0.03915718197822571, 0.08762107044458389, -0.13913199305534363, -0.11050227284431458, -0.07598737627267838, -0.07567457854747772, 0.08479885011911392, -0.221311554312706, -0.265516459941864, 0.08187931776046753, -0.08414196968078613, -0.01734272949397564, 0.08410787582397461, -0.18475057184696198, 0.0909428521990776, 0.01582854613661766, 0.26631519198417664, 0.01570012792944908, -0.03619823977351189, 0.2735768258571625, -0.24002651870250702, -0.27717000246047974, -0.22370733320713043, -0.12508051097393036, -0.20630605518817902, -0.05568932741880417, 0.08128298819065094, -0.11712609231472015, 0.030259063467383385, -0.02609041891992092, -0.07850626111030579, -0.025030467659235, -0.20355233550071716, 0.02945239469408989, 0.014856485649943352, 0.06181942671537399, -0.1698102355003357, -0.029285237193107605, -0.14233916997909546, 0.11675111949443817, 0.16560952365398407, -0.01814374513924122, 0.3762068450450897, 0.04307924211025238, -0.22817279398441315, -0.10600169748067856, -0.08550725877285004, -0.08924034237861633, -0.06994496285915375, 0.04311951622366905, -0.1287115514278412, 0.047092754393815994, -0.07776234298944473, 0.002367364941164851, -0.05414420738816261, -0.1018538847565651, 0.12266435474157333, -0.04405910521745682, 0.027815937995910645, 0.11240886896848679, -0.14743179082870483, 0.027210364118218422, 0.02714483067393303, 0.041107431054115295, 0.10437371581792831, -0.15925341844558716, -0.2854408919811249, 0.1083514615893364, -0.282097727060318, -0.11313196271657944, -0.17407499253749847, 0.20813605189323425, -0.11309844255447388, 0.24587924778461456, 0.0718134269118309, -0.05231142416596413, 0.036030255258083344, 0.1369994431734085, 0.06366398930549622, -0.15294422209262848, -0.11530967056751251, -0.1737363636493683, -0.13169945776462555, -0.17841346561908722, 0.06262527406215668, -0.15723887085914612, -0.07152082026004791, 0.20242099463939667, 0.13953740894794464, -0.004400266334414482, 0.1808278113603592, -0.03181956335902214, -0.008186436258256435, 0.06670331209897995, 0.015211960300803185, 0.11842585355043411, -0.039309918880462646, 0.11776126176118851, 0.22918128967285156, 0.057783983647823334, -0.05643942952156067, -0.0883418396115303, -0.03206578642129898, -0.022955363616347313, -0.0405728816986084, 0.12184584140777588, 0.016371753066778183, 0.013736545108258724, -0.15790319442749023, 0.11208858340978622, 0.07685614377260208, -0.08562647551298141, 0.011045794934034348, -0.020505385473370552, 0.044941749423742294, -0.08327211439609528, 0.20919205248355865, -0.09121373295783997, 0.005575553514063358, -0.031535789370536804, 0.054919175803661346, -0.045139048248529434, 0.0358898900449276, 0.00878254696726799, 0.10146871209144592, 0.2122678905725479, -0.11060994118452072, 0.07358544319868088, 0.014552040956914425, 0.0838436409831047, -0.06702858954668045, 0.04657020419836044, 0.11471952497959137, 0.06996648013591766, 0.07366516441106796, -0.26732146739959717, 0.09571978449821472, 0.41718149185180664, 0.0729372501373291, -0.002200324786826968, 0.04170548915863037, -0.17962446808815002, -0.26112765073776245, 0.09237167239189148, -0.1011243537068367, -0.08537208288908005, -0.02673863247036934, -0.04003790393471718, -0.07039303332567215, 0.19519351422786713, -0.04861364886164665, -0.07610280066728592, 0.03403105586767197, 0.026883913204073906, 0.16099417209625244, -0.03389127552509308, -0.13702267408370972, -0.06303239613771439, 0.06699667870998383, -0.2686558961868286, -0.062228716909885406, -0.18486301600933075, -0.07188299298286438, 0.1339493989944458, -0.17604371905326843, 0.1756783127784729, 0.06682964414358139, 0.05837292596697807, 0.04035719111561775, 0.09072108566761017, -0.04721386358141899, -0.008848622441291809, -0.01897663250565529, -0.12459402531385422, 0.01639648526906967, -0.14214752614498138, 0.13184531033039093, 0.09659022837877274, -0.0444193035364151, 0.032277561724185944, -0.07225682586431503, 0.09503544121980667, 0.12124664336442947, 0.05946393311023712, -0.10853680968284607, 0.14602597057819366, -0.11925142258405685, 0.09029769152402878, -0.22331605851650238, 0.12296536564826965, 0.006296833511441946, -0.05031300708651543, -0.26185905933380127, 0.03377155959606171, -0.006352478172630072, -0.11327312141656876, 0.07801748067140579, -0.06970050185918808, -0.008142247796058655, 0.04444989934563637, 0.005470471456646919, -0.22225089371204376, 0.07943756133317947, 0.17823418974876404, 0.0803680419921875, -0.16902483999729156, 0.06750927120447159, 0.289499968290329, -0.013264673762023449, -0.0024342245887964964, -0.11977607011795044, 0.16039402782917023, -0.061262570321559906, -0.13541384041309357, -0.10885638743638992, 0.06414227187633514, 0.09688755869865417, 0.029005996882915497, -0.0711899921298027, -0.00505970511585474, 0.15221768617630005, 0.026913555338978767, -0.1511535793542862, -0.12320099025964737, 0.050974953919649124, 0.004289261065423489, -0.23454029858112335, 0.07756295055150986, 0.09103933721780777, -0.07904677838087082, -0.16157905757427216, -0.06697071343660355, -0.03983588144183159, -0.18185479938983917, 0.030852915719151497}, {-0.14799368381500244, 0.474783331155777, 0.10279617458581924, 0.05727115273475647, 0.03828684613108635, 0.06394273787736893, -0.03143428638577461, 0.11386257410049438, -0.013553129509091377, -0.07437290996313095, -0.03291119635105133, 0.06904339790344238, -0.12800933420658112, 0.12377487868070602, 0.213094562292099, 0.10910970717668533, -0.10278716683387756, -0.041716914623975754, 0.05565488338470459, -0.005378409754484892, 0.2707744538784027, -0.09381826221942902, 0.011225974187254906, 0.0407547652721405, 0.08904294669628143, 0.06039576604962349, -0.013313379138708115, 0.07617317140102386, 0.20608516037464142, 0.10458763688802719, 0.01454756036400795, -0.08402106165885925, -0.005991390440613031, 0.1930343061685562, -0.001557316747494042, -0.0324544683098793, -0.06084609031677246, -0.09246838092803955, 0.17443877458572388, 0.05403059348464012, 0.11050843447446823, 0.006566936615854502, 0.09433013945817947, -0.05188633129000664, -0.15419825911521912, 0.15879957377910614, 0.09700138866901398, -0.16043461859226227, -0.14294712245464325, -0.023030178621411324, 0.014160984195768833, 0.04718482866883278, -0.2379475235939026, 0.01752658002078533, -0.06800451874732971, -0.06647822260856628, -0.0620950423181057, 0.10725667327642441, 0.003475925885140896, -0.01716478355228901, -0.3110102415084839, 0.013178813271224499, 0.009886256419122219, 0.23803967237472534, 0.10386977344751358, -0.034735146909952164, 0.12117620557546616, 0.030157145112752914, 0.28876206278800964, -0.03623379394412041, -0.22474925220012665, 0.039379216730594635, 0.015069986693561077, 0.10733497142791748, -0.05959208682179451, -0.04508101940155029, -0.0091811278834939, -0.08811327815055847, 0.27300354838371277, 0.0940595492720604, 0.04812521114945412, -0.25824710726737976, -0.050856295973062515, -0.09191343933343887, -0.12787674367427826, -0.12519524991512299, -0.0873820036649704, -0.011830464005470276, 0.08842454850673676, 0.15574701130390167, 0.007053227163851261, 0.011353230103850365, -0.05641866847872734, -0.07785246521234512, 0.012225623242557049, 0.1003301590681076, 0.12673254311084747, 0.032165661454200745, 0.12431757152080536, 0.0933288112282753, 0.02774788811802864, -0.004808597732335329, 0.16299685835838318, 0.20223525166511536, 0.13478727638721466, 0.3665623664855957, 0.06579866260290146, -0.030317233875393867, 0.07512684911489487, 0.18630434572696686, 0.016641385853290558, -0.011775159277021885, -0.09518027305603027, 0.17818684875965118, 0.08324209600687027, -0.0635390356183052, 0.16007746756076813, 0.13481393456459045, -0.09810808300971985, -0.11474666744470596, -0.02646598033607006, 0.106821708381176, 0.17715638875961304, 0.05624021589756012, 0.06586530804634094, 0.2119712382555008, 0.10487151890993118, -0.0701698288321495, -0.07649222016334534, -0.043535079807043076, -0.13680851459503174, -0.09856110066175461, 0.011718165129423141, -0.0020456258207559586, -0.00907075870782137, 0.10179264098405838, 0.33008813858032227, -0.06670632213354111, 0.01717211864888668, 0.199982687830925, -0.09204283356666565, -0.058777108788490295, 0.06903214752674103, -0.17037439346313477, 0.1623009741306305, 0.007621079683303833, -0.16085921227931976, -0.20337578654289246, 0.0339466892182827, 0.1214635893702507, -0.0913354903459549, 0.1305740624666214, 0.059932585805654526, 0.11063628643751144, 0.18263870477676392, 0.06562618911266327, -0.0009888872737064958, 0.08874929696321487, -0.06355384737253189, -0.09912216663360596, 0.05134149640798569, 0.015823345631361008, -0.09773916751146317, 0.14618156850337982, 0.13596661388874054, 0.0028489683754742146, 0.07271778583526611, -0.06939546763896942, -0.03956921398639679, -0.05523255094885826, 0.03540810942649841, -0.11024685949087143, 0.01949365623295307, -0.01289225835353136, -0.015653813257813454, 0.1871451437473297, 0.15640735626220703, -0.0043237339705228806, -0.04960591718554497, -0.05182407796382904, 0.02572254091501236, -0.0806569755077362, 0.08147905021905899, 0.06423670053482056, 0.03984347730875015, -0.006144956219941378, -0.08203912526369095, 0.06217929720878601, -0.270656555891037, 0.2793910503387451, -0.11696204543113708, -0.0778355523943901, -0.03639868274331093, 0.0013904450461268425, 0.15953071415424347, -0.12647783756256104, -0.10081708431243896, -0.18383511900901794, 0.14970767498016357, 0.051167458295822144, 0.02954961359500885, 0.2570488452911377, -0.03762233629822731, -0.03959008306264877, -0.06932740658521652, -0.014970554038882256, -0.11214137077331543, -0.05874299257993698, 0.1360630989074707, 0.05840618535876274, -0.005958803929388523, -0.17650821805000305, 0.013489234261214733, 0.17657527327537537, -0.25860241055488586, -0.1316964328289032, -0.005045784637331963, -0.3102770745754242, 0.07016637921333313, 0.03472276031970978, 0.012232145294547081, 0.10114655643701553, -0.08401601761579514, 0.007474484387785196, 0.01390061341226101, -0.01891103759407997, -0.32585492730140686, 0.055587947368621826, -0.03235342353582382, -0.0006496055284515023, -0.026928436011075974, -0.1129269003868103, -0.21049633622169495, -0.17634925246238708, -0.0025852632243186235, -0.2688073515892029, 0.128667414188385, -0.15370212495326996, -0.025485657155513763, 0.1662566363811493, -0.05053500086069107, 0.12054505199193954, -0.011325824074447155, 0.04263174161314964, -0.045279361307621, 0.14066627621650696, -0.20364971458911896, 0.09250106662511826, 0.0062311249785125256, -0.05830607935786247, 0.06613723933696747, 0.08222366124391556, 0.12973476946353912, -0.17493809759616852, -0.025588970631361008, -0.22574593126773834}, {-0.19877393543720245, 0.26054996252059937, -0.16495046019554138, -0.11554796993732452, 0.16028118133544922, -0.16608741879463196, 0.09345842152833939, 0.032197751104831696, -0.12732143700122833, 0.14900068938732147, -0.17070718109607697, 0.32869890332221985, -0.012936686165630817, -0.02347109280526638, 0.049323540180921555, 0.004161512944847345, 0.14789700508117676, 0.0436069518327713, -0.04124448448419571, -0.017856214195489883, 0.2203652709722519, 0.030113140121102333, 0.05868718400597572, 0.18453113734722137, 0.03478599339723587, 0.06324631720781326, -0.20907065272331238, -0.039919544011354446, -0.02253497578203678, -0.15521949529647827, 0.08439049869775772, -0.1051836609840393, -0.015866950154304504, -0.005471032112836838, 0.09808158874511719, 0.03202223405241966, -0.19943362474441528, 0.03922231122851372, -0.028994224965572357, 0.045663923025131226, 0.030481986701488495, 0.03209999203681946, 0.1673307865858078, -0.05411777272820473, -0.11452510952949524, 0.10687949508428574, -0.11900833994150162, -0.07286679744720459, -0.04461051523685455, 0.19254879653453827, -0.17958928644657135, 0.049131907522678375, -0.0195109024643898, 0.19128713011741638, -0.2639523446559906, 0.0944211483001709, 0.07299994677305222, 0.030477607622742653, -0.1636318564414978, -0.016869118437170982, -0.04439642280340195, -0.1180429756641388, 0.16877910494804382, 0.1150125116109848, -0.0658058300614357, 0.003413607832044363, -0.0009061353048309684, 0.0814567282795906, 0.3412242531776428, 0.11544869840145111, -0.0018015909008681774, -0.14310099184513092, -0.10426505655050278, 0.11759612709283829, 0.11359678208827972, 0.0645967572927475, -0.26785808801651, -0.27564677596092224, 0.14719662070274353, -0.1138966754078865, -0.010754182003438473, 0.11863355338573456, 0.3372907042503357, -0.04106922075152397, 0.008604230359196663, -0.1459600180387497, 0.12196806073188782, -0.022784721106290817, 0.04769659414887428, -0.08371825516223907, 0.0619422011077404, -0.10772519558668137, -0.25943005084991455, 0.016982298344373703, 0.10898224264383316, 0.233478844165802, 0.0423421747982502, 0.09210038930177689, 0.07147461175918579, -0.08746924996376038, 0.09834658354520798, 0.12242672592401505, 0.02932191640138626, -0.018117602914571762, -0.012445252388715744, -0.5955328345298767, 0.1341845840215683, 0.09867852181196213, 0.030100544914603233, -0.10592266172170639, -0.18778623640537262, -0.17810240387916565, -0.05672096833586693, 0.053943049162626266, 0.01989743299782276, 0.1835777908563614, -0.09408091008663177, 0.051800310611724854, 0.08420515060424805, 0.09694424271583557, -0.07300347089767456, -0.12203369289636612, 0.06669703125953674, 0.09364914149045944, 0.031980838626623154, 0.25106576085090637, 0.01259926613420248, 0.06637414544820786, 0.1098773181438446, -0.024021362885832787, 0.07919394224882126, 0.1515514999628067, 0.09932547062635422, -0.01849272847175598, -0.09314849227666855, 0.05911963805556297, 0.08704918622970581, -0.08010207861661911, 0.09393337368965149, 0.003962856717407703, -0.1731235235929489, 0.028788886964321136, -0.12422380596399307, -0.05593790486454964, 0.04041921719908714, -0.10427673161029816, -0.012645922601222992, 0.11532557755708694, 0.060826919972896576, 0.029140908271074295, -0.12286878377199173, -0.06041756272315979, -0.00308470637537539, 0.006606012117117643, -0.06015485152602196, 0.006760238204151392, 0.08309540897607803, 0.08401906490325928, 0.19244356453418732, -0.030085036531090736, -0.04771706089377403, 0.05098342522978783, -0.01227849256247282, 0.056031353771686554, 0.018214261159300804, -0.04873998835682869, 0.014237483032047749, 0.21382318437099457, -0.1409084051847458, -0.09000235795974731, -0.0028726677410304546, -0.09589109569787979, -0.0386737585067749, -0.00298541272059083, -0.02929547429084778, -0.12773923575878143, 0.040774788707494736, 0.03391271084547043, 0.08162469416856766, -0.14011292159557343, -0.051536720246076584, 0.19509808719158173, -0.029552465304732323, -0.11738120764493942, 0.11830591410398483, -0.030209356918931007, -0.07534704357385635, -0.02571219764649868, -0.0391826406121254, -0.08447389304637909, -0.026104582473635674, -0.017832664772868156, -0.01850052736699581, -0.09499140083789825, -0.03519892692565918, -0.019040429964661598, -0.17543691396713257, 0.12067056447267532, 0.11354626715183258, 0.06791485100984573, -0.0937422588467598, 0.08974719792604446, 0.04450739920139313, -0.0996292307972908, 0.03754035383462906, -0.10325579345226288, 0.019020987674593925, 0.0893174558877945, 0.011979502625763416, 0.06229853257536888, 0.0757526233792305, -0.08031190186738968, 0.019609615206718445, 0.30458104610443115, 0.10192857682704926, -0.14956846833229065, 0.1535763144493103, 0.07456307858228683, 0.14041376113891602, 0.07096367329359055, 0.2664124071598053, 0.1506720781326294, -0.08029819279909134, -0.017511392012238503, -0.06460480391979218, 0.10468456149101257, -0.19152207672595978, -0.16808974742889404, 0.10977219045162201, -0.09570759534835815, -0.1548108458518982, -0.03427913039922714, -0.17386002838611603, -0.1275111883878708, 0.08995489031076431, -0.08133213222026825, 0.02761172689497471, -0.2163306027650833, 0.06354273110628128, -0.0877823606133461, -0.14569659531116486, 0.08011593669652939, 0.10672374814748764, 0.08167936652898788, 0.03646289184689522, 0.07559597492218018, -0.2269183099269867, -0.0004489010025281459, 0.14848116040229797, -0.1642550677061081, -0.03986835479736328, 0.2144273966550827, -0.10411436855792999, 0.017764480784535408, -0.09210794419050217, -0.17366421222686768}, {-0.09402653574943542, 0.07423698902130127, -0.08793198317289352, -0.04794042184948921, 0.15651722252368927, 0.16006766259670258, 0.17400765419006348, -0.04958725348114967, 0.0454733669757843, -0.10517703741788864, 0.09356939792633057, -0.01857338659465313, 0.046197570860385895, -0.26900768280029297, -0.08572179079055786, -0.058241620659828186, 0.03726920485496521, 0.12277501821517944, -0.09200232475996017, 0.16247344017028809, 0.07355347275733948, -0.10210812091827393, -0.1655169129371643, -0.05134870857000351, 0.23177295923233032, -0.03795579820871353, 0.004071552772074938, 0.021134451031684875, -0.029573898762464523, -0.0753152146935463, -0.2004171460866928, -0.06619208306074142, 0.22371189296245575, -0.19248215854167938, -0.1416599154472351, 0.4743438959121704, 0.11132153123617172, 0.022096434608101845, 0.05916181206703186, 0.09195278584957123, 0.06003642827272415, 0.02199787274003029, -0.11256590485572815, -0.3158789873123169, 0.0059282733127474785, -0.04429268091917038, -0.17085841298103333, 0.05804798752069473, -0.05118907243013382, -0.18622630834579468, 0.15614350140094757, -0.0789642482995987, 0.02450188435614109, 0.043604206293821335, 0.03265751153230667, 0.07635287940502167, 0.049904145300388336, -0.052179209887981415, -0.11476805806159973, 0.13650323450565338, 0.04715298116207123, -0.17191986739635468, 0.13513676822185516, 0.04343537241220474, 0.06597404181957245, 0.09253305941820145, 0.11899497359991074, -0.04646015167236328, -0.0434100441634655, -0.1142868921160698, -0.10531081259250641, 0.16158415377140045, 0.17614275217056274, 0.026537157595157623, -0.28361043334007263, 0.22189861536026, -0.04859548062086105, -0.0456363819539547, -0.13552527129650116, -0.047232735902071, -0.04826341196894646, -0.07948045432567596, 0.016036687418818474, -0.10819370299577713, 0.13059407472610474, -0.2122756540775299, -0.014115316793322563, 0.06217913329601288, 0.04074237868189812, 0.027710791677236557, 0.01572222076356411, 0.1872962862253189, -0.024510037153959274, 0.1690710484981537, 0.011258273385465145, -0.13398775458335876, -0.13079336285591125, -0.0479544997215271, -0.18573133647441864, -0.21627259254455566, -0.11705432832241058, 0.05775011330842972, 0.037464361637830734, -0.05382975935935974, 0.14414547383785248, -0.00885695219039917, -0.0777735561132431, -0.005017575342208147, -0.3532508909702301, 0.1913338005542755, -0.23069500923156738, -0.047020453959703445, -0.2192569375038147, 0.06380071491003036, 0.21374809741973877, 0.30821672081947327, 0.09771568328142166, -0.11346627026796341, -0.0158033836632967, 0.010423757135868073, -0.09545756131410599, -0.012940318323671818, 0.024461818858981133, 0.0898432731628418, 0.02622753009200096, 0.0024665063247084618, 0.31848978996276855, 0.07868658006191254, -0.1801232546567917, -0.12316122651100159, 0.03500202298164368, 0.02328946441411972, -0.052769213914871216, -0.38854363560676575, 0.04229965806007385, 0.020537929609417915, -0.04331851750612259, -0.27267953753471375, 0.1039041131734848, -0.024391399696469307, -0.16796356439590454, -0.0588669627904892, -0.18736673891544342, 0.38816285133361816, -0.08760511875152588, 0.00861538015305996, -0.07871036976575851, 0.12531551718711853, 0.09156762063503265, 0.0626131221652031, -0.01012608502060175, -0.020369764417409897, 0.14715272188186646, -0.020361026749014854, -0.1287374496459961, -0.03383734077215195, 0.0006830894853919744, -0.1051500141620636, 0.10348112881183624, -0.21179211139678955, 0.08678475022315979, 0.10861840099096298, 0.089413583278656, -0.16520638763904572, -0.12806135416030884, 0.1734367460012436, 0.21329861879348755, 0.008024812676012516, -0.030997082591056824, 0.017558054998517036, -0.07692871987819672, 0.023925645276904106, 0.0960841178894043, 0.33900687098503113, 0.2009800523519516, 0.18165276944637299, 0.0796336978673935, 0.16024242341518402, -0.1115572527050972, -0.025578981265425682, 0.2501148581504822, -0.03932453691959381, 0.12323633581399918, -0.11247118562459946, 0.01673756167292595, 0.06672518700361252, 0.07128936797380447, -0.041286956518888474, -0.21205377578735352, 0.17071408033370972, 0.12489452958106995, 0.00505856703966856, 0.04053066670894623, 0.03887168690562248, 0.06819503754377365, 0.09629249572753906, -0.10594933480024338, -0.1946556270122528, 0.1023244857788086, 0.0825914666056633, -0.16451197862625122, -0.16889481246471405, -0.05432337895035744, -0.2166184037923813, 0.048646122217178345, -0.10868985205888748, -0.09051661193370819, 0.019247664138674736, -0.022661417722702026, 0.02326780930161476, 0.032754115760326385, 0.1051742434501648, 0.08607800304889679, -0.0999370664358139, -0.10113382339477539, 0.007662203628569841, 0.1539519727230072, 0.16874511539936066, 0.0712900385260582, 0.2029317170381546, -0.11859609186649323, 0.021443059667944908, -0.006500874646008015, 0.20904703438282013, 0.09545768052339554, 0.189173623919487, -0.06940115988254547, 0.0791957825422287, 0.06214981526136398, -0.17599259316921234, 0.023756342008709908, -0.040896229445934296, 0.053235266357660294, 0.16976019740104675, -0.06918156892061234, 0.11592516303062439, 0.28989049792289734, 0.052459947764873505, -0.028618603944778442, -0.12017005681991577, -0.1311907172203064, -0.06325423717498779, 0.0060248118825256824, -0.058086272329092026, -0.010669698938727379, -0.09197892248630524, 0.06981822848320007, -0.01993732340633869, 0.024803897365927696, -0.029135974124073982, -0.08937738090753555, 0.09138592332601547, 0.09092986583709717, 0.02540333941578865, -0.0798477903008461, 0.11909209936857224}, {0.2355131357908249, -0.18701045215129852, 0.19087576866149902, -0.18337653577327728, -0.004280563443899155, 0.021772699430584908, 0.06917853653430939, -0.003537700278684497, 0.25459232926368713, 0.1463375687599182, -0.048539310693740845, 0.06270716339349747, -0.043567556887865067, -0.11641966551542282, -0.1043071374297142, 0.04260171204805374, 0.0957803949713707, -0.11017334461212158, 0.0048796916380524635, 0.07662778347730637, -0.08728053420782089, -0.21848227083683014, 0.14692747592926025, -0.006528483238071203, -0.341749906539917, -0.1643441915512085, 0.1362634301185608, -0.061962030827999115, 0.05575885251164436, 0.09757432341575623, 0.07441036403179169, 0.19899165630340576, -0.15810944139957428, -0.13554351031780243, -0.014578474685549736, 0.16335038840770721, 0.0904533639550209, -0.02028915286064148, -0.03824479505419731, -0.05480196699500084, 0.1594659686088562, 0.14100532233715057, -0.08891768753528595, -0.13057957589626312, 0.17153379321098328, -0.004357616417109966, 0.21316683292388916, 0.08805401623249054, -0.05145309120416641, 0.23275060951709747, -0.20943500101566315, -0.024609504267573357, 0.22568394243717194, -0.029834410175681114, -0.0443899966776371, -0.02197185344994068, -0.12093327194452286, 0.024944303557276726, 0.10338625311851501, 0.017994631081819534, 0.24367620050907135, -0.13827519118785858, 0.3408375680446625, -0.1581905335187912, 0.17126482725143433, 0.08139024674892426, 0.006197527516633272, 0.010767209343612194, -0.036574024707078934, -0.15055152773857117, 0.056290403008461, 0.026066146790981293, -0.06834231317043304, -0.07618016749620438, 0.027183223515748978, -0.17897865176200867, 0.22600895166397095, -0.010917349718511105, -0.40894442796707153, -0.0852862149477005, -0.18321602046489716, -0.1490596979856491, 0.002306706039234996, -0.026324262842535973, -0.24752287566661835, -0.11116882413625717, 0.18054692447185516, -0.11563169211149216, 0.03578421473503113, -0.0320638045668602, -0.132949098944664, -0.05124618858098984, 0.014828006736934185, 0.0027923265006393194, -0.25272658467292786, 0.012889718636870384, 0.022650383412837982, 0.1336170732975006, 0.046475496143102646, -0.19416312873363495, 0.07643822580575943, -0.06994204968214035, -0.06761383265256882, 0.14807312190532684, -0.22357583045959473, -0.1272539645433426, -0.04707612842321396, -0.09488500654697418, 0.04583536833524704, 0.038485053926706314, -0.2069072425365448, -0.11371230334043503, -0.0341339185833931, -0.023964377120137215, 0.044332947582006454, -0.09707022458314896, -0.30392763018608093, -0.21026001870632172, -0.048056840896606445, -0.07887893915176392, -0.22697368264198303, 0.06689450144767761, 0.12409991770982742, 0.21086221933364868, 0.009078213013708591, -0.06042341887950897, 0.05507954955101013, 0.09272363036870956, -0.05179784074425697, -0.21495507657527924, -0.08566773682832718, 0.2125917226076126, -0.0366569459438324, 0.21442796289920807, 0.16077964007854462, -0.07906395941972733, 0.0251618679612875, 0.07927506417036057, -0.13292014598846436, -0.23378802835941315, -0.13733969628810883, -0.24443377554416656, -0.011118616908788681, 0.1360384225845337, -0.2386384904384613, 0.018439574167132378, 0.11522549390792847, -0.1628687083721161, 0.04432515427470207, 0.15807802975177765, 0.24534271657466888, 0.06212916597723961, -0.145722433924675, 0.0182436965405941, 0.04166160523891449, -0.11024533212184906, 0.026836836710572243, -0.02471725456416607, -0.14402908086776733, -0.03166763857007027, -0.06650155037641525, -0.03900916501879692, 0.14838601648807526, -0.07338257133960724, 0.0136260362342, -0.038391441106796265, 0.11995696276426315, -0.11981108039617538, 0.15527792274951935, -0.2820223867893219, 0.24282100796699524, -0.18606436252593994, -0.09700150042772293, 0.07025053352117538, 0.056021708995103836, -0.1785300076007843, -0.30538320541381836, -0.04527602717280388, -0.0017925978172570467, 0.008596586063504219, -0.2208288162946701, 0.059536948800086975, 0.051550835371017456, -0.25725609064102173, 0.10549696534872055, 0.29735681414604187, -0.1401117891073227, 0.00958600826561451, -0.25430813431739807, 0.3048601746559143, 0.20933164656162262, 0.32012656331062317, 0.002142102923244238, 0.14278925955295563, 0.019120406359434128, 0.028875939548015594, -0.03709631785750389, 0.14311614632606506, 0.08812188357114792, -0.010895065031945705, -0.20040981471538544, -0.4085538983345032, 0.07707498222589493, 0.1409328728914261, -0.13326193392276764, -0.02844611182808876, 0.13626904785633087, -0.23216310143470764, -0.09673155099153519, 0.12106259167194366, -0.07073386758565903, -0.10602284967899323, -0.020514721050858498, -0.25455784797668457, 0.022385086864233017, -0.06209595873951912, -0.016064725816249847, -0.18446685373783112, -0.24379763007164001, -0.16658829152584076, 0.08571667224168777, 0.12205836176872253, -0.03645946830511093, -0.1449442356824875, 0.38414907455444336, 0.012409989722073078, 0.13905873894691467, 0.039958760142326355, -0.006422170903533697, -0.14692078530788422, 0.08225876092910767, -0.1159476563334465, 0.07108966261148453, 0.08512590080499649, -0.17949046194553375, 0.07044164091348648, -0.019551612436771393, -0.0069408174604177475, -0.015121618285775185, -0.060566287487745285, -0.048979755491018295, 0.11170054227113724, 0.04735714569687843, 0.2547248899936676, 0.2615489959716797, 0.06220509856939316, 0.16644635796546936, -0.053008221089839935, 0.06482858955860138, 0.11652123183012009, -0.03192620724439621, 0.09550824016332626, -0.1796560436487198, 0.010768631473183632, -0.08139178901910782, 0.25096818804740906}, {-0.04217276722192764, 0.1214851662516594, 0.1501910239458084, -1.9763579985010438e-05, -0.011629891581833363, -0.2682766318321228, 0.056690335273742676, 0.25678300857543945, -0.2049863636493683, 0.002174211898818612, 0.02519792877137661, -0.20477335155010223, -0.2542053759098053, 0.08821242302656174, -0.13495732843875885, -0.0934937596321106, -0.13829755783081055, 0.13683441281318665, -0.00875137373805046, 0.08435148000717163, -0.16760100424289703, 0.03196757286787033, 0.008574831299483776, -0.01643562689423561, 0.014642944559454918, 0.11168430745601654, 0.13720853626728058, 0.11011546850204468, -0.12862686812877655, 0.0064417473040521145, 0.0970979556441307, 0.4172977805137634, -0.22332251071929932, 0.2043236792087555, 0.2642774283885956, -0.286983460187912, 0.0712563693523407, 0.028281603008508682, -0.020760543644428253, -0.07715258002281189, 0.10563333332538605, -0.15682289004325867, -0.07292014360427856, 0.15335892140865326, -0.05934305116534233, 0.01726566068828106, 0.11372387409210205, -0.043426431715488434, 0.06529822200536728, 0.10294049978256226, -0.224725604057312, 0.10645255446434021, -0.06285326182842255, 0.03400211036205292, 0.07277575135231018, 0.08802250027656555, -0.009711840189993382, -0.10642123222351074, -0.0338311567902565, -0.10983368009328842, 0.049017105251550674, 0.018371334299445152, 0.13254106044769287, 0.04421813040971756, 0.01620384491980076, -0.0931033045053482, -0.02749098651111126, 0.17286275327205658, 0.09326035529375076, 0.13994066417217255, 0.12631987035274506, -0.12246827781200409, -0.12509144842624664, 0.19054459035396576, 0.1209540069103241, 0.04133061319589615, -0.10228420048952103, 0.10231006145477295, 0.07840066403150558, 0.024658262729644775, -0.2190103530883789, -0.07162925601005554, 0.10764142125844955, -0.04875709116458893, -0.16668136417865753, -0.10335119813680649, -0.06628714501857758, 0.12896870076656342, 0.0578119270503521, 0.07356210798025131, 0.036257270723581314, -0.03109237365424633, -0.11315760761499405, -0.027520833536982536, 0.013963085599243641, -0.04521097242832184, 0.0032108055893331766, 0.05006907880306244, 0.048665851354599, 0.13887546956539154, -0.050520699471235275, 0.11069305986166, -0.002607040572911501, 0.20248429477214813, 0.001093147206120193, 0.005524882115423679, 0.0915224701166153, -0.11693141609430313, 0.23389393091201782, 0.1688685268163681, -0.18921786546707153, -0.0296204574406147, 0.20312382280826569, 0.2585541009902954, 0.07912993431091309, -0.07928431034088135, -0.12479600310325623, -0.04945667088031769, 0.013432781212031841, -0.07535041868686676, 0.10934172570705414, -0.2497940957546234, 0.10077384114265442, -0.15217645466327667, 0.001376823871396482, 0.026647642254829407, 0.04372374713420868, 0.2167484611272812, -0.12628977000713348, 0.3876473009586334, -0.16416533291339874, -0.3232792317867279, -0.19008107483386993, 0.05904323235154152, -0.030745819211006165, 0.029214344918727875, 0.17363274097442627, -0.008453009650111198, -0.05549948662519455, 0.05597146227955818, -0.022311827167868614, -0.0089055011048913, 0.04810884967446327, -0.061414845287799835, -0.00994204357266426, 0.08163157850503922, -0.010633450001478195, -0.12459976226091385, 0.2303725630044937, 0.07607271522283554, 0.09290317445993423, -0.0065348148345947266, -0.04683217778801918, -0.11942671984434128, 0.03853249177336693, 0.213196262717247, -0.3285091519355774, 0.1900729387998581, -0.14925497770309448, 0.14528372883796692, -0.015734918415546417, -0.05402825400233269, -0.04547266289591789, -0.19862762093544006, -0.031165607273578644, -0.07631640136241913, 0.11797301471233368, 0.012481728568673134, 0.21030955016613007, -0.029626930132508278, 0.05349889025092125, -0.03716044872999191, -0.051386408507823944, -0.2423618882894516, 0.08052731305360794, -0.05051353573799133, 0.031029123812913895, -0.06882237643003464, 0.10066577792167664, 0.11603594571352005, -0.21002015471458435, -0.22649146616458893, -0.06914284825325012, -0.10302982479333878, 0.10903464257717133, -0.06029417738318443, 0.12748511135578156, -0.08566777408123016, -0.012593463994562626, 0.17013610899448395, -0.031167099252343178, 0.2757260799407959, -0.08544792979955673, -0.1369597315788269, -0.1663701981306076, -0.27530255913734436, 0.06119838356971741, 0.12978725135326385, 0.14732450246810913, -0.01837952993810177, 0.07588957250118256, 0.07371695339679718, -0.08505871146917343, -0.12428118288516998, 0.03569703549146652, 0.08622848242521286, 0.03316904604434967, -0.24281156063079834, -0.15648475289344788, 0.11950409412384033, 0.15576057136058807, 0.053677354007959366, 0.07712413370609283, 0.15366871654987335, -0.04239768907427788, -0.18058758974075317, -0.09903641045093536, -0.10171934217214584, -0.06036543846130371, -0.09327033162117004, 0.21248029172420502, 0.09916862100362778, -0.1797366887331009, 0.1944677084684372, -0.08789155632257462, 0.18128153681755066, -0.13219580054283142, 0.00943747442215681, -0.04628295078873634, 0.047060176730155945, 0.0072066825814545155, -0.018433760851621628, -0.017551541328430176, -0.1964665651321411, -0.04757211357355118, -0.20684914290905, -0.1807592362165451, 0.07181492447853088, -0.19027648866176605, 0.0741460919380188, 0.03714952990412712, 0.08893024176359177, -0.11446253955364227, 0.0685243085026741, 0.03819813206791878, 0.03269548341631889, -0.036900270730257034, -0.1598864197731018, 0.22302888333797455, -0.10836568474769592, -0.18624664843082428, -0.06152220442891121, -0.08460409194231033, -0.06931504607200623, 0.14249953627586365, 0.13434723019599915}, {0.06315390020608902, 0.11679373681545258, -0.02374361641705036, -0.054011888802051544, -0.10283113270998001, -0.055010728538036346, 0.07654330134391785, 0.07515452802181244, 0.07904422283172607, -0.0642474815249443, 0.0031831671949476004, 0.1844225525856018, -0.007653611712157726, -0.07835737615823746, 0.004198542796075344, 0.10195809602737427, -0.09334670007228851, 0.2003503143787384, 0.044484943151474, 0.2410443127155304, -0.12585389614105225, -0.08019476383924484, -0.07144324481487274, -0.10398009419441223, -0.11390369385480881, -0.11054708063602448, 0.21386654675006866, -0.13363952934741974, 0.06471498310565948, 0.1268865317106247, -0.06462255865335464, -0.26389530301094055, -0.11777977645397186, 0.23050794005393982, -0.020531410351395607, -0.1366620510816574, 0.1223907396197319, 0.0700775757431984, -0.10060276091098785, 0.06486205011606216, 0.12753796577453613, 0.007413196377456188, -0.14994369447231293, -0.18046966195106506, 0.09723436087369919, -0.06368972361087799, -0.012227940373122692, 0.12350033968687057, -0.10344909131526947, 0.0909300297498703, 0.05966438353061676, -0.25775453448295593, 0.02245674468576908, 0.22930561006069183, 0.025936009362339973, 0.14227451384067535, -0.131496399641037, -0.07072912156581879, 0.09958347678184509, 0.03849290683865547, 0.05167045444250107, -0.18273478746414185, 0.1217920184135437, -0.027099039405584335, 0.02502916380763054, -0.08630926162004471, -0.0511171817779541, -0.22464902698993683, 0.07040785998106003, 0.10319025814533234, -0.10915759205818176, 0.060504429042339325, -0.2763440012931824, -0.3257708251476288, 0.0665353387594223, -0.19438566267490387, 0.02980811335146427, -0.05251535400748253, -0.0020351633429527283, -0.17372499406337738, 0.022601287811994553, -0.10457509011030197, -0.10037042945623398, -0.07441172003746033, -0.02103765495121479, 0.014382860623300076, -0.05990451201796532, -0.03987849876284599, 0.0030672799330204725, -0.07172536104917526, -0.03422195836901665, -0.07276660948991776, -0.11398939788341522, 0.04097180441021919, -0.028050199151039124, 0.008205089718103409, 0.012496023438870907, -0.027344558387994766, 0.09615539759397507, -0.03506643697619438, -0.17640458047389984, -0.23219531774520874, 0.009565447457134724, 0.052788976579904556, -0.3150191903114319, 0.2049131542444229, 0.023612717166543007, 0.0972837284207344, -0.09018690139055252, 0.022470377385616302, 0.11997414380311966, -0.1881110668182373, -0.09503132849931717, -0.052639711648225784, 0.031086739152669907, -0.15732203423976898, 0.05500883236527443, 0.034602969884872437, -0.17891016602516174, -0.02330225333571434, -0.021609095856547356, 0.20698168873786926, 0.01724538579583168, 0.13057513535022736, 0.12497677654027939, -0.29633617401123047, 0.17750538885593414, 0.05542388930916786, 0.14296582341194153, 0.09591330587863922, 0.18842221796512604, 0.018779564648866653, 0.04234584793448448, 0.1386699676513672, 0.10105659067630768, -0.22969989478588104, -0.10478819161653519, 0.04655670002102852, 0.052355047315359116, -0.2439069151878357, -0.0049142478965222836, -0.22765061259269714, 0.08461861312389374, 0.08777780830860138, -0.02594739943742752, 0.009904724545776844, -0.037818536162376404, -0.016823163256049156, -0.2335043102502823, 0.1266305297613144, -0.06571006029844284, 0.19516831636428833, 0.09894384443759918, -0.18057706952095032, -0.023490047082304955, 0.005570653360337019, 0.03634919226169586, -0.009900948032736778, -0.16116097569465637, 0.0799320861697197, -0.19761519134044647, -0.02608291245996952, 0.1312251091003418, -0.25770726799964905, 0.09145984798669815, 0.05315307527780533, -0.11548901349306107, 0.1858931928873062, -0.018077686429023743, 0.07232435047626495, 0.1931961476802826, -0.20623165369033813, 0.01536607462912798, -0.18957467377185822, 0.016214553266763687, 0.04065410792827606, -0.026638353243470192, 0.00942699983716011, -0.09186561405658722, -0.029836505651474, -0.009861835278570652, -0.09821950644254684, -0.06397195160388947, 0.029151786118745804, 0.15585751831531525, 0.08771591633558273, 0.06664004176855087, 0.11300291121006012, 0.00983425322920084, 0.033744290471076965, 0.09689882397651672, 0.10200916975736618, -0.10885210335254669, -0.010772943496704102, -0.036785632371902466, 0.15465502440929413, 0.0077788871712982655, -0.09076522290706635, -0.04429427906870842, 0.14451301097869873, 0.12771593034267426, -0.14259864389896393, 0.07777071744203568, -0.0172142144292593, 0.011972365900874138, -0.016558995470404625, 0.026153139770030975, -0.07093644887208939, -0.0019656859803944826, 0.16106384992599487, -0.09879118949174881, 0.21843107044696808, -0.049531735479831696, -0.04536993056535721, -0.13048206269741058, -0.1525166630744934, -0.005341594107449055, 0.05819499492645264, 0.09668967872858047, -0.2157801389694214, -0.059899941086769104, 0.037499818950891495, -0.14958754181861877, 0.07000237703323364, -0.04856997728347778, -0.06979892402887344, -0.031054098159074783, 0.13180965185165405, -0.17754605412483215, 0.11745823174715042, -0.07968183606863022, -0.1439756453037262, -0.01282297819852829, 0.07751336693763733, -0.04262985661625862, 0.10115025192499161, 0.13299645483493805, -0.14488111436367035, -0.2221362590789795, 0.07365732640028, -0.08858535438776016, 0.027639249339699745, 0.317770391702652, -0.12394716590642929, -0.0197567380964756, 0.1076982319355011, -0.09090568125247955, -5.1371491281315684e-05, -0.048544902354478836, -0.017387501895427704, -0.3505140244960785, -0.27364781498908997, 0.19272002577781677, 0.14582648873329163, -0.10459444671869278, -0.006052904296666384}, {0.39219555258750916, 0.39945998787879944, 0.14077872037887573, 0.24699105322360992, -0.4044186472892761, -0.026238199323415756, -0.04945856332778931, -0.20275120437145233, -0.28721320629119873, 0.1257774829864502, -0.0340157188475132, 0.010989328846335411, 0.09751830995082855, 0.2019679844379425, 0.15035364031791687, -0.13658636808395386, 0.2229662835597992, -0.2157098799943924, 0.13749291002750397, -0.019648374989628792, 0.19945472478866577, 0.11499416828155518, 0.03480645641684532, -0.17241275310516357, -0.18226972222328186, 0.1118980124592781, 0.32756757736206055, -0.14644332230091095, -0.370047390460968, -0.12924212217330933, -0.10609158873558044, 0.03797036409378052, -0.09411396086215973, 0.15363049507141113, -0.11481020599603653, 0.16225898265838623, 0.11416532844305038, 0.05573481693863869, -0.0012251262087374926, 0.11055856943130493, -0.195956751704216, -0.16953745484352112, -0.22987355291843414, 0.04237864166498184, 0.13606350123882294, -0.10838855057954788, -0.13774511218070984, -0.24652740359306335, 0.0782332792878151, -0.09304472804069519, -0.07279731333255768, 0.11202068626880646, -0.02821032889187336, -0.14274287223815918, 0.11572910845279694, -0.28639280796051025, -0.2765766680240631, 0.21180877089500427, -0.20078113675117493, -0.05917820334434509, -0.041322607547044754, 0.020572785288095474, 0.2087029367685318, -0.11598999798297882, 0.20569173991680145, -0.36610037088394165, 0.21368125081062317, 0.24101807177066803, -0.13547730445861816, -0.06111488491296768, 0.11050364375114441, -0.13773532211780548, -0.21594348549842834, -0.2022753804922104, 0.26734238862991333, 0.15149511396884918, -0.06330269575119019, 0.3533097505569458, -0.0812591090798378, -0.11284897476434708, -0.04185682162642479, -0.15814226865768433, 0.15788528323173523, 0.25702887773513794, -0.04773690551519394, -0.18257196247577667, -0.01864018477499485, -0.010945068672299385, 0.14461849629878998, -0.23547793924808502, 0.09435298293828964, 0.1931459605693817, -0.3015594184398651, 0.08499915152788162, 0.05457953363656998, 0.23587748408317566, -0.023785674944519997, -0.06101640686392784, -0.018973417580127716, 0.2809900641441345, 0.043523482978343964, 0.041550617665052414, 0.0713844895362854, -0.12374766170978546, -0.327644944190979, 0.3435101807117462, -0.24841651320457458, -0.0551825687289238, -0.01985827647149563, 0.08536607772111893, -0.29123660922050476, -0.04349023848772049, 0.2002718448638916, 0.16289092600345612, 0.15687870979309082, -0.07406850159168243, 0.21661712229251862, 0.04274922236800194, -0.3193422257900238, -0.018109474331140518, -0.061360765248537064, -0.15996402502059937, -0.017344575375318527, 0.347486674785614, -0.030323900282382965, 0.23495464026927948, -0.08018646389245987, 0.08627621829509735, -0.0243673212826252, 0.20323754847049713, -0.015340025536715984, -0.09874989092350006, 0.09309618175029755, 0.04704323038458824, -0.3743111193180084, -0.2662230134010315, 0.10696026682853699, 0.06469190865755081, 0.07306816428899765, -0.0008399548241868615, -0.11165657639503479, 0.3141360878944397, 0.1469702273607254, -0.0694439634680748, 0.12610191106796265, 0.05679325759410858, 0.2949239909648895, 0.17485609650611877, 0.06400873512029648, 0.22415730357170105, 0.08712781220674515, 0.254280686378479, 0.055383771657943726, -0.07455872744321823, -0.07416292279958725, 0.0922224298119545, -0.1745276302099228, -0.39638563990592957, -0.21870917081832886, 0.0385020449757576, 0.2537694275379181, -0.07964421808719635, -0.016510725021362305, 0.12876105308532715, 0.2682751715183258, 0.06411375105381012, -0.17425981163978577, -0.0009532360127195716, 0.04862690344452858, -0.013158767484128475, -0.16602113842964172, 0.028816673904657364, 0.1994507759809494, 0.19735406339168549, 0.11121015250682831, -0.03439575806260109, 0.14588774740695953, -0.015825750306248665, -0.09081046283245087, -0.014379293657839298, 0.10353715717792511, 0.16942203044891357, 0.12564916908740997, -0.23570901155471802, 0.1452045887708664, -0.01771276444196701, 0.34014081954956055, 0.3169425129890442, 0.04960157349705696, 0.02702774852514267, 0.07102984935045242, -0.036110084503889084, 0.005442522466182709, 0.10536849498748779, -0.327494740486145, 0.05635732784867287, 0.34119632840156555, -0.16446709632873535, 0.042100075632333755, 0.00334819289855659, 0.004281202796846628, -0.01818467490375042, -0.06357181817293167, 0.005620739888399839, -0.3754662871360779, -0.07410942763090134, -0.00020648021018132567, -0.17722606658935547, 0.0745306983590126, -0.15156680345535278, -0.2650531530380249, 0.2904098629951477, -0.030706804245710373, 0.0009030966903083026, -0.11250010132789612, 0.19554515182971954, -0.30469101667404175, 0.1774137020111084, -0.09713906794786453, 0.06114475429058075, -0.00830851774662733, 0.3120349943637848, 0.028277980163693428, 0.1011217013001442, -0.008320536464452744, 0.18560203909873962, -0.27798882126808167, -0.15673837065696716, 0.15320052206516266, -0.03560730069875717, -0.02804603986442089, 0.0035264724865555763, 0.23715059459209442, -0.15620213747024536, -0.2657133936882019, 0.12418139725923538, 0.23116560280323029, -0.23783931136131287, 0.19899173080921173, 0.4062690734863281, 0.17190439999103546, 0.019812706857919693, 0.06010477989912033, -0.06030545383691788, -0.16193172335624695, -0.07169968634843826, 0.03993640094995499, 0.23824110627174377, 0.016903486102819443, -0.033966921269893646, 0.3114124834537506, -0.23177582025527954, 0.06106776371598244, -0.03251669555902481, -0.04280157759785652, -0.05140313506126404}, {-0.008207368664443493, -0.06166381388902664, -0.057927317917346954, 0.14324739575386047, -0.05974108353257179, -0.023051315918564796, -0.24095119535923004, 0.0871887132525444, 0.062419138848781586, 0.39083367586135864, -0.057299088686704636, 0.03285543620586395, -0.04762200638651848, 0.00033431139308959246, -0.03580894321203232, 0.008913084864616394, 0.14653056859970093, -0.10360196232795715, -0.10507660359144211, 0.07718955725431442, -0.19276514649391174, -0.002154047368094325, 0.23123140633106232, 0.029689453542232513, 0.14209440350532532, -0.19693143665790558, 0.03303082287311554, -0.000296690413961187, -0.02011597715318203, -0.1290418803691864, -0.1780085563659668, -0.09712833911180496, 0.2150246649980545, 0.03874698653817177, 0.15789254009723663, 0.19506540894508362, -0.06697634607553482, -0.08287122845649719, -0.10615243762731552, -0.008980573154985905, -0.1335875242948532, 0.012654495425522327, -0.16106396913528442, 0.06577787548303604, 0.18944883346557617, -0.018536938354372978, -0.1645963191986084, 0.1369604766368866, 0.19441069662570953, 0.09196415543556213, 0.15512795746326447, 0.05648164823651314, 0.1264195591211319, -0.0238215159624815, -0.07230227440595627, -0.007985036820173264, -0.024932226166129112, 0.17908324301242828, -0.1447439193725586, -0.03134599328041077, 0.02701246365904808, -0.07620212435722351, -0.16633597016334534, -0.09887606650590897, -0.13155199587345123, -0.04723398759961128, -0.054162152111530304, 0.24610909819602966, -0.1316211074590683, 0.05450734123587608, -0.060657352209091187, -0.021881146356463432, 0.24251513183116913, -0.05750434845685959, -0.11330819129943848, 0.07382873445749283, 0.09311918914318085, -0.04751397669315338, 0.05263616517186165, 0.004025545436888933, 0.07639516144990921, -0.0008539062109775841, -0.052408188581466675, -0.3174988627433777, -0.12679587304592133, -0.035925280302762985, -0.1487511545419693, 0.1148749515414238, -0.06380298733711243, 0.08560516685247421, 0.14996342360973358, -0.07141629606485367, 0.17528407275676727, 0.12128394842147827, -0.09368045628070831, 0.06419859826564789, -0.09010548144578934, -0.06648092716932297, 0.09032710641622543, 0.12495864182710648, -0.2754504382610321, -0.033005498349666595, 0.114439956843853, 0.08742941915988922, 0.11368075758218765, 0.6160605549812317, 0.07967588305473328, 0.007428620010614395, -0.21825848519802094, 0.14010891318321228, -0.022273417562246323, -0.11730361729860306, 0.09755159169435501, -0.022152336314320564, 0.12612418830394745, 0.18469855189323425, 0.05560898408293724, 0.023233314976096153, 0.20975634455680847, -0.06074042245745659, 0.056456416845321655, 0.0029254634864628315, 0.004352007061243057, -0.009020769037306309, 0.035112325102090836, 0.12224192917346954, 0.1414916068315506, 0.18049342930316925, 0.07452698051929474, 0.02609163522720337, 0.14427560567855835, -0.12687845528125763, -0.035406503826379776, 0.14945854246616364, 0.03219849616289139, 0.09467710554599762, -0.05248723924160004, 0.09607674181461334, 0.09321479499340057, -0.0876699686050415, 0.03735228627920151, 0.18447715044021606, 0.040203794836997986, -0.15751083195209503, -0.06713221222162247, -0.035708073526620865, -0.035503968596458435, 0.019071431830525398, -0.009870634414255619, 0.24067676067352295, 0.042570821940898895, 0.032165367156267166, 0.023578980937600136, -0.04514243081212044, 0.06171358749270439, -0.014152427203953266, 0.21955092251300812, 0.062127817422151566, -0.3949255347251892, -0.16627228260040283, -0.21810704469680786, 0.16093984246253967, 0.19462762773036957, -0.04099542275071144, -0.0039704241789877415, 0.035310834646224976, -0.07857237756252289, -0.039624523371458054, 0.034180521965026855, 0.06232871115207672, -0.009698758833110332, -0.009043007157742977, -0.12977288663387299, 0.08215002715587616, -0.21417661011219025, -0.08552179485559464, 0.1767275035381317, 0.06710829585790634, -0.179958775639534, -0.01962292566895485, 0.16249996423721313, -0.33726680278778076, 0.2911154627799988, -0.03666316717863083, 0.05846641957759857, 0.0689307451248169, -0.2112157642841339, 0.018495984375476837, 0.0283497404307127, -0.018545767292380333, 0.12392710894346237, -0.03523614630103111, 0.09404677152633667, 0.09133151918649673, 0.054576560854911804, 0.03536153584718704, -0.1646757274866104, -0.08267910778522491, -0.0318058505654335, 0.18906302750110626, -0.006645291578024626, -0.05801142379641533, -0.033053796738386154, 0.01172206737101078, 0.08883552253246307, -0.10242049396038055, -4.693435403169133e-05, -0.07352432608604431, -0.09675553441047668, -0.15413923561573029, -0.0901905819773674, 0.03631914034485817, 0.04451483488082886, 0.14321574568748474, 0.13169901072978973, 0.11061171442270279, 0.07628606259822845, 0.07652180641889572, 0.07045646756887436, 0.296327143907547, -0.13940675556659698, 0.15474578738212585, 0.01122080534696579, -0.07355575263500214, -0.004664822015911341, -0.04917534813284874, -0.05913539230823517, 0.025526409968733788, -0.06141341105103493, -0.06391450017690659, -0.10931050777435303, 0.025988372042775154, 0.04808580130338669, -0.013550792820751667, 0.07509256154298782, 0.13409371674060822, 0.053677961230278015, -0.03930472955107689, -0.09603946655988693, 0.2669771909713745, -0.11839692294597626, -0.0031127177644521, 0.011888078413903713, 0.12454444915056229, -0.09176472574472427, 0.016084888949990273, 0.18745137751102448, -0.05572844669222832, -0.045053575187921524, -0.08822740614414215, -0.000704167177900672, -0.1239539310336113, 0.24709552526474, -0.11390306800603867, 0.01087136659771204, 0.100955531001091}, {0.0664946511387825, 0.11869151890277863, 0.184720978140831, -0.09761757403612137, -0.21612617373466492, -0.02055009827017784, -0.04232539236545563, -0.09439217299222946, -0.20466269552707672, 0.12143643945455551, -0.14699587225914001, -0.08112648129463196, 0.09835739433765411, -0.0863921195268631, -0.131127268075943, -0.00665117846801877, -0.08314783871173859, 0.10461539775133133, 0.007216715253889561, -0.04065477475523949, -0.16761472821235657, -0.18266573548316956, -0.15867294371128082, -0.01115700975060463, 0.09106361865997314, 0.02910754270851612, -0.1637343317270279, -0.11143118143081665, 0.20625968277454376, 0.12889403104782104, 0.017785005271434784, 0.21952761709690094, -0.3461839556694031, -0.1633845865726471, 0.03738719969987869, 0.10350146144628525, 0.17784279584884644, -0.003702544141560793, -0.07788112759590149, -0.23338203132152557, -0.07108775526285172, -0.021275749430060387, -0.027785958722233772, 0.1317089945077896, 0.013004628010094166, 0.09483974426984787, 0.2535819709300995, 0.21867352724075317, 0.27037230134010315, 0.020445138216018677, 0.22993899881839752, 0.04359987750649452, 0.0600651316344738, -0.16859254240989685, -0.107768714427948, -0.14817805588245392, 0.026137907058000565, 0.10521216690540314, 0.08610004186630249, 0.2003856897354126, -0.10793951153755188, 0.10692010819911957, 0.018188757821917534, -0.005807331297546625, -0.05659562721848488, 0.005291106645017862, -0.03463593125343323, 0.14728417992591858, -0.05262180045247078, 0.05740508809685707, -0.205741286277771, 0.10009952634572983, 0.2611798644065857, 0.09973236173391342, -0.38993144035339355, -0.10445120185613632, 0.1324181854724884, -0.21924136579036713, -0.09736417233943939, -0.06604088097810745, 0.11216283589601517, -0.10621490329504013, 0.10410050302743912, 0.05450337380170822, -0.019663164392113686, 0.22588413953781128, 0.20374096930027008, 0.0028226226568222046, 0.16076907515525818, -0.04203907772898674, -0.03777294605970383, -0.014525474980473518, 0.05222591385245323, -0.07761293649673462, -0.012818816117942333, 0.11689751595258713, -0.04948712885379791, -0.04320289567112923, 0.03165300562977791, 0.15857772529125214, 0.059080515056848526, -0.15547378361225128, -0.044187262654304504, -0.0042648050002753735, -0.1617405265569687, 0.11100374162197113, -0.04873712733387947, 0.31634435057640076, 0.06615719944238663, 0.043150320649147034, -0.05033998563885689, 0.017789354547858238, 0.0776204913854599, -0.16451400518417358, -0.08912858366966248, 0.03089374303817749, 0.10535096377134323, 0.041097648441791534, -0.04472839832305908, -0.011784544214606285, 0.08230827003717422, 0.018763916566967964, 0.17845232784748077, -0.01596442423760891, -0.14979428052902222, -0.19704453647136688, -0.06800719350576401, -0.07073915004730225, -0.023413769900798798, -0.044079456478357315, -0.12421423941850662, -0.07123665511608124, -0.04754304140806198, 0.02293016016483307, 0.287570059299469, 0.06767924875020981, -0.04685503989458084, -0.10846680402755737, -0.10020636767148972, 0.1083640456199646, 0.09427331387996674, -0.0973215401172638, 0.06621933728456497, -0.15514138340950012, -0.04758105427026749, -0.09167946875095367, 0.13944962620735168, 0.136271134018898, 0.026786185801029205, -0.15345199406147003, -0.015620277263224125, 0.14626601338386536, 0.0245505440980196, -0.05409087985754013, -0.13198193907737732, -0.017206940799951553, 0.27045145630836487, -0.2839023172855377, -0.25306475162506104, 0.09890570491552353, -0.04695271700620651, -0.07495952397584915, -0.048290353268384933, 0.0051105390302836895, 0.018907811492681503, -0.05796632915735245, -0.034256190061569214, -0.13768987357616425, -0.07149234414100647, 0.030110470950603485, -0.010194124653935432, -0.19050323963165283, 0.28263580799102783, 0.18377722799777985, -0.056274592876434326, -0.2289978563785553, 0.013378969393670559, 0.06381776183843613, 0.14684171974658966, -0.18853172659873962, 0.21320192515850067, -0.00751319108530879, -0.09043023735284805, 0.41910210251808167, -0.27766621112823486, -0.04871157556772232, -0.30721160769462585, 0.018674584105610847, 0.12307801097631454, -0.08895111829042435, 0.18108712136745453, -0.19996628165245056, 0.22945408523082733, -0.09669061750173569, -0.019725246354937553, 0.43707117438316345, 0.05389312654733658, -0.046654414385557175, 0.12496422976255417, -0.018904205411672592, -0.0941304862499237, -0.0018011071952059865, 0.07972292602062225, 0.031657248735427856, -0.10442932695150375, 0.11973122507333755, 0.011851086281239986, 0.15309162437915802, -0.06145249307155609, -0.07812098413705826, 0.03852437436580658, -0.47544482350349426, 0.04254933446645737, -0.09713688492774963, 0.07799626886844635, 0.11770330369472504, -0.00212469813413918, -0.17551074922084808, 0.014487986452877522, -0.213045135140419, 0.03983684629201889, -0.1002359688282013, 0.010847127996385098, -0.551511287689209, -0.08723223209381104, -0.39379000663757324, 0.15791144967079163, -0.20682939887046814, 0.06127479299902916, -0.1387888491153717, 0.10635004937648773, 0.23430731892585754, -0.0367826446890831, 0.03427669405937195, -0.040679771453142166, 0.20650999248027802, 0.04889732971787453, -0.06807149946689606, 0.205275759100914, 0.05043591931462288, -0.07341472059488297, -0.3658481538295746, 0.11948725581169128, -0.031379517167806625, 0.06401553004980087, 0.01578913815319538, -0.18755340576171875, 0.16020148992538452, -0.07761352509260178, 0.18319016695022583, -0.0484011173248291, 0.1497679501771927, 0.0924977958202362, -0.07754943519830704, 0.14638641476631165, 0.16528639197349548}, {-0.06695327162742615, -0.1018247902393341, -0.040269635617733, 0.14883053302764893, 0.02745552733540535, 0.027210848405957222, -0.042491666972637177, 0.23589272797107697, -0.10696090012788773, -0.025099050253629684, 0.3031194508075714, 0.1166064664721489, -0.005482405424118042, 0.27248361706733704, -0.18129776418209076, -0.06110071763396263, 0.24406147003173828, -0.07059277594089508, -0.20548593997955322, -0.1551453024148941, 0.28052109479904175, -0.01956770569086075, 0.06257224082946777, 0.07228361815214157, 0.030563998967409134, -0.15348957479000092, -0.2185627669095993, -0.23187954723834991, 0.08270284533500671, 0.0625959187746048, 0.08670858293771744, -0.04400673508644104, 0.11131875962018967, -0.11527564376592636, 0.016789961606264114, -0.1824115812778473, 0.1069309189915657, 0.025990819558501244, 0.021539783105254173, 0.08550416678190231, 0.03543294966220856, -0.18939951062202454, -0.000970956461969763, -0.09522770345211029, 0.10876204073429108, 0.05084662511944771, -0.010488161817193031, 0.1170380488038063, 0.015254695899784565, 0.1355566531419754, 0.07214275002479553, 0.05682605877518654, 0.2175268828868866, 0.013453364372253418, 0.023260517045855522, -0.026812473312020302, 0.08669927716255188, 0.02496451511979103, -0.0462610237300396, 0.020944317802786827, -0.09762492775917053, 0.07095211744308472, -0.12694428861141205, -0.10528895258903503, 0.09102551639080048, -0.06523232907056808, -0.1291682869195938, -0.1334390640258789, 0.01940886676311493, -0.04459289088845253, 0.07242508232593536, 0.20843401551246643, -0.06613921374082565, 0.1023322269320488, 0.3069904148578644, -0.10547582060098648, 0.0015752583276480436, 0.04812287166714668, -0.1579713076353073, 0.08816099911928177, -0.15037091076374054, -0.11246921867132187, 0.06889785081148148, -0.09291652590036392, 0.14035235345363617, 0.061860937625169754, -0.11337870359420776, 0.4169723689556122, 0.14028845727443695, 0.10100596398115158, 0.10656940937042236, -0.11732760071754456, 0.06451860070228577, 0.041073527187108994, -0.07891702651977539, 0.02389686182141304, 0.09845112264156342, 0.20818527042865753, 0.15817613899707794, 0.12203603237867355, -0.06121854484081268, -0.1002589613199234, -0.10011166334152222, 0.264922559261322, 0.19739915430545807, 0.2248028665781021, -0.08170211315155029, -0.26931503415107727, 0.12602953612804413, -0.04612426459789276, -0.09101346880197525, 0.04732431098818779, -0.14277420938014984, -0.2359183132648468, 0.10489267855882645, 0.16216705739498138, -0.052797820419073105, -0.16552861034870148, 0.1330593079328537, -0.018500132486224174, 0.1303628385066986, 0.06926145404577255, 0.07445422559976578, 0.11126842349767685, 0.2438596785068512, -0.02730816975235939, -0.06469984352588654, 0.028994102030992508, -0.2025655061006546, -0.2077573835849762, 0.25057321786880493, 0.1332770735025406, -0.20494084060192108, -0.04281457141041756, 0.022133657708764076, 0.25378555059432983, 0.07506294548511505, -0.0986505076289177, -0.21519829332828522, 0.18013334274291992, 0.0039993394166231155, 0.05192466825246811, -0.1369488537311554, -0.061645131558179855, 0.07146410644054413, 0.29734888672828674, -0.17402009665966034, 0.005167583469301462, 0.05885377526283264, -0.20433372259140015, -0.08890482783317566, -0.047220323234796524, 0.03235166519880295, -0.08980114012956619, 0.3034879267215729, -0.11177520453929901, 0.1574530452489853, -0.0343034453690052, -0.10970531404018402, 0.008752348832786083, 0.33906808495521545, -0.4361165165901184, -0.20106233656406403, 0.21163949370384216, 0.2965095639228821, 0.10252958536148071, -0.10821841657161713, 0.018078427761793137, -0.019849147647619247, 0.2677263915538788, -0.29815518856048584, -0.006046769674867392, -0.0769721269607544, -0.2223048061132431, 0.18935243785381317, -0.14309944212436676, -0.08075879514217377, -0.029037633910775185, 0.11602742969989777, -0.13238434493541718, -0.1928880661725998, 0.1364305466413498, 0.053015466779470444, -0.2240115851163864, -0.04693705216050148, -0.22688980400562286, 0.12192853540182114, -0.08364850282669067, -0.03708600625395775, 0.181541308760643, 0.05458346754312515, -0.04863155260682106, -0.19733858108520508, -0.19311681389808655, -0.043303102254867554, 0.018339693546295166, -0.01996733993291855, 0.05335943400859833, 0.13172343373298645, 0.12677039206027985, 0.042733874171972275, -0.3368544280529022, 0.15538182854652405, -0.1490296572446823, -0.12336856126785278, 0.010737589560449123, -0.04036625847220421, 0.10429072380065918, 0.19280551373958588, 0.10912004858255386, -0.07929439097642899, 0.09418591856956482, -0.11772456020116806, -0.06720071285963058, 0.029035193845629692, 0.021772058680653572, 0.15692675113677979, -0.10657605528831482, 0.06376150250434875, -0.012267671525478363, -0.022976670414209366, 0.14769718050956726, 0.004520009271800518, 0.0622905008494854, -0.1585213541984558, 0.15503427386283875, -0.08100035041570663, -0.1855410635471344, 0.2852191627025604, 0.01070227287709713, -0.06778403371572495, -0.10794378817081451, -0.15542905032634735, -0.1516352891921997, -0.00685701472684741, -0.13380813598632812, -0.17562708258628845, 0.15186186134815216, -0.007685933727771044, 0.09803177416324615, 0.10005815327167511, 0.10234706848859787, 0.36482831835746765, 0.09084026515483856, 0.05473880097270012, -0.059244100004434586, 0.056397631764411926, -0.04079552739858627, 0.2843482494354248, 0.07052978128194809, 0.10165223479270935, 0.11820758134126663, -0.06388634443283081, -0.02725713700056076, -0.02954762987792492, -0.16892921924591064}, {-0.1466818004846573, -0.16283006966114044, 0.007557461503893137, 0.051348961889743805, -0.17783088982105255, -0.14185893535614014, -0.0781702995300293, -0.21017885208129883, -0.17360259592533112, -0.044789720326662064, -0.06641498953104019, -0.5453429818153381, 0.05062074586749077, 0.002526927273720503, -0.011408991180360317, 0.11170393228530884, -0.05914502963423729, -0.044963281601667404, -0.37979012727737427, 0.03204196318984032, -0.16574755311012268, 0.015837086364626884, 0.07359886914491653, 0.02474801614880562, 0.30839040875434875, -0.24681392312049866, -0.03207695856690407, -0.11956963688135147, 0.15707573294639587, -0.05046970397233963, -0.22834764420986176, 0.14036987721920013, -0.16299039125442505, -0.06671993434429169, 0.16456091403961182, -0.0039182621985673904, -0.022939438000321388, 0.039744723588228226, 0.025446245446801186, -0.12043791264295578, 0.02310921810567379, -0.0715939924120903, -0.21067596971988678, 0.08646458387374878, 0.10649759322404861, -0.10303561389446259, 0.16147589683532715, 0.07776525616645813, 0.019922073930501938, -0.14835722744464874, 0.02062590792775154, 0.12531712651252747, -0.029183344915509224, -0.1080097183585167, 0.15850017964839935, -0.06042471528053284, -0.1463930457830429, -0.006383479107171297, -0.003235215786844492, -0.024382831528782845, -0.03207908198237419, 0.01601310633122921, 0.02769169770181179, 0.09096281975507736, -0.22477301955223083, -0.2467339187860489, -0.05416795611381531, 0.062121372669935226, 0.07719447463750839, -0.027307219803333282, -0.04446583241224289, 0.09172549098730087, 0.08274977654218674, -0.15914911031723022, 0.15921227633953094, -0.08103854954242706, 0.20105226337909698, 0.07933251559734344, -0.017855407670140266, 0.028264187276363373, -0.00532549200579524, -0.09992599487304688, -0.04060239717364311, -0.045600444078445435, -0.1108873263001442, 0.1316787749528885, -0.10842470079660416, 0.24327071011066437, -0.08417163789272308, 0.001322462921962142, -0.11998362094163895, 0.05468136817216873, 0.06905624270439148, 0.22253011167049408, -0.08919071406126022, -0.07011765241622925, 0.07288610190153122, -0.014749037101864815, 0.09669355303049088, 0.08689308911561966, 0.05014272779226303, -0.06626589596271515, 0.12831227481365204, -0.044718604534864426, -0.37237223982810974, 0.22654922306537628, -0.11120179295539856, 0.044399444013834, 0.06337453424930573, 0.18039602041244507, -0.1349838525056839, -0.12532633543014526, -0.052179332822561264, -0.07936009019613266, 0.040351659059524536, 0.04774659499526024, 0.12710779905319214, 0.11235615611076355, -0.09850341826677322, 0.12343548983335495, 0.04028867185115814, -0.17591677606105804, 0.27332618832588196, 0.060106758028268814, -0.5254411101341248, -0.10570615530014038, -0.04152673855423927, 0.02468465268611908, 0.13712123036384583, 0.014710288494825363, 0.0969824343919754, 0.061748698353767395, -0.008260027505457401, -0.028071966022253036, -0.09812668710947037, 0.2614065408706665, -0.07075277715921402, 0.080301932990551, 0.001984891016036272, 0.33703646063804626, -0.15597480535507202, 0.061878107488155365, -0.05233977362513542, 0.20440568029880524, 0.07999128103256226, 0.009190302342176437, 0.09701282531023026, -0.13261263072490692, 0.6176932454109192, 0.03716621547937393, 0.02572454698383808, -0.061252202838659286, 0.07909504324197769, -0.05223336070775986, -0.015359763987362385, -0.15695597231388092, -0.002430990803986788, -0.09823079407215118, -0.014099786058068275, 0.12626489996910095, 0.0934167429804802, -0.2488671839237213, 0.0018543374026194215, -0.08182363212108612, 0.04372824728488922, 0.12337026000022888, 0.07223735749721527, 0.16694578528404236, 0.2841632068157196, -0.13728268444538116, -0.16487424075603485, -0.004388561472296715, 0.019082115963101387, 0.11535385996103287, -0.09626515209674835, -0.08958754688501358, -0.11647149175405502, 0.06951537728309631, -0.03529205173254013, 0.03282519057393074, 0.06693249195814133, -0.5312461853027344, 0.05996784195303917, -0.01635274663567543, -0.15961283445358276, -0.11669352650642395, 0.17471253871917725, 0.14650306105613708, 0.20587052404880524, 0.029304975643754005, 0.1049070954322815, -0.18155796825885773, 0.04504472762346268, -0.022734839469194412, -0.43043896555900574, 0.08532752841711044, -0.0030227862298488617, -0.08442172408103943, -0.06682509928941727, -0.02882956713438034, -0.06926412135362625, -0.004903027322143316, 0.021463138982653618, -0.08792277425527573, 0.02631049044430256, -0.22894570231437683, -0.055359456688165665, -0.14200735092163086, -0.04310504347085953, -0.23057889938354492, 0.09553820639848709, 0.3168219029903412, -0.003411679994314909, -0.04683135077357292, 0.17508412897586823, 0.12148283421993256, 0.06564320623874664, -0.18577700853347778, 0.673129677772522, -0.23717835545539856, 0.029637912288308144, 0.07232386618852615, 0.11176220327615738, 0.2139291912317276, -0.02191590704023838, 0.14614567160606384, -0.061238426715135574, -0.10449996590614319, -0.05786421522498131, -0.012051433324813843, -0.1007002666592598, -0.06312385201454163, -0.06401178985834122, 0.07400780916213989, 0.07488255947828293, 0.04783666878938675, 0.1973152458667755, -0.022120915353298187, 0.001895917346701026, 0.11404818296432495, 0.05547673627734184, -0.023007411509752274, 0.133172407746315, -0.10373006761074066, 0.011272622272372246, -0.014387380331754684, -0.19157858192920685, -0.10041776299476624, -0.06557897478342056, 0.008209943771362305, -0.06597965210676193, 0.00300324778072536, -0.03040374629199505, 0.07962089031934738, 0.019326061010360718, 0.09347353875637054}, {-0.06036732345819473, 0.07960174977779388, 0.033556532114744186, 0.060876861214637756, -0.07037414610385895, -0.030558904632925987, -0.043623555451631546, 0.02869522012770176, -0.08571293205022812, 0.16656818985939026, 0.011011245660483837, 0.12225442379713058, -0.015855662524700165, -0.014671636745333672, 0.13681438565254211, -0.026011444628238678, 0.010221077129244804, -0.017781931906938553, 0.06067214906215668, -0.10252496600151062, -0.06285746395587921, -0.1335604041814804, 0.022185109555721283, -0.01121643465012312, 0.05512242391705513, -0.14916221797466278, -0.0847196951508522, 0.13369540870189667, -0.029034636914730072, -0.16940510272979736, 0.26679256558418274, -0.01714487187564373, 0.005848548375070095, -0.001467103254981339, -0.04705847054719925, 0.007437088061124086, -0.09167736023664474, -0.23822781443595886, 0.1385822296142578, 0.03178532049059868, -0.04920951649546623, -0.0038429549895226955, 0.05598524585366249, 0.028450002893805504, -0.027046063914895058, 0.01236250251531601, -0.05853846296668053, 0.1016760990023613, -0.0357234850525856, -0.09975984692573547, 0.059330035001039505, -0.11376262456178665, 0.07351560145616531, -0.07637029886245728, -0.046778835356235504, -0.12084991484880447, 0.04400477558374405, -0.07949285954236984, -0.030618317425251007, -0.06331421434879303, 0.015193898230791092, 0.016566934064030647, 0.04244000092148781, -0.07449857145547867, 0.08029355108737946, -0.07950243353843689, 0.15275681018829346, 0.07238059490919113, 0.10067247599363327, 0.09612084925174713, 0.1398434340953827, -0.09719008207321167, -0.05229027196764946, 0.0275541003793478, -0.1555432677268982, 0.09114744514226913, 0.0739041417837143, 0.34196344017982483, 0.01667608879506588, 0.10894493013620377, -0.17791157960891724, -0.05868701636791229, -0.1409427523612976, 0.12926225364208221, 0.04079192876815796, 0.011464616283774376, 0.14284341037273407, -0.12013337016105652, 0.05856986343860626, 0.03930681571364403, 0.06906410306692123, 0.12426561117172241, 0.11797844618558884, 0.024637587368488312, 0.05520179122686386, -0.06324559450149536, -0.050978999584913254, -0.01958329789340496, -0.03681125119328499, 0.07464350759983063, 0.034982938319444656, -0.09879115223884583, -0.12848925590515137, 0.14080119132995605, 0.09295391291379929, 0.04294038191437721, -0.045565079897642136, -0.04475875198841095, 0.16773511469364166, -0.042196307331323624, -0.03945735841989517, -0.035597264766693115, -0.06253430992364883, 0.20183317363262177, 0.16793526709079742, -0.10555611550807953, 0.004163908772170544, 0.042318250983953476, 0.0012571313418447971, -0.007752162404358387, 0.006577723193913698, -0.0745912566781044, -0.026396825909614563, 0.003673360450193286, 0.03513718768954277, -0.28928282856941223, 0.12222738564014435, 0.14425568282604218, 0.10614614933729172, 0.014417216181755066, 0.04029499366879463, 0.12451820820569992, -0.10993164777755737, -0.05161668732762337, -0.0651308223605156, -0.11359914392232895, -0.12572087347507477, -0.16040152311325073, 0.008985346183180809, -0.22191520035266876, -0.0937645435333252, 0.12376248091459274, 0.08511243760585785, 0.18190136551856995, 0.15856967866420746, 0.06152474880218506, 0.0868292897939682, -0.05184321478009224, -0.0919947475194931, 0.03830137848854065, -0.05392112210392952, 0.04742520675063133, 0.031115183606743813, 0.09208466857671738, 0.08105263859033585, -0.031215228140354156, -0.052785489708185196, -0.09484905004501343, 0.06042632833123207, -0.024910151958465576, -0.06666626781225204, -0.07419192045927048, 0.05918607488274574, -0.014646315015852451, -0.007098688744008541, 0.01142654474824667, 0.07291445881128311, -0.18821804225444794, -0.12150761485099792, 0.01719444803893566, -0.016492296010255814, 0.060895778238773346, -0.024921519681811333, -0.15900182723999023, -0.06626427173614502, 0.03615956008434296, 0.10595113784074783, -0.05211782827973366, 0.0414205826818943, -0.03773818165063858, -0.16574519872665405, -0.13031768798828125, -0.13094696402549744, -0.06685143709182739, -0.16986896097660065, -0.1162710189819336, -0.23433247208595276, 0.19068723917007446, 0.06309662014245987, -0.031780969351530075, -0.0041974252089858055, -0.03186425939202309, -0.24433332681655884, -0.10681506246328354, -0.07224872708320618, 0.14704866707324982, 0.17558881640434265, 0.1990591287612915, 0.005966401658952236, -0.031729359179735184, 0.17741048336029053, -0.07052788883447647, 0.021225009113550186, 0.01291887927800417, 0.08605282753705978, -0.144048273563385, 0.03735184296965599, 0.08718053251504898, -0.059997208416461945, -0.010951378382742405, -0.2662181854248047, -0.030310915783047676, -0.04287930205464363, 0.026127060875296593, -0.009144993498921394, 0.01159318070858717, 0.04543248191475868, 0.030617592856287956, -0.04493272677063942, -0.09921932220458984, -0.07556968927383423, -0.11569923907518387, -0.047748204320669174, 0.07612621784210205, -0.005166996270418167, -0.1890273243188858, -0.2573692202568054, -0.07016603648662567, 0.0262793879956007, -0.07398191094398499, 0.012868763878941536, 0.041942257434129715, -0.10692502558231354, 0.0029967408627271652, 0.0952208861708641, -0.1954326629638672, -0.030158095061779022, -0.11281796544790268, 0.10463117808103561, 0.04534219950437546, -0.17277005314826965, -0.05790933966636658, 0.017747720703482628, -0.03470318391919136, 0.11185465008020401, 0.045766204595565796, -0.042320262640714645, 0.16810786724090576, 0.07738563418388367, 0.07554762065410614, -0.050747547298669815, 0.04323997721076012, 0.045969750732183456, -0.10335180908441544, -0.10527930408716202, -0.058443304151296616}, {0.0719769224524498, -0.05976205691695213, 0.15387983620166779, -0.06850689649581909, 0.39514288306236267, 0.06946095079183578, 0.03383472561836243, -0.05713152140378952, 0.15242049098014832, 0.35270053148269653, 0.1852157860994339, 0.011437768116593361, 0.08797094970941544, -0.11727610975503922, -0.0981520414352417, 0.08875628560781479, 0.013801918365061283, -0.13632112741470337, -0.27042585611343384, 0.044147856533527374, 0.09231483936309814, 0.0498298741877079, -0.21817167103290558, 0.08465965837240219, 0.19778604805469513, -0.04388124868273735, -0.816323459148407, -0.19293582439422607, 0.04265681281685829, 0.12437718361616135, 0.051287587732076645, -0.12411065399646759, 0.2622569799423218, -0.11105456948280334, 0.2991260290145874, 0.07213614135980606, -0.09896913915872574, 0.09425670653581619, 0.11685256659984589, 0.27508965134620667, 0.056743234395980835, 0.17698027193546295, -0.08787872642278671, -0.13061030209064484, 0.03297048434615135, -0.10877640545368195, 0.1784159392118454, 0.10039116442203522, -0.06594850867986679, -0.03237591311335564, 0.13507995009422302, -0.14074796438217163, -0.07110188901424408, 0.068219393491745, -0.05311628431081772, 0.04638296365737915, -3.2613366784062237e-05, -0.025674045085906982, -0.016782104969024658, 0.2591862976551056, -0.03780948743224144, -0.1015557199716568, -0.14889784157276154, 0.1728356033563614, -0.2167886197566986, 0.21889792382717133, 0.012213501147925854, -0.3099800944328308, 0.07804591208696365, 0.23013778030872345, -0.10255822539329529, 0.039634544402360916, 0.23516030609607697, 0.14065535366535187, 0.004892220254987478, -0.11832952499389648, 0.05195682868361473, 0.2592720091342926, 0.10899513214826584, 0.17300669848918915, 0.10903903841972351, 0.0020844927057623863, 0.09674087911844254, -0.15077799558639526, -0.05603040009737015, 0.0755939930677414, 0.24794527888298035, 0.11316174268722534, -0.222085639834404, -0.04755546525120735, -0.11525451391935349, 0.06212720274925232, -0.06524234265089035, 0.2005111277103424, -0.046222984790802, 0.1573907434940338, 0.04134576767683029, 0.1153523325920105, -0.022169874981045723, 0.008021721616387367, 0.18106675148010254, -0.04458629712462425, -0.007657768204808235, 0.03082902356982231, 0.2968730926513672, -0.5285996198654175, 0.00898265466094017, 0.02482384815812111, 0.12869074940681458, -0.026408351957798004, 0.21468491852283478, 0.05641411617398262, -0.2444489598274231, 0.15776477754116058, 0.0035876482725143433, 0.24378472566604614, 0.18796822428703308, 0.014478391967713833, -0.09624040126800537, -0.11837320029735565, 0.021362075582146645, 0.2373933494091034, 0.09612442553043365, -0.13525548577308655, -0.14192138612270355, -0.36932116746902466, 0.03803760185837746, 0.1840531975030899, -0.0009069586521945894, -0.38818109035491943, 0.13428926467895508, -0.02179393172264099, -0.2482903003692627, -0.15904337167739868, -0.10318108648061752, 0.15523815155029297, 0.005301687866449356, 0.3423811197280884, 0.12293888628482819, -0.029395990073680878, -0.06711333245038986, 0.10513793677091599, 0.06428182125091553, 0.17016063630580902, 0.22806446254253387, -0.15652337670326233, -0.05659077316522598, 0.012190695852041245, 0.15886566042900085, 0.0008341912180185318, 0.054127246141433716, -0.040264200419187546, -0.19596035778522491, 0.15654772520065308, 0.02086743526160717, 0.153826504945755, 0.2087092250585556, -0.006485678255558014, 0.17243722081184387, 0.04093790799379349, 0.06472143530845642, 0.005973943509161472, -0.06049792096018791, -0.06590624898672104, 0.10448981076478958, -0.09442216902971268, 0.05005786567926407, -0.17579489946365356, -0.27122434973716736, -0.06046794727444649, -0.20558935403823853, -0.10744491219520569, -0.07891310751438141, -0.0741763487458229, 0.06259602308273315, 0.20456147193908691, -0.16022810339927673, -0.03497328236699104, 0.2167346477508545, -0.14842064678668976, 0.053147751837968826, -0.14967332780361176, 0.17603813111782074, -0.13369959592819214, -0.05632985755801201, -0.06494078785181046, -0.023547561839222908, -0.2470860779285431, -0.246821328997612, -0.03354305028915405, -0.15053202211856842, -0.14332862198352814, 0.16435034573078156, 0.014709687791764736, 0.34689006209373474, -0.2512499988079071, -0.15728293359279633, -0.11523313820362091, 0.10458911955356598, -0.04985838010907173, 0.008681223727762699, 0.11607488244771957, -0.18655385076999664, 0.08231256157159805, 0.12784090638160706, -0.21156932413578033, 0.11257416754961014, 0.09003648161888123, 0.32340550422668457, 0.05497698858380318, 0.18180342018604279, 0.16411466896533966, 0.021686729043722153, -0.023865198716521263, -0.2276211977005005, -0.13912895321846008, 0.2194545567035675, -0.06638063490390778, 0.3214789032936096, -0.11537906527519226, 0.1407744437456131, -0.1690450757741928, 0.38512328267097473, 0.19880376756191254, 0.0472789965569973, 0.029541753232479095, 0.18747559189796448, -0.023725537583231926, -0.1246614083647728, 0.09000249207019806, -0.15919512510299683, 0.12657469511032104, -0.08039427548646927, 0.01285147201269865, 0.17282336950302124, -0.13885705173015594, 0.1908644735813141, -0.12271413952112198, -0.051845014095306396, -0.04473019763827324, 0.0610477551817894, -0.031957633793354034, 0.19456812739372253, 0.5421642065048218, -0.06417752802371979, 0.06213291734457016, -0.017686855047941208, -0.054984766989946365, 0.07315167039632797, 0.28282880783081055, -0.1857667714357376, 0.1138702854514122, -0.097145676612854, 0.0924757719039917, 0.0508328340947628, 0.036325592547655106}, {0.011333446018397808, 0.09190761297941208, 0.18454603850841522, -0.039024386554956436, 0.07908548414707184, 0.02182517945766449, 0.11202806234359741, 0.10025482624769211, -0.10003712028265, -0.10018497705459595, 0.27352669835090637, 0.13224346935749054, -0.02616904117166996, -0.07434491813182831, 0.01315836887806654, 0.008566925302147865, -0.10101255029439926, 0.05900968238711357, -0.04362727701663971, 0.13912276923656464, -0.151151642203331, -0.12327628582715988, 0.12082606554031372, -0.3792358338832855, -0.10914042592048645, 0.03088374249637127, 0.15101030468940735, -0.0304974764585495, -0.1703568398952484, 0.02128325216472149, -0.08904706686735153, 0.07853411138057709, 0.1678943783044815, -0.03769834712147713, 0.02580036036670208, 0.11433910578489304, -0.09000205248594284, 0.09663562476634979, 0.004709822125732899, -0.010048758238554, -0.155978724360466, 0.10573409497737885, -0.10742324590682983, -0.16525065898895264, 0.031164202839136124, -0.01254836842417717, 0.022105282172560692, -0.07172449678182602, 0.056243639439344406, -0.3007161617279053, 0.13416269421577454, 0.0069414786994457245, -0.03642437234520912, 0.10936923325061798, -0.04949995130300522, -0.11198248714208603, -0.2111421525478363, 0.036862678825855255, 0.07795652002096176, 0.04324481636285782, -0.15187710523605347, -0.14844085276126862, 0.09661485254764557, -0.1112217903137207, -0.10323171317577362, 0.11589578539133072, -0.17485107481479645, -0.022273559123277664, -0.11132574081420898, 0.03802365064620972, -0.2905479669570923, 0.02903067320585251, 0.030285267159342766, -0.24211645126342773, -0.13700969517230988, -0.12290816754102707, 0.15181587636470795, -0.013307278044521809, 0.19486801326274872, 0.04897744953632355, 0.20165635645389557, 0.07870671898126602, 0.1370609849691391, -0.08049732446670532, -0.1552630066871643, 0.0690976157784462, 0.06456810235977173, -0.12181633710861206, 0.04294651001691818, 0.04576519504189491, -0.04924112558364868, 0.15589532256126404, 0.02719591185450554, 0.06846316158771515, -0.04342282935976982, -0.11827054619789124, 0.009977090172469616, 0.06832305341959, 0.055928975343704224, -0.017155881971120834, 0.04076119512319565, -0.006213328801095486, 0.02462300844490528, -0.11270707100629807, -0.1184992715716362, 0.10744880139827728, 0.08491183072328568, 0.1436430811882019, -0.03814713656902313, 0.10129216313362122, 0.198696106672287, -0.1057979092001915, 0.0958457663655281, -0.10698144882917404, 0.08699905127286911, -0.05449872091412544, -0.3454693853855133, 0.11649882048368454, -0.11963649839162827, 0.0067622424103319645, 0.05569892376661301, 0.07927858084440231, -0.1855175793170929, 0.009093132801353931, 0.0520394928753376, -0.0019525217358022928, -0.09586424380540848, -0.16543512046337128, 0.03634152188897133, -0.02676575817167759, 0.053702645003795624, 0.10207544267177582, 0.060190923511981964, -0.021168019622564316, -0.07813829928636551, 0.0754503607749939, -0.10633939504623413, 0.1660638004541397, 0.18726885318756104, -0.18314358592033386, -0.04291387274861336, -0.21612942218780518, 0.03595566004514694, -0.09610401839017868, 0.06618373095989227, 0.1082109585404396, -0.013067237101495266, -0.18666087090969086, -0.02703622728586197, 0.11791518330574036, -0.17830735445022583, -0.0431443490087986, -0.0625864714384079, 0.2658153176307678, -0.08528906106948853, -0.4317322373390198, -0.09135518968105316, -0.13966874778270721, 0.15690790116786957, 0.17063775658607483, -0.0488748736679554, 0.2032872587442398, 0.016573261469602585, 0.05282065272331238, 0.07459352910518646, 0.1238664984703064, -0.07076647877693176, -0.15084707736968994, -0.11435194313526154, 0.02967912144958973, 0.06255628913640976, 0.057786986231803894, 0.2686540186405182, 0.21519850194454193, 0.07910405844449997, 0.20081351697444916, 0.031999532133340836, 0.024603569880127907, 0.1334441751241684, -0.20513153076171875, 0.11057251691818237, -0.07026945799589157, 0.28216123580932617, 0.005350698251277208, -0.07847197353839874, -0.10215198248624802, -0.06557156145572662, 0.030825385823845863, 0.10861872136592865, 0.10073449462652206, -0.1353989839553833, 0.16678373515605927, 0.29295188188552856, 0.25663667917251587, 0.14765670895576477, -0.08879298716783524, -0.14154992997646332, -0.2341413050889969, -0.10668464004993439, 0.0801333412528038, 0.06932760030031204, 0.012234300374984741, -0.017198728397488594, 0.028492916375398636, -0.0994873121380806, 0.02065797708928585, -0.010913889855146408, 0.11750299483537674, -0.03500001132488251, 0.2245168685913086, 0.11180887371301651, 0.06551388651132584, -0.012522509321570396, 0.2515560984611511, 0.11860163509845734, 0.003827613778412342, -0.08235616981983185, 0.2663593590259552, 0.03544994071125984, 0.024156583473086357, 0.06458629667758942, -0.010872390121221542, -0.059522755444049835, 0.17827969789505005, 0.07444248348474503, 0.04143719747662544, 0.13551808893680573, -0.09369577467441559, 0.015035208314657211, 0.03942335769534111, 0.0178126972168684, -0.18679913878440857, 0.1000506803393364, 0.06524314731359482, 0.021872669458389282, 0.35869571566581726, 0.015517815947532654, 0.14402295649051666, -0.17155134677886963, -0.17234288156032562, -0.010948431678116322, -0.14357134699821472, -0.10554283857345581, -0.05042659863829613, -0.14263035356998444, 0.06338276714086533, -0.026166455820202827, 0.18060699105262756, -0.05676688253879547, -0.09467779099941254, -0.18137991428375244, 0.19729559123516083, -0.07266666740179062, 0.15038372576236725, 0.17750950157642365, 0.10242728888988495}, {0.14218129217624664, 0.24612802267074585, 0.35265934467315674, -0.01431964710354805, 0.33440080285072327, 0.16552837193012238, -0.0026481677778065205, 0.04952079802751541, 0.1595318764448166, 0.181026428937912, -0.1732199788093567, 0.035267893224954605, -0.04673030227422714, 0.043025605380535126, -0.07821604609489441, 0.05647837370634079, 0.06258024275302887, -0.1461712270975113, -0.1072290912270546, 0.08910094946622849, 0.16319072246551514, -0.16273702681064606, 0.11184103041887283, -0.022336868569254875, 0.09303044527769089, 0.09841761738061905, -0.2183791846036911, -0.15999580919742584, 0.031581610441207886, -0.19049546122550964, 0.03668977692723274, -0.12593936920166016, 0.026250416412949562, -0.013097717426717281, -0.12183505296707153, 0.09985391795635223, 0.13230037689208984, -0.0864354819059372, -0.012530541978776455, 0.13203179836273193, -0.09825584292411804, 0.014696133323013783, 0.10875760763883591, -0.0756644532084465, 0.13585609197616577, 0.095889151096344, 0.04517165943980217, -0.04234340041875839, -0.20667719841003418, 0.026109373196959496, 0.06450890004634857, 0.09845300018787384, 0.14118073880672455, -0.11116915196180344, -0.1507616639137268, -0.10964899510145187, 0.02186836302280426, 0.13193783164024353, -0.021113110706210136, 0.13815192878246307, -0.009160930290818214, 0.1943860501050949, 0.016649913042783737, -0.044489454478025436, 0.01447776798158884, 0.1793242245912552, 0.12604296207427979, -0.05145704373717308, 0.04297258332371712, -0.12463602423667908, -0.21598513424396515, -0.037301771342754364, 0.16476528346538544, 0.19523490965366364, -0.018422024324536324, -0.19612018764019012, 0.05807012692093849, 0.06238340213894844, -0.138019397854805, 0.025441119447350502, 0.10174206644296646, -0.09708931297063828, -0.006311185657978058, -0.164291650056839, 0.2123042196035385, 0.2416503131389618, 0.115489661693573, 0.030191048979759216, -0.11689998209476471, 0.02690500020980835, -0.051790524274110794, 0.09774981439113617, 0.030353739857673645, -0.03590921685099602, -0.29468387365341187, -0.0935937687754631, 0.05880308896303177, -0.05349667742848396, -0.11950203776359558, 0.0989602655172348, 0.10152899473905563, -0.06384554505348206, 0.05119621753692627, -0.07476967573165894, 0.006284985225647688, 0.0765082985162735, -0.20658981800079346, -0.12325239181518555, -0.054202351719141006, 0.10456143319606781, -0.09822455793619156, -0.31067535281181335, -0.0029435642063617706, -0.10045413672924042, 0.13955140113830566, 0.09510932862758636, 0.004010408651083708, -0.19916516542434692, -0.07546770572662354, -0.2760902941226959, -0.08047962188720703, -0.14948149025440216, 0.015697620809078217, -0.07750456780195236, 0.002013611374422908, -0.06353261321783066, 0.13051433861255646, 0.1166544258594513, 0.15696439146995544, -0.09694977104663849, -0.05714483559131622, -0.07120176404714584, -0.14152179658412933, -0.08529017120599747, -0.21770921349525452, 0.29781433939933777, 0.08706842362880707, -0.08632349967956543, 0.2520888149738312, -0.12373646348714828, 0.06020423024892807, -0.17094342410564423, 0.30592772364616394, 0.021115422248840332, -0.06402269750833511, 0.1270395964384079, -0.31821832060813904, -0.055303316563367844, 0.026930535212159157, -0.08251897245645523, 0.19884365797042847, -0.06374584883451462, 0.09979322552680969, -0.1845262199640274, 0.1181146577000618, -0.043372731655836105, 0.23298822343349457, -0.20003977417945862, -0.11203347891569138, -0.04586150124669075, -0.12231885641813278, -0.20849773287773132, -0.12067225575447083, 0.1608554869890213, -0.0020305640064179897, 0.09635728597640991, -0.21981631219387054, -0.0884789228439331, 0.04612022638320923, 0.0020287565421313047, -0.02275571972131729, -0.20965467393398285, 0.15032987296581268, 0.05882952734827995, 0.1326267123222351, -0.15359702706336975, -0.1416856348514557, 0.10715842247009277, -0.15108264982700348, -0.11453255265951157, -0.0032458435744047165, -0.052420735359191895, -0.022357964888215065, -0.05431927740573883, -0.08509054780006409, -0.047839269042015076, -0.04203416407108307, 0.063040591776371, -0.1288122683763504, 0.09838523715734482, -0.07781872898340225, 0.07053593546152115, 0.035278674215078354, 0.12260589003562927, 0.5158777832984924, 0.11401422321796417, -0.07573902606964111, -0.25852900743484497, 0.06414034962654114, -0.06354093551635742, -0.12943211197853088, -0.1640174835920334, 0.08315223455429077, 0.015155020169913769, 0.052087415009737015, -0.13975241780281067, -0.15791462361812592, -0.04074469208717346, 0.14400415122509003, 0.07250124961137772, -0.0681050568819046, -0.1507723182439804, -0.061356786638498306, -0.015781881287693977, -0.05826224014163017, -0.024460289627313614, -0.031241634860634804, -0.0680120661854744, 0.1198679730296135, -0.10443494468927383, -0.024050751700997353, 0.02196827158331871, -0.2949522137641907, 0.0540812723338604, 0.04767895117402077, -0.10125467926263809, -0.13891834020614624, 0.07419946789741516, -0.11189940571784973, 0.02610628679394722, -0.22981733083724976, 0.15484261512756348, 0.16532590985298157, 0.08296526968479156, -0.11954020708799362, -0.10048916935920715, 0.06351635605096817, -0.13100074231624603, -0.285025417804718, 0.03377639502286911, 0.07430080324411392, 0.011646907776594162, 0.2850390374660492, 0.02485210821032524, 0.10540781915187836, 0.13883334398269653, -0.07544980198144913, 0.1546018421649933, -0.007198095787316561, 0.04232441633939743, -0.1303589642047882, 0.04908951744437218, -0.012417525984346867, -0.042718805372714996, 0.1240011602640152, -0.03437735140323639}, {-0.14205972850322723, 0.02898012101650238, -0.11351287364959717, 0.05684162676334381, -0.0715692862868309, 0.036559443920850754, -0.03323718532919884, -0.0541553795337677, -0.14834094047546387, -0.19306109845638275, -0.18723011016845703, 0.10317957401275635, -0.051347989588975906, -0.022630972787737846, 0.006581776309758425, 0.04146110638976097, 0.033259034156799316, 0.31537050008773804, 0.10085039585828781, -0.15056514739990234, -0.02872944436967373, -0.13662442564964294, 0.03696290776133537, 0.11625262349843979, 0.13169927895069122, -0.13148382306098938, 0.11345211416482925, -0.040710557252168655, 0.14459560811519623, -0.12984563410282135, -0.09710902720689774, 0.037159450352191925, -0.081885926425457, 0.09239300340414047, -0.13236725330352783, -0.09381705522537231, -0.07685650140047073, -0.18405498564243317, 0.14081211388111115, 0.040486954152584076, -0.07071635872125626, -0.0863574668765068, 0.11887792497873306, -0.010905624367296696, -0.16657668352127075, -0.07206545025110245, 0.017719846218824387, -0.07031606137752533, 0.011799170635640621, 0.0400104857981205, -0.13412602245807648, 0.062237698584795, -0.11397845298051834, 0.07898002117872238, -0.161195307970047, 0.07503288984298706, 0.13261602818965912, 0.11647181957960129, -0.016155948862433434, 0.08926036208868027, -0.04948778077960014, -0.08205001801252365, -0.18050561845302582, -0.07253599166870117, 0.05449316278100014, 0.1111087054014206, -0.008030491881072521, -0.024908043444156647, 0.2343556135892868, 0.1263941526412964, -0.00795240793377161, -0.19147619605064392, 0.0642266795039177, 0.017023557797074318, -0.03856344521045685, -0.018825402483344078, -0.14350461959838867, -0.018122728914022446, 0.20373529195785522, 0.03421979025006294, -0.00021458696573972702, -0.0016510359710082412, 0.2106003761291504, 0.02884492464363575, -0.007858299650251865, -0.009944078512489796, 0.267435759305954, 0.07539209723472595, -0.05502527579665184, 0.22622154653072357, -0.011311143636703491, 0.02465161308646202, -0.05393572151660919, 0.005289270076900721, 0.09721880406141281, 0.008905284106731415, -0.004966391250491142, 0.0030277979094535112, -0.08368953317403793, 0.07333965599536896, 0.025165624916553497, -0.12107151001691818, 0.05590950325131416, -0.03848034888505936, -0.01456731092184782, -0.11158487200737, -0.14821775257587433, 0.09184017777442932, -0.06296142935752869, 0.06938352435827255, 0.05851155146956444, -0.06978017091751099, 0.06714998185634613, 0.06135082244873047, 0.015462592244148254, 0.0214694831520319, 0.19528289139270782, 0.013639223761856556, 0.08206932246685028, -0.09904388338327408, 0.06178213655948639, 0.1364865005016327, 0.06571385264396667, 0.22276578843593597, 0.3817439377307892, 0.04339749738574028, 0.12300343811511993, -0.03011760115623474, 0.02602660097181797, 0.13263632357120514, 0.08460282534360886, 0.010157360695302486, -0.1063387468457222, -0.011258875019848347, 0.09918399155139923, -0.1335029900074005, -0.04398460313677788, -0.006151605863124132, 0.13751205801963806, -0.02103355899453163, -0.07765312492847443, 0.07181249558925629, -0.00036553715472109616, -0.1097489520907402, -0.1132582277059555, -0.011122439987957478, 0.17078030109405518, 0.06987916678190231, -0.31825119256973267, -0.09697826951742172, 0.040619730949401855, 0.061151765286922455, 0.18478459119796753, 0.0007010489935055375, -0.20087505877017975, 0.15456996858119965, 0.048852380365133286, -0.2436414211988449, 0.14117757976055145, 0.2072446197271347, -0.1285744309425354, 0.1730583906173706, 0.04448645934462547, -0.10858507454395294, -0.1438266485929489, -0.06705811619758606, 0.02396363578736782, -0.041936323046684265, -0.09128478169441223, 0.01166543923318386, 0.1522476226091385, -0.12232774496078491, -0.015498276799917221, 0.11643010377883911, 0.04452149569988251, -0.0927920714020729, -0.020084043964743614, -0.01388082467019558, 0.1429772526025772, -0.004695709329098463, -0.007601399905979633, 0.050906430929899216, -0.02987593412399292, 0.2653021812438965, -0.15736840665340424, 0.15188992023468018, -0.47527313232421875, 0.08299372345209122, 0.08768496662378311, 0.03330099210143089, -0.20375986397266388, -0.28406134247779846, -0.013085303828120232, 0.1127362996339798, 0.14790599048137665, 0.16186264157295227, -0.08849532157182693, 0.10063104331493378, 0.033949099481105804, 0.05757015198469162, 0.14997847378253937, -0.08965371549129486, -0.025449739769101143, -0.09078626334667206, 0.033287227153778076, 0.07078737765550613, -0.05817246064543724, 0.0636165589094162, -0.033893782645463943, -0.08596231788396835, -0.03263161703944206, -0.3796301484107971, 0.10509982705116272, -0.2823520302772522, -0.13985726237297058, -0.023346347734332085, 0.05906644091010094, 0.049964357167482376, -0.11903386563062668, -0.03514396771788597, 0.1297721266746521, -0.05863790214061737, -0.1884208619594574, -0.40564846992492676, -0.14312438666820526, -0.3323623239994049, 0.012029347941279411, 0.006127397995442152, 0.0831514373421669, 0.12203744053840637, -0.016254564747214317, 0.024920495226979256, -0.15999294817447662, -0.007700036279857159, 0.036041487008333206, 0.07843229919672012, 0.18032436072826385, -0.011494975537061691, -0.0824359878897667, -0.01170213520526886, -0.23211465775966644, 0.23873841762542725, 0.03412777930498123, -0.14031104743480682, -0.024854805320501328, 0.038117390125989914, 0.009235081262886524, 0.07961156219244003, 0.12468520551919937, -0.16773433983325958, 0.14406275749206543, 0.20181719958782196, 0.09300901740789413, 0.0422433540225029, 0.1614416539669037, -0.13248340785503387}, {-0.09182772785425186, -0.24672344326972961, 0.16896672546863556, 0.22632087767124176, 0.15094037353992462, 0.003779061371460557, -0.32078078389167786, 0.06733835488557816, 0.057130273431539536, -0.08535635471343994, 0.024657439440488815, 0.04639964550733566, 0.05776442959904671, -0.07198631763458252, -0.04027654230594635, 0.18159501254558563, -0.16920965909957886, 0.03332313150167465, -0.022994326427578926, 0.02639327198266983, -0.09173167496919632, 0.1455676406621933, 0.04778749868273735, -0.16089507937431335, 0.32313206791877747, -0.05568572133779526, -0.2535380721092224, -0.1490405648946762, 0.048336584120988846, 0.048304229974746704, 0.01256704330444336, -0.04206144064664841, 0.1365077644586563, -0.1334923803806305, -0.02197444625198841, 0.21187061071395874, -0.09278848767280579, 0.07104329019784927, 0.07193513959646225, 0.21591193974018097, -0.017785105854272842, 0.24813216924667358, -0.256746768951416, 0.027741875499486923, 0.08438127487897873, 0.1697886735200882, 0.2031663954257965, 0.13111193478107452, -0.18968217074871063, 0.05249723047018051, 0.03565922752022743, -0.1790737807750702, -0.16596229374408722, -0.06736740469932556, 0.0007748760399408638, 0.06682411581277847, -0.1304786205291748, -0.20845599472522736, 0.2846444547176361, 0.4310321807861328, -0.07177631556987762, 0.15336686372756958, 0.05105387046933174, 0.2819124460220337, -0.12287571281194687, 0.07988975942134857, -0.33461108803749084, -0.10297206044197083, -0.12371993064880371, -0.03407961502671242, -0.04966696724295616, 0.13498443365097046, -0.05304807052016258, 0.025303902104496956, -0.04226883500814438, -0.29612764716148376, 0.10322583466768265, -0.17326395213603973, 0.023877669125795364, 0.16966407001018524, 0.049972061067819595, 0.04930650815367699, 0.09632161259651184, -0.13071158528327942, -0.05381232127547264, 0.0723649114370346, -0.20366157591342926, -0.024791767820715904, -0.14543753862380981, -0.00951401423662901, -0.01927599497139454, -0.23262187838554382, 0.19661296904087067, 0.165218323469162, 0.07605289667844772, -0.08095505833625793, 0.10182410478591919, 0.1687324345111847, -0.05805165693163872, 0.1313449591398239, -0.22212709486484528, -0.10280539840459824, -0.09189212322235107, -0.15961530804634094, 0.13955195248126984, 0.09102918952703476, 0.140264630317688, 0.14520996809005737, -0.12691305577754974, -0.09765152633190155, 0.010150659829378128, 0.2345006763935089, 0.05651550740003586, -0.02477145940065384, -0.03625597432255745, -0.2272878885269165, -0.024440480396151543, 0.06613615900278091, 0.0035241590812802315, -0.05794143304228783, 0.013496676459908485, 0.18272294104099274, 0.22571974992752075, -0.2779248058795929, -0.10674408078193665, -0.559967577457428, 0.055682405829429626, 0.0018598386086523533, -0.07381219416856766, -0.14114052057266235, -0.069758340716362, 0.19553066790103912, -0.11078133434057236, 0.012409052811563015, 0.014986852183938026, 0.19781838357448578, 0.24689382314682007, -0.1680871993303299, -0.25008320808410645, 0.12326798588037491, 0.057911261916160583, -0.1966974139213562, -0.25971993803977966, -0.043747883290052414, -0.10887391120195389, -0.07318130135536194, -0.09305724501609802, -0.04013390094041824, 0.0871577039361, 0.1639890819787979, -0.08538489043712616, -0.25481662154197693, -0.16867049038410187, -0.017930714413523674, 0.0072122132405638695, 0.08496333658695221, 0.15074485540390015, -0.00012946395145263523, -0.17595621943473816, -0.12687474489212036, 0.0705074593424797, 0.05496559292078018, 0.046334315091371536, -0.10803136974573135, -0.35417211055755615, -0.008304623886942863, 0.06629281491041183, -0.18088628351688385, -0.01951989345252514, -0.27090978622436523, -0.046051375567913055, -0.03642192855477333, 0.0389571376144886, 0.215809628367424, 0.04501025378704071, 0.21515578031539917, -0.1420120894908905, 0.15770331025123596, -0.022363295778632164, 0.01032306719571352, -0.007574195042252541, -0.09605937451124191, -0.07556705176830292, -0.22938239574432373, -0.016814537346363068, 0.030382154509425163, 0.14845286309719086, -0.2873038947582245, -0.18833231925964355, -0.02108178287744522, 0.0910429134964943, 0.20076724886894226, -0.02654454857110977, 0.23926857113838196, 0.26782238483428955, 0.05910370126366615, -0.20755089819431305, 0.18376870453357697, -0.04662837088108063, -0.2505587041378021, 0.022298941388726234, -0.03843320906162262, -0.03743312135338783, 0.20402030646800995, 0.10601045191287994, 0.2097836285829544, 0.27486294507980347, 0.21123377978801727, 0.13382400572299957, -0.24183876812458038, 0.06221725046634674, 0.20379023253917694, 0.05837331339716911, 0.0903858169913292, 0.08119172602891922, 0.09960418939590454, 0.18371781706809998, -0.04213469848036766, -0.0631524920463562, -0.2575210630893707, -0.04113738611340523, 0.04199685901403427, 0.06367882341146469, 0.24840056896209717, 0.08289696276187897, 0.02943188138306141, 0.08173305541276932, 0.0729641392827034, -0.03063046559691429, -0.08049126714468002, -0.10918937623500824, -0.041565362364053726, -0.07072458416223526, 0.22210080921649933, 0.11305170506238937, -0.09897767007350922, 0.0252398494631052, 0.127596914768219, 0.16364268958568573, -0.00978104118257761, 0.05988394841551781, -0.06932336091995239, 0.06100974231958389, 0.01495180744677782, 0.12763164937496185, 0.13652049005031586, -0.16259777545928955, -0.08761224895715714, -0.20918163657188416, 0.347278892993927, 0.014428926631808281, -0.2551766335964203, 0.048473674803972244, -0.024570448324084282, 0.26715853810310364, 0.08222106844186783}, {0.04559558629989624, 0.018290311098098755, 0.19449859857559204, -0.021088967099785805, 0.28983286023139954, 0.1750420331954956, -0.13570161163806915, 0.05464813485741615, 0.18194645643234253, -0.02092551626265049, 0.0007929244311526418, 0.19652990996837616, -0.09414765983819962, -0.024441275745630264, -0.12241397798061371, -0.11547017842531204, -0.17158864438533783, -0.021717406809329987, -0.01842443272471428, 0.09017843753099442, -0.09674076735973358, -0.024957915768027306, 0.010433875024318695, -0.042032308876514435, -0.2131197303533554, -0.02371937781572342, -0.034095536917448044, -0.23907558619976044, 0.09110365808010101, 0.016575928777456284, -0.2079259753227234, -0.14767715334892273, 0.05835742503404617, -0.026626896113157272, 0.11055372655391693, -0.023134341463446617, -0.12122546881437302, 0.11785971373319626, -0.17836159467697144, 0.25451648235321045, -0.13636185228824615, 0.003993656486272812, 0.068027563393116, -0.2944047749042511, -0.051307640969753265, 0.05271119251847267, -0.07371395826339722, 0.1515403538942337, -0.10799559950828552, -0.056134145706892014, 0.12206436693668365, -0.08547501266002655, -0.04602902755141258, 0.10649988800287247, 0.1369781643152237, -0.032035402953624725, -0.03737974911928177, 0.10047237575054169, 0.13344474136829376, 0.01475668977946043, 0.00937036331743002, 0.06004808098077774, 0.09572755545377731, -0.04753865674138069, 0.13928207755088806, 0.08868066221475601, -0.025607215240597725, 0.0863313227891922, -0.2197331190109253, 0.06698168069124222, 0.043706487864255905, 0.13138847053050995, 0.13186995685100555, 0.14113640785217285, -0.13820824027061462, -0.08706130087375641, 0.2002703845500946, -0.20000790059566498, 0.2154403179883957, -0.012051889672875404, 0.08770350366830826, 0.1797899752855301, -0.13151969015598297, -0.19444620609283447, 0.07975180447101593, 0.06053577736020088, 0.031215442344546318, 0.15324382483959198, -0.23095092177391052, -0.00921518076211214, -0.04845711216330528, 0.07815860956907272, 0.1387758105993271, 0.11681688576936722, -0.014224948361515999, -0.19030560553073883, 0.0028687871526926756, 0.02173108421266079, 0.05650273710489273, -0.1447976529598236, -0.17339269816875458, 0.10182778537273407, -0.005397578235715628, -0.20629435777664185, 0.040283989161252975, 0.07636051625013351, 0.038371678441762924, 0.04441561922430992, 0.1035158708691597, -0.14268289506435394, 0.2606179416179657, 0.08837281167507172, 0.22789765894412994, -0.0867781713604927, 0.032273780554533005, -0.16300977766513824, 0.11807216703891754, -0.17421098053455353, -0.008695034310221672, -0.06712961196899414, 0.03721180185675621, 0.20071163773536682, -0.13524538278579712, -0.4575311243534088, 0.13988900184631348, 0.009700753726065159, 0.08274947106838226, -0.10013537108898163, 0.13343489170074463, -0.19290995597839355, 0.008876731619238853, 0.1743641048669815, 0.05415970832109451, -0.03500943258404732, 0.013800266198813915, -0.22915400564670563, 0.005090727936476469, 0.10816246271133423, -0.12940895557403564, -0.1204296350479126, 0.12643831968307495, -0.2071133255958557, -0.06348028779029846, 0.2606300413608551, -0.2996399402618408, -0.09577004611492157, -0.12267506122589111, -0.014666890725493431, -0.17198657989501953, 0.014184284023940563, 0.04827988147735596, -0.13464847207069397, -0.00944429263472557, -0.010785041376948357, 0.10837970674037933, -0.035988952964544296, -0.01362853217869997, -0.007284867577254772, -0.044553156942129135, -0.1630759835243225, -0.20430417358875275, 0.0979698896408081, 0.21941274404525757, 0.00024188379757106304, -0.0960945338010788, -0.1361508071422577, -0.0011226583737879992, -0.3281269371509552, 0.05663185566663742, -0.02708139456808567, 0.09908807277679443, 0.0007004590006545186, -0.05067938566207886, 0.027824338525533676, 0.01818802021443844, -0.06354790925979614, 0.04098643362522125, 0.002000405453145504, -0.24701571464538574, 0.07140672206878662, 0.006543280091136694, 0.030651403591036797, -0.023204028606414795, -0.13150528073310852, 0.08120885491371155, 0.0699119046330452, -0.037335000932216644, -0.10470163077116013, 0.08340717852115631, 0.024947771802544594, 0.11377131938934326, 0.014449033886194229, -0.13457328081130981, 0.15465880930423737, -0.11309468746185303, -0.2899249792098999, 0.25949862599372864, 0.0788835659623146, 0.02885681949555874, -0.09236326813697815, 0.16469745337963104, 0.046714507043361664, 0.07909262925386429, 0.0663045272231102, 0.10992765426635742, -0.18247424066066742, 0.06711149215698242, 0.060817692428827286, 0.2007613480091095, 0.1160019189119339, 0.35759976506233215, 0.04773618280887604, -0.1230522021651268, -0.06153147295117378, -0.04394721984863281, 0.12166924774646759, 0.08024746924638748, -0.17583268880844116, 0.0022967916447669268, 0.0026032146997749805, -0.06826218217611313, -0.09065631777048111, 0.046428292989730835, 0.07905972748994827, 0.15648213028907776, 0.04105892777442932, 0.11624971032142639, 0.17152196168899536, -0.14982913434505463, -0.17185288667678833, 0.03216329962015152, -0.04941267892718315, 0.000870819203555584, 0.0582231730222702, -0.03529532253742218, -0.05518348515033722, 0.009551937691867352, 0.1190829649567604, -0.23588941991329193, -0.02959771826863289, -0.03780336678028107, 0.04100029543042183, 0.08209707587957382, -0.13701802492141724, 0.06389153748750687, 0.1255982518196106, 0.07770981639623642, -0.11334224045276642, 0.05418730154633522, 0.2520783245563507, 0.029481956735253334, -0.006849946454167366, 0.0322323739528656, 0.16482841968536377, 0.06956068426370621, 0.09282069653272629}, {0.21481391787528992, -0.051556311547756195, -0.08333209156990051, 0.1990792453289032, 0.2642863988876343, -0.3700709640979767, -0.255843847990036, -0.13006535172462463, -0.2302393913269043, -0.2935532331466675, 0.187169149518013, -0.1326853632926941, -0.06250926852226257, -0.06774017214775085, 0.10926898568868637, -0.10135041177272797, -0.09679754823446274, -0.03384501859545708, -0.07359662652015686, -0.2417077273130417, -0.08297546952962875, 0.11516118049621582, 0.04251733049750328, -0.1513909548521042, -0.0630941167473793, -0.01499025896191597, 0.21726395189762115, -0.07233375310897827, 0.03707650303840637, 0.21407288312911987, 0.2014491707086563, 0.007383509073406458, -0.17455703020095825, -0.2395326942205429, -0.02971014566719532, -0.17477543652057648, 0.15375854074954987, 0.014348458498716354, -0.007420353125780821, 0.0428004264831543, -0.030644919723272324, 0.15921220183372498, 0.2471190243959427, 0.32111674547195435, 0.05166645348072052, 0.4031049907207489, -0.027313783764839172, -0.16612696647644043, 0.1885351836681366, -0.1136825904250145, -0.03858465328812599, -0.10283990204334259, 0.04141669347882271, -0.24027396738529205, 0.09050384908914566, 0.1488284468650818, -0.18728972971439362, -0.13421602547168732, 0.10992693156003952, -0.3306032717227936, 0.08529983460903168, 0.1658775508403778, -0.10105451196432114, -0.08497900515794754, -0.06445155292749405, -0.02089131996035576, -0.014070423319935799, -0.1639394611120224, -0.1330360323190689, 0.12009235471487045, 0.2217641919851303, 0.1579912006855011, 0.04147830978035927, 0.1215035691857338, -0.08176317811012268, 0.04563654959201813, 0.0961725115776062, 0.04181038588285446, 0.007606001105159521, -0.0325617790222168, 0.1212594211101532, 0.2398311346769333, 0.05820198357105255, 0.05334708094596863, 0.2277422845363617, 0.09998779743909836, 0.06888320297002792, 0.1508193165063858, 0.004458788316696882, 0.030165933072566986, -0.06568250805139542, -0.0895826444029808, -0.10863788425922394, -0.03408721834421158, -0.10755568742752075, -0.25891101360321045, -0.06780936568975449, -0.049865324050188065, -0.1384918987751007, 0.01990766078233719, 0.17497143149375916, -0.1686888337135315, 0.21824133396148682, -0.09317632764577866, -0.06186945363879204, -0.25058138370513916, -0.012222702614963055, 0.20327337086200714, -0.1948087215423584, 0.03654288873076439, 0.1186661571264267, -0.24133194983005524, 0.16069167852401733, 0.023536594584584236, -0.07716540992259979, -0.20725654065608978, -0.0559709258377552, -0.09970004856586456, 0.05590088665485382, -0.00043249447480775416, 0.035611361265182495, -0.13643878698349, -0.10785872489213943, -0.09026052802801132, 0.12603642046451569, -0.0024368881713598967, -0.0884610265493393, -0.06921239197254181, 0.16599738597869873, 0.12182392925024033, -0.15618325769901276, -0.07861514389514923, -0.20120064914226532, -0.08571578562259674, 0.015858834609389305, -0.03773593530058861, -0.03354422748088837, 0.10141516476869583, 0.0024564401246607304, 0.20005014538764954, 0.07863246649503708, -0.1249508485198021, 0.021306898444890976, -0.09761904925107956, -0.259107768535614, -0.0271807461977005, -0.16281387209892273, 0.1802816540002823, -0.10509224981069565, -0.1838802546262741, -0.13451041281223297, -0.09777309745550156, -0.12906929850578308, -0.15978723764419556, -0.08439513295888901, -0.09706854075193405, -0.17327651381492615, -0.004273832775652409, -0.3076123297214508, 0.08392918109893799, 0.07890792936086655, 0.025631042197346687, 0.12441346794366837, -0.00014785892562940717, -0.1905619204044342, 0.2935238480567932, -0.3463863134384155, -0.22059017419815063, -0.06677426397800446, -0.06712454557418823, -0.04453567415475845, 0.09557326883077621, 0.0023966662120074034, -0.19337905943393707, -0.02266262099146843, -0.0012374018551781774, 0.15349894762039185, -0.24625536799430847, 0.020198382437229156, -0.0350131094455719, -0.09813842177391052, -0.17167812585830688, 0.00940883718430996, 0.168065145611763, -0.10184205323457718, -0.01926303282380104, -0.14290723204612732, -0.001996956067159772, -0.0036822299007326365, -0.2881188690662384, 0.15966112911701202, 0.09457409381866455, 0.19776295125484467, 0.24568937718868256, 0.20047038793563843, -0.009537409991025925, -0.05156302452087402, -0.21733936667442322, -0.1759037971496582, -0.18034960329532623, 0.15880794823169708, -0.04072942957282066, 0.06077561154961586, 0.054979197680950165, -0.10832412540912628, 0.12016095221042633, 0.06758473813533783, 0.07844983041286469, -0.1992943286895752, 0.15455842018127441, -0.1502077728509903, -0.12440858781337738, -0.11607994139194489, -0.15412260591983795, 0.020182335749268532, 0.17285498976707458, -0.00825192779302597, 0.17452766001224518, -0.16024720668792725, -0.00440157251432538, -0.07079880684614182, -0.1554946005344391, 0.10510489344596863, -0.11614864319562912, -0.25649821758270264, -0.2178417444229126, 0.1939520686864853, 0.05551895499229431, -0.04402807354927063, 0.016259362921118736, 0.12314962595701218, 0.03784332796931267, -0.1917935162782669, -0.16657227277755737, -0.0033864511642605066, 0.012475302442908287, -0.058243632316589355, -0.021513210609555244, 0.04199656844139099, 0.11091601103544235, -0.2229883372783661, -0.11290214210748672, 0.09755680710077286, -0.1001972183585167, 0.03169781714677811, 0.06796485930681229, 0.07500360906124115, 0.10664588958024979, -0.07772589474916458, 0.18316853046417236, 0.03467176854610443, -0.26436078548431396, 0.09572594612836838, 0.1753581166267395, 0.18338461220264435, -0.2417302131652832}, {-0.2173563539981842, -0.1562742292881012, 0.07604938745498657, -0.11409106850624084, 0.030516721308231354, 0.16897821426391602, -0.01374316867440939, 0.1167813390493393, -0.07558931410312653, 0.03321709856390953, -0.12831011414527893, -0.2394881397485733, -0.007858330383896828, -0.06936168670654297, -0.09032635390758514, 0.030453233048319817, -0.007405779790133238, 0.18577906489372253, -0.052773717790842056, 0.24083855748176575, -0.13898639380931854, -0.058031126856803894, 0.07308142632246017, 0.08148510754108429, 0.05838524177670479, 0.2895517647266388, -0.11413019150495529, -0.0032959452364593744, 0.1336570829153061, -0.08728146553039551, -0.14067965745925903, 0.1201254352927208, 0.11978240311145782, 0.2581022083759308, -0.01435157936066389, -0.10348982363939285, -0.19669194519519806, -0.12695704400539398, 0.06110014393925667, 0.008323726244270802, -0.004925064742565155, -0.16199606657028198, -0.07234904915094376, -0.19731508195400238, 0.03104078583419323, -0.03422611206769943, 0.3317817151546478, -0.010804440826177597, 0.022850150242447853, 0.04585469141602516, 0.1602809876203537, 0.1163121685385704, -0.03932194411754608, -0.0031093773432075977, 2.461597068759147e-05, -0.1273396760225296, 0.07755997776985168, -0.12577398121356964, 0.16697098314762115, 0.01768796145915985, 0.055976688861846924, 0.24847275018692017, -0.05901055783033371, 0.1358620822429657, -0.0191611647605896, 0.00025730906054377556, 0.06037793681025505, -0.07104555517435074, 0.10988857597112656, -0.08837518841028214, -0.09322850406169891, 0.11019754409790039, 0.06863696128129959, 0.2562088072299957, -0.11540842801332474, -0.1529683768749237, 0.10855696350336075, 0.07747630029916763, -0.18850021064281464, 0.20307305455207825, 0.017438936978578568, -0.07821034640073776, -0.027620943263173103, 0.09438728541135788, -0.10075905919075012, 0.16419441998004913, 0.12742163240909576, 0.1290295422077179, -0.05566622316837311, 0.15512436628341675, -0.020491337403655052, 0.1744566559791565, 0.021311338990926743, 0.07719065994024277, -0.14905515313148499, -0.008721653372049332, 0.10816296935081482, 0.10801823437213898, -0.05722608044743538, -0.23783579468727112, -0.09024368226528168, 0.13574089109897614, 0.07145203649997711, 0.011862951330840588, 0.06749942898750305, 0.15963487327098846, 0.06190580874681473, 0.08205609023571014, 0.09404385834932327, 0.19700390100479126, 0.0205052699893713, 0.09838272631168365, -0.12711894512176514, -0.11939704418182373, 0.026096751913428307, 0.03712305799126625, 0.006724864710122347, 0.14659065008163452, -0.1526450514793396, 0.013458039611577988, -0.040623318403959274, 0.026435568928718567, 0.21855473518371582, 0.10425282269716263, -0.14978182315826416, -0.33853256702423096, 0.05801471695303917, -0.005489374976605177, 0.01842530630528927, 0.14353421330451965, 0.1349853128194809, -0.11764918267726898, -0.04029227793216705, -0.04027024656534195, -0.09257956594228745, 0.25101059675216675, 0.10882952064275742, -0.06713977456092834, -0.13112086057662964, -0.016684917733073235, -0.22098633646965027, -0.16750158369541168, 0.5167747139930725, 0.03636828437447548, 0.02829144522547722, -0.02934422343969345, -0.04543108120560646, -0.08843119442462921, 0.10535164177417755, 0.04049691557884216, 0.08041846007108688, -0.23138977587223053, -0.0291252713650465, -0.08372201025485992, 0.18736732006072998, 0.1353333592414856, -0.10577604919672012, -0.17618466913700104, -0.03696482628583908, 0.040582265704870224, -0.08556948602199554, -0.16874060034751892, 0.1080893874168396, -0.12386876344680786, 0.00681004673242569, -0.2552192509174347, 0.030841197818517685, 0.021814528852701187, 0.24520699679851532, 0.14987191557884216, 0.016668153926730156, 0.09110966324806213, 0.06448965519666672, -0.09325025230646133, -0.1166469156742096, -0.031934890896081924, -0.0039120023138821125, 0.23734337091445923, 0.13261501491069794, -0.043203867971897125, 0.0036761490628123283, -0.24351060390472412, 0.08995474874973297, 0.07226885855197906, 0.0018543944461271167, -0.04624735563993454, -0.009900003671646118, 0.16949664056301117, -0.22142063081264496, 0.1479758620262146, -0.1086110919713974, 0.22517462074756622, 0.038981687277555466, 0.012260437943041325, 0.12594559788703918, 0.12600590288639069, 0.1148301362991333, -0.014576985500752926, 0.008870271034538746, -0.03402220457792282, -0.19049537181854248, -0.007656298577785492, 0.04347537085413933, -0.0504094623029232, -0.004955761134624481, -0.1893070489168167, 0.029586780816316605, -0.039982303977012634, 0.20526637136936188, -0.010303834453225136, 0.012069247663021088, 0.13104861974716187, -0.09009838104248047, 0.0370037816464901, -0.141447052359581, 0.02572912536561489, 0.07578542828559875, -0.13916590809822083, 0.1436546891927719, -0.3022632300853729, -0.0944589376449585, 0.16397376358509064, -0.30499473214149475, 0.079180046916008, 0.0329320915043354, -0.002270024735480547, 0.15574941039085388, 0.03373325988650322, 0.08629512041807175, 0.07276540994644165, 0.08715260773897171, -0.16110759973526, 0.014255009591579437, 0.014752261340618134, -0.12991057336330414, 0.08786413073539734, -0.047491785138845444, 0.003004293190315366, -0.13029931485652924, 0.11285857111215591, -0.07597890496253967, 0.01475465763360262, 0.1743883639574051, -0.02809424325823784, 0.03385226055979729, -0.05610138550400734, -0.11827835440635681, -0.023690219968557358, 0.1733856052160263, -0.06689084321260452, -0.2189953327178955, 0.14561724662780762, -0.09462954849004745, -0.08061579614877701, -0.0048561301082372665, -0.1940315216779709}, {0.04690590873360634, 0.11541192978620529, 0.09591308236122131, 0.07415954768657684, -0.14639690518379211, 0.049916647374629974, -0.0061252363957464695, -0.17603090405464172, 0.20942836999893188, 0.23541057109832764, -0.17761914432048798, -0.12897567451000214, -0.005330987740308046, -0.02893206849694252, -0.012250606901943684, 0.022673852741718292, -0.024166520684957504, 0.15361258387565613, 0.24867142736911774, 0.0625171959400177, -0.08129239082336426, -0.029971813783049583, -0.11443553119897842, -0.12482500076293945, -0.028752706944942474, 0.1345246583223343, 0.24494153261184692, 0.08310407400131226, -0.05744965001940727, 0.07503071427345276, -0.08520403504371643, 0.21863755583763123, -0.07824873924255371, -0.030876539647579193, 0.18302951753139496, 0.03175589069724083, 0.12916375696659088, 0.040078721940517426, -0.0481233187019825, 0.0010170061141252518, 0.009350450709462166, 0.08542376756668091, -0.15408416092395782, 0.044100046157836914, 0.19254808127880096, -0.04480716213583946, 0.15255269408226013, 0.07713762670755386, -0.23661354184150696, 0.12484083324670792, -0.4061317443847656, 0.3589625358581543, -0.10502046346664429, 0.008277766406536102, -0.006436212919652462, 0.1690685749053955, -0.03349900618195534, -0.05348439887166023, 0.17793278396129608, 0.2780550718307495, 0.11928382515907288, -0.027739981189370155, 0.26806753873825073, 0.013551857322454453, -0.21538834273815155, -0.18746674060821533, 0.1893082559108734, 0.08165928721427917, 0.22170329093933105, 0.24509991705417633, -0.03718947246670723, -0.18810194730758667, -0.27185535430908203, -0.21026089787483215, -0.02908596210181713, 0.02896582894027233, -0.002599417930468917, -0.06434588134288788, 0.09997628629207611, -0.37319129705429077, 0.03397544473409653, -0.106190025806427, 0.0894952341914177, 0.27397194504737854, 0.17267094552516937, -0.07296469062566757, 0.05024419724941254, -0.33995530009269714, -0.19989995658397675, -0.05343382805585861, -0.038023460656404495, -0.0431194081902504, -0.019770599901676178, -0.15902985632419586, 0.10199102014303207, 0.02778461202979088, 0.0548921637237072, -0.23909945785999298, 0.1739332675933838, 0.08831217139959335, -0.04133588820695877, -0.06443844735622406, -0.0808054655790329, -0.052055202424526215, -0.19789354503154755, -0.22851230204105377, -0.07473773509263992, 0.002178172580897808, -0.13199111819267273, 0.18218079209327698, 0.04036780446767807, -0.1638886034488678, 0.2029704600572586, -0.0587274432182312, -0.07834093272686005, -0.03280116245150566, -0.18866224586963654, 0.3015516698360443, 0.07334081083536148, 0.13778188824653625, 0.0747605562210083, 0.001318154507316649, -0.004126267042011023, -0.07257118076086044, 0.08460607379674911, -0.14759927988052368, -0.09347090125083923, 0.08582144975662231, 0.07948252558708191, 0.07363079488277435, 0.1351366639137268, -0.05416318401694298, 0.11503935605287552, 0.12307503819465637, 0.0562184676527977, -0.17712631821632385, -0.007705680560320616, 0.16084030270576477, 0.07986566424369812, -0.03797243535518646, 0.04962091147899628, -0.19403360784053802, 0.10505779832601547, -0.19721998274326324, -0.11265431344509125, -0.11996903270483017, 0.17671144008636475, -0.1422184407711029, -0.14471964538097382, 0.17423629760742188, 0.012164798565208912, 0.17442184686660767, -0.14405126869678497, 0.046093277633190155, 0.09313295036554337, 0.08687292039394379, 0.09697844088077545, 0.04822110757231712, 0.07095102220773697, 0.4631136357784271, -0.09164012968540192, 0.24614541232585907, 0.04366973042488098, -0.0692133828997612, 0.07053855061531067, 0.02302705869078636, 0.2282167375087738, 0.3093622326850891, 0.03170103207230568, -0.017435966059565544, 0.11463499814271927, -0.10336051136255264, -0.0976918563246727, 0.08009587228298187, -0.2541006803512573, -0.10994785279035568, -0.008378610014915466, -0.10094732791185379, -0.04044672101736069, 0.12258291244506836, -0.050771940499544144, -0.312556654214859, -0.06684371083974838, 0.14753614366054535, 0.0713057890534401, -0.015103920362889767, -0.12323574721813202, 0.047628093510866165, -0.026355154812335968, 0.1352880746126175, -0.13532903790473938, -0.13405722379684448, -0.1342625468969345, -0.02343483828008175, -0.028197845444083214, 0.06159023568034172, 0.11499112099409103, 0.08445042371749878, -0.032012198120355606, -0.03524422273039818, -0.1808769553899765, -0.011795584112405777, 0.052320703864097595, 0.12338010966777802, 0.15546582639217377, -0.09873469173908234, -0.14984716475009918, -0.31476375460624695, -0.07296192646026611, 0.11110690236091614, 0.026786288246512413, 0.03370092436671257, -0.15067550539970398, -0.013678930699825287, -0.08528762310743332, -0.2246936708688736, -0.17730070650577545, -0.11208314448595047, -0.31425759196281433, 0.015282710082828999, 0.12261748313903809, -0.08899492025375366, -0.06716349720954895, -0.16303926706314087, 0.013479326851665974, 0.04309375211596489, -0.2628943622112274, 0.13942213356494904, 0.14689861238002777, -0.09446719288825989, -0.31056147813796997, 0.3041207790374756, 0.008238239213824272, 0.05335374176502228, -0.12966904044151306, 0.0006708992295898497, -0.11354141682386398, -0.024786140769720078, 0.26075878739356995, 0.2647002339363098, -0.06029174476861954, -0.020657332614064217, 0.015033871866762638, 0.068564772605896, 0.06674275547266006, 0.1708744317293167, -0.11333175748586655, 0.03970399498939514, 0.1253218799829483, -0.06382892280817032, -0.03617745265364647, -0.03265567496418953, 0.044339556246995926, -0.15375369787216187, 0.12620411813259125, 0.32515478134155273}, {-0.1413840502500534, 0.25177714228630066, -0.17000871896743774, -0.04097365587949753, 0.2568381130695343, -0.053230781108140945, -0.05097116529941559, 0.18665362894535065, 0.13535338640213013, -0.09075357764959335, 0.15155191719532013, 0.2944245934486389, -0.14096465706825256, 0.17933033406734467, 0.024734707549214363, 0.05095691233873367, 0.12806111574172974, -0.044946786016225815, -0.09407939016819, -0.03774695843458176, -0.06180046126246452, -0.029402412474155426, 0.06266363710165024, 0.1898651123046875, -0.35758069157600403, -0.009426593780517578, -0.004316831938922405, -0.00214164680801332, 0.00363709032535553, -0.1068054810166359, 0.20418670773506165, -0.1681254655122757, -0.14651525020599365, 0.08283690363168716, 0.15335305035114288, -0.1664157658815384, 0.05878066271543503, 0.13626305758953094, -0.1972394436597824, -0.1445453017950058, 0.19758330285549164, -0.13346844911575317, 0.03612704947590828, 0.27490246295928955, 0.04484247416257858, -0.11872442811727524, -0.01881544105708599, -0.218277245759964, 0.09218208491802216, 0.02888178639113903, -0.1165493056178093, -0.054146908223629, -0.08932510018348694, 0.30025288462638855, 0.06432373821735382, 0.27714109420776367, 0.03569525107741356, 0.07629582285881042, 0.011686235666275024, -0.2460736483335495, -0.06938104331493378, 0.011267684400081635, -0.11041335016489029, -0.0845583975315094, 0.028654804453253746, 0.11866934597492218, -0.12489014863967896, -0.049188848584890366, 0.23671448230743408, 0.046786610037088394, 0.04309333860874176, 0.005156784784048796, -0.1469588279724121, 0.21029148995876312, 0.5005002617835999, -0.024931620806455612, -0.06784860789775848, 0.15323160588741302, 0.14732863008975983, 0.049834270030260086, -0.0004510314320214093, 0.15315188467502594, -0.008719196543097496, 0.17276619374752045, -0.09461023658514023, 0.1108398288488388, -0.05230683460831642, -0.08273693174123764, 0.1695362776517868, 0.1473248302936554, 0.11836688965559006, -0.08627989888191223, -0.15519408881664276, -0.09151534736156464, 0.13539674878120422, 0.15115413069725037, -0.10606282949447632, 0.08433637022972107, 0.17303970456123352, 0.4877215325832367, 0.044597726315259933, -0.09949100017547607, 0.04220060259103775, -0.0214182510972023, -0.07751519978046417, 0.40347397327423096, 0.05904582887887955, 0.20417559146881104, 0.24679894745349884, 0.07004774361848831, 0.08933045715093613, 0.223332017660141, -0.28022846579551697, 0.04542308673262596, 0.12968498468399048, -0.004390816669911146, 0.07238839566707611, 0.061062540858983994, -0.06243445351719856, 0.06839373707771301, -0.016394760459661484, 0.2008356750011444, 0.017366474494338036, -0.006671425886452198, 0.10423047095537186, 0.19574032723903656, -0.27937817573547363, -0.038388606160879135, -0.11013749241828918, -0.20604273676872253, -0.22271078824996948, -0.023461993783712387, -0.06096596270799637, -0.027666643261909485, 0.16244381666183472, -0.1955311894416809, -0.07723984867334366, -0.11434533447027206, -0.14866341650485992, -0.07757934182882309, -0.0088735893368721, -0.0633382797241211, 0.1255902200937271, -0.03761850669980049, 0.0544048547744751, 0.09153982251882553, 0.15462781488895416, 0.1155666932463646, -0.2060617357492447, -0.09529063105583191, -0.1531892716884613, 0.11873572319746017, 0.10102996975183487, 0.1893531233072281, -0.037236154079437256, -0.05771953612565994, -0.07342469692230225, 0.2791990339756012, 0.23223651945590973, 0.20462411642074585, -0.16158045828342438, 0.0941242128610611, -0.16176748275756836, -0.06762927770614624, 0.08318396657705307, -0.14199894666671753, 0.05313628539443016, 0.29308176040649414, -0.18269436061382294, -0.044562820345163345, -0.09276380389928818, 0.15266261994838715, -0.05012105032801628, -0.07169485837221146, 0.09528080374002457, 0.017829662188887596, -0.022908441722393036, -0.07581959664821625, 0.11401184648275375, 0.0066351452842354774, -0.1296643763780594, 0.09149375557899475, -0.05640190839767456, 0.3118656873703003, -0.050184331834316254, 0.1615811437368393, -0.1355043351650238, -0.07307383418083191, 0.04966570809483528, -0.12773747742176056, -0.1165003776550293, 0.10568300634622574, -0.1284763664007187, -0.04493211582303047, -0.10725655406713486, -0.08738570660352707, -0.04777531325817108, 0.3486986756324768, -0.08469533175230026, 0.1385992467403412, 0.06059764325618744, -0.06949729472398758, -0.051168665289878845, -0.06964722275733948, -0.030134834349155426, 0.1239599958062172, -0.014891215600073338, 0.1278161257505417, 0.07171936333179474, -0.19659657776355743, 0.017365431413054466, -0.19407245516777039, -0.027247605845332146, -0.009245449677109718, 0.15436656773090363, -0.1744929999113083, -0.00935615785419941, 0.12261300534009933, 0.14744941890239716, -0.07143594324588776, 0.15013264119625092, 0.04867198318243027, 0.13484610617160797, -0.18191395699977875, -0.13704131543636322, -0.2252529412508011, -0.03401394560933113, 0.017274295911192894, -0.05883131921291351, 0.13895048201084137, -0.005986434873193502, -0.06754323840141296, -0.10559158027172089, -0.046140190213918686, 0.0388045571744442, -0.15284155309200287, 0.29253754019737244, -0.21830590069293976, -0.07652831822633743, 0.4794119596481323, -0.07146057486534119, 0.10508108139038086, -0.059182099997997284, 0.139167919754982, -0.042859211564064026, -0.069316565990448, 0.06976128369569778, 0.13064171373844147, 0.08854029327630997, -0.07893873751163483, -0.04187744855880737, 0.11462979763746262, -0.06941063702106476, -0.2970747649669647, -0.011754105798900127, -0.07528728991746902}, {-0.22801679372787476, 0.049011390656232834, -0.03907855600118637, 0.219872385263443, -0.09783771634101868, 0.1308237463235855, -0.15935945510864258, -0.1353670358657837, 0.08039670437574387, -0.1304854154586792, 0.22630789875984192, -0.0662226527929306, 0.09183865785598755, 0.00027109411894343793, 0.0909479409456253, -0.06826632469892502, -0.018102409318089485, 0.043013766407966614, 0.1280878484249115, -0.09465062618255615, -0.12410026788711548, 0.04492836445569992, 0.11232026666402817, 0.1317722201347351, 0.08558680862188339, -0.03280794247984886, -0.030491923913359642, -0.12326420098543167, -0.05385824292898178, -0.1690932810306549, 0.15843546390533447, 0.03640616685152054, 0.13754451274871826, -0.04925323650240898, -0.0171415563672781, 0.24023079872131348, 0.08410148322582245, 0.21128761768341064, 0.17145197093486786, 0.008506649173796177, -0.05291513353586197, -0.11765488237142563, -0.07567673176527023, 0.09880227595567703, -0.126323863863945, -0.12202630192041397, -0.033701468259096146, 0.04838806763291359, 0.10749517381191254, 0.06213720515370369, 0.06412674486637115, 0.12412324547767639, -0.24256987869739532, -0.09828118234872818, -0.01637464202940464, 0.037417203187942505, 0.1417626440525055, 0.042543090879917145, -0.018725980073213577, -0.020282277837395668, 0.19776651263237, 0.05842164158821106, 0.008462290279567242, 0.14988020062446594, -0.20824027061462402, 0.01913261041045189, -0.06108863651752472, 0.07607150077819824, 0.10129937529563904, 0.11414071172475815, -0.05795854702591896, 0.05156165361404419, 0.01519627682864666, 0.13888736069202423, 0.056233081966638565, 0.08848384022712708, 0.053260281682014465, 0.057568203657865524, 0.32573631405830383, 0.10516250878572464, -0.05788905918598175, 0.22732119262218475, -0.09536594897508621, -0.055306777358055115, 0.11284508556127548, -0.15695062279701233, -0.1339993178844452, -0.019522525370121002, -0.19896605610847473, -0.036284420639276505, 0.14633329212665558, -0.029556943103671074, 0.1903587132692337, -0.012128452770411968, 0.08755183219909668, 0.18962083756923676, -0.10206445306539536, -0.15434935688972473, -0.02465912327170372, 0.015985749661922455, 0.0032704127952456474, 0.01700950227677822, -0.05220680683851242, -0.11184056103229523, -0.10581109672784805, 0.5003896355628967, 0.1327618658542633, -0.003130161901935935, -0.19803228974342346, 0.04338417947292328, -0.0734153538942337, -0.03283487260341644, -0.03249971196055412, -0.11253520846366882, 0.0005871735047549009, -0.008269386366009712, -0.02883191965520382, 0.009307428263127804, 0.008188332431018353, 0.007212936878204346, 0.1212928369641304, -0.09116563200950623, -0.05630049481987953, 0.20474636554718018, -0.010290324687957764, 0.036133188754320145, 0.15635761618614197, -0.007849149405956268, -0.056819092482328415, -0.1179911270737648, 0.05134544521570206, -0.0442688912153244, 0.025134673342108727, -0.11602625250816345, 0.023071974515914917, -0.13096152245998383, -0.06293023377656937, -0.07916412502527237, 0.022863758727908134, 0.13976939022541046, -0.008155460469424725, 0.08672251552343369, -0.16193699836730957, -0.11986969411373138, 0.010334516875445843, -0.09242424368858337, 0.16978366672992706, 0.22114378213882446, 0.03379666805267334, 0.06916920840740204, -0.1302546262741089, -0.15545488893985748, 0.17067565023899078, 0.19233079254627228, 0.016714803874492645, 0.28408095240592957, -0.0853259265422821, -0.10490556061267853, 0.02816278301179409, -0.058815594762563705, 0.03434987738728523, 0.1998918056488037, 0.14487163722515106, -0.15487878024578094, 0.09117329120635986, 0.04152034595608711, 0.016951238736510277, 0.08853159099817276, 0.013259484432637691, -0.008580920286476612, -0.14993825554847717, -0.2220401018857956, 0.1965249478816986, 0.29733413457870483, 0.13671423494815826, -0.011388282291591167, -0.005134198814630508, -0.15381430089473724, 0.07071363925933838, -0.053956303745508194, 0.2168777883052826, -0.15315456688404083, 0.04907640442252159, -0.09833045303821564, -0.06406394392251968, 0.11261259019374847, 0.026343822479248047, 0.028261486440896988, 0.15661871433258057, -0.008051293902099133, -0.0019324712920933962, -0.15828897058963776, -0.0519944466650486, 0.0335310623049736, 0.03292893245816231, -0.0012800851836800575, -0.11073117703199387, -0.05070440098643303, -0.036527641117572784, 0.1359403431415558, -0.030271757394075394, 0.11951740831136703, 0.21927839517593384, -0.268599271774292, 0.18051528930664062, 0.009431500919163227, 0.22448016703128815, 0.11551891267299652, -0.013098743744194508, -0.14629527926445007, -0.030393656343221664, -0.04032221809029579, -0.06550250202417374, 0.03417905420064926, 0.03330431506037712, 0.018892373889684677, 0.052516356110572815, 0.06645022332668304, 0.01658817194402218, 0.17490892112255096, 0.14221790432929993, -0.06142650172114372, -0.0016201038379222155, 0.0859360545873642, -0.4613838195800781, 0.12254665791988373, 0.10877725481987, 0.028415556997060776, 0.16147620975971222, -0.0881083756685257, -0.18433266878128052, 0.1371513158082962, -0.057454776018857956, 0.09381471574306488, -0.005995433311909437, 0.09442075341939926, 0.14581695199012756, -0.13011464476585388, 0.13260968029499054, 0.029133370146155357, -0.1924387812614441, -0.021463772282004356, -0.041841987520456314, -0.17698732018470764, -0.052809860557317734, 0.01895255222916603, -0.029082631692290306, -0.0748388022184372, 0.040914829820394516, -0.0763954222202301, -0.11048462986946106, 0.11813244968652725, 0.2832984924316406, 0.07127737253904343, 0.1035994291305542, -0.35410821437835693}, {-0.010763008147478104, -0.019213370978832245, 0.028895266354084015, -0.0424317829310894, -0.0756666287779808, -0.04568939656019211, 0.19473797082901, -0.17094333469867706, -0.027532661333680153, 0.09358963370323181, -0.1029602587223053, 0.12339234352111816, -0.30950725078582764, 0.10665729641914368, -0.061216212809085846, -0.07116475701332092, 0.050915904343128204, 0.0023267471697181463, 0.09422671049833298, 0.17129911482334137, -0.08056390285491943, -0.19249197840690613, -0.061047039926052094, -0.1732856035232544, -0.20192454755306244, 0.04658486694097519, 0.012417234480381012, -0.0403367355465889, -0.0466393381357193, 0.1413879245519638, 0.11713874340057373, 0.02593751810491085, -0.1623443216085434, 0.0016061844071373343, -0.1752150058746338, -0.23649510741233826, -0.06380268931388855, -0.08974456042051315, -0.1820208728313446, 0.04067390412092209, -0.13681098818778992, -0.02764762192964554, 0.0919935405254364, 0.07228939980268478, 0.06022275239229202, -0.0690917894244194, 0.09260448813438416, -0.15518027544021606, -0.0027474057860672474, 0.010374654084444046, -0.07204237580299377, 0.0750613585114479, 0.06554118543863297, 0.040263283997774124, 0.05152546614408493, 0.021973730996251106, 0.08469260483980179, -0.0555402971804142, 0.19388103485107422, 9.49222612689482e-06, -0.09449194371700287, 0.0390821173787117, 0.012116755358874798, -0.0389903225004673, -0.024271836504340172, -0.12669254839420319, -0.0571063831448555, 0.016719384118914604, -0.11985184252262115, 0.03207435458898544, 0.0759914293885231, -0.06935673207044601, -0.1315949410200119, 0.05556756258010864, 0.08419518172740936, 0.05780888721346855, 0.14743173122406006, 0.08853626996278763, 0.21690280735492706, 0.1007927656173706, 0.1730579286813736, -0.004292242228984833, 0.11106700450181961, 0.13373415172100067, -0.2872050404548645, 0.22808155417442322, 0.059614378958940506, -0.1914432793855667, 0.04184795916080475, -0.023983081802725792, -0.12619765102863312, 0.03066285513341427, 0.07185755670070648, -0.0751211941242218, 0.0834236815571785, -0.07770280539989471, -0.002709143329411745, -0.05359834432601929, -0.1547531634569168, 0.08145502209663391, -0.0017221900634467602, -0.11289456486701965, 0.0044531566090881824, -0.15468473732471466, -0.2407141476869583, -0.2770163118839264, -0.012573156505823135, 0.217568501830101, -0.05642891675233841, -0.013909712433815002, -0.05074638873338699, 0.01916922815144062, 0.14371737837791443, -0.2529256045818329, 0.13294535875320435, -0.1089584082365036, -0.16793859004974365, -0.06536656618118286, -0.18238221108913422, 0.054980434477329254, 0.034614820033311844, -0.10553058236837387, -0.013446412049233913, -0.11597558856010437, 0.09448270499706268, -0.10929515957832336, 0.042556148022413254, -0.012316941283643246, 0.15934020280838013, 0.060338344424963, -0.0075713228434324265, -0.14498968422412872, 0.048417750746011734, 0.07772036641836166, -0.22515669465065002, -0.029295507818460464, 0.016729500144720078, 0.08931353688240051, 0.13173513114452362, -0.12873829901218414, -0.06911520659923553, 0.12461292743682861, 0.16206306219100952, -0.02494611218571663, -0.08198972791433334, 0.16588425636291504, 0.0015856479294598103, -0.11136071383953094, 0.0051827700808644295, -0.017279883846640587, -0.11508185416460037, -0.05408845469355583, -0.11762149631977081, 0.12998686730861664, -0.17036141455173492, -0.037997767329216, 0.14848949015140533, -0.24615426361560822, 0.08283493667840958, 0.0848141610622406, -0.07723146677017212, -0.07991141080856323, 0.17872337996959686, -0.021732239052653313, 0.047990791499614716, 0.1533633917570114, 0.09820427745580673, -0.10437411814928055, 0.09421011805534363, 0.23978842794895172, -0.11337526887655258, -0.05523483827710152, -0.09928623586893082, -0.09902068227529526, -0.10729828476905823, 0.17217043042182922, -0.03431221470236778, -0.06677451729774475, 0.03631509095430374, -0.2699356973171234, -0.04077107459306717, 0.01908576861023903, 0.046724822372198105, 0.2406144142150879, 0.13890579342842102, -0.025691736489534378, -0.07623596489429474, 0.09656523913145065, -0.05735781416296959, 0.009776197373867035, 0.04396267235279083, 0.08465135842561722, 0.2105293720960617, 0.04216141253709793, -0.13766880333423615, 0.0873192846775055, -0.06367626786231995, -0.011207266710698605, 0.08517949283123016, -0.09140799939632416, 0.04109317064285278, 0.07052671909332275, -0.07521797716617584, 0.07170882076025009, 0.21990220248699188, 0.15010102093219757, -0.04039396345615387, -0.039268799126148224, 0.09574050456285477, -0.08135788887739182, -0.017942503094673157, -0.15542905032634735, 0.08689285069704056, 0.24555811285972595, -0.15161284804344177, 0.02679590694606304, -0.0696873664855957, 0.01736169308423996, 0.008726944215595722, -0.1435198038816452, -0.021791385486721992, 0.041363224387168884, -0.12996803224086761, -0.21558673679828644, -0.15993154048919678, -0.37081095576286316, 0.08031036704778671, -0.019950585439801216, -0.19765043258666992, 0.06910374760627747, -0.14273689687252045, -0.012410176917910576, -0.059054095298051834, -0.206698939204216, 0.1594836264848709, -0.095608651638031, -0.032692309468984604, 0.036892376840114594, 0.23246537148952484, 0.038369063287973404, -0.054828543215990067, -0.044543396681547165, 0.12562678754329681, 0.07827174663543701, -0.07160833477973938, -0.03114592842757702, 0.00030668091494590044, 0.18485593795776367, -0.19602835178375244, 0.0632188692688942, -0.1470983475446701, -0.03649004548788071, -0.01899176649749279, 0.021564358845353127, 0.09574229270219803, -0.15098124742507935}, {-0.09478099644184113, -0.19481323659420013, -0.08364848792552948, 0.028387684375047684, 0.17322608828544617, -0.11875845491886139, -0.15547114610671997, 0.19959956407546997, -0.16496311128139496, -0.16181278228759766, 0.30857259035110474, 0.05645078793168068, -0.08704350888729095, 0.005367849487811327, -0.15027035772800446, 0.1303105503320694, 0.0673220157623291, -0.12905018031597137, -0.04698168486356735, 0.0297060776501894, 0.18623526394367218, -0.038461536169052124, 0.13649000227451324, 0.028977997601032257, 0.04817074164748192, 0.23444192111492157, -0.08941733837127686, 0.00251305871643126, 0.01871994510293007, 0.12768030166625977, 0.2112746387720108, 0.14375895261764526, 0.17699186503887177, 0.0946325957775116, 0.1202392578125, -0.021656690165400505, 0.002429160987958312, 0.14535680413246155, 0.11093495786190033, 0.10450433194637299, 0.15355463325977325, -0.03626806288957596, 0.14549586176872253, 0.19310443103313446, 0.028753839433193207, 0.09727319329977036, -0.06010781228542328, 0.050858642905950546, -0.060931675136089325, 0.07028460502624512, -0.030158841982483864, 0.23501570522785187, -0.22322629392147064, 0.20215731859207153, -0.06884915381669998, 0.025293393060564995, -0.0551849901676178, 0.008543637581169605, -0.11526195704936981, -0.02946515567600727, -0.17293907701969147, -0.10639624297618866, 0.05997546389698982, 0.3335019648075104, 0.2655363380908966, 0.16567429900169373, -0.06602924317121506, -0.14224088191986084, 0.15712447464466095, 0.18250848352909088, 0.008386600762605667, 0.1466916799545288, -0.018095578998327255, -0.005443105939775705, -0.024133455008268356, -0.1683163046836853, -0.35030511021614075, -0.08361569792032242, 0.04042169079184532, -0.002674758667126298, -0.023647332563996315, 0.11407700181007385, 0.01981591433286667, -0.21999166905879974, 0.11546750366687775, -0.0828036218881607, -0.15994136035442352, 0.056098975241184235, -0.17244930565357208, 0.020390570163726807, -0.00962336827069521, -0.0026827012188732624, -0.10957580804824829, -0.01435448694974184, 0.06695996969938278, 0.07770765572786331, 0.06973639130592346, -0.046299178153276443, 0.015226967632770538, -0.042023878544569016, -0.03466340899467468, 0.004147575702518225, 0.028872858732938766, 0.10245724022388458, 0.029529994353652, 0.010571270249783993, -0.014891191385686398, -0.13708630204200745, 0.017600128427147865, -0.03574558347463608, 0.15184643864631653, -0.08865153789520264, -0.10210234671831131, 0.15934492647647858, 0.06874578446149826, -0.03794668987393379, 0.1289897859096527, 0.058883536607027054, 0.09601081162691116, 0.20380321145057678, 0.21180856227874756, 0.05604002624750137, -0.007574905641376972, 0.04777423292398453, 0.019263096153736115, 0.04866635799407959, 0.21698808670043945, -0.016658447682857513, 0.15385062992572784, -0.1165878176689148, 0.07223497331142426, 0.033638935536146164, -0.09058298170566559, -0.06608644127845764, -0.09646407514810562, 0.2149665355682373, 0.09578379988670349, 0.03468919172883034, -0.015879733487963676, 0.23997727036476135, -0.002507144818082452, 0.22862856090068817, -0.10817061364650726, -0.054763760417699814, 0.034376781433820724, -0.16286279261112213, -0.17255312204360962, 0.014853490516543388, 0.05571210756897926, 0.09117726981639862, -0.08034839481115341, 0.039541393518447876, 0.1962023824453354, -0.20444756746292114, 0.10597014427185059, 0.11875507980585098, -0.20165416598320007, 0.08922120183706284, 0.07345100492238998, -0.12346716225147247, -0.019927654415369034, -0.07764849811792374, -0.08938460052013397, 0.0067603993229568005, -0.0435287244617939, 0.0026017145719379187, -0.10730868577957153, 0.03899507224559784, 0.026944633573293686, -0.10192764550447464, -0.09239750355482101, 0.08376697450876236, 0.11224504560232162, -0.18068169057369232, 0.20135846734046936, -0.1917700469493866, -0.04037223383784294, -0.05737467110157013, -0.06439650058746338, 0.046388350427150726, -0.0984349474310875, 0.07370337098836899, 0.16006970405578613, -0.06603994220495224, -0.07575041055679321, -0.12725745141506195, 0.2762703001499176, -0.04396802559494972, 0.06243075430393219, -0.034454572945833206, -0.025076663121581078, 0.04789919778704643, -0.060032956302165985, 0.0650210902094841, 0.08645057678222656, -0.3226427733898163, -0.16440974175930023, 0.030232451856136322, -0.015753474086523056, -0.1628199815750122, 0.19646307826042175, 0.1688416600227356, 0.04083262383937836, 0.08546081930398941, -0.09624474495649338, -0.026921968907117844, 0.12449570745229721, 0.027649778872728348, 0.0038943139370530844, -0.017726926133036613, 0.16963520646095276, -0.04374361410737038, -0.11814279109239578, 0.043035589158535004, -0.13263610005378723, -0.0894634872674942, 0.0014727722154930234, -0.16233909130096436, 0.14049287140369415, 0.04590626060962677, -0.01718718931078911, 0.03481598198413849, 0.014852683059871197, 0.11870274692773819, -0.022628050297498703, 0.07611957937479019, -0.0596812441945076, -0.058646056801080704, 0.04770316556096077, 0.04578118398785591, 0.012904133647680283, -0.2311020940542221, -0.43818962574005127, -0.17781272530555725, 0.06575709581375122, -0.2993757128715515, -0.03229105845093727, 0.12935855984687805, -0.03943889960646629, -0.10280507802963257, 0.055416613817214966, 0.011429982259869576, -0.14979733526706696, -0.18945153057575226, -0.026210378855466843, -0.1025601401925087, -0.10611597448587418, -0.1655435562133789, -0.23185111582279205, -0.12813110649585724, 0.012834630906581879, -0.15830066800117493, -0.020196564495563507, -0.016253499314188957, -0.17747777700424194, -0.2915075421333313}, {0.03335018455982208, -0.23387038707733154, 0.0023264240007847548, -0.14185693860054016, -0.002408078871667385, 0.07581907510757446, 0.03426013141870499, -0.12618927657604218, -0.04729242995381355, 0.06676381081342697, -0.20961415767669678, -0.22973327338695526, -0.1300382763147354, -0.08571702986955643, -0.038646914064884186, 0.07771643996238708, -0.16182862222194672, -0.20734423398971558, 0.0312468521296978, -0.14655780792236328, -0.01268810871988535, 0.11553893983364105, 0.007684437092393637, 0.1627788245677948, 0.15014126896858215, 0.39117738604545593, -0.08696633577346802, 0.13929247856140137, 0.10366842150688171, 0.07559695839881897, -0.2617536783218384, 0.09269613027572632, 0.07095830887556076, 0.10996037721633911, 0.028167501091957092, -0.35980528593063354, 0.00382155179977417, -0.05845252797007561, 0.009943149983882904, -0.15713287889957428, 0.03391101211309433, -0.08287239074707031, -0.04103529825806618, 0.054490406066179276, -0.07356823980808258, 0.12700924277305603, 0.1576015204191208, -0.10786717385053635, -0.04087110981345177, -0.05356413125991821, -0.13712868094444275, -0.027763940393924713, -0.1058521643280983, 0.1970987468957901, -0.040458861738443375, 0.10428497940301895, -0.024371007457375526, -0.08652510493993759, 0.15054772794246674, -0.08380980044603348, 0.02098124288022518, 0.21506783366203308, 0.31613749265670776, -0.08706880360841751, 0.03071299009025097, -0.02082735300064087, 0.05787387862801552, 0.14018817245960236, 0.20015797019004822, -0.045556869357824326, 0.15271614491939545, 0.055791810154914856, -0.34101590514183044, 0.06409172713756561, 0.037878017872571945, -0.3097277879714966, -0.08860581368207932, 0.18367063999176025, -0.07099569588899612, -0.2650163173675537, -0.11284296959638596, 0.1953934282064438, 0.14061850309371948, 0.16396401822566986, -0.07084573060274124, 0.12521368265151978, -0.018667621538043022, -0.1804913878440857, 0.06792837381362915, 0.02694290690124035, -0.14271865785121918, 0.04638102650642395, -0.23125256597995758, -0.16033075749874115, 0.16923092305660248, 0.19733184576034546, 0.03142877668142319, -0.3292236030101776, -0.03928220272064209, -0.13192132115364075, 0.10553411394357681, -0.06359916925430298, 0.025176025927066803, -0.020854584872722626, 0.3557213544845581, 0.011829082854092121, 0.06387659907341003, 0.10363133251667023, 0.07494500279426575, -0.0016538831405341625, 0.1415501832962036, 0.1059596911072731, 0.03425034135580063, -0.02826225943863392, 0.10580862313508987, -0.07032304257154465, -0.026765182614326477, -0.0597548708319664, 0.09856542944908142, -0.1369282305240631, 0.022709136828780174, -0.2090335190296173, 0.008393557742238045, 0.04204977676272392, 0.14107246696949005, -0.1039424017071724, 0.06630779802799225, 0.03993428871035576, 0.2195100486278534, -0.04450011998414993, -0.21206462383270264, -0.30059993267059326, 0.27731406688690186, 0.09550036489963531, 0.1643962264060974, -0.07013298571109772, 0.23153294622898102, 0.06519516557455063, 0.290672242641449, 0.05843207240104675, -0.058935172855854034, 0.11468971520662308, 0.35735639929771423, -0.08274339139461517, -0.01677667535841465, -0.16676436364650726, 0.19794756174087524, 0.10845314711332321, -0.09558668732643127, -0.045614566653966904, 0.0070822155103087425, 0.14156226813793182, -0.18429160118103027, -0.201210156083107, -0.27452152967453003, 0.1941182166337967, -0.1658703088760376, 0.16781914234161377, 0.18915753066539764, 0.006328280549496412, 0.055037643760442734, -0.03146889805793762, 0.04767802730202675, -0.15833549201488495, -0.050896547734737396, -0.03710417449474335, 0.04297730699181557, -0.028553137555718422, -0.03417184576392174, -0.07123123854398727, -0.08163062483072281, -0.21852971613407135, -0.1873002052307129, -0.3477773070335388, 0.05841107293963432, -0.004207352176308632, -0.1351626217365265, 0.02670862153172493, -0.03975958749651909, 0.11801164597272873, -0.0510987788438797, -0.05079171806573868, -0.14293043315410614, 0.11167845129966736, -0.057612597942352295, -0.13033421337604523, -0.1961592733860016, 0.025129329413175583, 0.12377684563398361, -0.1786302775144577, 0.09955298155546188, -0.0426979623734951, 0.04937782883644104, -0.12083251029253006, 0.06831197440624237, -0.05301199108362198, -0.0002728204126469791, 0.0690428763628006, -0.01012390572577715, -0.04864663630723953, 0.01675308309495449, 0.28054848313331604, 0.02184615097939968, 0.004245201125741005, -0.06529566645622253, -0.007287372834980488, 0.040063850581645966, -0.17291446030139923, 0.10888063907623291, 0.054422955960035324, 0.12644623219966888, -0.2056163251399994, -0.10914673656225204, -0.18160755932331085, -0.18872499465942383, -0.07619159668684006, -0.06693387776613235, -0.16524100303649902, -0.02144516259431839, -0.08489784598350525, 0.08791374415159225, 0.1595534086227417, -0.08134844899177551, -0.2885245084762573, -0.05683673545718193, -0.20965352654457092, -0.04420209676027298, -0.030291300266981125, -0.06722496449947357, -0.015260094776749611, 0.018657267093658447, 0.16823041439056396, -0.1743747889995575, -0.3854195773601532, 0.1047675684094429, 0.10553667694330215, -0.24783232808113098, -0.13923045992851257, 0.15090826153755188, 0.08821505308151245, -0.030978471040725708, 0.11824284493923187, -0.02198699675500393, 0.027798984199762344, 0.04059538617730141, -0.003418596927076578, -0.02237129956483841, -0.015393874607980251, 0.017000945284962654, -0.05762256309390068, 0.030378812924027443, 0.0014047859003767371, -0.12050707638263702, -0.0005055946530774236, 0.1758127510547638, -0.15135248005390167}, {-0.15913498401641846, -0.0487830750644207, 0.0019672794733196497, -0.003195235738530755, 0.05775032564997673, 0.08560255914926529, 0.036668561398983, -0.014563280157744884, -0.12557576596736908, -0.18943175673484802, 0.11335520446300507, 0.05175361409783363, 0.1667051613330841, -0.08025304973125458, 0.13239403069019318, -0.005253843497484922, -0.11418821662664413, -0.0490683950483799, 0.11464472115039825, -0.16934141516685486, -0.11696787923574448, -0.0496903657913208, 0.021832076832652092, -0.05931071937084198, -0.08739910274744034, -0.07733380794525146, 0.004475449677556753, -0.06371979415416718, 0.041057899594306946, 0.0020489597227424383, 0.07338111847639084, 0.02679361216723919, -0.12740536034107208, -0.148374542593956, 0.04065317660570145, -0.2024422287940979, 0.29095083475112915, 0.1014750674366951, 0.17203080654144287, 0.0041918023489415646, -0.07571633160114288, 0.08910328894853592, 0.1717286854982376, -0.18769416213035583, 0.031123951077461243, 0.1339276134967804, 0.05369630455970764, -0.015202379785478115, -0.19137738645076752, -0.10839546471834183, 0.0872916430234909, 0.07529133558273315, -0.23107339441776276, -0.1248834952712059, 0.0021596436854451895, 0.2408721148967743, -0.041587945073843, -0.13065262138843536, 0.09662774950265884, -0.040058184415102005, -0.20833919942378998, -0.09362827241420746, 0.1521272361278534, 0.13767294585704803, -0.019752096384763718, -0.03864327818155289, -0.0440349206328392, -0.12064336240291595, 0.10966771841049194, -0.019507506862282753, -0.13933293521404266, -0.03668709099292755, 0.27522680163383484, 0.018696507439017296, 0.0794377401471138, 0.0236984696239233, -0.014377390965819359, 0.16205660998821259, 0.48295360803604126, -0.043268460780382156, -0.16673122346401215, 0.08377421647310257, 0.2673156261444092, 0.13363824784755707, -0.018071042373776436, -0.11096564680337906, -0.046885617077350616, 0.1918729543685913, 0.07440702617168427, 0.10642954707145691, -0.03226309269666672, -0.057214800268411636, 0.1323036402463913, -0.06967838108539581, 0.008699106052517891, -0.2002970427274704, -0.1362135112285614, 0.04983947426080704, 0.23110322654247284, -0.1622263491153717, 0.017378905788064003, 0.08814122527837753, 0.08761033415794373, -0.1023864895105362, 0.04411659762263298, -0.3120911419391632, 0.07370007783174515, 0.16776278614997864, 0.10024131089448929, -0.11478180438280106, 0.07809749990701675, -0.009799221530556679, -0.05691426247358322, -0.20475248992443085, 0.0892002210021019, -0.1816747635602951, -0.00794671755284071, -0.12318491190671921, 0.001839452888816595, -0.06972775608301163, 0.04913150519132614, 0.12645669281482697, -0.0691135823726654, -0.10594239830970764, 0.04236973077058792, -0.21586906909942627, -0.05975839123129845, -0.2112448662519455, -0.0451524443924427, -0.17276042699813843, -0.05207962915301323, 0.10691963881254196, -0.23590071499347687, -0.06022120639681816, 0.04494737833738327, 0.041601695120334625, -0.032237786799669266, 0.02141733653843403, 0.00010860676411539316, 0.11427119374275208, 0.047134388238191605, -0.04709063842892647, -0.18858395516872406, 0.21787406504154205, 0.09998785704374313, -0.04227304831147194, -0.11476851999759674, 0.04182711988687515, -0.07659775018692017, -0.06200327351689339, -0.10665849596261978, -0.1061362624168396, 0.01297756377607584, 0.06083393096923828, -0.07106046378612518, -0.07126501202583313, 0.06262236088514328, 0.10031833499670029, 0.07456386834383011, 0.0011913637863472104, 0.06563985347747803, -0.0555347241461277, 0.01103634387254715, -0.017898280173540115, 0.05812427029013634, -0.013614851050078869, -0.25162580609321594, 0.03020542860031128, -0.18642018735408783, -0.10581685602664948, -0.013492519035935402, -0.13598446547985077, -0.013210699893534184, -0.03300759196281433, -0.06818703562021255, 0.10401854664087296, 0.08281595259904861, -0.2040310800075531, 0.07343479245901108, -0.13151907920837402, -0.06422261893749237, 0.09725935757160187, -0.15831144154071808, -0.016171198338270187, -0.049814917147159576, -0.06447233259677887, 0.0923973023891449, 0.10155168920755386, 0.014403518289327621, 0.10108453035354614, -0.09769724309444427, 0.09638594090938568, 0.14571160078048706, -0.05701671168208122, 0.0038893120363354683, -0.21377161145210266, 0.09895367175340652, -0.0550866425037384, 0.11151353269815445, 0.06307805329561234, 0.07086662948131561, 0.05452955514192581, -0.11828093230724335, 0.06306947022676468, -0.06730393320322037, 0.0822676420211792, 0.015714963898062706, -0.01093776524066925, 0.13957177102565765, 0.20103885233402252, 0.19275160133838654, 0.06264587491750717, -0.03949883207678795, 0.201668843626976, -0.16798220574855804, -0.021348142996430397, -0.24367527663707733, 0.04331917688250542, -0.16185994446277618, 0.0071177356876432896, 0.09626366198062897, -0.09190390259027481, 0.10513031482696533, 0.2548753619194031, -0.08574842661619186, 0.1444977968931198, 0.028917882591485977, -0.06183726340532303, 0.040163345634937286, 0.007286818698048592, -0.12248454988002777, 0.008049850352108479, -0.09496963024139404, -0.07974309474229813, -0.18923166394233704, -0.10993696004152298, -0.15277551114559174, 0.06029671058058739, -0.02493029646575451, -0.19906270503997803, 0.10401961952447891, 0.005464404821395874, 0.09387736022472382, -0.1474447399377823, -0.0941600352525711, 0.012547024525702, -0.030531160533428192, -0.19470937550067902, 0.025232329964637756, 0.06897652894258499, -0.12842212617397308, -0.0038239210844039917, 0.08723977208137512, 0.2561855912208557, 0.34201568365097046, 0.0966598391532898}, {0.05308987945318222, 0.06685835868120193, 0.10310585796833038, 0.16697169840335846, -0.1973552703857422, -0.060566775500774384, -0.055412657558918, 0.07161381840705872, -0.35532450675964355, -0.0452001579105854, 0.0824102833867073, 0.03021378070116043, 0.07264775037765503, 0.10521592944860458, -0.17449097335338593, 0.03648180142045021, -0.11256926506757736, 0.11676569283008575, 0.18229611217975616, -0.10804442316293716, -0.13173961639404297, 0.06902197748422623, -0.014284839853644371, -0.10360538214445114, -0.3033236861228943, -0.022147968411445618, 0.18765199184417725, -0.09628546237945557, 0.03379601240158081, -0.3196115493774414, 0.11361192911863327, 0.20117750763893127, 0.022979887202382088, 0.22070014476776123, 0.03997723013162613, -0.4159662425518036, -0.059350959956645966, 0.09297060966491699, -0.23094502091407776, 0.015734069049358368, 0.03184731677174568, 0.07710140943527222, -0.3256567418575287, -0.2317824810743332, -0.15067817270755768, -0.14775361120700836, 0.21910308301448822, -0.23062413930892944, -0.028727203607559204, -0.06890416890382767, -0.12465767562389374, 0.1161271333694458, 0.061854876577854156, -0.5810925364494324, 0.27398279309272766, -0.15030179917812347, -0.01789529249072075, -0.026843933388590813, 0.011148390360176563, 0.008488224819302559, -0.13997246325016022, 0.1267157942056656, 0.029636533930897713, -0.2450670599937439, 0.03607495501637459, -0.2299962192773819, 0.018281759694218636, -0.07892972975969315, 0.11374843865633011, 0.03860234469175339, 0.007328323554247618, -0.09875895082950592, -0.07559794932603836, 0.08217615634202957, -0.1566537469625473, 0.04778038710355759, -0.12563706934452057, -0.09551022201776505, 0.1920797973871231, 0.15711204707622528, -0.2497493028640747, -0.13942797482013702, 0.06516190618276596, 0.19427448511123657, 0.018458230420947075, -0.026668278500437737, 0.025959517806768417, 0.062435030937194824, 0.26179662346839905, -0.08766450732946396, -0.07687249779701233, 0.0197579488158226, 0.1984691619873047, -0.014255341142416, -0.08812234550714493, -0.003413586877286434, 0.005677712149918079, 0.07281989604234695, 0.25878244638442993, -0.10992242395877838, 0.004415806382894516, -0.1095660999417305, 0.28134167194366455, -0.14667755365371704, -0.1363992691040039, -0.4804006516933441, 0.05318216606974602, -0.006840761750936508, 0.16084054112434387, 0.23554012179374695, -0.050901684910058975, 0.00010203037527389824, 0.04080014303326607, -0.042055193334817886, -0.06693904101848602, -0.16292549669742584, 0.05303723365068436, 0.11914995312690735, -0.09680944681167603, 0.14419829845428467, 0.0014988811453804374, -0.12634757161140442, 0.03549804911017418, -0.19272755086421967, -0.20922373235225677, -0.19407984614372253, -0.05538741126656532, -0.030845308676362038, 0.0008942802087403834, 0.31978967785835266, 0.051806043833494186, 0.04691862687468529, 0.1909378170967102, 0.09053178876638412, -0.11743206530809402, -0.01466650515794754, -0.10963887721300125, -0.02752518653869629, -0.1377428025007248, 0.0105611402541399, -0.13242419064044952, -0.12151838093996048, 0.13223488628864288, -0.0823565423488617, -0.1994500458240509, 0.16688020527362823, -0.06197286769747734, -0.04789996147155762, 0.15077239274978638, -0.0023900140076875687, -0.009923217818140984, -0.18689130246639252, 0.15967224538326263, 0.08407087624073029, 0.0667831152677536, 0.07992692291736603, -0.20839376747608185, -0.1021374836564064, 0.021359872072935104, -0.20281362533569336, 0.08022712171077728, -0.08735401928424835, 0.1551586538553238, -0.1842375546693802, 0.027716219425201416, -0.040551263839006424, -0.17303277552127838, 0.1219964325428009, 0.026608243584632874, 0.03577452525496483, -0.0586572028696537, -0.09158839285373688, -0.06237424537539482, -0.3063295781612396, 0.037872932851314545, 0.2670218348503113, 0.17313553392887115, -0.02977723628282547, -0.05435158684849739, -0.2131173312664032, -0.28836938738822937, -0.04062982648611069, -0.18206657469272614, 0.07727029174566269, 0.05731716752052307, -0.10205760598182678, 0.23902150988578796, -0.05270272493362427, 0.14123867452144623, 0.23612584173679352, 0.31941041350364685, 0.25896021723747253, -0.0895056501030922, -0.14357855916023254, -0.04447762668132782, -0.002350209979340434, 0.006319577340036631, 0.3522122800350189, 0.10938750207424164, 0.09739213436841965, 0.028372731059789658, -0.08414164930582047, -0.10969289392232895, -0.0944661870598793, 0.011635237373411655, -0.13099351525306702, 0.11067645996809006, -0.03125116974115372, 0.19076162576675415, 0.1943746656179428, 0.16475103795528412, -0.034890998154878616, 0.05846305191516876, 0.21495255827903748, 0.16328544914722443, 0.1791902482509613, -0.1730392575263977, 0.07445321977138519, -0.02220744453370571, -0.35619619488716125, -0.12307554483413696, 0.03815672546625137, -0.2807959020137787, 0.09577812254428864, -0.0676233246922493, 0.02107132226228714, 0.0699339285492897, -0.010750853456556797, -0.13592582941055298, 0.1190786138176918, -0.01608584076166153, -0.1678919792175293, -0.11708538979291916, -0.2244284451007843, -0.18059894442558289, 0.08433710038661957, -0.12443030625581741, 0.03508691489696503, -0.29128512740135193, -0.10352350026369095, 0.0748310461640358, 0.0030790597666054964, 0.007502463646233082, 0.11575503647327423, -0.0918719694018364, -0.11846816539764404, 0.026824012398719788, -0.0057814354076981544, 0.22259972989559174, 0.06773711740970612, -0.06473632901906967, 0.13533879816532135, 0.05684100091457367, 0.1960272192955017, -0.0364195816218853, 0.07170431315898895}, {-0.022275812923908234, 0.1251988708972931, 0.060889218002557755, -0.10292834788560867, -0.028473811224102974, 0.12908726930618286, 0.1372275948524475, 0.07050328701734543, 0.09999725222587585, -0.13621968030929565, -0.10512581467628479, 0.028052514418959618, -0.02947576902806759, 0.0891595333814621, 0.10342975705862045, 0.05525274574756622, -0.04010169953107834, 0.07741082459688187, -0.07687824219465256, 0.1855325847864151, -0.08097516000270844, -0.062660813331604, -0.05388718843460083, 0.35537588596343994, -0.007363487966358662, -0.11824577301740646, -0.03642524406313896, -0.06918911635875702, -0.007527695037424564, -0.08233769237995148, -0.19076082110404968, -0.1432284414768219, -0.1223045215010643, 0.04020078107714653, 0.15457700192928314, -0.20963406562805176, -0.10726805031299591, -0.14145001769065857, -0.2640548348426819, -0.054189346730709076, 0.06742767989635468, -0.2277969866991043, -0.056919801980257034, -0.04904480651021004, 0.0904662236571312, -0.1247037723660469, -0.01119585894048214, 0.03699686378240585, -0.08913031220436096, 0.02355659380555153, -0.17453983426094055, -0.14680437743663788, -0.03459439054131508, 0.06101364642381668, -0.0599994882941246, -0.0616815984249115, 0.19864584505558014, 0.08440817892551422, 0.05292503535747528, 0.05770739167928696, 0.12296401709318161, -0.05408622696995735, 0.0288932416588068, -0.6236944794654846, 0.009987599216401577, 0.2455328404903412, -0.005081448238343, 0.09860529005527496, 0.08079240471124649, 0.10907942056655884, 0.055403824895620346, -0.02916300669312477, 0.05516262352466583, 0.0553947351872921, 0.09206657111644745, -0.16706539690494537, -0.009647293016314507, -0.04334302246570587, -0.012780571356415749, 0.10013607889413834, -0.05679228529334068, -0.07255659997463226, 0.14935420453548431, 0.026217669248580933, 0.12892483174800873, -0.032292440533638, 0.10525713860988617, -0.022644175216555595, 0.04380464181303978, 0.1355995535850525, -0.08816812932491302, -0.023746559396386147, 0.07728853821754456, 0.07971642166376114, 0.22559095919132233, 0.12475211918354034, 0.2545985281467438, -0.09185149520635605, -0.025283828377723694, -0.14641769230365753, 0.1341952085494995, 0.13026142120361328, -0.08796889334917068, -0.004269012715667486, -0.14000508189201355, -0.21972626447677612, 0.08377578854560852, 0.07381081581115723, 0.11934327334165573, -0.0543103963136673, -0.011235281825065613, 0.061656318604946136, -0.12895068526268005, -0.21790772676467896, 0.011764435097575188, -0.12318262457847595, 0.143461674451828, -0.1668468713760376, 0.012093373574316502, -0.16512465476989746, -0.011403697542846203, -0.2747885286808014, -0.0334768183529377, -0.021541543304920197, 0.10994669795036316, 0.05659200996160507, -0.09479349106550217, -0.0010276465909555554, -0.11413677036762238, -0.061722710728645325, -0.016497105360031128, -0.029048342257738113, 0.027087999507784843, -0.027010297402739525, -0.24153843522071838, -0.23862919211387634, 0.19893278181552887, -0.0052969749085605145, -0.015175877138972282, 0.0691281110048294, 0.11003067344427109, -0.17088396847248077, -0.011488689109683037, 0.05988229066133499, 0.12302682548761368, -0.16372233629226685, 0.08409377187490463, 0.0424695648252964, -0.06502973288297653, -0.274679571390152, -0.1310715675354004, 0.12635159492492676, -0.008568434976041317, -0.07276862859725952, 0.04231053218245506, 0.2699606418609619, -0.02220139652490616, -0.12029801309108734, 0.059143513441085815, -0.049911972135305405, -0.023562075570225716, 0.10059355199337006, -0.04832412302494049, -0.12766645848751068, 0.22019825875759125, -0.21409109234809875, -0.02700171247124672, 0.14523547887802124, -0.24875478446483612, 0.1495145559310913, 0.05850931629538536, -0.10576987266540527, 0.015112276189029217, -0.20839357376098633, -0.13359609246253967, -0.17562337219715118, -0.029098881408572197, 0.06444773077964783, 0.1376219242811203, -0.04989936947822571, 0.06564854085445404, -0.05452840030193329, -0.17373888194561005, 0.1896161139011383, -0.052300043404102325, -0.13595399260520935, -0.10764092206954956, -0.13341666758060455, -0.08318187296390533, -0.03965853527188301, 0.07709651440382004, -0.11536220461130142, -0.10060788691043854, -0.14064420759677887, -0.04888526722788811, -0.08625338226556778, 0.1987454742193222, 0.16713836789131165, 0.016970187425613403, 0.2615991532802582, -0.06673254072666168, -0.07776488363742828, 0.11500721424818039, -0.09602727741003036, -0.015271097421646118, 0.010204981081187725, -0.04483501985669136, -0.041119564324617386, -0.10872355848550797, 0.19124223291873932, 0.1625797003507614, -0.13260243833065033, 0.01735195890069008, 0.0928773432970047, 8.377528138225898e-05, 0.0055022030137479305, 0.06372172385454178, -0.08347923308610916, 0.0935935229063034, 0.07701907306909561, 0.04996448755264282, -0.06699053943157196, -0.027723880484700203, -0.12022745609283447, -0.19165167212486267, -0.04686162993311882, -0.0214875265955925, 0.0965932235121727, 0.022849319502711296, 0.05632108077406883, -0.0565606988966465, -0.08948260545730591, -0.08572378754615784, -0.14673510193824768, 0.019976412877440453, 0.07259975373744965, 0.20304585993289948, -0.2788955569267273, -0.025932081043720245, -0.18178574740886688, 0.08304266631603241, 0.16482016444206238, 0.11688467115163803, 0.12184563279151917, -0.20112425088882446, -0.04327864572405815, -0.07810889929533005, 0.24996723234653473, 0.2015717774629593, -0.16378165781497955, 0.044697705656290054, 0.15232841670513153, -0.02123095467686653, 0.07792448252439499, 0.008375018835067749, -0.029864151030778885}, {-0.04822526127099991, 0.008720566518604755, -0.0819852277636528, -0.07400646805763245, -0.06828385591506958, -0.10401049256324768, 0.0983450785279274, 0.03227361664175987, 0.12253809720277786, -0.0033775712363421917, -0.05049772933125496, -0.2334970384836197, 0.02733742445707321, 0.1527249962091446, 0.05896705389022827, -0.06340484321117401, 0.29110950231552124, -0.0034065102227032185, 0.14262829720973969, 0.164664164185524, 0.2705071270465851, 0.0808010846376419, 0.060430482029914856, 0.04508562386035919, 0.029149560257792473, 0.0505438931286335, 0.23249997198581696, 0.06985779851675034, 0.029971614480018616, 0.042165786027908325, -0.06240009889006615, 0.18808606266975403, -0.09511776268482208, 0.22856850922107697, -0.1256793737411499, -0.10963135212659836, -0.01986492983996868, -0.1552008092403412, 0.02634189836680889, -0.2378017008304596, -0.14202138781547546, -0.0837847962975502, -0.09971576184034348, 0.09114667028188705, 0.13205014169216156, -0.1304924041032791, -0.05444154515862465, -0.07871761918067932, 0.13949735462665558, 0.3769831359386444, 0.08467185497283936, 0.13981439173221588, 0.22277110815048218, 0.050328873097896576, 0.0182487852871418, -0.1455235332250595, -0.0628860741853714, 0.11500817537307739, -0.15907414257526398, -0.27355843782424927, 0.1273183524608612, -0.0021249083802103996, -0.35105082392692566, -0.024311484768986702, 0.0022270632907748222, -0.026105618104338646, -0.033277153968811035, -0.3309623599052429, 0.09758103638887405, 0.02531091496348381, 0.0534031018614769, -0.09876664727926254, 0.040493834763765335, 0.04578729718923569, 0.37956899404525757, 0.1575433760881424, -0.15927593410015106, 0.11566635221242905, 0.16588729619979858, 0.2037193775177002, -0.33948513865470886, -0.05412638559937477, 0.16374005377292633, 0.2321961224079132, -0.06536951661109924, 0.10852725058794022, 0.10913584381341934, -0.09258800745010376, 0.0998261347413063, -0.06046091765165329, 0.1740913987159729, 0.08770443499088287, -0.2209952026605606, -0.05704013630747795, 0.15555885434150696, 0.22037558257579803, 0.05804233253002167, -0.06911224871873856, -0.12429273873567581, 0.46236976981163025, 0.22625461220741272, 0.11609766632318497, 0.10298825055360794, -0.10227829217910767, -0.04099419340491295, 0.3534700870513916, -0.036806147545576096, 0.007663522846996784, 0.05945313349366188, -0.0014672790421172976, -0.15951262414455414, -0.04125094786286354, -0.08180423825979233, 0.245338574051857, 0.07229281961917877, 0.11085081100463867, 0.08436939865350723, 0.05662110447883606, 0.08396568894386292, -0.02888156659901142, -0.03159806504845619, -0.09628906100988388, 0.07709982246160507, 0.15526588261127472, 0.07517968863248825, -0.026454534381628036, -0.13422149419784546, -0.09159290045499802, -0.1652286797761917, 0.16512179374694824, -0.08133866637945175, -0.16459733247756958, 0.18975608050823212, 0.09327917546033859, -0.2812192738056183, -0.14002937078475952, 0.13869397342205048, -0.02484799176454544, 0.01876244693994522, 0.111001156270504, -0.1116117537021637, 0.3037797212600708, 0.3961679935455322, -0.2275448888540268, 0.09288229793310165, 0.021133700385689735, 0.10073964297771454, -0.04422564432024956, -0.11862169951200485, 0.019551275297999382, 0.0443304069340229, 0.5647457838058472, 0.1273040920495987, 0.009935456328094006, 0.04234209656715393, 0.2362704575061798, -0.0005584320169873536, 0.14797137677669525, 0.0683131217956543, 0.29952681064605713, 0.20466332137584686, 0.01578994281589985, -0.2500493824481964, 5.490317562362179e-05, 0.16837629675865173, -0.12498893588781357, 0.10737016797065735, 0.09457459300756454, -0.04037529602646828, 0.2904317080974579, -0.07718321681022644, 0.3818909525871277, -0.06345445662736893, -0.06329788267612457, -0.24644437432289124, -0.17074978351593018, 0.0060992054641246796, -0.05337098240852356, 0.04280221834778786, 0.01563096046447754, 0.08713145554065704, -0.1552838832139969, -0.04334622621536255, -0.21429476141929626, -0.08983796834945679, -0.29751861095428467, 0.1053907722234726, -0.048861127346754074, -0.09437614679336548, -0.06468211114406586, 0.06162523478269577, 0.024466488510370255, -0.1843917816877365, -0.5001593232154846, -0.018890274688601494, -0.0796307921409607, -0.12768404185771942, 0.09311646968126297, 0.0933699831366539, 0.12562301754951477, -0.315174400806427, 0.15605919063091278, -0.003140832530334592, -0.14985966682434082, -0.09264795482158661, -0.07075135409832001, -0.028251800686120987, -0.12614141404628754, -0.03541351482272148, 0.03780525550246239, -0.020794624462723732, 0.018093841150403023, -0.019461987540125847, 0.012972586788237095, 0.013985710218548775, -0.0715814083814621, -0.120781809091568, 0.18755526840686798, 0.22838175296783447, 0.07228084653615952, 0.2262607365846634, -0.014963416382670403, -0.08069387078285217, 0.09075763821601868, 0.19479183852672577, 0.21411018073558807, -0.33161208033561707, 0.2115091234445572, 0.3176116347312927, 0.32507196068763733, 0.01580744981765747, 0.1283867061138153, 0.20892421901226044, -0.16248895227909088, -0.10938379913568497, -0.06516765058040619, -0.025433380156755447, -0.09054699540138245, 0.15674805641174316, 0.3708741366863251, 0.24893908202648163, -0.10840857774019241, -0.11583930253982544, 0.05283229798078537, -0.07367600500583649, -0.16234193742275238, -0.050849247723817825, -0.005362804047763348, 0.1517857313156128, -0.06294229626655579, 0.03184156492352486, 0.33240649104118347, -0.16686196625232697, -0.2309904843568802, -0.14682623744010925, 0.11714927107095718}, {-0.02321990579366684, 0.1913926899433136, 0.013100184500217438, 0.05635708197951317, 0.049599289894104004, 0.13411551713943481, -0.113850899040699, 0.053004175424575806, 0.08788249641656876, -0.05945346876978874, 0.10594837367534637, 0.20643173158168793, 0.14714908599853516, 0.13148358464241028, -0.02759857475757599, -0.017892075702548027, 0.16270633041858673, -0.07711105048656464, 0.028445737436413765, -0.16782507300376892, -0.0506216436624527, 0.06644971668720245, 0.07275909185409546, -0.08488800376653671, -0.04506346955895424, -0.04222138598561287, 0.08553065359592438, -0.0006685034022666514, -0.09912828356027603, 0.057687126100063324, -0.30682864785194397, 0.09242243319749832, 0.3152080774307251, 0.035932041704654694, 0.05492065101861954, 0.10601130872964859, -0.005924981087446213, 0.175803080201149, 0.14945608377456665, 0.044062986969947815, 0.033933863043785095, 0.1922866404056549, 0.04377242922782898, -0.0580776073038578, 0.10522755980491638, 0.1457062065601349, -0.12245992571115494, 0.07187458872795105, 0.005240095779299736, -0.07200539112091064, 0.0895107239484787, 0.12312338501214981, 0.03917478770017624, -0.031362246721982956, -0.03868395835161209, -7.604782877024263e-05, -0.08563587069511414, -0.1236904039978981, 0.0847395658493042, 0.03150108456611633, -0.08809812366962433, 0.03842729702591896, 0.003931421786546707, 0.2096991091966629, -0.09930554032325745, 0.030747270211577415, -0.12856554985046387, 0.021099969744682312, 0.02442404255270958, 0.09656421840190887, 0.05825386196374893, 0.1479157656431198, -0.5782274603843689, -0.312333881855011, 0.01816735416650772, -0.04092870652675629, -0.021809963509440422, -0.1689770221710205, 0.12688498198986053, -0.3366612195968628, 0.019193535670638084, 0.06458628922700882, -0.009354773908853531, 0.04149089753627777, 0.09251119196414948, -0.016083166003227234, -0.3264036476612091, 0.30181312561035156, 0.18391083180904388, -0.08716792613267899, 0.03380151465535164, -0.14546051621437073, 0.09863422065973282, 0.07745994627475739, 0.15127286314964294, -0.03864669427275658, -0.037791620939970016, 8.221777534345165e-05, 0.18131351470947266, -0.05275498330593109, 0.04440005496144295, -0.03072718158364296, -0.009589089080691338, -0.059508323669433594, 0.25806331634521484, 0.1076713278889656, 0.08737839758396149, -0.04839996248483658, 0.013766518794000149, -0.015290162526071072, 0.18387484550476074, -0.24589987099170685, -0.16476282477378845, 0.055619124323129654, 0.0630311369895935, -0.10737434029579163, -0.039095163345336914, -0.07764005661010742, -0.04063321650028229, 0.06742418557405472, -0.03745470196008682, 0.11638957262039185, -0.15002894401550293, -0.033868856728076935, -0.03227344527840614, 0.06197391822934151, -0.06870503723621368, 0.028165750205516815, 0.09665727615356445, -0.06291825324296951, -0.046096812933683395, 0.12854836881160736, 0.5082565546035767, -0.044771868735551834, -0.05344920977950096, 0.021155381575226784, 0.032600037753582, -0.08572018891572952, -0.020281841978430748, 0.1331438571214676, 0.16961200535297394, -0.03428146615624428, -0.09149940311908722, 0.18345685303211212, 0.2952810525894165, -0.07064304500818253, -0.20041775703430176, 0.14328433573246002, -0.09394507855176926, -0.055550411343574524, -0.17931440472602844, 0.13172195851802826, 0.0054222955368459225, 0.03282289206981659, -0.09638550132513046, -0.08296890556812286, 0.03982282429933548, 0.2077273279428482, -0.024893101304769516, -0.07051686942577362, -0.032376114279031754, -0.043399982154369354, -0.08879078924655914, -0.04258975386619568, -0.002098886528983712, 0.13961289823055267, 0.03145425394177437, 0.20822052657604218, 0.10160169005393982, -0.10057585686445236, 0.04603301361203194, -0.007629851810634136, 0.09275586903095245, 0.11399677395820618, 0.05737329646945, 0.29985740780830383, 0.11070650070905685, -0.00804462842643261, -0.10459417849779129, -0.06196022778749466, 0.014651356264948845, 0.20670533180236816, 0.028631269931793213, -0.1461450159549713, 0.00271219783462584, -0.07431571930646896, 0.10444121062755585, -0.12795023620128632, -0.01708873175084591, -0.08068928122520447, 0.042414695024490356, 0.06943472474813461, -0.01870291493833065, 0.07594520598649979, -0.1903972178697586, -0.055641159415245056, 0.05746474489569664, 0.09210742264986038, -0.16231684386730194, 0.06566013395786285, -0.013988420367240906, -0.09036069363355637, 0.22242429852485657, -0.1717802733182907, -0.053282853215932846, 0.05196504667401314, 0.033798202872276306, 0.06955002248287201, -0.1008651852607727, 0.02767670899629593, -0.30192577838897705, 0.05337509885430336, -0.08004163950681686, 0.19119399785995483, 0.12876616418361664, 0.04581576585769653, -0.0103113679215312, -0.013886498287320137, -0.04469228535890579, 0.10309457033872604, -0.24087688326835632, -0.027478983625769615, 0.263262540102005, 0.19516153633594513, 0.08588224649429321, 0.14028345048427582, -0.3145526945590973, -0.04839770868420601, -0.08905088156461716, -0.015138702467083931, -0.018414977937936783, -0.17729032039642334, -0.004003401845693588, -0.06419216841459274, -0.06731319427490234, 0.22224576771259308, 0.2414560168981552, -0.0382099486887455, 0.03026915341615677, -0.13285064697265625, 0.08942971378564835, -0.06385244429111481, -0.02334119752049446, 0.04808136448264122, 0.1115143895149231, 0.06279391795396805, 0.13114312291145325, -0.017350127920508385, -0.1876499056816101, -0.2041671872138977, 0.03484116494655609, 0.0720440223813057, -0.08160265535116196, 0.14404524862766266, 0.24783127009868622, -0.1126544252038002}, {-0.02362050674855709, -0.016983700916171074, 0.04515285789966583, 0.06529594212770462, 0.032259080559015274, -0.18655070662498474, -0.04834495857357979, -0.39121943712234497, -0.09827545285224915, -0.044257719069719315, -0.0030069276690483093, 0.07297980040311813, 0.0966167077422142, -0.11138809472322464, -0.015697039663791656, 0.19270311295986176, 0.03749386593699455, -0.26973673701286316, 0.059827253222465515, -0.012474165298044682, 0.040946368128061295, 0.04028382897377014, 0.07650059461593628, -0.20573464035987854, -0.04502790421247482, -0.06987720727920532, 0.04034588113427162, -0.18710364401340485, 0.14081580936908722, 0.02686934918165207, 0.05287580564618111, 0.2873508930206299, -0.02650653012096882, -0.031738538295030594, 0.036872562021017075, 0.2072526216506958, 0.13174748420715332, -0.0072682942263782024, 0.16047944128513336, 0.03404967859387398, 0.13156798481941223, 0.03466246649622917, -0.05827353149652481, 0.07483348250389099, -0.1887923926115036, 0.10416539013385773, 0.11918070167303085, 0.010596026666462421, 0.18376898765563965, 0.06489728391170502, 0.1394285261631012, 0.32291656732559204, -0.018865926191210747, -0.07827676832675934, 0.1590002179145813, 0.07927040755748749, -0.11781644821166992, -0.06490728259086609, -0.00839692261070013, 0.15286226570606232, 0.11049389839172363, 0.08629952371120453, 0.14280347526073456, 0.34285396337509155, 0.05708572641015053, -0.37127065658569336, 0.0012054936960339546, 0.14874781668186188, 0.18539728224277496, 0.1083204448223114, -0.027465185150504112, -0.07443508505821228, -0.07365590333938599, 0.10228276252746582, -0.21939197182655334, -0.06481503695249557, -0.0748433768749237, 0.1030367836356163, -0.14387331902980804, -0.1930292844772339, -0.09765909612178802, -0.0061219544149935246, -0.04064178839325905, -0.046153388917446136, -0.4192316234111786, 0.10453353822231293, -0.015545924194157124, -0.035040680319070816, 0.08812626451253891, -0.0359378382563591, -0.0698859915137291, -0.3191833794116974, 0.05039082467556, -0.0002838315849658102, -0.02910534478724003, 0.1900903582572937, -0.1617317795753479, 0.024331368505954742, 0.12697669863700867, -0.1156461164355278, 0.021832630038261414, -0.2073182463645935, -0.0005934463697485626, 0.13248752057552338, 0.04106200858950615, 0.03676958382129669, 0.049406956881284714, -0.18841552734375, 0.2441411018371582, -0.06304813176393509, -0.19226092100143433, -0.01523925457149744, 0.0023248668294399977, 0.026907727122306824, -0.04546581208705902, 0.08627311140298843, -0.0447283498942852, -0.014496543444693089, -0.0031270761974155903, 0.17228248715400696, 0.14988546073436737, 0.18860207498073578, 0.2711566984653473, 0.16749051213264465, 0.00041655483073554933, -0.15892688930034637, -0.02265639416873455, -0.051218412816524506, -0.09881322830915451, 0.21331088244915009, -0.0648520439863205, -0.2476031333208084, 0.14416837692260742, 0.08617891371250153, 0.12887932360172272, 0.23991020023822784, -0.14841917157173157, -0.11090942472219467, 0.04394630715250969, 0.16956903040409088, -0.15536008775234222, 0.09385593980550766, 0.13210782408714294, -0.2768174409866333, -0.01964513771235943, -0.006116772536188364, -0.0745893344283104, 0.07831033319234848, -0.12373265624046326, 0.12515762448310852, 0.021646928042173386, -0.007157517597079277, -0.10566773265600204, -0.026243453845381737, -0.11702694743871689, 0.059292491525411606, 0.13862411677837372, 0.1262311190366745, -0.13463008403778076, 0.059358641505241394, 0.053420376032590866, -0.18603403866291046, -0.1782638430595398, 0.1403750330209732, -0.09377596527338028, 0.197669118642807, -0.1069120317697525, -0.047071900218725204, 0.023403536528348923, 0.1657664030790329, -0.10734249651432037, 0.0413513146340847, -0.036487750709056854, 0.09244363009929657, 0.19786447286605835, -0.16181500256061554, 0.007509192917495966, -0.010743353515863419, -0.035316552966833115, -0.002210664562880993, 0.017815211787819862, 0.1263960897922516, 0.08805955201387405, -0.14768488705158234, -0.024648524820804596, 0.11027746647596359, 0.19190791249275208, 0.060787033289670944, 0.21367032825946808, 0.0846087858080864, 0.054358627647161484, 0.04872621223330498, 0.009192854166030884, 0.137368306517601, -0.3317265808582306, -0.09665709733963013, -0.050935421139001846, 0.00078946741996333, -0.053027309477329254, -0.13084475696086884, -0.007660473231226206, 0.22251856327056885, 0.054110944271087646, -0.053076185286045074, -0.07381448894739151, 0.10138045251369476, -0.12854212522506714, -0.150309756398201, 0.08574610948562622, -0.41573870182037354, -0.0436229407787323, 0.15778139233589172, -0.10582828521728516, 0.1998603790998459, -0.08601489663124084, 0.21398550271987915, 0.021480416879057884, -0.09134062379598618, -0.055739499628543854, -0.181212916970253, 0.31790629029273987, 0.10417213290929794, -0.059908248484134674, 0.25862592458724976, 0.05374404788017273, 0.1429554671049118, -0.1595873385667801, -0.43628576397895813, 0.04614289849996567, -0.16220463812351227, 0.09645289927721024, 0.031827833503484726, -0.12881077826023102, -0.0050627305172383785, -0.048199933022260666, -0.20290999114513397, 0.031194377690553665, -0.09586609899997711, 0.18089282512664795, -0.033169928938150406, -0.04611987993121147, -0.3019619882106781, 0.11583881825208664, -0.017579806968569756, 0.047742754220962524, -0.2140493541955948, -0.16966596245765686, -0.14499053359031677, 0.08867212384939194, -0.008940301835536957, 0.06918046623468399, 0.04741745442152023, 0.12408091127872467, -0.0679343119263649, -0.06634429097175598, -0.05519114434719086}, {-0.14530082046985626, 0.015381956472992897, -0.06238219514489174, 0.02879389002919197, 0.1515098661184311, -0.06074579060077667, -0.09038519114255905, -0.01062875147908926, 0.18726596236228943, -0.17618000507354736, -0.1461150199174881, -0.15460771322250366, 0.1075381487607956, -0.2350994497537613, 0.3104941248893738, -0.07522765547037125, -0.06164563447237015, 0.09190516918897629, -0.20194850862026215, 0.04201759025454521, -0.025202676653862, 0.06156250834465027, -0.12665705382823944, 0.20409314334392548, 0.03204381465911865, 0.04185380041599274, -0.3905722200870514, 0.07159320265054703, 0.2375410944223404, -0.06653685867786407, -0.06567132472991943, -0.017680125311017036, 0.33024704456329346, -0.11282787472009659, -0.03354215994477272, -0.035692233592271805, -0.1342010796070099, -0.0963476225733757, 0.11888784915208817, -0.10326244682073593, 0.1352710723876953, 0.2656346559524536, 0.11410337686538696, -0.05830604210495949, -0.1278345137834549, -0.0923943817615509, 0.08926673978567123, 0.08254318684339523, -0.11942482739686966, 0.049299322068691254, -0.05556197836995125, -0.2716148793697357, 0.09193582832813263, 0.28988853096961975, -0.07936493307352066, 0.0992804691195488, 0.16015750169754028, 0.12725961208343506, 0.08655165135860443, 0.040463317185640335, 0.19031721353530884, 0.09991943091154099, -0.07184462249279022, 0.4365783929824829, 0.11762115359306335, 0.31910666823387146, 0.10470574349164963, -0.05424266681075096, 0.0210653655230999, 0.2592126429080963, -0.09249691665172577, 0.00267151091247797, 0.048189159482717514, 0.09871935099363327, -0.09855599701404572, 0.12616360187530518, -0.05836694315075874, 0.037446361035108566, 0.18464207649230957, 0.22871115803718567, 0.06133733689785004, -0.15434233844280243, 0.04555996507406235, -0.23398394882678986, -0.13524314761161804, 0.18755510449409485, 0.07047004997730255, 0.0868915244936943, -0.067610003054142, 0.03612339869141579, -0.11302994191646576, 0.006169802509248257, 0.09208158403635025, -0.029043221846222878, 0.18874290585517883, 0.03631654009222984, 0.112870953977108, -0.3550078868865967, -0.20622099936008453, 0.025519218295812607, -0.06757191568613052, 0.037099964916706085, -0.3386244475841522, 0.0029468992725014687, 0.4078061878681183, -0.17494824528694153, -0.03397213667631149, 0.13541677594184875, -0.08104435354471207, 0.10494042187929153, 0.03287094086408615, 0.21872884035110474, -0.05905074626207352, 0.08637946844100952, -0.0678444430232048, 0.013724124059081078, 0.2445504516363144, 0.054543279111385345, -0.16187915205955505, -0.2703113257884979, 0.13194234669208527, 0.2898517847061157, 0.052334874868392944, 0.012115824036300182, -0.04065144434571266, -0.48005980253219604, 0.12413214147090912, -0.04359803721308708, -0.0063681225292384624, -0.3122587502002716, -0.057101089507341385, -0.14642708003520966, -0.238767609000206, -0.040747467428445816, 0.0007684651645831764, 0.08764815330505371, 0.11797499656677246, 0.21476340293884277, 0.0148489223793149, 0.1970846951007843, -0.01908165030181408, 0.11883184313774109, 0.1718410849571228, 0.16949491202831268, 0.04388107731938362, 0.05608347803354263, 0.02082236483693123, -0.09287698566913605, -0.043498847633600235, 0.05810195580124855, 0.03312459588050842, -0.4044930338859558, 0.12065280228853226, 0.02909008599817753, -0.24472297728061676, 0.1149793490767479, -0.1631830483675003, 0.24036724865436554, 0.3684995174407959, -0.16104833781719208, -0.01557269599288702, 0.016941722482442856, 7.422662747558206e-05, -0.016007075086236, 0.02233070507645607, -0.0361219123005867, -0.013119985349476337, -0.04512472078204155, -0.09545330703258514, -0.23286649584770203, 0.18668097257614136, -0.13360272347927094, -0.19333785772323608, 0.035607364028692245, -0.21459278464317322, -0.06698374450206757, 0.06725437194108963, -0.12695786356925964, 0.13066066801548004, 0.020854566246271133, -0.00422263890504837, 0.015195410698652267, -0.07693574577569962, 0.06566672027111053, -0.021927671507000923, -0.031048808246850967, -0.29606783390045166, -0.06395916640758514, 0.05565255880355835, -0.040196433663368225, -0.15831705927848816, -0.1604839265346527, -0.24026921391487122, -0.1486508846282959, 0.16999474167823792, -0.23974426090717316, -0.0687779113650322, 0.1817322075366974, -0.09538223594427109, 0.17646390199661255, 0.21619057655334473, 0.07612350583076477, -0.12933142483234406, -0.09489022940397263, 0.2111969143152237, 0.0059942142106592655, 0.07748506963253021, -0.05303308740258217, 0.16089260578155518, -0.19310325384140015, 0.1057971939444542, -0.13331620395183563, 0.20728078484535217, -0.11762159317731857, -0.3588656783103943, 0.011071845889091492, 0.23161005973815918, -0.028751470148563385, 0.1187194436788559, -0.04853018373250961, 0.04017859697341919, -0.011704209260642529, 0.09248887747526169, -0.24524907767772675, -0.10249227285385132, -0.12117251008749008, -0.12483347952365875, -0.11796785145998001, -0.1894829273223877, 0.03516968712210655, 0.16976067423820496, 0.0017711501568555832, -0.050308119505643845, -0.05737951025366783, 0.1407216489315033, -0.05010409653186798, 0.02721572481095791, 0.10215958952903748, 0.04292312636971474, -0.1561552733182907, 0.028435900807380676, -0.03897618129849434, -0.09826340526342392, -0.3156852126121521, -0.13929469883441925, 0.23309572041034698, 0.0380479097366333, -0.16384130716323853, 0.007909520529210567, -0.04740310087800026, -0.04047907143831253, 0.22452419996261597, -0.08318295329809189, -0.014196052215993404, -0.1946713924407959, -0.1874035894870758}, {0.006539513822644949, -0.11900916695594788, -0.1942799687385559, -0.021861035376787186, 0.04743511974811554, -0.07275451719760895, 0.035153135657310486, 0.024386601522564888, -0.05624235421419144, 0.25994184613227844, -0.19623270630836487, 0.17945951223373413, -0.017596108838915825, -0.03010520525276661, 0.0502438098192215, 0.20293490588665009, 0.07428082823753357, -0.09962248802185059, -0.16036194562911987, -0.040954019874334335, 0.3677942454814911, -0.01044632587581873, -0.07094121724367142, 0.029540063813328743, 0.19382712244987488, 0.11610348522663116, -0.07251649349927902, -0.010968898423016071, 0.16000930964946747, 0.05151663348078728, -0.05813714489340782, -0.12417830526828766, 0.21082106232643127, -0.06884673237800598, 0.03975945711135864, -0.03835979476571083, -0.09832670539617538, -0.031637195497751236, -0.1378558725118637, 0.12506261467933655, 0.06145083159208298, -0.09424297511577606, 0.025076229125261307, -0.1815844625234604, 0.03310766443610191, -0.13056659698486328, 0.08642034232616425, 0.07017739862203598, -0.05285234749317169, 0.17812691628932953, 0.030634107068181038, -0.00022720776905771345, -0.00649680569767952, 0.047277845442295074, 0.028637118637561798, 0.19786447286605835, -0.022585256025195122, 0.011345124803483486, 0.12728875875473022, -0.04726814478635788, -0.1441212296485901, -0.010583396069705486, -0.09096899628639221, 0.037172067910432816, 0.1459290087223053, 0.008203869685530663, -0.017806004732847214, 0.037364959716796875, 0.09421035647392273, 0.07019374519586563, -0.07998953759670258, -0.14828789234161377, -0.013037888333201408, -0.21203191578388214, -0.17830903828144073, 0.008205387741327286, -0.1396072804927826, -0.3785310685634613, -0.14737574756145477, 0.2866381108760834, 0.16127513349056244, 0.023615024983882904, -0.009120402857661247, -0.20384426414966583, 0.17605112493038177, -0.1168418750166893, 0.001639347756281495, 0.01441974937915802, 0.010080487467348576, -0.02320702001452446, 0.058574043214321136, -0.1429077684879303, -0.0315125435590744, -0.07680369168519974, 0.10325895249843597, 0.26488640904426575, -0.03172994405031204, 0.1005985289812088, 0.06164662167429924, 0.006423580460250378, 0.10467629134654999, -0.09053654968738556, 0.14922387897968292, 0.027381323277950287, 0.029046157374978065, -0.9170757532119751, -0.1964779943227768, 0.08824902772903442, 0.08151321858167648, 0.03724101558327675, 0.09968552738428116, -0.11360479891300201, -0.09296117722988129, -0.158843532204628, 0.07246672362089157, -0.06248495355248451, 0.002245897427201271, 0.03596924617886543, 0.07250794023275375, 0.030198832973837852, 0.09618484228849411, 0.19350427389144897, -0.010974840261042118, -0.029620805755257607, -0.08211503177881241, 0.002779528498649597, -0.16154471039772034, 0.10633347928524017, 0.09503617882728577, -0.1743614375591278, 0.010343853384256363, 0.0732257068157196, 0.03395934775471687, 0.18701182305812836, 0.08778231590986252, -0.015773538500070572, -0.027031555771827698, 0.05103061720728874, -0.1561855524778366, 0.06256362795829773, -0.2856270372867584, 0.013804716058075428, 0.011863207444548607, 0.10658910870552063, 0.07090631872415543, -0.04594368487596512, -0.14035405218601227, 0.0488428920507431, 0.08048827946186066, -0.0932195708155632, -0.020246610045433044, 0.10274534672498703, -0.04507986456155777, 0.0577247217297554, -0.04304344952106476, 0.07074584811925888, 0.01765453815460205, 0.3153574466705322, 0.08909988403320312, 0.09909404814243317, 0.1343323439359665, -0.025916174054145813, 0.042360227555036545, 0.10933547466993332, -0.08108275383710861, 0.013511032797396183, 0.05061325430870056, 0.07182498276233673, -0.026555193588137627, -0.026423446834087372, 0.14603827893733978, -0.09498865902423859, -0.19530528783798218, -0.18889091908931732, -0.0634712278842926, -0.1156327947974205, -0.2071516513824463, -0.023228153586387634, 0.10611454397439957, 0.17253649234771729, -0.06826946884393692, 0.12539523839950562, 0.1870969831943512, -0.13325323164463043, 0.18206797540187836, 0.10037214308977127, -0.15637758374214172, -0.03074004128575325, -0.08746881037950516, 0.07436003535985947, -0.09132832288742065, 0.05285629257559776, -0.130000501871109, -0.17501546442508698, 0.02674570493400097, -0.09439803659915924, -0.03217089921236038, 0.1430293619632721, 0.08945633471012115, 0.08145265281200409, 0.14042697846889496, 0.05099932476878166, -0.07897757738828659, 0.11543016135692596, 0.09694517403841019, 0.14380010962486267, -0.283754825592041, 0.1216726303100586, 0.0314716212451458, -0.05717069283127785, 0.002826168667525053, -0.08344337344169617, 0.05464499071240425, -0.05604497715830803, 0.12611240148544312, -0.06265800446271896, 0.12072627991437912, -0.19081075489521027, 0.14825879037380219, -0.05544546619057655, 0.025141796097159386, -0.04185193404555321, 0.07353103905916214, 0.010191877372562885, -0.06179383397102356, 0.013650832697749138, -0.1349838227033615, 0.0971330925822258, 0.16941037774085999, 0.04698110744357109, -0.09042874723672867, -0.04047773405909538, -0.11052112281322479, -0.06636367738246918, -0.023925378918647766, -0.030218755826354027, -0.11041915416717529, -0.03861631825566292, -0.11364675313234329, -0.158396914601326, 0.025147298350930214, -0.15629026293754578, 0.06560179591178894, 0.16438868641853333, 0.102009616792202, 0.0018509867368265986, 0.23462605476379395, -0.08948904275894165, 0.13336986303329468, 0.07634079456329346, -0.09864339977502823, 0.0003583848010748625, -0.03268945589661598, -0.019390976056456566, -0.12262707203626633, 0.07652978599071503}, {-0.0053254649974405766, -0.018132321536540985, -0.08944746851921082, -0.21739131212234497, 0.06771130859851837, -0.06733470410108566, 0.15945425629615784, 0.08991219848394394, -0.021351588889956474, -0.05936482548713684, 0.20286288857460022, -0.01907714270055294, -0.013734041713178158, 0.19505293667316437, -0.08389659970998764, -0.07229055464267731, -0.1474597305059433, -0.048617277294397354, 0.02648809179663658, 0.08739472180604935, -0.021239886060357094, -0.09624875336885452, 0.14361955225467682, -0.12846849858760834, 0.11327114701271057, 0.08661871403455734, 0.03255679830908775, 0.2159239947795868, -0.29620933532714844, 0.06277734786272049, 0.1661594957113266, -0.1121784970164299, -0.13711677491664886, -0.08007995784282684, -0.14538788795471191, 0.02760395221412182, 0.13389535248279572, 0.09616977721452713, 0.20205120742321014, -0.02888326719403267, 0.031721726059913635, -0.03772081434726715, 0.2006765604019165, 0.07499384135007858, 0.17291468381881714, 0.08969895541667938, -0.053758326917886734, -0.02030368149280548, -0.07238497585058212, -0.13587751984596252, 0.0288267582654953, -0.07826750725507736, -0.214178204536438, 0.20776064693927765, -0.023125717416405678, -0.09423308819532394, -0.22675634920597076, -0.318501353263855, 0.044586606323719025, 0.027417073026299477, -0.34118154644966125, -0.041514284908771515, -0.09807215631008148, 0.16185733675956726, -0.04191046953201294, 0.1406838297843933, -0.25318923592567444, -0.30731603503227234, -0.024426555261015892, -0.03227873891592026, -0.1458558291196823, 0.08293214440345764, -0.027545886114239693, -0.005960593465715647, -0.1584242582321167, 0.1786920428276062, 0.05965124070644379, 0.03996196761727333, -0.044160112738609314, 0.09076496213674545, 0.10887155681848526, 0.3144625723361969, 0.10607071965932846, 0.20675331354141235, 0.016201678663492203, -0.031624969094991684, 0.016048405319452286, -0.03251506760716438, 0.26512202620506287, 0.24668189883232117, -0.06902840733528137, 0.0037740168627351522, -0.03983401879668236, -0.08702075481414795, -0.1184113621711731, -0.06637811660766602, -0.04027985781431198, -0.11379799246788025, 0.12588335573673248, 0.25162574648857117, -0.047069139778614044, -0.14847791194915771, 0.24537712335586548, -0.2190295308828354, -0.16666994988918304, 0.08644304424524307, -0.12335164099931717, 0.007491614203900099, 0.12309839576482773, 0.06309869140386581, 0.03987734392285347, 0.06718558818101883, -0.16250388324260712, 0.17423832416534424, 0.17463040351867676, 0.03369547426700592, -0.12620407342910767, 0.04541228339076042, 0.0005778064369224012, 0.2697654366493225, 0.11379124224185944, -0.11087705940008163, -0.0019921238999813795, 0.06309600919485092, -0.1565408706665039, 0.08131860941648483, 0.0330435149371624, 0.0006167176179587841, -0.12055128812789917, -0.053069472312927246, 0.12805436551570892, 0.023067690432071686, -0.04863238334655762, -0.13250352442264557, 0.1818455159664154, 0.43209755420684814, -0.013319081626832485, -0.048704635351896286, 0.2447648048400879, -0.05924876406788826, 0.0578678734600544, 0.0754326730966568, -0.1300498992204666, -0.08623819798231125, -0.03848281875252724, -0.26994773745536804, 0.16819682717323303, 0.04255041852593422, 0.23613236844539642, 0.15698227286338806, 0.019056405872106552, 0.03768375143408775, -0.07634235173463821, -0.12711478769779205, -0.18857212364673615, -0.076400026679039, -0.2285664975643158, 0.011693031527101994, 0.15989567339420319, -0.0001276469265576452, -0.01950801908969879, -0.19048458337783813, 0.05959150567650795, -0.0939449667930603, -0.14437037706375122, -0.038970403373241425, -0.01723744161427021, 0.005865554790943861, 0.02930201217532158, -0.13319170475006104, -0.06527937948703766, 0.20002281665802002, 0.16236978769302368, 0.01536592561751604, -0.008988789282739162, 0.014958464540541172, 0.14232854545116425, 0.20333757996559143, 0.014753203839063644, -0.09673374891281128, 0.09584396332502365, 0.004975170362740755, 0.14650480449199677, 0.1583867371082306, -0.026719415560364723, 0.03309480473399162, 0.39361268281936646, 0.0921073630452156, 0.19605225324630737, -0.13115960359573364, 0.0677126944065094, 0.008449504151940346, 0.06059185415506363, 0.11065762490034103, -0.08966543525457382, -0.0417487770318985, 0.0569843165576458, -0.0526684895157814, -0.1435476541519165, -0.14562423527240753, -0.23523157835006714, 0.17071028053760529, -0.06777417659759521, -0.06955726444721222, 0.059427518397569656, 0.21531398594379425, 0.03857564181089401, 0.06474262475967407, -0.006071086041629314, -0.03263722360134125, 0.08987096697092056, 0.3109954297542572, -0.027529573068022728, -0.06929546594619751, 0.11873205006122589, 0.019457098096609116, 0.055121760815382004, 0.012187414802610874, 0.08517897129058838, 0.032257843762636185, 0.11282406747341156, -0.03238177299499512, -0.027053609490394592, 0.16844354569911957, -0.06825637817382812, -0.022067217156291008, -0.12376796454191208, -0.04416581615805626, -0.17531892657279968, 0.19886894524097443, 0.10967057943344116, -0.10427837818861008, -0.04053916037082672, 0.03639688715338707, 0.03778015449643135, -0.08307521045207977, -0.11784569174051285, 0.014209468849003315, 0.18098139762878418, 0.02038370631635189, 0.11052010953426361, -0.18713949620723724, -0.3058623671531677, -0.20599907636642456, -0.011045137420296669, -0.14237874746322632, -0.15590690076351166, 0.1774934083223343, -0.17679272592067719, 0.009646986611187458, 0.15375670790672302, -0.05807608366012573, 0.07165324687957764, 0.020982708781957626, -0.01424773782491684, -0.030940894037485123}, {0.10179007053375244, -0.01328347995877266, -0.127025306224823, -0.07260677218437195, -0.0502571240067482, -0.18372264504432678, 0.06559924781322479, -0.19161900877952576, -0.07112041860818863, 0.1174604594707489, -0.07055710256099701, 0.1266065090894699, -0.016988370567560196, -0.07257828861474991, 0.02936078980565071, 0.012446011416614056, -0.30229461193084717, -0.06607020646333694, 0.08903324604034424, 0.04084538295865059, 0.1390429437160492, 0.08043862879276276, -0.03610739856958389, -0.1058855727314949, -0.022869274020195007, -0.023293370380997658, 0.03520289808511734, 0.1753062903881073, -0.005103526636958122, -0.08829952031373978, 0.23659271001815796, 0.05931788682937622, -0.1332108974456787, 0.06940796971321106, -0.15421517193317413, 0.1746165156364441, 0.07877704501152039, -0.25725504755973816, 0.029830852523446083, -0.08290247619152069, 0.0716000646352768, 0.1009836420416832, 0.03775781765580177, 0.084903784096241, 0.24712839722633362, -0.020907605066895485, 0.1108657643198967, 0.2604934573173523, 0.10238050669431686, -0.09135067462921143, 0.1984211951494217, 0.024590902030467987, 0.08172868937253952, -0.09655716270208359, -0.012868653982877731, 0.14663228392601013, 0.14163464307785034, 0.3287543058395386, -0.1307378113269806, 0.039696104824543, 0.08063536137342453, 0.0036898960825055838, -0.023561488837003708, -0.03986014425754547, -0.11931224912405014, 0.04951798915863037, -0.045616086572408676, -0.05960267782211304, -0.028803111985325813, -0.004193906206637621, -0.015255200676620007, -0.06707915663719177, 0.11711059510707855, -0.1150803491473198, 0.1628599762916565, 0.07493392378091812, -0.18063734471797943, 0.5452595353126526, 0.06690359860658646, -0.18763327598571777, -0.11840401589870453, 0.06578018516302109, 0.21474015712738037, 0.1367696076631546, -0.2758522033691406, 0.1705448031425476, -0.04479515925049782, 0.10757651180028915, -0.005413256585597992, -0.020200449973344803, 0.06904854625463486, 0.23675163090229034, -0.0323098748922348, 0.11699141561985016, 0.13372033834457397, 0.12337411195039749, -0.042441289871931076, 0.01574561558663845, -0.13692687451839447, 0.15064015984535217, -0.05561027675867081, -0.13059291243553162, 0.017593013122677803, -0.03658078983426094, -0.31873318552970886, 0.09312032163143158, -0.012860494665801525, -0.04154133424162865, -0.05614927038550377, 0.026631353422999382, -0.2100609838962555, 0.20403625071048737, 0.0654846727848053, 0.07165124267339706, -0.007141427136957645, 0.5281258821487427, 0.030100705102086067, 0.026458263397216797, -0.0967760682106018, 0.11353368312120438, 0.05103589966893196, -0.13668498396873474, 0.13034549355506897, 0.20962433516979218, 0.11277072876691818, -0.26470908522605896, 0.14851704239845276, 0.17467428743839264, -0.11047278344631195, 0.12529590725898743, -0.13095559179782867, -0.2010234296321869, 0.018765203654766083, 0.1022971048951149, -0.1629946082830429, 0.13997529447078705, -0.10183163732290268, 0.1218305230140686, -0.11315040290355682, -0.3401561975479126, -0.005133575294166803, 0.09652437269687653, 0.2725975215435028, -0.019619066268205643, -0.045601483434438705, -0.059223491698503494, 0.21832901239395142, -0.02742248773574829, 0.05431694909930229, -0.13114136457443237, 0.020832164213061333, 0.1930653303861618, 0.089035265147686, -0.021808438003063202, -0.12113019078969955, -0.04160861298441887, -0.013763162307441235, -0.12379109859466553, 0.10588550567626953, -0.14136521518230438, 0.220773383975029, -0.09831748902797699, -0.21509861946105957, -0.15634186565876007, 0.037954263389110565, 0.14047469198703766, -0.023375539109110832, 0.0009067211067304015, 0.18039633333683014, 0.4353410005569458, -0.08978330343961716, -0.08938595652580261, 0.18566261231899261, 0.17445893585681915, -0.036312952637672424, 0.11651057749986649, 0.04599347338080406, -0.05991094559431076, 0.005249603185802698, -0.04040493071079254, -0.06189686805009842, 0.05785154178738594, -0.11922653019428253, 0.04991255700588226, -0.14134244620800018, -0.07030877470970154, -0.11878975480794907, -0.09315207600593567, 0.013839363120496273, 0.011744479648768902, 0.14524687826633453, -0.031136440113186836, -0.0192263126373291, -0.2052595019340515, -0.2229154109954834, 0.06584107130765915, -0.11186228692531586, -0.07501278817653656, -0.15963993966579437, 0.17701546847820282, -0.07096271216869354, 0.16459470987319946, -0.14389954507350922, 0.0023517527151852846, -0.09124325215816498, 0.2863852083683014, -0.1451970338821411, -0.2089090347290039, -0.046526066958904266, -0.11007004976272583, -0.04066953435540199, -0.10674494504928589, 0.09950148314237595, -0.018155712634325027, -0.009798425249755383, -0.07784426957368851, -0.04994107410311699, 0.03496461734175682, -0.11519883573055267, -0.04361291974782944, 0.17287753522396088, 0.23450876772403717, -0.05166597291827202, -0.13899259269237518, -0.10336242616176605, -0.08915619552135468, 0.24197831749916077, -0.06583444029092789, 0.03702392056584358, 0.15050533413887024, -0.06376441568136215, -0.06476674973964691, -0.014774213545024395, 0.04857410117983818, 0.09382147341966629, 0.08689261972904205, -0.20441053807735443, -0.2510579824447632, 0.09687181562185287, 0.08753900974988937, 0.2621486186981201, -0.03370150551199913, -0.10212244093418121, -0.009688816964626312, 0.12502123415470123, -0.1477971374988556, -0.15343411266803741, -0.05064205080270767, -0.1436292976140976, -0.39412152767181396, -0.01776145026087761, 0.2655647397041321, 0.03050599806010723, 0.06635307520627975, 0.0013888588873669505, -0.006117492914199829}, {-0.05472161993384361, -0.047124482691287994, -0.36514273285865784, 0.012741828337311745, 0.07648731768131256, -0.02002802863717079, 0.1604607254266739, 0.03350457176566124, -0.16115674376487732, 0.02667166292667389, 0.0595904216170311, -0.29529571533203125, -0.09460246562957764, -0.11126040667295456, -0.17608143389225006, 0.013505000621080399, 0.05239629000425339, 0.20752324163913727, -0.13049206137657166, -0.16318738460540771, 0.04451235011219978, -0.06767536699771881, 0.11009498685598373, 0.16986870765686035, 0.13598786294460297, 0.022072847932577133, -0.15931516885757446, 0.010387277230620384, 0.03473864868283272, -0.10330653190612793, 0.028985921293497086, -0.17294207215309143, 0.11351949721574783, -0.13148167729377747, -0.14584238827228546, 0.21100832521915436, 0.013981524854898453, 0.26771825551986694, 0.18047088384628296, 0.09240905940532684, 0.24524112045764923, -0.042967941612005234, -0.11570260673761368, -0.0338314026594162, 0.006126320455223322, -0.23161005973815918, -0.061373770236968994, 0.00027881894493475556, 0.0744619369506836, -0.053135309368371964, 0.002370847389101982, 0.14667291939258575, -0.07178884744644165, 0.05532427132129669, -0.08204682916402817, 0.12891916930675507, 0.29981255531311035, -0.16165053844451904, -0.10008072853088379, 0.20613665878772736, 0.06578349322080612, 0.057531967759132385, 0.0019424932543188334, 0.1874653846025467, 0.16265694797039032, 0.058850325644016266, 0.07902778685092926, -0.0541200116276741, -0.06903054565191269, -0.14246654510498047, -0.07068517804145813, 0.04472387954592705, -0.05792323872447014, -0.039360761642456055, 0.17511752247810364, 0.09900122135877609, -0.15664619207382202, 0.05380668491125107, 0.04229985922574997, -0.025030173361301422, 0.0589061863720417, 0.09652115404605865, -0.06505761295557022, 0.003500634338706732, 0.032697033137083054, 0.04091513901948929, -0.09168495982885361, -0.27000388503074646, 0.00482197105884552, -0.09909527748823166, -0.12748286128044128, -0.037894219160079956, -0.23428775370121002, -0.06430686265230179, 0.08249557018280029, 0.0456886850297451, -0.024642936885356903, -0.29605165123939514, 0.04608084261417389, -0.18533307313919067, -0.03430115059018135, -0.10240491479635239, 0.02959042228758335, -0.05405686795711517, 0.07378272712230682, 0.28862619400024414, 0.11095904558897018, 0.04245408624410629, 0.0851205587387085, 0.009731624275445938, 0.0984087735414505, -0.08342444151639938, -0.05140741169452667, -0.023717490956187248, -0.07847883552312851, 0.06099966540932655, 0.020487911999225616, 0.0035167515743523836, 0.1291218101978302, 0.09559061378240585, 0.1347152590751648, 0.02099437080323696, 0.008154620416462421, 0.15071745216846466, 0.042834747582674026, 0.21946749091148376, 0.0368909053504467, -0.03841382637619972, 0.017527207732200623, -0.1164543554186821, 0.10637204349040985, -0.017745036631822586, 0.0700904056429863, -0.2679414749145508, 0.21437028050422668, -0.04887239634990692, -0.010156073607504368, -0.04653134569525719, 0.02219334989786148, 0.13576917350292206, 0.0632442757487297, -0.025023972615599632, 0.08966971933841705, -0.06972530484199524, 0.024964038282632828, -0.07314906269311905, 0.21085676550865173, 0.08075513690710068, 0.17267724871635437, 0.06774511933326721, 0.06455676257610321, -0.05937977880239487, 0.09146032482385635, -0.03514557704329491, -0.02311915159225464, 0.17192651331424713, 0.00010238796676276252, 0.09402818232774734, 0.18073506653308868, 0.016183646395802498, -0.12452216446399689, 0.2120843231678009, -0.11569128930568695, -0.011055774055421352, -0.09256341308355331, 0.0644068494439125, -0.11433560401201248, 0.1867755651473999, 0.06424744427204132, -0.10098052024841309, 0.033695898950099945, -0.050207749009132385, -0.0042238663882017136, 0.14407598972320557, 0.188656285405159, 0.030744588002562523, 0.010596157982945442, -0.04766880348324776, -0.05917234346270561, 0.28093278408050537, -0.07504185289144516, -0.2691879868507385, 0.029612386599183083, 0.029489457607269287, -0.05870593339204788, 0.2119058072566986, 0.2895931303501129, 0.085635706782341, 0.21362459659576416, -0.09038890898227692, 0.038386691361665726, -0.16885283589363098, -0.19798937439918518, 0.06331383436918259, -0.06449457257986069, 0.04268147051334381, -0.18508732318878174, -0.08185131102800369, 0.10811547189950943, -0.06219649314880371, -0.09421044588088989, 0.05255522578954697, 0.21415671706199646, -0.19701199233531952, 0.018981460481882095, -0.15461717545986176, -0.2302173227071762, -0.13272447884082794, -0.12075427919626236, -0.19392387568950653, -0.0914706438779831, 0.059441275894641876, -0.14255768060684204, -0.13586334884166718, -0.05602928251028061, -0.05456820875406265, 0.0684383437037468, 0.09216991066932678, -0.09634025394916534, 0.11532718688249588, 0.07126563042402267, -0.012470929883420467, -0.025439251214265823, 0.08038369566202164, 0.009515891782939434, -0.06070515513420105, -0.024865256622433662, -0.04794887825846672, 0.021893590688705444, -0.23405537009239197, 0.07385264337062836, 0.020024288445711136, -0.19681930541992188, 0.1276538223028183, -0.08670520782470703, 0.05939765274524689, 0.010175252333283424, 0.08516945689916611, -0.05134008079767227, 0.13677318394184113, -0.10314612835645676, 0.27236685156822205, -0.2776978611946106, 0.023052344098687172, 0.10882348567247391, -0.050060614943504333, 0.0856143906712532, -0.19305121898651123, -0.004408544860780239, -0.0011062419507652521, 0.09638530015945435, 0.06766398251056671, 0.09039865434169769, -0.13283228874206543, -0.11418823152780533, -0.27525147795677185}, {-0.03068852610886097, -0.09104639291763306, -0.19175858795642853, 0.05646101385354996, -0.2120170146226883, 0.0457284115254879, 0.10780374705791473, -0.13736608624458313, 0.033708520233631134, -0.005730662494897842, -0.020462239161133766, -0.42123374342918396, 0.06138414889574051, -0.02869439125061035, -0.20347324013710022, -0.03094922937452793, -0.14832110702991486, 0.05539781227707863, 0.2310783714056015, 0.023306965827941895, -0.00532693462446332, -0.014053664170205593, -0.015957484021782875, -0.16253013908863068, 0.11114835739135742, -0.07417906075716019, 0.008323037065565586, 0.11524507403373718, 0.20297059416770935, -0.06531332433223724, -0.12531836330890656, 0.0601537749171257, 0.091548852622509, -0.053371019661426544, -0.13191135227680206, -0.002225190866738558, -0.12330986559391022, 0.26939964294433594, 0.22214198112487793, 0.024886950850486755, 0.01681833155453205, -0.10872142016887665, -0.036441728472709656, 0.027780234813690186, -0.024541795253753662, -0.050865065306425095, -0.14816468954086304, 0.1438222974538803, -0.02170947752892971, -0.12013441324234009, -0.11837423592805862, 0.05318130925297737, -0.04125913232564926, -0.13735851645469666, -0.057972948998212814, 0.02260127291083336, -0.13658158481121063, -0.11384367197751999, -0.07839040458202362, 0.01905778981745243, 0.11152507364749908, 0.24646365642547607, 0.09950120747089386, 0.1716853678226471, -0.0975310355424881, -0.08648703992366791, -0.0887107402086258, -0.021645881235599518, -0.054253045469522476, 0.21408438682556152, -0.09589561820030212, -0.1993740200996399, -0.17951488494873047, -0.1342141479253769, -0.07574691623449326, -0.04248114302754402, -0.16177819669246674, 0.06284046173095703, 0.10297324508428574, 0.056107282638549805, -0.1224587932229042, -0.08344116061925888, -0.13468149304389954, -0.11186409741640091, 0.007130587473511696, 0.05501779541373253, -0.2636815309524536, 0.06330885738134384, -0.24903830885887146, -0.10922112315893173, -0.09254401177167892, -0.18164286017417908, 0.1315600425004959, -0.09180330485105515, 0.011358463205397129, -0.04633217677474022, -0.04615139961242676, -0.02719508856534958, 0.11363927274942398, -0.05114862695336342, -0.06381967663764954, -0.10854280740022659, 0.19549523293972015, -0.08465076237916946, 0.28466543555259705, 0.03275885805487633, 0.03613932430744171, -0.019950205460190773, 0.009508570656180382, -0.026383982971310616, -0.017178811132907867, -0.23216301202774048, 0.2980673909187317, 0.054391130805015564, -0.057790592312812805, 0.12580670416355133, -0.08594253659248352, 0.11740551888942719, -0.09696981310844421, 0.03421425446867943, 0.20102068781852722, 0.08333459496498108, -0.048948734998703, 0.03617064654827118, 0.16238617897033691, 0.15191154181957245, 0.08979615569114685, -0.06689675897359848, 0.23678040504455566, 0.11417011171579361, -0.0028455837164074183, -0.13817346096038818, 0.21076342463493347, -0.12384477257728577, 0.36141538619995117, -0.05719996988773346, 0.15892930328845978, -0.04683703929185867, 0.11687003821134567, -0.12505315244197845, -0.09154558181762695, -0.09238456189632416, 0.0409652441740036, -0.03778734430670738, 0.048860348761081696, -0.016112595796585083, -0.02381202206015587, 0.048028040677309036, -0.02994898520410061, 0.29273721575737, 0.036146149039268494, -0.13715483248233795, 0.04272782802581787, -0.03955958038568497, -0.11519236862659454, -0.0032180470880120993, -0.22493252158164978, 0.2830231785774231, -0.295132040977478, 0.07823924720287323, -0.053275782614946365, 0.13965603709220886, 0.08810022473335266, 0.0071262335404753685, 0.09794863313436508, 0.12662066519260406, 0.1379222422838211, 0.11977995187044144, 0.046256691217422485, -0.03673197329044342, 0.008473284542560577, 0.04652765765786171, -0.036308735609054565, -0.08570732921361923, 0.060231488198041916, 0.05859162658452988, -0.0694815069437027, -0.07916422188282013, 0.013052859343588352, 0.17119503021240234, -0.12946343421936035, -0.32054081559181213, 0.08944280445575714, -0.11015575379133224, 0.04150884971022606, 0.13650605082511902, 0.14469082653522491, 0.022801952436566353, 0.2187637984752655, 0.24659746885299683, 0.012664170004427433, 0.016544194892048836, -0.1453787088394165, -0.043231021612882614, -0.01844012551009655, -0.006122906226664782, -0.035883236676454544, -0.16248100996017456, 0.14502869546413422, 0.11808864027261734, -0.040955930948257446, 0.061864327639341354, 0.01621956005692482, -0.10574471950531006, 0.10715039074420929, -0.026779336854815483, -0.1037575900554657, 0.04637349396944046, -0.10598550736904144, -0.15316137671470642, -0.030926411971449852, -0.055340901017189026, 0.0864073783159256, -0.1061035618185997, -0.08997641503810883, 0.08771783858537674, 0.08241310715675354, -0.05831516161561012, -0.3031388521194458, 0.17434152960777283, 0.0851922407746315, -0.09078559279441833, -0.025875426828861237, -0.1109122559428215, -0.14462910592556, 0.28420379757881165, 0.2290128618478775, 0.0064378464594483376, -0.10613333433866501, -0.04757322371006012, 0.08359038084745407, -0.09260306507349014, -0.03533234819769859, -0.1419280469417572, -0.018602920696139336, 0.14346528053283691, -0.092701256275177, -0.1407725214958191, 0.011825203895568848, 0.1440645009279251, -0.07268756628036499, 0.13862669467926025, -0.14658929407596588, -0.12540332973003387, 0.07384053617715836, 0.0749146044254303, 0.10090641677379608, -0.2702022194862366, 0.014733686111867428, -0.157083198428154, -0.09351444244384766, 0.049961384385824203, 0.135685995221138, 0.02104136161506176, -0.03153759986162186, 0.008803692646324635}, {0.13663634657859802, 0.030310360714793205, 0.09440353512763977, 0.023734228685498238, -0.01663600467145443, -0.03282449394464493, -0.09977436810731888, 0.11896549165248871, 0.11609135568141937, -0.00034758661058731377, 0.09699208289384842, -0.07874605804681778, 0.18135623633861542, 0.003524462692439556, -0.08954038470983505, -0.01687653735280037, 0.38677799701690674, -0.061942413449287415, -0.28995656967163086, -0.0007714651874266565, 0.09337100386619568, -0.04345110058784485, -0.11171799153089523, 0.27106398344039917, 0.4094177186489105, -0.2771313786506653, -0.3396259546279907, 0.13877767324447632, 0.10530265420675278, 0.1766609251499176, -0.1485452950000763, -0.039204295724630356, 0.08992414176464081, -0.1428164839744568, 0.25029945373535156, -0.1457405835390091, -0.2817235291004181, -0.23821206390857697, 0.035419389605522156, -0.22953735291957855, 0.23643577098846436, 0.013057597912847996, 0.42859721183776855, 0.08723220974206924, 0.040325429290533066, -0.07161372154951096, 0.0822855606675148, 0.027817513793706894, -0.01839008741080761, 0.014593514613807201, -0.13915401697158813, 0.22920043766498566, -0.035008203238248825, 0.4112001061439514, -0.21514756977558136, -0.1410859078168869, -0.23041707277297974, 0.07719182968139648, -0.15761326253414154, -0.08739352226257324, 0.20648694038391113, 0.06817659735679626, -0.32804566621780396, 0.11878565698862076, 0.11306530982255936, 0.06635881960391998, 0.014324775896966457, -0.24095629155635834, -0.05255552753806114, 0.011638212949037552, 0.22125989198684692, -0.2262776643037796, -0.26964735984802246, -0.08982600271701813, 0.12492156773805618, 0.15634876489639282, 0.045519568026065826, -0.09795418381690979, 0.042674656957387924, 0.07263810932636261, -0.19509314000606537, -0.11563310027122498, 0.039948977530002594, 0.1812058985233307, -0.08303551375865936, 0.19197598099708557, -0.042157746851444244, -0.06644340604543686, -0.008165126666426659, -0.17411158978939056, 0.13381697237491608, 0.03362560272216797, -0.06624850630760193, 0.10922731459140778, -0.02847420424222946, -0.002821285743266344, 0.2642187774181366, 0.047487109899520874, -0.09455009549856186, 0.12712796032428741, 0.19819049537181854, 0.06769328564405441, -0.022085988894104958, 0.3192899525165558, 0.5316489934921265, -0.33128851652145386, 0.15944640338420868, -0.018039830029010773, 0.16259340941905975, -0.09393703937530518, -0.14868010580539703, 0.07424240559339523, -0.05914049968123436, 0.5627033114433289, -0.056340981274843216, 0.07025451213121414, -0.09826590120792389, 0.1443353146314621, 0.06884121894836426, 0.040433675050735474, -0.018637487664818764, 0.07259080559015274, 0.31672361493110657, 0.2831129729747772, -0.35766562819480896, -0.20006167888641357, -0.06583772599697113, 0.13984520733356476, -0.06283343583345413, -0.13410954177379608, -0.10080628842115402, -0.11228445172309875, -0.02112395502626896, -0.11376716941595078, 0.021687492728233337, 0.04800976440310478, -0.07121114432811737, 0.1671639084815979, -0.1778656393289566, 0.17427220940589905, 0.0371646024286747, -0.011232997290790081, 0.14669249951839447, 0.09382233768701553, 0.3709273934364319, -0.11542176455259323, 0.09954039007425308, -0.12380652874708176, 0.5424057841300964, -0.09057561308145523, -0.033034633845090866, 0.059726495295763016, 0.1763669103384018, -0.02533179149031639, 0.0805610790848732, 0.09014275670051575, -0.10688723623752594, -0.04724494367837906, 0.007386638782918453, -0.11729804426431656, 0.0003356792440172285, -0.1374252885580063, -0.008563408628106117, 0.3554190695285797, 0.12733542919158936, -0.07081601768732071, -0.09639447182416916, 0.06711030751466751, -0.11898546665906906, 0.10348538309335709, 0.010540876537561417, 0.26064810156822205, -0.2483547329902649, -0.10071485489606857, -0.19228410720825195, -0.04812294989824295, 0.1651608794927597, -0.13398702442646027, 0.22407394647598267, -0.010094600729644299, -0.1373755782842636, 0.001517044031061232, 0.09253659099340439, -0.41493985056877136, -0.3185320496559143, -0.24071726202964783, -0.2735617458820343, 0.08782464265823364, -0.09549062699079514, -0.024660738185048103, -0.11471737921237946, -0.07866606116294861, -0.1041000708937645, -0.15817396342754364, -0.43148335814476013, 0.03017091378569603, -0.017632879316806793, 0.0071067484095692635, 0.24845285713672638, -0.07287134975194931, 0.26160088181495667, 0.06439000368118286, 0.013442961499094963, -0.24530155956745148, 0.0014110219199210405, 0.21252880990505219, -0.052370332181453705, -0.056903816759586334, -0.30679553747177124, 0.0183124803006649, -0.09127476066350937, 0.12505711615085602, 0.08811689913272858, 0.1507192701101303, 0.07533375173807144, -0.11292843520641327, 0.0050659473054111, -0.07409712672233582, 0.4110201597213745, -0.3100784122943878, -0.0717558041214943, 0.2382400631904602, -0.02150708995759487, 0.04118920862674713, -0.002034141682088375, 0.01298553217202425, -0.16225717961788177, 0.23436766862869263, 0.02718978002667427, 0.10992381721735, 0.22937476634979248, 0.08577675372362137, -0.19560539722442627, -0.16576413810253143, 0.189146488904953, -0.029796427115797997, 0.22539445757865906, 0.08589765429496765, 0.12833869457244873, 0.09396453201770782, -0.10843373090028763, 0.14212797582149506, -0.16987459361553192, 0.18002621829509735, -0.13420112431049347, 0.07669597119092941, -0.006189411506056786, -0.04582716524600983, 0.15899251401424408, 0.03777783736586571, 0.13959886133670807, 0.18221832811832428, -0.11060425639152527, 0.3903777301311493, 0.03635094314813614, -0.05898506939411163}, {0.1357199102640152, 0.050337616354227066, 0.2363223284482956, -0.21956732869148254, 0.19651924073696136, -0.09946926683187485, 0.22683493793010712, -0.003337917383760214, 0.09526559710502625, -0.013357581570744514, -0.02395690605044365, 0.0031596017070114613, -0.07777741551399231, -0.04997118562459946, -0.2779313623905182, 0.07352152466773987, -0.013906118459999561, -0.054363228380680084, -0.15725436806678772, -8.607313793618232e-05, 0.01784341409802437, 0.01905863918364048, -0.022424422204494476, 0.08517871797084808, 0.04591386020183563, 0.01065739430487156, -0.014733756892383099, -0.08459064364433289, -0.04935908317565918, -0.05139002576470375, -0.025643305853009224, 0.05697695165872574, -0.05704174190759659, 0.0037363881710916758, -0.03294266760349274, -0.14083564281463623, 0.021495630964636803, -0.17107243835926056, -0.057628754526376724, -0.03269320726394653, 0.13009461760520935, 0.04361014440655708, 0.18329231441020966, -0.07071444392204285, 0.17344066500663757, 0.02588752470910549, 0.05562419071793556, 0.13862693309783936, -0.11080291122198105, 0.0069786855019629, -0.08018088340759277, -0.05535535886883736, 0.19719409942626953, 0.0044152988120913506, -0.03461623564362526, -0.00921222846955061, 0.040041372179985046, 0.2519497573375702, -0.04658212512731552, 0.0605347715318203, -0.028611252084374428, -0.16677117347717285, 0.0009314912022091448, 0.06033826991915703, -0.012339632026851177, -0.009442450478672981, -0.09940624237060547, -0.10156417638063431, -0.031117867678403854, 0.013816146180033684, -0.057131581008434296, 0.004573270212858915, -0.08043026924133301, -0.09345926344394684, -0.1305859237909317, 0.010200032964348793, 0.14866551756858826, -0.2531201243400574, -0.05746777355670929, -0.06269668787717819, 0.1271948367357254, -0.049676068127155304, 0.03518176078796387, 0.15446802973747253, 0.10380053520202637, 0.08058790862560272, 0.003964296076446772, 0.02718356065452099, -0.014905586838722229, -0.10093960911035538, -0.17148637771606445, 0.06445124745368958, -0.00942924153059721, 0.22227869927883148, -0.2529713809490204, 0.003505989443510771, 0.005237364675849676, 0.26160717010498047, -0.05842050537467003, 0.08297543227672577, -0.02416854351758957, -0.21469038724899292, -0.21838673949241638, 0.052512265741825104, 0.10684207081794739, -0.34929215908050537, -0.08336687833070755, -0.10421137511730194, -0.06107664108276367, -0.08690613508224487, -0.07512691617012024, -0.05483296886086464, -0.05333046242594719, 0.11993396282196045, 0.045350536704063416, -0.14308571815490723, -0.22381694614887238, -0.09946522861719131, 0.04231422021985054, -0.10462020337581635, -0.04948568716645241, -0.0313633531332016, 0.015419930219650269, 0.06791449338197708, 0.05003483220934868, 0.06920427829027176, -0.08364126831293106, 0.10255465656518936, 0.007302310783416033, -0.039723847061395645, 0.04643189162015915, -0.023743193596601486, 0.07742878049612045, 0.16866004467010498, -0.2437475621700287, 0.20709702372550964, 0.04058799147605896, 0.10006799548864365, 0.03788227587938309, -0.17584162950515747, -0.1592334806919098, -0.31182193756103516, 0.14384320378303528, -0.09671888500452042, 0.22718258202075958, 0.046553127467632294, -0.029660698026418686, -0.202846959233284, 0.06388544291257858, -0.024363020434975624, 0.11104333400726318, -0.11011464893817902, -0.07912817597389221, -0.13945643603801727, 0.046733129769563675, 0.11008016765117645, 0.07287386059761047, 0.20245449244976044, -0.17986604571342468, -0.10153711587190628, 0.0425461009144783, -0.10749426484107971, -0.06652010232210159, 0.10453161597251892, 0.06601545959711075, 0.042087629437446594, 0.3752867579460144, 0.15087954699993134, 0.08981024473905563, 0.0812981054186821, -0.09580281376838684, -0.025076689198613167, 0.0872451588511467, -0.11120572686195374, -0.023831063881516457, -0.1525467038154602, -0.15131306648254395, 0.13784272968769073, 0.08080509305000305, -0.03493886813521385, 0.08748579025268555, 0.00021132770052645355, 0.0975017100572586, -0.039192523807287216, -0.096927210688591, 0.012240340933203697, -0.02891976200044155, 0.051218632608652115, -0.12915493547916412, -0.26422950625419617, 0.06506103277206421, 0.10535642504692078, 0.12760570645332336, 0.042496196925640106, 0.3000051975250244, -0.028926681727170944, 0.10687865316867828, -0.25735917687416077, 0.04111255332827568, 0.10610509663820267, -0.0688028559088707, 0.016255751252174377, -0.006491196341812611, 0.07558146864175797, 0.18582825362682343, -0.16037556529045105, -0.3590279519557953, -0.16794410347938538, 0.12482378631830215, -0.01702822931110859, -0.0018021967262029648, 0.09812527149915695, 0.021088622510433197, 0.2866800129413605, -0.041507795453071594, -0.11645185947418213, 0.005802534054964781, -0.19298288226127625, 0.1157880648970604, -0.06656757742166519, 0.07046441733837128, 0.004241258837282658, 0.07879091054201126, 0.006659802980720997, -0.0043593947775661945, 0.053097181022167206, -0.10944938659667969, 0.012144391424953938, 0.09646480530500412, 0.03482457622885704, -0.21011392772197723, -0.17174659669399261, 0.036815863102674484, -0.05088755115866661, 0.23683305084705353, -0.14243756234645844, -0.10670618712902069, 0.1500532329082489, 0.19830752909183502, -0.01224178820848465, -0.1788494884967804, 0.11326336860656738, 0.16715295612812042, 0.1408006101846695, -0.07440081983804703, -0.026529356837272644, 0.0739816352725029, 0.0402781218290329, 0.046152643859386444, -0.115628682076931, 0.014328399673104286, 0.09565500915050507, -0.3326740860939026, -0.03120736964046955, 0.17109927535057068, 0.07456634938716888}, {-0.07570507377386093, -0.1270996779203415, -0.14328859746456146, -0.01288477424532175, -0.34219661355018616, 0.16081106662750244, 0.17938116192817688, 0.15259766578674316, -0.14287832379341125, 0.2713485360145569, 0.2052442580461502, -0.19030165672302246, 0.04859776422381401, 0.1155405193567276, 0.04878558591008186, -0.1491524577140808, -0.22702893614768982, 0.2247953861951828, 0.2898525893688202, -0.12656456232070923, -0.015251236036419868, 0.037743084132671356, -0.09317931532859802, -0.1496383249759674, -0.1377895623445511, 0.1627068817615509, 0.13081298768520355, 0.04246180132031441, -0.003644027514383197, -0.07355435192584991, 0.22743551433086395, -0.024438997730612755, -0.033463895320892334, 0.023444319143891335, 0.2680061161518097, -0.3133007884025574, -0.13524433970451355, 0.22128035128116608, -0.00015359501412604004, 0.07944579422473907, -0.058317456394433975, 0.1570826917886734, -0.1424701064825058, -0.07415639609098434, 0.013796530663967133, -0.027684541419148445, 0.1541898399591446, 0.006467945873737335, 0.08817983418703079, -0.17773506045341492, -0.14009715616703033, -0.015909545123577118, 0.09296411275863647, -0.44510418176651, 0.15297654271125793, 0.07008136808872223, 0.030781134963035583, -0.16282474994659424, 0.04323422163724899, 0.06707712262868881, 0.01944468542933464, 0.11034481972455978, 0.13051024079322815, -0.102089062333107, 0.06302609294652939, -0.04364396259188652, -0.17340543866157532, 0.1704786866903305, -0.07902489602565765, -0.11742940545082092, 0.011663442477583885, 0.11710533499717712, -0.013464020565152168, 0.08940451592206955, -0.08039148896932602, 0.1023130863904953, 0.028797533363103867, -0.05080947279930115, 0.04225659370422363, 0.1321539729833603, -0.18949012458324432, 0.14903825521469116, 0.10094801336526871, 0.02129727602005005, 0.10968410223722458, 0.024466698989272118, -0.07945749908685684, -0.11083129793405533, 0.06195194274187088, 0.21268397569656372, 0.013161154463887215, -0.035149216651916504, 0.14710767567157745, 0.07293415069580078, -0.13219445943832397, -0.24507281184196472, -0.054501719772815704, 0.1587798297405243, 0.190205380320549, 0.3436264097690582, 0.009253416210412979, 0.13517585396766663, 0.350894033908844, -0.08443481475114822, -0.07421448081731796, 0.17728252708911896, -0.016762273386120796, 0.24921314418315887, 0.3427206873893738, 0.1625242829322815, -0.01398520078510046, 0.035572491586208344, 0.11788605153560638, -0.18337155878543854, 0.040851153433322906, -0.15404796600341797, 0.03229006007313728, 0.04463862255215645, -0.11582916975021362, -0.07174129039049149, 0.09099646657705307, 0.06104141101241112, -0.14600446820259094, -0.10533209890127182, -0.07609216123819351, -0.02272069826722145, -0.08713120222091675, 0.08582375943660736, 0.08254160732030869, 0.2003808319568634, -0.15367379784584045, -0.11852949857711792, -0.06364551931619644, 0.11445736885070801, 0.06555718928575516, -0.017087643966078758, 0.023896759375929832, -0.04679427295923233, -0.19911974668502808, -0.06761694699525833, -0.0002814210020005703, -0.089693583548069, -0.056151606142520905, -0.061102721840143204, -0.29686740040779114, 0.021665621548891068, -0.25751230120658875, 0.0907546877861023, -0.020216578617691994, 0.10305075347423553, 0.20668593049049377, 0.11560486257076263, 0.050233665853738785, 0.040758952498435974, 0.04535369202494621, 0.10329017788171768, -0.06956049799919128, 0.1982799470424652, 0.12700492143630981, -0.2119777798652649, 0.31617432832717896, 0.027333728969097137, -0.20356808602809906, -0.08705706149339676, -0.010468699038028717, -0.24201811850070953, -0.16172802448272705, 0.09242616593837738, 0.2329210489988327, 0.06880168616771698, -0.12839306890964508, 0.09657680243253708, 0.00965646468102932, -0.4048655331134796, -0.09043707698583603, -0.0036760696675628424, 0.08392803370952606, 0.13820113241672516, -0.01391424611210823, -0.12584534287452698, -0.09826302528381348, -0.12729591131210327, -0.16053150594234467, -0.01280206348747015, -0.05996749550104141, -0.2491641491651535, 0.10863485932350159, -0.22999924421310425, 0.18070286512374878, -0.07106241583824158, -0.02689642459154129, 0.17338886857032776, -0.18212109804153442, -0.1828785538673401, -0.09621240943670273, -0.13935379683971405, 0.0710371881723404, 0.17678302526474, -0.23738497495651245, 0.05621631070971489, 0.12442103773355484, -0.01148789282888174, -0.08346173167228699, 0.06928722560405731, -0.09102426469326019, -0.061621975153684616, -0.16784416139125824, -0.18249952793121338, -0.04045316204428673, 0.267740398645401, 0.1079549789428711, 0.13030275702476501, 0.1116102933883667, 0.14932380616664886, -0.014145751483738422, 0.11608372628688812, 0.011765377596020699, -0.05422023683786392, -0.16006971895694733, -0.16973140835762024, 0.016183778643608093, -0.09282958507537842, -0.041625600308179855, 0.11387810111045837, -0.0016455658478662372, 0.09057939052581787, 0.20430587232112885, 0.15547263622283936, -0.1411817967891693, -0.09571998566389084, 0.24139085412025452, 0.09011167287826538, -0.18752144277095795, -0.14487749338150024, -0.018901927396655083, -0.0905224084854126, -0.3449760973453522, -0.1137760654091835, 0.03989386186003685, 0.18502089381217957, 0.2600342929363251, -0.12572665512561798, -0.18011876940727234, 0.09006532281637192, 0.017221424728631973, -0.05442633852362633, 0.11711828410625458, 0.020757900550961494, -0.026979656890034676, 0.02038056030869484, -0.10904449224472046, -0.14491786062717438, 0.152965247631073, 0.09856866300106049, 0.06244530528783798, -0.031619518995285034}, {0.11168629676103592, -0.08409911394119263, 0.01267255935817957, 0.03373820334672928, 0.0014259468298405409, 0.268899142742157, 0.10415090620517731, 0.05090870335698128, -0.02685563638806343, 0.07517463713884354, -0.061902452260255814, -0.11641356348991394, 0.024520259350538254, 0.03265533968806267, 0.011984479613602161, -0.1578347384929657, 0.06234997510910034, 0.21023178100585938, -0.07064377516508102, 0.027728436514735222, -0.13463276624679565, -0.010070725344121456, -0.06691275537014008, 0.00974228885024786, 0.08896497637033463, -0.0034071479458361864, -0.022191615775227547, 0.16084693372249603, -0.09527763724327087, -0.011303787119686604, 0.08083920925855637, -0.013951209373772144, 0.03595753386616707, 0.05838094279170036, -0.19902770221233368, -0.2018204927444458, -0.10105833411216736, -0.02964111417531967, -0.1836574226617813, -0.080921471118927, 0.017161879688501358, 0.05205783247947693, -0.04749226197600365, 0.061173856258392334, -0.04310283809900284, -0.13055968284606934, 0.010006047785282135, -0.02638198249042034, -0.007775872014462948, 0.05994902923703194, -0.19781088829040527, -0.08716244250535965, 0.064748115837574, 0.06712473183870316, -0.022173134610056877, 0.02400241792201996, -0.06063400208950043, -0.010380402207374573, 0.06800506263971329, -0.12928876280784607, 0.1663486659526825, 0.03864005208015442, -0.2935321629047394, -0.2713440954685211, 0.03230953589081764, 0.056241992861032486, 0.06233198568224907, -0.29286259412765503, -0.04702993854880333, -0.06350622326135635, 0.2494237720966339, -0.05589636042714119, 0.17696934938430786, 0.03920205682516098, -0.08566166460514069, -0.06524481624364853, 0.002487935358658433, -0.03731643036007881, -0.11316335201263428, 0.28654444217681885, -0.29622775316238403, -0.11394613981246948, -0.017404111102223396, 0.11191479116678238, 0.042365662753582, 0.010082481428980827, 0.14532947540283203, -0.16612178087234497, 0.023418081924319267, -0.0761183351278305, 0.07787496596574783, 0.03511173650622368, -0.05853055790066719, -0.0052591292187571526, 0.02279338240623474, 0.0016209299210458994, 0.2682124376296997, -0.08809881657361984, -0.22810323536396027, -0.00757022388279438, -0.044715456664562225, 0.11791500449180603, 0.094058558344841, 0.018440471962094307, 0.07115798443555832, 0.016486460343003273, 0.1455289125442505, -0.007080376613885164, 0.05062874034047127, 0.0847192332148552, 0.03572550415992737, -0.009900933131575584, -0.0025400773156434298, -0.07360390573740005, -0.03654078021645546, -0.060271456837654114, 0.07037461549043655, -0.06458160281181335, 0.13022813200950623, -0.11408192664384842, 0.0618768110871315, -0.2708882987499237, -0.18181897699832916, -0.19042366743087769, 0.009146594442427158, -0.05293358862400055, 0.11863715201616287, -0.08327707648277283, -0.019331661984324455, 0.026068802922964096, 0.0686056837439537, 0.021160496398806572, -0.1375911682844162, -0.03831631690263748, -0.2542085647583008, -0.12912005186080933, 0.11117366701364517, 0.12987595796585083, -0.1643907129764557, 0.18104606866836548, -0.03646673262119293, -0.08901248127222061, 0.13560354709625244, 0.022878587245941162, -0.07409092038869858, 0.03196687996387482, 0.20045329630374908, 0.040152646601200104, 0.02507936768233776, 0.017267422750592232, 0.19554758071899414, 0.08042393624782562, 0.005649248603731394, 0.051335595548152924, 0.17751885950565338, 0.06580998003482819, -0.14516110718250275, -0.12414126843214035, 0.10294817388057709, -0.025892559438943863, -0.008994007483124733, 0.14426246285438538, 0.030049122869968414, -0.06171248108148575, 0.17254948616027832, -0.0734279677271843, 0.19242428243160248, 0.1710241138935089, -0.04277827590703964, 0.04547971859574318, 0.050321999937295914, -0.045034945011138916, -0.1074502170085907, -0.12100894004106522, -0.19723457098007202, 0.04040141403675079, -0.10230273008346558, 0.09879187494516373, -0.0915772020816803, -0.11901674419641495, 0.041560493409633636, -0.03501845896244049, -0.18565721809864044, 0.01605357602238655, -0.061009205877780914, -0.08954591304063797, 0.09484575688838959, -0.06292588263750076, -0.16489598155021667, 0.1251598745584488, -0.032675646245479584, 0.025838138535618782, -0.14777974784374237, -0.3443853557109833, 0.10875903815031052, 0.1398538053035736, 0.14597675204277039, 0.06573407351970673, 0.05829935148358345, 0.18350565433502197, -0.008987161330878735, 0.07897006720304489, -0.010659671388566494, -0.032667241990566254, 0.02074139006435871, 0.0671737864613533, 0.03505002707242966, 0.04371064901351929, -0.05535233020782471, 0.1675257384777069, 0.08160249143838882, 0.03603753075003624, 0.15062583982944489, -0.1800113320350647, -0.011980706825852394, -0.15087857842445374, 0.02443704754114151, 0.1484953761100769, 0.04538236930966377, -0.023345043882727623, -0.04063039273023605, -0.05679847672581673, -0.05902291089296341, -0.03708085045218468, 0.04177612066268921, 0.043686479330062866, -0.010186683386564255, 0.3511197566986084, 0.0734918937087059, 0.32806986570358276, 0.08121491223573685, 0.07105934619903564, 0.19115962088108063, -0.03608550876379013, -0.013778718188405037, -0.08156250417232513, -0.023134302347898483, -0.01011076383292675, 0.05651681497693062, -0.016517406329512596, 0.06951449066400528, -0.05784739553928375, -0.0721186175942421, -0.11244488507509232, -0.11259068548679352, 0.138580784201622, 0.10545089095830917, 0.051833778619766235, 0.013715248554944992, 0.0814046785235405, 0.007138246204704046, -0.10935898125171661, 0.047872044146060944, -0.007515381556004286, -0.23456552624702454, 0.13019904494285583}, {-0.03678739443421364, -0.023191111162304878, -0.06599220633506775, -0.007712432183325291, 0.18405702710151672, 0.09358221292495728, 0.12184452265501022, 0.02352510392665863, -0.00785827450454235, 0.19149817526340485, -0.00973414909094572, -0.25986209511756897, 0.017098000273108482, 0.05646843463182449, 0.025891415774822235, -0.06614936143159866, 0.051612257957458496, -0.009805786423385143, 0.011298338882625103, 0.05670716613531113, 0.1613326519727707, -0.028698373585939407, -0.06221741437911987, -0.16851159930229187, 0.13283666968345642, -0.04585526883602142, 0.07682864367961884, -0.1844436675310135, -0.08128920942544937, 0.028393015265464783, -0.034566715359687805, 0.07410064339637756, 0.024145279079675674, -0.09536943584680557, 0.027982475236058235, 0.1659708172082901, -0.136305034160614, 0.11899024993181229, 0.25348442792892456, 0.20127180218696594, -0.1876828819513321, 0.10310777276754379, -0.040084321051836014, 0.1867116391658783, -0.008186353370547295, 0.15499091148376465, -0.009608018212020397, 0.11136890202760696, -0.03385862708091736, 0.15403026342391968, 0.06537418812513351, 0.03341281786561012, 0.06821375340223312, -0.048710618168115616, -0.10369326174259186, -0.14648126065731049, 0.11169823259115219, -0.23107190430164337, -0.017894690856337547, 0.49467742443084717, 0.01760580763220787, -0.16416223347187042, 0.006463685072958469, 0.0015202617505565286, -0.009963804855942726, 0.04576786234974861, 0.05642171576619148, 0.00778689980506897, -0.04811477288603783, 0.04610590636730194, -0.2182072103023529, -0.05821148306131363, 0.254008024930954, 0.08947833627462387, 0.0762716606259346, 0.24675530195236206, -0.31480637192726135, 0.16365668177604675, 0.08296466618776321, 0.059086401015520096, 0.028545694425702095, 0.08557318896055222, -0.1431368887424469, -0.0018559617456048727, -0.010507571510970592, -0.18875236809253693, 0.03449840098619461, 0.22035104036331177, -0.12627284228801727, -0.007744395639747381, 0.0694776400923729, -0.021766573190689087, 0.22138294577598572, -0.09932567924261093, 0.12958486378192902, 0.12169092893600464, -0.03859712556004524, -0.14826737344264984, -0.19128531217575073, 0.1752059906721115, 0.002344107488170266, 0.21576546132564545, 0.10232193768024445, -0.04402710869908333, 0.07487504184246063, 0.23759132623672485, 0.019034413620829582, -0.14654213190078735, -0.21070511639118195, 0.021596893668174744, -0.14321598410606384, -0.13565857708454132, 0.02679521217942238, -0.027366911992430687, 0.08481702208518982, 0.10642971098423004, 0.23890374600887299, 0.030417123809456825, 0.152712881565094, 0.18602189421653748, -0.0829397663474083, -0.18087247014045715, 0.0365544855594635, -0.02901020087301731, -0.0684196725487709, 0.10101919621229172, -0.030307337641716003, 0.09674910455942154, -0.1687619835138321, -0.07903613150119781, -0.00280765350908041, -0.034978169947862625, -0.12845760583877563, -0.34323906898498535, 0.010530778206884861, -0.2102964073419571, 0.037061337381601334, 0.12379617989063263, -0.04350103437900543, -0.08490713685750961, -0.021348387002944946, 0.3262757360935211, -0.25124654173851013, -0.0750199407339096, -0.3314274847507477, -0.1803855001926422, 0.012483691796660423, 0.10971110314130783, 0.1560954600572586, 0.17395219206809998, -0.1420765370130539, -0.011732766404747963, 0.05553309619426727, -0.06857378035783768, -0.09614215046167374, -0.044027380645275116, -0.09852562844753265, -0.10693655163049698, 0.09303569048643112, -0.06309366226196289, 0.07226452231407166, 0.1934480369091034, -0.05406012386083603, 0.07703828066587448, -0.11996711045503616, 0.1678648740053177, 0.2542773187160492, 0.04930967092514038, -0.17134900391101837, 0.06714505702257156, -0.19594807922840118, -0.030926130712032318, -0.2732715904712677, 0.15992853045463562, -0.142539381980896, -0.022797083482146263, -0.03491416946053505, 0.031384751200675964, 0.08590187877416611, 0.1059141531586647, 0.09884464740753174, -0.2442254275083542, -0.008074947632849216, -0.10162493586540222, -0.015775730833411217, -0.015591669827699661, -0.02439340390264988, 0.03536749258637428, 0.06493007391691208, 0.06963688880205154, -0.16236300766468048, -0.20304711163043976, 0.06243550032377243, 0.2328450083732605, -0.026854142546653748, 0.026501374319195747, -0.12900950014591217, -0.24832551181316376, 0.0411636121571064, -0.10873941332101822, -0.10799026489257812, -0.050954028964042664, -0.053896255791187286, -0.14844675362110138, -0.03766326606273651, 0.057372767478227615, -0.058128178119659424, -0.11134494841098785, -0.1661779284477234, -0.05084514617919922, 0.1337212175130844, 0.030074596405029297, -0.10414199531078339, 0.011000020429491997, -0.018473871052265167, 0.07892671972513199, 0.21322432160377502, 0.2470528483390808, -0.08985539525747299, 0.21991263329982758, 0.06397989392280579, -0.2299395501613617, 0.029940500855445862, 0.1398475617170334, 0.1250721514225006, -0.00968808215111494, -0.02613992616534233, 0.0964127704501152, -0.09093708544969559, -0.11211331933736801, 0.11798640340566635, 0.15454700589179993, 0.1400006115436554, -0.10623016953468323, 0.07451982796192169, -0.011410333216190338, 0.024644626304507256, 0.09268894046545029, 0.009224016219377518, -0.014799635857343674, -0.1355762481689453, -0.01898614689707756, 0.051857512444257736, -0.1604614406824112, 0.09070160239934921, -0.20341305434703827, -0.06922953575849533, -0.04640188440680504, -0.14066393673419952, 0.0498691163957119, 0.148018479347229, -0.048900824040174484, 0.24385082721710205, 0.07427354156970978, 0.01799127645790577, -0.12846660614013672}, {-0.09541712701320648, 0.015995878726243973, 0.15966476500034332, -0.0004165028512943536, 0.10893645137548447, 0.02150764875113964, -0.13737677037715912, -0.08382312953472137, 0.013645299710333347, 0.07344692945480347, -0.004072151146829128, 0.13238312304019928, -0.06753768026828766, 0.0305060762912035, -0.13672496378421783, -0.01655755378305912, -0.012457124888896942, 0.10198128968477249, 0.16737543046474457, 0.1849750578403473, 0.13794733583927155, 0.08602679520845413, 0.1251201331615448, 0.01726350374519825, 0.05306681990623474, 0.11838296055793762, 0.002661612816154957, -0.0516994334757328, 0.0028129920829087496, 0.10200613737106323, -0.004574208986014128, 0.06928607076406479, -0.08716779202222824, 0.1869751363992691, 0.078719362616539, -0.1054329201579094, 0.0017316935118287802, 0.02757963351905346, -0.1283656805753708, -0.010535342618823051, 0.08766438812017441, -0.12017221003770828, 0.07861033827066422, 0.062232498079538345, 0.22322888672351837, 0.1639832854270935, 0.0526515431702137, -0.09976629912853241, -0.052820708602666855, 0.08054523915052414, -0.07548344135284424, -0.004231790080666542, 0.005552730523049831, -0.04585183411836624, -0.04236146807670593, 0.10265965014696121, 0.2320694774389267, -0.095679372549057, -0.0062776971608400345, 0.15584713220596313, -0.15978312492370605, -0.04780196398496628, 0.15763430297374725, -0.11738583445549011, -0.016816433519124985, -0.13884337246418, -0.008410573936998844, 0.10454338788986206, -0.17758046090602875, 0.12817545235157013, -0.0003084912896156311, -0.042702969163656235, -0.16469402611255646, -0.02637152373790741, -0.18899841606616974, 0.003652342827990651, -0.1857997328042984, -0.1250263899564743, 0.2515304982662201, -0.34838342666625977, 0.057425908744335175, 0.08881069719791412, -0.05364146828651428, -0.04440324380993843, -0.07666315138339996, -0.01626754365861416, 0.07673651725053787, 0.02514832094311714, -0.06416542083024979, -0.042293425649404526, -0.10826335847377777, -0.12672385573387146, 0.1192510649561882, -0.015234286896884441, 0.043732382357120514, 0.021821441128849983, -0.03365473821759224, -0.08149285614490509, -0.11278346180915833, 0.08945324271917343, -0.07669229060411453, -0.09967857599258423, -0.10026504844427109, 0.06907609105110168, 0.006893066223710775, 0.6366556882858276, 0.14380590617656708, -0.11695782840251923, -0.04521381855010986, -0.005359821952879429, 0.12350055575370789, 0.06045428290963173, -0.19095371663570404, -0.05178316682577133, 0.04452347010374069, -0.21575012803077698, -0.10332279652357101, -0.23709312081336975, 0.009051631204783916, -0.0035070551093667746, 0.032126303762197495, -0.02328309789299965, 0.02938191406428814, -0.07497140765190125, 0.22965846955776215, -0.09777659177780151, 0.03979484736919403, -0.08749149739742279, -0.020278513431549072, 0.007938448339700699, 0.0720626562833786, -0.041340697556734085, 0.0654740184545517, 0.08569427579641342, 0.1351039707660675, -0.05397579073905945, 0.1988261193037033, -0.10176093131303787, 0.12063589692115784, -0.10393615812063217, 0.14416898787021637, -0.11623366922140121, -0.131873220205307, -0.11156801879405975, -0.025586972013115883, -0.10389828681945801, 0.04870510473847389, -0.08514861017465591, -0.13979412615299225, 0.13935315608978271, -0.12482579797506332, -0.0729835256934166, -0.15337549149990082, -0.16949860751628876, -0.18098585307598114, 0.14693209528923035, 0.004760834854096174, 0.05164092406630516, -0.24734151363372803, -0.1583402007818222, 0.02535187639296055, -0.03431503102183342, -0.03287757933139801, 0.1311321258544922, -0.11137738078832626, 0.0007156325737014413, 0.2275131791830063, -0.17281237244606018, 0.01514652743935585, -0.03583347797393799, -0.037436194717884064, -0.17663897573947906, -0.13430404663085938, -0.11782094091176987, 0.28948405385017395, 0.23111750185489655, -0.06020223721861839, -0.01199405174702406, -0.01979651488363743, -0.008360549807548523, -0.056318096816539764, 0.24057234823703766, 0.13213622570037842, 0.05355740338563919, 0.09983593225479126, 0.1884329915046692, -0.04889523237943649, -0.036816179752349854, 0.070635586977005, -0.14394447207450867, 0.02120458334684372, 0.08943910896778107, 0.11807256937026978, 0.28008899092674255, -0.06724774092435837, -0.05904282629489899, -0.155090793967247, 0.008929876610636711, -0.0090947225689888, -0.3455917239189148, 0.05752873048186302, -0.02733626775443554, 0.0052391705103218555, 0.02837206982076168, -0.03255691006779671, 0.12477516382932663, -0.06351214647293091, 0.036081910133361816, 0.11895884573459625, -0.054695047438144684, -0.001496703363955021, 0.015603858046233654, 0.08292333036661148, 0.23548637330532074, -0.14276203513145447, 0.13151799142360687, -0.2358514815568924, -0.05149379000067711, -0.08766761422157288, -0.11693927645683289, 0.09262358397245407, 0.0807926207780838, -0.08502271771430969, -0.01362696848809719, -0.18670333921909332, -0.07181503623723984, 0.0660887211561203, 0.05588655173778534, 0.062379851937294006, -0.04045265540480614, -0.07971598207950592, 0.054252125322818756, -0.13414469361305237, 0.16712774336338043, 0.1280340999364853, -0.10336840897798538, -0.14634302258491516, 0.08750902861356735, 0.14014455676078796, 0.10347253829240799, 0.06319252401590347, 0.18678906559944153, 0.08247440308332443, 0.24457700550556183, -0.11358045786619186, -0.10881515592336655, -0.035594429820775986, -0.033585477620363235, -0.054555319249629974, 0.24513405561447144, -0.05049102380871773, -0.3244466483592987, -0.14045673608779907, -0.01665842905640602, 0.10052672773599625, 0.02294035628437996}, {-0.0028185516130179167, -0.0963519811630249, -0.14416082203388214, -0.08332322537899017, -0.006673906464129686, -0.07789306342601776, 0.14811959862709045, -0.19911517202854156, 0.043622709810733795, 0.11305892467498779, -0.00892819743603468, -0.13317391276359558, 0.07791135460138321, -0.04911835119128227, 0.24731162190437317, -0.2290438711643219, -0.05339163914322853, 0.23569203913211823, -0.07948781549930573, 0.018801430240273476, 0.12076572328805923, -0.10051868855953217, -0.114252470433712, 0.04674530401825905, 0.1948811113834381, 0.01041859108954668, -0.07796823233366013, 0.04738224297761917, -0.28402018547058105, 0.12472620606422424, -0.06011287495493889, -0.1089724451303482, 0.03646104410290718, 0.1523289829492569, -0.26525062322616577, 0.14828196167945862, -0.12903611361980438, -0.12909534573554993, 0.04103860259056091, 0.008494692854583263, -0.04993050917983055, 0.040631115436553955, -0.07132218778133392, 0.04402579367160797, 0.097859688103199, -0.03961242735385895, -0.17249660193920135, 0.01823057048022747, 0.16541986167430878, -0.1199965626001358, -0.10043220221996307, -0.05983949452638626, -0.11255984753370285, 0.10449644178152084, -0.16599895060062408, -0.12102822214365005, -0.15475702285766602, -0.1630372405052185, 0.016827354207634926, -0.09544659405946732, 0.06454528868198395, -0.06684020161628723, -0.27741146087646484, -0.07351768761873245, -0.02025260031223297, 0.02329639531672001, 0.013183866627514362, -0.1702377051115036, -0.1296633630990982, 0.08879389613866806, 0.18450140953063965, -0.2027641236782074, -0.0004566260613501072, -0.2569146454334259, -0.07727562636137009, 0.0934232845902443, -0.07448853552341461, -0.07352527230978012, 0.03715266287326813, 0.09171514958143234, -0.0019529361743479967, 0.1489739716053009, 0.1440669298171997, 0.12164837121963501, -0.12474290281534195, -0.0835808739066124, -0.07359222322702408, -0.1836826652288437, 0.1630730926990509, 0.04624352604150772, -0.091824010014534, -0.022319050505757332, -0.031040068715810776, -0.03744475543498993, 0.13374443352222443, -0.016319630667567253, 0.09027570486068726, 0.14724859595298767, -0.1505248248577118, 0.050548214465379715, -0.12852774560451508, 0.03924984857439995, -0.06337321549654007, -0.10926522314548492, -0.20697090029716492, 0.07811157405376434, 0.13543300330638885, 0.23693013191223145, 0.08143889904022217, -0.10810044407844543, 0.17480704188346863, 0.045551810413599014, 0.023753119632601738, 0.13944555819034576, 0.023394180461764336, 0.01753782294690609, 0.05865556374192238, 0.07275372743606567, -0.020847171545028687, 0.11923516541719437, -0.04981156438589096, -0.05867966637015343, -0.036125969141721725, -0.0020674581173807383, -0.18145085871219635, 0.00018640226335264742, -0.17166867852210999, -0.11646110564470291, 0.20365819334983826, 0.009872686117887497, -0.10265907645225525, 0.08431365340948105, -0.014972160570323467, 0.04331604763865471, -0.022140376269817352, -0.0793491005897522, -0.1473570317029953, 0.055926475673913956, 0.15274251997470856, -0.04233015701174736, 0.056097280234098434, 0.03716724365949631, 0.014881330542266369, 0.06394027173519135, -0.07948245853185654, -0.08281389623880386, 0.08336975425481796, 0.11307961493730545, 0.16196250915527344, -0.0069807544350624084, -0.043098658323287964, 0.1628240942955017, -0.20973238348960876, -0.055806487798690796, -0.1416231244802475, -0.1310877650976181, 0.2912754714488983, -0.035851553082466125, 0.0815892145037651, 0.0830642506480217, -0.03146539255976677, 0.15569528937339783, 0.10846181213855743, 0.024728484451770782, -0.025388898327946663, 0.15002955496311188, 0.04682295769453049, 0.06657042354345322, 0.21440576016902924, 0.1382318139076233, 0.12101756781339645, 0.09597574919462204, 0.14002098143100739, 0.1799650490283966, -0.08583443611860275, 0.15006570518016815, 0.19196826219558716, 0.02167835645377636, 0.11501526087522507, -0.08557815849781036, 0.009384911507368088, -0.027349146082997322, -0.04243358224630356, 0.2173570990562439, -0.0888853371143341, 0.13199545443058014, -0.06794199347496033, 0.23176629841327667, -0.026171931996941566, -0.11758536100387573, -0.09518852084875107, 0.16694484651088715, -0.11810743808746338, -0.058602385222911835, 0.09356414526700974, 0.2979021966457367, 0.06356125324964523, 0.04140397161245346, -0.18980465829372406, 0.15281538665294647, -0.22335921227931976, -0.10460339486598969, -0.09851048141717911, -0.15482963621616364, 0.19654712080955505, 0.037548474967479706, 0.2179308980703354, 0.019665759056806564, -0.07332819700241089, -0.07620184123516083, -0.1533849835395813, 0.03575710579752922, 0.2652314305305481, -0.03002195991575718, 0.11033246666193008, -0.08719617128372192, -0.04646628350019455, -0.10653173178434372, 0.20741215348243713, 0.12996912002563477, -0.02892160415649414, 0.02116847224533558, -0.06446253508329391, -0.012304075993597507, -0.004507792182266712, -0.18219779431819916, 0.10604145377874374, 0.00759389903396368, 0.0018997584702447057, 0.011274394579231739, -0.03370650112628937, 0.07941408455371857, 0.06375803053379059, 0.2052309513092041, 0.1942421942949295, 0.04118723049759865, 0.336998850107193, -0.004773936700075865, -0.03956363722681999, -0.030254250392317772, -0.1159946620464325, -0.3023970425128937, -0.03702320531010628, -0.11141350865364075, -0.03230935335159302, 0.05994046479463577, 0.028782814741134644, 0.31212320923805237, -0.13269446790218353, 0.0334542952477932, 0.2206558883190155, 0.04986507073044777, 0.0023687530774623156, -0.020380130037665367, -0.06295477598905563, -0.09785240888595581}, {-0.15107403695583344, -0.008391599170863628, 0.15860703587532043, 0.024141956120729446, -0.16369573771953583, 0.16122476756572723, 0.11918388307094574, 0.08352942764759064, -0.07834421098232269, 0.04814491420984268, 0.028496406972408295, -0.5818693041801453, 0.0016541420482099056, -0.1433432698249817, -0.15321962535381317, 0.11210035532712936, -0.004553689621388912, -0.03803359344601631, -0.03788474202156067, -0.24240893125534058, 0.10499487817287445, -0.18534013628959656, -0.08100354671478271, -0.10774757713079453, 0.2689550518989563, 0.013774103485047817, 0.18135932087898254, -0.27061381936073303, 0.004330994561314583, -0.24388433992862701, -0.13752618432044983, 0.14961232244968414, -0.07382681220769882, 0.13968247175216675, 0.015691597014665604, -0.15566499531269073, 0.13034659624099731, -0.02634883113205433, 0.018046986311674118, -0.13745267689228058, -0.1211242750287056, -0.09927328675985336, 0.08072423934936523, 0.01302920188754797, 0.22597374022006989, 0.017691144719719887, 0.07033253461122513, 0.11613141745328903, 0.05969719588756561, -0.15267744660377502, -0.12432494014501572, 0.22119928896427155, 0.10001351684331894, -0.0729549452662468, -0.0388663150370121, -0.10896211862564087, -0.08643516153097153, 0.09582087397575378, -0.039107032120227814, 0.16889771819114685, -0.06124148890376091, -0.10789419710636139, -0.1620173156261444, -0.05077623203396797, -0.04993966221809387, -0.04757485166192055, -0.0074165137484669685, -0.09583595395088196, 0.1332034021615982, 0.0023590174969285727, -0.22418342530727386, -0.05906660854816437, 0.23728162050247192, -0.08777882903814316, -0.06653306633234024, -0.005952539853751659, 0.01842462457716465, 0.0812947005033493, 0.09572884440422058, 0.19156621396541595, -0.13914713263511658, -0.0327049158513546, 0.09761333465576172, -0.04982418566942215, -0.09077388793230057, -0.1368328034877777, -0.03264157101511955, 0.06416074931621552, -0.0042259409092366695, 0.05849330127239227, -0.055423419922590256, 0.03539750352501869, 0.2508888840675354, 0.14909400045871735, 0.01706298440694809, 0.1499084234237671, -0.08262048661708832, -0.029049141332507133, 0.04656608775258064, -0.2157597839832306, -0.07571031898260117, -0.12110790610313416, 0.2505935728549957, -0.0828489363193512, -0.038508832454681396, -0.08577453345060349, -0.04190846160054207, -0.08768092840909958, -0.07402615994215012, 0.05411412939429283, -0.14427849650382996, -0.2716106176376343, -0.020378323271870613, -0.09341834485530853, 0.16943658888339996, 0.01490597054362297, 0.08828567713499069, 0.06859958916902542, -0.15712064504623413, -0.06039318069815636, -0.03853568434715271, -0.01395278051495552, 0.0958804190158844, 0.0653795674443245, 0.06256232410669327, 0.056650545448064804, 0.0163236316293478, 0.03061126172542572, -0.030510153621435165, -0.02719411440193653, -0.07289698719978333, -0.17272959649562836, -0.1773422658443451, -0.06684127449989319, 0.19281533360481262, 0.05623708292841911, -0.007836760021746159, 0.018976639956235886, 0.21441255509853363, 0.11711481958627701, -0.02905152179300785, 0.03452170640230179, 0.026398610323667526, 0.11639821529388428, 0.062321871519088745, -0.009433828294277191, -0.2023719847202301, 0.014533213339745998, 0.07330285757780075, 0.1841963231563568, -0.02377971075475216, 0.014560622163116932, 0.04993787407875061, 0.019815267994999886, 0.041974034160375595, 0.035084281116724014, 0.22809432446956635, -0.07296112179756165, -0.1787717491388321, -0.009878544136881828, 0.0009811681229621172, -0.10432270914316177, -0.05274083837866783, 0.027188407257199287, 0.06856370717287064, 0.11503467708826065, 0.06878165155649185, 0.10073451697826385, 0.14253124594688416, 0.25623223185539246, -0.12886162102222443, -0.21569283306598663, -0.18707378208637238, -0.11744805425405502, -0.1987265646457672, -0.07750111073255539, 0.047856830060482025, -0.035649318248033524, -0.009999207220971584, 0.12131804972887039, 0.05097624659538269, -0.5835292935371399, 0.049290403723716736, -0.22683534026145935, -0.23371495306491852, -0.19958136975765228, -0.04497792199254036, 0.07820790261030197, 0.07226091623306274, -0.16655410826206207, 0.0301736518740654, 0.12192004919052124, 0.2187015414237976, -0.013419633731245995, -0.12597446143627167, -0.030346916988492012, -0.1915605217218399, -0.14034974575042725, -0.02007833682000637, 0.15165160596370697, -0.1470079869031906, -0.0704779326915741, 0.03836970403790474, 0.024286344647407532, -0.13729336857795715, -0.10971090942621231, 0.02169639617204666, -0.26914411783218384, -0.09759785234928131, 0.1846163123846054, 0.05995280295610428, 0.05899306759238243, -0.04058372601866722, 0.0654904767870903, -0.1661069095134735, -0.00787041150033474, -0.015798870474100113, 0.11130262166261673, -0.06676298379898071, 0.021855536848306656, -0.011005382053554058, -0.12687450647354126, -0.10920950025320053, 0.1882602870464325, 0.11368263512849808, 0.1271083801984787, -0.05830725282430649, -0.00841789785772562, 0.0781853124499321, 0.07215604931116104, -0.02195626311004162, 0.15286608040332794, 0.06042229384183884, -0.15904675424098969, -0.0761028453707695, -0.011555438861250877, -0.025991978123784065, 0.11000426113605499, 0.022478951141238213, -0.2854750156402588, 0.0911593958735466, -0.0012616364983841777, 0.1861090362071991, 0.043877013027668, 0.06310983002185822, -0.1818988025188446, -0.05954037234187126, -0.169515460729599, 0.017488980665802956, -0.16668520867824554, -0.20182034373283386, 0.17547036707401276, 0.13616207242012024, 0.05134142190217972, 0.01565362699329853, 0.1556175947189331}, {-0.0074188923463225365, -0.22337396442890167, 0.23035383224487305, -0.2111625075340271, -0.20008018612861633, -0.03694050386548042, -0.19859002530574799, 0.033440131694078445, 0.029750537127256393, 0.018907729536294937, 0.0009505401831120253, 0.28279244899749756, 0.15466351807117462, 0.2559301257133484, 0.1591033935546875, 0.1439915895462036, 0.12387621402740479, -0.12428929656744003, 0.11405322700738907, -0.2624015510082245, -0.08652595430612564, -0.06171715259552002, 0.1029013991355896, 0.022732216864824295, 0.14190521836280823, -0.006154128350317478, -0.03641843795776367, 0.06774019449949265, 0.09325364232063293, 0.11437702178955078, 0.210785374045372, 0.18559947609901428, -0.024958325549960136, -0.01555892638862133, 0.12626416981220245, 0.05831267312169075, -0.19269773364067078, 0.15703392028808594, 0.2140093296766281, -0.0451250784099102, -0.06199266389012337, 0.10559532791376114, -0.15972107648849487, -0.0042917546816170216, -0.008781380020081997, 0.04637548327445984, 0.051065247505903244, 0.13680678606033325, 0.012698506936430931, -0.05572439357638359, -0.02409999445080757, 0.25164470076560974, -0.0875079408288002, -0.12959758937358856, -0.022040318697690964, 0.14917030930519104, -0.13242533802986145, -0.11752814054489136, 0.35639750957489014, 0.15853458642959595, 0.09704311192035675, -0.15230070054531097, 0.17646846175193787, 0.06010153889656067, 0.26449647545814514, 0.02973174676299095, -0.07670366019010544, 0.1523456871509552, -0.10329887270927429, 0.20859378576278687, -0.029943278059363365, 0.13410229980945587, 0.1984570473432541, -0.04978051036596298, 0.3045862317085266, -0.3334965109825134, -0.08647887408733368, 0.3765720725059509, 0.05318690091371536, -0.20309098064899445, 0.17386886477470398, 0.10331963002681732, -0.07932332903146744, -0.18351387977600098, -0.10285643488168716, -0.18342794477939606, -0.2535213828086853, 0.005155195016413927, -0.1275452822446823, 0.14762744307518005, 0.04190325736999512, -0.12777557969093323, -0.05055374279618263, 0.1076512336730957, -0.18434405326843262, -0.05172888934612274, 0.02579822950065136, 0.06383002549409866, -0.01248449832201004, 0.08479481935501099, -0.0892670527100563, 0.13756683468818665, -0.13451117277145386, -0.07421153038740158, -0.014340057037770748, 1.0938438177108765, -0.005554017145186663, 0.20672759413719177, 0.09413539618253708, -0.21018719673156738, -0.022982599213719368, -0.059060391038656235, -0.08155883848667145, -0.15673993527889252, -0.12122678756713867, -0.2131502777338028, 0.12828204035758972, 0.013509636744856834, -0.040835995227098465, -0.2137676179409027, 0.03480680286884308, -0.0264056958258152, -0.0546402744948864, -0.20952767133712769, -0.06633937358856201, 0.20896103978157043, 0.06512679159641266, -0.07448470592498779, 0.056495681405067444, 0.11851342767477036, -0.06986388564109802, -0.05282001942396164, -0.1717374175786972, 0.10521949827671051, 0.335442453622818, -0.11022312939167023, 0.28388088941574097, -0.041534505784511566, -0.019454041495919228, -0.11118356883525848, 0.09777567535638809, 0.010798909701406956, -0.14013709127902985, 0.009741033427417278, 0.13660874962806702, -0.014301353134214878, -0.06518352776765823, 0.06087531894445419, 0.04634467884898186, 0.0204772986471653, 0.15566250681877136, -0.061276886612176895, 0.13996388018131256, -0.4095112085342407, -0.004092110320925713, 0.01301911287009716, 0.03823317587375641, -0.21133601665496826, 0.15097178518772125, -0.2045668512582779, -0.007153312209993601, -0.007534658070653677, 0.0303165502846241, -0.06497497111558914, -0.189874529838562, -0.014731823466718197, 0.3580482602119446, 0.02208803966641426, 0.10944299399852753, -0.11667007952928543, -0.03982365503907204, -0.08643696457147598, -0.03855833411216736, -0.006726702209562063, 0.10877637565135956, -0.20117323100566864, 0.055885493755340576, 0.10650946199893951, 0.10882899910211563, -0.0706295371055603, -0.0771460011601448, 0.4315023422241211, 0.06476853787899017, -0.06081999093294144, -0.15396307408809662, -0.0050778198055922985, -0.1315937340259552, 0.05040730535984039, 0.12858131527900696, 0.0425935760140419, -0.1576841026544571, 0.013239046558737755, 0.028988072648644447, 0.18906129896640778, 0.004661122802644968, -0.22852441668510437, -0.027461161836981773, 0.09933507442474365, -0.1413242667913437, -0.019901854917407036, 0.20188577473163605, 0.18195375800132751, -0.02137751504778862, 0.4371013045310974, -0.09241503477096558, 0.07495153695344925, 0.052705515176057816, 0.09840933233499527, -0.08306755870580673, -0.039885591715574265, 0.14015184342861176, 0.20479778945446014, 0.1649695485830307, -0.18214534223079681, -0.15648603439331055, 0.08269200474023819, -0.07232224196195602, -0.17204144597053528, -0.042442575097084045, 0.09341534227132797, -0.08295760303735733, 0.03929136320948601, 0.18035149574279785, 0.02620791643857956, -0.12601083517074585, 0.0029529351741075516, 0.18648481369018555, 0.07962913066148758, -0.004818524233996868, -0.05101398751139641, -0.005339618772268295, 0.1889188587665558, 0.09407656639814377, 0.10881584882736206, 0.10805165767669678, 0.0020347789395600557, 0.00804981216788292, 0.011009391397237778, 0.03911448270082474, 0.4249432682991028, -0.007072792388498783, 0.025533316656947136, -0.06255152076482773, 0.09894224256277084, 0.08878788352012634, 0.058159131556749344, 0.0596507266163826, 0.031088845804333687, 0.09599032998085022, -0.07987822592258453, 0.048939298838377, -0.2407526671886444, -0.14781489968299866, -0.06735234707593918, 0.06866645812988281, -0.014417523518204689}, {0.022844724357128143, 0.07109679281711578, -0.1745808869600296, 0.0315282829105854, 0.010456062853336334, -0.07199500501155853, 0.1750732809305191, -0.26986250281333923, 0.09991122037172318, -0.04492930695414543, 0.1025175005197525, -0.09435474127531052, -0.08233287185430527, 0.15615704655647278, -0.10273011028766632, 0.057566866278648376, 0.12683866918087006, -0.05052468553185463, -0.0277274027466774, -0.06370822340250015, 0.09308335185050964, -0.04083618149161339, -0.047516822814941406, -0.03155141323804855, 0.061298344284296036, 0.03031851351261139, -0.27100324630737305, -0.07308433204889297, -0.04649519547820091, 0.10901908576488495, 0.16658782958984375, -0.09681420773267746, -0.02905855141580105, 0.1664561927318573, 0.04331706091761589, -0.16560783982276917, 0.0006525007775053382, -0.0428994782269001, 0.007957384921610355, 0.050524741411209106, 0.011563301086425781, -0.12326458841562271, -0.042243484407663345, 0.10985298454761505, -0.047087687999010086, 0.05640066787600517, -0.04224666580557823, 0.052852481603622437, 0.08663672208786011, 0.11872383207082748, -0.1899360716342926, -0.34575971961021423, -0.024674858897924423, 0.093352310359478, -0.1141587346792221, 0.014959064312279224, -0.17456045746803284, -0.2015751153230667, -0.05883781984448433, 0.10874781757593155, 0.06438687443733215, 0.0728292390704155, 0.05366568639874458, -0.13489754498004913, -0.05009602755308151, -0.02645232155919075, 0.031132372096180916, 0.0766114890575409, 0.07607165724039078, 0.01228503230959177, 0.013179817236959934, 0.15622639656066895, -0.2939649522304535, -0.03128191456198692, 0.01815798319876194, 0.04108768701553345, 0.20140692591667175, -0.06065664812922478, 0.30909886956214905, 0.04396403953433037, -0.03534219413995743, 0.05593760684132576, -0.16337202489376068, -0.08539344370365143, -0.05060567334294319, 0.0050436994060873985, 0.06614825129508972, -0.23858992755413055, 0.12649445235729218, -0.005327990278601646, 0.17365704476833344, -0.07792957872152328, -0.029952216893434525, -0.10782822221517563, 0.3364463448524475, 0.2602829933166504, 0.009785739704966545, -0.08633112907409668, -0.13118986785411835, 0.09090033173561096, -0.09735676646232605, -0.14041933417320251, 0.07708696275949478, 0.04679527506232262, 0.2230358123779297, 0.28078678250312805, 0.0138573432341218, 0.09203200787305832, -0.022768406197428703, 0.06188974529504776, -0.17076580226421356, 0.018915323540568352, 0.10154213756322861, -0.04351238161325455, 0.032140713185071945, 0.055077239871025085, 0.022494349628686905, 0.07857239246368408, 0.13844646513462067, -0.1087723821401596, 0.20675137639045715, -0.17820699512958527, 0.18227294087409973, 0.038829173892736435, -0.023493461310863495, -0.012789249420166016, -0.12871821224689484, 0.06037971004843712, -0.10185328125953674, -0.07865188270807266, 0.059414032846689224, -0.18779399991035461, 0.18372397124767303, 0.08391576260328293, -0.1497824788093567, -0.0019034607103094459, -0.14330199360847473, -0.09783656895160675, 0.12247052043676376, 0.05868038907647133, -0.023690732195973396, -0.026501964777708054, 0.1306518167257309, -0.23865142464637756, 0.02289654314517975, 0.025229930877685547, 0.04873773083090782, 0.08765507489442825, 0.010665991343557835, -0.054338470101356506, -0.3049335479736328, 0.1277633011341095, 0.03824910521507263, 0.11036711931228638, -0.0008747017127461731, -0.040144387632608414, 0.12823809683322906, -0.1316595822572708, -0.06826163083314896, -0.2031247764825821, -0.09981314092874527, -0.07072578370571136, 0.17831993103027344, 0.09857743233442307, -0.12589311599731445, 0.29309675097465515, 0.0039809709414839745, -0.00272600119933486, -0.010433245450258255, 0.08730138838291168, 0.08733034133911133, -0.15298718214035034, -0.0361667238175869, -0.10679640620946884, -0.24900126457214355, -0.08205963671207428, -0.04564971104264259, 0.037252623587846756, 0.12585578858852386, -0.11907491832971573, 0.18501803278923035, -0.07145348936319351, 0.08851102739572525, -0.11837885528802872, 0.030095931142568588, 0.05714847892522812, -0.033654917031526566, -0.04929117485880852, -0.16723395884037018, -0.11672674864530563, 0.20564939081668854, 0.05866990610957146, -0.0409657321870327, -0.10630758106708527, 0.030515581369400024, 0.13994400203227997, -0.18988575041294098, 0.13587039709091187, -0.04829501733183861, -0.004336593672633171, 0.005273648537695408, 0.03211532533168793, 0.14425291121006012, 0.14388546347618103, 0.10796055197715759, 0.08934388309717178, 0.09243101626634598, -0.0846838727593422, 0.02608664520084858, -0.10942111164331436, -0.30725768208503723, -0.12417925894260406, -0.13761478662490845, 0.12430820614099503, -0.02220350131392479, -0.01162237860262394, -0.041735902428627014, 0.05104011297225952, 0.1539216935634613, -0.23783303797245026, 0.08425199240446091, 0.06377053260803223, -0.1763465702533722, -0.10621915757656097, 0.0388677716255188, -0.024783214554190636, -0.06020525097846985, 0.03165752440690994, -0.09544570744037628, 0.02778451144695282, -0.23899051547050476, 0.12331758439540863, -0.1836838722229004, -0.06403833627700806, 0.07499442994594574, -0.07084636390209198, -0.10301606357097626, -0.09124687314033508, 0.3242880702018738, -0.03179215267300606, 0.13945820927619934, -0.12575139105319977, 0.07720913738012314, 0.15179316699504852, 0.16757437586784363, -0.1493440866470337, 0.0032377103343605995, -0.14511308073997498, 0.006036593113094568, 0.0333976075053215, 0.12285066395998001, -0.00980517826974392, 0.0038106064312160015, -0.14713603258132935, -0.03414135053753853, -0.2255934476852417}, {0.0373242162168026, 0.16815289855003357, -0.08027621358633041, 0.01049169059842825, 0.2745552957057953, 0.20823505520820618, 0.019762998446822166, 0.20551958680152893, -0.025050316005945206, 0.13129495084285736, 0.004611624870449305, 0.14554744958877563, 0.07909709960222244, -0.03544705733656883, 0.17768946290016174, -0.14210909605026245, -0.13618485629558563, 0.2362496256828308, 0.13671806454658508, -0.05737028643488884, -0.050913237035274506, -0.0133441761136055, 0.005093749146908522, 0.09094011038541794, -0.13853806257247925, 0.023263409733772278, 0.07087168842554092, -0.04263169690966606, -0.02676583081483841, 0.11239118874073029, 0.03176604211330414, -0.37024080753326416, 0.007106195669621229, -0.006660176441073418, -0.12854696810245514, -0.05804930627346039, -0.07474759966135025, -0.16756659746170044, 0.013403658755123615, 0.22001661360263824, 0.07213718444108963, -0.06221408024430275, -0.03344997391104698, -0.12210828810930252, -0.004395920783281326, 0.014024375006556511, 0.20111368596553802, -0.08633878827095032, -0.11967839300632477, 0.18496263027191162, -0.026537252590060234, -0.30507466197013855, -0.10937333852052689, -0.05803370848298073, -0.010878052562475204, -0.07744736969470978, 0.19189517199993134, 0.09233950078487396, 0.15497776865959167, 0.05094332993030548, 0.02055063284933567, -0.14182248711585999, -0.16886253654956818, -0.18849068880081177, 0.012366554699838161, 0.07083547115325928, -0.005429473239928484, -0.09148529917001724, 0.08540219068527222, 0.08757699280977249, -0.046613264828920364, -0.04647640883922577, 0.18691180646419525, 0.0472523495554924, -0.09524036943912506, -0.021845035254955292, 0.02906135469675064, -0.1486525684595108, 0.02527214027941227, -0.016750846058130264, 0.00013707208563573658, -0.08892174810171127, 0.051618922501802444, -0.2584991157054901, -0.03763826563954353, 0.10248184949159622, 0.15410152077674866, -0.00904684979468584, -0.01323326863348484, 0.04746229201555252, -0.02828330732882023, -0.025043558329343796, 0.059826284646987915, -0.07555201649665833, 0.06478732079267502, 0.16304075717926025, 0.034081194549798965, -0.024572400376200676, -0.28759631514549255, 0.1710311472415924, -0.0035564214922487736, 0.14574034512043, 0.133816659450531, -0.012519651092588902, -0.16583985090255737, -0.04786491021513939, -0.04099661484360695, 0.2400328814983368, 0.1054539829492569, 0.2049265205860138, -0.028296684846282005, 0.11655597388744354, 0.08250942826271057, -0.18705005943775177, 0.00837879627943039, -0.018056446686387062, -0.026501044631004333, -0.09191826730966568, 0.1235901266336441, -0.21125124394893646, 0.017451971769332886, -0.2607841193675995, 0.1420603096485138, -0.14294683933258057, 0.15394224226474762, -0.11106240749359131, 0.16436363756656647, -0.004852558020502329, 0.006046958267688751, -0.11960156261920929, 0.028216833248734474, 0.03681517392396927, -0.0013409245293587446, -0.0019239607499912381, -0.43020713329315186, -0.07489364594221115, 0.13273848593235016, 0.19424286484718323, 0.08989597111940384, -0.07742039114236832, -0.03378090262413025, -0.10723388195037842, 0.005982912611216307, -0.028055189177393913, -0.013422906398773193, -0.07386691868305206, 0.14654706418514252, 0.009946558624505997, 0.0034787915647029877, -0.05071475729346275, -0.05888577923178673, -0.043726179748773575, -0.09657638520002365, 0.029352501034736633, -0.07187285274267197, 0.07413113117218018, 0.03609678894281387, -0.202340766787529, 0.1752721667289734, 0.0744667500257492, 0.05011788383126259, -0.040698762983083725, 0.09756731241941452, -0.12988857924938202, 0.17667968571186066, -0.4248153865337372, -0.1256732940673828, -0.11088259518146515, -0.10467260330915451, -0.04437120258808136, 0.05116216838359833, -0.12331598252058029, -0.0016901502385735512, 0.03639468923211098, -0.0042177531868219376, -0.01618291437625885, -0.10948295146226883, 0.027554217725992203, -0.05482562258839607, -0.027494145557284355, 0.09494215995073318, 0.15758493542671204, -0.0473368763923645, 0.3945002555847168, 0.0833786278963089, -0.23344425857067108, -0.38357672095298767, 0.015004624612629414, -0.1340644508600235, 0.14928428828716278, -0.028309481218457222, -0.030460752546787262, -0.02591043896973133, -0.13047988712787628, 0.2767125964164734, 0.05814104527235031, 0.2371264547109604, 0.020131468772888184, 0.01778208650648594, -0.024426139891147614, -0.03805948793888092, 0.019900064915418625, -0.05223473906517029, -0.10979695618152618, 0.19382743537425995, 0.02909855544567108, 0.10093402117490768, 0.12024179100990295, -0.0041320351883769035, 0.23872067034244537, 0.11203981935977936, -0.1534092277288437, 0.09096958488225937, -0.08361335098743439, -0.11284302175045013, -0.14168211817741394, -0.09489311277866364, -0.050051212310791016, 0.008839137852191925, -0.07957076281309128, -0.02296125702559948, -0.07354823499917984, -0.11400663107633591, -0.11249108612537384, -0.08638385683298111, -0.2928485870361328, 0.0750473365187645, 0.2377067506313324, -0.07541700452566147, -0.018660617992281914, -0.025066755712032318, 0.058001015335321426, -0.022615717723965645, -0.007082585711032152, 0.08916381746530533, -0.21392861008644104, 0.04694037139415741, -0.03528125584125519, -0.08353755623102188, -0.046729449182748795, 0.12381160259246826, 0.06107907369732857, 0.09996666014194489, 0.09267791360616684, -0.03690650314092636, 0.08003721386194229, -0.11348026990890503, 0.12180808931589127, 0.13024672865867615, -0.16157393157482147, 0.09859725087881088, 0.031028596684336662, 0.1644352823495865, -0.1348583847284317, -0.03445752337574959, -0.035321786999702454}, {-0.004541942849755287, -0.13431422412395477, -0.017040571197867393, 0.07852434366941452, -0.19351176917552948, 0.012150920927524567, -0.17354397475719452, 0.08717743307352066, -0.0339633971452713, 0.10209399461746216, 0.010663465596735477, 0.12845712900161743, 0.15868999063968658, 0.058614011853933334, -0.046046387404203415, 0.07465097308158875, -0.09889735281467438, 0.00916210375726223, -0.030470799654722214, -0.15460948646068573, -0.24885278940200806, -0.21573925018310547, -0.005027448758482933, -0.1759738028049469, 0.251508504152298, 0.05218837037682533, -0.06863942742347717, 0.03968265280127525, 0.1715112030506134, -0.019275186583399773, 0.228012353181839, -0.0037338004913181067, 0.07326071709394455, 0.07460208982229233, -0.10197106748819351, -0.042579784989356995, 0.07418275624513626, -0.061022426933050156, 0.03413650020956993, -0.17919446527957916, -0.08675137907266617, 0.11818969249725342, 0.12189658731222153, -0.07467807829380035, -0.015791183337569237, -0.0525968074798584, -0.017364609986543655, -0.10467097908258438, 0.058268506079912186, -0.07139839977025986, 0.1366093009710312, -0.3046310245990753, 0.001063992502167821, 0.08486362546682358, 0.04559312388300896, 0.06571837514638901, 0.08673271536827087, 0.030846156179904938, -0.23484651744365692, 0.20869489014148712, 0.17701362073421478, 0.02611961029469967, -0.06832460314035416, 0.017268173396587372, 0.16107259690761566, 0.17969448864459991, 0.05037618800997734, -0.009877143427729607, -0.029073597863316536, 0.032700538635253906, 0.016154181212186813, -0.09138774126768112, 0.3354191780090332, 0.30442801117897034, -0.5450937747955322, 0.08425648510456085, -0.04396877810359001, -0.324470192193985, 0.05613446608185768, 0.1261560022830963, 0.016091613098978996, -0.033180393278598785, -0.378183513879776, 0.18214082717895508, 0.016976376995444298, -0.0033443851862102747, 0.15877068042755127, 0.06886782497167587, -0.19107232987880707, 0.1468222737312317, -0.020838415250182152, -0.16235968470573425, 0.08330275118350983, 0.04373437538743019, 0.02054663375020027, 0.04591713473200798, 0.010683479718863964, 0.15191826224327087, -0.09956834465265274, 0.28851383924484253, 0.03066978231072426, -0.17176464200019836, -0.010662270709872246, -0.0561666414141655, 0.0038224938325583935, -0.18867723643779755, -0.15750376880168915, 0.04478996992111206, -0.12141990661621094, -0.1083621010184288, -0.06079626455903053, 0.01918601244688034, -0.0902947261929512, 0.06790347397327423, -0.031884074211120605, -0.07394324988126755, -0.023643387481570244, 0.10604622215032578, -0.04603510722517967, -0.12068389356136322, -0.013483054004609585, 0.16395427286624908, 0.0396699532866478, -0.14880795776844025, -0.13023099303245544, -0.12438330054283142, -0.05819432809948921, 0.12274271994829178, -0.03927795588970184, 0.10448291897773743, 0.0901423990726471, -0.15706710517406464, -0.26965129375457764, -0.07501067966222763, 0.1538570523262024, -0.02888285554945469, 0.05582709237933159, -0.11494989693164825, -0.04190709814429283, 0.2338385134935379, -0.02729872800409794, 0.02606343850493431, -0.0761469230055809, -0.08935830742120743, 0.06676903367042542, -0.016651254147291183, 0.14796538650989532, -0.0787821039557457, -0.08635102957487106, 0.04342931509017944, 0.053808897733688354, -0.011017967015504837, -0.06615591049194336, -0.08932756632566452, 0.0269013624638319, 0.08165103942155838, -0.18381956219673157, -0.2611910104751587, -0.2607574462890625, -0.02365253120660782, 0.02975495345890522, 0.03029993362724781, 0.0755910873413086, 0.18273191154003143, -0.2507861256599426, -0.1832750141620636, -0.13266809284687042, -0.26255539059638977, 0.05972573161125183, -0.13918378949165344, 0.10016553103923798, 0.11470665782690048, -0.09690853208303452, -0.06595397740602493, -0.13424821197986603, 0.06990731507539749, 0.09378957003355026, 0.0268560741096735, -0.00056832330301404, -0.09573130309581757, -0.018066788092255592, 0.12170484662055969, 0.25177526473999023, 0.15510183572769165, -0.04739740118384361, -0.036125533282756805, -0.2723568379878998, 0.10482516884803772, -0.0333673469722271, 0.016103405505418777, 0.02964816987514496, -0.052710577845573425, -0.15232326090335846, 0.3011096715927124, 0.09107585996389389, 0.10204334557056427, -0.037854958325624466, -0.16718776524066925, -0.04017522931098938, -0.17722246050834656, 0.044687867164611816, 0.06858190149068832, -0.23124709725379944, 0.177671417593956, 0.10076862573623657, -0.00441237585619092, 0.010174499824643135, 0.039444711059331894, -0.0020679286681115627, 0.08224524557590485, 0.11633217334747314, -0.27184098958969116, -0.3152977526187897, 0.034967292100191116, 0.026138827204704285, 0.17703385651111603, 0.05030438303947449, 0.0031059798784554005, 0.24232174456119537, -0.039314933121204376, -0.10205444693565369, -0.0021248492412269115, 0.2065081000328064, -0.16332492232322693, -0.02868855744600296, -0.12001856416463852, -0.05619904026389122, 0.07719217985868454, -0.11556707322597504, 0.27380868792533875, 0.006513582542538643, 0.019226323813199997, 0.1581965684890747, -0.07180546224117279, -0.15456421673297882, -0.09335807710886002, 0.1321648210287094, 0.08222023397684097, 0.026674168184399605, -0.057623010128736496, 0.033982858061790466, -3.188267510267906e-05, -0.1819687783718109, 0.041772887110710144, -0.0880061686038971, -0.030027901753783226, -0.03477754816412926, 0.2854006886482239, 0.13361527025699615, 0.20918507874011993, 0.14555230736732483, -0.42516353726387024, -0.08587376028299332, -0.12650690972805023, -0.06997707486152649, -0.17534351348876953}, {-0.053403791040182114, 0.04678523540496826, 0.02835637517273426, -0.015739601105451584, -0.07378331571817398, -0.38776010274887085, -0.07486983388662338, 0.0016322197625413537, 0.1057482659816742, 0.03371644392609596, -0.17259559035301208, 0.17522789537906647, 0.02439386397600174, -0.029305659234523773, 0.060858409851789474, 0.060794103890657425, 0.14062218368053436, -0.06206899136304855, -0.15864156186580658, 0.14800117909908295, 0.039652928709983826, -0.022425029426813126, 0.02250855043530464, 0.26009050011634827, 0.0038258349522948265, 0.04690898209810257, 0.06922100484371185, 0.13660669326782227, -0.0403982549905777, -0.08747466653585434, -0.06850548833608627, 0.043662283569574356, -0.0631994903087616, 0.12401121854782104, 0.05458465963602066, -0.008332003839313984, -0.18335239589214325, -0.11329657584428787, -0.08388232439756393, -0.042324770241975784, 0.2844427525997162, 0.10406305640935898, -0.016036594286561012, -0.11741743981838226, 0.12031051516532898, -0.16828563809394836, -0.039378054440021515, -0.084756039083004, -8.526146120857447e-05, 0.20669510960578918, 0.004516233690083027, 0.1342362016439438, 0.09709969907999039, -0.010625917464494705, -0.010522203519940376, 0.07302223145961761, -0.08311735093593597, -0.18173539638519287, -0.053027842193841934, -0.02172682248055935, 0.12813931703567505, -0.11107988655567169, 0.05707321688532829, 0.024637136608362198, 0.02659623511135578, -0.08614572137594223, -0.16955061256885529, 0.022660918533802032, -0.03262436389923096, -0.03651929274201393, 0.006491581443697214, -0.23658257722854614, -0.0014137865509837866, 0.04874364659190178, -0.018314389511942863, -0.007798733189702034, -0.3292965292930603, 0.08721388876438141, 0.010403772816061974, -0.23131181299686432, -0.16055528819561005, -0.21564514935016632, -0.12623979151248932, -0.04958612099289894, -0.33127105236053467, 0.013602742925286293, -0.23757310211658478, -0.030319493263959885, 0.11247792094945908, -0.07825309783220291, -0.12988051772117615, -0.018655484542250633, -0.13945730030536652, 0.06421270966529846, 0.12791715562343597, 0.08333569020032883, 0.09127996116876602, 0.1012534350156784, 0.04343809559941292, -0.061694785952568054, 0.1052733063697815, -0.00711760064586997, -0.1681809276342392, 0.19434842467308044, -0.014613122679293156, 0.025825412943959236, 0.17084185779094696, -0.00974667351692915, 0.17547576129436493, 0.02336473949253559, -0.09344551712274551, -0.018204929307103157, -0.1350221335887909, 0.19788050651550293, 0.20126381516456604, -0.12499064952135086, 0.03210461512207985, 0.058638956397771835, 0.060193780809640884, 0.0375247485935688, -0.07398392260074615, -0.03656995669007301, 0.029373006895184517, 0.05290970206260681, -0.4292711019515991, 0.1387716829776764, 0.1595408171415329, -0.11644832044839859, -0.06796154379844666, 0.018785811960697174, -0.02326362207531929, -0.10208073258399963, 0.0984681099653244, -0.10916539281606674, -0.2312745302915573, 0.16117307543754578, -0.0071436562575399876, 0.05633252114057541, -0.17787837982177734, 0.1841539442539215, -0.057596929371356964, 0.0433201864361763, 0.09791015833616257, 0.07879641652107239, 0.2248639166355133, 0.07131529599428177, -0.05269074812531471, -0.15124428272247314, 0.3684737980365753, 0.10772679001092911, -0.1610763669013977, -0.1028323844075203, 0.0928235724568367, 0.010370821692049503, 0.06550332903862, 0.14448291063308716, -0.0769263431429863, -0.05929741635918617, -0.007317437790334225, 0.061830271035432816, -0.1149270161986351, -0.04812033474445343, -0.17612196505069733, -0.006569596007466316, 0.07211042940616608, -0.24798360466957092, 0.19440889358520508, -0.08537302911281586, 0.40013667941093445, 0.31220391392707825, 0.0682019367814064, -0.009393354877829552, -0.08302845805883408, -0.02798394300043583, 0.04182006046175957, 0.26069337129592896, 0.12910638749599457, 0.15179342031478882, -0.00017112711793743074, -0.0314137265086174, -0.08591865003108978, 0.1331384778022766, 0.14067605137825012, 0.09329605102539062, 0.04715435951948166, -0.1342279464006424, 0.02709621749818325, 0.12212911248207092, -0.09732785075902939, 0.19732043147087097, -0.1034095510840416, 0.0013590905582532287, -0.26553764939308167, 0.04418569803237915, -0.2385585755109787, -0.27214398980140686, 0.19945810735225677, 0.15729846060276031, 0.1258130967617035, -0.07677415013313293, -0.07366029918193817, 0.23664183914661407, -0.061793241649866104, 0.01056607998907566, 0.05094777047634125, -0.05012292414903641, 0.007384952157735825, -0.23071518540382385, -0.12767015397548676, -0.1724080592393875, 0.2613552212715149, 0.30579259991645813, 0.0002579180581960827, 0.08573304861783981, -0.1818396896123886, -0.1380574256181717, -0.11005160957574844, -0.06774453818798065, 0.3944154977798462, 0.023896928876638412, 0.16993139684200287, 0.22170299291610718, -0.15049190819263458, 0.4246607720851898, 0.07551265507936478, 0.17573362588882446, -0.15881651639938354, 0.029270926490426064, 0.20378005504608154, -0.04998950660228729, 0.055581673979759216, -0.04199434444308281, -0.00689671840518713, 0.11220421642065048, -0.09270123392343521, -0.15636318922042847, 0.03662976995110512, -0.05104966461658478, -0.10275141149759293, 0.04889330640435219, 0.08143244683742523, -0.05421174317598343, -0.0875420793890953, -0.09390238672494888, -0.17510387301445007, -0.17027102410793304, -0.16833585500717163, 0.12964481115341187, 0.07072801142930984, 0.03498320281505585, 0.2377668023109436, 0.08612160384654999, -0.13476960361003876, -0.11889515072107315, 0.09631513059139252, -0.04442410543560982}, {0.013370322063565254, 0.19964316487312317, 0.21008673310279846, 0.13696734607219696, -0.012577274814248085, -0.1503024399280548, -0.1540033370256424, 0.07479548454284668, -0.001048391219228506, -0.2913168668746948, -0.16232667863368988, -0.031655728816986084, -0.15023173391819, 0.05156799405813217, -0.08716518431901932, 0.08017436414957047, -0.1130203902721405, 0.2271861582994461, -0.1360889971256256, -0.0997309461236, -0.16147103905677795, 0.1734502613544464, 0.019087260589003563, 0.11584999412298203, 0.04619395732879639, -0.24617670476436615, 0.06308384984731674, 0.02707711234688759, -0.11624068766832352, 0.08567101508378983, 0.11419020593166351, 0.2365896850824356, 0.07456698268651962, 0.06871195882558823, -0.2047988921403885, -0.1009850949048996, -0.1134508028626442, -0.18787021934986115, -0.11246982216835022, -0.14876721799373627, 0.07168041169643402, 0.031493525952100754, -0.1636594533920288, 0.12276358902454376, -0.12289942800998688, -0.21607184410095215, 0.059475548565387726, -0.06614702194929123, 0.07078776508569717, 0.1915508508682251, -0.0601128488779068, -0.05402464419603348, -0.08207109570503235, -0.020346365869045258, 0.16377367079257965, 0.05784237012267113, 0.18923519551753998, 0.09598051011562347, 0.14688698947429657, -0.3298143744468689, 0.1446591317653656, 0.08862554281949997, 0.1816161721944809, 0.16773384809494019, 0.17309114336967468, -0.027414100244641304, -0.13835130631923676, -0.023371446877717972, -0.021151892840862274, 0.022051703184843063, 0.004427230916917324, -0.02521168254315853, -0.18594682216644287, 0.07627453655004501, 0.20066098868846893, -0.13466693460941315, -0.06331783533096313, 0.18643125891685486, -0.11011532694101334, -0.07530324906110764, -0.037315886467695236, -0.06559079140424728, -0.017242545261979103, -0.023410139605402946, 0.03056965582072735, -0.08463474363088608, -0.08325713127851486, 0.019762247800827026, 0.08908390998840332, -0.11125115305185318, 0.042213618755340576, 0.1569099724292755, -0.0377223901450634, -0.018242070451378822, -0.03127521276473999, -0.035684142261743546, -0.09801706671714783, -0.15864841639995575, -0.04193775728344917, -0.10008691251277924, 0.1312916874885559, -0.037853483110666275, -0.1376028060913086, 0.21711614727973938, 0.050314027816057205, 0.06352086365222931, 0.022184479981660843, 0.006529724225401878, -0.007888172753155231, -0.044069696217775345, -0.0717349573969841, 0.16063301265239716, 0.13757948577404022, 0.06750299036502838, -0.23512236773967743, -0.1321554332971573, -0.03671915829181671, -0.03516077995300293, 0.23187528550624847, -0.05975061282515526, -0.05233665183186531, 0.0492430105805397, -0.004418472293764353, -0.02175588719546795, -0.008120113052427769, -0.18344295024871826, 0.02659502625465393, -0.08606510609388351, -0.051333699375391006, 0.09980399906635284, 0.2814924418926239, 0.03138278052210808, 0.06993603706359863, -0.04720740020275116, 0.15186624228954315, 0.11874344944953918, 0.08512436598539352, -0.12597070634365082, -0.0231869388371706, -0.07170344889163971, 0.09646914899349213, -0.20096592605113983, -0.10647173225879669, 0.07527771592140198, 0.046607792377471924, -0.02010222338140011, -0.05422162637114525, -0.14187167584896088, 0.17140032351016998, -0.21728430688381195, 0.07961319386959076, -0.11300519108772278, 0.19342012703418732, -0.34959515929222107, 0.1269107311964035, 0.004889992065727711, -0.14766991138458252, 0.052318498492240906, -0.02922721579670906, 0.12931984663009644, -0.21624109148979187, -0.2777050733566284, 0.11184844374656677, 0.02842503786087036, -0.025064237415790558, -0.005335664376616478, -0.22149477899074554, -0.12294058501720428, -0.064767986536026, -0.1219417080283165, 0.20804138481616974, -0.0660247653722763, 0.03145132586359978, -0.17920808494091034, 0.07662564516067505, -0.03956522420048714, -0.23200532793998718, -0.042265355587005615, -0.011560413055121899, 0.00957365334033966, -0.153533473610878, 0.09942468255758286, -0.02195652946829796, 0.08299587666988373, 0.16047975420951843, -0.040316518396139145, -0.23536036908626556, 0.07633163034915924, 0.1827901154756546, 0.01820039190351963, 0.17894673347473145, -0.01779191941022873, -0.00647909427061677, 0.0007411791011691093, -0.048751119524240494, -0.1798466444015503, 0.03317306935787201, 0.2020508348941803, 0.10479620844125748, -0.03644194081425667, 0.1917155683040619, 0.07416413724422455, 0.3176168203353882, 0.05835884436964989, -0.11410050839185715, 0.029719285666942596, 0.29907897114753723, 0.08908261358737946, 0.24160590767860413, -0.10856901109218597, -0.12508057057857513, -0.1924630105495453, -0.08540313690900803, -0.22133716940879822, -0.04173370823264122, -0.11301465332508087, -0.051989372819662094, -0.24163003265857697, 0.06567328423261642, -0.24973522126674652, 0.05264006927609444, 0.11368793994188309, 0.08725608885288239, -0.14191199839115143, 0.009279920719563961, -0.16807115077972412, 0.021393373608589172, -0.13222482800483704, 0.0852561742067337, 0.05774332955479622, -0.16431273519992828, -0.00256899930536747, -0.35899826884269714, -0.10513416677713394, 0.007363506127148867, -0.08822400122880936, -0.20914021134376526, 0.2163897603750229, -0.009540123865008354, 0.14186373353004456, -0.048883773386478424, 0.3472134470939636, -0.03983086347579956, -0.18083259463310242, 0.19477829337120056, 0.03363887593150139, 0.08814899623394012, 0.08013298362493515, 0.0315520353615284, -0.034628868103027344, -0.05153023451566696, -0.04793959856033325, -0.07866284996271133, 0.1438365876674652, -0.09632445126771927, -0.06006960570812225}, {-0.12463537603616714, 0.07740995287895203, -0.2549806237220764, -0.21731282770633698, 0.06203581020236015, 0.04775777459144592, -0.02522248961031437, 0.07236310839653015, -0.1561715304851532, 0.0029705814085900784, 0.09920446574687958, 0.17938685417175293, -0.061534423381090164, -0.1946462094783783, 0.028734922409057617, 0.08806483447551727, -0.16152559220790863, 0.1028328537940979, -0.03229145333170891, 0.10484380275011063, -0.02500096708536148, 0.011583789251744747, -0.11992834508419037, 0.1057579442858696, 0.07383852452039719, 0.2351544350385666, 0.12787017226219177, -0.03276645019650459, 0.022971084341406822, -0.04918697476387024, -0.06280366331338882, -0.18069754540920258, -0.030360782518982887, -0.017641380429267883, 0.11531858146190643, 0.08254720270633698, -0.04414527118206024, -0.04176176339387894, 0.12104742974042892, 0.004092616960406303, 0.09832216799259186, -0.21947133541107178, 0.08886604756116867, -0.10935674607753754, -0.08457725495100021, 0.113314688205719, 0.14368852972984314, -0.09084375947713852, -0.13681742548942566, -0.03378121927380562, 0.019612230360507965, 0.09470540285110474, -0.0996154323220253, 0.3222462236881256, 0.04309297725558281, 0.22262929379940033, 0.18389341235160828, -0.11458748579025269, -0.20892202854156494, 0.1823626309633255, -0.22246652841567993, -0.10774834454059601, -0.1349366456270218, 0.1826179474592209, 0.10160985589027405, -0.10066133737564087, 0.09324806928634644, 0.10900290310382843, 0.05362965911626816, -0.009072275832295418, -0.3675425946712494, -0.04543016850948334, 0.03402993083000183, 0.02080726809799671, -0.18493546545505524, 0.21986080706119537, 0.0017849354771897197, -0.022864557802677155, 0.3482601046562195, 0.03307339921593666, 0.2298380583524704, 0.21214081346988678, 0.11555697023868561, -0.1389816701412201, -0.010251547209918499, -0.114328533411026, -0.08447803556919098, 0.06621618568897247, -0.038629453629255295, 0.14023619890213013, -0.02672308124601841, 0.062089383602142334, -0.17259736359119415, -0.022335397079586983, -0.1088269054889679, 0.004301310982555151, -0.049364615231752396, 0.07509571313858032, 0.01884232647716999, 0.09599903225898743, -0.049236517399549484, 0.0029271712992340326, 0.06928393989801407, -0.19811135530471802, -0.05537078529596329, 0.4727359116077423, -0.0279321800917387, 0.24189718067646027, 0.16834473609924316, -0.08555875718593597, 0.1364147961139679, 0.13876225054264069, -0.07004018127918243, 0.11948730796575546, 0.08914962410926819, -0.16288872063159943, 0.04823198541998863, 0.1316794604063034, -0.10537824034690857, -0.0424371175467968, 0.023734193295240402, 0.1228126734495163, 0.032999392598867416, -0.04381118342280388, 0.1529071182012558, -0.03509492054581642, 0.06561361998319626, 0.09955885261297226, -0.2631103992462158, 0.020634304732084274, -0.09468138217926025, 0.07849209010601044, -0.13072557747364044, -0.07036728411912918, 0.21151503920555115, 0.06799862533807755, -0.04060738906264305, -0.058360982686281204, 0.13967692852020264, -0.19415631890296936, -0.016787061467766762, 0.15688422322273254, -0.2733043134212494, 0.007899893447756767, 0.14894407987594604, -0.07201752066612244, -0.04021307826042175, 0.08454436808824539, 0.02785436250269413, -0.011162389069795609, -0.16705794632434845, 0.0024091496597975492, -0.06590534001588821, 0.06869760155677795, 0.0017029654700309038, 0.10102152824401855, 0.07988946884870529, 0.3866334855556488, 0.097742959856987, 0.0057272957637906075, -0.0942075327038765, -0.13388893008232117, -0.06928437948226929, -0.15628421306610107, -0.01791432499885559, -0.11873938888311386, 0.1948588788509369, -0.05466330423951149, 0.0952007845044136, 0.09641208499670029, -0.039557572454214096, 0.027445947751402855, 0.1792987585067749, 0.053539276123046875, 0.08165309578180313, -0.007157446350902319, 0.03661832958459854, -0.10605189949274063, -0.2180132120847702, 0.18669411540031433, -0.014583389274775982, 0.16709016263484955, 0.06519239395856857, 0.15851549804210663, 0.0626273825764656, -0.11181745678186417, -0.2772611677646637, -0.004950488917529583, 0.11036301404237747, -0.23725125193595886, -0.11343858391046524, 0.041935261338949203, 0.20458340644836426, 0.08591736853122711, 0.02788543328642845, -0.2198733687400818, 0.07202687114477158, -0.11206312477588654, -0.06647665053606033, -0.10343927890062332, -0.016421686857938766, 0.1278572678565979, -0.05257268622517586, -0.022143982350826263, 0.012713400647044182, 0.09872863441705704, -0.2671120762825012, 0.027711067348718643, 0.18673576414585114, -0.17743965983390808, 0.08932707458734512, -0.10937994718551636, 0.08528745919466019, -0.08220341056585312, -0.05273739993572235, 0.002278911881148815, -0.17393061518669128, -0.17848117649555206, -0.09377621114253998, -0.0814463198184967, 0.02322579175233841, -0.12294407933950424, 0.03382659703493118, -0.1472819447517395, -0.26718586683273315, -0.10435457527637482, -0.057672563940286636, -0.031018299981951714, -0.1367068886756897, 0.032130077481269836, -0.17742854356765747, -0.10815563797950745, -0.1962333619594574, 0.1349392980337143, 0.06054733693599701, -0.003623203607276082, 0.10327287763357162, -0.031060166656970978, -0.179486945271492, 0.029313592240214348, 0.02848884090781212, 0.13241450488567352, 0.009137162007391453, -0.1368447244167328, 0.13183066248893738, -0.09670182317495346, 0.0708308070898056, -0.017672309651970863, 0.07931435853242874, -0.13300468027591705, 0.012357180006802082, 0.09572049975395203, 0.02547932229936123, -0.05777658522129059, 0.05669192969799042, -0.19912852346897125}, {-0.19040291011333466, -0.10817041248083115, -0.25125420093536377, 0.015885522589087486, 0.012040882371366024, 0.019192354753613472, -0.03406580165028572, -0.060669928789138794, -0.09236133098602295, 0.06222265213727951, 0.0013332495000213385, -0.18448501825332642, -0.06699652224779129, 0.01951221562922001, 0.23304744064807892, 0.07768599689006805, -0.12504884600639343, -0.13906605541706085, 0.21188735961914062, 0.015097355470061302, 0.045812610536813736, 0.12910036742687225, 0.027701575309038162, 0.041602011770009995, -0.08544666320085526, -0.008432830683887005, 0.039798930287361145, -0.055038418620824814, 0.006971340160816908, 0.1472157984972, -0.3393295109272003, 0.011386385187506676, 0.05680885910987854, 0.05682031810283661, -0.05102613940834999, -0.1298871785402298, -0.013751537539064884, 0.18787696957588196, 0.08389656245708466, -0.04352908208966255, -0.10615391284227371, -0.008684121072292328, 0.06343459337949753, 0.0016055312007665634, -0.07089966535568237, 0.03010823018848896, -0.13231322169303894, -0.034541189670562744, 0.014799504540860653, -0.04887969046831131, 0.04313086345791817, 0.02272428572177887, -0.2997255027294159, 0.04536040872335434, 0.02330285683274269, -0.12417221069335938, 0.055869508534669876, 0.08000106364488602, 0.03619033098220825, 0.02733326517045498, -0.20524805784225464, -0.16999800503253937, -0.14578916132450104, -0.07872845977544785, -0.007398735731840134, -0.004240105394273996, -0.16222739219665527, 0.22552673518657684, 0.09092974662780762, 0.13300670683383942, -0.0468938872218132, 0.10335234552621841, -0.21242272853851318, -0.09046168625354767, -0.020699474960565567, -0.007014145143330097, 0.1500624269247055, -0.002504479605704546, 0.1902480274438858, -0.01366332732141018, -0.11690495163202286, 0.06362573057413101, 0.1877194494009018, -0.07468542456626892, 0.193545863032341, -0.13524088263511658, 0.00798889435827732, -0.04490949213504791, 0.10273104161024094, 0.03830079734325409, 0.07234997302293777, 0.13837353885173798, -0.07747974246740341, -0.16084106266498566, 0.016600454226136208, 0.1298428773880005, -0.12221734970808029, -0.08608702570199966, 0.06822271645069122, 0.050319574773311615, 0.06749460101127625, 0.06342125684022903, 0.19516664743423462, 0.006570884492248297, 0.06830434501171112, 0.18484775722026825, 0.036192722618579865, 0.28367456793785095, -0.04244403541088104, 0.049892883747816086, -0.026358187198638916, -0.07503732293844223, 0.09119676798582077, -0.10525622963905334, 0.05415894836187363, 0.055293694138526917, -0.14591476321220398, 0.0018904071766883135, 0.00774572417140007, -0.08546977490186691, 0.08368662744760513, -0.059503547847270966, 0.11834100633859634, 0.2631482779979706, 0.09362801909446716, 0.2837890088558197, -0.08600981533527374, 0.08099757134914398, 0.05763447284698486, 0.049942996352910995, -0.02482651360332966, 0.019153395667672157, 0.26050540804862976, -0.07559877634048462, 0.17246589064598083, -0.14440645277500153, -0.004275267478078604, -0.1672883927822113, 0.2960677444934845, -0.055298853665590286, -0.05678273364901543, 0.16023992002010345, 0.08653499186038971, -0.1228262186050415, 0.14975014328956604, -0.15333132445812225, 0.07569454610347748, 0.07075173407793045, -0.11580044031143188, 0.0394878163933754, -0.13607412576675415, -0.004212378524243832, -0.04725079610943794, 0.07379171997308731, -0.11166910082101822, 0.16102086007595062, -0.19808100163936615, 0.15174607932567596, 0.003266699379310012, 0.049361467361450195, -0.07858030498027802, 0.027236662805080414, 0.1645023226737976, -0.12367421388626099, -0.19471009075641632, 0.21010282635688782, -0.16635321080684662, -0.06475361436605453, -0.041922006756067276, 0.06014837697148323, -0.17899014055728912, -0.13274718821048737, -0.09419295936822891, -0.08628927171230316, -0.011068052612245083, 0.02029423415660858, 0.2153354287147522, -0.07079417258501053, 0.07118292152881622, 0.020137550309300423, 0.23114916682243347, -0.19773192703723907, 0.040943026542663574, 0.08123476058244705, 0.060203928500413895, -0.10308438539505005, 0.002186507685109973, 0.00351496203802526, -0.054225582629442215, -0.037669021636247635, -0.030028728768229485, -0.08975765109062195, 0.033181384205818176, -0.08198964595794678, -0.24685823917388916, 8.512857311870903e-05, -0.05409922078251839, -0.22079741954803467, 0.018617957830429077, 0.16110436618328094, 0.06971003860235214, 0.13151517510414124, -0.1981135904788971, -0.24624042212963104, -0.07618705183267593, 0.09892134368419647, 0.12577098608016968, -0.10051418840885162, -0.14875613152980804, 0.08680450916290283, -0.047991372644901276, -0.1503455489873886, 0.04729001969099045, -0.19351619482040405, -0.023540008813142776, 0.03820989280939102, -0.03103473410010338, 0.02561727538704872, -0.1400575488805771, 0.20097757875919342, 0.11313051730394363, 0.033055759966373444, 0.1727115511894226, -0.11952637881040573, -0.1510232388973236, -0.008599269203841686, -0.13993312418460846, -0.0004235219967085868, -0.0595674067735672, -0.0786575973033905, -0.030471576377749443, 0.07985001802444458, 0.07305781543254852, -0.1240985095500946, -0.06264059245586395, 0.15982073545455933, 0.15396712720394135, -0.13199591636657715, 0.058916088193655014, -0.004233261104673147, 0.15434454381465912, 0.01608818955719471, -0.0727936252951622, -0.17020800709724426, -0.1717880666255951, -0.021927358582615852, 0.023430747911334038, 0.12683649361133575, -0.03958681970834732, -0.05334033817052841, 0.048278894275426865, 0.150652214884758, 0.1112411618232727, -0.07046576589345932, 0.20361019670963287, -0.33657321333885193}, {0.05386924371123314, -0.04770691692829132, -0.06163337826728821, 0.07643502950668335, -0.10866346955299377, -0.14261439442634583, 0.01253057923167944, 0.061821091920137405, -0.08043539524078369, -0.008107504807412624, 0.018820367753505707, -0.2507706582546234, -0.25004589557647705, -0.06750745326280594, 0.09916682541370392, -0.04259304702281952, -0.11487235128879547, -0.0031356909312307835, 0.05898190662264824, 0.18103395402431488, -0.0163828507065773, -0.04362305998802185, -0.059251103550195694, -0.0352727510035038, 0.06183270364999771, 0.1902187466621399, -0.0004779843147844076, 0.08844156563282013, -0.10732323676347733, -0.0032770861871540546, 0.2962610721588135, 0.10509251058101654, -0.10245682299137115, 0.17760783433914185, -0.03150559961795807, -0.13167652487754822, 0.0585402175784111, -0.12402231246232986, -0.10166275501251221, 0.15461203455924988, 0.037268832325935364, -0.13201728463172913, -0.003483494510874152, 0.13505211472511292, -0.08275236934423447, -0.015804681926965714, -0.020961271598935127, -0.11445625871419907, 0.16240587830543518, -0.058180682361125946, -0.12227127701044083, 0.14336811006069183, 0.05714360624551773, 0.001202225685119629, -0.03699528053402901, 0.08301545679569244, 0.15099109709262848, -0.06799639016389847, 0.026061171665787697, -0.09920019656419754, 0.09478642046451569, -0.034469589591026306, 0.14040903747081757, -0.19633108377456665, -0.012637908570468426, 0.08237490057945251, 0.23175878822803497, 0.05502486601471901, 0.013531701639294624, 0.11308777332305908, 0.1087760254740715, -0.07280042767524719, 0.6297755241394043, 0.18466173112392426, -0.010742591693997383, 0.027771523222327232, 0.0068049561232328415, 0.010882154107093811, -0.05577048659324646, 0.16350345313549042, -0.07299001514911652, 0.09234599769115448, -0.03014661744236946, -0.1440889537334442, -0.08017980307340622, -0.06790738552808762, 0.14368866384029388, -0.037429697811603546, -0.28504279255867004, -0.1196717619895935, 0.0056813242845237255, -0.05240074172616005, -0.04535433650016785, -0.2717394232749939, -0.0015483774477615952, 0.021063346415758133, -0.013351635076105595, 0.16240078210830688, -0.20448128879070282, -0.015987416729331017, -0.05457796901464462, 0.13525915145874023, 0.05331948772072792, -0.0018961462192237377, -0.15730971097946167, -0.12247104197740555, -0.00726200919598341, -0.03597487136721611, 0.15824060142040253, 0.2264968603849411, -0.07459641993045807, 0.1787099987268448, 0.2809547185897827, 0.10023026913404465, -0.10904691368341446, 0.1010306179523468, -0.08415549248456955, 0.08372782915830612, 0.04779302701354027, 0.053657226264476776, 0.05109837278723717, 0.07571406662464142, -0.014862798154354095, -0.0448848120868206, 0.04906279966235161, 0.026228077709674835, -0.02730816975235939, 0.014096449129283428, -0.09583302587270737, 0.07454805821180344, -0.059792667627334595, -0.06406041234731674, -0.4889635741710663, 0.060676589608192444, 0.18796995282173157, -0.10189595818519592, 0.027416398748755455, -0.02184308134019375, -0.06666699796915054, -0.016236545518040657, 0.009161830879747868, -0.061344508081674576, 0.146531879901886, 0.01476308237761259, -0.43715187907218933, -0.16068382561206818, 0.10067038983106613, -0.02089444361627102, 0.06003427878022194, 0.14047452807426453, 0.019210953265428543, 0.02410178631544113, -0.15531080961227417, 0.021156782284379005, -0.13944435119628906, 0.06966117769479752, -0.20762227475643158, -0.1732727736234665, 0.011607671156525612, -0.1453714668750763, -0.0269194133579731, 0.05834841728210449, -0.027460994198918343, -0.0772719457745552, 0.0032835861202329397, -0.016964411363005638, -0.1253555864095688, 0.014425663277506828, 0.08299797028303146, 0.1543574184179306, 0.06704721599817276, 0.014805309474468231, -0.060279056429862976, -0.1818535476922989, -0.04178100824356079, -0.16029241681098938, -0.1466056853532791, -0.19474107027053833, -0.007182397413998842, 0.14346754550933838, -0.03025050461292267, -0.16468378901481628, -0.09948033094406128, 0.14066439867019653, 0.05680272728204727, 0.09756060689687729, -0.062301404774188995, 0.1190260574221611, -0.01723441854119301, 0.04790275916457176, 0.0411333367228508, 0.02673577517271042, -0.11334571242332458, -0.054883819073438644, -0.03577221557497978, 0.05086662247776985, 0.12471000105142593, 0.17608273029327393, -0.09179415553808212, 0.0066645145416259766, 0.06578664481639862, 0.033045027405023575, -0.03424539044499397, 0.03862595930695534, -0.05176762118935585, -0.011476467363536358, 0.08762115985155106, -0.2080254852771759, -0.03630837798118591, 0.041074078530073166, 0.17240683734416962, -0.18619900941848755, -0.0006853712256997824, 0.06811243295669556, 0.08058770000934601, -0.04277724400162697, 0.014427443034946918, 0.08502055704593658, -0.040280383080244064, 0.06475057452917099, 0.09685070067644119, -0.009158557280898094, -0.2204855978488922, -0.25159427523612976, -0.07426232099533081, -0.17337007820606232, 0.1116318553686142, 0.14438582956790924, 0.08115331083536148, -0.043453436344861984, 0.102730393409729, -0.04564810171723366, -0.0015352258924394846, 0.01453368365764618, -0.002487526973709464, -0.12523382902145386, -0.1292167752981186, -0.0017036102944985032, -0.000489870784804225, -0.1631985604763031, -0.0068174321204423904, 0.009509299881756306, -0.11295929551124573, -0.05215468630194664, 0.05143917351961136, -0.08097846060991287, -0.03824971988797188, -0.05973776802420616, -0.016286790370941162, 0.07686330378055573, -0.0736188143491745, -0.06213252618908882, 0.02660823054611683, -0.018237916752696037, -0.17551538348197937, 0.10870546102523804}, {-0.14901012182235718, 0.01437419094145298, -0.06193499267101288, -0.12015644460916519, 0.05341874808073044, 0.1924390196800232, -0.017375923693180084, 0.021060597151517868, 0.19495758414268494, 0.34048810601234436, -0.054417796432971954, -0.3027231991291046, -0.051914412528276443, -0.06843438744544983, 0.18905076384544373, -0.0785246267914772, -0.11598548293113708, -0.04044754430651665, -0.06840159744024277, -0.10626529902219772, 0.007793202064931393, -0.02662443369626999, -0.07974815368652344, 0.07188405096530914, -0.15440838038921356, -0.03086693584918976, -0.019616397097706795, -0.07678203284740448, 0.17081943154335022, 0.1792677938938141, 0.2525424063205719, 0.02844065986573696, 0.06950946152210236, -0.21791031956672668, 0.21152958273887634, -0.1083240807056427, -0.13746277987957, 0.02123326063156128, 0.16557706892490387, -0.02090715803205967, -0.06895606219768524, 0.041206713765859604, -0.05877039209008217, 0.11912307143211365, -0.0021211798302829266, -0.137995183467865, 0.34380024671554565, -0.08884549140930176, -0.08447900414466858, 0.053108058869838715, -0.3261456787586212, -0.03160397708415985, -0.0933673307299614, 0.1429363489151001, -0.021644888445734978, -0.004558283369988203, 0.055321551859378815, -0.18811163306236267, -0.019259057939052582, 0.10632343590259552, 0.09823158383369446, 0.24132345616817474, -0.05242222175002098, 0.3321959972381592, -0.05532756447792053, 0.08159013837575912, 0.16833865642547607, -0.14347700774669647, -0.09324591606855392, 0.1693907231092453, -0.07888820767402649, 0.055590953677892685, 0.047911934554576874, 0.1862695962190628, 0.26643186807632446, -0.06439581513404846, 0.10977169126272202, 0.4218659996986389, 0.047171857208013535, 0.008622623048722744, 0.3257916569709778, -0.06628450006246567, 0.048880595713853836, -0.07739759981632233, -0.0699230283498764, 0.11736910790205002, 0.04178975895047188, -0.17259769141674042, -0.11556282639503479, 0.06904701143503189, 0.10178547352552414, -0.018920646980404854, 0.03527845814824104, -0.285725861787796, -0.12137864530086517, -0.14632101356983185, -0.0013020309852436185, -0.15532341599464417, -0.17209194600582123, 0.05721244215965271, -0.12264017760753632, 0.1656198352575302, 0.14590264856815338, -0.07848560065031052, -0.00760128116235137, -0.03663933277130127, 0.03966393694281578, -0.055000461637973785, 0.20839554071426392, -0.05511262267827988, -0.04463764652609825, 0.034951213747262955, 0.014010393060743809, 0.11511456966400146, 0.03673413395881653, 0.0966186672449112, 0.023138055577874184, 0.028193484991788864, -0.1112339124083519, -0.0342160165309906, 0.05149060860276222, -0.11339601874351501, 0.0985054150223732, 0.09985414147377014, 0.12533120810985565, -0.16009706258773804, -0.2495187669992447, -0.1454225331544876, 0.018188098445534706, 0.11980533599853516, -0.12667858600616455, 0.08103294670581818, -0.13183727860450745, 0.22881203889846802, 0.030524473637342453, 0.2209080457687378, -0.04751862213015556, 0.30345359444618225, 0.0382518544793129, 0.15776756405830383, 0.11768149584531784, 0.07201538980007172, -0.11335328966379166, 0.10179906338453293, -0.14524836838245392, -0.1290425807237625, 0.2400418072938919, -0.06065405160188675, 0.05309310927987099, -0.2159148007631302, 0.1734747290611267, 0.0782247930765152, -0.14296714961528778, -0.06335831433534622, -0.11656174808740616, -0.05365997552871704, -0.0653998851776123, 0.11304149776697159, 0.07928866893053055, -0.03642840310931206, 0.006481691263616085, 0.011034290306270123, -0.014624283649027348, -0.14772863686084747, -0.045816998928785324, 0.3257139325141907, 0.04981221258640289, -0.137160524725914, 0.15825530886650085, -0.12250279635190964, -0.0460125170648098, 0.029693931341171265, -0.1611657589673996, -0.1384028047323227, -0.37719854712486267, 0.015738913789391518, -0.07011228054761887, -0.19505122303962708, -0.03508652001619339, -0.1429726630449295, -0.1473090648651123, -0.2622140944004059, 0.14679251611232758, -0.10159631818532944, 0.042642828077077866, -0.03732060268521309, 0.0004607026057783514, 0.12420420348644257, -0.11830763518810272, 0.2673493027687073, 0.05528344213962555, -0.04359429329633713, 0.15257865190505981, -0.003181186970323324, 0.06233997270464897, -0.24485471844673157, -0.17487968504428864, -0.06774288415908813, -0.146067813038826, 0.06475681811571121, 0.11670368909835815, 0.05305494740605354, -0.2050480842590332, -0.0002962580183520913, -0.00555522320792079, 0.21573637425899506, 0.21169477701187134, -0.07046151906251907, 0.016100991517305374, -0.004511551465839148, -0.1047312319278717, -0.16092708706855774, -0.15780670940876007, 0.01448911428451538, 0.015777043998241425, -0.1525370478630066, 0.11513388156890869, -0.10693427175283432, -0.09579270333051682, -0.18646466732025146, -0.04507942497730255, -0.07140535116195679, -0.17303486168384552, -0.11363271623849869, 0.004637293517589569, -0.03991774469614029, 0.09654191881418228, -0.04856017231941223, -0.07947463542222977, 0.10586118698120117, 0.06550100445747375, 0.2302497923374176, 0.12407655268907547, 0.004697463475167751, -0.029638411477208138, -0.2306894212961197, 0.037144873291254044, -0.20357544720172882, 0.1554695963859558, 0.03508614003658295, 0.08271182328462601, -0.018862323835492134, 0.10148059576749802, -0.0900743380188942, 0.15360906720161438, 0.18012264370918274, -0.007431133184581995, -0.028712788596749306, 0.00411811051890254, 0.15509864687919617, -0.12044534832239151, 0.025287125259637833, -0.12592174112796783, -0.0013223590794950724, -0.14273248612880707, 0.19548244774341583}, {0.08523372560739517, -0.24815529584884644, -0.12915530800819397, 0.21249014139175415, 0.2531336843967438, -0.04959428310394287, -0.036462076008319855, 0.1405220925807953, -0.14813300967216492, -0.02840546891093254, 0.024356398731470108, 0.02391055040061474, -0.05244128033518791, -0.03281848877668381, -0.13501863181591034, -0.015297573059797287, -0.12608839571475983, 0.07869184017181396, -0.27857333421707153, 0.13783609867095947, -0.1432550847530365, -0.0002585647162050009, -0.06429489701986313, -0.14061804115772247, 0.10231036692857742, -0.060524679720401764, 0.023266227915883064, -0.03065670281648636, -0.03079303354024887, -0.156354621052742, 0.09481873363256454, 0.29608336091041565, 0.003792550414800644, 0.06093364581465721, 0.025203242897987366, 0.03749849274754524, 0.20706121623516083, 0.2237364947795868, -0.08750138431787491, -0.026722807437181473, 0.043458785861730576, -0.04872254282236099, 0.08470908552408218, 0.06903167068958282, 0.2653101980686188, 0.08066463470458984, -0.06578730791807175, 0.2588207423686981, 0.134086474776268, -0.08739951252937317, 0.14138993620872498, -0.015508010983467102, 0.045458391308784485, -0.08166003227233887, -0.15184617042541504, -0.1709950715303421, 0.15402552485466003, -0.28157299757003784, -0.07652904093265533, 0.09200352430343628, 0.09613259136676788, 0.07428600639104843, -0.1264624148607254, 0.013129416853189468, 0.014773095026612282, -0.04368877038359642, -0.0006417916156351566, -0.21204115450382233, -0.22417394816875458, -0.02117055468261242, 0.10964188724756241, -0.10292017459869385, 0.037155840545892715, 0.23919571936130524, 0.04141971096396446, 0.2822924852371216, -0.0177816953510046, -0.22838228940963745, 0.03755303472280502, 0.18155398964881897, 0.0820860043168068, -0.08424602448940277, -0.05742960423231125, 0.04617432877421379, 0.05628722161054611, 0.09529072046279907, 0.024975836277008057, 0.07851722091436386, 0.012018260546028614, 0.12101617455482483, -0.047719866037368774, -0.03640461340546608, 0.13852155208587646, -0.02267969772219658, -0.023224802687764168, -0.007756448816508055, -0.07708939164876938, -0.015536774881184101, -0.26895928382873535, -0.051759250462055206, -0.05595846101641655, -0.045053768903017044, -0.06961426138877869, 0.0142296077683568, -0.14636839926242828, 0.08699239790439606, -0.10566435009241104, -0.20306187868118286, -0.016768205910921097, 0.1052757129073143, 0.05796058848500252, -0.038948629051446915, -0.00783542264252901, 0.008248396217823029, -0.013990710489451885, 0.1305391490459442, 0.20583680272102356, 0.06355637311935425, 0.013771124184131622, 0.3107690215110779, -0.01981593295931816, -0.18477900326251984, -0.05927994102239609, 0.011404497548937798, 0.1386808604001999, -0.14654144644737244, 0.13970370590686798, 0.008070023730397224, -0.17190539836883545, 0.040916141122579575, 0.20711247622966766, 0.12983132898807526, 0.03511679545044899, -0.16220802068710327, -0.18709446489810944, -0.02360190823674202, -0.18831978738307953, 0.12234769016504288, 0.02025407925248146, -0.1535831093788147, 0.2608504593372345, 0.24860620498657227, 0.013584673404693604, 0.07178515195846558, -0.16141662001609802, 0.3315119743347168, 0.044793665409088135, 0.0464898943901062, 0.010186263360083103, -0.040386173874139786, -0.43534377217292786, -0.0018203839426860213, 0.004731158260256052, -0.25116702914237976, -0.15525022149085999, -0.0019710937049239874, -0.011916723102331161, -0.21006464958190918, -0.09749846160411835, 0.04625633358955383, 0.0013518144842237234, -0.07072500139474869, -0.11830676347017288, -0.10279980301856995, -0.13210482895374298, 0.08473830670118332, -0.1613820195198059, 0.09908714890480042, 0.020040061324834824, 0.025537023320794106, -0.06896204501390457, 0.18654775619506836, 0.02395685948431492, 0.10170908272266388, -0.12195847928524017, -0.032780881971120834, -0.08741415292024612, -0.08666406571865082, 0.11225319653749466, 0.04202936962246895, -0.054896458983421326, -0.10397186875343323, 0.10590515285730362, -0.1471063196659088, -0.005570428911596537, -0.20664726197719574, 0.3119504749774933, -0.04247726500034332, 0.07112766802310944, -0.6611495614051819, 0.07680622488260269, -0.1324901133775711, 0.15301251411437988, 0.06779444217681885, -0.10161905735731125, -0.000550358381588012, 0.07315041869878769, -0.07769409567117691, -0.025803349912166595, -0.20447003841400146, -0.10258239507675171, -0.0006528512458316982, 0.19423843920230865, -0.11108992993831635, -0.018924664705991745, 0.126039057970047, -0.037156831473112106, 0.210368350148201, 0.03360600024461746, -0.16754020750522614, -0.06091490760445595, 0.16456733644008636, -0.2133687138557434, -0.06769081205129623, 0.16563087701797485, 0.03686068207025528, 0.05455375090241432, 0.1714332401752472, -0.021417880430817604, 0.22517691552639008, 0.011875838041305542, -0.19909003376960754, -0.07126306742429733, 0.2224932461977005, 0.19781911373138428, 0.153406023979187, -0.15771159529685974, 0.05857414752244949, 0.03176792711019516, 0.2115870863199234, 0.07897229492664337, -0.1627400815486908, -0.005073316395282745, 0.1238451898097992, 0.02326079271733761, -0.06916467845439911, 0.19321787357330322, 0.31876081228256226, 0.0743793249130249, -0.04073460027575493, -0.09483224898576736, 0.08393865823745728, 0.11502695083618164, -0.20383964478969574, 0.014012821018695831, -0.10768884420394897, 0.07791541516780853, -0.08180737495422363, -0.16068458557128906, -0.012622436508536339, -0.023652922362089157, -0.20469295978546143, -0.2800617814064026, -0.0062048956751823425, -0.07911640405654907, -0.002817165572196245}, {-0.005138028413057327, 0.19676151871681213, 0.1867075115442276, 0.06823931634426117, 0.11323489248752594, 0.010298113338649273, -0.1154990866780281, -0.059988535940647125, -0.06511363387107849, -0.10067295283079147, -0.061193618923425674, 0.24369704723358154, -0.10443726181983948, 0.06539137661457062, -0.12905409932136536, 0.057588472962379456, 0.3827575743198395, 0.05322680622339249, -0.1219736710190773, -0.021802589297294617, 0.012377265840768814, 0.19572626054286957, -0.19550052285194397, 0.21460597217082977, 0.026053622364997864, 0.10781815648078918, 0.269233375787735, 0.03976741433143616, -0.0372496172785759, 0.10721330344676971, -0.007896187715232372, -0.1493736356496811, -0.048805367201566696, 0.09616290032863617, -0.04230755195021629, 0.25784894824028015, -0.09178342670202255, -0.1646270453929901, 0.10172868520021439, 0.10601085424423218, 0.13198678195476532, 0.012769917026162148, -0.017277073115110397, 0.008534184657037258, 0.03693247213959694, -0.11231997609138489, 0.052579011768102646, -0.08292225748300552, -0.10892485827207565, 0.2159319370985031, 0.11256083101034164, 0.20668728649616241, 0.21575477719306946, 0.15463389456272125, 0.16224448382854462, 0.042022500187158585, -0.10538417845964432, 0.23199358582496643, -0.15494748950004578, -0.1756904125213623, -0.11457373946905136, 0.05513721704483032, -0.10293000191450119, 0.0454784519970417, -0.036914389580488205, 0.06222343444824219, -0.05733581632375717, 0.10568007826805115, 0.2884804308414459, -0.012576567940413952, -0.08645620942115784, 0.03346314653754234, -0.26313483715057373, -0.09860463440418243, 0.06553050875663757, 0.1204291507601738, 0.1038864254951477, 0.031688012182712555, -0.08901064097881317, -0.15668396651744843, 0.12003473937511444, 0.021926263347268105, -0.02444111555814743, 0.17014451324939728, 0.09726008772850037, 0.07743999361991882, 0.19862966239452362, 0.06393348425626755, -0.011109122075140476, 0.0438009575009346, -0.048799753189086914, 0.14113225042819977, -0.030747337266802788, -0.02515564300119877, -0.12742845714092255, 0.20084653794765472, 0.05583861097693443, -0.07435467839241028, 0.025725698098540306, 0.18374113738536835, 0.16961880028247833, 0.025119680911302567, -0.043139249086380005, 0.06924692541360855, 0.16356146335601807, 0.30762210488319397, 0.08614622056484222, 0.14500871300697327, 0.09695009142160416, -0.17334532737731934, -0.14577119052410126, 0.005691874772310257, -0.38761407136917114, 0.17889125645160675, 0.24910762906074524, 0.10451177507638931, 0.03585086762905121, 0.041188932955265045, -0.04990357533097267, -0.23483902215957642, -0.09498204290866852, 0.19012165069580078, 0.4496416747570038, 0.1435142308473587, -0.18022610247135162, 0.06634358316659927, 0.01049835979938507, 0.12355184555053711, -0.19728519022464752, 0.26857951283454895, 0.027820918709039688, -0.5474309325218201, 0.08985904604196548, 0.00041333664557896554, 0.043379634618759155, 0.21487653255462646, -0.1176786944270134, -0.04490620270371437, 0.0036591829266399145, 0.11630122363567352, -0.11075950413942337, 0.09712949395179749, 0.2426096796989441, -0.34641870856285095, 0.3043472468852997, 0.045856572687625885, -0.2845006585121155, 0.033153437077999115, -0.13011972606182098, -0.08037244528532028, -0.08630840480327606, -0.10200442373752594, 0.006747574079781771, 0.20793431997299194, -0.09534004330635071, -0.11388345807790756, 0.05036516860127449, 0.04473388195037842, 0.10667332261800766, -0.005335661116987467, 0.05250488221645355, -0.2409406453371048, -0.1878521591424942, 0.08137556165456772, 0.004308012779802084, -0.08533424139022827, -0.05507301911711693, 0.18087877333164215, 0.03478289395570755, 0.16518257558345795, -0.089249387383461, 0.08874398469924927, 0.1733560711145401, 0.19070804119110107, 0.004313770215958357, 0.062332116067409515, 0.020909160375595093, -0.023570485413074493, 0.09890066087245941, 0.04407421499490738, 0.18929636478424072, 0.25933730602264404, -0.05998266115784645, -0.1702941507101059, -0.08854908496141434, 0.05317867174744606, 0.04925594478845596, 0.12995564937591553, -0.3920961618423462, -0.0017988440813496709, 0.14088059961795807, 0.05273808538913727, 0.17947612702846527, 0.047564905136823654, 0.366159588098526, 0.3261158764362335, -0.06123688444495201, -0.010780161246657372, -0.014200309291481972, 0.10248862951993942, -0.12465352565050125, -0.2022058367729187, 0.1284739226102829, -0.16272716224193573, -0.01660594902932644, -0.09300549328327179, -0.039844807237386703, -0.23491668701171875, 0.07560532540082932, -0.035508111119270325, -0.10475727170705795, 0.009372565895318985, -0.04376435652375221, 0.13644248247146606, 0.2265736162662506, -0.26127195358276367, 0.0767194852232933, -0.14720043540000916, 0.3230554759502411, -0.021930962800979614, 0.02645600214600563, 0.3804551661014557, 0.07612446695566177, 0.09579427540302277, 0.041396837681531906, 0.13870158791542053, -0.2401151955127716, -0.30199217796325684, 0.004913318436592817, 0.2827264368534088, 0.029541632160544395, -0.11472781747579575, 0.06078304350376129, 0.026732884347438812, -0.04077140614390373, 0.2981370687484741, 0.30998408794403076, -0.08481230586767197, 0.027098888531327248, 0.5571251511573792, -0.15577489137649536, -0.11866338551044464, -0.026590455323457718, 0.27532958984375, 0.021058309823274612, -0.1628039926290512, -0.004145802929997444, 0.07667021453380585, 0.06526865065097809, -0.33679598569869995, -0.10976672917604446, 0.04915168508887291, -0.30092504620552063, -0.3461235463619232, 0.09257696568965912, 0.05744297802448273}, {-0.1030503511428833, 0.27654027938842773, 0.1664527952671051, 0.2745991051197052, 0.08120246231555939, -0.0073622651398181915, -0.2786634564399719, 0.11355303227901459, 0.08229651302099228, -0.4318351745605469, 0.15988366305828094, 0.32375818490982056, -0.04649268835783005, -0.014435996301472187, -0.011779243126511574, -0.026868438348174095, 0.03223162144422531, -0.0011780584463849664, -0.00808995496481657, -0.049941182136535645, 0.0032108572777360678, -0.06416920572519302, -0.012160798534750938, 0.18510285019874573, -0.03494642302393913, 0.04729273170232773, -0.03503108397126198, -0.10306432843208313, -0.27764737606048584, 0.0035723771434277296, 0.29053163528442383, 0.17021596431732178, 0.0798179879784584, 0.17250275611877441, -0.0028795322868973017, -0.033698149025440216, 0.15980419516563416, 0.07597768306732178, 0.1480545997619629, -0.001533225062303245, -0.009482263587415218, -0.09008852392435074, 0.024138174951076508, -0.01196363102644682, 0.13418911397457123, 0.06595262885093689, -0.048829905688762665, -0.12214597314596176, -0.1084471195936203, -0.042943909764289856, 0.2749076783657074, 0.009471978060901165, -0.0038568663876503706, -0.19919437170028687, -0.06726181507110596, 0.02635936066508293, -0.10541138797998428, 0.16692744195461273, -0.031167076900601387, -0.17784401774406433, 0.21527403593063354, 0.09562694281339645, 0.18443477153778076, 0.355715811252594, -0.14222261309623718, 0.11518678814172745, 0.16809406876564026, 0.11701489984989166, 0.10693909227848053, -0.16187545657157898, -0.17065487802028656, 0.03955269232392311, 0.14484211802482605, 0.3148207366466522, 0.11866358667612076, 0.0198250450193882, -0.07172095775604248, -0.11064620316028595, 0.12945996224880219, 0.1014692559838295, -0.1287597417831421, -0.12450229376554489, 0.06274625658988953, -0.12650875747203827, 0.027112402021884918, 0.24558046460151672, -0.2284562885761261, 0.2639746367931366, -0.09248407930135727, -0.07104585319757462, 0.07672470808029175, 0.11029540002346039, 0.3130394518375397, -0.1272118240594864, 0.18945738673210144, 0.03364553675055504, -0.15883897244930267, 0.05420868098735809, 0.011198543943464756, -0.2165241837501526, -0.03249045088887215, 0.11561839282512665, -0.07580401003360748, 0.06180456653237343, 0.3441978693008423, 0.3622788190841675, 0.044663723558187485, 0.011565526016056538, -0.17784133553504944, 0.0998915433883667, -0.07143857330083847, 0.1949034184217453, -0.20712293684482574, 0.11705894023180008, -0.07137219607830048, 0.10252130776643753, 0.046415504068136215, -0.21903912723064423, -0.08284355700016022, -0.03813198581337929, 0.16159865260124207, 0.03050229512155056, 0.17334739863872528, 0.41973942518234253, -0.14867514371871948, -0.17676952481269836, 0.28461048007011414, -0.1786089390516281, -0.19783355295658112, 0.04241609945893288, 0.14348022639751434, 0.20180922746658325, -0.13963626325130463, -0.12596146762371063, -0.10715028643608093, 0.0786585658788681, 0.22198271751403809, -0.07666490972042084, -0.22741876542568207, 0.2167876958847046, 0.07801470905542374, -0.13974657654762268, 0.12963545322418213, -0.2687626779079437, 0.10012262314558029, 0.11724314093589783, -0.045717962086200714, 0.03693597763776779, 0.03595055639743805, 0.1272267997264862, 0.00730300135910511, 0.041237760335206985, -0.08378224074840546, 0.12319231778383255, 0.10691934078931808, 0.1294754296541214, -0.09259519726037979, -0.10561292618513107, -0.006091240793466568, -0.3982958495616913, 0.10115606337785721, -0.16252219676971436, 0.05427174270153046, 0.02412843331694603, 0.09295632690191269, 0.051154542714357376, -0.37769845128059387, 0.020031560212373734, 0.02004660665988922, -0.017577555030584335, -0.08476222306489944, -0.026445580646395683, 0.05368603765964508, -0.08774615824222565, 0.06584639102220535, -0.026742160320281982, 0.1490355134010315, -0.2787030041217804, -0.0011702415067702532, -0.13393881916999817, 0.15419673919677734, 0.31033724546432495, 0.007067566271871328, -0.09329260140657425, -0.037125688046216965, -0.10662151128053665, 0.13584032654762268, -0.001341884839348495, -0.08308132737874985, -0.265878826379776, -0.11878858506679535, -0.1488230675458908, -0.13048748672008514, -0.12350888550281525, -0.03729219734668732, -0.08768080919981003, -0.0633825808763504, 0.2045481950044632, 0.19769103825092316, 0.19502629339694977, -0.22174273431301117, 0.12562903761863708, 0.27539390325546265, -0.17277657985687256, 0.06168689578771591, 0.10380679368972778, 0.37291544675827026, 0.1817484050989151, 0.19073309004306793, -0.04419848322868347, -0.03169877827167511, 0.10905823111534119, -0.09955823421478271, 0.18967190384864807, 0.0013978771166875958, 0.2838563323020935, 0.18994814157485962, -0.09776072949171066, 0.10032148659229279, -0.02141672931611538, 0.12363827228546143, -0.024561800062656403, -0.033549487590789795, 0.19521093368530273, -0.15598031878471375, 0.24223583936691284, -0.07840358465909958, -0.06809426844120026, 0.3100634515285492, -0.02432272583246231, 0.049221549183130264, 0.10988757759332657, -0.07051154226064682, -0.042144276201725006, -0.17906975746154785, -0.25042298436164856, 0.05840316787362099, -0.00782474223524332, -0.10253157466650009, 0.0566260851919651, 0.14709122478961945, 0.06015661731362343, 0.1838376224040985, -0.14707675576210022, -0.031203975901007652, 0.218950092792511, -0.07136452198028564, -0.2577956020832062, 0.007396896835416555, -0.12860117852687836, -0.017822179943323135, 0.171019047498703, 0.10960830748081207, 0.03813188895583153, -0.03298402950167656, -0.15709595382213593}, {-0.0768546462059021, 0.1123274490237236, 0.14934328198432922, -0.14505010843276978, -0.26007795333862305, 0.023366130888462067, -0.10500961542129517, -0.1574994921684265, -0.0001640602422412485, -0.1205090582370758, 0.15982387959957123, 0.05836082994937897, 0.07471320033073425, 0.09967683255672455, 0.09155194461345673, -0.09778295457363129, -0.20160947740077972, 0.09372551739215851, -0.22238296270370483, -0.006703071296215057, -0.0796821340918541, -0.2577831745147705, -0.01537083275616169, 0.041872788220644, 0.10154690593481064, -0.16042117774486542, 0.16559764742851257, 0.14576667547225952, -0.12312308698892593, 0.03104735165834427, -0.14819636940956116, -0.1692153364419937, -0.002301071770489216, 0.06422312557697296, -0.11799754202365875, 0.03137875720858574, -0.07917840778827667, -0.037017613649368286, 0.13476397097110748, -0.15209217369556427, 0.13056355714797974, 0.06501412391662598, 0.0727759301662445, -0.10334930568933487, -0.09588165581226349, 0.08736436069011688, 0.2713644504547119, -0.15723995864391327, 0.04422342777252197, -0.28368374705314636, -0.020885763689875603, -0.00359852472320199, -0.09589216113090515, 0.23702596127986908, 0.002231697551906109, -0.08466754853725433, -0.22156290709972382, 0.09809494018554688, 0.046891164034605026, -0.05871566757559776, -0.23862247169017792, -0.18887518346309662, -0.11905686557292938, 0.2079218327999115, 0.011274592950940132, -0.12546907365322113, 0.19985051453113556, 0.08714118599891663, 0.06504464149475098, 0.012964818626642227, -0.09239797294139862, 0.035356733947992325, -0.053282249718904495, 0.06693219393491745, -0.2555288076400757, -0.08234640210866928, 0.18706299364566803, 0.11764084547758102, -0.07128327339887619, 0.07737157493829727, 0.014800661243498325, -0.15698368847370148, -0.027007145807147026, -0.03511522337794304, -0.07741102576255798, 0.014429142698645592, 0.07692891359329224, -0.02984992042183876, 0.059437669813632965, 0.0712956041097641, 0.041490338742733, -0.20643268525600433, -0.13678985834121704, -0.016975251957774162, 0.025143343955278397, -0.09241293370723724, -0.11644855886697769, -0.13978992402553558, 0.05341041460633278, 0.11767127364873886, -0.1435825228691101, -0.06830412149429321, 0.13209718465805054, -0.1279185265302658, 0.03673168271780014, -0.5702887773513794, 0.10677958279848099, 0.03716195374727249, 0.15170538425445557, -0.04854009672999382, 0.12882837653160095, 0.10619454830884933, -0.1445946842432022, -0.05405428260564804, 0.057208385318517685, -0.1633179783821106, -0.25946927070617676, 0.21029528975486755, -0.06361206620931625, -0.11165501177310944, -0.07127609848976135, 0.21690250933170319, 0.12942685186862946, 0.07496911287307739, 0.04507803916931152, -0.16918253898620605, 0.04005405306816101, 0.011142309755086899, 0.11619141697883606, 0.01743234694004059, 0.08747539669275284, 0.009868152439594269, 0.11974302679300308, -0.19386041164398193, 0.07121340185403824, 0.3137182891368866, -0.11070100218057632, 0.07669702172279358, -0.04959156736731529, -0.18862299621105194, 0.05949011072516441, 0.033871714025735855, 0.13452555239200592, -0.06752094626426697, 0.12572748959064484, 0.06672869622707367, -0.17474177479743958, -0.21383239328861237, -0.024453792721033096, 0.10774979740381241, -0.08957894891500473, 0.07029881328344345, -0.0035703429020941257, 0.08646510541439056, 0.02648218721151352, -0.10715696960687637, 0.031056895852088928, 0.4301430881023407, -0.13507287204265594, -0.014279341325163841, -0.021247422322630882, -0.17039547860622406, -0.1682540327310562, 0.0758189707994461, 0.07658138871192932, 0.00029836990870535374, -0.059386905282735825, -0.18764086067676544, -0.18456754088401794, 0.03298241272568703, -0.06031589210033417, -0.11165225505828857, 0.0021823940332978964, -0.044478002935647964, -0.13307344913482666, 0.07376782596111298, 0.14118559658527374, -0.028944704681634903, -0.06806990504264832, -0.0623011514544487, 0.1607091724872589, -0.03125167265534401, 0.16361276805400848, 0.22531919181346893, -0.08444762974977493, -0.3689047396183014, -0.2890626788139343, 0.048321474343538284, 0.0209015142172575, 0.2234703153371811, -0.0023379141930490732, 0.2086568921804428, 0.2239796370267868, -0.005989163648337126, -0.014525026082992554, -0.20169509947299957, 0.17176367342472076, -0.1884271651506424, 0.046401236206293106, 0.11353648453950882, 0.09535684436559677, 0.055045295506715775, -0.10808786004781723, 0.053186025470495224, -0.05741668492555618, -0.2914982736110687, -0.03032686561346054, 0.0705934464931488, -0.08250725269317627, 0.04163704439997673, 0.05108243227005005, -0.03952770680189133, -0.07347765564918518, -0.04148771986365318, -0.1504923403263092, 0.008367713540792465, -0.015165219083428383, -0.05063973739743233, -0.050513509660959244, -0.10428065806627274, -0.14869634807109833, 0.12854726612567902, -0.05580994486808777, -0.12174725532531738, -0.0025903733912855387, -0.13563373684883118, 0.13978607952594757, -0.22357064485549927, -0.19527919590473175, 0.06548542529344559, -0.08265455812215805, -0.12507864832878113, -0.06545604765415192, -0.0544201023876667, 0.11954505741596222, -0.06702365726232529, 0.08649472892284393, -0.061090171337127686, -0.0023884966503828764, -0.1199575737118721, -0.001413258840329945, 0.08101589977741241, -0.03585275635123253, 0.019440628588199615, -0.062195949256420135, 0.04728350415825844, -0.06623978912830353, 0.3584323823451996, 0.05509655550122261, 0.027345044538378716, -0.008003565482795238, 0.10731783509254456, -0.20575657486915588, 0.1052948608994484, -0.057420313358306885, 0.11994960904121399}, {0.06361234933137894, 0.15457892417907715, -0.021008972078561783, -0.0037826995830982924, -0.1293633133172989, -0.025442473590373993, -0.04313603788614273, 0.22551384568214417, -0.13841669261455536, -0.05288451910018921, 0.010516027919948101, 0.0871388241648674, -0.006606082431972027, -0.11362877488136292, 0.010176797397434711, 0.2238023728132248, 0.08463706076145172, 0.26719796657562256, -0.10004356503486633, -0.16619452834129333, 0.07769624143838882, 0.19340728223323822, 0.20161855220794678, 0.1137523353099823, -0.03179537504911423, -0.1114298552274704, -0.05480378866195679, -0.0513770692050457, -0.05032752454280853, -0.13154777884483337, -0.07561271637678146, -0.10682912170886993, -0.03559717908501625, -0.05791135132312775, 0.06357349455356598, 0.21137212216854095, 0.31199783086776733, 0.09113727509975433, 0.021092498674988747, 0.01778811402618885, 0.08745599538087845, -0.10440368950366974, -0.01642206311225891, 0.056797999888658524, 0.045920487493276596, 0.07639529556035995, -0.12333617359399796, 0.03581294044852257, 0.041721343994140625, 0.05142318829894066, 0.1912754625082016, 0.026674477383494377, 0.09949174523353577, -0.0957595556974411, 0.13954389095306396, 0.019723769277334213, 0.012571749277412891, 0.10543761402368546, 0.041072577238082886, -0.12225162237882614, 0.14541926980018616, -0.17736129462718964, 0.02385721541941166, -0.055114440619945526, -0.008127376437187195, -0.07498693466186523, -0.09061912447214127, 0.03638875484466553, 0.21186384558677673, -0.02690843492746353, -0.08250913769006729, 0.005263442173600197, 0.3081117570400238, 0.027359576895833015, 0.14878897368907928, -0.08189426362514496, -0.060095466673374176, 0.0051836599595844746, -0.20256288349628448, 0.07741887867450714, 0.06255092471837997, -0.10497702658176422, 0.15099407732486725, -0.07353587448596954, 0.03789965435862541, 0.04957107454538345, -0.04933803528547287, 0.15090173482894897, 0.11834671348333359, -0.041742321103811264, 0.041070036590099335, 0.25108492374420166, 0.04596523195505142, 0.03698721528053284, -0.015742000192403793, -0.010654883459210396, -0.09567737579345703, -0.11427102982997894, 0.33707818388938904, -0.05900759622454643, 0.06485674530267715, -0.03184167668223381, -0.11095590144395828, 0.1217578873038292, -0.0954137071967125, 0.1899448037147522, -0.12455319613218307, -0.1619502454996109, -0.10244888812303543, 0.009440410882234573, -0.1124715656042099, -0.10092641413211823, 0.051619499921798706, -0.0770648941397667, 0.1719089299440384, 0.2709202468395233, -0.12469326704740524, -0.09283379465341568, -0.0027062518056482077, 0.13371050357818604, -0.08610320836305618, 0.05788788944482803, -0.07352925837039948, 0.21804064512252808, -0.01019100472331047, 0.03622620552778244, 0.012090645730495453, -0.002042661188170314, -0.047939714044332504, 0.015580167062580585, -0.09529616683721542, -0.08317448943853378, -0.23281069099903107, -0.06892706453800201, 0.22978456318378448, 0.01586896926164627, -0.12738358974456787, -0.049311619251966476, -0.08223632723093033, 0.10555792599916458, 0.06749077141284943, -0.02497069351375103, 0.047585103660821915, -0.04006476700305939, 0.1439080685377121, 0.06735411286354065, 0.01931692659854889, -0.06552179902791977, 0.058802444487810135, 0.0931849554181099, 0.022819286212325096, -0.06348881125450134, -0.053419243544340134, -0.18839897215366364, -0.030492590740323067, -0.0652947723865509, -0.26617997884750366, 0.08490048348903656, 0.09047511219978333, -0.14302347600460052, -0.029974471777677536, -0.02734975703060627, 0.11035504937171936, 0.048900019377470016, -0.05184277892112732, -0.21994738280773163, -0.15867532789707184, -0.10637711733579636, 0.12614166736602783, -0.016013868153095245, -0.21000152826309204, -0.01914125308394432, 0.1363726407289505, 0.2106509804725647, 0.10030952841043472, -0.11465296149253845, -0.08621631562709808, -0.029126035049557686, 0.07539968937635422, 0.09879559278488159, 0.07894004136323929, 0.023005634546279907, 0.10761090368032455, 0.11507641524076462, 0.1005866676568985, 0.18927203118801117, -0.08299840241670609, -0.06143391504883766, 0.11808978021144867, 0.04754069447517395, 0.19252674281597137, -0.08493293821811676, 0.14253468811511993, 0.21521708369255066, -0.1010446846485138, 0.0946885496377945, -0.19335557520389557, -0.006903169676661491, -0.08217890560626984, -0.007658965885639191, -0.10791424661874771, -0.03531383350491524, 0.03445909917354584, -0.10262687504291534, -0.14968198537826538, -0.15053431689739227, 0.2203475534915924, -0.09821368008852005, -0.05389450117945671, 0.16223417222499847, 0.1442718207836151, -0.07766672223806381, 0.0071260882541537285, -0.11320377886295319, 0.09275919198989868, 0.017319757491350174, 0.33318620920181274, -0.01819419302046299, 0.05327288806438446, 0.1728205382823944, 0.07730092853307724, 0.12927518784999847, 0.025736728683114052, 0.026788201183080673, 0.09568187594413757, 0.004579185042530298, -0.07683480530977249, -0.18301846086978912, 0.1126449927687645, -0.062243927270174026, -0.10120955854654312, -0.03657493367791176, 0.16934256255626678, 0.058924999088048935, -0.11691679060459137, 0.2514828145503998, 0.16695095598697662, 0.2186894714832306, -0.3238726854324341, 0.025714633986353874, -0.0808335617184639, 0.09965812414884567, 0.008717597462236881, -0.24654336273670197, -0.05681104585528374, -0.047273606061935425, 0.015824133530259132, -0.023776205256581306, -0.15117216110229492, -0.22470587491989136, -0.016020098701119423, 0.10798979550600052, -0.10372721403837204, -0.0414845235645771, 0.01626136153936386, -0.009246798232197762}, {0.017209971323609352, -0.03220340609550476, -0.0965786725282669, 0.08778481185436249, 0.2570662200450897, 0.08672504872083664, 0.02631247602403164, -0.18282565474510193, -0.13439325988292694, -0.06066179648041725, -0.1992681473493576, 0.09492886066436768, -0.04771127179265022, 0.027116499841213226, -0.03650275245308876, 0.050989795476198196, 0.15605425834655762, -0.027005886659026146, -0.06332340836524963, 0.03540726751089096, 0.3860021233558655, 0.04548826813697815, -0.011844726279377937, -0.10299805551767349, 0.026441358029842377, 0.18063703179359436, -0.14397448301315308, -0.16501231491565704, 0.097325399518013, -0.2101423442363739, -0.008466023951768875, -0.03998812288045883, 0.026631804183125496, 0.05623755231499672, -0.18711473047733307, 0.12043101340532303, 0.042200684547424316, 0.018140718340873718, -0.15713953971862793, 0.18493308126926422, 0.024669144302606583, 0.06111440062522888, -0.06216826289892197, 0.06775838881731033, -0.26872554421424866, 0.03620680049061775, -0.006998795084655285, -0.04253774136304855, 0.044600389897823334, -0.03947913646697998, -0.05227644369006157, 0.183970108628273, -0.023302512243390083, 0.10461147129535675, -0.13843707740306854, -0.11029984056949615, 0.28746187686920166, 0.0729663223028183, -0.03938138857483864, -0.00353804393671453, -0.03948920592665672, -0.044835563749074936, 0.053030095994472504, -0.1305793672800064, -0.13693130016326904, -0.04174396023154259, 0.1996980756521225, 0.09496709704399109, 0.06386395543813705, -0.017011351883411407, 0.06867977231740952, -0.04001932218670845, 0.06372354924678802, 0.17612938582897186, -0.0038347113877534866, 0.13889971375465393, -0.31884515285491943, -0.17913368344306946, 0.2550872266292572, -0.02147633023560047, -0.08158877491950989, 0.04290411248803139, 0.12493561953306198, -0.11275641620159149, 0.021013397723436356, -0.19976702332496643, 0.10458310693502426, 0.17113745212554932, 0.07370956242084503, -0.08878067135810852, -0.026841605082154274, 0.08691761642694473, -0.004925748333334923, 0.2028181254863739, 0.1457633376121521, -0.041696615517139435, -0.0607384592294693, -0.153338223695755, 0.08388489484786987, -0.18894381821155548, 0.224355086684227, 0.07147994637489319, 0.013776366598904133, -0.19273202121257782, -0.13779574632644653, -0.11285079270601273, 0.006106454413384199, -0.058011364191770554, -0.25654810667037964, 0.1334402859210968, 0.16937194764614105, 0.008894901722669601, -0.005884083919227123, -0.20658338069915771, -0.07181210815906525, -0.12795114517211914, 0.34071025252342224, -0.023041091859340668, -0.035171180963516235, -0.007174745202064514, 0.06059185788035393, 0.12043410539627075, -0.07158263027667999, 0.17834711074829102, 0.00875494908541441, 0.12879815697669983, 0.1622946858406067, -0.19398044049739838, -0.036755986511707306, 0.15616518259048462, 0.27744072675704956, -0.05176137015223503, 0.03529500216245651, -0.009437494911253452, -0.014601331204175949, -0.151018887758255, 0.2018251121044159, 0.1770922988653183, 0.006364199798554182, -0.08402939140796661, -0.18701298534870148, 0.10109344124794006, 0.1573193222284317, 0.07088340818881989, 0.13748325407505035, -0.0917852446436882, 0.024854371324181557, 0.26278528571128845, -0.0626043826341629, -0.05044452100992203, -0.3768443763256073, -0.08366385847330093, -0.13438257575035095, 0.1214524507522583, 0.005681390408426523, 0.36193034052848816, -0.20739009976387024, -0.167371928691864, 0.12546880543231964, -0.1441558450460434, 0.09164309501647949, 0.21606799960136414, 0.06108871474862099, -0.05702830106019974, 0.025977401062846184, 0.246461421251297, -0.011128539219498634, 0.15706710517406464, -0.1324378252029419, 0.10648882389068604, 0.013490498997271061, -0.2429511696100235, -0.09911838173866272, 0.100929856300354, -0.22055326402187347, 0.09125347435474396, -0.053202953189611435, -0.0400654561817646, 0.17157457768917084, 0.049256421625614166, 0.005440992768853903, -0.05326467007398605, 0.18163955211639404, 0.03307117894291878, -0.007335873786360025, -0.010950785130262375, 0.17255499958992004, -0.16212235391139984, -0.228409543633461, 0.08820348232984543, -0.0808909609913826, -0.26157763600349426, 0.14734619855880737, 0.03444887325167656, -0.030092403292655945, -0.02611703984439373, -0.025009367614984512, 0.08935503661632538, 0.15125709772109985, 0.013935091905295849, 0.04533437266945839, -0.028645608574151993, 0.005609131418168545, -0.2567615807056427, -0.1340588927268982, -0.08954458683729172, 0.0742545872926712, -0.021530013531446457, 0.11408405750989914, 0.06717617809772491, 0.029275430366396904, 0.14392241835594177, 0.025204194709658623, 0.04430046305060387, -0.16821080446243286, -0.055607736110687256, 0.1393454521894455, 0.039385873824357986, 0.162479430437088, 0.13900364935398102, -0.16883620619773865, -0.0916990339756012, -0.14353546500205994, 0.24932077527046204, -0.2988910973072052, 0.10576409101486206, -0.10117002576589584, 0.018027858808636665, 0.03488478437066078, 0.012422308325767517, 0.10898187756538391, 0.018802838400006294, 0.018907291814684868, -0.14013256132602692, 0.21566200256347656, 0.02738308534026146, 0.0052299196831882, 0.030542124062776566, 0.06904216855764389, -0.17697365581989288, -0.145809143781662, 0.08005039393901825, 0.12207099050283432, 0.20071575045585632, -0.048804473131895065, -0.05942687764763832, 0.006495548412203789, -0.11841321736574173, -0.017286600545048714, -0.0894918441772461, 0.019854627549648285, -0.08114204555749893, -0.08844510465860367, -0.018443910405039787, 0.04942914843559265, -0.370859831571579}, {-0.16614766418933868, -0.08956088870763779, 0.014504469931125641, 0.011513978242874146, -0.0980762168765068, -0.1382087618112564, 0.25789782404899597, -0.14435048401355743, 0.041179634630680084, 0.04983920231461525, 0.04714846983551979, 0.07608767598867416, 0.007706583943217993, 0.0450408048927784, -0.04972466826438904, -0.03585583716630936, 0.17968499660491943, -0.16564643383026123, 0.09688268601894379, 0.01484756637364626, -0.02385188266634941, -0.07612475752830505, -0.16082851588726044, 0.05134767293930054, 0.02922838367521763, -0.16573573648929596, -0.05047355219721794, 0.15394636988639832, -0.0609029158949852, -0.04800867289304733, -0.19846272468566895, -0.051397040486335754, 0.1389438956975937, 0.08842360228300095, -0.42142796516418457, -0.04667602851986885, -0.08406661450862885, -0.01559444796293974, 0.008281287737190723, -0.0359683521091938, -0.029792290180921555, -0.01460712868720293, -0.12298567593097687, -0.14507904648780823, -0.13639667630195618, -0.31350579857826233, -0.15469017624855042, 0.27504462003707886, -0.06073768436908722, 0.040280189365148544, 0.012694604694843292, 0.008769713342189789, 0.045151032507419586, 0.07208795100450516, -0.009816925041377544, -0.14033345878124237, -0.2507278323173523, 0.0692470371723175, -0.08329129219055176, -0.020672719925642014, -0.023053772747516632, -0.20572678744792938, -0.14376075565814972, -0.15206852555274963, 0.051086727529764175, 0.023545384407043457, 0.11036994308233261, -0.01293625682592392, 0.12385142594575882, 0.02953919768333435, 0.06776727735996246, -0.05551527068018913, 0.06739098578691483, 0.04296533763408661, 0.06018974632024765, 0.0734025314450264, -0.06594020873308182, 0.044856175780296326, -0.13357052206993103, 0.20047377049922943, -0.032589126378297806, -0.058279454708099365, 0.09298017621040344, 0.012798396870493889, -0.0697188526391983, -0.21723853051662445, 0.09349719434976578, 0.007289206143468618, 0.09115073829889297, -0.15096308290958405, -0.00215901923365891, 0.020659521222114563, -0.11231628805398941, -0.03024842031300068, 0.1392238289117813, -0.0047593084163963795, 0.13197435438632965, -0.03263365849852562, -0.054092030972242355, 0.07672744244337082, 0.00965195707976818, 0.019189096987247467, 0.01527285948395729, 0.13877734541893005, 0.07296440750360489, -0.42735835909843445, 0.01249817293137312, -0.20595623552799225, -0.023157956078648567, 0.0536518432199955, -0.03970673680305481, -0.22179467976093292, -0.10221988707780838, 0.06453081965446472, 0.09108012914657593, 0.2156781256198883, -0.051262225955724716, 0.012400077655911446, 0.08054304867982864, 0.042381271719932556, -0.08452319353818893, 0.0674542710185051, -0.05555262044072151, 0.14966589212417603, 0.12254271656274796, 0.15906573832035065, -0.03149072825908661, 0.07686658948659897, 0.07946156710386276, -0.09103155881166458, 0.0447300560772419, -0.010269606485962868, -0.05475219339132309, 0.07953988015651703, -0.11438363790512085, -0.1677141934633255, 0.021551407873630524, -0.031336460262537, -0.05759483203291893, 0.24460886418819427, -0.10667707771062851, 0.22700877487659454, -0.08349540829658508, 0.04063039645552635, 0.07946393638849258, 0.04617541655898094, 0.09975527226924896, 0.010721022263169289, 0.024229547008872032, 0.08962810039520264, -0.05815509334206581, 0.1318143755197525, -0.12230198085308075, 0.05291255936026573, -0.13178661465644836, -0.13282066583633423, -0.1485767364501953, -0.1339137852191925, 0.08446025103330612, 0.15165193378925323, 0.014090490527451038, 0.15127889811992645, -0.012101203203201294, 0.06569303572177887, -0.05622279644012451, 0.1372496634721756, 0.23182900249958038, 0.08285947889089584, -0.0849737823009491, -0.004587071482092142, 0.017775164917111397, 0.26776671409606934, 0.0459364578127861, -0.029437586665153503, -0.08097431063652039, -0.05979767069220543, -0.11022593080997467, 0.2159612774848938, -0.0981099084019661, 0.06627298146486282, -0.13595055043697357, 0.2629742920398712, -0.041014958173036575, 0.07669734954833984, -0.017675895243883133, 0.03873215988278389, 0.10636620223522186, 0.1138339713215828, -0.06625114381313324, 0.0397011898458004, -0.12325017154216766, 0.15631304681301117, -0.015602098777890205, -0.1826644092798233, -0.0092674121260643, 0.136775940656662, -0.3037498891353607, 0.06159552186727524, 0.01284044049680233, 0.2787086069583893, -0.05688359960913658, -0.28242969512939453, 0.0016888761892914772, 0.09597982466220856, 0.10363298654556274, 0.07193075120449066, -0.10068812221288681, -0.06201855093240738, -0.08368197083473206, 0.046902645379304886, -0.08780784159898758, 0.06903456896543503, 0.18774329125881195, 0.017856691032648087, 0.18694181740283966, -0.13834454119205475, 0.07293646037578583, 0.020911959931254387, -0.13269004225730896, 0.17623938620090485, -0.07624468207359314, -0.2079753875732422, 0.047690022736787796, 0.11628834903240204, 0.060767002403736115, -0.010936341248452663, -0.11423216760158539, -0.013781677931547165, 0.09751652926206589, -0.042542196810245514, 0.12456692010164261, -0.06576637178659439, 0.22031082212924957, -0.03320654481649399, 0.1130458265542984, -0.16462430357933044, 0.18862736225128174, -0.13203801214694977, -0.14738070964813232, -0.17033042013645172, 0.0851370319724083, -0.05801210552453995, -0.26949596405029297, 0.011266103014349937, -0.03182634711265564, -0.05885317921638489, -0.09630395472049713, 0.12865152955055237, 0.014461221173405647, -0.08030182123184204, -0.13645392656326294, 0.10871861129999161, 0.023860622197389603, 0.07038439065217972, -0.007445438299328089, -0.06960655003786087}, {-0.11348267644643784, 0.10786381363868713, 0.05861598998308182, 0.0785941630601883, -0.02997039072215557, 0.08913443237543106, 0.010194730944931507, 0.04467839375138283, 0.13932828605175018, 0.15312477946281433, -0.06594541668891907, -0.009163908660411835, 0.05154520645737648, -0.027445122599601746, -0.18872025609016418, -0.06681837141513824, -0.04518033564090729, -0.05123820900917053, -0.13784834742546082, 0.06433895230293274, -0.07195093482732773, -0.029477808624505997, -0.04360627010464668, 0.16353189945220947, 0.26775601506233215, -0.08758555352687836, 0.250645250082016, -0.10403534770011902, -0.13564275205135345, 0.022186538204550743, 0.11590807884931564, -0.14703750610351562, -0.10699471086263657, -0.13858844339847565, 0.05273134633898735, -0.1863918900489807, 0.06295957416296005, 0.21022014319896698, 0.06707406044006348, -0.05651404708623886, 0.0741674154996872, 0.026502899825572968, 0.08879105001688004, -0.06668612360954285, -0.06402475386857986, 0.11241918802261353, -0.10491210222244263, 0.07361847162246704, -0.019107306376099586, -0.06319597363471985, -0.2773272395133972, 0.06353666633367538, -0.10013033449649811, -0.16319316625595093, 0.012499900534749031, 0.04116060584783554, -0.12639732658863068, -0.2123277336359024, -0.008003351278603077, -0.23355522751808167, 0.051589567214250565, 0.02878285013139248, 0.022816477343440056, 0.034375205636024475, 0.022826019674539566, 0.041837818920612335, -0.14976221323013306, -0.12042831629514694, 0.15799376368522644, 0.05409150943160057, -0.03786398470401764, 0.02977609448134899, -0.17459948360919952, 0.09248000383377075, 0.17519967257976532, -0.20561762154102325, 0.05987618491053581, 0.31077516078948975, 0.0984172597527504, -0.0751258060336113, -0.1507463902235031, -0.08801478147506714, 0.044485967606306076, 0.046599000692367554, 0.15562903881072998, -0.03385215625166893, -0.0947021022439003, 0.048671308904886246, -0.035946086049079895, 0.1682395488023758, 0.09282543510198593, -0.0036769513972103596, 0.09212713688611984, -0.008989757858216763, -0.061119820922613144, 0.004522989504039288, 0.15084968507289886, 0.016951287165284157, 0.0007869070395827293, 0.06146566569805145, -0.19269198179244995, 0.04225343465805054, 0.12045180052518845, -0.17464801669120789, 0.032825831323862076, -0.3405206799507141, 0.11827414482831955, -0.12568168342113495, 0.20209911465644836, -0.0607142336666584, -0.07858073711395264, 0.18274755775928497, 0.09058097004890442, -0.14275319874286652, 0.2415473759174347, -0.13850265741348267, -0.14893633127212524, -0.05850067734718323, -0.2278454452753067, -0.09955337643623352, -0.09737364947795868, 0.19053764641284943, 0.004169465973973274, -0.21082435548305511, -0.3070692718029022, 0.08129176497459412, -0.011958267539739609, -0.13800014555454254, -0.1791004240512848, -0.1389530897140503, 0.12978708744049072, -0.10177715867757797, 0.08483977615833282, -0.019238635897636414, -0.005124149844050407, 0.13428881764411926, 0.01893962360918522, -0.02770249918103218, -0.013531602919101715, -0.030092135071754456, -0.005900952965021133, -0.2507874667644501, 0.06314714252948761, -0.1482764333486557, -0.3118157386779785, 0.049301475286483765, -0.046780552715063095, 0.021927934139966965, 0.17607741057872772, 0.023321101441979408, -0.0514877550303936, 0.050184763967990875, -0.046600185334682465, -0.19404177367687225, 0.13925707340240479, -0.016257070004940033, 0.07589839398860931, -0.05049113929271698, -0.07286544144153595, 0.1232079416513443, -0.03828629106283188, -0.1578046828508377, 0.10850847512483597, 0.002430039457976818, 0.12733949720859528, -0.12080845981836319, 0.11283241212368011, 0.19796495139598846, 0.0853734090924263, 0.12427979707717896, 0.012861956842243671, -0.03027721308171749, -0.057466596364974976, -0.14095231890678406, -0.03511860594153404, -0.08039245754480362, 0.13496997952461243, 0.04747101664543152, -0.019718298688530922, -0.05361190810799599, -0.0948476791381836, -0.27085819840431213, 0.15400353074073792, -0.4191177189350128, 0.13027161359786987, -0.1947302669286728, 0.35371333360671997, -0.012257929891347885, -0.09787175059318542, -0.09442079812288284, 0.07682615518569946, 0.08629990369081497, -0.09398907423019409, -0.3000323176383972, -0.06954586505889893, 0.03265848383307457, 0.21912051737308502, 0.02107323333621025, 0.03550902381539345, 0.05384359136223793, 0.013737991452217102, 0.20787447690963745, -0.07502684742212296, -0.12145659327507019, 0.020168833434581757, 0.16261756420135498, -0.043109066784381866, -0.07006735354661942, -0.033442746847867966, 0.13862967491149902, -0.11100147664546967, 0.38143694400787354, 0.13787035644054413, 0.09562913328409195, -0.18763978779315948, -0.20472203195095062, -0.11824662983417511, 0.03275655210018158, 0.22733063995838165, -0.10098908096551895, -0.11622593551874161, -0.12832129001617432, 0.018395982682704926, 0.47304967045783997, 0.09566524624824524, 0.36199265718460083, -0.1325504034757614, -0.028094666078686714, 0.046400297433137894, 0.1542062610387802, -0.19494114816188812, -0.018924517557024956, -0.0895908921957016, -0.0891476422548294, 0.032183483242988586, -0.0425063818693161, 0.050566233694553375, 0.03592216968536377, -0.08203433454036713, 0.1266724020242691, -0.08272901922464371, -0.05917290598154068, 0.17005816102027893, 0.05233148857951164, 0.029034674167633057, 0.054326336830854416, 0.05326063185930252, -0.08005587011575699, 0.0161321721971035, -0.11385724693536758, -0.19121220707893372, 0.322730153799057, -0.028649827465415, 0.033518437296152115, 0.03291737660765648, -0.0350942499935627}, {-0.1466706395149231, 0.18845099210739136, 0.006729572080075741, -0.15344679355621338, -0.1034541055560112, 0.16062550246715546, 0.009633767418563366, 0.0014586676843464375, 0.08152937144041061, -0.15560705959796906, -0.03340248391032219, 0.08660385012626648, 0.03762662038207054, 0.13541193306446075, -0.11752942949533463, -0.17535088956356049, -0.25604063272476196, -0.16934454441070557, 0.0881962701678276, 0.2406783252954483, 0.04429040104150772, -0.14967332780361176, -0.0695992186665535, -0.005160014610737562, 0.06838252395391464, -0.05950535461306572, 0.02174345590174198, -0.020796623080968857, -0.05530250072479248, 0.039414215832948685, -0.15635353326797485, 0.08639203011989594, 0.048228729516267776, 0.09465303272008896, 0.037980400025844574, 0.0030101612210273743, 0.05309121683239937, 0.10341818630695343, 0.024909067898988724, -0.0025880096945911646, -0.042636316269636154, -0.012851126492023468, 0.1524312049150467, -0.16852718591690063, -0.001031538937240839, 0.10001515597105026, -0.0393047109246254, -0.06664033979177475, 0.13786950707435608, -0.10068444907665253, 0.10161928832530975, 0.22378265857696533, -0.033765558153390884, 0.04725543409585953, -0.16589365899562836, 0.1043463945388794, -0.1569860279560089, 0.03348055109381676, -0.046052176505327225, 0.14104536175727844, -0.21699225902557373, -0.09259708225727081, 0.08023010194301605, 0.09380478411912918, 0.0348440483212471, -0.12333934009075165, -0.0030425640288740396, 0.1485864222049713, 0.13190943002700806, -0.16297797858715057, -0.10865370184183121, 0.0873667523264885, -0.2147558331489563, 0.08182533085346222, -0.4335704445838928, 0.08890285342931747, -0.0281866192817688, -0.6131415367126465, 0.15673400461673737, -0.07110550254583359, 0.022444244474172592, -0.10682987421751022, -0.1398838311433792, -0.09201662987470627, 0.23157374560832977, -0.0694340392947197, -0.06686811149120331, 0.08149422705173492, 0.0032245388720184565, 0.12274175137281418, 0.033448174595832825, 0.06894300878047943, 0.054299209266901016, -0.15881958603858948, -0.20175637304782867, 0.002053403528407216, 0.08435261994600296, -0.039891112595796585, -0.009320656768977642, 0.1359131783246994, 0.048070479184389114, -0.0842021107673645, 0.09725461155176163, -0.042775608599185944, 0.0997776910662651, 0.2611992061138153, 0.06474831700325012, 0.12454032897949219, -0.028367523103952408, -0.13015329837799072, -0.06679871678352356, -0.10074511915445328, -0.010265357792377472, 0.15774813294410706, -0.0264701209962368, -0.05656750872731209, 0.16108828783035278, 0.01735217310488224, 0.055505067110061646, -0.08216401189565659, -0.003244226798415184, 0.14898590743541718, 0.08771814405918121, 0.06770201772451401, 0.06158973649144173, -0.09822545945644379, -0.009151175618171692, 0.21587274968624115, 0.019727742299437523, 0.1367611587047577, 0.01473900768905878, 0.013025807216763496, 0.10430686920881271, -0.10625763982534409, 0.029221802949905396, 0.19058306515216827, 0.012737398035824299, -0.2681257426738739, 0.14193490147590637, -0.05865054577589035, -0.3804941773414612, -0.19225047528743744, -0.1346205323934555, -0.2960316836833954, 0.004959645215421915, -0.13122069835662842, -0.14918911457061768, 0.2308054268360138, -0.056562963873147964, 0.1378718614578247, -0.13368064165115356, 0.10527896136045456, -0.09163147956132889, 0.1227647066116333, 0.21922509372234344, 0.16349384188652039, 0.13673478364944458, 0.2272842824459076, -0.26061394810676575, 0.19933217763900757, -0.1284959763288498, -0.18946968019008636, 0.1153540387749672, 0.09217257797718048, -0.07286220788955688, 0.08445049822330475, 0.0930105596780777, -0.27671587467193604, -0.05881562829017639, -0.05562817305326462, 0.1535150706768036, 0.14003963768482208, 0.03562891110777855, 0.17900677025318146, 0.019695239141583443, 0.1490219235420227, 0.0473724864423275, -0.01283513754606247, 0.09112422168254852, 0.07557760924100876, 0.04825715348124504, -0.08234574645757675, 0.05643153190612793, 0.03717437759041786, 0.10827271640300751, 0.10973035544157028, -0.030365781858563423, 0.05996917560696602, -0.08221909403800964, 0.18610642850399017, -0.062291115522384644, 0.019380327314138412, 0.053946543484926224, 0.07587116956710815, 0.026077600196003914, -0.14088226854801178, -0.06167295202612877, -0.32905957102775574, 0.09596022963523865, -0.19477735459804535, -0.10900034755468369, 0.12759126722812653, -0.16203449666500092, 0.008162074722349644, 0.032278601080179214, -0.022121332585811615, -0.4571940004825592, 0.04289635643362999, 0.12691280245780945, 0.11354556679725647, -0.13738411664962769, 0.03292889520525932, 0.025729605928063393, 0.1197115108370781, 0.04419119283556938, -0.04479158669710159, -0.024609381332993507, -0.19853335618972778, -0.023315753787755966, 0.03772028535604477, 0.04466409236192703, 0.22043770551681519, -0.08618286997079849, 0.01197847630828619, -0.10574749857187271, 0.07362271845340729, -0.09951482713222504, -0.06330744922161102, -0.12213877588510513, -0.08470109105110168, 0.06879588961601257, 0.2596476972103119, 0.0202949121594429, 0.004054642282426357, 0.10142616927623749, 0.003606309648603201, 0.013549375347793102, -0.12556862831115723, 0.10762306302785873, 0.269025981426239, 0.05092152953147888, -0.0937081053853035, 0.045263610780239105, 0.08941992372274399, 0.015360585413873196, 0.25688132643699646, -0.0046714553609490395, 0.08232130855321884, -0.10327892005443573, 0.0217612124979496, 0.07541518658399582, -0.07381626963615417, -0.17227530479431152, 0.010745900683104992, -0.015321138314902782, -0.09376046806573868}, {0.07572910934686661, 0.05044299736618996, 0.008566047996282578, -0.006615307182073593, 0.12887173891067505, 0.13466569781303406, 0.1811305731534958, 0.048137784004211426, -0.07956042885780334, -0.09336034953594208, -0.16959063708782196, -0.22857901453971863, 0.0043447888456285, -0.01333774346858263, 0.03319096192717552, 0.04270677641034126, 0.12697279453277588, 0.016012810170650482, 0.04387480020523071, -0.008290491066873074, -0.5389214754104614, -0.07529092580080032, 0.03595645725727081, -0.08265433460474014, -0.2944267690181732, -0.16410645842552185, 0.10207914561033249, -0.07853204756975174, 0.007804452907294035, -0.06504157185554504, -0.22625599801540375, 0.2447224110364914, 0.06875795871019363, -0.2564001679420471, 0.0804707333445549, -0.0030039458069950342, -0.03046766296029091, -0.0009203265071846545, -0.10601790249347687, -0.11454960703849792, -0.0822368860244751, -0.027357835322618484, 0.027950681746006012, -0.12187328189611435, -0.009044230915606022, 0.008398613892495632, -0.16310644149780273, -0.0168722216039896, 0.02299857698380947, -0.0553438663482666, 0.11855433136224747, -0.03106420859694481, -0.131927028298378, -0.04140129312872887, -0.0170879028737545, -0.1528841108083725, -0.08523474633693695, 0.021106766536831856, -0.10740553587675095, 0.035315729677677155, -0.09763579070568085, -0.19087986648082733, -0.23893195390701294, -0.07082340121269226, -0.07066576182842255, 0.07485926896333694, -0.06154581159353256, -0.2815246880054474, 0.2527128756046295, 0.0256523285061121, -0.031707633286714554, -0.032090481370687485, -0.2152845412492752, -0.11315801739692688, -0.11444597691297531, -0.09572034329175949, -0.09876661747694016, 0.03872767835855484, 0.12241199612617493, -0.23452024161815643, 0.16029754281044006, 0.02517789602279663, 0.03260580450296402, 0.4115452170372009, -0.0990426242351532, 0.04081559553742409, 0.14452213048934937, -0.13434864580631256, -0.01914307102560997, 0.026990016922354698, -0.08398912101984024, 0.07388719916343689, 0.06776172667741776, -0.023631172254681587, -0.06134999915957451, -0.004076202400028706, 0.3241933286190033, -0.039558295160532, -0.04831516370177269, -0.007847854867577553, -0.014644118025898933, -0.17783600091934204, 0.0942000076174736, 0.05829011648893356, -0.27660807967185974, -0.060217682272195816, -0.07231946289539337, -0.07995340973138809, -0.14428548514842987, 0.13041622936725616, 0.02608199417591095, 0.12122295051813126, 0.19154520332813263, 0.09758522361516953, 0.06525000929832458, -0.09680027514696121, -0.07141046971082687, 0.05110810697078705, 0.006154603324830532, 0.042211197316646576, -0.0769662857055664, -0.03954516723752022, 0.017361588776111603, 0.02728659100830555, 0.08424226194620132, 0.02414564974606037, -0.04619770124554634, 0.13473376631736755, -0.12780915200710297, 0.04205132648348808, -0.13545478880405426, 0.14758937060832977, 0.03584862872958183, 0.13564719259738922, -0.10233186185359955, 0.050983354449272156, 0.044737450778484344, 0.0029246460180729628, 0.09869737923145294, -0.011326591484248638, 0.05912040174007416, 0.13349159061908722, 0.04723375290632248, -0.07913773506879807, -0.2386835366487503, 0.060581326484680176, 0.17361435294151306, 0.05671076104044914, -0.1093115359544754, -0.1378440260887146, 0.0271195899695158, -0.0003148522519040853, -0.13474895060062408, -0.18534275889396667, -0.02866222709417343, -0.121314138174057, -0.08281750977039337, -0.1827164739370346, 0.10291831195354462, 0.09706464409828186, -0.1192144975066185, -0.002269444288685918, 0.1719122976064682, -0.26088380813598633, -0.09811286628246307, -0.014789747074246407, -0.042599622160196304, 0.08725383877754211, 0.3452836275100708, 0.03907346725463867, -0.17831698060035706, -0.1082758754491806, 0.1673092395067215, 0.0008223527693189681, 0.12313343584537506, 0.06875527650117874, -0.1327582746744156, -0.1193329468369484, -0.08020368963479996, -0.18522746860980988, -0.0824282243847847, -0.1621192991733551, 0.09515267610549927, 0.2080712467432022, -0.14664024114608765, 0.028856927528977394, -0.10742737352848053, 0.19514277577400208, -0.019403163343667984, 0.022919774055480957, 0.10084623098373413, -0.27645736932754517, -0.039987266063690186, -0.10097203403711319, -0.004800485447049141, 0.21105003356933594, 0.10201915353536606, 0.1869685798883438, 0.12422648072242737, 0.017460161820054054, -0.04554803669452667, 0.17077480256557465, -0.082992322742939, 0.10097970813512802, -0.03548035770654678, -0.09674727916717529, -0.00669519929215312, -0.05739286541938782, 0.11809749901294708, -0.005613264627754688, 0.0767737552523613, -0.07980555295944214, -0.030424000695347786, 0.11653034389019012, 0.003663524752482772, -0.16084103286266327, -0.13147026300430298, 0.03796529769897461, 0.013562945649027824, -0.13090620934963226, -0.058814600110054016, 0.009291782043874264, 0.047344960272312164, -0.4468911588191986, 0.11534488201141357, -0.26624852418899536, 0.08815135061740875, 0.027549434453248978, 0.11676353216171265, 0.11972616612911224, -0.05879625305533409, -0.20938798785209656, 0.065644271671772, 0.1598011702299118, 0.2051214724779129, 0.0630708783864975, 0.006663724314421415, 0.16526995599269867, -0.4075853228569031, 0.0964198186993599, -0.01916886679828167, -0.048251062631607056, -0.019898269325494766, -0.1872147023677826, -0.10047135502099991, -0.07306202501058578, -0.13761265575885773, -0.030905816704034805, -0.18670593202114105, -0.09680548310279846, -0.07163682579994202, -0.13135379552841187, 0.11509087681770325, 0.09005485475063324, 0.023056454956531525, 0.11051996797323227}, {-0.009972474537789822, -0.20397885143756866, -0.03734925016760826, -0.07908027619123459, -0.17663781344890594, 0.040167003870010376, -0.1139422357082367, -0.2667781412601471, 0.01989174447953701, -0.053787119686603546, 0.0907650962471962, 0.20963262021541595, -0.1281827837228775, 0.029855066910386086, -0.11874895542860031, 0.03962375968694687, 0.15016411244869232, -0.07063499838113785, 0.24064205586910248, 0.10713250935077667, 0.1644205003976822, 0.07565410435199738, -0.029480399563908577, -0.07785916328430176, -0.17666925489902496, -0.11055093258619308, 0.19163134694099426, -0.07840968668460846, -0.051909975707530975, -0.12052657455205917, -0.1896161586046219, 0.16749358177185059, -0.050897713750600815, 0.038154881447553635, 0.24053606390953064, -0.1108858659863472, 0.11870573461055756, 0.12139519304037094, -0.0726318359375, 0.06043730676174164, -0.15520837903022766, -0.050211455672979355, -0.07006248086690903, -0.007936547510325909, 0.13532133400440216, -0.10432099550962448, -0.10394440591335297, 0.09813647717237473, 0.06123155355453491, 0.13928434252738953, 0.09082955121994019, 0.062194544821977615, 0.2844020128250122, 0.04598848521709442, -0.04777173325419426, -0.08891844749450684, -0.18928633630275726, 0.055231451988220215, -0.025332681834697723, -0.16485252976417542, 0.14787741005420685, -0.19512636959552765, 0.27626582980155945, -0.046750351786613464, -0.21126413345336914, -0.16870708763599396, 0.08839733898639679, 0.32340338826179504, -0.17209310829639435, -0.10932546108961105, -0.06154877319931984, -0.18759667873382568, -0.0877349004149437, -0.09853047877550125, -0.12412920594215393, -0.11479642242193222, 0.07569955289363861, -0.19032955169677734, -0.06920921802520752, -0.2920002341270447, -0.07017891854047775, -0.04890848323702812, -0.08217824995517731, 0.03423810005187988, 0.10203360766172409, -0.2461993545293808, -0.004636281635612249, -0.007423384580761194, 0.049052607268095016, -0.1778043657541275, 0.06778029352426529, -0.21970906853675842, 0.16053961217403412, -0.06703604012727737, 0.1608988344669342, 0.10056537389755249, -0.14630955457687378, -0.2167220264673233, 0.1819821447134018, -0.11188671737909317, -0.09066599607467651, 0.02288816124200821, -0.0960923433303833, 0.18013747036457062, -0.22153227031230927, -0.1530149132013321, 0.028931889683008194, -0.2556614875793457, -0.1552131325006485, 0.2137734442949295, -0.24336816370487213, -0.0988088846206665, 0.1136574000120163, 0.030422668904066086, 0.10723166912794113, -0.11723307520151138, -0.10231921821832657, -0.07115468382835388, 0.0803450345993042, 0.04592946916818619, -0.018563244491815567, -0.021840743720531464, 0.01593565195798874, 0.047581519931554794, 0.15723049640655518, 0.567742109298706, 0.13575634360313416, 0.30130863189697266, 0.018032588064670563, 0.03108103573322296, 0.07141322642564774, -0.0618683323264122, 0.09493691474199295, -0.012052081525325775, 0.11635158956050873, -0.2496439665555954, -0.07603053748607635, -0.0631263479590416, -0.10157959163188934, -0.05827806517481804, -0.009422151371836662, -0.1377018392086029, -0.10786636918783188, 0.07044383883476257, -0.10649164021015167, 0.019107932224869728, 0.24757808446884155, 0.07690157741308212, -0.15329554677009583, 0.1434861123561859, -0.11415331065654755, 0.13058839738368988, -0.034960683435201645, 0.1970125138759613, 0.007090975064784288, 0.16614876687526703, 0.13667349517345428, -0.07995954155921936, -0.008784257806837559, 0.1667894721031189, -0.0034534898586571217, 0.09711593389511108, 0.015177110210061073, -0.1473599672317505, -0.137184739112854, -0.03935966268181801, 0.09910425543785095, -0.08115007728338242, -0.005246276035904884, 0.1692216992378235, 0.016892334446310997, -0.21102741360664368, 0.03430468216538429, -0.06490997225046158, 0.045601021498441696, -0.05112976208329201, -0.05936260521411896, -0.2557826042175293, -0.0006555960280820727, 0.10185263305902481, 0.1886935830116272, 0.07242728024721146, -0.006740998476743698, 0.028943847864866257, 0.06114906072616577, -0.03424149751663208, 0.16816699504852295, 0.018374299630522728, 0.0378304049372673, 0.035739246755838394, 0.15398898720741272, 0.03917655721306801, -0.03560109809041023, 0.05710548907518387, -0.2497519850730896, 0.019681373611092567, -0.1697302907705307, -0.08326952159404755, -0.06889273226261139, 0.0016798285068944097, -0.05809066444635391, -0.13791711628437042, 0.24219441413879395, 0.2009013444185257, -0.2787838280200958, 0.015867218375205994, 0.08098674565553665, -0.07930493354797363, -0.2397226244211197, 0.09299563616514206, -0.03656758368015289, 0.0851922407746315, -0.14418353140354156, 0.06644537299871445, 0.14646203815937042, 0.0324091762304306, -0.0815265104174614, 0.01366625539958477, -0.2229386270046234, 0.18949522078037262, 0.09172693639993668, -0.01138685829937458, -0.02256522700190544, 0.10371699184179306, -0.015774911269545555, 0.0972360298037529, -0.3313539922237396, 0.1805039793252945, 0.15397150814533234, -0.28569796681404114, -0.2953053414821625, 0.07838018983602524, 0.10050379484891891, -0.028709670528769493, -0.14802153408527374, 0.13311438262462616, -0.03409794345498085, -0.0631730780005455, 0.17867784202098846, -0.010010735131800175, -0.24029342830181122, -0.06100662425160408, 0.19438053667545319, -0.10579890012741089, 0.02472156286239624, -0.018785374239087105, -0.07538996636867523, 0.021891169250011444, -0.04869712144136429, 0.05488229915499687, 0.0345589704811573, -0.26387354731559753, 0.06660332530736923, -0.05002358555793762, 0.031720615923404694, 0.10679569095373154}, {-0.23226743936538696, -0.045468784868717194, 0.003740832209587097, 0.0023300242610275745, 0.040244415402412415, -0.10983791202306747, -0.055803027004003525, -0.1113094612956047, 0.04407305270433426, 0.008858815766870975, 0.1412457823753357, -0.015760572627186775, -0.08941406011581421, 0.0533592626452446, -0.07546953856945038, 0.021358340978622437, 0.15263305604457855, -0.136534184217453, -0.03278852254152298, -0.016249878332018852, 0.006820010486990213, -0.09743130207061768, 0.03707876056432724, -0.21569865942001343, -0.013787142001092434, 0.06033182516694069, -0.07006515562534332, -0.01660347916185856, 0.06586463004350662, 0.19756922125816345, -0.1000368520617485, -0.024309294298291206, 0.012700618244707584, 0.13548392057418823, 0.23895300924777985, 0.11257628351449966, -0.13539361953735352, 0.12010658532381058, 0.10509169101715088, -0.07644280791282654, 0.03212188556790352, 0.09699682146310806, 0.03583462908864021, -0.00030170296668075025, 0.1816370189189911, 0.027833376079797745, 0.18595053255558014, 0.06902189552783966, -0.10699799656867981, 0.0792684555053711, 0.06477317214012146, 0.07795895636081696, -0.20754462480545044, 0.018834630027413368, 0.09279859811067581, 0.12001747637987137, -0.11314032226800919, 0.06776595860719681, -0.01676117815077305, 0.24136145412921906, -0.1769999861717224, 0.04059961065649986, 0.15625284612178802, 0.2770710587501526, -0.07283981144428253, -0.013745324686169624, 0.027391327545046806, 0.05049270763993263, 0.02211865782737732, 0.11691237986087799, -0.1680116057395935, 0.10662306845188141, -0.23411820828914642, -0.022119078785181046, -0.12349001318216324, 0.13026362657546997, -0.010506685823202133, 0.17654497921466827, 0.19234830141067505, -0.1863642930984497, 0.2768625319004059, 0.02815139666199684, -0.16877785325050354, -0.10072572529315948, -0.2098974734544754, 0.06655322760343552, -0.18395951390266418, 0.04813919588923454, -0.13190539181232452, 0.04564542695879936, 0.0029321913607418537, -0.04109634459018707, 0.1251084953546524, 0.020431675016880035, 0.16384489834308624, 0.09127688407897949, -0.09188372641801834, -0.06373152881860733, 0.053620319813489914, -0.023329580202698708, -0.059292927384376526, 0.012164744548499584, -0.04346892610192299, -0.10315699130296707, -0.018241174519062042, 0.12056177109479904, 0.09606829285621643, -0.07098747789859772, -0.06555481255054474, -0.011381695978343487, -0.018837295472621918, -0.12431668490171432, -0.16809575259685516, 0.14581409096717834, 0.14585600793361664, -0.10747787356376648, 0.04846654459834099, -0.0002604113251436502, -0.09190215170383453, -0.05955391377210617, -0.11498355120420456, 0.15285807847976685, 0.15708725154399872, 0.21450917422771454, 0.039166778326034546, -0.12976250052452087, 0.05849645286798477, 0.006171919871121645, -0.0760083869099617, -0.09951066970825195, 0.028381742537021637, -0.07034988701343536, 0.19810500741004944, -0.031079180538654327, 0.09370012581348419, 0.3690149486064911, 0.18980325758457184, -0.19210240244865417, 0.14638912677764893, -0.006963314954191446, -0.17484869062900543, 0.10523705184459686, 0.03338836133480072, 0.06278406828641891, -0.08763277530670166, -0.0875178650021553, -0.12285598367452621, -0.15546628832817078, -0.001979237888008356, 0.13001710176467896, -0.1025509387254715, 0.10446647554636002, 0.04315333068370819, -0.12297753244638443, -0.20447257161140442, 0.03176611661911011, 0.08644670248031616, 0.05056647211313248, 0.02387268655002117, 0.029235970228910446, -0.1388019323348999, -0.10439625382423401, 0.154000386595726, 0.08890584111213684, -0.20472657680511475, 0.2145482301712036, 0.2051229625940323, -0.09534184634685516, 0.12382086366415024, -0.007936541922390461, -0.056574612855911255, 0.12559175491333008, 0.040282268077135086, 0.054590098559856415, 0.11968924105167389, 0.06900254637002945, -0.06180083379149437, -0.026552889496088028, -0.009531079791486263, -0.08833926171064377, -0.0040708803571760654, 0.12132398039102554, 0.08941257745027542, -0.1353478580713272, 0.05770047754049301, 0.07978782057762146, 0.10402244329452515, 0.08157949149608612, -0.03581443428993225, 0.18253298103809357, -0.08971008658409119, 0.15799838304519653, 0.15703614056110382, 0.18249061703681946, -0.07179072499275208, -0.07512827962636948, -0.16936515271663666, -0.14314311742782593, 0.0612354502081871, -0.17601653933525085, -0.09292275458574295, 0.08569874614477158, -0.04180349037051201, 0.07955349236726761, 0.10197681188583374, -0.007915406487882137, -0.26497629284858704, -0.10064756125211716, -0.14749416708946228, -0.29756808280944824, -0.2290676236152649, 0.025227857753634453, 0.01574750803411007, 0.11255250871181488, -0.026427173987030983, 0.016597477719187737, 0.026436101645231247, -0.04142279177904129, -0.03472374379634857, -0.14411728084087372, 0.014816265553236008, 0.09751573204994202, -0.041750717908144, 0.05009187012910843, 0.051459573209285736, 0.14722739160060883, -0.12529148161411285, -0.03596533462405205, 0.006164418533444405, -0.10728095471858978, -0.11347545683383942, -0.02714460901916027, -0.041971247643232346, 0.05637334659695625, 0.20616190135478973, 0.01047208346426487, -0.0662139430642128, -0.10536664724349976, 0.0674782544374466, 0.03292810171842575, -0.0028750658966600895, 0.03827066347002983, -0.06655928492546082, 0.10844922810792923, 0.08606351166963577, 0.01262288261204958, -0.16839495301246643, -0.0630979835987091, -0.08563890308141708, 0.12760908901691437, 0.006609239149838686, -0.1391935795545578, -0.019186776131391525, -0.15938757359981537, 0.20786608755588531, -0.025865960866212845}, {0.11384572833776474, -0.07175977528095245, 0.09204540401697159, -0.035430800169706345, -0.11990246176719666, -0.09029317647218704, -0.12291296571493149, 0.07414945960044861, 0.053496744483709335, 0.05259900167584419, 0.05018201470375061, -0.05599163845181465, -0.059159159660339355, 0.02754932828247547, -0.04289273917675018, -0.06866702437400818, -0.17992088198661804, -0.050546422600746155, -0.1329832822084427, -0.14954356849193573, -0.22484661638736725, -0.0391479916870594, 0.034945048391819, 0.007022908888757229, -0.04000632464885712, 0.0145607003942132, 0.09337803721427917, 0.010180287063121796, 0.08810646831989288, 0.1123286783695221, 0.04732771962881088, -0.004399557132273912, 0.13922053575515747, 0.1550428420305252, 0.026651468127965927, -0.0223323255777359, -0.06616352498531342, 0.09176990389823914, 0.06719492375850677, -0.023336589336395264, 0.14955566823482513, -0.045453477650880814, -0.09954065084457397, 0.09474766999483109, 0.11140720546245575, 0.07176139950752258, -0.04038574546575546, -0.2811952531337738, 0.21350592374801636, -0.29209104180336, -0.055632516741752625, -0.011129681020975113, -0.15938028693199158, 0.13890297710895538, -0.08473537117242813, -0.019793597981333733, -0.06681153923273087, -0.08952087163925171, 0.007767310366034508, -0.04472619667649269, 0.010733717121183872, 0.0647890493273735, -0.29056957364082336, 0.1229260116815567, -0.0030529715586453676, 0.036238327622413635, 0.11244402080774307, -0.07578357309103012, -0.1946086883544922, 0.14297257363796234, 0.04922676086425781, -0.03629168123006821, -0.03248011693358421, -0.08248042315244675, 0.07837790250778198, 0.25846338272094727, 0.006700826343148947, 0.1154387965798378, 0.07383710145950317, 0.2765325903892517, 0.11672371625900269, -0.015326350927352905, -0.11681346595287323, -0.061753515154123306, -0.06942085921764374, -0.10260334610939026, -0.08898358792066574, 0.10776029527187347, -0.02924347296357155, 0.02028690278530121, 0.002463788026943803, -0.09957146644592285, -0.056037768721580505, -0.05566667020320892, -0.013234992511570454, -0.06485854834318161, -0.010471195913851261, 0.14711466431617737, 0.09950975328683853, 0.09930681437253952, -0.05541794002056122, -0.057060472667217255, 0.1872437298297882, 0.04491741955280304, 0.05888373777270317, 0.5086904168128967, 0.09639408439397812, 0.06975693255662918, 0.10190588980913162, -0.14165902137756348, 0.0609324611723423, 0.049221307039260864, 0.23949098587036133, 0.03804001212120056, 0.07214856147766113, -0.02484501339495182, -0.11771022528409958, 0.013844778761267662, 0.12316222488880157, 0.0116801206022501, 0.2346785068511963, -0.07291889190673828, -0.2558750510215759, -0.15717996656894684, 0.12845662236213684, -0.10249935835599899, 0.006158142816275358, -0.14524394273757935, -0.01158579345792532, -0.0033628325909376144, 0.04149768128991127, 0.26744458079338074, 0.022572224959731102, 0.045848507434129715, 0.09095264971256256, -0.10911194980144501, 0.002369408030062914, 0.22906924784183502, 0.012524744495749474, 0.15157687664031982, 0.18354061245918274, -0.05753195658326149, -0.10100331902503967, 0.029879793524742126, 0.004046197980642319, 0.041259005665779114, 0.03753301501274109, -0.22800438106060028, -0.016406452283263206, 0.05612960830330849, -0.11870189011096954, -0.012288945727050304, 0.07939425110816956, -0.09193055331707001, -0.21177607774734497, -0.027815889567136765, -0.2057996541261673, 0.22543959319591522, -0.07287295907735825, -0.24262112379074097, -0.0661616250872612, 0.04476672038435936, -0.014501829631626606, -0.04347876086831093, -0.04662449285387993, -0.05807111784815788, -0.12623481452465057, -0.281290203332901, -0.0481085330247879, 0.20325307548046112, -0.005371410399675369, 0.10439151525497437, -0.13350285589694977, 0.029896298423409462, -0.12558495998382568, 0.1276920586824417, 0.12711217999458313, -0.18647640943527222, 0.1228327602148056, -0.007190646603703499, -0.14479151368141174, -0.22687703371047974, -0.03356704115867615, -0.05804084986448288, 0.010917914099991322, -0.12119749933481216, 0.1481790542602539, -0.08209304511547089, 0.12502333521842957, 0.027087420225143433, -0.09392532706260681, 0.02049129642546177, -0.035282257944345474, 0.0962505042552948, -0.09380163252353668, -0.19174063205718994, -0.10034933686256409, -0.15490049123764038, -0.02485864982008934, -0.004122886341065168, 0.29990431666374207, 0.08970770984888077, -0.13568581640720367, -0.03838835284113884, 0.00010138950165128335, 0.023210207000374794, -0.05056151747703552, 0.10378221422433853, 0.05115136131644249, -0.2243281453847885, 0.034110069274902344, -0.008340664207935333, -0.24815633893013, 0.21694570779800415, -0.098091259598732, 0.06262755393981934, -0.07191753387451172, -0.07788271456956863, -0.07388482987880707, 0.20406411588191986, -0.1129414439201355, -0.17978908121585846, 0.07016291469335556, 0.06888414174318314, -0.05742790177464485, 0.12701034545898438, 0.13813604414463043, -0.06411904096603394, -0.027468722313642502, 0.04306167736649513, 0.0691620483994484, -0.01932184398174286, -0.1474160999059677, 0.03211744502186775, -0.0038723682519048452, 0.047585029155015945, -0.08656816184520721, 0.10512763261795044, -0.06361300498247147, 0.07313111424446106, -0.06410612910985947, 0.060188110917806625, -0.20231470465660095, 0.07331693172454834, -0.27874237298965454, 0.05731889232993126, 0.045021627098321915, -0.1337239295244217, 0.037710677832365036, 0.03216340392827988, -0.13489770889282227, 0.014640814624726772, -0.06997689604759216, -0.04281742870807648, 0.026195546612143517, -0.04954780638217926}, {0.005842460319399834, -0.22694078087806702, -0.2039983868598938, 0.04362617805600166, -0.05320100486278534, -0.12992669641971588, -0.014031263068318367, -0.11125168949365616, -0.10675109177827835, -0.18137094378471375, 0.10127780586481094, 0.08489509671926498, -0.04529796913266182, 0.050808507949113846, 0.10604635626077652, 0.11663513630628586, 0.2663406729698181, -0.24186761677265167, 0.06291870027780533, -0.16056326031684875, 0.21303893625736237, -0.08204127848148346, 0.1887352168560028, 0.03695331886410713, -0.03542191907763481, 0.04287915304303169, 0.10062858462333679, -0.02078896015882492, -0.04987703636288643, 0.1935320496559143, -0.015359093435108662, 0.14349102973937988, 0.09626509249210358, 0.19435229897499084, -0.10473506152629852, 0.02806096151471138, -0.0076387436129152775, 0.11244969069957733, -0.16237309575080872, 0.025629742071032524, 0.014676415361464024, 0.12060288339853287, -0.16460087895393372, 0.06210067868232727, 0.03163357079029083, -0.05602090433239937, 0.10344093292951584, 0.0430109016597271, -0.000829312193673104, -0.19996698200702667, -0.26376450061798096, 0.1372823268175125, -0.08208463340997696, -0.14307048916816711, -0.06189202517271042, -0.031519677489995956, -0.10933996737003326, 0.08832308650016785, -0.023483026772737503, -0.2168186902999878, -0.09686434268951416, -0.20465213060379028, 0.060141488909721375, -0.1307782381772995, -0.02081119827926159, -0.17174403369426727, -0.0684514045715332, 0.21149171888828278, 0.029450848698616028, -0.01591458171606064, 0.20316511392593384, -0.1136854887008667, -0.21266382932662964, -0.16093941032886505, 0.07348030060529709, -0.1244933009147644, 0.00020013020548503846, -0.2029498666524887, -0.12167004495859146, 0.17799484729766846, -0.11097745597362518, -0.1854383945465088, 0.031090715900063515, -0.05567804351449013, 0.03595665097236633, -0.2604055404663086, -0.23019370436668396, -0.016084836795926094, 0.14042329788208008, 0.06647586822509766, 0.02728174813091755, 0.020983221009373665, 0.26835307478904724, 0.019034598022699356, 0.3013806939125061, -0.07564746588468552, -0.1955711543560028, 0.10768353193998337, 0.16703732311725616, -0.3475729823112488, -0.027815967798233032, 0.10306813567876816, -0.226361483335495, 0.08300362527370453, 0.13441357016563416, -0.3878116309642792, -0.001267566462047398, 0.0004002740897703916, 0.048004165291786194, -0.14558005332946777, 0.09831281751394272, -0.3567863702774048, 0.0715741291642189, 0.021526794880628586, 0.03308427706360817, -0.04861430078744888, 0.012877125293016434, -0.1539871096611023, 0.04673226922750473, 0.1386484056711197, -0.12586478888988495, 0.005438616033643484, 0.014418178237974644, 0.31282666325569153, -0.1445748656988144, 0.36782214045524597, -0.019717799499630928, -0.17259015142917633, 0.0041234963573515415, -0.018611842766404152, 0.045794058591127396, -0.08795125782489777, 0.14578956365585327, 0.09821473807096481, 0.0778002068400383, -0.290117084980011, 0.08239167928695679, 0.1449524313211441, -0.12518449127674103, 0.23222313821315765, -0.07265329360961914, -0.0638226866722107, 0.11161532998085022, 0.13263240456581116, 0.09290225803852081, 0.02000190131366253, 0.06639414280653, -0.12241499871015549, 0.006902881897985935, -0.07613862305879593, 0.04638395458459854, 0.0020991370547562838, 0.04278569296002388, 0.06445956975221634, -0.050103768706321716, -0.016662757843732834, -0.03600762039422989, 0.06763424724340439, 0.011800405569374561, -0.06656986474990845, 0.023577747866511345, 0.10906421393156052, 0.09944605827331543, 0.08306474983692169, -0.05824415758252144, 0.0994061678647995, -0.0031182419043034315, 0.1544296145439148, 0.02274462953209877, -0.04076777398586273, -0.0780784860253334, 0.09031784534454346, 0.012470485642552376, 0.001363379298709333, -0.0653054267168045, 0.14152847230434418, 0.1125953197479248, -0.0991312637925148, 0.1196952760219574, -0.05049809440970421, -0.04515351727604866, 0.16939187049865723, -0.0254505667835474, -0.24417325854301453, -0.08909211307764053, 0.10198365151882172, 0.41871216893196106, 0.07975003123283386, 0.1885630041360855, 0.15035755932331085, -0.14980706572532654, 0.23318393528461456, -0.15336912870407104, 0.11410650610923767, -0.18580740690231323, -0.35480642318725586, -0.2497801035642624, 0.176613450050354, -0.056935399770736694, 0.11967694014310837, -0.08621085435152054, 0.2200229912996292, -0.17116659879684448, 0.017935071140527725, 0.07583386451005936, 0.14402617514133453, 0.10458623617887497, 0.024242520332336426, -0.43841174244880676, 0.13661162555217743, -0.037921011447906494, 0.2805444896221161, 0.03614266589283943, 0.04866500571370125, 0.11104337126016617, -0.09565440565347672, -0.11067228019237518, 0.015346900559961796, -0.039433080703020096, 0.06654118001461029, 0.006236129906028509, -0.06124123930931091, -0.1800607293844223, 0.37820908427238464, 0.0482858344912529, 0.34788891673088074, -0.2682749032974243, 0.03610696271061897, 0.14953306317329407, -0.21312987804412842, 0.08242791891098022, -0.06541559100151062, 0.038822006434202194, -0.1354944109916687, -0.011161659844219685, 0.14138109982013702, -0.14911943674087524, 0.029734425246715546, 0.07184399664402008, -0.1337142139673233, -0.004974303767085075, -0.13047084212303162, 0.07753262668848038, 0.16511760652065277, -0.1663314700126648, 0.1598885953426361, -0.004386688116937876, -0.132011279463768, -0.10683856904506683, -0.03465641662478447, 0.08665332198143005, -0.08226776868104935, 0.11477788537740707, 0.023582421243190765, -0.08850295841693878, -0.15464872121810913}, {0.1509789675474167, 0.21550330519676208, 0.14001527428627014, 0.21637608110904694, 0.11805935949087143, 0.03004777990281582, -0.11848700791597366, -0.08798535168170929, 0.17752514779567719, 0.0841817781329155, -0.3584861755371094, -0.11500032991170883, 0.0042839995585381985, 0.08963363617658615, -0.08707443624734879, 0.06365830451250076, 0.021990463137626648, -0.06294795125722885, -0.09038901329040527, -0.01314019225537777, 0.05028410628437996, 0.22322846949100494, 0.0006507115322165191, 0.11671128123998642, -0.008497320115566254, -0.023648688569664955, -0.26019611954689026, -0.09686227142810822, -0.055986467748880386, -0.13130606710910797, -0.04545688256621361, 0.028578348457813263, -0.1630435436964035, -0.08398555964231491, 0.029393980279564857, 0.14951945841312408, 0.130081444978714, 0.03384103253483772, -0.13073070347309113, 0.09948458522558212, 0.06833269447088242, -0.05738646537065506, 0.06151963770389557, 0.20652376115322113, 0.27201685309410095, -0.022559165954589844, 0.20653054118156433, -0.18894121050834656, -0.0724603682756424, 0.02339036948978901, 0.1877032369375229, 0.13572221994400024, 0.2802676856517792, 0.08500535041093826, 0.11745475977659225, 0.049143221229314804, 0.2651064693927765, -0.020715920254588127, 0.08006428927183151, -0.2304801642894745, 0.0983387902379036, -0.07603377103805542, 0.1106962263584137, 0.03188210353255272, 0.05737990885972977, 0.1155211478471756, 0.07492459565401077, 0.08689117431640625, 0.0023891280870884657, 0.1228315681219101, -0.07720858603715897, 0.11839284747838974, 0.023505406454205513, 0.3019807040691376, 0.4912773072719574, -0.17991460859775543, -0.2714676260948181, -0.07177133113145828, -0.256971150636673, -0.028953101485967636, 0.017233286052942276, -0.1506163477897644, -0.09174516797065735, 0.06116196885704994, -0.07227557897567749, 0.06576444953680038, 0.2304772436618805, -0.09561115503311157, 0.003215499222278595, -0.25061941146850586, -0.053279608488082886, 0.10401187092065811, 0.02746921218931675, -0.0115504190325737, -0.017600595951080322, 0.15079614520072937, 0.02525433897972107, 0.05748689919710159, 0.08575042337179184, 0.31477320194244385, -0.06368345022201538, 0.009902038611471653, -0.23884546756744385, 0.023303944617509842, -0.17490364611148834, 0.5500408411026001, 0.061819128692150116, 0.0014334684237837791, -0.2539108395576477, 0.26722294092178345, -0.053023774176836014, 0.0006377845420502126, -0.08863995969295502, -0.2129650115966797, -0.006969335954636335, 0.02066696807742119, 0.039570894092321396, -0.00837074127048254, 0.1085679680109024, 0.09285668283700943, 0.058534663170576096, 0.005374943371862173, 0.15139423310756683, -0.0239756740629673, -0.06611289083957672, -0.04360545426607132, 0.1993822157382965, -0.027644775807857513, -0.16326521337032318, -0.002693726448342204, 0.04789116606116295, 0.11417394131422043, -0.027362657710909843, -0.06685743480920792, 0.03256192430853844, 0.19513311982154846, 0.3182738125324249, -0.10233253240585327, 0.12062215805053711, 0.14087125658988953, 0.2698083519935608, -0.12254563719034195, 0.17526260018348694, -0.15498346090316772, 0.14125899970531464, 0.005976909771561623, 0.1619970202445984, 0.02310938574373722, -0.044518716633319855, 0.07006248831748962, 0.17712043225765228, -0.04478038102388382, -0.12977853417396545, -0.3202652931213379, 0.04125720262527466, 0.26364707946777344, -0.0005925981677137315, -0.16972939670085907, -0.1996874213218689, 0.15851563215255737, -0.09329424053430557, -0.02156110480427742, 0.21245209872722626, -0.015946485102176666, -0.021139778196811676, 0.10649452358484268, -0.004728229250758886, 0.036885228008031845, -0.06149309128522873, -0.11267784237861633, 0.1288084089756012, -0.3061629831790924, 0.0758567750453949, 0.06631483137607574, 0.020512573421001434, -0.028773557394742966, -0.19788207113742828, -0.11799173057079315, -0.018532367423176765, 0.3375013470649719, -0.0356464758515358, -0.23738540709018707, 0.04502405971288681, 0.0035969724413007498, 0.10510276257991791, -0.03739117830991745, 0.10741135478019714, 0.06566083431243896, 0.01251209992915392, 0.08768002688884735, 0.128988578915596, -0.018568968400359154, 0.09809231013059616, -0.02700200490653515, 0.13608960807323456, -0.17494475841522217, -0.13802044093608856, -0.14203818142414093, 0.09873349219560623, -0.04743671044707298, -0.19968532025814056, 0.11995966732501984, 0.08175846189260483, 0.06265266984701157, 0.010431366972625256, -0.2313404530286789, 0.41079282760620117, 0.0527343787252903, -0.030126750469207764, -0.09419314563274384, -0.025790730491280556, 0.09465214610099792, -0.25851181149482727, -0.10944466292858124, 0.004691227804869413, 0.04972545802593231, 0.07717965543270111, -0.23463992774486542, 0.10919599235057831, -0.08673838526010513, 0.03079713135957718, 0.048280827701091766, 0.06791411340236664, 0.09998087584972382, 0.14858712255954742, 0.11066372692584991, -0.12802916765213013, -0.017738083377480507, 0.2827061712741852, 0.11624264717102051, -0.39355117082595825, 0.04783281311392784, 0.07709413021802902, -0.0343671478331089, -0.2031375914812088, -0.04064144194126129, 0.11049758642911911, 0.16128124296665192, 0.0651552826166153, 0.4872385263442993, -0.09556841105222702, 0.2995961606502533, 0.19230735301971436, -0.008462316356599331, 0.059946876019239426, 0.11741901189088821, -0.045046817511320114, -0.09607742726802826, -0.07139391452074051, 0.11082141101360321, -0.10901951044797897, -0.03624459356069565, -0.12837663292884827, -0.2114543467760086, -0.16994713246822357, -0.047113146632909775}, {-0.08141552656888962, -0.24795353412628174, -0.22932115197181702, -0.1076611578464508, -0.2555070221424103, -0.1779426485300064, -0.1786450743675232, -0.19000709056854248, -0.06195573881268501, 0.030574552714824677, 0.2677832245826721, -0.32292914390563965, 0.02731381542980671, -0.04373447597026825, 0.07828351110219955, 0.008795622736215591, 0.05997070297598839, -0.05230746045708656, -0.038119785487651825, -0.1650194227695465, -0.12148625403642654, -0.15614494681358337, 0.006235486827790737, 0.08438193053007126, 0.06352858990430832, 0.3073403835296631, -0.16776151955127716, 0.06656094640493393, 0.13146023452281952, 0.19052505493164062, -0.08323592692613602, 0.0016285418532788754, 0.051906898617744446, 0.111040398478508, 0.19089210033416748, -0.303467720746994, -0.0555867962539196, 0.07191432267427444, 0.00588815426453948, -0.3048052489757538, -0.03845331072807312, 0.03682468831539154, 0.18304443359375, 0.32646965980529785, 0.013635870069265366, 0.11926492303609848, -0.06655213236808777, -0.022734561935067177, -0.00014745687076356262, 0.007697111461311579, -0.24677205085754395, 0.060627106577157974, 0.012073684483766556, 0.09828997403383255, -0.03880533576011658, 0.06561070680618286, -0.20508678257465363, 0.02634800225496292, -0.15711671113967896, -0.07265474647283554, 0.0794040709733963, 0.06419048458337784, -0.21639735996723175, 0.15901343524456024, -0.06293268501758575, -0.11975252628326416, -0.05430436134338379, 0.011095557361841202, 0.003099633613601327, -0.03888237103819847, 0.13485348224639893, 0.055957190692424774, -0.1696651428937912, 0.1399921476840973, -0.1722594052553177, -0.041662249714136124, 0.11947035789489746, -0.1796111911535263, 0.14969104528427124, 0.1418919861316681, 0.05069173872470856, 0.04132266342639923, 0.06464661657810211, 0.2795703113079071, -0.22131139039993286, 0.1245872899889946, -0.12685437500476837, -0.2662436366081238, 0.013265685178339481, 0.020387299358844757, 0.0748111680150032, -0.0902218297123909, -0.07076943665742874, -0.06987956166267395, -0.03321407362818718, -0.13278266787528992, -0.06268828362226486, 0.08034376800060272, 0.16212202608585358, 0.07994676381349564, 0.028090352192521095, -0.039532773196697235, -0.11011891812086105, 0.18667645752429962, 0.12227680534124374, -0.12245643138885498, 0.11550584435462952, 0.03213449940085411, -0.13343581557273865, -0.057944998145103455, -0.021519534289836884, -0.0486675426363945, 0.006709031295031309, 0.07867059111595154, -0.1819550096988678, -0.011877838522195816, -0.03579377755522728, 0.14705969393253326, -0.09073910117149353, -0.07571972906589508, 0.03464952111244202, -0.026486482471227646, -0.037694789469242096, -0.004098947625607252, -0.165145605802536, -0.06373760849237442, 0.03581485524773598, 0.08760389685630798, 0.01923384703695774, 0.17198820412158966, -0.21131908893585205, -0.28369584679603577, 0.10527436435222626, 0.13139192759990692, -0.10716760903596878, 0.03572288528084755, -0.0005643019103445113, 0.12319371104240417, 0.08488369733095169, 0.19766367971897125, -0.030936749652028084, 0.13760973513126373, 0.072586789727211, -0.03889653459191322, 0.33246126770973206, -0.14475667476654053, 0.36208629608154297, -0.005987726151943207, 0.1011267602443695, 0.10080832242965698, -0.11378837376832962, 0.03398583084344864, 0.05974376201629639, 0.09519375115633011, 0.031247364357113838, -0.05780681222677231, 0.01998985931277275, 0.04077332466840744, 0.06532084941864014, 0.10033805668354034, 0.05717998743057251, 0.0633818581700325, -0.18013961613178253, 0.1895674467086792, -0.005515699274837971, 0.049910981208086014, -0.04992298036813736, 0.13379426300525665, 0.050393324345350266, 0.011623717844486237, -0.12458893656730652, 0.18821865320205688, -0.2099461853504181, -0.2977144122123718, -0.16629229485988617, 0.00038308839430101216, -0.0058158570900559425, -0.1354532092809677, 0.04303025081753731, 0.01522387657314539, -0.0968334749341011, -0.08889885246753693, -0.18036291003227234, -0.02938229590654373, -0.15858620405197144, -0.23812301456928253, 0.045028865337371826, -0.17904095351696014, 0.08868245780467987, 0.006104369647800922, 0.16131103038787842, 0.024732688441872597, 0.010989557020366192, -0.15205712616443634, 0.15163806080818176, 0.22565415501594543, -0.08373567461967468, 0.07893073558807373, 0.06922568380832672, 0.1500764936208725, 0.1412980705499649, 0.04970676079392433, -0.04912571236491203, 0.07823710888624191, 0.04540286213159561, 0.22769421339035034, 0.037065763026475906, -0.06999734789133072, -0.149360790848732, -0.18074193596839905, -0.22739934921264648, 0.012904487550258636, 0.19431224465370178, 0.05512378364801407, 0.2538436949253082, 0.019820189103484154, -0.09833290427923203, 0.08562980592250824, 0.11033984273672104, 0.009685688652098179, 0.06189452111721039, 0.09208355844020844, -0.052981454879045486, -0.08062773197889328, 0.10713144391775131, 0.1214112788438797, -0.1965048909187317, -0.03028828091919422, 0.02337011508643627, 0.036965273320674896, 0.09783106297254562, 0.23633114993572235, -0.05202293023467064, -0.28858762979507446, 0.07531531900167465, -0.13982386887073517, -0.005126291885972023, -0.03137395903468132, 0.12980251014232635, 0.002529027173295617, 0.036910511553287506, 0.030875958502292633, -0.0642363429069519, 0.017471248283982277, -0.11401493102312088, -0.11285211145877838, 0.02590862847864628, 0.083910271525383, 0.07902919501066208, 0.10535190254449844, 0.21903075277805328, -0.06401656568050385, -0.2651495933532715, -0.09555508196353912, -0.026926835998892784, -0.08175213634967804}, {-0.0967465341091156, -0.010639991611242294, -0.009350107051432133, 0.2346068024635315, -0.03321823850274086, 0.03328370675444603, 0.33580705523490906, -0.32315438985824585, 0.2006353735923767, 0.17235316336154938, 0.04650247097015381, -0.38321784138679504, 0.14138081669807434, -0.21490608155727386, -0.01824892871081829, -0.12473665177822113, 0.16466374695301056, 0.04869581386446953, -0.1717710793018341, -0.08075550198554993, -0.28675970435142517, -0.08402138203382492, -0.3931833505630493, 0.22595198452472687, 0.2072858214378357, 0.11846304684877396, -0.08424437046051025, 0.007303176447749138, -0.13648387789726257, -0.1375584751367569, -0.07793041318655014, -0.05074742063879967, 0.029111089184880257, 0.10664235055446625, -0.05267203226685524, 0.16816383600234985, -0.05802402272820473, -0.13331164419651031, 0.21293863654136658, -0.026982665061950684, 0.09180702269077301, 0.00021620237384922802, -0.2210795134305954, -0.1281944364309311, -0.10531413555145264, 0.001762384781613946, -0.0659673884510994, 0.18947961926460266, -0.056144800037145615, -0.13118542730808258, -0.069706492125988, -0.0654989704489708, 0.0854533314704895, -0.027921006083488464, -0.036113377660512924, 0.049698859453201294, -0.03374854847788811, -0.008810291066765785, -0.3550485670566559, 0.40299150347709656, -0.1949588656425476, -0.04238666594028473, 0.06259983777999878, 0.12284867465496063, -0.03859802708029747, 0.07133116573095322, 0.21627876162528992, -0.14546144008636475, -0.06887190043926239, -0.07360759377479553, -0.026288794353604317, -0.10786561667919159, 0.042471569031476974, -0.018882280215620995, -0.2408437877893448, 0.17504267394542694, -0.1778266727924347, -0.1631832718849182, -0.05383213981986046, 0.27393069863319397, 0.09151674807071686, 0.046628668904304504, -0.18105287849903107, 0.06788603961467743, 0.0138170775026083, -0.09378938376903534, -0.019789298996329308, -0.012738906778395176, 0.032793015241622925, -0.22101528942584991, -0.025342578068375587, -0.19482140243053436, 0.10615935921669006, 0.050802189856767654, 0.08037061244249344, 0.2674618065357208, -0.05511332303285599, -0.044999778270721436, -0.16886912286281586, -0.04566670581698418, -0.18068350851535797, 0.048501208424568176, -0.07654977589845657, -0.06611723452806473, -0.009739617817103863, -1.2052568197250366, -0.0782218649983406, -0.07437209784984589, 0.003125953720882535, 0.10108736157417297, -0.022073127329349518, -0.09600397944450378, 0.14491108059883118, 0.11885672062635422, 0.06975715607404709, 0.0639808252453804, -0.04890541732311249, 0.12912455201148987, 0.08617499470710754, -0.04809512197971344, 0.05309304594993591, -0.1967780739068985, 0.032179396599531174, 0.22354339063167572, -0.004223008640110493, -0.23146545886993408, -0.06418800354003906, 0.27635064721107483, 0.029258660972118378, 0.030816884711384773, -0.06481747329235077, 0.0003145781811326742, -0.009453671984374523, 0.02770104445517063, -0.14192302525043488, 0.4100706875324249, -0.1250229924917221, -0.2820036709308624, 0.06624846160411835, -0.04533402621746063, -0.3621285855770111, -0.04920761659741402, 0.24470311403274536, 0.20358777046203613, -0.07228656113147736, -0.06488563865423203, -0.13472910225391388, 0.061188437044620514, 0.1554221659898758, -0.10142969340085983, -0.06519053131341934, -0.02980237826704979, 0.08887653052806854, 0.2767863869667053, 0.1325376331806183, -0.014461901038885117, 0.16429045796394348, -0.18246231973171234, 0.06643713265657425, 0.3303832709789276, 0.12033678591251373, -0.0988013967871666, -0.06667515635490417, -0.36438602209091187, 0.11914195865392685, 0.18489539623260498, 0.2069684863090515, 0.1833052784204483, 0.04410281032323837, -0.027125490829348564, -0.07296013087034225, 0.07073333114385605, 0.008756240829825401, 0.05086535960435867, -0.05494893714785576, 0.02844480611383915, 0.02682144194841385, 0.0519026517868042, -0.09241370111703873, -0.008364615961909294, 0.05365752801299095, -0.5516437292098999, -0.17849740386009216, -0.12872199714183807, -0.00883639045059681, 0.07386325299739838, -0.06318839639425278, 0.06149769574403763, 0.035199057310819626, -0.044849783182144165, -0.0911867693066597, -0.07937324047088623, -0.04426352307200432, 0.08600426465272903, -0.04301954060792923, 0.1425647884607315, -0.09717828035354614, -0.41190218925476074, 0.16053685545921326, -0.15889766812324524, 0.051930975168943405, -0.09022640436887741, 0.014679202809929848, 0.1841169148683548, 0.11052156984806061, 0.026394644752144814, -0.6500860452651978, -0.14616011083126068, -0.2216770052909851, -0.09984444081783295, 0.0035899290814995766, -0.201382577419281, -0.1452280730009079, -0.1282764971256256, 0.08343717455863953, -0.3081763684749603, 0.15604811906814575, -0.022219033911824226, 0.05414414405822754, -0.10281471908092499, 0.23802457749843597, -0.025263413786888123, -0.1516975611448288, -0.0741942897439003, 0.06550287455320358, -0.06490263342857361, -0.13137643039226532, -0.06138342618942261, -0.06525404751300812, 0.019925447180867195, -0.11933305114507675, -0.0645749643445015, 0.07779563963413239, -0.05373584106564522, -0.08750011771917343, 0.13531935214996338, -0.06085323914885521, -0.3091115355491638, 0.15644429624080658, -0.22322088479995728, -0.008418425917625427, -0.04839624464511871, -0.07922212034463882, -0.20563958585262299, 0.12694035470485687, 0.08488771319389343, 0.08375146239995956, -0.093243807554245, -0.12066788971424103, 0.025925925001502037, 0.1330128014087677, -0.07852587103843689, -0.01333765871822834, 0.036096904426813126, -0.15022847056388855, -0.08579789847135544}, {-0.22256509959697723, 0.004548045806586742, -0.028496326878666878, 0.2074892520904541, -0.014698377810418606, -0.1676303893327713, 0.015212991274893284, -0.17784331738948822, -0.08920116722583771, -0.06621303409337997, -0.011231287382543087, -0.13634319603443146, 0.013867453671991825, -0.10073258727788925, -0.036088693886995316, -0.04382134601473808, 0.27741581201553345, 0.04630475491285324, 0.22664235532283783, -0.038077227771282196, 0.24565570056438446, -0.04875720292329788, -0.006916898302733898, 0.09154652059078217, 0.02167814038693905, -0.008960195817053318, -0.05160749703645706, 0.1413719654083252, -0.08661633729934692, -0.11215005815029144, -0.1479758322238922, 0.06888901442289352, -0.09699679166078568, -0.07450681179761887, -0.07118550688028336, 0.06418132781982422, 0.04657917842268944, -0.1677028387784958, 0.26752394437789917, 0.0696902796626091, -0.052150677889585495, -0.0013458546018227935, 0.15163591504096985, -0.01284877210855484, 0.18897902965545654, -0.07776271551847458, -0.007471118122339249, -0.13194848597049713, 0.03559347614645958, -0.02211904339492321, 0.12701983749866486, 0.21275122463703156, 0.02869630604982376, -0.06633604317903519, 0.1175357848405838, -0.06651007384061813, -0.2247401773929596, 0.04379316791892052, -0.19687239825725555, -0.038084372878074646, -0.1937461644411087, -0.09402093291282654, -0.20561157166957855, 0.15382131934165955, -0.04244876652956009, -0.10103915631771088, 0.03974998742341995, -0.1414758414030075, 0.07042556256055832, 0.07625458389520645, -0.11634379625320435, -0.2732318341732025, 0.009096459485590458, 0.1320301741361618, 0.07103584706783295, 0.040145572274923325, -0.10493817180395126, 0.13916808366775513, 0.08350052684545517, 0.08869785070419312, 0.20792722702026367, 0.09930520504713058, 0.10879377275705338, -0.02975918911397457, -0.18415877223014832, -0.031063256785273552, 0.0356864258646965, 0.00786291342228651, 0.07707562297582626, -0.05094820633530617, -0.1425219625234604, 0.050179097801446915, -0.009110122919082642, 0.016515977680683136, -0.16979750990867615, 0.020132586359977722, -0.015658849850296974, -0.05529104918241501, -0.20380523800849915, 0.3819260597229004, 0.07829547673463821, -0.241012841463089, 0.12765245139598846, -0.08982846140861511, -0.12061543762683868, -0.12276194244623184, 0.00048588242498226464, -0.06539351493120193, -0.08879152685403824, 0.1534668654203415, -0.09943710267543793, -0.22842226922512054, -0.06036496162414551, -0.0011244597844779491, 0.010380398482084274, 0.13068461418151855, -0.11051137745380402, -0.0024933009408414364, 0.0457928292453289, 0.03186309337615967, 0.10412681847810745, -0.04152897745370865, 0.15909479558467865, 0.22102732956409454, 0.2974799871444702, 0.17224130034446716, 0.11605151742696762, -0.022334404289722443, -0.23674972355365753, 0.006158567499369383, -0.06634598970413208, -0.1853034794330597, -0.03528933972120285, -0.035677749663591385, -0.12908926606178284, 0.4533003270626068, -0.22364845871925354, -0.17737238109111786, 0.06272883713245392, -0.03766107186675072, -0.22192534804344177, 0.08644244819879532, 0.06270959973335266, -0.0018808242166414857, 0.007568296976387501, 0.07482622563838959, -0.1524672657251358, 0.026450946927070618, -0.11272066086530685, 0.028315067291259766, -0.12656369805335999, -0.08994689583778381, -0.044705912470817566, 0.17874056100845337, -0.23583777248859406, 0.08575525879859924, 0.020569099113345146, -0.2103416621685028, -0.07208525389432907, 0.5139985084533691, -0.1797339767217636, 0.014670019038021564, 0.009413577616214752, 0.062445659190416336, 0.018054218962788582, 0.13106107711791992, 0.06816338747739792, -0.07217486202716827, 0.09516434371471405, -0.08582393079996109, -0.0024571213871240616, -0.01925264485180378, 0.12669111788272858, 0.05624181032180786, 0.049731045961380005, 0.028323231264948845, -0.019824566319584846, -0.055958960205316544, -0.08711864054203033, -0.011135277338325977, 0.13543017208576202, -0.3121564984321594, 0.020634569227695465, 0.03229610621929169, 0.09077322483062744, -0.043659958988428116, -0.125715970993042, -0.022165898233652115, -0.15708355605602264, 0.10024891793727875, -0.002705024555325508, 0.005330415442585945, 0.2138439416885376, 0.13904128968715668, 0.12241731584072113, -0.02413102239370346, -0.0638255849480629, -0.2950761318206787, 0.029058251529932022, 0.08791298419237137, -0.12009835988283157, -0.05274083465337753, 0.02741296775639057, 0.06406262516975403, -0.010275526903569698, 0.004063902888447046, -0.2199387103319168, -0.29732435941696167, -0.03599296137690544, -0.03487087041139603, -0.01792256347835064, -0.017018819227814674, 0.006975236814469099, -0.026613296940922737, -0.041123583912849426, -0.31042858958244324, 0.08477957546710968, 0.048043593764305115, -0.10584644973278046, -0.13173094391822815, 0.3122675120830536, -0.05257348343729973, 0.14509256184101105, -0.0756366029381752, 0.049573156982660294, -0.1038251668214798, -0.03255721181631088, -0.14450354874134064, 0.19075536727905273, 0.23546408116817474, -0.058872926980257034, -0.00128382898401469, 0.10024488717317581, -0.027791807428002357, 0.025329018011689186, -0.04028831422328949, 0.00018941162852570415, -0.06547430157661438, 0.10844743251800537, 0.4240211546421051, 0.1376996636390686, -0.010023572482168674, -0.22822555899620056, 0.15303511917591095, -0.0457187183201313, -0.02062041498720646, 0.11612004041671753, -0.0820147842168808, -0.04001816734671593, -0.03251384571194649, -0.16041825711727142, -0.07000713795423508, -0.12559986114501953, -0.1498010754585266, 0.06782347708940506, -0.10755208134651184}, {0.03574560210108757, 0.07805486023426056, 0.03254231810569763, -0.02360324189066887, -0.13375578820705414, 0.21463073790073395, 0.018685802817344666, 0.17048946022987366, -0.0900721549987793, 0.04141686484217644, -0.019612040370702744, -0.038523901253938675, 0.007854598574340343, 0.09477890282869339, 0.17298057675361633, -0.1344178318977356, -0.06581619381904602, 0.1318342685699463, 0.08802979439496994, 0.05461842566728592, -0.011623416095972061, 0.027491163462400436, -0.01383972354233265, 0.04392629489302635, -0.05689443275332451, 0.047379717230796814, -0.01958003267645836, 0.11698874086141586, -0.05030009150505066, -0.034961577504873276, -0.07392411679029465, 0.016077864915132523, -0.11822280287742615, 0.09463519603013992, -0.040160927921533585, -0.07293613255023956, 0.036272935569286346, -0.22784645855426788, -0.16063055396080017, -0.015796564519405365, -0.04710033908486366, -0.0034889711532741785, 0.0047497558407485485, 0.1570957601070404, -0.06144541874527931, -0.09079574793577194, 0.2708326280117035, -0.09849219769239426, 0.1146935448050499, -0.00728892907500267, -0.16144193708896637, -0.08507585525512695, 0.14121876657009125, -0.07245728373527527, 0.14351047575473785, -0.18333201110363007, -0.02690531313419342, 0.09941604733467102, -0.018817275762557983, -0.05069783329963684, -0.00991754699498415, 0.06511746346950531, 0.03726746514439583, -0.23370470106601715, 0.0330437533557415, 0.10882089287042618, -0.08987102657556534, -0.09750299155712128, -0.03747955337166786, -0.05118783935904503, -0.08543645590543747, -0.16618207097053528, 0.26386043429374695, 0.21697232127189636, 0.026547526940703392, -0.0725197046995163, 0.18888141214847565, 0.17041578888893127, -0.17978134751319885, -0.030149610713124275, -0.08254688233137131, 0.009353896602988243, -0.08465801179409027, 0.10191947221755981, 0.05960078909993172, 0.08871466666460037, 0.14431416988372803, -0.055438004434108734, -0.051418013870716095, -0.001708009047433734, 0.11310423165559769, 0.12131186574697495, 0.1410282999277115, -0.1024957150220871, 0.013801616616547108, -0.05962427705526352, 0.10571768134832382, -0.08334311097860336, -0.3065352141857147, 0.09620381891727448, 0.01863946206867695, 0.19059620797634125, -0.051273688673973083, 0.17919564247131348, -0.0521240234375, 0.010692313313484192, -0.06427545845508575, -0.020682906731963158, -0.050508052110672, 0.04224659129977226, -0.025415487587451935, 0.16547003388404846, 0.10235011577606201, -0.01912633515894413, -0.21241088211536407, 0.043622419238090515, -0.10170998424291611, -0.07255838811397552, -0.051348648965358734, -0.1150609627366066, -0.09377145022153854, -0.13934923708438873, 0.2341815084218979, 0.05140908434987068, 0.13781510293483734, -0.028703799471259117, -0.14212344586849213, -0.020209236070513725, -0.036423277109861374, -0.06278575956821442, -0.10175999253988266, -0.1700800210237503, -0.053000759333372116, -0.016109660267829895, -0.13113835453987122, -0.050725456327199936, -0.05693330243229866, 0.0153116500005126, 0.08987974375486374, -0.13184821605682373, -0.09684895724058151, 0.14081893861293793, 0.15766699612140656, 0.12715046107769012, 0.02340957708656788, -0.1550019234418869, 0.11739100515842438, 0.10148444771766663, -0.1651967316865921, -0.16585882008075714, 0.14682018756866455, 0.1942192167043686, -0.09139271080493927, 0.07982533425092697, -0.021127983927726746, 0.038601286709308624, -0.06720544397830963, -0.2807392477989197, 0.09795888513326645, 0.05611877515912056, 0.030375657603144646, 0.017150018364191055, 0.015629958361387253, 0.09755656868219376, 0.13204099237918854, 0.03709389269351959, 0.05730637162923813, 0.04016280919313431, -0.02882542461156845, 0.10020113736391068, -0.12075169384479523, 0.048024166375398636, -0.10199473798274994, -0.16309450566768646, -0.1939985156059265, 0.08572448790073395, -0.038007017225027084, 0.08583874255418777, 0.016655830666422844, -0.05804436281323433, 0.04689490795135498, -0.06768936663866043, -0.025355877354741096, 0.21130575239658356, -0.10075201094150543, -0.1395515352487564, -0.1001121774315834, -0.05458072945475578, -0.13823965191841125, 0.02906043455004692, 0.0503779873251915, -0.007123757619410753, -0.017635103315114975, -0.12684154510498047, -0.1566966325044632, 0.04899369180202484, 0.2635740339756012, -0.07288982719182968, -0.04839981719851494, 0.17236991226673126, 0.010067642666399479, -0.03248240798711777, -0.09243568778038025, 0.07918404042720795, -0.14608094096183777, 0.03313569352030754, 0.06965611129999161, -0.10571449249982834, -0.07302214205265045, 0.19929242134094238, -0.037650201469659805, -0.20157793164253235, 0.05031725764274597, -0.2512236535549164, 0.011774214915931225, -0.08362925052642822, 0.030770396813750267, -0.1307435929775238, -0.2078986018896103, -0.07609973102807999, -0.046714045107364655, 0.21773453056812286, 0.03126373887062073, -0.2079654186964035, 0.041122663766145706, -0.12412052601575851, 0.1215735673904419, 0.05991758406162262, 0.03492521494626999, 0.11155631393194199, -0.039763934910297394, 0.15023332834243774, 0.17697900533676147, -0.07168436795473099, -0.03433351591229439, -0.1354246288537979, 0.03131016716361046, -0.15325453877449036, 0.12719692289829254, 0.026637662202119827, 0.19248971343040466, -0.03852428123354912, 0.036633431911468506, -0.015850737690925598, -0.1357375681400299, 0.0292338989675045, 0.13192012906074524, 0.16409268975257874, -0.0022058836184442043, -0.12961147725582123, -0.046145930886268616, -0.09505371004343033, 0.003014832502231002, -0.1036621481180191, 0.034071240574121475, 0.09677073359489441}, {0.0016749510541558266, 0.3796158730983734, 0.08572971820831299, -0.16132773458957672, 0.1831357628107071, -0.29414933919906616, 0.07076564431190491, 0.11839701235294342, -0.01052016019821167, 0.01336560770869255, -0.19052840769290924, 0.16607211530208588, -0.20574678480625153, -0.028221547603607178, -0.020479362457990646, 0.030160199850797653, 0.1752491593360901, -0.16108083724975586, -0.1711738109588623, 0.03746531531214714, 0.14149312674999237, 0.038232747465372086, -0.046588554978370667, 0.03036372922360897, -0.026724636554718018, 0.08653947710990906, -0.07596699148416519, -0.20194126665592194, -0.06953366100788116, -0.05754505470395088, -0.06944621354341507, -0.06841789186000824, -0.1721198707818985, 0.24833187460899353, 0.1759360432624817, 0.0837404802441597, 0.09414976835250854, -0.11472312361001968, -0.029475806280970573, 0.19826625287532806, 0.059411756694316864, -0.0683397501707077, -0.3265399932861328, -0.03200872614979744, -0.0026098445523530245, -0.279254674911499, 0.28297603130340576, -0.08134891092777252, -0.008562157861888409, 0.19749176502227783, 0.25814521312713623, -0.28736162185668945, 0.16276709735393524, 0.14541010558605194, -0.04343350604176521, 0.1119958907365799, 0.16051165759563446, 0.19668538868427277, -0.15621283650398254, -0.16143229603767395, -0.0677972063422203, -0.06202608719468117, 0.16532325744628906, -0.06683631986379623, 0.11692136526107788, 0.1978738158941269, 0.08842592686414719, 0.15439479053020477, 0.16617104411125183, -0.07320062816143036, -0.23667354881763458, -0.06546548753976822, 0.06615578383207321, 0.044331107288599014, 0.058349609375, 0.05731799080967903, -0.05775836855173111, 0.0020351321436464787, 0.08703192323446274, -0.1446591019630432, -0.21194273233413696, -0.09905968606472015, 0.3169958293437958, -0.03601665794849396, 0.11271266639232635, -0.004183470271527767, 0.1491873562335968, -0.13082115352153778, 0.03709506615996361, -0.04660124331712723, 0.152474507689476, 0.14235402643680573, -0.1610700935125351, 0.11080535501241684, 0.08708468824625015, 0.3850290775299072, -0.1060178354382515, -0.08179638534784317, 0.030526177957654, 0.057354267686605453, 0.08666767179965973, -0.10036991536617279, -0.3503749966621399, -0.043997615575790405, 0.1245146319270134, 0.060263533145189285, 0.11149986833333969, -0.016532784327864647, 0.06448866426944733, 0.10552836209535599, -0.016516877338290215, 0.06487568467855453, -0.46111002564430237, 0.10400710999965668, 0.10934845358133316, 0.2383415848016739, -0.0012303936528041959, -0.04806283861398697, -0.024648860096931458, 0.02063666842877865, -0.15802952647209167, -0.024991828948259354, 0.21634548902511597, -0.0238481517881155, 0.062292102724313736, 0.01626765727996826, -0.10677333921194077, 0.004862881265580654, -0.1903982013463974, -0.17037777602672577, -0.04644165188074112, -0.3670763373374939, -0.19233350455760956, 0.10779277235269547, 0.06528051197528839, 0.01847943291068077, -0.08339033275842667, -0.08386259526014328, 0.10779238492250443, -0.005899928044527769, -0.09269707649946213, 0.044310491532087326, 0.17561887204647064, -0.14374378323554993, -0.04194245487451553, 0.027311135083436966, 0.09614091366529465, 0.08418567478656769, -0.222237229347229, -0.11291243880987167, -0.03099215403199196, 0.28285783529281616, -0.02617742493748665, -0.0058010113425552845, 0.1354241669178009, 0.057709284126758575, 0.0476735420525074, -0.06510406732559204, 0.2620159685611725, -0.06881009042263031, 0.04075261205434799, 0.012052562087774277, -0.14362533390522003, 0.25607913732528687, 0.09622371941804886, -0.28578460216522217, -0.15126435458660126, 0.0291595458984375, -0.04859847575426102, 0.5197969079017639, 0.08337011188268661, -0.21440957486629486, -0.05315029248595238, -0.03724772855639458, -0.03771733492612839, 0.0024553928524255753, -0.09022119641304016, 0.023116152733564377, -0.17813405394554138, -0.08490857481956482, 0.26812800765037537, 0.028085142374038696, 0.16080214083194733, 0.02148636244237423, 0.04143825173377991, 0.027295811101794243, -0.09562467038631439, -0.06274046003818512, -0.11885114014148712, 0.05818849429488182, 0.11679933220148087, 0.07715415209531784, 0.007217558100819588, 0.040615346282720566, 0.22576653957366943, -0.22269520163536072, -0.006233299151062965, -0.03363919258117676, 0.04198798909783363, 0.11396178603172302, -0.278014600276947, -0.09266062080860138, -0.14300882816314697, -0.05142168700695038, -0.08990591019392014, -0.0541527085006237, 0.05352512001991272, -0.05129220336675644, -0.02743954211473465, 0.27445682883262634, 0.15061460435390472, -0.0344732329249382, -0.05240308493375778, 0.08038310706615448, 0.08330336958169937, -0.1629028022289276, 0.3317605257034302, -0.09755712002515793, 0.13416123390197754, -0.2849678099155426, 0.03755824640393257, 0.14211983978748322, -0.12965357303619385, -0.03856160491704941, 0.12722450494766235, 0.050166450440883636, -0.056319598108530045, -0.06180219352245331, 0.003488152753561735, -0.04926655441522598, -0.1855139285326004, 0.15473003685474396, 0.3181656301021576, 0.02189727872610092, -0.0166397113353014, 0.07255745679140091, 0.1475234478712082, -0.08083005994558334, -0.0518844835460186, 0.3222874104976654, -0.06079721078276634, -0.05038905888795853, 0.10054604709148407, 0.1493319869041443, 0.039268508553504944, -0.0889011099934578, -0.15895545482635498, 0.024339230731129646, 0.11493860185146332, -0.11541617661714554, -0.06722069531679153, 0.2108716368675232, -0.02758534625172615, -0.35157692432403564, -0.10828954726457596, 0.035790879279375076}, {-0.11127911508083344, 0.23684898018836975, 0.004299398511648178, -0.018414516001939774, 0.09738311171531677, 0.13176365196704865, 0.1508094072341919, 0.1080484688282013, -0.018413439393043518, -0.10228224843740463, -0.10778326541185379, -0.21263335645198822, 0.017958860844373703, -0.018964063376188278, -0.20559120178222656, -0.14010097086429596, -0.13898378610610962, 0.07690823078155518, -0.13061308860778809, 0.12893949449062347, -0.32777509093284607, -0.01836518570780754, -0.05564257130026817, 0.05542199686169624, 0.14530256390571594, -0.012549092061817646, -0.1667664349079132, 0.19324912130832672, -0.07420049607753754, -0.18293671309947968, 0.12511534988880157, -0.0013296983670443296, 0.02305300533771515, -0.10331529378890991, -0.1581752598285675, 0.1394597440958023, -0.1032867282629013, 0.02083732932806015, -0.020779961720108986, 0.15589162707328796, -0.03500467911362648, -0.07033427804708481, 0.2177262306213379, 0.016522452235221863, -0.05202489346265793, 0.07059268653392792, 0.1200568825006485, -0.11188949644565582, -0.05522983521223068, -0.2128046602010727, -0.06350088119506836, -0.062324605882167816, -0.08843949437141418, 0.10046891868114471, -0.06770581007003784, -0.05693454667925835, 0.12400992959737778, 0.21348881721496582, -0.02218993380665779, 0.047711268067359924, 0.003491683630272746, 0.12677161395549774, -0.13042539358139038, 0.051646675914525986, 0.2047308087348938, -0.03467205539345741, 0.05872276797890663, -0.06154510751366615, 0.06433644890785217, 0.00876463670283556, -0.13764271140098572, 0.053736086934804916, 0.0920589342713356, 0.13844898343086243, 0.020428931340575218, 0.06469655781984329, -0.1984950602054596, 0.07292750477790833, 0.024216145277023315, 0.3248043954372406, -0.03335385397076607, -0.025369636714458466, 0.16373002529144287, -0.0858784094452858, 0.04789251461625099, 0.21107178926467896, 0.0314164012670517, -0.16454972326755524, 0.03556996211409569, 0.11906542629003525, -0.03038441762328148, 0.06215304881334305, -0.08500449359416962, 0.04720636457204819, -0.19279269874095917, -0.18792904913425446, 0.03710775449872017, -0.06103992089629173, -0.2754587233066559, 0.056642085313797, 0.085427425801754, 0.09846339374780655, -0.06629930436611176, -0.10603340715169907, 0.15093687176704407, -0.15146301686763763, -0.0974045991897583, 0.2161606401205063, -0.06039843335747719, -0.13345953822135925, 0.2633095979690552, 0.13621735572814941, 0.05883489549160004, 0.10905405879020691, 0.014103337191045284, -0.11019360274076462, 0.12726382911205292, -0.042994532734155655, -0.17935951054096222, -0.16593411564826965, 0.13853192329406738, 0.02173861674964428, -0.09315814077854156, -0.08486240357160568, -0.1321118026971817, -0.3439919352531433, 0.12528471648693085, -0.051597513258457184, 0.0410219244658947, 0.015036971308290958, -0.0064499713480472565, -0.015690812841057777, -0.07084008306264877, -0.08193475008010864, -0.06848227977752686, 0.2349531054496765, -0.10132857412099838, -0.14449289441108704, 0.19221770763397217, 0.11759249866008759, -0.03552046790719032, -0.20722022652626038, 0.15276172757148743, 0.028504502028226852, -0.05390745773911476, 0.20203186571598053, -0.07908181846141815, -0.0878630131483078, 0.05860135704278946, -0.13173329830169678, 0.16026583313941956, -0.0052414326928555965, -0.15378108620643616, 0.17177502810955048, 0.2013474553823471, -0.03247798979282379, -0.1874915510416031, -0.03841645270586014, -0.07232331484556198, -0.01214827224612236, 0.07669766992330551, -0.19682621955871582, -0.025896385312080383, -0.03033163957297802, 0.13407108187675476, -0.2612968981266022, -0.1915697157382965, -0.04614318162202835, 0.17128601670265198, -0.03167307376861572, 0.198374941945076, 0.003988293930888176, -0.05871032178401947, 0.08971354365348816, -0.06860676407814026, 0.05731363222002983, 0.07879991829395294, 0.06472115963697433, 0.06800347566604614, -0.009196112863719463, 3.5585784644354135e-05, -0.11283261328935623, 0.12847039103507996, -0.12861019372940063, -0.05542878806591034, -0.05320179462432861, 0.22555775940418243, 0.10183419287204742, 0.050760723650455475, 0.0022169449366629124, -0.10815848410129547, 0.1456102430820465, -0.05754993483424187, 0.09443055838346481, 0.11852888762950897, -0.09496237337589264, 0.07631739974021912, -0.08459580689668655, -0.014370953664183617, -0.0141079630702734, -0.0787307396531105, -0.05423025041818619, 0.023690197616815567, -0.128664031624794, 0.09509218484163284, -0.10069214552640915, -0.16141867637634277, -0.10211127996444702, 0.2842574119567871, -0.055452533066272736, 0.0073258597403764725, 0.16031518578529358, 0.2158774584531784, -0.04143740236759186, -0.16184207797050476, -0.054162345826625824, -0.08627829700708389, -0.12280447781085968, 0.1176878958940506, -0.0405009500682354, -0.01409547682851553, -0.011384788900613785, -0.04257788881659508, 0.12351398169994354, 0.0016920337220653892, 0.22707802057266235, 0.2370189130306244, -0.06799191981554031, 0.10392411053180695, 0.03857323154807091, 0.26630547642707825, 0.014641751535236835, 0.040322329849004745, 0.13190357387065887, -0.01730966567993164, 0.006483909673988819, -0.22170551121234894, -0.06430510431528091, -0.05972662940621376, 0.013712814077734947, -0.03208089619874954, 0.07966820150613785, -0.2099914699792862, -0.07966294884681702, 0.030001699924468994, 0.07504899054765701, 0.055414505302906036, 0.09201455861330032, 0.014259252697229385, -0.15125738084316254, -0.07282144576311111, 0.1026465892791748, -0.15648935735225677, 0.019749879837036133, -0.022409845143556595, 0.13457629084587097}, {0.048923879861831665, -0.14047923684120178, 0.11633401364088058, -0.13265472650527954, -0.20310841500759125, 0.055912598967552185, -0.05430836230516434, 0.08655386418104172, -0.021173756569623947, -0.10065168142318726, 0.12545762956142426, -0.11705851554870605, -0.061515312641859055, -0.0811832994222641, -0.0017707565566524863, 0.11063429713249207, 0.010380887426435947, -0.010197203606367111, -0.20408686995506287, 0.024165721610188484, -0.2171899527311325, 0.01615477353334427, 0.04579498991370201, -0.16208535432815552, 0.08562348037958145, 0.09701453894376755, 0.05020943656563759, 0.0008582965238019824, 0.06517940759658813, 0.14956244826316833, -0.042126502841711044, -0.06473664939403534, -0.18104448914527893, 0.09021729230880737, -0.052874788641929626, 0.05930311977863312, -0.018121711909770966, 0.013232441619038582, 0.0979703813791275, 0.02016826532781124, 0.01841731369495392, 0.09678756445646286, -0.13036729395389557, -0.12457722425460815, 0.03551090881228447, 0.0827632024884224, 0.3202129900455475, -0.029579052701592445, 0.1200070008635521, -0.15174542367458344, 0.06452961266040802, -0.013579128310084343, 0.004662976134568453, -0.14959663152694702, 0.17510606348514557, 0.03441682457923889, 0.07353001087903976, -0.07147229462862015, -0.015609249472618103, 0.03714606538414955, 0.0480390302836895, 0.2749791145324707, -0.12887126207351685, 0.09163149446249008, -0.09445267170667648, -0.119649238884449, -0.10281118005514145, -0.060656193643808365, 0.05247483402490616, 0.007141336798667908, -0.1044890433549881, 0.1350395232439041, -0.13875484466552734, 0.18440425395965576, -0.2812235355377197, -0.05188194289803505, 0.33883073925971985, 0.08108130097389221, -0.31562408804893494, 0.24538977444171906, 0.05783243104815483, -0.05443461239337921, -0.18458427488803864, -0.060543231666088104, -0.02389254793524742, 0.2933099567890167, -0.06942717730998993, 0.043368205428123474, 0.03473525494337082, 0.03266703709959984, -0.08001552522182465, 0.0390494130551815, 0.04770016297698021, 0.03320115804672241, -0.05652295798063278, -0.015703069046139717, -0.022233417257666588, 0.04372367635369301, 0.09184570610523224, -0.27297547459602356, -0.08382146060466766, -0.09156743437051773, 0.28181201219558716, 0.14253798127174377, -0.0545797161757946, 0.0298038087785244, -0.01963876001536846, -0.01607358083128929, 0.04124627262353897, 0.027319245040416718, 0.004124070052057505, 0.06856278330087662, -0.01579376496374607, -0.015278504230082035, -0.09091071039438248, -0.05455474555492401, -0.15913808345794678, -0.10014520585536957, -0.13287071883678436, 0.030351027846336365, -0.03434531018137932, 0.11313319951295853, 0.5197513699531555, -0.18431492149829865, -0.08563434332609177, -0.20754364132881165, -0.12136553227901459, -0.04005899652838707, 0.07888942956924438, -0.07664479315280914, -0.36150795221328735, -0.12526094913482666, 0.1786605566740036, 0.07407518476247787, 0.20802077651023865, 0.20600005984306335, -0.08226384967565536, -0.10454663634300232, -0.0532756932079792, -0.016651328653097153, -0.07288869470357895, -0.20699456334114075, 0.42241963744163513, 0.10216277092695236, 0.040669035166502, -0.012299280613660812, 0.12882277369499207, -0.021083738654851913, 0.010451394133269787, -0.16032782196998596, -0.00022152223391458392, 0.04681779071688652, 0.05768230929970741, -0.02433229796588421, 0.05160783231258392, 0.051812078803777695, -0.09477101266384125, -0.04253312572836876, 0.041459325700998306, -0.15210750699043274, -0.07740969955921173, -0.08649646490812302, 0.015447703190147877, -0.10509754717350006, -0.04298665001988411, -0.05752890184521675, -0.010442624799907207, 0.018594341352581978, 0.20621472597122192, -0.04523954913020134, -0.08137378841638565, 0.042471736669540405, 0.05110595002770424, 0.05949854105710983, -0.055875662714242935, 0.06064308062195778, 0.06898204237222672, 0.14771783351898193, -0.15470710396766663, 0.08506524562835693, 0.06056922301650047, -0.1947937309741974, -0.008904358372092247, 0.07800192385911942, 0.04584125801920891, 0.012867824174463749, -0.049061473459005356, -0.07892219722270966, 0.01740686595439911, -0.09604751318693161, 0.09189330041408539, -0.06027715280652046, 0.022603796795010567, -0.06820396333932877, -0.003537443932145834, 0.10179716348648071, 0.2841040790081024, -0.12862761318683624, 0.00012724897533189505, -0.0869489535689354, -0.043491676449775696, -0.10124356299638748, 0.07304781675338745, 0.10293985158205032, 0.025455337017774582, -0.05038216710090637, 0.06463192403316498, 0.007682327181100845, 0.025272756814956665, 0.038942694664001465, 0.12974244356155396, 0.015836182981729507, 0.033355940133333206, -0.02283329889178276, 0.12565693259239197, 0.18767927587032318, 0.06584788113832474, -0.2061711996793747, 0.12128681689500809, -0.42452964186668396, -0.13772664964199066, 0.09846987575292587, -0.11805900931358337, -0.17739464342594147, 0.10801175981760025, -0.09575273096561432, 0.035905104130506516, 0.06403420865535736, -0.060741886496543884, 0.18951085209846497, 0.010900959372520447, -0.01971505768597126, 0.03915926441550255, 0.032981712371110916, -0.01574680581688881, 0.0796390175819397, -0.14411839842796326, 0.03645014017820358, -0.12452629208564758, -0.0672951266169548, -0.04667172580957413, -0.0008155656396411359, -0.003012188710272312, 0.02810438722372055, 0.03855355083942413, 0.021216893568634987, 0.10928905010223389, 0.11599250882863998, -0.08944851160049438, -0.12209322303533554, 0.013836955651640892, 0.04516996443271637, 0.021006843075156212, -0.06057409197092056, 0.2180900275707245, 0.05091197043657303}, {0.1369757056236267, 0.1626206934452057, -0.013460841961205006, -0.03150665760040283, -0.10476098209619522, 0.07997938990592957, -0.2128753811120987, -0.14010155200958252, -0.16295039653778076, 0.01975800096988678, 0.015283710323274136, -0.06803594529628754, 0.0008527804166078568, -0.010213913396000862, 0.1430356502532959, 0.132664293050766, -0.13883648812770844, 0.22867318987846375, 0.15361671149730682, -0.030193783342838287, -0.06920663267374039, 0.16616851091384888, 0.015227172523736954, -0.0865299329161644, -0.05890078470110893, -0.07909847795963287, 0.04853547364473343, 0.1572917401790619, 0.18442192673683167, -0.214852973818779, -0.17007054388523102, -0.05101659893989563, 0.2667660415172577, 0.12948426604270935, -0.19337520003318787, 0.048589613288640976, 0.08654437214136124, -0.08305393904447556, -0.15273210406303406, -0.23206448554992676, -0.08369062840938568, -0.09756115078926086, -0.14593861997127533, -0.019233765080571175, 0.15817148983478546, -0.20201362669467926, -0.1040043756365776, -0.07810691744089127, 0.06481262296438217, 0.07471668720245361, -0.19183751940727234, 0.04550914093852043, 0.1690904051065445, 0.05713009461760521, 0.026295099407434464, -0.10299618542194366, -0.054026614874601364, 0.22396378219127655, 0.20744922757148743, -0.29629838466644287, -0.21475505828857422, -0.015369340777397156, 0.07004918158054352, 0.15814776718616486, 0.2160981148481369, -0.15571971237659454, -0.07000527530908585, -0.038167569786310196, -0.13651250302791595, 0.017940213903784752, -0.057045962661504745, -0.01496278028935194, -0.19853933155536652, -0.03058774024248123, 0.30386650562286377, 0.09749500453472137, 0.025397732853889465, 0.3442448079586029, -0.25787246227264404, -0.2685469090938568, -0.04835927486419678, 0.08021733909845352, 0.0568196177482605, 0.12571343779563904, -0.08834430575370789, 0.11727837473154068, -0.1293443739414215, -0.17436960339546204, -0.002515325788408518, 0.1491984874010086, 0.16767409443855286, 0.08961689472198486, -0.14879681169986725, 0.07479649782180786, -0.14478279650211334, -0.23297668993473053, 0.12319950759410858, 0.0019359037978574634, -0.06413080543279648, 0.2522122263908386, -0.023242134600877762, 0.05756539851427078, -0.18853303790092468, 0.04806480556726456, 0.03055662289261818, 0.6174726486206055, -0.029364779591560364, 0.09681843221187592, 0.04481034353375435, 0.08230102807283401, -0.1298711895942688, 0.0661793202161789, -0.07235921919345856, -0.01579052396118641, -0.1909407079219818, 0.09422565996646881, -0.0031556093599647284, 0.26421085000038147, -0.21604467928409576, -0.20084388554096222, -0.12186650931835175, -0.005406504962593317, -0.06147143989801407, 0.11873766034841537, 0.1730758398771286, 0.020321272313594818, 0.007417299319058657, -0.22069182991981506, 0.011319062672555447, -0.04345094785094261, 0.0194198377430439, -0.1643206924200058, 0.2006099820137024, 0.026574796065688133, -0.05430063605308533, -0.08213488757610321, -0.13675746321678162, -0.12402396649122238, 0.17820274829864502, -0.22179439663887024, -0.08140253275632858, -0.003239687765017152, -0.0879298746585846, -0.08962636440992355, 0.0353240966796875, 0.15552139282226562, -0.2639099657535553, -0.19930630922317505, -0.13040761649608612, -0.03134062513709068, 0.08871418982744217, -0.019065681844949722, 0.22808176279067993, 0.003988557029515505, -0.13412296772003174, -0.18711629509925842, -0.014371045865118504, 0.03460557013750076, 0.05014035850763321, 0.0020209485664963722, 0.047885969281196594, 0.13269168138504028, 0.034260887652635574, 0.15651530027389526, -0.07027336210012436, 0.06351631879806519, -0.3059835135936737, 0.061898745596408844, 0.06887144595384598, 0.11377011984586716, 0.024505890905857086, -0.008914551697671413, -0.09108752012252808, -0.012396334670484066, 0.08538613468408585, 0.029779372736811638, -0.0322970412671566, 0.023917434737086296, -0.09235018491744995, -0.07403343170881271, 0.11779015511274338, -0.0789545476436615, -0.10270069539546967, -0.07797747850418091, -0.2387474626302719, -0.11036180704832077, -0.015945378690958023, -0.0207541361451149, -0.030020056292414665, 0.177286759018898, 0.06686890125274658, 0.12705698609352112, -0.157978817820549, -0.20086288452148438, -0.25905415415763855, 0.24086634814739227, 0.08092319220304489, 0.2460748553276062, -0.13938745856285095, 0.16539661586284637, 0.0796949565410614, 0.18312601745128632, -0.18941335380077362, 0.18346941471099854, -0.025808585807681084, -0.050545934587717056, -0.05372892692685127, -0.26412972807884216, 0.01516436506062746, -0.15724623203277588, 0.15095147490501404, 0.013430270366370678, 0.22634711861610413, -0.24201931059360504, -0.16185574233531952, 0.05828467383980751, -0.14480075240135193, 0.08930780738592148, -0.21715854108333588, 0.17360903322696686, 0.12676189839839935, 0.09215303510427475, -0.25617873668670654, 0.003977986518293619, 0.24045264720916748, 0.20406372845172882, 0.0010922744404524565, 0.1278950721025467, -0.025725241750478745, -0.011359039694070816, 0.227699875831604, -0.055399347096681595, 0.2698976397514343, 0.08095526695251465, -0.2837689518928528, -0.13906706869602203, 0.04552588239312172, -0.00702474033460021, -0.10280490666627884, 0.2645111680030823, 0.1754097044467926, 0.030403198674321175, -0.25728103518486023, -0.15509985387325287, -0.06540170311927795, -0.03561491519212723, 0.04228264465928078, -0.08820406347513199, 0.12125983089208603, -0.12814077734947205, -0.10912631452083588, 0.017536811530590057, 0.11064190417528152, -0.19114850461483002, -0.10658916085958481, 0.3989737033843994}, {-0.033367276191711426, -0.03376353532075882, -0.06623505800962448, 0.02913788892328739, -0.05359058082103729, 0.1315043568611145, 0.07809539139270782, 0.0879485234618187, -0.010466210544109344, 0.05176319554448128, 0.015338672325015068, -0.06556132435798645, -0.24231570959091187, -0.02803131751716137, 0.02932886779308319, 0.05631748214364052, -0.13218706846237183, 0.18080991506576538, 0.07505825906991959, 0.09056451171636581, -0.4011273682117462, -0.031128479167819023, -0.18012608587741852, 0.06782504171133041, -0.04934316128492355, 0.035515859723091125, 0.09422150254249573, 0.29392480850219727, -0.021100476384162903, -0.005300098564475775, 0.003921005874872208, 0.09280500560998917, 0.19299279153347015, 0.011988446116447449, 0.011422254145145416, -0.05668816342949867, -0.05508652701973915, 0.2316468209028244, 0.07124342769384384, -0.014731836505234241, 0.1339748650789261, -0.14639291167259216, -0.2591029405593872, -0.06210649013519287, 0.12383508682250977, -0.22256144881248474, -0.07842731475830078, 0.09128479659557343, 0.0627012625336647, 0.046110205352306366, -0.23916271328926086, 0.028679361566901207, -0.08703251928091049, 0.14240939915180206, -0.009196625091135502, 0.08869706839323044, -0.05313314124941826, -0.09828079491853714, 0.03984043747186661, -0.179647296667099, 0.10699789226055145, -0.06755594164133072, -0.09090331196784973, 0.05654679611325264, 0.03558850660920143, -0.017436189576983452, -0.005093551240861416, -0.08470761030912399, 0.036605123430490494, 0.03410487622022629, 0.20593079924583435, 0.032399218529462814, -0.03789882734417915, -0.14787453413009644, 0.042825546115636826, 0.10359595715999603, 0.004738240968436003, -0.004421606194227934, 0.08751489222049713, -0.15907467901706696, -0.05669322982430458, 0.09074719995260239, 0.0693877637386322, 0.10792090743780136, 0.028922297060489655, -0.10900816321372986, -0.1244937926530838, -0.20503680408000946, -0.006629975512623787, -0.04511527344584465, -0.06036289781332016, -0.09353601187467575, -0.18286964297294617, -0.03634679690003395, 0.15821243822574615, -0.04993775114417076, -0.016031399369239807, -0.03879987820982933, 0.04983975738286972, 0.011789422482252121, -0.10412617772817612, 0.05154964327812195, 0.09797253459692001, 0.010544721968472004, 0.10464802384376526, 0.19243116676807404, 0.09217250347137451, 0.23508140444755554, 0.26525241136550903, 0.08930615335702896, 0.05810120329260826, 0.08653993904590607, 0.009312699548900127, 0.20898188650608063, 0.005592236761003733, -0.07138919085264206, 0.06719212234020233, 0.0874311700463295, -0.049969807267189026, 0.02848479337990284, 0.02344108372926712, -0.05690392851829529, -0.04789785295724869, -0.12585727870464325, 0.0218970850110054, 0.09700276702642441, 0.09890478849411011, 0.011963262222707272, 0.007383291609585285, -0.007700518239289522, 0.015851812437176704, 0.11723443120718002, -0.05581541731953621, -0.04355546832084656, 0.05188584700226784, -0.12610580027103424, -0.005725303199142218, 0.1102508082985878, -0.0038993696216493845, -0.02229059673845768, -0.00880927499383688, 0.046423450112342834, -0.044614631682634354, -0.09226086735725403, -0.08923070132732391, 0.10826677083969116, 0.22955472767353058, 0.09686527401208878, -0.033491671085357666, 0.16479258239269257, -0.023083843290805817, 0.028986459597945213, -0.044422704726457596, 0.03397613763809204, 0.040566250681877136, 0.01372711081057787, -0.1633945107460022, 0.1607164889574051, 0.1732577681541443, 0.10429952293634415, -0.18190960586071014, 0.18434597551822662, 0.08057694137096405, -0.40508392453193665, 0.01899663172662258, -0.12227330356836319, 0.08960985392332077, 0.11869735270738602, 0.0156837310642004, 0.013121669180691242, 0.31268802285194397, 0.19875897467136383, 0.036558132618665695, -0.04911672696471214, -0.20818094909191132, 0.020703131332993507, -0.07129193097352982, -0.13857045769691467, -0.09282396733760834, 0.12081113457679749, -0.050806235522031784, -0.10654211789369583, -0.09262977540493011, -0.035502802580595016, -0.0750475749373436, 0.06028670072555542, 0.10021559149026871, -0.02466663159430027, 0.2007409781217575, 0.015203570947051048, 0.021388832479715347, 0.04706662893295288, 0.019266515970230103, -0.16613444685935974, -0.18885059654712677, -0.00011405480472603813, 0.046504613012075424, 0.13860896229743958, 0.01937289722263813, 0.14777100086212158, -0.14706683158874512, 0.11508134752511978, -0.006378320045769215, 0.03333006054162979, -0.12348116189241409, 0.006726372987031937, -0.028869889676570892, -0.023984599858522415, -0.09565270692110062, -0.05697436258196831, 0.1396290361881256, 0.017720632255077362, 0.016230856999754906, -0.21384352445602417, 0.05198660492897034, -0.1302788257598877, 0.026145529001951218, 0.040904778987169266, -0.11868640035390854, 0.10003555566072464, -0.03700883686542511, 0.007395398803055286, -0.09682303667068481, 0.043963465839624405, 0.07940573990345001, 0.2592707872390747, 0.07560532540082932, 0.1193576231598854, 0.10684847086668015, 0.04848574846982956, 0.013303295709192753, -0.02920200116932392, 0.1703469157218933, 0.04926988482475281, -0.062325287610292435, -0.03341282531619072, -0.004592966288328171, 0.022008530795574188, 0.03865968808531761, -0.02353779412806034, -0.0698239803314209, 0.03011307865381241, -0.4768078625202179, -0.09311994910240173, 0.05034321919083595, -0.011585723608732224, 0.005125862080603838, 0.016686057671904564, 0.15575265884399414, 0.09031938016414642, 0.042817097157239914, 0.020751040428876877, -0.02972918190062046, -0.03396168351173401, -0.32334089279174805, 0.09507355093955994}, {-0.047275278717279434, 0.033478304743766785, -0.03277964144945145, 0.032196659594774246, -0.03804945573210716, 0.23204676806926727, -0.1170015037059784, 0.19074024260044098, 0.21582143008708954, 0.019805362448096275, -0.1440945416688919, -0.10616432875394821, 0.15454892814159393, 0.0007985266274772584, 0.027288325130939484, -0.07685644924640656, -0.11764760315418243, 0.04638189449906349, 0.09746307134628296, 0.07557076960802078, -0.30220383405685425, -0.012373548001050949, -0.02988235279917717, -0.024165909737348557, 0.01579873450100422, -0.08863120526075363, -0.034089893102645874, 0.10020004957914352, 0.07696530967950821, 0.007017497438937426, -0.2504679262638092, 0.013406422920525074, 0.20624388754367828, -0.08250018954277039, 0.042274560779333115, -0.07388075441122055, -0.0377267450094223, -0.07253210246562958, 0.062298115342855453, 0.03399229049682617, 0.09818162024021149, 0.05242307856678963, -0.15191029012203217, 0.13247115910053253, 0.026794828474521637, -0.03470158576965332, 0.04047689586877823, 0.09978940337896347, -0.021170156076550484, 0.16213811933994293, -0.2692267894744873, 0.26042503118515015, -0.06325023621320724, 0.21758867800235748, -0.21702851355075836, 0.20386923849582672, 0.01073493156582117, -0.014644608832895756, 0.006658789236098528, -0.12662313878536224, -0.04638930410146713, -0.16405005753040314, 0.03884685039520264, 0.0573001354932785, 0.13658568263053894, 0.09637777507305145, 0.07413938641548157, 0.09188805520534515, 0.010544443503022194, -0.011100825853645802, 0.3447225093841553, -0.13340279459953308, 0.0066586933098733425, -0.1511964499950409, -0.013482620008289814, -0.012621373869478703, -0.022578632459044456, -0.004747123923152685, -0.5449493527412415, -0.26880085468292236, -0.16663387417793274, -0.07454024255275726, 0.09600989520549774, -0.047443799674510956, 0.1360378861427307, -0.2138877511024475, -0.036215465515851974, -0.24296315014362335, -0.014094105921685696, 0.11634968221187592, 0.028874440118670464, -0.000365587038686499, 0.027729276567697525, 0.09413415193557739, 0.08156830817461014, -0.020939834415912628, 0.1724412590265274, -0.070032499730587, -0.11660941690206528, -0.06124044582247734, 0.08058838546276093, 0.16213032603263855, -0.046979907900094986, 0.07394929230213165, 0.2761799693107605, 0.2331080436706543, 0.020684480667114258, -0.10683038830757141, 0.28361257910728455, -0.08242210745811462, 0.01072484627366066, 0.06910182535648346, 0.06114291399717331, -0.021128898486495018, -0.1691102385520935, -0.05572875961661339, 0.08057288825511932, -0.04791366308927536, 0.11839501559734344, -0.07642123103141785, -0.17472316324710846, -0.07879681885242462, -0.27332109212875366, -0.14174214005470276, 0.0653865784406662, -0.057267364114522934, -0.006416330114006996, -0.07808728516101837, 0.15457649528980255, 0.04666939005255699, 0.02849831059575081, -0.006181958131492138, 0.07039526849985123, 0.06633687764406204, -0.02629297971725464, -0.08976640552282333, -0.026894818991422653, -0.035595763474702835, -0.11270631849765778, 0.12121834605932236, -0.16696016490459442, 0.0710080936551094, -0.17376123368740082, -0.023950407281517982, -0.07205838710069656, 0.02676074579358101, 0.06081205978989601, -0.02737308293581009, 0.07931006699800491, -0.11411010473966599, 0.22316806018352509, -0.003802985418587923, 0.037523239850997925, -0.017837245017290115, 0.08844820410013199, -0.07570294290781021, 0.003712520468980074, 0.09788090735673904, 0.13958945870399475, 0.17927789688110352, -0.05777684971690178, 0.1255532205104828, -0.07574525475502014, -0.11875879764556885, 0.13502806425094604, -0.15299545228481293, 0.10829219222068787, 0.10620748996734619, -0.20499801635742188, 0.08006929606199265, 0.19799335300922394, 0.29772594571113586, -0.11933224648237228, -0.07814311981201172, -0.22926123440265656, 0.032306008040905, 0.13082648813724518, 0.04454540088772774, -0.10023856163024902, -0.08639396727085114, 0.1021462008357048, -0.07166993618011475, -0.24490764737129211, -0.3280889391899109, -0.17764298617839813, 0.06082221865653992, 0.2017429620027542, -0.06925193220376968, -0.032225631177425385, -0.062403421849012375, -0.03188618645071983, -0.11124614626169205, 0.0541519820690155, -0.11782661825418472, -0.08652002364397049, -0.09070493280887604, 0.17658500373363495, -0.04424217343330383, -0.10359354317188263, 0.05759884789586067, -0.018942255526781082, 0.1115526631474495, -0.08692032098770142, 0.01721138320863247, 0.038633719086647034, 0.1081906408071518, 0.07518135756254196, 0.04491797089576721, -0.0866042822599411, 0.019785333424806595, 0.01912669837474823, 0.20984765887260437, -0.016988391056656837, -0.18316912651062012, 0.008118151687085629, 0.09385474026203156, -0.23609119653701782, 0.04913187026977539, 0.014024048112332821, 0.12204869091510773, -0.2006772756576538, -0.20731715857982635, -0.08917074650526047, 0.2868674695491791, 0.059984635561704636, 0.4235139787197113, 0.04637465998530388, 0.14352679252624512, -0.10395082831382751, -0.0008237151778303087, 0.29298484325408936, -0.007306773215532303, 0.13756543397903442, 0.028348196297883987, -0.07330532371997833, 0.12283886224031448, -0.14124441146850586, -0.06615091115236282, -0.005460233427584171, 0.21051879227161407, -0.021555395796895027, -0.054019633680582047, -0.058372631669044495, -0.04510222002863884, -0.0760190412402153, 0.05629736930131912, 0.14093780517578125, -0.02870788425207138, -0.09666206687688828, -0.007489408832043409, 0.267221063375473, -0.035726722329854965, -0.10593812167644501, -0.0038256430998444557, -0.4069291353225708, 0.2672656178474426}, {0.03819568082690239, -0.15142351388931274, 0.06257140636444092, -0.05556076020002365, -0.03841615468263626, 0.0053811403922736645, 0.09509006887674332, -0.14842557907104492, 0.05617145448923111, 0.10785709321498871, 0.022318318486213684, -0.07177276909351349, 0.3163096606731415, -0.2504345178604126, 0.159480482339859, -0.09184926748275757, 0.10959853231906891, -0.08451284468173981, -0.19961003959178925, -0.13439854979515076, -0.1584978997707367, 0.076518714427948, -0.11847507953643799, -0.1921234279870987, 0.14033429324626923, -0.040596023201942444, -0.576874852180481, 0.05657866597175598, -0.0677480697631836, -0.06578865647315979, 0.08415987342596054, 0.05428510531783104, 0.2701614499092102, -0.02584664337337017, 0.024822723120450974, -0.11020240932703018, -0.025602733716368675, -0.10888876020908356, 0.003042721888050437, 0.03858804702758789, -0.3132859468460083, 0.14527085423469543, -0.08465499430894852, 0.04567596688866615, -0.00022447336232289672, -0.17106734216213226, 0.22147901356220245, -0.1740293949842453, -0.2181273251771927, -0.21510303020477295, -0.15988528728485107, -0.20558352768421173, -0.26584580540657043, 0.2401108592748642, -0.28246429562568665, -0.12909090518951416, 0.11372460424900055, -0.09647338837385178, 0.13872285187244415, 0.042747169733047485, 0.266676127910614, 0.006839493289589882, 0.23402859270572662, 0.14328709244728088, -0.07579406350851059, 0.30807921290397644, -0.010550948791205883, -0.1595660001039505, 0.03444572165608406, 0.1849936991930008, 0.1199222281575203, 0.003166817594319582, 0.13821636140346527, 0.015700213611125946, -0.2266012281179428, -0.06424568593502045, -0.10572946071624756, 0.08701908588409424, 0.06675904989242554, 0.20063959062099457, 0.11644011735916138, -0.19844424724578857, -0.07988885045051575, -0.0736987441778183, -0.31533852219581604, 0.19138020277023315, 0.04839921370148659, -0.07087664306163788, -0.3202284872531891, -0.1476326435804367, -0.11994627863168716, -0.040887702256441116, -0.13453544676303864, 0.09527979791164398, -0.20766861736774445, 0.03947120159864426, 0.08394164592027664, -0.03269701078534126, -0.04400014132261276, -0.17757850885391235, 0.007445834577083588, 0.19991017878055573, -0.05422955006361008, -0.10171685367822647, 0.10475189983844757, -0.48289403319358826, 0.19491800665855408, 0.11777021735906601, 0.058914221823215485, -0.14681501686573029, 0.0864994078874588, 0.20638498663902283, -0.26672831177711487, 0.04796368256211281, -0.29302161931991577, 0.053876012563705444, -0.03528996929526329, -0.18178801238536835, 0.11455992609262466, -0.06872232258319855, -0.05434044077992439, 0.1964234560728073, 0.1564720720052719, -0.04688679426908493, -0.06667722016572952, -0.42371585965156555, 0.1358243077993393, -0.09657064825296402, 0.149549201130867, 0.048930563032627106, 0.08316678553819656, 0.03541974350810051, -0.11556980013847351, 0.028778351843357086, -0.11533866077661514, 0.013731067068874836, 0.2376754879951477, 0.1783798336982727, 0.10606682300567627, -0.13560853898525238, 0.009910747408866882, -0.14103548228740692, 0.20316889882087708, 0.2053069919347763, 0.09756843745708466, -0.369041383266449, 0.16682110726833344, -0.03797603398561478, 0.22672106325626373, 0.03400830924510956, 0.09564929455518723, -0.3838171660900116, 0.0557003952562809, 0.06492259353399277, -0.07745492458343506, 0.052779316902160645, 0.08516304194927216, -0.18301084637641907, 0.08600503951311111, 0.1441548764705658, -0.029645530506968498, 0.1636648029088974, 0.3288315534591675, 0.005478661507368088, -0.1704704463481903, 0.01003201212733984, 0.16910900175571442, -0.05626296624541283, -0.12355529516935349, -0.1616491675376892, 0.2321782410144806, 0.07453104853630066, -0.04996606335043907, -0.10324687510728836, -0.06871536374092102, 0.15938128530979156, -0.17163826525211334, 0.07621639966964722, 0.17056892812252045, -0.082115538418293, 0.10448431968688965, -0.03737017139792442, 0.17497524619102478, 0.02952142246067524, -0.3454705476760864, 0.10356246680021286, 0.03306704759597778, -0.1867539882659912, -0.007444862741976976, 0.12529057264328003, -0.12218867242336273, 0.0752921998500824, -0.3188951015472412, 0.014275052584707737, 0.1390494555234909, -0.08326048403978348, -0.01873863860964775, 0.12437316030263901, 0.24490918219089508, 0.008532088249921799, -0.06524700671434402, 0.04215943068265915, -0.16569679975509644, 0.20600302517414093, 0.214564248919487, -0.12492343038320541, 0.18437731266021729, 0.23176540434360504, 0.18486303091049194, -0.33858346939086914, -0.04174737259745598, -0.16066813468933105, -0.2242545187473297, 0.0073733581230044365, -0.23371130228042603, 0.039796143770217896, 0.2912270128726959, 0.11995553970336914, 0.23404110968112946, -0.21169513463974, -0.13644297420978546, -0.06881503760814667, 0.18257960677146912, -0.1723339855670929, -0.06626424193382263, -0.016489403322339058, 0.279562771320343, 0.1912996470928192, -0.08790779113769531, 0.08430297672748566, -0.044941067695617676, -0.07429210841655731, -0.027203300967812538, 0.10936276614665985, 0.2147751748561859, -0.057673655450344086, 0.2566181719303131, 0.04681186005473137, 0.16038748621940613, -0.2255723625421524, 0.02797534130513668, 0.26160213351249695, -0.2103007435798645, 0.2990254759788513, -0.08109218627214432, 0.04752528667449951, 0.038582976907491684, -0.04716593027114868, -0.25016120076179504, 0.15578371286392212, 0.08039955049753189, -0.05251587554812431, -0.12937892973423004, 0.3507070243358612, 0.3248741030693054, -0.06467904895544052}, {-0.006166429258882999, 0.11956819891929626, 0.020539794117212296, 0.20802177488803864, -0.0015728420112282038, -0.03714148327708244, -0.5377691984176636, -0.16527292132377625, -0.032308489084243774, -0.13707037270069122, 0.19734437763690948, 0.032958511263132095, 0.07863537967205048, -0.035769037902355194, 0.06137582287192345, 0.11290579289197922, 0.143514946103096, -0.049033161252737045, -0.007565096020698547, -0.0016477216267958283, 0.08920874446630478, 0.08732236176729202, 0.2416778802871704, 0.07911311835050583, -0.04577605053782463, -0.14330913126468658, 0.06451335549354553, -0.059209249913692474, -0.0961189717054367, -0.1751234233379364, -0.02981467731297016, 0.15292005240917206, -0.22564299404621124, 0.23943670094013214, 0.1531704068183899, 0.054158203303813934, 0.04038887098431587, 0.281711220741272, -0.051844075322151184, -0.14057931303977966, -0.055873092263936996, -0.05603566765785217, 0.03841433674097061, 0.2661518156528473, -0.011317053809762001, -0.0342140756547451, 0.01084924302995205, -0.0909838080406189, -0.06977236270904541, -0.046937402337789536, -0.07562777400016785, -0.049144573509693146, -0.24305607378482819, -0.17638616263866425, -0.029303506016731262, -0.037682440131902695, 0.03901386260986328, -0.22836047410964966, 0.21002861857414246, -0.40163764357566833, 0.08269114792346954, 0.06290023028850555, 0.2520744800567627, -0.0012481388403102756, -0.19489508867263794, -0.14043645560741425, 0.3298851251602173, 0.02036583609879017, -0.06715645641088486, -0.08572296053171158, -0.004732904024422169, 0.20382636785507202, 0.1593286693096161, 0.0211741104722023, 0.020603779703378677, -0.3551076352596283, 0.20178954303264618, -0.0390816368162632, 0.154994934797287, -0.039576608687639236, -0.2774742543697357, -0.08993972837924957, 0.08926066011190414, -0.01419780869036913, -0.03224542364478111, 0.12079551815986633, -0.16892030835151672, 0.0018397062085568905, -0.1568409502506256, 0.19309458136558533, 0.1507623940706253, -0.131493479013443, 0.3078318238258362, 0.017261136323213577, 0.3017021715641022, 0.07900801301002502, -0.00246862368658185, 0.20107311010360718, 0.030029794201254845, -0.190232515335083, 0.007549028377979994, 0.2554742991924286, -0.037722036242485046, 0.008386821486055851, -0.04126271232962608, 0.26227065920829773, 0.17177316546440125, -0.10543707013130188, -0.06038278713822365, -0.12344461679458618, 0.20862530171871185, -0.06886401772499084, -0.12363316863775253, -0.10836181044578552, -0.16577894985675812, 0.05564567446708679, -0.2698676884174347, -0.025299346074461937, -0.06513099372386932, -0.21884877979755402, -0.16458725929260254, -0.07470328360795975, -0.11505325883626938, 0.05667714774608612, -0.02762865647673607, 0.06221376359462738, -0.06678058207035065, -0.08349082618951797, -0.1078711524605751, 0.03578539565205574, 0.012972225435078144, -0.005992507562041283, -0.12552674114704132, 0.04917933791875839, -0.063574880361557, -0.013297043740749359, 0.0881759375333786, 0.1114407405257225, -0.07115654647350311, 0.21849597990512848, 0.22713793814182281, -0.19267746806144714, -0.15617652237415314, -0.09922558814287186, 0.1453186720609665, -0.04623173922300339, -0.01798325963318348, -0.2236369550228119, -0.03668365627527237, 0.20390206575393677, -0.13230247795581818, 0.09040419012308121, 0.05387043207883835, 0.19990715384483337, -0.04580949991941452, 0.11783762276172638, -0.004180256277322769, -0.11241010576486588, -0.20964471995830536, -0.056522436439991, -0.22033722698688507, 0.0009443492162972689, 0.18487319350242615, 0.1973418891429901, 0.1356610506772995, 0.06706693023443222, -0.3922712802886963, -0.01093046460300684, 0.12056418508291245, 0.01637580245733261, 0.15392395853996277, -0.0032630213536322117, 0.034797102212905884, 0.024598609656095505, 0.3592081069946289, -0.005370917730033398, -0.2579124867916107, -0.1243068128824234, 0.13543012738227844, -0.02589474618434906, 0.016832944005727768, 0.013831081800162792, -0.08890842646360397, -0.19175615906715393, 0.029782693833112717, 0.06981640309095383, 0.11417168378829956, 0.0852988138794899, 0.1274302750825882, 0.04790869727730751, -0.09647797793149948, 0.2807064950466156, 0.23250024020671844, 0.07627099007368088, -0.02228645794093609, -0.0478905513882637, -0.1103694960474968, 0.048283517360687256, 0.05290787294507027, 0.10341662913560867, -0.16795946657657623, 0.14997237920761108, 0.13748498260974884, -0.14474725723266602, -0.04707243666052818, 0.0780242457985878, 0.5776950120925903, 0.18365366756916046, -0.1989537626504898, 0.09575711190700531, 0.049760423600673676, 0.0017702943878248334, -0.15397854149341583, 0.227179616689682, 0.011775498278439045, 0.2604101300239563, 0.12975099682807922, -0.09581784904003143, -0.1225733533501625, 0.006814020220190287, 0.036908384412527084, -0.04604373127222061, 0.022747891023755074, 0.17484888434410095, -0.09948278218507767, 0.2157973349094391, -0.08684330433607101, 0.07336769253015518, 0.1447896510362625, -0.12888196110725403, -0.19697599112987518, -0.10099855810403824, -0.04149239882826805, 0.12158762663602829, -0.07919713854789734, 0.022818785160779953, 0.2352462261915207, -0.023047523573040962, -0.01906655542552471, -0.04950045421719551, 0.10602124780416489, 0.03244843706488609, 0.04500618577003479, -0.05163354054093361, 0.12097717821598053, -0.10349437594413757, -0.017777562141418457, -0.08150790631771088, -0.2464977353811264, -0.10356570780277252, -0.14940255880355835, 0.12090142071247101, -0.00185328908264637, 0.22098812460899353, 0.23841720819473267, -0.07338053733110428}, {0.04861634969711304, 0.06461995840072632, -0.12596037983894348, -0.2450399249792099, 0.011249626986682415, 0.12424441426992416, 0.25880634784698486, 0.11353245377540588, 0.014306580647826195, 0.2929242253303528, 0.013803704641759396, -0.17462342977523804, 0.056248847395181656, -0.20880991220474243, -0.3676077425479889, -0.008849051780998707, -0.04603219032287598, 0.28480157256126404, 0.012302265502512455, 0.21604464948177338, -0.318807452917099, 0.035225413739681244, -0.03947139531373978, 0.1329265534877777, 0.09280022233724594, -0.016941437497735023, 0.01421247236430645, 0.07053613662719727, -0.1313653141260147, 0.0009491948876529932, -0.23686836659908295, 0.020688427612185478, 0.07430944591760635, 0.10219483077526093, -0.3044689893722534, 0.30891457200050354, 0.028744876384735107, 0.12369103729724884, -0.08474612981081009, -0.13937148451805115, 0.10130289942026138, -0.12788330018520355, -0.21115076541900635, -0.06854881346225739, -0.15378157794475555, -0.17829182744026184, -0.10182906687259674, 0.191961869597435, 0.016164299100637436, -0.019125090911984444, -0.10161425173282623, 0.14403396844863892, 0.018205538392066956, -0.06483114510774612, -0.04515991732478142, -0.05166253820061684, -0.04700758680701256, -0.12476193904876709, 0.05498344078660011, 0.03336295485496521, 0.1185590997338295, 0.08532442897558212, -0.1446026861667633, 0.11717309057712555, -0.024441875517368317, 0.08272434026002884, 0.0011362310033291578, -0.26352083683013916, -0.03522174432873726, 0.07716470211744308, 0.12065459787845612, -0.1274828165769577, 0.017399951815605164, -0.17914408445358276, -0.10406739264726639, 0.004633218515664339, -0.029484447091817856, -0.04089612513780594, -0.25340715050697327, 0.12410075217485428, 0.050459932535886765, 0.05685524269938469, -0.031136875972151756, 0.11727598309516907, -0.010884364135563374, -0.09880386292934418, 0.03768382966518402, -0.09746895730495453, -0.0497586652636528, -0.19987966120243073, 0.04480764642357826, -0.1314581036567688, 0.1150694265961647, 0.09453172236680984, 0.03134946525096893, 0.09284462034702301, -0.050518859177827835, 0.15013644099235535, -0.2766093313694, -0.005626752506941557, 0.008296225219964981, -0.003514489158987999, -0.1173257827758789, -0.0031530247069895267, -0.06514078378677368, -0.32919013500213623, -0.03845665603876114, -0.11955204606056213, 0.0654023066163063, 0.13429513573646545, -0.03139578178524971, 0.04421669617295265, 0.0369991771876812, 0.098424032330513, 0.1307906210422516, 0.08766848593950272, 0.08727585524320602, -0.00032234060927294195, 0.18817497789859772, 0.2187163531780243, 0.06594499945640564, -0.08310963958501816, -0.04396514594554901, -0.11094459891319275, 0.08612371981143951, 0.03946460038423538, 0.025626637041568756, 0.1363581269979477, 0.15067054331302643, -0.1872403621673584, -0.0003394504019524902, -0.04851648584008217, 0.022991469129920006, -0.03548170626163483, -0.2683697044849396, -0.1462649703025818, -0.2311534881591797, -0.25598856806755066, -0.009656756184995174, -0.23040041327476501, -0.05071387067437172, 0.11973074078559875, 0.009048417210578918, -0.09264053404331207, -0.0713031142950058, 0.013664638623595238, 0.11588698625564575, 0.05207511782646179, -0.12269864976406097, -0.00930172298103571, -0.11077076196670532, -0.08009166270494461, 0.05856398120522499, 0.16237670183181763, -0.0246747974306345, 0.048854097723960876, 0.018166735768318176, 0.004526523407548666, -0.0061495681293308735, 0.30888187885284424, -0.19032405316829681, 0.20261842012405396, -0.19598405063152313, -0.17799893021583557, -0.06220559403300285, -0.0897509902715683, 0.3367772400379181, 0.09899605810642242, 0.13995133340358734, -0.045624442398548126, 0.12719815969467163, 0.10508604347705841, 0.1372665911912918, 0.2951377332210541, 0.30583059787750244, -0.12867371737957, -0.1908782720565796, 0.19064253568649292, -0.17541508376598358, 0.21540603041648865, -0.10652019828557968, -0.22647804021835327, -0.09804452955722809, 0.08363544195890427, 0.09763401001691818, 0.25745368003845215, 0.11935532093048096, -0.010270238853991032, 0.0014474994968622923, -0.14555387198925018, -0.04728357866406441, -0.13963495194911957, 0.012219351716339588, -0.07046680897474289, -0.009721552953124046, 0.0838911235332489, -0.005720476154237986, -0.07843954861164093, 0.022308239713311195, 0.08189328014850616, -0.1147734522819519, -0.23333169519901276, 0.07727817445993423, 0.1417696475982666, 0.19578243792057037, 0.08976467698812485, -0.2735730707645416, -0.08242855221033096, -0.20880450308322906, -0.20366942882537842, 0.08444203436374664, -0.12605604529380798, -0.10763122886419296, -0.08239606022834778, 0.12863501906394958, -0.027980437502264977, 0.18316243588924408, 0.016414180397987366, 0.0806899145245552, 0.14649207890033722, 0.17696066200733185, 0.01485778484493494, -0.19031667709350586, -0.0809512510895729, -0.08054114133119583, 0.057209379971027374, -0.04250828176736832, 0.26415568590164185, 0.044444210827350616, 0.1198950782418251, -0.1028217300772667, -0.17683400213718414, 0.166748046875, 0.20732147991657257, 0.01965363509953022, 0.008584878407418728, 0.2773144245147705, 0.003768766997382045, -0.12145926803350449, -0.13151010870933533, -0.0536094531416893, 0.18805894255638123, -0.16944454610347748, -0.11403275281190872, 0.08910103142261505, -0.3147689402103424, -0.010237568989396095, -0.18111157417297363, -0.02694927155971527, -0.2930407226085663, -0.06266069412231445, -0.23914219439029694, 0.050932902842760086, 0.14720593392848969, -0.2996821105480194, 0.06787140667438507}, {0.03207771107554436, 0.257333368062973, 0.36204102635383606, 0.11838269978761673, 0.14233645796775818, 0.003455135738477111, 0.08273584395647049, -0.04208076000213623, -0.1915612369775772, 0.10692664980888367, -0.011922160163521767, 0.041377946734428406, -0.10728950053453445, 0.0059483167715370655, 0.10601993650197983, -0.0783238336443901, -0.09262418746948242, -0.16954651474952698, 0.04460138455033302, 0.08322060108184814, -0.11627581715583801, -0.11326573044061661, 0.04176205396652222, 0.04548024386167526, -0.016856234520673752, 0.14929074048995972, -0.17700721323490143, 0.033460915088653564, -0.04300336539745331, 0.003036130452528596, 0.008526258170604706, -0.049578022211790085, 0.290920227766037, 0.10143797844648361, 0.0912972018122673, -0.018666395917534828, 0.029765719547867775, -0.023072190582752228, 0.0667828768491745, 0.0044677057303488255, -0.2293928861618042, 0.04999769106507301, 0.11462000757455826, 0.19437827169895172, 0.055012285709381104, -0.09623677283525467, 0.00912474561482668, -0.2675876021385193, 0.025033213198184967, -0.21184557676315308, 0.051165785640478134, -0.03367731347680092, -0.037138618528842926, 0.17666538059711456, -0.03338480740785599, 0.03482212498784065, 0.05595903471112251, 0.12139516323804855, 0.06100843846797943, -0.09590300917625427, -0.06813912838697433, -0.16625039279460907, 0.20422126352787018, 0.10329611599445343, -0.0715155154466629, 0.018346691504120827, -0.0031413910910487175, 0.17108432948589325, 0.022444061934947968, -0.07467769086360931, -0.05662262812256813, -0.11338093131780624, 0.03212801367044449, 0.10728155076503754, 0.30192074179649353, 0.07448780536651611, 0.17110659182071686, -0.07017193734645844, 0.3350661098957062, -0.04888206720352173, -0.011391800828278065, 0.04163643345236778, 0.04125804081559181, -0.21883006393909454, -0.15431736409664154, -0.10630419850349426, 0.09762115031480789, 0.041522178798913956, -0.05994454398751259, -0.06373684108257294, -0.015844406560063362, 0.28958117961883545, -0.23544734716415405, -0.10166167467832565, -0.06628982722759247, -0.17376868426799774, -0.3215790390968323, 0.016138188540935516, 0.10598577558994293, -0.01880890503525734, 0.16074982285499573, -0.14989401400089264, 0.03840101510286331, -0.08193551003932953, 0.4631706178188324, 0.10562169551849365, -0.12537802755832672, -0.13108374178409576, -0.10399308055639267, -0.16666410863399506, -0.08528847992420197, -0.03460410237312317, 0.24262958765029907, -0.11407838761806488, 0.14062952995300293, 0.04864117130637169, -0.12881386280059814, 0.033653661608695984, -0.13586381077766418, 0.1453622579574585, -0.02608402445912361, 0.046742264181375504, -0.11219524592161179, -0.02022177167236805, 0.10263384878635406, 0.07330001145601273, 0.018672505393624306, 0.06833069026470184, 0.1462630331516266, -0.3326464295387268, -0.06819459795951843, 0.061111968010663986, 0.027395104989409447, 0.07895208895206451, 0.02354482375085354, -0.2283041924238205, 0.12425310909748077, 0.19759826362133026, 0.18646939098834991, -0.03991606831550598, 0.072242870926857, 0.13735701143741608, 0.0046191285364329815, 0.068989098072052, 0.06281008571386337, 0.07713177800178528, -0.056289516389369965, 0.16396698355674744, -0.013220538385212421, 0.3027973771095276, -0.03063262067735195, -0.07126124203205109, -0.221109077334404, 0.041131019592285156, -0.12697112560272217, 0.002902324078604579, -0.01941332407295704, 0.35470157861709595, -0.06530693173408508, 0.012912549078464508, -0.10227525234222412, 0.28304076194763184, 0.2426602840423584, 0.034097250550985336, 0.07862105965614319, -0.10034792125225067, 0.019443687051534653, -0.049641288816928864, -0.019899319857358932, -0.2782876789569855, -0.14782601594924927, 0.006518418900668621, -0.09659604728221893, -0.03428569436073303, -0.2968553602695465, 0.05200303718447685, -0.01814200170338154, -0.13172461092472076, 0.176946222782135, 0.19767099618911743, 0.22324037551879883, -0.025060957297682762, 0.33803266286849976, 0.11996588110923767, 0.045175760984420776, -0.10725007206201553, -0.3924403488636017, 0.09906385093927383, -0.1328437328338623, 0.0933072492480278, 0.0747508779168129, 0.2831028997898102, 0.40018054842948914, 0.3610612154006958, 0.19426994025707245, -0.10704341530799866, -0.18037153780460358, -0.12336442619562149, 0.06860603392124176, -0.00854514166712761, 0.10955306142568588, 0.060975611209869385, -0.24694319069385529, -0.43665486574172974, -0.025224819779396057, -0.14091356098651886, 0.13732904195785522, 0.07888953387737274, 0.09425386786460876, 0.14052297174930573, 0.006603628862649202, -0.15190331637859344, 0.13218934834003448, 0.472949743270874, -0.06721145659685135, -0.07442503422498703, -0.06873113662004471, 0.3626149892807007, 0.029508041217923164, 0.20957380533218384, 0.10908526182174683, 0.10711915045976639, 0.06436373293399811, -0.15811830759048462, -0.011750776320695877, -0.15854263305664062, 0.10805252939462662, 0.09315663576126099, -0.017952412366867065, -0.1819726973772049, -0.1821034997701645, 0.11381419003009796, -0.07968159765005112, -0.08744284510612488, 0.22196675837039948, -0.0707746148109436, -0.19744016230106354, 0.20725634694099426, 0.06083857640624046, -0.15738129615783691, -0.27057796716690063, -0.001118911779485643, -0.15487180650234222, 0.09962651133537292, 0.0005529560148715973, 0.010379341430962086, -0.030085647478699684, 0.09880965948104858, -0.061378493905067444, 0.13399234414100647, 0.013087062165141106, -0.07676418125629425, -0.08335316926240921, -0.058135025203228, 0.17990568280220032, -0.06667108833789825}, {-0.12771981954574585, 0.15198324620723724, -0.3243377208709717, -0.0995887890458107, 0.026462841778993607, -0.0491521917283535, 0.14084693789482117, 0.11529736965894699, -0.2022417038679123, -0.13513655960559845, 0.06670596450567245, -0.03168832138180733, -0.07474393397569656, -0.1185787171125412, -0.05348699539899826, -0.0346575602889061, -0.27597326040267944, 0.2863162159919739, -0.03248272091150284, 0.043448708951473236, -0.053870588541030884, 0.048957888036966324, -0.08090076595544815, -0.0043793534860014915, -0.00582025945186615, 0.08468145877122879, 0.11282679438591003, -0.013630230911076069, 0.15483371913433075, 0.04334884509444237, 0.13623777031898499, -0.14210331439971924, -0.0073127406649291515, 0.03899023309350014, 0.2937038838863373, -0.044141724705696106, 0.07971006631851196, 0.17204193770885468, 0.09082295000553131, -0.15198646485805511, 0.07034335285425186, -0.17088842391967773, -0.13707193732261658, -0.19525745511054993, 0.09002795815467834, 0.16239087283611298, 0.15962384641170502, -0.01952633075416088, 0.07016247510910034, -0.13192972540855408, -0.1919897496700287, 0.023395072668790817, -0.09483198821544647, -0.03620753809809685, 0.04867624118924141, -0.015831993892788887, -0.0664505735039711, -0.3383912742137909, -0.17400914430618286, -0.032065048813819885, -0.028216781094670296, 0.2033291906118393, -0.023471305146813393, 0.06281739473342896, 0.16323056817054749, 0.12365810573101044, 0.145999476313591, -0.1143723800778389, 0.030694657936692238, 0.11728192120790482, 0.10316912829875946, -0.008735103532671928, -0.0053544556722044945, 0.011205509305000305, -0.1962556689977646, 0.04752728343009949, -0.12258045375347137, 0.250575453042984, 0.02799707092344761, 0.20436322689056396, 0.10333912819623947, 0.09198898822069168, -0.04811732843518257, 0.21990077197551727, 0.19614726305007935, -0.1203334704041481, 0.026157615706324577, -0.03343414515256882, 0.12333431839942932, 0.2085631787776947, -0.025099290534853935, -0.0790044516324997, -0.08706226944923401, -0.05830630660057068, 0.07835342735052109, -0.11780637502670288, 0.09128051996231079, -0.15589940547943115, -0.1379016786813736, 0.09456777572631836, -0.1412048488855362, 0.01867910474538803, 0.15569357573986053, -0.0474807508289814, -0.05402961000800133, -0.12426699697971344, -0.2172713428735733, 0.044014230370521545, 0.15166574716567993, 0.11325892060995102, 0.06349319964647293, 0.08098600059747696, -0.0003616055764723569, 0.07465174794197083, -0.0006668788846582174, -0.029757361859083176, 0.12621349096298218, 0.12005932629108429, 0.0494377575814724, -0.1442987322807312, 0.08368652313947678, -0.03934130817651749, -0.02198503352701664, -0.010183611884713173, -0.11529333144426346, -0.05330434814095497, 0.022820085287094116, -0.0707048550248146, 0.03539030998945236, 0.06730794161558151, 0.20246323943138123, 0.2468351274728775, -0.06853799521923065, -0.24796034395694733, 0.00882234051823616, 0.23178520798683167, -0.017586082220077515, -0.21968556940555573, -0.07148287445306778, 0.04026758298277855, -0.07900366187095642, 0.026418747380375862, -0.20740430057048798, -0.07514268904924393, -0.21195007860660553, -0.14583832025527954, 0.051836367696523666, 0.09640585631132126, 0.08005160093307495, -0.11696802824735641, -0.1386638730764389, 0.06894022226333618, 0.23253637552261353, 0.031606271862983704, 0.11186002939939499, 0.03589412197470665, 0.008738338947296143, -0.0031973556615412235, 0.2006215900182724, 0.04217539355158806, 0.015405084937810898, -0.1344671994447708, 0.05090848356485367, -0.08722344040870667, -0.054196156561374664, -0.01207600999623537, -0.16104871034622192, -0.03494265675544739, 0.020580735057592392, -0.1344827562570572, -0.006483531091362238, -0.06971628218889236, 0.09645622968673706, -0.044284965842962265, -0.17470358312129974, 0.02599640190601349, 0.09823155403137207, 0.13499216735363007, -0.06516498327255249, 0.006885511800646782, -0.023817315697669983, 0.1705196052789688, -0.009394449181854725, -0.04396053031086922, -0.038704439997673035, -0.04439631849527359, 0.3528262972831726, 0.006643018685281277, 0.015761718153953552, -0.15430481731891632, -0.005494784563779831, 0.0903058722615242, -0.1721610575914383, -0.06599916517734528, 0.08117841929197311, -0.074437215924263, 0.16587747633457184, 0.14201007783412933, -0.06906569749116898, -0.037243958562612534, 0.02821946144104004, 0.03441735357046127, 0.01400414202362299, 0.016212599352002144, -0.19590914249420166, -0.08843864500522614, -0.09679096937179565, 0.20758742094039917, -0.023413656279444695, 0.03587355092167854, -0.0666947141289711, 0.29611945152282715, -0.013185148127377033, -0.23314601182937622, -0.06814375519752502, 0.06179676204919815, -0.11729242652654648, -0.15514881908893585, -0.080750472843647, -0.16521380841732025, 0.016291137784719467, -0.011947080492973328, -0.0033262046054005623, 0.27774807810783386, 0.015599244274199009, 0.1221078410744667, -0.0225068386644125, -0.07435242086648941, -0.0046373093500733376, -0.04661932587623596, 0.1835472583770752, 0.15524040162563324, -0.05210224911570549, -0.1778358519077301, -0.10812432318925858, -0.18615877628326416, 0.06121198832988739, -0.13553452491760254, 0.10923490673303604, -0.011586587876081467, 0.004268397111445665, -0.1435636579990387, 0.0881391242146492, 0.07758299261331558, 0.027447326108813286, 0.009525702334940434, 0.10123274475336075, -0.015022135339677334, 0.0030332147143781185, -0.02616083435714245, -0.13860078155994415, -0.024161607027053833, -0.0071647558361291885, -0.2354898899793625, -0.15655426681041718, -0.06520205736160278}, {-0.19926856458187103, 0.00041130679892376065, -0.07229334861040115, 0.03903999179601669, 0.051534030586481094, -0.09502093493938446, -0.24715355038642883, -0.1328466385602951, -0.20333455502986908, 0.16561347246170044, 0.005123230163007975, 0.0031444220803678036, 0.020031211897730827, -0.05193282291293144, 0.019125312566757202, 0.014131417497992516, 0.08560949563980103, 0.012040814384818077, -0.08374125510454178, -0.02359255589544773, -0.08324843645095825, -0.0034133726730942726, 0.06169100105762482, 0.024653665721416473, -0.16010484099388123, -0.07425999641418457, 0.03550058975815773, 0.13185179233551025, 0.10280171781778336, 0.03198761120438576, 0.04234577715396881, -0.34908056259155273, 0.05919221043586731, 0.18057133257389069, 0.19259944558143616, 0.0521242618560791, 0.009094317443668842, -0.06725911051034927, 0.02628762647509575, 0.05034063756465912, -0.02950885146856308, -0.20211465656757355, -0.19597645103931427, -0.09601666033267975, 0.10762431472539902, -0.24199116230010986, -0.009872077032923698, -0.010072385892271996, 0.05630110949277878, -0.017442122101783752, 0.1731773465871811, 0.007435233797878027, 0.14210115373134613, 0.167729914188385, 0.13084329664707184, -0.09407055377960205, -0.1368478238582611, 0.0802367553114891, -0.07959190756082535, -0.41606050729751587, -0.02109723538160324, -0.11090546101331711, -0.05313467979431152, 0.00012675610196311027, 0.01220934372395277, 0.011402408592402935, -0.011610429733991623, 0.030300868675112724, 0.21312429010868073, 0.10879477858543396, 0.05279277637600899, -0.20437473058700562, 0.05411739647388458, 0.09847363084554672, 0.19817407429218292, 0.09375069290399551, 0.31870201230049133, 0.06017038971185684, 0.06775402277708054, 0.1395147293806076, 0.05395602434873581, 0.04756335914134979, 0.19791154563426971, -0.16967186331748962, 0.005207598675042391, -0.10887855291366577, 0.05890345945954323, -0.07778255641460419, -0.08510366082191467, 0.029163626953959465, 0.12575975060462952, 0.005197808146476746, -0.19593952596187592, 0.04374345764517784, 0.18286016583442688, -0.05962427332997322, -0.13071098923683167, 0.2562258839607239, 0.10342703014612198, 0.07665413618087769, -0.03152544051408768, 0.011065008118748665, -0.051460228860378265, -0.014168165624141693, 0.06462952494621277, 0.10953270643949509, -0.1592310070991516, 0.12046592682600021, 0.41770708560943604, 0.007878882810473442, -0.17620033025741577, -0.05772670730948448, -0.09146077930927277, 0.07728947699069977, -0.05226719006896019, 0.18603739142417908, -0.04664314165711403, -0.026513678953051567, 0.12700869143009186, 0.015402112156152725, -0.14011605083942413, 0.045418817549943924, -0.09994734823703766, 0.1036621555685997, 0.005011146888136864, -0.06332078576087952, -0.08225036412477493, -0.0636640414595604, -0.07039009779691696, -0.0278144683688879, 0.07410941272974014, 0.00961213931441307, -0.014784899540245533, -0.1071639433503151, -0.05166551470756531, 0.02566838636994362, -0.17000310122966766, 0.06990652531385422, -0.186086967587471, -0.040280167013406754, 0.21188198029994965, -0.015835603699088097, 0.08735357224941254, 0.293066143989563, 0.11516381800174713, 0.13130858540534973, 0.10054134577512741, 0.07501593232154846, 0.010288248769938946, -0.1052950918674469, -0.2462959587574005, 0.1083456352353096, -0.0011845908593386412, 0.14714282751083374, -0.09856881946325302, -0.2458423525094986, -0.1486731469631195, 0.14314201474189758, 0.09468325227499008, 0.03280983865261078, -0.15850448608398438, -0.023536020889878273, -0.0024197488091886044, -0.0924428179860115, -0.05818555876612663, 0.021360833197832108, -0.13549315929412842, -0.07747436314821243, -0.11588294804096222, 0.13324742019176483, -0.012702696025371552, 0.1289539337158203, 0.12425285577774048, 0.019304761663079262, 0.04239305108785629, -0.025561345741152763, 0.17754599452018738, 0.005140315741300583, 0.050414618104696274, -0.10151038318872452, -0.04331442713737488, -0.05318864434957504, 0.07697407901287079, -0.044261470437049866, -0.02579784393310547, -0.019966334104537964, -0.02155599743127823, -0.2206774652004242, 0.028156239539384842, 0.16894137859344482, -0.014264460653066635, 0.01734030246734619, -0.08515148609876633, 0.0681346133351326, -0.10362762957811356, -0.045750223100185394, 0.01585369184613228, 0.09045843034982681, -0.01300861220806837, 0.0669708177447319, -0.10994293540716171, -0.21405453979969025, -0.2049243003129959, -0.05363772436976433, -0.016146356239914894, 0.019273564219474792, 0.42172348499298096, -0.03622693940997124, -0.1258630007505417, 0.07661769539117813, -0.053497832268476486, -0.1867988258600235, 0.029312364757061005, 0.12707474827766418, 0.2877429127693176, -0.02881956472992897, 0.19965821504592896, 0.07607454806566238, 0.07626089453697205, -0.004460304509848356, 0.03721868246793747, 0.09365347027778625, 0.010845143347978592, 0.06170499324798584, 0.07032588124275208, 0.12206897884607315, -0.1297692358493805, -0.1423368901014328, 0.05307518690824509, -0.07365184277296066, 0.15242192149162292, -0.031057795509696007, 0.11833325773477554, 0.08180714398622513, -0.09813801199197769, 0.04793257638812065, 0.2494448572397232, -0.050657402724027634, -0.28889232873916626, -0.12699124217033386, 0.0034033176489174366, 0.08177058398723602, -0.1816522628068924, -0.16516947746276855, -0.31499868631362915, 0.01591438427567482, 0.13576354086399078, 0.05679482966661453, 0.130063995718956, -0.016797583550214767, 0.15835745632648468, 0.07333008199930191, -0.07641565799713135, -0.1323792040348053, -0.11290256679058075, -0.046072423458099365}, {0.08068043738603592, -0.04398312792181969, 0.025006137788295746, 0.09472965449094772, -0.022709790617227554, 0.2077701985836029, 0.023837529122829437, -0.08828248083591461, 0.05472424998879433, -0.0042705596424639225, 0.03380423039197922, -0.02399151585996151, -0.07332290709018707, 0.07198165357112885, 0.08938033878803253, 0.010638732463121414, -0.0580173060297966, -0.005435416474938393, 0.1324031502008438, -0.033713094890117645, -0.04349169507622719, 0.038143448531627655, -0.0897122472524643, -0.03556162863969803, -0.04674263671040535, 0.08955136686563492, 0.13962729275226593, 0.006343713495880365, -0.24370205402374268, -0.03764118626713753, -0.5629488229751587, 0.1288926899433136, 0.5174959897994995, 0.1668553203344345, 0.15919923782348633, -0.043020885437726974, 0.024833574891090393, 0.005150225479155779, -0.09044355154037476, 0.12477345019578934, -0.006571675650775433, 0.04536468908190727, -0.013678314164280891, 0.08766403049230576, -0.03691377118229866, -0.15422292053699493, -0.16958336532115936, -0.01889622211456299, -0.14294065535068512, 0.011940522119402885, 0.06193312630057335, 0.13925068080425262, 0.07689382880926132, 0.0442662239074707, -0.029721178114414215, -0.05027080699801445, -0.08105365186929703, -0.12084744870662689, -0.023623166605830193, -0.15144968032836914, -0.1970497965812683, -0.07778531312942505, 0.25878649950027466, 0.2822013199329376, -0.12951456010341644, -0.14650623500347137, 0.08429044485092163, 0.23167076706886292, 0.026182614266872406, -0.05071748048067093, -0.037042077630758286, -0.048724398016929626, -0.5160057544708252, -0.1410418301820755, 0.16479061543941498, 0.09771877527236938, 0.09706512838602066, -0.007890944369137287, -0.0058164168149232864, -0.256887286901474, 0.06169775128364563, 0.10612501949071884, -0.04079210013151169, -0.22666671872138977, -0.05825130641460419, -0.23273903131484985, -0.3095383644104004, 0.10625534504652023, 0.032018739730119705, -0.35582777857780457, 0.07983356714248657, 0.05080241337418556, -0.02212933823466301, -0.1408170461654663, 0.04187958687543869, 0.05342397838830948, -0.05135995149612427, -0.05916345864534378, -0.016918901354074478, 0.04189109057188034, 0.003272848902270198, 0.20633357763290405, -0.09652926027774811, 0.07895606011152267, 0.7797796130180359, 0.1926877200603485, 0.13739371299743652, -0.05498804524540901, 0.02928107976913452, 0.008102132007479668, -0.033571790903806686, 0.048647090792655945, -0.031877659261226654, -0.06615237891674042, 0.05673312768340111, -0.036973144859075546, 0.02562776394188404, -0.0008958204416558146, -0.01815635710954666, -0.17101186513900757, -0.16905584931373596, -0.3097398281097412, -0.08324427902698517, -0.04610568284988403, -0.03159232810139656, 0.15503476560115814, -0.029037050902843475, 0.03256501629948616, 0.050854746252298355, -0.26040884852409363, -0.2165279984474182, -0.21465067565441132, 0.3941017985343933, -0.07075527310371399, -0.17732638120651245, -0.03117453306913376, 0.07570356130599976, -0.014985707588493824, 0.11961744725704193, -0.05754195526242256, -0.034397970885038376, 0.029747383669018745, 0.0664471760392189, 0.038608476519584656, 0.08893311768770218, 0.07710519433021545, -0.1625491827726364, 0.025195175781846046, 0.005268235225230455, 0.13864348828792572, -0.001901382813230157, -0.025860680267214775, -0.02610999345779419, 0.2175120860338211, -0.018354268744587898, 0.009846070781350136, 0.08484610170125961, 0.37305745482444763, 0.05936264619231224, -0.07769469916820526, -0.06403090804815292, 0.09276727586984634, -0.0053444853983819485, -0.010145386680960655, 0.04330186918377876, -0.02813868783414364, 0.24238625168800354, 0.0367438867688179, 0.38157984614372253, -0.18875114619731903, -0.010596520267426968, -0.0029452648013830185, -0.14384907484054565, -0.024608738720417023, -0.22290709614753723, 0.1541249305009842, 0.03711649030447006, -0.007232978008687496, -0.00684393011033535, 0.012804008089005947, 0.1412777304649353, -0.025138430297374725, -0.15069462358951569, -0.38144320249557495, 0.09067068248987198, 0.045967429876327515, 0.45409542322158813, 0.02141427807509899, -0.05433424934744835, -0.005245001520961523, 0.029339678585529327, 0.0746733620762825, 0.19211307168006897, 0.048127926886081696, -0.24979662895202637, -0.41430649161338806, 0.19088956713676453, -0.10001396387815475, 0.04633360356092453, 0.12052813172340393, -0.06814002245664597, 0.13066482543945312, -0.03724446892738342, 0.06538590788841248, 0.14552833139896393, -0.03092070482671261, 0.018917860463261604, -0.4510546326637268, -0.010839895345270634, 0.05245314538478851, 0.12746982276439667, 0.3429763913154602, 0.04773426055908203, 0.11120644211769104, -0.08793473243713379, 0.015799634158611298, -0.09957847744226456, -0.04559773951768875, -0.09833979606628418, 0.25123336911201477, -0.04070998355746269, 0.07827411592006683, 0.08316925913095474, 0.47143805027008057, 0.0038140220567584038, 0.40577244758605957, -0.07279906421899796, 0.14379730820655823, 0.03305395692586899, -0.16211947798728943, -0.00531548960134387, 0.020863289013504982, 0.30010148882865906, -0.003061396535485983, -0.13614070415496826, -0.07543502002954483, -0.1641843318939209, -0.035547465085983276, 0.07690726220607758, -0.03003520518541336, 0.07202725857496262, 0.21319031715393066, -0.04291202500462532, 0.0333845429122448, 0.02313055843114853, -0.21695317327976227, -0.017579413950443268, -0.17664557695388794, -0.0230939332395792, 0.17723138630390167, -0.0243424940854311, -0.06141756847500801, 0.10012637823820114, 0.05909692496061325, -0.21912860870361328, 0.07373648136854172}, {0.021425046026706696, 0.15451370179653168, 0.1900910884141922, -0.010500107891857624, 0.049414101988077164, 0.08480468392372131, 0.03840559720993042, 0.08010759949684143, 0.07820955663919449, 0.22775807976722717, -0.10027745366096497, 0.14264540374279022, -0.18060047924518585, -0.09537123143672943, 0.1515103131532669, 0.035966526716947556, -0.06950721144676208, -0.09624623507261276, -0.14862564206123352, 0.03343828395009041, -0.0518985316157341, 0.08373180031776428, -0.09002827107906342, 0.025449426844716072, -0.12526626884937286, 0.25257885456085205, -0.24434182047843933, 0.009951414540410042, -0.17268168926239014, 0.09420951455831528, 0.04661226645112038, 0.01644965261220932, 0.02876737341284752, -0.13806167244911194, 0.06446563452482224, 0.03428225591778755, 0.0014686849899590015, 0.05634743347764015, -0.10396314412355423, 0.061119139194488525, 0.059321071952581406, -0.07624516636133194, -0.1572418510913849, 0.1369735151529312, 0.05170649290084839, -0.0898229330778122, 0.015962760895490646, -0.18771801888942719, 0.047554269433021545, 0.13095034658908844, -0.2580726146697998, 0.10059428215026855, 0.014445251785218716, 0.01515574287623167, -0.03336029872298241, 0.02902277000248432, 0.04351193457841873, -0.26876115798950195, -0.04443434253334999, -0.0036993459798395634, 0.13715974986553192, -0.11610063910484314, 0.14332590997219086, -0.11332046985626221, -0.25583598017692566, 0.0010565468110144138, -0.062629334628582, -0.042145080864429474, 0.0901440903544426, -0.00992370955646038, 0.04982171580195427, 0.009817661717534065, -0.1331981122493744, -0.03127303346991539, 0.062056828290224075, -0.04442217946052551, 0.06961814314126968, 0.06768863648176193, 0.06240933761000633, -0.008392381481826305, -0.025491995736956596, 0.048528220504522324, 0.11253419518470764, 0.03550318256020546, -0.17458578944206238, -0.03450732305645943, 0.03987332433462143, -0.2536035478115082, 0.0570162795484066, -0.17292778193950653, 0.0966961458325386, -0.015129204839468002, -0.2008950114250183, -0.14076876640319824, 0.04233848303556442, 0.08565976470708847, 0.009924422949552536, -0.15041936933994293, 0.0063550532795488834, -0.28981825709342957, 0.049695275723934174, 0.15825669467449188, 0.0038727663923054934, -0.08230695128440857, -0.09427686035633087, -0.14297917485237122, -0.058152295649051666, 0.040863554924726486, 0.26251766085624695, -0.01100453082472086, 0.04327649250626564, 0.040996115654706955, -0.10923383384943008, -0.023661822080612183, 0.020394334569573402, 0.10179436951875687, -0.10881524533033371, -0.23381482064723969, -0.04543807730078697, 0.12080241739749908, -0.046689197421073914, -0.18480879068374634, 0.048459142446517944, -0.0320499949157238, -0.1720084249973297, -0.048652589321136475, -0.07188253849744797, -0.16836349666118622, 0.22011372447013855, 0.0015575042925775051, 0.14709098637104034, 0.10508035123348236, -0.016013799235224724, 0.09015991538763046, 0.0011844629188999534, -0.24846844375133514, 0.05171126499772072, 0.15713798999786377, -0.00950565468519926, 0.024929165840148926, 0.04552152380347252, -0.1540321409702301, 0.007940815761685371, -0.039228394627571106, -0.13462990522384644, -0.15299415588378906, 0.31196361780166626, 0.05087031051516533, 0.10827047377824783, 0.16469308733940125, -0.1702626347541809, 0.015183527022600174, -0.16846966743469238, 0.1790817826986313, -0.08450068533420563, -0.2350785881280899, -0.04078146070241928, 0.01756354235112667, 0.3076417148113251, -0.16251423954963684, 0.0036383976694196463, 0.1483711302280426, -0.16123966872692108, -0.25648143887519836, -0.0689794272184372, 0.10415709018707275, -0.09193079173564911, 0.11850207298994064, -0.1031220480799675, -0.047731827944517136, 0.08337443321943283, 0.026063265278935432, 0.07383120059967041, 0.11516644060611725, 0.06806769967079163, -0.17015135288238525, 0.019452039152383804, 0.06456796079874039, 0.06882400810718536, 0.008347315713763237, -0.17859600484371185, 0.05972421169281006, -0.1444052904844284, -0.03885627165436745, 0.14473049342632294, 0.17718839645385742, 0.1875286102294922, -0.10370642691850662, 0.0581667385995388, 0.21728400886058807, 0.055887218564748764, 0.21461166441440582, -0.005295787006616592, -0.01585630513727665, -0.13931289315223694, 0.10719131678342819, -0.019328434020280838, 0.24793598055839539, 0.07903897017240524, -0.17923817038536072, -0.10706476867198944, 0.05715429410338402, -0.27929410338401794, -0.09960882365703583, -0.0816388726234436, -0.0028621447272598743, 0.4245283901691437, 0.16220854222774506, 0.1794002652168274, -0.0451105460524559, 0.043717026710510254, -0.25634124875068665, -0.1691378951072693, 0.22729036211967468, 0.10346731543540955, 0.0014993264339864254, 0.07485013455152512, -0.008692895993590355, 0.013070591725409031, -0.042316604405641556, 0.10818490386009216, -0.16762012243270874, -0.19626940786838531, -0.09059686958789825, -0.028179986402392387, -0.032478466629981995, 0.05719152092933655, 0.0612042173743248, 0.015309314243495464, 0.031871553510427475, -0.14541488885879517, 0.023608433082699776, -0.20414678752422333, 0.12815290689468384, -0.09117018431425095, -0.03162655606865883, 0.15539740025997162, 0.026450134813785553, -0.22210414707660675, -0.42335203289985657, -0.11494329571723938, -0.12202665954828262, 0.0003519245656207204, 0.07642421126365662, 0.07973816990852356, -0.03619954362511635, -0.20371688902378082, 0.09032457321882248, 0.18905417621135712, -0.09998533874750137, 0.09580198675394058, -0.0032913554459810257, -0.00018759278464131057, 0.09169887751340866, 0.07251736521720886, 0.13934943079948425}, {-0.07356345653533936, -0.10691770166158676, -0.01268420834094286, -0.15606051683425903, -0.0889805257320404, -0.2011149376630783, -0.04480387270450592, 0.028135810047388077, -0.10762143135070801, 0.002781738992780447, 0.11912724375724792, 0.560524046421051, 0.031595759093761444, -0.055169228464365005, 0.17147603631019592, 0.25661224126815796, 0.046481940895318985, -0.08581940084695816, 0.21825118362903595, -0.0077613056637346745, 0.09164749085903168, 0.055787038058042526, 0.004126606974750757, -0.016843119636178017, -0.252734899520874, 0.03050459548830986, 0.05832298472523689, -0.0047445823438465595, 0.030395973473787308, 0.029620425775647163, 0.06414598971605301, -0.22489769756793976, 0.015867311507463455, -0.07057234644889832, -0.06754136085510254, 0.33053186535835266, 0.028344562277197838, -0.028428738936781883, 0.10213004797697067, -0.034196991473436356, 0.04275231808423996, 0.03270141780376434, 0.05853511765599251, -0.22032679617404938, -0.12372000515460968, -0.0848952978849411, -0.060600440949201584, -0.04408961161971092, -0.19221733510494232, -0.058346331119537354, 0.01876753568649292, -0.15422922372817993, -0.27652639150619507, 0.14086441695690155, 0.07284657657146454, 0.18649360537528992, 0.07015036791563034, -0.13026705384254456, 0.07170990109443665, 0.10564929246902466, -0.006272445432841778, -0.13671520352363586, 0.16117380559444427, 0.024704240262508392, 0.16107310354709625, 0.06211276352405548, 0.018267789855599403, 0.024744858965277672, 0.20844410359859467, 0.04225919768214226, -0.09950210899114609, 0.06926029920578003, 0.05961634963750839, -0.30894097685813904, 0.11679912358522415, -0.008085534907877445, -0.15902888774871826, -0.10632861405611038, 0.01759304851293564, -0.2947416305541992, 0.0038197189569473267, 0.10525821894407272, 0.09835480153560638, -0.057184308767318726, -0.08220863342285156, -0.07456016540527344, -0.1436600536108017, -0.12546135485172272, 0.032445501536130905, 0.1865447759628296, 0.08132808655500412, 0.03824494406580925, -0.16202470660209656, -0.19302549958229065, -0.03887109085917473, -0.08636821806430817, 0.10142602026462555, 0.19345426559448242, 0.14487600326538086, 0.2275146096944809, -0.12934663891792297, -0.11638306826353073, -0.25349950790405273, -0.048333678394556046, 0.015287184156477451, -0.10094831138849258, -0.15287582576274872, 0.030221423134207726, 0.21127833425998688, -0.2548552453517914, 0.013991856016218662, 0.10037704557180405, -0.09802793711423874, -0.09898286312818527, 0.0648290291428566, -0.10056674480438232, 0.11962217092514038, 0.013528551906347275, -0.006281946320086718, 0.10751181095838547, 0.11891689896583557, -0.019780553877353668, -0.1358492225408554, -0.102703757584095, 0.00023163293371908367, -0.12687301635742188, 0.0014259058516472578, -0.07990723103284836, 0.2554628551006317, 0.03811883181333542, -0.10902797430753708, 0.13934148848056793, -0.08337194472551346, -0.12048900127410889, 0.046160757541656494, 0.16921420395374298, 0.03647629916667938, -0.1403423696756363, 0.16941531002521515, -0.009854384697973728, -0.029503002762794495, -0.04789195582270622, -0.20223559439182281, -0.07478027790784836, -0.04099626839160919, -0.1402898132801056, -0.1328243762254715, 0.25704053044319153, -0.07860957086086273, 0.007301455829292536, 0.06186007335782051, -0.16168349981307983, 0.03558678179979324, -0.11515834182500839, -0.06252280622720718, -0.007167232688516378, -0.08722000569105148, 0.22446197271347046, 0.019332336261868477, -0.03965119272470474, 0.08166683465242386, -9.646296348364558e-06, 0.17144706845283508, 0.06946322321891785, 0.045362889766693115, -0.11362318694591522, -0.012822160497307777, -0.03459474444389343, 0.15059512853622437, -0.12787708640098572, 0.09222519397735596, -0.059760771691799164, 0.03431911766529083, 0.09414453059434891, 0.3497295081615448, 0.08699731528759003, -0.05244552344083786, -0.039744872599840164, 0.23443491756916046, 0.13416989147663116, -0.16731075942516327, 0.8128513693809509, -0.02848980948328972, 0.21268214285373688, -0.062377553433179855, 0.2752316892147064, -0.09894177317619324, 0.01605077087879181, 0.27227404713630676, -0.038631439208984375, -0.009181246161460876, 0.2916185259819031, -0.07257115840911865, 0.17832209169864655, 0.1577816605567932, -0.18450100719928741, 0.09446344524621964, 0.15840357542037964, -0.08954819291830063, -0.09287672489881516, 0.11466450989246368, 0.09655100852251053, 0.06615138798952103, 0.01628974638879299, 0.1853639781475067, 0.0003922560135833919, 0.03574850410223007, 0.2551085352897644, 0.1309262365102768, -0.07788420468568802, 0.13066411018371582, -0.21520881354808807, 0.17705731093883514, 0.1873726099729538, 0.0005532254581339657, 0.053708020597696304, -0.11320256441831589, 0.07381594926118851, -0.03770626336336136, -0.005182176362723112, 0.24930405616760254, -0.059875842183828354, -0.041160061955451965, -0.14835672080516815, -0.16179347038269043, -0.21871793270111084, 0.10896168649196625, -0.14991137385368347, -0.08935872465372086, -0.019319908693432808, -0.09210623800754547, -0.16667433083057404, -0.16189268231391907, 0.1293736845254898, 0.17854927480220795, -0.11725050956010818, -0.09125771373510361, 0.09027514606714249, -0.1554405242204666, -0.09300268441438675, -0.005101985763758421, 0.13660480082035065, 0.015318525023758411, -0.03324352204799652, 0.0007733560050837696, -0.03188598155975342, 0.029031630605459213, -0.05053827911615372, -0.055342935025691986, -0.11637763679027557, 0.1151343286037445, 0.03345978632569313, 0.03785930946469307, 0.02298104390501976, 0.15480875968933105, -0.09468413144350052}, {-0.17556831240653992, 0.01211783941835165, -0.027996240183711052, -0.1112770065665245, 0.02673286944627762, 0.1435343325138092, -0.09443328529596329, 0.051233064383268356, -0.10407533496618271, 0.1448645293712616, 0.03151710331439972, 0.05310790613293648, 0.1359746754169464, -0.06021248921751976, 0.042076706886291504, -0.11895495653152466, -0.020456532016396523, -0.09553233534097672, 0.1426745057106018, 0.004399093799293041, 0.03658553212881088, 0.009534147568047047, 0.22379663586616516, 0.13440530002117157, 7.684780575800687e-05, -0.5157940983772278, -0.2331685572862625, -0.05654082074761391, 0.13384050130844116, 0.11447833478450775, 0.09132831543684006, -0.30237483978271484, 0.11253687739372253, -0.2715264558792114, 0.06654590368270874, 0.16382786631584167, -0.03762482851743698, 0.030077001079916954, 0.1401623785495758, -0.06036783382296562, 0.038011349737644196, 0.09540221095085144, -0.028201688081026077, 0.1005677580833435, -0.07113289088010788, 0.04590675234794617, -0.09959638118743896, 0.21112437546253204, 0.08670952171087265, -0.09077905118465424, -0.17988039553165436, 0.04163522273302078, 0.18307851254940033, 0.012546364217996597, -0.08519530296325684, -0.12724614143371582, 0.15973098576068878, 0.08908869326114655, -0.029393309727311134, 0.22095727920532227, 0.17869435250759125, 0.08615735173225403, 0.12027861177921295, 0.16324937343597412, -0.029201006516814232, 0.3896256685256958, 0.2225697934627533, -0.12334569543600082, -0.05972202494740486, -0.20637722313404083, -0.01824771799147129, 0.25309836864471436, -0.27661120891571045, 0.02436574175953865, -0.33232125639915466, -0.17233087122440338, 0.018703775480389595, -0.0035951370373368263, 0.02249714359641075, -0.24299737811088562, 0.21401704847812653, -0.10781712085008621, -0.08096914738416672, -0.06811445206403732, 0.454351007938385, -0.32599467039108276, 0.18494848906993866, 0.007130899000912905, 0.11632668972015381, 0.009593752212822437, 0.07383554428815842, -0.1098940297961235, 0.02889876440167427, 0.06350625306367874, 0.06409032642841339, -0.03990833833813667, -0.03537900745868683, 0.051686521619558334, -0.22549819946289062, 0.034114059060811996, -0.05757156014442444, 0.1342185139656067, 0.006052170414477587, 0.17325960099697113, 0.39069047570228577, -0.08626112341880798, 0.19777269661426544, -0.10341689735651016, -0.27893730998039246, -0.04937596246600151, -0.04108644649386406, -0.1800779402256012, -0.06985139101743698, 0.09976986050605774, -0.1524503380060196, 0.08785513043403625, 0.10350604355335236, -0.173873171210289, 0.04499061033129692, -0.1435423046350479, 0.05701686069369316, -0.22449079155921936, -0.11345888674259186, 0.10969813913106918, 0.404063880443573, 0.09176737070083618, 0.11655296385288239, 0.03413247689604759, -0.13495776057243347, -0.25780752301216125, -0.12037204951047897, 0.20187677443027496, 0.262221097946167, -0.021158652380108833, -0.05821365863084793, -0.09559929370880127, -0.16673898696899414, -0.13863146305084229, -0.08049013465642929, 0.09039007872343063, 0.037481196224689484, -0.046721238642930984, -0.39931514859199524, 0.1475062370300293, 0.06561338156461716, -0.013882137835025787, 0.11173717677593231, -0.00488683907315135, -0.3524017035961151, -0.07320630550384521, -0.07357321679592133, 0.14004218578338623, -0.004977960139513016, 0.3111301064491272, 0.27022314071655273, -0.11965075135231018, 0.1039760410785675, 0.12989476323127747, -0.01883183978497982, -0.1663651168346405, -0.027590550482273102, 0.04281087592244148, 0.10432545095682144, -0.04397879168391228, -0.12972067296504974, 0.18140776455402374, 0.035982366651296616, -0.1979912668466568, -0.03986131772398949, 0.03351456671953201, -0.08053041994571686, -0.08808235079050064, 0.045674677938222885, 0.01037768367677927, -0.018061842769384384, 0.07488984614610672, 0.06939756870269775, 0.10035985708236694, -0.16481420397758484, 0.02315792255103588, 0.2231534868478775, 0.20305490493774414, -0.37079504132270813, -0.03183156996965408, -0.07651177793741226, 0.1311490386724472, 0.06268684566020966, 0.028861865401268005, -0.13803203403949738, -0.03679525852203369, 0.29647356271743774, 0.03934039548039436, -0.24148038029670715, 0.022705983370542526, 0.27107682824134827, 0.15457691252231598, -0.25838595628738403, 0.010113442316651344, 0.08278030902147293, -0.05594521015882492, 0.17405995726585388, -0.1765166074037552, -0.02836579829454422, -0.04429250210523605, -0.24027827382087708, 0.21458612382411957, 0.1856618970632553, 0.04503495246171951, -0.031284406781196594, 0.11439011245965958, -0.10318534821271896, -0.12199336290359497, -0.13726706802845, -0.0956621989607811, 0.2842123210430145, -0.011535284109413624, -0.008227361366152763, 0.14502999186515808, -0.29502469301223755, 0.020839622244238853, -0.2885085940361023, -0.11322594434022903, 0.10012996941804886, -0.0967465341091156, -0.01765192113816738, 0.01054011844098568, -0.058256011456251144, -0.02920909784734249, 0.05796850472688675, 0.018061814829707146, -0.03499651327729225, 0.051390860229730606, 0.010720059275627136, 0.12897416949272156, -0.21313633024692535, -0.24190503358840942, 0.1280258595943451, 0.005683739669620991, 0.031943920999765396, 0.12528516352176666, 0.2892661988735199, 0.10990062355995178, 0.1572132557630539, 0.032046083360910416, 0.08430493623018265, 0.02461947128176689, -0.0033214485738426447, 0.1240282654762268, -0.13370488584041595, -0.01614031381905079, 0.09487652033567429, 0.08580268919467926, 0.015040955506265163, -0.06632862240076065, 0.09053156524896622, -0.13413318991661072}, {-0.0913359746336937, -0.04055725783109665, 0.03201408311724663, -0.11289841681718826, -0.05744793266057968, 0.1048060804605484, -0.00020875167683698237, 0.13627199828624725, 0.24626833200454712, -0.11223243921995163, 0.1313658505678177, -0.05719290301203728, -0.03423471376299858, -0.17034976184368134, -0.05840148776769638, -0.0728897899389267, -0.0548994354903698, -0.05112651363015175, -0.07632256299257278, -0.12182342261075974, -0.3271843194961548, -0.032996244728565216, -0.09463972598314285, -0.04025949165225029, 0.03638290986418724, 0.09332287311553955, 0.04536807909607887, 0.24412687122821808, 0.28064435720443726, 0.13048437237739563, -0.12962861359119415, -0.05101272091269493, 0.14343668520450592, -0.036106351763010025, 0.12606488168239594, -0.011813442222774029, -0.118674635887146, 0.1708088517189026, 0.04832121729850769, 0.026975635439157486, 0.08520560711622238, 0.06675232946872711, -0.18625257909297943, -0.09817561507225037, -0.11794263124465942, 0.03184477612376213, -0.1418808400630951, -0.1436336487531662, 0.05173295736312866, -0.1529674530029297, -0.2902311384677887, -0.005666584242135286, -0.11140300333499908, 0.25843873620033264, -0.040098968893289566, 0.20493246614933014, -0.1447085291147232, 0.031083811074495316, -0.000748967460822314, 0.033464465290308, -0.07278972864151001, 0.08460597693920135, 0.10772597044706345, 0.07557034492492676, 0.03863966464996338, 0.158283069729805, 0.1582123339176178, -0.03557535260915756, -0.03721361607313156, -0.1111399307847023, 0.09443587809801102, 0.10056447982788086, -0.03022872656583786, -0.037888798862695694, -0.04247768595814705, -0.017800472676753998, -0.060433655977249146, 0.01244309451431036, 0.017398281022906303, -0.05999768152832985, -0.09787918627262115, -0.18757499754428864, -0.000494861276820302, 0.03533831238746643, 0.2286170870065689, -0.04296764358878136, -0.09741616249084473, 0.14455866813659668, -0.10924158245325089, 0.1685907244682312, -0.02834554761648178, -0.17040525376796722, -0.04167969897389412, -0.006408908404409885, -0.12969449162483215, -0.13860777020454407, -0.059796229004859924, 0.07326989620923996, 0.16975758969783783, -0.11366721242666245, 0.06958525627851486, 0.15910618007183075, 0.06428773701190948, -0.03958088159561157, 0.1832236498594284, -0.1354682892560959, 0.02424960397183895, 0.07494614273309708, 0.22641736268997192, -0.15047916769981384, 0.11863649636507034, 0.055786002427339554, 0.06592617183923721, 0.10749568790197372, -0.014838991686701775, -0.06464231759309769, 0.08833812177181244, -0.0173165462911129, -0.032448992133140564, -0.1104155108332634, 0.18152014911174774, 0.05363178625702858, -0.22162394225597382, 0.0019597657956182957, 0.04461060091853142, 0.0033922302536666393, 0.0440957210958004, -0.04954611882567406, -0.006075229961425066, 0.08749028295278549, -0.10583320260047913, 0.11191435158252716, -0.13050463795661926, 0.14078491926193237, 0.021384958177804947, -0.0840630978345871, -0.006236166227608919, -0.040100954473018646, -0.05983501672744751, 0.2673473358154297, -0.2679487466812134, -0.17678917944431305, -0.07989322394132614, 0.09203403443098068, -0.020098164677619934, -0.08176311105489731, -0.12235470861196518, 0.1004711464047432, 0.05005461722612381, 0.0692901760339737, 0.08817747235298157, -0.056953854858875275, 0.1372746080160141, 0.08078867942094803, 0.08087021112442017, -0.0574190691113472, -0.12910906970500946, 0.39559483528137207, 0.013490107841789722, 0.02001199871301651, -0.13766396045684814, 0.02461000718176365, 0.008416851051151752, -0.15930278599262238, 0.15337000787258148, -0.11835300177335739, -0.059373438358306885, -0.03248206526041031, -0.03366011008620262, -0.01968272589147091, 0.1594175547361374, 0.26099568605422974, -0.04209956154227257, 0.010926838032901287, -0.13840581476688385, -0.043210018426179886, -0.06134895607829094, 0.13961918652057648, 0.06401136517524719, -0.16268528997898102, -0.0611434131860733, 0.02508530765771866, -0.06794188916683197, -0.17480117082595825, -0.09690624475479126, -0.0744919627904892, 0.22762468457221985, -0.03505686670541763, 0.03980370983481407, 0.29851216077804565, -0.006143837235867977, 0.23644323647022247, -0.19552205502986908, -0.18596331775188446, -0.04519980773329735, 0.0036385376006364822, 0.11942857503890991, 0.10491162538528442, 0.11119658499956131, -0.0017534628277644515, 0.06335697323083878, 0.08352483063936234, -0.053664471954107285, -0.1124662384390831, -0.09072455763816833, 0.0695895254611969, -0.17907804250717163, 0.24257783591747284, 0.05623609572649002, 0.08664647489786148, 0.024190649390220642, 0.049127668142318726, 0.018772903829813004, 0.11147432029247284, -0.1942068189382553, 0.06914646178483963, 0.019568592309951782, 0.02479536272585392, -0.09391690790653229, 0.16052104532718658, -0.05189557373523712, -0.05542996525764465, -0.12619569897651672, 0.02680821157991886, 0.1670328825712204, 0.1727074533700943, -0.11281956732273102, 0.10675115138292313, 0.09179580211639404, 0.06582077592611313, 0.33822160959243774, 0.01466201338917017, -0.023334084078669548, -0.23104295134544373, 0.04584122821688652, -0.00800956692546606, -0.11550416797399521, -0.02060098387300968, -0.035573966801166534, -0.12216905504465103, -0.018569573760032654, -0.027890680357813835, -0.3236401081085205, -0.04014863073825836, 0.06110531464219093, 0.05394332855939865, 0.007363677024841309, -0.022313423454761505, 0.1564602255821228, -0.011832105927169323, 0.054377101361751556, 0.13282525539398193, -0.09060543775558472, 0.16856689751148224, -0.025920337066054344, 0.25419285893440247}, {-0.11677388101816177, -0.16179098188877106, -0.015733076259493828, 0.06379412114620209, 0.0085663553327322, 0.027388038113713264, -0.06973335146903992, -0.2776331901550293, -0.038988884538412094, 0.07375545054674149, -0.05848635360598564, -0.8438235521316528, -0.005783722270280123, 0.05881129205226898, 0.08620724081993103, -0.05576257407665253, -0.03858735039830208, -0.00858287513256073, -0.22452332079410553, 0.04284452274441719, -0.12343399971723557, -0.19963540136814117, 0.048270341008901596, -0.014107187278568745, 0.31101059913635254, 0.02316315285861492, 0.05859334021806717, 0.036052100360393524, 0.29062575101852417, 0.06522760540246964, -0.32451242208480835, 0.05506142973899841, 0.05967843905091286, 0.05624474957585335, -0.13433577120304108, -0.03381874039769173, -0.1686123162508011, -0.10878871381282806, 0.0751529186964035, -0.057909004390239716, -0.1519586741924286, 0.0010171065805479884, -0.08361580222845078, 0.06635267287492752, 0.13697071373462677, -0.1617049276828766, 0.078616201877594, 0.012666854076087475, 0.14026300609111786, -0.07216551899909973, -0.17778344452381134, 0.016428010538220406, 0.016003884375095367, 0.10152266174554825, -0.021425440907478333, -0.08578308671712875, -0.22690214216709137, 0.17445239424705505, -0.04925411939620972, -0.16016168892383575, -0.1336917281150818, 0.0814724713563919, -0.26506781578063965, 0.004738709423691034, -0.06327173113822937, -0.08206135779619217, -0.0035234324168413877, -0.1119859591126442, -0.15869764983654022, -0.10323838889598846, 0.18543244898319244, -0.06443151086568832, -0.10618570446968079, -0.13945738971233368, 0.0395682267844677, -0.004994775168597698, 0.1912507861852646, 0.03391508758068085, -0.12320482730865479, 0.23528513312339783, -0.021768787875771523, 0.01760644093155861, -0.02694724313914776, -0.07593028992414474, -0.11777620762586594, 0.14676980674266815, -0.014672630466520786, -0.01957973651587963, 0.0018382718553766608, -0.10257680714130402, 0.03248279541730881, 0.10996498167514801, -0.12710295617580414, 0.24656909704208374, 0.08305352181196213, 0.012647162191569805, -0.012942628934979439, 0.02321566455066204, -0.22829073667526245, -0.030421076342463493, -0.09147468954324722, 0.0016230359906330705, 0.3961881697177887, 0.05012070760130882, 0.157134547829628, -0.000550846743863076, 0.10288254916667938, 0.06075878068804741, 0.08773569762706757, 0.18280857801437378, -0.12888430058956146, -0.06387226283550262, 0.07499374449253082, 0.12743324041366577, -0.19882091879844666, -0.010663660243153572, 0.11014940589666367, 0.20837655663490295, 0.07129328697919846, -0.3891330361366272, -0.1103859394788742, -0.030662165954709053, 0.0009102858020924032, 0.15199261903762817, -0.06005777046084404, -0.10130472481250763, 0.0063662040047347546, 0.030417300760746002, 0.11939192563295364, 0.057150453329086304, -0.23037594556808472, -0.11406896263360977, 0.28751713037490845, 0.10476608574390411, -0.12140440940856934, 0.17201390862464905, 0.042847175151109695, -0.09728286415338516, 0.20120516419410706, -0.037901002913713455, -0.048025134950876236, 0.07528232038021088, 0.035704899579286575, 0.13297779858112335, -0.006511485669761896, -0.05319911241531372, 0.02655806392431259, -0.002547775162383914, 0.15127338469028473, -0.05426983907818794, -0.04608233645558357, -0.02447846345603466, -0.06379237771034241, -0.13602621853351593, -0.08829167485237122, -0.056521449238061905, 0.3285626769065857, -0.11590726673603058, -0.18222454190254211, 0.12824858725070953, 0.05091744661331177, 0.01526093203574419, 0.10492046922445297, 0.11034601181745529, -0.026929238811135292, 0.10251837223768234, 0.03602197766304016, 0.04476708546280861, 0.1445806622505188, 0.02709244377911091, 0.020912569016218185, 0.057813528925180435, -0.034415390342473984, 0.10553484410047531, -0.1755143702030182, -0.013797304593026638, -0.14266689121723175, 0.06779785454273224, -0.06191173568367958, -0.012248999439179897, 0.10523518919944763, -1.1597250699996948, -0.03874649479985237, -0.0476367212831974, 0.005246794316917658, -0.12434917688369751, 0.03638945519924164, -0.12672606110572815, -0.09542737901210785, 0.1980435997247696, 0.013924787752330303, 0.015581628307700157, 0.07582630217075348, -0.2787341773509979, -0.05237501487135887, 0.04983711242675781, -0.07360353320837021, -0.13475026190280914, 0.09278632700443268, 0.13269975781440735, 0.09211008250713348, 0.19035564363002777, 0.010516048409044743, 0.036100123077631, -0.023615825921297073, -0.0869571715593338, 0.12661562860012054, -0.27142220735549927, -0.14760765433311462, -0.042289841920137405, 0.01735740900039673, 0.12604688107967377, 0.23619601130485535, 0.07571261376142502, -0.0795159861445427, 0.0893407016992569, -0.04423664137721062, 0.00219808891415596, 0.11340872943401337, -0.027296239510178566, 0.07820148020982742, 0.041860032826662064, 0.09916787594556808, 0.256031334400177, -0.019599147140979767, 0.04352728649973869, 0.09436438232660294, -0.013561150059103966, 0.09179055690765381, 0.3172779679298401, 0.0512705072760582, 0.07714922726154327, 0.0691215768456459, -0.07770482450723648, -0.0890379250049591, -0.037353478372097015, 0.0027918871492147446, 0.05403370410203934, -0.0007121728267520666, 0.04517948627471924, 0.1770770400762558, 0.01595488376915455, -0.2552473247051239, 0.10633096098899841, -0.014906712807714939, -0.06305339187383652, 0.02852942980825901, 0.006892672739923, 0.07282819598913193, 0.055847518146038055, -0.12999020516872406, -0.0016918671317398548, 0.05871138721704483, 0.01236318051815033, 0.07721161842346191, 0.1827792525291443}, {0.0306364968419075, 0.035841938108205795, 0.12597569823265076, -0.07285899668931961, -0.05494356527924538, 0.06616824120283127, 0.3416564464569092, -0.4324008524417877, 0.18653351068496704, 0.145488902926445, -0.2569146752357483, 0.0239793062210083, -0.18225045502185822, -0.011117258109152317, 0.1336759328842163, 0.012238742783665657, 0.002755142282694578, 0.02744007296860218, -0.04569842666387558, 0.01965792290866375, -0.12377970665693283, -0.034903429448604584, -0.1783733367919922, -0.13148440420627594, -0.28433364629745483, 0.40106186270713806, 0.2979552149772644, 0.22765709459781647, -0.1900421530008316, 0.16842491924762726, -0.1905762106180191, 0.29355618357658386, 0.020293302834033966, -0.018679538741707802, 0.03520311415195465, 0.07813403010368347, 0.1002049669623375, 0.168650284409523, -0.05270727351307869, -0.24785268306732178, 0.005431830883026123, 0.09035317599773407, -0.050290290266275406, 0.13644978404045105, 0.44323447346687317, 0.05245660990476608, 0.16235195100307465, -0.16004914045333862, -0.16688565909862518, 0.10198473185300827, -0.050798188894987106, -0.018764106556773186, 0.1117294579744339, 0.2702060341835022, -0.23858800530433655, 0.3294025957584381, -0.09136442840099335, 0.16604197025299072, -0.09312823414802551, -0.35947877168655396, -0.21139346063137054, -0.17029733955860138, -0.0801200196146965, -0.13143138587474823, 0.16897733509540558, -0.33316993713378906, -0.4210958182811737, 0.21516041457653046, 0.10819897800683975, 0.47896936535835266, 0.3103252053260803, -0.4699845612049103, -0.295116662979126, -0.231073796749115, 0.1437053680419922, 0.0027672320138663054, 0.12849590182304382, -0.022530486807227135, 0.03860640525817871, -0.15064139664173126, -0.025820229202508926, 0.17863403260707855, 0.1096741259098053, 0.3721262514591217, -0.38210710883140564, 0.3373710811138153, 0.06010821461677551, -0.43110063672065735, -0.4082361161708832, -0.13258978724479675, -0.21723715960979462, -0.05126678943634033, -0.26561203598976135, -0.034882623702287674, -0.12546657025814056, 0.09867437928915024, 0.08465184271335602, -0.22526545822620392, 0.19096742570400238, -0.14772626757621765, 0.1055535227060318, 0.022453265264630318, -0.07201001793146133, -0.2809247672557831, -0.10516879707574844, -0.15485845506191254, 0.14245422184467316, 0.27785471081733704, -0.09008464217185974, -0.1062403991818428, 0.28637003898620605, -0.27269700169563293, 0.36606475710868835, -0.2267003059387207, 0.16888739168643951, -0.0984979048371315, -0.003268195316195488, 0.18286743760108948, 0.15984131395816803, 0.3196008503437042, 0.13800014555454254, -0.1344069242477417, 0.1281912624835968, 0.10774625092744827, 0.14732787013053894, 0.10592412948608398, -0.19219543039798737, 0.01971944235265255, 0.5046663880348206, 0.4213670790195465, 0.21288421750068665, -0.4527043402194977, 0.4323665499687195, 0.06889957189559937, -0.013178405351936817, -0.20143447816371918, 0.1701984703540802, -0.010984829626977444, 0.04887494444847107, -0.3524129092693329, 0.061853956431150436, 0.1481356918811798, 0.4624558985233307, 0.06624554842710495, 0.17553265392780304, 0.03707806393504143, 0.17872615158557892, 0.053883686661720276, -0.2439812868833542, 0.13485102355480194, 0.24842560291290283, -0.045668717473745346, -0.2892164885997772, -0.2162592113018036, -0.17116199433803558, -0.08345553278923035, -0.09818270057439804, -0.09579245746135712, 0.16970962285995483, 0.6784202456474304, -0.06544522196054459, 0.2691994309425354, 0.11990617215633392, -0.20606686174869537, 0.2104780226945877, -0.08421651273965836, -0.01553631853312254, 0.2692926526069641, 0.08088061213493347, -0.28683745861053467, -0.0017567456234246492, 0.04736346751451492, -0.19593636691570282, 0.147002175450325, -0.16564132273197174, 0.06330238282680511, -0.10805895924568176, 0.13283535838127136, 0.05844900384545326, 0.13979288935661316, -0.12078695744276047, -0.5534291863441467, 0.278657466173172, 0.0734936073422432, 0.023907436057925224, -0.23467284440994263, -0.2046200931072235, -0.09235439449548721, -0.21651442348957062, 0.12828846275806427, -0.20344460010528564, 0.07581007480621338, 0.17844019830226898, -0.17258433997631073, -0.1506698876619339, 0.1460265815258026, 0.06517595052719116, -0.01957649365067482, 0.07765045762062073, 0.23129865527153015, -0.01093803346157074, 0.22621959447860718, 0.04030229151248932, 0.16848333179950714, 0.2523878216743469, -0.13410669565200806, -0.3039167821407318, -0.3178447484970093, 0.08618902415037155, 0.07809426635503769, -0.46596482396125793, 0.15567408502101898, -0.0012006922625005245, 0.10163477808237076, -0.11807473003864288, -0.1856253445148468, 0.1705186665058136, -0.11369621753692627, 0.07867713272571564, 0.20027324557304382, 0.1372728943824768, 0.2413797825574875, 0.06767686456441879, 0.057989370077848434, -0.10489067435264587, 0.3683830797672272, -0.2993539869785309, 0.4741903841495514, 0.12054518610239029, 0.08806454390287399, -0.0943964496254921, 0.35416853427886963, 0.04802071675658226, -0.22590479254722595, -0.04698216915130615, 0.29096731543540955, -0.13478775322437286, 0.13061915338039398, 0.22146451473236084, 0.020651675760746002, -0.05445588007569313, 0.14577771723270416, -0.29998263716697693, 0.010221173986792564, -0.17736801505088806, 0.046742994338274, -0.1552518606185913, 0.13877752423286438, 0.21491043269634247, 0.032792042940855026, 0.02237214706838131, -0.3886511027812958, -0.06618783622980118, -0.1080562174320221, -0.11265529692173004, 0.055808525532484055}, {0.016148606315255165, -0.028248537331819534, -0.20420701801776886, 0.10144621878862381, -0.0797581598162651, -0.13130712509155273, 0.13381588459014893, 0.061672355979681015, -0.13782085478305817, -0.35938510298728943, -0.06797397881746292, -0.11422833055257797, -0.10024015605449677, -0.06565699726343155, -0.24373707175254822, -0.09611255675554276, -0.12441467493772507, 0.11571695655584335, 0.24935677647590637, 0.01059708371758461, -0.33350613713264465, -0.05020293965935707, -0.046356841921806335, -0.10943219065666199, -0.1480792611837387, 0.10825897753238678, 0.19194447994232178, 0.272467702627182, -0.04237481579184532, 0.07207144051790237, 0.17622433602809906, 0.29638949036598206, -0.09583060443401337, 0.2483658790588379, -0.028392469510436058, -0.19313833117485046, 0.0003347209421917796, -0.041899047791957855, 0.21797920763492584, -0.2029138058423996, 0.05699355527758598, 0.010261861607432365, -0.1106957197189331, 0.039247337728738785, 0.06481281667947769, -0.11106578260660172, 0.02381046861410141, 0.19999495148658752, 0.011892805807292461, -0.07741158455610275, -0.07811671495437622, 0.41824057698249817, -0.16620224714279175, -0.16056054830551147, 0.011166735552251339, 0.15460939705371857, 0.029514145106077194, -0.035318270325660706, 0.06752445548772812, -0.09794643521308899, -0.0977727472782135, 0.20845824480056763, 0.11882273852825165, -0.0363554023206234, 0.0006454631220549345, -0.23079730570316315, -0.026303725317120552, 0.20255476236343384, -0.10973116010427475, 0.05412561818957329, 0.02915400266647339, -0.05374465882778168, 0.20389999449253082, -0.005011457484215498, 0.203628808259964, 0.11749699711799622, -0.0921732485294342, 0.19832302629947662, 0.45619887113571167, -0.05663171783089638, -0.16183768212795258, -0.11219140887260437, 0.012099827639758587, -0.10780278593301773, -0.17383380234241486, 0.2681910991668701, -0.14174072444438934, -0.19697441160678864, -0.08834729343652725, 0.0270222295075655, 0.0001680878922343254, 0.1920154094696045, 0.08768051117658615, -0.17425282299518585, -0.051163628697395325, -0.22838051617145538, 0.09748610854148865, -0.024247555062174797, 0.034060876816511154, -0.12514542043209076, -0.14816658198833466, -0.04246090352535248, 0.04520782083272934, -0.013972600921988487, -0.11788257211446762, 0.2592063546180725, -0.13561688363552094, -0.04568161815404892, 0.22867831587791443, -0.05347376689314842, 0.10959652066230774, -0.10841241478919983, 0.25211411714553833, 0.1324923038482666, -0.006934735458344221, -0.12998734414577484, -0.13873368501663208, 0.2438335120677948, 0.000267756637185812, 0.0012148205423727632, -0.04691757634282112, -0.051439229398965836, -0.043906472623348236, 0.00023705365310888737, 0.08694722503423691, -0.16486495733261108, 0.07457728683948517, -0.1754392683506012, 0.24626010656356812, 0.1517898142337799, -0.14920347929000854, -0.004532426130026579, -0.12635083496570587, 0.16826525330543518, 0.04598941653966904, 0.026012659072875977, -0.0866849422454834, -0.07576078921556473, -0.011082978919148445, -0.3518177270889282, 0.05729720741510391, -0.03489924594759941, 0.3243766129016876, -0.0058747269213199615, -0.24080036580562592, 0.052180759608745575, -0.0673074945807457, 0.11955787241458893, -0.13661986589431763, 0.054847296327352524, -0.06703027337789536, 0.03581665828824043, -0.07447560876607895, -0.22354359924793243, -0.1709304004907608, -0.04730668291449547, -0.07048622518777847, 0.2081478387117386, -0.08791901916265488, 0.2217201441526413, -0.24987785518169403, 0.08899880200624466, 0.02580356039106846, -0.053449518978595734, 0.06320220977067947, 0.08691130578517914, 0.23257751762866974, 0.08409549295902252, 0.2038045972585678, -0.22488567233085632, 0.10810961574316025, 0.034339699894189835, 0.18939386308193207, -0.2564582824707031, -0.0812920406460762, -0.022900067269802094, 0.0936092734336853, -0.17262159287929535, 0.029724927619099617, -0.16484417021274567, -0.0834595113992691, 0.006051445379853249, 0.12490840256214142, -0.08200877159833908, 0.14418423175811768, -0.11371801048517227, 0.02883310243487358, 0.006542597431689501, -0.07686258852481842, -0.13654817640781403, -0.07664719223976135, 0.11645658314228058, -0.06594464182853699, -0.1008385643362999, -0.15423978865146637, -0.08424760401248932, 0.05967259779572487, 0.11951051652431488, -0.16848760843276978, 0.16926033794879913, 0.041468407958745956, 0.21635162830352783, 0.32536837458610535, 0.07487881183624268, 0.08816968649625778, 0.13299435377120972, -0.26634034514427185, -0.029778048396110535, -0.037580739706754684, -0.03109479323029518, 0.01409525703638792, 0.21158790588378906, 0.08660010248422623, 0.0451124906539917, -0.00013272480282466859, -0.24509930610656738, -0.10448046773672104, -0.07940345257520676, -0.16216515004634857, -0.0527777373790741, -0.11767900735139847, 0.059838324785232544, -0.18456460535526276, 0.13889266550540924, -0.18142220377922058, 0.14777140319347382, -0.037414781749248505, 0.025620702654123306, 0.08145248889923096, -0.016946397721767426, 0.2041616141796112, 0.03551561012864113, -0.10354262590408325, -0.2369227409362793, 0.04435141384601593, -0.24758855998516083, -0.132740318775177, 0.054180461913347244, 0.173920139670372, 0.2543308138847351, 0.005471955519169569, 0.14025627076625824, -0.2560270130634308, -0.04909052699804306, 0.1805901974439621, -0.06249184161424637, 0.06723461300134659, -0.14690500497817993, -0.0480068139731884, -0.039080481976270676, -0.12184538692235947, -0.07932756841182709, -0.011809430085122585, -0.17407359182834625, -0.14080753922462463, -0.24802690744400024}, {0.08761513233184814, 0.07889988273382187, 0.20243370532989502, 0.04094889014959335, 0.24345748126506805, 0.007227574475109577, -0.006469144020229578, -0.009253166615962982, 0.09677539765834808, -0.043425559997558594, 0.14630389213562012, -0.04885634779930115, -0.05014300346374512, -0.10698498785495758, -0.13182583451271057, -0.19111302495002747, 0.0823897197842598, -0.2759254276752472, -0.23824404180049896, -0.07572788745164871, -0.08466153591871262, 0.04722273349761963, -0.06166567653417587, 0.0354023240506649, 0.10942413657903671, -0.12404362112283707, -0.03198912739753723, 0.051724500954151154, -0.061281442642211914, 0.056898463517427444, 0.10199885815382004, -0.14848768711090088, 0.20686040818691254, -0.08685644716024399, 0.4653971791267395, -0.02438148483633995, 0.19519183039665222, 0.06283805519342422, 0.1865447610616684, 0.15652446448802948, 0.03597376123070717, 0.0522419810295105, 0.15641047060489655, 0.04086994007229805, 0.6006156206130981, 0.14632894098758698, 0.040570151060819626, -0.23692911863327026, 0.016369599848985672, 0.00912463292479515, 0.06947105377912521, -0.00910215638577938, 0.00585375027731061, 0.08452373743057251, -0.09708435088396072, 0.06430225819349289, -0.01825747825205326, -0.11638367921113968, 0.02544039487838745, -0.02450060099363327, -0.0053077926859259605, 0.07932837307453156, -0.32867977023124695, 0.025461925193667412, -0.2902905344963074, 0.08606217056512833, -0.1292266845703125, 0.0570133700966835, 0.2466418445110321, 0.006792874541133642, 0.11420074105262756, 0.07666903734207153, -0.19860094785690308, 0.17594996094703674, 0.13091929256916046, 0.10234759002923965, -0.19016744196414948, -0.1779032200574875, -0.04310210049152374, 0.23928727209568024, 0.19098936021327972, -0.13890241086483002, -0.06440933048725128, -0.10638180375099182, -0.0022532474249601364, 0.041655994951725006, -0.004611435811966658, 0.0619770884513855, -0.1211506649851799, 0.17440861463546753, -0.12402799725532532, -0.09448032826185226, 0.09414499998092651, -0.03511828929185867, -0.19627532362937927, -0.01124661322683096, -0.1507759839296341, 0.232621431350708, -0.10197556018829346, -0.1097172200679779, -0.03559091314673424, -0.015740325674414635, 0.1332024186849594, -0.05424942821264267, 0.255026638507843, -0.21078738570213318, 0.14265453815460205, -0.10310718417167664, 0.07771823555231094, 0.06222781538963318, 0.12447751313447952, 0.04515736177563667, -0.0093353521078825, -0.021888885647058487, 0.2815895080566406, 0.06156439334154129, -0.11851254105567932, -0.05144169181585312, 0.14118286967277527, 0.04533626139163971, -0.06242569163441658, -0.2813573181629181, 0.007945891469717026, 0.04744246229529381, 0.151372030377388, -0.09760962426662445, -0.02289876714348793, 0.21661561727523804, 0.09122839570045471, 0.0790148451924324, 0.008174455724656582, 0.0924842581152916, 0.043833691626787186, -0.03705460950732231, -0.11758723855018616, 0.10369830578565598, 0.15067075192928314, 0.23860837519168854, -0.1217603012919426, 0.2978452444076538, 0.11486329138278961, 0.023872708901762962, -0.005864166188985109, -0.2098308950662613, -0.07480521500110626, 0.10992582887411118, 0.07323729991912842, -0.09998873621225357, -0.14774593710899353, 0.24415212869644165, 0.020200714468955994, -0.13053672015666962, -0.024237731471657753, -0.15005731582641602, 0.10495474934577942, 0.00620532501488924, -0.07332363724708557, -0.18135368824005127, -0.3269056975841522, 0.01873597875237465, -0.0003244789259042591, 0.008517521433532238, -0.21560350060462952, -0.14804306626319885, -0.026221584528684616, -0.05482027307152748, -0.2113553285598755, -0.10211392492055893, -0.036526620388031006, 0.1957651972770691, -0.09438387304544449, 0.24390558898448944, -0.005364542827010155, 0.043624091893434525, 0.04303644225001335, -0.12285280227661133, 0.08283849060535431, 0.040908798575401306, 0.01461431197822094, -0.0483119897544384, 0.022150922566652298, -0.3508337736129761, 0.03181598708033562, -0.015140360221266747, 0.35156479477882385, -0.08102358877658844, -0.1089756041765213, -0.11296511441469193, -0.02885916270315647, -0.06002325937151909, 0.13739988207817078, -0.0736987367272377, 0.142544686794281, 0.07742927968502045, 0.3469821810722351, -0.18700841069221497, -0.05497923865914345, -0.2043834924697876, -0.0669945776462555, 0.06701484322547913, -0.13799838721752167, -0.052824221551418304, 0.039532627910375595, -0.23759102821350098, 0.09216578304767609, -0.20437109470367432, -0.0340690016746521, 0.11359041184186935, 0.1227383092045784, -0.13975638151168823, -0.1159772276878357, 0.021320557221770287, -0.22117675840854645, 0.3746006190776825, 0.11288374662399292, 0.08720103651285172, 0.3067464828491211, 0.16866719722747803, 0.06783533841371536, 0.09669586271047592, -0.19274340569972992, -0.02162996493279934, -0.04242166131734848, -0.04093870148062706, 0.15048842132091522, -0.09283234924077988, -0.18294386565685272, 0.037692416459321976, -0.18866141140460968, 0.02018800750374794, 0.14606286585330963, -0.09241972863674164, 0.07052329927682877, 0.03654535859823227, 0.18217109143733978, -0.2133519947528839, 0.11287575215101242, 0.14546073973178864, -0.17200128734111786, -0.0363866426050663, 0.07952050864696503, -0.10615573823451996, -0.02940019778907299, 0.2252373993396759, -0.0016180140664801002, -0.08732280880212784, 0.013149156235158443, 0.032606903463602066, -0.1332831233739853, -0.08086812496185303, -0.006750107742846012, 0.06477073580026627, -0.1398339867591858, -0.1767330765724182, -0.037345778197050095, -0.10044461488723755}, {0.04607388749718666, 0.2382047176361084, -0.2464088350534439, -0.04711345583200455, -0.1502513587474823, 0.06413103640079498, -0.12037862092256546, 0.134698286652565, 0.02131836488842964, -0.1868615299463272, 0.18979959189891815, 0.05358946695923805, -0.10238607227802277, 0.08770549297332764, 0.2928386330604553, -0.050640206784009933, -0.23679518699645996, 0.060052305459976196, 0.15274471044540405, -0.13673575222492218, 0.2425571233034134, 0.2593717575073242, 0.17175757884979248, -0.09933214634656906, -0.21803171932697296, 0.08308187127113342, 0.3777281641960144, 0.09057410061359406, 0.09843800961971283, 0.24620425701141357, -0.2298002690076828, -0.05708363279700279, -0.3085748553276062, -0.060114260762929916, 0.018575789406895638, 0.16359855234622955, 0.053076375275850296, -0.11833862215280533, -0.030622821301221848, -0.12403696775436401, 0.2824450433254242, -0.09105820208787918, 0.04948526620864868, 0.2514738440513611, -0.17836534976959229, 0.08915869146585464, 0.07151903212070465, -0.05582449957728386, 0.057729825377464294, 0.09182178229093552, -0.2559817433357239, 0.10303325206041336, 0.05402631685137749, -0.11735769361257553, 0.054831959307193756, 0.12517622113227844, 0.13554425537586212, -0.10894817113876343, 0.13612490892410278, -0.31385645270347595, 0.05136600136756897, 0.1232495903968811, 0.000629638263490051, -0.04214305058121681, 0.16037040948867798, -0.05194748565554619, 0.19679410755634308, -0.01867811754345894, -0.05860835313796997, 0.16005796194076538, 0.0549333393573761, 0.11243492364883423, -0.11366600543260574, -0.16169165074825287, 0.16868099570274353, -0.2155809849500656, 0.06426691263914108, 0.372691810131073, -0.22113822400569916, -0.028052350506186485, -0.028520923107862473, 0.15360064804553986, 0.05762655660510063, 0.20689482986927032, 0.04600140452384949, -0.012802710756659508, -0.1465405970811844, -0.07084627449512482, 0.13294103741645813, 0.26821866631507874, 0.31808337569236755, -0.07620953023433685, -0.08636072278022766, -0.017885342240333557, -0.004735858645290136, -0.057976625859737396, -0.016777461394667625, -0.05457431823015213, 0.06387489289045334, 0.32695573568344116, 0.036962274461984634, -0.08521278202533722, 0.11650198698043823, 0.046511463820934296, -0.15854549407958984, 0.5484596490859985, -0.26963427662849426, -0.039461079984903336, 0.056942276656627655, -0.019239265471696854, 0.10205195844173431, -0.15274108946323395, 0.16547104716300964, -0.1071244403719902, 0.017132233828306198, -0.019914118573069572, -0.011629790998995304, 0.017255496233701706, -0.10515376925468445, -0.20047889649868011, -0.06897979229688644, 0.08326544612646103, 0.007571158930659294, 0.14832042157649994, 0.21408218145370483, 0.4520827829837799, -0.11715859919786453, -0.05756247043609619, -0.021011264994740486, 0.03726133331656456, -0.21600398421287537, -0.10502259433269501, 0.2276400625705719, 0.13588036596775055, 0.15002521872520447, -0.14960108697414398, -0.0005514058284461498, 0.18265120685100555, -0.09214535355567932, 0.029302343726158142, 0.1723761260509491, 0.13726122677326202, 0.002489052712917328, -0.035199880599975586, -0.023434635251760483, -0.1913772076368332, -0.1630416363477707, 0.14769825339317322, -0.07091853022575378, -0.27996110916137695, -0.05305476114153862, 0.20618893206119537, 0.10162851214408875, -0.2323736846446991, 0.07901208847761154, 0.010322021320462227, -0.22016222774982452, 0.34491202235221863, 0.16092965006828308, -0.19626964628696442, 0.026740530505776405, -0.007301460951566696, -0.19548676908016205, 0.15145482122898102, -0.1263592392206192, -0.0031505392398685217, -0.13251641392707825, 0.08279010653495789, 0.022709228098392487, 0.18112340569496155, 0.022159045562148094, -0.1722346693277359, 0.07099877297878265, 0.010469017550349236, 0.37859678268432617, -0.17765989899635315, -0.09808715432882309, -0.1911056786775589, -0.24534623324871063, 0.21124079823493958, -0.03887127339839935, 0.08974158763885498, -0.2657013237476349, 0.3328726291656494, 0.2861480712890625, 0.13450129330158234, -0.3945591151714325, -0.22362473607063293, 0.16770975291728973, -0.12085175514221191, 0.04642436280846596, -0.04756809398531914, 0.0471675843000412, -0.12947773933410645, -0.31102362275123596, -0.07383210957050323, -0.09373405575752258, -0.0572170689702034, -0.20033447444438934, 0.09492196142673492, 0.37104353308677673, -0.11252481490373611, 0.16210296750068665, 0.08539159595966339, 0.030686676502227783, 0.1639729142189026, 0.04858674481511116, -0.13070781528949738, -0.2622518837451935, 0.1421346217393875, 0.06425780802965164, -0.3672609031200409, 0.10012884438037872, -0.3343955874443054, 0.20254595577716827, -0.35952863097190857, -0.12431791424751282, -0.24721525609493256, -0.31573233008384705, 0.13767129182815552, -0.028110764920711517, -0.2471381276845932, 0.040897320955991745, -0.3741662800312042, -0.25497257709503174, -0.40475553274154663, -0.10199591517448425, -0.1256253719329834, 0.01867164671421051, 0.07476773113012314, 0.003249366069212556, 0.22768180072307587, -0.1891077607870102, -0.08276369422674179, 0.014354648999869823, 0.16598650813102722, 0.046817801892757416, -0.12007398158311844, -0.06574270874261856, 0.26859551668167114, 0.25852638483047485, 0.05916620418429375, 0.0020542507991194725, -0.2404826581478119, 0.048711828887462616, 0.08286328613758087, 0.06283138692378998, -0.10188861191272736, -0.18837499618530273, -0.20301564037799835, 0.08106835186481476, 0.006335278041660786, -0.030788350850343704, -0.10100378096103668, -0.13617944717407227, 0.0484393946826458}, {0.29194560647010803, -0.058508340269327164, 0.1678132861852646, -0.18491406738758087, 0.20144401490688324, 0.1783578246831894, 0.0170112494379282, 0.16334447264671326, 0.12637954950332642, 0.279130220413208, -0.07303913682699203, -0.17435190081596375, 0.03721960261464119, 0.0871196985244751, -0.08397424966096878, -0.14911215007305145, -0.0383523553609848, 0.06990494579076767, -0.08886003494262695, -0.056820157915353775, -0.1175738126039505, 0.1187230795621872, -0.05397295579314232, -0.05466951057314873, 0.14162971079349518, -0.18544702231884003, -0.07788830995559692, -0.11156968027353287, -0.12049762159585953, -0.12222085893154144, -0.09218128025531769, -0.23556865751743317, -0.03392396494746208, 0.004553454928100109, 0.10389973223209381, -0.05098048597574234, -0.14040394127368927, -0.02536153607070446, -0.07141824066638947, 0.03137131780385971, -0.25523093342781067, 0.20642247796058655, -0.06293757259845734, -0.014474908821284771, 0.17163094878196716, -0.07065790146589279, 0.1140323355793953, -0.18824879825115204, -0.06503019481897354, -0.03998558223247528, 0.003138289088383317, -0.23285038769245148, 0.22815895080566406, -0.020294567570090294, -0.0895669013261795, 0.04353584349155426, -0.14850831031799316, -0.0018733812030404806, -0.039351873099803925, 0.09291095286607742, 0.20879662036895752, 0.11011534929275513, -0.16777536273002625, -0.1541018784046173, -0.09987179189920425, 0.20431286096572876, 0.05555659532546997, -0.06976892054080963, -0.12115852534770966, -0.06197120249271393, -0.04078681766986847, -0.03399479389190674, 0.005116162821650505, 0.10365375131368637, 0.057285651564598083, -0.049558937549591064, 0.03631336987018585, 0.0459771528840065, -0.19591891765594482, 0.16061316430568695, 0.10102630406618118, 0.019233355298638344, 0.041632167994976044, 0.055857814848423004, 0.11533766239881516, 0.09267215430736542, 0.06762145459651947, -0.03068486414849758, -0.22767166793346405, -0.00932207889854908, -0.007133333012461662, 0.17997263371944427, 0.2324429154396057, -0.12902091443538666, -0.23117098212242126, -0.15217174589633942, -0.08580143004655838, 0.13393037021160126, -0.0863146260380745, -0.005139730870723724, -0.15222980082035065, 0.08126562088727951, 0.16917790472507477, -0.03539955988526344, 0.05594200640916824, -0.09631654620170593, 0.07492987811565399, -0.04554179310798645, -0.17521417140960693, 0.07274503260850906, 0.03359270095825195, -0.06447967886924744, 0.015514567494392395, -0.06875547766685486, -0.02574172429740429, 0.0783025324344635, -0.1850074976682663, -0.029957382008433342, -0.0010085017420351505, 0.023061485961079597, -0.12120846658945084, -0.11424281448125839, 0.13835257291793823, -0.05236303433775902, -0.027903374284505844, -0.031176121905446053, -0.07468467205762863, -0.01870790310204029, 0.13351744413375854, -0.034462880343198776, 0.08689679205417633, -0.09776969254016876, -0.16441214084625244, 0.1884949654340744, -0.026222236454486847, -0.10074155032634735, 0.09595546126365662, 0.22135229408740997, 0.12569479644298553, 0.04839928820729256, 0.10086043924093246, 0.03546075150370598, -0.009377678856253624, -0.13741900026798248, -0.17391327023506165, -0.10736498981714249, 0.12933728098869324, -0.24251358211040497, 0.008177521638572216, 0.025456005707383156, 0.2524319887161255, 0.07083262503147125, -0.09368503093719482, -0.06956525146961212, 0.09507951140403748, -0.20785696804523468, 0.14839324355125427, -0.237367182970047, -0.10429539531469345, -0.14024175703525543, 0.020977096632122993, 0.09759336709976196, 0.098537877202034, -0.08784890919923782, 0.11549413204193115, 0.14692449569702148, 0.016033051535487175, -0.0005619915318675339, -0.06206534430384636, -0.046363115310668945, 0.005152380093932152, -0.012099811807274818, -0.22327978909015656, -0.026432065293192863, -0.024003883823752403, -0.09419030696153641, -0.021926527842879295, 0.05926554650068283, -0.21737639605998993, -0.09349564462900162, 0.12088118493556976, -0.4959363043308258, 0.032086391001939774, -0.13159655034542084, -0.02607063390314579, -0.18180081248283386, 0.23498208820819855, 0.020819371566176414, -0.10730559378862381, 0.010319485329091549, 0.04612679407000542, 0.07420409470796585, 0.1718052476644516, -0.09061812609434128, -0.0664195716381073, 0.1505279690027237, 0.10255035758018494, -0.21168994903564453, 0.035161156207323074, 0.10582637786865234, -0.021862367168068886, -0.25118058919906616, -0.20757463574409485, 0.06505721062421799, 0.012552397325634956, 0.007262995932251215, 0.00887221097946167, -0.1193172037601471, -0.007054426707327366, 0.2543037533760071, -0.027539758011698723, 0.1975710093975067, -0.1043253242969513, -0.14532561600208282, 0.040537118911743164, 0.08840751647949219, -0.21994411945343018, -0.04009270668029785, 0.003621390089392662, -0.1387900561094284, -0.06622432917356491, 0.05894238501787186, 0.14749109745025635, 0.13571001589298248, 0.20267820358276367, 0.14070163667201996, 0.014788776636123657, 0.19304968416690826, -0.1553044617176056, 0.10321033000946045, -0.08428560197353363, 0.0018714402103796601, 0.17092753946781158, 0.06016607582569122, -0.24994933605194092, -0.09735185652971268, 0.15491701662540436, -0.07023326307535172, -0.18530330061912537, 0.009111620485782623, -0.09852433949708939, 0.00016960025823209435, 0.2719265818595886, 0.010547188110649586, -0.07199341058731079, 0.14887526631355286, 0.014185977168381214, 0.04803301766514778, 0.04601778835058212, 0.007440111134201288, -0.03797145187854767, -0.07866596430540085, 0.006960313767194748, 0.06670084595680237, 0.06338148564100266, 0.2141019105911255}, {0.05352066457271576, -0.08917345106601715, -0.024975642561912537, 0.03022560477256775, -0.003722290275618434, -0.20896726846694946, -0.074490025639534, -0.08349745720624924, 0.09366818517446518, -0.010075584053993225, -0.10316247493028641, 0.1425430327653885, 0.13437345623970032, 0.022373264655470848, 0.03796384856104851, -0.11773916333913803, 0.20379087328910828, -0.09605135768651962, 0.10088273882865906, 0.007356737740337849, -0.16838271915912628, 0.11085543781518936, -0.01940235123038292, -0.13682666420936584, -0.006001986563205719, 0.0005387385608628392, -0.009001893922686577, 0.05371503159403801, 0.09606856852769852, 0.11881005018949509, 0.0920054018497467, 0.2575209140777588, -0.2661769986152649, 0.08534446358680725, 0.007077871356159449, 0.015221335925161839, -0.028962593525648117, -0.04050494730472565, -0.07962434738874435, -0.06261669099330902, 0.08243495970964432, 0.13441242277622223, -0.04358081519603729, 0.10454878211021423, 0.06672412157058716, 0.0192567128688097, -0.15887700021266937, -0.04580926522612572, -0.06270184367895126, 0.15494196116924286, -0.1235174611210823, 0.05732140690088272, 0.050374094396829605, -0.14208927750587463, -0.08508830517530441, -0.14875836670398712, 0.028082963079214096, -0.01071045733988285, 0.010909181088209152, -0.05623043328523636, 0.2005222588777542, 0.0454288125038147, 0.11235862970352173, -0.17921890318393707, -0.06763796508312225, -0.0706855058670044, -0.08953974395990372, -0.06712865829467773, -0.14354382455348969, -0.029612306505441666, 0.13516703248023987, 0.04814820736646652, -0.023794306442141533, -0.014072904363274574, -0.04196193814277649, -0.07944390177726746, -0.1352025270462036, -0.10988418757915497, -0.07332877814769745, -0.02469421550631523, -0.19617809355258942, -0.16984765231609344, -0.026941930875182152, 0.2066880315542221, -0.1961984783411026, -0.0529773086309433, 0.05727037414908409, -0.20887605845928192, 0.08073713630437851, -0.10844098776578903, -0.05641595646739006, -0.0287095345556736, 0.05437492951750755, 0.09691322594881058, -0.0010814875131472945, 0.06462222337722778, 0.10265779495239258, 0.07489422708749771, -0.12457359582185745, 0.2345385104417801, 0.152877077460289, 0.058998193591833115, -0.3270409107208252, 0.06619893014431, -0.27561643719673157, 0.05407186597585678, 0.06613260507583618, -0.0771578848361969, 0.092669777572155, -0.23895394802093506, 0.04697198048233986, -0.14171196520328522, -0.03381083533167839, 0.08162295818328857, -0.13091515004634857, -0.13859133422374725, -0.014930786564946175, -0.1078718826174736, 0.10647807270288467, -0.0020423848181962967, 0.10274292528629303, -0.12083368748426437, -0.005851734429597855, 0.17651323974132538, -0.043515466153621674, -0.006065393798053265, -0.13991829752922058, 0.17085766792297363, 0.03296279162168503, -0.01654849387705326, 0.0808146595954895, 0.09009683132171631, -0.07759248465299606, 0.24297834932804108, -0.06853648275136948, -0.09488660842180252, 0.14880916476249695, -0.008220040239393711, -0.008489500731229782, 0.06553177535533905, -0.0976279154419899, 0.11526284366846085, -0.02433796599507332, -0.13264228403568268, -0.12968340516090393, -0.04657810181379318, 0.1770801842212677, -0.10364212840795517, -0.026937291026115417, -0.060369037091732025, -0.018199389800429344, -0.046465128660202026, -0.07953929156064987, 0.04530255123972893, -0.06272131204605103, 0.03618551790714264, -0.022534651681780815, -0.2199908345937729, 0.005238744430243969, 0.07366184890270233, 0.12834058701992035, 0.13349273800849915, -0.0372544564306736, -0.11849045753479004, 0.0531352162361145, 0.031072845682501793, -0.052512284368276596, 0.08519517630338669, -0.06499025225639343, 0.08481094986200333, -0.12194783985614777, 0.006640346720814705, 0.09990298002958298, 0.08254977315664291, 0.050254594534635544, -0.03882961720228195, -0.13464076817035675, -0.08294172585010529, 0.05341699719429016, -0.009513260796666145, -0.03727959096431732, 0.033133186399936676, -0.292293906211853, -0.1375097781419754, -0.030287228524684906, -0.052322402596473694, 0.16946686804294586, 0.03311837464570999, -0.05183243751525879, 0.15507206320762634, -0.09816789627075195, 0.08519153296947479, -0.28458601236343384, 0.018102392554283142, -0.11201174557209015, 0.07104811817407608, 0.030857767909765244, 0.3279547095298767, -0.04539697244763374, -0.13188530504703522, 0.040075354278087616, 0.040290672332048416, -0.02079319767653942, 0.09713291376829147, -0.19080007076263428, -0.059759631752967834, 0.10229917615652084, 0.14090444147586823, -0.05616585910320282, -0.14477132260799408, -0.06255748867988586, -0.07008885592222214, -0.11962836980819702, -0.14789104461669922, 0.03154440224170685, -0.09138526767492294, -0.026316294446587563, 0.05430719628930092, 0.028397176414728165, 0.03149821236729622, 0.12781447172164917, -0.012091100215911865, 0.12325306236743927, -0.02045479603111744, 0.2791416049003601, 0.022559376433491707, 0.11203508824110031, 0.06749237328767776, 0.007265371270477772, 0.0511552095413208, -0.03337258845567703, -0.0062076859176158905, 0.02128337323665619, -0.055665865540504456, 0.026022499427199364, 0.012574765831232071, -0.19493608176708221, 0.05610659718513489, 0.21381066739559174, 0.18169395625591278, 0.06800591200590134, 0.04323475435376167, -0.02267679013311863, 0.1536356806755066, 0.13357742130756378, 0.06417001783847809, -0.07856492698192596, -0.03539938107132912, 0.0853361114859581, 0.17730757594108582, 0.20065294206142426, 0.025808367878198624, -0.18027545511722565, -0.13184978067874908, 0.03344181180000305, 0.1981264352798462}, {0.07679593563079834, -0.30316078662872314, -0.051070764660835266, 0.002484882716089487, -0.08831623941659927, 0.030162803828716278, -0.04394644498825073, -0.007845531217753887, -0.16333745419979095, -0.0575421117246151, 0.06680344045162201, 0.1335558146238327, -0.04540048912167549, -0.07896070927381516, -0.08137653023004532, -0.10796113312244415, 0.008644256740808487, 0.005620875861495733, 0.14016248285770416, 0.00869071390479803, -0.11038171499967575, -0.13863907754421234, 0.051117848604917526, 0.00025598399224691093, -0.07389484345912933, -0.013464588671922684, 0.030606381595134735, 0.21195410192012787, -0.08618704229593277, 0.0714816004037857, -0.034466035664081573, 0.257428914308548, 0.13766200840473175, 0.1512661874294281, -0.16163435578346252, -0.012296331115067005, -0.25029057264328003, 0.02425382100045681, -0.12384799122810364, -0.04869914799928665, 0.1415344923734665, 0.09833702445030212, -0.03023226000368595, -0.08892844617366791, -0.023281030356884003, 0.013035771436989307, -0.15839333832263947, 0.04630092903971672, 0.06742951273918152, -0.1458292305469513, -0.30390453338623047, -0.027471257373690605, -0.06656448543071747, -0.1506689339876175, -0.07317868620157242, -0.037110310047864914, 0.07127652317285538, 0.02630368061363697, 0.017892947420477867, -0.010123497806489468, 0.1051102727651596, -0.0026945271529257298, 0.07752569019794464, -0.011023261584341526, 0.09565965831279755, -0.112755186855793, -0.030235188081860542, -0.0060390387661755085, 0.07075726985931396, -0.0065184105187654495, 0.13553789258003235, 0.030000915750861168, 0.004495483357459307, -0.12046656012535095, -0.29009342193603516, 0.023700997233390808, -0.031184159219264984, -0.06917212903499603, -0.01791064254939556, 0.00565732829272747, -0.20447300374507904, -0.09491678327322006, -0.10291445255279541, 0.16285644471645355, 0.06502357870340347, -0.13073119521141052, -0.12673203647136688, -0.16857127845287323, 0.07535392791032791, 0.0176201444119215, -0.08542891591787338, -0.2736547291278839, 0.06992161273956299, -0.01587446592748165, 0.082131028175354, -0.1451326310634613, 0.14583390951156616, 0.04471676051616669, -0.12707218527793884, -0.4746476411819458, 0.08411499857902527, 0.08560043573379517, -0.13664692640304565, -0.20310239493846893, 0.16553492844104767, -0.20679457485675812, 0.15889781713485718, 0.07881692051887512, 0.0790354534983635, -0.06870219111442566, 0.1093037873506546, 0.07469198107719421, -0.057945191860198975, 0.06466247141361237, 0.023301701992750168, -0.20273686945438385, -0.005548210348933935, -0.04854603856801987, 0.10205748677253723, -0.0781187117099762, 0.024562986567616463, -0.12167193740606308, -0.29637032747268677, -0.12607164680957794, 0.030299875885248184, 0.043927717953920364, 0.007965036667883396, -0.09092571586370468, 0.08284948021173477, 0.09605752676725388, -0.004301742184907198, 0.1328592449426651, -0.060947682708501816, -0.0099602360278368, -0.13437499105930328, -0.20193538069725037, 0.05906396731734276, 0.09704414010047913, -0.1273954063653946, 0.21386972069740295, -0.018866991624236107, -0.1291864514350891, -0.17514938116073608, 0.053559426218271255, -0.050782572478055954, 0.10651887953281403, 0.048069506883621216, -0.04207015410065651, -0.03710050880908966, -0.029011262580752373, -0.11465536803007126, -0.039102524518966675, 0.07239029556512833, 0.1361657828092575, 0.04043733701109886, 0.04252760112285614, -0.09778117388486862, -0.025474587455391884, 0.08878152817487717, -0.2818484306335449, -0.07659470289945602, 0.12009134143590927, -0.07588190585374832, -0.014152809046208858, 0.10649652779102325, -0.19325746595859528, 0.1762734055519104, 0.0989091545343399, -0.10708807408809662, 0.03801663592457771, 0.179080069065094, 0.17986269295215607, -0.0054658884182572365, 0.006464793812483549, -0.034432295709848404, 0.16848480701446533, 0.17381420731544495, 0.046250052750110626, -0.05015300214290619, -0.012114220298826694, 0.005830388981848955, 0.13894855976104736, -0.13177208602428436, 0.033817704766988754, -0.19398453831672668, 0.013122111558914185, 0.26621681451797485, 0.008655916899442673, 0.06831490248441696, 0.0967685878276825, -0.12349601089954376, -0.08261014521121979, -0.19486263394355774, -0.21946915984153748, -0.15371441841125488, 0.16569294035434723, 0.14038273692131042, 0.364995539188385, 0.08506772667169571, 0.11280887573957443, -0.07506778091192245, -0.04736487567424774, 0.06463859230279922, 0.051246386021375656, -0.017470035701990128, -0.015701327472925186, -0.02822771854698658, 0.22950990498065948, 0.017062481492757797, 0.0487542487680912, 0.09420670568943024, 0.05186088755726814, 0.013775988481938839, 0.0006255529006011784, -0.10168144106864929, 0.041614875197410583, -0.10607484728097916, 0.0887482538819313, -0.20069003105163574, 0.22041520476341248, -0.07183120399713516, -0.048884276300668716, -0.057268016040325165, 0.0006351410411298275, -0.115105539560318, 0.012475430965423584, 0.03218604251742363, 0.1302030086517334, -0.03919322416186333, -0.0848337784409523, 0.12086103856563568, -0.17877867817878723, -0.010111064650118351, 0.048673953860998154, 0.17747917771339417, -0.031196851283311844, -0.00943945161998272, -0.01482990849763155, 0.05578058958053589, -0.4537574350833893, 0.10450436174869537, -0.07822737097740173, -0.17895576357841492, 0.0686449408531189, -0.25025349855422974, 0.09878796339035034, -0.136225625872612, 0.10067076981067657, -0.006250685080885887, -0.014075287617743015, 0.1245575025677681, -0.05362137779593468, 0.007505957502871752, 0.10357865691184998, -0.05638597160577774, 0.05448237806558609}, {-0.09063496440649033, 0.20831643044948578, 0.12350306659936905, -0.010571605525910854, -0.0023934445343911648, 0.0643787607550621, -0.056660015136003494, -0.19393539428710938, -0.14749911427497864, -0.0486861914396286, -0.00957762822508812, 0.1934564709663391, 0.12854605913162231, 0.06287799030542374, 0.12981386482715607, 0.15602277219295502, -0.07218372076749802, -0.24292638897895813, 0.0729941725730896, 0.05232565477490425, 0.006808473262935877, 0.1472921520471573, -0.038627151399850845, -0.023787107318639755, -0.15992118418216705, -0.26179832220077515, 0.19581907987594604, -0.1592373102903366, 0.11000010371208191, -0.07366012781858444, -0.1189606562256813, 0.039254020899534225, -0.15220968425273895, -0.13424932956695557, -0.15606947243213654, 0.13696737587451935, 0.09445072710514069, 0.15369966626167297, 0.11008577793836594, -0.10600482672452927, -0.11624780297279358, -0.020468605682253838, -0.11195071041584015, 0.022199148312211037, -0.07635093480348587, -0.03965739905834198, -0.1368524432182312, -0.051702287048101425, -0.018127888441085815, 0.023426713421940804, 0.22183392941951752, -0.1376807540655136, 0.040205150842666626, -0.0758502259850502, 0.06710399687290192, 0.037446144968271255, -0.03836832568049431, -0.11970985680818558, -0.0010796849383041263, 0.056802716106176376, 0.041249897330999374, -0.1937667727470398, 0.06426839530467987, -0.020352603867650032, 0.10856536775827408, -0.303300142288208, 0.16549380123615265, 0.38635894656181335, 0.12184369564056396, 0.13288269937038422, 0.20626656711101532, -0.0006896043778397143, 0.0036142817698419094, -0.3215954899787903, 0.029087897390127182, 0.050698474049568176, 0.09315046668052673, 0.17836123704910278, 0.04179789870977402, -0.23351243138313293, -0.13240395486354828, -0.1567201465368271, 0.13682608306407928, -0.139256089925766, 0.11066897213459015, -0.2673473656177521, -0.11231400072574615, 0.17362606525421143, -0.016885951161384583, -0.07351411879062653, -0.02377263829112053, -0.0918986052274704, 0.035226739943027496, 0.3693079352378845, -0.11877046525478363, -0.017772983759641647, 0.12584355473518372, -0.04982798174023628, 0.1833488941192627, 0.06662639230489731, 0.0019244550494477153, 0.05265459790825844, -0.32752376794815063, -0.06818837672472, 0.004834032617509365, -0.20418189465999603, -0.04367144778370857, -0.04903068765997887, -0.2151695042848587, 0.007488888222724199, 0.2066437304019928, 0.023634281009435654, 0.027430115267634392, -0.14448335766792297, 0.06346801668405533, 0.126846581697464, 0.09952066838741302, -0.19073115289211273, 0.18736812472343445, 0.028794100508093834, 0.09819501638412476, -0.03198850899934769, -0.156018927693367, -0.1442321240901947, 0.07047536969184875, -0.04149017855525017, 0.018904849886894226, -0.0073093026876449585, 0.04283997416496277, -0.00850062444806099, 0.3357463777065277, 0.21569859981536865, 0.013103245757520199, 0.115000419318676, 0.21318496763706207, -0.193660169839859, 0.16221614181995392, -0.05175980180501938, -0.011370336636900902, -0.3738084137439728, -0.19960030913352966, -0.04679044708609581, -0.3393848240375519, -0.0405009426176548, 0.09204331040382385, 0.06448224186897278, 0.07779920101165771, -0.15526439249515533, 0.1097768247127533, 0.07917093485593796, 0.07693807780742645, -0.023095954209566116, 0.002423191210255027, 0.12921611964702606, 0.05850199609994888, -0.0029775493312627077, -0.12260472774505615, 0.17631842195987701, -0.06587422639131546, -0.11235837638378143, -0.02078602835536003, 0.05157551169395447, 0.266348659992218, -0.06835224479436874, -0.12471656501293182, -0.07829149067401886, -0.08750387281179428, -0.10906461626291275, -0.14469951391220093, -0.18039706349372864, 0.09155828505754471, -0.15628930926322937, -0.17914380133152008, 0.1554158329963684, 0.1988765001296997, -0.03996502235531807, -0.10409768670797348, 0.1405521184206009, 0.1552577167749405, 0.12034797668457031, 0.1588457226753235, 0.45325225591659546, 0.12377887219190598, -0.11773508042097092, 0.22582140564918518, 0.3290177285671234, 0.08097218722105026, 0.13982830941677094, 0.15389828383922577, 0.15455278754234314, -0.11113147437572479, -0.19576652348041534, -0.10125233232975006, 0.09405060857534409, -0.22222420573234558, -0.25347957015037537, 0.07186601310968399, 0.030092699453234673, 0.05429008603096008, 0.15559686720371246, 0.12767352163791656, -0.025069983676075935, -0.08688292652368546, 0.03307030349969864, 0.03932927921414375, -0.15329328179359436, 0.1533205211162567, 0.3694094717502594, -0.08434806019067764, -0.07922527194023132, 0.15438008308410645, -0.07831194251775742, -0.03382372483611107, 0.16903044283390045, -0.0050719366408884525, 0.1319389045238495, 0.29982486367225647, 0.15373748540878296, 0.1460224837064743, -0.004581743385642767, 0.11255055665969849, -0.07079757004976273, 0.06961674243211746, -0.014232068322598934, 0.1160697266459465, 0.053105901926755905, 0.0643785148859024, 0.13362304866313934, 0.03421842306852341, -0.23329338431358337, -0.10801629722118378, -0.15757443010807037, 0.3793630003929138, 0.24685262143611908, -0.08419079333543777, 0.07979069650173187, 0.1432684063911438, -0.04761200398206711, -0.24437429010868073, -0.11040188372135162, -0.3744826316833496, 0.08495259284973145, 0.20731046795845032, -0.19088290631771088, -0.016384271904826164, -0.007242739666253328, 0.179924875497818, -0.08383432775735855, -0.11208264529705048, -0.013226494193077087, 0.20612581074237823, 0.0328071303665638, -0.007826026529073715, 0.0975811630487442, 0.006778213661164045, 0.11176188290119171}, {0.13792209327220917, -0.1286405324935913, -0.09489922225475311, -0.15634144842624664, -0.23643560707569122, 0.11079876124858856, 0.128694549202919, -0.09173274040222168, -0.21222171187400818, -0.22087445855140686, 0.03550956770777702, -0.04044227674603462, -0.10821057111024857, 0.09055215865373611, 0.05602851137518883, -0.04114929214119911, -0.12081572413444519, -0.04276866838335991, -0.04816357418894768, -0.06720994412899017, 0.14376024901866913, -0.047672875225543976, 0.036166734993457794, 0.1167120411992073, 0.020087886601686478, 0.15104196965694427, -0.06903489679098129, 0.1587112694978714, 0.0934879407286644, 0.018375849351286888, -0.07661717385053635, -0.1637669950723648, 0.07747470587491989, -0.04522274434566498, -0.23528069257736206, -0.03696468845009804, 0.17619772255420685, -0.11997837573289871, -0.028895294293761253, -0.044537391513586044, 0.09873947501182556, -0.14379383623600006, 0.05927078425884247, -0.05435324087738991, -0.04617033153772354, 0.0896860659122467, -0.012068909592926502, -0.07757376879453659, 0.021955279633402824, 0.08561165630817413, 0.28359565138816833, -0.06313809007406235, 0.025467287749052048, 0.10312387347221375, 0.0781443640589714, -0.008897925727069378, -0.0168745294213295, 0.15990948677062988, 0.009403239004313946, -0.30002689361572266, -0.25576716661453247, 0.0159196387976408, 0.19855783879756927, 0.16755247116088867, 0.1607174426317215, -0.02597404085099697, 0.06819318234920502, 0.20503920316696167, -0.11899717897176743, -0.09710399806499481, -0.02388518489897251, -0.019045978784561157, -0.022837206721305847, -0.013249997980892658, 0.07850664854049683, 0.17446915805339813, 0.24749255180358887, 0.1283024251461029, -0.03102572076022625, -0.14044874906539917, 0.1645241528749466, -0.06274375319480896, -0.07926807552576065, -0.16358408331871033, -0.00705848028883338, 0.022288866341114044, -0.03764881193637848, 0.11508940905332565, -0.03392164409160614, 0.06291638314723969, 0.09434493631124496, 0.1936771273612976, -0.17524564266204834, -0.06532450765371323, -0.009224241599440575, 0.0001811839611036703, -0.07474961876869202, 0.46401119232177734, 0.12049009650945663, -0.0565190389752388, 0.17218482494354248, -0.10787861049175262, -0.21814310550689697, 0.09981030225753784, 0.32516610622406006, -0.07722478359937668, -0.13051998615264893, 0.13240160048007965, -0.01899069733917713, -0.15872618556022644, 0.010656214319169521, 0.07840169221162796, 0.053731583058834076, 0.27216482162475586, -0.15224912762641907, 0.11681453883647919, -0.0939711406826973, -0.054420486092567444, -0.008993430063128471, 0.09818011522293091, -0.01943695917725563, 0.2703978419303894, 0.021936016157269478, -0.02676413394510746, 0.01369295921176672, 0.02501513995230198, -0.20940424501895905, 0.07277999073266983, -0.02861936390399933, -0.09067320078611374, -0.06675266474485397, -0.14469633996486664, 0.12018673866987228, 0.0012947593349963427, 0.24258391559123993, -0.017513001337647438, -0.13739624619483948, 0.014834421686828136, -0.0722399577498436, -0.21943819522857666, -0.12435761839151382, 0.06298576295375824, 0.13978736102581024, -0.029700949788093567, 0.20514246821403503, -0.10170499235391617, -0.04386605694890022, -0.12675194442272186, 0.10703916847705841, -0.05093817412853241, -0.02617810294032097, 0.22239138185977936, -0.04991105943918228, -0.028969507664442062, 0.07717638462781906, 0.03961102291941643, -0.07753483951091766, 0.2660197913646698, 0.004440602846443653, 0.09677936136722565, 0.014829154126346111, -0.08132846653461456, -0.13882113993167877, 0.19707252085208893, 0.008562951348721981, 0.04000087454915047, -0.00485081085935235, -0.15243278443813324, 0.02469319850206375, -0.028518712148070335, 0.09574124962091446, -0.061359602957963943, 0.05615151301026344, -0.08884687721729279, 0.08387826383113861, 0.03996463492512703, 0.06046844646334648, -0.16814003884792328, 0.12390600144863129, 0.07171694189310074, 0.0814078077673912, 0.33825793862342834, 0.0267200730741024, 0.32297879457473755, -0.08693943917751312, -0.022287752479314804, -0.5724674463272095, 0.021333280950784683, -0.10416033118963242, 0.03616040199995041, -0.055654630064964294, -0.10690684616565704, 0.1807539016008377, 0.012185238301753998, -0.002436496317386627, -0.0677720457315445, -0.006977526005357504, -0.19338728487491608, -0.0743173137307167, 0.013060332275927067, 0.13641592860221863, -0.035135701298713684, -0.0825636088848114, 0.059004273265600204, 0.09181775897741318, -0.029015585780143738, -0.04420597851276398, 0.13218826055526733, 0.011564331129193306, -0.026333723217248917, 0.06315425038337708, -0.34974387288093567, 0.19943153858184814, -0.11518057435750961, -0.1061900407075882, -0.0834822878241539, 0.031158864498138428, -0.052988745272159576, -0.0684463158249855, 0.08517936617136002, 0.1399163454771042, 0.09311430156230927, 0.21600280702114105, -0.5634797215461731, 0.1202462911605835, -0.514892041683197, 0.1531803160905838, -0.07467121630907059, 0.17353738844394684, -0.12638285756111145, 0.007154475431889296, 0.026205189526081085, -0.040541741997003555, 0.024095386266708374, 0.2898005545139313, 0.020267397165298462, 0.1673131138086319, -0.006871811114251614, -0.02413301356136799, -0.11757467687129974, 0.10651910305023193, 0.08823016285896301, -0.07511681318283081, -0.07995083183050156, -0.08741724491119385, -0.049023013561964035, 0.16890890896320343, 0.009517299011349678, 0.10727397352457047, -0.11374828964471817, -0.005235792603343725, -0.008819951675832272, -0.10733191668987274, 0.07965443283319473, 0.06933433562517166, 0.005259217694401741}, {0.07766162604093552, -0.11206133663654327, 0.14258231222629547, 0.0900166854262352, 0.10219726711511612, 0.10556217283010483, -0.2165805697441101, 0.09307461977005005, -0.05470084771513939, -0.01343241985887289, -0.18393082916736603, 0.040392402559518814, -0.01696157455444336, -0.038702238351106644, 0.02187357284128666, -0.1456984132528305, -0.15448221564292908, 0.08427407592535019, -0.231676384806633, 0.11819498240947723, -0.07655242085456848, -0.07990904152393341, -0.0186388548463583, -0.04565129801630974, -0.044745370745658875, 0.08409349620342255, -0.23980672657489777, -0.0462939590215683, -0.004516412504017353, 0.050788749009370804, -0.21837036311626434, -0.03473573178052902, 0.19227738678455353, -0.10611501336097717, -0.31685560941696167, 0.20216605067253113, 0.024704864248633385, -0.06707745045423508, 0.027442602440714836, 0.1271636039018631, -0.12010295689105988, 0.16293716430664062, -0.14367835223674774, -0.11096221208572388, -0.05110878497362137, -0.04476441815495491, 0.0752369612455368, 0.02823231928050518, -0.05597422271966934, -0.20617792010307312, 0.3969070613384247, -0.27011218667030334, -0.040361642837524414, 0.33111506700515747, 0.0046548619866371155, -0.22971846163272858, -0.041305460035800934, 0.11665237694978714, 0.14122802019119263, 0.05559680983424187, 0.00752604752779007, -0.07957865297794342, -0.14789126813411713, -0.03597162663936615, 0.1366327404975891, 0.12289709597826004, -0.19367201626300812, 0.09380476921796799, -0.16023406386375427, -0.16070556640625, -0.05984428897500038, -0.03700665384531021, 0.012335558421909809, -0.1561156064271927, -0.4079054594039917, 0.058010999113321304, 0.18409334123134613, -0.004171750973910093, -0.27531349658966064, -0.0807856097817421, 0.05494289472699165, 0.08790149539709091, 0.02246689423918724, -0.22950290143489838, 0.06406854838132858, 0.02922903560101986, 0.2777095437049866, 0.18217071890830994, -0.24421174824237823, 0.046030838042497635, -0.1640046089887619, 0.01828691177070141, -0.2462528944015503, 0.1219610795378685, -0.17168879508972168, -0.3039078414440155, -0.07575485855340958, -0.052855562418699265, -0.18767379224300385, -0.017357606440782547, -0.05194319039583206, 0.0005785874091088772, -0.04976426437497139, -0.09652084857225418, 0.1238405629992485, 0.5071945190429688, -0.016818808391690254, 0.04275520518422127, -0.24805694818496704, 0.04957982152700424, -0.15082719922065735, 0.011775982566177845, 0.15643081068992615, 0.1751926839351654, -0.30310073494911194, 0.25986990332603455, -0.030430080369114876, -0.05097667872905731, -0.07540745288133621, 0.019906487315893173, -0.06086552515625954, -0.02312016487121582, 0.01038892287760973, -0.10449017584323883, 0.1353943794965744, -0.06824968010187149, 0.004054728429764509, -0.004040851257741451, 0.015077793970704079, -0.07373543083667755, -0.09148349612951279, -0.07023308426141739, -0.0058886464685201645, -0.2511195242404938, 0.027277877554297447, 0.1095605194568634, -0.1729804277420044, -0.033998042345047, 0.07108753174543381, -0.3133229613304138, 0.31705978512763977, -0.010197581723332405, -0.049571070820093155, 0.29708942770957947, 0.036462411284446716, 0.1795337200164795, -0.13285280764102936, -0.06809403002262115, -0.06348082423210144, -0.3539721667766571, 0.007788868620991707, 0.04089588299393654, -0.0919952541589737, -0.029573148116469383, 0.03839152678847313, -0.2630845010280609, -0.23584432899951935, -0.13055042922496796, -0.15171702206134796, -0.13971379399299622, -0.164383202791214, 0.02720526233315468, 0.006250116508454084, -0.09675897657871246, -0.0643673911690712, 0.03608078509569168, -0.007174598053097725, -0.4706621468067169, -0.13643702864646912, -0.0237246286123991, 0.11178238689899445, 0.4601815938949585, 0.04954000562429428, 0.14120036363601685, 0.15582650899887085, 0.1372903734445572, 0.031686097383499146, 0.11262747645378113, -0.1558842957019806, 0.2011536806821823, 0.23386450111865997, -0.03449440374970436, 0.2755601704120636, 0.167527973651886, -0.059046272188425064, 0.16629257798194885, 0.03808329626917839, 0.1327940970659256, -0.15754330158233643, 0.008818456903100014, 0.0636020377278328, -0.025632956996560097, 0.01758737489581108, 0.13533052802085876, 0.3875526487827301, 0.14265865087509155, 0.007889230735599995, -0.19058050215244293, 0.009747843258082867, -0.03043203614652157, 0.04302190989255905, -0.1431616097688675, -0.004291652701795101, 0.05532044544816017, -0.03569423779845238, -0.09617367386817932, 0.020935483276844025, -0.09101849049329758, 0.032274916768074036, -0.008290214464068413, -0.0640183612704277, 0.00542763015255332, 0.07702793926000595, -0.2568056285381317, 0.11147473007440567, 0.1643713265657425, 0.002302657114341855, 0.03309875726699829, 0.10252025723457336, 0.04605706408619881, -0.3144512474536896, 0.08267208933830261, 0.13954982161521912, 0.07060379534959793, 0.003949694335460663, 0.008399667218327522, 0.12787503004074097, -0.016273029148578644, -0.07891706377267838, -0.013357533141970634, 0.17364385724067688, -0.07858186960220337, 0.14035044610500336, 0.23640060424804688, -0.035745151340961456, 0.13629770278930664, 0.12471337616443634, 0.2772756516933441, -0.340685099363327, -0.0580911785364151, 0.08213832229375839, 0.010346095077693462, 0.037901051342487335, -0.10657093673944473, -0.07720958441495895, -0.038210172206163406, 0.1776353418827057, 0.1866132766008377, -0.17463147640228271, 0.05592590197920799, -0.007856011390686035, -0.041730403900146484, -0.009935326874256134, 0.11541818827390671, 0.13500849902629852, -0.022439269348978996}, {-0.015822015702724457, 0.3261440098285675, 0.21960492432117462, 0.012055394239723682, 0.1260332614183426, 0.09677572548389435, 0.185832217335701, 2.0375577150844038e-05, -0.05153845250606537, -0.07386405766010284, 0.035499949008226395, -0.20281033217906952, -0.15812572836875916, -0.03649559244513512, 0.02244696393609047, 0.18605448305606842, -0.1371905356645584, -0.2330363243818283, -0.103008933365345, 0.15695448219776154, 0.0702100619673729, -0.10529646277427673, -0.075319804251194, -0.12336842715740204, -0.049317698925733566, -0.002205680822953582, 0.002929194364696741, 0.10508928447961807, 0.012991895899176598, 0.015609722584486008, -0.07349572330713272, -0.12410241365432739, 0.12107225507497787, -0.049625054001808167, 0.07530221343040466, 0.011913301423192024, -0.03349697217345238, -0.09153840690851212, 0.10523610562086105, 0.09815236926078796, -0.09591881185770035, -0.08823642134666443, -0.05908854678273201, -0.03606922924518585, -0.16679492592811584, 0.03419192135334015, -0.07050275057554245, -0.02554532140493393, 0.07355799525976181, -0.0960024893283844, 0.12386664003133774, 0.0831417366862297, -0.0828787088394165, -0.15720394253730774, -0.009969445876777172, 0.2085108608007431, 0.07806459814310074, 0.04080226644873619, -0.048140618950128555, 0.2536579668521881, -0.2504795789718628, 0.011287616565823555, -0.11473455280065536, 0.22174379229545593, 0.10361669212579727, 0.036296162754297256, 0.1382811814546585, 0.2493579387664795, 0.355704665184021, -0.06526582688093185, -0.1288919597864151, 0.1582450270652771, -0.10559920966625214, 0.004935938399285078, -0.005087617319077253, 0.02191493660211563, -0.024278096854686737, 0.17600853741168976, -0.06610042601823807, -0.22511474788188934, 0.27760612964630127, -0.0859551802277565, 0.05496566742658615, -0.267097532749176, 0.12903478741645813, -0.03231265768408775, -0.2890782952308655, 0.07381974905729294, -0.019409460946917534, 0.31848278641700745, -0.007222085725516081, 0.2584880292415619, 0.059715814888477325, -0.04477287828922272, -0.32523855566978455, 0.11260604858398438, -0.14532025158405304, 0.05705128237605095, 0.1412520706653595, -0.014095742255449295, -0.1490550935268402, -0.03802093490958214, 0.08325178921222687, 0.02238425426185131, 0.05836699903011322, 0.10863429307937622, 0.02617591992020607, 0.05011296272277832, -0.05668020248413086, 0.06046478450298309, -0.062028154730796814, 0.0509350560605526, -0.02043740451335907, -0.10933153331279755, -0.10472318530082703, 0.09286138415336609, 0.16081678867340088, 0.019602617248892784, 0.057235606014728546, 0.08171591907739639, 0.10442803055047989, 0.05829445272684097, 0.18304035067558289, -0.08190090209245682, 0.011720916256308556, -0.009650573134422302, -0.03515763580799103, 0.08640945702791214, -0.10958097875118256, -0.09420032054185867, 0.06702004373073578, -0.024031439796090126, 0.08439427614212036, 0.00972804706543684, 0.17722168564796448, 0.22932057082653046, 0.0637485682964325, -0.13457328081130981, 0.2165721207857132, -0.0448862761259079, -0.2539120018482208, -0.14736758172512054, -0.1087869182229042, -0.07834623008966446, -0.07567740976810455, -0.02113404870033264, -0.2134292721748352, 0.05985850840806961, 0.06566471606492996, -0.10580579191446304, 0.15845516324043274, -0.09131260216236115, 0.05631449818611145, -0.095259889960289, 0.23645536601543427, 0.03675292059779167, 0.05866100266575813, 0.26161667704582214, -0.0996256023645401, -0.29719385504722595, -0.133247971534729, -0.1192944347858429, -0.2225170135498047, 0.16157138347625732, -0.017045149579644203, 0.06660273671150208, 0.16618908941745758, -0.18995581567287445, 0.12268273532390594, -0.12415775656700134, 0.07591216266155243, -0.05852876231074333, 0.02195551246404648, -0.05158909037709236, 0.02421080693602562, -0.024176014587283134, -0.07927122712135315, 0.09282423555850983, -0.0016614831984043121, -0.053020428866147995, 0.24423177540302277, 0.010860585607588291, 0.16158193349838257, 0.15880756080150604, 0.2759757339954376, 0.01823882758617401, -0.09766766428947449, 0.05358484759926796, -0.0367143340408802, 0.1548602432012558, 0.02067931927740574, 0.21397648751735687, 0.22937163710594177, 0.10501965135335922, -0.007809373550117016, -0.2604691684246063, -0.14961926639080048, -0.40915921330451965, 0.010224099270999432, 0.02870871126651764, -0.07211771607398987, 0.2756611406803131, -0.13127340376377106, 0.0386543832719326, 0.11084792762994766, 0.14309632778167725, -0.25141701102256775, -0.34729519486427307, 0.22282527387142181, -0.19479389488697052, 0.13501866161823273, -0.0229581780731678, 0.0039193411357700825, 0.07941018044948578, -0.07379240542650223, 0.008346005342900753, 0.009680114686489105, -0.11944828182458878, -0.07650773972272873, 0.04080263152718544, 0.05290331318974495, 0.01131116971373558, -0.1496228575706482, -0.09393355995416641, -0.16170422732830048, -0.05089336261153221, -0.04595761373639107, -0.06845416873693466, -0.08110898733139038, -0.08552885055541992, 0.13700281083583832, 0.006146884988993406, 0.024893619120121002, 0.023624707013368607, 0.028870340436697006, -0.12197404354810715, -0.14809858798980713, 0.15185342729091644, 0.07988837361335754, -0.06939566880464554, 0.23694485425949097, -0.001210555899888277, -0.03412675857543945, -0.013347922824323177, 0.1155267059803009, -0.03133469447493553, -0.016408400610089302, 0.03218851238489151, 0.017486480996012688, -0.1640833616256714, -0.12030086666345596, 0.26946839690208435, -0.030003521591424942, 0.1295834332704544, -0.07840673625469208, 0.10436072945594788}, {0.060993533581495285, -0.036772482097148895, -0.007126506883651018, 0.0046856943517923355, 0.2871708273887634, -0.02483941987156868, 0.15141867101192474, 0.09263384342193604, 0.09089720249176025, 0.3003734052181244, -0.26397407054901123, -0.3885391056537628, -0.024486318230628967, -0.11920007318258286, -0.21242542564868927, -0.1304766684770584, 0.04806577041745186, -0.12698134779930115, -0.38614311814308167, -0.15749086439609528, 0.06756306439638138, -0.018510842695832253, 0.006436568219214678, 0.039637885987758636, 0.24426409602165222, 0.08340802788734436, -0.4681861400604248, 0.05331309884786606, 0.0023544789291918278, 0.013008478097617626, 0.04552126303315163, -0.11050868034362793, 0.38302746415138245, -0.19539861381053925, 0.09099454432725906, -0.09058008342981339, -0.008322074078023434, -0.06785916537046432, 0.08120614290237427, 0.08173684030771255, -0.04115821421146393, 0.09762176871299744, 0.16934482753276825, -0.09770471602678299, 0.12383191287517548, -0.05518316477537155, 0.28213977813720703, 0.00562753202393651, -0.111948661506176, -0.0035350394900888205, 0.18302062153816223, -0.13022758066654205, -0.18004794418811798, 0.21473996341228485, -0.03744586929678917, -0.061059024184942245, 0.10713299363851547, 0.06055599823594093, -0.24641461670398712, -0.009147286415100098, -0.15046179294586182, -0.20377399027347565, -0.23054398596286774, 0.00748041458427906, 0.13581609725952148, 0.053564075380563736, -0.004171634092926979, 0.01081452053040266, 0.053366634994745255, -0.003200582228600979, 0.06461746990680695, 0.03241417557001114, 0.12307068705558777, 0.28484976291656494, -0.05686882510781288, 0.1105540469288826, -0.12357143312692642, -0.03867119550704956, -0.01747286133468151, 0.0743444487452507, 0.16858726739883423, -0.09819581359624863, -0.1026722714304924, -0.19862206280231476, -0.21277019381523132, 0.10599258542060852, 0.03439389169216156, -0.02199382521212101, -0.13636407256126404, -0.08890796452760696, -0.056615762412548065, -0.05333535745739937, -0.21109674870967865, 0.22907231748104095, 0.07303658872842789, 0.060874685645103455, -0.08519936352968216, 0.040672946721315384, 0.038993868976831436, -0.057337209582328796, -0.024471886456012726, -0.08924736082553864, -0.10919877886772156, 0.07417211681604385, 0.3379008173942566, -0.6855331063270569, -0.042380187660455704, -0.01005790289491415, 0.12499669194221497, 0.08303093910217285, -0.17482702434062958, 0.036014359444379807, -0.0706280767917633, 0.14525863528251648, -0.06951040029525757, 0.09699002653360367, 0.01898239180445671, 0.09727738052606583, 0.02614891715347767, 0.18681278824806213, -0.11798933893442154, 0.09494110196828842, -0.0023481170646846294, -0.17411959171295166, -0.1407352089881897, -0.21841447055339813, 0.0335165336728096, 0.11648215353488922, 0.20263564586639404, -0.18933382630348206, 0.12083354592323303, 0.16882586479187012, -0.1851740926504135, -0.195030078291893, 0.026964174583554268, 0.06880424916744232, -0.02241378091275692, 0.20407670736312866, -0.10460661351680756, 0.2114260196685791, 0.041848715394735336, 0.022589359432458878, -0.029937658458948135, -0.03833116218447685, 0.13177210092544556, 0.08664403855800629, 0.1041240468621254, 0.07008359581232071, 0.19974806904792786, 0.030070634558796883, -0.09941578656435013, -0.07031088322401047, -0.13754358887672424, -0.0648050308227539, 0.043675586581230164, 0.12001947313547134, 0.18464352190494537, -0.07985594123601913, -0.16370615363121033, 0.021718068048357964, -0.014723517000675201, 0.14309054613113403, -0.22682920098304749, 0.05782319977879524, 0.11300505697727203, -0.19067199528217316, -0.30469396710395813, -0.2413630336523056, -0.21399842202663422, -0.0337066687643528, 0.018161922693252563, -0.006088164169341326, -0.09256792813539505, 0.006786594167351723, -0.15671157836914062, 0.11372093111276627, -0.09333724528551102, -0.05065526068210602, 0.19342032074928284, 0.08979082852602005, -0.14932861924171448, -0.341861367225647, 0.26145172119140625, 0.030318500474095345, -0.21181611716747284, -0.032948169857263565, -0.30011650919914246, 0.03095279075205326, 0.19610093533992767, -0.05730145424604416, -0.006246382836252451, -0.04108063131570816, -0.04313458874821663, -0.14659897983074188, 0.31171339750289917, -0.0523667074739933, -0.1725071519613266, -0.059423115104436874, -0.09255541115999222, 0.164302259683609, -0.22399383783340454, 0.17904147505760193, -0.11041553318500519, -0.06039794906973839, -0.11752539128065109, -0.17998553812503815, -0.21376508474349976, -0.047708816826343536, 0.10921967029571533, -0.05905923247337341, 0.07494404911994934, 0.07299723476171494, -0.06336330622434616, 0.11428848654031754, -0.18485045433044434, 0.12341952323913574, 0.17136947810649872, 0.2580389082431793, 0.3138517737388611, -0.08479750156402588, 0.0029395914170891047, -0.31687378883361816, 0.112433522939682, 0.08120843768119812, 0.11202192306518555, 0.04323410987854004, -0.0300207007676363, -0.01436366606503725, 0.012667275033891201, 0.057671066373586655, 0.09538818150758743, 0.17634983360767365, -0.1260715126991272, 0.11376824975013733, -0.0556534007191658, -0.14597176015377045, -0.00404941663146019, 0.17346231639385223, -0.03444349020719528, -0.19954033195972443, -0.06590054929256439, -0.06120377779006958, -0.06713960319757462, 0.16582216322422028, -0.059896085411310196, 0.007571076042950153, -0.1719667613506317, -0.05770546942949295, 0.3386355936527252, 0.04055560752749443, 0.02554495818912983, 0.004954630509018898, -0.18845538794994354, 0.008536101318895817, 0.00844360701739788, -0.17074543237686157}, {-0.07373407483100891, 0.13946203887462616, -0.08326245844364166, 0.055222395807504654, -0.08450043946504593, -0.12470093369483948, -0.025436611846089363, -0.11036920547485352, 0.07001613080501556, -0.1308043897151947, -0.06461331248283386, 0.10353091359138489, -0.008599374443292618, 0.2287818342447281, 0.18567556142807007, -0.07233946025371552, 0.08140582591295242, 0.04193275421857834, -0.16930566728115082, -0.07559815794229507, -0.08394844830036163, 0.09168287366628647, -0.026743805035948753, -0.05395630747079849, -0.056612804532051086, -0.11837593466043472, -0.18078817427158356, 0.009721259586513042, 0.2396262288093567, -0.034727685153484344, -0.02115681767463684, -0.05104053393006325, -0.20912615954875946, -0.06475547701120377, -0.06364019960165024, 0.15673434734344482, 0.027111662551760674, -0.1092507615685463, 0.10646945238113403, -0.0651678517460823, 0.11657153815031052, -0.03829532861709595, 0.288568913936615, 0.010373531840741634, -0.11916881054639816, 0.11512899398803711, 0.07744695991277695, -0.1762196123600006, 0.02501223236322403, 0.06455355882644653, 0.22082971036434174, 0.010577206499874592, -0.013749249279499054, -0.12216625362634659, 0.030917029827833176, 0.004298219457268715, -0.014926305040717125, 0.20900613069534302, -0.05576318874955177, -0.08769621700048447, -0.05629495531320572, 0.03396796062588692, 0.11342417448759079, 0.08559155464172363, 0.04417863488197327, -0.046179283410310745, -0.08232861757278442, 0.08353200554847717, -0.17819738388061523, 0.12147893756628036, -0.05893810838460922, 0.06169222295284271, 0.10508812218904495, 0.13097873330116272, 0.4184602200984955, 0.05220220983028412, 0.020886939018964767, -0.10763329267501831, 0.08023271709680557, 0.08955538272857666, 0.16698478162288666, 0.005606231279671192, -0.11661577969789505, -0.13580939173698425, -0.26897943019866943, 0.11409088224172592, 0.10106112062931061, -0.002945909509435296, 0.17334701120853424, -0.05566989257931709, 0.1318446397781372, 0.09661631286144257, -0.14320304989814758, 0.12020288407802582, 0.09535842388868332, 0.11767507344484329, -0.2729140818119049, 0.0483345165848732, -0.07691031694412231, 0.6755956411361694, 0.03132674843072891, -0.13755713403224945, -0.26347261667251587, 0.19016920030117035, -0.1687428057193756, 0.446586012840271, -0.022410914301872253, -0.005608099978417158, -0.11078081279993057, -0.11676134914159775, -0.094142846763134, 0.10262495279312134, -0.11871862411499023, -0.06273974478244781, 0.13141296803951263, 0.016591893509030342, -0.11152823269367218, -0.04062405973672867, 0.13832110166549683, -0.014271875843405724, 0.2059980183839798, -0.06422828882932663, 0.13473041355609894, 0.08623401075601578, -0.045796241611242294, -0.0841807872056961, 0.04436267539858818, 0.012667772360146046, -0.2720802426338196, -0.06393150985240936, 0.005166441202163696, -0.05105985328555107, 0.018405823037028313, 0.055371955037117004, 0.03186215087771416, 0.2618999183177948, -0.1265273243188858, -0.02434907853603363, 0.05515851825475693, 0.037149880081415176, -0.051308948546648026, 0.011548490263521671, 0.1441994458436966, -0.24345546960830688, 0.065746009349823, -0.02259868197143078, 0.08964813500642776, -0.10633070766925812, 0.06530819088220596, 0.024002674967050552, -0.13597384095191956, -0.15068207681179047, 0.03629634901881218, 0.03380995988845825, -0.15366299450397491, -0.08451492339372635, 0.04845401272177696, 0.008219476789236069, -0.0003921265888493508, 0.11703603714704514, -0.07872869819402695, -0.17084664106369019, 0.07179278880357742, 0.09927666932344437, -0.01159379817545414, 0.06416114419698715, -0.09228456020355225, -0.00045969864004291594, 0.11098960787057877, -0.012002812698483467, 0.042448412626981735, -0.027821602299809456, -0.0175529345870018, 0.10423384606838226, 0.1176564171910286, -0.039084453135728836, 0.009081636555492878, -0.1731310486793518, 0.14881274104118347, -0.2006123661994934, -0.05506253242492676, 0.05305783823132515, 0.20881250500679016, -0.06584382802248001, -0.08823639899492264, 0.06612846255302429, -0.27752503752708435, 0.1088157519698143, 0.09836854040622711, -0.0029704999178647995, 0.028900843113660812, 0.013895350508391857, 0.15481431782245636, 0.028115656226873398, -0.03267015516757965, -0.0779392421245575, -0.08301384747028351, -0.2420695275068283, 0.01005130261182785, -0.16766025125980377, 0.06902914494276047, 0.030468089506030083, 0.19582173228263855, 0.07859295606613159, 0.09636637568473816, -0.08972494304180145, 0.08998428285121918, -0.03772334009408951, -0.05986407771706581, -0.19015799462795258, -0.0033008730970323086, 0.00409774761646986, -0.0410454161465168, 0.002642547246068716, 0.21880891919136047, -0.1221865564584732, -0.01836712285876274, -0.21727922558784485, 0.1575053334236145, -0.21819700300693512, 0.3097276985645294, -0.019493820145726204, -0.05095909908413887, -0.0028323358856141567, 0.16189144551753998, -0.04908832535147667, 0.09829382598400116, 0.025813207030296326, -0.10189547389745712, 0.026073632761836052, -0.19532731175422668, -0.06124941259622574, -0.110897958278656, 0.12135203182697296, 0.01995384879410267, -0.0388755165040493, -0.07119724899530411, 0.13991333544254303, 0.05587129667401314, 0.5782723426818848, -0.11859118938446045, 0.03925164416432381, 0.03311074525117874, -0.021566646173596382, 0.07765071839094162, 0.014344130642712116, 0.1255674809217453, -0.0651925727725029, 0.12233442813158035, 0.11078447103500366, -0.07201756536960602, -0.008065853267908096, -0.1116265058517456, -0.21600663661956787, 0.10453042387962341, 0.03478255495429039}, {-0.1912272870540619, 0.11489304900169373, 0.17824886739253998, 0.30413419008255005, 0.13310584425926208, 0.0520135760307312, -0.21626485884189606, -0.004455799236893654, 0.21483410894870758, -0.27953338623046875, 0.1605619490146637, 0.0005825641565024853, -0.0044295224361121655, 0.1580200493335724, -0.14338919520378113, -0.08570628613233566, 0.05710688978433609, 0.010233242996037006, 0.18873988091945648, 0.07087284326553345, 0.12036144733428955, -0.07111254334449768, 0.07963071018457413, 0.00037274070200510323, -0.007924480363726616, -0.061943355947732925, 3.07379086734727e-05, -0.1572198122739792, -0.07753172516822815, 0.03451007604598999, 0.015460830181837082, 0.15820904076099396, -0.22568556666374207, 0.08596116304397583, 0.10141885280609131, -0.15112978219985962, 0.1710939109325409, 0.21882300078868866, -0.20387861132621765, -0.055531781166791916, -0.016638707369565964, 0.12568889558315277, 0.18533644080162048, 0.018160659819841385, 0.09600125998258591, 0.2227214127779007, 0.0017948350869119167, -0.033646754920482635, -0.029892660677433014, 0.060094982385635376, 0.11161457747220993, -0.05239339917898178, 0.03326360508799553, -0.07027755677700043, -0.002615186385810375, -0.03853761777281761, -0.246244877576828, -0.07823549956083298, 0.0005190744996070862, -0.028874171897768974, -0.0628499910235405, -0.03493071347475052, 0.1697567105293274, -0.23984253406524658, -0.17247281968593597, -0.013118615373969078, 0.07001496106386185, 0.10602639615535736, -0.1849997490644455, 0.026103487238287926, 0.03665386512875557, 0.10068870335817337, 0.0863693356513977, -0.14230087399482727, -0.38996031880378723, 0.0018174045253545046, 0.07568201422691345, -0.3171961009502411, 0.27023786306381226, -0.12681205570697784, -0.338213711977005, 0.04867466539144516, -0.32747960090637207, 0.027484314516186714, 0.2045765221118927, 0.10715136677026749, -0.17455552518367767, 0.16555480659008026, 0.009693789295852184, 0.014741995371878147, 0.03899935632944107, -0.15022964775562286, 0.25951069593429565, 0.02562662400305271, 0.08185820281505585, 0.021763881668448448, -0.06593342125415802, -0.12136201560497284, 0.1559235155582428, -0.2688027024269104, -0.00670661311596632, -0.017626943066716194, 0.049053966999053955, -0.08926300704479218, -0.14220304787158966, 0.0011527867754921317, 0.06076961010694504, 0.0385499969124794, -0.04934787377715111, -0.09452322125434875, 0.029412420466542244, 0.02792883664369583, -0.033126864582300186, -0.03482041507959366, 0.034004732966423035, 0.026228390634059906, -0.003986837342381477, -0.08155594021081924, -0.04035009816288948, 0.06680171936750412, 0.04878764972090721, 0.15126393735408783, -0.10045745223760605, -0.12683874368667603, -0.04055626690387726, 0.09501626342535019, -0.05497942119836807, 0.0403236448764801, -0.30677807331085205, -0.2139153927564621, -0.07814791053533554, -0.03972989320755005, -0.12586413323879242, 0.07627101242542267, 0.03460896760225296, -0.12550455331802368, -0.11564639955759048, -0.10415682196617126, -0.024845238775014877, 0.20580057799816132, -0.10805636644363403, -0.33631432056427, -0.05294928327202797, 0.03592609241604805, 0.1564440131187439, 0.02596324495971203, -0.4104999005794525, 0.03670108690857887, 0.055451810359954834, 0.08007713407278061, -0.1292509287595749, 0.09129161387681961, 0.02646864578127861, 0.033229898661375046, 0.15484711527824402, 0.13180674612522125, 0.02661464922130108, 0.05949738621711731, -0.3782467246055603, -0.08480715751647949, -0.048464592546224594, -0.0798480361700058, 0.018179161474108696, 0.21972426772117615, 0.19501164555549622, -0.08977433294057846, -0.2424396276473999, 0.01132100448012352, -0.1017848551273346, 0.10136106610298157, 0.09004079550504684, 0.16977807879447937, -0.08934468775987625, -0.12144137918949127, 0.03609739616513252, 0.09612904489040375, 0.10516086965799332, -0.06308259814977646, 0.05981778725981712, 0.012464861385524273, 0.08671770989894867, -0.09530182927846909, -0.11300283670425415, -0.11132364720106125, -0.053336773067712784, 0.06692556291818619, 0.34815528988838196, -0.008708233013749123, 0.07659465819597244, 0.03210751712322235, 0.03582015633583069, 0.14093026518821716, -0.07716960459947586, 0.08463326096534729, -0.06119835749268532, -0.07554332911968231, 0.09060624241828918, 0.08528898656368256, -0.09822291880846024, -0.0455450564622879, -0.08269218355417252, -0.0410778746008873, -0.014237774536013603, -0.011111625470221043, 0.09212491661310196, 0.007928241044282913, -0.03250795602798462, 0.10542666167020798, 0.04282055422663689, 0.286701500415802, 0.0033581138122826815, 0.2897109389305115, -0.12032908946275711, 0.16759003698825836, -0.1781829297542572, 0.05814371630549431, -0.09421976655721664, 0.18175850808620453, -0.09670440107584, -0.07209046930074692, -0.03676113113760948, 0.128462016582489, 0.17082707583904266, 0.262101411819458, 0.058848023414611816, 0.2569851875305176, -0.09695597738027573, 0.03557613119482994, 0.1595250368118286, 0.02963734045624733, -0.0815344899892807, -0.150227889418602, -0.12473185360431671, -0.2049465775489807, -0.30759599804878235, 0.11831294745206833, -0.0951591357588768, 0.03056076355278492, 0.008481493219733238, -0.03962993994355202, -0.006181582808494568, 0.04879935085773468, 0.2690907120704651, -0.19652721285820007, -0.06665074080228806, 0.09063924103975296, 0.0089423144236207, -0.15790128707885742, -0.2531159520149231, 0.17261329293251038, -0.16960975527763367, -0.03304487094283104, -0.0012128263479098678, 0.1382085531949997, 0.2651872932910919, -0.10819060355424881}, {0.0632295161485672, 0.05065711587667465, 0.10956791043281555, 0.0865008607506752, 0.3558000922203064, -0.12328742444515228, 0.151274174451828, -0.05035718157887459, 0.18554413318634033, -0.01906515099108219, -0.052559394389390945, 0.10539280623197556, 0.04319515824317932, 0.020755579695105553, 0.3027403950691223, -0.04783814772963524, 0.1989632099866867, -0.14840126037597656, -0.024934962391853333, 0.1532028615474701, 0.15437261760234833, 0.07924742996692657, 0.046156980097293854, 0.052328094840049744, -0.21253602206707, -0.015352600254118443, -0.03950027748942375, -0.017481429502367973, 0.0859798863530159, -0.11481776088476181, 0.04711025953292847, 0.08983435481786728, -0.2025851458311081, -0.08506597578525543, -0.15630891919136047, -0.1419423520565033, 0.04537633806467056, 0.010066112503409386, -0.04293319582939148, 0.004600598476827145, -0.04950632154941559, 0.01404883898794651, 0.19413956999778748, 0.10406824946403503, -0.07475926727056503, -0.10110970586538315, -0.13317468762397766, -0.03563499450683594, -0.09258253872394562, -0.08234201371669769, -0.09651666134595871, -0.02093259058892727, -0.0037069120444357395, -0.1484745591878891, -0.014163228683173656, -0.1346319019794464, 0.05786876380443573, -0.0658516213297844, -0.07123240828514099, -0.017234835773706436, 0.030309975147247314, 0.05844423919916153, 0.05705202370882034, 0.03692908585071564, 0.027138935402035713, 0.10742637515068054, -0.05351140722632408, -0.08739084005355835, -0.31624874472618103, 0.060930270701646805, -0.01777493953704834, 0.041049424558877945, 0.09871082752943039, 0.13624486327171326, 0.20065860450267792, 0.19690798223018646, -0.24019166827201843, 0.05823345482349396, 0.3535335659980774, 0.0736820176243782, 0.03361564129590988, 0.053576525300741196, 0.3624304533004761, -0.04390227422118187, 0.09662648290395737, 0.05084007605910301, 0.11336202919483185, 0.00964080449193716, -0.04804200306534767, -0.010901663452386856, 0.032591622322797775, 0.2229223996400833, 0.006253325380384922, -0.14256232976913452, -0.10089186578989029, 0.09512168169021606, -0.005911952815949917, 0.14005902409553528, -0.11563483625650406, -0.15296489000320435, 0.12952439486980438, 0.13676761090755463, 0.19951720535755157, -0.016606539487838745, -0.08772319555282593, -0.17668722569942474, -0.0007828663801774383, -0.19227224588394165, -0.04901275411248207, -0.017465582117438316, 0.06653749197721481, 0.09585708379745483, -0.040289826691150665, -0.053069841116666794, 0.18279929459095, 0.015248019248247147, 0.022547351196408272, -0.007419428788125515, 0.05473954975605011, 0.13410598039627075, -0.13645824790000916, -0.2284557968378067, 0.013830780051648617, -0.00637256633490324, 0.1607200652360916, -0.16088934242725372, -0.14293597638607025, 0.07582864165306091, -0.07034635543823242, -0.028467873111367226, -0.04102249816060066, -0.0349106565117836, 0.030070902779698372, -0.1836422085762024, -0.04785244166851044, 0.08242389559745789, 0.17879296839237213, 0.097883440554142, 0.04221942648291588, 0.09840512275695801, -0.10118523985147476, 0.1311623901128769, -0.1165926605463028, 0.01832536794245243, -0.003478556638583541, -0.01286494079977274, -0.038414131850004196, 0.22920937836170197, -0.12143302708864212, 0.1476425677537918, -0.08703955262899399, -0.09252316504716873, -0.0954500362277031, -0.04475822299718857, -0.1877633035182953, 0.01818344183266163, 0.060981858521699905, -0.425431489944458, -0.04230860248208046, 0.0034474709536880255, 0.17102722823619843, -0.09144280850887299, 0.2064247876405716, 0.108257956802845, -0.15115372836589813, 0.20636922121047974, 0.004281326197087765, 0.14956147968769073, 0.09474185854196548, -0.017551768571138382, -0.2652585506439209, -0.10021466761827469, -0.0993661880493164, -0.03981233388185501, -0.29780054092407227, 0.29743310809135437, 0.0661652684211731, -0.020217258483171463, 0.10155124962329865, -0.18539831042289734, -0.007460607681423426, -0.1841491162776947, 0.09550240635871887, -0.09532193839550018, 0.10935492068529129, -0.048812542110681534, 0.11962613463401794, 0.10127828270196915, -0.12912367284297943, -0.05089379847049713, 0.06644140928983688, 0.09129578620195389, 0.019379625096917152, -0.10427749156951904, 0.08991242945194244, 0.12994709610939026, 0.032811034470796585, -0.0339110903441906, -0.05340507999062538, -0.10360944271087646, -0.09930595010519028, 0.1053868755698204, -0.17120012640953064, -0.10328099876642227, -0.061441320925951004, -0.007929299026727676, 0.19149893522262573, 0.13774457573890686, 0.1253879964351654, -0.021066466346383095, -0.02449638955295086, 0.1783609688282013, 0.1857338398694992, 0.3550957441329956, 0.029564877972006798, 0.03591584041714668, -0.005681069567799568, -0.02201125957071781, 0.040885452181100845, 0.003870480926707387, 0.09073158353567123, 0.10420109331607819, 0.08930577337741852, 0.11736933887004852, -0.05160757526755333, 0.08872845768928528, -0.13123191893100739, 0.1115211546421051, 0.015229504555463791, 0.07225555926561356, -0.11352364718914032, -0.0740356519818306, 0.06947788596153259, -0.20308995246887207, -0.1229754313826561, -0.23050588369369507, 0.048088688403367996, 0.09950797259807587, 0.02936835028231144, -0.0579555481672287, 0.013337613083422184, -0.030090145766735077, 0.12615783512592316, 0.08504758775234222, -0.021301427856087685, -0.04392286390066147, -0.09703379124403, 0.01954956352710724, -0.2760624885559082, 0.06520764529705048, -0.010131093673408031, -0.1415601223707199, 0.07014931738376617, 0.014691716991364956, 0.30071166157722473, -0.2479156106710434}, {-0.06965026259422302, -0.013932643458247185, 0.0385114811360836, 0.08093654364347458, 0.04304327070713043, 0.08164265751838684, -0.04251062124967575, -0.2037147730588913, 0.15953975915908813, -0.3095047175884247, -0.15938514471054077, -0.1577751487493515, 0.1098974198102951, 0.15302133560180664, 0.20244960486888885, 0.16748455166816711, 0.09676547348499298, 0.3001070022583008, 0.13213111460208893, -0.15182532370090485, 0.3328658640384674, 0.004542612936347723, -0.02457529865205288, 0.07963444292545319, -0.0274338535964489, -0.036914024502038956, -0.053672391921281815, -0.02040250599384308, 0.06499390304088593, 0.03292000666260719, -0.18511879444122314, 0.048635419458150864, 0.1032060831785202, 0.00770802004262805, -0.35117751359939575, 0.07576113194227219, -0.09905722737312317, -0.06954239308834076, 0.11678212881088257, -0.07884864509105682, 0.09867575764656067, 0.15452006459236145, 0.10093087702989578, -0.03867525979876518, -0.394792377948761, -0.1132201999425888, -0.01707741618156433, 0.24717700481414795, -0.21853914856910706, 0.1653415560722351, 0.20362381637096405, -0.15501169860363007, -0.05272394418716431, -0.1979193538427353, 0.0891108512878418, -0.14353632926940918, -0.11272354423999786, 0.07952027022838593, -0.006329563912004232, -0.035935938358306885, -0.09639372676610947, 0.011140973307192326, -0.05847613885998726, 0.18576152622699738, 0.197016641497612, -0.07029665261507034, 0.0016459161415696144, 0.06838902086019516, -0.09039029479026794, 0.04121990129351616, 0.04123397916555405, -0.13693028688430786, 0.23987509310245514, 0.060820598155260086, 0.30993983149528503, -0.08127754926681519, -0.12601874768733978, 0.12574753165245056, -0.2071329653263092, 0.10715696960687637, -0.06187712028622627, -0.09143401682376862, 0.02393808402121067, -0.11314348131418228, -0.17888915538787842, -0.02177589200437069, -0.04420192167162895, 0.013754873536527157, 0.1842220574617386, -0.1567082703113556, -0.06701260060071945, -0.012190272100269794, -0.06212802603840828, 0.08487394452095032, 0.07713823765516281, -0.03504834324121475, 0.0632418841123581, -0.35058459639549255, 0.05242988467216492, -0.20323915779590607, 0.05970015376806259, -0.18184705078601837, -0.19047465920448303, 0.05858916416764259, 0.13150320947170258, -0.36504194140434265, -0.18120363354682922, 0.11746640503406525, -0.05627492070198059, -0.010619308799505234, -0.09453379362821579, 0.2576095163822174, 0.09231813251972198, 0.040017303079366684, -0.19079694151878357, 0.015580198727548122, 0.16147641837596893, 0.004209036938846111, 0.08810410648584366, -0.046839311718940735, 0.20468896627426147, 0.18182022869586945, 0.050416894257068634, 0.08592914789915085, 0.09990033507347107, 0.10487986356019974, -0.050680018961429596, -0.22483383119106293, 0.08529167622327805, -0.13322405517101288, -0.03848795220255852, -0.13671071827411652, 0.02603289671242237, 0.016066640615463257, 0.11912939697504044, 0.1241418719291687, 0.0260707326233387, 0.07918138802051544, 0.03264720365405083, 0.050839755684137344, -0.2211693525314331, -0.026448553428053856, -0.007207796443253756, 0.001430463045835495, 0.32146382331848145, 0.1385483294725418, -0.1136789470911026, -0.16331113874912262, 0.031788360327482224, -0.06815756857395172, 0.32244545221328735, -0.09047035872936249, 0.0966026708483696, 0.05816509947180748, -0.053705304861068726, 0.22185127437114716, 0.0500323511660099, 0.018372442573308945, 0.027065375819802284, 0.011960756033658981, 0.05487094447016716, 0.031858235597610474, -0.08510047942399979, 0.15894527733325958, 0.01912403292953968, -0.17666932940483093, -0.030550438910722733, -0.013264243490993977, -0.03245803713798523, 0.055788323283195496, 0.1460401564836502, -0.12133204191923141, 0.05850055813789368, 0.09905695170164108, -0.24805735051631927, 0.14404228329658508, -0.006497182417660952, -0.018601346760988235, 0.16875071823596954, -0.02804017998278141, -0.06700444966554642, -0.08453131467103958, -0.0308933574706316, 0.03113953210413456, -0.2916713058948517, -0.25535252690315247, 0.12224763631820679, -0.02492010034620762, 0.07888320088386536, 0.07881641387939453, -0.11905223876237869, -0.08161093294620514, 0.05920180305838585, -0.08878230303525925, 0.12511876225471497, -0.16877220571041107, 0.04948094114661217, -0.045370109379291534, -0.06781613081693649, 0.10243017226457596, 0.1813892424106598, 0.024742627516388893, 0.3172432482242584, 0.2353430837392807, -0.0034585006069391966, -0.051707468926906586, 0.039408307522535324, -0.16314320266246796, -0.04212868958711624, -0.04821892827749252, 0.12368902564048767, 0.020274745300412178, 0.09948309510946274, -0.39528122544288635, -0.17379401624202728, -0.03920949995517731, -0.15808705985546112, -0.03125651925802231, -0.18700233101844788, 0.23072034120559692, 0.02268378995358944, -0.044310539960861206, 0.004321179818361998, 0.18180304765701294, 0.19299417734146118, 0.13491731882095337, 0.0972202718257904, -0.13226506114006042, 0.3224770426750183, -0.01247346680611372, 0.028137393295764923, 0.004845867399126291, -0.13742457330226898, -0.14785726368427277, -0.014485999010503292, -0.06958553940057755, -0.08747299760580063, -0.006554626394063234, -0.020532207563519478, -0.12087807059288025, 0.11323118209838867, 0.022749722003936768, 0.074697345495224, -0.273786336183548, -0.08386031538248062, 0.08255912363529205, -0.04905858263373375, -0.058434080332517624, 0.15510012209415436, 0.052568674087524414, 0.0668865442276001, 0.31365063786506653, -0.009345969185233116, 0.27516549825668335, -0.10924209654331207, 0.1137990802526474}, {0.030331891030073166, 0.059875231236219406, 0.12102939933538437, 0.08389969170093536, 0.1760988086462021, -0.021012939512729645, -0.18310844898223877, 0.24980102479457855, 0.2678673267364502, -0.16087117791175842, -0.11525055766105652, 0.4180097281932831, -0.20906734466552734, -0.11221155524253845, -0.08047462999820709, 0.21427494287490845, 0.09610901027917862, -0.21974265575408936, -0.23594146966934204, 0.097300685942173, -0.10985452681779861, -0.06954081356525421, 0.15376801788806915, -0.024661902338266373, -0.07645260542631149, -0.07487890124320984, -0.08948536217212677, 0.15392985939979553, -0.06752010434865952, -0.014859400689601898, 0.037407487630844116, 0.09477506577968597, -0.0612921342253685, 0.05048287287354469, -0.2513187825679779, -0.06266649812459946, -0.060330115258693695, 0.04508310928940773, -0.2817158102989197, 0.051017243415117264, 0.0952986627817154, -0.05198076367378235, 0.03796639293432236, -0.21172237396240234, -0.3696601986885071, 0.21713565289974213, -0.017465412616729736, 0.19778087735176086, 0.08766277879476547, -8.797956979833543e-06, 0.0843782126903534, -0.17135699093341827, 0.036156803369522095, 0.0046241264790296555, 0.046626169234514236, -0.11900148540735245, 0.16281916201114655, 0.04097476601600647, -0.06796450167894363, -0.01877599023282528, -0.06975266337394714, 0.19397060573101044, -0.13530775904655457, -0.051392894238233566, 0.09428434073925018, -0.04755285754799843, -0.21614813804626465, 0.11136609315872192, 0.1617974042892456, -0.1073179692029953, 0.08453599363565445, -0.008874332532286644, -0.01087169162929058, 0.03580606356263161, -0.7199934720993042, 0.037724267691373825, -0.14088749885559082, -0.5531855821609497, -0.06403305381536484, 0.10278470069169998, -0.08333572000265121, 0.11911311000585556, -0.17128700017929077, -0.07972510159015656, 0.05725185573101044, 0.15459789335727692, 0.043013691902160645, -0.06153401359915733, 0.03028055466711521, 0.08287014812231064, 0.11293187737464905, -0.13276982307434082, -0.09295392036437988, 0.070749931037426, -0.1535637527704239, -0.0162131916731596, -0.047272782772779465, -0.006310014054179192, 0.12651030719280243, -0.29133278131484985, -0.022062158212065697, 0.02659846469759941, -0.09154106676578522, 0.1514236181974411, 0.04300127923488617, -0.6188345551490784, -0.007595784030854702, 0.0230691097676754, 0.06939971446990967, -0.2009449601173401, 0.10860384255647659, 0.05042898654937744, 0.02421657182276249, 0.044978249818086624, -0.009347403421998024, -0.05952950567007065, -0.006499920971691608, -0.07015610486268997, -0.009038114920258522, 0.04937570169568062, -0.04699648916721344, -0.032694585621356964, -0.18061968684196472, -0.08990529924631119, 0.006836977321654558, 0.040156006813049316, -0.07747169584035873, 0.07727450877428055, 0.08277049660682678, 0.07977055758237839, 0.0526968315243721, 0.10948165506124496, 0.1016797125339508, 0.06032863259315491, -0.1383281946182251, -0.03125721216201782, 0.0014406614936888218, -0.21625083684921265, -0.11116965860128403, -0.010511061176657677, -0.2570556700229645, 0.08648540824651718, -0.07943113893270493, -0.11782369017601013, -0.20660658180713654, 0.04366963356733322, 0.016525812447071075, -0.004641937557607889, -0.07836847752332687, -0.04646315053105354, 0.0931709036231041, -0.12694557011127472, 0.1097756177186966, -0.22980497777462006, 0.16622896492481232, 0.15397019684314728, 0.01295642089098692, 0.18506552278995514, 0.023033207282423973, -0.14289483428001404, 0.05047277361154556, -0.17039208114147186, -0.06271765381097794, 0.17640754580497742, 0.05459911748766899, 0.010822954587638378, -0.02864139899611473, -0.0698615238070488, -0.07302150875329971, 0.033008724451065063, 0.1538679450750351, 0.05800878629088402, -0.07611867040395737, 0.0790892243385315, 0.08663798868656158, 0.06675226986408234, -0.08120144158601761, 0.12966318428516388, 0.10442111641168594, 0.02240312658250332, 0.010754805989563465, 0.26480013132095337, -0.14855195581912994, 0.11202263832092285, 0.016284767538309097, 0.06153477355837822, -0.12881670892238617, 0.06856539100408554, -0.036380842328071594, -0.06020215153694153, -0.21938487887382507, 0.07258466631174088, -0.09483280032873154, -0.07794743031263351, 0.07767807692289352, -0.029889965429902077, 0.0995921716094017, 0.03347757086157799, 0.02404789626598358, -0.13738596439361572, 0.06911317259073257, 0.06352808326482773, 0.11021069437265396, 0.17979015409946442, 0.007047788240015507, -0.07991097867488861, -0.15329968929290771, 0.20170994102954865, 0.27054259181022644, -0.05357876047492027, 0.14019988477230072, -0.12632939219474792, 0.0979246124625206, 0.11291525512933731, 0.017687195912003517, 0.021395210176706314, 0.1776547133922577, 0.05371656268835068, 0.04788542538881302, 0.005635352339595556, -0.22755688428878784, 0.14298115670681, 0.024295350536704063, -0.16288194060325623, -0.09963223338127136, -0.1290132850408554, -0.11595317721366882, 0.005394269246608019, -0.25045523047447205, -0.05512863025069237, 0.09389578551054001, -0.2195780724287033, -0.19938987493515015, 0.04499488323926926, 0.0725008100271225, -0.11691496521234512, -0.17122061550617218, 0.07505977898836136, -0.029773438349366188, -0.4080844521522522, 0.05440890043973923, -0.04486672952771187, -0.1699480414390564, -0.018358658999204636, 0.08545048534870148, -0.02342982590198517, -0.06997642666101456, 0.05973502993583679, 0.058292511850595474, 0.024741550907492638, 0.1388334184885025, -0.14235487580299377, -0.01347600668668747, 0.07339894026517868, -0.17593170702457428, -0.040214769542217255}, {-0.034777723252773285, -0.3304319977760315, 0.05474153533577919, -0.08491332083940506, 0.24664591252803802, 0.019913533702492714, 0.385701984167099, -0.03332715854048729, 0.30776792764663696, 0.0009930036030709743, -0.026789715513586998, 0.2609279155731201, 0.0738612562417984, 0.0012453884119167924, 0.0743413195014, -0.04788322001695633, 0.17336741089820862, -0.04889577627182007, 0.18143056333065033, 0.29465800523757935, -0.142617329955101, 0.007080915383994579, -0.1237475797533989, 0.12527650594711304, 0.14301960170269012, 0.20005519688129425, -0.12879261374473572, -0.2566579580307007, 0.26795053482055664, 0.2277490347623825, 0.054809339344501495, -0.18127205967903137, 0.1004505455493927, -0.19339223206043243, 0.12857769429683685, -0.07032477110624313, -0.09123421460390091, 0.05469712242484093, -0.07576056569814682, 0.1266896277666092, 0.07914479821920395, -0.06306002289056778, 0.14600585401058197, -0.0069938283413648605, -0.274139940738678, 0.19829760491847992, -0.059898823499679565, 0.05010824277997017, -0.05275658145546913, -0.0668095201253891, 0.16154715418815613, 0.096320241689682, -0.3008950352668762, 0.18268133699893951, 0.08741603046655655, 0.23823519051074982, -0.22638578712940216, -0.28057074546813965, 0.0871204286813736, 0.1703542172908783, -0.055665597319602966, -0.18906153738498688, -0.13388769328594208, -0.019928889349102974, 0.02295578643679619, 0.1829081028699875, -0.0007289413479156792, 0.015625234693288803, 0.025520477443933487, 0.24047549068927765, 0.04192667827010155, -0.21349658071994781, 0.18449103832244873, 0.00538341561332345, -0.22608068585395813, -0.10625685006380081, -0.05328367277979851, -0.1744474172592163, 0.3406597971916199, -0.05104529857635498, 0.034091461449861526, 0.19946928322315216, 0.2501342296600342, -0.21038523316383362, 0.0364791601896286, -0.3171796202659607, 0.056308019906282425, 0.061304859817028046, -0.16074547171592712, 0.0470978245139122, -0.17187821865081787, -0.21574616432189941, -0.04467864707112312, 0.04426919296383858, 0.20371153950691223, -0.23813007771968842, 0.022599734365940094, -0.03204231709241867, 0.050006091594696045, -0.12626518309116364, -0.04647231474518776, 0.016353005543351173, -0.2255626916885376, -0.23573285341262817, -0.0325627438724041, -0.8910722136497498, -0.2073340266942978, 0.2113707959651947, -0.01432921551167965, 0.08376326411962509, 0.05431528389453888, 0.13138365745544434, 0.027973705902695656, 0.018337957561016083, 0.32374241948127747, -0.08751586824655533, 0.003526894375681877, 0.18724849820137024, -0.07711397856473923, -0.11981731653213501, 0.17555345594882965, 0.11635265499353409, 0.1766359955072403, -0.3894599378108978, -0.05790822207927704, 0.08534212410449982, -0.06690460443496704, 0.061521977186203, 0.018925851210951805, -0.30123022198677063, 0.13337984681129456, 0.16101114451885223, -0.32011765241622925, 0.0843811184167862, 0.1666366010904312, -0.06856667995452881, 0.08618805557489395, 0.008854875341057777, 0.07412831485271454, 0.17384964227676392, -0.11976787447929382, -0.08376223593950272, -0.16750799119472504, 0.128381609916687, -0.040684230625629425, -0.3119506239891052, -0.24363389611244202, -0.031497981399297714, -0.12334652245044708, -0.057806409895420074, -0.1260623037815094, -0.04486311227083206, -0.15384866297245026, 0.004695037845522165, -0.16953136026859283, 0.19832812249660492, 0.18446826934814453, 0.008891604840755463, -0.1232394278049469, 0.15514549612998962, -0.15390229225158691, -0.0006587624084204435, -0.10272224992513657, 0.051688000559806824, -0.08712486922740936, -0.09434294700622559, 0.37544435262680054, 0.2019675225019455, -0.10096810013055801, -0.050931092351675034, 0.21494664251804352, -0.04900886490941048, 0.024922633543610573, -0.1249852254986763, -0.09411545097827911, -0.09099361300468445, -0.003792259143665433, -0.20153237879276276, 0.12896358966827393, 0.0650944784283638, -0.09996744990348816, 0.3686268627643585, 0.034148406237363815, -0.28522828221321106, 0.12347999215126038, 0.008600854314863682, 0.2240118533372879, 0.04120080918073654, -0.08534034341573715, -0.07775278389453888, 0.012148507870733738, 0.25823333859443665, 0.09292180091142654, 0.005965172313153744, 0.14457900822162628, -0.2129300981760025, 0.14951327443122864, -0.07758642733097076, -0.15223993360996246, 0.006388332694768906, 0.14277029037475586, 0.02371697500348091, 0.08386116474866867, 0.15144143998622894, -0.01473009493201971, 0.028146114200353622, -0.40941664576530457, -0.1151665523648262, -0.1833304613828659, -0.12260064482688904, -0.016311630606651306, 0.14068195223808289, 0.2599954903125763, 0.2841136157512665, 0.001120865112170577, -0.20469456911087036, -0.11918579787015915, 0.24024958908557892, 0.24417012929916382, -0.016372328624129295, 0.20158711075782776, -0.13112017512321472, -0.24157145619392395, 0.316784143447876, -0.15436574816703796, 0.235151007771492, 0.2724553644657135, 0.06485982984304428, -0.014450150541961193, -0.12487253546714783, -0.16923445463180542, 0.16651271283626556, -0.26248374581336975, 0.28406232595443726, 0.33286574482917786, -0.1242576465010643, -0.07863961905241013, -0.09353327006101608, -0.046452153474092484, -0.2921527922153473, -0.06923763453960419, 0.2567608952522278, 0.07047221064567566, 0.12317702174186707, 0.07455676048994064, -0.06150078773498535, 0.01350194402039051, 0.018320320174098015, -0.151119664311409, -0.0777953565120697, -0.07733388245105743, -0.0031775343231856823, 0.05992930382490158, 0.2369816154241562, 0.3174895644187927, 0.1698431819677353}, {0.08148892223834991, -0.07020138949155807, 0.141590878367424, -0.10060027241706848, 0.09200259298086166, 0.16146734356880188, -0.019120845943689346, -0.20404133200645447, 0.015993373468518257, -0.14163243770599365, -0.004160827957093716, 0.12990812957286835, 0.04277320206165314, 0.06811212748289108, 0.06090836226940155, -0.1092989593744278, 0.0842212587594986, -0.23605655133724213, -0.07676808536052704, 0.02185773476958275, -0.11007288098335266, -0.033638130873441696, -0.013995785266160965, 0.14108465611934662, -0.10999048501253128, -0.07528209686279297, -0.06195120885968208, -0.06949031352996826, 0.21266567707061768, 0.04298437386751175, -0.05496567115187645, 0.2624748647212982, -0.06110512092709541, 0.15052078664302826, 0.025029342621564865, 0.010677923448383808, 0.08540136367082596, -0.10701491683721542, -0.16881434619426727, -0.026012087240815163, 0.0071545010432600975, 0.07101815193891525, 0.016521301120519638, -0.038597434759140015, -0.138240247964859, 0.18411579728126526, -0.17259439826011658, 0.07242053002119064, -0.005141985137015581, -0.2594304084777832, 0.03594954311847687, -0.053229574114084244, 0.14682820439338684, -0.12869100272655487, 0.10532736778259277, -0.16629719734191895, -0.1122654378414154, 0.13564638793468475, -0.15733325481414795, 0.019167549908161163, -0.0749482735991478, -0.079202800989151, 0.35511514544487, -0.08335565030574799, -0.15276433527469635, 0.23539435863494873, 0.12482460588216782, 0.16427461802959442, -0.15321983397006989, 0.05085724964737892, -0.06166886165738106, 0.17144852876663208, 0.31255099177360535, -0.08009293675422668, -0.1733884960412979, 0.056822244077920914, 0.1648404598236084, 0.04216012731194496, -0.06641683727502823, -0.07366036623716354, -0.1265566200017929, -0.06202784180641174, -0.21118347346782684, -0.07104086875915527, 0.0971774011850357, 0.15150418877601624, 0.13937318325042725, 0.253311425447464, 0.10767033696174622, -0.08747229725122452, 0.032575033605098724, 0.033814720809459686, 0.20631243288516998, -0.020918115973472595, -0.13367798924446106, 0.07909147441387177, -0.07285214960575104, 0.021543247625231743, 0.14044629037380219, -0.0027010999619960785, 0.2232288420200348, -0.06833521276712418, -0.475391149520874, 0.0466499887406826, 0.1116936057806015, -0.0887097492814064, 0.05754159018397331, 0.15602263808250427, -0.24390023946762085, -0.06233799457550049, -0.0009176277671940625, -0.12569798529148102, -0.25335949659347534, 0.17606468498706818, -0.1423589587211609, -0.004825398791581392, -0.14698979258537292, -0.1572299301624298, 0.03562936186790466, -0.26657170057296753, -0.21949219703674316, 0.031489890068769455, -0.1536015123128891, 0.11340946704149246, 0.08420363813638687, -0.02357388660311699, 0.05335976928472519, 0.19159682095050812, -0.04652921482920647, 0.019534824416041374, -0.13350839912891388, 0.0746302455663681, -0.40867042541503906, -0.12301737815141678, -0.01812414824962616, -0.27125686407089233, 0.01454567164182663, -0.058922477066516876, -0.2105625718832016, 0.06899668276309967, -0.210236057639122, -0.12750567495822906, -0.048636894673109055, -0.07825079560279846, 0.029250632971525192, -0.11811120063066483, -0.5882296562194824, 0.08057261258363724, -0.09391501545906067, -0.43262144923210144, -0.16046175360679626, 0.0182963740080595, -0.013901954516768456, 0.3383566737174988, 0.16426080465316772, 0.1567913293838501, 0.13954615592956543, 0.001717045553959906, -0.03118617832660675, -0.05691521242260933, 0.19108541309833527, 0.15492643415927887, 0.15254761278629303, 0.36482828855514526, 0.048670608550310135, 0.05146430432796478, 0.15804855525493622, -0.27637696266174316, -0.06933436542749405, 0.0492367148399353, -0.0049539548344910145, 0.13224543631076813, 0.15466737747192383, -0.03420237824320793, -0.07353564351797104, 0.028886137530207634, 0.12076759338378906, 0.01918381080031395, 0.04436039552092552, -0.09009894728660583, 0.14280007779598236, 0.22781561315059662, -0.0015831387136131525, -0.037452176213264465, -0.1445850431919098, 0.1795727014541626, -0.10867280513048172, -0.01679498702287674, -0.027635063976049423, 0.11927136778831482, 0.00638506980612874, 0.1652154177427292, 0.031627681106328964, 0.011576078832149506, 0.10465648770332336, 0.12597469985485077, 0.06572619825601578, -0.055817801505327225, 0.06309192627668381, 0.011687304824590683, -0.09848948568105698, 0.3182078003883362, -0.008474522270262241, 0.11434327065944672, -0.14917680621147156, -0.08063561469316483, -0.12917950749397278, 0.23482561111450195, 0.19561615586280823, 0.09842606633901596, 0.2261875420808792, -0.02643594704568386, 0.11564818769693375, 0.01104193925857544, -0.10346347093582153, 0.15112771093845367, -0.24835214018821716, 0.08836232125759125, 0.0551433339715004, -0.029920997098088264, -0.05914890021085739, 0.07573128491640091, 0.07750251144170761, -0.14877086877822876, -0.17958007752895355, -0.00022081943461671472, 0.1722826063632965, -0.08763032406568527, -0.0009729679441079497, -0.029471317306160927, -0.10172469913959503, -0.08139178901910782, -0.010240928269922733, 0.026874031871557236, -0.03364745154976845, 0.11415199935436249, 0.09806089848279953, -0.1891995221376419, 0.179474875330925, 0.006658143829554319, 6.796551315346733e-05, -0.11908464133739471, 0.050879575312137604, 0.269870400428772, -0.029121050611138344, -0.1543423980474472, 0.15489274263381958, 0.14989237487316132, -0.0970844179391861, 0.08583177626132965, 0.12988030910491943, -0.018459994345903397, -0.07507885247468948, 0.09071531891822815, 0.18453513085842133, -0.026495935395359993}, {-0.1802627444267273, 0.039111051708459854, 0.09251604974269867, 0.012195584364235401, -0.15942485630512238, -0.12399040907621384, -0.23738953471183777, -0.007833749055862427, 0.02748912014067173, -0.057247623801231384, 0.037520136684179306, -0.09250622987747192, 0.007869934663176537, 0.19378532469272614, 0.1428040713071823, -0.10545824468135834, -0.11845725774765015, -0.04084175452589989, -0.010025294497609138, -0.2550833821296692, -0.0578901544213295, -0.18991908431053162, 0.14380963146686554, 0.13201555609703064, 0.11191675066947937, -0.05628630891442299, 0.2583812177181244, -0.10412148386240005, -0.1532512903213501, -0.06589639186859131, -0.06295185536146164, 0.003726760158315301, -0.0002161937882192433, -0.1315000355243683, -0.05237503722310066, -0.13193166255950928, -0.16030068695545197, -0.12054752558469772, 0.19378221035003662, -0.08969026058912277, 0.0670190304517746, -0.06792239844799042, -0.08195378631353378, 0.11531591415405273, 0.26212894916534424, 0.12376988679170609, 0.17900480329990387, 0.024071309715509415, 0.03384730964899063, -0.10515192151069641, -0.17508772015571594, 0.0868161991238594, -0.1550820767879486, 0.21565526723861694, -0.10125277936458588, 0.09139689058065414, -0.04127516224980354, 0.09984391182661057, 0.053924303501844406, 0.20671780407428741, 0.010542737320065498, 0.035936564207077026, -0.004384439438581467, 0.02032954804599285, 0.06436282396316528, 0.012342117726802826, -0.07465872913599014, -0.04956870526075363, 0.05710379406809807, -0.06046905741095543, 0.04832766577601433, 0.01705978438258171, 0.09166210889816284, -0.027026325464248657, -0.13832911849021912, -0.14230941236019135, -0.07110253721475601, 0.2613047957420349, 0.08280397206544876, 0.1249610185623169, -0.05521486699581146, -0.10612430423498154, -0.14172740280628204, -0.05263853445649147, -0.01199381798505783, 0.048034071922302246, -0.009410853497684002, -0.005312267225235701, -0.020244525745511055, 0.1222301498055458, 0.07597435265779495, 0.1493178904056549, -0.01865454763174057, 0.06131232529878616, -0.02406402677297592, -0.03148430213332176, -0.0014605859760195017, 0.04510798305273056, 0.2615263760089874, 0.09815629571676254, -0.13480858504772186, 0.26452580094337463, 0.3117428123950958, 0.004371763672679663, 0.0739240050315857, 1.9004119634628296, 0.054422859102487564, -0.020637881010770798, 0.18018198013305664, -0.1390504240989685, -0.15346573293209076, 0.01601240411400795, -0.07372581958770752, 0.019084595143795013, -0.010582847520709038, 0.09849467128515244, -0.03195011988282204, -0.01029663160443306, 0.13979001343250275, -0.13428646326065063, -0.008186789229512215, 0.07773654907941818, -0.020416805520653725, 0.08037310838699341, 0.15918764472007751, 0.08227065205574036, 0.0702441930770874, -0.14846691489219666, -0.004837220534682274, -0.06527075916528702, 0.14354491233825684, 0.07573248445987701, 0.09529222548007965, -0.18823406100273132, -0.017508311197161674, 0.30638688802719116, 0.1616562306880951, 0.17996713519096375, 0.06598462909460068, 0.22231629490852356, -0.009558227844536304, 0.025646410882472992, -0.12822763621807098, 0.13272783160209656, 0.06529957801103592, 0.024867147207260132, -0.038522038608789444, -0.04750761762261391, 0.290130615234375, -0.1357298046350479, 0.20508985221385956, 0.1386660635471344, 0.06645895540714264, -0.39708295464515686, 0.03998688980937004, 0.13942186534404755, 0.058437280356884, -0.09681128710508347, 0.09588391333818436, -0.5473693013191223, 0.12909160554409027, -0.03399698808789253, -0.026114648208022118, 0.1787300854921341, -0.42510491609573364, -0.05510389059782028, 0.03919705003499985, -0.17066437005996704, -0.12274634838104248, 0.052568838000297546, -0.04799265041947365, -0.10644910484552383, -0.09460362046957016, -0.1230548694729805, -0.06748936325311661, -0.05461043119430542, 0.053434547036886215, 0.11128054559230804, 0.07924021035432816, -0.17921920120716095, 0.09744399785995483, 0.20716863870620728, 0.1739829033613205, 0.23471933603286743, 0.10568486154079437, -0.14731818437576294, -0.5733771920204163, -0.03781930357217789, -0.11904007941484451, 0.11614328622817993, 0.05900734290480614, 0.11935863643884659, 0.09470725804567337, 0.14811880886554718, -0.0544973649084568, -0.11190120130777359, -0.09638770669698715, -0.2786227762699127, -0.21393416821956635, 0.12443022429943085, 0.19411280751228333, 0.01811584271490574, 0.04612550884485245, 0.016899308189749718, -0.10133776068687439, 0.016595924273133278, -0.1465708613395691, 0.10207197070121765, 0.1340746134519577, 0.0035344711504876614, -0.008709140121936798, 0.006787641439586878, -0.09431532025337219, -0.0089575732126832, -0.17193415760993958, 0.06802254915237427, -0.1186557337641716, 0.08652861416339874, -0.1462528258562088, 0.010022436268627644, -0.12198103219270706, 0.044495370239019394, 0.1322798728942871, -0.16588528454303741, 0.06733939796686172, -0.22525076568126678, -0.08746366947889328, -0.07951199263334274, -0.02317247912287712, -0.06341899186372757, 0.2387823909521103, 0.17680929601192474, -0.07769235223531723, 0.016894614323973656, 0.25972387194633484, 0.005970177240669727, -0.012559248134493828, 0.12875515222549438, 0.09984289109706879, 0.3598364293575287, -0.17655137181282043, 0.14945295453071594, 0.10204989463090897, 0.05600499361753464, -0.15736611187458038, 0.20520690083503723, 0.02057499997317791, 0.018605830147862434, 0.08012308180332184, 0.04715856537222862, 0.10398393869400024, 0.19268342852592468, 0.11239898204803467, 0.036475323140621185, 0.06380034983158112, 0.02142064832150936}, {-0.1308077722787857, -0.17076589167118073, 0.0066609736531972885, 0.060786791145801544, 0.02010880783200264, -0.06789374351501465, -0.16061894595623016, 0.027938777580857277, -0.0009504624758847058, 0.0756678506731987, 0.10567744076251984, -0.09067219495773315, -0.027752378955483437, -0.1942809522151947, -0.06529965996742249, 0.014533708803355694, -0.02283209003508091, -0.009930852800607681, -0.06554209440946579, -0.0034269951283931732, -0.015727682039141655, 0.04877820611000061, 0.03808534890413284, -0.03177348151803017, -0.09415923058986664, -0.09310075640678406, 0.10640237480401993, 0.04036750644445419, -0.022386306896805763, -0.05745725333690643, -0.05365677550435066, 0.08778074383735657, 0.19166934490203857, -0.016664862632751465, 0.1370576173067093, -0.04620996117591858, -0.030465468764305115, 0.2701696753501892, 0.22374272346496582, 0.04694338142871857, -0.05018886923789978, 0.03337623551487923, -0.08023059368133545, 0.008481712080538273, 0.003013297915458679, -0.1414414346218109, -0.03134310618042946, 0.08067130297422409, -0.1630314439535141, -0.15271008014678955, 0.011378566734492779, 0.0073367697186768055, 0.003218082943931222, -0.16105641424655914, 0.004708619322627783, 0.12194082140922546, -0.17584781348705292, -0.21028611063957214, 0.17053915560245514, 0.0266436580568552, -0.059736546128988266, 0.14394201338291168, 0.0458977036178112, 0.3547040820121765, -0.0262268278747797, -0.04223000630736351, -0.04111749678850174, -0.048476334661245346, -0.08185325562953949, 0.331431120634079, 0.08377999067306519, -0.04398111253976822, -0.15973789989948273, -0.06238635256886482, 0.03859884291887283, 0.07765372097492218, 0.008245379664003849, -0.09562216699123383, -0.14168104529380798, -0.023100323975086212, 0.023195959627628326, 0.031098511070013046, -0.04571223631501198, -0.2273504137992859, 0.0051467386074364185, 0.07952046394348145, -0.360090434551239, 0.16973881423473358, -0.145266592502594, -0.11666688323020935, 0.038638658821582794, -0.0821523517370224, 0.03352578356862068, 0.047631748020648956, 0.008288685232400894, -0.08583664149045944, -0.11504040658473969, -0.007294884417206049, 0.24552297592163086, -0.11893273890018463, -0.2165188491344452, -0.037788014858961105, -0.004359737038612366, 0.1857355237007141, 0.14922089874744415, -0.03889497369527817, 0.02850397676229477, 0.06946223229169846, 0.051774557679891586, 0.09676292538642883, 0.1387559324502945, 0.09028246998786926, 0.022861694917082787, 0.2792108654975891, -0.03568931296467781, 0.15387697517871857, 0.006961156614124775, 0.013399924151599407, 0.10842254012823105, -0.03150657191872597, -0.007648604456335306, 0.23795168101787567, 0.0176834836602211, -0.017275013029575348, -0.05525985360145569, -0.11891879141330719, 0.07146169245243073, 0.023051124066114426, 0.1709650158882141, -0.19489315152168274, 0.12467172741889954, 0.10005249083042145, 0.22045747935771942, -0.07849176973104477, 0.19137462973594666, 0.15606093406677246, 0.10494699329137802, 0.17459890246391296, -0.09435363113880157, 0.17340050637722015, 0.04882067069411278, 0.06737070530653, -0.05386881157755852, 0.06719896197319031, -0.08479153364896774, -0.06438790261745453, 0.01025041751563549, -0.11557646095752716, 0.2382880449295044, 0.26745226979255676, 0.07939412444829941, -0.10637465864419937, -0.15517976880073547, -0.11045366525650024, -0.009524589404463768, 0.06935370713472366, -0.05143257975578308, 0.21966075897216797, -0.161576509475708, 0.032334763556718826, -0.11191795021295547, -0.0732855573296547, 0.22034619748592377, -0.01260882243514061, -0.03563453257083893, -0.08143927901983261, 0.01643768511712551, 0.04689783975481987, -0.0070202406495809555, -0.03938566520810127, 0.017672469839453697, 0.22507372498512268, -0.12070350348949432, -0.03365451470017433, 0.1574484407901764, -0.04131202772259712, -0.05413540452718735, -0.07322801649570465, 0.12430677562952042, 0.20291908085346222, -0.01548673864454031, -0.0661330595612526, 0.16866284608840942, -0.2225901335477829, 0.15792542695999146, 0.004647351801395416, 0.05738155543804169, 0.15249477326869965, 0.1526334434747696, 0.1673451066017151, 0.00650365836918354, 0.08415251225233078, 0.052589625120162964, 0.057000331580638885, -0.22142355144023895, -0.1574217528104782, -0.11059378832578659, -0.04162020608782768, 0.034161947667598724, -0.09521260857582092, 0.0707264393568039, 0.051761526614427567, 0.0349113866686821, -0.12056532502174377, 0.007504219189286232, -0.26647281646728516, 0.06176931783556938, -0.07025846093893051, 0.0458313524723053, -0.1700531244277954, 0.1327366828918457, 0.29359155893325806, -0.016326822340488434, -0.01954260654747486, -0.09608012437820435, 0.004943999461829662, 0.18074995279312134, -0.06780063360929489, -0.06889865547418594, 0.013852201402187347, 0.09742461144924164, -0.15404456853866577, 0.18298989534378052, 0.18638549745082855, -0.009949385188519955, 0.16658443212509155, -0.03582337498664856, 0.057858213782310486, -0.024761894717812538, -0.10441075265407562, 0.11193888634443283, -0.031488463282585144, -0.007037419360131025, 0.024851171299815178, -0.0021898134145885706, 0.13882046937942505, -0.14431098103523254, 0.10830149799585342, -0.2142971158027649, -0.058857567608356476, -0.15711350739002228, 0.12288262695074081, -0.11284250766038895, -0.09294716268777847, 0.009874854236841202, -0.11600863933563232, -0.027530113235116005, -0.3079569935798645, -0.1365453004837036, 0.19551892578601837, 0.11558093875646591, -0.07083192467689514, 0.13907267153263092, 0.12421949952840805, 0.13324680924415588, -0.0007396035944111645}, {0.06629574298858643, -0.07760469615459442, 0.014462891034781933, 0.009659496136009693, 0.010323888622224331, 0.3179704546928406, 0.17984531819820404, 0.03275448828935623, 0.12299833446741104, -0.016637740656733513, -0.05111696571111679, -0.0014973293291404843, 0.09066402912139893, -0.06249519810080528, 0.08208198845386505, -0.009554826654493809, -0.023131532594561577, 0.14905013144016266, -0.07002531737089157, 0.06828685849905014, -0.06515687704086304, 0.02510332502424717, 0.0902227908372879, 0.047343604266643524, -0.19583894312381744, 0.01562582328915596, -0.048079948872327805, -0.0922812893986702, -0.15835361182689667, -0.26447343826293945, 0.052978817373514175, 0.01888006180524826, 0.0885998085141182, -0.06347474455833435, -0.1304776519536972, 0.05912843719124794, 0.19665339589118958, 0.11847878247499466, -0.18151095509529114, -0.03431713953614235, -0.2881986200809479, 0.00048353674355894327, -0.13972847163677216, 0.05762016400694847, 0.018909664824604988, -0.2506190836429596, -0.17142803966999054, -0.07184736430644989, 0.07103231549263, -0.21211081743240356, -0.0016868544043973088, -0.2990567982196808, -0.07743990421295166, -0.011979199014604092, 0.0454036146402359, -0.17514796555042267, 0.11771088093519211, -0.036297507584095, 0.11054765433073044, -0.11072811484336853, 0.14508529007434845, -0.13585734367370605, 0.12702853977680206, -0.003920160233974457, 0.06525194644927979, 0.11293688416481018, -0.10437432676553726, 0.0436495766043663, -0.1975337266921997, 0.14170436561107635, 0.06038838252425194, 0.011031207628548145, 0.17818239331245422, 0.10868086665868759, -0.023499183356761932, -0.00742168165743351, -0.025295622646808624, -0.2146826535463333, -0.11640587449073792, -0.09568379074335098, -0.2730708420276642, -0.09984102845191956, 0.20623306930065155, -0.011535362340509892, -0.0017009156290441751, 0.056483663618564606, 0.014051866717636585, 0.0198341216892004, -0.20985019207000732, 0.0015571959083899856, -3.829736670013517e-06, 0.20007333159446716, -0.12010011076927185, 0.063581183552742, -0.10274790972471237, -0.12024290859699249, 0.04382224380970001, -0.2290932536125183, 0.13475920259952545, -0.1562129557132721, 0.0420282818377018, 0.3070482313632965, -0.1283615678548813, -0.09118662029504776, -0.29251518845558167, 0.0189141146838665, -0.08801581710577011, 0.03430323675274849, -0.11305858194828033, 0.1979881078004837, -0.047309279441833496, -0.1379135698080063, 0.0844217985868454, 0.013029004447162151, -0.221963033080101, 0.2647784352302551, 0.18650095164775848, -0.08627794682979584, 0.05326233431696892, 0.12021977454423904, -0.11714199185371399, 0.1523258239030838, -0.2765692472457886, -0.3969121277332306, -0.05142245441675186, -0.027177438139915466, 0.1545889973640442, -0.16358798742294312, 0.22282660007476807, -0.016079029068350792, -0.051384929567575455, 0.17631474137306213, 0.015796953812241554, -0.2041708379983902, 0.13569296896457672, -0.28076061606407166, 0.09412302076816559, -0.02700677700340748, -0.10406122356653214, -0.1755901277065277, 0.02684277854859829, -0.17953556776046753, 0.04774348810315132, 0.052028778940439224, -0.10902265459299088, 0.022335046902298927, 0.23279224336147308, 0.18245822191238403, -0.018209366127848625, 0.029918229207396507, 0.05493166670203209, -0.051790203899145126, 0.0982392281293869, -0.15213575959205627, 0.10288695245981216, 0.03181099519133568, -0.018681593239307404, -0.011831174604594707, 0.11860613524913788, 0.04188111796975136, 0.013414851389825344, 0.18739819526672363, 0.21056224405765533, -0.007857982069253922, -0.035301316529512405, 0.06161041557788849, -0.030114520341157913, -0.03214344009757042, 0.020029423758387566, -0.2551714777946472, 0.1466357558965683, -0.2093164026737213, -0.006171477492898703, 0.13531306385993958, -0.038033511489629745, 0.08492040634155273, -0.00713301682844758, -0.01863250508904457, 0.21851204335689545, 0.17364077270030975, 0.2931309640407562, 0.09299737215042114, -0.18041597306728363, 0.08526364713907242, 0.07415040582418442, 0.03987465426325798, 0.0903666615486145, -0.10995122790336609, 0.1582745611667633, 0.11957360804080963, 0.10841747373342514, -0.10334949940443039, -0.2545684576034546, -0.22411911189556122, -0.14251749217510223, -0.07300686091184616, 0.2089083045721054, 0.20353597402572632, 0.024427326396107674, 0.21004237234592438, 0.025357134640216827, 0.004998629447072744, -0.12268683314323425, -0.15530842542648315, -0.15348009765148163, -0.013495633378624916, 0.15547071397304535, -0.005560610443353653, 0.008291302248835564, -0.046487271785736084, 0.08644353598356247, -0.0021514645777642727, 0.04352928325533867, 0.04407903552055359, -0.13702663779258728, -0.005706456024199724, 0.013420600444078445, 0.07578090578317642, 0.008837639354169369, 0.1891864389181137, 0.058050718158483505, 0.07794340699911118, 0.00938995461910963, -0.1161438450217247, -0.15177549421787262, -0.012972833588719368, 0.19751785695552826, -0.010508166626095772, 0.1585443615913391, 0.1002563089132309, -0.1226118803024292, -0.002624162007123232, 0.08824922889471054, 0.12993402779102325, -0.11584849655628204, 0.048186227679252625, -0.09881706535816193, 0.0844169482588768, -0.12637726962566376, -0.046711403876543045, -0.19899293780326843, 0.0544644333422184, 0.07137302309274673, -0.1352176070213318, -0.15734606981277466, 0.1282871514558792, 0.01656722091138363, 0.11829943209886551, -0.2163568139076233, -0.11715925484895706, -0.10445411503314972, 0.3075835406780243, 0.15890416502952576, 0.11260075867176056, 0.0030108473729342222, 0.098907969892025}, {0.0057788388803601265, 0.20215874910354614, 0.2542496919631958, 0.21792174875736237, 0.05487698316574097, 0.13523000478744507, -0.06515800952911377, 0.09286623448133469, -0.0449589341878891, 0.04988642781972885, -0.11254522949457169, 0.23811686038970947, 0.13248445093631744, -0.014254133217036724, 0.08125098049640656, 0.09260533004999161, 0.07141604274511337, 0.04038754850625992, -0.32477501034736633, -0.03619293496012688, 0.006755764130502939, 0.15323863923549652, 0.2292737364768982, 0.05303705483675003, -0.0350860171020031, -0.19510193169116974, 0.043561115860939026, -0.12790930271148682, -0.03759090229868889, -0.03784581646323204, -0.037598755210638046, -0.054360609501600266, -0.06356987357139587, 0.11777849495410919, 0.05326443910598755, 0.2082168161869049, 0.19018512964248657, 0.023125357925891876, -0.12385967373847961, 0.16254474222660065, -0.07023021578788757, -0.009047335013747215, -0.021644582971930504, 0.15116947889328003, 0.26135191321372986, 0.11982297897338867, 0.1274106502532959, -0.15042367577552795, 0.07455508410930634, 0.034362491220235825, 0.15220783650875092, -0.005046399310231209, 0.12587249279022217, -0.07665900141000748, 0.1650853157043457, 0.0822121724486351, 0.23130883276462555, 0.0841144323348999, 0.03851037472486496, -0.05046669393777847, -0.12960094213485718, -0.04276962950825691, 0.05977050960063934, -0.040471069514751434, -0.00022301478020381182, -0.08110596239566803, 0.0024923314340412617, 0.2320539504289627, 0.03749376907944679, -0.13873930275440216, 0.06146615371108055, -0.08297703415155411, -0.05394582822918892, -0.13410109281539917, 0.017564767971634865, -0.04320171847939491, 0.18647539615631104, -0.11773035675287247, -0.326579213142395, -0.13296234607696533, 0.1578867882490158, -0.06594188511371613, 0.06025422737002373, -0.22811421751976013, 0.041202642023563385, 0.20028014481067657, 0.007256319746375084, 0.08766771852970123, 0.16776219010353088, -0.1622498631477356, -0.019465824589133263, -0.024173500016331673, 0.09478702396154404, 0.15152090787887573, 0.0734698548913002, -0.11647406965494156, -0.02700139954686165, 0.06954959779977798, 0.29953712224960327, 0.3382378816604614, 0.10569075495004654, 0.04217777028679848, 0.12163788825273514, -0.11462201923131943, -0.021343115717172623, 0.4043402075767517, -0.14187054336071014, 0.06295594573020935, -0.04452204704284668, 0.05191904306411743, -0.20809143781661987, -0.05821674317121506, -0.06919779628515244, 0.003266426268965006, -0.026197876781225204, 0.20536741614341736, -0.022532271221280098, -0.11184114962816238, -0.0034764152951538563, 0.03160598874092102, -0.13778413832187653, 0.2019987404346466, -0.021248288452625275, 0.21007701754570007, 0.11991981416940689, -0.01982022263109684, -0.06468512862920761, -0.11073839664459229, 0.02998267486691475, 0.1519474983215332, 0.34579184651374817, -0.022430753335356712, 0.04559297487139702, -0.11593194305896759, 0.28369373083114624, -0.12414059787988663, -0.08989399671554565, -0.10468021780252457, -0.03726397827267647, -0.16422775387763977, -0.03546087071299553, -0.16812072694301605, 0.0028338574338704348, -0.2280157506465912, 0.11490295082330704, -0.003092678729444742, 0.06958543509244919, -0.12573617696762085, -0.08553292602300644, 0.07526139914989471, -0.20850017666816711, 0.09850651770830154, 0.008490298874676228, -0.01373275462538004, 0.04842501878738403, -0.04754963517189026, -0.07224546372890472, 0.13098639249801636, -0.13403534889221191, 0.09775852411985397, -0.01611674576997757, -0.0941549614071846, 0.041230689734220505, -0.08564741909503937, 0.0010666422313079238, -0.06864126771688461, -0.27673664689064026, -0.2583063840866089, -0.18535512685775757, 0.18480674922466278, -0.07893064618110657, 0.10730280727148056, 0.18168464303016663, 0.19955429434776306, 0.3160781264305115, -0.1658806949853897, 0.13370497524738312, 0.15043899416923523, -0.09046421945095062, 0.030517928302288055, 0.08612695336341858, -0.017657142132520676, -0.022776078432798386, -0.13278140127658844, 0.09218230843544006, 0.18730740249156952, 0.03933995962142944, 0.10176258534193039, 0.12952084839344025, -0.0267082117497921, 0.17650137841701508, -0.05071593075990677, -0.00396621972322464, 0.24844175577163696, -0.12534812092781067, 0.05114109814167023, -0.0210625808686018, -0.026440296322107315, -0.13142624497413635, -0.15084217488765717, 0.02772686444222927, -0.10259518027305603, -0.0922999233007431, -0.1275031715631485, -0.4007464647293091, 0.015921903774142265, 0.22148144245147705, 0.07673399895429611, -0.067659392952919, 0.053720761090517044, 0.05025473237037659, 0.09120763093233109, -0.30589208006858826, 0.15544936060905457, 0.21883399784564972, 0.21930041909217834, 0.09254053980112076, 0.02198377437889576, 0.16354374587535858, -0.1405743509531021, 0.11971062421798706, 0.10610345751047134, -0.09795444458723068, 0.16975101828575134, 0.17482838034629822, 0.10559504479169846, -0.25771570205688477, -0.08050063252449036, 0.06702131032943726, -0.06688861548900604, 0.009121851064264774, -0.07341035455465317, 0.04571298509836197, 0.08517202734947205, -0.3044416308403015, 0.18200676143169403, 0.28232309222221375, 0.05446184054017067, -0.2652393877506256, 0.5053439736366272, -0.19619229435920715, -0.2286088764667511, 0.19879454374313354, -0.061734117567539215, -0.04410212114453316, -0.07524064928293228, -0.12245160341262817, 0.18512898683547974, -0.1560630053281784, -0.13088035583496094, -0.03227384015917778, 0.025249365717172623, -0.14488601684570312, -0.09450340270996094, 0.0015111888060346246, 0.04009019955992699}, {-0.015141106210649014, -0.10022778064012527, -0.11858455836772919, 0.21884390711784363, 0.17416448891162872, -0.14001582562923431, -0.04117899015545845, 0.12503857910633087, 0.0027692292351275682, 0.016099775210022926, 0.02694675326347351, 0.113254114985466, -0.1340150237083435, -0.18513289093971252, 0.1874256283044815, 0.16796205937862396, 0.0665997639298439, 0.0962028056383133, -0.20348221063613892, 0.11507335305213928, -0.09487329423427582, 0.12971067428588867, 0.025277497246861458, -0.08095809817314148, -0.15292572975158691, -0.014455018565058708, 0.14287538826465607, -0.11025642603635788, -0.05080708488821983, -0.03955661877989769, 0.16784442961215973, 0.022119883447885513, 0.10050118714570999, 0.14291131496429443, 0.2794111669063568, 0.09288575500249863, 0.12116695940494537, 0.011603414081037045, 0.10574793070554733, 0.11438435316085815, 0.05226778984069824, -0.07608116418123245, -0.07023616135120392, -0.036943092942237854, -0.15460553765296936, -0.06657760590314865, -0.08183656632900238, -0.026405885815620422, 0.10152833163738251, -0.11954227834939957, 0.12779952585697174, -0.1861986666917801, -0.11175786703824997, -0.07135330140590668, 0.013892358168959618, 0.07043712586164474, 0.2131316065788269, -0.031369201838970184, 0.019848637282848358, 0.008008494973182678, -0.042388562113046646, 0.13389022648334503, 0.3443931043148041, 0.041635483503341675, -0.0750504657626152, 0.11987746506929398, 0.08355381339788437, -0.03649338334798813, -0.015712348744273186, -0.06686408817768097, -0.23882363736629486, 0.05107437074184418, 0.03684081882238388, -0.11043216288089752, -0.013062096200883389, 0.031169770285487175, 0.0304271150380373, -0.07155639678239822, -0.0027194651775062084, -0.07970588654279709, 0.0878288522362709, 0.039898429065942764, -0.025334734469652176, -0.1065085381269455, 0.08210509270429611, 0.034665923565626144, -0.03250941261649132, 0.19807259738445282, 0.08304596692323685, -0.11175898462533951, -0.07673240453004837, 0.09659142792224884, 0.09540888667106628, -0.11770191043615341, 0.13953156769275665, -0.058895956724882126, -0.023717613890767097, 0.08904904127120972, -0.006043264176696539, 0.005599769297987223, -0.04384233430027962, -0.1977115273475647, -0.02894016169011593, 0.015991922467947006, -0.04675848037004471, -0.02397138625383377, -0.13215668499469757, -0.11747599393129349, 0.18834464251995087, -0.01382214855402708, 0.2708195447921753, 0.19490978121757507, -0.04870122671127319, -0.061061955988407135, 0.1609400361776352, -0.11282806098461151, -0.12466046214103699, -0.1494898945093155, -0.1793489009141922, 0.09028126299381256, 0.05622648075222969, 0.04113277047872543, -0.09009527415037155, -0.006841746624559164, 0.210173100233078, -0.13206079602241516, 0.005467157810926437, 0.09438782185316086, -0.08619939535856247, -0.20668888092041016, -0.16979871690273285, 0.23173384368419647, 0.004798495210707188, 0.050778038799762726, 0.10161405056715012, 0.12612824141979218, -0.08702633529901505, -0.13685627281665802, 0.06987298280000687, -0.09535474330186844, 0.14633995294570923, -0.4009123742580414, 0.1171250194311142, 0.02971533127129078, -0.19837842881679535, 0.001890092622488737, -0.04019662365317345, 0.049102865159511566, -0.10420822352170944, -0.016154184937477112, -0.3924260139465332, -0.16939233243465424, -0.06899407505989075, 0.07624562829732895, -0.10770987719297409, -0.013114645145833492, 0.16713498532772064, 0.00838835071772337, -0.028049364686012268, -0.03237881138920784, -0.014814520254731178, -0.012271729297935963, -0.08250709623098373, -0.0810459703207016, -0.03283453732728958, -0.11029545962810516, -0.0461348295211792, -0.11540041118860245, -0.10590843856334686, -0.0893745869398117, 0.03348740190267563, 0.05630376562476158, 0.0532359853386879, 0.11537851393222809, 0.179873526096344, 0.13690507411956787, -0.00947575643658638, -0.08611647039651871, -0.015854278579354286, -0.020088260993361473, 0.020586248487234116, -0.000972310604993254, -0.008224327117204666, 0.08995196968317032, 0.009846299886703491, 0.28296729922294617, -0.07438481599092484, -0.006876325700432062, 0.020316317677497864, 0.03505559265613556, 0.07709631323814392, 0.08947012573480606, -0.1114123985171318, 0.2598441243171692, -0.08771111071109772, -0.009100254625082016, 0.09508655220270157, -0.016892923042178154, -0.0957908183336258, -0.16855402290821075, 0.02785872109234333, 0.02690489962697029, 0.10763614624738693, 0.12741054594516754, 0.04840148240327835, 0.023123204708099365, 0.12732051312923431, 0.3077951669692993, 0.06814694404602051, -0.12614990770816803, 0.07489137351512909, -0.1437712013721466, -0.1567308008670807, 0.2608412206172943, 0.1568000465631485, 0.017165498808026314, 0.04731576889753342, 0.03266517072916031, 0.02626767009496689, -0.10314293205738068, -0.0788770467042923, -0.06273332983255386, 0.06757109612226486, -0.047237932682037354, -0.06304747611284256, -0.1746034473180771, -0.13584426045417786, -0.15687203407287598, 0.03411317244172096, -0.08440369367599487, 0.00030741203227080405, -0.15863759815692902, -0.16111716628074646, 0.16033530235290527, -0.0780130997300148, -0.00713195139542222, -0.06963245570659637, 0.17603427171707153, -0.1122455894947052, -0.12935344874858856, -0.05932055041193962, 0.08275380730628967, 0.052871569991111755, -0.08824776113033295, 0.11223476380109787, 0.248104065656662, -0.059206318110227585, -0.026341164484620094, -0.05934885889291763, 0.10198692977428436, -0.0634254664182663, 0.11927258223295212, 0.12712222337722778, -0.04137016832828522, -0.13591349124908447, -0.14244486391544342}, {-0.0013720336137339473, -0.163221076130867, 0.14437472820281982, 0.022068912163376808, 0.06061156094074249, 0.111725814640522, -0.07791236788034439, -0.04329856112599373, 0.05195053294301033, 0.10965225845575333, -0.04573971778154373, -0.6842944025993347, -0.1124989315867424, -0.12787936627864838, -0.014558680355548859, -0.19651351869106293, -0.06915245950222015, -0.12728847563266754, -0.06460683047771454, 0.13364335894584656, -0.19458019733428955, -0.05505723878741264, -0.0718563050031662, -0.011750202625989914, 0.13338662683963776, -0.058189135044813156, 0.19695386290550232, -0.0014730365946888924, -0.07230434566736221, -0.04403642192482948, -0.1650640219449997, 0.13669364154338837, 0.05769088491797447, 0.035328350961208344, -0.22920989990234375, 0.027185365557670593, -0.047383636236190796, -0.12205681949853897, -0.1667037457227707, 0.016573825851082802, -0.0056877825409173965, 0.025037385523319244, -0.23577061295509338, -0.029460173100233078, 0.05747571960091591, -0.13702578842639923, 0.13469864428043365, -0.04576273262500763, -0.1620815396308899, 0.10382819920778275, 0.16822880506515503, 0.09280098229646683, -0.10436135530471802, -0.19591131806373596, 0.054830122739076614, -0.2770073413848877, -0.07827051728963852, 0.008998237550258636, 0.07095208764076233, -0.01176116056740284, 5.90791751164943e-05, 0.12162095308303833, 0.03249002620577812, 0.0038474153261631727, -0.14334914088249207, -0.08841368556022644, 0.01828710362315178, -0.1355094015598297, -0.08893706649541855, 0.06285862624645233, -0.029200419783592224, -0.12610919773578644, 0.003176875412464142, 0.09196999669075012, -0.07253803312778473, -0.12120170891284943, 0.04996620863676071, 0.027771031484007835, -0.05227590352296829, 0.12332187592983246, 0.09214429557323456, -0.07470117509365082, 0.0046657598577439785, 0.034425411373376846, -0.05773596093058586, -0.02210172824561596, 0.01970003731548786, -0.00037084665382280946, -0.053443655371665955, -0.1797182559967041, -0.05499250069260597, 0.07715802639722824, 0.10754109919071198, 0.08727820217609406, 0.10230766981840134, -0.14644236862659454, 0.12998664379119873, -0.03400533273816109, -0.31968119740486145, -0.23292028903961182, -0.1041572242975235, -0.015707483515143394, 0.3351713716983795, 0.078411765396595, -0.0671774297952652, -0.05527964606881142, 0.06927363574504852, -0.03389288857579231, -0.15286189317703247, 0.33581793308258057, 0.046290453523397446, -0.13684377074241638, -0.10238079726696014, 0.035024650394916534, -0.1232803463935852, 0.12127954512834549, -0.017476819455623627, 0.0639432743191719, 0.05696820095181465, -0.06623045355081558, -0.04787779599428177, -0.1338759958744049, 0.13152150809764862, -0.1349698007106781, 0.054619185626506805, -0.028388341888785362, -0.0062804981134831905, -0.10985103249549866, -0.05253639444708824, -0.07400993257761002, -0.10587070137262344, -0.1846928596496582, 0.036519549787044525, -0.005501989740878344, -0.1930120885372162, -0.09656717628240585, 0.11470743268728256, 0.051192235201597214, -0.030312154442071915, -0.11141753196716309, 0.08659826219081879, -0.046186499297618866, 0.2536030113697052, 0.3176526427268982, -0.04205380007624626, -0.04694453254342079, 0.14903584122657776, 0.016389034688472748, -0.02199798822402954, 0.04779917374253273, -0.12322711944580078, -0.035559237003326416, -0.07114069163799286, 0.08761487901210785, -0.0020755380392074585, -0.0031804072204977274, -0.10264883935451508, -0.4077461361885071, 0.15483063459396362, 0.14274151623249054, -0.12111900001764297, 0.034409359097480774, 0.04823924973607063, 0.04683992266654968, -0.1596287339925766, 0.035762447863817215, 0.17104573547840118, 0.03749135881662369, 0.12962029874324799, -0.08562491834163666, 0.05995491147041321, 0.010827225632965565, 0.002340170554816723, 0.07730649411678314, -0.051939550787210464, -0.03204913064837456, -0.15939576923847198, 0.07000939548015594, -0.007130953948944807, -0.09251965582370758, 0.041960716247558594, -0.7859251499176025, 0.030628615990281105, 0.01822398602962494, 0.0296859722584486, -0.022373046725988388, 0.06910213828086853, 0.006828798446804285, -0.07726158201694489, 0.09897179901599884, 0.19927547872066498, -0.024270566180348396, 0.01156732626259327, -0.042043473571538925, -0.18247738480567932, 0.1300821453332901, 0.17639794945716858, 0.015985293313860893, 0.1838523894548416, 0.11751136183738708, -0.13337820768356323, 0.0908130630850792, 0.04038449004292488, -0.04421994462609291, -0.08023593574762344, 0.07472384721040726, 0.2478157877922058, -0.3166898787021637, -0.039475612342357635, 0.041928038001060486, 0.020668331533670425, 0.04230641573667526, -0.0022842236794531345, -0.16727279126644135, -0.007333952467888594, 0.060115013271570206, 0.018965130671858788, -0.11933749169111252, -0.07435502856969833, 0.04289395734667778, -0.004663567058742046, 0.03863723576068878, 0.22400261461734772, 0.2244749665260315, 0.12985368072986603, 0.08938638865947723, 0.156820148229599, 0.0607663169503212, 0.11781276017427444, 0.14761555194854736, -0.03793492168188095, 0.14504843950271606, 0.24512705206871033, -0.05146428197622299, -0.019221773371100426, 0.0245047640055418, -0.1423281878232956, 0.04846831411123276, 0.17470209300518036, -0.05719611793756485, 0.05804493650794029, -0.05969994515180588, 0.05267839506268501, 0.17690163850784302, 0.14044047892093658, -0.11631355434656143, 0.13317365944385529, 0.029938317835330963, 0.029853610321879387, 0.009676540270447731, -0.13028526306152344, 0.005765998270362616, 0.17193560302257538, 0.02002946101129055, 0.05984629690647125, 0.2016746699810028}, {-0.06693457067012787, 0.013850793242454529, 0.08269914984703064, 0.05728478357195854, 0.39540940523147583, -0.14766423404216766, 0.07194897532463074, -0.025108199566602707, 0.07426182925701141, -0.34250113368034363, 0.14069165289402008, -0.17736846208572388, -0.13810060918331146, 0.18682679533958435, 0.19392919540405273, -0.238917738199234, 0.09875943511724472, 0.023718178272247314, -0.08098465204238892, -0.0014326800592243671, -0.002516028471291065, -0.026022719219326973, -0.08924707025289536, -0.17637281119823456, -0.20406782627105713, -0.1917097121477127, -0.13584327697753906, -0.04073590785264969, -0.17775225639343262, 0.05224616080522537, -0.02228434383869171, -0.21851615607738495, 0.13889959454536438, 0.09790785610675812, -0.06844685226678848, -0.09129700809717178, 0.1182987317442894, 0.03182919695973396, 0.013020625337958336, 0.030266597867012024, 0.019997144117951393, -0.08795158565044403, 0.17111046612262726, -0.009033712558448315, -0.16503824293613434, -0.042083028703927994, 0.08429741859436035, -0.1882064938545227, 0.06881902366876602, -0.14669926464557648, 0.22127430140972137, -0.1757843941450119, 0.05332431569695473, 0.011098025366663933, 0.12672092020511627, -0.22805088758468628, -0.07159926742315292, 0.15524664521217346, -0.12190189957618713, -0.009243997745215893, 0.16146771609783173, 0.06866657733917236, 0.15350712835788727, -0.0652829110622406, -0.026539841666817665, 0.09753552079200745, 0.16431650519371033, 0.05942629277706146, -0.17039334774017334, -0.015827873721718788, -0.010347889736294746, -0.1706579327583313, 0.05859900638461113, 0.06920478492975235, 0.007994068786501884, 0.3782230615615845, 0.003913749475032091, -0.20349140465259552, 0.16734588146209717, 0.1537638157606125, 0.06997489184141159, 0.024010974913835526, -0.15361902117729187, -0.17648762464523315, -0.017063599079847336, -0.014611966907978058, 0.2725837826728821, 0.07632797956466675, 0.028903473168611526, -0.2500225305557251, 0.0871274322271347, -0.03464557230472565, 0.11552783101797104, -0.12045064568519592, -0.03359995037317276, 0.15171875059604645, 0.026977427303791046, 0.14961408078670502, -0.3166331648826599, 0.20088940858840942, 0.0715937614440918, 0.013292192481458187, 0.15291835367679596, -0.06758525967597961, 0.23786675930023193, -0.23078641295433044, -0.21091321110725403, 0.2060909867286682, 0.11123669892549515, 0.02676728181540966, -0.07912024855613708, 0.08820220828056335, 0.08516484498977661, -0.08101409673690796, 0.004295664839446545, -0.015750087797641754, -0.10031062364578247, 0.040447622537612915, -0.034058600664138794, 0.07029203325510025, 0.11948718875646591, -0.11180545389652252, 0.12192355841398239, -0.20261308550834656, 0.12397009879350662, -0.08159718662500381, -0.1719139814376831, 0.0782231017947197, -0.15846627950668335, -0.339909166097641, -0.247688427567482, 0.1627732664346695, -0.1571459025144577, -0.10686331242322922, -0.2474565953016281, 0.009607437998056412, -0.1541210412979126, -0.09046804904937744, 0.11349692940711975, -0.06446471810340881, -0.0913984626531601, -0.11786267161369324, -0.11442884057760239, 0.05301059037446976, -0.0564785972237587, 0.15496349334716797, 0.02671472541987896, 0.030022183433175087, 0.051053132861852646, 0.12267183512449265, -0.23484137654304504, 0.15453937649726868, -0.11345729976892471, 0.09491021931171417, 0.15709932148456573, -0.09017419815063477, -0.0628582090139389, -0.27448374032974243, 0.006963138934224844, 0.15668421983718872, 0.07011868804693222, -0.009953770786523819, 0.021796677261590958, -0.021066157147288322, 0.051520343869924545, 0.09195089340209961, -0.07244069129228592, -0.16574062407016754, 0.009854983538389206, -0.2517881393432617, -0.03421138972043991, 0.1713986098766327, 0.04676014184951782, -0.11028124392032623, 0.04629243537783623, -0.038197290152311325, 0.10581004619598389, -0.10871271789073944, 0.24248598515987396, -0.11853262782096863, 0.1366688460111618, -0.24898695945739746, 0.13509811460971832, -0.03369302675127983, -0.08151007443666458, 0.09247709065675735, 0.18726922571659088, -0.01841505989432335, -0.1540074348449707, 0.1044783666729927, 0.13062502443790436, 0.19784708321094513, 0.016884390264749527, 0.07519862800836563, -0.11016867309808731, 0.49314701557159424, -0.1797725409269333, -0.15313591063022614, -0.08815983682870865, -0.008456950075924397, 0.09994499385356903, -0.2103683352470398, -0.1447119265794754, -0.004815566819161177, -0.11256735771894455, 0.05797342583537102, -0.2569670081138611, -0.03779191896319389, 0.042653825134038925, 0.04731067270040512, 0.12523123621940613, -0.006051418371498585, 0.0009826690657064319, 0.2841995656490326, 0.04384492337703705, -0.10969433188438416, -0.19687505066394806, 0.067050501704216, -0.043950941413640976, -0.20811304450035095, 0.06642496585845947, 0.0977797657251358, 0.2199215292930603, 0.1698104441165924, 0.0038185722660273314, 0.11603185534477234, -0.1338152289390564, -0.06323330849409103, 0.2757951021194458, -0.09632185846567154, -0.013078897260129452, -0.1693868488073349, 0.2405625283718109, 0.09416309744119644, -0.080175019800663, -0.2857760488986969, 0.026806361973285675, -0.02260330133140087, 0.10936711728572845, -0.16064205765724182, 0.1112934872508049, -0.17384788393974304, 0.19300906360149384, -0.038294728845357895, -0.09215457737445831, -0.14136449992656708, -0.10686706006526947, 0.09271378815174103, 0.024863753467798233, 0.076102115213871, -0.05050497502088547, 0.06977210193872452, -0.014942980371415615, 0.06842368841171265, 0.12570053339004517, 0.07220147550106049}, {0.12326780706644058, 0.10672980546951294, 0.13141554594039917, -0.21280381083488464, -0.0674053281545639, -0.039864398539066315, -0.09233763068914413, -0.1304554045200348, 0.19756625592708588, 0.9839399456977844, 0.045798156410455704, -0.2600608766078949, 0.0008017530781216919, 0.16007579863071442, 0.07981875538825989, -0.0482524149119854, -0.0002775687025859952, -0.26142698526382446, 0.14923684298992157, -0.03697284683585167, 0.08742377161979675, 0.09062980115413666, -0.06060430780053139, 0.10962311178445816, -0.07597003132104874, 0.03778650239109993, -0.16336731612682343, 0.051061999052762985, 0.034144818782806396, 0.2882132828235626, -0.09606964886188507, -0.04384269937872887, 0.028692936524748802, -0.03299400955438614, 0.128029003739357, -0.026939906179904938, 0.021321680396795273, -0.04752565175294876, -0.058004796504974365, 0.10257828235626221, -0.27215027809143066, 0.15001077950000763, -0.03208458796143532, 0.15652410686016083, 0.11556192487478256, -0.166078120470047, 0.09926972538232803, 0.012918778695166111, -0.21489740908145905, 0.04078550636768341, -0.23243914544582367, 0.07421592622995377, 0.02188800647854805, 0.025189615786075592, 0.11692450195550919, 0.07681112736463547, -0.06697791069746017, 0.047254566103219986, 0.03106045164167881, -0.2285548597574234, 0.08202425390481949, -0.09015447646379471, 0.2636064291000366, 0.09409601986408234, -0.052611421793699265, 0.052754417061805725, 0.09600703418254852, 0.16222406923770905, 0.21244652569293976, 0.16473254561424255, -0.0010403299238532782, -0.04183785244822502, -0.22879748046398163, -0.12652568519115448, 0.1545749306678772, -0.19170202314853668, -0.15230008959770203, 0.10041625797748566, -0.33976253867149353, -0.3058960735797882, 0.04464293643832207, -0.10693003237247467, -0.03675905615091324, -0.052980292588472366, -0.15300384163856506, 0.09251019358634949, 0.0003566186933312565, -0.11349837481975555, -0.2967919111251831, 0.021426912397146225, -0.06880016624927521, -0.03327978402376175, -0.05277763679623604, -0.10588841140270233, -0.12857045233249664, -0.0017400271026417613, 0.04797713831067085, -0.15946342051029205, 0.04530319571495056, -0.08129920810461044, -0.02183348871767521, 0.18134598433971405, 0.05755142122507095, 0.15403079986572266, 0.1794339269399643, -0.008668108843266964, 0.08733298629522324, -0.1463349312543869, -0.045330021530389786, -0.11176031827926636, 0.0942039042711258, -0.0376591794192791, -0.02640998363494873, 0.12505266070365906, -0.1537257730960846, -0.007740912958979607, -0.2019093930721283, 0.005742531269788742, -0.14900784194469452, -0.004794278182089329, -0.18448695540428162, 0.08809687942266464, 0.025824176147580147, -0.18244268000125885, 0.10038268566131592, 0.06477882713079453, -0.11967937648296356, 0.045703496783971786, 0.10840927064418793, 0.060483116656541824, -0.11468953639268875, -0.0945649966597557, 0.18395976722240448, 0.09573275595903397, 0.160322904586792, -0.07249765843153, 0.23463192582130432, 0.08131953328847885, 0.02953534573316574, -0.19174280762672424, 0.10372089594602585, 0.12484990805387497, 0.05753207579255104, 0.04009734094142914, -0.07372693717479706, -0.24581323564052582, 0.12584516406059265, -0.11161179840564728, -0.0387813001871109, 0.11258763074874878, 0.4475104510784149, -0.03207209333777428, 0.04607219994068146, -0.18640704452991486, -0.140448197722435, 0.05320720374584198, -0.10462916642427444, 0.10729605704545975, -0.08197285979986191, -0.024752814322710037, -0.08094238489866257, 0.2321220189332962, 0.030442506074905396, -0.09616516530513763, -0.02244391106069088, -0.05232001096010208, 0.13654407858848572, 0.07016468048095703, -0.0266110822558403, -0.011244439519941807, -0.06398627907037735, -0.05857772007584572, -0.16811436414718628, -0.0075606112368404865, 0.12014197558164597, -0.04769373685121536, -0.20349977910518646, 0.13541272282600403, -0.018629219383001328, 0.1975853443145752, 0.0018062423914670944, -0.08899214118719101, -0.09088671952486038, 0.012299541383981705, 0.06109366565942764, 0.07263032346963882, -0.1148209273815155, -0.0749426931142807, -0.06251297146081924, 0.4250584840774536, -0.012368238531053066, 0.3438666760921478, -0.06713739037513733, -0.01646675355732441, 0.04581518843770027, -0.20015719532966614, 0.051942795515060425, 0.059044867753982544, 0.09819631278514862, -0.0700879618525505, -0.20582172274589539, 0.09952894598245621, -0.09530907869338989, 0.04066018387675285, 0.18089079856872559, -0.23213593661785126, 0.06911290436983109, -0.1026059165596962, -0.32614827156066895, -0.2191057950258255, -0.09342235326766968, 0.04668490216135979, -0.12545719742774963, -0.2120961844921112, -0.15489895641803741, 0.12247902899980545, -0.15993623435497284, 0.04072197899222374, -0.06431448459625244, -0.05802224203944206, 0.008988812565803528, -0.09214956313371658, 0.009827216155827045, -0.14908601343631744, -0.03110678680241108, -0.06445179879665375, -0.08232207596302032, 0.10668771713972092, -0.056327372789382935, 0.009286263026297092, -0.045991480350494385, 0.09456241875886917, 0.12131098657846451, -0.08448345959186554, 0.45567941665649414, 0.03986145928502083, -0.045577287673950195, 0.007599415257573128, -0.03244706615805626, -0.08255282789468765, 0.08891294151544571, 0.15371915698051453, 0.07681801915168762, 0.14730671048164368, 0.16477276384830475, 0.167811781167984, -0.0005459681851789355, -0.11866891384124756, 0.1262502819299698, -0.014956001192331314, 0.036448948085308075, 0.02765464037656784, -0.19040438532829285, 0.020576488226652145, 0.04040854796767235, 0.12195907533168793}, {-0.006628768518567085, -0.0449339933693409, -0.09261083602905273, -0.07124222815036774, -0.19560310244560242, -0.0023940224200487137, 0.10678619891405106, 0.19215276837348938, -0.05374053120613098, 0.05226987227797508, 0.08917965739965439, 0.060757771134376526, -0.1642448604106903, -0.013587997294962406, -0.08036959916353226, 0.17929506301879883, 0.04638930410146713, 0.047311872243881226, 0.04402537643909454, -0.10929232090711594, 0.04238617792725563, 0.008337719365954399, -0.07029852271080017, 0.12832452356815338, 0.07243085652589798, 0.1423630267381668, 0.019443562254309654, 0.20756830275058746, 0.3020339608192444, 0.3263349235057831, -0.13258102536201477, 0.05054973065853119, -0.08305701613426208, 0.2636118531227112, 0.09150820225477219, -0.007525973487645388, -0.15720659494400024, -0.14337295293807983, 0.25697770714759827, -0.08710706233978271, 0.2518334686756134, -0.03516988828778267, 0.3005870282649994, -0.2214229553937912, -0.11707361787557602, -0.12808024883270264, 0.29177844524383545, 0.13170824944972992, 0.001823201309889555, 0.18543627858161926, -0.03774557262659073, 0.18655294179916382, -0.01479963306337595, 0.1114792600274086, -0.03338506817817688, 0.12789230048656464, -0.061564333736896515, 0.20787931978702545, -0.06483571976423264, -0.05910824239253998, 0.13878312706947327, 0.13449527323246002, 0.07288041710853577, 0.24097397923469543, 0.14743657410144806, 0.04786526411771774, 0.05964424088597298, -0.0193768423050642, 0.12812648713588715, -0.012782459147274494, -0.04710714891552925, 0.14642693102359772, -0.1857159584760666, -0.2435481995344162, 0.037565991282463074, -0.011408363468945026, 0.039846159517765045, 0.30143412947654724, -0.24791312217712402, -0.12423458695411682, -0.025875935330986977, -0.07661394774913788, -0.057544734328985214, 0.05688498541712761, 0.1178467720746994, 0.3246496617794037, -0.17177510261535645, 0.02477756328880787, -0.08005768060684204, 0.1660473346710205, 0.05641254037618637, -0.13692861795425415, -0.06793360412120819, -0.0022001387551426888, -0.12117266654968262, 0.0682925134897232, -0.038009971380233765, -0.0483708456158638, -0.04407468065619469, 0.08788180351257324, 0.09263844043016434, 0.013361366465687752, -0.1122148260474205, 0.0388467013835907, 0.08949438482522964, -0.0633237287402153, -0.0334990918636322, -0.10936681926250458, 0.0617915540933609, -0.11463329941034317, -0.11875735968351364, -0.05266251415014267, -0.0668189749121666, 0.2369040995836258, 0.10787682235240936, -0.01677658222615719, -0.07471583783626556, 0.11135765165090561, -0.023645218461751938, -0.159094899892807, -0.21424143016338348, 0.24893070757389069, 0.20633916556835175, 0.05966321378946304, 0.07995006442070007, -0.08365549892187119, -0.047617580741643906, 0.24092428386211395, 0.014122271910309792, 0.015252302400767803, -0.04151036962866783, -0.08802222460508347, -0.029580138623714447, 0.1542768031358719, 0.38666433095932007, 0.1694488674402237, 0.007626349572092295, -0.1621352881193161, -0.07765401154756546, -0.005895080976188183, -0.22883111238479614, -0.07601798325777054, 0.048054423183202744, 0.12468396872282028, 0.17396561801433563, -0.018951350823044777, -0.016515742987394333, -0.20586863160133362, -0.050465911626815796, -0.26038894057273865, 0.15378741919994354, 0.053027987480163574, 0.13015367090702057, 0.06928536295890808, -0.07479263842105865, 0.003128873184323311, -0.13596166670322418, 0.278428316116333, 0.17440982162952423, -0.2082141637802124, -0.05019563436508179, -0.06623721867799759, -0.08016326278448105, -0.032873596996068954, -0.09241030365228653, 0.017454015091061592, -0.07665002346038818, -0.09652478992938995, 0.0023993977811187506, 0.16029977798461914, 0.032094329595565796, 0.10674119740724564, -0.027259856462478638, -0.14250697195529938, 0.09449997544288635, -0.23390589654445648, 0.13724850118160248, -0.0523068867623806, -0.20674759149551392, -0.021286386996507645, -0.0495833158493042, 0.2908932864665985, -0.3351104259490967, -0.06266000121831894, -0.07329302281141281, -0.005861725192517042, -0.2620087265968323, -0.11637765169143677, -0.18134325742721558, -0.00983558502048254, -0.08974945545196533, -0.07122398912906647, -0.053382206708192825, -0.20305870473384857, 0.20786839723587036, -0.13309425115585327, -0.0628725066781044, 0.08075132966041565, -0.010530401021242142, 0.18206608295440674, 0.24006113409996033, 0.05664194002747536, 0.05339379236102104, 0.10219479352235794, 0.08472216129302979, 0.17705003917217255, -0.21222975850105286, 0.055212970823049545, -0.02059360407292843, 0.10738136619329453, 0.23180916905403137, -0.12047217786312103, 0.09506069123744965, -0.15201090276241302, 0.07656950503587723, -0.18675939738750458, 0.0487656369805336, -0.27256256341934204, -0.3015945553779602, -0.37786468863487244, 0.004031486809253693, 0.1082267239689827, 0.026118282228708267, -0.2123798131942749, 0.04945217818021774, 0.045963071286678314, -0.04055599495768547, 0.020445486530661583, -0.09141802042722702, 0.06448432803153992, 0.1332467496395111, -0.011591620743274689, -0.1062832921743393, -0.2800729274749756, 0.07524845749139786, -0.03071703389286995, -0.01686663180589676, 0.003949103411287069, 0.10946889966726303, 0.07998722046613693, 0.11665616929531097, 0.057316508144140244, -0.08719407021999359, 0.0025338674895465374, 0.16824105381965637, 0.029775669798254967, 0.18315920233726501, 0.014647173695266247, 0.1256924420595169, -0.24927394092082977, 0.023454826325178146, 0.15585510432720184, -0.20879054069519043, -0.11591163277626038, -0.08330422639846802, 0.296473890542984}, {0.07441555708646774, 0.1097257062792778, 0.04707609489560127, -0.09633664041757584, -0.1798577904701233, -0.21675916016101837, -0.05949290469288826, 0.15197092294692993, -0.13505880534648895, -0.07797614485025406, -0.09410683810710907, -0.034036293625831604, 0.007005985360592604, -0.021635839715600014, 0.0407068245112896, 0.2526353597640991, 0.03396577760577202, -0.05821958929300308, -0.11469188332557678, -0.1664019674062729, 0.1488015353679657, -0.00962983537465334, 0.015548200346529484, -0.29943108558654785, 0.011439849622547626, 0.02482299692928791, -0.032918740063905716, 0.1473332792520523, -0.04872593283653259, 0.12477008253335953, 0.2580704092979431, 0.0481216236948967, 0.040828853845596313, -0.12447769194841385, -0.07870210707187653, 0.04417654126882553, 0.0052924505434930325, -0.10151271522045135, 0.11789654195308685, -0.1022544577717781, -0.025176040828227997, -0.21524381637573242, -0.049704328179359436, 0.19588150084018707, 0.011868857778608799, -0.1565810590982437, -0.08010168373584747, 0.2427692413330078, -0.000808162905741483, -0.09696079790592194, 0.003999966662377119, 0.0020977663807570934, -0.10600219666957855, -0.04386395215988159, -0.05414989963173866, -0.0005641997558996081, 0.017995597794651985, 0.14579766988754272, 0.14747631549835205, -0.06213400140404701, 0.07616470754146576, -0.054389480501413345, 0.12129616737365723, -0.07319169491529465, 0.10722770541906357, 0.08037007600069046, -0.09539122134447098, 0.14905738830566406, -0.000742661184631288, 0.1890586018562317, 0.006818213500082493, 0.12042694538831711, 0.13960939645767212, 0.06403188407421112, 0.052328165620565414, 0.07882826775312424, 0.12843720614910126, 0.12642483413219452, 0.11691834032535553, -0.007901160977780819, -0.06296473741531372, 0.06917978078126907, -0.04269048571586609, -0.16522763669490814, -0.23942488431930542, -0.15952268242835999, -0.13403865694999695, -0.007136527448892593, -0.049444351345300674, 0.09851888567209244, 0.017474474385380745, -0.014874886721372604, -0.13137878477573395, -0.0529928058385849, 0.02514839544892311, 0.1364290565252304, -0.07867395877838135, 0.03622306138277054, -0.09999565035104752, -0.3099270462989807, -0.06227493658661842, -0.05643846094608307, -0.11893883347511292, 0.028232388198375702, 0.1050805151462555, 0.4479433596134186, 0.1907729059457779, -0.05573023483157158, 0.019181381911039352, -0.03712194785475731, 0.05192677676677704, -0.0037400308065116405, -0.08708729594945908, 0.0745982900261879, -0.1004582941532135, 0.24333259463310242, -0.06670200079679489, -0.055143192410469055, 0.16327112913131714, 0.04364687576889992, 0.06504946202039719, -0.23593750596046448, -0.08830073475837708, -0.04881039634346962, 0.046148013323545456, 0.1458183228969574, 0.15679484605789185, 0.09849124401807785, -0.1283402293920517, -0.08276031166315079, -0.027423733845353127, 0.047558195888996124, 0.0015820189146324992, -0.018212176859378815, 0.020436109974980354, -0.2404850274324417, -0.04740293696522713, 0.06715915352106094, -0.07684116810560226, -0.025427961722016335, 0.17292052507400513, 0.10840696841478348, -0.10213001817464828, -0.006821471732109785, -0.00787090789526701, -0.09887120127677917, -0.05274106189608574, -0.053656481206417084, 0.17202284932136536, 0.11015462130308151, -0.03592338040471077, -0.13289830088615417, -0.035349950194358826, -0.3389066159725189, -0.12378447502851486, -0.0630945935845375, 0.014170215465128422, -0.13378804922103882, 0.11572663486003876, -0.5259323120117188, -0.11389122903347015, 0.19050928950309753, 0.008382460102438927, -0.010001975111663342, 0.006820880342274904, -0.1486290842294693, -0.13194520771503448, -0.10306239128112793, -0.0025024437345564365, -0.03241617605090141, -0.01758786477148533, 0.07734216004610062, -0.09546194970607758, 0.10072784125804901, 0.12132895737886429, 0.06317457556724548, 0.06612034887075424, 0.013074913062155247, 0.1325218826532364, 0.03960077837109566, 0.022403152659535408, 0.06416937708854675, 0.08054859936237335, 0.17592944204807281, -0.011044420301914215, 0.10091274231672287, -0.1753200739622116, -0.13566124439239502, 0.08979570120573044, -0.02639225870370865, -0.018719570711255074, -0.0889972448348999, 0.20625874400138855, 0.00028259350801818073, -0.13777966797351837, 0.02111552283167839, -0.07124471664428711, 0.0818990021944046, -0.055227890610694885, 0.09749434143304825, 0.026653481647372246, 0.05750828608870506, -0.024371061474084854, 0.029242679476737976, 0.04664183408021927, 0.05463526397943497, 0.29597726464271545, 0.14236922562122345, 0.04518686607480049, 0.1223173663020134, -0.07662353664636612, -0.17803044617176056, 0.053922515362501144, 0.10564257204532623, 0.07433606684207916, 0.0756610855460167, -0.017962267622351646, 0.031900081783533096, 0.008723246864974499, 0.14226362109184265, -0.0870567038655281, -0.09842369705438614, -0.029544081538915634, -0.21035709977149963, -0.11934284120798111, -0.09017203748226166, 0.04627877473831177, 0.04688849672675133, -0.12583830952644348, -0.12327823787927628, 0.11957287788391113, -0.07327437400817871, -0.071390300989151, 0.2226303219795227, 0.14359642565250397, -0.14835456013679504, 0.16186296939849854, 0.11420371383428574, 0.13054391741752625, -0.4291619658470154, -0.07394102215766907, 0.18523739278316498, -0.15490250289440155, -0.16365352272987366, -0.08439605683088303, -0.16019225120544434, 0.04481924697756767, -0.1055968627333641, 0.08886012434959412, 0.03811183199286461, 0.11755257844924927, -0.05668110027909279, -0.03775360435247421, 0.018924983218312263, -0.08724934607744217, -0.11747743934392929}, {-0.1516810655593872, 0.02615058235824108, -0.34597060084342957, 0.010425947606563568, -0.016537323594093323, 0.16749069094657898, 0.07923861593008041, 0.07699888199567795, 0.09915560483932495, -0.046870555728673935, 0.007195393554866314, -0.046602513641119, -0.08297579735517502, 0.0035952513571828604, -0.03370246663689613, 0.0812322273850441, -0.09758760035037994, 0.3339864909648895, -0.20378684997558594, 0.10674861073493958, 0.08805213868618011, 0.05082951858639717, 0.1056084856390953, -0.03895153850317001, -0.08073453605175018, -0.18764369189739227, 0.029355250298976898, -0.12392044812440872, 0.08369298279285431, 0.11537621915340424, 0.10220377892255783, 0.03894370049238205, -0.04241764545440674, 0.07041759788990021, -0.09116926044225693, -0.005852367728948593, 0.08780069649219513, 0.2016088217496872, -0.06408698111772537, -0.013810142874717712, 0.13535383343696594, -0.20818199217319489, -0.15850232541561127, -0.06235678493976593, 0.11430392414331436, -0.12192769348621368, -0.040494829416275024, 0.007825630716979504, 0.08893176168203354, 0.02887313812971115, -0.10020726919174194, -0.1122748851776123, -0.17199014127254486, 0.04151885583996773, -0.1220034658908844, 0.30577266216278076, 0.028770387172698975, -0.27377116680145264, -0.07014398276805878, 0.11669307202100754, 0.22366391122341156, -0.020285293459892273, 0.16507607698440552, 0.19246971607208252, -0.004027057439088821, 0.0672617182135582, 0.03796304017305374, -0.06768961995840073, 0.10096883028745651, 0.03264499083161354, -0.03748385235667229, 0.19827508926391602, 0.33092981576919556, 0.052255190908908844, 0.01633821614086628, 0.008829592727124691, -0.13880740106105804, -0.033658143132925034, 0.15302255749702454, 0.3153765797615051, 0.026368776336312294, -0.05694025754928589, 0.022325804457068443, -0.03308587521314621, 0.03180659934878349, 0.06695645302534103, -0.06394683569669724, -0.04728957638144493, 0.11130482703447342, 0.03418363630771637, 0.029778236523270607, -0.21674181520938873, 0.20954713225364685, -0.09387806802988052, 0.3232381045818329, 0.318244993686676, 0.06516508758068085, 0.06138630956411362, -0.20525550842285156, 0.3347633481025696, -0.16813363134860992, 0.005037355236709118, 0.12781964242458344, -0.010232259519398212, -0.34035927057266235, 0.2534830570220947, 0.03188338130712509, -0.12763932347297668, -0.12565390765666962, 0.24466000497341156, -0.10170719027519226, 0.14726974070072174, 0.012592297978699207, -0.14055241644382477, 0.04597263038158417, 0.024792304262518883, 0.1379915475845337, -0.0003308883460704237, 0.19832773506641388, 0.07866007834672928, 0.22804969549179077, -0.017078904435038567, 0.04437299072742462, -0.14695888757705688, -0.02340123988687992, -0.16941849887371063, -0.13943764567375183, 0.010180912911891937, 0.08268516510725021, -0.13891881704330444, 0.06334133446216583, 0.22872774302959442, -0.33700233697891235, -0.07273726165294647, 0.05420587584376335, -0.16514210402965546, -0.022706346586346626, 0.13301213085651398, -0.13165056705474854, 0.30869823694229126, -0.08238187432289124, 0.08350863307714462, -0.08432978391647339, -0.16854755580425262, -0.22213047742843628, 0.06529171764850616, 0.278482049703598, 0.019316909834742546, 0.05828336998820305, 0.005453864112496376, -0.1324252486228943, -0.04474155604839325, 0.1402343511581421, -0.11921151727437973, 0.00892594363540411, -0.0791192352771759, -0.10926268994808197, 0.1308412253856659, 0.3774639666080475, -0.07514540106058121, -0.004416708834469318, -0.00320114241912961, -0.17166103422641754, -0.284903347492218, -0.109889917075634, 0.006750121712684631, -0.07709774374961853, 0.17000140249729156, -0.10277701914310455, -0.10596247762441635, -0.03105453960597515, 0.07836355268955231, -0.12172237038612366, 0.1224261224269867, -0.004209958948194981, -0.08122190088033676, -0.14284339547157288, -0.07412942498922348, 0.02165604569017887, 0.29721611738204956, 0.10534238070249557, -0.16174328327178955, -0.16671600937843323, 0.08361716568470001, 0.133283793926239, 0.12463965266942978, 0.20023582875728607, 0.022130686789751053, 0.02521246299147606, 0.1592128574848175, -0.02548317238688469, -0.21092693507671356, -0.31886500120162964, -0.19147706031799316, 0.03491857647895813, -0.08039746433496475, -0.14743465185165405, 0.3502976596355438, -0.12826316058635712, -0.20996403694152832, 0.3262200355529785, -0.09316044300794601, -0.01563151180744171, 0.04966218024492264, -0.026777008548378944, -0.232514888048172, -0.02280174382030964, 0.10364105552434921, 0.06575744599103928, -0.09249740839004517, 0.12320131063461304, -0.35073599219322205, 0.0066962591372430325, -0.1876019984483719, 0.03508706018328667, -0.05171879008412361, 0.03991774469614029, -0.09164614975452423, 0.011835501529276371, -0.16235773265361786, 0.06069377809762955, 0.0062906439416110516, -0.16014206409454346, -0.24059808254241943, -0.1775681972503662, -0.2189444601535797, 0.06266018748283386, 0.084972083568573, -0.04605342447757721, -0.008564324118196964, -0.15549930930137634, -0.012905221432447433, 0.021192537620663643, 0.06258788704872131, -0.09650745987892151, -0.06804382055997849, 0.3384242355823517, 0.0054575614631175995, 0.014319434762001038, -0.04527830705046654, -0.1290046125650406, 0.08758621662855148, 0.12076263129711151, -0.08429989963769913, -0.06660116463899612, -0.013822323642671108, -0.2248799353837967, -0.40130680799484253, -0.0185776986181736, -0.05258568748831749, 0.09506730735301971, -0.051787909120321274, 0.09345760941505432, -0.47750434279441833, -0.13393472135066986, -0.1366286277770996}, {0.11564042419195175, 0.00533938966691494, -0.06460887938737869, 0.05115755647420883, 0.003962297458201647, -0.31074264645576477, -0.20426426827907562, -0.12561078369617462, 0.04403337091207504, 0.22809651494026184, 0.1938508152961731, -0.3176432251930237, -0.08662194758653641, 0.0385749414563179, 0.0664479210972786, 0.13284313678741455, 0.08625087887048721, -0.1313798725605011, -0.05306296795606613, -0.1632259488105774, -0.09974876046180725, 0.027398189529776573, -0.0049491641111671925, 0.025038739666342735, 0.006302475463598967, -0.06670506298542023, 0.1167774498462677, -0.019602095708251, 0.2094087451696396, 0.12021172046661377, -0.1605285108089447, -0.07687895745038986, 0.05979292094707489, -0.09156351536512375, 0.28756850957870483, 0.10688517242670059, 0.07669170945882797, 0.036352045834064484, 0.03716232627630234, -0.16555438935756683, 0.03336568921804428, 0.029572347179055214, 0.0007681448478251696, 0.06707379966974258, 0.08891258388757706, -0.04312288388609886, 0.11709076166152954, 9.289651643484831e-05, -0.10215671360492706, -0.11624958366155624, 0.1475348174571991, 0.013873293995857239, 0.06066657602787018, 0.10003088414669037, 0.11588425189256668, 0.2083851844072342, -0.03480755165219307, -0.05101776868104935, 0.08281096071004868, 0.0816088393330574, -0.19963058829307556, 0.1906197965145111, 0.1749085634946823, 0.3421342670917511, -0.15119676291942596, 0.16772589087486267, -0.008002923801541328, -0.002811867743730545, -0.02109634131193161, -0.1358262151479721, -0.12500883638858795, 0.1887338012456894, -0.16317781805992126, -0.046698376536369324, 0.009101311676204205, -0.07640515267848969, 0.11791126430034637, 0.2397565394639969, -0.11433377861976624, -0.1454007476568222, 0.34489017724990845, -0.03235914185643196, -0.14517615735530853, -0.0043154386803507805, -0.15519775450229645, 0.02728165127336979, -0.12483908981084824, 0.09532616287469864, -0.026218118146061897, -0.014192348346114159, 0.08159767836332321, -0.026932844892144203, -0.03155459463596344, -0.06632507592439651, 0.12174981087446213, 0.03523489087820053, -0.06502852588891983, 0.029370533302426338, 0.1650451421737671, -0.0908079519867897, -0.02701732888817787, 0.09831218421459198, 0.1911514699459076, 0.004352263640612364, 0.05453912913799286, 0.053038090467453, 0.12796415388584137, -0.29766103625297546, -0.03899974375963211, -0.002085769781842828, -0.0786556527018547, 0.04938755929470062, -0.21925945580005646, -0.014869589358568192, -0.10635527223348618, -0.09262576699256897, -0.006156693212687969, 0.15691804885864258, -0.19602948427200317, -0.027056820690631866, -0.29163020849227905, -0.042653970420360565, -0.004148496314883232, -0.09245555102825165, -0.1595144122838974, -0.024231834337115288, -0.09896306693553925, 0.047143131494522095, -0.02870996668934822, -0.1056370958685875, -0.21646448969841003, -0.0767049789428711, 0.09491337090730667, 0.14052817225456238, 0.06760597974061966, 0.16377508640289307, 0.06710395961999893, -0.03346497192978859, 0.03109179437160492, -0.13902871310710907, 0.12712053954601288, 0.27241361141204834, -0.10422668606042862, -0.06507347524166107, -0.05240024998784065, 0.10099398344755173, -0.1250775307416916, -0.10261604934930801, 0.08236384391784668, 0.05297297239303589, 0.030692847445607185, 0.0659128874540329, -0.0696750208735466, -0.006094274576753378, -0.06548920273780823, -0.22066622972488403, 0.08071810007095337, 0.09953372925519943, 0.10713229328393936, 0.0604819729924202, -0.17677392065525055, 0.09257102757692337, 0.1068982407450676, 0.0004130991583224386, -0.07806755602359772, 0.054908741265535355, 0.2778127193450928, 0.03580738231539726, 0.1671932488679886, -0.2663780748844147, -0.07674883306026459, -0.05894266068935394, 0.13649819791316986, 0.05970902740955353, -0.024847395718097687, -0.023366762325167656, -0.27153658866882324, 0.010485348291695118, 0.004215271212160587, 0.16021013259887695, -0.12569209933280945, -0.2620828449726105, 0.03853533789515495, -0.1336343139410019, -0.05889501795172691, -0.10933485627174377, -0.026688460260629654, 0.0611492283642292, -0.03732258081436157, 0.156359001994133, -0.07643815129995346, 0.17582769691944122, 0.16401407122612, 0.18076366186141968, -0.0317724384367466, -0.01601404696702957, -0.11815013736486435, 0.011697560548782349, -0.01983780413866043, -0.13064827024936676, -0.05015172064304352, 0.11312459409236908, -0.011883698403835297, 0.17340834438800812, 0.003932980354875326, -0.09730211645364761, 0.19137351214885712, -0.011309163644909859, -0.13138075172901154, -0.253473162651062, -0.3038570284843445, -0.1107453852891922, 0.04898173362016678, 0.06093895807862282, 0.3315995931625366, 0.040100015699863434, -0.003562995931133628, 0.09471841901540756, 0.06876658648252487, -0.046218231320381165, -0.07821512222290039, -0.014934368431568146, 0.13236470520496368, -0.145035058259964, 0.24287894368171692, -0.08677112311124802, 0.048883821815252304, 0.054223619401454926, -0.16939903795719147, 0.05134136602282524, -0.03280048444867134, -0.1203317865729332, 0.1425534337759018, -0.0954887717962265, 0.049322035163640976, 0.146484375, -0.09909192472696304, -0.13615036010742188, 0.009213052690029144, -0.006571864243596792, -0.11085864901542664, 0.2635546028614044, 0.023955950513482094, 0.13335750997066498, 0.28666725754737854, -0.11025384068489075, -0.0015288578579202294, -0.12468303740024567, -0.09303363412618637, 0.2822543978691101, -0.10610834509134293, -0.206364244222641, -0.0029770378023386, -0.06575621664524078, -0.06194934621453285, 0.07823561877012253}, {-0.03679145127534866, 0.04629649221897125, -0.09098146855831146, -0.13865219056606293, 0.09667054563760757, 0.14425699412822723, -0.03768577054142952, 0.15078085660934448, -0.11641713976860046, 0.06818836182355881, -0.1039678156375885, -0.350090891122818, -0.11340060830116272, 0.006466621067374945, 0.08572185784578323, 0.080485500395298, -0.06481032073497772, 0.12450800836086273, -0.10092410445213318, 0.39059677720069885, 0.21612288057804108, 0.03085808828473091, 0.04145599901676178, 0.23571041226387024, 0.21510806679725647, -0.07803621143102646, -0.05543222278356552, -0.01771155185997486, -0.006946910172700882, 0.17275528609752655, -0.012260446324944496, -0.10245715826749802, -0.020216448232531548, 0.10692114382982254, -0.0008513690554536879, -0.26575392484664917, 0.05086996778845787, 0.14329881966114044, -0.15650880336761475, 0.10139071941375732, 0.16374853253364563, -0.30107903480529785, -0.17676444351673126, -0.15392684936523438, 0.04071837291121483, -0.2476266324520111, 0.17162880301475525, 0.06557770818471909, 0.014337869361042976, 0.07858213782310486, -0.05176526680588722, 0.0439101867377758, -0.21919235587120056, -0.1481139063835144, 0.04876871034502983, 0.07502856850624084, 0.26657673716545105, -0.17347103357315063, 0.07149142026901245, 0.011173051781952381, 0.052459824830293655, 0.00256537483073771, 0.06879062950611115, -0.19825561344623566, -0.029745569452643394, 0.009431199170649052, -0.01797960326075554, 0.05523184314370155, 0.14811977744102478, 0.015375052578747272, 0.1886564940214157, -0.16859090328216553, 0.7779971957206726, 0.056948162615299225, 0.012320514768362045, 0.17048965394496918, 0.1333988755941391, 0.11798066645860672, -0.05919170379638672, 0.08285846561193466, 0.02418360859155655, 0.005103733390569687, 0.18492193520069122, -0.14368471503257751, 0.1263742595911026, 0.036049310117959976, 0.4102979898452759, 0.0016343562165275216, -0.08159241825342178, -0.08162999898195267, 0.1611877977848053, -0.006412076763808727, -0.0329279750585556, -0.10374307632446289, -0.06498413532972336, 0.00940503366291523, 0.0014356927713379264, -0.100916787981987, 0.024367472156882286, 0.1864655762910843, -0.057394541800022125, -0.15799938142299652, 0.07765641808509827, -0.0064018769189715385, -0.3031699061393738, 0.30053678154945374, -0.17998842895030975, 0.06571868807077408, -0.08374499529600143, 0.2777421772480011, -0.08047007024288177, 0.2216625064611435, 0.17796066403388977, -0.14356084167957306, 0.14131414890289307, 0.2514364719390869, -0.1913166046142578, -0.019218845292925835, -0.03323664888739586, 0.19031491875648499, -0.055188558995723724, 0.19988524913787842, 0.1743260771036148, -0.10456199198961258, 0.18840214610099792, 0.03074352815747261, 0.013748366385698318, 0.017231659963726997, 0.1810474395751953, -0.03420794755220413, 0.12135356664657593, -0.10985331982374191, -0.32117393612861633, -0.19796901941299438, 0.30400699377059937, -0.13121667504310608, 0.028437210246920586, 0.00474584149196744, -0.12095913290977478, -0.00797455757856369, -0.2887442111968994, 0.013946440070867538, -0.02984311245381832, 0.051685139536857605, -0.3270915448665619, -0.05725737288594246, -0.022148454561829567, -0.028903020545840263, 0.05556752160191536, 0.030220407992601395, 0.032865434885025024, 0.049244970083236694, -0.12903526425361633, -0.11500781029462814, 0.0554036982357502, -0.05793115124106407, -0.16366331279277802, 0.11821521818637848, 0.1576785445213318, 0.004375264514237642, 0.02309248223900795, -0.022333065047860146, 0.10927900671958923, -0.011758017353713512, -0.2581019401550293, 0.015427500009536743, 0.07250169664621353, -0.09971586614847183, -0.19823956489562988, 0.07667279243469238, 0.07763726264238358, -0.06718777865171432, 0.0356009267270565, -0.10011707991361618, 0.11672210693359375, -0.08015111833810806, -0.04896939545869827, -0.07134555280208588, 0.10126369446516037, 0.05479365214705467, 0.08774727582931519, -0.3083720803260803, 0.08146325498819351, 0.3065655827522278, 0.01965121552348137, 0.07667382806539536, 0.002474962268024683, 0.0978030413389206, 0.046637315303087234, 0.2634376585483551, 0.16530895233154297, 0.02219507470726967, -0.03612581267952919, -0.19436585903167725, 0.10613010078668594, -0.15201328694820404, -0.08462464809417725, 0.012554734013974667, 0.006056043319404125, -0.11599508672952652, 0.11009762436151505, -0.08218853920698166, -0.16859416663646698, 0.031972404569387436, 0.01635279506444931, -0.2759424149990082, -0.0674772784113884, -0.13260167837142944, 0.15127389132976532, 0.15534383058547974, 0.2626085579395294, -0.1302935928106308, -0.011216710321605206, 0.07801762223243713, 0.07544974982738495, -0.011916026473045349, 0.1038404181599617, -0.007247020024806261, 0.10657111555337906, 0.02604258805513382, 0.1837361752986908, -0.07408549636602402, -0.2885458767414093, -0.15030799806118011, 0.08822415769100189, -0.2347889393568039, -0.06677792966365814, 0.2056320309638977, 0.13361267745494843, -0.025054266676306725, -0.05150911584496498, 0.012395464815199375, 0.018166912719607353, -0.04573768377304077, 0.06984012573957443, -0.0173660758882761, 0.03536079451441765, 0.015886196866631508, -0.25262922048568726, 0.02550783008337021, -0.039707984775304794, 0.06734028458595276, -0.039092324674129486, -0.05061647295951843, -0.35465338826179504, 0.09023860841989517, -0.0587962307035923, -0.08667852729558945, -0.0550302155315876, -0.07542458921670914, -0.16155366599559784, 0.040365587919950485, 0.2078154981136322, -0.23771630227565765, -0.13953647017478943, -0.11031913012266159}, {-0.24911652505397797, -0.016638942062854767, -0.1043144166469574, -0.06356343626976013, 0.08686140924692154, -0.0011032185284420848, -0.07885754853487015, -0.387601375579834, -0.06736449897289276, 0.06151575222611427, 0.03726679086685181, -0.16447384655475616, 0.020396726205945015, -0.13913141191005707, 0.02819267474114895, 0.05925901606678963, 0.11581040918827057, 0.03269470855593681, -0.031865838915109634, -0.04928046464920044, -0.049453217536211014, -0.017560910433530807, 0.14425623416900635, -0.15454073250293732, 0.14373745024204254, -0.08939845114946365, 0.07528062164783478, -0.02714388445019722, 0.1025463342666626, -0.15039648115634918, -0.006690965965390205, 0.14178970456123352, -0.03378262743353844, -0.0210175309330225, -0.22260424494743347, 0.31553423404693604, -0.0155580909922719, -0.007997725158929825, 0.24705365300178528, 0.04317475110292435, -0.03562557324767113, 0.1744106113910675, -0.12015603482723236, -0.026348480954766273, -0.10421988368034363, 0.011342016980051994, 0.10733305662870407, 0.07524527609348297, -0.12316884100437164, 0.09823904931545258, 0.02522413060069084, 0.05308990553021431, -0.029736537486314774, -0.0873015895485878, 0.19285979866981506, 0.16387373208999634, -0.07509712129831314, -0.18682102859020233, 0.06304317712783813, 0.01458163745701313, -0.14770065248012543, 0.23079939186573029, 0.1521611362695694, 0.5126211643218994, -0.09279783815145493, -0.27606427669525146, -0.12335599958896637, 0.1528577357530594, 0.09915770590305328, 0.06801172345876694, -0.08936283737421036, -0.06305745244026184, -0.27770644426345825, 0.03766544908285141, 0.015500920824706554, 0.025783754885196686, 0.04750391095876694, 0.18321122229099274, 0.08539706468582153, -0.06661290675401688, 0.0400710292160511, 0.1436612457036972, -0.12559643387794495, 0.058237224817276, -0.25129345059394836, -0.028396563604474068, -0.12006235867738724, -0.10400813817977905, -0.03776460886001587, -0.04674631357192993, -0.05071622505784035, -0.10185559093952179, -0.002497659530490637, 0.06896331161260605, 0.06200304254889488, 0.05774982273578644, -0.24269475042819977, -0.11463576555252075, 0.11636075377464294, 0.043539393693208694, -0.04982049763202667, -0.05969519168138504, -0.03019869141280651, -0.07457632571458817, 0.04589013010263443, 0.20134678483009338, -0.03179897367954254, 0.029400834813714027, -0.22285525500774384, -0.021489376202225685, -0.19513575732707977, -0.041403546929359436, 0.006964984815567732, -0.05946655943989754, 0.025730092078447342, 0.05867140367627144, -0.06179540976881981, 0.12477686256170273, 0.0764855444431305, 0.17328466475009918, 0.2536000609397888, 0.041380345821380615, 0.07970787584781647, 0.19172361493110657, -0.1405058652162552, -0.03754201903939247, 0.15006844699382782, -0.06367908418178558, -0.06783884763717651, 0.015199916437268257, -0.0639512687921524, -0.14041224122047424, 0.3137378990650177, -0.019361557438969612, 0.12312597781419754, 0.240083247423172, -0.12208563834428787, -0.11339510977268219, 0.20826901495456696, -0.08911842852830887, 0.044097792357206345, 0.05132005363702774, -0.036838408559560776, 0.00712770689278841, -0.13574257493019104, -0.007607053965330124, 0.17109596729278564, 0.08000791817903519, 0.11520598083734512, 0.18592458963394165, -0.09284605085849762, -0.030186159536242485, -0.0060260784812271595, -0.05199707671999931, -0.17337006330490112, -0.1946030855178833, 0.004879281390458345, 0.10906266421079636, 0.09462286531925201, 0.03796345368027687, 0.07647339254617691, -0.09679217636585236, -0.07663112133741379, -0.14218813180923462, -0.17939043045043945, 0.3195874094963074, 0.02310970239341259, 0.017288383096456528, 0.1884378045797348, -0.12263938784599304, -0.0775003656744957, -0.03174496442079544, -0.0005431879544630647, 0.2720729112625122, 0.1686444878578186, -0.03920463100075722, -0.09139814972877502, -0.10866636037826538, 0.09608695656061172, 0.02187042497098446, 0.050212860107421875, -0.1828380674123764, 0.04559773579239845, -0.08170018345117569, -0.016788899898529053, 0.1436348259449005, 0.23614151775836945, -0.022548217326402664, 0.15511807799339294, -0.11862408369779587, 0.022723248228430748, -0.07749127596616745, -0.03812888264656067, 0.13937561213970184, -0.3534514904022217, 0.09305626153945923, -0.16336560249328613, 0.08715169131755829, -0.03318095952272415, -0.1192035898566246, -0.0030174737330526114, 0.11393687129020691, -0.044584084302186966, 0.11263678967952728, 0.07767125964164734, -0.035170797258615494, 0.17738854885101318, -0.1641143262386322, -0.035947252064943314, -0.24463312327861786, -0.03799748048186302, 0.03181777521967888, -0.0952935740351677, 0.116994708776474, -0.045020394027233124, 0.0406646728515625, 0.028356684371829033, -0.03433474898338318, 0.07199336588382721, -0.1396692842245102, 0.24273711442947388, 0.08529345691204071, 0.012887921184301376, 0.15198394656181335, 0.0720905065536499, 0.10898023843765259, -0.08523613959550858, -0.23610514402389526, -0.016990290954709053, -0.18324394524097443, 0.020592642948031425, 0.0245102196931839, -0.07549850642681122, 0.15121449530124664, -0.12613658607006073, -0.10032732039690018, -0.14535470306873322, -0.019752632826566696, 0.08377379924058914, -0.0763004794716835, -0.05937653034925461, -0.18732203543186188, 0.05075262859463692, -0.08507364988327026, 0.19163736701011658, -0.10843414068222046, -0.1546270102262497, -0.07064199447631836, 0.057543911039829254, 0.10379673540592194, 0.027648501098155975, 0.07960503548383713, 0.16704966127872467, -0.07474665343761444, 0.10734062641859055, 0.08272264897823334}, {-0.002192982705309987, 0.20509764552116394, -0.02376597560942173, -0.008270150981843472, 0.1635333150625229, 0.11328890174627304, 0.1007576733827591, -0.07859577238559723, 0.10565072298049927, 0.18436963856220245, 0.1578759104013443, 0.24680595099925995, -0.07423491030931473, 0.08009284734725952, -0.0028925398364663124, -0.29233527183532715, 0.024588454514741898, 0.05136320739984512, 0.2103584110736847, 0.2801094949245453, 0.10418201237916946, -0.07951096445322037, -0.13974235951900482, 0.13613438606262207, -0.3939049243927002, 0.1327168047428131, -0.0033422750420868397, -0.04980887472629547, -0.2206195592880249, -0.14052222669124603, -0.1347852349281311, -0.05835297703742981, 0.10312514007091522, 0.08047529309988022, 0.1610274612903595, -0.030810406431555748, 0.056239210069179535, 0.1638902872800827, -0.05744416266679764, 0.07584014534950256, -0.16321921348571777, -0.10535266995429993, 0.12060932070016861, -0.05279984697699547, -0.055088020861148834, -0.016769148409366608, -0.016741521656513214, -0.2036454975605011, 0.12591980397701263, 0.048394571989774704, -0.10858187079429626, -0.03809304162859917, -0.022391440346837044, 0.08399080485105515, -0.06727281957864761, -0.2188517153263092, -0.20991702377796173, 0.0915556252002716, -0.11458650976419449, 0.13251715898513794, -0.20203916728496552, -0.19035083055496216, 0.2089425027370453, 0.16971728205680847, 0.08404674381017685, 0.10600545257329941, 0.023492297157645226, 0.04257883504033089, -0.09352842718362808, -0.25779637694358826, -0.08051210641860962, -0.1503734588623047, -0.26434916257858276, 0.03741363808512688, 0.03068450093269348, 0.17849195003509521, 0.02382178232073784, 0.33325737714767456, 0.040029458701610565, -0.39760327339172363, 0.024758586660027504, -0.028860658407211304, 0.1272282749414444, 0.13285183906555176, 0.19416795670986176, -0.06870624423027039, 0.09849020093679428, -0.025784185156226158, -0.031575627624988556, 0.0774175152182579, 0.14708958566188812, 0.22798392176628113, -0.03458845615386963, -0.06716082990169525, 0.01279500126838684, -0.27350911498069763, 0.03439057618379593, -0.08455292135477066, 0.11619831621646881, 0.0010800468735396862, 0.08622600883245468, 0.25211548805236816, 0.11348579078912735, -0.039153456687927246, -0.10826905071735382, -0.2107960730791092, -0.15652556717395782, -0.156268909573555, 0.02253783866763115, 0.14874784648418427, -0.2728215754032135, -0.10894706100225449, 0.19179745018482208, 0.320744127035141, 0.011717187240719795, -0.08901312947273254, 0.06815395504236221, -0.18500018119812012, -0.36993294954299927, 0.17297199368476868, -0.22992976009845734, 0.06672393530607224, -0.09635594487190247, 0.02496635913848877, 0.34719064831733704, 0.10113257169723511, -0.20717428624629974, -0.03459525853395462, 0.03916553780436516, -0.1959133744239807, -0.04563338682055473, 0.10458852350711823, 0.2472800463438034, 0.03329526633024216, -0.00974274892359972, -0.2184583842754364, -0.24206724762916565, 0.06773410737514496, -0.02374091185629368, -0.21723583340644836, -0.027817320078611374, 0.097064308822155, 0.08844143152236938, 0.1137554943561554, -0.24817420542240143, 0.02089373767375946, -0.1550111174583435, 0.0157119520008564, -0.3099527060985565, 0.045041274279356, -0.031549714505672455, 0.1683785766363144, -0.0025473730638623238, 0.04432648792862892, -0.17636431753635406, 0.007688462268561125, 0.254238486289978, -0.0034112436696887016, 0.08214720338582993, 0.20965398848056793, 0.0881468653678894, 0.1841353327035904, -0.02286369353532791, 0.1681474894285202, 0.05889395996928215, 0.32832202315330505, 0.29949840903282166, 0.1888091266155243, -0.11449838429689407, -0.061097651720047, 0.01810896396636963, 0.19758006930351257, -0.11636734753847122, -0.08154169470071793, -0.09004499763250351, -0.06578581035137177, 0.01441823411732912, 0.023013891652226448, 0.11692433059215546, -0.18153846263885498, -0.08376658707857132, 0.10299491137266159, 0.10207293182611465, -0.08959004282951355, 0.3453916907310486, -0.031804945319890976, 0.17338016629219055, 0.05379784107208252, -0.12981119751930237, 0.2716420590877533, -0.050855886191129684, 0.10361061245203018, 0.03358951956033707, -0.04480322077870369, 0.04039228707551956, 0.19337840378284454, 0.022883903235197067, -0.025222446769475937, -0.030277926474809647, -0.006744446698576212, 0.027692731469869614, 0.057214777916669846, 0.0007572844042442739, -0.00922088697552681, -0.039426546543836594, 0.06485427170991898, -0.13899794220924377, -0.0654277428984642, 0.15699797868728638, -0.25140616297721863, -0.19189608097076416, 0.20457273721694946, 0.08820595592260361, 0.13607004284858704, -0.1598803997039795, -0.051291752606630325, -0.10626980662345886, 0.08116002380847931, -0.1737413853406906, 0.10748418420553207, -0.08591744303703308, -0.0827965959906578, -0.17348764836788177, 0.01092628762125969, -0.012321380898356438, 0.2453622817993164, -0.20748946070671082, -0.04656042903661728, -0.042786166071891785, 0.052058443427085876, 0.3199681043624878, 0.01615241914987564, -0.02163807675242424, -0.15037016570568085, -0.0871570035815239, -0.09232501685619354, 0.08193007111549377, 0.0160888209939003, -0.0657360628247261, 0.2580525875091553, -0.04351058602333069, 0.013340175151824951, 0.07276571542024612, -0.06833400577306747, 0.07290691137313843, 0.15913090109825134, 0.007289149798452854, 0.004033795557916164, -0.013130787760019302, 0.06027894839644432, -0.05897468328475952, 0.13342685997486115, -0.050623923540115356, 0.17960374057292938, -0.055245012044906616, 0.04326541721820831}, {0.13438472151756287, 0.1576988250017166, 0.028528908267617226, 0.13674016296863556, -0.013558121398091316, 0.061715804040431976, -0.15034928917884827, 0.10318715870380402, 0.04218712076544762, -0.027461327612400055, -0.289069265127182, 0.3287099599838257, 0.059548065066337585, 0.20275065302848816, 0.08691568672657013, -0.1299673318862915, 0.1645267903804779, 0.10953512042760849, -0.03176403045654297, 0.08461152762174606, -0.08643033355474472, 0.15967991948127747, 0.22933261096477509, 0.028241368010640144, -0.10619060695171356, 0.16194766759872437, -0.1563001573085785, 0.006423234473913908, -0.19660817086696625, -0.16594050824642181, 0.006447615567594767, 0.0015413700602948666, -0.08562403917312622, 0.11579672247171402, -0.16093988716602325, -0.1145189180970192, -0.028722960501909256, -0.11515428125858307, -0.23717156052589417, -0.08224542438983917, 0.0033106356859207153, 0.037527237087488174, -0.004608191084116697, -0.09051673859357834, 0.017509330064058304, -0.1600135862827301, 0.12042541056871414, -0.10004130750894547, -0.008617245592176914, 0.16536130011081696, -0.1386609971523285, -0.08242698013782501, -0.1347939670085907, -0.0616973377764225, -0.08150405436754227, -0.19421979784965515, 0.1267765462398529, -0.015622682869434357, 0.026350412517786026, -0.20428906381130219, 0.2345527708530426, 0.16545623540878296, 0.006408799439668655, -0.26351282000541687, 0.1697583794593811, 0.049084972590208054, -0.09717992693185806, 0.034753795713186264, 0.11629857867956161, 0.04442662000656128, 0.1883373260498047, -0.018994882702827454, 0.17239680886268616, -0.14155513048171997, 0.16349798440933228, -0.11407337337732315, -0.07982297241687775, -0.19408763945102692, -0.036364030092954636, -0.06263403594493866, -0.06651818007230759, 0.010428112000226974, -0.09353595972061157, 0.07386307418346405, 0.11755035072565079, -0.008636494167149067, -0.0029692885000258684, -0.07683023065328598, 0.020793424919247627, -0.04796190932393074, 0.030406739562749863, 0.03642035275697708, 0.04590423405170441, -0.03406321257352829, 0.11530594527721405, 0.13598646223545074, 0.1854439526796341, 0.0946023017168045, -0.1077471449971199, -0.11401413381099701, 0.003952080849558115, 0.00922423880547285, -0.19386257231235504, -0.10707508027553558, 0.004627793561667204, 0.4612857699394226, -0.04025813937187195, 0.050795651972293854, 0.03662671893835068, 0.0012489469954743981, -0.03397829458117485, -0.08601056784391403, -0.28055375814437866, -0.09987452626228333, -0.08473461866378784, -0.08855120837688446, -0.017041446641087532, -0.1899900734424591, 0.056190375238657, -0.05552085489034653, 0.12346973270177841, -0.30043697357177734, -0.0014445072738453746, 0.10216327756643295, 0.03992898017168045, 0.022586876526474953, 0.14947640895843506, 0.030872255563735962, 0.0952054038643837, 0.09307749569416046, 0.12634479999542236, -0.013612539507448673, -0.10763335973024368, 0.18275588750839233, -0.4047742187976837, -0.1537199467420578, -0.0016926790121942759, 0.10659997165203094, -0.07108922302722931, 0.01978977769613266, 0.04150392860174179, -0.12870876491069794, 0.226852685213089, -0.09046309441328049, 0.34782758355140686, 0.05375690385699272, 0.3101710379123688, -0.010510151274502277, -0.07746578007936478, -0.02652202919125557, 0.049886081367731094, 0.11660420894622803, 0.07291831821203232, -0.026442350819706917, -0.022285370156168938, 0.1403350830078125, 0.08142614364624023, -0.0880841538310051, 0.139519602060318, -0.24219393730163574, 0.048861101269721985, -0.115700863301754, 0.08743979781866074, 0.08114123344421387, 0.1162162572145462, -0.3875453472137451, -0.06638132035732269, 0.05959261581301689, -0.08384519070386887, -0.03886030986905098, 0.27354711294174194, -0.0738232210278511, 0.05671337991952896, 0.024405546486377716, -0.01888640783727169, 0.05875701457262039, 0.1270386427640915, -0.04910336807370186, 0.057912081480026245, 0.018807826563715935, 0.13951224088668823, 0.1873808652162552, -0.11267838627099991, 0.020853739231824875, -0.02133355103433132, 0.027524542063474655, -0.023264147341251373, -0.2260740101337433, 0.04128929600119591, -0.01929149217903614, -0.09356102347373962, -0.07750815898180008, -0.29781004786491394, -0.1266261488199234, -0.08086211234331131, 0.13881170749664307, 0.21128137409687042, 0.20696814358234406, -0.029618317261338234, 0.07824788987636566, -0.04431022331118584, -0.183208167552948, 0.10853473097085953, -0.02160867676138878, -0.04700418934226036, 0.12214510142803192, 0.2105632722377777, 0.10729967802762985, 0.10166174918413162, -0.07405127584934235, -0.008297616615891457, -0.10424862056970596, -0.09488879144191742, -0.11839605122804642, 0.03447206690907478, 0.1117415651679039, -0.07039270550012589, -0.1343718022108078, 0.06083691120147705, 0.04656170681118965, 0.08450502157211304, 0.18061357736587524, -0.12475776672363281, -0.092142753303051, 0.10780644416809082, 0.05837516114115715, -0.0387398786842823, 0.12560318410396576, 0.1708923727273941, 0.016834335401654243, -0.13849082589149475, -0.00182136963121593, 0.11376854032278061, 0.21509981155395508, -0.1959645301103592, -0.11956837773323059, 0.29901233315467834, 0.04170761629939079, 0.1256503015756607, 0.11547602713108063, 0.2315656840801239, -0.03302128240466118, 0.12768414616584778, -0.057625237852334976, -0.2744174599647522, 0.26546353101730347, -0.06906537711620331, 0.26026690006256104, -0.13635358214378357, -0.2115420252084732, 0.20961417257785797, -0.013720804825425148, -0.09995575994253159, -0.11921955645084381, -0.16796310245990753, 0.03957061469554901}, {0.06584721058607101, -0.3412390649318695, -0.03363122418522835, 0.0013284069718793035, -0.27654898166656494, -0.17963576316833496, 0.04328112304210663, -0.081442691385746, -0.169280007481575, -0.045831553637981415, 0.07765494287014008, 0.01954776421189308, 0.06847622990608215, -0.02771386131644249, 0.02126549929380417, 0.08576997369527817, 0.07865919917821884, -0.19696252048015594, -0.06607115268707275, 0.1824459731578827, 0.1450895071029663, -0.08258986473083496, -0.015216442756354809, -0.2119230479001999, -0.19932147860527039, -0.02073979191482067, 0.1769496202468872, 0.005492277909070253, 0.031132647767663002, 0.1105419471859932, 0.07499358803033829, 0.2958000898361206, -0.04024122655391693, 0.04782242700457573, 0.04800298437476158, -0.12641099095344543, -0.07195630669593811, 0.01108703762292862, -0.17190831899642944, 0.025082867592573166, 0.11393458396196365, -0.04991847276687622, -0.2859971821308136, -0.14462703466415405, -0.259095698595047, 0.13389216363430023, -0.04004034772515297, -0.039668817073106766, 0.05445103347301483, -0.05440100282430649, -0.13888053596019745, 0.32033368945121765, -0.0219737458974123, -0.11306778341531754, -0.09442061930894852, -0.08879300206899643, -0.07855672389268875, 0.003563061123713851, -0.054943058639764786, 0.07919108122587204, 0.10059107095003128, -0.03915848955512047, 0.15051154792308807, 0.12607045471668243, 0.0852249339222908, -0.3980289697647095, -0.05563940852880478, 0.06809479743242264, -0.09088004380464554, 0.07289375364780426, 0.20551906526088715, 0.013505050912499428, -0.0014449540758505464, -0.09931927919387817, -0.11105368286371231, 0.1860678642988205, -0.09502103179693222, -0.1201159805059433, -0.11664145439863205, -0.1685754656791687, -0.11639408022165298, 0.09539110958576202, -0.3099316954612732, -0.0540032759308815, -0.27245041728019714, -0.06000567227602005, -0.2776947617530823, 0.010111244395375252, -0.05825253948569298, -0.07804130762815475, -0.04169655218720436, -0.44940030574798584, 0.11964891105890274, 0.0935833752155304, 0.21914662420749664, -0.041233472526073456, 0.047994136810302734, 0.06153017655014992, 0.32161855697631836, -0.24138151109218597, -0.17550598084926605, -0.19974364340305328, -0.09878863394260406, 0.12985429167747498, -0.35722243785858154, -0.3344925343990326, 0.07651367038488388, -0.018530866131186485, 0.12258195132017136, 0.008341310545802116, -0.04402485489845276, -0.06602374464273453, 0.18599630892276764, 0.041930850595235825, 0.020407790318131447, -0.004262440837919712, -0.08432286232709885, 0.07002653181552887, 0.13232296705245972, 0.2844362258911133, -0.010728910565376282, 0.10943536460399628, -0.1677105873823166, 0.17643339931964874, -0.09345731884241104, 0.0888681635260582, 0.17552365362644196, 0.010230068117380142, 0.05146550014615059, 0.20364739000797272, 0.11195708066225052, 0.186751589179039, 0.07968797534704208, -0.009556244127452374, 0.049133673310279846, -0.12931765615940094, -0.03215522691607475, -0.09486616402864456, -0.2689651846885681, -0.06033961847424507, -0.0067495983093976974, 0.10459773987531662, -0.23495806753635406, -0.022323543205857277, -0.09573265165090561, 0.12231101095676422, -0.11694952100515366, -0.12812061607837677, 0.19268397986888885, 0.1231263130903244, -0.07950489968061447, -0.057103682309389114, 0.08138870447874069, -0.0235789455473423, -0.1394949108362198, -0.11892211437225342, -0.060060132294893265, -0.038103457540273666, 0.12366674840450287, 0.02939877286553383, -0.10032996535301208, -0.07533367723226547, 0.2715853154659271, 0.0135984867811203, -0.24742402136325836, -0.04405083507299423, 0.09683435410261154, -0.05866900458931923, 0.17452867329120636, -0.05262153595685959, 0.12644967436790466, 0.1477578580379486, 0.1063055694103241, -0.05557384714484215, 0.057213716208934784, 0.07381732761859894, -0.0219629667699337, -0.05779152363538742, 0.04836688190698624, 0.015323367901146412, -0.02296469733119011, 0.05579676851630211, -0.07910605520009995, -0.14939577877521515, 0.29652637243270874, 0.17871344089508057, 0.1404004842042923, 0.11385990679264069, 0.07617742568254471, 0.11143805086612701, 0.044058140367269516, -0.10381960868835449, -0.09693901240825653, 0.24694587290287018, -0.3566083312034607, 0.11554776132106781, 0.014737469144165516, 0.2603556215763092, 0.12605565786361694, -0.21145904064178467, -0.016652055084705353, 0.10225898027420044, -0.040038637816905975, -0.08589227497577667, 0.012061958201229572, 0.044480275362730026, 0.0768023431301117, -0.01538910809904337, -0.127452552318573, 0.02312687411904335, 0.13602718710899353, 0.08872725069522858, -0.16277270019054413, 0.1171761155128479, 0.05524049326777458, 0.002607730682939291, 0.08957374095916748, -0.14470072090625763, -0.06658253818750381, -0.04545506834983826, 0.04633390158414841, -0.014439395628869534, -0.05719638243317604, 0.010560222901403904, -0.11354474723339081, -0.045260392129421234, -0.046491093933582306, 0.02425115928053856, -0.028123600408434868, -0.12394949048757553, -0.02579985372722149, -0.1507568508386612, -0.026540184393525124, 0.34765467047691345, -0.06081733480095863, 0.29678329825401306, 0.04306348040699959, 0.08334274590015411, -0.16253551840782166, -0.4385456442832947, -0.04605726897716522, 0.0077628642320632935, -0.09314299374818802, -0.0395272932946682, -0.12604358792304993, -0.27249622344970703, 0.11666013300418854, -0.2410793900489807, -0.09202469140291214, 0.21528147161006927, 0.22335954010486603, -0.44607701897621155, 0.09801173955202103, 0.0648358166217804, -0.03579656407237053, 0.1281837671995163}, {0.033290471881628036, -0.08897169679403305, -0.00434421980753541, -0.01760515198111534, 0.01819697953760624, 0.048531968146562576, 0.02792850323021412, -0.05535011738538742, 0.03091009147465229, -0.010562798008322716, -0.06899677217006683, -0.5853807926177979, 0.09257079660892487, -0.08771122246980667, 0.085175521671772, -0.13644766807556152, -0.14077387750148773, 0.02293979749083519, -0.07743840664625168, 0.06818893551826477, -0.07211440056562424, -0.18911106884479523, 0.0027256635949015617, -0.13054068386554718, 0.2202475219964981, -0.0751747190952301, 0.19421741366386414, 0.034681208431720734, 0.0029373555444180965, -0.05757753178477287, -0.01756196655333042, 0.18192872405052185, 0.03934483230113983, -0.05966716632246971, -0.07789064943790436, -0.02970208041369915, -0.06922870129346848, -0.061493389308452606, -0.27925992012023926, -0.03409276530146599, -0.08698014169931412, 0.0070435656234622, -0.25613507628440857, 0.027896584942936897, 0.06999760866165161, -0.02335435338318348, 0.04929212108254433, 0.027611644938588142, -0.04416743665933609, -0.004601247143000364, 0.14146430790424347, 0.19068613648414612, 0.12583960592746735, -0.13283424079418182, 0.005437727086246014, -0.20782552659511566, -0.09321161359548569, -0.016295278444886208, -0.004202274605631828, 0.15808947384357452, 0.09943042695522308, 0.16691549122333527, 0.027763137593865395, -0.0680437907576561, -0.16814841330051422, -0.11463090032339096, 0.185388445854187, -0.08583135902881622, -0.02382100373506546, -0.07305064797401428, 0.02229904755949974, -0.12211748957633972, 0.0461820550262928, 0.0643635019659996, -0.046210289001464844, -0.1276887059211731, 0.052306924015283585, 0.11164048314094543, -0.11248326301574707, 0.08746118098497391, -0.037586186081171036, -0.1351834535598755, -0.028452660888433456, 0.010787957347929478, -0.0730234682559967, 0.05475539341568947, 0.059450700879096985, -0.04352281615138054, -0.05160475894808769, -0.339982271194458, -0.08959689736366272, 0.1506470888853073, 0.18957139551639557, 0.051951587200164795, 0.0395360067486763, -0.0069626206532120705, 0.09099633991718292, -0.054562103003263474, -0.31265851855278015, -0.16390693187713623, -0.059557050466537476, 0.05185110867023468, 0.3465802073478699, 0.03147624060511589, -0.12102842330932617, 0.08213093131780624, 0.04752826690673828, 0.11673484742641449, -0.10593888908624649, 0.18703560531139374, -0.03193530812859535, -0.008953308686614037, -0.016095807775855064, 0.10989562422037125, -0.12721627950668335, 0.16243872046470642, -0.010013790801167488, 0.014674553647637367, 0.018808605149388313, -0.06101052090525627, -0.0013979187933728099, -0.11836723983287811, 0.07547720521688461, 0.1478162556886673, 0.0821402296423912, -0.10977647453546524, -0.002049510832875967, -0.13359224796295166, 0.09014979749917984, 0.05682925134897232, -0.029465200379490852, -0.08130496740341187, 0.07395486533641815, -0.026975199580192566, -0.19255918264389038, -0.22306422889232635, -0.014709832146763802, 0.05026673898100853, -0.03317130357027054, -0.18500591814517975, -0.06001320853829384, -0.10993710905313492, 0.022169824689626694, 0.20526304841041565, -0.15706861019134521, -0.07260657101869583, -0.012948674149811268, -0.061257172375917435, 0.013891663402318954, 0.021302763372659683, -0.03612494841217995, -0.029022522270679474, -0.0010526758851483464, 0.04207424819469452, 0.05543053150177002, 0.11836525797843933, 0.1906445324420929, -0.26521486043930054, 0.10534662753343582, 0.16604949533939362, -0.030956018716096878, -0.005586210172623396, -0.0918634757399559, 0.10904527455568314, -0.09171555936336517, 0.0860118418931961, 0.016436981037259102, -0.13487304747104645, -0.015283818356692791, 0.06533295661211014, 0.08864687383174896, 0.06989197432994843, -0.09013273566961288, 0.0032949107699096203, -0.21283774077892303, 0.03105258196592331, -0.030215110629796982, -0.02216612547636032, 0.01918718032538891, -0.06498600542545319, 0.033093877136707306, -0.7640671133995056, -0.03348133713006973, -0.08870552480220795, 0.001064626732841134, -0.24165771901607513, 0.05818474665284157, -0.04940909892320633, -0.1635182499885559, 0.08819060772657394, 0.12244196236133575, 0.031131966039538383, -0.10407691448926926, 0.030229149386286736, -0.26630860567092896, 0.09760341048240662, -0.0914449691772461, -0.032590094953775406, 0.0893588662147522, 0.02411724254488945, -0.06493745744228363, 0.052119117230176926, 0.01932121440768242, 0.04258245229721069, -0.16433203220367432, 0.022843673825263977, -0.06122979149222374, -0.18297135829925537, 0.09121435135602951, 0.06693247705698013, -0.045925576239824295, 0.1958533376455307, 0.025482866913080215, -0.08863286674022675, -0.038320086896419525, 0.21176709234714508, -0.07975483685731888, 0.03329745680093765, -0.10700657963752747, 0.03499956801533699, -0.023751163855195045, -0.040006738156080246, 0.03680654242634773, 0.2857990860939026, 0.04800940677523613, 0.08460204303264618, 0.03305288404226303, 0.09409414976835251, 0.0807671993970871, 0.06764229387044907, 0.12158197164535522, 0.1666657030582428, 0.19416341185569763, -0.05508577078580856, -0.01909872330725193, -0.20701389014720917, -0.0037633157335221767, -0.11277322471141815, 0.1255125105381012, 0.09773557633161545, 0.09057942777872086, -0.07605772465467453, -0.056191153824329376, -0.06114931404590607, -0.00604210002347827, 0.0011576444376260042, -0.03854797035455704, 0.0003375816158950329, -0.009558971039950848, -1.2901713262181147e-06, -0.1664654016494751, 0.04035381227731705, 0.1422164887189865, 0.07548213005065918, 0.07605604082345963, 0.07529472559690475}, {0.01364654116332531, 0.09583371877670288, 0.022755296900868416, 0.021192610263824463, 0.06113561615347862, 0.15527676045894623, 0.02960542030632496, 0.056099988520145416, -0.10222867131233215, 0.011782231740653515, -0.05963776633143425, -0.35001805424690247, 0.008844281546771526, 0.158223956823349, -0.1202523335814476, 0.0338151715695858, -0.06286929547786713, 0.1174987405538559, -0.1334865689277649, -0.15207719802856445, -0.048569198697805405, 0.07784566283226013, 0.08276090770959854, 0.0005866868305020034, 0.10477244108915329, -0.16790546476840973, -0.020929889753460884, 0.005912669003009796, -0.10214881598949432, -0.08246289193630219, -0.003163532353937626, -0.08028940856456757, 0.08892562240362167, 0.04195955768227577, -0.05885929614305496, -0.11187737435102463, 0.02897891215980053, -0.13227961957454681, -0.14007677137851715, 0.029463717713952065, -0.12812373042106628, 0.0735069066286087, -0.10864078253507614, -0.021121308207511902, 0.21348823606967926, 0.01760196126997471, 0.29372939467430115, -0.02723522298038006, -0.037435345351696014, 0.13440757989883423, 0.010232282802462578, 0.057428259402513504, 0.3307827115058899, 0.12006549537181854, 0.15785352885723114, -0.1651991307735443, 0.08026286959648132, 0.08525341004133224, -0.06722281128168106, 0.07165562361478806, -0.01878804713487625, 0.034333374351263046, -0.29829806089401245, -0.2685432732105255, -0.1497585028409958, 0.024322712793946266, 0.012070068158209324, -0.16145341098308563, -0.033851295709609985, -0.12746357917785645, -0.08098988980054855, 0.0827445462346077, 0.008978668600320816, 0.04183182492852211, 0.1261620968580246, -0.0845898762345314, 0.09516829997301102, -0.050065383315086365, 0.15405285358428955, 0.4047237038612366, 0.08230667561292648, -0.04181807115674019, -0.25017040967941284, -0.07504203915596008, -0.14238762855529785, -0.09432901442050934, -0.013858328573405743, 0.16193076968193054, 0.10243640840053558, -0.1601179838180542, 0.1948598176240921, -0.013561307452619076, 0.26070138812065125, 0.09953257441520691, -0.21016567945480347, 0.14219290018081665, -0.026908330619335175, 0.18485435843467712, -0.07866790145635605, 0.033605702221393585, -0.09664040803909302, -0.10522985458374023, 0.33369722962379456, 0.10499512404203415, -0.060710158199071884, 0.3858530521392822, -0.12064599245786667, -0.12168343365192413, 0.005110687110573053, 0.17815501987934113, -0.18020182847976685, -0.07801342010498047, -0.03836957365274429, 0.01420555915683508, 0.12668581306934357, 0.009788213297724724, -0.19378137588500977, -0.1511005312204361, -0.09807607531547546, 0.13305288553237915, -0.30070534348487854, -0.10236546397209167, 0.38751596212387085, -0.13256286084651947, -0.12182407826185226, 0.003943650983273983, -0.05315498635172844, 0.08859684318304062, -0.09851126372814178, -0.16948898136615753, 0.25420960783958435, 0.23098988831043243, 0.006937454454600811, -0.029754364863038063, -0.04086587578058243, -0.22773396968841553, -0.02358352206647396, -0.050086989998817444, -0.145077645778656, 0.14419406652450562, 0.008538071066141129, -0.06308098137378693, -0.025029294192790985, -0.1058126911520958, -0.12363182008266449, 0.09764690697193146, 0.09805577248334885, -0.26268911361694336, 0.02045769803225994, -0.02758096531033516, -0.011113680899143219, -0.14460964500904083, -0.08964869379997253, -0.06066213548183441, -0.015802666544914246, -0.03755717724561691, -0.042650941759347916, -0.24428339302539825, 0.12017118185758591, -0.05619105324149132, -0.1623477190732956, -0.13976000249385834, -0.038904234766960144, -0.23458229005336761, 0.2191050499677658, 0.06054041534662247, 0.11142546683549881, -0.00388380722142756, -0.16931365430355072, -0.06416166573762894, 0.09996003657579422, 0.14100663363933563, -0.11404004693031311, 0.058450695127248764, 0.07014696300029755, -0.05857120826840401, -0.09302691370248795, 0.4748587906360626, -0.3203381896018982, 0.17385853826999664, 0.008197096176445484, -0.7175754904747009, -0.04482860863208771, -0.136257603764534, 0.1413997858762741, -0.16164937615394592, 0.0906866267323494, 0.022089309990406036, -0.045071858912706375, 0.19265127182006836, 0.11555533856153488, -0.3561093509197235, 0.10616438835859299, 0.03815252706408501, -0.07734270393848419, 0.2640763223171234, 0.08989621698856354, 0.010111680254340172, -0.06123277172446251, 0.081690214574337, -0.083974689245224, -0.2492763102054596, -0.04609140753746033, 0.11423085629940033, -0.1430421769618988, -0.009113986045122147, -0.0893762856721878, -0.05295303091406822, -0.05182486027479172, 0.1639908254146576, -0.13238802552223206, -0.11511804908514023, -0.2666662633419037, 0.036152567714452744, 0.09950092434883118, 0.015452161431312561, 0.0808582678437233, -0.10219412297010422, 0.0995919480919838, -0.24266840517520905, -0.2524590492248535, 0.01843870058655739, -0.0260678231716156, 0.01872832328081131, 0.06821371614933014, 0.13180717825889587, -0.10398493707180023, 0.11339099705219269, 0.15250162780284882, 0.07388734817504883, 0.16323518753051758, 0.012642459012567997, 0.017353080213069916, -0.03128115460276604, -0.2292005866765976, 0.004942011088132858, -0.0701451227068901, 0.1428178995847702, -0.06949090957641602, -0.055599797517061234, 0.03994043543934822, 0.0026021618396043777, 0.014843858778476715, 0.010991761460900307, -0.0572943352162838, -0.025304662063717842, 0.032642003148794174, -0.06405744701623917, -0.019200095906853676, 0.028868963941931725, -0.2914571464061737, 0.049602359533309937, 0.02662123553454876, -0.1645749807357788, -0.05049235001206398, 0.11784997582435608}, {-0.04039480537176132, 0.02008899487555027, -0.10164704918861389, -0.011785161681473255, -0.012541543692350388, 0.19416984915733337, 0.11623553186655045, -0.11415110528469086, -0.09433522820472717, 0.11282709985971451, -0.08283990621566772, -0.6295635104179382, -0.08021537214517593, 0.03430391848087311, 0.11200343817472458, -0.18736471235752106, -0.14265550673007965, 0.04611197113990784, -0.3123529851436615, 0.002234902698546648, -0.00629345141351223, -0.07886119186878204, 0.032490186393260956, 0.028908124193549156, 0.19836170971393585, 0.005136293359100819, 0.13035213947296143, 0.08161699771881104, 0.10235045850276947, 0.1319837123155594, -0.1481368988752365, 0.17441898584365845, 0.1087837815284729, -0.0454372763633728, -0.11956017464399338, -0.08226575702428818, -0.09655552357435226, -0.051256343722343445, -0.07023195177316666, 0.020786400884389877, 0.030929608270525932, -0.05621807277202606, -0.0420844629406929, 0.07161501795053482, 0.12652964890003204, -0.11819661408662796, -0.04382387176156044, 0.07936818897724152, 0.06332670897245407, -0.021930648013949394, 0.11179300397634506, 0.18920227885246277, 0.1235642060637474, -0.010525265708565712, 0.13159491121768951, -0.1397029310464859, -0.13518370687961578, 0.002712214132770896, -0.03666920214891434, -0.13553152978420258, -0.08349321782588959, 0.09700127691030502, -0.054551687091588974, -0.11938752979040146, -0.028568539768457413, -0.08215252310037613, -0.01507437415421009, -0.05113385245203972, -0.07130709290504456, 0.009203750640153885, 0.07864239811897278, 0.0417279377579689, -0.04059110954403877, 0.2297677993774414, 0.12628020346164703, 0.013019062578678131, 0.013173983432352543, 0.15252545475959778, -0.12254592031240463, 0.2749994397163391, 0.025094076991081238, 0.08771136403083801, 0.0527864433825016, -0.00377883343026042, -0.1595374196767807, 0.13161325454711914, 0.14111685752868652, -0.009283527731895447, -0.02964918315410614, -0.18203790485858917, 0.07427505403757095, 0.09967730194330215, 0.025562966242432594, 0.010613120160996914, -0.013715957291424274, -0.005694806110113859, -0.05106228590011597, -0.1250963807106018, -0.3881526291370392, 0.025521207600831985, -0.044295791536569595, -0.0629873052239418, 0.2665322422981262, 0.16111567616462708, 0.033695071935653687, 0.26192304491996765, 0.07561787217855453, 0.06311579793691635, -0.02192934788763523, 0.1700906604528427, -0.08634567260742188, -0.11178668588399887, 0.13629265129566193, 0.08949349075555801, 0.061353594064712524, 0.08631550520658493, 0.020571326836943626, 0.05881078541278839, -0.015152866952121258, -0.10363590717315674, -0.03825991600751877, -0.06053389236330986, -0.05788079649209976, 0.16850973665714264, 0.014600535854697227, 0.049651850014925, -0.027997231110930443, -0.06424833089113235, 0.14106421172618866, 0.04705042392015457, 0.019412150606513023, 5.1449598686303943e-05, 0.06660649180412292, -0.02597828209400177, -0.21358118951320648, -0.10400594025850296, -0.07370748370885849, 0.05115227401256561, 0.020106276497244835, -0.06416907161474228, 0.05973004177212715, -0.005146156996488571, 0.14996157586574554, 0.06875704228878021, -0.04722100496292114, 0.06251049041748047, 0.19423168897628784, -0.02683280222117901, 0.018796509131789207, -0.013620811514556408, -0.0028899272438138723, -0.00595932686701417, 0.024698687717318535, -0.014792473055422306, 0.08072414249181747, -0.03254858776926994, 0.03660140186548233, -0.04378334805369377, -0.08332447707653046, 0.14698725938796997, -0.0976189523935318, 0.04179202765226364, -0.0635424479842186, -0.051276713609695435, 0.05277803912758827, 0.17520089447498322, -0.061886366456747055, -0.04856044054031372, -0.017937621101737022, -0.010349706746637821, 0.001862284610979259, 0.14244617521762848, -0.06236638128757477, -0.06580889225006104, -0.24809059500694275, -0.09810076653957367, 0.12922708690166473, -0.1668136715888977, -0.05746128410100937, -0.09029232710599899, 0.0422547347843647, -1.1513592004776, 0.10598818212747574, -0.08368334174156189, -0.014233442023396492, -0.04627922922372818, 0.07881239056587219, -0.07850349694490433, -0.26813313364982605, -0.06594227254390717, 0.1106732040643692, -0.0817684456706047, 0.05044383928179741, -0.13177387416362762, -0.17952361702919006, -0.03476288914680481, -0.04757560417056084, -0.07472734898328781, -0.0023405253887176514, 0.04695791378617287, 0.08874338120222092, 0.1372566521167755, 0.05113513767719269, 0.11152789741754532, -0.13655544817447662, -0.05908817797899246, -0.09265880286693573, -0.2001577615737915, 0.12464708834886551, -0.001095038140192628, -0.13443662226200104, 0.1383531093597412, -0.08988818526268005, -0.12010234594345093, 0.029044844210147858, 0.03606348857283592, 0.08450394868850708, 0.025398319587111473, 0.09777383506298065, -0.044948726892471313, -0.0766124576330185, 0.0635317787528038, 0.010081244632601738, 0.24618293344974518, 0.09125242382287979, 0.08072534948587418, 0.045418839901685715, 0.13562323153018951, 0.11354973912239075, 0.21124064922332764, 0.0041370573453605175, 0.06399202346801758, 0.0029360780026763678, -0.09249900281429291, -0.1770002841949463, -0.11738482117652893, 0.02701716311275959, -0.06441635638475418, 0.07349073886871338, 0.31030336022377014, -0.05721276253461838, 0.02402477338910103, -0.003156395861878991, -0.03247428312897682, 0.0030613585840910673, -0.03655272349715233, 0.10126544535160065, 0.02109598182141781, -0.03186542168259621, 0.13091614842414856, -0.06865835934877396, 0.006246655248105526, 0.003700014902278781, 0.008153675124049187, -0.07706837356090546, 0.0069018141366541386}, {-0.14878268539905548, -0.023117952048778534, 0.0022665781434625387, 0.05124032869935036, -0.15405425429344177, -0.12578167021274567, -0.13940614461898804, 0.04693334922194481, 0.2127096951007843, -0.10628901422023773, 0.13178598880767822, 0.4702489376068115, 0.018653081730008125, -0.0712021142244339, 0.0939406156539917, -0.07741949707269669, -0.14671210944652557, -0.053260911256074905, 0.16058023273944855, 0.0252612866461277, 0.008171206340193748, -0.07797592878341675, -0.06648021936416626, -0.1249709352850914, -0.2485017627477646, 0.165002703666687, -0.15626098215579987, 0.02742559276521206, -0.012456611730158329, 0.1338820904493332, 0.08981622010469437, -0.24321384727954865, -0.15410974621772766, -0.04388563707470894, -0.06557098776102066, 0.16684424877166748, -0.016749223694205284, 0.21773552894592285, -0.03278542682528496, 0.10027046501636505, -0.10647629201412201, -0.04883395880460739, -0.04853237792849541, 0.08736438304185867, 0.08960674703121185, 0.06707123667001724, -0.0819602757692337, -0.01254073716700077, -0.05538313835859299, -0.07125380635261536, 0.019995523616671562, -0.7147568464279175, -0.14472858607769012, -0.13320355117321014, -0.1032886803150177, 0.019537584856152534, 0.07490315288305283, -0.09375345706939697, -0.04562738165259361, -0.05809910595417023, -0.13371576368808746, -0.020176902413368225, 0.149720698595047, 0.11807635426521301, -0.07479657977819443, 0.23060354590415955, -0.09962216764688492, 0.19126056134700775, -0.34940409660339355, -0.22296182811260223, -0.024995744228363037, 0.06185290589928627, 0.04256861284375191, -0.12664243578910828, 0.0297293271869421, -0.016638631001114845, 0.04813191294670105, -0.1240057572722435, 0.01858152449131012, -0.17383290827274323, 0.06298995018005371, 0.07186363637447357, -0.002584458328783512, -0.1280561238527298, -0.02700727805495262, -0.04588732495903969, -0.05736337602138519, -0.05676676705479622, -0.12649478018283844, 0.08332647383213043, -0.09126272052526474, -0.04650867357850075, 0.02982821688055992, 0.10866258293390274, -0.18596602976322174, 0.021444052457809448, -0.009232927113771439, 0.12541456520557404, -0.054917510598897934, 0.11303093284368515, -0.17105533182621002, -0.11610836535692215, -0.2936825454235077, 0.008827252313494682, -0.06765058636665344, 0.03464978560805321, -0.05568563565611839, -0.041280653327703476, -0.14893600344657898, -0.07172694057226181, 0.15745948255062103, 0.20381253957748413, 0.11430531740188599, -0.14122550189495087, -0.09473678469657898, -0.24571171402931213, -0.11125966161489487, 0.11967039108276367, -0.0187616515904665, 0.021975070238113403, 0.3291574716567993, -0.15484848618507385, -0.19896520674228668, -0.011110457591712475, 0.1270199418067932, 0.14672815799713135, 0.009959191083908081, -0.02316734381020069, -0.024696115404367447, -0.31976020336151123, 0.01559054758399725, -0.01660173386335373, -0.011024653911590576, 0.016986964270472527, -0.05880742892622948, 0.022098645567893982, -0.11293867975473404, 0.11204967647790909, 0.04185239598155022, -0.2803254723548889, 0.12893880903720856, -0.06365659832954407, -0.06395301967859268, 0.006053994409739971, 0.14824701845645905, 0.07016412913799286, -0.06731392443180084, -0.09243961423635483, -0.157934308052063, 0.1771521270275116, 0.04061683267354965, 0.03262846916913986, -0.13566990196704865, 0.026302997022867203, -0.07930709421634674, 0.041196346282958984, -0.07654841244220734, 0.10879678279161453, -0.01397042814642191, 0.020585721358656883, 0.03381193429231644, 0.008689610287547112, 0.06275729835033417, 0.0937856137752533, 0.02699529193341732, -0.12116836756467819, -0.13850310444831848, -0.04176924377679825, -0.0776187852025032, -0.4364212155342102, 0.15498103201389313, 0.08929572254419327, -0.0957224890589714, 0.2201966792345047, 0.1312902867794037, -0.11325277388095856, -0.10285487025976181, 0.0331481508910656, 0.030485957860946655, -0.07248635590076447, -0.073776476085186, 0.43088966608047485, -0.11226456612348557, 0.260355681180954, 0.2103203386068344, 0.09762216359376907, -0.13912248611450195, 0.05251966416835785, 0.05232871323823929, 0.029422976076602936, -0.09511742740869522, 0.08693361282348633, 0.08674117177724838, 0.33612260222435, 0.1705130934715271, -0.04136516898870468, -0.015053050592541695, -0.08722661435604095, 0.01228178758174181, -0.06353357434272766, 0.005378790199756622, 0.07945126295089722, 0.1344674527645111, 0.19210079312324524, 0.027597809210419655, 0.1132560521364212, -0.03759083151817322, 0.381600946187973, 0.044686634093523026, -0.04728512465953827, 0.08038685470819473, -0.13790087401866913, 0.06178969517350197, -0.02621752768754959, 0.15811245143413544, 0.022851433604955673, -0.12683767080307007, -0.14231185615062714, -0.23827891051769257, 0.2317219078540802, 0.05450599640607834, -0.021849242970347404, 0.20279356837272644, -0.261572003364563, -0.17182986438274384, -0.05112959444522858, 0.3282874822616577, 0.08208931237459183, 0.008839582093060017, -0.2195233255624771, -0.059332817792892456, -0.03215538710355759, 0.12639379501342773, 0.10958962887525558, -0.16030529141426086, -0.1336648166179657, -0.2256505787372589, 0.37606462836265564, -0.13021786510944366, 0.06468340754508972, -0.13453403115272522, -0.07275966554880142, -0.10358721017837524, 0.004318363033235073, 0.024547109380364418, 0.08710160106420517, -0.0416492260992527, -0.006539694499224424, 0.003931622486561537, 0.38607433438301086, -0.014474154449999332, 0.009644396603107452, -0.07216393947601318, -0.14520925283432007, -0.06035088002681732, 0.10457036644220352}, {0.012943140231072903, -0.13454753160476685, 0.1630670577287674, -0.06964407116174698, 0.009600993245840073, 0.010323618538677692, 0.051531121134757996, -0.0876050591468811, -0.10668247193098068, 0.0017337879398837686, 0.0960635393857956, 0.023597078397870064, -0.04799075424671173, -0.06161145120859146, -0.12620103359222412, 0.14914748072624207, 0.15382812917232513, -0.01732771471142769, -0.14406192302703857, 0.03161901608109474, -0.055991824716329575, -0.12234165519475937, 0.024020493030548096, -0.04255080595612526, 0.10984200984239578, -0.07540879398584366, -0.07730896025896072, -0.1156766265630722, 0.013599884696304798, -0.0009542029583826661, 0.05072886869311333, -0.08766400814056396, -0.15571075677871704, 0.1437768042087555, 0.15562783181667328, 0.1088712215423584, -0.03880099579691887, 0.003906481433659792, 0.024235345423221588, 0.06845574826002121, 0.09012531489133835, -0.032871562987565994, -0.053252387791872025, -0.15562866628170013, -0.14200201630592346, 0.15232405066490173, 0.049108896404504776, 0.018600601702928543, -0.0817808136343956, -0.026150111109018326, 0.25754514336586, -0.04160637781023979, 0.08538033813238144, 0.05684458836913109, 0.14830641448497772, 0.08402460813522339, 0.05038343369960785, -0.016314171254634857, -0.11418411135673523, 0.28654441237449646, -0.1632491648197174, 0.19879914820194244, 0.09673385322093964, 0.08441391587257385, -0.04139742627739906, 0.004709664732217789, 0.045224420726299286, 0.1427372843027115, 0.06147623434662819, 0.04700176790356636, -0.21933728456497192, 0.08773424476385117, 0.19251985847949982, -0.02977299503982067, -0.5153102278709412, -0.057640545070171356, -0.03243519365787506, -0.16768360137939453, 0.0548211894929409, 0.003931672312319279, 0.24902892112731934, -0.0070563978515565395, -0.27142030000686646, -0.03482098877429962, -0.07041142135858536, 0.14984990656375885, 0.0526139959692955, 0.2323264330625534, 0.041239019483327866, 0.020603155717253685, -0.13889293372631073, 0.027981746941804886, 0.1234707310795784, 0.20855246484279633, -0.031085312366485596, 0.06063603237271309, -0.1005338802933693, 0.034562848508358, 0.13415850698947906, -0.30810442566871643, 0.07194623351097107, -0.16880358755588531, -0.1270316243171692, 0.058832839131355286, -0.17017598450183868, -0.20317469537258148, 0.06755711883306503, -0.02522849664092064, -0.07534943521022797, 0.017224404960870743, -0.016927193850278854, -0.07537565380334854, 0.05727796256542206, -0.138427272439003, 0.10656019300222397, -0.057521551847457886, 0.008297097869217396, -0.051584307104349136, -0.07104337960481644, 0.030033063143491745, -0.132883220911026, 0.12275116890668869, 0.11366895586252213, 0.06770087033510208, -0.009671180509030819, -0.11709246039390564, -0.0006314837373793125, 0.03547921031713486, -0.08246978372335434, -0.05402400717139244, -0.01705484837293625, -0.04902040213346481, -0.09360991418361664, 0.06250111013650894, 0.0991864874958992, 0.15527968108654022, -0.02511216513812542, -0.019562149420380592, 0.0341353714466095, 0.016921816393733025, -0.02116558700799942, 0.033336322754621506, 0.12802140414714813, 0.1051865816116333, -0.05803807079792023, -0.025548119097948074, -0.11765515804290771, -0.14381439983844757, 0.038747455924749374, -0.010463902726769447, -0.013313021510839462, -0.06645597517490387, 0.06786837428808212, -0.025781646370887756, -0.039199505001306534, 0.07707492262125015, 0.031958092004060745, 0.030446777120232582, -0.14811041951179504, 0.008530240505933762, 0.03648587316274643, -0.2159174084663391, 0.027365000918507576, 0.006223376840353012, -0.07216374576091766, 0.03666646406054497, 0.05347220599651337, -0.11908194422721863, 0.008763132616877556, -0.1415582001209259, 0.04396463558077812, -0.04804620146751404, -0.009446985088288784, 0.11070399731397629, 0.1517050713300705, 0.049659162759780884, -0.08187735825777054, 0.056088242679834366, -0.10996268689632416, 0.11132550239562988, -0.07972633838653564, -0.0008091272320598364, 0.15410014986991882, 0.0547032430768013, -0.023602666333317757, 0.22846856713294983, -0.04989747330546379, -0.0388585701584816, 0.10055067390203476, 0.03577045723795891, -0.0874234139919281, -0.06237254664301872, 0.1530086249113083, 0.38587260246276855, -0.035899125039577484, -0.09466974437236786, -0.20569132268428802, -0.18041574954986572, 0.12759308516979218, -0.26747873425483704, 0.055797141045331955, -0.027328884229063988, 0.15840573608875275, 0.07856716215610504, 0.015093836933374405, -0.025824885815382004, -0.236882284283638, 0.06095072254538536, 0.036642685532569885, 0.014448664151132107, -0.04237218201160431, 0.032796043902635574, 0.11988198757171631, 0.029365185648202896, -0.006470078136771917, 0.026758266612887383, 0.13476385176181793, -0.12328699231147766, 0.13986308872699738, -0.19033055007457733, -0.20850802958011627, 0.143363356590271, 0.17368455231189728, 0.04976443573832512, 0.0027242794167250395, -0.0042444858700037, -0.046752940863370895, -0.22466489672660828, -0.007123256102204323, -0.13293679058551788, -0.037711575627326965, -0.11062399297952652, -0.06220359355211258, 0.11004938930273056, -0.026616623625159264, 0.13396337628364563, 0.029377534985542297, -0.003724602283909917, -0.05060587450861931, -0.27729174494743347, -0.20476298034191132, 0.14848174154758453, 0.10827292501926422, 0.07101711630821228, -0.023730874061584473, -0.03269065171480179, 0.008438118733465672, -0.033445265144109726, -0.012909811921417713, 0.06162862852215767, -0.11373341828584671, -0.08706891536712646, -0.019343677908182144, 0.06070757284760475, 0.04994242265820503, -0.02106548845767975}, {0.0033366824500262737, 0.10114790499210358, -0.058714188635349274, 0.1827017068862915, 0.03529603034257889, 0.02706276625394821, 0.05614688619971275, -0.01990618370473385, -0.3818225860595703, -0.2588130533695221, -0.08212655037641525, -0.01633569598197937, 0.17310987412929535, 0.05520664528012276, -0.020094318315386772, 0.17497049272060394, 0.106003537774086, 0.05988283082842827, -0.11498785763978958, -0.3021507263183594, 0.011527310125529766, -0.02321818843483925, 0.2760849893093109, 0.23140931129455566, 0.21720021963119507, -0.023856107145547867, 0.007508035749197006, 0.04895227774977684, -0.13730230927467346, -0.20562441647052765, 0.3018245995044708, -0.12375131249427795, -0.09611959010362625, 0.09830489754676819, -0.005540765821933746, -0.1033114567399025, 0.09267453104257584, -0.19412760436534882, 0.10146577656269073, 0.1112934872508049, -0.126266747713089, 0.00017323654901701957, -0.048679180443286896, -0.025989603251218796, 0.11933350563049316, -0.1552867889404297, 0.08044576644897461, -0.0066758934408426285, 0.11385401338338852, 0.02236870303750038, -0.014666417613625526, 0.3247443437576294, 0.17909976840019226, -0.016758542507886887, -0.02432357706129551, -0.10197149962186813, 0.08399021625518799, 0.19550451636314392, -0.2375604808330536, 0.028412634506821632, -0.0822400450706482, -0.254576176404953, -0.08226991444826126, 0.10665963590145111, -0.04146069660782814, 0.12170639634132385, 0.053336773067712784, -0.05133659020066261, 0.13035373389720917, -0.21782292425632477, -0.06875170022249222, 0.07415803521871567, -0.11506890505552292, 0.2811850905418396, 0.15365418791770935, 0.18224774301052094, -0.2534995675086975, 0.14137747883796692, 0.013686036691069603, 0.18473327159881592, 0.14960092306137085, 0.01135426014661789, 0.11365810036659241, -0.13842450082302094, -0.012791560031473637, -0.17461560666561127, 0.26357731223106384, 0.11686845123767853, 0.09328633546829224, 0.0350819006562233, -0.13228890299797058, 0.21070711314678192, -0.10626152157783508, 0.03420919179916382, 0.04965086653828621, -0.009369091130793095, -0.09400833398103714, -0.11974239349365234, 0.10461245477199554, 0.1251075118780136, 0.11838679015636444, 0.18906830251216888, 0.21044722199440002, 0.32092294096946716, -0.0418211929500103, 0.7787538766860962, -0.15301616489887238, -0.045518893748521805, 0.11258392035961151, 0.378126859664917, -0.07095695286989212, -0.18022437393665314, 0.062069643288850784, -0.04382086545228958, -0.11365597695112228, -0.003962422721087933, 0.13096824288368225, -0.16646502912044525, 0.1052209883928299, 0.008051805198192596, -0.057356055825948715, 0.1319393813610077, -0.12125416845083237, 0.17861171066761017, 0.03594207018613815, 0.07661756873130798, 0.012565969489514828, -0.11764128506183624, 0.09596094489097595, 0.15753993391990662, 0.029878567904233932, 0.055906664580106735, -0.045455560088157654, -0.3337762653827667, -0.06227995827794075, 0.0850108191370964, 0.05945327505469322, -0.005511659663170576, -0.02538572996854782, -0.07179544121026993, 0.07787737995386124, -0.19540128111839294, -0.08549332618713379, 0.07296976447105408, -0.030351931229233742, 0.1282128244638443, -0.13612954318523407, 0.0918925479054451, -0.01479550264775753, 0.23346276581287384, 0.14963047206401825, 0.10002820193767548, 0.037979889661073685, -0.19366909563541412, 0.02195744961500168, 0.0347474068403244, -0.055981654673814774, 0.00725410133600235, -0.028189683333039284, -0.048954445868730545, -0.08653423935174942, 0.04307914525270462, -0.03420956805348396, -0.10747500509023666, 0.14794683456420898, 0.20565493404865265, -0.14366157352924347, -0.039734143763780594, -0.046438612043857574, -0.024018781259655952, 0.039059173315763474, -0.1855350136756897, 0.0952291265130043, -0.09728088974952698, -0.18516960740089417, 0.043877892196178436, 0.17849412560462952, -0.1818750500679016, -0.021009279415011406, -0.04818527773022652, 0.07823463529348373, -0.11169921606779099, 0.008275416679680347, -0.021487580612301826, -0.022573227062821388, -0.21590986847877502, -0.3817809224128723, 0.22355328500270844, -0.057841602712869644, 0.04584215208888054, -0.07956786453723907, 0.05351106822490692, 0.1764814704656601, 0.0682697668671608, 0.20178180932998657, 0.06359466910362244, 0.03924870863556862, -0.07055149972438812, -0.09400247782468796, -0.03959863260388374, -0.08358553797006607, 0.17401540279388428, -0.011714152991771698, 0.021047545596957207, -0.2081815004348755, -0.16416826844215393, 0.0030415079090744257, -0.008633147925138474, 0.036764562129974365, 0.10171721130609512, -0.13194923102855682, -0.18369702994823456, -0.09527212381362915, -0.23248587548732758, -0.05835750699043274, 0.14246967434883118, 0.03451593965291977, -0.03240017965435982, 0.0401870459318161, -0.00854397937655449, 0.06947048753499985, 0.12512420117855072, -0.08463820815086365, -0.1298910677433014, -0.035463277250528336, -0.20599530637264252, -0.22299517691135406, 0.03755510598421097, 0.19545283913612366, 0.17251813411712646, 0.08213765174150467, 0.1218271479010582, 0.10705367475748062, -0.08009345084428787, -0.24883948266506195, -0.07122685015201569, 0.22643300890922546, 0.288068562746048, -0.28351154923439026, 0.33224451541900635, 0.010452456772327423, 0.08309997618198395, -0.11260392516851425, -0.12280368059873581, -0.1783851683139801, 0.006937812548130751, -0.05679697543382645, 0.03817921504378319, 0.13996979594230652, 0.1435798555612564, -0.13199488818645477, 0.15928931534290314, -0.013797979801893234, -0.04185634106397629, -0.15770624577999115, -0.016949748620390892}, {0.0334940031170845, -0.14400234818458557, -0.21263112127780914, -0.10940975695848465, 0.17708063125610352, -0.11994022876024246, -0.02720647305250168, -0.11144842207431793, -0.09066282957792282, -0.04601486027240753, -0.11667357385158539, 0.015485643409192562, -0.1351052075624466, -0.020370686426758766, -0.009097098372876644, 0.11024518311023712, 0.032228708267211914, 0.1238507628440857, -0.043794963508844376, 0.10104885697364807, -0.12444036453962326, 0.026177773252129555, 0.11682341992855072, -0.1323825567960739, -0.0750977024435997, 0.28459954261779785, 0.4278344511985779, -0.14546072483062744, 0.10407376289367676, -0.16572333872318268, 0.2782702147960663, -0.07376985251903534, -0.3721647560596466, 0.07236910611391068, -0.048656582832336426, 0.11703500151634216, -0.011138644069433212, -0.05050477758049965, 0.10002323240041733, 0.08546330034732819, -0.01249676663428545, -0.07523509860038757, -0.07443121820688248, -0.032369911670684814, 0.053928833454847336, 0.22772295773029327, -0.010112310759723186, 0.12681418657302856, 0.182816281914711, 0.07194173336029053, 0.22185513377189636, -0.08203441649675369, -0.007609705440700054, -0.06038852408528328, 0.17650818824768066, -0.019348831847310066, 0.23109541833400726, 0.10061173141002655, 0.1134713664650917, 0.07550255954265594, 0.03681169077754021, -0.0968233272433281, 0.07471781969070435, -0.050049081444740295, 0.043077580630779266, -0.056780941784381866, -0.04256905987858772, 0.21361443400382996, 0.028773415833711624, -0.026204882189631462, -0.12007198482751846, 0.021569374948740005, 0.07632546126842499, 0.11156809329986572, 0.0515175499022007, -0.029375024139881134, -0.14789755642414093, -0.0037840143777430058, -0.09636720269918442, 0.1645514816045761, 0.1656728833913803, 0.160636767745018, 0.010433722287416458, -0.10612209141254425, 0.008067182265222073, 0.18615010380744934, -0.019724031910300255, 0.020990358665585518, -0.03304613381624222, 0.19855301082134247, -0.13073308765888214, 0.10831505060195923, -0.034233804792165756, -0.13322725892066956, -0.16130085289478302, 0.22790299355983734, -0.16982316970825195, 0.0855022445321083, -0.06402160227298737, -0.06024594232439995, -0.046439699828624725, -0.13452011346817017, -0.06715060025453568, -0.08550825715065002, -0.4306183457374573, 0.05938836932182312, -0.08754124492406845, 0.036798521876335144, -0.12187495827674866, -0.043579209595918655, 0.015550077892839909, 0.029953908175230026, -0.00998601969331503, -0.13266846537590027, 0.053923770785331726, -0.09286002814769745, -0.02711685746908188, 0.04680584371089935, -0.12353624403476715, 0.09330222010612488, -0.02529127337038517, 0.06282338500022888, 0.24149125814437866, 0.12090537697076797, 0.00824794266372919, 0.14456497132778168, -0.01478940062224865, -0.06260139495134354, 0.07876396924257278, 0.13108685612678528, -0.12009117752313614, -0.30703580379486084, -0.0016724471934139729, 0.0456366129219532, 0.16992437839508057, 0.03740788623690605, 0.133291557431221, -0.028294866904616356, 0.09130720049142838, -0.39928174018859863, -0.004943672567605972, 0.045719973742961884, 0.2729244530200958, -0.0964985340833664, 0.10491808503866196, -0.1255224198102951, 0.019677385687828064, -0.07667692005634308, -0.19778281450271606, -0.04865211620926857, -0.06545382738113403, 0.09252617508172989, 0.0805368423461914, -0.008770518004894257, -0.0889941081404686, -0.09786638617515564, -0.12185110151767731, -0.1493820697069168, 0.14168567955493927, 0.009083029814064503, 0.07961339503526688, -0.20588859915733337, -0.12386258691549301, 0.024733997881412506, -0.21288235485553741, -0.0885707214474678, -0.05659763142466545, -0.016753816977143288, 0.07616037130355835, 0.0005763954250141978, -0.0964360311627388, -0.23327454924583435, 0.12789525091648102, 0.09182337671518326, 0.058341119438409805, -0.004835654981434345, -0.02674766443669796, 0.031004933640360832, 0.0173677746206522, 0.034041628241539, -0.06417255848646164, -0.07144293189048767, 0.04818094149231911, 0.26114147901535034, 0.17767080664634705, -0.02685144729912281, -0.38151705265045166, 0.17735695838928223, -0.0472540482878685, -0.27481862902641296, 0.06246426701545715, 0.03658118471503258, 0.11380024999380112, 0.20588365197181702, 0.23287008702754974, 0.04087236896157265, -0.08992141485214233, -0.25790977478027344, -0.32322657108306885, -0.18190090358257294, 0.0928049236536026, 0.03665947914123535, 0.1409408003091812, 0.050022874027490616, -0.13858847320079803, -0.12036021798849106, -0.09202296286821365, -0.12193827331066132, 0.040459468960762024, 0.026330580934882164, 0.13775509595870972, -0.1066073328256607, -0.05055319890379906, -0.004269519820809364, 0.08490663766860962, 0.12060979753732681, 0.08901076763868332, -0.24595415592193604, 0.09141746908426285, 0.03937392309308052, 0.06647185236215591, 0.31720229983329773, 0.003969557583332062, -0.17096655070781708, -0.1564074456691742, -0.31521034240722656, 0.20133692026138306, -0.11477240920066833, -0.10669006407260895, -0.09081503748893738, 0.046231623739004135, -0.07321519404649734, 0.03801807761192322, -0.020913420245051384, -0.0646485686302185, 0.0632532387971878, -0.07593395560979843, -0.0009784415597096086, -0.05211741849780083, 0.1870213747024536, -0.013258255086839199, 0.008385151624679565, 0.11978602409362793, 0.03796406462788582, 0.057661667466163635, -0.18001949787139893, 0.01820608228445053, 0.08928724378347397, 0.18764451146125793, -0.36041489243507385, -0.23117658495903015, -0.05673745274543762, -0.0168361347168684, -0.04704366251826286, 0.23506034910678864, -0.09785952419042587}, {-0.07644736021757126, -0.30742117762565613, 0.0369100421667099, -0.02544623799622059, -0.07308798283338547, 0.2439245879650116, -0.11362197995185852, 0.16735336184501648, -0.025463735684752464, -0.08584993332624435, -0.05604294687509537, -0.20970670878887177, 0.10358268767595291, 0.05831914022564888, -0.04997709393501282, 0.1018451452255249, -0.23076783120632172, 0.03477265313267708, 0.06115548312664032, 0.04951341822743416, -0.12476710230112076, -0.05940653011202812, 0.07646645605564117, -0.07321115583181381, 0.11141174286603928, 0.2020200788974762, 0.03698168694972992, -0.055371060967445374, -0.15751397609710693, -0.27660641074180603, 0.051015134900808334, -0.016025034710764885, -0.09661417454481125, 0.003936354536563158, -0.13816210627555847, -0.1108563095331192, 0.06758550554513931, -0.06689874827861786, 0.05875604227185249, -0.024223798885941505, -0.24603524804115295, -0.006258619017899036, 0.0036780252121388912, 0.12557607889175415, 0.06559090316295624, 0.06153823435306549, -0.0193962212651968, 0.05551129952073097, 0.09982573986053467, -0.03431324660778046, 0.027017327025532722, -0.10974036157131195, 0.017625698819756508, -0.14473295211791992, 0.02716168761253357, -0.18616311252117157, 0.010683078318834305, -0.13050511479377747, 0.15476401150226593, -0.1513775885105133, -0.07823879271745682, 0.277616024017334, 0.03614537790417671, 0.16373977065086365, -0.04252679646015167, -0.028608541935682297, -0.12548279762268066, 0.160883828997612, -0.13871516287326813, -0.07341884076595306, -0.14328914880752563, 0.0038408117834478617, 0.04226808249950409, 0.1397118717432022, -0.018087957054376602, -0.2087143361568451, 0.4095546305179596, 0.012013105675578117, 0.13130222260951996, 0.08911479264497757, -0.1370043307542801, 0.059373196214437485, 0.015777036547660828, 0.05014564469456673, -0.20024317502975464, 0.0736021026968956, -0.09123498201370239, -0.04250272735953331, 0.03056459315121174, -0.04036826640367508, 0.026306983083486557, 0.13703469932079315, 0.07697991281747818, 0.05672318860888481, 0.027961868792772293, -0.04297427088022232, 0.00026176180108450353, 0.04612642899155617, 0.04272579401731491, -0.26721709966659546, -0.037804245948791504, -0.11256402730941772, -0.11876804381608963, -0.08762849867343903, 0.10598451644182205, 0.2859637141227722, -0.015958013013005257, 0.14740531146526337, 0.13145430386066437, 0.04579378664493561, -0.06598090380430222, 0.1908666044473648, -0.08322427421808243, -0.12940946221351624, -0.03354582190513611, -0.02192593552172184, -0.11961083114147186, -0.054734017699956894, -0.08373446762561798, -0.14057809114456177, -0.08907939493656158, -0.11189533770084381, 0.014031928963959217, -0.11549732834100723, -0.1601952761411667, -0.10332344472408295, 0.19733040034770966, 0.03838444873690605, 0.06862261891365051, 0.10369710624217987, -0.03750171884894371, -0.11815252155065536, 0.08561248332262039, 0.0064307572320103645, -0.18984492123126984, 0.2674436867237091, 0.03284034878015518, -0.1354840099811554, 0.11818864941596985, -0.1140625923871994, 0.1641828417778015, -0.17986701428890228, 0.3204686641693115, -0.0618177130818367, 0.28540489077568054, 0.1988074779510498, -0.006392038892954588, -0.009326865896582603, 0.15286538004875183, -0.013970442116260529, 0.030968621373176575, 0.02010258659720421, -0.10297289490699768, -0.047964006662368774, 0.24825404584407806, -0.03355860337615013, 0.07205705344676971, -0.14002643525600433, -0.19312617182731628, -0.04613673686981201, -0.10791075974702835, -0.18394127488136292, -0.019760189577937126, -0.0015372911002486944, 0.008887575939297676, 0.053230736404657364, 0.04232022538781166, -0.14886356890201569, 0.04279768839478493, -0.01999014802277088, 0.04786591976881027, 0.020104020833969116, 0.015807906165719032, -0.06736665964126587, -0.02282833307981491, 0.15527251362800598, -0.11082074791193008, 0.032823346555233, 0.06645984947681427, -0.03578713908791542, 0.021464021876454353, -0.14983819425106049, 0.18174313008785248, 0.04344066232442856, -0.13410523533821106, -0.013216666877269745, 0.003027824219316244, 0.023855341598391533, 0.11730965971946716, -0.14570684731006622, 0.006526212207973003, -0.154985710978508, 0.17746074497699738, 0.04529253765940666, 0.03133770823478699, -0.022508373484015465, 0.13330493867397308, -0.17019601166248322, -0.011147283017635345, -0.013865050859749317, -0.23085971176624298, 0.00955503061413765, 0.08464166522026062, 0.06754831224679947, -0.13150279223918915, -0.01361035369336605, 0.46882128715515137, 0.19938144087791443, 0.13645155727863312, -0.14151304960250854, 0.0643947497010231, 0.09132437407970428, 0.11682283133268356, -0.10112819075584412, 0.12395498901605606, 0.2850641906261444, 0.09396514296531677, -0.01913725957274437, 0.058641467243433, -0.0012299234513193369, -0.07355362176895142, 0.09479822963476181, -0.07299990952014923, -0.031704407185316086, 0.08933800458908081, 0.15064500272274017, 0.24289050698280334, 0.03407726809382439, 0.1593908816576004, -0.05211764946579933, -0.20365861058235168, -0.009802404791116714, 0.19722899794578552, 0.06054304912686348, -0.020061984658241272, -0.12234775722026825, -0.01712394505739212, 0.044292036443948746, 0.23537495732307434, -0.007008476182818413, 0.015474557876586914, 0.08781595528125763, 0.1300983875989914, -0.07933579385280609, 0.31379348039627075, 0.07301206886768341, 0.059381239116191864, 0.07325776666402817, -0.01555374264717102, -0.0560271181166172, -0.15011143684387207, -0.05664591118693352, 0.2120099514722824, 0.08663392812013626, 0.12098832428455353, -0.09836656600236893}, {0.1534963697195053, 0.1223779171705246, 0.11507297307252884, 0.12492302805185318, -0.03903505578637123, 0.02380240708589554, 0.2747511863708496, -0.14275558292865753, 0.04951616749167442, -0.07002568244934082, 0.23214372992515564, 0.16645492613315582, 0.12678855657577515, 0.24498263001441956, 0.06834066659212112, 0.09232675284147263, 0.014751425944268703, 0.014838702976703644, 0.037336722016334534, 0.06962738186120987, -0.06642147898674011, 0.03639962896704674, 0.008300873450934887, -0.11235672980546951, -0.05447041988372803, 0.008023879490792751, 0.12376154214143753, -0.16715066134929657, -0.011379636824131012, -0.12340047955513, -0.031195206567645073, 0.025648824870586395, -0.01287816185504198, -0.12515494227409363, 0.09980720281600952, -0.09751732647418976, 0.063148632645607, 0.16741031408309937, -0.11187530308961868, -0.07440820336341858, 0.018543191254138947, -0.04953017458319664, -0.03908929228782654, 0.21702827513217926, 0.1007300540804863, 0.16245287656784058, -0.23034432530403137, -0.0282343327999115, 0.13379982113838196, -0.10734254121780396, 0.05256785824894905, 0.16582567989826202, -0.28709548711776733, 0.05377599596977234, 0.08104356378316879, -0.06229696422815323, -0.1499856412410736, -0.3775794804096222, -0.16104960441589355, 0.08082647621631622, -0.2138388454914093, -0.11407285928726196, -0.14418183267116547, -0.009253174066543579, 0.12259222567081451, -0.22071516513824463, -0.10472743213176727, -0.11293385922908783, -0.09937701374292374, 0.169158473610878, -0.03506557270884514, 0.33907803893089294, -0.12031522393226624, 0.08246206492185593, 0.004223796539008617, 0.22556406259536743, -0.12893863022327423, -0.014429669827222824, 0.3296282887458801, 0.1532273292541504, 0.15778838098049164, 0.161057710647583, 0.09254268556833267, 0.11547356098890305, 0.08145053684711456, -0.1526201367378235, -0.13650454580783844, 0.16410505771636963, -0.04251839220523834, 0.32884493470191956, 0.07121070474386215, -0.04174567013978958, 0.11972997337579727, 0.12397275120019913, 0.12672032415866852, 0.026312340050935745, 0.2534114122390747, 0.23053881525993347, 0.21029341220855713, 0.12027540802955627, -0.05031578242778778, 0.0022086373064666986, 0.21619100868701935, 0.01159563846886158, -0.08373551070690155, 0.16739961504936218, -0.03028387576341629, 0.16621175408363342, -0.03938296064734459, -0.04178188368678093, 0.0477883405983448, -0.03678380697965622, -0.10750038176774979, -0.04715164378285408, 0.24555912613868713, 0.020275680348277092, 0.16874557733535767, 0.15707771480083466, -0.11631199717521667, 0.002630742499604821, 0.08470404893159866, -0.122743621468544, -0.056700121611356735, 0.10158850997686386, -0.3271421492099762, 0.14486394822597504, -0.008217324502766132, 0.03221355751156807, -0.33186355233192444, 0.06343086808919907, 0.3991798758506775, 0.1459582895040512, -0.1031002625823021, -0.00819169357419014, -0.30361276865005493, -0.008836004883050919, -0.08377958834171295, 0.12928958237171173, 0.10409163683652878, 0.1098598763346672, -0.026028461754322052, -0.04599500447511673, -0.10880468040704727, -0.06500744819641113, 0.11088877171278, -0.051852986216545105, -0.06615719944238663, 0.12929178774356842, 0.10670312494039536, -0.10645771026611328, -0.34824690222740173, 0.014110934920608997, -0.11731332540512085, -0.1568361073732376, -0.08046044409275055, -0.05831186845898628, 0.2155546247959137, -0.05857568979263306, -0.017754657194018364, 0.012752365320920944, -0.023883046582341194, 0.01283591240644455, -0.047574374824762344, 0.1134670302271843, -0.13085870444774628, 0.09747075289487839, -0.051246292889118195, -0.03044469654560089, -0.07301178574562073, 0.09723491221666336, 0.12523429095745087, 0.012574635446071625, 0.10752510279417038, -0.04455622658133507, 0.02487143874168396, -0.30867162346839905, 0.04210377857089043, 0.03148281201720238, 0.16589608788490295, -0.08656533062458038, -0.05964822322130203, 0.21496246755123138, 0.07040417939424515, 0.06837277114391327, 0.1373366117477417, -0.039121199399232864, 0.34306827187538147, -0.014223264530301094, 0.14222979545593262, 0.04113394021987915, 0.08268208056688309, -0.02232385240495205, 0.05201920121908188, 0.05585651472210884, 0.006376889534294605, 0.08926499634981155, -0.13303400576114655, 0.011123206466436386, -0.11762543767690659, 0.08930952101945877, -0.13998162746429443, -0.05894698202610016, 0.04731083661317825, -0.14819495379924774, 0.24575923383235931, 0.1775135099887848, 0.008951332420110703, 0.34133976697921753, 0.030160849913954735, -0.197414368391037, 0.08784717321395874, -0.0044072517193853855, -0.021305736154317856, 0.19275718927383423, 0.1844932585954666, 0.0055489265359938145, 0.1306852251291275, 0.11116761714220047, 0.2131427377462387, 0.0838412493467331, -0.26032283902168274, -0.04829537123441696, -0.05675262585282326, -0.062234487384557724, -0.1506444364786148, 0.03296337276697159, 0.06809091567993164, -0.06104631721973419, -0.08470702171325684, -0.01876627467572689, -0.2642187774181366, -0.11346553266048431, -0.2511011064052582, 0.06537578999996185, -0.11973782628774643, 0.17156969010829926, 0.10781662166118622, -0.014255429618060589, 0.01649801805615425, -0.07536587864160538, -0.04288209602236748, -0.18915730714797974, 0.022932620719075203, 0.14478981494903564, 0.06583890318870544, -0.16268648207187653, -0.08381928503513336, 0.04531874880194664, -0.003307715756818652, -0.15029489994049072, -0.05788073316216469, -0.12294900417327881, 0.04485752433538437, -0.023385722190141678, 0.2466500848531723, -0.11278144270181656}, {-0.06850938498973846, -0.13984012603759766, -0.07691509276628494, -0.14520996809005737, 0.13802459836006165, -0.12941157817840576, -0.09974058717489243, -0.07621823996305466, 0.038474470376968384, 0.01789523847401142, 0.06572159379720688, 0.0560973659157753, -0.06628730893135071, -0.0230580884963274, -0.15047726035118103, 0.024673044681549072, 0.019981175661087036, -0.05944633483886719, -0.305739164352417, 0.04649842157959938, -0.2202930748462677, 0.13024407625198364, -0.028707390651106834, 0.09071183949708939, -0.0035105631686747074, 0.08586052805185318, -0.2236696183681488, 0.2955762445926666, 0.18864037096500397, -0.03604671731591225, 0.12830190360546112, -0.14412106573581696, 0.0811135396361351, 0.18412762880325317, 0.04349321871995926, -0.08235715329647064, 0.02493112161755562, -0.24708561599254608, -0.043533388525247574, -0.01641552895307541, 0.1064869686961174, 0.04807160794734955, -0.044436633586883545, -0.09649481624364853, -0.10549764335155487, -0.12375378608703613, 0.13858076930046082, -0.014641992747783661, -0.04563130438327789, 0.017903577536344528, 0.16542449593544006, -0.09693897515535355, 0.042457256466150284, 0.19232803583145142, -0.002457865746691823, -0.04225565493106842, 0.10913017392158508, 0.03681362420320511, -0.07164895534515381, -0.11001648008823395, 0.04880281165242195, -0.2040705680847168, -0.030310215428471565, 0.08963187038898468, 0.07072170078754425, 0.08824831992387772, 0.0033803796395659447, -0.11703203618526459, -0.052773699164390564, -0.0879187062382698, -0.14111928641796112, 0.13079579174518585, -0.17188195884227753, 0.14949746429920197, 0.1017918735742569, 0.14141497015953064, 0.11826823651790619, 0.0205631572753191, 0.09630628675222397, -0.024427462369203568, 0.319846510887146, 0.0561988465487957, -0.005538443569093943, -0.04223184660077095, -0.03652813285589218, 0.05844490975141525, 0.1872260570526123, 0.06710362434387207, -0.05002681165933609, 0.0009857133263722062, 0.08038800209760666, 0.0353887677192688, -0.26467275619506836, -0.05456084385514259, 0.006495704408735037, 0.05436475947499275, -0.10229268670082092, 0.04723567143082619, 0.029095323756337166, -0.1075836792588234, -0.12603195011615753, -0.10836179554462433, -0.0059387595392763615, -0.02803758531808853, 0.110943503677845, 0.36403071880340576, -0.03957153111696243, -0.15488457679748535, -0.022323209792375565, 0.09735336899757385, -0.03849159926176071, 0.06455571949481964, -0.17337553203105927, 0.1312771737575531, 0.12355176359415054, 0.1293855905532837, 0.006752958986908197, -0.11164100468158722, -0.01703607849776745, -0.0011331704445183277, -0.09395553916692734, -0.08854380249977112, 0.19531165063381195, 0.05485088750720024, -0.013126390986144543, -0.050781603902578354, 0.04842560738325119, 0.11481255292892456, -0.11687979847192764, -0.03042367473244667, 0.15153972804546356, 0.04917223006486893, 0.23693354427814484, -0.0713571161031723, 0.01916249841451645, 0.20318526029586792, -0.0708533227443695, 0.02031596750020981, 0.14155612885951996, -0.20101340115070343, 0.09223361313343048, 0.011072935536503792, 0.06205098703503609, 0.1028282642364502, 0.11575420945882797, -0.002429935382679105, -0.10848124325275421, -0.09795138984918594, 0.07928170263767242, -0.08055949211120605, -0.16477160155773163, -0.03181501477956772, -0.006381242070347071, -0.11169155687093735, -0.029102088883519173, 0.04196834936738014, -0.10092109441757202, -0.03787427023053169, 0.11262816190719604, 0.013768348842859268, -0.13652107119560242, -0.0869937315583229, -0.010328572243452072, -0.052837878465652466, 0.005179050378501415, 0.09313727915287018, -0.3881360590457916, -0.10512584447860718, 0.025929337367415428, 0.000172514482983388, 0.12953884899616241, 0.17396163940429688, -0.13010913133621216, -0.10409576445817947, 0.007751081138849258, 0.08542456477880478, 0.12276115268468857, -0.16226761043071747, -0.25109565258026123, 0.12840737402439117, -0.09147815406322479, -0.3063267767429352, 0.15553197264671326, 0.32259121537208557, 0.06617235392332077, -0.08143894374370575, -0.41543856263160706, -0.16785143315792084, 0.056111522018909454, -0.014667320996522903, 0.07699330896139145, -0.08175288140773773, 0.21887212991714478, -0.0902111828327179, -0.1490766853094101, -0.13554514944553375, -0.13098858296871185, -0.09860427677631378, -0.12775644659996033, -0.07647325098514557, 0.06611792743206024, -0.13714367151260376, -0.07989984005689621, 0.19050897657871246, 0.0623931810259819, -0.11140824854373932, 0.14134624600410461, 0.12281618267297745, 0.16656121611595154, -0.13213004171848297, -0.09863299131393433, -0.2962782382965088, -0.12718471884727478, -0.09725473076105118, -0.0699307769536972, -0.14098723232746124, 0.15400296449661255, 0.003822699189186096, 0.15327037870883942, -0.10019538551568985, -0.10170696675777435, 0.06099805608391762, -0.11876558512449265, -0.46614354848861694, -0.04435689374804497, -0.4627017378807068, -0.09683046489953995, 0.025496017187833786, -0.14324668049812317, 0.0542428158223629, 0.07435759156942368, -0.10227847844362259, -0.1591222584247589, -0.012073361314833164, 0.11406456679105759, -0.2574922442436218, 0.028298962861299515, 0.012330781668424606, -0.1912425458431244, 0.18933530151844025, 0.04240858182311058, 0.08624632656574249, -0.16924144327640533, -0.03897378221154213, -0.016585158184170723, -0.0036840436514467, 0.0001588237937539816, -0.0014570519560948014, -0.10125064104795456, -0.13249802589416504, 0.043620236217975616, -0.1898815780878067, -0.24247077107429504, -0.008037826046347618, -0.10862118750810623, -0.0719284638762474}, {0.01378457061946392, 0.0998063012957573, -0.07051083445549011, -0.08128854632377625, -0.09293264150619507, 0.021913176402449608, 0.08168435096740723, 0.025363123044371605, -0.03491967171430588, 0.017046945169568062, 0.04047730937600136, -0.039188675582408905, -0.1140502318739891, -0.032732583582401276, -0.07196467369794846, 0.19857613742351532, -0.18010766804218292, -0.030188528820872307, 0.16752159595489502, 0.01727130077779293, -0.49600163102149963, 0.044176723808050156, -0.01366572454571724, 0.20779532194137573, 0.01744658127427101, -0.07109344750642776, 0.013087051920592785, 0.3138667643070221, 0.15595848858356476, -0.017341062426567078, -0.09885881096124649, 0.011288478970527649, 0.1509154736995697, -0.19282664358615875, -0.0643782988190651, 0.0057100895792245865, -0.10506075620651245, 0.34902435541152954, 0.06450658291578293, 0.021097969263792038, 0.2669996917247772, 0.0004913583979941905, -0.20394682884216309, -0.1452825963497162, -0.059405338019132614, -0.04534349590539932, -0.05585271865129471, 0.041791342198848724, -0.038375966250896454, -0.03069181740283966, -0.2987799644470215, -0.06426942348480225, -0.04625052958726883, 0.021109145134687424, 0.019065458327531815, 0.014753730967640877, -0.0243147574365139, -0.057068340480327606, 0.08462953567504883, -0.08500071614980698, 0.0725809708237648, 0.03456072881817818, 0.08380798250436783, 0.04697369784116745, 0.036504074931144714, 0.07959800213575363, 0.16676224768161774, 0.02965637668967247, 0.008629306219518185, -0.10345946252346039, 0.10257546603679657, -0.11561773717403412, -0.0025539835914969444, -0.04280262812972069, 0.0694156214594841, -0.04772372171282768, -0.12378457933664322, -0.06176460161805153, -0.14131051301956177, -0.04639556258916855, -0.09220937639474869, -0.0675596296787262, 0.0738467201590538, -0.22632846236228943, 0.23231512308120728, -0.09691557288169861, -0.01834840700030327, -0.10369103401899338, -0.07853589951992035, 0.16966857016086578, -0.06651583313941956, -0.15246054530143738, -0.04195059835910797, 0.15638624131679535, -0.1441330909729004, 0.023794565349817276, -0.1084766536951065, 0.02664010412991047, 0.14330296218395233, 0.03913076967000961, -0.14004342257976532, 0.02875247225165367, -0.10755539685487747, 0.00979665108025074, 0.0722164586186409, -0.005859249271452427, -0.03625289723277092, -0.007570820860564709, 0.17254674434661865, -0.026356475427746773, 0.17053060233592987, 0.13124585151672363, -0.04989416152238846, 0.06415260583162308, 0.06378988176584244, -0.03258698433637619, -0.19860078394412994, -0.24341845512390137, -0.14828358590602875, -0.0830628052353859, -0.07053202390670776, 0.13142046332359314, -0.06009722501039505, -0.03188527002930641, 0.03304154425859451, -0.03910258412361145, 0.13177984952926636, -0.10303663462400436, -0.02821030281484127, -0.04632848501205444, -0.12251849472522736, 0.06482776999473572, 0.1068212017416954, 0.08092708140611649, 0.05941012501716614, -0.12533752620220184, -0.037291090935468674, 0.056393664330244064, -0.0861612856388092, -0.15535302460193634, -0.22620345652103424, 0.014565003104507923, 0.09522873163223267, -0.09642861038446426, -0.06643616408109665, 0.0763130709528923, 0.03429703786969185, -0.046030640602111816, 0.11956171691417694, 0.05163153260946274, 0.12352050095796585, -0.04237253591418266, -0.03759277984499931, 0.13102702796459198, -0.08543437719345093, -0.14065910875797272, -0.2478608787059784, 0.30713924765586853, 0.17981815338134766, 0.22228896617889404, -0.08452189713716507, 0.15339723229408264, 0.022501884028315544, -0.1456025242805481, -0.05412908270955086, -0.07769975066184998, 0.15429769456386566, -0.030049819499254227, 0.18207520246505737, -0.09112567454576492, 0.2438007891178131, 0.10870499908924103, -0.04012510925531387, -0.018818572163581848, -0.1631111055612564, -0.09577471017837524, -0.1504446566104889, -0.10645190626382828, -0.11046002805233002, 0.0005399996298365295, 0.021125540137290955, 0.13935962319374084, -0.15889549255371094, -0.09108901768922806, -0.12982194125652313, 0.012168945744633675, 0.2515161633491516, -0.008990321308374405, 0.08590058982372284, 0.017546964809298515, -0.08410072326660156, 0.2475096583366394, -0.12860313057899475, -0.07533178478479385, -0.10755069553852081, -0.12443750351667404, 0.03173041343688965, 0.21699891984462738, 0.047125209122896194, 0.13072705268859863, -0.046125903725624084, -0.17931711673736572, 0.06091291829943657, 0.03328000381588936, -0.059838518500328064, -0.13754987716674805, -0.07495909929275513, 0.021618321537971497, 0.02086167223751545, -0.05033266171813011, -0.03511779010295868, -0.0032473003957420588, 0.1585511863231659, -0.03743111714720726, 0.0065365685150027275, -0.11219263076782227, 0.004996073432266712, 0.18721364438533783, -0.16899363696575165, 0.10330376774072647, 0.08093728125095367, -0.0814938098192215, 0.018137291073799133, -0.035830818116664886, 0.11296702921390533, 0.09812559187412262, 0.0087829465046525, 0.1081058457493782, 0.04777754098176956, -0.03190440312027931, 0.29266121983528137, -0.027998382225632668, 0.056483194231987, 0.006838332861661911, 0.11605154722929001, -0.036216240376234055, -0.01833907701075077, 0.007525761611759663, -0.032078102231025696, 0.06190555542707443, -0.14051704108715057, -0.053049929440021515, -0.4944918155670166, -0.05547454208135605, -0.07440850883722305, 0.16588447988033295, 0.038459401577711105, -0.10575306415557861, 0.12842333316802979, -0.03959028795361519, -0.05991339311003685, 0.0800003781914711, -0.20928151905536652, -0.030704788863658905, -0.08990255743265152, 0.11867327243089676}, {-0.04752993583679199, -0.17060966789722443, 0.021205049008131027, -0.12353570759296417, -0.09733105450868607, -0.02274068258702755, 0.14958061277866364, -0.0044536166824400425, 0.21466459333896637, 0.08071556687355042, 0.15452854335308075, 0.0640905573964119, 0.008889044634997845, -0.17983193695545197, -0.1027832105755806, -0.02314853109419346, -0.10321210324764252, -0.10550925880670547, 0.1173987165093422, 0.12854604423046112, -0.11951019614934921, -0.12889355421066284, -0.1874115765094757, 0.1702374368906021, 0.27718326449394226, 0.06407460570335388, -0.08553744107484818, -0.12144012004137039, -0.18814343214035034, -0.05153682827949524, 0.09602771699428558, 0.13314217329025269, 0.04548335820436478, 0.08280514925718307, 0.08537760376930237, 0.08725148439407349, -0.2862638831138611, 0.30745217204093933, 0.044876858592033386, 0.0743715837597847, -0.21609905362129211, -0.03192239627242088, 0.10464705526828766, -0.12029006332159042, -0.18759450316429138, 0.03666778281331062, -0.049875274300575256, 0.006182103883475065, -0.15704886615276337, -0.0625453069806099, -0.06287188082933426, -0.10383718460798264, -0.08550485223531723, 0.07231880724430084, -0.030279116705060005, 0.07188879698514938, -0.32181838154792786, -0.3051997721195221, -0.22720980644226074, 0.3222351670265198, -0.22224919497966766, -0.05982859432697296, 0.25209590792655945, 0.28733813762664795, -0.16890442371368408, 0.06688249856233597, 0.24212488532066345, 0.010413204319775105, -0.23005975782871246, -0.034160975366830826, -0.019979218021035194, -0.1000465527176857, 0.1374533325433731, 0.0007796311401762068, 0.03397958725690842, -0.009227629750967026, 0.02913864701986313, -0.10531512647867203, 0.1910633146762848, -0.15028254687786102, 0.13939692080020905, -0.051349107176065445, -0.08408864587545395, -0.01836838386952877, -0.04297536239027977, -0.12567736208438873, -0.07272415608167648, 0.015249799937009811, -0.12103002518415451, -0.15062372386455536, -0.18118581175804138, -0.03850448876619339, -0.06967341154813766, 0.041745658963918686, 0.07746636122465134, 0.041774600744247437, 0.09624879062175751, 0.04055728390812874, -0.1144758090376854, -0.1423078328371048, 0.006071825511753559, 0.02269928529858589, -0.3348464369773865, -0.1170990914106369, 0.1060437560081482, -0.49170005321502686, 0.26340770721435547, -0.043313924223184586, -0.05443694069981575, 0.07150042057037354, -0.1605367660522461, 0.07365000993013382, -0.14186693727970123, 0.09551652520895004, 0.17034709453582764, 0.0005177448620088398, -0.17812705039978027, 0.03326418995857239, -0.3481038808822632, -0.041051093488931656, 0.02082938142120838, 0.1424509882926941, -0.11442624032497406, -0.20350870490074158, -0.278373658657074, -0.20457389950752258, -0.04854310676455498, -0.04125825688242912, -0.1491028368473053, -0.10947901755571365, -0.039002418518066406, -0.058497898280620575, -0.240990549325943, 0.008864015340805054, 0.05268678441643715, 0.09753584861755371, 0.024958966299891472, -0.08083073049783707, 0.0782657042145729, 0.1344231218099594, -0.14935530722141266, 0.0051342155784368515, -0.08204188197851181, 0.12079408764839172, -0.10188561677932739, -0.22934502363204956, -0.1661534458398819, -0.051618073135614395, 0.2205348014831543, 0.22653067111968994, -0.1191675141453743, -0.03359777107834816, -0.15147905051708221, 0.2077217698097229, -0.10117164999246597, -0.0013276595855131745, 0.20448410511016846, -0.20843257009983063, 0.1040656715631485, 0.316457062959671, 0.21407857537269592, -0.0711396187543869, -0.08659080415964127, -0.012162547558546066, -0.07424045354127884, 0.04836343228816986, 0.6524518728256226, 0.20803053677082062, 0.2575167715549469, -0.011992933228611946, -0.1340675801038742, -0.015174774453043938, -0.155356302857399, 0.025452306494116783, 0.1901572048664093, 0.11224641650915146, -0.16575747728347778, 0.05283099785447121, -0.006350028328597546, -0.21974335610866547, -0.03791792318224907, 0.24197475612163544, -0.1832823008298874, -0.3724372684955597, -0.06684770435094833, 0.1470213234424591, 0.5091026425361633, 0.05168730020523071, -0.1792875975370407, 0.2927376329898834, -0.16004988551139832, 0.14155957102775574, -0.05348631367087364, -0.04231568053364754, 0.11950341612100601, 0.12567952275276184, -0.2672405540943146, 0.06739992648363113, 0.2926769256591797, -0.1401389092206955, -0.2981438636779785, 0.0030534302350133657, -0.22874625027179718, -0.13600248098373413, -0.27216652035713196, 0.28751689195632935, -0.2605192959308624, -0.10562736541032791, 0.09940022230148315, -0.16084407269954681, -0.031144116073846817, 0.1879391223192215, -0.07907279580831528, 0.42599016427993774, 0.11352190375328064, -0.06095479428768158, 0.034797631204128265, 0.24731691181659698, 0.1887848973274231, -0.09607715904712677, 0.007982180453836918, -0.03925380855798721, -0.0661243349313736, 0.26517531275749207, 0.3165512979030609, 0.2880879044532776, 0.1156129539012909, 0.0817108228802681, -0.05592364817857742, -0.04596944898366928, -0.15632574260234833, -0.19101053476333618, 0.1533852070569992, 0.13936775922775269, -0.0010169136803597212, -0.024913761764764786, 0.028052721172571182, -0.266260027885437, 0.2235138863325119, -0.0896514505147934, 0.0078058261424303055, 0.13152648508548737, -0.002960889833047986, 0.3945213258266449, -0.012117444537580013, 0.016384346410632133, -0.04005087912082672, -0.10845132172107697, 0.02267487533390522, 0.11488219350576401, -0.10070982575416565, -0.2171056717634201, -0.10440383851528168, 0.15804965794086456, 0.22362101078033447, 0.16390149295330048}, {-0.007923394441604614, -0.07949710637331009, -7.60192924644798e-05, 0.0763910785317421, -0.08571573346853256, -0.2739497721195221, -0.01811607927083969, 0.23849982023239136, -0.09233954548835754, -0.2690691649913788, 0.1521000862121582, 0.008450512774288654, -0.2748718559741974, -0.043551232665777206, -0.1283734142780304, 0.1872897893190384, -0.06246654689311981, -0.06594867259263992, -0.09935838729143143, -0.058592088520526886, -0.2327439934015274, 0.01665838435292244, -0.14578141272068024, -0.015449224971234798, 0.018887488171458244, 0.2239384949207306, 0.15947510302066803, 0.18442986905574799, 0.19154828786849976, 0.0035264133475720882, -0.011734872125089169, -0.04486740007996559, 0.03890251740813255, 0.1298314332962036, -0.07938995212316513, 0.018949372693896294, -0.09365719556808472, 0.038562286645174026, 0.10156325995922089, -0.01981539838016033, 0.20925511419773102, -0.2186267226934433, -0.09508970379829407, -0.0718817189335823, -0.07900667190551758, -0.0331432931125164, 0.06617271155118942, 0.02715587429702282, 0.18107803165912628, 0.0774989128112793, -0.2334153652191162, -0.11866569519042969, 0.0100215720012784, 0.028507422655820847, 0.049732379615306854, 0.24405644834041595, -0.10811999440193176, 0.03703996166586876, -0.09331594407558441, -0.2053191065788269, -0.21945661306381226, 0.02047594264149666, -0.2645345628261566, 0.26840895414352417, 0.010393669828772545, -0.18291346728801727, 0.03936077281832695, -0.12404118478298187, -0.07323292642831802, -0.04887780174612999, 0.1699160635471344, 0.003540125209838152, -0.010021760128438473, -0.11758436262607574, -0.08707647770643234, 0.12749654054641724, 0.12307041883468628, -0.07404930144548416, -0.05154239013791084, -0.030185364186763763, 0.014087035320699215, 0.15737766027450562, -0.0355706587433815, 0.04578843712806702, 0.022750593721866608, -0.014470763504505157, -0.13499093055725098, -0.11320409178733826, 0.05623467639088631, 0.013966876082122326, 0.11156988888978958, -0.35694432258605957, -0.04953313246369362, 0.06983863562345505, 0.17789967358112335, -0.09015493839979172, 0.03042728826403618, 0.18711340427398682, 0.06585215032100677, -0.015112270601093769, -0.0696801170706749, -0.08112052083015442, -0.01717892475426197, 0.07119765132665634, 0.0632873997092247, 0.168816477060318, 0.13068999350070953, -0.046679433435201645, 0.3221788704395294, 0.17499910295009613, -0.09517279267311096, 0.2454533874988556, -0.04022709280252457, 0.3519216477870941, -0.07582450658082962, -0.07157982140779495, -0.15968279540538788, 0.16304375231266022, -0.03728412464261055, 0.1294662356376648, 0.05420566350221634, 0.11294419318437576, 0.0250144861638546, -0.12483996152877808, 0.09016864001750946, -0.11501341313123703, -0.028771359473466873, 0.07860058546066284, -0.20298516750335693, -0.029073670506477356, -0.08071226626634598, -0.12834157049655914, -0.10822552442550659, 0.14367292821407318, 0.23171652853488922, 0.02077355794608593, -0.057332027703523636, -0.028754431754350662, -0.02566472627222538, 0.061497077345848083, 0.042910151183605194, 0.17570894956588745, -0.05268494039773941, -0.05249427631497383, -0.028739238157868385, 0.10265957564115524, 0.018388541415333748, -0.10115895420312881, 0.009028967469930649, 0.1355964094400406, 0.01268446072936058, -0.03250658139586449, -0.14949992299079895, 0.08775734156370163, 0.030793102458119392, -0.03365050628781319, -0.22296851873397827, 0.3023722469806671, 0.15466737747192383, -0.1429806649684906, -0.0024913279339671135, 0.023801611736416817, -0.011190230026841164, -0.012869341298937798, -0.05141923204064369, -0.22088536620140076, -0.13144634664058685, -0.062050022184848785, -0.018277067691087723, 0.2088630050420761, -0.03459714725613594, 0.11422032862901688, -0.022168230265378952, -0.14657771587371826, 0.11575019359588623, -0.018145238980650902, 0.18690377473831177, -0.0743379071354866, 0.06533323228359222, -0.05604712292551994, -0.007102294825017452, -0.14830425381660461, -0.07192444056272507, -0.25832223892211914, -0.023309476673603058, -0.05421004816889763, 0.08831062912940979, -0.09811456501483917, -0.05861029773950577, -0.14642548561096191, 0.041163936257362366, -0.041733548045158386, -0.05105741694569588, -0.07640277594327927, -0.12374482303857803, 0.24344809353351593, -0.11679628491401672, 0.13029129803180695, 0.06317505240440369, 0.07301962375640869, 0.06565474718809128, 0.05006307363510132, -0.052263349294662476, 0.04383515939116478, 0.08451355993747711, 0.09699737280607224, -0.03223266452550888, -0.08437830209732056, 0.006893981248140335, -0.014189899899065495, 0.1748863160610199, -0.019312748685479164, -0.04157945141196251, -0.17215070128440857, 0.1486821323633194, 0.0010160935344174504, -0.016339579597115517, -0.012785164639353752, -0.16318455338478088, -0.12283459305763245, -0.026532426476478577, 0.04279538244009018, -0.03098033368587494, 0.24104298651218414, 0.031158950179815292, 0.2264857441186905, -0.18128159642219543, 0.1293470710515976, -0.03491208702325821, 0.10595274716615677, 0.20677705109119415, -0.22157134115695953, 0.0951162576675415, -0.04969988390803337, 0.017104696482419968, -0.10533801466226578, -0.13130342960357666, 0.06851685047149658, -0.07037521153688431, -0.10138791054487228, 0.06977687776088715, -0.19135792553424835, -0.117647185921669, 0.06809449195861816, -0.07090356200933456, -0.09850937128067017, 0.16924293339252472, -0.0022748089395463467, 0.08269570022821426, 0.08622611314058304, 0.09007096290588379, 0.01987510733306408, -0.2278955727815628, -0.010161297395825386, -0.07131931930780411, 0.16884306073188782}, {-0.0180355217307806, 0.24644352495670319, -0.09999387711286545, -0.08786974847316742, -0.08296286314725876, -0.06155247613787651, 0.028811559081077576, 0.1375410556793213, -0.1342470347881317, 0.026901014149188995, -0.10684750229120255, -0.14020249247550964, -0.05674558877944946, 0.09038593620061874, 0.08933834731578827, 0.006383662112057209, -0.01708485558629036, 0.03478299826383591, -0.028283994644880295, 0.16310839354991913, -0.16280193626880646, -0.15831305086612701, 0.039313945919275284, 0.22075694799423218, -0.16823691129684448, 0.31369057297706604, 0.0007150454330258071, 0.18590936064720154, 0.13224807381629944, -0.09941351413726807, -0.15728193521499634, -0.19528239965438843, 0.0941065102815628, 0.10570360720157623, 0.17200934886932373, 0.14732907712459564, -0.23944759368896484, 0.1966603547334671, 0.1513124704360962, 0.02376856841146946, -0.04770609363913536, -0.13950757682323456, -0.13614019751548767, -0.04952693358063698, 0.05460578203201294, -0.11692383885383606, -0.05493518337607384, -0.26283150911331177, 0.00880532804876566, 0.038364410400390625, -0.04286821186542511, 0.2006811648607254, -0.18334226310253143, 0.24126213788986206, 0.03414585068821907, 0.0781366229057312, 0.012405981309711933, -0.08638116717338562, -0.008631878532469273, -0.004540467169135809, -0.059135567396879196, -0.078517384827137, 0.08819978684186935, 0.14632335305213928, 0.028182610869407654, -0.0562833771109581, -0.03938767686486244, 0.14726465940475464, 0.38498353958129883, -0.014329446479678154, 0.016250234097242355, -0.05236231908202171, -0.22333237528800964, -0.25131115317344666, 0.16610166430473328, 0.11898395419120789, -0.0575086772441864, 0.050372228026390076, 0.0012112183030694723, -0.17073357105255127, -0.1196361854672432, -0.027967069298028946, 0.0361475832760334, -0.2169119417667389, 0.03487834334373474, -0.11465322971343994, -0.08861511945724487, -0.11288996040821075, 0.020846951752901077, 0.12035750597715378, -0.0025754005182534456, 0.07324580103158951, -0.18823476135730743, 0.05956953763961792, 0.04275555536150932, 0.08911939710378647, -0.030673302710056305, 0.11447856575250626, 0.26513412594795227, -0.02419136092066765, -0.13029845058918, 0.1046501025557518, -0.01845504902303219, -0.0841766968369484, 0.018357476219534874, 0.5594601631164551, 0.04070890322327614, 0.0779888778924942, 0.04587492346763611, -0.08219055831432343, -0.07094205170869827, 0.07944584637880325, -0.08793597668409348, 0.05757775902748108, -0.023530438542366028, 0.2036324292421341, 0.11332905292510986, 0.08373895287513733, -0.1354905217885971, -0.08338912576436996, -0.006285922136157751, 0.2639543116092682, -0.021406082436442375, 0.06111243739724159, -0.04328928887844086, -0.056254055351018906, 0.11049848049879074, -0.2698573172092438, 0.049880556762218475, 0.14183199405670166, 0.03238620609045029, 0.0016969084972515702, -0.039212021976709366, 0.17255519330501556, 0.21999672055244446, -0.18752168118953705, -0.020213324576616287, 0.01741156168282032, -0.06934400647878647, -0.13965526223182678, -0.055941611528396606, 0.03872060775756836, -0.2107282131910324, -0.22379948198795319, -0.1261930614709854, 0.02723015286028385, 0.1406220942735672, -0.06844016164541245, -0.11235254257917404, 0.09254487603902817, -0.1522766649723053, -0.1359492987394333, 0.20376905798912048, 0.01584615185856819, 0.15546150505542755, -0.1308748722076416, -0.10958003997802734, 0.46634039282798767, 0.2649182975292206, 0.1513165831565857, -0.1870814561843872, 0.10607565939426422, 0.20250988006591797, -0.09099875390529633, 0.09855861961841583, -0.2693183422088623, -0.02861376851797104, 0.060636479407548904, 0.06903674453496933, -0.02208193577826023, 0.10247596353292465, 0.030982108786702156, -0.029837625101208687, 0.15139633417129517, 0.05094939470291138, 0.273569792509079, 0.025064153596758842, -0.00955111812800169, 0.016401156783103943, 0.04512268677353859, 0.14508621394634247, 0.07127833366394043, 0.07401610910892487, -0.11914078891277313, 0.23405417799949646, 0.13958469033241272, -0.048791807144880295, -0.04865960031747818, 0.21909134089946747, 0.11979810148477554, 0.02367027848958969, 0.22526201605796814, -0.3074192702770233, -0.22865723073482513, 0.021441930904984474, -0.1044330894947052, -0.1376032680273056, 0.3947235941886902, -0.18336260318756104, 0.08062765002250671, -0.14221277832984924, -0.049704719334840775, -0.045679721981287, -0.101849764585495, 0.07229483127593994, 0.0657147467136383, 0.18589849770069122, -0.3619319200515747, -0.048966918140649796, 0.047584909945726395, 0.07476120442152023, 0.24648383259773254, -0.28304389119148254, 0.22680017352104187, -0.08826345205307007, -0.15884986519813538, -0.11581476032733917, -0.11441554129123688, -0.13627482950687408, 0.03367886692285538, 0.29484984278678894, 0.17574714124202728, -0.34077805280685425, 0.07533460110425949, 0.11310993134975433, 0.08850943297147751, -0.029535692185163498, 0.08640541136264801, -0.22504891455173492, -0.2906647324562073, 0.2604342997074127, 0.1200704276561737, 0.17425350844860077, 0.12039469182491302, 0.12077992409467697, 0.15567104518413544, 0.1200968325138092, -0.15664798021316528, -0.13197734951972961, 0.34115031361579895, -0.06476336717605591, -0.06874001771211624, -0.47267621755599976, -0.07609561830759048, 0.013117414899170399, 0.05664224550127983, -0.11036550998687744, -0.033107273280620575, 0.0981672927737236, -0.4928472638130188, 0.0668552964925766, 0.29235517978668213, 0.04993154853582382, -0.25072285532951355, -0.016454674303531647, 0.027864178642630577}, {-0.024387171491980553, -0.28181546926498413, -0.19066794216632843, 0.0333627425134182, 0.02043420448899269, 0.07625414431095123, -0.24433346092700958, -0.20199374854564667, 0.15406125783920288, 0.05046169087290764, -0.1578260213136673, 0.02259868010878563, 0.0026920088566839695, 0.003604816971346736, 0.10238293558359146, -0.07607631385326385, -0.09093385189771652, 0.12406492233276367, -0.20261940360069275, -0.05850265547633171, 0.0428384393453598, 0.033457979559898376, -0.12267827242612839, 0.05491277202963829, -0.037142761051654816, 0.15986324846744537, -0.05852866545319557, 0.04485531896352768, -0.0221711453050375, 0.092910997569561, 0.231194406747818, 0.18035626411437988, -0.03507472574710846, 0.22703973948955536, -0.051698554307222366, 7.000728510320187e-05, 0.1719149500131607, 0.03260234743356705, -0.018304765224456787, 0.014782508835196495, 0.11645377427339554, 0.16793349385261536, -0.041169773787260056, 0.11882274597883224, 0.039239294826984406, -0.033419497311115265, 0.12931138277053833, -0.018528005108237267, -0.07464085519313812, 0.05759158357977867, -0.24821627140045166, -0.1696510761976242, 0.018546564504504204, -0.13251708447933197, -0.03820910304784775, -0.027106765657663345, 0.07601165026426315, -0.17464813590049744, 0.037697289139032364, -0.17417648434638977, 0.14438964426517487, 0.1343717724084854, 0.037283021956682205, 0.12933920323848724, 0.2273852527141571, -0.038958072662353516, 0.0343642495572567, -0.28156429529190063, -0.34426137804985046, 0.009631973691284657, 0.022030284628272057, 0.0835636705160141, -0.013090892694890499, 0.17187440395355225, 0.19377046823501587, -0.09623786062002182, -0.2681420147418976, 0.08175785839557648, -0.13566936552524567, -0.03523102030158043, 0.20811434090137482, 0.07954436540603638, -0.0902562290430069, 0.2034231722354889, 0.172590970993042, 0.16398072242736816, -0.02967771887779236, -0.1782464236021042, -0.10452438145875931, -0.2606320381164551, 0.041420627385377884, -0.04414879158139229, 0.05339963734149933, 0.01756378822028637, 0.09359918534755707, -0.11754205077886581, 0.14980104565620422, -0.22049884498119354, 0.1349250227212906, -0.3250110447406769, 0.029322510585188866, 0.1217598021030426, -0.04731658846139908, -0.00231689284555614, -0.006137466058135033, 0.12831993401050568, 0.11297804862260818, -0.15989834070205688, -0.04773252457380295, 0.03263411670923233, 0.08744342625141144, 0.01018123235553503, 0.1672106832265854, -0.030262278392910957, -0.16873401403427124, -0.3844047486782074, 0.008951404131948948, -0.10195241868495941, 0.1338968425989151, -0.0034439668525010347, -0.11850050091743469, 0.07947766780853271, -0.03948768973350525, 0.06048649549484253, 0.06007910519838333, -0.07951947301626205, 0.08516833186149597, 0.07401726394891739, 0.0986485555768013, -0.006615600548684597, 0.2272263616323471, 0.08027736097574234, 0.1222739890217781, -0.11546706408262253, 0.09595208615064621, 0.1342875063419342, -0.05061041936278343, -0.10689348727464676, 0.08779536932706833, 0.09610314667224884, -0.028165945783257484, -0.06432732194662094, -0.02405819483101368, 0.2276826798915863, -0.22432123124599457, 0.069921113550663, -0.08102777600288391, 0.1587529480457306, -0.0054350304417312145, -0.06416838616132736, 0.3153286576271057, -0.001477208104915917, 0.008709009736776352, -0.15015354752540588, -0.07780911028385162, 0.1974853128194809, -0.12131276726722717, -0.19906337559223175, 0.11808764189481735, -0.13631121814250946, 0.019884679466485977, -0.0011071859626099467, -0.09157483279705048, -0.12429830431938171, -0.11298203468322754, 0.08662551641464233, 0.026857877150177956, 0.07532738149166107, -0.007151290774345398, -0.09650127589702606, -0.0834539532661438, 0.1951664686203003, -0.06077182665467262, -0.029229357838630676, 0.18324892222881317, -0.12019994854927063, 0.07798762619495392, -0.08021944761276245, -0.007600270211696625, 0.11454907059669495, -0.01036431547254324, -0.10496237128973007, -0.31348392367362976, -0.018969528377056122, 0.08166757225990295, -0.0011710341786965728, 0.13653138279914856, 0.035513486713171005, 0.007544858381152153, -0.05860442668199539, -0.03484582528471947, 0.08338601887226105, -0.06306660175323486, -0.045113611966371536, -0.09795918315649033, -0.15912483632564545, -0.026878759264945984, -0.026537373661994934, -0.11632565408945084, -0.3732052445411682, 0.11434726417064667, 0.007030357141047716, 0.04870375618338585, 0.35341745615005493, -0.03893209248781204, -0.05764478072524071, 0.38674166798591614, 0.0888645350933075, 0.05838119611144066, -0.18562738597393036, -0.061689555644989014, -0.0549466498196125, -0.006674750242382288, -0.06067533418536186, 0.028768489137291908, -0.0750373974442482, -0.035568222403526306, -0.11696869879961014, 0.02098553255200386, -0.12933246791362762, 0.1033245399594307, -0.17280463874340057, -0.38555729389190674, -0.06177552044391632, 0.09615308791399002, -0.006565942894667387, 0.03938993439078331, 0.2428285926580429, 0.21930943429470062, 0.0865277349948883, -0.11891273409128189, 0.018766328692436218, -0.06141602620482445, -0.0659598782658577, -0.15015263855457306, -0.08524683862924576, -0.06442748010158539, 0.12086699903011322, 0.1158963218331337, -0.021875740960240364, 0.030075469985604286, 0.19428549706935883, -0.038486823439598083, 0.05578799173235893, 0.06759387254714966, 0.13371850550174713, 0.19756272435188293, -0.05768246576189995, -0.31280067563056946, 0.15902100503444672, 0.06984958052635193, -0.3386479318141937, -0.03137819841504097, 0.0356718972325325, -0.013685647398233414, -0.27481809258461}, {-0.17526860535144806, -0.352730929851532, -0.15520521998405457, -0.03653499856591225, 0.07954538613557816, -0.06732401996850967, -0.12886293232440948, -0.16401708126068115, -0.20093941688537598, -0.04278752580285072, -0.06609699130058289, -0.10060416162014008, 0.04229895770549774, -0.04634829983115196, 0.03492886573076248, -0.010356124490499496, 0.07528851926326752, -0.24912525713443756, 0.08890391141176224, -0.097343809902668, 0.3995618522167206, 0.04860270768404007, -0.07441602647304535, -0.12607155740261078, -0.08949519693851471, 0.11631950736045837, -0.12026168406009674, -0.020503632724285126, -0.012023091316223145, -0.02197345905005932, -0.014237537980079651, 0.13249371945858002, 0.04730589687824249, 0.05604378506541252, -0.1687217354774475, 0.09974319487810135, 0.08685702085494995, -0.08344686776399612, 0.04452931135892868, -0.03534472733736038, -0.02183784358203411, 0.021676775068044662, -0.08518176525831223, -0.19723984599113464, -0.1520151048898697, -0.0019412727560847998, -0.18224133551120758, 0.018311526626348495, -0.23852378129959106, -0.0067820302210748196, -0.020479558035731316, -0.13763128221035004, 0.13088873028755188, -0.06465108692646027, 0.01537925936281681, -0.10571766644716263, -0.006409673485904932, 0.12470997869968414, -0.3155713975429535, 0.10503530502319336, -0.18395505845546722, -0.1004083901643753, 0.0808311253786087, 0.08232158422470093, 0.058029331266880035, 0.12197066098451614, 0.033289194107055664, -0.05346735939383507, -0.011110946536064148, 0.06337305158376694, -0.15738479793071747, -0.022397689521312714, 0.03360246494412422, 0.2695891857147217, -0.25483885407447815, 0.15185026824474335, -0.08571978658437729, 0.038421377539634705, 0.29542824625968933, 0.18294969201087952, 0.04522610828280449, 0.02019409090280533, -0.17106251418590546, 0.16418351233005524, -0.18422900140285492, 0.11149042844772339, 0.13858091831207275, 0.015350175090134144, 0.03589897230267525, -0.18330399692058563, 0.08391522616147995, -0.18342061340808868, 0.1377955824136734, -0.11235933005809784, 0.003322803182527423, 0.25489652156829834, -0.08272760361433029, 0.05308912321925163, -0.019190644845366478, -0.1791406124830246, 0.015966514125466347, -0.10951075702905655, 0.07248961925506592, -0.10776348412036896, 0.17032675445079803, -0.7132670283317566, 0.10929963737726212, -0.19557367265224457, -0.18630939722061157, -0.16139701008796692, 0.14345236122608185, -0.18011167645454407, -0.07579309493303299, 0.022124500945210457, 0.007134891115128994, -0.09840108454227448, 0.036240383982658386, 0.0951429083943367, 0.0036372244358062744, 0.24067574739456177, -0.1029551699757576, -0.07112491875886917, -0.045334286987781525, 0.17344240844249725, 0.17168346047401428, 0.19213047623634338, -0.06718076765537262, 0.17026761174201965, -0.1738673597574234, 0.11067455261945724, -0.10072388499975204, -0.06563262641429901, 0.22905774414539337, 0.022868582978844643, -0.06320358067750931, 0.08748815953731537, 0.060022708028554916, 0.01948828622698784, 0.18767644464969635, -0.011827186681330204, -0.08963389694690704, 0.27987053990364075, -0.03431238234043121, -0.011639255098998547, 0.21985797584056854, -0.20064349472522736, 0.00810623075813055, -0.10414616018533707, -0.17284925282001495, 0.022211140021681786, -0.1419956237077713, 0.13749875128269196, -0.11641250550746918, 0.11881674826145172, 0.040055714547634125, 0.20090311765670776, 0.10230863839387894, -0.21520347893238068, -0.053582798689603806, 0.10723350197076797, -0.029944173991680145, 0.011634359136223793, 0.11723746359348297, 0.2254749834537506, -0.14379867911338806, 0.07829815149307251, 0.009940504096448421, 0.03790906071662903, -0.05681772157549858, 0.030353059992194176, -0.0020711799152195454, 0.003235951531678438, -0.03856270760297775, 0.09377291053533554, -0.06352757662534714, 0.06602708995342255, -0.01908390410244465, -0.12414038926362991, -0.09472516179084778, -0.0732690617442131, -0.2516323924064636, -0.026932716369628906, 0.11425463110208511, 0.0795932412147522, -0.12414474785327911, 0.10517114400863647, 0.04221450537443161, 0.08766891807317734, -0.15898889303207397, 0.019125042483210564, -0.012101702392101288, -0.1866573989391327, 0.25614261627197266, 0.12071233987808228, 0.24295514822006226, 0.11969317495822906, -0.3731677234172821, -0.1944698840379715, 0.06495659053325653, 0.062488947063684464, 0.003620252711698413, 0.1746671348810196, -0.022108370438218117, 0.10767054557800293, 0.08965682238340378, 0.06508158147335052, -0.10302931070327759, -0.06933822482824326, -0.07415445894002914, 0.04720279201865196, -0.021834200248122215, -0.06610307842493057, 0.14172345399856567, 0.1801917999982834, 0.02859528176486492, -0.10963490605354309, -0.08166830241680145, 0.03167364373803139, -0.14684301614761353, 0.1510082483291626, 0.17225539684295654, 0.09523921459913254, 0.16108191013336182, -0.12699785828590393, -0.02320997416973114, -0.07667664438486099, -0.1696799248456955, -0.10846072435379028, 0.1622408926486969, 0.1241048127412796, -0.07131822407245636, -0.15485486388206482, 0.012934747152030468, -0.03551103547215462, 0.23189358413219452, 0.17098914086818695, 0.10778321325778961, -0.05125780031085014, 0.11221341043710709, -0.26341453194618225, -0.09650275856256485, 0.2287488877773285, 0.005272346083074808, -0.012183894403278828, -0.0113059775903821, -0.3260977268218994, 0.05415235832333565, 0.04980171471834183, 0.12609587609767914, 0.011414299719035625, 0.13027112185955048, -0.23782943189144135, -0.08858928829431534, -0.03671065717935562, 0.23349550366401672, 0.058479588478803635}, {-0.030355190858244896, 0.2713339626789093, 0.09612327069044113, -0.03157835081219673, -0.15639697015285492, 0.0951051339507103, 0.05544716492295265, 0.05318398401141167, -0.10610882937908173, 0.10127700865268707, 0.1151590496301651, 0.3448113203048706, 0.0075734928250312805, 0.019399946555495262, -0.009825186803936958, -0.008704661391675472, 0.022836152464151382, -0.0007067482802085578, -0.12263241410255432, 0.15968360006809235, -0.014219974167644978, -0.07169493287801743, 0.20261022448539734, -0.16581465303897858, -0.05112572759389877, -0.10173285752534866, 0.010837032459676266, -0.10661744326353073, -0.06661605834960938, -0.08440405130386353, -0.12280812114477158, 0.13288265466690063, 0.06377436965703964, 0.0035815455485135317, -0.03848797455430031, 0.04249614477157593, -0.014100348576903343, -0.09138106554746628, -0.06662868708372116, 0.020988885313272476, -0.08081846684217453, 0.14116732776165009, 0.21979796886444092, -0.034238241612911224, 0.005577511619776487, 0.013008376583456993, 0.026887182146310806, 0.08397052437067032, -0.10103220492601395, -0.12653523683547974, 0.12463775277137756, -0.2746405303478241, -0.030301997438073158, -0.15301696956157684, -0.07412336766719818, -0.00916630495339632, -0.14519299566745758, -0.0377957783639431, 0.00475645437836647, 0.07365057617425919, -0.017895542085170746, 0.02767583727836609, -0.07434402406215668, 0.1146816611289978, -0.11005208641290665, -0.09318813681602478, 0.09126470983028412, -0.0592137910425663, -0.10390161722898483, 0.1649901419878006, -0.07466087490320206, -0.016625357791781425, -0.23748363554477692, 0.098251573741436, -0.13397347927093506, 0.03615780174732208, -0.03408436104655266, 0.10783107578754425, 0.024112556129693985, 0.21214084327220917, -0.1603485494852066, -0.03375853970646858, 0.03665783628821373, 0.03543601557612419, 0.022439036518335342, 0.07728663831949234, 0.1606924831867218, 0.05195103958249092, 0.017957279458642006, 0.03548302501440048, 0.030995719134807587, -0.05893407016992569, -0.005066073499619961, 0.34793227910995483, 0.1032828837633133, -0.11580085754394531, 0.06748998910188675, 0.02468857541680336, -0.10586874186992645, -0.19186218082904816, -0.05719086900353432, 0.0655989870429039, -0.03947484865784645, -0.006768689025193453, -0.11472231894731522, 0.1747967004776001, 0.162093386054039, -0.05785929784178734, -0.0896037146449089, 0.04678987339138985, -0.06919775903224945, 0.06375600397586823, -0.0724727213382721, -0.029955169185996056, 0.10369613766670227, 0.12136722356081009, 0.0019511491991579533, -0.11913250386714935, -0.21631832420825958, 0.0004970072186551988, 0.0016891192644834518, 0.11823758482933044, 0.08198758959770203, 0.20398667454719543, -0.02153030037879944, 0.12743009626865387, 0.08848108351230621, -0.010419829748570919, -0.04968981817364693, 0.053445857018232346, 0.09474657475948334, 0.161294624209404, -0.2440834939479828, 0.08019032329320908, 0.15800438821315765, -0.1974589228630066, 0.05819518491625786, -0.2207251787185669, 0.007664601318538189, -0.30168417096138, 0.06633263826370239, 0.03916525840759277, 0.3116269111633301, 0.12544839084148407, 7.469215051969513e-05, 0.054814308881759644, 0.018288400024175644, -0.18286000192165375, -0.1796172559261322, 0.04134145751595497, -0.015043169260025024, 0.07748451828956604, 0.057550616562366486, -0.02113284170627594, 0.09682156890630722, -0.07788662612438202, -0.02890627831220627, -0.19659370183944702, 0.13780248165130615, 0.21248774230480194, -0.020364459604024887, 0.014859405346214771, 0.20017191767692566, -0.30513259768486023, 0.03539117053151131, -0.04603363946080208, -0.12775811553001404, 0.38934049010276794, -0.16810345649719238, -0.015999553725123405, -0.0858793705701828, -0.08914554119110107, -0.12112925946712494, -0.03147503361105919, -0.021593820303678513, 0.352899432182312, 0.08191043883562088, 0.0040953694842755795, -0.23984023928642273, 0.06691328436136246, -0.10002744942903519, 0.07503350079059601, -0.21572335064411163, -0.012524271383881569, 0.02879839576780796, -0.14667099714279175, 0.1096523329615593, -0.20057980716228485, 0.04266124591231346, -0.16837038099765778, -0.02097938023507595, 0.11801672726869583, -0.28999584913253784, -0.15138112008571625, 0.0023209559731185436, -0.05937042087316513, 0.5984822511672974, 0.13261868059635162, -0.05521540343761444, -0.007178891450166702, 0.04948367178440094, -0.239881232380867, -0.06923995912075043, 0.015607853420078754, 0.04308686405420303, 0.09301863610744476, 0.18389204144477844, -0.06120298430323601, 0.0209992416203022, -0.013478676788508892, -0.007134535815566778, 0.01258873287588358, -0.04785110801458359, 0.056738585233688354, 0.09680100530385971, -0.09779416769742966, 0.04014775902032852, 0.007439993787556887, -0.1158394068479538, -0.02701697312295437, 0.006661756429821253, 0.04079930856823921, 0.027038438245654106, 0.058556828647851944, -0.10675361752510071, -0.008892256766557693, -0.05132163688540459, -0.027400603517889977, -0.13080276548862457, -0.015233942307531834, -0.11881458759307861, -0.07218058407306671, -0.03519975021481514, -0.03663972020149231, 0.17447762191295624, 0.17174476385116577, -0.06003535911440849, -0.037990469485521317, -0.10899161547422409, 0.06980320066213608, -0.1652505099773407, 0.13207240402698517, 0.19847562909126282, -0.04202960804104805, 0.02446068823337555, 0.022091755643486977, -0.03260380029678345, 0.08531656116247177, -0.06952589750289917, -0.15559151768684387, -0.06855045258998871, -0.02219432406127453, 0.13212436437606812, 0.1557810753583908, -0.051685914397239685, -0.02543734386563301}, {0.1946677565574646, 0.10141615569591522, 0.06587076932191849, 0.13058343529701233, 0.20481109619140625, -0.02231992781162262, -0.028587020933628082, -0.25905606150627136, 0.26770323514938354, 0.0010305161122232676, -0.22515293955802917, 0.008573009632527828, 0.4107343554496765, 0.13375604152679443, 0.17396609485149384, -0.13289983570575714, 0.3146584928035736, -0.12494006007909775, -0.23183034360408783, -0.22452016174793243, 0.24961304664611816, 0.13228681683540344, 0.087984099984169, 0.34638920426368713, -0.07060950994491577, -0.25681251287460327, -0.21057216823101044, -0.249273419380188, 0.05018376559019089, 0.13173630833625793, -0.14636628329753876, -0.1876865178346634, -0.04647968336939812, -0.16210348904132843, -0.07220066338777542, 0.13878630101680756, -0.03540646284818649, -0.20387956500053406, -0.17726056277751923, 0.038575273007154465, -0.00967198796570301, 0.3019672930240631, 0.14717523753643036, -0.08697686344385147, 0.1956188976764679, -0.015181547030806541, 0.1264560967683792, 0.2138792872428894, -0.11104344576597214, 0.156538724899292, 0.1470886766910553, 0.08283402770757675, 0.22832150757312775, -0.10168741643428802, 0.18308451771736145, -0.2547647953033447, -0.13204209506511688, 0.2287949025630951, 0.14120586216449738, 0.07048481702804565, -0.018228581175208092, -0.10053461045026779, -0.08552800863981247, -0.18042217195034027, -0.08383078128099442, 0.026620106771588326, -0.12558937072753906, 0.15854763984680176, 0.13948774337768555, -0.13113239407539368, -0.17378206551074982, -0.06999102979898453, -0.10345283895730972, -0.049221500754356384, -0.12177001684904099, -0.07352478802204132, -0.04945223405957222, 0.0461709201335907, -0.026304909959435463, 0.053475260734558105, 0.11161220073699951, 0.08977212756872177, -0.03801511228084564, 0.03158845752477646, 0.04225025326013565, -0.028719497844576836, 0.2941935360431671, 0.11270980536937714, 0.061614301055669785, -0.025027155876159668, 0.09603698551654816, 0.27728021144866943, 0.02963283471763134, 0.22912812232971191, 0.00979005079716444, -0.009342542849481106, -0.12092512100934982, 0.1513635367155075, -0.17528894543647766, 0.07022062689065933, 0.20859341323375702, -0.08027441054582596, -0.14402098953723907, -0.05262890085577965, 0.10626054555177689, 0.15609978139400482, -0.16066259145736694, 0.17699719965457916, -0.2300930619239807, -0.17229536175727844, -0.2519974410533905, -0.11166799813508987, -0.09662622213363647, 0.05750475823879242, -0.11743752658367157, 0.1554480642080307, 0.07486096769571304, -0.008915032260119915, -0.08229178190231323, -0.07509485632181168, -0.2721042335033417, 0.13511835038661957, 0.11456381529569626, 0.003739939769729972, -0.020680924877524376, 0.11590997129678726, 0.010298727080225945, 0.09628506004810333, -0.04642896354198456, -0.18827545642852783, -0.05159804970026016, -0.15641364455223083, 0.10281147062778473, -0.01705516129732132, -0.1394597738981247, -0.041395530104637146, 0.045342426747083664, -0.22169096767902374, -0.03425087779760361, 0.10238038748502731, -0.09910593181848526, -0.08738873153924942, 0.0811491534113884, -0.21864053606987, 0.40393415093421936, 0.03519479185342789, -0.019681330770254135, 0.02435518242418766, -0.01432942133396864, -0.18570899963378906, 0.07594287395477295, 0.01972852274775505, -0.04361877217888832, 0.04430282115936279, 0.09411826729774475, 0.06826536357402802, 0.24641582369804382, -0.33880558609962463, -0.08829933404922485, 0.16056625545024872, 0.08548017591238022, -0.09809710830450058, 0.020028410479426384, 0.38865789771080017, -0.018950046971440315, 0.08256938308477402, 0.13843373954296112, 0.08027531206607819, -0.030702030286192894, 0.02061748504638672, -0.16452299058437347, 0.14141695201396942, -0.0477278046309948, 0.13389845192432404, 0.03684905171394348, -0.06546670198440552, -0.09255219995975494, 0.07867743819952011, -0.09848993271589279, -0.1648973822593689, 0.11232935637235641, 0.05948770046234131, 0.12788903713226318, 0.053787462413311005, -0.17324768006801605, 0.11963760852813721, -0.08919277042150497, 0.06265614181756973, -0.35061243176460266, -0.11272938549518585, 0.12768469750881195, -0.2495545595884323, 0.20137202739715576, 0.0543396957218647, 0.16394031047821045, 0.025940708816051483, -0.11846400797367096, -0.20723816752433777, -0.041320957243442535, 0.08889605849981308, -0.17224034667015076, -0.0661122053861618, 0.01837090030312538, 0.1165546253323555, -0.1886357069015503, -0.08281226456165314, -0.13109800219535828, -0.07464448362588882, -0.15463928878307343, -0.004248622339218855, -0.07923556119203568, 0.01257127895951271, -0.04647678881883621, -0.15060538053512573, 0.09746231138706207, -0.024055220186710358, 0.10956673324108124, -0.13993506133556366, 0.008011732250452042, -0.08792699128389359, -0.14722688496112823, 0.29383933544158936, 0.17656977474689484, -0.06985748559236526, 0.06249343603849411, -0.07524202764034271, -0.020153841003775597, -0.07943897694349289, 0.10965514928102493, -0.06600263714790344, -0.006382331717759371, 0.13757053017616272, 0.17581972479820251, 0.12041137367486954, -0.04037923738360405, 0.111739881336689, 0.23970603942871094, 0.04606239125132561, 0.029409421607851982, 0.09809213876724243, -0.0689970999956131, 0.038394495844841, 0.2607620358467102, 0.08171030133962631, 0.07703319191932678, -0.030519282445311546, 0.009046829305589199, 0.20994214713573456, 0.012396476231515408, -0.021984128281474113, -0.02056301385164261, -0.03779004141688347, -0.11910183727741241, -0.04040367156267166, 0.08994891494512558, -0.0926656499505043}, {-0.034871917217969894, -0.10109756886959076, -0.006138913799077272, 0.15378277003765106, 0.17903175950050354, 0.14902126789093018, -0.07070212811231613, 0.005338574759662151, -0.09973631799221039, 0.07957486063241959, -0.195135697722435, 0.1409904956817627, -0.010738407261669636, 0.18371903896331787, 0.2749888002872467, -0.0071670906618237495, 0.18544240295886993, 0.009691646322607994, 0.015512716956436634, -0.08811449259519577, 0.08165901154279709, -0.0048790923319756985, -0.18470875918865204, 0.33129143714904785, 0.12974075973033905, 0.09506642818450928, -0.2581608295440674, -0.10431167483329773, -0.20444434881210327, 0.12057919800281525, 0.09106069058179855, 0.09166039526462555, 0.2214323729276657, -0.0955682247877121, -0.17093810439109802, 0.07398276031017303, 0.06961756944656372, -0.11747264117002487, 0.21279847621917725, -0.1486118584871292, 0.09407038986682892, 0.1268671751022339, 0.012723433785140514, 0.13442279398441315, -0.10691403597593307, 0.18208107352256775, -0.057299576699733734, 0.05707371234893799, 0.06694730371236801, 0.059164755046367645, -0.19819676876068115, -0.0284233707934618, -0.041473139077425, 0.08863504976034164, -0.17491962015628815, 0.17395998537540436, 0.24078984558582306, -0.11952350288629532, -0.19836357235908508, 0.40337154269218445, 0.11483611166477203, 0.1400415301322937, 0.11894658207893372, 0.2762971818447113, 0.05383444204926491, 0.06765083223581314, 0.023268453776836395, -0.026483409106731415, 0.16998767852783203, 0.15348674356937408, 0.0007218383834697306, 0.016487834975123405, -0.04424023628234863, 0.19916987419128418, 0.02815292589366436, 0.0885457918047905, -0.6532984375953674, -0.34355080127716064, -0.05732210725545883, -0.02675694413483143, -0.031044216826558113, -0.006774877198040485, -0.14761292934417725, 0.153249591588974, -0.05263591185212135, -0.047239262610673904, -0.14601194858551025, -0.011910474859178066, 0.0721360445022583, -0.08644901216030121, 0.03706236556172371, -0.17827695608139038, -0.10060693323612213, 0.09811875224113464, -0.07453283667564392, -0.0269631277769804, -0.06067882105708122, -0.21897397935390472, -0.17814314365386963, 0.07353388518095016, -0.011017661541700363, 0.10999499261379242, -0.3096582591533661, -0.024244168773293495, 0.22031313180923462, 0.14521226286888123, 0.1469094604253769, -0.06833753734827042, -0.08679284155368805, -0.05344181880354881, -0.03077256865799427, -0.042164724320173264, 0.028852203860878944, -0.0023707153741270304, -0.29318398237228394, -0.20715998113155365, 0.3874463438987732, 0.0733797699213028, 0.23075057566165924, 0.11984317004680634, 0.053800854831933975, -0.08748795092105865, 0.08363131433725357, -0.06933921575546265, 0.04716057330369949, -0.12341462820768356, 0.2864988446235657, -0.1371750831604004, 0.07569848001003265, -0.2856028974056244, 0.24094875156879425, 0.13002251088619232, 0.18051807582378387, -0.12580011785030365, -0.03815010190010071, 0.029679197818040848, 0.1425343006849289, 0.03603852540254593, -0.009029805660247803, -0.04629862308502197, -0.006449046544730663, 0.1747165322303772, 0.2976132035255432, -0.11571346968412399, 0.02625410631299019, 0.04229980334639549, -0.00334174488671124, 0.0861411765217781, 0.08110831677913666, -0.15016485750675201, 0.10466931015253067, 0.004534377250820398, 0.06470025330781937, -0.024789150804281235, -0.03513573855161667, -0.003674461040645838, -0.05135544016957283, -0.026982618495821953, 0.060117755085229874, -0.16635556519031525, 0.1307671219110489, 0.03466299548745155, -0.20231597125530243, -0.028275029733777046, -0.12591272592544556, -0.10641129314899445, 0.3548910617828369, -0.13393719494342804, 0.04086115211248398, -0.007827119901776314, -0.1959841102361679, -0.03156391158699989, -0.17284958064556122, 0.05324200540781021, -0.024299263954162598, 0.12208837270736694, 0.06418748199939728, 0.07147491723299026, 0.026159007102251053, 0.1263081431388855, -0.14510132372379303, 0.19151170551776886, -0.03821616619825363, -0.061445452272892, -0.08310989290475845, -0.002298978855833411, 0.06537123769521713, 0.0033246746752411127, -0.1726495921611786, -0.2409951090812683, -0.0567815788090229, -0.04269426688551903, -0.20425404608249664, 0.11067593097686768, 0.19620105624198914, -0.2582975924015045, -0.1558532565832138, 0.14453630149364471, 0.04192014038562775, -0.4700920283794403, 0.1527460366487503, 0.12882548570632935, 0.11773782968521118, 0.054058987647295, 0.053679242730140686, -0.15316320955753326, -0.07117246091365814, 0.11947262287139893, 0.28070804476737976, -0.2553374171257019, -0.06209771707653999, 0.03910338506102562, -0.17727130651474, -0.02717306651175022, -0.047640904784202576, 0.11856461316347122, 0.009706302545964718, -0.06127253919839859, 0.06702257692813873, 0.012142812833189964, 0.045526325702667236, -0.04392354190349579, 0.07508266717195511, 0.03653803840279579, -0.09859023988246918, 0.07551910728216171, -0.06999289244413376, 0.15659798681735992, 0.10409257560968399, 0.0991751179099083, 0.06752882897853851, -0.04693760722875595, -0.02792946994304657, -0.044002655893564224, 0.13512718677520752, -0.13234756886959076, 0.03439377248287201, 0.38274601101875305, 0.13730815052986145, 0.239252969622612, 0.013359883800148964, -0.09226622432470322, -0.09455156326293945, 0.15678144991397858, 0.10823235660791397, 0.12166676670312881, -0.11664873361587524, -0.1315849870443344, 0.08818770200014114, -0.054837316274642944, 0.17884361743927002, -0.12861384451389313, -0.16390448808670044, -0.05037183314561844, 0.060672275722026825, -0.11883023381233215}, {0.11168307065963745, 0.030011890456080437, 0.03789125010371208, 0.1034899652004242, -0.3461867868900299, -0.07052939385175705, 0.06781339645385742, 0.16125160455703735, -0.031962208449840546, 0.001191602204926312, 0.06945932656526566, -0.012137610465288162, 0.018251454457640648, -0.029808081686496735, -0.03791048750281334, 0.03971708193421364, 0.11270707845687866, -0.17338097095489502, 0.11704502254724503, -0.1031610369682312, 0.47786155343055725, 0.025812026113271713, 0.04087255895137787, 0.054632414132356644, -0.00813395343720913, -0.1860823780298233, 0.0438302606344223, 0.032334983348846436, -0.12681427597999573, -0.022736283019185066, -0.007438586559146643, 0.010104388929903507, 0.1768946498632431, -0.1739249974489212, 0.07868102192878723, 0.03692666068673134, 0.029337309300899506, 0.029361525550484657, -0.14378142356872559, -0.25126850605010986, -0.050200123339891434, 0.09513793140649796, -0.029751120135188103, 0.08582646399736404, -0.07333032041788101, 0.04647073894739151, -0.0726819708943367, -0.20064572989940643, 0.09537870436906815, 0.1637706160545349, 0.11034584045410156, 0.16153985261917114, 0.08762909471988678, 0.12998412549495697, -0.02770296484231949, 0.06576193124055862, -0.007529062684625387, 0.16652779281139374, -0.11195086687803268, -0.06578671187162399, 0.08469372242689133, -0.4655456244945526, -0.004088343121111393, -0.15504568815231323, 0.037812430411577225, 0.22548681497573853, -0.03314615413546562, -0.0660761371254921, 0.10068903863430023, 0.012354190461337566, 0.19843661785125732, -0.11653847247362137, 0.0836290791630745, 0.010179083794355392, -0.061827704310417175, 0.04256605729460716, -0.02551133558154106, -0.10596168786287308, -0.1954004466533661, -0.24292878806591034, -0.032047927379608154, -0.09879685193300247, -0.11471247673034668, 0.01318846084177494, 0.04700447991490364, -0.337258517742157, -0.12467866390943527, 0.08258115500211716, -0.007217416539788246, -0.12311600893735886, 0.03817438706755638, -0.04928017780184746, -0.1198129653930664, -0.02367355301976204, 0.10179385542869568, 0.08629787713289261, 0.06021735444664955, 0.013332579284906387, 0.06315939873456955, 0.09498818963766098, 0.10146200656890869, 0.06423086673021317, 0.014553084969520569, 0.19460712373256683, 0.09757435321807861, -0.14121533930301666, -0.1610773652791977, -0.054768119007349014, -0.11714799702167511, 0.020832838490605354, -0.12800762057304382, -0.020318930968642235, -0.04306879639625549, 0.05452145263552666, -0.017715880647301674, 0.044931381940841675, -0.0307603906840086, -0.06905178725719452, 0.21086297929286957, -0.12709251046180725, -0.0863584578037262, 0.002539945300668478, -0.12181823700666428, -0.15551504492759705, 0.0976288914680481, 0.3328685462474823, 0.037020862102508545, 0.022836506366729736, -0.009593035094439983, -0.14548757672309875, -0.035385649651288986, -0.09664642065763474, -0.19305026531219482, -0.17345137894153595, -0.03296058997511864, -0.449956476688385, 0.04163186252117157, -0.05654474347829819, -0.06405778229236603, 0.07292040437459946, -0.09125247597694397, 0.33646905422210693, -0.33951592445373535, 0.003958902321755886, -0.07713787257671356, 0.16511031985282898, 0.0349818654358387, -0.07892679423093796, 0.05852003023028374, -0.04422846436500549, -0.23256759345531464, 0.0899832621216774, 0.004680791404098272, 0.016937844455242157, 0.07893138378858566, 0.06121173873543739, 0.005409505683928728, -0.0002566063485573977, 0.18319915235042572, -0.011732330545783043, 0.19663700461387634, 0.25353455543518066, -0.19577932357788086, -0.03458136320114136, 0.027855850756168365, -0.03412910923361778, 0.13513359427452087, 0.01600147970020771, 0.0450623482465744, 0.08197442442178726, -0.2355770468711853, -0.05835045874118805, -0.041822757571935654, -0.03529445454478264, 0.024151291698217392, -0.060832854360342026, 0.01453413162380457, 0.062126096338033676, -0.0963355004787445, 0.0930832177400589, 0.02760915458202362, 0.10787677764892578, -0.01411568745970726, 0.06725339591503143, -0.03661251440644264, -0.06440528482198715, -0.05339755490422249, -0.02158418297767639, 0.22057697176933289, 0.03969242796301842, -0.030655480921268463, -0.11876486241817474, -0.2845071852207184, -0.18020111322402954, 0.0004892150172963738, 0.0635029599070549, 0.05407887324690819, 0.3220982551574707, -0.040440887212753296, 0.18119339644908905, 0.04841981828212738, 0.018185246735811234, -0.11211978644132614, 0.06917661428451538, -0.07320605218410492, 0.06351475417613983, 0.08401354402303696, -0.21137242019176483, -0.2562708258628845, 0.0633101537823677, 0.07892953604459763, -0.1275026947259903, -0.044342041015625, 0.06857102364301682, 0.009658348746597767, 0.13036848604679108, -0.11231489479541779, 0.1413368135690689, -0.07957810908555984, 0.24003705382347107, -0.025343773886561394, -0.1037740707397461, 0.13458837568759918, -0.13495203852653503, -0.001988822827115655, -0.12863565981388092, -0.10837826877832413, 0.04088236764073372, -0.026847422122955322, -0.023123377934098244, 0.08015669137239456, -0.019192485138773918, -0.013256353326141834, -0.04881509393453598, 0.009305360727012157, -0.028503429144620895, 0.2013348489999771, -0.0508834607899189, -0.10076718032360077, -0.05979517847299576, -0.18129631876945496, 0.005577359814196825, -0.09320556372404099, -0.00788082554936409, -0.15439078211784363, -0.052889540791511536, 0.02895904704928398, 0.031923990696668625, -0.13030320405960083, 0.042300816625356674, 0.07015357166528702, -0.09653608500957489, -0.01753326505422592, -0.000996118295006454, 0.12557178735733032, -0.044265758246183395}, {0.06302927434444427, 0.04275403544306755, 0.12411166727542877, -0.2827415466308594, 0.11399360746145248, 0.026406381279230118, 0.13571520149707794, -0.1346750557422638, -0.02533063478767872, 0.48136311769485474, -0.1622627079486847, 0.22327536344528198, 0.04140368103981018, 0.04144454374909401, -0.10012730956077576, 0.08674293011426926, 0.0913834199309349, -0.1399182379245758, -0.17188262939453125, 0.32213008403778076, -0.04929961636662483, -0.03932155668735504, -0.26535484194755554, 0.03809953108429909, 0.038786958903074265, 0.001856906688772142, 0.04883655160665512, -0.13976728916168213, -0.046872805804014206, 0.06377154588699341, -0.27114084362983704, 0.136618971824646, -0.030986124649643898, 0.09006448090076447, 0.0775412917137146, 0.0032494792249053717, 0.00932646170258522, 0.08146486431360245, 0.09371665120124817, 0.1037784069776535, -0.10938327014446259, 0.14907386898994446, 0.02457820065319538, -0.28421929478645325, 0.14192619919776917, 0.005299846641719341, -0.06689538061618805, 0.31576189398765564, -0.2594461441040039, 0.11582974344491959, -0.04461723193526268, 0.034278664737939835, 0.09220436215400696, 0.028094947338104248, -0.057938944548368454, 0.07564938068389893, -0.023124760016798973, 0.14347369968891144, -0.10286290943622589, 0.1973608285188675, -0.045082006603479385, -0.06715267151594162, 0.10884790122509003, -0.0161597803235054, 0.05693243816494942, -0.046129532158374786, 0.06830611824989319, 0.21279625594615936, 0.07525338977575302, 0.23870837688446045, -0.026598656550049782, -0.040781911462545395, -0.13257181644439697, -0.01612444967031479, 0.030471187084913254, -0.07243544608354568, 0.009071185253560543, -0.05679669231176376, -0.267640620470047, -0.3762287497520447, 0.05498616397380829, 0.010025789961218834, -0.012265809811651707, -0.14746332168579102, -0.03162002936005592, -0.05994688346982002, -0.034330252557992935, -0.05549687519669533, -0.24026024341583252, -0.18342800438404083, -0.13531535863876343, 0.044249050319194794, 0.02805166132748127, 0.0488249734044075, -0.09780921787023544, 0.2407364696264267, 0.015147490426898003, 0.021804947406053543, -0.004053274169564247, -0.006455363240092993, -0.0834508016705513, 0.025374731048941612, 0.0341416671872139, -0.04948301985859871, -0.04021211341023445, -0.5473566055297852, 0.0317484475672245, 0.002281421096995473, -0.05781666561961174, 0.11925741285085678, -0.14910419285297394, -0.12983272969722748, -0.13112971186637878, 0.02703559771180153, -0.0168620515614748, 0.07610014826059341, -0.14305467903614044, -0.08850809186697006, 0.12652450799942017, -0.24311502277851105, -0.1656055748462677, 0.04649763181805611, 0.2158380150794983, 0.03711087256669998, -0.06805521249771118, 0.07115203887224197, 0.07901972532272339, 0.27541738748550415, 0.13547822833061218, -0.10203170031309128, 0.16141784191131592, -0.15585994720458984, 0.03975478187203407, 0.2780516743659973, 0.07924696058034897, 0.005573948845267296, 0.12967272102832794, -0.05048832669854164, -0.001017698086798191, 0.04195680096745491, -0.2257341742515564, -0.041519343852996826, -0.07440847903490067, 0.17761395871639252, 0.03450046107172966, -0.07608622312545776, 0.12129421532154083, 0.03167454153299332, 0.14702770113945007, 0.020158901810646057, 0.22431321442127228, 0.12237101048231125, -0.2346397042274475, 0.07595764845609665, -0.03419588506221771, 0.07963620871305466, 0.2006230503320694, 0.1252959966659546, -0.11083970963954926, 0.16085542738437653, -0.0015924934996291995, -0.060048505663871765, 0.13456925749778748, -0.15952971577644348, -0.022541869431734085, 0.08572561293840408, 0.5613557696342468, -0.06913041323423386, -0.06376351416110992, -0.009638159535825253, 0.1476559042930603, -0.046928469091653824, -0.14160944521427155, 0.0693148672580719, -0.055114876478910446, -0.003636552719399333, -0.3748636245727539, 0.18347714841365814, -0.05065197870135307, 0.09124528616666794, 0.0140991834923625, 0.15999241173267365, -0.008523339405655861, -0.2727534770965576, 0.2009362131357193, 0.0003679230867419392, 0.07892405241727829, -0.062366046011447906, -0.03901895135641098, 0.14467622339725494, -0.03187897428870201, 0.07808627188205719, 0.01800014078617096, 0.08370979875326157, -0.05930856615304947, -0.037019696086645126, -0.2593097984790802, -0.1904810667037964, 0.18479646742343903, -0.09244464337825775, -0.1541806012392044, -0.13079899549484253, -0.0350867360830307, 0.10454494506120682, 0.16585224866867065, -0.0688171312212944, -0.21922141313552856, -0.13222360610961914, 0.0789918601512909, -0.05949283018708229, -0.06971821933984756, 0.08574701845645905, 0.10633376985788345, 0.1804395169019699, 0.14968647062778473, -0.1357005387544632, 0.03185628354549408, -0.06578446179628372, 0.13395683467388153, 0.020892295986413956, 0.0303023848682642, 0.05104662850499153, -0.042012784630060196, 0.044203273952007294, 0.2453233152627945, 0.04257665574550629, -0.2042706161737442, -0.006823343690484762, -0.057562604546546936, -0.06346520781517029, -0.2652745842933655, 0.12826260924339294, 0.02110157161951065, -0.1404765248298645, 0.05871939659118652, 0.043925683945417404, -0.15659208595752716, -0.09182607382535934, -0.06040170043706894, -0.055178556591272354, -0.1748206615447998, 0.2872576415538788, 0.011198463849723339, 0.43944603204727173, 0.17958427965641022, -0.22047245502471924, 0.20112065970897675, 0.1798686385154724, -0.0883336067199707, -0.07126037031412125, 0.07292507588863373, 0.016574833542108536, -0.02011050283908844, 0.02698211930692196, 0.10589797794818878, 0.19293370842933655}, {0.04707122594118118, -0.015762824565172195, 0.04720694199204445, -0.2363797426223755, -0.17190338671207428, 0.17592015862464905, 0.3307284414768219, -0.07683536410331726, 0.05338413268327713, -0.0017447739373892546, 0.11381503939628601, -0.04878360778093338, -0.11057505011558533, 0.046182360500097275, -0.14135576784610748, 0.0834156721830368, -0.009687134996056557, 0.028254132717847824, -0.1214013397693634, -0.2330576777458191, 0.10100365430116653, -0.0321928970515728, -0.16431865096092224, -0.07300607115030289, 0.11156818270683289, 0.0017191244987770915, -0.09470634907484055, -0.12340603023767471, -0.013172204606235027, 0.01709531992673874, 0.18269240856170654, -0.11720988154411316, 0.13427942991256714, -0.013077589683234692, -0.22158947587013245, 0.12498737871646881, -0.1007433533668518, -0.15657754242420197, 0.03547783941030502, 0.025941872969269753, -0.0035486521665006876, -0.2736054062843323, -0.22684065997600555, -0.04952862113714218, -0.18105247616767883, 0.10410107672214508, -0.1183241754770279, 0.27374598383903503, -0.015394693240523338, 0.08343391865491867, -0.08244515210390091, -0.19028151035308838, 0.001947058830410242, -0.15921461582183838, 0.07206297665834427, 0.15314887464046478, -0.10822839289903641, 0.08046679198741913, -0.04005234315991402, 0.3059430718421936, 0.2786807715892792, 0.07979511469602585, -0.27189379930496216, -0.08604174107313156, -0.025632692500948906, 0.10396426916122437, 0.19137601554393768, -0.09692123532295227, 0.10708347707986832, 0.16995538771152496, -0.09416904300451279, 0.1633428931236267, -0.1053575649857521, -0.0007755403639748693, 0.13064861297607422, 0.09800493717193604, 0.11557996273040771, 0.13355526328086853, -0.2006978988647461, 0.24472679197788239, -0.11184903979301453, -0.011967542581260204, 0.005196192301809788, 0.18293976783752441, 0.20555271208286285, 0.004946521483361721, 0.01024166401475668, 0.010029885917901993, -0.056802522391080856, 0.1124391108751297, 0.047102365642786026, 0.05535588413476944, 0.07630449533462524, 0.10266897827386856, -0.09596846252679825, 0.22373786568641663, -0.0018013921799138188, -0.1046629473567009, -0.04658623784780502, -0.11843927949666977, -0.22896485030651093, 0.009877052158117294, 0.0187219250947237, -0.08750242739915848, 0.17077334225177765, -0.221633180975914, -0.011216854676604271, 0.11889811605215073, 0.060656845569610596, -0.05734478309750557, -0.006128661800175905, -0.07537782192230225, 0.04667205736041069, -0.19235950708389282, -0.09677735716104507, 0.18666096031665802, -0.02566036768257618, -0.041409581899642944, 0.029769079759716988, -0.06899094581604004, 0.11306611448526382, -0.016250204294919968, -0.06885390728712082, -0.09011954069137573, -0.28132447600364685, 0.04131618142127991, 0.042066868394613266, 0.07752051204442978, 0.15500906109809875, -0.01777791976928711, -0.0932735726237297, 0.06165599822998047, 0.07199389487504959, 0.018469713628292084, 0.021170204505324364, -0.04339229315519333, -0.00858304649591446, 0.11186323314905167, -0.052513256669044495, 0.17209534347057343, -0.23450198769569397, -0.005938564892858267, -0.011571971699595451, 0.06235087662935257, 0.06018413230776787, 0.1476903110742569, -0.03859051316976547, -0.006078863516449928, 0.043932169675827026, -0.24057582020759583, 0.23067818582057953, 0.11777883023023605, 0.12539418041706085, -0.03768802061676979, 0.13855904340744019, -0.06980013847351074, 0.008489185012876987, 0.006469221785664558, -0.043735966086387634, 0.048274699598550797, 0.1597011387348175, -0.024951238185167313, -0.09762672334909439, -0.08488170802593231, 0.01845216378569603, -0.010593644343316555, 0.1577167510986328, 0.13278189301490784, 0.10874377936124802, -0.06827650219202042, -0.032391782850027084, -0.11298447847366333, 0.015277217142283916, 0.08266352117061615, -0.1580202579498291, -0.00392085500061512, 0.14492380619049072, 0.11302731186151505, -0.007277066353708506, 0.10063760727643967, 0.12423022091388702, -0.03548410162329674, 0.02697046846151352, -0.031054526567459106, -0.09954418987035751, -0.10092739015817642, 0.12068082392215729, -0.14618054032325745, 0.12721824645996094, -0.04068111628293991, 0.007698607165366411, -0.12941381335258484, -0.23887431621551514, -0.15513844788074493, 0.11695130914449692, 0.16771478950977325, 0.04129190742969513, -0.12237825989723206, -0.010585426352918148, 0.2007591426372528, -0.08938627690076828, -0.09634726494550705, 0.210257887840271, 0.007821531035006046, 0.13997291028499603, 0.23909246921539307, -0.5444818735122681, 0.06745368242263794, 0.03233069181442261, 0.09584326297044754, -0.06543638557195663, -0.060803670436143875, 0.09494016319513321, -0.3791011869907379, 0.04581901431083679, -0.200251966714859, 0.030377738177776337, -0.03317707031965256, -0.04907466471195221, 0.09941662102937698, -0.19591635465621948, -0.08002903312444687, 0.07742217183113098, -0.0584939606487751, -0.1486532837152481, 0.01680874265730381, 0.02145206369459629, -0.09617134928703308, -0.056087855249643326, 0.2163456529378891, 0.05109284073114395, 0.08974619209766388, -0.09737454354763031, -0.09525993466377258, 0.022681482136249542, 0.24074366688728333, -0.05137079581618309, -0.1657860428094864, 0.18987421691417694, 0.07031279802322388, 0.12785999476909637, -0.08388026803731918, 0.1136322095990181, -0.057595156133174896, -0.14758656919002533, 0.014725025743246078, 0.04958618804812431, -0.011301607824862003, 0.19653105735778809, -0.08989905565977097, 0.1511770784854889, 0.04433620721101761, 0.07705973833799362, -0.18498556315898895, 0.006254048552364111, -0.028547145426273346}, {-0.17659412324428558, -0.10593067854642868, 0.09872528165578842, 0.2122480422258377, -0.005305249243974686, 0.0640997514128685, -0.10723530501127243, 0.04307970777153969, -0.01357210986316204, -0.06452617794275284, 0.16567830741405487, -0.09009705483913422, 0.018196024000644684, -0.04429427906870842, 0.046546820551157, -0.0213936660438776, -0.23546192049980164, 0.10129673779010773, -0.03554658964276314, 0.03543476760387421, -0.03746698424220085, 0.004122787155210972, -0.030944017693400383, -0.04788796976208687, 0.07461824268102646, -0.013561354018747807, -0.012935926206409931, 0.12198247760534286, -0.13120579719543457, -0.05354147031903267, -0.2908622920513153, -0.016938727349042892, 0.20339015126228333, -0.009012661874294281, -0.0027064739260822535, 0.12858596444129944, 0.12296543270349503, 0.05889630317687988, 0.2087763100862503, 0.019995691254734993, -0.1276513785123825, 0.02815921977162361, 0.0039343638345599174, -0.09893307834863663, 0.15273715555667877, 0.035971056669950485, 0.022295301780104637, 0.06465278565883636, -0.13728801906108856, -0.1894177496433258, 0.09299279004335403, -0.05941313877701759, -0.032908786088228226, 0.19370751082897186, 0.11513037234544754, -0.030254356563091278, -0.0667843222618103, -0.171161487698555, 0.014534841291606426, 0.05695265159010887, -0.13578911125659943, -0.008967682719230652, -0.08157342672348022, 0.25295406579971313, -0.05776603892445564, 0.0199069045484066, 0.0837354063987732, -0.09613287448883057, -0.06860842555761337, -0.24638384580612183, -0.13020573556423187, 0.08342048525810242, 0.2890620827674866, -0.007848790846765041, -0.03368833661079407, -0.022263016551733017, 0.04510251805186272, -0.06135422736406326, -0.07996638119220734, 0.027826350182294846, 0.15384531021118164, 0.03395013511180878, 0.10296513140201569, -0.03978649154305458, 0.010102611035108566, 0.03552604466676712, 0.12988488376140594, 0.159446582198143, -0.22244426608085632, -0.027183981612324715, -0.05126417800784111, 0.011828904040157795, -0.024044588208198547, 0.09385942667722702, -0.10446727275848389, -0.33234021067619324, -0.13208727538585663, -0.00034222917747683823, 0.017305724322795868, -0.09985391795635223, -0.17757456004619598, 0.014708499424159527, -0.13108476996421814, -0.00506235659122467, 0.27310827374458313, 0.07995451986789703, -0.03532366827130318, -0.02096370793879032, -0.0954035148024559, -0.01931072399020195, 0.021482836455106735, 0.008965331129729748, 0.12755295634269714, -0.039934415370225906, 0.008816535584628582, 0.00039250330883078277, 0.13852329552173615, 0.09167170524597168, 0.0015837213722988963, 0.004316647071391344, -0.21278783679008484, -0.07213135808706284, -0.07855398207902908, -0.11640623211860657, -0.024218352511525154, 0.02122405916452408, 0.006973241455852985, -0.09070877730846405, 0.14109860360622406, -0.013879981823265553, 0.20715466141700745, 0.17419718205928802, -0.24672381579875946, -0.14652982354164124, 0.016629740595817566, 0.08461381494998932, 0.03686755523085594, 0.08809980750083923, 0.04310464859008789, 0.2837366461753845, 0.1288262903690338, 0.16771486401557922, -0.06480739265680313, 0.08908185362815857, -0.06405379623174667, 0.08511832356452942, -0.09530901908874512, -0.028704416006803513, 0.02836139127612114, 0.11848635971546173, 0.1372210532426834, -0.07494737952947617, -0.09326096624135971, 0.011939749121665955, -0.049198854714632034, -0.15545566380023956, -0.13874347507953644, 0.19412268698215485, 0.04969952628016472, -0.01642332784831524, 0.07107890397310257, 0.060545310378074646, 0.2447301149368286, -0.016036786139011383, -0.05353398621082306, -0.09117575734853745, 0.013475953601300716, -0.09988477826118469, -0.09004882723093033, -0.21170751750469208, 0.07760649174451828, 0.20009782910346985, 0.2034955620765686, 0.04450567439198494, 0.11304893344640732, 0.11280116438865662, 0.06352192163467407, 0.07467429339885712, -0.012717878445982933, 0.13511642813682556, -0.06022675707936287, -0.13885267078876495, -0.004458112642168999, -0.03045204095542431, -0.10129350423812866, 0.048978060483932495, 0.2807551324367523, 0.14648954570293427, -0.10825598239898682, -0.10133527219295502, -0.038234736770391464, -0.09096236526966095, 0.1655140221118927, 0.08904249966144562, 0.21402892470359802, -0.022088542580604553, 0.10937956720590591, -0.1979023963212967, -0.12184453755617142, 0.06092379614710808, -0.18641474843025208, 0.05094494670629501, -0.12483834475278854, 0.03380829468369484, -0.169040709733963, -0.22673986852169037, 0.08057097345590591, 0.07465406507253647, 0.0006835191161371768, -0.055153485387563705, -0.005624189507216215, 0.23252803087234497, 0.06591515988111496, -0.061179883778095245, -0.09507693350315094, 0.1131301075220108, 0.1314234733581543, 0.07071148604154587, -0.028157323598861694, 0.0655510425567627, -0.32349342107772827, -0.1742578148841858, 0.10033528506755829, 0.2595181167125702, 0.17637795209884644, 0.25098350644111633, -0.06878446787595749, -0.05600620433688164, -0.02907225303351879, 0.009718833491206169, 0.17511804401874542, 0.0027979493606835604, 0.14938193559646606, 0.13689738512039185, -0.07326646894216537, 0.16528832912445068, -0.02030746452510357, 0.32666444778442383, -0.162196546792984, -0.1401786506175995, -0.2309829443693161, -0.12496360391378403, -0.10834470391273499, -0.15193618834018707, -0.02596663497388363, -0.13309626281261444, -0.10758160054683685, -0.07091774046421051, -0.17173194885253906, 0.20441310107707977, -0.1215447261929512, -0.022481882944703102, -0.03945186734199524, 0.14220687747001648, -0.09203135967254639, 0.06366395950317383}, {-0.1540202498435974, -0.053326088935136795, -0.14113305509090424, 0.11448095738887787, 0.1702127605676651, 0.020136529579758644, -0.33457043766975403, -0.0904095470905304, 0.0210657250136137, -0.2787800431251526, 0.16479839384555817, 0.030199168249964714, 0.023927025496959686, 0.16431564092636108, -0.10013149678707123, 0.06652992218732834, 0.12927158176898956, 0.033762119710445404, -0.1045115515589714, 0.05315753445029259, -0.05344810336828232, -0.07899292558431625, 0.03321895748376846, 0.1486956924200058, 0.1282920241355896, -0.29934969544410706, -0.03619762510061264, -0.10100146383047104, 0.29011958837509155, 0.0515621080994606, 0.14386413991451263, 0.13082358241081238, -0.04658111557364464, 0.052604008466005325, -0.05651135742664337, 0.036885231733322144, 0.047370102256536484, -0.050973545759916306, -0.08372320979833603, 0.03147115185856819, -0.1026218980550766, -0.0005339168128557503, -0.16217586398124695, -0.06995706260204315, -0.4889119565486908, 0.2368621677160263, -0.06818974763154984, -0.041101668030023575, 0.11584495007991791, -0.07252801954746246, 0.09298143535852432, 0.03252619132399559, 0.23492549359798431, -0.2187163531780243, 0.03601924702525139, -0.19632932543754578, -0.02854326367378235, -0.007807732094079256, -0.18552221357822418, -0.08522719144821167, 0.11668554693460464, 0.18501967191696167, -0.01197634357959032, 0.11092904210090637, 0.1314574033021927, 0.1250278353691101, 0.1899043470621109, -0.23028787970542908, -0.0692872628569603, -0.11528018116950989, -0.04770985245704651, 0.22625528275966644, 0.3235754370689392, 0.1445070058107376, -0.3322557508945465, 0.17824265360832214, 0.007439338602125645, -0.25774699449539185, -0.19058264791965485, 0.3588866889476776, 0.2624203860759735, 0.12867605686187744, -0.3793438673019409, -0.10786696523427963, 0.37709200382232666, -0.2101002186536789, -0.07052737474441528, 0.19871209561824799, 0.20750662684440613, -0.08492674678564072, 0.19484174251556396, -0.24342738091945648, 0.10283172130584717, 0.014368204399943352, -0.017952468246221542, 0.14148657023906708, 0.0489363893866539, -0.06540850549936295, -0.0417051799595356, 0.27370303869247437, -0.17373545467853546, 0.09685803204774857, -0.30245864391326904, 0.35911116003990173, -0.02905360981822014, 0.3133060038089752, 0.03181101381778717, -0.00578981963917613, -0.17281945049762726, 0.14519305527210236, -0.21457871794700623, 0.03422608971595764, 0.0452776737511158, 0.0969897210597992, 0.01443022582679987, -0.07676488906145096, 0.09200221300125122, -0.05674966052174568, 0.06176448240876198, 0.008617197163403034, 0.26222899556159973, -0.02112937718629837, -0.22827047109603882, 0.20763389766216278, -0.0799759030342102, -0.1670243889093399, 0.2603721618652344, 0.23088230192661285, -0.33762791752815247, -0.14095522463321686, 0.10100451856851578, 0.17869193851947784, -0.2618144154548645, 0.059170644730329514, 0.023010166361927986, 0.22167789936065674, -0.1972123235464096, -0.2104109525680542, -0.24575945734977722, 0.23781469464302063, -0.09494489431381226, -0.1220875084400177, -0.1587030589580536, -0.049301765859127045, 0.0805865153670311, 0.2121952623128891, -0.07879884541034698, 0.031516820192337036, 0.01038823276758194, -0.20004232227802277, 0.037873320281505585, 0.07501034438610077, 0.19575859606266022, 0.13966815173625946, 0.05384393036365509, -0.01220636535435915, -0.13299643993377686, -0.08088140189647675, 0.004163800738751888, -0.1738647073507309, 0.03904055804014206, -0.24481874704360962, 0.22874769568443298, 0.07610396295785904, -0.06457895040512085, 0.07796682417392731, -0.004582738503813744, -0.03799249976873398, 0.06934725493192673, 0.09928151220083237, -0.04699110612273216, 0.13527779281139374, 0.09005725383758545, -0.10157231986522675, -0.07704048603773117, -0.15537171065807343, 0.05771079659461975, 0.03187309950590134, 0.012554874643683434, -0.27726873755455017, -0.0046499245800077915, -0.024133168160915375, -0.24668358266353607, -0.31107857823371887, -0.009467442519962788, 0.08264989405870438, 0.16816098988056183, 0.04718031734228134, -0.11747485399246216, 0.07827247679233551, -0.006518391892313957, -0.20859406888484955, -0.15659727156162262, 0.13625796139240265, 0.16303355991840363, 0.12282141298055649, -0.066016785800457, 0.2369948923587799, -0.06293901056051254, -0.3589867353439331, 0.1959294229745865, -0.04624442011117935, 0.024432940408587456, 0.2730589807033539, -0.1274453103542328, 0.1560867726802826, 0.030276168137788773, 0.3923812210559845, 0.060803044587373734, -0.011696412228047848, -0.07248715311288834, -0.037709664553403854, -0.21297384798526764, -0.09116040915250778, 0.21431902050971985, 0.08381608128547668, 0.07400710135698318, -0.13345341384410858, 0.10577820241451263, -0.03929290547966957, 0.010848676785826683, -0.0787644162774086, -0.10135840624570847, 0.20396745204925537, 0.1012294813990593, 0.2839064598083496, 0.05728127434849739, -0.20560237765312195, 0.23729544878005981, 0.15947416424751282, -0.002848035655915737, -0.0808175727725029, 0.010866908356547356, 0.10491154342889786, -0.34441250562667847, 0.28134220838546753, -0.07026290893554688, 0.08217808604240417, 0.0023176439572125673, -0.11627249419689178, -0.008421062491834164, -0.06260593980550766, 0.0940113365650177, -0.03831813484430313, 0.03174861520528793, -0.058171436190605164, 0.004943249747157097, -0.11722920089960098, -0.03051712177693844, 0.27469733357429504, 0.07417623698711395, -0.2468547523021698, -0.14485028386116028, -0.18482592701911926, 0.09918131679296494, 0.25800371170043945}, {-0.12081173807382584, 0.06045529246330261, 0.1594187319278717, -0.0058990586549043655, -0.018274206668138504, -0.02513648197054863, 0.03395913168787956, 0.05239030718803406, 0.12418460845947266, -0.031232329085469246, 0.10788268595933914, 0.4066911041736603, 0.23126395046710968, 0.08258236199617386, 0.047679413110017776, 0.11130955070257187, 0.16654829680919647, -0.11124876886606216, 0.10909049957990646, 0.14586332440376282, -0.17834177613258362, 0.20880527794361115, 0.01673128642141819, -0.1349853277206421, -0.0528169609606266, 0.10087314993143082, -0.09686540812253952, 0.23547933995723724, -0.2631959319114685, -0.17529752850532532, -0.16423043608665466, -0.025437962263822556, 0.021390849724411964, -0.14038386940956116, 0.05472242832183838, 0.15423054993152618, 0.007014753762632608, 0.11683118343353271, -0.05602572485804558, -0.1862172782421112, -0.15518657863140106, 0.05128119885921478, -0.027171997353434563, -0.06660404056310654, 0.01657550036907196, -0.17579247057437897, -0.027690988034009933, -0.18082794547080994, 0.11178217083215714, -0.2669718861579895, 0.1184331551194191, -0.004420772194862366, -0.10984187573194504, 0.16580457985401154, -0.054406553506851196, 0.14422090351581573, -0.0820717141032219, 0.013174193911254406, -0.07912217825651169, -0.24017225205898285, 0.03341415897011757, -0.32152771949768066, -0.06016552075743675, 0.05315805599093437, -0.01224062591791153, 0.1094922348856926, -0.09754863381385803, -0.16123688220977783, -0.2717576026916504, 0.08429750800132751, -0.003563351696357131, 0.08974859863519669, 0.2148263305425644, -0.10905413329601288, 0.14701759815216064, 0.10673124343156815, 0.18668876588344574, -0.01754014939069748, -0.14827725291252136, -0.13759849965572357, 0.013596178032457829, 0.04534124583005905, 0.07259781658649445, -0.06124528497457504, 0.02131826803088188, -0.08589161187410355, 0.02077663689851761, -0.14304491877555847, 0.06072815880179405, 0.1328088343143463, -0.03246559575200081, -0.19037410616874695, -0.2542848587036133, 0.08037823438644409, 0.03955046460032463, -0.21309888362884521, 0.09586220234632492, 0.3570428192615509, 0.1941569447517395, 0.019852299243211746, 0.07878435403108597, -0.03573279827833176, -0.214369535446167, -0.12171459943056107, 0.058416206389665604, 0.0993134006857872, 0.15888486802577972, -0.08744991570711136, 0.14048761129379272, -0.1299017071723938, 0.013028481975197792, 0.0001878512412076816, -0.021798383444547653, -0.008777548559010029, -0.08167054504156113, 0.34987252950668335, -0.10416279733181, 0.1232515275478363, 0.04093245789408684, 0.07823000103235245, -0.17383258044719696, 0.06988202035427094, 0.006472575478255749, 0.08812948316335678, 0.11093422025442123, -0.12586195766925812, -0.011947997845709324, 0.2274990677833557, 0.009249511174857616, -0.040217652916908264, -0.09206805378198624, -0.06644850224256516, -0.1684780716896057, 0.019917219877243042, 0.13670967519283295, 0.05073963478207588, -0.17158128321170807, -0.05639120191335678, 0.1657753735780716, -0.07937651127576828, 0.11029258370399475, 0.024911843240261078, -0.039021529257297516, 0.10697633773088455, 0.15680137276649475, -0.1004561260342598, 0.0664568617939949, -0.022432437166571617, -0.10565102845430374, 0.011302663013339043, 0.016154654324054718, -0.005351473111659288, 0.0038177340757101774, 0.07682600617408752, 0.004152465146034956, -0.04353681951761246, -0.05721721425652504, 0.013019981794059277, 0.036186475306749344, -0.1299116164445877, -0.07605069875717163, 0.02640877477824688, 0.06275554746389389, 0.06360413134098053, -0.135603666305542, -0.0005086348974145949, -0.06702793389558792, 0.005956884007900953, -0.028286922723054886, 0.1424962878227234, -0.06690387427806854, 0.07013317197561264, 0.013792191632091999, 0.1669226884841919, 0.2533698081970215, 0.19248539209365845, -0.11440872400999069, 0.08750198781490326, -0.0810268223285675, 0.085550457239151, 0.1778809279203415, 0.5645382404327393, 0.12413039803504944, -0.08159118890762329, -0.054859329015016556, 0.14134638011455536, -0.04220066964626312, 0.12425367534160614, 0.018231818452477455, -0.07539814710617065, -0.091501384973526, 0.0027464511804282665, 0.07011771202087402, 0.00643132533878088, -0.05595582351088524, 0.09671244770288467, 0.03092072531580925, 0.07134983688592911, -0.14478130638599396, -0.024324096739292145, -0.09991327673196793, 0.02916211076080799, 0.08558137714862823, -0.02755144238471985, 0.10642274469137192, -0.08525034785270691, 0.1262955516576767, -0.07292397320270538, -0.0064737373031675816, -0.015968848019838333, 0.10233183205127716, 0.08031683415174484, 0.09460654109716415, 0.160403311252594, 0.01855429634451866, -0.12814760208129883, 0.0762350857257843, -0.06172395497560501, 0.1209447979927063, 0.020335519686341286, 0.1764487773180008, 0.06298478692770004, 0.025321856141090393, 0.015033619478344917, -0.02344569005072117, 0.08085578680038452, -0.0012186442036181688, -0.06722331047058105, -0.046985458582639694, 0.11757553368806839, -0.11078234016895294, -0.20174339413642883, 0.18559283018112183, 0.31324303150177, 0.047623444348573685, 0.1964157372713089, 0.15446606278419495, 0.09703310579061508, -0.10067305713891983, -0.19176585972309113, 0.1880446970462799, -0.06349427253007889, -0.15795786678791046, -0.023807505145668983, 0.07230562716722488, -0.2870185673236847, 0.04794299229979515, -0.13202959299087524, -0.12251366674900055, -0.32864344120025635, -0.0015482959570363164, 0.045934244990348816, -0.12382359057664871, 0.18891872465610504, 0.07593321800231934, 0.07453872263431549}, {-0.16676642000675201, -0.021571476012468338, 0.04358820617198944, 0.023196639493107796, 0.09589133411645889, -0.011566358618438244, 0.3016580641269684, -0.08752907812595367, -0.1435956358909607, -0.062298912554979324, -0.1976519674062729, 0.24406598508358002, -0.054128967225551605, -0.11473342031240463, -0.21933284401893616, 0.09242305159568787, 0.06696075946092606, 0.12789396941661835, 0.05875887721776962, 0.15342916548252106, 0.19764715433120728, -0.03582584485411644, -0.15362600982189178, 0.030917534604668617, 0.15778563916683197, 0.125249445438385, 0.006856608670204878, -0.10613926500082016, -0.2659134268760681, -0.40493425726890564, 0.1966000199317932, 0.17512015998363495, -0.16373156011104584, 0.11678476631641388, -0.09020955860614777, -0.13400834798812866, -0.034106042236089706, -0.04058400169014931, 0.05004522204399109, 0.1482793092727661, 0.01015438325703144, -0.012227259576320648, 0.23967720568180084, -0.19997158646583557, -0.07982536405324936, -0.06936166435480118, 0.040419481694698334, 0.07824863493442535, -0.050123851746320724, 0.0013944857055321336, 0.15609706938266754, -0.005731202661991119, 0.048756394535303116, -0.3193439543247223, -0.05650833994150162, 0.010181859135627747, 0.24448156356811523, -0.046754054725170135, -0.03355410695075989, 0.2924228608608246, 0.033225167542696, 0.13776668906211853, 0.18554013967514038, -0.05880296975374222, -0.08856002241373062, -0.10376322269439697, -0.06333580613136292, 0.1946745663881302, -0.1589670032262802, -0.0033128175418823957, -0.1612202525138855, -0.039027608931064606, 0.11094105243682861, 0.07628727704286575, 0.13086432218551636, 0.07384315133094788, -0.08701076358556747, -0.12739069759845734, 0.03750485181808472, -0.06268008053302765, -0.02877618744969368, -0.021468060091137886, 0.08412426710128784, -0.19121824204921722, -0.01835796609520912, 0.07485076040029526, 0.021841328591108322, 0.009566607885062695, -0.09225351363420486, -0.0709802508354187, -0.15161268413066864, 0.018501142039895058, 0.17300373315811157, 0.041995130479335785, -0.07349735498428345, 0.09158176183700562, 0.07237155735492706, -0.04485919326543808, -0.10286717116832733, 0.022104792296886444, 0.027196258306503296, -0.03004545159637928, 0.035401489585638046, -0.17071552574634552, -0.26827573776245117, -0.4084269106388092, -0.002551231998950243, -0.12209423631429672, -0.036314088851213455, 0.1683579683303833, 0.10581304877996445, -0.045544903725385666, 0.061618078500032425, -0.20660391449928284, 0.19324880838394165, -0.0859435573220253, 0.027471136301755905, -0.08289048820734024, -0.06549103558063507, 0.041092973202466965, 0.0985858142375946, 0.06576517224311829, 0.04369381442666054, -0.10266842693090439, 0.008150557987391949, -0.0532747246325016, 0.056064803153276443, 0.13225261867046356, -0.02986905537545681, -0.1279170662164688, 0.1915464550256729, 0.08290176093578339, -0.10751684755086899, -0.14436137676239014, 0.012117665261030197, -0.0022436128929257393, -0.06127260997891426, -0.19353562593460083, 0.11779002100229263, -0.29283273220062256, 0.07848355174064636, -0.05290756747126579, -0.10829183459281921, 0.15484747290611267, -0.1524902582168579, 0.05738989636301994, 0.032067663967609406, 0.029731158167123795, -0.03154263645410538, 0.025197096168994904, -0.1005936786532402, 0.07509466260671616, -0.12137959897518158, -0.009120752103626728, -0.02792336605489254, 0.18389369547367096, 0.03608296066522598, -0.29004237055778503, 0.09335684776306152, 0.2394532710313797, 0.03683485463261604, -0.027028527110815048, -0.050460055470466614, 0.006534822750836611, 0.0031862470787018538, 0.006606060545891523, 0.1983197033405304, -0.017879877239465714, -0.0035536803770810366, 0.053140297532081604, 0.18133379518985748, -0.10981159657239914, 0.03666916489601135, -0.049645643681287766, 0.11179553717374802, -0.10264620184898376, -0.13077569007873535, -0.06725329160690308, -0.09584514051675797, 0.1064605861902237, -0.006128028966486454, 0.10519832372665405, 0.053156767040491104, 0.0259598046541214, 0.1204395443201065, -0.061881568282842636, -0.17032836377620697, -0.06458957493305206, 0.06554143875837326, -0.13899843394756317, -0.09467338025569916, -0.11399085074663162, -0.006484602577984333, 0.1623590588569641, -0.07205810397863388, -0.048345234245061874, -0.07149355858564377, -0.17874081432819366, -0.003608396276831627, -0.04675673320889473, -0.32831868529319763, -0.06463882327079773, 0.14680145680904388, -0.0767158567905426, -0.007712221704423428, 0.0958038792014122, -0.25901103019714355, 0.06521420180797577, -0.0033907003235071898, -0.23599861562252045, 0.12935830652713776, -0.07107048481702805, -0.0725429505109787, 0.13442306220531464, -0.06666896492242813, -0.10095969587564468, 0.13765256106853485, -0.06101680174469948, 0.05921735614538193, -0.141586571931839, 0.05756189674139023, 0.010826277546584606, -0.04915502667427063, -0.08672517538070679, 0.1430612951517105, -0.084108367562294, 0.10124095529317856, 0.08437254279851913, 0.13639800250530243, -0.055032920092344284, -0.39335906505584717, -0.17536142468452454, -0.0688772201538086, -0.06621343642473221, -0.11916135251522064, -0.14346803724765778, -0.05542154982686043, 0.22865326702594757, -0.1031975969672203, -0.03140150383114815, 0.07790103554725647, 0.09857842326164246, 0.15074197947978973, -0.008621386252343655, 0.0935092493891716, 0.014648029580712318, -0.05942826345562935, -0.07055498659610748, -0.02738204598426819, -0.12330229580402374, -0.044381242245435715, 0.13541527092456818, 0.028699195012450218, 0.17028667032718658, -0.21314379572868347, -0.18788953125476837}, {0.0038650226779282093, -0.46871262788772583, 0.0386810377240181, -0.057525984942913055, -0.23601099848747253, -0.09980333596467972, -0.18702326714992523, -0.03658498451113701, 0.09263977408409119, 0.11720834672451019, -0.018768511712551117, -0.25490254163742065, -0.09268324077129364, 0.016128001734614372, -0.19830508530139923, 0.2970685064792633, 0.1287367045879364, -0.1820582151412964, 0.18406833708286285, -0.21766439080238342, -0.1381816267967224, 0.004803917836397886, 0.08077643066644669, -0.06304270029067993, -0.04502324387431145, -0.10339130461215973, 0.044020410627126694, -0.1896907091140747, -0.13083486258983612, 0.16342371702194214, 0.04245634004473686, -0.12602552771568298, 0.06469868868589401, 0.01306242123246193, 0.10313420742750168, -0.21940173208713531, 0.14438804984092712, 0.0895257219672203, 0.03057188354432583, -0.032949257642030716, -0.09031016379594803, 0.09401525557041168, 0.06914026290178299, 0.07113343477249146, -0.16076737642288208, 0.2005661576986313, -0.029564592987298965, 0.02841310016810894, -0.03549212962388992, -0.028932668268680573, 0.18659651279449463, -0.13709646463394165, -0.03316040709614754, 0.14453670382499695, -0.00016830449749249965, 0.1263550966978073, -0.08648687601089478, 0.07615458220243454, -0.04803312197327614, 0.10674475878477097, -0.08549758046865463, 0.00035326145007275045, -0.21254302561283112, 0.06598865985870361, 0.03218317776918411, -0.1704871654510498, 0.08484983444213867, -0.03584880009293556, -0.22505342960357666, 0.1113000363111496, 0.052925772964954376, 0.21793769299983978, -0.024882156401872635, 0.1471937894821167, -0.5110822916030884, 0.00961838848888874, -0.0785999447107315, 0.2541821599006653, 0.004270171280950308, 0.11702064424753189, 0.02442314475774765, 0.04639778286218643, -0.1119869202375412, -0.04012836143374443, -0.07499536126852036, -0.16320611536502838, -0.07789776474237442, 0.1310379058122635, 0.025087183341383934, 0.2666771113872528, 0.09127750992774963, -0.1130765900015831, -0.09718358516693115, 0.12078870832920074, -0.0370776504278183, 0.006518959067761898, -0.0016593456966802478, 0.015851924195885658, 0.18784615397453308, -0.3115619421005249, -0.14813481271266937, -0.05376157537102699, 0.05809088796377182, 0.06217342987656593, 0.0448707714676857, -0.36961719393730164, -0.08271483331918716, -0.09958142042160034, -0.16035398840904236, -0.29399415850639343, -0.018690582364797592, 0.00030064021120779216, 0.11840064823627472, -0.07675155997276306, -0.11571021378040314, -0.19172357022762299, -0.135629341006279, -0.0770634114742279, 0.13345672190189362, 0.1629500687122345, 0.030567141249775887, -0.042666543275117874, 0.03417406603693962, -0.11763694882392883, -0.11794326454401016, 0.16138744354248047, 0.06137219816446304, 0.022004524245858192, 0.051488712430000305, 0.15652300417423248, 0.039933089166879654, 0.09670741111040115, 0.20034503936767578, -0.11672041565179825, 0.12417346984148026, -0.01713237538933754, 0.15892353653907776, 0.19130045175552368, 0.05223767086863518, 0.1448979675769806, 0.03198934346437454, 0.03451964259147644, 0.03126062452793121, 0.0641111209988594, -0.048497334122657776, -0.09602408856153488, 0.0550956204533577, -0.31423601508140564, 0.17724235355854034, -0.04109610617160797, 0.03291776031255722, -0.15456758439540863, 0.21673551201820374, -0.11776409298181534, 0.19943048059940338, 0.08560991287231445, 0.0235151220113039, 0.08065363764762878, -0.06235089898109436, -0.1824280321598053, -0.1235080361366272, -0.2817148268222809, 0.14058201014995575, -0.18004782497882843, -0.03207818791270256, 0.11523513495922089, 0.05068740248680115, -0.2821134626865387, -0.10769717395305634, 0.035831280052661896, -0.10418859124183655, 0.030700311064720154, -0.019578196108341217, -0.16299039125442505, 0.04154760017991066, -0.05885996297001839, 0.01020130142569542, -0.08383165299892426, -0.04739684239029884, -0.0392550528049469, -0.11143144965171814, -0.31379544734954834, 0.05486184358596802, -0.017817925661802292, 0.09332650899887085, -0.1102452501654625, -0.18544699251651764, 0.15811675786972046, 0.08922600001096725, 0.04207964986562729, -0.10912179201841354, 0.07502661645412445, 0.31070029735565186, 0.25060728192329407, -0.002108771586790681, -0.1483650505542755, -0.1552732139825821, -0.26079991459846497, 0.0607108436524868, -0.10187109559774399, 0.20442292094230652, 0.25470760464668274, -0.12836511433124542, 0.271988183259964, 0.05610214173793793, -0.05907271429896355, -0.06259019672870636, 0.05647330358624458, -0.09084607660770416, 0.016717130318284035, -0.04468631371855736, -0.19473347067832947, -0.1905333250761032, -0.010723080486059189, 0.16585253179073334, 0.14980296790599823, 0.03315389156341553, -0.0704088807106018, 0.09470424801111221, -0.00845237448811531, -0.26241302490234375, -0.1156717985868454, 0.2861151397228241, -0.19678950309753418, -0.10364729911088943, 0.013886790722608566, -0.01893005333840847, -0.0851794183254242, -0.240425705909729, 0.0877424106001854, -0.057290561497211456, 0.034605998545885086, -0.20250080525875092, -0.19641755521297455, -0.06926297396421432, 0.0045213038101792336, -0.4473261535167694, 0.21236930787563324, 0.23029693961143494, -0.2745930850505829, -0.2346435934305191, 0.04435260593891144, 0.17502938210964203, -0.008163153193891048, 0.010321876965463161, -0.021541086956858635, 0.18665747344493866, -0.14126074314117432, 0.1174360066652298, 0.1719202995300293, 0.05913934111595154, -0.33935168385505676, 0.037042953073978424, 0.2570851147174835, 0.009385533630847931, -0.038247913122177124}, {-0.1108512282371521, -0.10303960740566254, -0.019538406282663345, -0.10403839498758316, -0.02140466310083866, -0.15704241394996643, -0.10410897433757782, 0.13597775995731354, -0.06267871707677841, -0.0010965511901304126, 0.19031715393066406, -0.2503759264945984, -0.09135881811380386, -0.048503611236810684, -0.0751420333981514, 0.045850448310375214, -0.22920668125152588, -0.09387550503015518, 0.1368158757686615, 0.08960511535406113, 0.12500585615634918, 0.06105152890086174, 0.04195278137922287, 0.08358737826347351, 0.018105659633874893, 0.3460122346878052, -0.0020560487173497677, -0.020585859194397926, 0.03257257863879204, 0.08531085401773453, -0.14399631321430206, 0.01778746210038662, 0.20014704763889313, 0.13860563933849335, 0.17293912172317505, -0.04636391997337341, 0.18545404076576233, 0.09890498965978622, 0.027369951829314232, 0.030842946842312813, 0.2048974335193634, -0.3136149048805237, -0.1049242839217186, 0.10349667072296143, 0.043495938181877136, -0.009068439714610577, 0.11656144261360168, 0.09941407293081284, 0.026080181822180748, -0.04812387749552727, 0.025464963167905807, 0.09285571426153183, -0.1975671797990799, 0.11304755508899689, 0.07379812747240067, 0.1633618324995041, -0.13214030861854553, 0.046833984553813934, 0.09075669944286346, 0.1437986046075821, -0.21199561655521393, -0.04445663094520569, -0.012613390572369099, 0.16566790640354156, 0.10046489536762238, 0.12420420348644257, -0.02718408964574337, 0.04504552111029625, 0.18403252959251404, 0.06719299405813217, -0.0031941549386829138, 0.025386670604348183, 0.06419651210308075, 0.0449320450425148, -0.26692914962768555, 0.0938941240310669, 0.148943692445755, -0.04468442499637604, 0.16551589965820312, -0.0022097546607255936, 0.1028379276394844, 0.022172605618834496, 0.10302069783210754, -0.12758882343769073, -0.029334967955946922, 0.04393080994486809, -0.07399371266365051, 0.05570573732256889, -0.03813031315803528, 0.13777494430541992, 0.04012142866849899, -0.11707678437232971, 0.013669481500983238, -0.014397130347788334, 0.09493985772132874, 0.010639914311468601, -0.01774861104786396, 0.05322274938225746, 0.06579788029193878, -0.16721613705158234, -0.05124256759881973, 0.047966428101062775, -0.03759213164448738, -0.09897769242525101, 0.2519558370113373, -0.16773711144924164, -0.054377324879169464, 0.06418842077255249, 0.231417715549469, 0.05380009859800339, -0.04705731198191643, 0.1298818439245224, -0.01957886293530464, -0.007830921560525894, -0.2795320451259613, 0.04213239625096321, -0.07578280568122864, -0.04828175529837608, -0.1975044459104538, -0.030705304816365242, -0.025866718962788582, 0.09292343258857727, 0.10525747388601303, -0.19684284925460815, 0.06998399645090103, -0.15571333467960358, 0.02804369479417801, 0.16844986379146576, 0.02187349833548069, -0.16324859857559204, -0.13250769674777985, -0.007615773938596249, -0.17724823951721191, -0.08537256717681885, 0.06196538358926773, 0.29648557305336, 0.1679936647415161, -0.11316113919019699, 0.1753486841917038, 0.10409029573202133, -0.01346637960523367, 0.007556796073913574, 0.16142146289348602, 0.004956220742315054, 0.004735417198389769, -0.07419031113386154, 0.00587803591042757, 0.05233193188905716, 0.0259736068546772, 0.10365001112222672, -0.131522998213768, -0.05860395357012749, -0.028842750936746597, 0.07197579741477966, 0.01089082658290863, -0.1530044972896576, 0.1643643081188202, 0.21358631551265717, -0.0815262570977211, -0.26135432720184326, 0.036796681582927704, -0.1356537938117981, -0.13239029049873352, 0.012891953811049461, -0.18224823474884033, 0.04347708821296692, 0.15876860916614532, -7.825173815945163e-05, 0.09643617272377014, -0.010231668129563332, -0.014140673913061619, 0.02213982306420803, 0.17224417626857758, -0.014601379632949829, -0.0693792849779129, 0.18726611137390137, 0.029002320021390915, 0.1651998907327652, 0.016424788162112236, 0.04902946576476097, 0.03630229830741882, -0.06653814017772675, 0.08797238767147064, 0.0602482333779335, 0.006978383753448725, 0.05421286076307297, 0.31434884667396545, -0.10426081717014313, -0.024748586118221283, 0.02177799865603447, 0.06705936789512634, 0.10615035146474838, 0.056292176246643066, 0.03180820867419243, 0.12486588209867477, -0.0969759151339531, 0.1280636340379715, -0.0223248228430748, -0.1049678772687912, -0.11894604563713074, -0.1795336753129959, 0.15348319709300995, -0.06068478152155876, 0.012430987320840359, -0.11222007125616074, 0.04400492459535599, 0.23619240522384644, -0.23970122635364532, 0.11986598372459412, -0.044794026762247086, 0.09399658441543579, 0.1500100940465927, 0.16027303040027618, 0.04736986756324768, -0.03747839108109474, 0.024306930601596832, 0.11414098739624023, -0.02468273788690567, -0.02857639268040657, -0.14629128575325012, 0.04083786904811859, -0.007905704900622368, -0.29387056827545166, 0.15130198001861572, -0.05348698049783707, 0.23669777810573578, 0.17026051878929138, 0.05844277888536453, 0.028425702825188637, -0.06984387338161469, -0.07791388034820557, -0.06740038841962814, 0.017689278349280357, -0.054858434945344925, -0.09006552398204803, -0.0019682436250150204, 0.14231738448143005, 0.03242306038737297, 0.00658051623031497, -0.4346880614757538, -0.011516330763697624, -0.06660100072622299, 0.030315836891531944, -0.028141150251030922, 0.2576213777065277, -0.06433561444282532, -0.045561533421278, -0.1158381775021553, -0.10174590349197388, -0.06538312882184982, -0.017522934824228287, -0.0910593792796135, 0.03704644739627838, 0.06703522801399231, -0.018858786672353745, 0.0013853437267243862}, {-0.2036779224872589, 0.21144482493400574, 0.23089836537837982, -0.08136804401874542, -0.030044056475162506, 0.0524224117398262, 0.2330199033021927, -0.021894728764891624, -0.11928505450487137, 0.011012834496796131, -0.011556229554116726, 0.19272911548614502, 0.13118118047714233, 0.07345277070999146, -0.06906772404909134, -0.1223873570561409, -0.0636826753616333, 0.12043716758489609, -0.04519648849964142, 0.08117799460887909, -0.4129786789417267, 0.13648191094398499, -0.008638580329716206, 0.026268383488059044, 0.018908513709902763, -0.09248293191194534, 0.028842482715845108, 0.18042954802513123, -0.05365847796201706, -0.36739298701286316, 0.008730783127248287, 0.07133417576551437, 0.2593139111995697, -0.03548413887619972, -0.0035655128303915262, -0.11713452637195587, -0.03927111253142357, 0.21898797154426575, 0.18420188128948212, -0.10660576075315475, -0.0864589735865593, 0.11243750154972076, 0.004142467398196459, 0.019398517906665802, 0.30597543716430664, 0.09362418949604034, 0.007535891141742468, -0.15680086612701416, -0.011898651719093323, 0.010553861036896706, -0.10530942678451538, 0.0983959436416626, 0.05790155008435249, -0.12881362438201904, -0.04514550790190697, -0.02198772132396698, -0.20373184978961945, 0.0919402614235878, 0.04212634637951851, -0.10435370355844498, -0.041324399411678314, -0.06121792271733284, -0.02683327905833721, -0.02356618642807007, -0.12645064294338226, -0.06335624307394028, 0.15854337811470032, -0.11181624233722687, -0.11664047837257385, -0.13588763773441315, -0.10501010715961456, -0.0530761256814003, 0.12004967033863068, 0.08528988063335419, 0.9187360405921936, -0.10129697620868683, 0.044157352298498154, -0.017711283639073372, 0.03300899267196655, 0.22286605834960938, -0.012653009966015816, -0.10698750615119934, 0.21376578509807587, -0.06422317773103714, 0.022722747176885605, -0.0975058376789093, 0.05654924362897873, -0.11588075011968613, 0.127156063914299, -0.1695382446050644, 0.04381432384252548, 0.18798480927944183, 0.15658164024353027, 0.10399387031793594, -0.010917098261415958, -0.026831526309251785, -0.017362887039780617, 0.18124982714653015, 0.05177833512425423, 0.4707443118095398, -0.046729154884815216, 0.17902827262878418, 0.11060141772031784, -0.1065007820725441, 0.17488591372966766, 0.3334835469722748, -0.011577019467949867, 0.11415683478116989, 0.1428944617509842, 0.19323091208934784, -0.07591961324214935, -0.07602189481258392, -0.0775698646903038, 0.048484038561582565, 0.04390445724129677, 0.34851738810539246, 0.07285497337579727, 0.10106681287288666, -0.03599486127495766, 0.029103092849254608, 0.06747781485319138, -0.01434654276818037, -0.046683408319950104, 0.020846843719482422, -0.13359478116035461, -0.036936208605766296, -0.0004555297491606325, -0.29414770007133484, -0.055428288877010345, -0.2011963427066803, -0.06827835738658905, -0.21673965454101562, -0.24305212497711182, -0.04605842009186745, -0.11279674619436264, -0.240943044424057, 0.054491423070430756, -0.1208924725651741, -0.14150108397006989, 0.11057034134864807, -0.011803912930190563, -0.16718779504299164, 0.10074128955602646, -0.12705661356449127, -0.0652412623167038, 0.20746096968650818, -0.03432140126824379, -0.06429440528154373, -0.039099421352148056, 0.34281331300735474, -0.039303094148635864, 0.10791525989770889, -0.07103312015533447, 0.25294989347457886, -0.09054812788963318, -0.05400272086262703, 0.1747591346502304, -0.024956941604614258, -0.061021242290735245, 0.30218955874443054, 0.10800013691186905, 0.22255626320838928, -0.014974148012697697, -0.1711277812719345, 0.14033737778663635, 0.02728409878909588, 0.09389350563287735, 0.1338501125574112, 0.035803377628326416, -0.26175999641418457, 0.0048735192976891994, 0.004284406080842018, 0.15243594348430634, -0.05223797261714935, -0.10019616037607193, 0.05730258673429489, -0.020773472264409065, 0.05651494115591049, 0.1315927952528, -0.138010635972023, 0.07271505892276764, -0.1305115520954132, 0.022941453382372856, -0.19150948524475098, -0.014209013432264328, -0.11283745616674423, 0.3460054397583008, -0.04884521663188934, 0.060787949711084366, -0.03586988523602486, 0.11032960563898087, 3.926891440642066e-05, -0.05700324475765228, 0.036121562123298645, -0.04927890747785568, 0.10864683240652084, 0.02091054432094097, 0.003734440077096224, -0.10095541179180145, 0.3403209447860718, -0.1934942752122879, -0.18082664906978607, 0.24183018505573273, -0.4088156521320343, 0.013802113011479378, -0.20418956875801086, 0.10952712595462799, -0.09428810328245163, -0.10244503617286682, -0.0491061694920063, 0.0036871563643217087, 0.0568220354616642, -0.13128258287906647, 0.11186224222183228, 0.03577323257923126, -0.23250335454940796, 0.030269810929894447, 0.20829389989376068, 0.04334545508027077, 0.04577860236167908, -0.12305570393800735, 0.08886463195085526, -0.013765337876975536, 0.3166057765483856, 0.1441996544599533, 0.3047269880771637, -0.08620356768369675, 0.34817180037498474, 0.17850184440612793, 0.11696793138980865, -0.15573425590991974, 0.04431629553437233, 0.24301016330718994, 0.045272503048181534, -0.08891135454177856, -0.11259815841913223, -0.12583404779434204, -0.05729365721344948, -0.021869001910090446, 0.5236444473266602, 0.013473697006702423, 0.08521197736263275, -0.017375841736793518, -0.00448735011741519, -0.12774355709552765, 0.09140072762966156, -0.09545425325632095, 0.02208593115210533, 0.031827494502067566, -0.006200235337018967, -0.1490323841571808, 0.3077225089073181, -0.1161804273724556, -0.10871624201536179, -0.10545814037322998, 0.2994981110095978}, {-0.04766983166337013, 0.1100248172879219, -0.08770643919706345, 0.1438228338956833, -0.05105450376868248, -0.1602473258972168, 0.05458318069577217, -0.10226983577013016, -0.1295839250087738, -0.08834018558263779, -0.013733994215726852, -0.013646150007843971, -0.14857187867164612, 0.24370531737804413, 0.10446503013372421, 0.06035662442445755, -0.027550721541047096, 0.06067920848727226, 0.12510763108730316, -0.2610200047492981, 0.003220078768208623, 0.03078475221991539, -0.016507579013705254, -0.0552130751311779, -0.22252069413661957, -0.09569230675697327, -0.004015428479760885, -0.02152671478688717, 0.07465711981058121, -0.0035472146701067686, 0.13774903118610382, 0.04889737442135811, 0.07618748396635056, 0.0029947636649012566, -0.09252426028251648, -0.18613369762897491, 0.058329273015260696, 0.04349839314818382, 0.050298187881708145, -0.02178553305566311, -0.16116079688072205, 0.03128144517540932, 0.015879707410931587, -0.03623415529727936, -0.23437869548797607, -0.020298806950449944, 0.030088383704423904, -0.06347726285457611, 0.0034426231868565083, -0.18083159625530243, 0.030930422246456146, -0.1311291605234146, -0.07707639038562775, -0.006700754631310701, -0.01796538569033146, -0.0723443254828453, -0.028873225674033165, 0.1822751760482788, -0.028424059972167015, -0.008430128917098045, -0.17171752452850342, -0.09648781269788742, -0.007124852854758501, -0.14295469224452972, -0.037265364080667496, 0.08005369454622269, -0.06059170886874199, 0.16730152070522308, 0.03476118668913841, 0.1751897931098938, -0.01316269300878048, 0.08297519385814667, 0.20501631498336792, 0.11549495905637741, -0.004025357775390148, 0.0017280778847634792, 0.08355873823165894, 0.011295229196548462, 0.43606290221214294, -0.18381541967391968, -0.11271387338638306, -0.0214226134121418, 0.16742739081382751, -0.08481251448392868, 0.0461345836520195, 0.10078634321689606, 0.0544610470533371, 0.05685312673449516, 0.057312458753585815, 0.21435970067977905, 0.096263587474823, 0.02482282742857933, 0.056485556066036224, -0.18492993712425232, 0.23259329795837402, 0.04243120178580284, -0.15346623957157135, -0.06788181513547897, -0.08310560137033463, 0.0037111740093678236, 0.05844292789697647, -0.15356335043907166, -0.010280795395374298, -0.0036481188144534826, 0.11221501976251602, 0.2615845501422882, -0.0999942496418953, 0.16835492849349976, -0.08955463021993637, -0.06869475543498993, 0.07212855666875839, 0.03328634053468704, -0.0014463497791439295, 0.042731475085020065, -0.2022363394498825, 0.25199946761131287, 0.08696875721216202, 0.060356903821229935, 0.17765507102012634, 0.06017852574586868, 0.20174956321716309, 0.12871846556663513, 0.002253206679597497, -0.0695369616150856, 0.1133233904838562, 0.023917458951473236, 0.063502237200737, -0.0059106359258294106, 0.08635679632425308, 0.00474375206977129, 0.08000897616147995, 0.07512495666742325, 0.02175186015665531, 0.06072065606713295, 0.11025447398424149, -0.09332968294620514, -0.09224648773670197, -0.019807549193501472, -0.04820319637656212, 0.006467652507126331, 0.09788458794355392, 0.05046216398477554, -0.0074668461456894875, -0.2980811297893524, -0.06807246804237366, 0.053514741361141205, 0.034382324665784836, -0.000429782725404948, -0.13227032124996185, -0.0753326416015625, -0.2060498595237732, 0.06998862326145172, 0.14007379114627838, 0.006849047262221575, -0.03477662801742554, -0.03769805282354355, -0.013062960468232632, -0.04249373450875282, 0.05175496265292168, -0.12630236148834229, 0.015527944080531597, 0.11710742861032486, 0.07610119879245758, 0.11430911719799042, 0.03749557211995125, 0.1173173040151596, -0.1473783701658249, -0.15065670013427734, -0.05040159448981285, 0.026882529258728027, -0.0752035602927208, -0.11418158560991287, 0.08702842146158218, -0.17665910720825195, -0.22224539518356323, 0.17204415798187256, 0.23904657363891602, -0.11173418909311295, 0.25500747561454773, 0.002204555319622159, 0.07950201630592346, 0.05728040635585785, 0.12570229172706604, 0.24254347383975983, -0.09239301085472107, -0.13121730089187622, -0.17663781344890594, -0.1682230532169342, 0.23511677980422974, 0.025889931246638298, 0.12839177250862122, -0.23049619793891907, 0.13343128561973572, -0.0002652507973834872, -0.11733337491750717, -0.017598984763026237, -0.26691392064094543, -0.18939918279647827, -0.015358193777501583, 0.11183067411184311, 0.1076371893286705, 0.10362442582845688, -0.01952371932566166, -0.13410405814647675, 0.04518340528011322, -0.10123972594738007, -0.017649762332439423, 0.0522519126534462, -0.14217570424079895, 0.11341997981071472, 0.19494211673736572, -0.21548478305339813, 0.12276449054479599, -0.1633712351322174, -0.04293101653456688, 0.12293591350317001, -0.16345559060573578, -0.049794752150774, -0.16329343616962433, 0.0021347824949771166, 0.059919070452451706, -0.061945702880620956, -0.13710948824882507, -0.3003164529800415, -0.20311202108860016, -0.09919073432683945, 0.14622566103935242, -0.09439404308795929, -0.12463951855897903, -0.01787542551755905, -0.13187767565250397, 0.21843434870243073, 0.00895313173532486, -0.18143345415592194, 0.20267342031002045, -0.1732558012008667, -0.01648104563355446, -0.10790073126554489, 0.060513678938150406, 0.07664186507463455, 0.11747971922159195, -0.22764278948307037, 0.07520484179258347, 0.06624990701675415, -0.1642601191997528, -0.0842040628194809, -0.03958465903997421, -0.0938837081193924, -0.036986343562603, 0.014771766029298306, 0.12484469264745712, 0.019525228068232536, 0.14420226216316223, 0.04221441596746445, 0.1869707852602005, 0.061734944581985474}, {-0.0007105224067345262, -0.3291698098182678, 0.1813441812992096, -0.09142550826072693, -0.018430639058351517, -0.19665199518203735, 0.06758025288581848, -0.017248013988137245, 0.001473541371524334, -0.08576101809740067, 0.002599355997517705, 0.1928464025259018, -0.12247339636087418, 0.04171907156705856, -0.1536361426115036, -0.07396500557661057, -0.024446742609143257, -0.16739685833454132, -0.07636892795562744, -0.08890563249588013, -0.33922743797302246, 0.05823685973882675, -0.11483129113912582, -0.26436570286750793, 0.02640465646982193, 0.2112664133310318, -0.12786808609962463, 0.05590175464749336, -0.23891562223434448, -0.13637544214725494, 0.12508513033390045, 0.08074329048395157, 0.16295284032821655, 0.11963757127523422, -0.04760270193219185, 0.08169379085302353, -0.013254412449896336, 0.018865574151277542, -0.1177859902381897, -0.049965016543865204, -0.13550545275211334, 0.022712599486112595, -0.1837906390428543, 0.028657637536525726, 0.0667361244559288, -0.07490057498216629, -0.18606075644493103, -0.03399579972028732, -0.018302178010344505, 0.0749220997095108, 0.28938373923301697, 0.003630459774285555, 0.02400221861898899, 0.25674793124198914, -0.044070757925510406, -0.13413575291633606, -0.07424499839544296, 0.050702743232250214, 0.019109664484858513, 0.005799844395369291, 0.0034372585359960794, -0.06625034660100937, 0.1545451134443283, -0.2563946545124054, 0.02900504879653454, -0.07176866382360458, 0.18974895775318146, 0.18710632622241974, -0.3536366820335388, 0.002714750822633505, 0.0628533661365509, -0.0655311644077301, 0.39285165071487427, 0.23644042015075684, 0.1486743986606598, -0.19973117113113403, -0.012590070255100727, 0.04224546253681183, 0.073058120906353, -0.031720273196697235, 0.03049488365650177, -0.1009567454457283, -0.123720683157444, 0.1911531686782837, -0.18471382558345795, 0.035513296723365784, -0.07418694347143173, 0.0722416341304779, 0.1271848827600479, -0.2304832935333252, -0.04101293906569481, 0.07070683687925339, -0.19942699372768402, -0.1322552114725113, -0.07241757959127426, 0.039230260998010635, 0.1578955054283142, -0.03548396751284599, 0.09178140759468079, -0.04740379378199577, 0.006724400445818901, 0.057548776268959045, -0.15510615706443787, 0.12539023160934448, -0.1706247329711914, -0.07373972982168198, 0.013546700589358807, -0.007864762097597122, -0.23852814733982086, 0.12942557036876678, 0.06100473552942276, -0.10978572070598602, 0.4018320143222809, -0.11646641790866852, -0.008244347758591175, 0.1816227287054062, -0.06964807212352753, -0.05263921990990639, 0.02289176732301712, 0.00788271613419056, -0.15881744027137756, -0.15158888697624207, -0.1912713348865509, 0.27098748087882996, -0.1910920888185501, 0.14525413513183594, 0.07236318290233612, -0.09770826250314713, 0.10402566194534302, -0.06840979307889938, -0.07033942639827728, -0.15476736426353455, -0.26233452558517456, -0.005937329959124327, 0.008165030740201473, -0.19928893446922302, 0.049957506358623505, -0.15784215927124023, 0.009164114482700825, -0.10483568906784058, 0.08520956337451935, -0.052282970398664474, 0.06200706213712692, 0.11125927418470383, -0.22725826501846313, 0.12424783408641815, 0.015852566808462143, 0.029496757313609123, 0.21087764203548431, 0.08952885866165161, 0.17380419373512268, 0.057080354541540146, -0.12561477720737457, 0.03299223631620407, -0.06976603716611862, 0.05391760915517807, -0.11914262175559998, -0.19383230805397034, -0.08905497938394547, 0.06279003620147705, 0.047594938427209854, 0.1518602967262268, -0.032395221292972565, -0.08741162717342377, 0.007125235162675381, 0.15224149823188782, -0.10071831196546555, 0.0664060041308403, 0.28092458844184875, 0.051039330661296844, -0.09250809997320175, 0.07446018606424332, 0.02567768096923828, 0.15413276851177216, 0.1716175228357315, 0.2541547119617462, 0.04423996061086655, -0.08769833296537399, 0.10605999082326889, -0.10547671467065811, 0.009944832883775234, -0.08600018173456192, 0.03026939556002617, 0.22882765531539917, -0.23082266747951508, -0.031697578728199005, 0.021498315036296844, 0.23179097473621368, -0.011911231093108654, -0.04741983115673065, -0.002182644559070468, -0.06626817584037781, 0.13985711336135864, 0.3645726144313812, -0.1656172126531601, 0.06032121926546097, -0.05104109272360802, 0.183158740401268, -0.08768799901008606, -0.1693619042634964, -0.17384149134159088, 0.13576655089855194, -0.08501571416854858, 0.03936787694692612, -0.03570270538330078, 0.09668905287981033, 0.04214495047926903, -0.058480363339185715, 0.051913514733314514, 0.09939281642436981, -0.14677166938781738, 0.030435066670179367, -0.09479508548974991, 0.037757810205221176, -0.13227438926696777, -0.2079804390668869, -0.06145309656858444, 0.14277403056621552, 0.21895664930343628, 0.10607360303401947, -0.17425177991390228, 0.06874280422925949, 0.07169848680496216, -0.11824657768011093, -0.2042544186115265, -0.07220955938100815, 0.09969352930784225, -0.2626969516277313, 0.0022519640624523163, -0.07442817091941833, 0.05207613483071327, 0.024729952216148376, 0.22941067814826965, -0.0462687723338604, -0.0023196018300950527, -0.01544284075498581, -0.230324849486351, -0.010072564706206322, 0.09178965538740158, -0.14696688950061798, 0.09581193327903748, 0.000326006964314729, -0.2097340077161789, 0.013055176474153996, 0.013198957778513432, 0.007638030685484409, -0.07606402039527893, 0.009003454819321632, -0.06055878847837448, 0.2643952965736389, 0.04924653097987175, -0.40163496136665344, 0.08305607736110687, -0.061898745596408844, 0.05022655054926872, -0.03170965239405632}, {0.19520600140094757, -0.04669756814837456, 0.04684709385037422, 0.020641664043068886, -0.052335139364004135, -0.017293374985456467, -0.09893907606601715, -0.03663858398795128, 0.009808468632400036, 0.2388487160205841, 0.0719531998038292, 0.09152311831712723, -0.14051508903503418, -0.08863753825426102, 0.28248661756515503, 0.0628342479467392, 0.02422253228724003, -0.16338050365447998, 0.005525162909179926, 0.05876858904957771, 0.1113574355840683, -0.05659538879990578, 0.21000109612941742, -0.2208210527896881, -0.12262564152479172, 0.04243318736553192, 0.1739768236875534, -0.0024917020928114653, 0.10513201355934143, -0.003674087580293417, -0.09767862409353256, 0.28886085748672485, -0.02869720384478569, -0.09541267901659012, 0.04862101003527641, 0.10559853166341782, 0.029760977253317833, -0.06152968108654022, -0.09113064408302307, -0.05744721367955208, -0.006901659071445465, 0.0692126452922821, -0.0917997881770134, 0.19195711612701416, -0.011449184268712997, -0.03932711482048035, -0.06615573167800903, -0.15073183178901672, 0.0511273555457592, -0.005559407640248537, 0.11042993515729904, -0.06162860989570618, 0.019112737849354744, -0.20676040649414062, 0.07889488339424133, -0.053292904049158096, -0.08960115164518356, -0.11730215698480606, 0.0580216683447361, 0.06737681478261948, -0.016850728541612625, -0.22599750757217407, 0.19664514064788818, -0.09847249835729599, -0.01162874884903431, -0.13043813407421112, -0.08335366100072861, 0.13290776312351227, -0.0617896243929863, 0.11644737422466278, 0.042353369295597076, -0.007414007559418678, -0.09029272943735123, -0.06958141177892685, 0.09493879973888397, 0.08354981243610382, 0.027211399748921394, 0.16053861379623413, -0.22615163028240204, -0.16490736603736877, -0.10742154717445374, 0.02329518087208271, -0.18349581956863403, 0.0571686290204525, -0.3379940092563629, -0.0647316426038742, -0.09464053809642792, -0.027549883350729942, -0.07688538730144501, 0.15094318985939026, -0.04184668883681297, -0.08043762296438217, -0.06925556063652039, 0.019909881055355072, 0.04190324991941452, 0.05829635262489319, -0.04486904665827751, 0.1774337738752365, 0.05191957950592041, -0.1831228882074356, -0.009259932674467564, 0.021244700998067856, -0.07836554944515228, -0.07279043644666672, 0.026938360184431076, -0.09541294723749161, 0.1935899406671524, -0.04994289204478264, 0.060918066650629044, -0.18453308939933777, -0.07563289999961853, -0.0027135824784636497, 0.014680210500955582, -0.000713014043867588, 0.05480081960558891, -0.05554715916514397, -0.028730962425470352, 0.055895574390888214, -0.16461677849292755, -0.00871353317052126, -0.15898750722408295, 0.11815322190523148, 0.016337282955646515, -0.11731661856174469, -0.054114051163196564, 0.3600511848926544, 0.028732318431138992, -0.05120088905096054, -0.1299847513437271, 0.04178624227643013, -0.22836147248744965, -0.12073218077421188, 0.050850436091423035, 0.14255236089229584, -0.11017771810293198, -0.4134189784526825, 0.04088596627116203, 0.06843039393424988, -0.05115219205617905, 0.017611414194107056, -0.06053900346159935, 0.24020074307918549, -0.12950292229652405, 0.010431711561977863, -0.08028654009103775, -0.09320523589849472, -0.0019010566174983978, -0.15761153399944305, 0.052957262843847275, 0.022876061499118805, -0.04981851950287819, -0.017343228682875633, -0.12082827091217041, -0.04330512136220932, -0.18370063602924347, -0.1283886730670929, -0.02699432522058487, -0.010234815068542957, 0.26299911737442017, -0.12603530287742615, 0.17373378574848175, 0.2630144953727722, -0.07570885866880417, 0.05488616228103638, 0.008923651650547981, -0.05881112813949585, 0.03582994267344475, 0.09951801598072052, 0.12303607165813446, 0.1201399639248848, -0.11011874675750732, 0.14030757546424866, -0.1997509002685547, 0.02799776755273342, 0.1817856878042221, 0.12226903438568115, 0.031229292973876, -0.0812755823135376, 0.14552836120128632, -0.04722350463271141, 0.0906575471162796, 0.26191312074661255, 0.11435016989707947, 0.039851829409599304, 0.07250818610191345, 0.13811053335666656, 0.02164730243384838, 0.06899063289165497, 0.12786880135536194, 0.44110268354415894, -0.09819629043340683, 0.022995058447122574, 0.0974910631775856, 0.003561914199963212, -0.10492406040430069, -0.013317267410457134, 0.006246279459446669, 0.11115700006484985, 0.1528560370206833, 0.06113692745566368, -0.03312330320477486, 0.17980673909187317, -0.2685617506504059, -0.12484148144721985, -0.12206649780273438, -0.023707689717411995, 0.0011063935235142708, -0.1492171585559845, -0.11123233288526535, 0.09979739040136337, 0.14182017743587494, -0.10442916303873062, -0.051974643021821976, 0.1866380274295807, 0.1601114124059677, 0.051692478358745575, -0.26219141483306885, 0.3431394696235657, -0.04091177135705948, 0.14517299830913544, 0.17684383690357208, -0.071754589676857, 0.015842590481042862, -0.12031785398721695, 0.1772279441356659, -0.1184205412864685, 0.08848877251148224, -0.09284570068120956, -0.022968290373682976, -0.18414977192878723, -0.10246157646179199, -0.13941575586795807, 0.08244737237691879, 0.0869898721575737, 0.15442168712615967, 0.3801758885383606, -0.044060464948415756, 0.05866377055644989, -0.016802236437797546, -0.15413253009319305, 0.09152131527662277, 0.037328701466321945, -0.03440343588590622, 0.2212279736995697, -0.3455047607421875, -0.16419436037540436, -0.04815734177827835, -0.14480113983154297, -0.05152741074562073, 0.07821442931890488, 0.07640933990478516, -0.036189477890729904, -0.1083851084113121, 0.17053504288196564, -0.04369690269231796, 0.26855897903442383}, {-0.1352827101945877, 0.03525003418326378, 0.1409008651971817, 0.11397348344326019, 0.045191604644060135, -0.015230223536491394, 0.097786083817482, 0.031684622168540955, 0.44407621026039124, 0.09778927266597748, -0.08641348034143448, 0.12083020806312561, -0.10633689910173416, -0.05534327030181885, -0.29950156807899475, -0.10409194231033325, -0.14364011585712433, -0.19934417307376862, -0.1473216861486435, 0.11702148616313934, -0.1666763573884964, -0.08536235243082047, -0.10492809861898422, 0.08310171961784363, 0.12288148701190948, -0.15841162204742432, 0.08531709015369415, 0.02560782991349697, -0.18674306571483612, -0.18993663787841797, 0.16992898285388947, -0.04448821768164635, 0.08526645600795746, -0.010089905001223087, 0.05345907807350159, -0.048753511160612106, 0.020309127867221832, 0.12703227996826172, -0.037655968219041824, 0.04038761183619499, 0.06800668686628342, 0.11885209381580353, 0.1361590325832367, -0.2808113396167755, 0.10511656850576401, -0.0071460106410086155, -0.1643678992986679, 0.044506434351205826, 0.007550486363470554, -0.06155728921294212, 0.43063047528266907, -0.1866145133972168, 0.3752409815788269, 0.13624709844589233, 0.009449794888496399, 0.14849944412708282, 0.10045866668224335, 0.034034304320812225, -0.18035189807415009, 0.07011742144823074, -0.07823825627565384, -0.05322851985692978, 0.02176634594798088, -0.04439948499202728, 0.20903916656970978, 0.16153529286384583, 0.13700130581855774, -0.21343466639518738, -0.05623408034443855, -0.09376109391450882, -0.0765766054391861, -0.10830245912075043, 0.013914761133491993, 0.29564255475997925, -0.03675477206707001, -0.011991075240075588, 0.14460135996341705, -0.13217058777809143, -0.16536636650562286, 0.036892425268888474, 0.1946459859609604, -0.12287408113479614, -0.28866150975227356, -0.018502460792660713, 0.007153131999075413, 0.10097447037696838, 0.01053666602820158, -0.04283798858523369, 0.0030147156212478876, 0.07461188733577728, 0.041856709867715836, 0.035589687526226044, -0.11413887143135071, 0.14895376563072205, -0.2527066171169281, 0.005206211935728788, -0.053332094103097916, 0.21558967232704163, -0.04760739952325821, -0.03039122372865677, -0.09249655157327652, -0.12480128556489944, 0.2072572261095047, 0.17128634452819824, -0.09405722469091415, -0.14935395121574402, -0.18326939642429352, -0.18749184906482697, 0.0595620796084404, -0.0073840259574353695, 0.08436626940965652, -0.014362435787916183, -0.06234022229909897, 0.16742853820323944, 0.11040705442428589, 0.0060097347013652325, -0.11931133270263672, -0.03725384548306465, -0.05978671461343765, 0.0560961589217186, -0.25116074085235596, 0.12249308079481125, -0.008623105473816395, 0.007741784211248159, -0.014519518241286278, 0.0012638786574825644, -0.10384956002235413, 0.03858252242207527, -0.1431083083152771, -0.039323631674051285, -0.1441357582807541, -0.054979316890239716, -0.1493798941373825, 0.06944585591554642, 0.03519461676478386, 0.14704865217208862, -0.3140658438205719, -0.1847594529390335, 0.026560591533780098, -0.3735012412071228, 0.008009406737983227, -0.1727263480424881, -0.028753122314810753, 0.16315729916095734, -0.05982355773448944, 0.17912481725215912, -0.2475445717573166, -0.020651251077651978, -0.0888025164604187, 0.012179038487374783, -0.10931715369224548, 0.1155179888010025, -0.1344504952430725, -0.11273796856403351, 0.06916581094264984, -0.16917665302753448, -0.00923155341297388, -0.11627057939767838, -0.07583848387002945, 0.032117825001478195, -0.07878133654594421, 0.024437343701720238, -0.18134364485740662, 0.057341527193784714, 0.22327172756195068, 0.02313239499926567, -0.06196051836013794, -0.08524920791387558, 0.008006138727068901, 0.03674279525876045, 0.20272000133991241, 0.23892539739608765, 0.11605732887983322, 0.025858234614133835, 0.2741020917892456, -0.023683959618210793, -0.18192173540592194, 0.0908084586262703, -0.46050363779067993, 0.06304844468832016, -0.18919837474822998, -0.2761790156364441, 0.24113552272319794, -0.00588212301954627, 0.04192512109875679, -0.11451877653598785, -0.18088868260383606, 0.11456310749053955, 0.10518087446689606, -0.11715450137853622, -0.06380681693553925, 0.12983672320842743, 0.23731306195259094, 0.20865364372730255, 0.07110287249088287, 0.01805798150599003, -0.12114706635475159, -0.28359273076057434, 0.010753600858151913, -0.08595815300941467, 0.03971576318144798, -0.20630978047847748, 0.09312328696250916, 0.1699482798576355, -0.04772825166583061, 0.045222677290439606, -0.2589626908302307, -0.07881773263216019, 0.09294900298118591, -0.032137222588062286, 0.024447450414299965, -0.06543351709842682, -0.16466426849365234, 0.22093988955020905, 0.29036644101142883, -0.09831221401691437, 0.03102937713265419, 0.023785144090652466, 0.055058714002370834, -0.08359959721565247, -0.08823370188474655, 0.05721530690789223, -0.06904276460409164, -0.16121017932891846, -0.00472964346408844, -0.10303893685340881, -0.1086132749915123, -0.0005314943846315145, -0.0013847608352079988, 0.06412822753190994, 0.03198868781328201, -0.27503421902656555, 0.0027590596582740545, -0.0866757333278656, -0.08537878096103668, 0.025768455117940903, 0.022229880094528198, 0.16893580555915833, -0.06759744137525558, 0.05242446810007095, -0.032809603959321976, 0.09972703456878662, -0.12255983799695969, 0.24216677248477936, 0.30954065918922424, -0.012536735273897648, 0.08324100077152252, 0.07960119843482971, 0.02176898717880249, 0.13150283694267273, -0.20336982607841492, -0.05039048194885254, -0.1334581971168518, -0.05874602124094963, -0.20135344564914703, -0.11312773823738098}}
local m3 = {{0.009516230784356594, 0.0008425090927630663, -0.017556974664330482, 0.02476123720407486, -0.034104764461517334, -0.011140045709908009, 0.003947657998651266, -0.03618647903203964, -0.01101678516715765, 0.00030229284311644733, 0.014600289985537529, -0.002630482194945216, 0.007838805206120014, -0.0005172564415261149, -0.0037844639737159014, -0.006547580938786268, 0.01933031529188156, -0.018469903618097305, 0.019318249076604843, -0.00700779166072607, 0.001528410823084414, -0.020212141796946526, -0.011535385623574257, -0.01706167683005333, 0.0065964930690824986, 0.0052375649102032185, -0.018697207793593407, -0.006334342062473297, -0.009692884981632233, -0.022722972556948662, 0.02197665348649025, -0.0021998595912009478, 0.006585733965039253, -0.014808631502091885, -0.017702076584100723, 0.0002919542312156409, 0.00327065191231668, -0.030447693541646004, -0.009339185431599617, -0.026839585974812508, 0.017142461612820625, 0.01338908076286316, 0.025845259428024292, 0.005769837647676468, -0.012223068624734879, -0.007683220785111189, 0.014349495992064476, 0.0004278783162590116, 0.015890449285507202, -0.006705868989229202, -0.003823523875325918, -0.014069996774196625, 0.011142758652567863, -0.018648669123649597, 0.016940785571932793, -0.025992073118686676, 0.01406420860439539, 0.013590030372142792, -0.002768441569060087, -0.009542546235024929, 0.022065678611397743, -0.02242337539792061, -0.016950998455286026, -0.010218249633908272, -0.01981775090098381, 0.027655651792883873, 0.018984152004122734, -0.016047650948166847, -0.016686448827385902, -0.009950535371899605, 0.024117877706885338, -0.014002557843923569, 0.004535323474556208, -0.00038597718230448663, -0.027086971327662468, 0.007953743450343609, 0.002249007346108556, 0.04770814999938011, 0.001457726233638823, 0.012946750037372112, 0.006412704475224018, -0.025540916249155998, -0.011068882420659065, 0.028563037514686584, -0.003233348485082388, 0.025422489270567894, 0.0053801205940544605, -0.021807070821523666, 0.02243175357580185, -0.006487073376774788, 0.001496594282798469, -0.004277585074305534, -0.006896098610013723, -0.019910087808966637, -0.0025084929075092077, 0.012772764079272747, -0.03876601532101631, -0.003019219497218728, -0.009426457807421684, -0.031368497759103775, 0.004159184172749519, -0.006035245954990387, -0.0029123704880476, -0.006636423524469137, -0.0035145285073667765, 0.008156322874128819, -0.0005139620625413954, -0.016334783285856247, 0.0019581655506044626, -0.0024109994992613792, -0.01843879371881485, -0.022009186446666718, 0.009026749059557915, -0.011627603322267532, -0.011687044985592365, -0.004435592330992222, -0.009052526205778122, 0.017921777442097664, 0.0008525729645043612, 0.022320745512843132, -0.027022631838917732, -0.0200740285217762, -0.003349972190335393, 0.03742954507470131, 0.0010988248977810144, 0.007118892390280962, 0.00021665613166987896, 0.010995729826390743, -0.007925386540591717, 0.015884919092059135, -0.017087919637560844, -0.0005352856242097914, 0.015408642590045929, 0.00386289251036942, -0.004619075451046228, 0.003382954513654113, -0.0530519001185894, -0.018984798341989517, -0.006341319531202316, 0.010616196319460869, 0.016283299773931503, 0.020095815882086754, -0.00814361684024334, 0.016138175502419472, -0.008159846998751163, 0.009877338074147701, -0.0068206884898245335, 0.002201693132519722, -0.001729658804833889, 0.006905004382133484, -0.009472863748669624, 0.025870058685541153, 0.0066243726760149, 0.004255575593560934, -0.023563643917441368, -0.0028897391166538, -0.014729859307408333, -0.024066759273409843, 0.00457024946808815, -0.0041436399333179, 0.0031475743744522333, 0.014590512961149216, -0.008705848827958107, 0.006075338926166296, -0.015089615248143673, 0.030635613948106766, -0.0043508997187018394, 0.0025448654778301716, 0.011285186745226383, 0.007540541235357523, -0.015763312578201294, -0.010544901713728905, 0.008412657305598259, -0.013742371462285519, -0.018784286454319954, 0.018428392708301544, 0.0014808729756623507, -0.016916777938604355, 0.018245549872517586, -0.011576131917536259, -0.010058755986392498, 0.016016339883208275, -0.013500366359949112, -0.01397818885743618, -0.034403856843709946, -0.001066056895069778, 0.0019345765467733145, 0.027844615280628204, 0.019027497619390488, -0.0022448834497481585, -0.003115667961537838, -0.011595644056797028, 0.014047691598534584, -0.0016164345433935523, 0.012677852995693684, 0.036068208515644073, 0.0008942726417444646, 0.015752581879496574, 0.008709403686225414, -0.008060343563556671, -0.00984722375869751, -0.003615785390138626, 0.02247409150004387, 0.023780543357133865, -0.007079333532601595, 0.01203968096524477, 0.009503107517957687, 0.008054562844336033, 0.01721828430891037, -0.02885228767991066, -0.03997931256890297, -0.01605120673775673, -0.015479487366974354, 0.012691352516412735, 0.021244022995233536, -0.004887139890342951, -0.026704948395490646, -0.012986408546566963, -0.01060620415955782, 0.0023795815650373697, 0.012588812038302422, -0.015462898649275303, 0.0017738793976604939, -0.011820616200566292, 0.004449889995157719, -0.0033463456202298403, -0.002535987878218293, 0.008822017349302769, 0.014481422491371632, 0.002254184801131487, 0.0061652883887290955, 0.005045725964009762, 0.012107813730835915, -0.023617373779416084, -0.028004281222820282, -0.008011085912585258, -0.0066820173524320126, -0.004162310156971216, 0.02792629599571228, -0.011432718485593796, 0.015466920100152493, -0.022573737427592278, -0.0029438300989568233, 0.013579148799180984, -0.00571155920624733, -0.011766740120947361, -0.008909419178962708, 0.0036842371337115765, -0.002201792784035206, -0.004365865606814623, -0.0016026725061237812, -0.014274471439421177, -0.012967300601303577, 0.009246519766747952, -0.005445297807455063, -0.012169386260211468}, {-0.007590109016746283, -0.015226324088871479, -0.01936487667262554, 0.01825759746134281, -0.013535871170461178, -0.009696812368929386, -0.017519645392894745, -0.04729209467768669, -0.022525029256939888, -0.0021153236739337444, 0.009510267525911331, 0.0012993805576115847, 0.00469357380643487, 0.004240724258124828, -0.01648826338350773, -0.019619068130850792, 0.032116565853357315, -0.011659444309771061, 0.013761722482740879, -0.013110073283314705, -0.004519683308899403, -0.01892625167965889, 0.0013649307657033205, -0.0055369180627167225, 0.01545690931379795, -0.015313676558434963, -0.004621818661689758, 0.015052556060254574, -0.013125649653375149, -0.010916763916611671, 0.003982905298471451, 0.001623731106519699, 0.014982229098677635, 0.003687085583806038, 0.006846158765256405, 0.012513352558016777, -0.0007587760337628424, -0.0010262016439810395, -0.0018667643889784813, -0.008902168832719326, 0.00035189659683965147, -0.0002636231074575335, 0.03490040451288223, 0.004443496465682983, -0.0008746160892769694, -0.012557346373796463, 0.004579611122608185, 0.0003080856113228947, 0.02423810586333275, -0.023087408393621445, -0.006130782887339592, -0.005581478122621775, 0.005769954062998295, -0.010756741277873516, 0.009944381192326546, -0.025181714445352554, 0.015809770673513412, 0.013138975016772747, -0.005869508720934391, 0.0005891875480301678, 0.010413714684545994, -0.005268310196697712, -0.015704622492194176, 0.0035414891317486763, -0.016161659732460976, -0.010335930623114109, 0.019943444058299065, -0.003407084383070469, -0.021652953699231148, -0.020066801458597183, -0.0033369779121130705, -0.022028610110282898, -0.0030241182539612055, -0.011023239232599735, -0.02473306655883789, 0.028166403993964195, 0.004025509115308523, 0.004041837528347969, -0.010517994873225689, 0.017216039821505547, -0.0027763883117586374, 0.012995027005672455, -0.00895696971565485, 0.021551471203565598, -0.03130025416612625, 0.01028954703360796, 0.020757172256708145, 0.004444947000592947, 0.02110021747648716, -0.003111395286396146, 0.01178586483001709, 0.022088272497057915, -0.018173281103372574, -0.022928336635231972, 0.00621790112927556, 0.017653873190283775, -0.02157801203429699, -0.005031109321862459, -0.01595361717045307, -0.004998175427317619, 0.03678659349679947, 0.0032345103099942207, 0.0036946723703294992, -0.004425710532814264, -0.015800807625055313, -0.033373333513736725, 0.011918140575289726, -0.02092246152460575, 0.011166481301188469, -0.0020085477735847235, 0.0009891634108498693, -0.039071422070264816, -0.014018678106367588, -0.004473668523132801, -0.010667731985449791, 0.015636280179023743, -0.01575690507888794, 0.025409819558262825, -0.011061831377446651, 0.029274556785821915, -0.02861957810819149, -0.013412408530712128, -0.005670497193932533, 0.0438079871237278, 0.0054590050131082535, 0.011302627623081207, -0.009768051095306873, 0.021853603422641754, -0.00848652794957161, 0.003034197259694338, -0.0044207144528627396, -0.0087096206843853, -0.007040024735033512, 0.011773958802223206, -0.005840090103447437, 0.00223195506259799, -0.025845050811767578, 0.009639382362365723, -0.008440998382866383, 0.004390998277813196, 0.01706058718264103, 0.023157138377428055, -0.010781120508909225, 0.014144510962069035, 0.016775276511907578, 0.012368679977953434, 0.023479240015149117, 0.016723698005080223, 5.00639407619019e-06, 0.009027963504195213, -0.0230551864951849, -0.0006024864851497114, -0.004101395141333342, 0.020192977041006088, -0.013947349041700363, -0.005808277986943722, 0.01705341413617134, -0.015745216980576515, 0.008301871828734875, -0.01008005440235138, 0.007441685535013676, 0.007821629755198956, -0.013535941950976849, 0.028776034712791443, -0.012076559476554394, 0.039971038699150085, -0.016873834654688835, -0.011951959691941738, 0.009439138695597649, 0.011934302747249603, -0.01338525116443634, -0.0067474632523953915, 0.0015662327641621232, 0.001570140477269888, -0.013184594921767712, 0.017014726996421814, -0.009509020484983921, 0.0007358701550401747, 0.010408381931483746, -0.023840222507715225, -0.0024208363611251116, -0.007675213739275932, -0.006080166436731815, 0.004789482336491346, -0.02817421779036522, 0.011454442515969276, 0.0027335770428180695, 0.030040111392736435, -0.02406599000096321, -0.00854284968227148, -0.012107327580451965, -0.0019609308801591396, -0.0037539799232035875, 0.003097700886428356, 0.032863929867744446, 0.03152243047952652, 0.0024620972108095884, 0.0007262192666530609, 0.025603849440813065, 0.005584006663411856, -0.024957366287708282, 0.03410886600613594, -0.016803957521915436, 0.012213903479278088, -0.002596050500869751, 0.014870602637529373, 0.003174393204972148, -0.00114353746175766, -0.01796645298600197, -0.01220078207552433, -0.019876699894666672, -0.0006640731589868665, -0.009778519161045551, 0.01515509095042944, 0.017730774357914925, 0.014530881308019161, -0.009114068932831287, 0.003793342038989067, -0.010740953497588634, -0.020241914317011833, 0.002764577278867364, -0.01856117881834507, 0.0059276120737195015, -0.013842090032994747, 0.0073028565384447575, 0.0031008124351501465, -0.008458309806883335, -0.008315648883581161, 0.01937875524163246, -0.00864332914352417, 0.01035802811384201, -0.011891745962202549, 0.029905568808317184, -0.01879408024251461, -0.025164203718304634, -0.01971389725804329, 0.004721560515463352, 0.0044879917986691, 0.037351835519075394, -0.01232849434018135, 0.025206394493579865, -0.0073651839047670364, -0.001914725173264742, 0.007498154882341623, 0.0059073432348668575, 0.007023074198514223, -0.01936310902237892, 0.000573323224671185, 0.003093864768743515, -0.005956127308309078, -0.014066199772059917, -0.012564566917717457, -0.007793369237333536, -0.009168628603219986, -0.00015887917834334075, -0.0020096919033676386}, {0.02172829955816269, -0.016375428065657616, 0.007196477614343166, 0.006014196667820215, -0.029383797198534012, 0.002031948883086443, -0.01749579608440399, -0.029751047492027283, -0.02095905691385269, 0.01894228905439377, -0.013176401145756245, 0.001929040183313191, -0.013319291174411774, -0.009567518718540668, -0.01284423004835844, -0.01599092222750187, 0.007177943829447031, -0.008343586698174477, 0.017836101353168488, 0.0009963435586541891, 0.0047603207640349865, -0.021679116412997246, -0.008871542289853096, -0.014723343774676323, 0.0043336013332009315, -0.008508165366947651, -0.004462245386093855, 0.003026034915819764, -0.014628422446548939, -0.016567768529057503, 0.01689547300338745, 0.02370557375252247, -0.010542117990553379, 0.01016010157763958, -0.012154624797403812, -0.0010147748980671167, 0.0013704641023650765, -0.013636498712003231, -0.01476086862385273, -0.003596850670874119, 0.006937878206372261, -0.005605297163128853, 0.020744571462273598, 0.01589834876358509, 0.00960615649819374, -0.018864229321479797, 0.035183146595954895, -0.0017851042794063687, 0.019234947860240936, -0.011642011813819408, -0.0011696297442540526, -0.005540372338145971, 0.02597923018038273, 0.0076804449781775475, 0.021342391148209572, -0.03686897084116936, 0.00528999837115407, -0.005738184787333012, -0.018464790657162666, -0.02648201584815979, 0.011713284067809582, 0.005978471599519253, -0.019079722464084625, 0.004064411856234074, 0.006078596692532301, 0.004103063140064478, 0.008364600129425526, -0.02624600939452648, -0.009944682009518147, -0.0032273242250084877, 0.031049318611621857, -0.004951335955411196, -0.01160014420747757, 0.014444052241742611, -0.011813215911388397, 0.0028832126408815384, -0.0017483721021562815, 0.02450511045753956, -0.007898606359958649, 0.01153878029435873, 0.0011090473271906376, -0.010950984433293343, -0.035494424402713776, 0.026303458958864212, -0.01446271687746048, 0.004045260138809681, -0.002397067379206419, -0.011089476756751537, 0.020824937149882317, -0.006248764228075743, -0.003622868098318577, 0.02227148972451687, -0.004677889868617058, -0.0062704142183065414, 0.0059309666976332664, 0.032063309103250504, -0.030771536752581596, -0.016912590712308884, -0.009426425211131573, -0.0026212893426418304, 0.0020707319490611553, -0.007280214689671993, -0.019267285242676735, -0.028511207550764084, -0.011351315304636955, -0.010253548622131348, -0.002277899766340852, -0.023947536945343018, -0.009798897430300713, 0.02319190837442875, -0.02510075643658638, -0.052871622145175934, -0.004379321355372667, 0.017194468528032303, -0.021289929747581482, 4.511755832936615e-05, -0.013633206486701965, 0.009070592932403088, -0.002302271779626608, 0.01542308647185564, -0.015020222403109074, 0.003437163308262825, -0.008964592590928078, 0.015777045860886574, 0.015450268983840942, 0.0029461034573614597, 0.01899658888578415, 0.007095722481608391, -0.008504245430231094, 0.01927993632853031, -0.0058064828626811504, 0.006208889186382294, 0.00284641538746655, -1.816429357859306e-05, -0.021691810339689255, -0.014111047610640526, -0.019330034032464027, 0.007639887277036905, -0.0023505771532654762, 0.030282972380518913, -0.008394541218876839, 0.020482469350099564, -0.0004626007576007396, -0.00039429456228390336, 0.015228299424052238, 0.024478336796164513, 0.011053920723497868, 0.028988460078835487, 0.0032947887666523457, 0.015337608754634857, -0.0041770897805690765, 0.028497159481048584, -0.007649855222553015, 0.01659230887889862, -0.021463187411427498, -0.0008898132364265621, 0.019892387092113495, -0.0071233841590583324, 0.013552695512771606, -0.006656463257968426, -0.007129342295229435, 0.0086144944652915, -0.012063361704349518, 0.022804688662290573, -0.01528972014784813, 0.02310345135629177, -0.009008262306451797, 0.0235008392482996, -0.0017054459312930703, -0.0008415256743319333, -0.015939870849251747, -0.006499995477497578, 0.011600925587117672, -0.017873523756861687, -0.0035586573649197817, 0.023855341598391533, -0.013128659687936306, -0.0018592013511806726, 0.009256934747099876, -0.0017534507205709815, -0.016997341066598892, -0.007682705298066139, -0.011393635533750057, 0.007517698220908642, -0.02004385180771351, -0.007044267375022173, -0.0009246181580238044, 0.030682789161801338, -0.01446966826915741, -0.005653560161590576, 0.0026935229543596506, -0.011547203175723553, 0.0075895413756370544, 0.00164568773470819, -0.007723810616880655, 0.006610256619751453, -0.018564727157354355, 0.005180854815989733, -0.0051748561672866344, 0.02474612556397915, -0.008606484159827232, 0.018338078632950783, 0.013753552921116352, 0.014696115627884865, -0.019493775442242622, 0.00782967358827591, 0.01862955652177334, 0.008631521835923195, -0.011061522178351879, -0.02474864572286606, -0.026630019769072533, -0.019714033231139183, -0.029417628422379494, 0.019269073382019997, 0.010673441924154758, 0.02563672699034214, 0.0040639531798660755, -0.0055110822431743145, 0.002759817987680435, -0.007196258753538132, 0.0017267093062400818, -0.014302384108304977, -0.0055408482439816, -0.020269569009542465, 0.008005211129784584, 0.0036043955478817225, 0.01432893332093954, -0.012100206688046455, 0.006494155153632164, 0.02531949058175087, -0.020709404721856117, 0.0028371671214699745, 0.026041897013783455, -0.006666043773293495, -0.0017509728204458952, -0.015659121796488762, -0.01103716716170311, 0.0003982192138209939, 0.026317009702324867, -0.018084049224853516, 0.012571120634675026, 0.009550453163683414, 0.007068605627864599, 0.03603328764438629, -0.008795413188636303, -0.01275001373142004, -0.0002779025526251644, 0.0005754769663326442, 0.011469341814517975, 0.006001724395900965, -0.009291849099099636, -0.01687118224799633, -0.017793474718928337, 0.004899237770587206, -0.0030163254123181105, -0.017422296106815338}, {-0.0007734656683169305, -0.005490418057888746, -0.00947553850710392, -0.0005714368307963014, -0.023578522726893425, -0.01522116083651781, 0.0006528841331601143, -0.030659863725304604, -0.023297613486647606, -0.010912911035120487, 0.016695350408554077, -0.0007614976493641734, 0.0041679744608700275, -0.02474462427198887, 0.008607831783592701, -0.02079252526164055, 0.007931407541036606, -0.0016275059897452593, 0.004558288957923651, -0.023738602176308632, -0.021692171692848206, -0.00040599057683721185, -0.0030972326640039682, -0.03917372226715088, -0.006825135089457035, 0.005461304448544979, -0.002781420946121216, 0.013473842293024063, -0.016121437773108482, -0.017190853133797646, 0.011064632795751095, 0.008456742390990257, 0.008823205716907978, -0.01171831414103508, -0.03661000356078148, -0.012560308910906315, 0.014310857281088829, -0.015173212625086308, 0.005293028894811869, -0.009007005020976067, 0.006102500017732382, -0.0007112132734619081, 0.024729324504733086, 0.025013474747538567, -0.011915599927306175, -0.011286985129117966, 0.01446882076561451, -0.010068637318909168, 0.022488394752144814, 0.0006917097489349544, 0.010405235923826694, 0.0031596473418176174, 0.021029945462942123, -0.003365454962477088, -0.008273264393210411, -0.018047655001282692, 0.01341576874256134, 0.020388303324580193, -0.013073895126581192, 0.005771576892584562, 0.009249669499695301, -0.011791784316301346, -0.016862783581018448, -0.0007579753291793168, -0.015289741568267345, 0.005682844202965498, 0.0004580053500831127, -0.010040326043963432, -0.030524685978889465, -0.009677184745669365, 0.014961665496230125, -0.02489641308784485, -0.03392466530203819, 0.020612813532352448, -0.006494757253676653, 0.013585732318460941, 0.005468116141855717, 0.01416840311139822, -0.0016095370519906282, 0.01723678596317768, -0.013510055840015411, 0.018941372632980347, -0.016828978434205055, 0.006035689264535904, -0.008634130470454693, -0.0037072962149977684, 0.010980176739394665, -0.019952189177274704, 0.026195406913757324, -0.01319168508052826, 0.012599373236298561, 0.001162296044640243, -0.023757122457027435, -0.003674133913591504, 0.01738812029361725, 0.01422065682709217, -0.002134795533493161, 0.012422259896993637, -0.02349730208516121, -0.04293084517121315, 0.023246105760335922, -0.012434556148946285, -0.016106482595205307, -0.010410723276436329, -0.02027176506817341, -0.003046517027541995, 0.023462211713194847, -0.014421459287405014, 0.03196427971124649, -0.015252766199409962, -0.005185799673199654, -0.03407170623540878, 0.004620466846972704, -0.009277760982513428, -0.01686781458556652, 0.006342229433357716, -0.007112063001841307, 0.013457107357680798, -0.0008018855005502701, 0.010444660671055317, -0.024660680443048477, 0.0009804789442569017, 0.004896992351859808, 0.03308173269033432, 0.026527799665927887, 0.008377308957278728, -0.0030427013989537954, 0.021147027611732483, -0.021063271909952164, 0.023107275366783142, -0.02098470740020275, -0.010616674087941647, 0.007296390365809202, 0.00996690709143877, -0.01126476377248764, -0.009603272192180157, -0.007881776429712772, 0.0030911448411643505, 0.00014500734687317163, 0.020067324861884117, 0.012392861768603325, 0.019995765760540962, 0.009935496374964714, 0.019497860223054886, 0.00514821894466877, 0.017580047249794006, 0.007759423460811377, 0.037251487374305725, 0.005693904589861631, 0.017783205956220627, -0.010990572161972523, 0.032280631363391876, -0.0008889927994459867, 0.014717645943164825, -0.011446942575275898, 0.0026552157942205667, 0.009499586187303066, -0.021656015887856483, -0.0076051936484873295, -0.011629796586930752, 0.0007040800410322845, 0.0023416541516780853, -0.005884958431124687, 0.016368575394153595, 0.003801875514909625, 0.034958358854055405, -0.01432736311107874, -0.019504297524690628, -0.004393147770315409, -0.014234315603971481, 0.005136615131050348, -0.010802710428833961, 0.010326915420591831, 0.0019505280070006847, -0.019945980980992317, 0.006531441584229469, -0.0019551333971321583, -0.019201088696718216, 0.009624917060136795, 0.0018360050162300467, -0.005375401116907597, 0.002645770087838173, -0.0074601443484425545, 0.0053286561742424965, -0.02320380136370659, -0.007791001815348864, -0.002614150522276759, 0.021822614595294, -0.006860541179776192, -0.011007680557668209, -0.011319104582071304, -0.014645718038082123, 0.007406441029161215, 0.008166263811290264, 0.025415631011128426, -0.0021293286699801683, -0.010553320869803429, 0.00416965689510107, 0.003730426076799631, -0.0012946412898600101, -0.014259684830904007, -0.0033355001360177994, 0.02534487098455429, 0.026990331709384918, -0.02858905866742134, 0.0036768836434930563, 0.020633449777960777, -0.005107937380671501, -0.012204269878566265, -0.021233374252915382, -0.03965268284082413, 0.0012067236239090562, -0.016015950590372086, 0.013085825368762016, 0.01244837511330843, -0.0027919276617467403, -0.025909923017024994, -0.003917166031897068, 0.008597983978688717, -0.01874048076570034, 0.006921451538801193, -0.00812405627220869, -0.005946632474660873, -0.005946566350758076, 0.0031969966366887093, 0.004049188457429409, -0.019364401698112488, 0.0015853868098929524, 0.0079466812312603, 0.016563432291150093, 0.009658769704401493, 0.004360693506896496, 0.01590653508901596, 0.0013553195167332888, -0.001638223184272647, 0.00020839441276621073, -0.0001601901021786034, 0.005697656888514757, 0.02924988977611065, -0.021270016208291054, -0.006490189116448164, -0.007581797428429127, -0.00984554085880518, 0.021479539573192596, 0.0005236625438556075, -0.00942846480756998, -0.003650878556072712, 0.003986208699643612, 0.012279434129595757, -0.018765244632959366, -0.008985694497823715, -0.02324969880282879, -0.008540959097445011, -2.1241363356239162e-05, -0.00996354129165411, 0.002849609823897481}, {-0.007218670565634966, -0.01568148098886013, 0.012458264827728271, 0.017833324149250984, -0.04169558733701706, -0.010812084190547466, -0.010687909089028835, -0.02510705031454563, -0.020145056769251823, 0.016135500743985176, -0.006860199384391308, -0.015683604404330254, 0.01158219389617443, 0.027210325002670288, -0.006838861387223005, -0.012201427482068539, 0.04177387058734894, -0.014932607300579548, 0.012231414206326008, 0.002081113401800394, 0.009683896787464619, -0.013181940652430058, 0.0038695847615599632, -0.026329416781663895, -0.010619055479764938, -0.0015015698736533523, -0.010344159789383411, -0.0015939754666760564, -0.006987718865275383, -0.019094431772828102, 0.00951456930488348, 0.009174203500151634, 0.0056772539392113686, 0.005898584611713886, -0.011653445661067963, -0.014634153805673122, 0.0022039858158677816, -0.029208801686763763, 0.00903764832764864, -0.029294755309820175, -0.0029242313466966152, 0.012185567058622837, 0.02413659170269966, -0.0013345809420570731, -0.015197142027318478, -0.00093869399279356, 0.0069100200198590755, -0.01520485244691372, 0.029880698770284653, -0.00435820734128356, -0.008885391987860203, -0.0027048506308346987, 0.010473527014255524, -0.005390673875808716, 0.01211486104875803, -0.003079089568927884, -0.0010416824370622635, 0.0014760455815121531, -0.026967383921146393, -0.01437467336654663, -0.005886764265596867, -0.005947364494204521, -0.006710287183523178, -0.0065001994371414185, -0.017155809327960014, -0.008915681391954422, 0.02144690789282322, -0.02507402002811432, -0.015590730123221874, -0.0011310753179714084, 0.029125647619366646, 0.0004425654769875109, 0.005339845083653927, -0.0005886115250177681, -0.005008078180253506, 0.016252363100647926, -0.00733210239559412, 0.025181656703352928, -0.004562194459140301, 0.0031030771788209677, -0.017663544043898582, -0.0012903197202831507, -0.004101820755749941, 0.03406586870551109, -0.033437538892030716, 0.0032173290383070707, 0.011606207117438316, -0.02223675698041916, 0.02606823295354843, -0.004001236986368895, -0.004664918873459101, -0.008015819825232029, -0.0180053748190403, 0.00184432917740196, 0.00033672808785922825, 0.028930656611919403, -0.006855836138129234, -0.017318274825811386, -0.01196317095309496, -0.024337690323591232, 0.00645692041143775, 0.0005735213635489345, -0.01596640981733799, -0.009649903513491154, 0.0036458836402744055, -0.017530951648950577, 0.013524573296308517, 0.007333430461585522, -0.006169395055621862, -0.0031890145037323236, -0.012244493700563908, -0.025209181010723114, -0.010247420519590378, -0.012221485376358032, 0.004682606551796198, 0.00020054866035934538, -0.0025227065198123455, 0.04090670123696327, -0.008085394278168678, 0.011710822582244873, -0.016202658414840698, -0.014695077203214169, -0.002112537855282426, 0.013862130232155323, 0.012726728804409504, -0.012852894142270088, 0.0010098840575665236, 0.025330975651741028, -0.010722089558839798, 0.018513202667236328, -0.00790420826524496, -0.012240240350365639, 0.011445877142250538, 0.011203229427337646, -0.012834696099162102, 0.006251109763979912, -0.007632527034729719, 0.006084330379962921, 0.013413209468126297, 0.001300170086324215, 0.030488822609186172, 0.01392568089067936, 0.008336595259606838, 0.015057381242513657, 0.002787022851407528, 0.014874817803502083, 0.014326614327728748, 0.011164886876940727, 0.00689765764400363, 0.02045426517724991, -0.02107006125152111, 0.03072875179350376, -0.006674540229141712, 0.002546799834817648, -0.028229624032974243, -0.004668689798563719, 0.025488076731562614, -0.0031871998216956854, -0.0016015778528526425, -0.010264049284160137, 0.01219701487571001, 0.013083559460937977, -0.01909875124692917, 0.009188299998641014, -0.010040133260190487, 0.03921625018119812, -0.00597385922446847, 0.006880535744130611, -0.01876199245452881, 0.005730913951992989, -0.010907081887125969, -0.014804817736148834, 0.00019956407777499408, 0.007500539533793926, -0.021929439157247543, 0.018964501097798347, 0.010077730752527714, -0.0086882458999753, -0.002971107605844736, -0.03466314822435379, 0.0030647609382867813, 0.00847618468105793, -0.015903526917099953, -0.004778638947755098, -0.02438976801931858, -0.0036349561996757984, 0.0052380007691681385, 0.0317259356379509, -0.0009593376307748258, -0.010022792033851147, 0.013784892857074738, 0.024106450378894806, 0.031291812658309937, -0.01486590038985014, 0.03216289356350899, 0.025561604648828506, -0.0070957294665277, 0.025299711152911186, -0.005603605881333351, 0.00418867589905858, -0.0026159952394664288, -0.005507958587259054, -0.005409972742199898, 0.018757766112685204, -0.0004460324125830084, 0.014043631963431835, 0.010652480646967888, 0.012208147905766964, -0.0022551321890205145, -0.018271619454026222, -0.02461710199713707, -0.000924963504076004, -0.01752949319779873, 0.00742710568010807, 0.01192639023065567, -0.0014920916873961687, -0.024152997881174088, -0.013501428999006748, -0.01123570092022419, -0.00767122907564044, 0.007512019481509924, -0.008356187492609024, 0.005853721406310797, 0.001859417650848627, 0.006380770821124315, 0.0032335547730326653, 0.012281696312129498, 0.002026994014158845, 0.01949983835220337, -0.005570829380303621, 0.0009505194029770792, 0.01161486841738224, 0.024498775601387024, 0.012507880106568336, -0.02998238243162632, -0.009293233044445515, -0.019439324736595154, 0.012442429549992085, 0.030842889100313187, -0.015882156789302826, 0.021218644455075264, -0.004558404441922903, -0.006011990364640951, 0.01433173380792141, 0.0017002944368869066, -0.020248612388968468, 0.0038149221800267696, 0.012256445363163948, 0.007475745398551226, -0.012821991927921772, -0.0053976369090378284, -0.02145528607070446, -0.006273518316447735, 0.022310009226202965, 0.005069062579423189, -0.014562287367880344}, {-0.003953819163143635, -0.00518829096108675, 0.0015512278769165277, -0.003637762740254402, -0.018117759376764297, -0.010837526991963387, -0.004447766579687595, -0.024329211562871933, -0.01635618880391121, 0.013162839226424694, -0.01676257885992527, 0.005492108408361673, 0.020181726664304733, -0.004194511566311121, -0.006641428917646408, 0.002078950172290206, 0.017054613679647446, -0.012355325743556023, -0.00012712727766484022, -0.015622477047145367, -0.006009342614561319, -0.01881234347820282, -0.006921293213963509, -0.0381021723151207, 0.014100818894803524, -0.0011702569900080562, -0.011219841428101063, -0.0024854487273842096, -0.02357598952949047, 0.0017119431868195534, 0.017103606835007668, 0.008077915757894516, 0.01604704186320305, 0.014192604459822178, -0.02408718131482601, 0.012062515132129192, -0.016270697116851807, -0.021870877593755722, 0.012780897319316864, -0.03902626037597656, -0.002020140178501606, -0.0016695780213922262, 0.010298029519617558, 0.005783008877187967, -0.01313758734613657, -0.023481637239456177, 0.007511570118367672, 0.0008178458083420992, -0.002069595968350768, 0.016308996826410294, -0.0012731143506243825, -0.005303563084453344, 0.015133678913116455, -0.004776765126734972, -0.006501868832856417, -0.027762848883867264, -0.002255240920931101, 0.007833335548639297, -0.03823237121105194, -0.02913389541208744, 0.026890112087130547, -0.005783036816865206, -0.012803733348846436, -0.0058152033016085625, -0.0017359282355755568, 0.01044000219553709, 0.022843709215521812, -0.010002263821661472, -0.022509321570396423, -0.02397231012582779, 0.004650591406971216, 0.009388613514602184, -0.012985809706151485, 0.02980702742934227, -0.008122017607092857, 0.04026881977915764, -0.00014298185124062002, 0.02459815703332424, 0.012322887778282166, -0.012561730109155178, -0.008721779100596905, 0.007354856003075838, -0.00410126056522131, 0.03247205168008804, -0.02441263385117054, -0.0002530492492951453, 0.015012615360319614, -0.014045779593288898, 0.020398074761033058, -0.023417571559548378, -0.0012007427867501974, -0.015448757447302341, 0.002036241116002202, -0.012194025330245495, -0.012055665254592896, 0.02031560242176056, -0.016458634287118912, -0.020525066182017326, -0.021543394774198532, -0.004684017971158028, 0.008508262224495411, 0.007632412947714329, -0.010205832310020924, -0.002878326689824462, -0.0038426124956458807, -3.9081402064766735e-05, 0.008925686590373516, 0.006456822156906128, 0.0006107789813540876, -0.012213954702019691, -0.03187599033117294, -0.034786563366651535, -0.002561873523518443, -0.01619853638112545, -0.01488733571022749, 0.02600627951323986, -0.03179733827710152, 0.02544385753571987, -0.020300960168242455, 0.03555619716644287, -0.010506294667720795, -0.005972953978925943, 0.0024807641748338938, 0.027939094230532646, 0.010068888776004314, 0.002980975667014718, 0.0060747847892344, 0.006093479227274656, -0.003988845273852348, 0.001059897942468524, 0.008079097606241703, -0.02951628342270851, 0.0026727335061877966, -0.002843348076567054, -0.0064576235599815845, -0.0005975530948489904, -0.002554420381784439, 0.0076915426179766655, 0.00609572883695364, 0.013430585153400898, 0.014446648769080639, 0.020464394241571426, -0.001344122109003365, 0.016147591173648834, 0.0028958716429769993, 0.0010465405648574233, 0.003284506034106016, 0.008717996068298817, -0.005085369572043419, 0.008163521997630596, -0.0154649643227458, 0.0330178365111351, -0.019669122993946075, 0.0014866210985928774, -0.02521604485809803, 0.0069023133255541325, 0.009210461750626564, -0.01177884265780449, -0.008015447296202183, -0.02719142846763134, -0.008043413050472736, 0.002114500617608428, -0.0038571804761886597, 0.03561687096953392, -0.01078538317233324, 0.028891252353787422, 0.00881256815046072, 0.004147701896727085, 0.00981960166245699, 0.01679459773004055, 0.01894514262676239, 0.007809364702552557, 0.014116030186414719, -0.002221587812528014, -0.014621521346271038, -0.0028598576318472624, -0.011312931776046753, -0.024820975959300995, 0.01846674643456936, -0.0128769027069211, 0.007181345950812101, -0.007278354838490486, -0.0002222079929197207, 0.001497414312325418, -0.03664124384522438, -0.0002987227344419807, -0.002884254092350602, 0.02612455189228058, 0.0014621512964367867, -0.01730618253350258, -0.01780163124203682, -0.009356528520584106, -0.0023174365051090717, -0.0038529098965227604, 0.00404988182708621, 0.012599043548107147, -0.025702079758048058, 0.00503816781565547, -0.005193590652197599, 0.018077239394187927, -0.015647338703274727, -0.011768457479774952, 0.007789638824760914, 0.005932907108217478, 0.001956467516720295, 0.029551614075899124, 0.005663440562784672, 0.010842089541256428, -0.005737829487770796, -0.014938636682927608, -0.013061639852821827, 6.619384657824412e-05, -0.0076991962268948555, 0.007431713864207268, 0.03459123894572258, 0.003699440509080887, -0.017269229516386986, 0.0028806577902287245, -0.014605335891246796, -0.00948887038975954, 0.00013672361092176288, 0.0009289517183788121, 0.007650021929293871, -0.0028847611974924803, -0.003611050546169281, 0.01482043694704771, -0.010185805149376392, 0.00454570772126317, 0.013524878770112991, 0.02960103563964367, -0.008319920860230923, -0.014519138261675835, 0.023563707247376442, -0.0028574271127581596, 0.0069935936480760574, -0.01857582852244377, 0.006633406970649958, -0.0015108861261978745, 0.030941927805542946, -0.018940431997179985, -0.00903186947107315, -0.00047483405796810985, 0.011836876161396503, 0.02599669247865677, -0.00026998051907867193, -0.020923947915434837, 0.0041482821106910706, 0.015313589945435524, -0.004038582090288401, -0.0025928185787051916, -0.016167286783456802, -0.011486747302114964, 0.0013606393476948142, 0.013573480769991875, -0.002304790075868368, -0.01066178921610117}, {0.013548114337027073, 0.002349619287997484, 4.512706436798908e-05, 0.008685036562383175, -0.01642652601003647, 0.003947135992348194, -0.030783336609601974, -0.025350941345095634, -0.03962798789143562, -0.011375293135643005, -0.009440778754651546, -0.017476297914981842, 0.0008096218225546181, -0.0032091005705296993, -0.008405533619225025, -0.011085454374551773, 0.02035217359662056, -0.025344718247652054, 0.004713785834610462, -0.007992890663444996, -0.002820125548169017, -0.012288716621696949, -0.020838722586631775, -0.014940261840820312, 0.01534164510667324, 0.0004833546990994364, -0.006453178357332945, 0.022044619545340538, 0.005198986269533634, -0.0016469369875267148, 0.006365957669913769, -0.005389452446252108, 0.005785189103335142, 0.00904049538075924, -0.01071125641465187, 0.009290608577430248, 0.019660385325551033, -0.023722995072603226, -0.004972824826836586, -0.04718027636408806, 0.017176935449242592, -0.010565659962594509, 0.02347061038017273, 0.018939824774861336, 0.018150966614484787, -0.011596200056374073, 0.009786786511540413, -0.013126426376402378, 0.021054379642009735, -0.013553518801927567, 0.009184522554278374, -0.009036654606461525, 0.02820858731865883, 0.0021484463941305876, 0.014968262985348701, -0.031233690679073334, -0.033826444298028946, 0.012742060236632824, -0.008327160030603409, -0.0009534505661576986, 0.002875338075682521, 0.007923616096377373, -0.003565977793186903, -0.0015317805809900165, -0.012731566093862057, 0.019351888447999954, -0.007698386441916227, -0.034290820360183716, 0.00615000631660223, -0.00014107234892435372, 0.009938512928783894, 0.004193222615867853, -0.01599346287548542, 0.0028031636029481888, -0.006620591040700674, 0.01828903704881668, 0.0012507436331361532, 0.019581040367484093, -0.0020207869820296764, -0.0004368203808553517, 0.011125942692160606, -0.00560831930488348, -0.0030055392999202013, 0.014682943001389503, -0.013604866340756416, -0.021691976115107536, 0.00579766696318984, -0.019849540665745735, 0.020413199439644814, -0.005371231585741043, 0.01060163788497448, 0.0007700991700403392, -0.013014783151447773, -0.01617325469851494, 0.013482260517776012, 0.013375784270465374, -0.012038643471896648, -0.011314496397972107, -0.020686281844973564, -0.011594608426094055, 0.025973279029130936, -0.0102120041847229, 0.0014275462599471211, -0.008469738066196442, -0.0036873766221106052, -0.036382950842380524, 0.014258931390941143, -0.011370887979865074, 0.007798435166478157, -0.0023355246521532536, -0.006413205526769161, -0.03914416953921318, 0.011797037906944752, -0.008085249923169613, -0.0020727484952658415, -0.009015281684696674, -0.026411110535264015, 0.004863023292273283, -0.01764661818742752, 0.014011603780090809, -0.01291246060281992, -0.025351310148835182, -0.011841868050396442, 0.030361084267497063, 0.0013440167531371117, 0.0014407612616196275, -0.012540465220808983, 0.033095601946115494, -0.003629879327490926, 0.005947655998170376, 0.006325423717498779, -0.006777734961360693, 0.013622038066387177, -0.004573763348162174, -0.007128434721380472, -0.010612500831484795, -0.02208610624074936, -0.01862046681344509, 0.00898987241089344, 0.0012098719598725438, 0.011913564056158066, 0.019082333892583847, 0.0145249729976058, -0.011039814911782742, 0.004710105247795582, 0.010180561803281307, 3.564433427527547e-05, 0.03207077458500862, 0.010672084055840969, 0.021775010973215103, -0.013544379733502865, 0.007469736970961094, -0.017409222200512886, -0.01568298414349556, -0.03169799596071243, -0.010213975794613361, 0.000392628600820899, 0.0026262581814080477, -0.009931501932442188, -0.009721728973090649, -0.0027045116294175386, 0.004182750359177589, -0.024072347208857536, 0.011047529056668282, -0.012411901727318764, 0.05902339518070221, 0.003914273343980312, -0.0022175104822963476, 0.011501702480018139, 0.007633994799107313, 0.01033641304820776, -0.007025820203125477, 0.011953750625252724, 0.01590483821928501, -0.0068223620764911175, 0.020569268614053726, 0.0065772561356425285, -0.013827460817992687, 0.010642103850841522, 0.007645400241017342, -0.004449129104614258, -0.015500614419579506, 0.001187161193229258, 0.012666944414377213, -0.014130555093288422, -0.008932574652135372, 0.007328538689762354, 0.033954210579395294, -0.005767214111983776, -0.0059980773366987705, -0.0016202564584091306, -0.009531146846711636, -0.005885991267859936, 0.0026710662059485912, 0.009602987207472324, 0.013556952588260174, -0.0013439605245366693, -0.009115531109273434, 0.0025837558787316084, 0.0270773284137249, -0.004397917073220015, -0.006041236221790314, 0.00505772465839982, 0.00966115016490221, 0.0011073242640122771, 0.027691517025232315, 0.01638895273208618, 0.004332850221544504, -0.012159885838627815, -0.020107638090848923, -0.02738294005393982, 0.008312403224408627, -0.010675588622689247, 0.013783155009150505, 0.007972920313477516, -0.005043238401412964, -0.029217546805739403, -0.013211638666689396, -0.0095677450299263, 0.011227847076952457, 0.015396658331155777, 0.00258630677126348, 3.783710053539835e-05, 0.0032431522849947214, -0.020786769688129425, -0.011157275177538395, 0.013388516381382942, 0.0034884673077613115, 0.016625072807073593, 0.00311213918030262, -0.003721612738445401, 0.0023970878683030605, 0.020874658599495888, -0.01072012074291706, -0.002225872129201889, -0.01833568699657917, -0.0024055808316916227, -0.019377458840608597, 0.04120117798447609, -0.022431332617998123, -0.002478797687217593, -0.013335470110177994, 0.0037859315052628517, 0.005770091433078051, -0.0003261589736212045, -0.003185560228303075, -0.02019874006509781, -0.001162620261311531, 0.006926380097866058, 0.0010122007224708796, -0.005920144729316235, -0.02486332878470421, -0.0018138802843168378, 0.008725881576538086, -0.024035587906837463, -0.015589534305036068}, {-0.0013083834201097488, -0.013160696253180504, 0.009471124969422817, 0.010663754306733608, -0.023961136117577553, -0.005317772272974253, 0.001276305876672268, -0.037840813398361206, -0.030052918940782547, -0.006868089083582163, -0.013883301988244057, 0.011831355281174183, -0.005524851381778717, 0.026876814663410187, -0.005910662934184074, 0.0005256737349554896, 0.03172418475151062, -0.013925082981586456, 0.011832033284008503, -0.011476223357021809, 0.0007603706326335669, -0.023970620706677437, -0.0281978789716959, -0.021403098478913307, 0.006792538799345493, 0.014926949515938759, -0.0029490103479474783, 0.01539294607937336, -0.02064857818186283, -0.022820819169282913, -0.0021365093998610973, -0.0007597892545163631, 0.0013181770918890834, 0.009058901108801365, -0.03558905050158501, 0.0041389768011868, 0.008765317499637604, -0.003591854590922594, -0.015716874971985817, -0.016724780201911926, 0.01668877899646759, -0.002460707910358906, 0.013778218999505043, 0.018372802063822746, 0.001921660965308547, -0.0015740987146273255, 0.003190841292962432, 0.0017516810912638903, 0.020719271153211594, 0.0040293806232512, -0.01209327857941389, -0.0026044542901217937, 0.03937404975295067, -0.00830327719449997, 1.0590646525088232e-05, -0.005021991673856974, -0.0033521330915391445, 0.00911992508918047, -0.007058559451252222, 0.006355233956128359, 0.003735568141564727, -0.006546306423842907, -0.015429632738232613, -0.010792426764965057, -0.009938955307006836, 0.012556413188576698, 0.023755181580781937, -0.009301675483584404, -0.002489204751327634, -0.008568506687879562, 0.015259667299687862, 0.0012004124000668526, -0.003754186676815152, 0.005864674225449562, -0.023138493299484253, 0.023028716444969177, -0.004302525892853737, 0.02523796819150448, -0.0041005900129675865, 0.021701941266655922, 0.0004970196750946343, -0.015119425021111965, -0.01031999010592699, 0.03214283287525177, -0.006476176902651787, 0.00601930869743228, -0.006832097191363573, -0.012545319274067879, 0.0190330371260643, -0.017546027898788452, -0.006097158882766962, 0.006916003301739693, -0.01932566612958908, -0.026868807151913643, 0.014445638284087181, 0.006823405623435974, -0.023816604167222977, -0.009180114604532719, -0.025531820952892303, -0.011084808968007565, 0.005021251272410154, 0.0038779587484896183, -0.0030977604910731316, -0.01140359602868557, -0.01997300423681736, -0.0057692090049386024, -0.015650995075702667, -0.009854729287326336, -0.024333149194717407, -0.013879874721169472, -0.02003341354429722, -0.04124021530151367, 0.009340081363916397, -0.008343956433236599, -0.012455779127776623, -0.003526067128404975, -0.027735358104109764, 0.014551467262208462, 0.0018112665275111794, 0.02891494892537594, -0.011447298340499401, -0.0238446407020092, 0.024292033165693283, 0.025477956980466843, 0.019554955884814262, 0.019730350002646446, 0.027514925226569176, 0.018502194434404373, -0.035111263394355774, 0.00995622482150793, -0.019983412697911263, -0.00455221114680171, 0.004565113224089146, 0.013615177012979984, -0.016889141872525215, 0.01334378495812416, -0.012485280632972717, -0.021473707631230354, 0.01922641694545746, 0.018617048859596252, 0.003073703031986952, 0.023547474294900894, -0.017318345606327057, 0.005653231870383024, 0.010479067452251911, 0.02744998224079609, -0.018359960988163948, 0.0012794863432645798, -0.0018823230639100075, -0.007296439725905657, -0.020284831523895264, 0.007644757628440857, 0.010099520906805992, 0.002217657398432493, 0.002785968594253063, -0.005481771193444729, -0.003519001416862011, -0.010306881740689278, 0.018888557329773903, -0.010165642015635967, 0.011271432042121887, -0.011315257288515568, -0.01535625010728836, 0.003414192469790578, -0.005260934587568045, 0.04885999858379364, -0.011817211285233498, -0.005959892645478249, 0.009324789978563786, -0.0006380014237947762, -0.0062426128424704075, -0.010438707657158375, 0.014934509992599487, 0.0072125596925616264, -0.027329113334417343, 0.0009158648899756372, 0.004068972542881966, -0.022202204912900925, 0.016988566145300865, -0.024049561470746994, -0.020346026867628098, 0.024296263232827187, -0.006938773673027754, -0.0028346762992441654, -0.03853903338313103, -0.03758721053600311, -0.015802836045622826, 0.003996854182332754, -0.006276958156377077, -0.004782083444297314, -0.01001307275146246, -0.003383514005690813, 0.013057581149041653, -0.004053026437759399, -0.0009741687681525946, 0.016145560890436172, -0.025509987026453018, -0.0008752525900490582, 0.004661834333091974, 0.01810638979077339, -0.023259125649929047, -0.01860889047384262, 0.014916875399649143, 0.014439363032579422, -0.004666288383305073, 0.01679198071360588, 0.019088201224803925, 0.012208744883537292, 0.008861218579113483, -0.024586739018559456, -0.03438113257288933, -0.015727374702692032, -0.02004249021410942, 0.018195681273937225, 0.0075075505301356316, 0.015398812480270863, -0.021575981751084328, -0.0034713794011622667, 0.010047540068626404, 0.009551895782351494, -0.0008783343364484608, -0.012898348271846771, -0.0043508512899279594, -0.002771850908175111, 0.01491888239979744, 0.009776012040674686, 0.004042074084281921, 0.020656051114201546, 0.003946260083466768, 0.013537030667066574, -0.0002602205495350063, -0.018030188977718353, 0.006137717980891466, -0.006780028343200684, -0.013743323273956776, 0.00936441496014595, 0.010264258831739426, -0.013615674339234829, 0.022287994623184204, -0.012318439781665802, 0.015957999974489212, 0.00162521842867136, -0.006016520783305168, 0.01669888012111187, 0.007180899381637573, -0.00878896564245224, 0.019883425906300545, 0.001324823242612183, -0.0040191104635596275, 0.009662903845310211, -0.010077104903757572, -0.024165714159607887, 0.00117534922901541, 0.013733083382248878, -0.003588865976780653, -0.011955498717725277}, {0.0012605361407622695, -0.009808849543333054, -0.0052636172622442245, 0.01942910999059677, -0.022374119609594345, -0.010482171550393105, -0.016109298914670944, -0.027570240199565887, -0.002603736473247409, 0.003668734570965171, 0.007515984587371349, 7.836845907149836e-05, -0.004031942691653967, 0.01498168520629406, 0.0021758649963885546, -0.033706795424222946, 0.005170492455363274, 0.00021952291717752814, 0.005595183931291103, -0.0020168933551758528, 0.008988280780613422, 0.002844879636541009, -0.0053876666352152824, -0.017819425091147423, -0.0044930968433618546, -0.010195557959377766, 0.00980970449745655, 0.007193438243120909, -0.010591027326881886, -0.018530527129769325, 0.014111759141087532, -0.012540411204099655, -0.019133277237415314, -0.005212123040109873, -0.037564221769571304, -0.018299303948879242, -0.011803830042481422, -0.031878117471933365, -0.013568136841058731, -0.039706356823444366, 0.019153350964188576, 0.014052781276404858, 0.032075732946395874, 0.03632912039756775, -0.02062627486884594, -0.010086989030241966, -0.011869511567056179, 0.0160346869379282, 0.02033347450196743, -0.011101913638412952, -0.005005987361073494, -0.000870438467245549, 0.02728925831615925, -0.021981131285429, 0.021452706307172775, -0.021002456545829773, -0.016440499573946, -0.015531371347606182, -0.006286997813731432, -0.029930105432868004, 0.009596203453838825, 0.005630544386804104, -0.0013361654710024595, -0.01660667546093464, -0.01726437918841839, 0.005790280178189278, 0.010561726056039333, -0.019832853227853775, -0.010568834841251373, -0.00916271936148405, 0.003612667554989457, -0.010007611475884914, -0.003860766300931573, 0.01977469027042389, 0.0009173237485811114, 0.030921200290322304, 0.0040921783074736595, 0.018901903182268143, -0.0014210426015779376, 0.013585072942078114, -0.006823234260082245, -0.008790947496891022, -0.01723909005522728, 0.03640446811914444, -0.01486689317971468, 0.00421116640791297, 0.013244370929896832, -0.01019868440926075, 0.009756138548254967, -0.022338366135954857, -0.02660554274916649, -0.0069373189471662045, 0.0076526920311152935, -0.017987024039030075, 0.014460892416536808, 0.023101627826690674, -0.0175490602850914, -0.008615581318736076, -0.021117525175213814, -0.020107783377170563, 0.015268861316144466, -0.002848314354196191, -0.006112993694841862, -0.006232354324311018, -0.019937463104724884, -0.013286448083817959, -0.007326404098421335, -0.020328115671873093, 0.0005248137749731541, -0.016473358497023582, -0.030279332771897316, -0.03577567636966705, -0.003206989960744977, -0.008094110526144505, -0.012172982096672058, -0.0030275830067694187, 0.000406023784307763, 0.0283247921615839, 0.008597740903496742, 0.021329181268811226, -0.013436385430395603, -0.018228286877274513, 0.004780035465955734, 0.02957763522863388, 0.01964171975851059, -0.014672579243779182, 0.005953257437795401, 0.020213326439261436, -0.00048023529234342277, 0.005437713582068682, -0.004133830312639475, -0.0086929090321064, -0.0023691286332905293, 0.009960561990737915, -0.017811806872487068, -1.790698297554627e-05, -0.018207145854830742, -0.0071335118263959885, 0.009709407575428486, 0.0025334148667752743, 0.005013437941670418, 0.023724889382719994, -0.006795847322791815, 0.01136196032166481, 0.01014228630810976, -0.006953386124223471, 0.009202074259519577, 0.0435352623462677, 0.005385323893278837, 0.010662155225872993, -0.0062886811792850494, 0.0030079015996307135, -0.018419304862618446, 0.008503173477947712, -0.00448300875723362, 0.003592410357668996, -0.004911893513053656, -0.019926752895116806, 0.002231242833659053, -0.013178372755646706, 0.016160842031240463, 0.0038282079622149467, -0.018421990796923637, 0.016130253672599792, -0.008119740523397923, 0.021035822108387947, -0.015179972164332867, 0.004763410426676273, -0.009690920822322369, -0.0015380226541310549, -0.0018348683370277286, -0.006006310228258371, -0.003742302069440484, -0.004828101489692926, -0.011815309524536133, 0.019403915852308273, -0.0051626237109303474, -0.005378107074648142, 0.0038974261842668056, -0.028132284060120583, 0.0024026527535170317, -0.007023109123110771, -0.0013530583819374442, 0.0007351331878453493, -0.030838921666145325, 0.022719457745552063, 0.017542792484164238, 0.028648750856518745, 0.005150462035089731, -0.01300709880888462, 0.012095728889107704, -0.0076343100517988205, 0.0037148999981582165, -0.0011843778192996979, -0.0003380125272087753, 0.021168788895010948, -0.015645982697606087, 0.012510653585195541, 0.010161631740629673, 0.006183795165270567, -0.003670232370495796, 0.0017824708484113216, 0.005288219545036554, 0.007377722766250372, -0.0013473372673615813, 0.005667729303240776, 0.006401468999683857, -0.008195766247808933, -0.0033831738401204348, -0.02315874956548214, -0.02861236035823822, 0.006630929186940193, -0.02381361648440361, 0.014002921059727669, 0.01940576732158661, 0.0030688163824379444, -0.01693594828248024, 0.0063420990481972694, 0.002804983640089631, -0.0068880910985171795, 0.008105387911200523, -0.004091225564479828, 0.01619456522166729, -0.014549112878739834, 0.003247014945372939, 0.02418483793735504, 0.0032599284313619137, 0.03255447745323181, 0.0018705078400671482, 0.013689382001757622, -0.007328242529183626, -0.00673460541293025, 0.010130749084055424, -0.012809963896870613, -0.0019302036380395293, -0.010715405456721783, 0.01219385489821434, -0.002222284907475114, 0.02730867825448513, -0.015077849850058556, 0.031164200976490974, 0.01098919752985239, -0.0032692195381969213, 0.024342218413949013, -0.002903462154790759, -0.010281223803758621, 0.004728496540337801, -0.010130717419087887, 0.011859186924993992, -0.011379720643162727, -0.012979989871382713, -0.029571859166026115, -0.0004963262472301722, 0.008770562708377838, -0.0009745703428052366, -0.011855043470859528}, {0.011375932954251766, -0.006963755004107952, -0.004522410687059164, 0.012954480946063995, -0.021127430722117424, 0.013565192930400372, -0.005130276083946228, -0.027765359729528427, -0.01697680354118347, 0.010549851693212986, 0.002991247223690152, 0.00973657425493002, -0.011674481444060802, -0.00353240012191236, 0.005721484310925007, -0.012944674119353294, 0.016991794109344482, -0.006176207680255175, 0.010781599208712578, -0.027426840737462044, -0.010263221338391304, -0.020650146529078484, -0.008886124938726425, -0.022470753639936447, -0.008237998932600021, -0.011149091646075249, 0.007957510650157928, 0.010706329718232155, -0.009233467280864716, -0.010087912902235985, 0.017165180295705795, 0.006578849628567696, 0.009536483325064182, 0.008587464690208435, -0.009303483180701733, 0.001152705866843462, 0.02217787504196167, -0.007461274974048138, -0.0031581646762788296, -0.018189379945397377, 0.0008580450667068362, -0.011103110387921333, 0.0028746933676302433, 0.0036261610221117735, 0.0012674741446971893, -0.00882070790976286, -0.010037370957434177, -0.01892400160431862, 0.022406911477446556, -0.001814347691833973, 0.003047144738957286, 0.008321482688188553, 0.019312886521220207, 0.010930721648037434, 0.0003444908943492919, -0.012263447977602482, 0.016974961385130882, -0.015469896607100964, -0.01194717176258564, -0.01616993173956871, 0.007397579960525036, 0.00022775634715799242, 0.002644449006766081, 0.01101300586014986, -0.008851340971887112, 0.017794571816921234, 0.015247181989252567, -0.008505315519869328, -0.013856102712452412, -0.007182313594967127, -0.0031734465155750513, -0.014202987775206566, 0.003999718464910984, 0.021160250529646873, -0.01641816832125187, 0.01815471053123474, -0.002908000722527504, 0.0189492329955101, -0.002205357188358903, 0.003975339233875275, 0.009114044718444347, -0.0021992071997374296, -0.018190965056419373, 0.026127513498067856, -0.03431505337357521, 0.01223421934992075, 0.02712821215391159, -0.02134537138044834, 0.015308023430407047, -0.005023417994379997, -0.0049869827926158905, -0.0046619391068816185, -0.00894088763743639, -0.030964041128754616, -0.009887111373245716, 0.022599652409553528, -0.03451535105705261, -0.005577977281063795, -0.0107286861166358, -0.01223963126540184, 0.020293347537517548, 0.015199552290141582, 0.00049161771312356, 0.0038355656433850527, -0.005880431737750769, -0.010916199535131454, 0.00041333254193887115, -0.007726604584604502, -0.006102506536990404, -0.0007488297997042537, -0.017737388610839844, -0.0473579503595829, 0.011162380687892437, -0.021359674632549286, 0.0009306726860813797, 0.004034773446619511, -0.021070223301649094, 0.01590690203011036, -0.007399085909128189, 0.0025990460999310017, -0.021350426599383354, -0.005554649047553539, -0.01027799490839243, 0.012578506022691727, 0.007519719190895557, 0.005700487177819014, 0.0028473809361457825, 0.02808806300163269, -0.038733694702386856, 0.02650376968085766, -0.035072773694992065, -0.004328079521656036, 0.008231885731220245, 0.013695718720555305, -0.011159295216202736, 0.0020751189440488815, -0.027690574526786804, -0.013738742098212242, -0.0004393563140183687, 0.004486306570470333, 0.009231258183717728, 0.01751786284148693, -0.0017657192656770349, 0.020969897508621216, 0.007987108081579208, -0.006399674341082573, -0.009710234589874744, 0.009285569190979004, 0.015177719295024872, -0.0003177251492161304, -0.012060025706887245, 0.02025994285941124, 0.001991095021367073, 0.011703422293066978, -0.013511700555682182, -0.0052256337366998196, 0.01171258743852377, -0.019767725840210915, 0.010561192408204079, -0.0038374357391148806, -0.01305962260812521, 0.02700253203511238, -0.01287760678678751, 0.0183488130569458, 0.007653963752090931, 0.040770888328552246, -0.024620510637760162, -0.016372809186577797, -0.0011763459770008922, 0.013456870801746845, -0.000266788323642686, -0.0013562769163399935, 0.01519982609897852, 0.0024531055241823196, -0.0029671837110072374, 0.010493370704352856, 0.0038731226231902838, -0.02571329101920128, 0.008719019591808319, 0.013873713091015816, -0.028571246191859245, 0.0013964319368824363, -0.010500130243599415, -0.0067561413161456585, -0.019754942506551743, -0.02142925001680851, 0.0073570734821259975, 0.02271585911512375, -0.005681857466697693, -0.007844706065952778, 0.0017417469061911106, -0.020442798733711243, 0.01084974780678749, -0.0021828473545610905, 0.009948331862688065, 0.004680561367422342, -0.030810588970780373, 0.014586633071303368, 0.01047779992222786, 0.010057410225272179, -0.010821063071489334, -0.014672422781586647, 0.0011395602487027645, 0.021160997450351715, 0.0032834187150001526, 0.02795635350048542, -0.00453480938449502, 0.0128238620236516, -0.007855531759560108, -0.018054330721497536, -0.01925450935959816, 0.003971032798290253, -0.036638736724853516, 0.009421642869710922, 0.025604041293263435, -0.005514000542461872, -0.008905906230211258, 0.022022360935807228, -0.004786811303347349, -0.005858795717358589, 0.0006015335093252361, -0.004205174278467894, -0.017667528241872787, 0.0033373446203768253, 0.0019569292198866606, 0.002093746792525053, -0.0026689483784139156, -0.004381235688924789, 0.015419313684105873, -0.006996101699769497, -0.01336812973022461, -0.005020002368837595, 0.0037393970414996147, 0.003671047044917941, -0.01168212853372097, -0.007543567102402449, 0.015621872618794441, -0.0198014248162508, 0.013805055059492588, -0.015276470221579075, 0.005313726142048836, 0.011379570700228214, -0.00600552698597312, 0.012942761182785034, -0.007071933709084988, -0.00896715559065342, -0.011780847795307636, 0.011431684717535973, 0.011695323511958122, 0.009204708971083164, 0.014362312853336334, -0.022708402946591377, -0.0025058321189135313, 0.0014450507005676627, 0.0028388830833137035, -0.008610500022768974}, {-0.0054435450583696365, -0.025513004511594772, -0.010360530577600002, 0.017091110348701477, -0.035942383110523224, -0.009646694175899029, -0.017853999510407448, -0.022665396332740784, -0.026300176978111267, -0.002252867678180337, 0.005075495224446058, -0.0016521160723641515, -0.0070958747528493404, 0.0024278764612972736, 0.00218875240534544, 0.004804147407412529, 0.01960841193795204, -0.01993374153971672, -0.002231104066595435, -0.006431649439036846, -0.016489306464791298, -0.01932699978351593, -0.01428291667252779, -0.007381804753094912, -0.019942723214626312, -0.0032439338974654675, -0.014398135244846344, 0.01603214628994465, 0.00035055517219007015, -0.022568393498659134, 0.026267586275935173, 0.008882586844265461, -0.0018134963465854526, 0.024647248908877373, -0.010079422034323215, -0.0059768459759652615, -0.01664053276181221, -0.009422915056347847, 0.0003177992766723037, -0.031139878556132317, -0.009144578129053116, -0.0025233726482838392, 0.017493372783064842, 0.03697824105620384, -0.004437917843461037, 0.009152336977422237, -0.006400708109140396, 0.0022842902690172195, 0.014165034517645836, -0.013155952095985413, -0.017680786550045013, -0.006637589540332556, 0.031674474477767944, 0.009094138629734516, -0.01618783362209797, -0.010803623124957085, 0.01879291608929634, 0.010648315772414207, -0.01424841582775116, -0.012136491015553474, -0.011524700559675694, -0.005546045023947954, 0.0012236511101946235, 0.010627195239067078, -0.01254358422011137, 0.012782598845660686, 0.014230472967028618, -0.009369865991175175, -0.005620141513645649, -0.0009428352932445705, 0.0014640262816101313, -0.00521579897031188, 0.014577025547623634, 0.016228096559643745, 0.005378067027777433, 0.029428882524371147, -0.009706336073577404, 0.014392728917300701, 0.013448022305965424, 0.010827861726284027, -0.005205952562391758, 0.003074342617765069, -0.0011308400426059961, 0.01207792479544878, -0.015710817649960518, 0.002432262059301138, 0.012927964329719543, 0.006718978751450777, -0.0025620197411626577, -0.030555913224816322, -0.022330274805426598, 0.015623235143721104, -0.02159399725496769, -0.00337759661488235, 0.003184480592608452, 0.016945403069257736, -0.01758449710905552, -0.009151180274784565, -0.0031330992933362722, -0.007463580928742886, 0.0056200227700173855, -0.00741611048579216, 0.004716329742223024, -0.01611967757344246, -0.004747945815324783, 0.0018913050880655646, 0.001306757447309792, -0.013490591198205948, -0.005796230863779783, -0.010772483423352242, -0.004741578362882137, -0.030913220718503, -0.012403305619955063, 0.015758125111460686, -0.010237427428364754, -0.008528869599103928, -0.009181400761008263, 0.03958341106772423, -0.0008540782728232443, 0.013563316315412521, -0.012481335550546646, -0.02806061878800392, 0.012993590906262398, 0.020238598808646202, 0.01755426824092865, 0.010434778407216072, -0.003755384124815464, 0.011395987123250961, 0.004214525688439608, 0.017119694501161575, -0.025422079488635063, -0.008971527218818665, 0.0025547142140567303, 0.020458346232771873, -0.01789914257824421, -0.007365835830569267, -0.04345357045531273, -0.01771528832614422, 0.013944584876298904, 0.004537422209978104, 0.01156388409435749, 0.02873125858604908, -0.008108587935566902, 0.014039627276360989, 0.0027094311080873013, 0.01679915376007557, -0.00034839569707401097, 0.015701202675700188, -0.006203855387866497, 0.0028972390573471785, -0.011172995902597904, 0.012714646756649017, -0.010147362016141415, 0.028948640450835228, -0.025847075507044792, -0.0138883450999856, -0.0036925822496414185, -0.021061602979898453, -0.0001249217166332528, -0.008152959868311882, -0.02274981141090393, 0.011329876258969307, -0.0020765247754752636, 0.0019500714261084795, 0.006068309303373098, 0.026578545570373535, -0.0028168426360934973, 0.006368707399815321, 0.00739717110991478, 0.007937546819448471, 0.012325362302362919, 0.014865115284919739, -0.010772128589451313, 0.01730223558843136, -0.01591583900153637, 0.008964324370026588, 0.007797428406774998, -0.01733970083296299, 0.008994550444185734, -0.021706806495785713, -0.002136676572263241, -0.00684770755469799, -0.015711965039372444, 0.006739391945302486, -0.026014814153313637, -0.004527375567704439, -0.0033989630173891783, 0.020095769315958023, -0.009710886515676975, -0.030802704393863678, 0.011623088270425797, -0.00634390814229846, 0.01771627552807331, -0.010622205212712288, -0.005900074727833271, 0.02056148834526539, -0.011045478284358978, 0.002977431286126375, -0.0007888980326242745, 0.021008631214499474, -0.01136629469692707, 0.0035320711322128773, -0.005138910375535488, 0.006839372217655182, -0.006274539511650801, 0.00842499639838934, -8.269448153441772e-05, -0.003821056801825762, -0.004329843912273645, -0.026327000930905342, -0.03841489180922508, -0.002311665564775467, -0.03142239525914192, 0.0210806205868721, 0.015138238668441772, -0.004060560371726751, -0.03644565865397453, -0.006072614341974258, -0.0026378328911960125, 0.0016366466879844666, 0.00680495984852314, 0.0021744421683251858, 0.000439280120190233, -0.0024861141573637724, -0.011190964840352535, -0.0042950487695634365, -0.0017950284527614713, -0.0023566451855003834, 0.010047032497823238, 0.016392813995480537, -0.00525240320712328, 0.0023554121144115925, -0.0007928049308247864, 0.006491573061794043, -0.022034570574760437, 0.008033426478505135, -0.0017634938703849912, 0.007568819914013147, 0.031628143042325974, -0.0083471005782485, -0.0029323094058781862, 0.013335827738046646, 0.004467277321964502, 0.021224144846200943, -0.005471230484545231, -0.00011570886272238567, -0.018544096499681473, -0.006737144198268652, 0.005783294793218374, -0.003526256885379553, 0.01364697702229023, -0.026637427508831024, -0.0077468934468925, 0.023283179849386215, -0.00873086228966713, -0.012186035513877869}, {-0.00017718783055897802, -0.004258126951754093, -0.0027601371984928846, 0.018893729895353317, -0.03474640101194382, -0.028403131291270256, -0.008765638805925846, -0.016149073839187622, -0.028726862743496895, 0.007964742369949818, -0.0014612345257773995, 0.0001582913100719452, 0.014479399658739567, 0.025498531758785248, 0.012726761400699615, 0.0006878278218209743, 0.02738363854587078, -0.0030213487334549427, 0.0010751431109383702, -0.013553580269217491, 0.02117273584008217, -0.008578591048717499, -0.012786595150828362, -0.012315903790295124, 0.009101827628910542, -0.005151150748133659, -0.0039941794238984585, 0.012186000123620033, -0.009620662778615952, -0.028056614100933075, 0.020633619278669357, -0.001571577973663807, 0.01154930517077446, 0.011642452329397202, -0.010652347467839718, 0.00013109573046676815, 0.02026952989399433, -0.018304377794265747, -0.01359606347978115, -0.026718787848949432, 0.014512457884848118, -0.006873400416225195, 0.010276572778820992, 0.011743069626390934, 0.004149206914007664, -0.008425804786384106, -0.007065509911626577, 0.00862186774611473, 0.004043411463499069, -0.00961528904736042, -0.0008776902104727924, 0.0015863666776567698, 0.0337650440633297, -0.006900092586874962, 0.010455222800374031, -0.026307646185159683, -0.022129520773887634, 0.020119713619351387, -0.031523216515779495, -0.014520746655762196, -0.003493823576718569, 0.001237225253134966, -0.01933550275862217, -0.007722363341599703, -0.005055129062384367, 0.003344191238284111, 0.013316689990460873, -0.00132177141495049, -0.014928395859897137, -0.010253209620714188, 0.0033063285518437624, -0.012256919406354427, 0.0031059598550200462, 0.011269419454038143, -0.011591235175728798, 0.0074934870935976505, -0.019467636942863464, 0.0360906645655632, -0.0051576984114944935, 0.0009699970833025873, -0.010229569859802723, -0.029520975425839424, -0.023937180638313293, 0.035442523658275604, -0.0235726498067379, -0.002534229075536132, 0.02560858055949211, -0.007972030900418758, 0.022332532331347466, -0.02557063288986683, 0.004818096291273832, 0.004882661625742912, -0.008864960633218288, -0.024264413863420486, -0.0009444041643291712, 0.011926027946174145, -0.029179321601986885, 0.0017314368160441518, -0.017919467762112617, 0.003470492083579302, 0.031271692365407944, 0.009270199574530125, -0.012295764870941639, -0.003377222456037998, -0.006971842609345913, -0.002266154857352376, -0.01152804959565401, -0.027195388451218605, 0.014889468438923359, -0.010863118804991245, 0.0011246033245697618, -0.020429560914635658, 0.004489429295063019, -0.016264911741018295, -0.008701203390955925, -0.00759353069588542, -0.0010971586452797055, 0.015258069150149822, 0.0013935862807556987, 0.019164294004440308, -0.016911815851926804, -0.012845522724092007, 0.013843124732375145, 0.012053843587636948, 0.011406125500798225, 0.017038192600011826, 0.0038622780703008175, 0.0029982656706124544, -0.014305477030575275, 0.019010022282600403, -0.025973105803132057, -0.007840470410883427, 0.013124577701091766, -0.00020004474208690226, -0.02315085381269455, -0.012517298571765423, -0.025300268083810806, -0.019748007878661156, 0.005895541049540043, 0.00897401012480259, 0.017229359596967697, 0.015753164887428284, -0.00962945818901062, 0.021829498931765556, 0.003538905642926693, 0.011156593449413776, 0.004603748209774494, 0.027289243414998055, 0.006675090175122023, 0.006841599475592375, -0.007738530170172453, 0.017407942563295364, -0.0153201287612319, -0.0008063142304308712, -0.022052312269806862, -0.008527963422238827, 0.021219858899712563, -0.03064122423529625, 0.015534781850874424, -0.0023036685306578875, -0.0016795751871541142, -0.0033877224195748568, -0.017752043902873993, 6.188992119859904e-05, -0.004166669212281704, 0.04950666055083275, -0.0075967684388160706, 0.009888140484690666, 0.015192861668765545, -0.019397282972931862, 0.002519618719816208, 0.003522699000313878, -0.011815295554697514, -0.006952325347810984, -0.01611490733921528, 0.017167188227176666, 0.0006073208060115576, -0.008213328197598457, -0.00029510652530007064, -0.005309045314788818, -0.0018059969879686832, -0.001928903628140688, -0.011259551160037518, -0.008843588642776012, -0.03352906554937363, -0.01001672726124525, -0.006834493950009346, 0.015279069542884827, -0.0086094681173563, -0.018261712044477463, 0.018402433022856712, 0.0010469999397173524, -0.005944278556853533, -0.011887699365615845, -0.0015348807210102677, 0.015885965898633003, -0.03791818022727966, -0.02850721962749958, 0.003733922028914094, 0.01878516748547554, -0.012473336420953274, -0.018459046259522438, 0.009665246121585369, 0.01049499399960041, 0.002135292859748006, 0.02946021966636181, 0.011742732487618923, 0.011514515615999699, -0.008110164664685726, -0.03403772413730621, -0.03472701087594032, -0.016609232872724533, -0.03253834322094917, 0.015333103016018867, -0.0011637062998488545, -0.005796364042907953, -0.014480901882052422, 0.009843029081821442, -0.0038026010151952505, 0.002711239270865917, -0.00678608613088727, 0.0020640608854591846, 0.006925093475729227, -0.006943490356206894, -0.00812872126698494, 0.008993721567094326, 0.012739213183522224, 0.012903700582683086, 0.023013748228549957, 0.0075779142789542675, 0.0019433608977124095, 0.017096225172281265, 0.02704099379479885, -0.015453001484274864, -0.015086915343999863, -0.024026377126574516, 0.004557518754154444, -0.006899977568536997, 0.036996304988861084, -0.022400934249162674, 0.008845694363117218, 0.003228683490306139, 0.0023643390741199255, 0.011842492036521435, -0.00042968877824023366, 0.008533598855137825, -0.01140554714947939, 0.022579360753297806, 0.012488679960370064, -0.001391664263792336, -0.010075532831251621, -0.009683026932179928, 0.0071098944172263145, 0.016514942049980164, 0.002282222732901573, -0.004680648446083069}, {-0.014656408689916134, -0.020100155845284462, -0.01830463856458664, -0.0009842057479545474, -0.018098188564181328, 0.0011761784553527832, 0.0023629157803952694, -0.04616440832614899, -0.0030912202782928944, 0.007016644347459078, -0.012553785927593708, -0.0010284606833010912, 0.008711392059922218, 0.024866977706551552, -0.0033414300996810198, -0.010686507448554039, 0.02486949786543846, -0.012655885890126228, 0.00882184412330389, -0.005121149122714996, 0.001964172814041376, -0.004587146453559399, -0.008656250312924385, -0.031144211068749428, 0.003014377783983946, 0.01774582453072071, -0.006121808663010597, -0.012345471419394016, -0.001510566333308816, -0.003466600552201271, 0.00899386778473854, -0.007056316826492548, 0.005917824804782867, 0.00899051409214735, -0.045279283076524734, -0.006071793846786022, -0.01225912757217884, -0.02537759765982628, 0.008986445143818855, -0.04005463793873787, 0.00041211768984794617, 0.013048365712165833, 0.02459568902850151, 0.04067371040582657, -0.005985241383314133, 0.008231878280639648, -0.003036123001947999, -0.015366570092737675, 0.02326173335313797, -0.01045971643179655, 0.003006247105076909, -0.0004703109443653375, 0.00815447699278593, 0.004017199389636517, 0.0034364464227110147, -0.025722535327076912, 0.0016921962378546596, 0.0060777366161346436, -0.010154048912227154, -0.019974742084741592, 0.002633076161146164, 0.012696183286607265, -0.01873137056827545, -0.00466365460306406, -0.014517208561301231, -0.0011160265421494842, 0.0036725241225212812, -0.010889479890465736, -0.02986184135079384, -0.005258537828922272, 0.0022322251461446285, -0.011521602049469948, -0.018332965672016144, 0.02897375077009201, -0.011113903485238552, 0.02214890532195568, 0.0030674326699227095, 0.02966296300292015, 0.008235213346779346, -0.0004428837855812162, -0.009248979389667511, 0.008335836231708527, -0.00366294477134943, 0.02287467010319233, -0.01517857052385807, 0.02015955001115799, -0.00018756194913294166, -0.009529278613626957, 0.01509888842701912, 0.002982380334287882, -0.022446783259510994, 0.005489778704941273, -0.005150623619556427, -0.01580130122601986, 0.012873956002295017, 0.023411210626363754, -0.018522363156080246, -0.011997062712907791, -0.021874330937862396, -0.002372103976085782, 0.02891649678349495, 0.009532184340059757, -0.0057851518504321575, -0.010164248757064342, 0.005194142926484346, -0.011178378947079182, -0.008471901528537273, -0.02146163024008274, -0.0016318015987053514, -0.01634082943201065, -0.012995871715247631, -0.030102714896202087, -0.0007499423809349537, 0.0011191655648872256, -0.004962349776178598, -0.023170849308371544, -0.02871069684624672, 0.0269427802413702, 0.00494125671684742, 0.019433805719017982, -0.001272668014280498, -0.006877906154841185, -0.005200016312301159, 0.019586710259318352, 0.0168156735599041, 0.005856264848262072, -0.002360791666433215, 0.008213059976696968, -0.0046361293643713, 0.01932637393474579, -0.010802256874740124, -0.01674671098589897, 0.016791991889476776, 0.007816243916749954, -0.018033653497695923, 0.005924875847995281, -0.009025469422340393, -0.020238233730196953, -0.010989018715918064, 0.008175023831427097, 0.008075816556811333, 0.023775741457939148, 0.0030699889175593853, 0.013956095091998577, 0.0018203591462224722, 0.006163543090224266, 0.010459184646606445, 0.005729128140956163, -0.00282950303517282, -0.008363868109881878, -0.022048288956284523, 0.035473860800266266, -0.01069113053381443, 0.011203646659851074, -0.018395358696579933, -0.0011293021962046623, -0.024323632940649986, -0.014919574372470379, -0.0045829517766833305, -0.003919138573110104, -0.0047223796136677265, 5.134439925313927e-05, -0.0013817051658406854, -0.0011206765193492174, 0.007582309655845165, 0.03781662881374359, 0.0039732991717755795, 0.008159216493368149, 0.018665792420506477, 0.024041276425123215, -0.011115796864032745, 0.001498054014518857, 0.008689912036061287, -0.012082623317837715, 0.0007932230946607888, 0.00414273189380765, 0.009503026492893696, -0.011622698977589607, 0.018487557768821716, -0.011479267850518227, -0.01864463835954666, -0.02110031433403492, 0.009690001606941223, 0.0150445019826293, -0.032190605998039246, -0.004232867620885372, 0.001935038366355002, 0.027035079896450043, -0.018015746027231216, -0.012052513659000397, 0.008791797794401646, 0.00023393100127577782, -0.003771095769479871, -0.004745940677821636, 0.012403764761984348, 0.03210842236876488, -0.01509720180183649, -0.011263152584433556, 0.01492234319448471, 0.0007587686995975673, -0.011163069866597652, -0.010131639428436756, 0.009553827345371246, 5.305634840624407e-05, -0.007815471850335598, 0.021438585594296455, 0.0008418566430918872, -0.0034888831432908773, -0.01099548488855362, -0.01470221858471632, -0.034425318241119385, 0.01806419901549816, -0.014037582091987133, 0.02020259015262127, 0.022233739495277405, -0.025842389091849327, -0.022789230570197105, -0.000328707363223657, -0.008314988575875759, -0.0037408319767564535, -0.004586911294609308, -0.0001311424857703969, 0.024580545723438263, -0.011058365926146507, -0.01722409948706627, 0.008151103742420673, 0.007450689096003771, 0.00974260363727808, 0.027307620272040367, 0.01777086779475212, -0.010413131676614285, 0.01034783199429512, 0.010768157429993153, 0.005568292457610369, -0.030583979561924934, -0.003303767880424857, 0.0013695870293304324, -0.0027601781766861677, 0.024191204458475113, -0.015353989787399769, 0.018377184867858887, -0.009350663051009178, 0.0012015324318781495, 0.009521327912807465, 0.0015381978591904044, 0.019172072410583496, -0.01296582818031311, 0.006215283647179604, -0.013417481444776058, -0.0118864169344306, -0.005667176563292742, -0.007369861006736755, -0.02787180431187153, 0.015630850568413734, -0.0032218871638178825, -0.006252777297049761}, {-0.017507217824459076, -0.025052007287740707, -0.0016120638465508819, 0.00933756958693266, -0.01595878228545189, -0.021763235330581665, 0.0043703122064471245, -0.04061850160360336, -0.029740747064352036, 0.0031131005380302668, -0.00836388673633337, 0.0008356813923455775, -0.006129016168415546, 0.005019121803343296, 0.007235685363411903, -0.017768850550055504, 0.005539368372410536, -0.009989041835069656, 0.009591467678546906, -0.008995571173727512, -0.012685556896030903, -0.016227468848228455, -0.00846006814390421, -0.024982789531350136, -0.011277171783149242, -0.01177514810115099, -0.006165416445583105, 0.025995828211307526, -0.025534341111779213, -0.017011260613799095, 0.01444985531270504, -0.004093324765563011, -0.014797561801970005, 0.007490201853215694, -0.026262499392032623, -0.019715171307325363, 0.020222237333655357, 0.014494715258479118, -0.0035540300887078047, -0.03571359068155289, 0.01603856310248375, 0.006050892174243927, 0.005064947996288538, 0.006450778339058161, 0.017331046983599663, -0.007266895845532417, 0.01579057238996029, 0.006917126011103392, 0.005490433424711227, -0.009262596257030964, 0.0007429345860145986, -0.019426722079515457, 0.012194331735372543, 0.0029549074824899435, -0.0029618660919368267, -0.021859848871827126, -0.006894969381392002, 0.007387591525912285, -0.03180977702140808, -0.00804453156888485, 0.012983977794647217, -0.0024006410967558622, -0.001524288090877235, -0.01465856097638607, -0.007563174236565828, -0.006392782554030418, 0.02810566872358322, -0.007842999882996082, 0.0008978880359791219, -0.004832146223634481, 0.016057783737778664, -0.017817102372646332, 0.006620723754167557, 0.032336924225091934, -0.02563224360346794, 0.019514204934239388, 0.0005902863340452313, 0.02280360460281372, 0.0007292478112503886, -0.009740125387907028, 0.0020513494964689016, -0.01623350754380226, -0.01798965036869049, 0.024465342983603477, -0.009797406382858753, 0.00646722549572587, 0.010279378853738308, -0.0021759660448879004, 0.024668162688612938, -0.03098529577255249, -0.018818264827132225, -0.007137062959372997, 0.0008852629107423127, -0.018535204231739044, 0.005720581393688917, 0.037563446909189224, 0.0030550318770110607, 0.012031112797558308, -0.010944640263915062, -0.017327114939689636, 0.003214946947991848, -0.014276031404733658, -0.02962208166718483, 0.012851149775087833, -0.017340632155537605, -0.019205797463655472, 0.008459172211587429, -0.014898211695253849, 0.010201350785791874, 0.006899377331137657, -0.012060636654496193, -0.016732119023799896, -0.007764609530568123, -0.013104790821671486, -0.008126333355903625, -0.010786212980747223, -0.023398444056510925, 0.024584820494055748, 0.01554898452013731, 0.012253385037183762, 0.0013781554298475385, 0.00520133413374424, -0.016889663413167, 0.03789343312382698, 0.013383062556385994, 0.004390938673168421, -0.026265863329172134, 0.03232642635703087, -0.03664083778858185, 0.000708218605723232, -0.01890118233859539, -0.008174571208655834, -0.0074163153767585754, -0.00956382229924202, -0.021104242652654648, 0.004765416495501995, -0.021533530205488205, -0.009038184769451618, 0.002102636732161045, 0.0018332437612116337, 0.008580485358834267, 0.03224505856633186, -0.0016315452521666884, -0.0033913173247128725, 0.023674251511693, 0.015341099351644516, -0.00511541310697794, 0.01090411376208067, -0.0072667570784688, 0.00980499479919672, -0.00577280018478632, 0.0293769221752882, -0.024705596268177032, -0.0006692667957395315, -0.0017036930657923222, -0.026696354150772095, 0.000203741728910245, -0.00746863754466176, 0.013267339207231998, -0.0005942973075434566, -0.0070652468129992485, 0.020576084032654762, -0.0048197293654084206, 0.006533533800393343, -0.006763304118067026, 0.05205756425857544, -0.009118852205574512, 0.016040820628404617, 0.008038646541535854, 0.01170693151652813, 0.003291212720796466, 0.007608718704432249, 0.01554427482187748, -0.0019429564708843827, -0.031317029148340225, 0.016455169767141342, -0.004240213427692652, -0.0076970127411186695, 0.014419857412576675, -0.03038003481924534, -0.015887096524238586, -0.0004788829537574202, -0.00383333140052855, 0.0072941528633236885, -0.011244278401136398, -0.0005313073052093387, 0.0029262376483529806, 0.022141894325613976, -0.011030171997845173, 0.0001577199436724186, 0.00247736694291234, 0.004650291055440903, -0.001326891826465726, -0.01380114071071148, 0.012666193768382072, 0.031263090670108795, -0.032448749989271164, -0.00873932521790266, -0.007838105782866478, 0.011812550947070122, -0.010598178021609783, 0.004477217327803373, -0.01591719500720501, 0.007567218970507383, -0.014392724260687828, 0.02555137500166893, -0.0007476677419617772, -0.00432234862819314, -0.008016570471227169, -0.023372802883386612, -0.02933630906045437, 0.007317009381949902, -0.0008367639966309071, 0.013721199706196785, -0.0007223089342005551, 0.0040284269489347935, -0.026583557948470116, -0.01148170419037342, -0.007463932503014803, -0.015449104830622673, -0.014459817670285702, -0.0030959059949964285, 0.019708896055817604, 0.008004102855920792, -0.010420067235827446, 0.003885591635480523, 0.0030778339132666588, 0.005425970535725355, 0.035096749663352966, 0.014207358472049236, 5.550196874537505e-05, 0.0037446182686835527, 0.02663722261786461, 0.014483642764389515, -0.0178214218467474, 0.0028266317676752806, -0.0007977266795933247, -0.011210575699806213, 0.037468064576387405, -0.003368909237906337, 0.019909130409359932, 0.0008124517044052482, 0.021321816369891167, 0.0344267264008522, -0.019086243584752083, -0.008098997175693512, -0.004784381948411465, 0.0059271641075611115, 0.016861792653799057, 0.005632123909890652, 0.019115488976240158, -0.01577233523130417, -0.011089576408267021, 0.0031989356502890587, 0.003859013319015503, -0.006454997230321169}, {0.028258346021175385, -0.4972720146179199, 0.1810825616121292, -0.5645302534103394, 0.33600038290023804, -0.15396836400032043, -0.44376009702682495, 0.24122096598148346, 0.5315805077552795, 0.503084659576416, 0.3734099566936493, -0.003664760384708643, -0.5125231742858887, -0.9928629398345947, -0.7351546287536621, -0.6721168756484985, -0.47531506419181824, 0.1938408613204956, 0.27570840716362, -0.4495011568069458, -0.40743422508239746, -1.4637690782546997, -0.680782675743103, 0.20044492185115814, 0.12770651280879974, 0.02033120021224022, 0.4109744131565094, -0.1852307915687561, -0.2822483777999878, -0.05034418776631355, -0.15339983999729156, -0.5598859786987305, -0.2586931884288788, 0.14300230145454407, 0.30825528502464294, 0.08122394233942032, -1.2533457279205322, 0.09589673578739166, -0.015155050903558731, 0.23251685500144958, 0.47980573773384094, -0.8091523051261902, 0.5600199699401855, -1.0850157737731934, -0.3250928819179535, -0.18287184834480286, -0.09365301579236984, 0.41415226459503174, -1.1819963455200195, -0.3699929118156433, 0.17380419373512268, -0.06414677202701569, -0.14832307398319244, -0.2529546916484833, -0.5812671184539795, 0.10016290843486786, 0.12555162608623505, -0.18088988959789276, -0.36157554388046265, 0.3858749568462372, 0.5951583981513977, 0.12697966396808624, -0.16726154088974, 0.14418749511241913, -0.913019597530365, 0.2292749583721161, 2.302133560180664, -0.011829896830022335, -0.02373133972287178, 0.5333066582679749, -0.053128067404031754, -0.8004322648048401, 0.10297985374927521, -0.082671158015728, -1.1454925537109375, 0.2762748897075653, -0.6013025641441345, -0.07578061521053314, 0.23289185762405396, 0.06266918778419495, 0.38563060760498047, -0.37446129322052, -0.14403940737247467, 0.2903226912021637, 0.07616068422794342, 0.2639872133731842, 0.16409419476985931, 0.011407221667468548, -0.5665103197097778, -0.11109147220849991, -1.329203724861145, -0.2711270749568939, 0.21350416541099548, 0.22044232487678528, -0.2503994405269623, 0.01227947510778904, 0.07425867766141891, 0.09444413334131241, 0.09897905588150024, -1.0900681018829346, -0.5327155590057373, 0.19110076129436493, 0.4057256281375885, -0.8876257538795471, -0.5101635456085205, -1.28577721118927, 1.1530952453613281, 0.14269569516181946, 0.5262472629547119, 0.2135782390832901, 0.5883473753929138, 0.5519148111343384, -0.11030871421098709, 0.07977340370416641, 0.9201217293739319, -0.5730010867118835, 0.0022898491006344557, 0.05127619206905365, 0.09174679964780807, 0.2054898738861084, 0.05062225088477135, 0.005391065962612629, 0.00016398818115703762, -0.8773781657218933, -0.006468195468187332, -0.21425122022628784, 0.8546301126480103, 0.09895878285169601, 0.11843973398208618, -0.05451150983572006, 0.21083912253379822, 0.23827317357063293, 0.3463439345359802, 0.1495036780834198, 0.07303356379270554, 0.03483128547668457, 0.25766876339912415, 0.1807439625263214, 0.3584701418876648, -0.040774185210466385, 0.4200966954231262, -0.29172050952911377, 0.21346992254257202, 0.8812375068664551, 0.4508558511734009, -0.3248746395111084, -0.3935357332229614, -0.2746819257736206, -0.21416182816028595, 0.22112448513507843, -0.14403288066387177, -0.799942672252655, -0.41361141204833984, 0.05088284611701965, -0.35904571413993835, 0.32484176754951477, 0.20093262195587158, 0.18144458532333374, -0.33848950266838074, -0.10241781920194626, -0.36299657821655273, -0.08196509629487991, 0.3587988018989563, 0.06665283441543579, -0.5788521766662598, -0.29128602147102356, 0.14889641106128693, 0.04374608024954796, -0.18545889854431152, -0.36152708530426025, 0.46062859892845154, -0.12724480032920837, -0.5418948531150818, -0.022140339016914368, 0.07675313949584961, 0.8676083087921143, -1.03766667842865, -0.24099579453468323, 0.6607831120491028, -0.1092422753572464, -0.2694992125034332, -0.21458449959754944, 0.13122566044330597, 0.016258684918284416, -0.7342411875724792, -0.2205369919538498, 0.10755852609872818, 1.841643214225769, 0.2949698567390442, 0.19954998791217804, -1.700248122215271, 0.4098784923553467, 0.09398721903562546, 0.2700446844100952, 0.04059513285756111, -0.7265559434890747, 0.3128015100955963, -0.5036090612411499, 1.090883731842041, -0.22796382009983063, -0.26192817091941833, 0.11833220720291138, 0.10886833071708679, 0.141116663813591, 0.7447338104248047, -0.5847538113594055, -0.18562136590480804, 0.2696339190006256, -0.04903167486190796, -0.42477700114250183, -0.0440778024494648, 0.97069251537323, 0.2740851938724518, 0.5778946876525879, -0.4154319167137146, 0.30020812153816223, -0.024422181770205498, -0.22411483526229858, 0.3365117609500885, 0.09401967376470566, 0.4081580340862274, -0.20047378540039062, -0.45309269428253174, 0.9887734651565552, -0.21549712121486664, 0.2920537292957306, 0.29162198305130005, 0.08002090454101562, -0.5129297375679016, 0.07765737920999527, 0.24546967446804047, -1.092379093170166, -0.35693925619125366, -0.09901844710111618, 0.6708144545555115, 0.05758368968963623, 0.13330860435962677, -0.12244240939617157, -0.4746665954589844, -1.07804274559021, -0.43235915899276733, 0.7032191157341003, 0.09010104089975357, 0.5051947236061096, 0.023078422993421555, 0.3412480354309082, -1.0453377962112427, -0.2263062447309494, 0.22067296504974365, 0.21183961629867554, 0.014771874062716961, 0.12138287723064423, 0.09976600855588913, 0.8710254430770874, -0.27877092361450195, 0.2315821796655655}, {0.009099572896957397, 0.003632972249761224, 0.002643989399075508, 0.0061698779463768005, -0.03595808520913124, -0.010862896218895912, 0.003478463040664792, -0.02915678359568119, -0.028135720640420914, 0.0015475318068638444, 0.0042879581451416016, -0.00029594686930067837, -0.011703398078680038, 0.0005706844385713339, 0.0015403341967612505, -0.016000332310795784, 0.03072921372950077, -0.030741332098841667, 0.024357331916689873, -0.013635070063173771, 0.004482756834477186, -0.004796524066478014, -0.021153872832655907, -0.026194604113698006, -0.002124246908351779, -0.013829574920237064, -0.006791955791413784, 0.024395491927862167, -0.00931891705840826, -0.012480649165809155, 0.0023053877521306276, 0.007123054005205631, -0.004593239165842533, 0.01864852011203766, -0.018789827823638916, -0.00813260767608881, -0.009778762236237526, -0.0015771135222166777, 0.0007420987240038812, -0.024714017286896706, 0.003470859257504344, -0.006862236186861992, 0.029526378959417343, 0.0070123085752129555, -0.0033978840801864862, -0.008864632807672024, 0.011101429350674152, -0.03390602767467499, 0.029412750154733658, -0.01774471066892147, -0.019298702478408813, -0.010884782299399376, 0.015163284726440907, -0.006470840889960527, 0.006386805325746536, -0.01963729038834572, -0.002280701417475939, -0.0017512639751657844, -0.023720458149909973, -0.017435120418667793, 0.01913057640194893, -0.006695400457829237, -0.022618886083364487, -0.02296934649348259, -0.004949025809764862, 0.01393740065395832, 0.01747487485408783, -0.002522323979064822, -0.010882562026381493, -0.0038650394417345524, 0.0031534917652606964, -0.004555427934974432, 0.0010451001580804586, 0.0061056655831635, -0.0024527227506041527, 0.036686696112155914, 0.0009518366423435509, 0.0270469319075346, 0.014491907320916653, 0.006600463762879372, 0.0010273335501551628, 0.002478889422491193, -0.011726231314241886, 0.010133177042007446, -0.008218986913561821, 0.015384050086140633, 0.008213115856051445, -0.027145808562636375, 0.02332516573369503, -0.025003865361213684, -0.016131233423948288, 0.006271176505833864, -0.010115539655089378, -0.011150647886097431, -0.0034021399915218353, 0.02770032361149788, -0.004318464547395706, -0.02072199434041977, -0.013576359488070011, -0.03031376749277115, 0.00946848839521408, 0.011347830295562744, -0.0012537591392174363, -0.010689998045563698, 0.006102847866714001, -0.0021573619451373816, -0.0016589160077273846, -0.02287532016634941, -0.010302525013685226, -0.02062140218913555, -0.018611600622534752, -0.03912461921572685, 0.0011544935405254364, -0.02855878323316574, -0.0023861664813011885, -0.0030776087660342455, -0.013659275136888027, 0.013422872871160507, 0.00041271356167271733, 0.027187388390302658, -0.009407786652445793, -0.009075727313756943, -0.009293998591601849, 0.019405504688620567, -0.0017827108968049288, -0.0016020090552046895, -0.009774717502295971, 0.00822469498962164, -0.007186102215200663, 0.01739463582634926, 0.013988431543111801, -0.012547031044960022, 0.01313866674900055, 0.0043322499841451645, -0.004715251736342907, 0.024765385314822197, -0.012018820270895958, -0.02318306267261505, -0.001244339277036488, 0.007941369898617268, 0.017090724781155586, 0.016708627343177795, 0.005221027880907059, 0.02098263055086136, 0.02767297439277172, 0.018107546493411064, 0.010684137232601643, 0.024445340037345886, 0.0014750539558008313, -0.0004904756788164377, -0.008993510156869888, 0.027382925152778625, -0.014348283410072327, 0.007556472904980183, -0.013820555992424488, -0.009728904813528061, -0.007652831729501486, -0.010283547453582287, -0.003593962639570236, 0.0030205154325813055, 0.0006064735352993011, -0.002240576781332493, -0.019137931987643242, 0.006332994904369116, -0.0030224965885281563, 0.02036513388156891, -0.0059510767459869385, 0.008018926717340946, 0.011444601230323315, -0.004343073815107346, -0.009670363739132881, -0.0065802475437521935, 0.0021932150702923536, 0.001634223386645317, -0.03204219415783882, 0.009766427800059319, 0.008079090155661106, -0.018462367355823517, -0.0016814764821901917, -0.011389934457838535, 0.0033471756614744663, -0.013947898522019386, -0.017122594639658928, 0.00720431050285697, -0.02766214869916439, 0.010059681721031666, -0.004215688444674015, 0.025210268795490265, -0.013072558678686619, 0.0012012093793600798, -0.0012346053263172507, -0.008206921629607677, 0.01292751356959343, 0.003339081071317196, 0.013344504870474339, 0.021448176354169846, -0.018364423885941505, -0.00738578150048852, -0.0030138336587697268, 0.0019438544986769557, -0.01038963533937931, 0.005940671544522047, -0.004301924258470535, 0.022539591416716576, -0.006810121703892946, 0.01587621681392193, 0.006988402921706438, 0.007759717758744955, -0.01254070270806551, -0.03431643545627594, -0.008857295848429203, 0.004658299032598734, -0.008172444999217987, 0.0028058860916644335, 0.01620173640549183, -0.001556851901113987, -0.034874286502599716, -0.019462810829281807, 0.018236611038446426, 0.008586863987147808, -0.013781597837805748, 0.008967277593910694, 0.013696171343326569, -0.014070466160774231, -0.01178483571857214, 0.020736034959554672, -0.005318623501807451, -0.019484523683786392, 0.017650477588176727, 0.021966682747006416, -0.03207039460539818, 0.0017551752971485257, 0.012028088793158531, -0.005651908926665783, 0.0014085557777434587, -0.0006559600005857646, -0.0014569205231964588, -0.003727259347215295, 0.03074224479496479, -0.017548037692904472, 0.02257753536105156, 0.0018837501993402839, 0.0010748704662546515, 0.028588373214006424, -0.0015984228812158108, -0.023668259382247925, -0.011058642528951168, -0.008270121179521084, 0.0006589579861611128, -0.014044124633073807, -0.010611843317747116, -0.036337655037641525, -0.0016513331793248653, 0.021647751331329346, -0.011547038331627846, -0.0015199593035504222}, {0.002697011921554804, -0.01855507865548134, 0.000327567890053615, 0.0071112895384430885, -0.022743888199329376, 0.016975942999124527, -0.0060685863718390465, -0.02330092340707779, -0.023089686408638954, 0.015882765874266624, -0.009351399727165699, -0.015573936514556408, -0.016543559730052948, 0.01635606400668621, -0.00895270612090826, -0.0018583181081339717, 0.017652636393904686, -0.024156393483281136, -0.006514566484838724, -0.01656365394592285, -0.003549672430381179, -0.028492717072367668, 0.006548689678311348, -0.02870013564825058, 0.009578202851116657, 0.003117023967206478, 0.011905365623533726, 0.01934695988893509, -0.007758776657283306, 0.003095670836046338, 0.015517303720116615, 0.012908377684652805, -0.01756083406507969, 0.013982567936182022, -0.02273503504693508, -0.009630445390939713, 0.00228244555182755, -0.020380215719342232, -0.004962120205163956, -0.03138265758752823, -0.002792476210743189, 0.005464861169457436, 0.01294600497931242, 0.00555657222867012, 0.016039714217185974, -0.0011228855000808835, 0.005508078262209892, 0.01142031792551279, 0.01367114670574665, -0.014593315310776234, -0.011748522520065308, -0.007489638403058052, 0.021862633526325226, 0.00418624235317111, 0.0032465006224811077, -0.00855983141809702, -0.002566238632425666, -0.009016797877848148, -0.012067233212292194, -0.027162354439496994, 0.0010894557926803827, -0.0014977919636294246, -0.006461907643824816, 0.004176973830908537, -0.0013448854442685843, 0.01833203248679638, 0.009166838601231575, -0.026538413017988205, -0.01898275688290596, 0.0153659051284194, 0.0037414596881717443, -0.0002974590170197189, -0.0063810502178967, 0.018429705873131752, 0.0023195245303213596, 0.01703951321542263, 0.005404461175203323, 0.022776572033762932, -0.0036837353836745024, 0.012573737651109695, 0.002409694017842412, -0.0027448309119790792, -0.013593305833637714, 0.016864018514752388, -0.006264054216444492, 0.0029825964011251926, 0.034760620445013046, -0.014585188589990139, 0.017876770347356796, -0.007695688400417566, -0.00506356917321682, -0.000519449298735708, -0.003108626464381814, -0.013918462209403515, -0.004658181685954332, 0.024294856935739517, -0.008153460919857025, -0.006470350082963705, -0.023922733962535858, -0.006885431706905365, 0.028218457475304604, -0.010086254216730595, 0.00035861280048266053, -0.005395505111664534, 0.005584601312875748, 0.004523891024291515, 0.0005612183595076203, -0.008036729879677296, -0.0028094053268432617, -0.006838812027126551, -0.03365645930171013, -0.043137844651937485, -0.008312955498695374, -0.009989806450903416, -0.01001422293484211, -0.00686686672270298, -0.0189827848225832, 0.03277358412742615, 0.0020332380663603544, 0.02767525613307953, -0.028457943350076675, -0.00888974592089653, 0.0031558028422296047, 0.0151863694190979, 0.01328032836318016, 0.013091999106109142, -0.0037977437023073435, 0.021425869315862656, 0.007718014530837536, 0.010213717818260193, -0.013486032374203205, -0.004184052813798189, 0.0029908204451203346, -0.005452645476907492, 0.004062171094119549, -0.007278015837073326, -0.023364849388599396, 0.0011105408193543553, 0.014994846656918526, -0.0025959096383303404, 0.033442422747612, 0.01213167980313301, -0.002267182106152177, 0.0163484625518322, 0.006188573315739632, 0.02307148277759552, 0.010236215777695179, 0.000542336143553257, 0.004468264989554882, 0.016404546797275543, -0.0025110431015491486, 0.03841984272003174, 0.0009363045101054013, -0.010198116302490234, -0.01889379695057869, -0.0033810364548116922, -0.006482060998678207, -0.0006999436300247908, 0.014342647977173328, -0.026734452694654465, 0.005415897350758314, 0.0068273828364908695, 0.0063514444045722485, 0.0324576310813427, -0.0015727163990959525, 0.048535093665122986, -0.012626806274056435, 0.015408634208142757, 0.0022059311158955097, 0.0058200424537062645, -0.015571595169603825, -0.009991919621825218, 0.020749369636178017, -0.015031394548714161, -0.01762407086789608, 0.015613202005624771, 0.0008856008062139153, -0.024463223293423653, 0.02162335067987442, 0.002425665035843849, -0.013640085235238075, 0.007021785248070955, -0.019549107179045677, 0.011007196269929409, -0.016463203355669975, -0.0019533250015228987, -0.014450770802795887, 0.03567829728126526, 0.005599594209343195, -0.01325218752026558, 0.013144010677933693, 0.006720187142491341, 0.019990772008895874, 0.014887804165482521, 0.0015098131261765957, 0.03683477267622948, -0.01463120523840189, -0.009294556453824043, -0.0005306417588144541, 0.020772680640220642, -0.02334911935031414, -0.01759140007197857, -0.00040478890878148377, -0.0014386267866939306, -0.014221508987247944, 0.025955382734537125, 0.013010012917220592, 0.004233804065734148, 0.0060103219002485275, -0.020554514601826668, -0.03571360558271408, 0.006197980605065823, -0.018421081826090813, 0.027721311897039413, 0.0020408183336257935, -0.0015367006417363882, -0.010144888423383236, -0.013144087977707386, -0.0058394530788064, 0.02039206214249134, 0.0003793381911236793, -0.008335331454873085, 0.009952659718692303, -0.013997809030115604, -0.005348327569663525, 0.009571243077516556, -0.0011850576847791672, -0.0014918409287929535, 0.02724994160234928, 0.03355089947581291, -0.001932578394189477, -0.018763191998004913, 0.008174045011401176, -0.002952308626845479, -0.008130665868520737, 0.013801352120935917, -0.014590566977858543, -0.0013868175446987152, 0.03261040896177292, -0.0112534174695611, 0.005593193229287863, -0.010441605001688004, 0.0014897158835083246, 0.008900120854377747, 0.008728640154004097, -0.04401329904794693, -0.012741751968860626, 0.006443557795137167, 0.003888589795678854, -0.001992198172956705, -0.020416300743818283, -0.01548768486827612, -0.0032131043262779713, 0.007972261868417263, 0.015630312263965607, -0.013870837166905403}, {0.016854478046298027, -0.006360235624015331, 0.004473775625228882, 0.019174253568053246, -0.00046619438217021525, -0.00548374792560935, 0.00038149303873069584, -0.03869088366627693, -0.02968663163483143, -0.001619411283172667, -0.008759349584579468, -0.016032766550779343, 0.009831219911575317, 0.005787104368209839, -0.010914786718785763, -0.03892005234956741, 0.017167212441563606, -0.007809406612068415, 0.017558908089995384, 0.006919359788298607, -0.005617290735244751, -0.0015610279515385628, -0.002585854846984148, -0.00664130225777626, 0.009577593766152859, -0.0009886946063488722, 0.0002246572548756376, 0.022638345137238503, -0.00535778421908617, -0.016112545505166054, -0.0019072412978857756, 0.0034572796430438757, -0.01055974792689085, -0.00527865020558238, -0.020186319947242737, -0.007610945031046867, 0.006784686353057623, -0.00253185722976923, 0.00570992287248373, -0.03776270151138306, 0.004803212359547615, -0.007844178006052971, 0.007425547111779451, 0.022251470014452934, 0.010649697855114937, -0.022471124306321144, -0.007304131519049406, -0.021671276539564133, 0.03098004125058651, -0.021741187199950218, -0.004826594144105911, -0.013409288600087166, 0.012288977392017841, 0.005854816175997257, 0.017329685389995575, -0.017324041575193405, 0.0028883127961307764, 0.009907213971018791, -0.015883151441812515, -0.004297541920095682, 0.014685112051665783, -0.010138614103198051, -0.018199894577264786, -0.010622771456837654, -0.025038886815309525, 0.02684408240020275, -0.00024835014482960105, -0.025274034589529037, -0.011255861259996891, -0.0098959319293499, 0.007621514145284891, -0.013567788526415825, -0.004724876023828983, -0.0020386618562042713, -0.013689796440303326, 0.018205929547548294, -0.0005114105879329145, 0.026601070538163185, 0.0050108786672353745, 0.01414240337908268, -0.022456442937254906, -0.003245361614972353, -0.009348991326987743, 0.033716339617967606, -0.007084930781275034, 0.0015391288325190544, 0.0175729189068079, -0.00916909147053957, 0.014288593083620071, -0.016307273879647255, -0.0070929815992712975, 0.013332342728972435, 0.003067360259592533, -0.0020446518901735544, -0.009183987975120544, 0.021099647507071495, -0.009606373496353626, 0.007856311276555061, -0.00971053447574377, -7.818734593456611e-05, 0.012569037266075611, -0.009951178915798664, -0.0011313664726912975, 0.005283524747937918, 0.00967797078192234, -0.00952103827148676, 0.02409241534769535, 0.0029511619359254837, -0.011286810971796513, -0.016070585697889328, -0.03669019043445587, -0.026458708569407463, 0.0018773899646475911, 0.0032515244092792273, -0.015465530566871166, 0.005512474104762077, -0.013058722019195557, -0.005621636286377907, -0.010117574594914913, 0.013788769021630287, -0.012551335617899895, -0.031009787693619728, -0.003017494222149253, 0.021957531571388245, 0.007520024664700031, 0.015511571429669857, -0.007753903511911631, 0.013567257672548294, -0.010914823040366173, 0.018188806250691414, -0.00987410917878151, -0.01689257100224495, 0.013878711499273777, 0.007730219047516584, -0.015451939776539803, 0.0028587987180799246, -0.018774278461933136, -0.013952730223536491, -0.009010694921016693, 0.020705437287688255, 0.013625364750623703, 0.03352554515004158, 0.004702169448137283, 0.01642690598964691, -0.0105582345277071, 0.017066920176148415, -0.0005244883359409869, 0.016870375722646713, -0.01330406405031681, -0.005341545213013887, -0.008723214268684387, 0.02776954509317875, -0.008548065088689327, 0.010457624681293964, -0.022598445415496826, -0.01377933006733656, 0.004813568666577339, -0.010299723595380783, -0.0064259422942996025, -0.006254938896745443, 0.00608759606257081, -0.0017956770025193691, -0.0004229796468280256, 0.029148073866963387, -0.008855290710926056, 0.04108431935310364, -0.009893137961626053, -0.0044337050057947636, 0.0029249012004584074, 0.014074103906750679, 0.00032569875475019217, -0.007584042381495237, 0.002596598584204912, 0.004456562455743551, -0.01054474152624607, 0.0044814650900661945, -0.013270723633468151, -0.017245018854737282, 0.016060257330536842, -0.014208772219717503, 0.007196051999926567, -0.0023203531745821238, 0.006326787173748016, -0.00377434934489429, -0.02422352135181427, -0.016655558720231056, -0.012253562919795513, 0.026997549459338188, 0.0016912140417844057, 0.013857198879122734, -0.015711015090346336, 0.01760735921561718, -0.004719363991171122, 0.0028477630112320185, -0.0111250514164567, 0.007675019558519125, -0.015191216953098774, -0.008168837986886501, -0.004375395830720663, 0.012826759368181229, -0.01406213827431202, 0.013950555585324764, -0.005048192571848631, 0.02293766476213932, -0.00765953166410327, 0.03352491930127144, 0.01234881579875946, -0.007352088112384081, -0.0024016830138862133, -0.025586942210793495, -0.025533581152558327, -0.008095165714621544, -0.01144171692430973, 0.003707590512931347, 0.024593127891421318, -0.010364874266088009, -0.02649880200624466, 0.0013647348387166858, -0.0042604971677064896, -0.009111672639846802, 0.01382730808109045, 0.002771341009065509, -0.0050242128781974316, -0.010445493273437023, 0.010973090305924416, 0.0018093312392011285, 0.006014025304466486, 0.01340064499527216, 0.016557343304157257, 0.012741501443088055, -0.008595512248575687, -0.01499191764742136, 0.01800381951034069, 0.00320259272120893, -0.02147408202290535, 0.0013480940833687782, -0.011302759870886803, 0.003581734374165535, 0.041487403213977814, -0.011632505804300308, 0.007234635297209024, -0.018649660050868988, 0.001396150328218937, 0.036676786839962006, -0.016557909548282623, -0.016618335619568825, -0.0216368418186903, 0.012203233316540718, -0.0029370614793151617, 0.029458370059728622, 0.008405826054513454, -0.034791264683008194, -0.0210867989808321, -0.0026482068933546543, -0.002942516468465328, -0.02288063056766987}, {0.0016582689713686705, -0.0036993150133639574, -0.004329298622906208, 0.003564481157809496, -0.021755529567599297, -0.025741633027791977, -0.02461242489516735, -0.024944934993982315, -0.0242287740111351, 0.0012739337980747223, 0.011202797293663025, -0.010654645040631294, 0.0021256767213344574, 0.0167490653693676, 0.001793299918062985, -0.004775856621563435, 0.018376721069216728, -0.0016404956113547087, 0.012955920770764351, -0.006411249283701181, -0.003306563710793853, -0.006733549293130636, -0.02484539896249771, -0.010904882103204727, -0.008658264763653278, -0.005393936298787594, -0.0024585716892033815, 0.03672441095113754, -0.013724692165851593, -0.019740208983421326, 0.005320707801729441, -0.009559210389852524, 0.005596200469881296, 0.005922697950154543, 0.0052179559133946896, -0.008380591869354248, 0.024082105606794357, -0.021550575271248817, 0.012691505253314972, -0.026040280237793922, -0.0030774218030273914, -0.003833755385130644, 0.015618094243109226, 0.014475042931735516, -0.0072385696694254875, -0.01257376279681921, -0.004621121566742659, 0.0009352703928016126, 0.021444737911224365, -0.0234756451100111, 0.003840666264295578, -0.014348845928907394, -0.0001237959077116102, -0.0016473543364554644, 0.010462230071425438, -0.03004692867398262, 0.016974570229649544, -0.006254477426409721, -0.012672655284404755, -0.017859142273664474, 0.014528203755617142, -0.015422388911247253, -0.013755783438682556, 0.011492069810628891, -0.013302095234394073, 0.013955849222838879, 0.03214449808001518, -0.009977002628147602, -0.00788493175059557, 0.007828714326024055, 0.012670563533902168, -0.009775640442967415, 0.004686542320996523, 0.012986314482986927, -0.020563529804348946, 0.023312456905841827, 0.0008191050146706402, 0.02667868137359619, -0.005518907681107521, 0.0028171518351882696, -0.001522057456895709, 0.0055747381411492825, -0.03277365863323212, 0.034741614013910294, -0.021010924130678177, -0.006544067990034819, 0.002184369834139943, -0.01994379609823227, 0.02101258933544159, -0.010441671125590801, 0.009295234456658363, 0.0037960587069392204, -0.015438790433108807, -0.023135388270020485, 0.008059256710112095, 0.03454446420073509, -0.0207306407392025, 0.006711637135595083, -0.0023980382829904556, -0.019101582467556, 0.019368590787053108, -0.014148606918752193, 0.012846545316278934, -0.00647724000737071, -0.005479013547301292, -0.0074600703082978725, 0.0010884309886023402, -0.004774985834956169, 0.0017699534073472023, -0.0055074794217944145, -0.021124135702848434, -0.022876881062984467, 0.005295427516102791, -0.0071825142949819565, -0.003031713655218482, -0.003258694661781192, -0.028499653562903404, 0.02116020768880844, 0.008166449144482613, 0.010934618301689625, -0.016382340341806412, -0.017501140013337135, 0.002432455774396658, 0.027681594714522362, 0.0026159994304180145, 0.003787050489336252, -0.004731636960059404, 0.023810625076293945, -0.008263429626822472, 0.015393281355500221, -0.014701626263558865, -0.0016359384171664715, 0.019035426899790764, 0.019432300701737404, -0.012787654995918274, -0.006767838727682829, -0.00704059237614274, -0.009703353978693485, 0.012139128521084785, 0.014092457480728626, 0.01837451569736004, 0.008367428556084633, -0.013484768569469452, 0.03431212157011032, 0.012844105251133442, -0.0015659215860068798, -2.911620686063543e-05, 0.029096441343426704, -0.006340350024402142, 0.028098514303565025, -0.00536741130053997, 0.022400178015232086, -0.022110288962721825, 0.016281327232718468, -0.00876352097839117, 0.009492883458733559, 0.015426496043801308, -0.013542484492063522, 0.00726349838078022, -0.0034058019518852234, -0.00035489490255713463, 0.0158888828009367, -0.012846686877310276, 0.016383178532123566, 0.006172184366732836, 0.027519019320607185, -0.013522711582481861, 0.011721296235918999, 0.005695013329386711, 0.016033364459872246, -0.004107549786567688, -0.002958440687507391, -9.513432451058179e-05, 0.019887758418917656, -0.011169694364070892, 0.00455941166728735, 0.0027689021080732346, 0.002904170425608754, 0.01693691313266754, -0.009704646654427052, -0.014384383335709572, -0.005649386439472437, 0.0020854671020060778, 0.002790943020954728, -0.02736160159111023, -0.0031574920285493135, 0.012899506837129593, 0.027447231113910675, -0.004045257344841957, -0.008943838998675346, -0.004699896089732647, 0.01554980967193842, 0.014421933330595493, 0.012010562233626842, 0.007843021303415298, 0.017291387543082237, -0.009032497182488441, -0.0013609883608296514, -0.011453312821686268, 0.0125284967944026, -0.00948262121528387, -0.006083337124437094, 0.007262140512466431, 0.014824263751506805, -0.010513723827898502, 0.029258307069540024, -0.004875003360211849, 0.0011244110064581037, -0.006472367327660322, -0.03451703116297722, -0.010203111916780472, -0.015670178458094597, -0.015126420184969902, 0.004171658307313919, 0.02103334665298462, -0.016915151849389076, -0.026673218235373497, -0.00010760230361483991, -0.0015183392679318786, -0.004519790876656771, 0.010720796883106232, -0.00951461773365736, -0.00943705439567566, -0.016291523352265358, 0.007292959373444319, 0.00010030154953710735, -0.005315595772117376, 0.0005822496605105698, 0.016607871279120445, -0.003964581061154604, -0.007081196643412113, 0.010202573612332344, 0.005056072026491165, -0.0007301531150005758, -0.010300181806087494, -0.014534435234963894, -0.012678629718720913, 0.011120247654616833, 0.0637480840086937, -0.012282243929803371, 0.004769326653331518, -0.0029340172186493874, -0.001688426244072616, 0.016116445884108543, -0.006342501379549503, -0.014997199177742004, -0.018316073343157768, 0.0053854831494390965, -0.014633720740675926, 0.00853804312646389, 0.002962665632367134, -0.008892054669559002, 0.007823814637959003, 0.014566685073077679, -0.01139854732900858, -0.01831675134599209}, {-0.013738272711634636, -0.017300330102443695, 0.001236709184013307, 0.0005657181027345359, -0.02104681171476841, -0.015018654987215996, -0.018728161230683327, -0.035496775060892105, -0.026663050055503845, -0.0076889521442353725, 0.0022580106742680073, -0.002128469990566373, 0.010177556425333023, -0.010256459936499596, 0.0049433521926403046, -0.0033195328433066607, 0.011782619170844555, -0.0156997162848711, 0.023570995777845383, 0.0048287492245435715, 0.002343712141737342, -0.03190971165895462, -0.008862283080816269, -0.0010611377656459808, 0.004595314152538776, -0.005854479502886534, -0.005210761446505785, 0.020401792600750923, -0.023330356925725937, 0.004995061084628105, -0.010055978782474995, -0.01579863205552101, -0.02057049050927162, 0.005044267512857914, 0.002524992451071739, -0.00522298738360405, 0.018676308915019035, -0.017068585380911827, -0.008223236538469791, -0.0045713563449680805, 0.01435616984963417, -0.0017606402980163693, 0.008551327511668205, 0.0009987035300582647, -0.007030393462628126, -0.013838063925504684, 0.009023594669997692, -0.0018108129734173417, 0.021663356572389603, -0.01355236116796732, -0.009053594432771206, 0.006069360766559839, 0.030885126441717148, 0.009804186411201954, -0.0019997006747871637, -0.021562092006206512, -0.0017652051756158471, 0.009328247047960758, -0.02860388532280922, -0.010505571030080318, 0.0164544228464365, -0.011068328283727169, -0.01607953943312168, -0.014661157503724098, 0.0013723765732720494, 0.029823923483490944, 0.0003273244947195053, -0.0024831965565681458, -0.0015187014359980822, 0.0013695665402337909, 0.01116197369992733, -0.017532501369714737, -0.0036060854326933622, 0.011520609259605408, -0.013289544731378555, 0.008489935658872128, -0.008841152302920818, 0.029592210426926613, 0.010747455060482025, 0.005606158636510372, -0.03338850662112236, 0.006446587387472391, -0.010294833220541477, 0.02573929913341999, -0.021759238094091415, 0.004479000344872475, 0.0146402632817626, -0.019674114882946014, 0.010191413573920727, -0.023608628660440445, -0.01105671375989914, -0.008265440352261066, -0.00933406874537468, -0.0054648625664412975, 0.01220036018639803, 0.021413272246718407, -0.010014330968260765, 0.004740804899483919, -0.02376456744968891, 0.0009309329325333238, 0.01279327180236578, 0.0017951596528291702, 0.004391898866742849, -0.004990307614207268, -0.012137309648096561, -0.0012201538775116205, 0.0003116998414043337, -0.002552158897742629, -0.007896228693425655, -0.0037847107741981745, -0.019569382071495056, -0.01652950420975685, -0.005732310935854912, -0.008844632655382156, -0.01366960909217596, 0.003567902371287346, -0.022147662937641144, 0.022573357447981834, -0.01864965818822384, 0.009896479547023773, -0.027414442971348763, -0.008683091960847378, 0.005642191041260958, 0.026226568967103958, 0.00891813077032566, 0.03518332540988922, -0.01232586894184351, 0.02938239276409149, -0.0005770717980340123, 0.009045865386724472, -0.03908367455005646, -0.005279461853206158, 0.016884436830878258, 0.007833368144929409, -0.015275808051228523, -0.0022045543882995844, -0.025315286591649055, -0.026603499427437782, 0.026750735938549042, 0.017230737954378128, 0.015060185454785824, 0.020817222073674202, 0.028640933334827423, 0.01361194346100092, 0.001109732431359589, 0.01748731918632984, -0.009538602083921432, 0.004626153968274593, 0.01612905226647854, 0.015423713251948357, -0.013632475398480892, 0.023823056370019913, -0.003932540770620108, -0.003733160439878702, -0.0024356297217309475, -0.01803196407854557, 0.014273513108491898, -0.042463384568691254, 0.003717045998200774, -0.006951932795345783, 0.002794081112369895, 0.002120726276189089, -0.026847295463085175, 0.019451092928647995, 0.004146149381995201, 0.05510864406824112, -0.007741423323750496, -0.00019056789460591972, 0.014514671638607979, 0.03346523270010948, -0.003125651739537716, 0.012157727032899857, 0.002125933999195695, -0.010553725063800812, -0.019648585468530655, 0.010631480254232883, -0.017096538096666336, -0.004920739680528641, 0.010399584658443928, -0.016937151551246643, -0.01888926327228546, 0.01626730151474476, -0.002671163994818926, 0.0028337452095001936, -0.04501108452677727, -0.004121336154639721, -0.009902764111757278, 0.05075163394212723, -0.011420427821576595, 0.005415693391114473, -0.003932467661798, 0.00020183924061711878, -0.013683703728020191, -0.007115132641047239, 0.024540498852729797, 0.025512486696243286, -0.022015953436493874, 0.007012100424617529, 0.006221654824912548, -0.005030862521380186, -0.0034144839737564325, -0.013028381392359734, 0.0003339670365676284, 0.03570369631052017, 0.01122208684682846, 0.012799062766134739, 0.02119159698486328, 0.007177394814789295, -0.014623266644775867, -0.03610580787062645, -0.030649324879050255, -0.004834188614040613, -0.023934381082654, -0.011998171918094158, 0.013993417844176292, 0.00908853393048048, -0.029928507283329964, -0.022253340110182762, -0.006771692540496588, 0.0024034895468503237, 0.00036459448165260255, -0.019703233614563942, -0.010656040161848068, -0.01673610508441925, 0.007869199849665165, 0.017609182745218277, -0.008471588604152203, 0.0013601445825770497, 0.009843124076724052, 0.022957270964980125, -0.00015711371088400483, -0.002582905814051628, -0.0017035850323736668, 0.0052683050744235516, -0.012889106757938862, -0.0026796022430062294, 0.006083816289901733, 0.008983642794191837, 0.027192512527108192, -0.008707789704203606, 0.021461600437760353, 0.007252152543514967, 0.009783120825886726, 0.033221274614334106, 0.01138601265847683, 0.0061833797954022884, -0.00040343485306948423, 0.0018387864110991359, 0.010290159843862057, 0.020616674795746803, -0.008356942795217037, -0.017669640481472015, 0.0010578682413324714, 0.011173315346240997, 0.008801307529211044, -0.012988194823265076}, {0.0027957286220043898, -0.004217779263854027, -0.015609248541295528, 0.026754913851618767, -0.03768526762723923, -0.010946099646389484, -0.01217708084732294, -0.009434211999177933, -0.045835237950086594, 0.01603103242814541, 0.011908221989870071, 0.0018559424206614494, 0.002099403878673911, 0.002629282884299755, 0.012538140639662743, -0.024764209985733032, 0.030221065506339073, -0.017200635746121407, 0.0011196898994967341, -2.455263711453881e-06, -0.0011490120086818933, -0.003749599913135171, -0.010537220165133476, -0.014404461719095707, -0.004464109893888235, 0.0011370215797796845, -0.009042240679264069, 0.022556012496352196, 0.0037870274391025305, -0.03927018865942955, 0.0086390795186162, -0.005710375029593706, -8.364467066712677e-05, -0.01785999722778797, -0.023754043504595757, 0.011362985707819462, 0.0030075039248913527, -0.015873542055487633, -0.007848761975765228, -0.02080160565674305, 0.008933644741773605, 0.000427067483542487, 0.01715298555791378, 0.008068779483437538, 0.004936509300023317, -0.01692802459001541, -0.008840571157634258, -0.00031371263321489096, -0.007620012387633324, -0.009848996065557003, 0.00925156008452177, -0.0014528384199365973, 0.016847915947437286, -0.0055782548151910305, 0.007615923415869474, -0.022937772795557976, -0.0013723692391067743, 0.027500491589307785, -0.004505667835474014, -0.005928768310695887, 0.004913202952593565, -0.003185856156051159, 0.00429696636274457, -0.00300863035954535, -0.0022751828655600548, -0.002350029069930315, 0.0008644660119898617, -0.01834578812122345, -0.008999635465443134, -0.0068395864218473434, 0.014219539240002632, -0.011825851164758205, 0.011721407063305378, 0.0201142355799675, -0.002125760307535529, 0.0074144043028354645, -0.002298115286976099, 0.02106473408639431, 0.0067213536240160465, -0.0016763574676588178, -0.009714663960039616, -0.004495282657444477, -0.012800783850252628, 0.02399655617773533, 0.0024367335718125105, -0.0004640768747776747, 0.003923802170902491, 0.0030263394583016634, 0.018148131668567657, -0.006578981410712004, -0.011009794659912586, -0.020392151549458504, -0.002872732002288103, -0.016937995329499245, 0.0025062214117497206, 0.012492137961089611, -0.029822103679180145, -0.005234389565885067, -0.027625348418951035, -0.004934275988489389, 0.013149170204997063, -0.01274202298372984, -0.005978316068649292, -0.004892920143902302, 0.0016552425222471356, -0.015528333373367786, -0.0023560149129480124, -0.030658110976219177, -0.0035026962868869305, -0.007178574334830046, -0.018142057582736015, -0.04677719250321388, -0.0009741751127876341, -0.02135436423122883, -0.016922825947403908, -0.019055645912885666, -0.0030703560914844275, 0.02725597843527794, 0.008960235863924026, 0.011020839214324951, -0.026279816403985023, -0.01346048153936863, -0.012006924487650394, 0.03328265994787216, 0.015360435470938683, 0.0038267553318291903, 0.010818812996149063, 0.014849169179797173, -0.012931711040437222, 0.015520758926868439, -0.011497731320559978, -0.0073346602730453014, -0.01687736064195633, 0.010433713905513287, -0.010981205850839615, 0.019752368330955505, -0.021105818450450897, -0.010910757817327976, 0.0027931376826018095, -0.003883649129420519, -0.0005610280786640942, 0.03872371092438698, -0.005988392513245344, 0.002719978569075465, 0.01748994179069996, 0.024768337607383728, 0.009468350559473038, 0.008793451823294163, 0.019573114812374115, -0.0032340760808438063, -0.010360989719629288, 0.011671328917145729, 0.00016666532610543072, 0.014719187282025814, -0.014967210590839386, -0.02340158261358738, 0.0071756718680262566, -0.016561387106776237, -0.004130955785512924, -0.01817307621240616, -4.940683083987096e-06, 0.0032673580572009087, 0.003570136846974492, 0.016636859625577927, -0.0042715175077319145, 0.029061434790492058, 0.002266509225592017, -0.005068572703748941, 0.01862248405814171, -0.00019619011436589062, -0.0023756935261189938, 0.01602889783680439, -0.002890706295147538, -0.006654699798673391, -0.011090130545198917, 0.009361394681036472, 0.008521530777215958, -0.013803253881633282, 0.019632725045084953, -0.0002047328307526186, -0.013622689992189407, 0.02045765519142151, -0.02087392844259739, 0.013924866914749146, -0.04236096143722534, -0.019162697717547417, -0.009372994303703308, 0.021417802199721336, 0.013537799939513206, -0.012427283450961113, -0.007270066998898983, 0.008674473501741886, -0.0050622462294995785, -0.01655869372189045, 0.02215627208352089, 0.024742769077420235, -0.017202459275722504, -0.003134782426059246, -0.006257885135710239, -0.004813887644559145, -0.0058154938742518425, -0.005157724022865295, 0.02178630605340004, 0.015030576847493649, 0.004232816863805056, 0.01798163540661335, 0.00707532512024045, -0.001221311162225902, 0.004489756654947996, -0.028948038816452026, -0.024309922009706497, -0.0010777185671031475, -0.0229472778737545, 0.011738029308617115, -0.0009064172045327723, -0.010554666630923748, -0.026616040617227554, -0.0032796820160001516, 0.0069382451474666595, -0.006599305663257837, -0.0038924759719520807, -0.02573721669614315, -0.0034195254556834698, -0.021369626745581627, -0.00326200551353395, 0.009610832668840885, -0.008047621697187424, 0.009276073426008224, 0.027360066771507263, 0.013208803720772266, 0.0017265407368540764, -0.0030963898170739412, 0.020880751311779022, 0.0012791628250852227, -0.022708557546138763, -0.01658543199300766, 0.0016209924360737205, -0.007995753549039364, 0.045382309705019, -0.01327313669025898, 0.037515200674533844, -0.009624110534787178, -0.002479467075318098, 0.020547684282064438, 0.008070284500718117, -0.002786042634397745, -0.009268619120121002, 0.000403510028263554, 0.004388471599668264, -0.004799273796379566, 0.0055684903636574745, -0.02127131260931492, -0.0014404482208192348, 0.025047553703188896, -0.00806555524468422, 0.009125892072916031}, {0.02701590582728386, -0.014307424426078796, -0.0063116056844592094, 0.030157385393977165, -0.015141859650611877, 0.009262587875127792, -0.015316859818994999, -0.03827411308884621, -0.02919732965528965, -0.004244181793183088, -0.0017789319390431046, -0.027958644554018974, 0.010886617004871368, 0.030583128333091736, -0.013661925680935383, -0.013596477918326855, 0.01682121306657791, -0.011768278665840626, 0.011292079463601112, -0.0033553249668329954, -0.008749349042773247, -0.028239784762263298, -0.02923460677266121, -0.005725454539060593, -0.010671718046069145, -0.005606609862297773, -0.005815759301185608, 0.011487836949527264, -0.0074861375615000725, -0.0035833893343806267, 0.007386182900518179, -0.0032715629786252975, -0.011126278899610043, 0.007191534619778395, -0.025787517428398132, -0.004100319463759661, 0.020034853368997574, -0.010877124033868313, -0.00015388656174764037, -0.027850637212395668, -0.00031269664759747684, -0.01782085932791233, 0.00996236503124237, 0.010146950371563435, -0.01863393560051918, -0.02889794483780861, -0.01684734970331192, 0.00551668182015419, 0.01588287577033043, 0.012112389318645, 0.0017449541483074427, -0.001967934425920248, 0.02608759142458439, -0.004373452160507441, 0.010777981951832771, -0.012007590383291245, -0.022763662040233612, 0.0037930472753942013, -0.020147036761045456, -0.01743454486131668, -0.004474065266549587, 0.022988436743617058, 0.008816232904791832, -0.014474622905254364, 0.014383837580680847, 0.015577775426208973, 0.013799234293401241, 0.0020193762611597776, -0.013004902750253677, -0.001768838264979422, 0.011024161241948605, -0.03331761807203293, 0.0017453522887080908, 0.0056159039959311485, -0.01177658699452877, 0.02105887606739998, 0.003326431382447481, 0.014899455942213535, -0.0008925383444875479, 0.015165942721068859, 0.00903608463704586, -0.01464670617133379, -0.014985770918428898, 0.018022770062088966, 0.007136070169508457, -0.005498246289789677, 0.02479562722146511, -0.006364461965858936, 0.02563515119254589, 0.0008211600361391902, 0.00030767239513807, 0.014804789796471596, -0.020564699545502663, -0.016489380970597267, 0.005335614550858736, 0.03400516137480736, -0.020932521671056747, 0.007270919159054756, -0.018610309809446335, -0.010106437839567661, 0.016500411555171013, 0.0006414930685423315, -0.016914138570427895, 0.004599436651915312, -0.006139717064797878, 0.014039980247616768, 0.01331758126616478, -0.0012796134687960148, -0.013836516998708248, -0.016449110582470894, -0.014611730352044106, -0.03787318244576454, 0.0008631414966657758, -0.017357811331748962, -0.024610133841633797, -0.001175166922621429, 0.004391888156533241, 0.023235060274600983, -0.005638439673930407, 0.005215546581894159, -0.0071639930829405785, -0.0005690893158316612, -0.013411000370979309, 0.02684888057410717, 0.006054067984223366, 0.004928127396851778, -0.01747993193566799, 0.02592276781797409, -0.004456210415810347, 0.032501716166734695, -0.03580291569232941, -0.021386658772826195, 0.005089799407869577, 0.024402301758527756, -0.012889159843325615, -0.0020734055433422327, -0.033785343170166016, -0.03054044581949711, 0.010195241309702396, 0.0018451572395861149, 0.00045436996151693165, 0.018817612901329994, 0.014448776841163635, -0.0058719441294670105, -0.007403593510389328, -0.001850412110798061, 0.003991385456174612, 0.0045889983884990215, 0.0017807412659749389, 0.02134815789759159, -0.023646188899874687, 0.0225136149674654, -0.0011037986259907484, 0.00818511750549078, 0.0014734624419361353, -0.008422737941145897, 0.008602879010140896, -0.019321026280522346, 6.594711885554716e-05, -0.011592959053814411, 0.01107475534081459, 0.012917979620397091, -0.0043021938763558865, 0.013705287128686905, -0.014255843125283718, 0.04757218062877655, 0.006659704260528088, -0.014444136992096901, 0.007546807639300823, 0.01430194266140461, 0.011031639762222767, -0.0051485877484083176, -0.014426948502659798, 0.0005576311959885061, -0.007985394448041916, 0.01040769275277853, -0.010474683716893196, 0.00025131599977612495, 0.015598389320075512, -0.00620316993445158, 0.0007257001707330346, 0.013193272054195404, 0.0022865429054945707, -0.021506624296307564, -0.054764531552791595, -0.0023645467590540648, 0.01157534122467041, 0.021778050810098648, 0.0007922393269836903, -0.010596801526844501, -0.011178374290466309, -0.021503321826457977, -0.00035529248998500407, 0.0032142584677785635, 0.012535098008811474, 0.020207224413752556, -0.006079389713704586, -0.02092202939093113, 0.011713344603776932, 0.012347959913313389, -0.01322270929813385, -0.02858477644622326, 0.012690680101513863, 0.0035447643604129553, -0.009679870679974556, 0.01068134419620037, 0.0012006203178316355, 0.009857619181275368, -0.020563587546348572, -0.02231132984161377, -0.021589139476418495, 0.008246294222772121, -0.018683554604649544, 0.01332581415772438, 0.021727699786424637, 0.006215828936547041, -0.013187705539166927, -0.013329729437828064, 0.001032227068208158, -0.01287614181637764, -0.005469419993460178, 0.0032208270858973265, 0.011987386271357536, -0.024222657084465027, 0.0022251650225371122, 0.007961194962263107, 0.01010079961270094, 0.00838052574545145, 0.04294958710670471, 0.01574251987040043, -0.008941500447690487, -0.006655651610344648, -0.009450177662074566, -0.009940581396222115, -0.007641930133104324, -0.003533249022439122, 0.010794978588819504, 0.0011174725368618965, 0.04884105920791626, -0.021851148456335068, 0.027085373178124428, 0.00071241072146222, 0.0012628616532310843, 0.028819678351283073, 0.001131720724515617, -0.016263529658317566, -0.02042081020772457, -0.011105891317129135, 0.011317900381982327, -0.004834418185055256, -0.010339638218283653, -0.010002844966948032, 0.0024690907448530197, 0.00013888554531149566, -0.017440365627408028, -0.007060447242110968}, {0.005086972843855619, 0.009706269018352032, -0.0019630270544439554, 0.02516142837703228, -0.028819380328059196, -0.008376388810575008, -0.006750268395990133, -0.0439913235604763, -0.03625897318124771, -0.0002830413868650794, 0.012234572321176529, 0.013106132857501507, -0.012767341919243336, 0.0052213710732758045, 0.00025556545006111264, -0.001622588257305324, 0.02383149228990078, -0.001672298414632678, 0.00204672385007143, -0.0047032590955495834, -6.344555731629953e-05, 0.006662327796220779, -0.01206161268055439, -0.01900046318769455, 0.008029010146856308, 0.004213825333863497, -0.0022150566801428795, 0.013803718611598015, -0.014800540171563625, -0.012018879875540733, 0.002778897760435939, -0.0018382685957476497, -0.015128696337342262, -0.0020526889711618423, -0.014866729266941547, -0.002716028830036521, -0.005538186989724636, -0.0087950574234128, -0.023036956787109375, -0.01610489934682846, 0.009997268207371235, -0.005663963034749031, 0.011775690130889416, 0.0144983334466815, 0.0013049629051238298, 0.0016685368027538061, 0.010443506762385368, -0.013123774901032448, 0.007497658021748066, -0.011448933742940426, -0.012535314075648785, 0.0012742613907903433, 0.02907615527510643, -0.002399746561422944, 0.003906896337866783, -0.02374946139752865, 0.003454056568443775, -0.013405958190560341, -0.01367053110152483, 0.015105495229363441, -0.005121571011841297, -0.013236576691269875, 0.006306834053248167, -0.004814419895410538, 0.00848958920687437, 0.004477197304368019, 0.016590839251875877, -0.005031924229115248, -0.018653368577361107, -0.008543463423848152, 0.008875986561179161, -0.017905572429299355, -0.0051504685543477535, 0.0010707329493016005, -0.008837850764393806, 0.009803995490074158, 0.0078155267983675, 0.021615639328956604, 0.00486000394448638, 0.0033171505201607943, -0.02012786827981472, -0.0060426704585552216, -0.007256797514855862, 0.045751314610242844, -0.017067527398467064, -0.013981868512928486, 0.008096649311482906, -0.004578196443617344, 0.02331373281776905, -0.017980599775910378, -0.01574651710689068, 0.0013606512220576406, -0.012185702100396156, -0.030441515147686005, 0.02672184817492962, 0.017520926892757416, -0.01020493172109127, -0.010440696030855179, -0.007812817580997944, -0.001839484553784132, 0.01861388608813286, -0.003386824857443571, -0.022219642996788025, -0.014108015224337578, 0.003992470446974039, -0.004131624475121498, -0.0067969076335430145, -0.0001384519855491817, -0.006015679333359003, -0.012111050076782703, -0.0124788424000144, -0.04597507417201996, -0.001695118611678481, -0.02496965415775776, -0.01326317060738802, -0.010797818191349506, -0.008191663771867752, 0.029598647728562355, -0.032285548746585846, 0.01762205921113491, -0.013638492673635483, -0.022243786603212357, 0.011093532666563988, 0.028221851214766502, -0.001150481402873993, 0.009080481715500355, -0.011800318956375122, 0.01211421750485897, -0.01860513910651207, 0.002478211186826229, -0.015604960732161999, -0.0025127907283604145, 0.0201409999281168, 0.006802814546972513, -0.003023388097062707, -0.0020735077559947968, -0.027954844757914543, -0.016207030043005943, 0.04528007656335831, 0.021236520260572433, 0.01624045893549919, 0.019696634262800217, 0.022334259003400803, 0.007092715241014957, 0.0048513649962842464, 0.005121003370732069, -0.007648544851690531, 0.012677429243922234, 0.007387334946542978, -0.009768340736627579, -0.016847535967826843, 0.008127229288220406, -0.007590509485453367, -0.010245660319924355, -0.023482659831643105, -0.015157879330217838, -0.0058781420812010765, -0.02519126422703266, 0.01111048087477684, -0.011523636057972908, 0.0028193113394081593, -0.001516793854534626, -0.01650630123913288, 0.0062469481490552425, -0.0028053794521838427, 0.038870327174663544, -0.0027904845774173737, -0.0021639508195221424, -0.004888643976300955, 0.012620337307453156, -0.008952248841524124, 0.0013013252755627036, -0.005854095797985792, -0.011400814168155193, 0.0007183463894762099, 0.008727976121008396, -0.006824877578765154, -0.02066226676106453, 0.012728179804980755, -0.008478667587041855, -0.019151702523231506, 0.013931713066995144, -0.004873650148510933, -0.006517553701996803, -0.036846768110990524, 0.005246561951935291, -0.004978639539331198, 0.023131653666496277, -0.01364401075989008, 0.008550930768251419, 0.006072803866118193, -0.0006522329058498144, 0.017779842019081116, -0.01812605932354927, 0.0013868596870452166, 0.01909971795976162, -0.021968672052025795, 0.009943624958395958, 0.004699402954429388, 0.009881779551506042, -0.009148219600319862, -0.0026663837488740683, 0.00966176949441433, 0.024149253964424133, -0.0012267871061339974, 0.01811806857585907, -0.012333669699728489, 0.01005029957741499, -0.011771407909691334, -0.005928173195570707, -0.014550777152180672, -0.009631647728383541, -0.015253054909408092, 0.01120380125939846, 0.012574100866913795, 0.007281289901584387, -0.02523249015212059, 0.001120643108151853, 0.011853521689772606, -0.008861428126692772, 0.003686400130391121, -0.0028377289418131113, 0.0019342910964041948, -0.01267918199300766, 0.0004909202107228339, 0.017471615225076675, -0.002109629102051258, 0.010700133629143238, -0.00508581567555666, -0.006526992190629244, -0.00467153312638402, -0.018922334536910057, 0.012477499432861805, -0.0032131350599229336, -0.008808719925582409, -0.024800151586532593, 0.0050330148078501225, 0.0039039093535393476, 0.025660747662186623, -0.017316794022917747, 0.0065863411873579025, -0.004059476312249899, -0.0035750858951359987, 0.03503355756402016, -0.013020078651607037, -0.008117392659187317, -0.00977344997227192, 0.005969179794192314, -0.012987695634365082, 0.012127373367547989, -0.0204458050429821, -0.015927428379654884, -0.03267985209822655, 0.004964452236890793, -0.01148160919547081, -0.002211949322372675}, {-0.13807599246501923, 0.9958008527755737, -0.38085803389549255, 0.14036308228969574, -0.5472970604896545, 0.37131088972091675, 0.23967105150222778, 0.30211275815963745, -0.7650676965713501, -0.665885865688324, -0.14209257066249847, -0.04575112834572792, 1.3428564071655273, 0.6987465023994446, 0.8236506581306458, 1.634804368019104, 0.16964174807071686, 0.3263663351535797, 0.765220046043396, -0.2331426441669464, 0.9996675848960876, 2.4447498321533203, 0.9584096074104309, 0.030697345733642578, -0.20053096115589142, -0.058311786502599716, 2.022468090057373, -0.3198680877685547, 0.6033015251159668, 0.3307511806488037, -0.1693815141916275, 1.4837726354599, -0.509953498840332, -0.7803688049316406, 1.1256544589996338, 0.2042239010334015, 1.128243327140808, 0.5017570853233337, 0.15593545138835907, -0.9142122268676758, -0.08383505791425705, 1.5678707361221313, 1.407779574394226, 0.3920643627643585, 0.6160088181495667, 0.4423612058162689, 0.4532719552516937, -0.27371305227279663, 0.30162182450294495, 0.9922999739646912, 0.1704837530851364, 0.2920771539211273, -0.1692350208759308, 0.4285103976726532, 0.994130551815033, 1.128116488456726, -0.0008208023500628769, -0.5601466298103333, 0.9493725895881653, -0.12274579703807831, -1.438973307609558, -0.5811335444450378, -0.3170970678329468, -0.18785232305526733, 0.9172594547271729, -0.377613365650177, -1.7743213176727295, -0.7204593420028687, 0.7437013983726501, -1.0790390968322754, -0.4021615982055664, 0.1558469980955124, -0.6317262053489685, -0.44652798771858215, 1.0242947340011597, -0.42540600895881653, 0.1647723913192749, -0.33524131774902344, -0.04873877763748169, -0.062122538685798645, 0.05899422615766525, 0.6025299429893494, 0.46945443749427795, 0.5277450680732727, 0.39764076471328735, 0.04341588914394379, 0.00049022858729586, -0.1280500888824463, 0.8486993908882141, 0.562499463558197, 1.0869098901748657, 0.7556085586547852, -0.27925166487693787, 0.3834792375564575, -0.11170633882284164, -0.37061962485313416, -0.236570343375206, -0.014753260649740696, 1.0617412328720093, 1.7664523124694824, 1.2365747690200806, -0.15612654387950897, 0.2343222051858902, 0.32883188128471375, -0.12531034648418427, 1.9567290544509888, -1.8504201173782349, 0.23271511495113373, -0.50809246301651, -0.1792508214712143, -1.0562560558319092, -0.27126455307006836, 0.15692809224128723, -0.10692750662565231, 0.7999501824378967, 0.4204273223876953, 0.13780827820301056, 0.24649262428283691, -0.5132327675819397, -0.3625836670398712, 0.1382947713136673, 1.4214259386062622, 0.5950524806976318, 0.6427386403083801, 0.6077125668525696, -0.1349356323480606, -1.621017336845398, -0.7953460812568665, 0.4529038965702057, 0.525714099407196, 0.2771797776222229, 0.018790382891893387, -0.7985928654670715, -0.5186203718185425, 1.1600844860076904, -0.011740374378859997, 0.1783514767885208, -0.09879735112190247, -0.02718823403120041, 0.19238169491291046, -0.22330182790756226, -0.23575499653816223, -0.6145516037940979, -1.2417072057724, 0.4208735525608063, 0.005128245335072279, -0.6837257742881775, 0.18689337372779846, -0.7028688788414001, -0.6867625713348389, 0.31857919692993164, 0.5192115902900696, 0.20825038850307465, -0.6047376394271851, 0.60101318359375, -0.1383706033229828, -0.33969226479530334, 1.2533032894134521, 0.31995150446891785, 0.7552481293678284, 0.9762201309204102, -0.3310515284538269, -0.2582346200942993, 0.1006489247083664, 0.2203865647315979, -0.13051177561283112, -0.30994340777397156, 0.06578502804040909, -0.17009013891220093, 0.13243766129016876, -0.5214771032333374, -0.07976377755403519, 0.527364194393158, 0.0263685155659914, -0.02864079363644123, -1.661266803741455, 0.6093137860298157, 0.19892875850200653, -1.0828852653503418, 0.8608688712120056, 0.2192113995552063, -0.19016766548156738, 0.1448783278465271, -0.5146323442459106, -0.30361345410346985, -0.08414489775896072, 0.31470444798469543, 0.6677347421646118, 0.8888280391693115, -0.5899543762207031, 1.5831279754638672, 0.27214956283569336, 0.29568201303482056, 0.21192944049835205, -0.8106935024261475, -1.415269136428833, -0.4473802447319031, -0.3544597327709198, -1.8627680540084839, -0.4746645390987396, 0.7482116222381592, 0.29869580268859863, -0.3099307417869568, -0.4455907940864563, -1.6168570518493652, -0.6803581714630127, 0.27892202138900757, -0.3379386365413666, -0.2133038192987442, 0.7732887268066406, -0.19874846935272217, 0.10090626776218414, -0.4046030342578888, -0.5026067495346069, -0.4096979796886444, -0.32548657059669495, -0.2014649659395218, -0.35901781916618347, -0.6764171719551086, -0.5505348443984985, 0.9515145421028137, 0.47425577044487, 0.793151319026947, 0.5007591247558594, 0.2909347116947174, 0.4678312838077545, -0.9657090902328491, -0.5768554210662842, 0.30795490741729736, 0.9088689088821411, 0.29919394850730896, 0.8423497080802917, -0.40709224343299866, -0.07793671637773514, -0.5841571688652039, 0.15325073897838593, -0.282459020614624, -0.14505377411842346, -0.6017128825187683, 1.3782529830932617, -0.26871100068092346, -0.8112800717353821, 0.204164057970047, -0.7245027422904968, 0.43232494592666626, -0.31612905859947205, 0.2826373875141144, -0.5583146810531616, -0.2510840594768524, -0.2360856831073761, -0.5870264172554016, 0.39799004793167114, -0.11945898085832596, -0.8122803568840027, 0.34590578079223633, 0.02387049049139023}, {0.007699370384216309, -0.01154345739632845, -0.017788872122764587, 0.024601975455880165, -0.037221625447273254, 0.002602518768981099, -0.0026814821176230907, -0.017256110906600952, -0.030581502243876457, -0.0020883132237941027, 0.007101928815245628, -0.002216740045696497, 0.007038281764835119, -0.00910361111164093, -0.0016974471509456635, 0.0026694282423704863, 0.00890427641570568, -0.023145658895373344, 0.01282650325447321, -0.016608277335762978, 0.010039782151579857, -0.024834010750055313, -0.0017694063717499375, -0.0244891420006752, 0.0020975200459361076, 0.005458891857415438, -0.0004945914843119681, -0.003139869077131152, -0.02746272087097168, -0.006371206138283014, 0.014194960705935955, 0.007508496288210154, -0.004747724626213312, 0.01496528647840023, -0.015359720215201378, -0.021313456818461418, -0.006125309504568577, -0.0011834476608783007, 0.004388704430311918, -0.02115027792751789, 0.021801244467496872, 0.021552003920078278, 0.020732391625642776, 0.007756097242236137, -0.0049401032738387585, -0.003977223299443722, 0.0010385143104940653, 0.01164609007537365, 0.01464891154319048, -0.009386233054101467, 0.0031193261966109276, -0.0031765925232321024, 0.012002715840935707, -0.007824456319212914, 0.02107810415327549, -0.027074242010712624, -0.003962346818298101, 0.015000132843852043, -0.022623606026172638, -0.021688465029001236, 0.007682088762521744, -0.008349315263330936, 0.008702727034687996, -0.00874622818082571, -0.0024353214539587498, 0.010323375463485718, 0.02005743235349655, 0.0023181913420557976, -0.017810167744755745, -0.02345837838947773, 0.012271073646843433, -0.0024264336097985506, -0.011157739907503128, 0.0034745398443192244, -0.015009395778179169, 0.0010545956902205944, -5.04848976561334e-05, 0.01918759010732174, -0.008442984893918037, 0.0008353005396202207, -0.005767031107097864, 0.001593523076735437, -0.009579505771398544, 0.005416416563093662, -0.003311483422294259, -0.004848422948271036, 0.011076327413320541, -0.010485835373401642, 0.01742396131157875, -0.01910565234720707, -0.029098983854055405, -0.00021308864234015346, -0.013153763487935066, -0.02864380180835724, 0.0017954460345208645, 0.020846646279096603, -0.03824739158153534, -0.0030880055855959654, -0.016409780830144882, -0.01206604391336441, 0.011884626932442188, 0.012926524505019188, 0.014341098256409168, -0.00267665833234787, -0.011338657699525356, -0.01333540678024292, -0.01192132756114006, -0.01244791317731142, -0.009974905289709568, -0.0029307971708476543, -0.009080404415726662, -0.04160451889038086, -0.0007318907300941646, -0.002241740468889475, -0.011849935166537762, 0.012064487673342228, -0.003384576877579093, 0.04106413573026657, 0.009951499290764332, 0.029671253636479378, -0.017589809373021126, -0.033947959542274475, 0.01287137996405363, 0.008147752843797207, 0.014672745019197464, 0.005037902854382992, 0.007908747531473637, 0.021831588819622993, -0.01640336588025093, 0.018686095252633095, -0.018934575840830803, -0.005274403374642134, -0.0012779851676896214, -0.0016885906225070357, -0.006181146018207073, 0.010749614797532558, -0.014846893027424812, -0.0008735877927392721, 0.0008714294526726007, 0.0036965475883334875, 0.007018789649009705, 0.012475118972361088, -0.0012841616990044713, 0.004940352402627468, 0.011258440092206001, 0.000867260794620961, 0.00699113542214036, 0.011987722478806973, 0.00685877725481987, 0.012333033606410027, -0.016139084473252296, 0.02561046928167343, -0.002357503166422248, 0.01653148978948593, -0.03578982502222061, 0.005757228937000036, -0.007139305118471384, -0.024435503408312798, 0.007912890054285526, -0.02376263402402401, 0.016600295901298523, 0.005602696910500526, -0.006793344393372536, 0.0011946666054427624, -0.004442095290869474, 0.04039647430181503, -0.0019150503212586045, -0.004010961391031742, -0.008908860385417938, 0.019990192726254463, 0.025023626163601875, -0.005870970897376537, 0.0036918572150170803, -0.00895592849701643, 0.005480421707034111, 0.009383552707731724, 0.00661962665617466, -0.0033221407793462276, 0.0011373000452294946, -0.014572612009942532, -0.0017278965096920729, 0.006224242504686117, -0.005470207426697016, 0.011508938856422901, -0.04091320559382439, -0.01945948600769043, 0.007684964686632156, 0.03386758267879486, 0.008357501588761806, -0.008485591970384121, -0.00539065059274435, -0.002188298152759671, 0.008958888240158558, -0.006210811901837587, 0.00989502016454935, 0.044467341154813766, -0.03114045411348343, -0.00952580664306879, 0.0031286573503166437, 0.018744241446256638, -0.01989506371319294, 0.006254525389522314, 0.02184016816318035, 0.006934149190783501, -0.002207839395850897, 0.025101440027356148, -0.004725165199488401, 0.009661216288805008, -0.012973478063941002, -0.03333015367388725, -0.02428794838488102, 0.0020317567978054285, -0.014136208221316338, 0.01670045033097267, 0.0006145566003397107, 0.015099171549081802, -0.02716837264597416, -0.002477502916008234, 0.015519238077104092, -0.0019153256434947252, 0.0013717274414375424, -0.0007499305647797883, -0.005201893858611584, -0.009234128519892693, 0.0013053800212219357, -0.012395613826811314, -0.011399639770388603, 0.012387643568217754, 0.012511183507740498, 0.02312402054667473, -0.011855892837047577, -0.013695492409169674, 0.012236558832228184, 0.007786097936332226, -0.020075520500540733, -0.015446056611835957, -0.021268580108880997, 0.0030677039176225662, 0.04453666880726814, -0.003947929013520479, 0.020974716171622276, 0.013990674167871475, -0.008157141506671906, 0.019267287105321884, -0.0017902149120345712, 0.005918503273278475, -0.008424552157521248, 0.008086543530225754, -0.011162202805280685, 0.013248695991933346, -0.016025377437472343, -0.017331501469016075, -0.027154935523867607, 0.006376989651471376, -0.010849528945982456, -0.01577560044825077}, {0.006312937010079622, -0.014021393842995167, 0.014373737387359142, 0.020913606509566307, -0.032791562378406525, -0.011973158456385136, -0.008740890771150589, -0.021959131583571434, -0.019354335963726044, 0.004132706671953201, 0.00486079603433609, -0.0013688827166333795, -0.007392859552055597, 0.018043778836727142, 0.0032750931568443775, 0.00225816504098475, 0.013213159516453743, -0.028395431116223335, 0.007666524033993483, -0.009374622255563736, -0.005056735128164291, -0.01194762159138918, -0.03129937872290611, -0.026519332081079483, -0.010004294104874134, -0.01918698661029339, 0.0017336017917841673, 0.0065236338414251804, -0.017790520563721657, -0.01022565457969904, -0.007017753552645445, -0.004792171996086836, 0.010842941701412201, 0.004429408349096775, -0.01965808868408203, -0.008053258061408997, -0.0008682134211994708, -0.008001315407454967, -0.027607180178165436, -0.03168052062392235, -0.018404362723231316, 0.0034028212539851665, 0.02161405049264431, -0.004175856709480286, -0.0012370364274829626, -0.011318832635879517, 0.020426735281944275, 0.006991002708673477, 0.021060368046164513, -0.015828276053071022, -0.0005158452549949288, -0.005135924089699984, 0.014109148643910885, -0.009725639596581459, 0.0020997391548007727, -0.036522600799798965, -0.017984651029109955, -0.008801900781691074, -0.012723528780043125, -0.02580844610929489, 0.011069116182625294, -0.00436403090134263, -0.01537835318595171, -0.007819969207048416, -0.011888852342963219, 0.015137741342186928, 0.017815537750720978, -0.0015603748615831137, -0.011731905862689018, -0.019404806196689606, 0.004576786886900663, -0.020603958517313004, -0.003841645549982786, 0.004403884056955576, 0.0006721007521264255, 0.0137641541659832, -0.026099255308508873, 0.021106528118252754, 0.011123365722596645, -0.004439225886017084, -0.012231351807713509, -6.0487742302939296e-05, -0.008159621618688107, 0.01498531922698021, -0.012692820280790329, 0.011230239644646645, 0.021332625299692154, -0.015679193660616875, 0.02091497741639614, -0.03528958559036255, -0.0027539758011698723, 0.011855723336338997, -0.002218110952526331, -0.027270996943116188, -0.007893943227827549, 0.020987702533602715, -0.02423395775258541, -0.008377288468182087, -0.014163482934236526, -0.02275538444519043, 0.013327335007488728, -0.004272140096873045, -0.0225489754229784, -0.005034885834902525, -0.0057000829838216305, -0.007501459680497646, 0.011357292532920837, -0.01714594103395939, -0.020558765158057213, -0.01757267862558365, -0.009220896288752556, -0.048787157982587814, -0.006856714840978384, -0.02094709873199463, -0.01943218521773815, 0.0019119149073958397, -0.026726724579930305, 0.0238479096442461, -0.010526980273425579, 0.022991806268692017, 0.008788034319877625, 0.002488879021257162, -0.00475683156400919, 0.009997422806918621, 0.006940566468983889, 0.015306836925446987, 0.0027822840493172407, 0.019415948539972305, -0.004941255785524845, 0.02599492110311985, -0.0031956611201167107, 0.009458692744374275, -0.004536665976047516, -0.0030545275658369064, -0.025666790083050728, -0.00044128773151896894, -0.011729096062481403, -0.011636670678853989, -0.013878308236598969, 0.011928323656320572, 0.005517379846423864, 0.016222018748521805, -0.009323956444859505, 0.008079148828983307, -0.0064186775125563145, 0.009671187028288841, 0.0010436539305374026, 0.03621627762913704, 0.02092590369284153, 0.009684663265943527, -0.022047134116292, -0.0009821347193792462, -0.014502836391329765, 0.004205015022307634, -0.023736491799354553, -0.0036510052159428596, 0.006491340231150389, -0.009595430456101894, 0.018896112218499184, 0.00316603877581656, 0.006347988732159138, -0.010308058001101017, -0.012269612401723862, 0.0037168776616454124, -0.01585003361105919, 0.016715794801712036, -0.017155202105641365, 0.006893039681017399, 0.012667099945247173, 0.023223748430609703, -0.006477563641965389, 0.022577686235308647, -0.0037881697062402964, -0.007586577907204628, -0.01469136867672205, 0.001614664332009852, 0.021234318614006042, -0.006421022582799196, -0.003847152227535844, -0.021476544439792633, -0.005808393005281687, -0.0037431290838867426, 0.0023865760304033756, -0.0008676721481606364, -0.028527403250336647, -0.01834673248231411, 0.007996938191354275, 0.014201272279024124, -0.0007296576513908803, -0.013097667135298252, 0.024116627871990204, -0.011862263083457947, 0.0031734758522361517, -0.0021269621793180704, -0.007292069960385561, 0.022340215742588043, -0.010766888968646526, -0.005843544378876686, -0.0034703875426203012, 0.008017751388251781, -0.007667181082069874, 0.012386375106871128, -0.007923705503344536, 0.020720286294817924, -0.009190216660499573, 0.03912002965807915, -0.00031987461261451244, 0.009069016203284264, 0.007303932216018438, -0.026039423421025276, -0.0425797738134861, -0.009295115247368813, -0.025442758575081825, 0.017917459830641747, 0.031055953353643417, 0.004434535279870033, -0.024086035788059235, -0.0025787975173443556, -0.003762454492971301, -0.014410693198442459, -0.017744526267051697, -0.018073681741952896, 0.017432618886232376, 0.0005608612555079162, -0.015726923942565918, 0.008802449330687523, 0.015230236575007439, 0.006770451087504625, 0.02719356305897236, 0.00011778366024373099, -0.012542136944830418, 0.007576365023851395, 0.01854475773870945, -0.0015038879355415702, -0.012807638384401798, -0.014937474392354488, 0.01744788885116577, -0.022112173959612846, 0.03790470212697983, -0.007230916526168585, 0.02914435975253582, -0.010653886012732983, -0.00519454013556242, 0.014808929525315762, 0.005558008793741465, 0.004657278303056955, -0.018459752202033997, 0.010441293939948082, 0.012586905620992184, 0.0014872400788590312, 0.009243955835700035, 0.008519995026290417, -0.005349571816623211, 0.004488968290388584, 0.009244327433407307, -0.011540311388671398}, {0.18426038324832916, -0.0021008974872529507, 0.26694729924201965, 0.43324607610702515, -0.10653579235076904, -0.1119532659649849, -0.2330295294523239, 0.13496653735637665, 0.06428525596857071, 0.23401716351509094, -0.05009440705180168, 0.23763062059879303, -0.4249621629714966, -0.24916592240333557, -0.013875650241971016, -0.070929154753685, -0.15117038786411285, -0.20510825514793396, 0.021020440384745598, 0.157204732298851, -0.4195738732814789, 0.021214544773101807, -0.37329187989234924, -0.29357069730758667, -0.5563079118728638, 0.16576838493347168, -0.6752952933311462, -0.1310928612947464, -0.359776109457016, -0.18116535246372223, 0.33019155263900757, -0.26552966237068176, -0.01144248154014349, 0.381989061832428, -0.2851022779941559, 0.020601270720362663, 0.035210251808166504, -0.060803938657045364, -0.2006801962852478, -0.03437468037009239, -0.09538865834474564, -0.4240535795688629, -0.40517210960388184, 0.25174564123153687, -0.35047194361686707, -0.01848785951733589, 0.0467362217605114, 0.12775976955890656, -0.03211419656872749, -0.23958690464496613, 0.0017906759167090058, -0.2838798463344574, 0.1737271547317505, -0.13033370673656464, 0.2641523778438568, -0.18766744434833527, 0.22555048763751984, 0.21045401692390442, -0.2489260584115982, 0.010684621520340443, -0.10565366595983505, 0.22666700184345245, 0.3110465705394745, 0.1724139302968979, -0.4090389311313629, 0.08337514102458954, 0.5354811549186707, 0.3351849913597107, -0.3017374575138092, 0.23394106328487396, -0.05732439085841179, 0.4811452329158783, 0.48749852180480957, 0.17581462860107422, -0.09407640248537064, 0.1663905382156372, 0.16871574521064758, -0.006587465293705463, -0.056403499096632004, -0.7282756567001343, 0.08305887132883072, 0.08302897214889526, -0.3114475905895233, -0.0029936956707388163, -0.1725616306066513, 0.05263839289546013, -0.18241357803344727, 0.049120623618364334, -0.07019977271556854, -0.39065811038017273, -0.09777721762657166, -0.017576424404978752, 0.06273595988750458, -0.2113553285598755, 0.034908730536699295, 0.17616654932498932, -0.10961712151765823, 0.045837268233299255, 0.1739712357521057, -0.22238682210445404, -0.1714545339345932, -0.2134859412908554, -0.8819065690040588, -0.015776334330439568, 0.010259106755256653, 0.23885993659496307, -0.03544919192790985, -0.30792948603630066, -0.2542386054992676, 0.5877697467803955, -0.2459411770105362, 0.09236901253461838, 0.1956908404827118, -0.36650604009628296, -0.12606923282146454, 0.0007428618846461177, -0.20108194649219513, 0.1676999032497406, -0.30817171931266785, 0.43575215339660645, -0.1410350650548935, -0.16841132938861847, 0.058968435972929, -0.10112825781106949, -0.00592131307348609, 0.0049348329193890095, 0.30546364188194275, 0.5204115509986877, -0.3675256371498108, -0.30901458859443665, -0.36277109384536743, -0.4217090606689453, -0.4668750464916229, 0.43904194235801697, 0.06109914928674698, 0.06590060144662857, -0.1829075962305069, 0.04218599572777748, 0.05020220950245857, 0.029415711760520935, 0.25078675150871277, 0.10466311872005463, 0.2024962157011032, 0.00983024388551712, -0.3330387473106384, 0.39343035221099854, 0.327204167842865, 0.14345723390579224, -0.014800339937210083, 0.23494720458984375, -0.17317892611026764, 0.0426386296749115, -0.2644708454608917, 0.5892621874809265, -0.11954360455274582, -0.13868339359760284, 0.10679598897695541, -0.4098110496997833, 0.03256303817033768, -0.1755267083644867, -0.03960055857896805, -0.03787999972701073, 0.01035374402999878, -0.05576476827263832, -0.2056368738412857, -0.2012059986591339, -0.12273166328668594, 0.0037497414741665125, 0.6412628293037415, 0.05926240608096123, -0.27024024724960327, -0.3149482309818268, -0.04817497730255127, 0.1475788950920105, 0.2024729996919632, 0.030903931707143784, -0.31507641077041626, -0.2649882137775421, 0.5359505414962769, -0.3600286841392517, 0.07825198024511337, 0.24226264655590057, -0.0016569372965022922, 0.3973686397075653, -0.043543741106987, 0.3408639430999756, -0.14476965367794037, -0.050955601036548615, 0.2154979705810547, -0.12210220098495483, 0.3607068955898285, -0.6127798557281494, 0.06582421064376831, 0.07760266214609146, 0.26702773571014404, 1.0288857221603394, -0.14502516388893127, 0.18664540350437164, 0.11344794183969498, -0.026123523712158203, -0.4360249936580658, -0.23444874584674835, 0.18787169456481934, 0.4117524027824402, 0.19254009425640106, 0.29487344622612, 0.022720377892255783, -0.6494163870811462, 0.09294016659259796, -0.30120259523391724, 0.19156348705291748, -0.7038792371749878, -0.1741376519203186, 0.2014700323343277, 0.5041890144348145, 0.3742179572582245, -0.10520074516534805, -0.11014469712972641, -0.063881516456604, -0.04149271547794342, -0.17294622957706451, 0.2414882928133011, 0.004293626639991999, -0.8303593397140503, -0.23117366433143616, -0.24096274375915527, 0.15173088014125824, -0.093107670545578, 0.19893048703670502, -0.2529258728027344, -0.4122864007949829, -0.25791043043136597, 0.5140615701675415, 0.29272767901420593, -0.22562366724014282, -0.1235857605934143, 0.13767988979816437, -0.02075585350394249, 0.15505947172641754, -0.32734376192092896, 0.04534848406910896, 0.15791641175746918, 0.1793527603149414, -0.0024253998417407274, 0.07415397465229034, -0.47699180245399475, -0.2237880527973175, -0.05447931960225105, 0.17317380011081696, 0.015820719301700592, -0.13231615722179413, -0.6910858154296875, 0.34961554408073425, -0.13289378583431244, 0.042629845440387726, -0.16224199533462524}, {0.009537079371511936, -0.024990560486912727, -0.005621772725135088, 0.014902569353580475, -0.0246536023914814, -0.012019628658890724, -0.00916118361055851, -0.0165685024112463, -0.016396528109908104, 0.0030162865296006203, 0.00014593196101486683, -0.007017021998763084, -0.011600389145314693, 0.007557539269328117, 0.004254350904375315, -0.010443422943353653, 0.005510842893272638, -0.0076036592945456505, 0.004179785028100014, -0.014832334592938423, 0.006413305643945932, -0.018642740324139595, -0.021360890939831734, -0.0005186931230127811, 0.00652180565521121, 0.002158170100301504, 0.003165431087836623, 0.013614453375339508, -0.011897513642907143, -0.03146379068493843, 0.0024497827980667353, 0.009973609820008278, -0.023352494463324547, -0.009316050447523594, -0.008730301633477211, -0.008183381520211697, -0.007808983791619539, -0.011345388367772102, -0.011776930652558804, -0.031638868153095245, -0.006093787495046854, 0.0024069533683359623, 0.009608862921595573, 0.0026356629095971584, -0.0017419662326574326, 0.006701160687953234, 0.0032410267740488052, 0.010355313308537006, 0.026577366515994072, -0.017737826332449913, -0.019679920747876167, 0.007716264110058546, 0.026143047958612442, 0.01143919862806797, 0.010790356434881687, -0.015440771356225014, 0.008691729977726936, 0.01507595181465149, -0.020077256485819817, -0.018990369513630867, -0.022221747785806656, 0.002325073117390275, 0.0012826516758650541, -0.010593053884804249, 0.00640203058719635, 0.008127622306346893, 0.007065615616738796, -0.0035629982594400644, -0.0033497479744255543, -0.019104382023215294, 0.004934047814458609, -0.009939182549715042, -0.01114444900304079, 0.012315310537815094, -0.013665629550814629, 0.0008211469394154847, 0.00726408651098609, 0.007957084104418755, 0.009019399993121624, 0.001166882342658937, -0.007471594028174877, 0.005689320620149374, -0.02662106603384018, 0.020445410162210464, -0.016990313306450844, 0.018996402621269226, -0.009475498460233212, 0.00023150609922595322, 0.014601113274693489, -0.030674664303660393, -0.0015118372393772006, 0.01322943065315485, 0.01894417032599449, -0.012540815398097038, 0.007403110153973103, 0.025123653933405876, -0.01935010962188244, -0.014521054923534393, -0.013691411353647709, -0.027119120582938194, 0.0232606939971447, 0.002398813609033823, -0.015057254582643509, -0.011873417533934116, -0.013958856463432312, -0.026039835065603256, -0.02180876024067402, -0.005763087887316942, -0.003610817017033696, 0.007183405105024576, -0.0306780356913805, -0.021817130967974663, -0.014658305794000626, -0.0205584354698658, -0.008305558934807777, 0.005880947224795818, -0.020451374351978302, 0.022837085649371147, 0.0035605360753834248, 0.011210227385163307, -0.015715060755610466, -0.028424140065908432, -0.005780338309705257, 0.013559514656662941, 0.0021670653950423002, 0.02475465089082718, 0.0014004746917635202, 0.02789492718875408, -0.0056466530077159405, 0.007077932823449373, -0.011198548600077629, -0.0027877928223460913, -0.003969032783061266, 0.00494523486122489, -0.016625721007585526, 0.015746718272566795, -0.031141964718699455, -0.016254696995019913, -0.0018801564583554864, 0.003734532045200467, 0.015887176617980003, 0.024153048172593117, 0.004803128074854612, 0.0006901660235598683, 0.0033809663727879524, 0.01596127450466156, -0.006120646372437477, 0.01136142946779728, -0.0011755760060623288, -0.004651293624192476, -0.03446134924888611, 0.024925298988819122, -0.01665748655796051, -0.01960778422653675, -0.021463945508003235, -0.008570777252316475, 0.00650819344446063, -0.015564787201583385, 0.0015370401088148355, -0.020679453387856483, 0.0008653371478430927, -0.004674417898058891, -0.0063628340139985085, 0.008824283257126808, 0.00163795065600425, 0.055786628276109695, -0.0036607335787266493, 0.007848977111279964, -0.0029643471352756023, 0.020884834229946136, 0.011586792767047882, 0.00010500193457119167, -0.00927039049565792, -0.0020022636745125055, -0.01661420240998268, 0.010498441755771637, 0.0022043930366635323, -0.025145120918750763, 0.022510448470711708, -0.023011811077594757, -0.026560351252555847, -0.008243247866630554, -0.03227488696575165, 0.009798916056752205, -0.030847759917378426, -0.005842010956257582, 0.004050366580486298, 0.02623838186264038, 0.003679283894598484, -0.022345008328557014, 0.007747883442789316, -0.007286677602678537, 0.006171811372041702, -0.005927340593189001, 0.00461336225271225, 0.029739100486040115, 0.004780460149049759, 0.01813427172601223, 0.002610952826216817, 0.006701112259179354, -0.013602813705801964, -0.004792389459908009, 0.011864894069731236, -0.0036146058700978756, -0.005528360605239868, 0.03128151223063469, 0.011849293485283852, 0.009661007672548294, -0.006263983901590109, -0.01614214852452278, -0.019948164001107216, -0.011112851090729237, -0.02858014963567257, 0.007203032728284597, 0.016944365575909615, -0.009364704601466656, -0.015537027269601822, 0.009336702525615692, -0.0028071256820112467, -0.0009022183367051184, 0.0007918971823528409, -0.014325276017189026, -0.019124047830700874, -0.014357469975948334, -0.00042905285954475403, 0.005410561803728342, -0.007864761166274548, -0.005793163552880287, 0.0207943394780159, 0.02095683291554451, 0.005575018469244242, -0.003338092938065529, 0.011342670768499374, 0.0009183960501104593, -0.017010705545544624, 0.009740440174937248, 0.01606637053191662, -0.00974555779248476, 0.03456439822912216, -0.008471189066767693, 0.01058065053075552, -0.0015378366224467754, -0.01175497192889452, 0.024484459310770035, 0.0065000238828361034, -0.03225184604525566, -0.010917984880506992, 0.003580296179279685, 0.003465841058641672, 0.009161373600363731, -0.014289884828031063, -0.012312487699091434, -0.01939995214343071, 0.015786245465278625, 0.01123830210417509, -0.010119417682290077}, {-0.6040716171264648, -0.3005532920360565, -0.10568948090076447, -0.26454201340675354, 0.9376295804977417, -0.22227533161640167, 0.547606885433197, -0.11889904737472534, 0.6936873197555542, -0.5291278958320618, 0.2973232567310333, -0.2075149416923523, -0.5147068500518799, 0.2577449381351471, -0.2299681007862091, 0.04108230024576187, -0.17379118502140045, 0.0075359768234193325, -1.169764757156372, 0.46781694889068604, -0.08013172447681427, 0.07715253531932831, 0.4859809875488281, 0.5942016839981079, 0.6961230635643005, 0.09241491556167603, -0.6004629731178284, 0.08727282285690308, 0.42465436458587646, 0.45440056920051575, -0.6672317385673523, -0.20662643015384674, 1.0621927976608276, 0.016124257817864418, 0.7750363945960999, 0.19268420338630676, 0.40286338329315186, 0.5506264567375183, 0.762689471244812, 1.2742887735366821, -0.14954297244548798, -0.3638377785682678, -1.6108028888702393, 0.05042920261621475, 0.4558548033237457, 0.20124322175979614, -0.27952224016189575, -0.20153269171714783, 0.32646986842155457, 0.05140949785709381, 0.14616285264492035, 0.4870683550834656, -0.5461223125457764, 0.19172583520412445, -0.418533593416214, 0.35711854696273804, -0.7783846855163574, -0.25643107295036316, 0.1837124228477478, 0.25510308146476746, -0.16736000776290894, 0.1471274048089981, 0.3206644058227539, 0.717822790145874, 0.6810026168823242, -0.4805765450000763, -1.527035117149353, 0.4723628759384155, 0.22137829661369324, -0.07361730933189392, -0.36469998955726624, 0.2709304690361023, -0.7198094129562378, -0.396498441696167, 0.7966163754463196, -0.4645836353302002, 0.15210452675819397, -0.21068550646305084, 0.21767045557498932, 0.2938355505466461, 0.32729217410087585, 0.3327087461948395, 0.32964026927948, -1.2943942546844482, 0.0986706092953682, -0.4247114360332489, -0.6216331124305725, 0.8607040047645569, -0.5663437843322754, 0.3785648047924042, 0.2595905661582947, -0.38163578510284424, 0.2675098776817322, 0.15629376471042633, 0.0738200843334198, -0.46477341651916504, 0.6141787171363831, -0.0106206014752388, -0.07531746476888657, 0.13900180160999298, -0.8092325329780579, -0.22032421827316284, 0.37368232011795044, 0.43770086765289307, 1.0668216943740845, -0.314395546913147, 0.5340865254402161, 0.02514391764998436, 0.3572353720664978, -0.6961880922317505, 0.9491049647331238, 0.35558775067329407, -0.3036867678165436, 0.5123421549797058, -0.9358363747596741, 0.04511897265911102, 0.2870219349861145, -0.9642600417137146, 0.5909785032272339, -0.45668837428092957, 0.3914318084716797, -0.03201735019683838, -0.33922797441482544, -0.05036364123225212, -0.6776310205459595, 0.2798416018486023, 0.506151020526886, -0.38750046491622925, 0.21820780634880066, -0.5729243755340576, 0.14536239206790924, -0.06584463268518448, 1.071918249130249, -0.21700303256511688, -0.14306221902370453, 0.5038193464279175, 0.32249385118484497, -0.402691125869751, -0.18226854503154755, 0.2002956122159958, -0.7101866602897644, -0.28062915802001953, -0.22986872494220734, -0.1990058571100235, -0.34573638439178467, -0.38358578085899353, -0.35475242137908936, -0.25125208497047424, 0.6287448406219482, 0.14946573972702026, 0.25366318225860596, -0.5212588310241699, 0.5612882971763611, -0.3305581510066986, 0.05634239688515663, -0.01772465743124485, -0.022078873589634895, 0.47699132561683655, -0.24629323184490204, -0.17976193130016327, -0.5326741933822632, 0.03551945462822914, 0.37395012378692627, -0.2832677662372589, 0.25752028822898865, -0.1254851520061493, 0.46221673488616943, -0.06328321248292923, 0.06450250744819641, -0.21676523983478546, 0.2644243538379669, 0.4742904305458069, 0.41308075189590454, 0.2849336564540863, 0.3026282787322998, 0.6208623051643372, 0.867328941822052, 0.5292098522186279, -0.820292055606842, 0.4723318815231323, 0.11169013381004333, 0.16118600964546204, 0.10369405150413513, -0.7033466100692749, 1.6344239711761475, 0.29112541675567627, 0.7593635320663452, -2.8909857273101807, -0.6415083408355713, 0.2732832729816437, 0.04643671214580536, 0.29148492217063904, 0.11302582174539566, 0.0919627919793129, -0.23282083868980408, -0.9785445928573608, 0.5372404456138611, 0.14697182178497314, 0.006559670437127352, 0.08915798366069794, 0.3190438449382782, 0.2936175465583801, 0.15390290319919586, -0.6552149057388306, 0.24892191588878632, 0.2200658619403839, -0.3126238286495209, 0.023927049711346626, 0.22030377388000488, 0.3734290301799774, 0.597751259803772, 0.6255151629447937, 0.8108633756637573, -0.5005789995193481, -0.04300352931022644, -0.4484448730945587, 0.847671389579773, 0.582869827747345, 0.567875862121582, 0.34502843022346497, -0.8773329257965088, -0.338192343711853, -0.47729527950286865, 0.7891069650650024, 0.40072304010391235, 0.37279367446899414, -0.2929329574108124, -0.23292458057403564, -0.5364488959312439, -0.9863284826278687, -0.03481804206967354, 0.30932724475860596, -0.35732218623161316, 0.07820601761341095, 0.3649047911167145, 0.5254560708999634, -0.41594627499580383, 0.7318499684333801, -0.33578160405158997, 0.6310462951660156, -0.08033127337694168, -0.19516007602214813, -0.25941815972328186, -0.4565809965133667, -0.03502226620912552, 0.7639212608337402, 1.1068196296691895, -0.19954773783683777, -0.3997941315174103, -0.07931928336620331, 0.39248502254486084, 0.8109928369522095, -0.33772921562194824, -0.318228542804718, 0.13600443303585052, 0.039017435163259506}, {0.011806299909949303, -0.003236927790567279, -0.008766102604568005, 0.04864860698580742, -0.02953188121318817, -0.01864861696958542, -0.014813813380897045, -0.027996066957712173, -0.018343430012464523, 0.024359550327062607, -0.0051518711261451244, -0.01429772935807705, -0.01625717803835869, -0.016049092635512352, -0.009037754498422146, -0.002839979948475957, 0.0352618545293808, -0.013484196737408638, 0.034888267517089844, -0.004366835113614798, 0.025421326979994774, -0.015863938257098198, -0.015250978991389275, -0.016636399552226067, 0.007074013352394104, 0.009362290613353252, 0.008313401602208614, -0.00908725056797266, -0.0017832292942330241, 0.0017354179872199893, 0.02560143545269966, -0.024055713787674904, -0.020117074251174927, 0.008658159524202347, -0.024512261152267456, -0.0038952648174017668, -0.0018337131477892399, -0.03467710688710213, -0.012113173492252827, -0.025239909067749977, 0.012239325791597366, -0.008327626623213291, 0.018709374591708183, -0.000870531250257045, -0.006864565424621105, -0.00043912208639085293, 0.004758829716593027, 0.01031258050352335, 0.0005947305471636355, -0.0013309621717780828, -0.010447981767356396, -0.006228683516383171, 0.015118525363504887, 0.012378516606986523, -0.003698373679071665, -0.02612689696252346, -0.023789850994944572, -0.0010676326928660274, 0.0004170542524661869, -0.028351780027151108, 0.003098621964454651, 0.00850811880081892, -0.0055831377394497395, -0.020470978692173958, -0.0017438054783269763, -0.012604833580553532, 0.021240776404738426, -0.002526208758354187, -0.02081378363072872, -0.015370517037808895, -0.0036228876560926437, 0.003815337084233761, -0.007398694753646851, 0.009916422888636589, -0.01473726611584425, 0.007042901590466499, -0.009279859252274036, 0.01521268580108881, 0.007363828830420971, -0.010437892749905586, -0.017646122723817825, -0.00965737085789442, -0.0032371620181947947, 0.019392050802707672, -0.006722013466060162, -0.00513301370665431, 0.023086678236722946, -0.0169687382876873, 0.02099321037530899, -0.020879417657852173, -0.006731642410159111, -0.0047587137669324875, -0.011949541978538036, -0.013217642903327942, 0.003435272490605712, 0.023489437997341156, -0.03604355826973915, -0.00279191043227911, -0.010199292562901974, -0.011615089140832424, 0.008623222820460796, 0.007094135973602533, 0.012989545240998268, -0.011793935671448708, 0.0004998115473426878, 0.005298983305692673, -0.00181428377982229, -0.029931174591183662, 0.0022938523907214403, -0.003073731902986765, -0.017298920080065727, -0.029951684176921844, 0.019679000601172447, 0.005039574578404427, -0.020556114614009857, 0.011652011424303055, -0.0008498508250340819, 0.019016161561012268, -0.018343716859817505, 0.00181106710806489, -0.004530944395810366, -0.008680827915668488, -0.007001594174653292, 0.030387232080101967, 0.009279911406338215, 0.0033413288183510303, 0.0010392912663519382, 0.00902453251183033, -0.013558631762862206, 0.00361162843182683, -0.012520041316747665, 0.004105512518435717, 0.005161479115486145, 0.01347876526415348, -0.00784582644701004, -0.0008528924081474543, -0.04019691422581673, -0.004342357162386179, 0.019551163539290428, 0.011901287361979485, 9.69873508438468e-05, 0.023200465366244316, 0.0004393387062009424, 0.010450867004692554, 0.007165980059653521, 0.015508787706494331, 0.012353681027889252, -0.001809040317311883, -0.002875895006582141, 0.009168618358671665, -0.011841878294944763, 0.019477514550089836, -0.01852039247751236, 0.010402311570942402, -0.01147019024938345, -0.0029700039885938168, 0.01158544048666954, -0.02945164032280445, 0.005620531737804413, -0.025526518002152443, -0.013048806227743626, 0.013479181565344334, -0.007463506888598204, 0.004762648604810238, 0.0027397717349231243, 0.031505901366472244, -0.02104802057147026, 0.009739095345139503, -0.00030810985481366515, 0.016986461356282234, 0.007937345653772354, -0.007376708090305328, -0.008196445181965828, -0.019504345953464508, 0.002334985416382551, 0.028606006875634193, 0.0028598266653716564, 0.0007993835606612265, 0.01491951011121273, -0.021344894543290138, -0.006809981074184179, -0.005188378971070051, -0.015346981585025787, 0.0043462119065225124, -0.036342065781354904, 0.011084667406976223, -0.01648254133760929, 0.02443014457821846, -0.004841271787881851, -0.007712458726018667, -0.00635098572820425, -0.01373173389583826, 0.017535436898469925, -0.0027972133830189705, -0.0069130598567426205, 0.029727039858698845, -0.020805155858397484, 0.0020310310646891594, 0.0014422587119042873, 0.00899131502956152, -0.028009790927171707, 0.006186442915350199, 0.0002533298684284091, -0.000394880014937371, 0.006805882323533297, 0.029427146539092064, 0.0033986433409154415, 0.014914248138666153, -0.012281475588679314, -0.039592452347278595, -0.02798992209136486, -0.003481819061562419, -0.02847215160727501, 0.021147405728697777, 0.022645099088549614, 0.015248267911374569, -0.036778613924980164, 0.0012802665587514639, 0.003792086848989129, -0.0099136708304286, -0.003319463226944208, -0.0024630078114569187, 0.00804939866065979, 0.00586713058874011, 0.0010583880357444286, -0.012960311025381088, 0.0032599095720797777, 0.0012803527060896158, 0.021733902394771576, 0.008748538792133331, 0.005849834997206926, -0.013304453343153, 0.011348536238074303, -0.0013742519076913595, -0.008386291563510895, -0.005028904881328344, 0.003288864390924573, -0.0027715868782252073, 0.04270969331264496, -0.009596946649253368, 0.008499834686517715, 0.005494139157235622, -0.029074812307953835, 0.026302548125386238, -0.013204722665250301, -0.006730910390615463, -0.01843952015042305, 0.019680919125676155, -0.0023834079038351774, 0.0006270932499319315, -0.004096357151865959, -0.033363163471221924, -0.0053492202423512936, 0.006169002503156662, 0.0010053453734144568, -0.00197419710457325}, {0.4444071054458618, 0.03181363642215729, 0.17263221740722656, -0.07705102115869522, 0.03856243938207626, 0.222536101937294, 0.05506778880953789, 0.24939122796058655, 0.1436540186405182, 0.37491264939308167, -0.4191071689128876, 0.006227975245565176, 0.038580261170864105, 0.15280045568943024, 0.19975607097148895, -0.674767255783081, -0.030313048511743546, 0.01636962965130806, -0.2397242933511734, 0.3131544888019562, -0.12966324388980865, -0.7787423729896545, -0.09747347235679626, 0.01602802984416485, -0.158837229013443, -0.23594683408737183, -0.9588481187820435, 0.26689469814300537, 0.017785394564270973, -0.17729398608207703, 0.47830909490585327, -0.5132790803909302, -0.23539693653583527, 0.11320225149393082, -1.470154047012329, -0.3443997800350189, -0.37316370010375977, -0.6729084849357605, -0.5929474234580994, 0.1428099274635315, -0.254083514213562, 0.013630988076329231, -0.3847142159938812, 0.06359145790338516, -0.32412752509117126, -0.228513702750206, -0.21591109037399292, -0.05561095103621483, 0.16820916533470154, -0.18637683987617493, -0.4029633402824402, -0.34034669399261475, 0.21259735524654388, -0.27901744842529297, -0.430982381105423, -0.9150160551071167, 0.38292890787124634, 0.5695423483848572, -0.11833979189395905, -0.24463903903961182, 1.0737637281417847, 0.13925868272781372, 0.07167165726423264, -0.6187199354171753, -0.07756790518760681, 0.36252593994140625, -0.0008199459407478571, 0.17399035394191742, -0.3381188213825226, 0.5175755620002747, 0.688037097454071, 0.05970386043190956, 0.8225837349891663, 0.5194845795631409, -0.29042935371398926, 0.03830919787287712, 0.1489993929862976, 0.12524592876434326, -0.4063146114349365, 0.2585383951663971, -0.664811909198761, -0.5306280255317688, 0.06502820551395416, -0.19339291751384735, 0.1337706744670868, -0.0033904416486620903, 0.37644174695014954, -0.3827112913131714, -0.14978647232055664, 0.030221885070204735, 0.16953709721565247, -0.1297500878572464, 0.02277100645005703, -0.12832783162593842, 0.1775108128786087, 0.13566967844963074, 0.07290130108594894, -0.16904127597808838, -0.8073838949203491, -0.2534177601337433, -0.12721428275108337, 0.4478509724140167, -0.0027593716513365507, 0.3127255439758301, -0.23513856530189514, -0.3878098130226135, 0.18890978395938873, 0.23226824402809143, 0.03225830942392349, 0.18396610021591187, 0.19202500581741333, 0.15852999687194824, 0.045832954347133636, 0.09497129917144775, -0.4831309914588928, 0.009806330315768719, 0.12278323620557785, 0.01241363026201725, 0.2535827159881592, -0.2446357011795044, -0.015546342357993126, -0.813948392868042, -0.344881534576416, -0.16183380782604218, -0.07911821454763412, -0.030858110636472702, -0.0005276642041280866, 0.14399540424346924, -0.11769981682300568, 0.010846128687262535, 0.03080972470343113, 0.3992510139942169, -0.352948933839798, 0.028502052649855614, -0.7023676037788391, -0.6381381750106812, -0.0603981539607048, 0.4832155704498291, -0.3170103430747986, -0.6110259890556335, 0.01588234305381775, 0.0559125579893589, 0.3282488286495209, 0.2064148634672165, -0.3671424388885498, -0.07725738734006882, 0.9983111023902893, -0.11062438040971756, 0.21902984380722046, -0.19485405087471008, 0.01222077663987875, 0.28663375973701477, 0.07554588466882706, -0.004027427639812231, 0.2513599395751953, 0.1368473321199417, -0.10707185417413712, -1.0268601179122925, 0.1801975518465042, -0.017391575500369072, -0.06656547635793686, 0.3404242694377899, -0.22617852687835693, -0.23642873764038086, 0.265783429145813, -0.2588687241077423, 0.03371492400765419, -0.1915343999862671, -0.4074554741382599, 0.20985332131385803, 0.2171875387430191, 0.09435787051916122, -0.4647273123264313, -0.31933972239494324, -0.18645606935024261, -0.05182785540819168, -0.08665816485881805, 0.12065539509057999, 0.3703024685382843, -0.5341703295707703, 0.05388297513127327, 0.04269835352897644, -0.24225236475467682, 0.7381053566932678, 0.269631952047348, -0.1997947096824646, -1.0482738018035889, -0.2592891752719879, -0.6098309755325317, 0.39618510007858276, -0.3504886329174042, -0.3090338110923767, -0.6985588073730469, -0.5563820600509644, 0.5387558937072754, 1.4871104955673218, 0.168766051530838, 0.5137218832969666, 0.5399420261383057, 0.3929252028465271, -0.02269390970468521, -0.3614124357700348, -0.27473291754722595, 0.19679681956768036, 0.5555698871612549, 0.28553152084350586, -0.020066268742084503, 0.5956929326057434, 0.16233986616134644, 0.18302325904369354, 0.2216404229402542, -0.9356026649475098, 0.0979398712515831, -0.1099042296409607, 0.017625853419303894, 0.02328823134303093, 0.0896037295460701, 0.2604873776435852, -0.15078623592853546, 0.1877138465642929, -0.25084805488586426, -0.0903274193406105, 0.03298308700323105, -1.1682392358779907, -0.2443191111087799, -0.9878484010696411, 0.7902402877807617, 0.6966215968132019, -0.033735934644937515, -0.05536200851202011, 0.04255079850554466, 0.31429341435432434, 0.195112943649292, -0.1622905731201172, 0.007167617790400982, -0.357766717672348, 0.3967192471027374, -0.30032312870025635, 0.3348821997642517, -0.24595513939857483, 0.3285849690437317, 0.1796814203262329, -0.14599919319152832, 0.08365115523338318, -0.47248929738998413, 0.032425180077552795, 0.1920202672481537, 0.9191481471061707, 0.2526872754096985, 0.07774781435728073, 0.5022755861282349, -0.18875539302825928, 0.2592757046222687, 0.08283254504203796, -0.23840568959712982, 0.03675563260912895}, {-0.010278355330228806, -0.008501026779413223, 0.008849860168993473, 0.01052815094590187, -0.022455887869000435, -0.019129447638988495, -0.004562291782349348, -0.03080209344625473, -0.028886083513498306, 0.019939985126256943, -0.008606093004345894, -0.015298089012503624, 0.005824488587677479, -0.008185723796486855, -0.007892000488936901, -0.016660047695040703, 0.03173235058784485, -0.022640613839030266, 0.01512282807379961, -0.017052991315722466, 0.00991741567850113, -0.003882539225742221, -0.021173764020204544, -0.025892112404108047, 0.012145943939685822, 0.0036686663515865803, 0.006640564184635878, 0.0059541878290474415, -0.02672148495912552, 0.0016607801662757993, 0.013367421925067902, 0.0007502931985072792, 0.007825643755495548, 0.010871922597289085, -0.010343200527131557, -0.005398722365498543, -0.01337346900254488, -0.014720729552209377, 0.005665985867381096, -0.03723359853029251, 0.014472211711108685, 0.010336572304368019, 0.012532501481473446, 0.010093250311911106, 0.006986445747315884, -0.0032666753977537155, -0.002932352712377906, 0.009243671782314777, 0.020151296630501747, 0.00832840334624052, -0.012934125028550625, -0.013038408942520618, 0.017853697761893272, 0.014241751283407211, 0.008869508281350136, -0.020656876266002655, -0.013556605204939842, -0.015766486525535583, -0.016428297385573387, -0.00709604611620307, -0.0010128351859748363, -0.0011535739758983254, -0.006771797779947519, 0.002850597258657217, -0.015539180487394333, 0.007746958173811436, 0.0174734964966774, -0.0015158255118876696, -0.00801530946046114, -0.014108861796557903, 0.010062060318887234, 0.013291832059621811, -0.00857923086732626, 0.007958994247019291, 0.004127091262489557, 0.012439437210559845, -0.00024285791732836515, 0.02399810403585434, -0.01745053380727768, 0.008462036028504372, -0.013012325391173363, -0.016848132014274597, -0.010612361133098602, 0.015612547285854816, -0.038787078112363815, -0.008273715153336525, 0.010905180126428604, -0.01650470867753029, 0.031052598729729652, -0.016541210934519768, -0.010979530401527882, 0.006710430607199669, -0.0229532141238451, -0.002839549444615841, 0.004057697951793671, 0.018258346244692802, -0.01535808015614748, 0.010077268816530704, -0.026403840631246567, -0.01844433695077896, 0.012620106339454651, 0.011326208710670471, -0.010163183324038982, 0.002444760873913765, 0.0019278137478977442, -0.01620021089911461, 0.010469453409314156, -0.026738926768302917, -0.018260419368743896, 0.012234771624207497, -0.016063662245869637, -0.04154413193464279, 0.0013344708131626248, -0.02744918316602707, -0.011451862752437592, 0.0008537517860531807, -0.009899735450744629, 0.011437861248850822, 0.0016656506340950727, 0.0198544729501009, -0.020029595121741295, -0.021903350949287415, 0.0030796295031905174, 0.02548111043870449, 0.0052526541985571384, 0.023993972688913345, 0.006369547452777624, 0.007765945978462696, -0.007519206032156944, 0.014768959023058414, -0.009379814378917217, -0.02113731950521469, 0.0050170086324214935, 0.0051065548323094845, -0.019296081736683846, 0.013133717700839043, -0.018465768545866013, -0.008993667550384998, 0.022086139768362045, 0.00742963096126914, -0.005468886811286211, 0.01309353206306696, -0.0023115987423807383, 0.017334746196866035, -0.01428179070353508, -0.002912241965532303, 0.006250981241464615, 0.015820695087313652, -0.00851461011916399, 0.02207108959555626, -0.01330616232007742, 0.02749357558786869, -0.018098697066307068, 0.015552854165434837, -0.012453850358724594, -0.01579129882156849, -0.0018220230704173446, -0.015005012042820454, -0.004566626623272896, -0.008094008080661297, -0.006311929784715176, 0.010167762637138367, -0.016951700672507286, 0.035610947757959366, -0.007835006341338158, 0.02437666989862919, -0.015173530206084251, -0.0016873963177204132, -0.0011967013124376535, 0.003779425285756588, -0.0016693833749741316, 0.006692893337458372, -5.510901246452704e-05, 0.019021233543753624, -0.00836097914725542, 0.026691336184740067, 0.0023049302399158478, -0.008651024661958218, 0.0037295131478458643, -0.012081808410584927, -0.01953514851629734, -0.011496922932565212, -0.01737138070166111, -0.009077644906938076, -0.029544951394200325, 0.01340430323034525, 0.010995659977197647, 0.05096030235290527, -0.012708173133432865, -0.011139199137687683, -0.018061289563775063, -0.007868064567446709, 0.003082837210968137, -0.0031769948545843363, -0.0026017692871391773, 0.0036997657734900713, -0.002686689142137766, -0.004311135038733482, -0.0020531557966023684, 0.000816195853985846, -0.019746582955121994, -0.010987251996994019, -0.010806377045810223, 0.0022907813545316458, -0.020931052044034004, 0.02061326615512371, 0.008752202615141869, 0.002296456368640065, -0.002057786798104644, -0.021243760362267494, -0.03778959438204765, -0.002851661527529359, -0.00313089182600379, 0.003605789039283991, 0.010125218890607357, -0.0005990653298795223, -0.023457448929548264, -0.0069128768518567085, 0.01025379728525877, -0.018486661836504936, -0.007456987164914608, 0.004665770102292299, 0.0019559503998607397, -0.0014112184289842844, -0.0064889597706496716, -0.010670709423720837, -0.0011643549660220742, -0.01564043201506138, 0.010998877696692944, 0.011644561775028706, -0.020324649289250374, -0.01627245545387268, 0.006398228462785482, 0.006161104422062635, -0.01052925270050764, -0.014494221657514572, 0.00027845316799357533, -0.004083001054823399, 0.037052400410175323, -0.018272414803504944, 0.018207073211669922, 0.009507106617093086, 0.0026991318445652723, 0.03934942185878754, -0.00953281857073307, -0.0035179387778043747, 0.0037978163454681635, -0.0076621524058282375, -0.004071004688739777, 0.007358758244663477, -0.025937872007489204, -0.005227888002991676, -0.012486199848353863, 0.01954510435461998, 0.003400297136977315, -0.00624126335605979}}
local m4 = {-0.12119200825691223, -0.4320903420448303, 0.16360077261924744, -0.024315910413861275, 0.318928062915802, 0.28024089336395264, -0.11023423820734024, 0.1976889669895172, 0.5035359859466553, -0.12433239072561264, 0.4639778137207031, 0.4027350842952728, 0.12333646416664124, -0.25493040680885315, -0.02231675200164318, 0.1948857307434082, -0.4754638671875, 0.16065751016139984, -0.23279736936092377, 0.1479167342185974, 0.009400470182299614, 0.47699084877967834, 0.08560270071029663, 0.292190283536911, -0.3599409759044647, 0.058879803866147995, 0.11652722209692001, 0.017723476514220238, 0.31027185916900635, 0.42517009377479553, -0.16898401081562042, 0.0004393960989546031, 0.3184901475906372, -0.3918144702911377, 0.6982641816139221, 0.19490303099155426, -0.043992869555950165, 0.5211102962493896, 0.27624088525772095, 0.18817909061908722, 0.0014870389131829143, 0.23174692690372467, -0.3168533742427826, -0.41062405705451965, -0.11928929388523102, 0.36713600158691406, -0.22957241535186768, 0.03572351485490799, -0.42685163021087646, -0.005996388848870993, 0.0681905448436737, -0.18357662856578827, -0.4264618456363678, -0.10152142494916916, 0.08709192276000977, 0.5503253936767578, -0.18714933097362518, -0.395602285861969, 0.44956478476524353, -0.015746744349598885, -0.17238160967826843, 0.1827181577682495, 0.4743066430091858, 0.6574476957321167, 0.38722914457321167, -0.16221420466899872, -0.22226335108280182, 0.566750705242157, 0.31580430269241333, -0.029436055570840836, -0.09957416355609894, -0.05943959951400757, -0.4640403985977173, -0.41787704825401306, -0.019030319526791573, -0.2625054717063904, -0.03521683067083359, -0.5692240595817566, 0.030449818819761276, -0.8333264589309692, -0.0997619777917862, 0.33173683285713196, 0.023881589993834496, -0.4496639668941498, 0.1895037442445755, 0.03452831134200096, -0.4834495484828949, 0.26014867424964905, -0.4918281137943268, 0.21658116579055786, -0.019057705998420715, -0.29834115505218506, -0.04869748279452324, 0.17696478962898254, -0.15104009211063385, -0.7326403856277466, 0.2771260142326355, -0.11435204744338989, 0.7235895395278931, 0.31815558671951294, -0.49658554792404175, 0.170261412858963, -0.35118937492370605, -0.0026072559412568808, 0.3665490448474884, -0.16517239809036255, 0.2335260808467865, 0.16591136157512665, 0.12471108138561249, -0.21304485201835632, 0.7565355896949768, 0.5610060691833496, 0.33956265449523926, 0.36495935916900635, 0.05180160328745842, -0.6422286629676819, 0.24114453792572021, -0.38803631067276, -0.029255546629428864, -0.08472876995801926, 0.4624996781349182, 0.8540431261062622, -0.19887351989746094, -0.24511770904064178, -0.09459339827299118, -0.07290452718734741, 0.069530189037323, -0.3269287943840027, 0.29990383982658386, -0.2062879204750061, 0.04589494317770004, 0.40371009707450867, 0.2186051905155182, -0.00915728323161602, 0.944208562374115, 0.02223058231174946, 0.3416841924190521, -0.1464000791311264, -0.36764127016067505, -0.24689121544361115, -0.15031121671199799, -0.4667811989784241, -0.3748229444026947, 0.050587594509124756, -0.212965190410614, -0.24506749212741852, -0.3570350408554077, -0.09480604529380798, -0.08399725705385208, -0.07389301061630249, 0.2477569729089737, -0.4943859875202179, 0.21521520614624023, -0.14873568713665009, 0.24749773740768433, 0.15940918028354645, -0.4732681214809418, 1.1500868797302246, -0.05571204796433449, 0.11321435868740082, 0.05533626675605774, -0.10508105903863907, 0.1509464979171753, -0.1316273957490921, -0.1409706324338913, -0.6401874423027039, 0.07622697204351425, -0.02066471241414547, 0.026243390515446663, -0.3827364444732666, -0.056412018835544586, 0.5131473541259766, -0.1578075885772705, 0.02683178335428238, 0.3737300634384155, 0.269226998090744, 0.05207091569900513, 0.0038901555817574263, -0.059790484607219696, 0.24472108483314514, 0.02900165133178234, 0.38514867424964905, 0.059309568256139755, -0.06608220934867859, 0.7094714045524597, 0.30065619945526123, 0.2561238706111908, -0.6245588064193726, 0.12549398839473724, 0.16537179052829742, -0.17407894134521484, 0.29258841276168823, -0.2057512253522873, 0.12592479586601257, -0.13023057579994202, -0.9442114233970642, 0.3103829324245453, 0.41151106357574463, -0.13051091134548187, -0.2560710906982422, 0.5063381791114807, 0.245778426527977, -0.0909273698925972, -0.01295003853738308, -0.03543900325894356, -0.09522618353366852, 0.05559632554650307, 0.41493257880210876, 0.34701067209243774, 0.3603726029396057, 0.8147027492523193, 0.5282593369483948, -0.12134373188018799, -0.16668663918972015, -0.15000194311141968, 0.028442131355404854, 0.04981180280447006, 0.08199391514062881, -0.1215803250670433, -0.009129812009632587, 0.12319072335958481, -0.22715942561626434, -0.0439898744225502, 0.3909992277622223, -0.32664042711257935, 0.18921226263046265, 0.08403586596250534, -0.3656195402145386, -0.6538853049278259, -0.6281121373176575, 0.18768808245658875, -0.3358302712440491, -0.332994669675827, -0.11146729439496994, 0.3513216972351074, 0.20511022210121155, -0.4514368772506714, 0.42106395959854126, -0.4724952280521393, 0.35859474539756775, -0.2770954668521881, -0.020831327885389328, -0.19145646691322327, -0.1986144483089447, 0.19923116266727448, 0.3429737091064453, 0.2780303359031677, -0.37758925557136536, -0.08119022846221924, 0.36074256896972656, 0.18353813886642456, 0.1272091567516327, 0.02147340029478073, 0.5132419466972351, 0.24231310188770294, 0.24474278092384338}
local m5 = {-0.18638470768928528, -0.1896393597126007, -0.1889890730381012, -0.19049835205078125, -0.18795514106750488, -0.18933545053005219, -0.19069159030914307, -0.18700024485588074, -0.18877221643924713, -0.19134320318698883, -0.19095942378044128, -0.18640364706516266, -0.1892036348581314, -0.18770746886730194, 1.2487244606018066, -0.18736907839775085, -0.1877896934747696, -0.18898238241672516, -0.1888350248336792, -0.1865197867155075, -0.18802186846733093, -0.18859650194644928, -0.19018925726413727, 1.5589599609375, -0.18677210807800293, -0.18892286717891693, -0.11103861778974533, -0.18899531662464142, 1.566094994544983, -0.18721932172775269, 0.8411797285079956, -0.18822908401489258}

local function zipmap(f, ...)
    local out = {}
    local ars = {...}
    for i in pairs(ars[1]) do
        local arg = {}
        for _, arr in pairs(ars) do
            table.insert(arg, arr[i])
        end
        table.insert(out, f(table.unpack(arg, 1, #arg)))
    end
    return out
end

local function sum(arr)
    local acc = 0
    for _, v in pairs(arr) do
        acc = acc + v
    end
    return acc
end
local function pairsum(...) return zipmap(function(...) return sum {...} end, ...) end

local function dot(mat, vec)
    local out = {}
    for i, row in pairs(mat) do
        local acc = 0
        for j, val in pairs(row) do
            acc = acc + val * vec[j]
        end
        out[i] = acc
    end
    return out
end

local function zeroes(n)
    local out = {}
    for i = 1, n do
        table.insert(out, 0)
    end
    return out
end

local chars = "abcdefghijklmnopqrstuvwxyz-|. \n_"
local i_to_c_map = {}
local c_to_i_map = {}
for i = 1, #chars do
    i_to_c_map[i] = chars:sub(i, i)
    c_to_i_map[chars:sub(i, i)] = i
end

local function onehot(char)
    local x = zeroes(#chars)
    x[c_to_i_map[char]] = 1
    return x
end

local function softmax(v)
    local out = zipmap(math.exp, v)
    local s = sum(out)
    return zipmap(function(x) return x / s end, out)
end

local function run_step(char, prev)
    local new_hidden = zipmap(math.tanh, pairsum(dot(m1, onehot(char)), dot(m2, prev), m4))
    return softmax(pairsum(dot(m3, new_hidden), m5)), new_hidden
end

local function do_sample(opts, weights)
    local cweights = {}
    local acc = 0
    for i, w in pairs(weights) do
        acc = acc + w
        cweights[i] = acc
    end
    local value = math.random() * acc
    for i, cw in pairs(cweights) do
        if cw > value then
            return opts[i]
        end
    end
end

local function run_seq(input, n)
    local dist
    local hidden = zeroes(#m4)
    local buf = {}
    for i = 1, #input - 1 do
        dist, hidden = run_step(input:sub(i, i), hidden)
    end
    local char = input:sub(#input, #input)
    for i = 1, n do
        dist, hidden = run_step(char, hidden)
        local sample = do_sample(i_to_c_map, dist)
        table.insert(buf, sample)
        char = buf[#buf]
    end
    return table.concat(buf)
end

local function read_board(s)
    local out = {}
    for x in (s .. "\n"):gmatch "([xo\\.]+)\n" do
        x:gsub(".", function(a)
            table.insert(out, a)
        end)
    end
    return out
end

local function entry(s)
    local board = read_board(s)
    local output = read_board(run_seq(s .. "\n-\n", 11))
    local choices = {}
    for i = 1, #board do
        if board[i] == "." and output[i] == "x" then
            board[i] = "x"
            return table.concat(board)
        elseif board[i] == "." then
            table.insert(choices, i)
        end
    end
    board[choices[math.random(1, #choices)]] = "x"
    return table.concat(board)
end

local char = "e"
local prev = zeroes(#m4)
local new_hidden = zipmap(math.tanh, pairsum(dot(m1, onehot(char)), dot(m2, prev), m4))
local output_pre_bias = softmax(pairsum(dot(m3, new_hidden), m5))
for x, v in ipairs(output_pre_bias) do
    print((x-1) .. "|" .. v)
end